using System;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using System.IO;
using Symcon.Utils.MessageDispatcher;
using System.Collections.Generic;
namespace Symcon.LightRidge.PEAdvisor
{

    /// <summary>
    /// Summary description for AggregateFormula.
    /// </summary>
    public class AggregateFormula
    {
        public AggregateFormula()
        {
            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

            _lrrModes = LRRModel.ProjectModes;

            _objAggregatePropertyValues = LRRModel.AggregatePropertyValuesObj;

            _genericFormulas = new LRRGenericFormulas();
            _app.DBManagerObj.PopulateCollection(_genericFormulas, "", false);
        }

        private bool propValExists;

        public bool PropValExists
        {
            get { return propValExists; }
            set { propValExists = value; }
        }


        public Hashtable AggregateSiteComponents
        {
            get { return SiteComponents; }

        }

        private void AssignPriorityToFormula(LRRFormula lrrFormula,string modeID)
        {
            if (modeID == "1")
            {
                lrrFormula.Priority = 2;
            }
            if (modeID == "2")
            {
                lrrFormula.Priority = 3;
            }
            if (modeID == "3")
            {
                lrrFormula.Priority = 4;
            }
        }

        private Hashtable GenerateSimpleFormulaForProperty(LRRAggregateProperty objAggregateProperty, string sModeID, string sModeName, bool bOverwriteExisting, LRRFormulaParams formulaParamsToDelete, int nParam)
        {
            string sPropExpression = "";
            string sPropName = "";
            string sMasterProperty = "";
            string sFormulaExpression = "";
            LRRFormulaParams _formulaParams = new LRRFormulaParams();
            LRRFormula _lrrFormula;
            LRRAggregatePropertyValue objAggPropVal = objAggregateProperty.GetValueObjectForPropertyMode(sModeID);

            sPropName = objAggregateProperty.Name == null ? "" : objAggregateProperty.Name.ToString(Symcon.BusinessObjects.SymConverter.Culture);
            sFormulaExpression = _genericFormulas[0].Expression == null ? "" : _genericFormulas[0].Expression.ToString(Symcon.BusinessObjects.SymConverter.Culture);
            sMasterProperty = objAggregateProperty.MasterPropertyID == null ? "" : objAggregateProperty.MasterPropertyID.ToString(Symcon.BusinessObjects.SymConverter.Culture);

            if (objAggPropVal != null)
            {
                string sFormulaIDForAgg = objAggPropVal.FormulaID == null ? "" : objAggPropVal.FormulaID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                //Create Formula only if not exists
                if ((sFormulaIDForAgg != "") && (bOverwriteExisting))
                {
                    _lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(sFormulaIDForAgg);
                    if (_lrrFormula != null)
                    {
                        _lrrFormula.Expression = "";
                        LRRFormulaParams formulaParams = _lrrFormula.Params;
                        foreach (LRRFormulaParam objParam in formulaParams)
                        {
                            formulaParamsToDelete.Add(objParam);
                        }
                        foreach (LRRFormulaParam objParam in formulaParamsToDelete)
                        {
                            _lrrFormula.Params.Remove(objParam);
                        }
                    }
                }
                else
                {
                    _lrrFormula = new LRRFormula();
                }
                if (_lrrFormula != null)
                {
                    _lrrFormula.FormulaName = sPropName + "[A][" + sModeName + "]";
                    _lrrFormula.FormulaDescription = "Auto Generated";
                    _lrrFormula.PostCalc = true;
                    _lrrFormula.IsFormula = true;
                    AssignPriorityToFormula(_lrrFormula, sModeID);
                    if(!LRRModel.FormulasObj.Contains(_lrrFormula))
                           LRRModel.FormulasObj.Add(_lrrFormula);
                    objAggPropVal.FormulaObj = _lrrFormula;
                    IEnumerator SiteCompEnum = SiteComponents.Keys.GetEnumerator();
                    //LRRAggregateEquipments aggregateEquips = LRRModel.AggregatesObj.GetAggregateFromID(objAggregateProperty.AggregateID).AggregateEquipments;
                    LRRAggregateEquipments aggregateEquips = LRRModel.AggregateEquipmentsObj.GetAggregateEquipsFromParentAggregateID(objAggregateProperty.AggregateID);   
                    foreach (LRRAggregateEquipment aggregateEquip in aggregateEquips)
                    {
                        //Normal Case.
                        if (!string.IsNullOrEmpty(aggregateEquip.EquipmentID))
                        {
                            LRRSiteComponent objSiteComponent = SiteComponents[aggregateEquip.EquipmentID] as LRRSiteComponent;
                            LRRSiteComponentProperties objSiteComponentProps = LRRModel.SiteComponentPropertiesObj.GetSiteComponentPropertiesByComponentID(objSiteComponent.SiteComponentID);
                            if ((objSiteComponent != null) && (objSiteComponentProps.Count > 0))
                            {
                                LRRSiteComponentProperty objSiteCompProp = objSiteComponentProps.GetSiteCompPropFromMasterPropertyID(objSiteComponent.SiteComponentID, sMasterProperty);
                                if (objSiteCompProp != null)
                                {
                                    LRRSiteComponentPropertyValue objPropVal = objSiteCompProp.GetValueObjectForPropertyMode(sModeID);

                                    if (objPropVal != null)
                                    {
                                        LRRFormulaParam _formulaParam = new LRRFormulaParam();
                                        _formulaParam.FormulaID = _lrrFormula.ID;
                                        _formulaParam.ParamName = Convert.ToChar(nParam, Symcon.BusinessObjects.SymConverter.Culture).ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                        _formulaParam.ParamPath = "Aggregates\\" + objSiteCompProp.ID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                        _formulaParam.SourceOBJID = objSiteCompProp.ID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                        _formulaParam.SourceCollectioName = objPropVal.TypeOfObject.ToString();
                                        _formulaParam.CurrentValue = objPropVal.PropertyValue;
                                        _formulaParam.PropertyName = objSiteCompProp.Name;
                                        _formulaParam.ModeID = sModeID;
                                        _formulaParam.ModeName = sModeName;
                                        _formulaParam.UOM = objSiteCompProp.UOM;
                                        nParam = nParam + 1;
                                        _formulaParams.Add(_formulaParam);
                                    }
                                }
                            }
                        }
                        //Aggregate Of aggregate case.
                        else
                        {
                            LRRAggregate aggregateObj = LRRModel.AggregatesObj.GetAggregateFromID(aggregateEquip.ChildAggregateID) as LRRAggregate;
                            LRRAggregateProperty aggProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromMasterPropertyID(aggregateObj.AggregateID, sMasterProperty);
                            if (aggProp != null)
                            {
                                LRRAggregatePropertyValue aggPropValue = aggProp.Values.GetAggregateValueObjectForPropertyMode(sModeID, aggProp.ID);  
                                if (aggPropValue != null)
                                {
                                    LRRFormulaParam _formulaParam = new LRRFormulaParam();
                                    _formulaParam.FormulaID = _lrrFormula.ID;
                                    _formulaParam.ParamName = Convert.ToChar(nParam, Symcon.BusinessObjects.SymConverter.Culture).ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    _formulaParam.ParamPath = "Aggregates\\" + aggProp.ID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    _formulaParam.SourceOBJID = aggProp.ID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    _formulaParam.SourceCollectioName = aggPropValue.TypeOfObject.ToString();
                                    _formulaParam.CurrentValue = aggPropValue.PropertyValue;
                                    _formulaParam.PropertyName = aggProp.Name;
                                    _formulaParam.ModeID = sModeID;
                                    _formulaParam.ModeName = sModeName;
                                    _formulaParam.UOM = aggProp.UOM ;
                                    nParam = nParam + 1;
                                    _formulaParams.Add(_formulaParam);
                                }
                            }
                        }
                    }
                    if (sFormulaExpression != "WEIGHTED-AVERAGE")
                    {
                        if (_formulaParams.Count > 0)
                        {
                            _lrrFormula.Params = _formulaParams;
                            FormulaParamsForSimpleProp.Add(sModeID, _formulaParams);
                            FormulaExpression objFormulaExpression = new FormulaExpression(_formulaParams);
                            FormulaExp = getFormulaString(objFormulaExpression, sFormulaExpression, FormulaParamsForSimpleProp, objAggregateProperty);
                        }
                        return FormulaExp;
                    }
                    else
                    {
                        if (_formulaParams.Count > 0)
                        {
                            _lrrFormula.Params = _formulaParams;
                            FormulaParamsForSimpleProp.Add(sModeID, _formulaParams);
                        }
                        return FormulaParamsForSimpleProp;
                    }

                }
            }
            return FormulaParamsForSimpleProp;
                
            
        }

        #region Evaluate Formulas
        private void EvaluteFormulaForSimpleProp(LRRAggregateProperty objAggProperty, Hashtable Formulas)
        {
            IEnumerator iEnum = Formulas.GetEnumerator();
            while (iEnum.MoveNext())
            {
                DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
                string ModeId = dEntry.Key.ToString();
                LRRAggregatePropertyValue objAggPropVal = objAggProperty.GetValueObjectForPropertyMode(ModeId);
                FormulaExpressionObjects expObj = dEntry.Value as FormulaExpressionObjects;
                LRRFormulaParams formulaParams = expObj.Params;
                LRRFormulaParam fParam = formulaParams[0] as LRRFormulaParam;
                string sPropExpression = expObj.FormulaExpression;
                LRRFormula _lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(fParam.FormulaID);
                if (_lrrFormula != null)
                {
                    if (_lrrFormula.Params.Count > 0)
                    {
                        if (objAggPropVal != null)
                        {
                            string sAggPropValID = objAggPropVal.ID == null ? "" : objAggPropVal.ID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                            string sAggPropID = objAggPropVal.AggregateID == null ? "" : objAggPropVal.AggregateID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                            objAggPropVal.FormulaID = _lrrFormula.ID;
                            _lrrFormula.SourceCollectionName = objAggPropVal.TypeOfObject.ToString();
                            LRRFormulaDataStore.SourceCollectionName = objAggPropVal.TypeOfObject.ToString();
                            LRRFormulaDataStore.ObjectSource = typeof(LRRAggregatePropertyValue).ToString();
                            LRRFormulaDataStore.SourceObjectID = sAggPropValID;
                            LRRFormulaDataStore.SourcePropertyID = sAggPropID;
                            LRRFormulaDataStore.ModeID = ModeId;
                            _lrrFormula.SourceObjectID = sAggPropValID;
                            _lrrFormula.SourcePropertyID = sAggPropID;
                            _lrrFormula.ModeID = ModeId;
                            _lrrFormula.IsFormula = expObj.IsFormula;
                            FormulaExpression objFormulaExpression = new FormulaExpression(formulaParams);
                            if (_lrrFormula.IsFormula)
                            {
                                _lrrFormula.Expression = expObj.FormulaExpression;
                                objAggPropVal.PropertyValue = objFormulaExpression.EvaluateFormula(expObj.FormulaExpression, _lrrFormula.Params, _lrrFormula);
                            }
                            else
                                _lrrFormula.Expression = objFormulaExpression.EvaluateFormula(expObj.FormulaExpression, _lrrFormula.Params, _lrrFormula);   
                        }
                    }
                }
            }
        }

        private void EvaluteFormulaForGroupProp(LRRAggregateProperty objAggProperty, ArrayList Formulas)
        {
            for (int i = 0; i < Formulas.Count; i++)
            {
                FormulaExpressionObjects expObj = Formulas[i] as FormulaExpressionObjects;
                LRRFormulaParams formulaParams = expObj.Params;
                string aggGroupPropValueID = expObj.AggregateGroupPropValueID;
                string aggGroupPropID = expObj.AggregateGroupPropID;

                LRRAggregateGroupedPropertyValue aggGrpPropValue = LRRModel.AggregateGroupedPropertyValuesObj.GetAggregateGroupedPropertyValueFromID(aggGroupPropValueID);
                LRRFormulaParam fParam = formulaParams[0] as LRRFormulaParam;
                string sPropExpression = expObj.FormulaExpression;
                LRRFormula _lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(fParam.FormulaID);
                if (_lrrFormula != null)
                {
                    if (_lrrFormula.Params.Count > 0)
                    {
                        if (aggGrpPropValue != null)
                        {
                            aggGrpPropValue.FormulaID = _lrrFormula.ID;
                            _lrrFormula.SourceCollectionName = aggGrpPropValue.TypeOfObject.ToString();
                            LRRFormulaDataStore.SourceCollectionName = aggGrpPropValue.TypeOfObject.ToString();
                            LRRFormulaDataStore.SourceObjectID = aggGrpPropValue.AggregateGroupedPropertyValueID;
                            LRRFormulaDataStore.SourcePropertyID = aggGroupPropID;
                            LRRFormulaDataStore.ModeID = fParam.ModeID;
                            LRRFormulaDataStore.SourceLookupPropertyID = aggGrpPropValue.LookUpPropertiesID;
                            LRRFormulaDataStore.SourceLookupQualifierID = aggGrpPropValue.LookUpQualifierID;
                            LRRFormulaDataStore.ObjectSource = typeof(LRRAggregateGroupedPropertyValue).ToString();
                            _lrrFormula.SourceObjectID = aggGrpPropValue.AggregateGroupedPropertyValueID;
                            _lrrFormula.SourcePropertyID = LRRFormulaDataStore.SourcePropertyID;
                            _lrrFormula.ModeID = aggGrpPropValue.ModeID;
                            _lrrFormula.IsFormula = expObj.IsFormula;
                            FormulaExpression objFormulaExpression = new FormulaExpression(formulaParams);
                            if (_lrrFormula.IsFormula)
                            {
                                _lrrFormula.Expression = expObj.FormulaExpression;
                                aggGrpPropValue.PropertyValue = objFormulaExpression.EvaluateFormula(expObj.FormulaExpression, _lrrFormula.Params, _lrrFormula);
                            }
                            else
                            {
                                _lrrFormula.Expression = objFormulaExpression.EvaluateFormula(expObj.FormulaExpression, _lrrFormula.Params, _lrrFormula);
                            }
                        }
                    }
                }
            }
        }

        #endregion

        private Hashtable GenerateGroupedFormulaForProperty(LRRAggregateProperty objAggregateProperty, string sModeID, string sModeName, string sType, bool bOverwriteExisting, LRRFormulaParams formulaParamsToDelete, int nParam)
        {
            string sPropName = "";
            string sMasterProperty = "";
            string sFormulaExpression = "";
            Hashtable formula = new Hashtable();

            LRRAggregateGroupedPropertyValue objAggPropVal = null;
            LRRAggregateGroupedProperties objAggGroupedProperties = LRRModel.AggregateGroupedPropertiesObj.GetAggregateGroupPropFromSiteCompPropID(objAggregateProperty.ID);

            foreach (LRRAggregateGroupedProperty objAggGrpProp in objAggGroupedProperties)
            {

                sPropName = objAggregateProperty.Name == null ? "" : objAggregateProperty.Name.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                string sAttribName = objAggGrpProp.AttributeName == null ? "" : objAggGrpProp.AttributeName.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                sPropName = sPropName + "[" + sAttribName + "]";

                sFormulaExpression = _genericFormulas[0].Expression == null ? "" : _genericFormulas[0].Expression.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                sMasterProperty = objAggregateProperty.MasterPropertyID == null ? "" : objAggregateProperty.MasterPropertyID.ToString(Symcon.BusinessObjects.SymConverter.Culture);

                LRRLookUpQualifiers objLkupQuals = LRRModel.LookUpQualifiers.getLookUpQualifiersForPropType(sType);

                string sGrpPropID = objAggGrpProp.AggregateGroupedPropertyID;

                if (objLkupQuals.Count == 0) //case of data array
                {
                    sPropName = sPropName + "[DataArray]";
                    objAggPropVal = LRRModel.AggregateGroupedPropertyValuesObj.GetGroupedValuesFromGroupPropID(sGrpPropID).GetAggregateGroupedPropertyValueObjectForMode(sModeID);
                    Hashtable formulaHash = new Hashtable();
                    formulaHash = CreateGroupedFormula(sGrpPropID, sPropName, sMasterProperty, sFormulaExpression, objAggPropVal, null, null, sModeID, nParam, bOverwriteExisting, formulaParamsToDelete);
                    formula.Add(objAggGrpProp.AggregateGroupedPropertyID, formulaHash);
                }
                else
                {
                    sPropName = sPropName + "[Grouped]";
                    foreach (LRRLookUpQualifier objLkupQual in objLkupQuals)
                    {
                        string sLookupQualID = objLkupQual.LookUpQualifierID;
                        objAggPropVal = LRRModel.AggregateGroupedPropertyValuesObj.GetGroupedValuesFromGroupPropID(sGrpPropID).GetGroupedValuesFromLkupQualID(sLookupQualID).GetAggregateGroupedPropertyValueObjectForMode(sModeID);
                        CreateGroupedFormula(sGrpPropID, sPropName, sMasterProperty, sFormulaExpression, objAggPropVal, objLkupQual, objLkupQuals, sModeID, nParam, bOverwriteExisting, formulaParamsToDelete);
                    }
                }
            }
            return formula;
        }

        public Hashtable CreateGroupedFormula(string sGrpPropID, string sPropName, string sMasterProperty, string sFormulaExpression, LRRAggregateGroupedPropertyValue objAggPropVal, LRRLookUpQualifier objLkupQual, LRRLookUpQualifiers objLkupQuals, string sModeID, int nParam, bool bOverwriteExisting, LRRFormulaParams formulaParamsToDelete)
        {
            string sPropExpression = "";
            LRRFormulaParams _formulaParams = new LRRFormulaParams();
            LRRFormula _lrrFormula;

            if (objLkupQual != null)
            {
                sPropName = sPropName + "[" + LRRModel.Qualifiers.getQualifierObjectForQualifierID(objLkupQual.QualifierID).Name + "]";
            }

            if (objAggPropVal != null)
            {
                string sFormulaIDForAgg = objAggPropVal.FormulaID == null ? "" : objAggPropVal.FormulaID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                //Create Formula only if not exists
                if ((sFormulaIDForAgg != "") && (bOverwriteExisting))
                {
                    _lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(sFormulaIDForAgg);
                    if (_lrrFormula != null)
                    {
                        _lrrFormula.Expression = "";
                        LRRFormulaParams formulaParams = _lrrFormula.Params;
                        foreach (LRRFormulaParam objParam in formulaParams)
                        {
                            formulaParamsToDelete.Add(objParam);
                        }
                        foreach (LRRFormulaParam objParam in formulaParamsToDelete)
                        {
                            _lrrFormula.Params.Remove(objParam);
                        }
                    }
                }
                else
                    _lrrFormula = new LRRFormula();
            }
            else
                _lrrFormula = new LRRFormula();

            _lrrFormula.FormulaName = sPropName + "[A][" + LRRModel.Modes.GetModeByID(sModeID).ModeName + "]";
            _lrrFormula.FormulaDescription = "Auto Generated";
            _lrrFormula.IsFormula = false;
            AssignPriorityToFormula(_lrrFormula, sModeID);
            LRRModel.FormulasObj.Add(_lrrFormula);
            objAggPropVal.FormulaObj = _lrrFormula;
            _lrrFormula.PostCalc = true;
            IEnumerator SiteCompEnum = SiteComponents.Keys.GetEnumerator();

            while (SiteCompEnum.MoveNext())
            {
                string sComponentID = SiteCompEnum.Current.ToString();
                LRRSiteComponent objSiteComponent = SiteComponents[sComponentID] as LRRSiteComponent;
                LRRSiteComponentProperties objSiteComponentProps = LRRModel.SiteComponentPropertiesObj.GetSiteComponentPropertiesByComponentID(sComponentID);

                if ((objSiteComponent != null) && (objSiteComponentProps.Count > 0))
                {
                    LRRSiteComponentProperty objSiteCompProp = objSiteComponentProps.GetSiteCompPropFromMasterPropertyID(sComponentID, sMasterProperty);
                    if (objSiteCompProp != null)
                    {
                        LRRSiteComponentGroupProperties objSiteGroupedProperties = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromSiteCompPropID(objSiteCompProp.ID);
                        foreach (LRRSiteComponentGroupProperty objSiteGrpProp in objSiteGroupedProperties)
                        {
                            if (objSiteGrpProp.LookUpPropertiesID == LRRModel.AggregateGroupedPropertiesObj.GetAggregateGroupPropFromID(sGrpPropID).LookUpPropertiesID)
                            {
                                LRRSiteGroupPropertyQualifierValues objPropVals = new LRRSiteGroupPropertyQualifierValues();
                                List<IPropertyValue> lstVals = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValuesFromGroupPropID(objSiteGrpProp.GroupPropertyID);

                                foreach (IPropertyValue val in lstVals)
                                {
                                    objPropVals.Add(val);
                                }
                                //LRRSiteGroupPropertyQualifierValues objPropVals = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValuesFromGroupPropID(objSiteGrpProp.GroupPropertyID);
                                if (objLkupQuals == null) //case of data array
                                {
                                    LRRSiteGroupPropertyQualifierValue objPropVal = objPropVals.GetQualifierValueObjectForPropertyMode(sModeID);
                                    if (objPropVal != null)
                                    {
                                        CreateGroupedFormulaParams(objSiteGrpProp, objPropVal, _formulaParams, _lrrFormula, sModeID, nParam, sFormulaExpression);
                                        nParam = nParam + 1;
                                    }
                                }
                                else
                                {
                                    LRRSiteGroupPropertyQualifierValue objPropVal = objPropVals.GetGroupQualifierValuesFromLkupQualID(objLkupQual.LookUpQualifierID).GetGroupQualifierValuesFromLkupPropIDAndGrpPropID(objSiteGrpProp.LookUpPropertiesID, objSiteGrpProp.GroupPropertyID).GetQualifierValueObjectForPropertyMode(sModeID);
                                    if (objPropVal != null)
                                    {
                                        CreateGroupedFormulaParams(objSiteGrpProp, objPropVal, _formulaParams, _lrrFormula, sModeID, nParam, sFormulaExpression);
                                        nParam = nParam + 1;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            if (sFormulaExpression != "WEIGHTED-AVERAGE")
            {
                if (_formulaParams.Count > 0)
                {
                    _lrrFormula.Params = _formulaParams;
                    FormulaParamsForGrpProp = new Hashtable();
                    FormulaParamsForGrpProp.Add(objAggPropVal.AggregateGroupedPropertyValueID, _formulaParams);
                }
                return FormulaParamsForGrpProp;
            }
            else
            {
                if (_formulaParams.Count > 0)
                {
                    _lrrFormula.Params = _formulaParams;
                    FormulaParamsForGrpProp = new Hashtable();
                    FormulaParamsForGrpProp.Add(objAggPropVal.AggregateGroupedPropertyValueID, _formulaParams);
                }
                return FormulaParamsForGrpProp;
            }
        }
        public void CreateGroupedFormulaParams(LRRSiteComponentGroupProperty objSiteGrpProp, LRRSiteGroupPropertyQualifierValue objPropVal, LRRFormulaParams _formulaParams, LRRFormula _lrrFormula, string sModeID, int nParam, string sFormulaExpression)
        {
            if (objPropVal != null)
            {
                LRRFormulaParam _formulaParam = new LRRFormulaParam();
                _formulaParam.FormulaID = _lrrFormula.ID;
                _formulaParam.ParamName = Convert.ToChar(nParam, Symcon.BusinessObjects.SymConverter.Culture).ToString(Symcon.BusinessObjects.SymConverter.Culture);
                _formulaParam.ParamPath = "Aggregates\\" + objSiteGrpProp.GroupPropertyID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                _formulaParam.SourceOBJID = objSiteGrpProp.GroupPropertyID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                _formulaParam.SourceCollectioName = objPropVal.TypeOfObject.ToString();
                _formulaParam.CurrentValue = objPropVal.PropertyValue;
                _formulaParam.PropertyName = objSiteGrpProp.AttributeName;
                _formulaParam.SourceLookUpPropertyID = objPropVal.LookUpPropertiesID;
                _formulaParam.SourceLookUpQualifierID = objPropVal.LookUpQualifierID;
                _formulaParam.ModeID = sModeID;
                _formulaParam.ModeName = LRRModel.Modes.GetModeByID(sModeID).ModeName;
                _formulaParam.UOM = objSiteGrpProp.UOM;
                _formulaParams.Add(_formulaParam);
            }
        }

        public LRRFormulaParams CreateAggregatesFormulaExpression(string sAggregateID, string sAggregatePropID, string sFormulaExpression, bool bOverwriteExisting)
        {
            int nParam = 65;
            string sPropName = "";
            string sSiteComponentIDs = "";
            string sMasterProperty = "";
            string sAggProperty = "";
            string sModeName = "";
            LRRFormulaParams formulaParamsToDelete = new LRRFormulaParams();
            Hashtable EvaluateFormulas = new Hashtable();
            Hashtable EvaluatrFormulasForGrp = new Hashtable();
            ArrayList FormulaParamsCollection = new ArrayList();


            LRRAggregate objAggregate = LRRModel.AggregatesObj.GetAggregateFromID(sAggregateID);
            LRRAggregates objAggregates = LRRModel.AggregatesObj;


            sSiteComponentIDs = GetSiteComponentsForAggregates(objAggregate, objAggregates);

            if (sSiteComponentIDs != "")
            {

                LRRAggregateProperty objAggregateProperty = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(sAggregatePropID);

                _genericFormulas.ItemFilter = "FormulaID = '" + sFormulaExpression + "'";

                if (_genericFormulas.Count != 0)
                {

                    sFormulaExpression = _genericFormulas[0].Expression == null ? "" : _genericFormulas[0].Expression.ToString(Symcon.BusinessObjects.SymConverter.Culture);

                    sAggProperty = objAggregateProperty.ID == null ? "" : objAggregateProperty.ID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                    sMasterProperty = objAggregateProperty.MasterPropertyID == null ? "" : objAggregateProperty.MasterPropertyID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                    foreach (LRRMode lrrMode in _lrrModes)
                    {
                        nParam = 65;
                        sModeName = lrrMode.ModeName == null ? "" : lrrMode.ModeName.ToString(Symcon.BusinessObjects.SymConverter.Culture);

                        if (lrrMode.IsDefault)
                            continue;

                        string sType = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(sMasterProperty).Type;
                        if (sType == "1")
                        {
                            EvaluateFormulas = GenerateSimpleFormulaForProperty(objAggregateProperty, lrrMode.ModeId, sModeName, bOverwriteExisting, formulaParamsToDelete, nParam);
                        }
                        else
                        {
                            Hashtable formulaParmForMode = GenerateGroupedFormulaForProperty(objAggregateProperty, lrrMode.ModeId, sModeName, sType, bOverwriteExisting, formulaParamsToDelete, nParam);
                            EvaluatrFormulasForGrp.Add(lrrMode.ModeId, formulaParmForMode);
                        }
                    }


                    if (EvaluateFormulas.Count > 0)
                    {
                        #region Evaluate Formula for SimpleProp
                        if (sFormulaExpression != "WEIGHTED-AVERAGE")
                        {
                            if (EvaluateFormulas.Count > 0)
                            {
                                EvaluteFormulaForSimpleProp(objAggregateProperty, EvaluateFormulas);
                                FormulaExp.Clear();
                                FormulaParamsForSimpleProp.Clear();
                            }
                        }
                        else
                        {
                            FormulaExpression objFormulaExpression = new FormulaExpression(EvaluateFormulas);
                            FormulaExp = getFormulaString(objFormulaExpression, sFormulaExpression, EvaluateFormulas, objAggregateProperty);
                            EvaluteFormulaForSimpleProp(objAggregateProperty, FormulaExp);
                            propValExists = true;
                            FormulaExp.Clear();
                            FormulaParamsForSimpleProp.Clear();
                        }
                        #endregion
                    }
                    else if (EvaluatrFormulasForGrp.Count > 0)
                    {
                        #region Evaluate For GroupProp
                        ArrayList FExperssion = getFormulaString(EvaluatrFormulasForGrp, sFormulaExpression);
                        EvaluteFormulaForGroupProp(objAggregateProperty, FExperssion);
                        #endregion
                    }
                    else
                    {
                        //LRRMessagBox.ShowMessage("SID_PROP_VAL_NOT_EXISTS", "SID_LABLE", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        HasFormulaFailed = true;
                        propValExists = false;
                        return formulaParamsToDelete;
                    }
                }
            }
            SiteComponents.Clear();
            return formulaParamsToDelete;
        }


        public string GetSiteComponentsForAggregates(LRRAggregate objAggregate, LRRAggregates objAggregates)
        {
            string sSiteComponentIDs = "";
            string sSiteComponentID = "";
            string sAggregateID = "";

            //bool bAggregatable = objAggregate.IsAggregateOfAggregates;
            LRRAggregateEquipments objAggregateEquipments = LRRModel.AggregateEquipmentsObj.GetAggregateEquipsFromParentAggregateID(objAggregate.AggregateID);
            if (objAggregateEquipments.Count > 0)
            {
                foreach (LRRAggregateEquipment objAggregateEquipment in objAggregateEquipments)
                {
                    sSiteComponentID = "";
                    sAggregateID = "";
                    sAggregateID = objAggregateEquipment.ChildAggregateID == null ? "" : objAggregateEquipment.ChildAggregateID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                    if (sAggregateID != "")
                    {
                        //For this aggregate, get all the Child Aggregates
                        LRRAggregate objAgg = objAggregates.GetAggregateFromID(sAggregateID);

                        sSiteComponentID = GetSiteComponentsForAggregates(objAgg, objAggregates);

                        if (sSiteComponentIDs == "")
                            sSiteComponentIDs = "'" + sSiteComponentID + "'";
                        else
                            sSiteComponentIDs = sSiteComponentIDs + ",'" + sSiteComponentID + "'";

                        objAggregates.ItemFilter = "";
                    }
                    else
                    {
                        //For this aggregate, get all the equipment
                        sSiteComponentID = objAggregateEquipment.EquipmentID == null ? "" : objAggregateEquipment.EquipmentID.ToString(Symcon.BusinessObjects.SymConverter.Culture);

                        if (sSiteComponentIDs == "")
                            sSiteComponentIDs = "'" + sSiteComponentID + "'";
                        else
                            sSiteComponentIDs = sSiteComponentIDs + ",'" + sSiteComponentID + "'";


                        if (!SiteComponents.Contains(sSiteComponentID))
                            SiteComponents.Add(sSiteComponentID, LRRModel.SiteComponentsObj.GetSiteComponentByID(sSiteComponentID));
                    }
                }
            }
            else
            {
                LRRFormulaParams fParamsToRemove = new LRRFormulaParams(); 
                foreach (LRRAggregateProperty aggregateProp in objAggregate.AggregateProperties)
                {
                    foreach (LRRAggregatePropertyValue aggPropValue in aggregateProp.Values)
                    {
                        if (!string.IsNullOrEmpty(aggPropValue.FormulaID))
                        {
                            LRRFormula formulaObj = LRRModel.FormulasObj.GetFormulaObjFromID(aggPropValue.FormulaID);
                            if (formulaObj != null)
                            {
                                foreach (LRRFormulaParam fParam in formulaObj.Params)
                                {
                                    fParamsToRemove.Add(fParam);
                                }
                                foreach (LRRFormulaParam fParam in fParamsToRemove)
                                {
                                    formulaObj.Params.Remove(fParam);
                                }
                                _app.DBManagerObj.ExecuteBatchDelete(fParamsToRemove);
                                LRRModel.FormulasObj.Remove(formulaObj);
                            }
                        }
                    }
                }
            }
            return sSiteComponentIDs;
        }

        public Hashtable getFormulaString(FormulaExpression objFormulaExpression, string sFormulaExpression, Hashtable FormulaParamsForMode, LRRAggregateProperty aggProperty)
        {
            string sPropExpression = "";
            Hashtable FormulaExpression = new Hashtable();

            switch (sFormulaExpression)
            {
                case "SUM":
                    FormulaExpression = objFormulaExpression.SUM(FormulaParamsForMode);
                    break;

                case "AVG":
                    FormulaExpression = objFormulaExpression.AVERAGE(FormulaParamsForMode);
                    break;

                case "COUNT":
                    FormulaExpression = objFormulaExpression.COUNT(FormulaParamsForMode,aggProperty.MasterPropertyID);
                    break;

                case "MAX":
                    FormulaExpression = objFormulaExpression.MAX(FormulaParamsForMode);
                    break;

                case "MIN":
                    FormulaExpression = objFormulaExpression.MIN(FormulaParamsForMode);
                    break;

                case "WEIGHTED-AVERAGE":
                    FormulaExpression = objFormulaExpression.WEIGHTEDAVERAGE(FormulaParamsForMode, aggProperty);
                    break;

            }

            return FormulaExpression;
        }

        public ArrayList getFormulaString(Hashtable FormulaParams, string sFormulaExpression)
        {
            string sPropExpression = "";
            ArrayList FormulaExpressions = new ArrayList();
            IEnumerator iEnum = FormulaParams.GetEnumerator();
            while (iEnum.MoveNext())
            {
                DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
                Hashtable formulaParamsforMode = dEntry.Value as Hashtable;
                IEnumerator iEnumFparma = formulaParamsforMode.GetEnumerator();
                while (iEnumFparma.MoveNext())
                {
                    DictionaryEntry dEntryfParam = (DictionaryEntry)iEnumFparma.Current;
                    string aggGrpPropID = dEntryfParam.Key.ToString();
                    Hashtable fParamHash = dEntryfParam.Value as Hashtable;
                    IEnumerator iEnumInternal = fParamHash.GetEnumerator();
                    while (iEnumInternal.MoveNext())
                    {
                        DictionaryEntry dEntryInternal = (DictionaryEntry)iEnumInternal.Current;
                        string groupPropValueID = dEntryInternal.Key.ToString();
                        LRRFormulaParams formulaParams = dEntryInternal.Value as LRRFormulaParams;
                        FormulaExpression objFormulaExpression = new FormulaExpression(formulaParams);
                        FormulaExpressionObjects formulaExpObj = null;
                        switch (sFormulaExpression)
                        {
                            case "SUM":
                                formulaExpObj = objFormulaExpression.SUM(formulaParams);
                                formulaExpObj.AggregateGroupPropID = aggGrpPropID;
                                formulaExpObj.AggregateGroupPropValueID = groupPropValueID;
                                break;

                            case "AVG":
                                formulaExpObj = objFormulaExpression.AVERAGE(formulaParams);
                                formulaExpObj.AggregateGroupPropID = aggGrpPropID;
                                formulaExpObj.AggregateGroupPropValueID = groupPropValueID;
                                break;

                            case "COUNT":
                                formulaExpObj = objFormulaExpression.COUNT(formulaParams);
                                formulaExpObj.AggregateGroupPropID = aggGrpPropID;
                                formulaExpObj.AggregateGroupPropValueID = groupPropValueID;
                                break;

                            case "MAX":
                                formulaExpObj = objFormulaExpression.MAX(formulaParams);
                                formulaExpObj.AggregateGroupPropID = aggGrpPropID;
                                formulaExpObj.AggregateGroupPropValueID = groupPropValueID;
                                break;

                            case "MIN":
                                formulaExpObj = objFormulaExpression.MIN(formulaParams);
                                formulaExpObj.AggregateGroupPropID = aggGrpPropID;
                                formulaExpObj.AggregateGroupPropValueID = groupPropValueID;
                                break;
                            case "WEIGHTED-AVERAGE":
                                formulaExpObj = objFormulaExpression.WEIGHTEDAVERAGE(formulaParams, aggGrpPropID);
                                formulaExpObj.AggregateGroupPropID = aggGrpPropID;
                                formulaExpObj.AggregateGroupPropValueID = groupPropValueID;
                                break;

                        }
                        FormulaExpressions.Add(formulaExpObj);
                    }
                }
            }
            return FormulaExpressions;
        }


        #region Fields
        private Hashtable SiteComponentProperties = new Hashtable();
        private Hashtable SiteComponents = new Hashtable();
        private LRRFormulas _lrrFormulas = null;
        private LRRAggregatePropertyValues _objAggregatePropertyValues = null;
        private LRRGenericFormulas _genericFormulas = new LRRGenericFormulas();
        private LRRModes _lrrModes = null;
        public bool HasFormulaFailed = false;
        private LRRApp _app;
        private Hashtable FormulaParamsForSimpleProp = new Hashtable();
        private Hashtable FormulaParamsForGrpProp;
        private Hashtable FormulaExp = new Hashtable();
        #endregion
    }
}



