using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;

using Syncfusion.Windows.Forms.Grid;

using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.IOScreen;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.LightRidge.BusinessObjects.Utilities;
using Symcon.Utils.HeaderDispatcher; 
using Symcon.Utils.MessageLogger;
using System.Collections.Generic;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.Validator;

namespace Symcon.LightRidge.PEAdvisor.CommonViews
{
    /// <summary>
    /// Summary description for GridSavedViewController.
    /// </summary>
    public class GridSavedViewController : IUnifiedGridFactory	
	{
		public GridSavedViewController()
		{
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		}
		
		#region IUnifiedGridFactory Members
        /// <summary>
        /// Loads the panel.
        /// </summary>
        /// <param name="arrXCollection">The arr X collection.</param>
        /// <param name="arrYCollection">The arr Y collection.</param>
        /// <returns></returns>
		public Panel LoadPanel(ArrayList arrXCollection,ArrayList arrYCollection)
		{
			//Split according to master plant items
			int rowCount = 0;
			LRRSiteComponents siteComponents = new LRRSiteComponents();
            LRRAggregates aggregates = new LRRAggregates();
			object[,]  arrSplitComponents = SplitComponent(arrXCollection,arrYCollection,out rowCount);
			
			Panel pnlMain = new Panel();
			for(int i=0;i < rowCount;i++)
			{
				ArrayList arrXList = new ArrayList();
				ArrayList arrYList = new ArrayList();
				
				if(arrSplitComponents[i,0] != null)
				{
					arrXList = arrSplitComponents[i,0] as ArrayList;
				}
				if(arrSplitComponents[i,1] != null)
				{
					arrYList = arrSplitComponents[i,1] as ArrayList;
				}
				
				//Create Grid
				GridControl gridControlInstance = new GridControl();
				gridControlInstance.Dock = DockStyle.Fill;
				Panel pnlGrid = new Panel();
				pnlGrid.Dock = DockStyle.Top;
				pnlGrid.Controls.Add(gridControlInstance);
				
				//Add the Panel to panel Collection
				_pnlCollection.Add(pnlGrid);

				pnlMain.Controls.Add(pnlGrid);

				InitializeGrid(gridControlInstance);

				gridControlInstance.RowCount = arrYList.Count+ 2;
				gridControlInstance.ColCount = arrXList.Count * LRRModel.ProjectModes.Count + 2;

				//Row/Col Freeze
				gridControlInstance.Rows.FreezeRange(FREEZE_ROW_START,FREEZE_ROW_END);
				gridControlInstance.Cols.FreezeRange(FREEZE_COL_START,FREEZE_COL_END);

				GridControlManager gridControlManager = new GridControlManager();
				gridControlManager.RowList = arrYList;
				gridControlManager.ColList = arrXList;

				GridManagerHash.Add(gridControlInstance,gridControlManager);

				//Register the Binding Called Event
				gridControlManager.BindingManager.BindingCalled+=new Symcon.LightRidge.PEAdvisor.DataStore.LRRBindingManager.BindingCallDel(bindingManager_BindingCalled);

                gridControlManager.ForeColorChanged += new Symcon.LightRidge.PEAdvisor.CommonViews.GridControlManager.ForeColorChangesDel(gridManager_ForeColorChanged);
                gridControlManager.BackColorChanged += new Symcon.LightRidge.PEAdvisor.CommonViews.GridControlManager.BackColorChangesDel(gridManager_BackColorChanged);

				//register grid events
				registerGridEvents(gridControlInstance);

				gridControlInstance.ResetVolatileData();
				gridControlInstance.BeginUpdate();

                //Clear the Row/Col Headers
                ClearHeaders(gridControlManager);

				//Render x-axis
				CreateXAxis(gridControlInstance,gridControlManager,arrXList);

				//Render y-axis
				CreateYAxis(gridControlInstance,gridControlManager,arrYList);

				//Sort the Grid
				gridControlInstance.RaiseCellClick(PROPERTIES_COL,COMPONENT_ROW,new MouseEventArgs(MouseButtons.Left,1,1,1,1));
				InitializePropertyValues(gridControlInstance,gridControlManager);

				gridControlInstance.ResumeLayout();
				gridControlInstance.EndUpdate(true);
			}

			pnlMain.Dock = DockStyle.Fill;

            if (SearchQueryID != null)
            {
                LRRUnifiedViewSearchCrtiteria criteria = new LRRUnifiedViewSearchCrtiteria();
                _lrrApp.DBManagerObj.PopulateObject(criteria, typeof(LRRUnifiedViewSearchCrtiteria), " WHERE ID='" + SearchQueryID + "'");

                if (criteria != null)
                {
                    string userSettingsID = criteria.UserSettingsID;

                    if (userSettingsID != null)
                    {
                        LRRUserSetting userSettings = new LRRUserSetting();
                        _lrrApp.DBManagerObj.PopulateObject(userSettings, typeof(LRRUserSetting), " WHERE ID='" + userSettingsID + "'");
                        
                        if (userSettingsID != null)
                        {
                            _gridPersistance.SetBytesToUserSettingsXml(userSettings.UserSettings);
                            _gridPersistance.GridCollection = GetPanelCollection();
                            GridPersistance.LoadGridSettings(false, false);
                        }
                    }
                }
            }
            //Check forPersistance
            //if (SearchQueryID != null)
            //{
            //    if (ApplicationPersistanceUtil.HtUserQueries.ContainsKey(SearchQueryID))
            //    {
            //        GridPersistance.GctlSetting = ApplicationPersistanceUtil.HtUserQueries[SearchQueryID] as IOScreen.LRRGridControlSetting;
            //        GridPersistance.GridCollection = GetPanelCollection();
            //        GridPersistance.LoadGridSettings(false,false);
            //    }
            //}
			return pnlMain;
		}

        /// <summary>
        /// Gets the panel collection.
        /// </summary>
        /// <returns></returns>
		public ArrayList GetPanelCollection()
		{
			return _pnlCollection;
		}

        /// <summary>
        /// This method clears the row/col Headers collection
        /// </summary>
        /// <param name="gridManager">The grid manager.</param>
        private void ClearHeaders(GridControlManager gridManager)
        {
            gridManager.RowHeader.Clear();
            gridManager.ColHeader.Clear();
        }

        /// <summary>
        /// Saves the Query
        /// </summary>
		public void Save()
		{
            //Save the Property Values & QualifierValues
            LRRSiteComponentPropertyValues toSaveSiteCompPropValues = LRRModel.PropertyValuesObj.GetChanges();
            //LRRAggregatePropertyValues toSaveAggrPropValues = LRRModel.AggregatePropertyValuesObj.GetChanges();
            LRRSiteGroupPropertyQualifierValues toSaveSiteGrpQualifierValues = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();
            // toSaveOverrides added by ashwin 
            LRROverrides toSaveOverrides = LRRModel.ChildModel.Overrides.GetChanges();
            LRRGroupOverrides toSaveGroupOverrides = LRRModel.ChildModel.GroupOverrides.GetChanges();
            if (toSaveSiteCompPropValues.Count > 0 || toSaveSiteGrpQualifierValues.Count > 0)
            {
                //DialogResult dlgResult = MessageBox.Show(this,"Do you want to save the property values?","Question",MessageBoxButtons.YesNo,MessageBoxIcon.Question); 
                if (toSaveSiteCompPropValues.Count > 0)
                {
                    _lrrApp.DBManagerObj.CommitCollection(toSaveSiteCompPropValues);
                    LRRModel.PropertyValuesObj.RefreshCollection();

                }
                if (toSaveSiteGrpQualifierValues.Count > 0)
                {
                    _lrrApp.DBManagerObj.CommitCollection(toSaveSiteGrpQualifierValues);
                    LRRModel.SiteGroupPropertyQualifierValuesObj.RefreshCollection();
                }
                // saving of collection toSaveOverrides and toSaveGroupOverrides added by ashwin
                if (toSaveOverrides.Count > 0)
                {
                    _lrrApp.DBManagerObj.CommitCollection(toSaveOverrides);
                    LRRModel.ChildModel.Overrides.RefreshCollection();
                }
                if (toSaveGroupOverrides.Count > 0)
                {
                    _lrrApp.DBManagerObj.CommitCollection(toSaveGroupOverrides);
                    LRRModel.ChildModel.GroupOverrides.RefreshCollection();
                }
                /*
                foreach (LRROverride overd in RollBackPVOverrides)
                {
                    if (LRRModel.ChildModel.Overrides.Contains(overd))
                    {
                        LRRModel.ChildModel.Overrides.Remove(overd);
                    }
                }
                **/
                //LRRMessagBox.ShowMessage("SID_PROP_VALUE_SAVE", true);
                LRRMessagBox.ShowMessage("SID_SAVEDINFO", true);
            }
            else
            {
                LRRMessagBox.ShowMessage("SID_NODATASAVED", true);
            }
		}


        /// <summary>
        /// Gets or sets the search query.
        /// </summary>
        /// <value>The search query.</value>
		public UnifiedSearchCriteria  SearchQuery 
		{
			get{return _searchCriteria;}
			set
			{
				_searchCriteria = value;
			}
		}

        /// <summary>
        /// Gets or sets the search query ID.
        /// </summary>
        /// <value>The search query ID.</value>
        public string SearchQueryID
        {
            get { return _searchCriteriaID; }
            set
            {
                _searchCriteriaID = value;
            }
        }

		/// <summary>
		/// This method clears the bindings
		/// </summary>
		public void ClearAllBindings()
		{
			IEnumerator iEnum = GridManagerHash.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
				GridControlManager gridControlManager =  dEntry.Value as GridControlManager;
				ClearBindings(gridControlManager);
			}
		}

		/// <summary>
		/// Method changes the UOM Set
		/// </summary>
		/// <param name="measSysID"></param>
		public void ChangeUOMSet(string measSysID)
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = (GridControlManager)dEntry.Value;
				if(gridControl.CurrentCell.IsActive)
				{
					for(int iRow = DATA_ROW_START; iRow <= gridControl.RowCount;iRow ++)
					{
						try
						{
							//Get the Row Data from Grid Data Manager
							ArrayList     rowSortList = gridControlManager.RowHeader;
							//Get the row data
							CellHeader rowData = rowSortList[iRow - DATA_ROW_START] as CellHeader;
						
							string uomTypeID = rowData.UOMTypeID;
							string uiUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(measSysID,uomTypeID);
							rowData.UIUOMId = uiUOMID;

							//Add to UI UOM hash
							if(gridControlManager.UIUOMHash.ContainsKey(rowData.ID))
							{
								gridControlManager.UIUOMHash[rowData.ID] = uiUOMID;
							}
							else
							{
								gridControlManager.UIUOMHash.Add(rowData.ID,uiUOMID);
							}
						
							string uomDisplayName = LRRModel.ChildModel.GetUOMNameFromID(uiUOMID);
							gridControl.Model[iRow,UOM_COL].Text = uomDisplayName;
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridSimpleViewController),"Exception in changing UOMSet:- " + ex.Message,Logger.Level.Debug);
						}
					}
					InitializePropertyValues(gridControl,gridControlManager);
					break;
				}
			}
		}
		
		/// <summary>
		/// Method Unsubscribes the Reconnect Completed Event
		/// </summary>
		public void UnSubscribeReconnectCompleted()
		{
			this._lrrApp.ReconnectCompleted-=new EventHandler(_lrrApp_ReconnectCompleted);
		}
		/// <summary>
		/// Method clears the sorting
		/// </summary>
		public void ClearSort()
		{
			IOScreenUtilities.ClearSort(GridManagerHash);
		}
		
		#endregion
	
		#region Methods

        /// <summary>
        /// Splits the component.
        /// </summary>
        /// <param name="arrXCollection">The arr X collection.</param>
        /// <param name="arrYCollection">The arr Y collection.</param>
        /// <param name="rowCount">The row count.</param>
        /// <returns></returns>
		private object[,] SplitComponent(ArrayList arrXCollection,ArrayList arrYCollection, out int rowCount)
		{
			Hashtable htMasterComps = new Hashtable();
			Hashtable htProps = new Hashtable();

			rowCount = 0;
			
			//Split X-Collection
			for(int i=0;i < arrXCollection.Count;i++)
			{
				SearchCriteria criteria = arrXCollection[i] as SearchCriteria;
				ArrayList arrCondition = criteria.ArrListSearchCondition;
				for(int j=0;j < arrCondition.Count;j++)
				{
					SearchCondition condition = arrCondition[j] as SearchCondition;
                    if (criteria.ObjType == typeof(LRRSiteComponent))
                    {
                        LRRSiteComponent siteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(condition.Value);
                        string sMasterCompType = siteComponent.MasterComponentType;
                        if (!htMasterComps.ContainsKey(sMasterCompType))
                        {
                            ArrayList arrCriteria = new ArrayList();
                            arrCriteria.Add(criteria);
                            htMasterComps.Add(sMasterCompType, arrCriteria);
                        }
                        else
                        {
                            ArrayList arrCriteria = htMasterComps[sMasterCompType] as ArrayList;
                            arrCriteria.Add(criteria);
                        }
                    }
                    else if (criteria.ObjType == typeof(LRRAggregate))
                    {
                        LRRAggregate aggObj = LRRModel.AggregatesObj.GetAggregateFromID(condition.Value);
                        string sMasterCompType = aggObj.EquipmentTypeID;
                        if (!htMasterComps.ContainsKey(sMasterCompType))
                        {
                            ArrayList arrCriteria = new ArrayList();
                            arrCriteria.Add(criteria);
                            htMasterComps.Add(sMasterCompType, arrCriteria);
                        }
                        else
                        {
                            ArrayList arrCriteria = htMasterComps[sMasterCompType] as ArrayList;
                            arrCriteria.Add(criteria);
                        }
                    }
				}
			}

			//Split Y-Collection
			for(int i=0;i < arrYCollection.Count;i++)
			{
				SearchCriteria criteria = arrYCollection[i] as SearchCriteria;
				
				ArrayList arrCondition = criteria.ArrListSearchCondition;
				if(criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
				{
					SearchCondition condition = arrCondition[0] as SearchCondition;
					LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition.Value);
					string compID = null;
					if(masterProp != null)
					{
						compID = masterProp.ComponentID;
					}
					else
					{
						LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(condition.Value);
						LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
						compID = siteComp.MasterComponentType;
					}

					if(!htProps.ContainsKey(compID))
					{
						ArrayList arrCriteria = new ArrayList();
						arrCriteria.Add(criteria);
						htProps.Add(compID,arrCriteria);
					}
					else
					{
						ArrayList arrCriteria = htProps[compID] as ArrayList;
						arrCriteria.Add(criteria);
					}
				}
				if(criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
				{
					string masterPropID = null;
					for(int j=0; j < arrCondition.Count;j++)
					{
						SearchCondition condition = arrCondition[j] as SearchCondition;
						if(condition.Name == "MasterPropID")
							masterPropID = condition.Value;
					}
					LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(masterPropID);
					string compID = masterProp.ComponentID;
					if(!htProps.ContainsKey(compID))
					{
						ArrayList arrCriteria = new ArrayList();
						arrCriteria.Add(criteria);
						htProps.Add(compID,arrCriteria);
					}
					else
					{
						ArrayList arrCriteria = htProps[compID] as ArrayList;
						arrCriteria.Add(criteria);
					}
				}
                if (criteria.ObjType == typeof(LRRAggregatePropertyValue))
                {
                    SearchCondition condition = arrCondition[0] as SearchCondition;
                    LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition.Value);
                    string aggID = null;
                    if (masterProp != null)
                    {
                        aggID = masterProp.ComponentID;
                    }
                    else
                    {
                        LRRAggregateProperty aggProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(condition.Value);
                        LRRAggregate aggregate = LRRModel.AggregatesObj.GetAggregateFromID(aggProp.AggregateID);
                        aggID = aggregate.EquipmentTypeID;
                    }

                    if (!htProps.ContainsKey(aggID))
                    {
                        ArrayList arrCriteria = new ArrayList();
                        arrCriteria.Add(criteria);
                        htProps.Add(aggID, arrCriteria);
                    }
                    else
                    {
                        ArrayList arrCriteria = htProps[aggID] as ArrayList;
                        arrCriteria.Add(criteria);
                    }
                }											 
			}

			int nMasterComp = htMasterComps.Count;
			rowCount = nMasterComp;
			object[,] arrSplitComponent = new object[nMasterComp,2];
			
			int iCtr = 0;
			IDictionaryEnumerator iEnum = htMasterComps.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry) iEnum.Current;
				string compID = (string)dEntry.Key;
				ArrayList criteria = dEntry.Value as ArrayList;
				arrSplitComponent[iCtr,0] = criteria;

				ArrayList propCriteria = htProps[compID] as ArrayList;
				arrSplitComponent[iCtr,1] = propCriteria;

				iCtr++;
			}

			return arrSplitComponent;
		}


		/// <summary>
		/// Prints the selected grid
		/// </summary>
		public void Print()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					GridControlPrint ggPrint = new GridControlPrint();
					ggPrint.PrintData(gridControl,false);
					break;
				}
			}
		}

		/// <summary>
		/// Print Preview of the selected grid
		/// </summary>
		public void PrintPreview()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					GridControlPrint ggPrint = new GridControlPrint();
					ggPrint.PrintData(gridControl,true);
					break;
				}
			}
		}

		/// <summary>
		/// Exports the selected grid
		/// </summary>
		/// <param name="gridManagerHash"></param>
		/// <returns></returns>
		public GridControl Export()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					return gridControl;
				}
			}
			return null;
		}

		/// <summary>
		/// This method registers all the grid events
		/// </summary>
		/// <param name="gridControlInstance"></param>
		private void registerGridEvents(GridControl gridControlInstance)
		{
			gridControlInstance.CurrentCellCloseDropDown +=new Syncfusion.Windows.Forms.PopupClosedEventHandler(gridControlInstance_CurrentCellCloseDropDown);
			gridControlInstance.CurrentCellValidating +=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellValidating);
			gridControlInstance.CurrentCellStartEditing+=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellStartEditing);
			
			//add a sort header to the CellModels.
			gridControlInstance.CellModels.Add("ColumnHeaderCell", new GridSortColumnHeaderCellModel(gridControlInstance.Model));

			//catch the click
			gridControlInstance.CellClick +=new GridCellClickEventHandler(gridControlInstance_CellClick);

			//Event for row movement
			gridControlInstance.RowsMoving+=new GridRangeMovingEventHandler(gridControlInstance_RowsMoving);
			gridControlInstance.RowsMoved+=new GridRangeMovedEventHandler(gridControlInstance_RowsMoved);

			//Event for Col Movement
			gridControlInstance.ColsMoving+=new GridRangeMovingEventHandler(gridControlInstance_ColsMoving);

			//Event for preventing hidden columns from expanding
			//			gridControlInstance.ResizingColumns+=new GridResizingColumnsEventHandler(gridControlInstance_ResizingColumns);
			
			//Enabling/Disabling Left/Right Col Movement Buttons
			gridControlInstance.SelectionChanged+=new GridSelectionChangedEventHandler(gridControlInstance_SelectionChanged);
			
			//Event for Handling Key down
			gridControlInstance.KeyDown+=new KeyEventHandler(gridControlInstance_KeyDown);

			//Event for Painting Header text
			gridControlInstance.Paint+=new PaintEventHandler(gridControlInstance_Paint);

			//Event for after cell accepted changes
            gridControlInstance.CurrentCellAcceptedChanges += new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellAcceptedChanges);

            gridControlInstance.Model.ClipboardPaste += new Syncfusion.Windows.Forms.Grid.GridCutPasteEventHandler(Model_ClipboardPaste);


			//Event for Reconnect
			this._lrrApp.ReconnectCompleted+=new EventHandler(_lrrApp_ReconnectCompleted);


			//Context Menu for switching recording values
			ContextMenu contMenu = new ContextMenu();
			gridControlInstance.ContextMenu = contMenu;
			contMenu.Popup+=new EventHandler(contMenu_Popup);
		}

        void gridControlInstance_CurrentCellAcceptedChanges(object sender, System.ComponentModel.CancelEventArgs e)
        {
            GridControl grid = sender as GridControl;
            if (!(grid[UOM_ROW, grid.CurrentCell.ColIndex].Text == IOSchemes.UOM_HEADER_NAME))
            {
                if (grid.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.CommonViews.SavedViewForm))
                {
                    CommonViews.SavedViewForm.BPropertyValChanged = true;
                }
                else if (grid.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.CommonViews.UnifiedViewResultForm))
                {
                    UnifiedViewResultForm.BUnifiedResultFormChange = true;
                }
            }
        }

		/// <summary>
		/// Initializes the grid
		/// </summary>
		/// <param name="gridControlInstance"></param>
		public static void InitializeGrid(GridControl gridControlInstance)
		{
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.BackColor = Color.DarkBlue;
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.TextColor =  SystemColors.ActiveCaptionText;
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.CellType = "Static";
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.ReadOnly = true;

			gridControlInstance.BaseStylesMap["Row Header"].StyleInfo.BackColor = Color.FromArgb(192,192,192);

			gridControlInstance.Properties.RowHeaders = true;
			gridControlInstance.NumberedRowHeaders = false;
			gridControlInstance.AllowDragSelectedRows = true;
			gridControlInstance.BaseStylesMap["Row Header"].StyleInfo.Borders.Right = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.Solid, System.Drawing.SystemColors.WindowFrame);
			gridControlInstance.ColWidths.SetSize(0, 10);

			gridControlInstance.BaseStylesMap["Header"].StyleInfo.BackColor = Color.DarkBlue;
			gridControlInstance.ColStyles[1].CellType = "Static";
			gridControlInstance.RowStyles[1].CellType = "Static";
			gridControlInstance.RowStyles[2].CellType = "Static";
			gridControlInstance.ColStyles[1].ReadOnly = true;
			gridControlInstance.RowStyles[0].ReadOnly = true;
			gridControlInstance.RowStyles[1].ReadOnly = true;
			gridControlInstance.RowStyles[2].ReadOnly = true;;

			gridControlInstance.Model.Options.DefaultGridBorderStyle = GridBorderStyle.Solid;
			gridControlInstance.Model.Options.ExcelLikeSelectionFrame = false;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.WrapText = true;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.Trimming = StringTrimming.None;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.AutoSize = true;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.HorizontalAlignment = GridHorizontalAlignment.Center;
			gridControlInstance.Properties.BackgroundColor = Color.FromArgb(150,150,150);
			gridControlInstance.Properties.GridLineColor = Color.FromArgb(0,0,0);
			gridControlInstance.DefaultGridBorderStyle = GridBorderStyle.Solid;
			gridControlInstance.Properties.Buttons3D = false;
		}

        /// <summary>
        /// Creates the X axis.
        /// </summary>
        /// <param name="gridControl">The grid control.</param>
        /// <param name="gridControlManager">The grid control manager.</param>
        /// <param name="arrXCollection">The arr X collection.</param>
		private void CreateXAxis(GridControl gridControl,GridControlManager gridControlManager,ArrayList arrXCollection)
		{
            string equipmentTag = null;
            string plantItemTag = null;
            string plantLevelName = null;
            string strToolTIp = null;
			gridControl[ATTRIBUTE_ROW,ATTRIBUTE_COL].Text = "Attributes";
            gridControl.ColWidths.SetSize(ATTRIBUTE_COL, 300);
			gridControl[ATTRIBUTE_ROW,ATTRIBUTE_COL].BackColor = IOSchemes.ATTRIBUTES_BACK_COLOR;
			gridControl[ATTRIBUTE_ROW + 1,ATTRIBUTE_COL].BackColor = IOSchemes.ATTRIBUTES_BACK_COLOR;
			gridControl[UOM_ROW,UOM_COL].Text = "UOM";
			gridControl[UOM_ROW,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;
			gridControl[UOM_ROW + 1,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;

			int colIndex = COMPONENT_COL;
			int nModes = LRRModel.ProjectModes.Count;
            for (int i = 0; i < arrXCollection.Count; i++)
            {
                SearchCriteria criteria = arrXCollection[i] as SearchCriteria;
                ArrayList arrCondition = criteria.ArrListSearchCondition;
                SearchCondition condition = arrCondition[0] as SearchCondition;

                gridControl.CoveredRanges.Add(GridRangeInfo.Cells(COMPONENT_ROW, colIndex, COMPONENT_ROW, colIndex + nModes - 1));
                if (criteria.ObjType == typeof(LRRAggregate))
                {
                    LRRAggregate aggregate = LRRModel.AggregatesObj.GetAggregateFromID(condition.Value);
                    gridControl.Model[COMPONENT_ROW, colIndex].Text = "[Aggregate]" + aggregate.Name;
                    gridControl[COMPONENT_ROW, colIndex].BackColor = IOSchemes.COMPONENT_BACK_COLOR;
                    gridControl[COMPONENT_ROW, colIndex].TextAlign = GridTextAlign.Left;
                    gridControl[COMPONENT_ROW, colIndex].CellTipText = "[Aggregate]" + aggregate.Name;

                    LRRMasterComponent masterComp = LRRModel.Model.GetMasterComponentByID(aggregate.EquipmentTypeID);
                    //				gridControl.CoveredRanges.Add(GridRangeInfo.Cells(HEADER_ROW,HEADER_COL,HEADER_ROW,HEADER_COL + COL_SPAN - 1));
                    gridControl[HEADER_ROW, HEADER_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                    //				gridControl[HEADER_ROW,HEADER_COL].Text = masterComp.ComponentName;
                    gridControlManager.HeaderText = masterComp.ComponentName;

                    int nModesCol = colIndex;
                    foreach (LRRMode lrrMode in LRRModel.ProjectModes)
                    {
                        CellHeader colHeader = new CellHeader();
                        colHeader.SearchCriteriaObj = criteria;
                        colHeader.ModeID = lrrMode.ModeId;
                        colHeader.ID = aggregate.AggregateID;
                        gridControlManager.ColHeader.Add(colHeader);

                        gridControl[MODES_ROW, nModesCol].Text = lrrMode.ModeDisplayName;
                        gridControl[MODES_ROW, nModesCol].CellTipText = lrrMode.ModeName;
                        gridControl[MODES_ROW, nModesCol].BackColor = IOSchemes.GetModeBackColor(lrrMode.ModeId);
                        gridControl[MODES_ROW, nModesCol].TextColor = IOSchemes.GetModeTextColor(lrrMode.ModeId);
                        nModesCol++;
                    }

                    colIndex = nModesCol;
                }
                else
                {
                    LRRSiteComponent siteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(condition.Value);
                    if (siteComponent != null)
                    {
                        gridControl.Model[COMPONENT_ROW, colIndex].Text = siteComponent.Name;
                        gridControl[COMPONENT_ROW, colIndex].BackColor = IOSchemes.COMPONENT_BACK_COLOR;
                        gridControl[COMPONENT_ROW, colIndex].TextAlign = GridTextAlign.Left;

                        equipmentTag = siteComponent.EquipmentTag;
                        plantItemTag = siteComponent.Name;
                        LRRLevels plantLevelCol = new LRRLevels();
                        _lrrApp.DBManagerObj.PopulateCollection(plantLevelCol);
                        LRRLevel plantLevel = new LRRLevel();
                        plantLevel = plantLevelCol.getLevelByLevelID(siteComponent.PlantUnit);
                        if (plantLevel != null)
                        {
                            plantLevelName = plantLevel.Name;
                            strToolTIp = LRRHeaderText.GetHeaderString("SID_PLANTITEM") + ": " + plantItemTag
                                        + "\n" + LRRHeaderText.GetHeaderString("SID_EQUIPTAG") + ": " + equipmentTag
                                        + "\n" + LRRHeaderText.GetHeaderString("SID_LEVEL") + ": " + plantLevelName;

                            gridControl[COMPONENT_ROW, colIndex].CellTipText = strToolTIp;

                            LRRMasterComponent masterComp = LRRModel.Model.GetMasterComponentByID(siteComponent.MasterComponentType);
                            //				gridControl.CoveredRanges.Add(GridRangeInfo.Cells(HEADER_ROW,HEADER_COL,HEADER_ROW,HEADER_COL + COL_SPAN - 1));
                            gridControl[HEADER_ROW, HEADER_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                            //				gridControl[HEADER_ROW,HEADER_COL].Text = masterComp.ComponentName;
                            gridControlManager.HeaderText = masterComp.ComponentName;

                            int nModesCol = colIndex;
                            foreach (LRRMode lrrMode in LRRModel.ProjectModes)
                            {
                                CellHeader colHeader = new CellHeader();
                                colHeader.SearchCriteriaObj = criteria;
                                colHeader.ModeID = lrrMode.ModeId;
                                colHeader.ID = siteComponent.SiteComponentID;
                                gridControlManager.ColHeader.Add(colHeader);

                                gridControl[MODES_ROW, nModesCol].Text = lrrMode.ModeDisplayName;
                                gridControl[MODES_ROW, nModesCol].CellTipText = lrrMode.ModeName;
                                gridControl[MODES_ROW, nModesCol].BackColor = IOSchemes.GetModeBackColor(lrrMode.ModeId);
                                gridControl[MODES_ROW, nModesCol].TextColor = IOSchemes.GetModeTextColor(lrrMode.ModeId);
                                nModesCol++;
                            }


                            colIndex = nModesCol;
                        }
                    }
                    }
                
                }
            
		}

		private void _lrrApp_ReconnectCompleted(object sender, EventArgs e)
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				GridControl  gridControlInstance = (GridControl)iDictEnum.Key;
				GridControlManager  gridControlManager = (GridControlManager)iDictEnum.Value;
				InitializePropertyValues(gridControlInstance,gridControlManager);
			}
		}

        /// <summary>
        /// Creates the Y axis.
        /// </summary>
        /// <param name="gridControl">The grid control.</param>
        /// <param name="gridControlManager">The grid control manager.</param>
        /// <param name="arrYCollection">The arr Y collection.</param>
		private void CreateYAxis(GridControl gridControl,GridControlManager gridControlManager,ArrayList arrYCollection)
        {
            int rowIndex = PROPERTIES_ROW;
            for (int i = 0; i < arrYCollection.Count; i++)
            {
                SearchCriteria criteria = arrYCollection[i] as SearchCriteria;
                string rowType = criteria.ObjType.ToString();
                CellHeader rowHeader = new CellHeader();
                rowHeader.SearchCriteriaObj = criteria;
                gridControlManager.RowHeader.Add(rowHeader);

                string uomTypeID = null;
                string uiUOMID = null;

                ArrayList arrCondition = criteria.ArrListSearchCondition;
                if (criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
                {
                    SearchCondition condition = arrCondition[0] as SearchCondition;
                    LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition.Value);
                    if (masterProp != null)
                    {
                        string mpName = "[S] - " + masterProp.Name;                      
                        gridControl.Model[rowIndex, PROPERTIES_COL].WrapText = false;
                        gridControl.Model[rowIndex, PROPERTIES_COL].Text = mpName;
                        gridControl.Model[rowIndex, PROPERTIES_COL].CellTipText = mpName;
                        gridControl.Model[rowIndex, PROPERTIES_COL].TextAlign = GridTextAlign.Left;
                        uomTypeID = masterProp.MasterPropertyUOMTypeID;

                        string localeID = LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID;
                        string uiUomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localeID, uomTypeID);
                        rowHeader.UIUOMId = uiUomID;

                        // string newUIUOM = ChangedUIUOMDictionary.RetrieveUOM(this.GetType(), condition.Value);
                        //if (newUIUOM != null)
                        //    rowHeader.UIUOMId = newUIUOM;

                        rowHeader.Name = mpName;
                        rowHeader.ID = masterProp.PropID;
                    }
                    else
                    {
                        LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(condition.Value);
                        string spName = "[S] - " + siteCompProp.Name;
                        gridControl.Model[rowIndex, PROPERTIES_COL].Text = spName;
                        gridControl.Model[rowIndex, PROPERTIES_COL].CellTipText = spName;
                        gridControl.Model[rowIndex, PROPERTIES_COL].WrapText = false;
                        gridControl.Model[rowIndex, PROPERTIES_COL].TextAlign = GridTextAlign.Left;
                        uomTypeID = siteCompProp.UOMTypeID;
                        rowHeader.UIUOMId = siteCompProp.UIUomID;
                        rowHeader.Name = spName;
                        rowHeader.ID = siteCompProp.ID;
                    }
                    gridControl.Model[rowIndex, PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;

                }
                if (criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
                {
                    string lkUpPropertyID = null;
                    string lookUpQualifierID = null;
                    string masterPropID = null;
                    for (int j = 0; j < arrCondition.Count; j++)
                    {
                        SearchCondition condition = arrCondition[j] as SearchCondition;
                        if (condition.Name == "LookUpPropertiesID")
                            lkUpPropertyID = condition.Value;
                        if (condition.Name == "LookUpQualifierID")
                            lookUpQualifierID = condition.Value;
                        if (condition.Name == "MasterPropID")
                            masterPropID = condition.Value;
                    }
                    LRRLookUpProperty lkupProp = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(lkUpPropertyID);

                    string masterPropName = null;
                    string propTypeName = null;
                    LRRPropertyType propType = null;
                    LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(masterPropID);
                    if (masterProp != null)
                    {
                        masterPropName = masterProp.Name;
                        propType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(masterProp.Type);
                        propTypeName = propType.PropertyType;
                        uomTypeID = propType.UOMTypeID;

                        string localeID = LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID;
                        string uiUomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localeID, uomTypeID);
                        rowHeader.UIUOMId = uiUomID;
                    }
                    if (lookUpQualifierID != null)
                    {

                        LRRLookUpQualifier lrrLkUpQual = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
                        if (lrrLkUpQual != null)
                        {
                            LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);
                            string gpName = "[GP] - " + masterPropName + "/" + qual.Name + "/" + lkupProp.Name + "/" + propTypeName;
                            gridControl.Model[rowIndex, PROPERTIES_COL].Text = gpName;
                            gridControl.Model[rowIndex, PROPERTIES_COL].WrapText = false;

                            gridControl.Model[rowIndex, PROPERTIES_COL].CellTipText = gpName;
                            gridControl.Model[rowIndex, PROPERTIES_COL].TextAlign = GridTextAlign.Left;
                            gridControl.Model[rowIndex, PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                            uomTypeID = lrrLkUpQual.UOMTypeID;
                            rowHeader.Name = gpName;
                            rowHeader.UIUOMId = lrrLkUpQual.UIUomID;
                            rowHeader.ID = qual.QualifierID + "|" + lkupProp.LookUpPropertiesID + "|" + masterPropID;
                        }
                    }
                    else
                    {
                        string daName = "[DA] - " + masterPropName + "/" + lkupProp.Name + "/" + propTypeName;
                        gridControl.Model[rowIndex, PROPERTIES_COL].Text = daName;
                        gridControl.Model[rowIndex, PROPERTIES_COL].WrapText = false;                 
                        gridControl.Model[rowIndex, PROPERTIES_COL].CellTipText = daName;
                        gridControl.Model[rowIndex, PROPERTIES_COL].TextAlign = GridTextAlign.Left;
                        gridControl.Model[rowIndex, PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                        rowHeader.Name = daName;
                        rowHeader.ID = masterPropID + "|" + lkupProp.LookUpPropertiesID;
                        uomTypeID = propType.UOMTypeID;
                    }
                }
					 
                rowHeader.UOMTypeID = uomTypeID;
                if (SearchQueryID != null && SearchQueryID != "0")
                {
                    LRRSearchCriteriaUserSpecificUIUOM uiUOMObj = null;
                    LRRUnifiedViewSearchItemConditions srchConditions = new LRRUnifiedViewSearchItemConditions();
                    this._lrrApp.DBManagerObj.PopulateCollection(srchConditions, " where SearchCriteriaID = '" + SearchQueryID + "'");

                    if (srchConditions != null && srchConditions.Count != 0)
                    {
                        string lkUpPropertyID = null;
                        string lookUpQualifierID = null;
                        string masterPropID = null;
                        string siteComponentPropertyID = null;
                        string rowHeaderID = null;

                        string searchCriteriaID = srchConditions[0].SearchCriteriaID;
                        LRRUnifiedViewSearchItems uvSearchItems = new LRRUnifiedViewSearchItems();
                        LRRUnifiedViewSearchItemConditions uvSearchItemConditionsFromItemID = new LRRUnifiedViewSearchItemConditions();

                        _lrrApp.DBManagerObj.PopulateCollection(uvSearchItems, " where SearchCriteriaID = '" + searchCriteriaID + "' and Type = '" + rowType + "'");
                        foreach (LRRUnifiedViewSearchItem uvSearchItem in uvSearchItems)
                        {
                            uvSearchItemConditionsFromItemID = srchConditions.GetItemConditionsByItemID(uvSearchItem.ID);
                            if (uvSearchItem.Type == typeof(LRRSiteGroupPropertyQualifierValue).ToString())
                            {
                                foreach (LRRUnifiedViewSearchItemCondition searchItemCondition in uvSearchItemConditionsFromItemID)
                                {
                                    if (searchItemCondition.Name == "LookUpPropertiesID")
                                        lkUpPropertyID = searchItemCondition.ConditionValue;
                                    if (searchItemCondition.Name == "LookUpQualifierID")
                                        lookUpQualifierID = searchItemCondition.ConditionValue;
                                    if (searchItemCondition.Name == "MasterPropID")
                                        masterPropID = searchItemCondition.ConditionValue;
                                }
                                LRRLookUpProperty lkupProp = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(lkUpPropertyID);


                                if (lookUpQualifierID != null && masterPropID != null && lkUpPropertyID != null)
                                {
                                    LRRLookUpQualifier lrrLkUpQual = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
                                    if (lrrLkUpQual != null)
                                    {
                                        LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);
                                        rowHeaderID = qual.QualifierID + "|" + lkupProp.LookUpPropertiesID + "|" + masterPropID;
                                    }
                                }
                                else if (masterPropID != null && lkUpPropertyID != null)
                                {
                                    rowHeaderID = masterPropID + "|" + lkupProp.LookUpPropertiesID;
                                }
                                if (rowHeader.ID == rowHeaderID)
                                {
                                    LRRSearchCriteriaUserSpecificUIUOMs lrrSrchCriteriaItemGrpPropUOMS = new LRRSearchCriteriaUserSpecificUIUOMs();
                                    _lrrApp.DBManagerObj.PopulateCollection(lrrSrchCriteriaItemGrpPropUOMS, " WHERE PropertyID = '" + uvSearchItem.ID + "'");
                                    foreach (LRRSearchCriteriaUserSpecificUIUOM srchCriteriaUserSpecificUiUOM in lrrSrchCriteriaItemGrpPropUOMS)
                                    {
                                        uiUOMObj = srchCriteriaUserSpecificUiUOM;
                                        break;
                                    }
                                }
                                lkUpPropertyID = null;
                                lookUpQualifierID = null;
                                masterPropID = null;
                            }
                            else if (uvSearchItem.Type == typeof(LRRSiteComponentPropertyValue).ToString())
                            {
                                foreach (LRRUnifiedViewSearchItemCondition searchItemCondition in uvSearchItemConditionsFromItemID)
                                {
                                    if (searchItemCondition.Name == "SiteComponentPropertyID")
                                        siteComponentPropertyID = searchItemCondition.ConditionValue;
                                }
                                if (siteComponentPropertyID != null)
                                {
                                    rowHeaderID = siteComponentPropertyID;
                                }
                                if (rowHeader.ID == rowHeaderID)
                                {
                                    LRRSearchCriteriaUserSpecificUIUOMs lrrSrchCriteriaItemGrpPropUOMS = new LRRSearchCriteriaUserSpecificUIUOMs();
                                    _lrrApp.DBManagerObj.PopulateCollection(lrrSrchCriteriaItemGrpPropUOMS, " WHERE PropertyID = '" + uvSearchItem.ID + "'");
                                    foreach (LRRSearchCriteriaUserSpecificUIUOM srchCriteriaUserSpecificUiUOM in lrrSrchCriteriaItemGrpPropUOMS)
                                    {
                                        uiUOMObj = srchCriteriaUserSpecificUiUOM;
                                        break;
                                    }
                                }
                                siteComponentPropertyID = null;
                            }
                        }
                        if (uiUOMObj != null)
                        {
                            rowHeader.UIUOMId  = uiUOMObj.UIUomID;
                        }
                    }
                }
 
                LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomTypeID);
                string uomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_lrrApp.AppLocaleID, uomTypeID);
                rowHeader.UOMId = uomID;
                //Get UOM name from UOMID
                string uomName = "";
                string uiUOMName = "";
                string uomDisplayName = "";
                uiUOMID = rowHeader.UIUOMId;
                CommonViews.ChangedUIUOMDictionary.AddUOM(typeof(GridSavedViewController), rowHeader.ID, uiUOMID);

                if (uomID != null && uomID.Trim() != "")
                {
                    uomName = LRRModel.ChildModel.GetUOMNameFromID(uomID);
                }
                if (uiUOMID != null && uiUOMID.Trim() != "")
                {
                    uiUOMName = LRRModel.ChildModel.GetUOMNameFromID(uiUOMID);
                }
                if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
                {
                    uomDisplayName = uiUOMName;
                }
                else
                {
                    uomDisplayName = uomName;
                }

                rowHeader.UOMDisplayName = uomDisplayName;
                gridControl.Model[rowIndex, UOM_COL].Text = uomDisplayName;
                gridControl.Model[rowIndex, UOM_COL].CellType = "ComboBox";
                gridControl.Model[rowIndex, UOM_COL].DataSource = uoms;
                uoms.SortColumn = "Name";
                uoms.Sort();
                gridControl.Model[rowIndex, UOM_COL].DisplayMember = "Name";
                gridControl.Model[rowIndex, UOM_COL].ValueMember = "ID";
                gridControl.Model[rowIndex, UOM_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                gridControl.Model[rowIndex, UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;

                gridControl[rowIndex, PROPERTIES_COL].BackColor = IOSchemes.GRID_ROW_COLOR;
                gridControl[rowIndex, PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                gridControl[rowIndex, PROPERTIES_COL].TextMargins.Left = 5;
                rowIndex++;

            }
        }

        /// <summary>
        /// Initializes the property values.
        /// </summary>
        /// <param name="gridControl">The grid control.</param>
        /// <param name="gridControlManager">The grid control manager.</param>
		private void InitializePropertyValues(GridControl gridControl,GridControlManager gridControlManager)
		{
			//Clear the Bindings
			ClearBindings(gridControlManager);

			//Loop through the grid matrix,put property values,add bindings
            for (int iRow = DATA_ROW_START; iRow < gridControlManager.RowList.Count + DATA_ROW_START; iRow++)
            {
                for (int iCol = DATA_COL_START; iCol < gridControlManager.ColList.Count * LRRModel.ProjectModes.Count + DATA_COL_START; iCol++)
                {
                        CellHeader rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;
                        CellHeader colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;

                        string uiUOMID = rowData.UIUOMId;

                        object objPropValue = GetPropertyValueObject(rowData, colData);

                        if (objPropValue != null)
                        {
                            //						gridControl.Model[iRow,iCol].Text = siteCompPropValue.PropertyValue;
                            IPropertyValue iPropValue = objPropValue as IPropertyValue;
                            string uomID = iPropValue.GetUOMID();
                            if (iPropValue.GetID() == "")
                            {
                                int xyz = 0;
                            }

                            //check for EVVs,if yes then put an exclusive drop down
                            if (iPropValue.GetEditableState())
                            {
                                string uomTypeID = rowData.UOMTypeID;
                                if (uomTypeID != null && uomTypeID.Trim() != "")
                                {
                                    LRRUOMType lrrUOMType = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(uomTypeID);
                                    if (lrrUOMType != null && lrrUOMType.IsEVV && !LRRModel.ChildModel.ConstantEVVs.ContainsKey(uomTypeID))
                                    {
                                        LRREVVs evvs = LRRModel.ChildModel.GetEVVsFromUOMTypeID(uomTypeID);
                                        gridControl.Model[iRow, iCol].CellType = "ComboBox";
                                        gridControl.Model[iRow, iCol].DataSource = evvs;
                                        gridControl.Model[iRow, iCol].DisplayMember = "EVV";
                                        gridControl.Model[iRow, iCol].ValueMember = "EVV";
                                        gridControl.Model[iRow, iCol].DropDownStyle = GridDropDownStyle.Exclusive;
                                        //gridControl.Model[iRow, iCol].BackColor = "Color";
                                    }
                                }
                            }

                            //Add Bindings
                            gridControlManager.BindingManager.AddBinding(objPropValue, "PropertyValue", gridControl.Model[iRow, iCol], "Text");

                            //Add Colour Binding
                            if (!gridControlManager.PropValueCellAddress.ContainsKey(iPropValue.GetID()))
                            {
                                gridControlManager.PropValueCellAddress.Add(iPropValue.GetID(), new GridCellAddress(iRow, iCol));
                                iPropValue.ForeColorChanged += new EventHandler(gridControlManager.ForeColorChangedHandler);
                                iPropValue.BackColorChanged += new EventHandler(gridControlManager.BackColorChangedHandler);
                            }

                            //Set Colour
                            SetColor(iPropValue, gridControl.Model[iRow, iCol]);

                            //Convert property value according to UIUOM 
                            string propertyValue = iPropValue.GetPropertyValue();
                            string actualPropertyValue = iPropValue.GetActualPropertyValue();
                            if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
                            {
                                propertyValue = iPropValue.GetPropertyValueForUOM(uomID, uiUOMID, propertyValue);
                                actualPropertyValue = iPropValue.GetPropertyValueForUOM(uomID, uiUOMID, actualPropertyValue);
                            }

                            //Add to UI UOM hash
                            if (gridControlManager.UIUOMHash.ContainsKey(rowData.ID))
                            {
                                gridControlManager.UIUOMHash[rowData.ID] = uiUOMID;
                            }
                            else
                            {
                                gridControlManager.UIUOMHash.Add(rowData.ID, uiUOMID);
                            }



                            string lrrDataType = iPropValue.GetDataType();
                            if (lrrDataType == null || lrrDataType.Trim() == "")
                            {
                                propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                            }
                            gridControl.Model[iRow, iCol].Text = propertyValue;

                            //Set the Cell tool Tip
                            SetCellToolTipText(iPropValue, gridControl.Model[iRow, iCol], rowData.UIUOMId);

                            gridControl.Model[iRow, iCol].HorizontalAlignment = GridHorizontalAlignment.Right;
                            gridControl.Model[iRow, iCol].AutoSize = true;
                            gridControl.Model[iRow, iCol].Trimming = StringTrimming.EllipsisCharacter;
                        }
                        else
                        {
                            gridControl.Model[iRow, iCol].BackColor = IOSchemes.GRID_ROW_COLOR;
                            gridControl.Model[iRow, iCol].Borders.All = new GridBorder(GridBorderStyle.Solid, IOSchemes.GRID_PROPVALUE_LINE_COLOR, GridBorderWeight.ExtraThin);
                        }
                    
                    }
                
                }
            
			//Resize to fit
			gridControl.ColWidths.ResizeToFit(GridRangeInfo.Cells(DATA_ROW_START,DATA_COL_START,gridControl.RowCount,gridControl.ColCount),GridResizeToFitOptions.NoShrinkSize);
		}

        /// <summary>
        /// Gets the property value object.
        /// </summary>
        /// <param name="rowData">The row data.</param>
        /// <param name="colData">The col data.</param>
        /// <returns></returns>
		private object GetPropertyValueObject(CellHeader rowData,CellHeader colData)
		{
			SearchCriteria rowSrchCriteria = rowData.SearchCriteriaObj;
			SearchCriteria colSrchCriteria = colData.SearchCriteriaObj;

			//Get the Site Component
			string modeID = colData.ModeID;
			ArrayList arrConditionList = colSrchCriteria.ArrListSearchCondition;
            SearchCondition condition = arrConditionList[0] as SearchCondition;
            ArrayList arrRowConditionList = rowSrchCriteria.ArrListSearchCondition;
            if (colSrchCriteria.ObjType == typeof(LRRAggregate))
            {
                LRRAggregate aggregate = LRRModel.AggregatesObj.GetAggregateFromID(condition.Value);
                //Get the Property Values
                foreach (SearchCondition aggCondition in arrRowConditionList)
                {
                    if (aggCondition.ObjType == typeof(LRRAggregatePropertyValue))
                    {
                        //SearchCondition condition2 = arrRowConditionList[0] as SearchCondition;
                        LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(aggCondition.Value);
                        LRRAggregateProperty aggProperty = null;
                        if (masterProp != null)
                        {
                            aggProperty = aggregate.GetPropertyFromMasterPropID(masterProp.PropID);
                        }
                        else
                        {
                            object objAggProperty = LRRModel.GetPropertyObject(typeof(LRRAggregate), aggCondition.Value, aggregate.AggregateID);
                            if (objAggProperty != null)
                                aggProperty = objAggProperty as LRRAggregateProperty;
                        }
                        if (aggProperty != null)
                        {
                            List<IPropertyValue> arrPropvalues = LRRModel.AggregatePropertyValuesObj.GetPropValuesByPropID(aggProperty.ID);
                            if (arrPropvalues != null)
                            {
                                for (int i = 0; i < arrPropvalues.Count; i++)
                                {
                                    LRRAggregatePropertyValue aggPropValue = arrPropvalues[i] as LRRAggregatePropertyValue;
                                    if (aggPropValue != null && aggPropValue.ModeID == modeID)
                                    {
                                        return aggPropValue;
                                    }
                                }
                            }
                        }
                    }
                    else if (aggCondition.ObjType == typeof(LRRAggregateGroupedPropertyValue))
                    {
                        string lkUpPropertyIDFrAgg = null;
                        string lookUpQualifierIDFrAgg = null;
                        string masterPropIDFrAgg = null;
                        for (int j = 0; j < arrRowConditionList.Count; j++)
                        {
                            SearchCondition condition1 = arrRowConditionList[j] as SearchCondition;
                            if (condition1.Name == "LookUpPropertiesID")
                                lkUpPropertyIDFrAgg = condition1.Value;
                            if (condition1.Name == "LookUpQualifierID")
                                lookUpQualifierIDFrAgg = condition1.Value;
                            if (condition1.Name == "MasterPropID")
                                masterPropIDFrAgg = condition1.Value;
                        }

                        LRRAggregateProperty aggProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromMasterPropertyID(aggregate.AggregateID, masterPropIDFrAgg);
                        if (aggProp != null)
                        {
                            string aggGrpPropID = LRRModel.AggregateGroupedPropertiesObj.GetAggregateGroupPropID(aggProp.ID, lkUpPropertyIDFrAgg);
                            LRRLookUpQualifier lkUpQualObj = null;
                            if (lookUpQualifierIDFrAgg != null && lookUpQualifierIDFrAgg.Trim() != "")
                                lkUpQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierIDFrAgg);
                            string qualID = lkUpQualObj == null ? null : lkUpQualObj.QualifierID;
                            LRRAggregateGroupedPropertyValue aggGrpValueObj = LRRModel.AggregateGroupedPropertyValuesObj.GetGroupedValueObjectForPropertyMode(aggGrpPropID, lkUpPropertyIDFrAgg , qualID, modeID);
                            if (aggGrpValueObj != null)
                            {
                                return aggGrpValueObj;
                            }
                        }
                    }
                }
                return null;
            }
            else
            {
                LRRSiteComponent siteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(condition.Value);
                //Get the Property Values
                if (rowSrchCriteria.ObjType == typeof(LRRSiteComponentPropertyValue))
                {
                    SearchCondition condition2 = arrRowConditionList[0] as SearchCondition;
                    LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition2.Value);
                    LRRSiteComponentProperty siteCompProperty = null;
                    if (masterProp != null)
                    {
                        siteCompProperty = siteComponent.GetPropertyFromMasterPropID(masterProp.PropID);
                    }
                    else
                    {
                        object objSiteCompProperty = LRRModel.GetPropertyObject(typeof(LRRSiteComponent), condition2.Value, siteComponent.SiteComponentID);
                        if (objSiteCompProperty != null)
                            siteCompProperty = objSiteCompProperty as LRRSiteComponentProperty;
                    }
                    if (siteCompProperty != null)
                    {
                        List<IPropertyValue> arrPropvalues = LRRModel.PropertyValuesObj.GetPropValuesByPropID(siteCompProperty.ID);
                        if (arrPropvalues != null)
                        {
                            for (int i = 0; i < arrPropvalues.Count; i++)
                            {
                                LRRSiteComponentPropertyValue siteCompPropValue = arrPropvalues[i] as LRRSiteComponentPropertyValue;
                                if (siteCompPropValue != null && siteCompPropValue.ModeID == modeID)
                                {
                                    return siteCompPropValue;
                                }
                            }
                        }
                    }
                }
                if (rowSrchCriteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
                {
                    string lkUpPropertyID = null;
                    string lookUpQualifierID = null;
                    string masterPropID = null;
                    for (int j = 0; j < arrRowConditionList.Count; j++)
                    {
                        SearchCondition condition1 = arrRowConditionList[j] as SearchCondition;
                        if (condition1.Name == "LookUpPropertiesID")
                            lkUpPropertyID = condition1.Value;
                        if (condition1.Name == "LookUpQualifierID")
                            lookUpQualifierID = condition1.Value;
                        if (condition1.Name == "MasterPropID")
                            masterPropID = condition1.Value;
                    }

                    LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromMasterPropertyID(siteComponent.SiteComponentID, masterPropID);
                    if (siteCompProp != null)
                    {
                        string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompProp.ID, lkUpPropertyID);
                        LRRLookUpQualifier lkUpQualObj = null;
                        if (lookUpQualifierID != null && lookUpQualifierID.Trim() != "")
                            lkUpQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
                        string qualID = lkUpQualObj == null ? null : lkUpQualObj.QualifierID;
                        LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp, lkUpPropertyID, qualID, modeID);
                        if (qualValueObj != null)
                        {
                            return qualValueObj;
                        }
                    }
                }
						 
            }
            return null;
		}

		/// <summary>
		/// This method clears the bindings
		/// </summary>
		/// <param name="gridControlManager"></param>
		private void ClearBindings(GridControlManager gridControlManager)
		{
			if(gridControlManager.BindingManager != null)
			{
				gridControlManager.BindingManager.RemoveBinding();
				
			}
		}

		/// <summary>
		/// Method sets the fore/back/border colours for the grid cell
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="gsInfo"></param>
		private void SetColor(IPropertyValue iPropValue,GridStyleInfo gsInfo)
		{
			gsInfo.BackColor = iPropValue.GetBackColor();
			gsInfo.TextColor = iPropValue.GetForeColor();

			//Set Border Color and Border Thickness
			Color borderColor = iPropValue.GetBorderColor();
			int borderThickness = iPropValue.GetBorderThickness();
			if(borderThickness == 2)
            {	//gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid, borderColor,GridBorderWeight.Medium);
                GridBorder grdBorder = new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                gsInfo.Borders.Top = grdBorder;// new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                gsInfo.Borders.Left = grdBorder;// new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
            }
			else
			{
				gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid,IOSchemes.GRID_PROPVALUE_LINE_COLOR,GridBorderWeight.ExtraThin);
			}
			gsInfo.Font.Underline = iPropValue.GetUnderLine();
		}

		/// <summary>
		/// Changes property Values acording to UI UOM
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		private void changePropertyValueForUOM(GridControl gridControl,GridControlManager gridControlManager,int iRow,int iCol)
		{
			//Get the old UOM and new UOM
			string newUOM = gridControl.CurrentCell.Renderer.ControlValue == null ? "" : gridControl.CurrentCell.Renderer.ControlValue.ToString();
			
			for(int iColIndex =DATA_COL_START; iColIndex <= gridControl.ColCount;iColIndex++)
			{
				CellHeader colData = gridControlManager.ColHeader[iColIndex - DATA_COL_START] as CellHeader;
				CellHeader rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;

                //ChangedUIUOMDictionary.AddUOM(this.GetType(), rowData.ID.ToString(), newUOM);

				object objPropValue = GetPropertyValueObject(rowData,colData);

				if(objPropValue != null)
				{
					IPropertyValue iPropValue = objPropValue as IPropertyValue;
					string uomID = iPropValue.GetUOMID();

					//Convert property value according to UIUOM 
                    //TODO: Changed the method from PropertyValue to ActualPropertyValue 
					//string propertyValue = iPropValue.GetPropertyValue();
                    string propertyValue = iPropValue.GetActualPropertyValue();
                    propertyValue = iPropValue.GetPropertyValueForUOM(uomID,newUOM,propertyValue);
                   // propertyValue = ConvertUOM(uomID, newUOM, propertyValue, iPropValue.GetDataType());

                    string lrrDataType = iPropValue.GetDataType();
                    if (lrrDataType == null || lrrDataType.Trim() == "")
                    {
                        propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                    }
                    else
                    {
                        LRRDataType dataType = LRRModel.ChildModel.DataTypes.GetDataTypeObjByID(lrrDataType);
                        bool isText = dataType.Name.StartsWith(Validator.LRR_TEXT);
                        bool isNumber = dataType.Name.StartsWith(Validator.LRR_NUMBER);
                        if (isNumber)
                        {
                            string currentDataType = dataType.Name.Remove(0, Validator.LRR_NUMBER.Length);
                            //Get the precisions
                            string[] currPrecisions = currentDataType.Split(new char[] { ',' }, 2);
                            if (currPrecisions.Length > 1)
                            {
                                int iCurMantissa = Int32.Parse(currPrecisions[0], SymConverter.Culture);
                                int iCurPrecision = Int32.Parse(currPrecisions[1], SymConverter.Culture);

                                string currentPropertyValue = propertyValue;
                                if (currentPropertyValue != null && currentPropertyValue.Trim() != "")
                                {
                                    string[] currValuePrecisions = currentPropertyValue.Split(new char[] { '.' }, 2);
                                    if (currValuePrecisions.Length > 1)
                                    {
                                        int ind = currValuePrecisions[1].ToUpper().IndexOf("E");//Check for Scientific Notation...
                                        if (ind == -1)
                                        {
                                            if (currValuePrecisions[1].Length > iCurPrecision)
                                            {

                                                //string newPropertyValuePrecision  = currValuePrecisions[1].Substring(0,iCurPrecision + 1);
                                                string newPropertyValue = currentPropertyValue;// currValuePrecisions[0] + "." + newPropertyValuePrecision;
                                                int defaultLength = Symcon.BusinessObjects.SymConverter.Culture.NumberFormat.NumberDecimalDigits;
                                                //Symcon.BusinessObjects.SymConverter.Culture.NumberFormat.NumberDecimalDigits = currValuePrecisions[1].Length;
                                                Symcon.BusinessObjects.SymConverter.Culture.NumberFormat.NumberDecimalDigits = iCurPrecision;
                                                try
                                                {
                                                    object retValue = 0.0;
                                                    bool success = Validator.CanConvertToDouble(newPropertyValue, out retValue);
                                                    if (success)
                                                    {
                                                        double dPropertyValue = System.Math.Round((double)retValue, iCurPrecision);
                                                        newPropertyValue = dPropertyValue.ToString("N", Symcon.BusinessObjects.SymConverter.Culture);
                                                    }
                                                    Symcon.BusinessObjects.SymConverter.Culture.NumberFormat.NumberDecimalDigits = defaultLength;
                                                }
                                                catch { }
                                                propertyValue = newPropertyValue;
                                            }
                                        }
                                        else
                                        {
                                            propertyValue = currentPropertyValue;
                                        }
                                    }
                                    else
                                    {
                                        string newPropertyValue = currentPropertyValue;
                                        object retValue = 0.0;
                                        bool success = Validator.CanConvertToDouble(newPropertyValue, out retValue);
                                        if (success)
                                        {
                                            int defaultLength = Symcon.BusinessObjects.SymConverter.Culture.NumberFormat.NumberDecimalDigits;
                                            Symcon.BusinessObjects.SymConverter.Culture.NumberFormat.NumberDecimalDigits = 0;
                                            double dPropertyValue = System.Math.Round((double)retValue, 0);
                                            newPropertyValue = dPropertyValue.ToString("N", Symcon.BusinessObjects.SymConverter.Culture);
                                            Symcon.BusinessObjects.SymConverter.Culture.NumberFormat.NumberDecimalDigits = defaultLength;
                                        }
                                        propertyValue = newPropertyValue;
                                    }
                                }
                            }
                        }
                    }

					gridControl.Model[iRow,iColIndex].Text = propertyValue;

					//Set Cell Tool Tip text
					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iColIndex],newUOM);

                    rowData.UIUOMId = newUOM;
                    //Set the Display UOM
                    rowData.UOMDisplayName = LRRModel.ChildModel.GetUOMNameFromID(newUOM);

					//Set the UI UOM to Sortmanager Row 
					//					rowData.UIUOMId = newUOM;
					//
                    //Add to UI UOM hash
                    //if (gridControlManager.UIUOMHash.ContainsKey(rowData.ID))
                    //{
                    //    gridControlManager.UIUOMHash[rowData.ID] = newUOM;
                    //}
                    //else
                    //{
                    //    gridControlManager.UIUOMHash.Add(rowData.ID, newUOM);
                    //}
                    CommonViews.ChangedUIUOMDictionary.AddUOM(typeof(GridSavedViewController),rowData.ID,newUOM);
				}
			}
		}

        /// <summary>
		/// Property value changes
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>
        private bool changePropertyValue(GridControl gridControl, GridControlManager gridControlManager, int iRow, int iCol)
        {
            return changePropertyValue(gridControl, gridControlManager, iRow, iCol, null);
        }

        public static LRRSiteComponentPropertyValues RollBackPropertyValues
        {
            get { return _rollBackPropertyValues; }
            set { _rollBackPropertyValues = value; }
        }
        public static Hashtable RollBackSettingsHash
        {
            get { return _rollBackSettingsHash; }
            set { _rollBackSettingsHash = value; }
        }
        public static LRRSiteGroupPropertyQualifierValues RollBackQValues
        {
            get { return _rollBackQVa; }
            set { _rollBackQVa = value; }
        }
        public static Hashtable RollBackQVSettingsHash
        {
            get { return _rollBackQVSettingsHash; }
            set { _rollBackQVSettingsHash = value; }
        }
        public static bool RemoveOverride
        {
            get { return _removeOverride; }
            set { _removeOverride = value; }
        }
        public static LRROverrides RollBackPVOverrides
        {
            get { return _rollBackPVOverrides; }
            set { _rollBackPVOverrides = value; }
        }
        private static bool _removeOverride = false;
        private static LRROverrides _rollBackPVOverrides = new LRROverrides();
        private static LRRSiteComponentPropertyValues _rollBackPropertyValues = new LRRSiteComponentPropertyValues();
        private static ArrayList _arrRollBakcsettings = new ArrayList();
        public static Hashtable _rollBackSettingsHash = new Hashtable();
        private static Hashtable _rollBackQVSettingsHash = new Hashtable();
        private static ArrayList _arrRollBackSettingsQVs = new ArrayList();
        private static LRRSiteGroupPropertyQualifierValues _rollBackQVa = new LRRSiteGroupPropertyQualifierValues();
        public void ProcessRollBackforQV(object objPropValue, IPropertyValue iPropValue, bool removeOverride)
        {
            /*
            if (_lrrApp.RollBackQualifierValuesGSVC.Contains(objPropValue))
            {
                _lrrApp.RollBackQualifierValuesGSVC.Remove(objPropValue);
                _lrrApp.RollBackQualifierValuesGSVC.Add(objPropValue);
            }
            else
            {
                _lrrApp.RollBackQualifierValuesGSVC.Add(objPropValue);
            }

            if (RollBackQValues.Contains(objPropValue))
            {
                RollBackQValues.Remove(objPropValue);
                RollBackQValues.Add(objPropValue);
            }
            else
            {
                RollBackQValues.Add(objPropValue);
            }
            iPropValue = objPropValue as IPropertyValue;
            _arrRollBakcsettings.Clear();
            _arrRollBakcsettings.Add(iPropValue.GetPropertyValue());
            _arrRollBakcsettings.Add(iPropValue.GetActualPropertyValue());
            _arrRollBakcsettings.Add(iPropValue.GetErrorCode());
            _arrRollBakcsettings.Add(iPropValue.GetForeColor());
            _arrRollBakcsettings.Add(iPropValue.GetUnderLine());
            _arrRollBakcsettings.Add(((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(objPropValue)).TrustedState);
            if (removeOverride)
            {
                _arrRollBakcsettings.Add(!iPropValue.GetOverrideState());
            }
            else
            {
                _arrRollBakcsettings.Add(iPropValue.GetOverrideState());
            }
            
            if (!RollBackQVSettingsHash.ContainsKey(iPropValue))
            {
                RollBackQVSettingsHash.Add(iPropValue, _arrRollBakcsettings);

                if (!_lrrApp.RollBackQualifierValueGSVCSettingsHash.ContainsKey(iPropValue))
                {
                    _lrrApp.RollBackQualifierValueGSVCSettingsHash.Add(iPropValue, _arrRollBakcsettings);
                }

            }
             * */

        }
        public void ProcessRollBackforPV(object objPropValue, IPropertyValue iPropValue, bool removeOverride)
        {
            if (_lrrApp.RollBackPropertyValuesGSVC.Contains(objPropValue))
            {
                _lrrApp.RollBackPropertyValuesGSVC.Remove(objPropValue);
                _lrrApp.RollBackPropertyValuesGSVC.Add(objPropValue);
            }
            else
            {
                _lrrApp.RollBackPropertyValuesGSVC.Add(objPropValue);
            }
            if (RollBackPropertyValues.Contains(objPropValue))
            {
                RollBackPropertyValues.Remove(objPropValue);
                RollBackPropertyValues.Add(objPropValue);
            }
            else
            {
                RollBackPropertyValues.Add(objPropValue);
            }
            iPropValue = objPropValue as IPropertyValue;
            _arrRollBakcsettings.Clear();
            _arrRollBakcsettings.Add(iPropValue.GetPropertyValue());
            _arrRollBakcsettings.Add(iPropValue.GetActualPropertyValue());
            _arrRollBakcsettings.Add(iPropValue.GetErrorCode());
            _arrRollBakcsettings.Add(iPropValue.GetForeColor());
            _arrRollBakcsettings.Add(iPropValue.GetUnderLine());
            _arrRollBakcsettings.Add(((Symcon.LightRidge.BusinessObjects.LRRSiteComponentPropertyValue)(objPropValue)).TrustedState);
            //_arrRollBakcsettings.Add(iPropValue.GetOverrideState());
            if (removeOverride)
            {
                _arrRollBakcsettings.Add(!iPropValue.GetOverrideState());
            }
            else
            {
                _arrRollBakcsettings.Add(iPropValue.GetOverrideState());
            }

            if (!RollBackSettingsHash.ContainsKey(iPropValue))
            {
                RollBackSettingsHash.Add(iPropValue, _arrRollBakcsettings);
                if (!_lrrApp.RollBackPropertyValueGSVCSettingsHash.ContainsKey(iPropValue))
                {
                    _lrrApp.RollBackPropertyValueGSVCSettingsHash.Add(iPropValue, _arrRollBakcsettings);
                }
            }

        }
		/// <summary>
		/// Property value changes
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>
		private bool changePropertyValue(GridControl gridControl,GridControlManager gridControlManager,int iRow,int iCol, string currentCellValue)
        {
            string currentValue = gridControl.CurrentCell.Renderer.ControlText;
            
            if (currentCellValue != null)
                currentValue = currentCellValue;
            currentValue = Utilities.ConvertCultureSpecficToPlainString(currentValue, _lrrApp);

            //Get the Row Data from Grid Data Manager
            ArrayList rowSortList = gridControlManager.RowHeader;
            //Get the Col Data from Grid Data Manager
            ArrayList colHeaderCellList = gridControlManager.ColHeader;

            //Get the row data
            CellHeader rowData = rowSortList[iRow - DATA_ROW_START] as CellHeader;
            string rowCellID = rowData.ID;
            //Get the column data
            CellHeader colData = colHeaderCellList[iCol - DATA_COL_START] as CellHeader;
            //Type objectType = colData.ObjectType;
            string modeID = colData.ModeID;
            string colCellID = colData.ID;
            RollBackSettingsHash.Clear();
            RollBackQVSettingsHash.Clear();
            _lrrApp.RollBackPropertyValueGSVCSettingsHash.Clear();
            _lrrApp.RollBackQualifierValueGSVCSettingsHash.Clear();
            //Get the Property Value Object
            object objPropValue = GetPropertyValueObject(rowData, colData);
            IPropertyValue iPropValue ;
            if (objPropValue != null)
            {
                iPropValue = objPropValue as IPropertyValue;
                // rollaback collection added by ashwin
                //objPropValue.GetType().Name.Contains("Qualifier")
                if (objPropValue.GetType().Name.Contains("Qualifier"))
                {
                    ProcessRollBackforQV(objPropValue, iPropValue, false);
                }
                else
                {
                    ProcessRollBackforPV(objPropValue, iPropValue, false);
                }


                string uomID = iPropValue.GetUOMID();

                string localeID = LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID;
                string uiUomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localeID, rowData.UOMTypeID);

                string newUOM = "";

                if (rowData.UIUOMId == null)
                {
                    newUOM = rowData.UOMId;
                }
                else
                {
                    newUOM = rowData.UIUOMId;
                }
                
                string newpropValue = iPropValue.GetPropertyValueForUOM(newUOM,uomID, currentValue);//ConvertUOM(newUOM, uomID, currentValue, iPropValue.GetDataType());
                //newpropValue = ConvertUOM(newUOM, uiUomID, currentValue, iPropValue.GetDataType());

                SearchCriteria criteria = rowData.SearchCriteriaObj;

                if (iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
                {

                    bool prevOverrideState = iPropValue.GetOverrideState();
                    iPropValue.SetOverrideState(newpropValue);
                    bool currOverrideState = iPropValue.GetOverrideState();

                    //Measurement Mode
                    if (prevOverrideState != currOverrideState && modeID == "1")
                    {
                        iPropValue.SetTrustedState("-1");
                        if (iPropValue.GetType() == typeof(LRRAggregatePropertyValue))
                        {
                            _lrrApp.SwapBindings(iPropValue, true);
                            iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                        }
                        else if (iPropValue.GetType() == typeof(LRRAggregateGroupedPropertyValue))
                        {
                            _lrrApp.SwapGroupPropBindings(iPropValue, true);
                            iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                        }
                        else
                        {
                            if (criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
                                _lrrApp.SwapBindings(objPropValue, true);
                            if (criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
                                _lrrApp.SwapGroupPropBindings(objPropValue, true);
                            iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                        }
                    }

                    //Other Modes
                    if (prevOverrideState != currOverrideState && modeID != "1")
                    {
                        // line commented by ashwin
                        //iPropValue.SetTrustedState("-1");
                        if (iPropValue.GetType() == typeof(LRRAggregatePropertyValue))
                        {
                            _lrrApp.RemoveCascadedBindings(objPropValue, typeof(LRRAggregate), rowData.ID, colData.ID, true);
                            iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                        }
                        else if (iPropValue.GetType() == typeof(LRRAggregateGroupedPropertyValue))
                        {
                            _lrrApp.RemoveCascadedBindings(objPropValue, typeof(LRRAggregate), rowData.ID, colData.ID, true);
                            iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                        }
                        else
                        {
                            if (criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
                                _lrrApp.RemoveCascadedBindings(objPropValue, typeof(LRRSiteComponent), rowData.ID, colData.ID, true);
                            if (criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
                            {
                                LRRSiteGroupPropertyQualifierValue qualObj = objPropValue as LRRSiteGroupPropertyQualifierValue;
                                _lrrApp.RemoveCascadedBindings(objPropValue, qualObj.GroupPropertyID, qualObj.LookUpPropertiesID, qualObj.QualifierID, true);
                            }
                            iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());

                        }
                    }

                    iPropValue.SetPropertyValue(newpropValue);
                    //Set the Cell tool Tip
                    SetCellToolTipText(iPropValue, gridControl.Model[iRow, iCol], newUOM);

                    //Get the Property Object
                    if (criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
                    {
                        if (iPropValue.GetType() == typeof(LRRAggregatePropertyValue))
                        {
                            object objProp = LRRModel.GetPropertyObject(typeof(LRRAggregate), rowData.ID, colData.ID);
                            if (objProp != null)
                            {
                                //Populate Overrides Collection	
                                IProperty iProp = objProp as IProperty;
                                string propID = iProp.GetID();
                                PopulateOverrides(iPropValue, propID, colData.ID, modeID, uiUomID);
                            }
                        }
                        //else if(iPropValue.GetType() == typeof(LRRAggregateGroupedPropertyValue))
                        //{
                        //    object objProp = LRRModel.GetPropertyObject(typeof(LRRAggregate), rowData.ID, colData.ID);
                        //    IProperty iProp = objProp as IProperty;
                        //    string propID = iProp.GetID();
                        //    PopulateAggGroupOverrides(iPropValue, propID, colData.ID, modeID, uomID);
                        //}
                        else
                        {
                            object objProp = LRRModel.GetPropertyObject(typeof(LRRSiteComponent), rowData.ID, colData.ID);
                            if (objProp != null)
                            {
                                //Populate Overrides Collection	
                                IProperty iProp = objProp as IProperty;
                                string propID = iProp.GetID();
                                PopulateOverrides(iPropValue, propID, colData.ID, modeID, uiUomID);
                            }
                        }
                    }

                    //Get the Group Property Object
                    if (criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
                    {
                        if (iPropValue.GetType() == typeof(LRRAggregateGroupedPropertyValue))
                        {
                            LRRAggregateGroupedPropertyValue qualObj = objPropValue as LRRAggregateGroupedPropertyValue;
                            if (qualObj != null)
                            {
                                PopulateAggGroupOverrides(qualObj, qualObj.AggregateGroupPropertyID, qualObj.QualifierID);
                            }

                        }
                        else
                        {
                            LRRSiteGroupPropertyQualifierValue qualObj = objPropValue as LRRSiteGroupPropertyQualifierValue;
                            if (qualObj != null)
                            {
                                PopulateGroupOverrides(qualObj, qualObj.GroupPropertyID, qualObj.QualifierID);
                            }
                        }
                    }

                }
            }
            return true;
        }

        /// <summary>
        /// Cascades the changes to prop values.
        /// </summary>
        /// <param name="gridControl">The grid control.</param>
        /// <param name="gridControlManager">The grid control manager.</param>
        /// <param name="iRow">The i row.</param>
        /// <param name="iCol">The i col.</param>
        /// <returns></returns>
        private bool cascadeChangesToPropValues(GridControl gridControl, GridControlManager gridControlManager, int iRow, int iCol)
        {
            string currentValue = "";
            //Get the Row Data from Grid Data Manager
            ArrayList rowSortList = gridControlManager.RowHeader;
            //Get the Col Data from Grid Data Manager
            ArrayList colHeaderCellList = gridControlManager.ColHeader;

            //Get the row data
            CellHeader rowData = rowSortList[iRow - DATA_ROW_START] as CellHeader;
            string rowCellID = rowData.ID;
            //			string uomID     = rowData.UOMID;

            //Get the column data
            CellHeader colData = colHeaderCellList[iCol - DATA_COL_START] as CellHeader;
            Type objectType = colData.SearchCriteriaObj.ObjType;
            string modeID = colData.ModeID;
            string colCellID = colData.ID;

            //Get the Property Value Object
            object objPropValue = GetPropertyValueObject(rowData, colData); //LRRModel.GetPropertyValueObject(objectType, modeID, rowCellID, colCellID);
            if (objPropValue != null)
            {
                IPropertyValue iPropValue = objPropValue as IPropertyValue;
                string uomID = iPropValue.GetUOMID();
                string newUOM = rowData.UIUOMId;
                string newpropValue = iPropValue.GetPropertyValueForUOM(newUOM, uomID, currentValue);

                if (iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
                {
                    bool prevOverrideState = iPropValue.GetOverrideState();
                    iPropValue.SetOverrideState(newpropValue);
                    bool currOverrideState = iPropValue.GetOverrideState();

                    //Measurement Mode
                    if (prevOverrideState != currOverrideState && modeID == "1")
                    {
                        iPropValue.SetTrustedState("-1");
                        _lrrApp.SwapBindings(objPropValue, true);
                        iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                    }

                    //Other Modes
                    if (prevOverrideState != currOverrideState && modeID != "1")
                    {
                        iPropValue.SetTrustedState("-1");
                        _lrrApp.RemoveCascadedBindings(objPropValue, objectType, rowCellID, colCellID, true);
                        iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                    }

                    iPropValue.SetPropertyValue(newpropValue);
                    //Set the Cell tool Tip
                    SetCellToolTipText(iPropValue, gridControl.Model[iRow, iCol], newUOM);
                    //Populate Overrides Collection
                    object objProp = LRRModel.GetPropertyObject(objectType, rowCellID, colCellID);
                    if (objProp != null)
                    {
                        IProperty iProp = objProp as IProperty;
                        string propID = iProp.GetID();
                        PopulateOverrides(iPropValue, propID, colCellID, modeID, uomID);
                    }

                }
                //				SetColor(iRowIndex,iColIndex);
            }
            return true;
        }


	
		/// <summary>
		/// Populates Overrides Screen
		/// </summary>
		/// <param name="lrrSiteCompPropValue"></param>
		private void PopulateOverrides(IPropertyValue iPropValue,string propertyID,string compID,string modeID, string sNewUOM)
		{
            if (iPropValue.GetType() == typeof(LRRAggregatePropertyValue))
            {
                LRRAggregateOverride lrrModelOverride = LRRModel.ChildModel.AggregateOverrides.GetOverrideObjectByAggPropertyAndModeID(propertyID, modeID);
                if (iPropValue.GetOverrideState())
                {
                    if (lrrModelOverride != null)
                    {
                        sNewUOM = lrrModelOverride.UOM;
                        LRRModel.ChildModel.AggregateOverrides.Remove(lrrModelOverride);
                    }
                    LRRAggregateOverride lrraggOverride = new LRRAggregateOverride();
                    lrraggOverride.ModeID = modeID;
                    lrraggOverride.AggregatePropertyID = propertyID;
                    lrraggOverride.AggregateID = compID;

                    //Pre/Post Override Values
                    string modelUOM = iPropValue.GetUOMID();
                    lrraggOverride.PreOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualOverridedValue());
                    lrraggOverride.PostOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualPropertyValue());

                    lrraggOverride.UOM = sNewUOM;
                    //Assign Confidence Level
                    string dataConfidenceLevelID = null;
                    if (modeID == "1" || modeID == "2")
                    {
                        dataConfidenceLevelID = "3";
                    }
                    else
                    {
                        dataConfidenceLevelID = "1";
                    }
                    lrraggOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                    LRRModel.ChildModel.AggregateOverrides.Add(lrraggOverride);

                    iPropValue.DataConfidenceValueChanged += new EventHandler(iPropValue_DataConfidenceValueChanged);

                    iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
                }
                else
                {
                    if (lrrModelOverride != null)
                    {
                        iPropValue.DataConfidenceValueChanged -= new EventHandler(iPropValue_DataConfidenceValueChanged);
                        LRRModel.ChildModel.AggregateOverrides.Remove(lrrModelOverride);
                    }
                }
                //Update Overrides
                _lrrApp.UpdateAggregateOverridePoints();
            }
				else if ( iPropValue.GetType() == typeof(LRRAggregateGroupedProperty))
				{

					LRRAggGroupOverride lrrModelOverride=LRRModel.ChildModel.GroupAggOverrides.GetOverrideObjectByAggPropertyAndModeID(propertyID,modeID);
			    if(iPropValue.GetOverrideState())
				 {
					  if(lrrModelOverride != null)
				    {
						 	 sNewUOM = lrrModelOverride.UOM;
                 LRRModel.ChildModel.GroupAggOverrides.Remove(lrrModelOverride);
					 }
					 	
					LRRAggGroupOverride lrrGrpAggOverride = new LRRAggGroupOverride();
					lrrGrpAggOverride.ModeID = modeID;
					lrrGrpAggOverride.AggregatePropertyID=propertyID;
					lrrGrpAggOverride.AggregateID = compID;
					//Pre/Post Override Values
				    string modelUOM = iPropValue.GetUOMID();
					lrrGrpAggOverride.PreOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualOverridedValue());
					lrrGrpAggOverride.PostOverrideValue=iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualPropertyValue());
					 lrrGrpAggOverride.UOM = sNewUOM;
				    //Assign Confidence Level
				    string dataConfidenceLevelID = null;
					 if(modeID == "1" || modeID == "2")
				    {
					    dataConfidenceLevelID = "3";
				    }
				    else
				    {
					    dataConfidenceLevelID = "1";
				    }
					 lrrGrpAggOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                    LRRModel.ChildModel.GroupAggOverrides.Add(lrrGrpAggOverride);
					 iPropValue.DataConfidenceValueChanged+=new EventHandler(iPropValue_DataConfidenceValueChanged);
					iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);

				 }
				 else
				 {
					 if(lrrModelOverride != null)
				    {
					    iPropValue.DataConfidenceValueChanged-=new EventHandler(iPropValue_DataConfidenceValueChanged);
					    LRRModel.ChildModel.GroupAggOverrides.Remove(lrrModelOverride);
				    }
				 }
				 

				}

            else
            {
                LRROverride lrrModelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(propertyID, modeID);
                if (iPropValue.GetOverrideState())
                {
                    if (lrrModelOverride != null)
                    {
                        sNewUOM = lrrModelOverride.UOM;
                        LRRModel.ChildModel.Overrides.Remove(lrrModelOverride);
                    }
                    LRROverride lrrOverride = new LRROverride();
                    lrrOverride.IsTemplateOverride = false;
                    lrrOverride.ModeID = modeID;
                    lrrOverride.SiteComponentPropertyID = propertyID;
                    lrrOverride.SiteComponentID = compID;

                    lrrOverride.UserID = _lrrApp.UID;
                    //Pre/Post Override Values
                    string modelUOM = iPropValue.GetUOMID();
                    lrrOverride.PreOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualOverridedValue());
                    lrrOverride.PostOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualPropertyValue());

                    lrrOverride.UOM = sNewUOM;
                    //Assign Confidence Level
                    string dataConfidenceLevelID = null;
                    if (modeID == "1" || modeID == "2")
                    {
                        dataConfidenceLevelID = "3";
                    }
                    else
                    {
                        dataConfidenceLevelID = "1";
                    }
                    lrrOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                    LRRModel.ChildModel.Overrides.Add(lrrOverride);

                    iPropValue.DataConfidenceValueChanged += new EventHandler(iPropValue_DataConfidenceValueChanged);

                    iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
                }
                else
                {
                    if (lrrModelOverride != null)
                    {
                        iPropValue.DataConfidenceValueChanged -= new EventHandler(iPropValue_DataConfidenceValueChanged);
                        LRRModel.ChildModel.Overrides.Remove(lrrModelOverride);
                        // rollback part added by ashwin
                        /*
                        if (!_lrrApp.RollBackPVOverridesGSVC.Contains(lrrModelOverride))
                        {
                            _lrrApp.RollBackPVOverridesGSVC.Add(lrrModelOverride);
                        }
                         * */
                    }
                }
                //Update Overrides
                _lrrApp.UpdateOverridePoints();
            }

		}
        /// <summary>
        /// Populates the agg group overrides.
        /// </summary>
        /// <param name="iPropValue">The i prop value.</param>
        /// <param name="aggGroupPropID">The agg group prop ID.</param>
        /// <param name="qualifierID">The qualifier ID.</param>
		 private void PopulateAggGroupOverrides(IPropertyValue iPropValue, string aggGroupPropID, string qualifierID)
		 {
			   LRRAggGroupOverride  lrrAggModelOverride = LRRModel.ChildModel.GroupAggOverrides.GetOverrideObjectByValueID(iPropValue.GetID());
            if (iPropValue.GetOverrideState())
            {
                if (lrrAggModelOverride != null)
                {
                   
						  LRRModel.ChildModel.GroupAggOverrides.Remove(lrrAggModelOverride);
                }
                LRRAggGroupOverride lrrAggGroupOverride = new LRRAggGroupOverride();
					 string aggregatePropID = LRRModel.AggregateGroupedPropertiesObj.GetAggregateGroupPropFromID(aggGroupPropID).AggregatePropertyID;
                string aggregateID = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(aggregatePropID).AggregateID;
					 
					 string modeID = iPropValue.GetModeID();
                lrrAggGroupOverride.ModeID = modeID;
                lrrAggGroupOverride.AggregatePropertyID = aggregatePropID;
                lrrAggGroupOverride.AggregateID= aggregateID;
						//lrrAggGroupOverride.CurrentValue = iPropValue.GetPropertyValue();
                //Pre/Post Override Values
                lrrAggGroupOverride.PreOverrideValue = iPropValue.GetActualOverridedValue();
                lrrAggGroupOverride.PostOverrideValue = iPropValue.GetActualPropertyValue();

					 lrrAggGroupOverride.UOM = iPropValue.GetUOMID();
                lrrAggGroupOverride.AggGroupPropertyID= aggGroupPropID;
                lrrAggGroupOverride.AggregatePropertyValueID = iPropValue.GetID();
                lrrAggGroupOverride.QualifierID = qualifierID;
					 //Assign Confidence Level
                string dataConfidenceLevelID = null;
                if (modeID == "1" || modeID == "2")
                {
                    dataConfidenceLevelID = "3";
                }
                else
                {
                    dataConfidenceLevelID = "1";
                }
					  lrrAggGroupOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                LRRModel.ChildModel.GroupAggOverrides.Add(lrrAggGroupOverride);

                iPropValue.DataConfidenceValueChanged += new EventHandler(iPropValue_DataConfidenceValueChanged);
                iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
					_lrrApp.UpdataGrpAggOverridePoints();


            }
            else
            {
                if (lrrAggModelOverride != null)
                {
                    iPropValue.DataConfidenceValueChanged -= new EventHandler(iPropValue_DataConfidenceValueChanged);
                    LRRModel.ChildModel.GroupAggOverrides.Remove(lrrAggModelOverride);
                }
						_lrrApp.UpdataGrpAggOverridePoints();

            }

		 }


        /// <summary>
        /// Populates Overrides Screen
        /// </summary>
        /// <param name="lrrSiteCompPropValue"></param>
        private void PopulateGroupOverrides(IPropertyValue iPropValue, string groupPropID, string qualifierID)
        {
            LRRGroupOverride lrrModelOverride = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(iPropValue.GetID());
            if (iPropValue.GetOverrideState())
            {
                if (lrrModelOverride != null)
                {
                    LRRModel.ChildModel.GroupOverrides.Remove(lrrModelOverride);
                }
                LRRGroupOverride lrrGroupOverride = new LRRGroupOverride();

                string siteCompPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(groupPropID).SiteComponentPropID;
                string siteCompID = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID).ComponentID;

                string modeID = iPropValue.GetModeID();
                lrrGroupOverride.ModeID = modeID;
                lrrGroupOverride.SiteComponentPropertyID = siteCompPropID;
                lrrGroupOverride.SiteComponentID = siteCompID;

                //lrrGroupOverride.CurrentValue = iPropValue.GetPropertyValue();
                //Pre/Post Override Values
                lrrGroupOverride.PreOverrideValue = iPropValue.GetActualOverridedValue();
                lrrGroupOverride.PostOverrideValue = iPropValue.GetActualPropertyValue();

                lrrGroupOverride.UOM = iPropValue.GetUOMID();
                lrrGroupOverride.GroupPropertyID = groupPropID;
                lrrGroupOverride.SiteGroupPropertyQualifierValueID = iPropValue.GetID();
                lrrGroupOverride.QualifierID = qualifierID;

                //Assign Confidence Level
                string dataConfidenceLevelID = null;
                if (modeID == "1" || modeID == "2")
                {
                    dataConfidenceLevelID = "3";
                }
                else
                {
                    dataConfidenceLevelID = "1";
                }
                lrrGroupOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                LRRModel.ChildModel.GroupOverrides.Add(lrrGroupOverride);

                iPropValue.DataConfidenceValueChanged += new EventHandler(iPropValue_DataConfidenceValueChanged);
                iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
					  _lrrApp.UpdateGrpOverridePoints();

            }
            else
            {
                if (lrrModelOverride != null)
                {
                    iPropValue.DataConfidenceValueChanged -= new EventHandler(iPropValue_DataConfidenceValueChanged);
                    LRRModel.ChildModel.GroupOverrides.Remove(lrrModelOverride);
                    // rollback part added by ashwin
                    /*
                    if (!_lrrApp.RollBackQVOverridesGSVC.Contains(lrrModelOverride))
                    {
                        _lrrApp.RollBackQVOverridesGSVC.Add(lrrModelOverride);
                    }
                    **/
                }
            }
            //Update Overrides
            _lrrApp.UpdateGrpOverridePoints();
        }



        /// <summary>
        /// Sets the Cell Tool Tip Text
        /// </summary>
        /// <param name="iPropValue">The i prop value.</param>
        /// <param name="gsInfo">The gs info.</param>
        /// <param name="uiUOMID">The UI UOMID.</param>
        private void SetCellToolTipText(IPropertyValue iPropValue, GridStyleInfo gsInfo, string uiUOMID)
        {
            gsInfo.CellTipText = null;
            string uomID = iPropValue.GetUOMID();
            string actualPropValue = iPropValue.GetActualPropertyValue();
            //string equipmentTag = null;
            string histTag = null;

            if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
            {
                actualPropValue = ConvertUOM(uomID, uiUOMID, actualPropValue, iPropValue.GetDataType());
                
            }

            string strToolTip = actualPropValue;
            gsInfo.CellTipText = strToolTip;

            if (iPropValue.GetType().Name == "LRRSiteGroupPropertyQualifierValue")
            {
                string siteCompPropID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).SiteComponentPropID;
                string histTagID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).HistTagID;
                LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
                LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
                //equipmentTag = siteComp.EquipmentTag;
                histTag = histTagID == null || histTagID == "" ? null : LRRModel.HistDatas.GetHistDataFromHistTagUID(histTagID).HistTag;
            }
            else if (iPropValue.GetType().Name == "LRRSiteComponentPropertyValue")
            {
                string siteCompPropID = ((Symcon.LightRidge.BusinessObjects.LRRSiteComponentPropertyValue)(iPropValue)).SiteComponentPropertyID;
                LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
                LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
                //equipmentTag = siteComp.EquipmentTag;
                LRRHistPropCompPropMapping histComProp = LRRModel.HistPropMaps.GetHistMapObjFromValID(siteCompPropID);
                LRRHistData histdata = LRRModel.HistDatas.GetHistDataFromHistTagUID(siteCompProp.HistTagID);
                histTag = histdata == null ? null : histdata.HistTag;
            }
            else if (iPropValue.GetType().Name == "LRRAggregatePropertyValue")
            {
                histTag = null;
            }

            if (iPropValue.GetOverrideState())
            {
                string preOverrideValue = iPropValue.GetActualOverridedValue();
                if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
                {
                    preOverrideValue = ConvertUOM(uomID, uiUOMID, preOverrideValue, iPropValue.GetDataType());
                }

                string lrrDataType = iPropValue.GetDataType();
                //if (lrrDataType == null || lrrDataType.Trim() == "")
                //{
                //    actualPropValue = Utilities.FormatNumber(actualPropValue, _lrrApp);
                //    preOverrideValue = Utilities.FormatNumber(preOverrideValue, _lrrApp);
                //}
                // start of changes for handling the displaying of tool tip using the model override object by ashwin
                LRRSiteComponentPropertyValue scpv = null;
                LRRSiteGroupPropertyQualifierValue scgQV = null;
                if (iPropValue.GetType().Name == "LRRSiteComponentPropertyValue")
                {
                    scpv = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(iPropValue.GetPropertyID(), iPropValue.GetModeID());
                }
                else if (iPropValue.GetType().Name == "LRRSiteGroupPropertyQualifierValue")
                {
                    string lookupPropertyID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).LookUpPropertiesID;
                    string groupPropertyID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).GroupPropertyID;
                    string qialifierID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).QualifierID;
                    string modeID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).ModeID;
                    string lookupQualifierId = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).LookUpQualifierID;

                    if (lookupQualifierId != null && lookupQualifierId != "")
                    {
                        scgQV = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(groupPropertyID, lookupPropertyID, qialifierID, modeID);
                    }
                    else
                    {
                        scgQV = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValueFromGroupPropIDandModeID(groupPropertyID, modeID);
                    }
                    //scgQV = LRRModel.SiteGroupPropertyQualifierValuesObj.get
                }

                if (scpv != null)
                {
                    LRROverride modelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(scpv.SiteComponentPropertyID, scpv.ModeID);
                    if (modelOverride != null)
                    {
                        strToolTip = "Pre-Override Value :" + modelOverride.PreOverrideValue + "\n" +
                                        "Post-Override Value :" + modelOverride.PostOverrideValue;
                    }
                }
                else if (scgQV != null)
                {
                    LRRGroupOverride modelOverride = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(scgQV.SiteGroupPropertyQualifierValueID);
                    if (modelOverride != null)
                    {
                        strToolTip = "Pre-Override Value :" + modelOverride.PreOverrideValue + "\n" +
                            "Post-Override Value :" + modelOverride.PostOverrideValue;
                    }
                }
                // end of changes for handling the displaying of tool tip using the model override object by ashwin
                // below line of code commented by ashwin
                /*
                strToolTip = "Pre-Override Value :" + preOverrideValue + "\n" +
              "Post-Override Value :" + actualPropValue;
                */
                if (string.IsNullOrEmpty(histTag))
                {
                    gsInfo.CellTipText = strToolTip;
                }

                else
                {
                    if (iPropValue.GetModeID() == "1")
                    {
                        gsInfo.CellTipText = strToolTip + "\n" +
                                LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
                    }
                    else
                        gsInfo.CellTipText = strToolTip;

                }
            }
            else
            {
                string lrrDataType = iPropValue.GetDataType();
                if (lrrDataType == null || lrrDataType.Trim() == "")
                {
                    actualPropValue = Utilities.FormatNumber(actualPropValue, _lrrApp);
                }
                if (!string.IsNullOrEmpty(histTag) && iPropValue.GetModeID() == "1")

                    gsInfo.CellTipText = strToolTip + "\n" + LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
            }
        }

		/// <summary>
		/// Converts Property value from one UOM to another
		/// </summary>
		/// <param name="oldUOMValue"></param>
		/// <param name="newUOMValue"></param>
		/// <param name="propVal"></param>
		/// <returns></returns>
		private string ConvertUOM(string oldUOMValue,string newUOMValue,string propVal,string lrrDataTypeID)
		{
			string newVal = "";
			if(propVal != null && propVal != "")
			{
				if(oldUOMValue == newUOMValue)
				{
					newVal = propVal;
				}
				else
				{
					if((oldUOMValue != null && oldUOMValue != "") &&
						(newUOMValue != null && newUOMValue != ""))
					{
                        newVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(oldUOMValue, newUOMValue, propVal, lrrDataTypeID);
					}
				}
			}
			return newVal;
		}

		/// <summary>
		/// Sets State of the menu items
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="contMenu"></param>
		private void ProcessMenuItems(/*IProperty iProp,*/IPropertyValue iPropValue,ContextMenu contMenu)
		{
			bool overridedState = iPropValue.GetOverrideState();
			ErrorCode errorCode   = iPropValue.GetErrorCode();
			for(int i=0;i < contMenu.MenuItems.Count;i++)
			{
				MenuItem miItem = contMenu.MenuItems[i];
				switch(miItem.Index)
				{
					//Trend Values Menu
					case 0:
                        if (iPropValue is LRRSiteGroupPropertyQualifierValue)
                            miItem.Enabled = iPropValue.GetIsTrended();
                        else if (iPropValue is LRRSiteComponentPropertyValue)
                        {
                            LRRSiteComponentPropertyValue scpv = iPropValue as LRRSiteComponentPropertyValue;
                            string siteComponentPropertyID = scpv.SiteComponentPropertyID;
                            LRRSiteComponentProperty scp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteComponentPropertyID);
                            miItem.Enabled = scp.IsTrended;
                        }
                        else
                            miItem.Enabled = false;
						break;
					//Override Values Menu
					case 1:
						miItem.Enabled = overridedState;
						break;
					//Uncertain Input Menu
					case 2:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.UNCERTAIN)) == (ErrorCode.UNCERTAIN) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Trusted Input Menu
					case 3:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.TRUSTED)) == (ErrorCode.TRUSTED) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Temp Trusted Input Menu
					case 4:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.TEMPTRUSTED)) == (ErrorCode.TEMPTRUSTED) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Uncertain Override Menu
					case 5://3:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.UNCERTAIN)) == (ErrorCode.UNCERTAIN) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Trusted Override Menu
					case 6://4:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.TRUSTED)) == (ErrorCode.TRUSTED) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Temp Trusted Override Menu
					case 7://5:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.TEMPTRUSTED)) == (ErrorCode.TEMPTRUSTED) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
				}
			}
		}

		/// <summary>
		/// This method applies sorting
		/// </summary>
		/// <param name="col"></param>
		public void SortCol(int col,GridControl gridControl,GridControlManager gridControlManager)
		{
			if(gridControl.CurrentCell.IsEditing)
				gridControl.CurrentCell.EndEdit();
		
			GridStyleInfo style = new GridStyleInfo();
			style.Tag = "";

			//turn off the old sort arrow
			for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				if(col1 != col && gridControl[1, col1].CellType == "ColumnHeaderCell")
				{
					gridControl[1, col1].CellType = "Header";
					break;
				}

			//turn on new sort arrow
			gridControl[1, col].CellType = "ColumnHeaderCell";

			System.ComponentModel.ListSortDirection dir = System.ComponentModel.ListSortDirection.Ascending;
			if(gridControl[1, col].HasTag)
			{
				if((System.ComponentModel.ListSortDirection) gridControl[1, col].Tag == System.ComponentModel.ListSortDirection.Ascending)
					dir = System.ComponentModel.ListSortDirection.Descending;
			}

			gridControl[1, col].Tag = dir;

			int headerCount = 2;
			string colName = null;
			if(col == 1)
				colName = "Name";
			if(col == 2)
				colName = "UOMName";
			int direction = 0;
			if(dir == System.ComponentModel.ListSortDirection.Ascending)
				direction = 0;
			if(dir == System.ComponentModel.ListSortDirection.Descending)
				direction = 1;
			
			Comparer  comparer = new Comparer();
			gridControl.Data.SortByColumn(col,dir,headerCount);
			gridControl.Refresh();

			//Sort the Row List
			ArrayList sortManagerList = gridControlManager.RowHeader;
			comparer.SortColumn = colName;
			comparer.SortingOrder = direction;
			sortManagerList.Sort(comparer);

            //Sort By UOM
		    if(col == 2)
			{
                createRowHeaders(gridControl,gridControlManager);
				gridControl.Refresh();
			}
		}

        /// <summary>
        /// Method creates the row Headers
        /// </summary>
        /// <param name="gridControlInstance"></param>
        /// <param name="gridControlManager"></param>
        private void createRowHeaders(GridControl gridControlInstance, GridControlManager gridControlManager)
        {
            ArrayList rowSortManagerList = gridControlManager.RowHeader;

            for (int iRow = DATA_ROW_START; iRow < rowSortManagerList.Count + DATA_ROW_START; iRow++)
            {
                CellHeader rowManager = rowSortManagerList[iRow - DATA_ROW_START] as CellHeader;
                gridControlInstance[iRow, PROPERTIES_COL].Text = rowManager.Name;
                gridControlInstance[iRow, PROPERTIES_COL].BackColor = IOSchemes.GRID_ROW_COLOR;
                gridControlInstance[iRow, PROPERTIES_COL].CellTipText = rowManager.Name;
                gridControlInstance[iRow, PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                gridControlInstance[iRow, PROPERTIES_COL].TextMargins.Left = 5;


                LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(rowManager.UOMTypeID);
                gridControlInstance.Model[iRow, UOM_COL].Text = rowManager.UOMDisplayName;
                gridControlInstance.Model[iRow, UOM_COL].CellType = "ComboBox";
                gridControlInstance.Model[iRow, UOM_COL].DataSource = uoms;
                uoms.SortColumn = "Name";
                uoms.Sort();
                gridControlInstance.Model[iRow, UOM_COL].DisplayMember = "Name";
                gridControlInstance.Model[iRow, UOM_COL].ValueMember = "ID";
                gridControlInstance.Model[iRow, UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;
                gridControlInstance.Model[iRow, UOM_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
            }
        }

		/// <summary>
		/// Method hides the modes from the grids.
		/// </summary>
		public void HideModes()
		{
			Cursor.Current = Cursors.WaitCursor;
			Cursor.Show();

			ArrayList arrModeSelected = new ArrayList();
			foreach(LRRMode mode in LRRModel.ProjectModes)
			{
				if(!mode.ViewSelection)
					arrModeSelected.Add(mode.ModeId);
			}
			
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;
				//ValidatePropertyValues(gridControl,gridControlManager);
				ArrayList colList = gridControlManager.ColHeader;
				gridControl.ColHiddenEntries.Clear();
				for(int i=0;i < colList.Count;i++)
				{
					CellHeader colHeaderCell = colList[i] as CellHeader;
					if(arrModeSelected.Contains(colHeaderCell.ModeID))
					{
						gridControl.ColHiddenEntries.Add(new GridColHidden(i + DATA_COL_START));
					}
				}
				gridControl.Refresh();
			}

			Cursor.Current = Cursors.Default;
		}

		 	/// <summary>
		/// Method Validates the property Values and sets the colors on the grid cells
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		private void ValidatePropertyValues(GridControl gridControl,GridControlManager gridControlManager)
		{
			gridControl.BeginUpdate();
			//Get the Modes Count
			int iModesCount = LRRModel.ProjectModes.Count;
			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.SortManagerList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			//Get the Row Header from Grid Data Manager
			ArrayList		RowHeader = gridControlManager.RowHeader;
			//Get the Col Header from Grid Data Manager
			ArrayList		ColHeader = gridControlManager.ColHeader;

			//Loop through the grid matrix,put property values,add bindings
			for(int iRow = DATA_ROW_START;iRow < RowHeader.Count + DATA_ROW_START - 1; iRow++)
			{
				for(int iCol = DATA_COL_START; iCol < (ColHeader.Count - DATA_COL_START + 1) * iModesCount + DATA_COL_START; iCol++)
				{
					//Get the row data
					IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
					string rowCellID = rowData.ID;
					//Get the column data
					ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
					Type objectType = colData.ObjectType;
					string modeID   = colData.ModeID;
					string colCellID = colData.ID;

					//Get the Property Value Object
					object objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);
				
					//Put Property Values in grid cells
					if(objPropValue != null)
					{
						IPropertyValue iPropValue = objPropValue as IPropertyValue;
						iPropValue.Validate();
						SetColor(iPropValue,gridControl.Model[iRow,iCol]);
					}
				}
			}
			gridControl.EndUpdate(true);
		}

		
		/// <summary>
		/// Method moves the range of columns to left
		/// </summary>
		public void LeftColMovement()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;

				if(gridControl.CurrentCell.IsActive)
				{
					GridRangeInfo rangeInfo = gridControl.Selections.Ranges.ActiveRange;
					int range = rangeInfo.Right - rangeInfo.Left;
					int right = rangeInfo.Right;
					int left = rangeInfo.Left;
					int target = left - LRRModel.ProjectModes.Count;
					if(target < DATA_COL_START || rangeInfo.Top != HEADER_ROW)
						return;

					gridControl.ResetVolatileData();
					gridControl.BeginUpdate();

					Cursor.Current = Cursors.WaitCursor;
					Cursor.Show();

					gridControl.Cols.MoveRange(left,range + 1,target);
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;

					InitializePropertyValues(gridControl,gridControlManager);

					gridControl.EndUpdate();
					Cursor.Hide();
					break;
				}
			}
		}
		
		/// <summary>
		/// Method moves the range of columns to right
		/// </summary>
		public void RightColMovement()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;


				if(gridControl.CurrentCell.IsActive)
				{
					GridRangeInfo rangeInfo = gridControl.Selections.Ranges.ActiveRange;
					int range = rangeInfo.Right - rangeInfo.Left;
					int right = rangeInfo.Right;
					int left = rangeInfo.Left;
					
					if(right + LRRModel.ProjectModes.Count > gridControl.ColCount)
						return;

					gridControl.ResetVolatileData();
					gridControl.BeginUpdate();
					Cursor.Current = Cursors.WaitCursor;
					Cursor.Show();
					
					int target = left + LRRModel.ProjectModes.Count;
					gridControl.Cols.MoveRange(left,range + 1,target);

                    
                    //_gridPersistance._arrGridColMove[0].Add(left);
                    //_gridPersistance._arrGridColMove[1].Add(range + 1);
                    //_gridPersistance._arrGridColMove[2].Add(target);

					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;

					InitializePropertyValues(gridControl,gridControlManager);

					gridControl.EndUpdate();
					Cursor.Hide();
					break;
				}
			}
		}
		
		/// <summary>
		/// Method Hides the Components from the grid
		/// </summary>
		/// <param name="arrayUnSelectedRecords"></param>
		public void HideComponents(ArrayList arrayUnSelectedRecords)
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;
				ArrayList colList = gridControlManager.ColHeader;
				for(int i=0;i < colList.Count;i++)
				{
					CellHeader colHeaderCell = colList[i] as CellHeader;
					if(arrayUnSelectedRecords.Contains(colHeaderCell.ID))
					{
						gridControl.ColHiddenEntries.Add(new GridColHidden(i + DATA_COL_START));
					}
				}
				gridControl.Refresh();
			}
		}
		
		/// <summary>
		/// Method Hides the Properties from the grid
		/// </summary>
		/// <param name="arrayUnSelectedRecords"></param>
		public void HideProperties(ArrayList arrayUnSelectedRecords)
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;
				gridControl.RowHiddenEntries.Clear();
				ArrayList rowList = gridControlManager.RowHeader;
				for(int i=0;i < rowList.Count;i++)
				{
					CellHeader rowHeaderCell = rowList[i] as CellHeader;
					rowHeaderCell.IsVisible = true;
					if(arrayUnSelectedRecords.Contains(rowHeaderCell.ID))
					{
						rowHeaderCell.IsVisible = false;
						gridControl.RowHiddenEntries.Add(new GridRowHidden(i + DATA_ROW_START));
					}
				}
				gridControl.Refresh();
			}
		}
		
		/// <summary>
		/// Method saves the Unified View
		/// </summary>
		/// <param name="name"></param>
		/// <param name="desc"></param>
		/// <param name="levelID"></param>
		public void Save(string name,string desc,string levelID)
		{
			LRRUnifiedViewSearchCrtiteria srchCriteria = new LRRUnifiedViewSearchCrtiteria();
            IEnumerator iEnumUVSearchCriteria = LRRModel.ChildModel.UnifiedViewSearchCrtiterias.GetEnumerator();// _SearchCriterias.GetEnumerator();
            string searchQueryID = null;
            string userID = this._lrrApp.DBManagerObj.AccessControlModel.UserID;
            string userSettingsID = null;
            while (iEnumUVSearchCriteria.MoveNext())
            {
                LRRUnifiedViewSearchCrtiteria srch = iEnumUVSearchCriteria.Current as LRRUnifiedViewSearchCrtiteria;
                if (srch.Name == name)
                {
                    DialogResult dlgResult = LRRMessagBox.ShowMessage("SID_QUERYNAME_DUPLICATE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dlgResult == DialogResult.No)
                    {
                        return;
                    }

                    searchQueryID = srch.ID;

                    userSettingsID = srch.UserSettingsID;

                    srch.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                    _lrrApp.DBManagerObj.CommitObject(srch);
                    //					_SearchCriterias.Remove(srch);
                    //Remove from Model
                    LRRUnifiedViewSearchCrtiteria srchCriteriaUV = LRRModel.ChildModel.UnifiedViewSearchCrtiterias.GetUnifiedViewByID(srch.ID);
                    if (srchCriteria != null)
                    {
                        userID = srchCriteriaUV.UserID;
                        srchCriteriaUV.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                        LRRModel.ChildModel.UnifiedViewSearchCrtiterias.Remove(srchCriteriaUV);

                        //Remove from Levels and Level Link tables
                        LRRSearchGroupLevel lrrSrchGroupLevel = new LRRSearchGroupLevel();
                        lrrSrchGroupLevel.LevelID = srchCriteriaUV.ID;
                        lrrSrchGroupLevel.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                        _lrrApp.DBManagerObj.CommitObject(lrrSrchGroupLevel);

                        LRRSearchGroupLevelLink lrrSrchGroupLevelLink = new LRRSearchGroupLevelLink();
                        _lrrApp.DBManagerObj.PopulateObject(lrrSrchGroupLevelLink, typeof(LRRSearchGroupLevelLink), " where ChildID = '" + srchCriteriaUV.ID + "'");

                        //Delete the Object
                        lrrSrchGroupLevelLink.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                        _lrrApp.DBManagerObj.CommitObject(lrrSrchGroupLevelLink);

                    }
                    break;
                }
            }

            IOScreen.GridPersistanceUtilUnified gutil = _gridPersistance;
            gutil.GridCollection = _pnlCollection;
            gutil.SaveGridSettings(false, false);


            LRRUserSetting userSettings = new LRRUserSetting(); ;
            if (userSettingsID == null)
            {
                userSettingsID = userSettings.ID;
            }
            else
            {
                _lrrApp.DBManagerObj.PopulateObject(userSettings, typeof(LRRUserSetting), " WHERE ID='" + userSettingsID + "'");
                userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                _lrrApp.DBManagerObj.CommitObject(userSettings);
            }

            srchCriteria.UserSettingsID = userSettingsID;

            userSettings.UserSettings = gutil.GetUserSettingsXmlBytes();
            userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            LRRModel.ChildModel.UnifiedViewSearchCrtiterias.Add(srchCriteria);


			srchCriteria.Name = name;
			srchCriteria.Description = desc;
			srchCriteria.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            srchCriteria.TemplateClass = "Symcon.LightRidge.PEAdvisor.CommonViews.GridSavedViewController";
			_lrrApp.DBManagerObj.CommitObject(srchCriteria);
            this.SearchQueryID = srchCriteria.ID;

            _lrrApp.DBManagerObj.CommitObject(userSettings, _lrrApp.DBManagerObj.AccessControlModel.UserID);
			
			LRRUnifiedViewSearchCrtiterias srchItems = new LRRUnifiedViewSearchCrtiterias();
			LRRUnifiedViewSearchItemConditions srchItemConditions = new LRRUnifiedViewSearchItemConditions();
            LRRSearchCriteriaUserSpecificUIUOMs srchCriteriaUserSpecificUIUOMs = new LRRSearchCriteriaUserSpecificUIUOMs();
            //LRRUnifiedViewSearchItems srcItems = new LRRUnifiedViewSearchItems();

            

			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
            
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;

				//Get the X-Axis
				ArrayList rowList = gridControlManager.RowList;
				for(int iRow = 0;iRow < rowList.Count;iRow++)
				{
					string lkUpPropertyID = null;
                    string lookUpQualifierID = null;
                    string masterPropID = null;
                    string lkUpQualifierID = null;
                    string rowHeaderID = null;
                    string siteComponentPropertyID = null;
                    
					SearchCriteria criteria = rowList[iRow] as SearchCriteria;

					LRRUnifiedViewSearchItem srchItem = new LRRUnifiedViewSearchItem();
					srchItem.Type = criteria.ObjType.ToString();
					srchItem.SearchCriteriaID = srchCriteria.ID;
					srchItems.Add(srchItem);
					ArrayList arrConditionList = criteria.ArrListSearchCondition;
                    for (int i = 0; i < arrConditionList.Count; i++)
                    {
                        SearchCondition condition = arrConditionList[i] as SearchCondition;
                        LRRUnifiedViewSearchItemCondition srchItemCondition = new LRRUnifiedViewSearchItemCondition();
                        srchItemCondition.SearchCriteriaID = srchCriteria.ID;
                        srchItemCondition.SearchCriteriaItemID = srchItem.ID;
                        srchItemCondition.Name = condition.Name;
                        srchItemCondition.ConditionValue = condition.Value;
                        srchItemCondition.Oper = condition.Operator;
                        srchItemConditions.Add(srchItemCondition);
                    }
                    if (criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
                    {
                        foreach (SearchCondition srchItemCondition in arrConditionList)
                        {
                            if (srchItemCondition.Name == "LookUpPropertiesID")
                                lkUpPropertyID = srchItemCondition.Value;
                            if (srchItemCondition.Name == "LookUpQualifierID")
                                lookUpQualifierID = srchItemCondition.Value;
                            if (srchItemCondition.Name == "MasterPropID")
                                masterPropID = srchItemCondition.Value;
                        }
                        LRRLookUpProperty lkupProp = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(lkUpPropertyID);

                        if (masterPropID != null)
                        {
                            rowHeaderID = masterPropID;
                        }

                        if (lookUpQualifierID != null && masterPropID != null && lkUpPropertyID != null)
                        {
                            LRRLookUpQualifier lrrLkUpQual = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
                            if (lrrLkUpQual != null)
                            {
                                LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);
                                rowHeaderID = qual.QualifierID + "|" + lkupProp.LookUpPropertiesID + "|" + masterPropID;
                            }
                        }

                        else if (masterPropID != null && lkUpPropertyID != null)
                        {
                            rowHeaderID = masterPropID + "|" + lkupProp.LookUpPropertiesID;
                        }
                    }
                    else if (criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
                    {
                        foreach (SearchCondition srchCondition in arrConditionList)
                        {
                            if (srchCondition.Name == "SiteComponentPropertyID")
                                siteComponentPropertyID = srchCondition.Value;
                            if (siteComponentPropertyID != null)
                            {
                                rowHeaderID = siteComponentPropertyID;
                            }
                        }
                    }

                    string changedUIUomID = CommonViews.ChangedUIUOMDictionary.RetrieveUOM(typeof(GridSavedViewController), rowHeaderID);
                    if (changedUIUomID != null)
                    {
                        //Save in the User Specific UIUOMs
                        LRRSearchCriteriaUserSpecificUIUOM srchCriteriaUserSpecificUIUOM = new LRRSearchCriteriaUserSpecificUIUOM();
                        srchCriteriaUserSpecificUIUOM.PropertyID = srchItem.ID;
                        srchCriteriaUserSpecificUIUOM.UIUomID = changedUIUomID;
                        srchCriteriaUserSpecificUIUOM.IsVisible = true;
                        srchCriteriaUserSpecificUIUOMs.Add(srchCriteriaUserSpecificUIUOM);
                    }
                }



				//Get the Y-Axis
				ArrayList colList = gridControlManager.ColList;
				for(int iCol = 0;iCol < colList.Count;iCol++)
				{
					SearchCriteria criteria = colList[iCol] as SearchCriteria;

					LRRUnifiedViewSearchItem srchItem = new LRRUnifiedViewSearchItem();
					srchItem.Type = criteria.ObjType.ToString();
					srchItem.SearchCriteriaID = srchCriteria.ID;
					srchItems.Add(srchItem);

					ArrayList arrConditionList = criteria.ArrListSearchCondition;
					for(int i=0;i < arrConditionList.Count;i++)
					{
						SearchCondition condition = arrConditionList[i] as SearchCondition;
						LRRUnifiedViewSearchItemCondition srchItemCondition = new LRRUnifiedViewSearchItemCondition();
						srchItemCondition.SearchCriteriaID = srchCriteria.ID;
						srchItemCondition.SearchCriteriaItemID = srchItem.ID;
						srchItemCondition.Name = condition.Name;
						srchItemCondition.ConditionValue = condition.Value;
						srchItemCondition.Oper = condition.Operator;
						srchItemConditions.Add(srchItemCondition);
					}
				}
			}
			
			CommonViews.ChangedUIUOMDictionary.Clear(typeof(GridSavedViewController));
			if(srchItems.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(srchItems);
			if(srchItemConditions.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(srchItemConditions);
            if (srchCriteriaUserSpecificUIUOMs.Count > 0)
                _lrrApp.DBManagerObj.CommitCollection(srchCriteriaUserSpecificUIUOMs);
            srchCriteriaUserSpecificUIUOMs.RefreshCollection();

			//Group Levels
			LRRSearchGroupLevel lrrLevel = new LRRSearchGroupLevel();
			lrrLevel.LevelID = srchCriteria.ID;
            lrrLevel.UserID = _lrrApp.DBManagerObj.AccessControlModel.UserID;

           

			lrrLevel.Name = name;
			lrrLevel.Description = desc;
			lrrLevel.Type = "UnifiedView";
//			lrrLevel.IsPublished = isPublished;

			LRRSearchGroupLevelLink lrrLvlLink = new LRRSearchGroupLevelLink();
			lrrLvlLink.ChildID = srchCriteria.ID;;
			lrrLvlLink.ParentID = levelID;
			lrrLvlLink.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
			_lrrApp.DBManagerObj.CommitObject(lrrLevel);
			_lrrApp.DBManagerObj.CommitObject(lrrLvlLink);


            string searchID = srchCriteria.ID;
            //IOScreenUtilities.Save(searchID, GridManagerHash, false);

            //IOScreen.GridPersistanceUtilUnified gutil = _gridPersistance;
            //gutil.GridCollection = _pnlCollection;
            //gutil.SaveGridSettings(false, false);

            //LRRUserSetting userSettings = new LRRUserSetting(); ;
            //if (SearchQuery.SearchQuery.UserSettingsID == null)
            //{
            //    SearchQuery.SearchQuery.UserSettingsID = userSettings.ID;
            //}
            //else
            //{
            //    _lrrApp.DBManagerObj.PopulateObject(userSettings, typeof(LRRUserSetting), " WHERE ID='" + SearchQuery.SearchQuery.UserSettingsID + "'");
            //    userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
            //    _lrrApp.DBManagerObj.CommitObject(userSettings);
            //}

            //userSettings.UserSettings = gutil.GetUserSettingsXmlBytes();
            //userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            //_lrrApp.DBManagerObj.CommitObject(userSettings, SearchQuery.SearchQuery.UserID);

            IOScreenUtilities.Save(searchID, GridManagerHash, false);
            LRRMessagBox.ShowMessage("SID_SAVE_VIEW_SUCCESSFULLY", true);
            //if (ApplicationPersistanceUtil.HtUserQueries.ContainsKey(searchID))
            //    ApplicationPersistanceUtil.HtUserQueries[searchID] = gutil.GctlSetting;
            //else
            //    ApplicationPersistanceUtil.HtUserQueries.Add(searchID, gutil.GctlSetting);
		}

		#endregion

		#region Constants
		private int COMPONENT_ROW = 1;
		private int COMPONENT_COL = 3;
		private int MODES_ROW = 2;
		private int PROPERTIES_ROW = 3;
		private int PROPERTIES_COL = 1;
		private int ATTRIBUTE_ROW = 1;
		private int ATTRIBUTE_COL = 1;
		private int HEADER_ROW = 0;
		private int HEADER_COL = 1;
		private int DATA_ROW_START = 3;
		private int DATA_COL_START = 3;
		private int COL_SPAN = 5;
		private const int FREEZE_ROW_START = 0;
		private const int FREEZE_ROW_END = 1;
		private const int FREEZE_COL_START = 0;
		private const int FREEZE_COL_END= 1;

		private int UOM_COL = 2;
		private int UOM_ROW = 1;
		#endregion 

		#region Fields
		private Hashtable _gridManagerHash = new Hashtable();
		private ArrayList _pnlCollection = new ArrayList();
		private LRRApp	  _lrrApp;
		//For saving Queries
		private UnifiedSearchCriteria   _searchCriteria = new UnifiedSearchCriteria();
        private string _searchCriteriaID;
        private GridPersistanceUtilUnified _gridPersistance = new GridPersistanceUtilUnified();

        private LRRSearchCriteriaUserSpecificUIUOMs _searchCriteriaUserSpecficUIUOMs = new LRRSearchCriteriaUserSpecificUIUOMs();
		#endregion

		#region Properties
        /// <summary>
        /// Gets or sets the grid manager hash.
        /// </summary>
        /// <value>The grid manager hash.</value>
		public Hashtable GridManagerHash
		{
			get{return _gridManagerHash;}
			set
			{
				_gridManagerHash = value;
			}
		}

        /// <summary>
        /// Gets or sets the grid persistance.
        /// </summary>
        /// <value>The grid persistance.</value>
        private GridPersistanceUtilUnified GridPersistance
        {
            get { return _gridPersistance; }
            set
            {
                _gridPersistance = value;
            }
        }
		#endregion
		
		#region Event Handlers

        /// <summary>
        /// Handles the ForeColorChanged event of the gridManager control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Symcon.LightRidge.PEAdvisor.CommonViews.ColorEventArgs"/> instance containing the event data.</param>
        private void gridManager_ForeColorChanged(object sender, ColorEventArgs e)
        {
            GridControlManager gridControlManager = sender as GridControlManager;
            IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator() as IDictionaryEnumerator;
            while (iDictEnum.MoveNext())
            {
                GridControlManager gcManager = iDictEnum.Value as GridControlManager;
                if (gcManager == gridControlManager)
                {
                    GridControl gridControl = iDictEnum.Key as GridControl;
                    IPropertyValue iPropValue = e.PropValueObj as IPropertyValue;
                    gridControl.Model[e.Row, e.Col].TextColor = iPropValue.GetForeColor();
                    break;
                }
            }
        }

        /// <summary>
        /// Handles the BackColorChanged event of the gridManager control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Symcon.LightRidge.PEAdvisor.CommonViews.ColorEventArgs"/> instance containing the event data.</param>
        private void gridManager_BackColorChanged(object sender, ColorEventArgs e)
        {
            GridControlManager gridControlManager = sender as GridControlManager;
            IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator() as IDictionaryEnumerator;
            while (iDictEnum.MoveNext())
            {
                GridControlManager gcManager = iDictEnum.Value as GridControlManager;
                if (gcManager == gridControlManager)
                {
                    GridControl gridControl = iDictEnum.Key as GridControl;
                    IPropertyValue iPropValue = e.PropValueObj as IPropertyValue;
                    gridControl.Model[e.Row, e.Col].BackColor = iPropValue.GetBackColor();
                    break;
                }
            }
        }

        /// <summary>
        /// Handles the CurrentCellCloseDropDown event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.PopupClosedEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CurrentCellCloseDropDown(object sender, Syncfusion.Windows.Forms.PopupClosedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			gridControl.CurrentCell.ConfirmChanges();
			gridControl.Refresh();
		}

        /// <summary>
        /// Handles the CurrentCellValidating event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CurrentCellValidating(object sender, System.ComponentModel.CancelEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			//Get the Grid Data
			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			if(gridControlManager == null)
				return;
			//Check for UOM
			if(iRow > UOM_ROW && iCol == UOM_COL)
			{
				//Change the Property Values according to UOM
				changePropertyValueForUOM(gridControl,gridControlManager,iRow,iCol);
			}
			//Check for Property Value 
			if(iRow >= DATA_ROW_START && iCol >= DATA_COL_START)
			{
				bool result = changePropertyValue(gridControl,gridControlManager,iRow,iCol);
				if(!result)
				{
					gridControl.CurrentCell.ResetCurrentCellWithoutDeactivate();
					gridControl.CurrentCell.Refresh();
					e.Cancel = true;
				}
			}
		}

        /// <summary>
        /// Handles the CurrentCellStartEditing event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CurrentCellStartEditing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] == null)
			{
				e.Cancel = true;
				return;
			}
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

			if(iRow < DATA_ROW_START || iCol < DATA_COL_START)
				return;

			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			//Get the row data
			CellHeader colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;
			CellHeader rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;

			//Get the Property Value Object
			object objPropValue = GetPropertyValueObject(rowData,colData);
			if(objPropValue != null)
			{
				IPropertyValue iPropValue = objPropValue as IPropertyValue;
				if(!iPropValue.GetEditableState())
					e.Cancel = true;
			}
			else
			{
				e.Cancel = true;
			}
				
		}

        /// <summary>
        /// Handles the CellClick event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridCellClickEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CellClick(object sender, GridCellClickEventArgs e)
		{
			if(e.MouseEventArgs.Button == MouseButtons.Left
				&& e.RowIndex == 1 && (e.ColIndex == 1 || e.ColIndex == 2))
			{
				GridControl gridControl = sender as GridControl;
				gridControl.ResetVolatileData();
				gridControl.BeginUpdate();

				Cursor.Current = Cursors.WaitCursor;
				Cursor.Show();
				if(GridManagerHash[gridControl] != null)
				{
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
					SortCol(e.ColIndex,gridControl,gridControlManager);
					InitializePropertyValues(gridControl,gridControlManager);

					//Hide Rows
					ArrayList rowList = gridControlManager.RowHeader;
					gridControl.RowHiddenEntries.Clear();
					for(int i=0;i < rowList.Count;i++)
					{
						CellHeader sortManager = rowList[i] as CellHeader;
						if(!sortManager.IsVisible)
						{
							gridControl.RowHiddenEntries.Add(new GridRowHidden(i + DATA_ROW_START));
						}
					}
				}
				gridControl.EndUpdate();
				Cursor.Hide();
			}
		}

        /// <summary>
        /// Handles the KeyDown event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_KeyDown(object sender, KeyEventArgs e)
		{
            // Commented to support Copy/Paste functionality By Ctrl+C Ctrl+V.
            //if(e.Shift || e.Control || e.Alt)
            //{
            //    e.Handled = true;
            //}

            try
            {
                bool isDelete = true;
                GridControl gridControl = sender as GridControl;

                if (e.KeyCode == Keys.Delete && !gridControl.CurrentCell.IsEditing)
                {
                    Syncfusion.Windows.Forms.Grid.GridRangeInfoList rangeInfoList = new GridRangeInfoList();
                    gridControl.Selections.GetSelectedRanges(out rangeInfoList, true);
                    int top = rangeInfoList.ActiveRange.Top;
                    int bottom = rangeInfoList.ActiveRange.Bottom;
                    int right = rangeInfoList.ActiveRange.Right;
                    int left = rangeInfoList.ActiveRange.Left;
                    if (top < DATA_ROW_START || right <= DATA_COL_START)
                        return;
                    GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
                    //Get the Row Data from Grid Data Manager
                    ArrayList rowSortList = gridControlManager.RowHeader;
                    //Get the Col Data from Grid Data Manager
                    ArrayList colHeaderCellList = gridControlManager.ColHeader;
                    //Get the row data
                    if (GridManagerHash[gridControl] == null)
                    {
                        e.Handled = true;
                        return;
                    }
                    for (int iRow = top; iRow <= bottom; iRow++)
                    {
                        for (int iCol = left; iCol <= right; iCol++)
                        {
                            CellHeader rowData = rowSortList[iRow - DATA_ROW_START] as CellHeader;
                            string rowCellID = rowData.ID;
                            ////Get the column data
                            CellHeader colData = colHeaderCellList[iCol - DATA_COL_START] as CellHeader;
                            Type objectType = colData.SearchCriteriaObj.ObjType;
                            string modeID = colData.ModeID;
                            string colCellID = colData.ID;

                            //Get the Property Value Object
                            //object objPropValue = LRRModel.GetPropertyValueObject(objectType, modeID, rowCellID, colCellID);
                            object objPropValue = GetPropertyValueObject(rowData, colData);
                            if (objPropValue != null)
                            {
                                IPropertyValue iPropValue = objPropValue as IPropertyValue;
                                if (!iPropValue.GetEditableState())
                                {
                                    isDelete = false;
                                    e.Handled = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (isDelete)
                    {
                        DialogResult dlgResult = Symcon.Utils.MessageDispatcher.LRRMessagBox.ShowMessage("SID_DELETE_VALUE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dlgResult == DialogResult.No)
                        {
                            e.Handled = true;
                        }
                        else
                        {
                            for (int iRow = top; iRow <= bottom; iRow++)
                            {
                                for (int iCol = left; iCol <= right; iCol++)
                                {
                                    cascadeChangesToPropValues(gridControl, gridControlManager, iRow, iCol);
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(IOScreenUtilities), ex.Message, Logger.Level.Debug);
            }
		}

        /// <summary>
        /// Handles the RowsMoving event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridRangeMovingEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_RowsMoving(object sender, GridRangeMovingEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(e.Target < DATA_ROW_START || e.From < DATA_ROW_START)
			{
				e.Cancel = true;
				return;
			}
            
            _gridPersistance._arrGridRowMove[0].Add(e.From);
            _gridPersistance._arrGridRowMove[1].Add(e.Count);
            _gridPersistance._arrGridRowMove[2].Add(e.Target);
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;

				int oldIndex = e.From;
				int newIndex = e.Target;
				int countOFItems = e.Count;
				ArrayList sortManagerList = gridControlManager.RowHeader;
				int rowStart = DATA_ROW_START;

				//Update the Row List
				int toMoveIndex = -1;
				int lastInsertIndex = -1;

				//Upward Movement
				if(oldIndex > newIndex)
				{
					toMoveIndex = oldIndex + (countOFItems - 1);
					lastInsertIndex = newIndex;
				}
				else//Downward Movement
				{
					toMoveIndex = oldIndex;
					lastInsertIndex = newIndex + countOFItems - 1;
				}

				for(int i=0;i<countOFItems;i++)
				{
					//Remove 
					CellHeader sortManager = (CellHeader)sortManagerList[toMoveIndex - rowStart];
					//Update the new indexes
					sortManagerList.RemoveAt(toMoveIndex - rowStart);
					sortManagerList.Insert(lastInsertIndex - rowStart,sortManager);
				}
			}
		}

        /// <summary>
        /// Handles the RowsMoved event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridRangeMovedEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_RowsMoved(object sender, GridRangeMovedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
				InitializePropertyValues(gridControl,gridControlManager);
				if(gridControl.CurrentCell.IsEditing)
					gridControl.CurrentCell.EndEdit();
				//turn off the sort arrow
				for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				{
					if(gridControl[1, col1].CellType == "ColumnHeaderCell")
					{
						gridControl[1, col1].CellType = "Header";
						break;
					}
				}
			}
		}

        /// <summary>
        /// Handles the ColsMoving event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridRangeMovingEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_ColsMoving(object sender, GridRangeMovingEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
				 
				int oldIndex = e.From;
				int newIndex = e.Target;
				int countOFItems = e.Count;
				ArrayList colList = gridControlManager.ColHeader;
				int colStart = DATA_COL_START;

				//Update the Row List
				int toMoveIndex = -1;
				int lastInsertIndex = -1;

				//Upward Movement
				if(oldIndex > newIndex)
				{
					toMoveIndex = oldIndex + (countOFItems - 1);
					lastInsertIndex = newIndex;
				}
				else//Downward Movement
				{
					toMoveIndex = oldIndex;
					lastInsertIndex = newIndex + countOFItems - 1;
				}

				for(int i=0;i<countOFItems;i++)
				{
					//Remove 
					CellHeader colHeaderCell = (CellHeader)colList[toMoveIndex - colStart];
					//Update the new indexes
					colList.RemoveAt(toMoveIndex - colStart);
					colList.Insert(lastInsertIndex - colStart,colHeaderCell);
				}
			}
            _gridPersistance._arrGridColMove[0].Add(e.From);
            _gridPersistance._arrGridColMove[1].Add(e.Count);
            _gridPersistance._arrGridColMove[2].Add(e.Target);
		}

        /// <summary>
        /// Handles the Paint event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.PaintEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_Paint(object sender, PaintEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gcManager = GridManagerHash[gridControl] as GridControlManager;
				GridStyleInfo styleInfo = gridControl.BaseStylesMap["Column Header"].StyleInfo;
				int x = gridControl.RowIndexToVScrollPixelPos(HEADER_ROW);
				int y = gridControl.ColIndexToHScrollPixelPos(PROPERTIES_COL - 1);
				int height = gridControl.RowHeights.GetSize(HEADER_ROW);
				StringFormat sFormat = new StringFormat();
				sFormat.Alignment = StringAlignment.Near;
				sFormat.LineAlignment = StringAlignment.Center;
				Graphics g = e.Graphics;
				int width = gridControl.Width;
				g.DrawString(gcManager.HeaderText,styleInfo.GdipFont,new SolidBrush(styleInfo.TextColor),new RectangleF(x,y,width,height),sFormat);
			}
		}

        /// <summary>
        /// Handles the SelectionChanged event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridSelectionChangedEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_SelectionChanged(object sender, GridSelectionChangedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(e.Range.Top == HEADER_ROW && e.Range.Left >= DATA_COL_START)
			{
				if(e.Reason != GridSelectionReason.MouseMove)
					return;
				int iModes = 0;

				//Get the Selected Modes
				ArrayList arrModesSelected = new ArrayList();
				foreach(LRRMode mode in LRRModel.ProjectModes)
				{
					if(mode.ViewSelection)
					{
						iModes++;
						arrModesSelected.Add(mode);
					}
				}
				
				string firstSelModeID = null;
				string lastSelModeID  = null;
				if(arrModesSelected.Count > 0)
				{
					LRRMode firstSelMode = arrModesSelected[0] as LRRMode;
					LRRMode lastSelMode  = arrModesSelected[arrModesSelected.Count - 1] as LRRMode;

					firstSelModeID = firstSelMode.ModeId;
					lastSelModeID = lastSelMode.ModeId;
				}
				
				//Check for Column Movement
				if(GridManagerHash[gridControl] != null)
				{
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
					ArrayList colHeaderCellList = gridControlManager.ColHeader;

					CellHeader firstColHdrCell = colHeaderCellList[e.Range.Left - DATA_COL_START] as CellHeader;
					CellHeader lastColHdrCell = colHeaderCellList[e.Range.Right - DATA_COL_START] as CellHeader;

					if(firstColHdrCell.ModeID == firstSelModeID && lastColHdrCell.ModeID == lastSelModeID)
					{
						LRRMode firstMode = LRRModel.ProjectModes[0];
						LRRMode lastMode = LRRModel.ProjectModes[LRRModel.ProjectModes.Count - 1];
						
						int leftIncr = Int32.Parse(firstSelModeID,Symcon.BusinessObjects.SymConverter.Culture) - Int32.Parse(firstMode.ModeId,Symcon.BusinessObjects.SymConverter.Culture);
						int rightIncr = Int32.Parse(lastMode.ModeId,Symcon.BusinessObjects.SymConverter.Culture ) - Int32.Parse(lastSelModeID,Symcon.BusinessObjects.SymConverter.Culture);
						int leftIndex = e.Range.Left - leftIncr;
						int rightIndex = e.Range.Right + rightIncr;
						
						gridControl.Selections.ChangeSelection(gridControl.Selections.Ranges.ActiveRange,GridRangeInfo.Cells(HEADER_ROW,leftIndex,gridControl.RowCount,rightIndex),false);

						_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = true;
						_lrrApp.MainFrame.BarItemRightColMovement.Enabled = true;	

						return;
					}
					else
					{
						_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = false;
						_lrrApp.MainFrame.BarItemRightColMovement.Enabled = false;
					}
				}
			}
			else
			{
				_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = false;
				_lrrApp.MainFrame.BarItemRightColMovement.Enabled = false;
			}
		}

        /// <summary>
        /// Handles the DataConfidenceValueChanged event of the iPropValue control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void iPropValue_DataConfidenceValueChanged(object sender, EventArgs e)
		{
			
			IPropertyValue iPropValue = sender as IPropertyValue;
			string modeID = iPropValue.GetModeID();
			string propertyID = iPropValue.GetPropertyID();
            if (iPropValue.GetType() == typeof(LRRAggregatePropertyValue))
            {
                LRRAggregateOverride lrrModelOverride = LRRModel.ChildModel.AggregateOverrides.GetOverrideObjectByAggPropertyAndModeID(propertyID, modeID);
                if (lrrModelOverride != null)
                {
                    lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();
                    _lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);
                }
                //Update Overrides
                _lrrApp.UpdateAggregateOverridePoints(); 
            }
				else if(iPropValue.GetType() == typeof(LRRAggregateGroupedPropertyValue))
				{
					LRRAggGroupOverride lrrModelOverride =LRRModel.ChildModel.GroupAggOverrides.GetOverrideObjectByAggPropertyAndModeID(propertyID, modeID);
					if (lrrModelOverride != null)
                {
                    lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();
                    _lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);
                }
                //Update Overrides
                _lrrApp.UpdataGrpAggOverridePoints();

				}
				else if(iPropValue.GetType() == typeof(LRRSiteComponentGroupPropertyValue))
				{
					LRRGroupOverride  lrrModelOverride =LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(propertyID);
					if (lrrModelOverride != null)
                {
                    lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();
                    _lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);
                }
                //Update Overrides
                _lrrApp.UpdateGrpOverridePoints();
				}
            else
            {
                LRROverride lrrModelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(propertyID, modeID);
                if (lrrModelOverride != null)
                {
                    lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();
                    _lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);
                }
                //Update Overrides
                _lrrApp.UpdateOverridePoints();
            }
			
		}

        SetRecordedTrendedForm trendForm = new SetRecordedTrendedForm(true);
        public static bool _calledFromTrendedWindow = false;
        public static bool CalledFromTrendedWindow
        {
            get { return _calledFromTrendedWindow; }
            set
            {
                _calledFromTrendedWindow = value;
            }
        }
        private void trendValue(object sender, EventArgs e)
        {
            MenuItem miItem = sender as MenuItem;
            GridControl gridControl = miItem.Parent.GetContextMenu().SourceControl as GridControl;
            if (GridManagerHash[gridControl] == null)
                return;

            GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
            //Get the Col Data from Grid Data Manager
            ArrayList compHeaderCellList = gridControlManager.ColHeader;
            ArrayList attributeHeaderCellList = gridControlManager.RowHeader;
            trendForm.Text = "Trended Components Window :";
            CalledFromTrendedWindow = true;
            trendForm.BindTree(compHeaderCellList, attributeHeaderCellList, gridControlManager.HeaderText,true);

            trendForm.ShowDialog();
        }

        SetRecordedTrendedForm recordForm = new SetRecordedTrendedForm(false);
        private void recordValue(object sender, EventArgs e)
        {
            MenuItem miItem = sender as MenuItem;
            GridControl gridControl = miItem.Parent.GetContextMenu().SourceControl as GridControl;
            if (GridManagerHash[gridControl] == null)
                return;

            GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
            ArrayList compHeaderCellList = gridControlManager.ColHeader;
            ArrayList attributeHeaderCellList = gridControlManager.RowHeader;
            recordForm.Text = "Recorded Components Window :";
            CalledFromTrendedWindow = false;
            recordForm.BindTree(compHeaderCellList, attributeHeaderCellList, gridControlManager.HeaderText,false);
            recordForm.ShowDialog();
            
        }

        /// <summary>
        /// Handles the Popup event of the contMenu control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void contMenu_Popup(object sender, EventArgs e)
		{
			ContextMenu contMenu = sender as ContextMenu;
			contMenu.MenuItems.Clear();

			GridControl gridControl = contMenu.SourceControl as GridControl;
			if(gridControl == null)
				return;
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			CellHeader colData = new CellHeader();
			CellHeader rowData = new CellHeader();

			//Get the GridData
			if(GridManagerHash[gridControl] == null)
				return;
			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			
			//Record Properties Menu
			//if(iCol == PROPERTIES_COL && iRow > DATA_ROW_START - 1)
			
			if(iRow == COMPONENT_ROW && iCol > PROPERTIES_COL && iCol > UOM_COL)
			{
				MenuItem miItem = new MenuItem(LRRHeaderText.GetHeaderString("SID_RECORDE_PROP_VALUE"),new EventHandler(recordValue));
				//				GetObjectData(gridControlManager,iRow + DATA_ROW_START - 1,iCol,out colData,out rowData);
				//				miItem.Checked = rowData.IsRecorded;
				contMenu.MenuItems.Add(miItem);

				MenuItem miItemTrend = new MenuItem(LRRHeaderText.GetHeaderString("SID_TREND_PROP_VALUE"),new EventHandler(trendValue));
				contMenu.MenuItems.Add(miItemTrend);
			}

			//Property Values Overrides Menu
			if(iRow >= DATA_ROW_START && iCol >= DATA_COL_START)
			{
				//Get the GridData
				rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;
				colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;
				object objPropValue = GetPropertyValueObject(rowData,colData);

				if(objPropValue != null)
				{
//					IProperty iProp = LRRModel.GetPropertyObject(colData.ObjectType,rowData.ID,colData.ID) as IProperty;
					IPropertyValue iPropValue = objPropValue as IPropertyValue;

					//					if(iPropValue.GetPropertyValue() != null && iPropValue.GetPropertyValue() != "")
					//					{
					
					MenuItem miItemTrend = new MenuItem(LRRHeaderText.GetHeaderString("SID_VIEW_TREND"));
					//miItemTrend.IsParent = true;
                    foreach (string schemeName in UserControls.UserControlTrendOptions.HtSchemes.Keys)
                    {
                        MenuItem miItemTrendChild = new MenuItem(schemeName, new EventHandler(viewTrend));
                        miItemTrend.MenuItems.Add(miItemTrendChild);
                    }
					/*
					foreach(string schemeName in UserControls.UserControlTrendOptions.HtSchemes.Keys)
					{
						MenuItem miItemTrendChild = new MenuItem(schemeName,new EventHandler(viewTrend));
						miItemTrend.MenuItems.Add(miItemTrendChild);
					}*/
					miItemTrend.Index = 0;//6;
					contMenu.MenuItems.Add(miItemTrend);
					//					}

					if(!iPropValue.GetEditableState())
					{
						ProcessMenuItems(iPropValue,contMenu);
						return;
					}

					//Override Values Menu
					MenuItem miItem = new MenuItem(LRRHeaderText.GetHeaderString("SID_OVERRIDE_VAL_OFF"),new EventHandler(overrideValue));
					miItem.Index = 1;
					contMenu.MenuItems.Add(miItem);

					//Uncertain Input Menu
					MenuItem miItemUncertainInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_UNCERTAIN_INPUT"),new EventHandler(overrideValue));
					miItemUncertainInput.Index = 2;
					contMenu.MenuItems.Add(miItemUncertainInput);

					//Trusted Input Menu
					MenuItem miItemTrustedInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_TRUSTED_INPUT"),new EventHandler(overrideValue));
					miItemTrustedInput.Index = 3;
					contMenu.MenuItems.Add(miItemTrustedInput);

					//TempTrusted Input Menu
					MenuItem miItemTempTrustedInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_TEMP_TRUSTER_INPUT"),new EventHandler(overrideValue));
					miItemTempTrustedInput.Index = 4;
					contMenu.MenuItems.Add(miItemTempTrustedInput);

					//Uncertain Override Menu
					MenuItem miItemUncertainOverride = new MenuItem(LRRHeaderText.GetHeaderString("SID_UNCERTAIN_OVERRIDE"),new EventHandler(overrideValue));
					miItemUncertainOverride.Index = 5;//3;
					contMenu.MenuItems.Add(miItemUncertainOverride);

					//Trusted Override Menu
					MenuItem miItemTrustedOverride = new MenuItem(LRRHeaderText.GetHeaderString("SID_TRUSTED_OVERRIDE"),new EventHandler(overrideValue));
					miItemTrustedOverride.Index = 6;//4;
					contMenu.MenuItems.Add(miItemTrustedOverride);

					//TempTrusted Override Menu
					MenuItem miItemTempTrusted = new MenuItem(LRRHeaderText.GetHeaderString("SID_TEMP_TRUSTER_OVERRIDE"),new EventHandler(overrideValue));
					miItemTempTrusted.Index = 6;//5;
					contMenu.MenuItems.Add(miItemTempTrusted);

					ProcessMenuItems(iPropValue,contMenu);
				}
			}
		}

        public void viewTrend(object sender, EventArgs e)
        {
            MenuItem miItem = sender as MenuItem;
            string trendScheme = miItem.Text;
            ContextMenu contMenu = miItem.GetContextMenu();
            GridControl gridControl = contMenu.SourceControl as GridControl;

            if (GridManagerHash[gridControl] == null)
                return;

            int iRow = gridControl.CurrentCell.RowIndex;
            int iCol = gridControl.CurrentCell.ColIndex;

            CellHeader colData = new CellHeader();
            CellHeader rowData = new CellHeader();

            GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
            rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;
            colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;
            object objPropValue = GetPropertyValueObject(rowData, colData);

            if (objPropValue != null)
            {
                IPropertyValue iPropValue = objPropValue as IPropertyValue;
                if (iPropValue is LRRSiteGroupPropertyQualifierValue)
                    _lrrApp.LRRUIManagerObj.ShowTrend(trendScheme, iPropValue, rowData.UIUOMId);
                else if( iPropValue is LRRSiteComponentPropertyValue)
                {
                    LRRSiteComponentPropertyValue scpv = iPropValue as LRRSiteComponentPropertyValue;
                    string scpID = scpv.SiteComponentPropertyID;
                    LRRSiteComponentProperty scp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(scpID);
                    if (scp != null)
                    {
                        _lrrApp.LRRUIManagerObj.ShowTrend(trendScheme, scp as IProperty,colData.ModeID, rowData.UIUOMId);
                    }
                }
            }
        }

        /// <summary>
        /// Overrides the value.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		public void overrideValue(object sender, EventArgs e)
		{
			MenuItem miItem = sender as MenuItem;
			ContextMenu contMenu = miItem.GetContextMenu();
			GridControl gridControl = contMenu.SourceControl as GridControl;
            
			if(GridManagerHash[gridControl] == null)
				return;

			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			CellHeader colData = new CellHeader();
			CellHeader rowData = new CellHeader();

			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;
			colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;
			object objPropValue = GetPropertyValueObject(rowData,colData);
            //object objPropValue = GetPropertyValueObject(colData, rowData);

			if(objPropValue != null)
			{
				IPropertyValue iPropValue =  objPropValue as IPropertyValue;
//				IProperty iProp = null;

				//Check for Menu item Index
				switch(miItem.Index)
				{
					//Override Value On/Off
					case 1:
						iPropValue.SetOverrideState(false);
						SearchCriteria criteria = rowData.SearchCriteriaObj;
						if(colData.ModeID == "1")
						{
							_lrrApp.SwapBindings(objPropValue,false);
							iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
						}
						else
						{
							if(colData.ModeID == "2")
							{
								_lrrApp.SwapBindings(objPropValue,false);
								iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
							}
							else
							{
								_lrrApp.SwapBindings(objPropValue,false);
								iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
							}
							if(criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
								_lrrApp.RemoveCascadedBindings(objPropValue,typeof(LRRSiteComponent),rowData.ID,colData.ID,false);
							if(criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
							{
								if(objPropValue.GetType()==typeof(LRRAggregateGroupedPropertyValue))
								{
									LRRAggregateGroupedPropertyValue lrrAggGrpPropVal =  objPropValue as LRRAggregateGroupedPropertyValue;
									//_lrrApp.RemoveCascadedBindings(objPropValue,typeof(LRRAggregateGroupedPropertyValue),rowData.ID,colData.ID,true);
									_lrrApp.RemoveCascadedBindings(lrrAggGrpPropVal,lrrAggGrpPropVal.AggregateGroupPropertyID,lrrAggGrpPropVal.LookUpPropertiesID,lrrAggGrpPropVal.QualifierID,false);
								}
								else
								{ 
									LRRSiteGroupPropertyQualifierValue qualObj = objPropValue as LRRSiteGroupPropertyQualifierValue;
								_lrrApp.RemoveCascadedBindings(objPropValue,qualObj.GroupPropertyID,qualObj.LookUpPropertiesID,qualObj.QualifierID,false);

								}
								
							}
							//iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
						}
						
						//Set the Cell tool Tip
						SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],rowData.UIUOMId);

						//Get the Property Object
						if(criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
						{
							object objProp = LRRModel.GetPropertyObject(typeof(LRRSiteComponent),rowData.ID,colData.ID);
							if(objProp != null)
							{
								//Populate Overrides Collection	
								IProperty iProp = objProp as IProperty;
								string propID = iProp.GetID();
								PopulateOverrides(iPropValue,propID,colData.ID,colData.ModeID,rowData.UOMId);
							}
						}
						else
						{
							LRRSiteGroupPropertyQualifierValue qualObj = objPropValue as LRRSiteGroupPropertyQualifierValue;
							if(objPropValue.GetType()==typeof(LRRAggregateGroupedPropertyValue))
							{
								LRRAggregateGroupedPropertyValue objGroupPropValue = objPropValue as LRRAggregateGroupedPropertyValue;
								PopulateAggGroupOverrides(iPropValue,objGroupPropValue.AggregateGroupPropertyID,objGroupPropValue.QualifierID);
							}
							else
							{
								PopulateGroupOverrides(iPropValue,qualObj.GroupPropertyID,qualObj.QualifierID);
							}
						}

                        if (gridControl.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.CommonViews.SavedViewForm))
                            SavedViewForm.BPropertyValChanged = true;
                        else if (gridControl.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.CommonViews.UnifiedViewResultForm))
                            UnifiedViewResultForm.BUnifiedResultFormChange = true;
						break;
					//Uncertain Input
					case 2:
						iPropValue.SetTrustedState("0");
						break;
					//Trusted Input
					case 3:
						iPropValue.SetTrustedState("1");
						break;
					//Temp Trusted Input
					case 4:
						iPropValue.SetTrustedState("2");
						break;
					//Uncertain Override
					case 5://3:
						iPropValue.SetDataConfidenceLevelID("3");
						iPropValue.SetTrustedState("-1");
						break;
					//Trusted Override
					case 6://4:
						iPropValue.SetDataConfidenceLevelID("1");
						iPropValue.SetTrustedState("-1");
						break;
					//TempTrusted Override
					case 7://5:
						iPropValue.SetDataConfidenceLevelID("2");
						iPropValue.SetTrustedState("-1");
						break;
				}

				ProcessMenuItems(iPropValue,contMenu);
				//Set the Color
				SetColor(iPropValue,gridControl.Model[iRow,iCol]);
			}
		}

        /// <summary>
        /// Bindings the manager_ binding called.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="target">The target.</param>
		private void bindingManager_BindingCalled(object source, object target)
		{
			object src = source;
			object trg = target;
			GridStyleInfo gsInfo = target as GridStyleInfo;
			GridControl gridControl = gsInfo.GetActiveGridView() as GridControl;

			int iRow = gsInfo.CellIdentity.RowIndex;
			int iCol = gsInfo.CellIdentity.ColIndex;

			//Get the Grid Data
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
				
				//Get the row data
				CellHeader rowData = new CellHeader();
				//Get the column data
				CellHeader colData = new CellHeader();

                //Get the Property Value Object
                rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;
                colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;

				//Get the row data
				string rowCellID = rowData.ID;
				string uiUOMID   = rowData.UIUOMId;
				//				string uomID     = rowData.UOMID;

				//Get the column data
				string modeID   = colData.ModeID;
				string colCellID = colData.ID;


				object objPropValue = GetPropertyValueObject(rowData,colData);

				//Convert and put Property Values in grid cells
				if(objPropValue != null)
				{
					IPropertyValue iPropValue = objPropValue as IPropertyValue;
					string uomID = iPropValue.GetUOMID();
					//Convert property value according to UIUOM 
                    // commented by ashwin
					//string propertyValue = iPropValue.GetActualPropertyValue();
                    string propertyValue = iPropValue.GetPropertyValue();
					if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
					{
						propertyValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,propertyValue);
					}

                    string lrrDataType = iPropValue.GetDataType();
                    if (lrrDataType == null || lrrDataType.Trim() == "")
                    {
                        propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                    }
					//					gridControl.Model[iRow,iCol].Text = propertyValue;
					gsInfo.Text = propertyValue;
					SetCellToolTipText(iPropValue,gsInfo,uiUOMID);
				
					//Set the Color
					SetColor(iPropValue,gridControl.Model[iRow,iCol]);
				}
			}
		}

        private void Model_ClipboardPaste(object sender, Syncfusion.Windows.Forms.Grid.GridCutPasteEventArgs e)
        {
            GridModel gridControlModel = sender as GridModel;
            gridControlModel.CurrentCellInfo.GridView.CurrentCell.EndEdit();

            GridControl gridControl = gridControlModel.CurrentCellInfo.GridView as GridControl;

            if (GridManagerHash[gridControl] == null)
            {
                return;
            }

            GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;

            IDataObject data = Clipboard.GetDataObject();

            if (data.GetDataPresent(typeof(string)))
            {
                string s = (string)data.GetData(typeof(string));

                string[] sRows = s.Split(new string[] { "\r\n" }, StringSplitOptions.None);
                
                GridRangeInfoList rangeList;
                if (gridControl.Selections.GetSelectedRanges(out rangeList, true))
                {
                    foreach (GridRangeInfo range in rangeList)
                    {
                        int sRowIndex = 0;
                        for (int iRow = range.Top; iRow <= range.Bottom; iRow++)
                        {
                            if (sRowIndex < sRows.Length)
                            {
                                string sRow = sRows[sRowIndex++];
                                int sCellInRowIndex = 0;
                                string[] sCells = sRow.Split(new char[] { '\t' });
                                for (int iCol = range.Left; iCol <= range.Right; iCol++)
                                {
                                    if (sCellInRowIndex < sCells.Length)
                                    {
                                        string sCell = sCells[sCellInRowIndex++];

                                        if (iRow < DATA_ROW_START || iCol < DATA_COL_START)
                                            continue;


                                        //Get the row data
                                        CellHeader colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;
                                        CellHeader rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;

                                        string modeId = colData.ModeID;
                                        LRRMode mode = LRRModel.Modes.GetModeByID(modeId);

                                        if (!mode.ViewSelection)
                                        {
                                            sCellInRowIndex--;
                                            continue;
                                        }

                                        //Get the Property Value Object
                                        object objPropValue = GetPropertyValueObject(rowData, colData);
                                        if (objPropValue != null)
                                        {
                                            IPropertyValue iPropValue = objPropValue as IPropertyValue;
                                            if (!iPropValue.GetEditableState())
                                                continue;
                                        }
                                        else
                                        {
                                            continue;
                                        }

                                        gridControl[iRow, iCol].Text = sCell;
                                        changePropertyValue(gridControl, gridControlManager, iRow, iCol, sCell);
                                    }
                                }
                            }
                        }
                    }
                    e.Result = true;
                    e.Handled = true;
                }
            }
        }

		#endregion
	}
}
