using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.BusinessObjects; 
using Symcon.Utils.MessageDispatcher;

namespace Symcon.LightRidge.PEAdvisor.CommonViews
{
	/// <summary>
	/// Summary description for UnifiedViewResultForm.
	/// </summary>
	public class UnifiedViewResultForm : System.Windows.Forms.Form,ILRRUIContainer,ILRRUISpecificContainer
	{

		
		private TabControl			_parentTabControl;
		private TabPage				_parentTabPage;
		private System.Windows.Forms.Panel panel1;
        public static bool BUnifiedResultFormChange = false;
        private LRRApp _lrrApp;


		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public UnifiedViewResultForm()
		{
			InitializeComponent();
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		}

        public UnifiedViewResultForm(IUnifiedGridFactory gridView, LRRResultProperties resultComponents, LRRResultProperties resultAttributes)
		{
            InitForm(gridView, resultComponents, resultAttributes);
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		}

        public UnifiedViewResultForm(string lrrUnifiedViewID)
        {
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            LRRUnifiedViewSearchCrtiteria lrrUnifiedView = LRRModel.ChildModel.UnifiedViewSearchCrtiterias.GetUnifiedViewByID(lrrUnifiedViewID);
            OpenUnifiedView(lrrUnifiedView);
            this.Text = Symcon.Utils.HeaderDispatcher.LRRHeaderText.GetHeaderString("SID_RESULT") + "-" + "[" + lrrUnifiedView.Name + "]";
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(UnifiedViewResultForm));
			this.panel1 = new System.Windows.Forms.Panel();
			this.SuspendLayout();
			// 
			// panel1
			// 
			this.panel1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panel1.Location = new System.Drawing.Point(0, 0);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(832, 582);
			this.panel1.TabIndex = 2;
			// 
			// GroupedPlantPropertiesResultForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(832, 582);
			this.Controls.Add(this.panel1);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "GroupedPlantPropertiesResultForm";
			this.Text = "Result";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.ResumeLayout(false);

		}
		#endregion

        #region Private Methods

        private void OpenUnifiedView(LRRUnifiedViewSearchCrtiteria lrrUnifiedView)
        {
            ArrayList arrXCollection = new ArrayList();
            ArrayList arrYCollection = new ArrayList();

            CommonViewUtilities.GetCollection(lrrUnifiedView.ID, out arrXCollection, out arrYCollection);

            string templateClass = lrrUnifiedView.TemplateClass;
            if (templateClass == null || templateClass.Trim() == "")
            {
                templateClass = "Symcon.LightRidge.PEAdvisor.CommonViews.GridSavedViewController";
            }
            string searchCriteriaID = lrrUnifiedView.ID;
            Type gridType = Type.GetType(templateClass);
            object gridView = Activator.CreateInstance(gridType);
            GridView = gridView as IUnifiedGridFactory;
            GridView.SearchQueryID = lrrUnifiedView.ID;
            IUnifiedGridFactory objGrid = gridView as IUnifiedGridFactory;

            LRRUnifiedViewSearchItemConditions searchItemConditions = new LRRUnifiedViewSearchItemConditions();
            objGrid.LoadPanel(arrXCollection, arrYCollection);

            LRRResultProperties resultComponents = CommonViewUtilities.GetSiteComponents(arrXCollection);
            LRRResultProperties resultAttributes = CommonViewUtilities.GetPropeties(arrYCollection);

            ArrayList arrParams = new ArrayList();
            arrParams.Add(objGrid);
            arrParams.Add(resultComponents);
            arrParams.Add(resultAttributes);
            Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory unifiedGridView = arrParams[0] as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
            InitForm(unifiedGridView, resultComponents, resultAttributes);
        }

        private void InitForm(IUnifiedGridFactory gridView, LRRResultProperties resultComponents, LRRResultProperties resultAttributes)
        {
            InitializeComponent();
            GridView = gridView;
            ArrayList pnlCollection = gridView.GetPanelCollection();
            SetPanel(pnlCollection);
            gridView.HideModes();
            LRRModel.ProjectModes.ModesChanged += new EventHandler(ProjectModes_ModesChanged);

            _rsResultAttributesWindow = new ResultWindow();
            _rsResultAttributesWindow.PropertyCollection = resultAttributes;
            _rsResultAttributesWindow.Text = "Result Attributes Window";
            _rsResultAttributesWindow.BindGrid();
            _rsResultAttributesWindow.SelectedProperties += new Symcon.LightRidge.PEAdvisor.CommonViews.ResultWindow.SelectedPropertiesChanged(_rsResultAttributesWindow_SelectedProperties);

            //Show the Component List Form
            _rsResultComponentsWindow = new ResultWindow();
            _rsResultComponentsWindow.PropertyCollection = resultComponents;
            _rsResultComponentsWindow.Text = "Result Components Window";
            _rsResultComponentsWindow.BindGrid();
            _rsResultComponentsWindow.SelectedProperties += new Symcon.LightRidge.PEAdvisor.CommonViews.ResultWindow.SelectedPropertiesChanged(_rsResultComponentsWindow_SelectedProperties);
            
            //Form closing event for UnifiedViews Result form
            this.FormClosing += new FormClosingEventHandler(UnifiedViewResultForm_FormClosing);
            //btnExec.Click+=new EventHandler(inOutForm.btnUpdateModel_Click);
        }

        public bool _overrideRemoved = true;

        public void ProcessRollBackforPropertyValues(LRRSiteComponentPropertyValues scpvs)
        {
            foreach (LRRSiteComponentPropertyValue lscpv in scpvs)
            {
                
                LRROverride overrideobj = LRRModel.ChildModel.Overrides.GetChanges().GetOverrideObjectByPropertyAndModeID(lscpv.SiteComponentPropertyID, lscpv.ModeID);
                if (overrideobj != null) 
                {
                    LRRModel.ChildModel.Overrides.RollBackChanges(overrideobj);
                    lscpv.RollBackChanges(true);
                    /*
                    if (!_lrrApp.RollBackPVOverridesGSVC.Contains(overrideobj) && !_lrrApp.RollBackPVOverridesGTSVC.Contains(overrideobj))
                    {
                        _overrideRemoved = false;
                        lscpv.SetOverrideState(false);
                        overrideobj.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                        _lrrApp.DBManagerObj.CommitObject(overrideobj);
                        LRRModel.ChildModel.Overrides.Remove(overrideobj);
                        _lrrApp.RemoveCascadedBindings(lscpv, false);
                        lscpv.SetPropertyValue(lscpv.GetActualOverridedValue());
                    }
                     * */
                }
                /*
                if (GridSavedViewController.RollBackSettingsHash.Count > 0 && !_overrideRemoved)
                {
                    LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                    LRRSiteComponentPropertyValue lscpvChanged = LRRModel.GetPropertyValueObject(typeof(LRRSiteComponent), lscpv.ModeID, lscp.ID, lscp.ComponentID) as LRRSiteComponentPropertyValue;
                    Hashtable hashrollbackSettings = GridSavedViewController.RollBackSettingsHash;
                    if (lscpvChanged != null)
                    {
                        LRRSiteComponentPropertyValue scpv = GridSavedViewController.RollBackPropertyValues.GetPropValueBysiteCompAndModeID(lscpvChanged.SiteComponentPropertyID, lscpvChanged.ModeID);
                        if (scpv != null)
                        {
                            lscpvChanged = scpv;
                            _lrrApp.RemoveCascadedBindings(lscpvChanged, false);
                            lscpvChanged.SetPropertyValue(lscpvChanged.GetActualOverridedValue());
                        }
                    }
                    lscpvChanged.RollBackChanges(true);
                    ArrayList arrRollBackSettings = hashrollbackSettings[lscpvChanged] as ArrayList;
                    if (arrRollBackSettings != null)
                    {
                        lscpv.OverridedValue = null;
                        lscpv.PropertyValue = arrRollBackSettings[0].ToString();
                        lscpv.ActualPropertyValue = arrRollBackSettings[1].ToString();
                        lscpv.ErrCode = (Symcon.LightRidge.BusinessObjects.Utilities.ErrorCode)arrRollBackSettings[2];
                        lscpv.ForeColor = (System.Drawing.Color)arrRollBackSettings[3];
                        lscpv.UnderLine = (bool)arrRollBackSettings[4];
                        lscpv.TrustedState = arrRollBackSettings[5].ToString();
                    }
                    lscpv.IsOverrided = lscpvChanged.IsOverrided;
                    lscpv.SetOverrideState(false);
                    lscpv.Validate();
                }
                else if (GridTransposedSavedViewController.RollBackSettingsHash.Count > 0 && !_overrideRemoved)
                {
                    LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                    LRRSiteComponentPropertyValue lscpvChanged = LRRModel.GetPropertyValueObject(typeof(LRRSiteComponent), lscpv.ModeID, lscp.ID, lscp.ComponentID) as LRRSiteComponentPropertyValue;
                    Hashtable hashrollbackSettings = GridTransposedSavedViewController.RollBackSettingsHash;
                    if (lscpvChanged != null)
                    {
                        LRRSiteComponentPropertyValue scpv = GridTransposedSavedViewController.RollBackPropertyValues.GetPropValueBysiteCompAndModeID(lscpvChanged.SiteComponentPropertyID, lscpvChanged.ModeID);
                        if (scpv != null)
                        {
                            lscpvChanged = scpv;
                            _lrrApp.RemoveCascadedBindings(lscpvChanged, false);
                            lscpvChanged.SetPropertyValue(lscpvChanged.GetActualOverridedValue());
                        }
                    }
                    lscpvChanged.RollBackChanges(true);
                    ArrayList arrRollBackSettings = hashrollbackSettings[lscpvChanged] as ArrayList;
                    if (arrRollBackSettings != null)
                    {
                        lscpv.PropertyValue = arrRollBackSettings[0].ToString();
                        lscpv.ActualPropertyValue = arrRollBackSettings[1].ToString();
                        lscpv.ErrCode = (Symcon.LightRidge.BusinessObjects.Utilities.ErrorCode)arrRollBackSettings[2];
                        lscpv.ForeColor = (System.Drawing.Color)arrRollBackSettings[3];
                        lscpv.UnderLine = (bool)arrRollBackSettings[4];
                        lscpv.TrustedState = arrRollBackSettings[5].ToString();

                    }
                    lscpv.IsOverrided = lscpvChanged.IsOverrided;
                    lscpv.SetOverrideState(lscpv.IsOverrided);
                    lscpv.Validate();
                }
                _overrideRemoved = true;
                */
            }
            LRRModel.PropertyValuesObj.RefreshCollection();
        }
        public void ProcessRollBackforQValues(LRRSiteGroupPropertyQualifierValues scqvs)
        {
            foreach (LRRSiteGroupPropertyQualifierValue scQv in scqvs)
            {
                LRRGroupOverride overrideobj = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(scQv.SiteGroupPropertyQualifierValueID);
                if (overrideobj != null)
                {
                    LRRModel.ChildModel.GroupOverrides.RollBackChanges(overrideobj);
                    scQv.RollBackChanges(true);
                    /*
                    if (!_lrrApp.RollBackQVOverridesGSVC.Contains(overrideobj) && !_lrrApp.RollBackQVOverridesGTSVC.Contains(overrideobj))
                    {
                        if (overrideobj.ObjectState != Symcon.Globals.Enums.DBOperation.UNCHANGED)
                        {
                            overrideobj.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                            _lrrApp.DBManagerObj.CommitObject(overrideobj);
                            LRRModel.ChildModel.GroupOverrides.Remove(overrideobj);
                            _lrrApp.RemoveCascadedBindings(scQv, scQv.GroupPropertyID, scQv.LookUpPropertiesID, scQv.QualifierID, false);
                            scQv.SetPropertyValue(scQv.GetActualOverridedValue());
                        }

                    }
                    */
                }

                /*
                LRRSiteGroupPropertyQualifierValue siteCompQValuRollBackObj;
                LRRSiteGroupPropertyQualifierValue siteCompQValuModelObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(scQv.SiteGroupPropertyQualifierValueID);
                Hashtable hashrollbackSettings = GridSavedViewController.RollBackQVSettingsHash;

                if (GridSavedViewController.RollBackQVSettingsHash.Count > 0)
                {
                    if (siteCompQValuModelObj != null)
                    {
                        if (siteCompQValuModelObj.LookUpQualifierID != null && siteCompQValuModelObj.LookUpQualifierID.Trim() != "")
                        {
                            string lokUpQID = siteCompQValuModelObj.LookUpQualifierID;
                            siteCompQValuRollBackObj = GridSavedViewController.RollBackQValues.GetQualifierValueObjectForPropertyMode(scQv.GroupPropertyID, scQv.LookUpPropertiesID, scQv.QualifierID, scQv.ModeID);
                            if (siteCompQValuRollBackObj != null)
                            {
                                siteCompQValuModelObj = siteCompQValuRollBackObj;
                                _lrrApp.RemoveCascadedBindings(scQv, siteCompQValuModelObj.GroupPropertyID, siteCompQValuModelObj.LookUpPropertiesID, siteCompQValuModelObj.QualifierID, false);
                                siteCompQValuModelObj.SetPropertyValue(siteCompQValuModelObj.GetActualOverridedValue());
                            }
                        }
                        else
                        {
                            siteCompQValuRollBackObj = GridSavedViewController.RollBackQValues.GetGroupQualifierValueFromGroupPropIDandModeID(scQv.GroupPropertyID, scQv.ModeID);
                            if (siteCompQValuRollBackObj != null)
                            {
                                siteCompQValuModelObj = siteCompQValuRollBackObj;
                                _lrrApp.RemoveCascadedBindings(scQv, siteCompQValuModelObj.GroupPropertyID, siteCompQValuModelObj.LookUpPropertiesID, siteCompQValuModelObj.QualifierID, false);
                                siteCompQValuModelObj.SetPropertyValue(siteCompQValuModelObj.GetActualOverridedValue());
                            }
                        }
                    }
                    siteCompQValuModelObj.RollBackChanges(true);
                    ArrayList arrRollBackSettings = hashrollbackSettings[siteCompQValuModelObj] as ArrayList;
                    if (arrRollBackSettings != null)
                    {
                        scQv.PropertyValue = arrRollBackSettings[0].ToString();
                        scQv.ActualPropertyValue = arrRollBackSettings[1].ToString();
                        scQv.ErrCode = (Symcon.LightRidge.BusinessObjects.Utilities.ErrorCode)arrRollBackSettings[2];
                        scQv.ForeColor = (System.Drawing.Color)arrRollBackSettings[3];
                        scQv.UnderLine = (bool)arrRollBackSettings[4];
                        scQv.TrustedState = arrRollBackSettings[5].ToString();
                    }
                    scQv.IsOverrided = siteCompQValuModelObj.IsOverrided;
                    scQv.SetOverrideState(scQv.IsOverrided);
                    scQv.Validate();
                }
                else if (GridTransposedSavedViewController.RollBackQVSettingsHash.Count > 0)
                {
                    if (siteCompQValuModelObj != null)
                    {
                        if (siteCompQValuModelObj.LookUpQualifierID != null && siteCompQValuModelObj.LookUpQualifierID.Trim() != "")
                        {
                            string lokUpQID = siteCompQValuModelObj.LookUpQualifierID;
                            siteCompQValuRollBackObj = GridTransposedSavedViewController.RollBackQValues.GetQualifierValueObjectForPropertyMode(scQv.GroupPropertyID, scQv.LookUpPropertiesID, scQv.QualifierID, scQv.ModeID);
                            if (siteCompQValuRollBackObj != null)
                            {
                                siteCompQValuModelObj = siteCompQValuRollBackObj;
                                _lrrApp.RemoveCascadedBindings(scQv, siteCompQValuModelObj.GroupPropertyID, siteCompQValuModelObj.LookUpPropertiesID, siteCompQValuModelObj.QualifierID, false);
                                siteCompQValuModelObj.SetPropertyValue(siteCompQValuModelObj.GetActualOverridedValue());
                            }
                        }
                        else
                        {
                            siteCompQValuRollBackObj = GridTransposedSavedViewController.RollBackQValues.GetGroupQualifierValueFromGroupPropIDandModeID(scQv.GroupPropertyID, scQv.ModeID);
                            if (siteCompQValuRollBackObj != null)
                            {
                                siteCompQValuModelObj = siteCompQValuRollBackObj;
                                _lrrApp.RemoveCascadedBindings(scQv, siteCompQValuModelObj.GroupPropertyID, siteCompQValuModelObj.LookUpPropertiesID, siteCompQValuModelObj.QualifierID, false);
                                siteCompQValuModelObj.SetPropertyValue(siteCompQValuModelObj.GetActualOverridedValue());
                            }
                        }
                    }
                    siteCompQValuModelObj.RollBackChanges(true);
                    ArrayList arrRollBackSettings = hashrollbackSettings[siteCompQValuModelObj] as ArrayList;
                    if (arrRollBackSettings != null)
                    {
                        scQv.PropertyValue = arrRollBackSettings[0].ToString();
                        scQv.ActualPropertyValue = arrRollBackSettings[1].ToString();
                        scQv.ErrCode = (Symcon.LightRidge.BusinessObjects.Utilities.ErrorCode)arrRollBackSettings[2];
                        scQv.ForeColor = (System.Drawing.Color)arrRollBackSettings[3];
                        scQv.UnderLine = (bool)arrRollBackSettings[4];
                        scQv.TrustedState = arrRollBackSettings[5].ToString();
                    }
                    scQv.IsOverrided = siteCompQValuModelObj.IsOverrided;
                    scQv.SetOverrideState(false);
                    scQv.Validate();
                }
                */
            }
            LRRModel.SiteGroupPropertyQualifierValuesObj.RefreshCollection();
        }
        public void ProcessRollbackforPVOverrides(LRROverrides pvoverrides)
        {
            foreach (LRROverride ovrd in pvoverrides)
            {
                LRROverride overrideobj = LRRModel.ChildModel.Overrides.GetOverrideObjByID(ovrd.ID);
                    //GridSavedViewController.RollBackPVOverrides.GetOverrideObjByID(pvoverrides);
                    //LRRModel.ChildModel.Overrides.GetChanges().GetOverrideObjectByPropertyAndModeID(lscpv.SiteComponentPropertyID, lscpv.ModeID);
                if (overrideobj != null)
                {
                    LRRSiteComponentPropertyValue scpv = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(overrideobj.SiteComponentPropertyID, overrideobj.ModeID);
                    if (scpv != null)
                    {
                        scpv.SetOverrideState(true);
                        //scpv.SetOverrideValue(overrideobj.PostOverrideValue);
                        //scpv.OverridedValue = overrideobj.PostOverrideValue;
                        scpv.PropertyValue = overrideobj.PostOverrideValue;
                        scpv.IsOverrided = true;
                        scpv.Validate();
                        if (!LRRModel.ChildModel.Overrides.Contains(overrideobj))
                        {
                            LRRModel.ChildModel.Overrides.Add(overrideobj);
                        }
                    }
                }
            }
        }
        public void ProcessRollBackforQVOverrides(LRRGroupOverrides qvOverrides)
        {
            foreach (LRRGroupOverride groupeOverride in qvOverrides)
            {
                LRRGroupOverride grpOverrideobj = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(groupeOverride.SiteGroupPropertyQualifierValueID);
                if (grpOverrideobj != null)
                {
                    LRRSiteGroupPropertyQualifierValue scGQV = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(grpOverrideobj.SiteGroupPropertyQualifierValueID);
                    if (scGQV != null)
                    {
                        scGQV.SetOverrideState(true);
                        scGQV.PropertyValue = grpOverrideobj.PostOverrideValue;
                        scGQV.IsOverrided = true;
                        scGQV.Validate();
                        
                        if (!LRRModel.ChildModel.GroupOverrides.Contains(grpOverrideobj))
                        {
                            LRRModel.ChildModel.GroupOverrides.Add(grpOverrideobj);
                        }
                        
                    }
                }
            }
        }
        void UnifiedViewResultForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            //LRRSiteComponentPropertyValues toSaveSiteCompPropValues = LRRModel.PropertyValuesObj.GetChanges();

            LRRSiteComponentPropertyValues toSaveSiteCompPropValues = LRRModel.PropertyValuesObj.GetChanges();
            LRRAggregatePropertyValues toSaveAggrPropValues = LRRModel.AggregatePropertyValuesObj.GetChanges();
            LRRSiteGroupPropertyQualifierValues toSaveQValues = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();
            //LRROverrides tosavePVOverrides = _lrrApp.RollBackPVOverridesGSVC
                //LRRModel.ChildModel.Overrides.GetChanges();
            LRRGroupOverrides toSaveGroupOverrides = LRRModel.ChildModel.GroupOverrides.GetChanges();
            if (toSaveSiteCompPropValues.Count > 0 || toSaveQValues.Count > 0)
            {
                DialogResult dlgResult = LRRMessagBox.ShowMessage(this, "SID_SAVE_PROPVALUE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                // the no part for dialog result added by ashwin
                if (dlgResult == DialogResult.Yes)
                {
                    GridView.Save();
                }
                else
                {
                    ProcessRollBackforPropertyValues(toSaveSiteCompPropValues);
                    ProcessRollBackforQValues(toSaveQValues);
                    /*
                    ProcessRollbackforPVOverrides(_lrrApp.RollBackPVOverridesGSVC);
                    ProcessRollbackforPVOverrides(_lrrApp.RollBackPVOverridesGTSVC);
                    ProcessRollBackforQVOverrides(_lrrApp.RollBackQVOverridesGSVC);
                    ProcessRollBackforQVOverrides(_lrrApp.RollBackQVOverridesGTSVC);
                     * */
                        //toSaveGroupOverrides);
                    #region cmt1
                    /*
                    foreach (LRRSiteComponentPropertyValue lscpv in toSaveSiteCompPropValues)
                    {
                        LRROverride overrideobj = LRRModel.ChildModel.Overrides.GetChanges().GetOverrideObjectByPropertyAndModeID(lscpv.SiteComponentPropertyID, lscpv.ModeID);
                        if (overrideobj != null)
                        {
                            overrideobj.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                            _lrrApp.DBManagerObj.CommitObject(overrideobj);
                            LRRModel.ChildModel.Overrides.Remove(overrideobj);
                            _lrrApp.RemoveCascadedBindings(lscpv, false);
                            lscpv.SetPropertyValue(lscpv.GetActualOverridedValue());
                        }
                        if (GridSavedViewController.RollBackSettingsHash.Count > 0)
                        {
                            LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                            LRRSiteComponentPropertyValue lscpvChanged = LRRModel.GetPropertyValueObject(typeof(LRRSiteComponent), lscpv.ModeID, lscp.ID, lscp.ComponentID) as LRRSiteComponentPropertyValue;
                            Hashtable hashrollbackSettings = GridSavedViewController.RollBackSettingsHash;
                            if (lscpvChanged != null)
                            {
                                LRRSiteComponentPropertyValue scpv = GridSavedViewController.RollBackPropertyValues.GetPropValueBysiteCompAndModeID(lscpvChanged.SiteComponentPropertyID, lscpvChanged.ModeID);
                                if (scpv != null)
                                {
                                    lscpvChanged = scpv;
                                    _lrrApp.RemoveCascadedBindings(lscpvChanged, false);
                                    lscpvChanged.SetPropertyValue(lscpvChanged.GetActualOverridedValue());
                                }
                            }
                            lscpvChanged.RollBackChanges(true);
                            ArrayList arrRollBackSettings = hashrollbackSettings[lscpvChanged] as ArrayList;
                            if (arrRollBackSettings != null)
                            {
                                lscpv.PropertyValue = arrRollBackSettings[0].ToString();
                                lscpv.ActualPropertyValue = arrRollBackSettings[1].ToString();
                                lscpv.ErrCode = (Symcon.LightRidge.BusinessObjects.Utilities.ErrorCode)arrRollBackSettings[2];
                                lscpv.ForeColor = (System.Drawing.Color)arrRollBackSettings[3];
                                lscpv.UnderLine = (bool)arrRollBackSettings[4];
                                lscpv.TrustedState = arrRollBackSettings[5].ToString();

                            }
                            lscpv.IsOverrided = lscpvChanged.IsOverrided;
                            lscpv.SetOverrideState(false);
                            lscpv.Validate();
                        }
                        else if(GridTransposedSavedViewController.RollBackSettingsHash.Count > 0)
                        {
                            LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                            LRRSiteComponentPropertyValue lscpvChanged = LRRModel.GetPropertyValueObject(typeof(LRRSiteComponent), lscpv.ModeID, lscp.ID, lscp.ComponentID) as LRRSiteComponentPropertyValue;
                            Hashtable hashrollbackSettings = GridTransposedSavedViewController.RollBackSettingsHash;
                            if (lscpvChanged != null)
                            {
                                LRRSiteComponentPropertyValue scpv = GridTransposedSavedViewController.RollBackPropertyValues.GetPropValueBysiteCompAndModeID(lscpvChanged.SiteComponentPropertyID, lscpvChanged.ModeID);

                                if (scpv != null)
                                {
                                    lscpvChanged = scpv;
                                    _lrrApp.RemoveCascadedBindings(lscpvChanged, false);
                                    lscpvChanged.SetPropertyValue(lscpvChanged.GetActualOverridedValue());

                                }

                            }
                            lscpvChanged.RollBackChanges(true);
                            ArrayList arrRollBackSettings = hashrollbackSettings[lscpvChanged] as ArrayList;
                            if (arrRollBackSettings != null)
                            {
                                lscpv.PropertyValue = arrRollBackSettings[0].ToString();
                                lscpv.ActualPropertyValue = arrRollBackSettings[1].ToString();
                                lscpv.ErrCode = (Symcon.LightRidge.BusinessObjects.Utilities.ErrorCode)arrRollBackSettings[2];
                                lscpv.ForeColor = (System.Drawing.Color)arrRollBackSettings[3];
                                lscpv.UnderLine = (bool)arrRollBackSettings[4];
                                lscpv.TrustedState = arrRollBackSettings[5].ToString();

                            }
                            lscpv.IsOverrided = lscpvChanged.IsOverrided;
                            lscpv.SetOverrideState(false);

                            lscpv.Validate();

                        }
                    }
                    */
                    #endregion
                    /*
                    LRRModel.PropertyValuesObj.RefreshCollection();
                    LRRModel.SiteGroupPropertyQualifierValuesObj.RefreshCollection();
                    LRRModel.ChildModel.Overrides.RefreshCollection();
                    LRRModel.ChildModel.GroupOverrides.RefreshCollection();
                     * */
                    /*
                    LRROverrides toSaveOverrides = LRRModel.ChildModel.Overrides.GetChanges();
                    foreach(LRROverride overrideobj in toSaveOverrides )
                    {
                        overrideobj.RollBackChanges(true);
                        if (LRRModel.ChildModel.Overrides.Contains(overrideobj))
                        {
                            LRRModel.ChildModel.Overrides.Remove(overrideobj);
                        }
                    }
                    LRRModel.SiteGroupPropertyQualifierValuesObj.RefreshCollection();
                    LRRGroupOverrides tosaveGroupOverrides = LRRModel.ChildModel.GroupOverrides.GetChanges();
                    foreach (LRRGroupOverride gOverride in tosaveGroupOverrides)
                    {
                        gOverride.RollBackChanges(true);
                        if (LRRModel.ChildModel.GroupOverrides.Contains(gOverride))
                        {
                            LRRModel.ChildModel.GroupOverrides.Remove(gOverride);
                        }
                    }
                     * */
                    foreach (LRRAggregatePropertyValue aggPropVal in toSaveAggrPropValues)
                    {
                        aggPropVal.RollBackChanges(true);
                    }

                }
                BUnifiedResultFormChange = false;
                 
            }

            if (_gridView != null)
            {
                _gridView.ClearAllBindings();
                _gridView.UnSubscribeReconnectCompleted();
            }

        }

        #endregion

        #region ILRRUIContainer Members

        public new void Refresh()
		{
		}

		public void Save()
		{
            if (GridView != null)
            {
                // loop through rollback collection added by ashwin
                GridView.Save();
                BUnifiedResultFormChange = false;
            }
        }

        //public void CheckForChanges(IUnifiedGridFactory gridView)
        //{
        //}

		public void CopyMasterProperties()
		{
		
		}

		public void Paste()
		{
			
		}

		public void Copy()
		{
			
		}

		public void Print()
		{
			if(GridView != null)
				GridView.Print();
		}

		public void Delete()
		{
			
		}

		public void ArrangeColumns()
		{
			
		}

		public void ClearFilter()
		{
		}
		public void ImportToExcel()
		{
		}

		public  void RightLevel()
		{

		}
		public  void LeftLevel()
		{

		}
		public  void UpLevel()
		{

		}
		public  void DownLevel()
		{

		}


		public bool SupportsSelectionPaste()
		{
			return false;
		}

		public void SelectionPaste()
		{
		}

		public bool SupportsCopy()
		{
			// TODO:  Add ComplexEquip.SupportsCopy implementation
			return false;
		}

		public bool SupportsPaste()
		{
			// TODO:  Add ComplexEquip.SupportsPaste implementation
			return false;
		}

		public bool SupportsExport()
		{
			// TODO:  Add ComplexEquip.SupportsExport implementation
			return false;
		}

		public bool SupportsSave()
		{
			return true;
		}

		public bool SupportsRefresh()
		{
			// TODO:  Add ComplexEquip.SupportsRefresh implementation
			return false;
		}

		public bool SupportsDelete()
		{
			// TODO:  Add ComplexEquip.SupportsDelete implementation
			return false;
		}

		public bool SupportsPrint()
		{
			// TODO:  Add ComplexEquip.SupportsPrint implementation
			return true;
		}

		public bool SupportsArrangeColumns()
		{
			// TODO:  Add ComplexEquip.SupportsArrangeColumns implementation
			return false;
		}

		public bool SupportsClearFilter()
		{
			// TODO:  Add ComplexEquip.SupportsClearFilter implementation
			return false;
		}
		public bool SupportsMasterPropertiesCopy()
		{
			return false;
		}
		public bool SupportsInsertNewRows()
		{
			// TODO:  Add MasterProperties.SupportsClearFilter implementation
			return false;
		}

		public bool SupportsImportToExcel()
		{
			return false;
		}

		public bool SupportsLevel()
		{
			return false;
		}

		public void InsertNewRows()
		{
			
		}
		public void Export()
		{
		}
		
		public TabControl	ParentTabControl
		{
			set
			{
				_parentTabControl = value;
			}
			get{return _parentTabControl;}
		}
	
		public TabPage ParentTabPage
		{
			set
			{
				_parentTabPage = value;
			}
			get
			{
				return _parentTabPage;
			}
		}
		#endregion

		#region Methods

		private void SetPanel(ArrayList pnlCollection)
		{
			int heightOfEach = panel1.Height/pnlCollection.Count;
			int splitCnt = 0;
			for(int i=pnlCollection.Count-1;i>=0;i--)
			{
				Panel panel = pnlCollection[i] as Panel;
				panel.Height = heightOfEach;
				this.panel1.Controls.Add(panel);
				if(splitCnt < pnlCollection.Count - 1)
				{
					Splitter splitGrid = new Splitter();
					splitGrid.Dock = DockStyle.Top;
					this.panel1.Controls.Add(splitGrid);
					splitCnt++;
				}
			}
			if(this.panel1.Controls.Count > 0)
				this.panel1.Controls[0].Dock = DockStyle.Fill;
		}
		#endregion

		#region ILRRUIContainer Members

		public void CopyUOMTypes()
		{
		
		}

		public bool SupportsCopyUOMTypes()
		{
			return false;
		}

		#endregion
		#region ILRRUIContainer Members

		public void ClearSort()
		{
//			if(GridView != null)
//				GridView.ClearSort();
		}

		public bool SupportsClearSort()
		{
			return true;
		}

		#endregion

		#region ILRRUIContainer Members

		public void PrintPreview()
		{
			if(GridView != null)
				GridView.PrintPreview();
		}

		#endregion

		#region ILRRUIContainer Members

		public void ShowPropertyList()
		{
			if(_rsResultAttributesWindow != null && _rsResultAttributesWindow.PropertyCollection.Count > 0)
			{
				_rsResultAttributesWindow.Show();
				_rsResultAttributesWindow.IsWindowOpen = true;
			}
		}

		public bool SupportsResultsFilter()
		{
			return true;
		}

		#endregion

		#region ILRRUIContainer Members

		public void ShowComponentList()
		{
			if(_rsResultComponentsWindow != null && _rsResultComponentsWindow.PropertyCollection.Count > 0)
			{
				_rsResultComponentsWindow.Show();
				_rsResultComponentsWindow.IsWindowOpen = true;
			}
		}

		#endregion

		private void ProjectModes_ModesChanged(object sender, EventArgs e)
		{
			if(GridView != null)
				GridView.HideModes();
		}

		#region Properties
		private IUnifiedGridFactory GridView
		{
			get { return _gridView;}
			set
			{
				_gridView = value;
			}
		}
		#endregion

		#region
        private IUnifiedGridFactory _gridView;

		private ResultWindow _rsResultAttributesWindow;
		private ResultWindow _rsResultComponentsWindow;
		#endregion

		#region ILRRUIContainer Members

		public void LeftColMovement()
		{
			if(GridView != null)
				GridView.LeftColMovement();
		}

		public void RightColMovement()
		{
			if(GridView != null)
				GridView.RightColMovement();
			
		}

		#endregion

		#region ILRRUIContainer Members

		public void PageSetup()
		{
			GridGroupingPrint ggPrint = new GridGroupingPrint();
			ggPrint.PageSetUp();
		}

		public void Undo()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.Undo implementation
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.SupportsPageSetup implementation
			return true;
		}

		public bool SupportsUndo()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.SupportsUndo implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void OpenCompositeView()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.OpenCompositeView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.TransposeView implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.OpenQuery implementation
		}

		public void OpenView()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.OpenView implementation
		}

		public bool SupportsOpenCompositeView()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.SupportsOpenCompositeView implementation
			return false;
		}

		public bool SupportsTransposeView()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.SupportsTransposeView implementation
			return false;
		}

		public bool SupportsOpenQuery()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.SupportsOpenQuery implementation
			return false;
		}

		public bool SupportsOpenView()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.SupportsOpenView implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public bool SupportsCompositeView()
		{
			// TODO:  Add GroupedPlantPropertiesResultForm.SupportsCompositeView implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public bool PerformDirtyChk
		{
			get
			{
				// TODO:  Add GroupedPlantPropertiesResultForm.PerformDirtyChk getter implementation
				return true;
			}
		}

		#endregion

		#region ILRRUISpecificContainer Members

		public void ChangeUOMSet()
		{
//			IGroupGridFactory iGridView = GridView as IGroupGridFactory;
//			if(iGridView != null)
//			{
//				ChangeUOMSetDefinitionForm chngUOMSetDefn = new ChangeUOMSetDefinitionForm();
//				DialogResult dlgResult = chngUOMSetDefn.ShowDialog();
//				if(dlgResult == DialogResult.OK  && chngUOMSetDefn.MeasSysID != null)
//				{
//					iGridView.ChangeUOMSet(chngUOMSetDefn.MeasSysID);
//				}
//			}
		}

		public bool SupportsChangeUOMSet()
		{
			return true;
		}

		#endregion

		private void _rsResultAttributesWindow_SelectedProperties(ArrayList arrayUnSelectedRecords)
		{
			if(GridView != null)
				GridView.HideProperties(arrayUnSelectedRecords);
		}

		private void _rsResultComponentsWindow_SelectedProperties(ArrayList arrayUnSelectedRecords)
		{
			if(GridView != null)
			{
				GridView.HideModes();
				GridView.HideComponents(arrayUnSelectedRecords);
			}
		}
	}
}
