using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;

using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Lightridge.LRRUI.Controls;
using Symcon.Lightridge.LRRUI;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.PEAdvisor.ScreenManagement;   
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.LightRidge.PEAdvisor.CommonViews;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Diagram;

//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class CompositeView : System.Windows.Forms.Form	,ILRRUIContainer
	{
		#region Fields

		private System.Windows.Forms.Panel						pnlFull;
		private System.Windows.Forms.Panel						pnlGraphics;
		private System.Windows.Forms.Panel						pnlIO;
		private System.ComponentModel.IContainer				components;
		private TabControl										_parentTabControl;
		private TabPage											_parentTabPage;
		private LRRApp											_lrrApp;
		private string											viewID;
		private System.Windows.Forms.Splitter					splitter1;
		private Syncfusion.Windows.Forms.Tools.XPMenus.BarItem	barItemTransposeView;
        private Syncfusion.Windows.Forms.Tools.XPMenus.ComboBoxBarItem comboBoxBarItemZoom;
		private string											queryID;
		private string											type1;
		private string											type2;
		private LRRSearchCriterias								_lrrSearchCriterias = null;
		private LRRSiteComponents								_lrrSiteComponents = null;
		private LRRSavedViews									_lrrSavedViews = null;
		private LRRCompositeViews								_lrrCompositeViews = null;
		private LRRSearchCriteriaQualifiers						_LRRSearchCriteriaQualifiers = null;
		private LRRSearchCriteriaPropertyTypes					_lrrSearchCriteriaPropertyTypes = null;
		private LRRPropertyTypes								_objPropertyTypes = null;
		private bool											_bTransposeFlag	= true;
		private object											_gridView;
		private GraphicsManagers								gmsType1 = null;
		private GraphicsManagers								gmsType2 = null;
		private LRRSavedViews									lrrSavedViews = null;
		private ToolTip											_toolTip = null;
		//Property Window
		private Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow _rsProperties;
		//Component Window
		private Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow _rsComponents;
        private string                                          _compositeViewID = "";
        private int                                             _lastZoom = 100;
        private Canvas                                          _zoomCanvas = null;
        private ResultWindow                                    _resultComponentsWindow;
        private ResultWindow                                    _resultPropertiesWindow;
        private Syncfusion.Windows.Forms.Diagram.Controls.OverviewControl overviewControl1;
        private Syncfusion.Windows.Forms.Tools.DockingManager dockingManager1;
        private string                                          _currentCompositeViewName = "";
        



		#endregion

		#region Constructor and Designer Generated Code
		public CompositeView()
		{
            InitCompositeView();
		}

        public CompositeView(bool is1stChild)
        {
            InitCompositeView();
            if (is1stChild)
            {
                pnlGraphics.Name = "pnlGraphics1stChild";
                splitter1.Name = "splitter11stChild";
            }
            else
            {
                pnlGraphics.Name = "pnlGraphics2ndChild";
                splitter1.Name = "splitter12ndChild";
            }
        }

        public void InitCompositeView()
        {
            InitializeComponent();
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitDataSource();
            // hookup events called by ashwin
            gmsType1 = new GraphicsManagers();
            //gmsType1.HookUpEvents();
            gmsType2 = new GraphicsManagers();
            //gmsType2.HookUpEvents();
            lrrSavedViews = new LRRSavedViews();
            _toolTip = new ToolTip();
            gmsType1.ApplyModes();
            gmsType2.ApplyModes();

            LRRModel.ProjectModes.ModesChanged += new EventHandler(ProjectModes_ModesChanged);
            this.Closing += new CancelEventHandler(CompositeView_Closing);
            this.Load += new EventHandler(CompositeView_Load);
            this.Activated += new EventHandler(CompositeView_Activated);
            this.Deactivate += new EventHandler(CompositeView_Deactivate);
            _rsProperties = new Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow();
            _rsComponents = new Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow();
            _toolTip.AutoPopDelay = 2000;

            _lrrApp.MainFrame.MainFrameBarManager.Items.Add(comboBoxBarItemZoom);
            _lrrApp.MainFrame.CompositeViewBarObj.Items.Add(comboBoxBarItemZoom);

            comboBoxBarItemZoom.Visible = false;
            _resultComponentsWindow = new ResultWindow();
            _resultPropertiesWindow = new ResultWindow();

        }

        public CompositeView(LRRCompositeView compositeView)
        {
            InitializeComponent();
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitDataSource();
            // hookup events called by ashwin
            gmsType1 = new GraphicsManagers();
            //gmsType1.HookUpEvents();
            gmsType2 = new GraphicsManagers();
            //gmsType2.HookUpEvents();
            lrrSavedViews = new LRRSavedViews();
            _toolTip = new ToolTip();
            gmsType1.ApplyModes();
            gmsType2.ApplyModes();

            LRRModel.ProjectModes.ModesChanged += new EventHandler(ProjectModes_ModesChanged);
            this.Closing += new CancelEventHandler(CompositeView_Closing);
            this.Load += new EventHandler(CompositeView_Load);
            this.Activated += new EventHandler(CompositeView_Activated);
            this.Deactivate += new EventHandler(CompositeView_Deactivate);
            _rsProperties = new Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow();
            _rsComponents = new Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow();
            _toolTip.AutoPopDelay = 2000;
            _resultComponentsWindow = new ResultWindow();
            _resultPropertiesWindow = new ResultWindow();
            OpenCompositeView(compositeView);
            this.Text = "Composite View" + "-[" + compositeView.Name + "]";

            _lrrApp.MainFrame.MainFrameBarManager.Items.Add(comboBoxBarItemZoom);
            _lrrApp.MainFrame.CompositeViewBarObj.Items.Add(comboBoxBarItemZoom);

            comboBoxBarItemZoom.Visible = false;

            //_lrrApp.MainFrame.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(_lrrApp.MainFrame.CompositeViewBarObj, false);
            // _lrrApp.MainFrame.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(_lrrApp.MainFrame.CompositeViewBarObj, false);

        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
            //ApplicationPersistanceUtil.SaveGridSettings(this, this, _compositeViewID);
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
            gmsType1.Dispose();
            gmsType2.Dispose();
            gmsType1 = null;
            gmsType2 = null;
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(CompositeView));
            this.pnlFull = new System.Windows.Forms.Panel();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.pnlIO = new System.Windows.Forms.Panel();
            this.pnlGraphics = new System.Windows.Forms.Panel();
            this.overviewControl1 = new Syncfusion.Windows.Forms.Diagram.Controls.OverviewControl();
            this.barItemTransposeView = new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem();
            this.comboBoxBarItemZoom = new Syncfusion.Windows.Forms.Tools.XPMenus.ComboBoxBarItem();
            this.dockingManager1 = new Syncfusion.Windows.Forms.Tools.DockingManager(this.components);
            this.pnlFull.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.dockingManager1)).BeginInit();
            this.SuspendLayout();
            // 
            // pnlFull
            // 
            this.pnlFull.Controls.Add(this.splitter1);
            this.pnlFull.Controls.Add(this.pnlIO);
            this.pnlFull.Controls.Add(this.pnlGraphics);
            this.pnlFull.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlFull.Location = new System.Drawing.Point(0, 0);
            this.pnlFull.Name = "pnlFull";
            this.pnlFull.Size = new System.Drawing.Size(744, 486);
            this.pnlFull.TabIndex = 0;
            // 
            // splitter1
            // 
            this.splitter1.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.splitter1.Location = new System.Drawing.Point(0, 155);
            this.splitter1.Name = "splitter1";
            this.splitter1.Size = new System.Drawing.Size(744, 3);
            this.splitter1.TabIndex = 3;
            this.splitter1.TabStop = false;
            // 
            // pnlIO
            // 
            this.pnlIO.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.pnlIO.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlIO.Location = new System.Drawing.Point(0, 0);
            this.pnlIO.Name = "pnlIO";
            this.pnlIO.Size = new System.Drawing.Size(744, 158);
            this.pnlIO.TabIndex = 2;
            this.pnlIO.Click += new System.EventHandler(this.pnlIO_Click);
            // 
            // pnlGraphics
            // 
            this.pnlGraphics.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.pnlGraphics.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.pnlGraphics.Location = new System.Drawing.Point(0, 158);
            this.pnlGraphics.Name = "pnlGraphics";
            this.pnlGraphics.Size = new System.Drawing.Size(744, 328);
            this.pnlGraphics.TabIndex = 0;
            // 
            // overviewControl1
            // 
            this.overviewControl1.BackColor = System.Drawing.SystemColors.AppWorkspace;
            this.dockingManager1.SetEnableDocking(this.overviewControl1, true);
            this.overviewControl1.ForeColor = System.Drawing.Color.Red;
            this.overviewControl1.Location = new System.Drawing.Point(1, 1);
            this.overviewControl1.Name = "overviewControl1";
            this.overviewControl1.Size = new System.Drawing.Size(165, 127);
            this.overviewControl1.TabIndex = 0;
            // 
            // barItemTransposeView
            // 
            this.barItemTransposeView.CategoryIndex = -1;
            this.barItemTransposeView.ID = "";
            // 
            // comboBoxBarItemZoom
            // 
            this.comboBoxBarItemZoom.CategoryIndex = 0;
            this.comboBoxBarItemZoom.ChoiceList.AddRange(new string[] {
            "25%",
            "50%",
            "75%",
            "100%",
            "150%",
            "200%",
            "300%",
            "400%",
            "500%"});
            this.comboBoxBarItemZoom.Editable = false;
            this.comboBoxBarItemZoom.ID = "Zoom:";
            this.comboBoxBarItemZoom.Text = "Zoom:";
            this.comboBoxBarItemZoom.TextBoxValue = "100%";
            this.comboBoxBarItemZoom.Click += new System.EventHandler(this.comboBoxBarItemZoom_Click);
            // 
            // dockingManager1
            // 
            this.dockingManager1.AutoHideInterval = 200;
            this.dockingManager1.CloseEnabled = false;
            this.dockingManager1.DockLayoutStream = ((System.IO.MemoryStream)(resources.GetObject("dockingManager1.DockLayoutStream")));
            this.dockingManager1.HostControl = this;
            this.dockingManager1.PersistState = true;
            this.dockingManager1.ShowToolTips = true;
            this.dockingManager1.SetDockLabel(this.overviewControl1, "Pan and Zoom");
            this.dockingManager1.SetFloatOnly(this.overviewControl1, true);
            
            // 
            // CompositeView
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.BackColor = System.Drawing.SystemColors.InactiveCaptionText;
            this.ClientSize = new System.Drawing.Size(744, 486);
            this.Controls.Add(this.pnlFull);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Name = "CompositeView";
            this.Tag = "SID_22";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
            this.pnlFull.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.dockingManager1)).EndInit();
            this.ResumeLayout(false);

		}
		#endregion

		#endregion

		#region ILRRUIContainer Members

		public void LeftColMovement()
		{
			Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridView = GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
			if(iGridView != null)
				iGridView.LeftColMovement();
            else
            {
                Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory iUnifiedGridFactory = GridView as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
                if (iUnifiedGridFactory != null)
                    iUnifiedGridFactory.LeftColMovement();
            }
		}


		public void RightColMovement()
		{
			Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridView = GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
            if (iGridView != null)
                iGridView.RightColMovement();
            else
            {
                Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory iUnifiedGridFactory = GridView as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
                if(iUnifiedGridFactory != null)
                    iUnifiedGridFactory.RightColMovement();
            }

		}

		
		public void ShowPropertyList()
		{
			if(_rsProperties != null && _rsProperties.PropertyCollection.Count > 0)
			{
				_rsProperties.Show();
				_rsProperties.IsWindowOpen = true;
			}
            else if (_resultPropertiesWindow != null && _resultPropertiesWindow.PropertyCollection.Count > 0)
            {
                _resultPropertiesWindow.IsWindowOpen = true;
                _resultPropertiesWindow.Show();

            }
		}

		
		public bool SupportsResultsFilter()
		{
			return false;
		}


		public void PageSetup()
		{
			GridGroupingPrint ggPrint = new GridGroupingPrint();
			ggPrint.PageSetUp();
		}

		
		public void Undo()
		{
		}

		
		public bool SupportsPageSetup()
		{
			return true;
		}

		
		public bool SupportsUndo()
		{
			return false;
		}


		public void ShowComponentList()
		{
			if(_rsComponents != null && _rsComponents.PropertyCollection.Count > 0)
			{
				_rsComponents.Show();
				_rsComponents.IsWindowOpen = true;
			}
            else if (_resultComponentsWindow != null && _resultComponentsWindow.PropertyCollection.Count > 0)
            {
                _resultComponentsWindow.IsWindowOpen = true;
                _resultComponentsWindow.Show();

            }
		}


		public void OpenCompositeView()
		{
			_lrrApp.DBManagerObj.PopulateCollection(_lrrCompositeViews,"",false,true);
			_lrrCompositeViews = _lrrCompositeViews.FilterForUserByUserID();
			CompositeFileDialog openCompositeView = new CompositeFileDialog(_lrrCompositeViews);
			DialogResult dlgResult = openCompositeView.ShowDialog(this);
			if(dlgResult == DialogResult.OK)
			{
				LRRCompositeView lrrCompView = openCompositeView.ObjSelected as LRRCompositeView;
                _currentCompositeViewName = lrrCompView.Name;
				this.OpenCompositeView(lrrCompView);
			}
		}

		
		public void TransposeView()
		{
			if(_bTransposeFlag)
			{
				pnlGraphics.Dock = DockStyle.Right;
				pnlGraphics.Width = (int)((SystemInformation.WorkingArea.Width)* 0.50);
				splitter1.Dock = DockStyle.Right;
				pnlIO.Dock = DockStyle.Fill;
				SetPanelHeight(pnlIO);
				SetPanelHeight(pnlGraphics);
				_bTransposeFlag = false;
			}
			else
			{
				pnlGraphics.Dock = DockStyle.Bottom;
				pnlGraphics.Height = (int)((SystemInformation.WorkingArea.Height)* 0.50);
				splitter1.Dock = DockStyle.Bottom;
				pnlIO.Dock = DockStyle.Fill;
				SetPanelHeight(pnlIO);
				SetPanelHeight(pnlGraphics);
				_bTransposeFlag = true;
			}
		}

        public void TransposeViewWithoutResizing()
        {
            if (_bTransposeFlag)
            {
                pnlGraphics.Dock = DockStyle.Right;
                //pnlGraphics.Width = (int)((SystemInformation.WorkingArea.Width) * 0.50);
                splitter1.Dock = DockStyle.Right;
                pnlIO.Dock = DockStyle.Fill;
                SetPanelHeight(pnlIO);
                SetPanelHeight(pnlGraphics);
                _bTransposeFlag = false;
            }
            else
            {
                pnlGraphics.Dock = DockStyle.Bottom;
                //pnlGraphics.Height = (int)((SystemInformation.WorkingArea.Height) * 0.50);
                splitter1.Dock = DockStyle.Bottom;
                pnlIO.Dock = DockStyle.Fill;
                SetPanelHeight(pnlIO);
                SetPanelHeight(pnlGraphics);
                _bTransposeFlag = true;
            }
        }

		public void OpenQuery()
		{
			ScreenListView listView = new ScreenListView(1);
			DialogResult dlgResult = listView.ShowDialog(this);

			if(dlgResult == DialogResult.OK)
			{
				pnlIO.Controls.Clear();
				ListItem listItem = listView.SelectedItemObj as ListItem;

				Open(listItem,pnlIO,"IO");
			}
		}

		
		public void OpenView()
		{
			ScreenListView listView = new ScreenListView(1);
			DialogResult dlgResult = listView.ShowDialog(this);

			if(dlgResult == DialogResult.OK)
			{
				pnlGraphics.Controls.Clear();
				ListItem listItem = listView.SelectedItemObj as ListItem;
				Open(listItem,pnlGraphics,"Graphics");
			}
		}

		
		public bool SupportsOpenCompositeView()
		{
			return false;
		}

		
		public bool SupportsTransposeView()
		{
			return false;
		}

		
		public bool SupportsOpenQuery()
		{
			return false;
		}

		
		public bool SupportsOpenView()
		{
			return false;
		}

		
		public bool SupportsCompositeView()
		{
			return true;
		}


		public bool PerformDirtyChk
		{
			get
			{
				return true;
			}
		}


		public void CopyUOMTypes()
		{
		
		}

		
		public bool SupportsCopyUOMTypes()
		{
			return false;
		}


		public void PrintPreview()
		{
		}


		public void ClearSort()
		{
		}

		
		public bool SupportsClearSort()
		{
			// TODO:  Add CompositeView.SupportsClearSort implementation
			return false;
		}


		public void Refresh()
		{

		}

		
		public void Save()
		{
			if(viewID != null && queryID !=null)
			{
                
				SaveQueryForm saveQuery = new SaveQueryForm(_currentCompositeViewName, "Composite View","CompositeViews");
                LRRCompositeView compView;
				DialogResult result =  saveQuery.ShowDialog(this);
                if(result == DialogResult.OK)
				{
                    SaveCompositeView(saveQuery);
				}
			}
		}

        private void SaveCompositeView(SaveQueryForm saveQuery)
        {
            _currentCompositeViewName = saveQuery.queryName;
            string userSettingID = null;
            string compositeViewID = null;
            LRRCompositeViews compositeViews = LRRModel.ChildModel.CompositeViews;
            //LRRCompositeView compositeViewOld = null;
            if (compositeViews.Count > 0)
            {
                foreach (LRRCompositeView compositeView in compositeViews)
                {
                    if (compositeView.Name == saveQuery.queryName)
                    {
                        DialogResult dlgResult = LRRMessagBox.ShowMessage("SID_QUERYNAME_DUPLICATE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dlgResult == DialogResult.No)
                        {
                            return;
                        }
                        else
                        {
                            compositeViewID = compositeView.ID;
                            userSettingID = compositeView.UserSettingsID;
                            //compositeViewOld = compositeView;
                            compositeView.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                            _lrrApp.DBManagerObj.CommitObject(compositeView);

                            LRRSearchGroupLevel searchGrpLevel = LRRModel.SearchGroupLevels.GetLevelFromID(compositeView.ID);
                            if (searchGrpLevel != null)
                            {
                                searchGrpLevel.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                                _lrrApp.DBManagerObj.CommitObject(searchGrpLevel);
                            }

                            //Get Level Link object
                            LRRSearchGroupLevelLink lrrSrchGroupLevelLink = new LRRSearchGroupLevelLink();
                            _lrrApp.DBManagerObj.PopulateObject(lrrSrchGroupLevelLink, typeof(LRRSearchGroupLevelLink), " where ChildID = '" + compositeView.ID + "'");

                            //Delete Level Link Object
                            if (lrrSrchGroupLevelLink != null)
                            {
                                lrrSrchGroupLevelLink.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                                _lrrApp.DBManagerObj.CommitObject(lrrSrchGroupLevelLink);
                            }

                            compositeView.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                            LRRModel.ChildModel.CompositeViews.Remove(compositeView);
                        }
                        break;
                    }  
                }
            }

            LRRCompositeView compView = new LRRCompositeView();
            if (compositeViewID != null)
                compView.ID = compositeViewID;

            if (userSettingID != null)
                compView.UserSettingsID = userSettingID;

            compView.Name = saveQuery.queryName;
            compView.Description = saveQuery.queryDesc;
            compView.ViewID = viewID;
            compView.QueryID = queryID;
            compView.Type1 = type1;
            compView.Type2 = type2;

            _lrrApp.DBManagerObj.CommitObject(compView);
            LRRModel.ChildModel.CompositeViews.Add(compView);
            //Save in the levels
            //Group Levels
            LRRSearchGroupLevel lrrLevel = new LRRSearchGroupLevel();
            lrrLevel.LevelID = compView.ID;
            lrrLevel.Name = saveQuery.queryName;
            lrrLevel.Description = saveQuery.queryDesc;
            lrrLevel.Type = "CompositeViews";
            lrrLevel.IsPublished = saveQuery.IsPublished;
            lrrLevel.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;

            LRRSearchGroupLevelLink lrrLvlLink = new LRRSearchGroupLevelLink();
            lrrLvlLink.ChildID = compView.ID;
            lrrLvlLink.ParentID = saveQuery.SelectedLevel.LevelID;
            lrrLvlLink.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;

            _lrrApp.DBManagerObj.CommitObject(lrrLevel);
            _lrrApp.DBManagerObj.CommitObject(lrrLvlLink);
            LRRModel.ChildModel.SearchGroupLevels.Add(lrrLevel);
            _compositeViewID = compView.ID;
            string userSettingsID = ApplicationPersistanceUtil.SaveCompositeViewSettings(this, compView, _lrrApp);
            compView.UserSettingsID = userSettingsID;
            compView.ObjectState = Symcon.Globals.Enums.DBOperation.UPDATE;
            _lrrApp.DBManagerObj.CommitObject(compView);
            LRRMessagBox.ShowMessage("SID_COMPOSITEVIEW_SAVED", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

		public void CopyMasterProperties()
		{
		}

		
		public void Paste()
		{
		}

		
		public void Copy()
		{
		}

		
		public void Print()
		{
		}

		
		public void Delete()
		{
		}

		
		public void ArrangeColumns()
		{
		}

		
		public void ClearFilter()
		{
		}

		
		public void InsertNewRows()
		{
		}

		
		public void ImportToExcel()
		{
		}

		
		public void Export()
		{
		}

		
		public void SelectionPaste()
		{
		}

		
		public void RightLevel()
		{
		}

		
		public void LeftLevel()
		{
		}

		
		public void UpLevel()
		{
		}

		
		public void DownLevel()
		{
		}

		
		public bool SupportsMasterPropertiesCopy()
		{
			return false;
		}

		
		public bool SupportsCopy()
		{
			return false;
		}

		
		public bool SupportsPaste()
		{
			return false;
		}

		
		public bool SupportsExport()
		{
			return false;
		}

		
		public bool SupportsSave()
		{
			return true;
		}

		
		public bool SupportsRefresh()
		{
			return false;
		}

		
		public bool SupportsDelete()
		{
			return false;
		}

		
		public bool SupportsPrint()
		{
			return false;
		}

		
		public bool SupportsArrangeColumns()
		{
			return false;
		}

		
		public bool SupportsClearFilter()
		{
			return false;
		}

		
		public bool SupportsInsertNewRows()
		{
			return false;
		}

		
		public bool SupportsImportToExcel()
		{
			return false;
		}

		
		public bool SupportsSelectionPaste()
		{
			return false;
		}

		
		public bool SupportsLevel()
		{
			return false;
		}
		
		

		
		public TabControl	ParentTabControl
		{
			set
			{
				_parentTabControl = value;
			}
			get{return _parentTabControl;}
		}
	
		
		public TabPage ParentTabPage
		{
			set
			{
				_parentTabPage = value;
			}
			get
			{
				return _parentTabPage;
			}
		}


		#endregion

		#region Private Methods

        private void Zoom()
        {
            try
            {
                double dblZoom = double.Parse(comboBoxBarItemZoom.TextBoxValue.Replace("%", "").Trim(), Symcon.BusinessObjects.SymConverter.Culture);
                int nZoom = (int)dblZoom;
                comboBoxBarItemZoom.TextBoxValue = nZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";

                if (_zoomCanvas != null)
                {
                    _zoomCanvas.Zoom(nZoom.ToString());
                }
            }
            catch (Exception ex)
            {
                comboBoxBarItemZoom.TextBoxValue = _lastZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
                Logger.LogMessage(typeof(ExcelView), "buttonZoom_Click Event:" + ex.Message, Logger.Level.Debug);
            }
        }

		private void Open(ListItem listItem,Panel pnl,string panelType)
		{
			if(pnl.Name == "pnlIO")
			{
				ClearGraphicsManagersType1();
                this.dockingManager1.SetDockVisibility(this.overviewControl1, false);
			}
			else
			{
				ClearGraphicsManagersType2();
                this.dockingManager1.SetDockVisibility(this.overviewControl1, false);
			}
			switch(listItem.ListItemType)
			{
				case "Query" :
					LRRSearchCriteria  lrrSrchCriteria = listItem.ListItemObject as LRRSearchCriteria;
					ArrayList arrList = OpenQueryForSearchID(lrrSrchCriteria.ID);
					if(arrList == null || arrList.Count > 0)
					{
						SetPanel(arrList,pnl);
					}
					else
					{
						LRRMessagBox.ShowMessage(this,"SID_INVALID_QUERY","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Error);      
					}
					if(panelType == "IO")
					{
						queryID = lrrSrchCriteria.ID;
						type1 = "Query";
					}
					else
					{
						viewID = lrrSrchCriteria.ID;
						type2  = "Query";
					}
					Syncfusion.Windows.Forms.Tools.XPMenus.Bar resultsFilter = _lrrApp.LRRUIManagerObj.MainFrameObj.BarResultsFilter;
					_lrrApp.LRRUIManagerObj.MainFrameObj.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(resultsFilter,true);
					break;
				case "Graphics" :
					LRRSavedView lrrsvView = listItem.ListItemObject as LRRSavedView;
					Panel pnl1;
					if(pnl.Name == "pnlIO")
					{
						pnl1 = OpenFilesForType1(lrrsvView.ID);
					}
					else
					{
						pnl1 = OpenFilesForType2(lrrsvView.ID);
					}
                    if (pnl1 != null)
                    {
                        pnl1.Controls[0].Click += new EventHandler(pnl1_Click);
                        pnl1.Dock = DockStyle.Fill;
                        pnl.Controls.Add(pnl1);
                        pnl.BackColor = pnl1.BackColor;
                        if (panelType == "IO")
                        {
                            queryID = lrrsvView.ID;
                            type1 = "Graphics";
                        }
                        else
                        {
                            viewID = lrrsvView.ID;
                            type2 = "Graphics";
                        }
                    }
					break;
				case "CompositeViews" :
					LRRCompositeView  lrrCompView = listItem.ListItemObject as LRRCompositeView;

                    if (panelType == "IO" || panelType == "Graphics")
                    {
                        CompositeView compView = null;
                        if (panelType == "IO")
                            compView= new CompositeView(true);
                        else
                            compView = new CompositeView(false);
                        Control[] ctlArray = compView.Controls.Find("pnlFull", true);
                        if (ctlArray.Length > 0)
                        {
                            foreach (Control ctl in ctlArray)
                            {
                                if (ctl.Name == "pnlFull")
                                {
                                    pnl.Controls.Add(ctl);
                                }
                            }
                        }
                        compView.OpenCompositeView(lrrCompView);
                        if (panelType == "IO")
                        {
                            queryID = lrrCompView.ID;
                            type1 = "CompositeViews";
                        }
                        else
                        {
                            viewID = lrrCompView.ID;
                            type2 = "CompositeViews";
                        }

                        SetCanvasClickEventForInnerCVs(pnl);
                    }
                    else
                    {
                        OpenCompositeView(lrrCompView);
                    }
                    
					break;
				case "ExcelView" :
					LRRExcelView lrrExcelView = listItem.ListItemObject as LRRExcelView;
					//LRRChart  lrrChart = LRRModel.ChildModel.Charts.GetChartFromID(lrrCompView.QueryID);
					ExcelView ev = new ExcelView();
					Panel pnlEV = ev.Open(lrrExcelView.ID);
					pnlEV.Dock = DockStyle.Fill;
					pnl.Controls.Add(pnlEV);
					if(panelType == "IO")
					{
						queryID = lrrExcelView.ID;
						type1 = "ExcelView";
					}
					else
					{
						viewID = lrrExcelView.ID;
						type2  = "ExcelView";
					}
					break;
                case "UnifiedView":
                    LRRUnifiedViewSearchCrtiteria lrrUnifiedView = listItem.ListItemObject as LRRUnifiedViewSearchCrtiteria;
                    ArrayList pnlCollection = openUnifiedViewForSearchID(lrrUnifiedView.ID);

                    if (pnlCollection == null || pnlCollection.Count > 0)
                    {
                        SetPanel(pnlCollection, pnl);
                    }
                    if (panelType == "IO")
                    {
                        queryID = lrrUnifiedView.ID;
                        type1 = "UnifiedView";
                    }
                    else
                    {
                        viewID = lrrUnifiedView.ID;
                        type2 = "UnifiedView";
                    }
                    break;
				default :
					break;
			}
		}

        void pnl1_Click(object sender, EventArgs e)
        {
            //Panel pnl1 = sender as Panel;
            Canvas canvas = sender as Canvas;
            _zoomCanvas = canvas;
            if (canvas != null)
            {
                comboBoxBarItemZoom.Enabled = true;
                comboBoxBarItemZoom.TextBoxValue = canvas.View.Magnification.Width + "%";
                this.dockingManager1.SetDockVisibility(this.overviewControl1, true);
                this.overviewControl1.Model = canvas.Model;
                this.overviewControl1.View = canvas.View;
            }
            else
            {
                this.dockingManager1.SetDockVisibility(this.overviewControl1, false);
                comboBoxBarItemZoom.Enabled = false;
                
            }
        }


		private Panel OpenFilesForType1(string fileID)
		{
			try
			{
				gmsType1.CreateGraphicsManager();
				Canvas canvas = gmsType1.ActiveManager.LoadCanvas(fileID);
				lrrSavedViews.Clear();
				_lrrApp.DBManagerObj.PopulateCollection(lrrSavedViews);
				LRRSavedView lrsv =	lrrSavedViews.GetSavedViewFromID(fileID);
				canvas.FileDescription = lrsv.SavedViewDescription;
				canvas.FileName	= lrsv.SavedViewName;
				canvas.FileID = lrsv.ID	 ;
                (canvas as Control).Name = fileID;
				//	gms.OpenedViews.Add(lrsv.ID, canvas);
				//_lrrApp.DBManagerObj.PopulateCollection(gms.SelectedModes);
				
				gmsType1.ActiveManager.BindSymbols(gmsType1.SelectedModes,GraphicsManagers.PropertiesMappingMode.OPEN,GraphicsManagers.UserMode.OPERATOR);
				this.WireGmsType1Events();
				Panel pnl = new Panel();
				pnl.Controls.Add(canvas);
                canvas.View.BackgroundColor = canvas.Model.BackgroundStyle.Color;
               // overviewControl1.View = canvas.View;
                //overviewControl1.Model = canvas.Model;
				return pnl;
				
			}

			catch(Exception	ex)
			{
				//				MessageBox.Show(ex.Message);
				Logger.LogMessage(typeof(CompositeView),ex.Message  ,Logger.Level.Error);
				return null;
			}

			//			LRRSavedViewData lrrsvData = GetCanvasData(fileID);
			//			byte[] byteArray = lrrsvData.ViewBlob;
			//			MemoryStream ms = null;                          // Writes the BLOB to a file (*.bmp).
			//			try
			//			{
			//				ms = new MemoryStream(byteArray);
			//				Canvas canvas = new Canvas();
			//				canvas.LoadBinary(ms);
			//				Panel pnl = new Panel();
			//				pnl.Controls.Add(canvas);
			//				return pnl;
			//			}
			//			catch(Exception ex)
			//			{
			//				MessageBox.Show(ex.Message);
			//				return null;
			//			}
			//			finally
			//			{
			//				ms.Close();
			//			}
		}


		private void ClearGraphicsManagersType1()
		{
			if(gmsType1.ActiveManager != null)
			{
                gmsType1.ActiveManager.LRRBindingManagerObj.RemoveBinding();
				gmsType1.RemoveGraphicsManager(gmsType1.ActiveManager.ID);
				this.UnWireGmsType1Events();
			}
		}


		private Panel OpenFilesForType2(string fileID)
		{
			try
			{
				gmsType2.CreateGraphicsManager();
				Canvas canvas = gmsType2.ActiveManager.LoadCanvas(fileID);
				lrrSavedViews.Clear();
				_lrrApp.DBManagerObj.PopulateCollection(lrrSavedViews);
				LRRSavedView lrsv =	lrrSavedViews.GetSavedViewFromID(fileID);
				canvas.FileDescription = lrsv.SavedViewDescription;
				canvas.FileName	= lrsv.SavedViewName;
				canvas.FileID = lrsv.ID	 ;
                (canvas as Control).Name = fileID;
				//	gms.OpenedViews.Add(lrsv.ID, canvas);
				//_lrrApp.DBManagerObj.PopulateCollection(gms.SelectedModes);
				
				gmsType2.ActiveManager.BindSymbols(gmsType2.SelectedModes,GraphicsManagers.PropertiesMappingMode.OPEN,GraphicsManagers.UserMode.OPERATOR);
				this.WireGmsType2Events();
				Panel pnl = new Panel();
				pnl.Controls.Add(canvas);
                canvas.View.BackgroundColor = canvas.Model.BackgroundStyle.Color;
               // overviewControl1.View = canvas.View;
                //overviewControl1.Model = canvas.Model;
				return pnl;
				
			}

			catch(Exception	ex)
			{
				//				MessageBox.Show(ex.Message);
				Logger.LogMessage(typeof(CompositeView),ex.Message  ,Logger.Level.Error);
				return null;
			}

			//			LRRSavedViewData lrrsvData = GetCanvasData(fileID);
			//			byte[] byteArray = lrrsvData.ViewBlob;
			//			MemoryStream ms = null;                          // Writes the BLOB to a file (*.bmp).
			//			try
			//			{
			//				ms = new MemoryStream(byteArray);
			//				Canvas canvas = new Canvas();
			//				canvas.LoadBinary(ms);
			//				Panel pnl = new Panel();
			//				pnl.Controls.Add(canvas);
			//				return pnl;
			//			}
			//			catch(Exception ex)
			//			{
			//				MessageBox.Show(ex.Message);
			//				return null;
			//			}
			//			finally
			//			{
			//				ms.Close();
			//			}
		}


		private void ClearGraphicsManagersType2()
		{
			if(gmsType2.ActiveManager != null)
			{
                gmsType2.ActiveManager.LRRBindingManagerObj.RemoveBinding();
				gmsType2.RemoveGraphicsManager(gmsType2.ActiveManager.ID);
				this.UnWireGmsType2Events();
			}
		}


		private void WireGmsType1Events()
		{
			gmsType1.CanvasSelectionChanged+=new NodeCollectionEventHandler(gmsType1_CanvasSelectionChanged);
//			gmsType1.LabelTextChanged+=new Symcon.LightRidge.BusinessObjects.PropertyChanged(gmsType1_LabelTextChanged);
			gmsType1.LabelPropertyValueChanged+=new Symcon.LightRidge.BusinessObjects.PropertyChanged(gmsType1_LabelPropertyValueChanged);
			gmsType1.MouseEnter+=new NodeMouseEventHandler(gmsType1_MouseEnter);
			gmsType1.ShowAttributeDetails+=new EventHandler(gmsType1_ShowAttributeDetails);
			gmsType1.SelectDisplayUOM+=new EventHandler(gmsType1_SelectDisplayUOM);
			gmsType1.ViewTrend+=new EventHandler(gmsType1_ViewTrend);
			gmsType1.LabelDoubleClick+=new NodeMouseEventHandler(gmsType1_LabelDoubleClick);
			gmsType1.OverrideValueClick+=new EventHandler(gmsType1_OverrideValueClick);
            gmsType1.NodeDoubleClick += new NodeMouseEventHandler(gmsType1_NodeDoubleClick);
		}

        void gmsType1_NodeDoubleClick(object sender, NodeMouseEventArgs evtArgs)
        {
            if (gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() != typeof(LRRLabel))
            {
                if (evtArgs.Node.GetType().BaseType == typeof(Symbol))
                {
                    GraphicalLink gphLink = (evtArgs.Node.GetType().GetProperty("Link").GetValue(evtArgs.Node, null)) as GraphicalLink;
                    if (gphLink != null)
                    {
                        OpenLink(gphLink);

                    }
                }
            }
            evtArgs.ContinueOperation = true;
        }
	
		private void UnWireGmsType1Events()
		{
			gmsType1.CanvasSelectionChanged-=new NodeCollectionEventHandler(gmsType1_CanvasSelectionChanged);
//			gmsType1.LabelTextChanged-=new Symcon.LightRidge.BusinessObjects.PropertyChanged(gmsType1_LabelTextChanged);
			gmsType1.LabelPropertyValueChanged-=new Symcon.LightRidge.BusinessObjects.PropertyChanged(gmsType1_LabelPropertyValueChanged);
			gmsType1.MouseEnter-=new NodeMouseEventHandler(gmsType1_MouseEnter);
			gmsType1.ShowAttributeDetails-=new EventHandler(gmsType1_ShowAttributeDetails);
			gmsType1.SelectDisplayUOM-=new EventHandler(gmsType1_SelectDisplayUOM);
			gmsType1.ViewTrend-=new EventHandler(gmsType1_ViewTrend);
			gmsType1.LabelDoubleClick-=new NodeMouseEventHandler(gmsType1_LabelDoubleClick);
			gmsType1.OverrideValueClick-=new EventHandler(gmsType1_OverrideValueClick);
            gmsType1.NodeDoubleClick -= new NodeMouseEventHandler(gmsType1_NodeDoubleClick);
		}

		private void WireGmsType2Events()
		{
			gmsType2.CanvasSelectionChanged+=new NodeCollectionEventHandler(gmsType2_CanvasSelectionChanged);
//			gmsType2.LabelTextChanged+=new Symcon.LightRidge.BusinessObjects.PropertyChanged(gmsType2_LabelTextChanged);
			gmsType2.LabelPropertyValueChanged+=new Symcon.LightRidge.BusinessObjects.PropertyChanged(gmsType2_LabelPropertyValueChanged);
			gmsType2.MouseEnter+=new NodeMouseEventHandler(gmsType2_MouseEnter);
			gmsType2.ShowAttributeDetails+=new EventHandler(gmsType2_ShowAttributeDetails);
			gmsType2.SelectDisplayUOM +=new EventHandler(gmsType2_SelectDisplayUOM);
			gmsType2.ViewTrend+=new EventHandler(gmsType2_ViewTrend);
			gmsType2.LabelDoubleClick+=new NodeMouseEventHandler(gmsType2_LabelDoubleClick);
			gmsType2.OverrideValueClick+=new EventHandler(gmsType2_OverrideValueClick);
            gmsType2.NodeDoubleClick += new NodeMouseEventHandler(gmsType2_NodeDoubleClick);

		}

        void gmsType2_NodeDoubleClick(object sender, NodeMouseEventArgs evtArgs)
        {
            if (gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() != typeof(LRRLabel))
            {
                if (evtArgs.Node.GetType().BaseType == typeof(Symbol))
                {
                    GraphicalLink gphLink = (evtArgs.Node.GetType().GetProperty("Link").GetValue(evtArgs.Node, null)) as GraphicalLink;
                    if (gphLink != null)
                    {
                        OpenLink(gphLink);

                    }
                }
            }
            evtArgs.ContinueOperation = true;
        }
		
		private void UnWireGmsType2Events()
		{
			gmsType2.CanvasSelectionChanged-=new NodeCollectionEventHandler(gmsType2_CanvasSelectionChanged);
//			gmsType2.LabelTextChanged-=new Symcon.LightRidge.BusinessObjects.PropertyChanged(gmsType2_LabelTextChanged);
			gmsType2.LabelPropertyValueChanged-=new Symcon.LightRidge.BusinessObjects.PropertyChanged(gmsType2_LabelPropertyValueChanged);
			gmsType2.MouseEnter-=new NodeMouseEventHandler(gmsType2_MouseEnter);
			gmsType2.ShowAttributeDetails-=new EventHandler(gmsType2_ShowAttributeDetails);
			gmsType2.SelectDisplayUOM-=new EventHandler(gmsType2_SelectDisplayUOM);
			gmsType2.ViewTrend-=new EventHandler(gmsType1_ViewTrend);
			gmsType2.LabelDoubleClick-=new NodeMouseEventHandler(gmsType2_LabelDoubleClick);
			gmsType2.OverrideValueClick-=new EventHandler(gmsType2_OverrideValueClick);
            gmsType2.NodeDoubleClick -= new NodeMouseEventHandler(gmsType2_NodeDoubleClick);
		}

		
		private LRRSavedViewData GetCanvasData(string fileID)
		{
			try
			{
				string whereClause =  " WHERE ID = '"+ fileID + "'"; // Where Clause for Querying the DB
				LRRSavedViewDatas lrrSavedViewDatas = new LRRSavedViewDatas(); 
				_lrrApp.DBManagerObj.PopulateCollection(lrrSavedViewDatas,whereClause);//Fetch Saved View data using FileID
				LRRSavedViewData myobj = lrrSavedViewDatas[0];
				return myobj;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(CompositeView),ex.Message  ,Logger.Level.Error);
				return null;
			}
		}

		
		private void barItemOpenQuery_Click(object sender, System.EventArgs e)
		{
			_lrrSearchCriterias = LRRModel.ChildModel.SearchCriterias.GetSearchCriteriasWithoutNewQuery();//new LRRSearchCriterias();
			CompositeFileDialog openQuery = new CompositeFileDialog(_lrrSearchCriterias);
			DialogResult dlgResult = openQuery.ShowDialog(this);
			if(dlgResult == DialogResult.OK)
			{
				pnlIO.Controls.Clear();
				LRRSearchCriteria lrrSearchCriteria = openQuery.ObjSelected as LRRSearchCriteria;
				SetPanel(OpenQueryForSearchID(lrrSearchCriteria.ID),pnlIO);
				queryID = lrrSearchCriteria.ID;
			}
		}
		
		
		private void InitDataSource()
		{
			//			if(_lrrSearchCriterias != null)
			//				_lrrSearchCriterias.Clear();
			if(_lrrSiteComponents != null)
				_lrrSiteComponents.Clear();
			if(_lrrCompositeViews != null)
				_lrrCompositeViews.Clear();

			if(_lrrSearchCriteriaPropertyTypes != null)
				_lrrSearchCriteriaPropertyTypes.Clear();
			if(_LRRSearchCriteriaQualifiers != null)
				_LRRSearchCriteriaQualifiers.Clear();
			if(_objPropertyTypes != null)
				_objPropertyTypes.Clear();

			_lrrSearchCriterias = LRRModel.ChildModel.SearchCriterias.GetSearchCriteriasWithoutNewQuery();//new LRRSearchCriterias();

			_lrrSearchCriteriaPropertyTypes = new LRRSearchCriteriaPropertyTypes();
			_LRRSearchCriteriaQualifiers = new LRRSearchCriteriaQualifiers();

			_objPropertyTypes = new LRRPropertyTypes();


			//			_lrrApp.DBManagerObj.PopulateCollection(_lrrSearchCriterias," where ID <> '0'",false);
			

			_lrrApp.DBManagerObj.PopulateCollection(_lrrSearchCriteriaPropertyTypes,"",false,true);
			_lrrApp.DBManagerObj.PopulateCollection(_LRRSearchCriteriaQualifiers,"",false,true);
			_lrrApp.DBManagerObj.PopulateCollection(_objPropertyTypes,false);

			_lrrSiteComponents = new LRRSiteComponents();
			_lrrApp.DBManagerObj.PopulateCollection(_lrrSiteComponents,false);

			_lrrCompositeViews = new LRRCompositeViews();
			//			_lrrApp.DBManagerObj.PopulateCollection(_lrrCompositeViews);
		
		}
        private ArrayList openUnifiedViewForSearchID(string searchID)
        {
            ArrayList arrXCollection = new ArrayList();
            ArrayList arrYCollection = new ArrayList();

            Symcon.LightRidge.PEAdvisor.CommonViews.CommonViewUtilities.GetCollection(searchID, out arrXCollection, out arrYCollection);
            LRRUnifiedViewSearchCrtiteria lrrSearchCriteriaForUnifiedView = LRRModel.ChildModel.UnifiedViewSearchCrtiterias.GetUnifiedViewByID(searchID);
            string templateClass = lrrSearchCriteriaForUnifiedView.TemplateClass;
            if (templateClass == null || templateClass.Trim() == "")
            {
                templateClass = "Symcon.LightRidge.PEAdvisor.CommonViews.GridSavedViewController";
            }
            Type gridType = Type.GetType(templateClass);
            Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory gridView1;
            object gridView = Activator.CreateInstance(gridType);
            gridView1 = gridView as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
            gridView1.SearchQueryID = searchID;
            Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory objGrid = gridView1 as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
            objGrid.LoadPanel(arrXCollection, arrYCollection);

            LRRResultProperties resultComponents = Symcon.LightRidge.PEAdvisor.CommonViews.CommonViewUtilities.GetSiteComponents(arrXCollection);
            LRRResultProperties resultProperties = Symcon.LightRidge.PEAdvisor.CommonViews.CommonViewUtilities.GetPropeties(arrYCollection);

            ArrayList arrParams = new ArrayList();
            arrParams.Add(objGrid);
            arrParams.Add(resultComponents);
            arrParams.Add(resultProperties);
            Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory unifiedGridView = arrParams[0] as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
            ArrayList pnlCollection = unifiedGridView.GetPanelCollection();
            GridView = gridView1;
            gridView1.HideModes();

            //Show the Components Window
            _resultComponentsWindow.PropertyCollection = resultComponents;
            _resultComponentsWindow.Text = "Result Components Window";
            _resultComponentsWindow.BindGrid();
            _resultComponentsWindow.SelectedProperties += new Symcon.LightRidge.PEAdvisor.CommonViews.ResultWindow.SelectedPropertiesChanged(_resultComponentsWindow_SelectedProperties);
            _resultComponentsWindow.Owner = this;

            //Show the Properties Window
            _resultPropertiesWindow.PropertyCollection = resultProperties;
            _resultPropertiesWindow.Text = "Result Attributes Window";
            _resultPropertiesWindow.BindGrid();
            _resultPropertiesWindow.SelectedProperties += new Symcon.LightRidge.PEAdvisor.CommonViews.ResultWindow.SelectedPropertiesChanged(_resultPropertiesWindow_SelectedProperties);
            _resultPropertiesWindow.Owner = this;

            unifiedGridView.HideProperties(_resultPropertiesWindow.UnSelectedRecords);
            unifiedGridView.HideComponents(_resultComponentsWindow.UnSelectedRecords);

            return unifiedGridView.GetPanelCollection();

        }
		
		private ArrayList OpenQueryForSearchID(string searchID)
		{
			LRRModes lrrModes = LRRModel.ProjectModes;
			SearchRetrievalUtility searchUtil = new SearchRetrievalUtility();
			LRRSiteComponents lrrSiteComponents = searchUtil.PopulateSiteComponents(searchID);
		
			//Aggregates
			LRRAggregates lrrResultAggregates = searchUtil.PopulateAggregates(searchID);
			
			LRRSearchCriteria lrrSearchCriteria = LRRModel.ChildModel.SearchCriterias.GetSearchCriteriaByID(searchID);

			if(lrrSearchCriteria == null)
			{
				return null;
			}

			string sSource = lrrSearchCriteria.Source == null ? "" : lrrSearchCriteria.Source.ToString(Symcon.BusinessObjects.SymConverter.Culture);
			if(sSource == "I/O")
			{
				string templateClass = lrrSearchCriteria.TemplateClass; 
				switch(templateClass)
				{
					case "Symcon.LightRidge.PEAdvisor.GridSimpleView":
						templateClass = "Symcon.LightRidge.PEAdvisor.IOScreen.GridSimpleViewController";
						break;
					case "Symcon.LightRidge.PEAdvisor.GridTransposedView":
						templateClass = "Symcon.LightRidge.PEAdvisor.IOScreen.GridTransposedViewController";
						break;
					case "Symcon.LightRidge.PEAdvisor.GridSimpleGroupedModeView":
						templateClass = "Symcon.LightRidge.PEAdvisor.IOScreen.GridSimpleGroupedModeViewController";
						break;
					case "Symcon.LightRidge.PEAdvisor.GridTransposedGroupedModeView":
						templateClass = "Symcon.LightRidge.PEAdvisor.IOScreen.GridTransposedGroupedModeViewController";
						break;
					default :
						templateClass = templateClass;
						break;
				}
				
				Type gridType = Type.GetType(templateClass);
				object gridView = Activator.CreateInstance(gridType);
				GridView = gridView;
				Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory objGrid = gridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
				
				ArrayList custCollection = new ArrayList();
				custCollection.Add(lrrSiteComponents);
				custCollection.Add(lrrResultAggregates);
                objGrid.SearchQuery.SearchQuery.UserSettingsID = lrrSearchCriteria.UserSettingsID;

				LRRSearchCriteriaItemProperties  srchCriteriaItemProps = new LRRSearchCriteriaItemProperties();
				this._lrrApp.DBManagerObj.PopulateCollection(srchCriteriaItemProps," where SearchCriteriaID = '" + searchID + "'",false,true);

				objGrid.LoadPanel(custCollection,srchCriteriaItemProps);
				
				//Show Results Property Window
				LRRResultProperties resultProperties = SearchRetrievalUtility.ShowPropertiesWindow(lrrSiteComponents,lrrResultAggregates);
				_rsProperties.PropertyCollection = resultProperties;
				_rsProperties.Text = "Result Properties Window";
				_rsProperties.BindGrid();
				_rsProperties.SelectedProperties+=new Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow.SelectedPropertiesChanged(_rsProperties_SelectedProperties);
				_rsProperties.Owner = this;

				//Show the Component List Form
				LRRResultProperties resultComponents = SearchRetrievalUtility.ShowComponents(lrrSiteComponents,lrrResultAggregates);
				_rsComponents.PropertyCollection = resultComponents;
				_rsComponents.Text = "Result Components Window";
				_rsComponents.BindGrid();
				_rsComponents.SelectedProperties+=new Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow.SelectedPropertiesChanged(_rsComponents_SelectedProperties);
				_rsComponents.Owner = this;

				objGrid.HideProperties(_rsProperties.UnSelectedRecords);
				objGrid.HideComponents(_rsComponents.UnSelectedRecords);

				return objGrid.GetPanelCollection();
			}
			else
			{
				Type gridType = Type.GetType(lrrSearchCriteria.TemplateClass);
				object gridView = Activator.CreateInstance(gridType);
				GridView = gridView;
				Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory objGrid = gridView as Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory;

				LRRLookUpQualifiers objLookUpQualifiers = new LRRLookUpQualifiers();
				LRRSiteComponents objModelComponent = LRRModel.SiteComponentsObj;
				LRRSiteComponents objSiteCompCollection	 = new LRRSiteComponents();
				string qualIDs = "";
				ArrayList custCollection = new ArrayList();
				foreach(LRRSiteComponent objSiteComponent in lrrSiteComponents)
				{
					LRRSiteComponent objModelSiteComponent = objModelComponent.GetSiteComponentByID(objSiteComponent.SiteComponentID);
					objSiteCompCollection.Add(objModelSiteComponent);
				}
				LRRSearchCriteriaPropertyTypes srchPropTypes = _lrrSearchCriteriaPropertyTypes.FilterBySearchCriteriaID(lrrSearchCriteria.ID);
				if(srchPropTypes.Count > 0)
				{
					LRRSearchCriteriaPropertyType srchPropType = srchPropTypes[0];
					LRRPropertyType propTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(srchPropType.PropertyType);
					if(propTypeObj != null && propTypeObj.IsDataArray)
					{
						LRRPropertyTypes objPropTypes = new LRRPropertyTypes();
						foreach(LRRSearchCriteriaPropertyType srchCriteriaPropType in srchPropTypes)
						{
							LRRPropertyType lrrPropTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(srchCriteriaPropType.PropertyType);
							objPropTypes.Add(lrrPropTypeObj);
						}
						custCollection.Add(objPropTypes);
						custCollection.Add(objSiteCompCollection);
					}
					else
					{
						LRRSearchCriteriaQualifiers lrrSrchCriteriaQualifiers = new LRRSearchCriteriaQualifiers();
						_lrrApp.DBManagerObj.PopulateCollection(lrrSrchCriteriaQualifiers,"",false,true);
						LRRSearchCriteriaQualifiers lrrSrchCriteriaQuals = lrrSrchCriteriaQualifiers.FilterBySearchCriteriaID(searchID); 
						foreach(LRRSearchCriteriaQualifier lrrSearchCriteriaQual in lrrSrchCriteriaQuals)
						{
							if( qualIDs=="")
							{
								qualIDs="'" + lrrSearchCriteriaQual.QualifierID + "'";
							}
							else
								qualIDs=qualIDs + ",'" + lrrSearchCriteriaQual.QualifierID + "'";
				
						}

						_lrrApp.DBManagerObj.PopulateCollection(objLookUpQualifiers," WHERE LookupQualifierID IN (" + qualIDs + ")",false);
						custCollection.Add(objSiteCompCollection);
						custCollection.Add(objLookUpQualifiers);
					}
				}

				LRRSearchCriteriaItemGroupPropertiesUIUOMs srchCriteriaUIUOMs = null;
				srchCriteriaUIUOMs = new LRRSearchCriteriaItemGroupPropertiesUIUOMs();
				this._lrrApp.DBManagerObj.PopulateCollection(srchCriteriaUIUOMs," where SearchCriteriaID = '" + searchID + "'");

				objGrid.LoadPanel(custCollection,srchCriteriaUIUOMs);

				objGrid.HideModes();
				return objGrid.GetPanelCollection();
			}
			
		}

		
		private void SetPanel(ArrayList pnlCollection,Panel pnlGridContainer)
		{
			if(pnlCollection == null || pnlCollection.Count == 0)
			{
				LRRMessagBox.ShowMessage(this,"SID_INVALID_QUERY","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Error);      
				return;
			}
			int heightOfEach = pnlGridContainer.Height/pnlCollection.Count;
			int splitCnt = 0;
			for(int i=pnlCollection.Count-1;i>=0;i--)
			{
				Panel panel = pnlCollection[i] as Panel;
				panel.Height = heightOfEach;
				pnlGridContainer.Controls.Add(panel);
				if(splitCnt < pnlCollection.Count - 1)
				{
					Splitter splitGrid = new Splitter();
					splitGrid.Dock = DockStyle.Top;
					pnlGridContainer.Controls.Add(splitGrid);
					splitCnt++;
				}
			}
			if(pnlGridContainer.Controls.Count > 0)
				pnlGridContainer.Controls[0].Dock = DockStyle.Fill;
		}

		
		private void SetPanelHeight(Panel pnl)
		{
			int iNumPanels = pnl.Controls.Count/2;
			if(iNumPanels != 0)
			{
				int pnlHeight = pnl.Height - (SystemInformation.HorizontalScrollBarHeight);
				int gridPnlHeight = (pnlHeight)/iNumPanels;
				foreach(Control ctl in pnl.Controls)
				{
					if(ctl.GetType() == typeof(Panel))
					{
						ctl.Height = gridPnlHeight;
					}
				}
			}
		}


        private void SetCanvasClickEventForInnerCVs(Control ctl)
        {
            if (ctl is Panel)
            {
                foreach (Control ctlInner in ctl.Controls)
                {
                    SetCanvasClickEventForInnerCVs(ctlInner);
                }
            }
            else if (ctl is Canvas)
            {
                ctl.Click += new EventHandler(pnl1_Click);
            }
        }

		#endregion

		#region Private Properties

		private object GridView
		{
			get { return _gridView;}
			set
			{
				_gridView = value;
			}
		}

		#endregion

		#region Events

		public void OpenCompositeView(LRRCompositeView lrrCompView)
		{
            _currentCompositeViewName = lrrCompView.Name;
            viewID = lrrCompView.ViewID;
            queryID = lrrCompView.QueryID;
			pnlIO.Controls.Clear();
			pnlGraphics.Controls.Clear();
			ClearGraphicsManagersType1();
			ClearGraphicsManagersType2();
            this.dockingManager1.SetDockVisibility(this.overviewControl1, false);
            type1 = lrrCompView.Type1;
            type2 = lrrCompView.Type2;
			switch(lrrCompView.Type1)
			{
                case "CompositeViews":
                    LRRCompositeView lrrCompViewInner = LRRModel.ChildModel.CompositeViews.GetCompositeViewFromID(lrrCompView.QueryID) as LRRCompositeView;
                    CompositeView compView = new CompositeView(true);
                    Control[] ctlArray = compView.Controls.Find("pnlFull", true);
                    if (ctlArray.Length > 0)
                    {
                        foreach (Control ctl in ctlArray)
                        {
                            if (ctl.Name == "pnlFull")
                            {
                                pnlIO.Controls.Add(ctl);
                                compView.OpenCompositeView(lrrCompViewInner);
                                SetCanvasClickEventForInnerCVs(pnlIO);
                                break;
                            }
                        }
                    }
                    break;
				case "Query" :
					ArrayList arrList = OpenQueryForSearchID(lrrCompView.QueryID);
					SetPanel(arrList,pnlIO);
					if(arrList != null && arrList.Count > 0)
					{
						Syncfusion.Windows.Forms.Tools.XPMenus.Bar resultsFilter = _lrrApp.LRRUIManagerObj.MainFrameObj.BarResultsFilter;
						_lrrApp.LRRUIManagerObj.MainFrameObj.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(resultsFilter,true);
					}
					break;
				case "Graphics":
					Panel pnl2 = OpenFilesForType1(lrrCompView.QueryID);
                    pnl2.Controls[0].Click += new EventHandler(pnl1_Click);
					pnl2.Dock = DockStyle.Fill;
					pnlIO.Controls.Add(pnl2);
					break;
				case "ExcelView":
					//LRRChart  lrrChart = LRRModel.ChildModel.Charts.GetChartFromID(lrrCompView.QueryID);
					ExcelView ev = new ExcelView();
					Panel pnlEV = ev.Open(lrrCompView.QueryID);
					pnlEV.Dock = DockStyle.Fill;
					pnlIO.Controls.Add(pnlEV);
					break;
                case "UnifiedView":                   
                    ArrayList panelCollection = openUnifiedViewForSearchID(lrrCompView.QueryID);
                    SetPanel(panelCollection, pnlIO);
                    if (panelCollection != null && panelCollection.Count > 0)
                    {
                        Syncfusion.Windows.Forms.Tools.XPMenus.Bar resultsFilter = _lrrApp.LRRUIManagerObj.MainFrameObj.BarResultsFilter;
                        _lrrApp.LRRUIManagerObj.MainFrameObj.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(resultsFilter, true);
                    }
                    break;
				default:
					break;
			}

			switch(lrrCompView.Type2)
			{
                case "CompositeViews":
                    LRRCompositeView lrrCompViewInner = LRRModel.ChildModel.CompositeViews.GetCompositeViewFromID(lrrCompView.ViewID) as LRRCompositeView;
                    CompositeView compView = new CompositeView(false);
                    Control[] ctlArray = compView.Controls.Find("pnlFull", true);
                    if (ctlArray.Length > 0)
                    {
                        foreach (Control ctl in ctlArray)
                        {
                            if (ctl.Name == "pnlFull")
                            {
                                pnlGraphics.Controls.Add(ctl);
                                compView.OpenCompositeView(lrrCompViewInner);
                                SetCanvasClickEventForInnerCVs(pnlGraphics);
                                break;
                            }
                        }
                    }
                    break;
				case "Query" :
					ArrayList arrList = OpenQueryForSearchID(lrrCompView.ViewID);
					SetPanel(arrList,pnlGraphics);
					if(arrList != null && arrList.Count > 0)
					{
						Syncfusion.Windows.Forms.Tools.XPMenus.Bar resultsFilter = _lrrApp.LRRUIManagerObj.MainFrameObj.BarResultsFilter;
						_lrrApp.LRRUIManagerObj.MainFrameObj.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(resultsFilter,true);
					}
					break;
				case "Graphics":
					Panel pnl2 = OpenFilesForType2(lrrCompView.ViewID);
                    pnl2.Controls[0].Click += new EventHandler(pnl1_Click);
					pnl2.Dock = DockStyle.Fill;
					pnlGraphics.Controls.Add(pnl2);
					break;
				case "ExcelView":
					//LRRChart  lrrChart = LRRModel.ChildModel.Charts.GetChartFromID(lrrCompView.QueryID);
					ExcelView ev = new ExcelView();
                    Panel pnlEV = ev.Open(lrrCompView.ViewID);
					pnlEV.Dock = DockStyle.Fill;
					pnlGraphics.Controls.Add(pnlEV);
                    break;
                case "UnifiedView":
                    ArrayList panelCollection = openUnifiedViewForSearchID(lrrCompView.ViewID);
                    SetPanel(panelCollection, pnlGraphics);
                    if (panelCollection != null && panelCollection.Count > 0)
                    {
                        Syncfusion.Windows.Forms.Tools.XPMenus.Bar resultsFilter = _lrrApp.LRRUIManagerObj.MainFrameObj.BarResultsFilter;
                        _lrrApp.LRRUIManagerObj.MainFrameObj.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(resultsFilter, true);
                    }
                    break;
				default:
					break;
			}
            _compositeViewID = lrrCompView.ID;
            ApplicationPersistanceUtil.ApplyCompositeViewSettings(this, lrrCompView, _lrrApp);
		}

		
		private void barItemTransposeView_Click(object sender, System.EventArgs e)
		{
			if(_bTransposeFlag)
			{
				pnlGraphics.Dock = DockStyle.Right;
				pnlGraphics.Width = (int)((SystemInformation.WorkingArea.Width)* 0.50);
				splitter1.Dock = DockStyle.Right;
				pnlIO.Dock = DockStyle.Fill;
				SetPanelHeight(pnlIO);
				_bTransposeFlag = false;
			}
			else
			{
				pnlGraphics.Dock = DockStyle.Bottom;
				pnlGraphics.Height = (int)((SystemInformation.WorkingArea.Height)* 0.50);
				splitter1.Dock = DockStyle.Bottom;
				pnlIO.Dock = DockStyle.Fill;
				SetPanelHeight(pnlIO);
				_bTransposeFlag = true;
			}
		}

		
		private void ProjectModes_ModesChanged(object sender, EventArgs e)
		{
			Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridView = GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
			if(iGridView == null)
			{
				Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory iGroupGridView = GridView as Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory;
                if (iGroupGridView != null)
                    iGroupGridView.HideModes();
                else
                {
                    Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory iUnifiedGridFactory = GridView as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
                    if (iUnifiedGridFactory != null)
                        iUnifiedGridFactory.HideModes();
                }
			}
			else
			{
				iGridView.HideModes();
			}
		}

		
		private void CompositeView_Closing(object sender, CancelEventArgs e)
        {
            ILRRUIContainer cont = this as ILRRUIContainer;
            LRRCompositeView compView = LRRModel.ChildModel.CompositeViews.GetCompositeViewFromID(_compositeViewID);
            if (compView != null)
            {
                string userSettingsID = ApplicationPersistanceUtil.SaveCompositeViewSettings(this, compView, _lrrApp);
                compView.UserSettingsID = userSettingsID;
                compView.ObjectState = Symcon.Globals.Enums.DBOperation.UPDATE;
                _lrrApp.DBManagerObj.CommitObject(compView);
                LRRModel.ProjectModes.ModesChanged -= new EventHandler(ProjectModes_ModesChanged);
                //Clear All Bindings
                Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridView = GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
                if (iGridView != null)
                {
                    iGridView.ClearAllBindings();
                    iGridView.UnSubscribeReconnectCompleted();
                }

                _rsProperties.SelectedProperties -= new Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow.SelectedPropertiesChanged(_rsProperties_SelectedProperties);
                _rsComponents.SelectedProperties -= new Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow.SelectedPropertiesChanged(_rsComponents_SelectedProperties);
                _rsProperties.Dispose();
                _rsComponents.Dispose();
                _lrrApp.LRRUIManagerObj.HidePropertyDetailsDialog();
                ClearGraphicsManagersType1();
                ClearGraphicsManagersType2();

                //for bar CompositeView
                _lrrApp.MainFrame.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(_lrrApp.MainFrame.CompositeViewBarObj, false);
            }
		}

		
		private void CompositeView_Load(object sender, EventArgs e)
		{
            this.dockingManager1.SetDockVisibility(this.overviewControl1, false);
		}


		private void CompositeView_Deactivate(object sender, EventArgs e)
		{
            comboBoxBarItemZoom.Visible = false;

			if(_rsProperties != null)
				_rsProperties.Hide();
			if(_rsComponents != null)
				_rsComponents.Hide();
		}


        private void CompositeView_Activated(object sender, EventArgs e)
        {
            comboBoxBarItemZoom.Visible = true;

            if (_rsProperties != null && _rsProperties.IsWindowOpen)
                _rsProperties.Show();
            if (_rsComponents != null && _rsComponents.IsWindowOpen)
                _rsComponents.Show();
        }
		
		
		private void _rsProperties_SelectedProperties(ArrayList arrayUnSelectedRecords)
		{
			Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridView = GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
            if (iGridView != null)
                iGridView.HideProperties(arrayUnSelectedRecords);
		}

		
		private void _rsComponents_SelectedProperties(ArrayList arrayUnSelectedRecords)
		{
			Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridView = GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
			if(iGridView != null)
				iGridView.HideComponents(arrayUnSelectedRecords);
		}

        private void _resultComponentsWindow_SelectedProperties(ArrayList arrayUnSelectedRecords)
        {
                Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory iUnifiedGridFactory = GridView as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
                if (iUnifiedGridFactory != null)
                {
                    iUnifiedGridFactory.HideModes();
                    iUnifiedGridFactory.HideComponents(arrayUnSelectedRecords);
                    
                }
        }

        private void _resultPropertiesWindow_SelectedProperties(ArrayList arrayUnSelectedRecords)
        {
                Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory iUnifiedGridFactory = GridView as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
                if(iUnifiedGridFactory != null)
                    iUnifiedGridFactory.HideProperties(arrayUnSelectedRecords);
        }

		private void gmsType1_CanvasSelectionChanged(object sender, NodeCollectionEventArgs evtArgs)
		{
			gmsType1.ActiveManager.CanvasObj.Model.UpdateViews();
		}

	
		private void gmsType1_LabelPropertyValueChanged(object sender, string propertyName)
		{
			//this.gmsType1.ActiveManager.CanvasObj.Model.UpdateViews();
		}

		
		private void gmsType1_MouseEnter(object sender, NodeMouseEventArgs evtArgs)
		{
			GraphicsUtilities.ActivateToolTip(evtArgs,_toolTip,gmsType1);
		}

		
		private void gmsType1_ShowAttributeDetails(object sender, EventArgs e)
		{
			try
			{
				_lrrApp.LRRUIManagerObj.ShowPropertyDetailsDialog();
				_lrrApp.LRRUIManagerObj.UpdatePropertyDetailsPanel(gmsType1);
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(CompositeView),"gmsType1_LabelDoubleClick :"+ex.Message  ,Logger.Level.Debug);
			}

		}

		
		private void gmsType1_SelectDisplayUOM(object sender, EventArgs e)
		{
			if(gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
			{
				GraphicsUtilities.OnUOMChangeForGridCell(gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last,null);
			}
			else
			{
				LRRLabel selectedLbl = gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last as LRRLabel;

				Symbol smbl = selectedLbl.Parent as Symbol;
				if(selectedLbl != null)
				{
					foreach(LRRLabel lbl in smbl.Labels)
					{
						if(lbl.IsUOM && lbl.ParentID == selectedLbl.ParentID)
						{
							if(lbl.ID != null && lbl.Tag != null)
							{
								LRRUOMType lrrUOMType = lbl.Tag as LRRUOMType;
								if(lrrUOMType != null)
									GraphicsUtilities.ChangeUOMClick(lbl,lrrUOMType.UOMTypeID);
								break;
							}
						}
					}
				}
			}
			gmsType1.ActiveManager.CanvasObj.Model.UpdateViews();
		}

		
		private void gmsType1_ViewTrend(object sender, EventArgs e)
		{
		
			MenuItem mItemChild = sender as MenuItem;
            if (mItemChild != null)
            {
                string trendScheme = mItemChild.Text;

                if (gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
                {
                    GridCell gCell = gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last as GridCell;
                    IProperty iProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(gCell.PropertyID);
                    _lrrApp.LRRUIManagerObj.ShowTrend(trendScheme, iProp, gCell.ModeID, gCell.UIUOMID);
                }
                else
                {
                    LRRLabel selectedLbl = gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last as LRRLabel;
                    GraphicsUtilities.OnVeiwTrend(trendScheme, selectedLbl);
                }
            }
		}


		private void gmsType1_LabelDoubleClick(object sender, NodeMouseEventArgs evtArgs)
		{
			_lrrApp.LRRUIManagerObj.ShowPropertyDetailsDialog();
			_lrrApp.LRRUIManagerObj.UpdatePropertyDetailsPanel(gmsType1);
		}

		
		private void gmsType1_OverrideValueClick(object sender, EventArgs e)
		{
			if(gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
			{
				GridCell lbl =  gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last  as GridCell;
				object objPropValue = lbl.Tag ;
				IPropertyValue ipropVal =  lbl.Tag as  IPropertyValue;
				MenuItem miItem = sender as MenuItem;
				if(objPropValue != null)
				{
					LRRSiteComponentPropertyValue lrrPropVal = ipropVal as LRRSiteComponentPropertyValue;
					if(lrrPropVal != null)
					{
						
						
						LRRSiteComponentProperty lrrScp =  LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lbl.PropertyID);
						GraphicsSiteComponentUtilities.OnOverrideValueClickForSiteComponent(miItem,ipropVal,lrrScp);
						GraphicsUtilities.SetColor(lbl,ipropVal);
						gmsType1.ActiveManager.CanvasObj.Model.UpdateViews();
					}
					else
					{
						GraphicsSiteComponentUtilities.OnOverrideValueClickForGroupProp(miItem,ipropVal);
						GraphicsUtilities.SetColor(lbl,ipropVal);
						gmsType1.ActiveManager.CanvasObj.Model.UpdateViews();
					}
				}
			}
			else
			{
			
				LRRLabel lbl =  gmsType1.ActiveManager.CanvasObj.Controller.NodesHit.Last  as LRRLabel;
				object objPropValue = lbl.Tag ;
				IPropertyValue ipropVal =  lbl.Tag as  IPropertyValue;
				MenuItem miItem = sender as MenuItem;

				if(objPropValue != null)
				{
					LRRSiteComponentPropertyValue lrrPropVal = ipropVal as LRRSiteComponentPropertyValue;
					if(lrrPropVal != null)
					{
						LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
						LRRSiteComponentProperty lrrScp = parentLbl.Tag as LRRSiteComponentProperty;
						GraphicsSiteComponentUtilities.OnOverrideValueClickForSiteComponent(miItem,ipropVal,lrrScp);
					}
					else
					{
						GraphicsSiteComponentUtilities.OnOverrideValueClickForGroupProp(miItem,ipropVal);
					}
				}
			}
		}

		
		private void gmsType2_CanvasSelectionChanged(object sender, NodeCollectionEventArgs evtArgs)
		{
//			if(evtArgs.ChangeType == CollectionExChangeType.Insert)
//			{
//				if(evtArgs.Node.GetType() !=	typeof(Syncfusion.Windows.Forms.Diagram.SymbolLabel) &&	 gmsType2.SelectedModes.Count > 0 &&	_lrrApp.LRRUIManagerObj.IsPropertyDialogVisible)
//					_lrrApp.LRRUIManagerObj.UpdatePropertyDetailsPanel(gmsType2);
//			}
			gmsType2.ActiveManager.CanvasObj.Model.UpdateViews();

		}

		
		private void gmsType2_LabelPropertyValueChanged(object sender, string propertyName)
		{
			//this.gmsType2.ActiveManager.CanvasObj.Model.UpdateViews();
		}

		
		private void gmsType2_MouseEnter(object sender, NodeMouseEventArgs evtArgs)
		{
			GraphicsUtilities.ActivateToolTip(evtArgs,_toolTip,gmsType2);
		}

		
		private void gmsType2_ShowAttributeDetails(object sender, EventArgs e)
		{
			try
			{
				_lrrApp.LRRUIManagerObj.ShowPropertyDetailsDialog();
				_lrrApp.LRRUIManagerObj.UpdatePropertyDetailsPanel(gmsType2);
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(CompositeView),"gmsType2_LabelDoubleClick :"+ex.Message  ,Logger.Level.Debug);
			}

		}

		
		private void gmsType2_SelectDisplayUOM(object sender, EventArgs e)
		{
			if(gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
			{
				GraphicsUtilities.OnUOMChangeForGridCell(gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last,null);

			}
			else
			{

				LRRLabel selectedLbl = gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last as LRRLabel;

				Symbol smbl = selectedLbl.Parent as Symbol;
				if(selectedLbl != null)
				{
					foreach(LRRLabel lbl in smbl.Labels)
					{
						if(lbl.IsUOM && lbl.ParentID == selectedLbl.ParentID)
						{
							if(lbl.ID != null && lbl.Tag != null)
							{
								LRRUOMType lrrUOMType = lbl.Tag as LRRUOMType;
								if(lrrUOMType != null)
									GraphicsUtilities.ChangeUOMClick(lbl,lrrUOMType.UOMTypeID);
								break;
							}
						}
					}
				}
			}
			gmsType2.ActiveManager.CanvasObj.Model.UpdateViews();
		}


		private void gmsType2_ViewTrend(object sender, EventArgs e)
		{
			MenuItem mItemChild = sender as MenuItem;
			string trendScheme = mItemChild.Text;

			if(gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
			{
				GridCell gCell =  gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last as GridCell;
				IProperty iProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(gCell.PropertyID);
				_lrrApp.LRRUIManagerObj.ShowTrend(trendScheme,iProp,gCell.ModeID,gCell.UIUOMID);
			}
			else
			{
				LRRLabel selectedLbl = gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last as LRRLabel;
				GraphicsUtilities.OnVeiwTrend(trendScheme,selectedLbl);
			}
		}
		
		
		private void gmsType2_LabelDoubleClick(object sender, NodeMouseEventArgs evtArgs)
		{
			_lrrApp.LRRUIManagerObj.ShowPropertyDetailsDialog();
			_lrrApp.LRRUIManagerObj.UpdatePropertyDetailsPanel(gmsType2);
		}

		
		private void gmsType2_OverrideValueClick(object sender, EventArgs e)
		{
			if(gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
			{
				GridCell lbl =  gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last  as GridCell;
				object objPropValue = lbl.Tag ;
				IPropertyValue ipropVal =  lbl.Tag as  IPropertyValue;
				MenuItem miItem = sender as MenuItem;
				if(objPropValue != null)
				{
					LRRSiteComponentPropertyValue lrrPropVal = ipropVal as LRRSiteComponentPropertyValue;
					if(lrrPropVal != null)
					{
						
						
						LRRSiteComponentProperty lrrScp =  LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lbl.PropertyID);
						GraphicsSiteComponentUtilities.OnOverrideValueClickForSiteComponent(miItem,ipropVal,lrrScp);
						GraphicsUtilities.SetColor(lbl,ipropVal);
						gmsType2.ActiveManager.CanvasObj.Model.UpdateViews();
					}
					else
					{
						GraphicsSiteComponentUtilities.OnOverrideValueClickForGroupProp(miItem,ipropVal);
						GraphicsUtilities.SetColor(lbl,ipropVal);
						gmsType2.ActiveManager.CanvasObj.Model.UpdateViews();
					}
				}
			}
			else
			{
			
				LRRLabel lbl =  gmsType2.ActiveManager.CanvasObj.Controller.NodesHit.Last  as LRRLabel;
				object objPropValue = lbl.Tag ;
				IPropertyValue ipropVal =  lbl.Tag as  IPropertyValue;
				MenuItem miItem = sender as MenuItem;

				if(objPropValue != null)
				{
					LRRSiteComponentPropertyValue lrrPropVal = ipropVal as LRRSiteComponentPropertyValue;
					if(lrrPropVal != null)
					{
						LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
						LRRSiteComponentProperty lrrScp = parentLbl.Tag as LRRSiteComponentProperty;
						GraphicsSiteComponentUtilities.OnOverrideValueClickForSiteComponent(miItem,ipropVal,lrrScp);
					}
					else
					{
						GraphicsSiteComponentUtilities.OnOverrideValueClickForGroupProp(miItem,ipropVal);
					}
				}
			}
		}

        private void comboBoxBarItemZoom_Click(object sender, System.EventArgs e)
		{
			Zoom();
		}

		#endregion

        private void OpenLink(GraphicalLink gphLink)
        {
            string fileID = gphLink.ID;

            switch (gphLink.LinkTypeID)
            {
                case 1:
                    LRRSavedView lrrsView = LRRModel.SavedViews.GetSavedViewFromID(fileID);
                    LRRViewManager.OpenGraphics(lrrsView);
                    break;
                case 2:
                    LRRSearchCriteria lrrsc = LRRModel.ChildModel.SearchCriterias.GetSearchCriteriaByID(fileID);
                    LRRViewManager.OpenQuery(lrrsc);
                    break;
                case 3:
                    LRRUnifiedViewSearchCrtiteria lrrunifiedViewSearchCriteria = LRRModel.ChildModel.UnifiedViewSearchCrtiterias.GetUnifiedViewByID(fileID);
                    LRRViewManager.OpenUnifiedView(lrrunifiedViewSearchCriteria);
                    break;
                case 4:
                    LRRCompositeView compView = LRRModel.ChildModel.CompositeViews.GetCompositeViewFromID(fileID);
                    LRRViewManager.OpenCompositeView(compView);
                    break;
                case 5:
                    LRRExcelView excelView = LRRModel.ChildModel.ExcelViews.GetExcelViewFromID(fileID);
                    LRRViewManager.OpenExcelView(excelView);
                    break;
                case 6:
                    LRRMultiAttributeTrendInfo multiTrendInfo = LRRModel.ChildModel.MultiTrendInfosObj.GetMultiTrendFromID(fileID);
                    LRRViewManager.OpenMultiTrend(multiTrendInfo);
                    break;
                default:
                    break;
            }
        }

        private void pnlIO_Click(object sender, EventArgs e)
        {

        }
	}
}
