using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
    public partial class DBLayoutUC : UserControl
    {
        public DBLayoutUC()
        {
            InitializeComponent();
            
        }

        public DBLayoutUC(XmlDashBoard xmlDashBoard)
        {
            InitializeComponent();
            this.DashBoardXML = xmlDashBoard;
            CreateLayoutPanel();
            _modelTree = new ModelTree();
            _ctrlPropertyForm = new ControlPropertyForm();
        }

        #region Fields/Properties
        private XmlDashBoard _xmlDashBoard = new XmlDashBoard();
        public XmlDashBoard DashBoardXML
        {
            get { return _xmlDashBoard; }
            set { _xmlDashBoard = value; }
        }

        private ModelTree _modelTree;
        private ControlPropertyForm _ctrlPropertyForm;

        #endregion

        #region Methods

        /// <summary>
        /// Method creates the layout panel and the XML structure
        /// </summary>
        public void CreateLayoutPanel()
        {
            int iCols = this.DashBoardXML.Cols;
            int iRows = this.DashBoardXML.Rows;

            Cell[] cells = new Cell[iCols * iRows];
            this.DashBoardXML.Cells = cells;

            this.tableLayoutPanel.Controls.Clear();
            this.tableLayoutPanel.RowCount = iCols * iRows;
            this.tableLayoutPanel.ColumnCount = 4;

            int iIndex = 0;
            for (int iRow = 0; iRow < iRows; iRow++)
            {
                for (int iCol = 0; iCol < iCols; iCol++)
                {
                    //Cell Label
                    Label lblCell = new Label();
                    lblCell.Text = "Cell {" + iRow + "," + iCol + "}";

                    //Control Combo
                    ArrayList arrControlsList = new ArrayList();
                    System.Reflection.Assembly assembly = System.Reflection.Assembly.Load("Symcon.LightRidge.DashBoardControlsLibrary");
                    Type[] types = assembly.GetTypes();
                    for (int i = 0; i < types.Length; i++)
                    {
                        Type type = types[i];
                        if (type.Namespace == "Symcon.LightRidge.DashBoardControlsLibrary")
                        {
                            arrControlsList.Add(type.Name);
                        }
                    }
                    ComboBox cmbControls = new ComboBox();
                    cmbControls.DataSource = arrControlsList;

                    //Bind Button
                    Button btnBindCell = new Button();
                    btnBindCell.Text = "Bind Cell";
                    btnBindCell.Tag = iIndex;
                    btnBindCell.Click += new EventHandler(btnBindCell_Click);

                    //Property Grid
                    /*
                    Button btnPropertyGrid = new Button();
                    btnPropertyGrid.Text = "Properties";
                    btnPropertyGrid.Tag = iIndex;
                    btnPropertyGrid.Click += new EventHandler(btnPropertyGrid_Click);*/

                    Cell cell = new Cell();
                    this.DashBoardXML.Cells[iIndex] = cell;

                    cmbControls.DataBindings.Add("Text",cell,"ControlName");

                    this.tableLayoutPanel.Controls.Add(lblCell, 0, iIndex);
                    this.tableLayoutPanel.Controls.Add(cmbControls, 1, iIndex);
                    this.tableLayoutPanel.Controls.Add(btnBindCell, 2, iIndex);
                    //this.tableLayoutPanel.Controls.Add(btnPropertyGrid, 3, iIndex);
                    iIndex++;
                }
            }
            TableLookAndFeel(iRows, iCols);
        }

        private void TableLookAndFeel(int iRows, int iCols)
        {
            TableLayoutRowStyleCollection rowStyles = this.tableLayoutPanel.RowStyles;
            int rowHeight = 100 / iRows;
            foreach (RowStyle style in rowStyles)
            {
                style.SizeType = SizeType.AutoSize;
                style.Height = rowHeight;
            }

            TableLayoutColumnStyleCollection colStyles = this.tableLayoutPanel.ColumnStyles;
            int colHeight = 100 / iCols;
            foreach (ColumnStyle style in colStyles)
            {
                style.SizeType = SizeType.AutoSize;
                style.Width = colHeight;
            }
        }
        #endregion

        #region Event Handlers
        void btnBindCell_Click(object sender, EventArgs e)
        {
            Button btnBindCell = sender as Button;
            int cellIndex = (int)btnBindCell.Tag;
            _modelTree.CellControl = this.DashBoardXML.Cells[cellIndex];
            _modelTree.ShowDialog();
        }


        void btnPropertyGrid_Click(object sender, EventArgs e)
        {
            Button btnPropertyGrid = sender as Button;
            int cellIndex = (int)btnPropertyGrid.Tag;
            _ctrlPropertyForm.CellControl = this.DashBoardXML.Cells[cellIndex];
            _ctrlPropertyForm.BindControl();
            _ctrlPropertyForm.ShowDialog();
        }
        #endregion
    }
}
