using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Resources;
using System.Reflection;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters;
using Syncfusion.Windows.Forms.Diagram;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.DashBoardControlsLibrary;


namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
    public partial class DashBoardViewer : Form,ILRRUIContainer
    {
        #region Constructors

        public DashBoardViewer()
        {
            InitializeComponent();
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            this.AutoScroll = true;
            InitDataSource();
            OpenDashBoard();

            _lrrApp.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
            _lrrApp.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);

            _attrDetDialog = new AttributeDetailsDialog();
        }

        public DashBoardViewer(LRRDashBoardView lrrDashBoardView)
        {
            InitializeComponent();
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

            _lrrDashBoardView = lrrDashBoardView;

            InitDataSource();
            OpenDashBoard();
            this.AutoScroll = true;
            //Add Bindings of the controls
            foreach (INode node in this.diagram1.Model.Nodes)
            {
                Symbol symbol = node as Symbol;
                ControlNode ctrlNode = symbol.Nodes.First as ControlNode;
                AddBinding(ctrlNode.Control);
                ctrlNode.Control.GetType().GetEvent("AttributeDetailsCalled",BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public).AddEventHandler(ctrlNode.Control,System.Delegate.CreateDelegate(typeof(Symcon.LightRidge.DashBoardControlsLibrary.AttributeDetailsDel),this,"AttributeDetailsCalled"));
            }

            _lrrApp.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
            _lrrApp.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);

            this.diagram1.NodeDoubleClick +=new NodeMouseEventHandler(diagram1_NodeDoubleClick);
            _attrDetDialog = new AttributeDetailsDialog();
        }

        #endregion

        #region Event Handlers

        private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
        {
            this.diagram1.View.Update(true);
        }

        private void AttributeDetailsCalled(object sender, Symcon.LightRidge.DashBoardControlsLibrary.AttributeDetailsEventArgs e)
        {
            _attrDetDialog.UpdatePropertyDetailsPanel(e);
            _attrDetDialog.Owner = this;
            _attrDetDialog.Show();
            _attrDetDialog.WindowState = FormWindowState.Normal;
        }

        private void diagram1_NodeDoubleClick(object sender, NodeMouseEventArgs evtArgs)
        {
            if(evtArgs.Node.GetType().BaseType == typeof(SymbolBase))
			{
                Symbol symbol = evtArgs.Node as Symbol;
                ControlNode ctrlNode = symbol.Nodes.First as ControlNode;
				DashboardLink dhLink = (ctrlNode.Control.GetType().GetProperty("Link").GetValue(ctrlNode.Control, null)) as DashboardLink;
				if(dhLink != null )
				{
                    int linkTypeID = dhLink.LinkTypeID;
					string viewID =	dhLink.ID;
                    Symcon.LightRidge.PEAdvisor.ScreenManagement.ScreenListView listView = new Symcon.LightRidge.PEAdvisor.ScreenManagement.ScreenListView();
                    listView.OpenViews(linkTypeID,viewID);
				}
			}
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Method Initializes the Data Source
        /// </summary>
        private void InitDataSource()
        {
            _lrrDashBoardViewData = new LRRDashBoardViewData();
            _lrrApp.DBManagerObj.PopulateObject(_lrrDashBoardViewData,typeof(LRRDashBoardViewData)," where ID = '" + _lrrDashBoardView.ID +"'");
        }
        
        /// <summary>
        /// Method opens the dashboard
        /// </summary>
        private void OpenDashBoard()
        {
            LRRDashBoardViewData lrrDashBoardViewData = _lrrDashBoardViewData;
            byte[] byteArray = lrrDashBoardViewData.ViewBlob;
            if (byteArray == null)
                return;
            MemoryStream ms = null;                         
            try
            {
                ms = new MemoryStream(byteArray);
                if (ms != null)
                {
                    this.diagram1.LoadBinary(ms);
                    ms.Close();
                    byteArray = null;
                }
            }
            catch (Exception ex)
            {
                if (ms != null)
                {
                    ms.Close();
                }
            }
        }

        /// <summary>
        /// Method gets the source object
        /// </summary>
        /// <param Name1="sourceObjectName"></param>
        /// <param Name1="sourceObjectID"></param>
        /// <returns></returns>
        private object GetSourceObject(string sourceObjectName, string sourceObjectID)
        {
            object retValue = null;
            switch (sourceObjectName)
            {
                case "LRRSiteComponentPropertyValue":
                    retValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(sourceObjectID);
                    break;
                case "LRRSiteGroupPropertyQualifierValueID":
                    retValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sourceObjectID);
                    break;
                default:
                    break;
            }
            return retValue;
        }

        /// <summary>
        /// Method adds the binding
        /// </summary>
        /// <param name="objControl"></param>
        private void AddBinding(Control objControl)
        {
            string sourceObjectName = objControl.GetType().GetProperty("SourceObjectName").GetValue(objControl, null) == null ? "" : objControl.GetType().GetProperty("SourceObjectName").GetValue(objControl, null).ToString();
            string sourceObjectID = objControl.GetType().GetProperty("SourceObjectID").GetValue(objControl, null) == null ? "" : objControl.GetType().GetProperty("SourceObjectID").GetValue(objControl, null).ToString();
            string sourceProperty = objControl.GetType().GetProperty("SourceProperty").GetValue(objControl, null) == null ? "" : objControl.GetType().GetProperty("SourceProperty").GetValue(objControl, null).ToString();

            object sourceObject = GetSourceObject(sourceObjectName, sourceObjectID);

            try
            {
                _lrrBindingManager.AddBinding(sourceObject, sourceProperty, objControl, "TargetValue");
            }
            catch { }
        }
        #endregion

        #region Fields
        private LRRDashBoardViewData _lrrDashBoardViewData;
        private LRRDashBoardView _lrrDashBoardView;
        private LRRApp _lrrApp;

        private TabControl _parentTabControl;
        private TabPage _parentTabPage;

        private LRRBindingManager _lrrBindingManager = new LRRBindingManager();
        private AttributeDetailsDialog _attrDetDialog;
        #endregion

        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Copy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyMasterProperties()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Delete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DownLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Export()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void InsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenQuery()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public TabControl ParentTabControl
        {
            set
            {
                _parentTabControl = value;
            }
            get { return _parentTabControl; }
        }

        public TabPage ParentTabPage
        {
            set
            {
                _parentTabPage = value;
            }
            get
            {
                return _parentTabPage;
            }
        }

        public void Paste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool PerformDirtyChk
        {
            get { return false; }
        }

        public void Print()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PrintPreview()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Save()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void SelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowComponentList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowPropertyList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsArrangeColumns()
        {
            return false;
        }

        public bool SupportsClearFilter()
        {
            return false;
        }

        public bool SupportsClearSort()
        {
            return false;
        }

        public bool SupportsCompositeView()
        {
            return false;
        }

        public bool SupportsCopy()
        {
            return false;
        }

        public bool SupportsCopyUOMTypes()
        {
            return false;
        }

        public bool SupportsDelete()
        {
            return false;
        }

        public bool SupportsExport()
        {
            return false;
        }

        public bool SupportsImportToExcel()
        {
            return false;
        }

        public bool SupportsInsertNewRows()
        {
            return false;
        }

        public bool SupportsLevel()
        {
            return false;
        }

        public bool SupportsMasterPropertiesCopy()
        {
            return false;
        }

        public bool SupportsPageSetup()
        {
            return false;
        }

        public bool SupportsPaste()
        {
            return false;
        }

        public bool SupportsPrint()
        {
            return false;
        }

        public bool SupportsRefresh()
        {
            return false;
        }

        public bool SupportsResultsFilter()
        {
            return false;
        }

        public bool SupportsSave()
        {
            return false;
        }

        public bool SupportsSelectionPaste()
        {
            return false;
        }

        public bool SupportsUndo()
        {
            return false;
        }

        public void TransposeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Undo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}