using System;
using System.Collections.Generic;
using System.Text;
using Symcon.LightRidge.BusinessObjects;

namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
    [Serializable()]
    public class XmlDashBoard
    {
        private string _name;
        public string Name
        {
            get { return _name; }
            set 
            {
                _name = value;
                if (NameChanged != null)
                    NameChanged(this, null);
            }
        }


        private int _rows;
        public int Rows
        {
            get { return _rows; }
            set
            {
                _rows = value;
                if (RowsChanged != null)
                    RowsChanged(this, null);
            }
        }

        private int _cols;
        public int Cols
        {
            get { return _cols; }
            set 
            { 
                _cols = value;
                if (ColsChanged != null)
                    ColsChanged(this, null);
            }
        }

        private Cell[] _cells;
        public Cell[] Cells
        {
            get { return _cells; }
            set { _cells = value; }
        }

        public event EventHandler NameChanged;
        public event EventHandler RowsChanged;
        public event EventHandler ColsChanged;
    }

    [Serializable()]
    public class Cell
    {
        private string _controlName;
        public string ControlName
        {
            get { return _controlName; }
            set
            {
                _controlName = value;
                if (ControlNameChanged != null)
                    ControlNameChanged(this, null);
            }
        }


        private string _maxLimit;
        public string MaxLimit
        {
            get { return _maxLimit; }
            set 
            {
                _maxLimit = value;
                if (MaxLimitChanged != null)
                    MaxLimitChanged(this, "MaxLimit");
            }
        }

        private string _minLimit;
        public string MinLimit
        {
            get { return _minLimit; }
            set 
            {
                _minLimit = value;
                if (MinLimitChanged != null)
                    MinLimitChanged(this, "MinLimit");
            }
        }

        private float _width;
        public float Width
        {
            get { return _width; }
            set
            {
                _width = value;
                if (WidthChanged != null)
                    WidthChanged(this, "Width");
            }
        }

        private float _height;
        public float Height
        {
            get { return _height; }
            set
            {
                _height = value;
                if (HeightChanged != null)
                    HeightChanged(this, "Height");
            }
        }



        private BindingInfo _bindInfo = new BindingInfo();
        public BindingInfo BindInfo
        {
            get { return _bindInfo; }
            set { _bindInfo = value; }
        }

        public event EventHandler ControlNameChanged;
        public event PropertyChanged MaxLimitChanged;
        public event PropertyChanged MinLimitChanged;
        public event PropertyChanged WidthChanged;
        public event PropertyChanged HeightChanged;
    }

    [Serializable()]
    public class BindingInfo
    {
        private string _sourceObjectName;
        public string SourceObjectName
        {
            get { return _sourceObjectName; }
            set 
            {
                _sourceObjectName = value;
                if (SourceObjectNameChanged != null)
                    SourceObjectNameChanged(this, "SourceObjectName");
            }
        }

        private string _sourceObjectID;
        public string SourceObjectID
        {
            get { return _sourceObjectID; }
            set
            {
                _sourceObjectID = value;
                if (SourceObjectIDChanged != null)
                    SourceObjectIDChanged(this, "SourceObjectID");
            }
        }

        private string _sourceProperty;
        public string SourceProperty
        {
            get { return _sourceProperty; }
            set 
            {
                _sourceProperty = value;
                if (SourcePropertyChanged != null)
                    SourcePropertyChanged(this, "SourceProperty");
            }
        }

        private string _targetObject;
        public string TargetObject
        {
            get { return _targetObject; }
            set { _targetObject = value; }
        }

        private string _targetProperty;
        public string TargetProperty
        {
            get { return _targetProperty; }
            set { _targetProperty = value; }
        }

        private string _siteComponentName;
        public string SiteComponentName
        {
            get { return _siteComponentName; }
            set 
            {
                _siteComponentName = value;
                if (SiteComponentNameChanged != null)
                    SiteComponentNameChanged(this, "SiteComponentName");
            }
        }

        private string _siteComponentProperty;
        public string SiteComponentProperty
        {
            get { return _siteComponentProperty; }
            set
            {
                _siteComponentProperty = value;
                if (SiteComponentPropertyChanged != null)
                    SiteComponentPropertyChanged(this, "SiteComponentProperty");
            }
        }

        private string _modeName;
        public string ModeName
        {
            get { return _modeName; }
            set
            {
                _modeName = value;
                if (ModeNameChanged != null)
                    ModeNameChanged(this, "ModeName");
            }
        }

        private LRRSiteComponentProperties _selectedSiteCompProp = new LRRSiteComponentProperties();

        public LRRSiteComponentProperties SelectedSiteComponentProperties
        {
            get { return _selectedSiteCompProp; }
            set { _selectedSiteCompProp = value; }
        }

        private LRRSiteComponentGroupProperties _selectedGroupProps = new LRRSiteComponentGroupProperties();

        public LRRSiteComponentGroupProperties SelectedSiteCompGroups
        {
            get { return _selectedGroupProps; }
            set { _selectedGroupProps = value; }
        }


        private LRRModes _selectedMode = new LRRModes();

        public LRRModes SelectedModes
        {
            get { return _selectedMode; }
            set { _selectedMode = value; }
        }

        public event PropertyChanged SiteComponentNameChanged;
        public event PropertyChanged SiteComponentPropertyChanged;
        public event PropertyChanged ModeNameChanged;
        public event PropertyChanged SourceObjectNameChanged;
        public event PropertyChanged SourceObjectIDChanged;
        public event PropertyChanged SourcePropertyChanged;
    }

}
