using System;
using System.Collections;
using System.Reflection;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageLogger;
using System.Threading;
using Symcon.LightRidge.BusinessObjects;
namespace Symcon.LightRidge.PEAdvisor.DataStore
{
	/// <summary>
	/// Summary description for LRRBindingManager.
	/// </summary>
	public class LRRBindingManager
	{
		public delegate void BindingCallDel(object source,object target);
		private Mutex _mutex = new Mutex();
        private Delegate del; 
        /// <summary>
        /// Initializes a new instance of the <see cref="LRRBindingManager"/> class.
        /// </summary>
		public LRRBindingManager()
		{
			 _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            del = System.Delegate.CreateDelegate(typeof(Symcon.LightRidge.BusinessObjects.PropertyChanged), this, "PropertyChanged");
		}


		#region Properties

        public bool SuspendCascading
        {
            set { _suspendCascading = value; }
            //get { return _disableCascading; }
        }


        /// <summary>
        /// Calls the cascade bindings.
        /// </summary>
        /// <param name="onlyForDerivedPropertyValues">if set to <c>true</c> [only for derived property values].</param>
        public void CallCascadeBindings(bool onlyForDerivedPropertyValues, bool isPreCalc)
        {
            try
            {
                foreach (object key in _sourceObjectsList.Keys)
                {
                   LRRBindingCollection bindingCollection = (LRRBindingCollection)_sourceObjectsList[key];
                   if (bindingCollection != null)
                   {
                       ArrayList arList = bindingCollection["ActualPropertyValue"];
                       for (int i = 0; i < arList.Count; i++)
                       {
                           LRRBindingInfo bindingInfo = (LRRBindingInfo)arList[i];

                           
                           bool check = true;
                           if (onlyForDerivedPropertyValues)
                           {
                               check = false;
                               string formulaID = key.GetType().GetProperty("FormulaID").GetValue(key, null) as string;
                               if ((formulaID != null) && (formulaID != "") )
                               {
                                   if (formulaID == "631F0E52-87F9-4EF6-8123-3F11F7540CA0" || formulaID == "63BCE52F-6D8D-4FCD-9680-4A57930A5597")
                                   {
                                       int iabc = 01;
                                   }

                                   LRRFormula formula = LRRModel.FormulasObj.GetFormulaObjFromID(formulaID);
                                   if (formula != null)
                                   {
                                       if (isPreCalc)
                                       {
                                           if (formula.PreCalc)
                                           {
                                               check = true;
                                           }
                                       }
                                       else
                                       {
                                           if (formula.PostCalc)
                                           {
                                               check = true;
                                           }
                                       }
                                   }
                               }
                           }
                           object sourceValue = key.GetType().GetProperty("ActualPropertyValue", BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public).GetValue(key, null);

                           object[] args = new object[2];
                           args[0] = bindingInfo;
                           args[1] = sourceValue;
                           if (check)
                           {
                               bindingInfo.Target.GetType().GetProperty(bindingInfo.PropName).SetValue(bindingInfo.Target, sourceValue, null);
                               if (BindingCalled != null)
                                   BindingCalled(sourceValue, bindingInfo.Target);
                           }
                       }
                   }
                }
            }
            catch(Exception ex)
            {
                Logger.LogMessage(typeof(LRRBindingManager), ex.Message, Logger.Level.Debug);
            }
        }

		/// <summary>
		/// This event gets all the Events from the properties of the objects
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
		public void PropertyChanged(object sender,string propertyName)
		{

			try
			{
				//_mutex.WaitOne();
				if(_sourceObjectsList.ContainsKey(sender))
				{
					LRRBindingCollection bindingCollection = (LRRBindingCollection)_sourceObjectsList[sender];
					if(bindingCollection != null)
					{
						ArrayList arList = bindingCollection[propertyName];
						for(int i=0;i<arList.Count;i++)
						{
                            LRRBindingInfo bindingInfo = (LRRBindingInfo)arList[i];

                            if (_suspendCascading)
                            {
                                string direction = bindingInfo.Target.GetType().GetProperty("Direction").GetValue(bindingInfo.Target, null) as string;
                                if (direction == "3")
                                    continue;
                            }

                            object sourceValue = sender.GetType().GetProperty(propertyName,BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public).GetValue(sender,null);

                            bindingInfo.Target.GetType().GetProperty(bindingInfo.PropName).SetValue(bindingInfo.Target,sourceValue,null);
							if(BindingCalled != null)
							    BindingCalled(sourceValue,bindingInfo.Target);
						}
					}
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRBindingManager),ex.Message  ,Logger.Level.Debug);
			}
			finally
			{
				//_mutex.ReleaseMutex();
			}

		}

        /// <summary>
        /// Adds the binding.
        /// </summary>
        /// <param name="sourceObject">The source object.</param>
        /// <param name="sourceProperty">The source property.</param>
        /// <param name="targetObject">The target object.</param>
        /// <param name="targetProperty">The target property.</param>
        /// <returns></returns>
		public bool AddBinding(object sourceObject,string sourceProperty,object targetObject,string targetProperty)
		{
			LRRBindingCollection bindCollection = null;
			if(!_sourceObjectsList.ContainsKey(sourceObject))
			{
				bindCollection = new LRRBindingCollection();
				_sourceObjectsList.Add(sourceObject,bindCollection);
				sourceObject.GetType().GetEvent(sourceProperty + "Changed",BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public).AddEventHandler(sourceObject,del);
				bindCollection.Add(sourceProperty,new LRRBindingInfo(targetObject,targetProperty));
			}
			else
			{
				bindCollection = (LRRBindingCollection)_sourceObjectsList[sourceObject];
				sourceObject.GetType().GetEvent(sourceProperty + "Changed",BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public).AddEventHandler(sourceObject,del);
				bindCollection.Add(sourceProperty,new LRRBindingInfo(targetObject,targetProperty));
			}

			try
			{
				targetObject.GetType().GetProperty(targetProperty).SetValue(targetObject,sourceObject.GetType().GetProperty(sourceProperty).GetValue(sourceObject,null),null);
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRBindingManager),ex.Message  ,Logger.Level.Debug);
			}
			return true;
		}

        /// <summary>
        /// Adds the binding.
        /// </summary>
        /// <param name="sourceObject">The source object.</param>
        /// <param name="sourceProperty">The source property.</param>
        /// <param name="targetObject">The target object.</param>
        /// <param name="targetProperty">The target property.</param>
        /// <param name="setInitalState">if set to <c>true</c> [set inital state].</param>
        /// <returns></returns>
		public bool AddBinding(object sourceObject,string sourceProperty,object targetObject,string targetProperty,bool setInitalState)
		{
			LRRBindingCollection bindCollection = null;
			if(!_sourceObjectsList.ContainsKey(sourceObject))
			{
				bindCollection = new LRRBindingCollection();
				_sourceObjectsList.Add(sourceObject,bindCollection);
                sourceObject.GetType().GetEvent(sourceProperty + "Changed", BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public).AddEventHandler(sourceObject, del);
				bindCollection.Add(sourceProperty,new LRRBindingInfo(targetObject,targetProperty));
			}
			else
			{
				bindCollection = (LRRBindingCollection)_sourceObjectsList[sourceObject];
				bindCollection.Add(sourceProperty,new LRRBindingInfo(targetObject,targetProperty));
			}

			try
			{
				if(setInitalState)
					targetObject.GetType().GetProperty(targetProperty).SetValue(targetObject,sourceObject.GetType().GetProperty(sourceProperty).GetValue(sourceObject,null),null);
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRBindingManager),ex.Message  ,Logger.Level.Debug);
			}
			return true;
		}

        /// <summary>
        /// Removes the binding.
        /// </summary>
		public void RemoveBinding()
		{
			try
			{
				foreach(object obj in _sourceObjectsList.Keys)
				{
					LRRBindingCollection bindCollection = null;
					bindCollection = (LRRBindingCollection)_sourceObjectsList[obj];
					IDictionaryEnumerator ienum = bindCollection.GetEnumerator();
					while(ienum.MoveNext())
					{
						string  propertyName = (string)ienum.Key;
						//obj.GetType().GetEvent(propertyName + "Changed",BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public).RemoveEventHandler(obj,System.Delegate.CreateDelegate(typeof(Symcon.LightRidge.BusinessObjects.PropertyChanged),this,"PropertyChanged"));
						EventInfo ei = obj.GetType().GetEvent(propertyName + "Changed",BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public);
                        //MethodInfo mi = ei.GetRaiseMethod();
                        //Delegate del = System.Delegate.CreateDelegate(typeof(Symcon.LightRidge.BusinessObjects.PropertyChanged),this,"PropertyChanged");
						ei.RemoveEventHandler(obj,del);
                        
					}
					bindCollection.Clear();
				}
				_sourceObjectsList.Clear();
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRBindingManager),ex.Message  ,Logger.Level.Debug);
			}
		}

        /// <summary>
        /// Removes the binding.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="sourceProperty">The source property.</param>
        /// <param name="target">The target.</param>
		public void RemoveBinding(object source,string sourceProperty,object target)
		{
            try
            {
                string targetKeyValue = "";
                PropertyInfo[] targetProperties = target.GetType().GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
                IEnumerator iTargetPropEnum = targetProperties.GetEnumerator();
                while (iTargetPropEnum.MoveNext())
                {
                    PropertyInfo targetProp = (PropertyInfo)iTargetPropEnum.Current;
                    SymKeyFieldAttribute[] keyfield = (SymKeyFieldAttribute[])targetProp.GetCustomAttributes(typeof(SymKeyFieldAttribute),
                        true);
                    if (keyfield.Length > 0)
                    {
                        targetKeyValue = targetProp.GetValue(target, null).ToString();
                        break;
                    }
                }

                LRRBindingCollection bindCollection = null;
                if (source != null)
                {
                    bindCollection = (LRRBindingCollection)_sourceObjectsList[source];
                    if (bindCollection == null)
                        return;
                    ArrayList arlist = bindCollection[sourceProperty];
                    int foundIndex = -1;
                    for (int i = 0; i < arlist.Count; i++)
                    {
                        bool check = false;
                        LRRBindingInfo lrrBindInfo = (LRRBindingInfo)arlist[i];
                        PropertyInfo[] properties = lrrBindInfo.Target.GetType().GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
                        IEnumerator iPropEnum = properties.GetEnumerator();
                        while (iPropEnum.MoveNext())
                        {
                            PropertyInfo property = (PropertyInfo)iPropEnum.Current;
                            SymKeyFieldAttribute[] keyfield = (SymKeyFieldAttribute[])property.GetCustomAttributes(typeof(SymKeyFieldAttribute),
                                true);

                            if (keyfield.Length > 0)
                            {
                                string currentKeyValue = property.GetValue(lrrBindInfo.Target, null).ToString();

                                //Keys Match remove the object from the arraylist
                                if (targetKeyValue == currentKeyValue)
                                {
                                    foundIndex = i;
                                    check = true;
                                    break;
                                }
                            }

                        }
                        if (check)
                            break;
                    }
                    if (foundIndex != -1)
                        arlist.RemoveAt(foundIndex);
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(LRRBindingManager), ex.Message, Logger.Level.Debug);
            }
        
		}


        /// <summary>
        /// Removes the binding.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="sourceProperty">The source property.</param>
        /// <param name="target">The target.</param>
        /// <param name="flag">if set to <c>true</c> [flag].</param>
		public void RemoveBinding(object source,string sourceProperty,object target,bool flag)
		{
			try
			{
				LRRBindingCollection bindCollection = null;
				bindCollection = (LRRBindingCollection)_sourceObjectsList[source];
				ArrayList arlist = bindCollection[sourceProperty];
				for(int i=0;i<arlist.Count;i++)
				{
					LRRBindingInfo  lrrBindInfo = (LRRBindingInfo)arlist[i];
					if(lrrBindInfo.Target.Equals(target))
					{
						arlist.Remove(lrrBindInfo);
						break;
					}
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRBindingManager),ex.Message  ,Logger.Level.Debug);
			}
		}
		#endregion
		#region Delegates
		public delegate void PropertyBindingDel(LRRBindingInfo bindingInfo,string sourceValue);
		#endregion

		#region Fields
		private Hashtable				_sourceObjectsList = new Hashtable();
		public event BindingCallDel BindingCalled;
		private LRRApp					_app;
        private bool _suspendCascading = false;
		#endregion
	}

	public class LRRBindingCollection : DictionaryBase
	{
		
		public LRRBindingCollection()
		{

		}

        /// <summary>
        /// Adds the specified prop name.
        /// </summary>
        /// <param name="PropName">Name of the prop.</param>
        /// <param name="bindInfo">The bind info.</param>
		[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
		public void Add(string PropName,LRRBindingInfo bindInfo)
		{
			ArrayList arList = null;
			if(this.Dictionary[PropName] == null)
			{
				arList = new ArrayList();
				this.Dictionary.Add(PropName,arList);
				arList.Add(bindInfo);
			}
			else
			{
				arList = (ArrayList)this.Dictionary[PropName];
				if(arList != null)
					arList.Add(bindInfo);
			}
		}

        /// <summary>
        /// Gets or sets the <see cref="System.Collections.ArrayList"/> with the specified prop name.
        /// </summary>
        /// <value></value>
		public ArrayList this[string PropName]
		{
			[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
			get
			{
				ArrayList arList = this.Dictionary[PropName] as ArrayList;
				return arList;
			}
			[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
			set
			{
				this.Dictionary[PropName] = value;
			}
		}
	}

	public class LRRBindingInfo
	{
		public LRRBindingInfo(object targetObject,string targetProperty)
		{
			_obj = targetObject;
			_propName = targetProperty;
		}

		private object _obj;
        /// <summary>
        /// Gets or sets the target.
        /// </summary>
        /// <value>The target.</value>
		public object Target
		{
			[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
			get{return _obj;}
			[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
			set
			{
				_obj = value;
			}
		}

		private string _propName;
        /// <summary>
        /// Gets or sets the name of the prop.
        /// </summary>
        /// <value>The name of the prop.</value>
		public string PropName
		{
			[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
			get{return _propName;}
			[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
			set
			{
				_propName = value;
			}
		}
	}
}
