using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Data;
using Symcon.Utils.MessageLogger;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Tools.XPMenus;
using Syncfusion.Grouping;
using Syncfusion.ComponentModel;

using Symcon.Database;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;
using System.Collections.Generic;
using Syncfusion.Windows.Forms.Tools;

namespace Symcon.LightRidge.PEAdvisor
{
    public partial class FormulaScriptBrowser : Form, ILRRUIContainer
    {
        public FormulaScriptBrowser()
        {

            InitializeComponent();
            _lrrapp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            PopulateTreeView();
        }
        private LRRComponentTypes lrrCompTypes = null;
        private LRRApp _lrrapp;
        private Dictionary<string, List<LRRSiteComponentProperty>> siteCompPropsWithFormula = LRRModel.SiteCompPropsWithFormulas;
        private List<LRRSiteComponentProperty> lscpWithFormulas = null;
        private List<LRRHistData> histDataWithFormulas = LRRModel.HistDataWithFormulas;
        private TabControl _parentTabControl;
        private TabPage _parentTabPage;

        private void PopulateTreeView()
        {
            treeViewAdv1.SuspendLayout();
            treeViewAdv1.SelectedNodes.Clear();
            treeViewAdv1.Nodes.Clear();
            lrrCompTypes = new LRRComponentTypes();

            _lrrapp.DBManagerObj.PopulateCollection(lrrCompTypes, true);

            int[] folders = new int[1];
            folders[0] = 0;
            int[] leaf = new int[1];
            leaf[0] = 1;
            LRRMasterComponents masterComponents = LRRModel.Model;

            IEnumerator compTypeEnum = lrrCompTypes.GetEnumerator();
            while (compTypeEnum.MoveNext())
            {
                LRRMasterComponents masterComps = new LRRMasterComponents();
                LRRComponentType compType = compTypeEnum.Current as LRRComponentType;
                IEnumerator masterCompEnum = masterComponents.GetEnumerator();

                TreeNodeAdv tnav = new TreeNodeAdv();
                tnav.Text = compType.Name;
                tnav.LeftImageIndices = folders;

                while (masterCompEnum.MoveNext())
                {
                    LRRMasterComponent masterComp = masterCompEnum.Current as LRRMasterComponent;
                    if (compType.ID == masterComp.ComponentID)
                    {
                        compType.MasterComponentObj = masterComponents;
                        masterComp.SiteComponentsObj = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(masterComp.ID);
                        if (masterComp.SiteComponentsObj.Count > 0)
                        {
                            TreeNodeAdv child = new TreeNodeAdv();
                            child.Text = masterComp.ComponentName;
                            child.Tag = masterComp;
                            child.LeftImageIndices = folders;
                            foreach (LRRSiteComponent sc in masterComp.SiteComponentsObj)
                            {
                                if (siteCompPropsWithFormula.ContainsKey(sc.SiteComponentID))
                                {
                                    lscpWithFormulas = siteCompPropsWithFormula[sc.SiteComponentID];
                                    TreeNodeAdv grandChild = new TreeNodeAdv();
                                    string sLevelId = sc.PlantUnit == null ? "" : sc.PlantUnit.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    if (sLevelId != "")
                                    {
                                        LRRLevel levl = LRRModel.ChildModel.Levels.getLevelByLevelID(sLevelId);
                                        if (levl != null)
                                        {
                                            grandChild.Text = sc.Name + " [" + levl.Name + " ]";
                                        }
                                        else grandChild.Text = sc.Name + " [ Level not defined ]";
                                    }
                                    else
                                    {
                                        grandChild.Text = sc.Name + " [ Level not defined ]";
                                    }
                                    grandChild.Text = grandChild.Text + " [" + sc.EquipmentTag + "]";
                                    grandChild.Tag = sc;
                                    grandChild.LeftImageIndices = leaf;
                                    if (sc.Properties.Count > 0)
                                    {
                                        TreeNodeAdv folderGP = new TreeNodeAdv();
                                        folderGP.Text = LRRHeaderText.GetHeaderString("SID_GP");
                                        folderGP.LeftImageIndices = folders;
                                        TreeNodeAdv foldersimpleProp = new TreeNodeAdv();
                                        foldersimpleProp.Text = LRRHeaderText.GetHeaderString("SID_SIMPLE_PROP");
                                        foldersimpleProp.LeftImageIndices = folders; ;
                                        TreeNodeAdv folderDataArrayProp = new TreeNodeAdv();
                                        folderDataArrayProp.Text = LRRHeaderText.GetHeaderString("SID_DATA_ARRYAS");
                                        folderDataArrayProp.LeftImageIndices = folders;
                                        foreach (LRRSiteComponentProperty lscp in sc.Properties)
                                        {

                                            TreeNodeAdv propChildlvl = new TreeNodeAdv();
                                            propChildlvl.Text = lscp.Name;
                                            propChildlvl.Tag = lscp;
                                            if (lscp.Type != null)
                                            {
                                                LRRPropertyType propType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(lscp.Type);
                                                if (propType != null)
                                                {
                                                    if (propType.IsDataArray)
                                                    {
                                                        folderDataArrayProp.Nodes.Add(propChildlvl);
                                                        folderDataArrayProp.Nodes.Sort(SortOrder.Ascending);
                                                    }
                                                    else if (propType.ID == "1")
                                                    {
                                                        if (lscpWithFormulas.Contains(lscp))
                                                        {
                                                            foldersimpleProp.Nodes.Add(propChildlvl);
                                                            foldersimpleProp.Nodes.Sort(SortOrder.Ascending);
                                                        }
                                                    }
                                                    else
                                                    {
                                                        folderGP.Nodes.Add(propChildlvl);
                                                        folderGP.Nodes.Sort(SortOrder.Ascending);
                                                    }
                                                }
                                            }
                                        }
                                        if (folderGP.Nodes.Count > 0)
                                        {
                                            grandChild.Nodes.Add(folderGP);
                                        }
                                        if (foldersimpleProp.Nodes.Count > 0)
                                        {
                                            grandChild.Nodes.Add(foldersimpleProp);
                                        }
                                        if (folderDataArrayProp.Nodes.Count > 0)
                                        {
                                            grandChild.Nodes.Add(folderDataArrayProp);
                                        }
                                        grandChild.Nodes.Sort(SortOrder.Ascending);
                                    }
                                    child.Nodes.Add(grandChild);
                                    child.Nodes.Sort(SortOrder.Ascending);
                                }
                            }
                            if (child.Nodes.Count > 0)
                            {
                                tnav.Nodes.Add(child);
                                tnav.Nodes.Sort(SortOrder.Ascending);
                            }
                            LRRAggregates aggregates = LRRModel.AggregatesObj.GetAggregatesByMasterComponentID(masterComp.ID);
                            if (aggregates != null && aggregates.Count > 0)
                            {
                                TreeNodeAdv aggNode = new TreeNodeAdv();
                                aggNode.Text = "Aggregates";
                                child.Nodes.Add(aggNode);
                                foreach (LRRAggregate aggregate in aggregates)
                                {
                                    TreeNodeAdv grandChild = new TreeNodeAdv();
                                    grandChild.Text = aggregate.Name;
                                    grandChild.Tag = aggregate;
                                    grandChild.LeftImageIndices = leaf;
                                    foreach (LRRAggregateProperty aggregateProp in aggregate.AggregateProperties)
                                    {
                                        TreeNodeAdv grandAggChild = new TreeNodeAdv();
                                        grandAggChild.Text = aggregateProp.Name;
                                        grandAggChild.Tag = aggregateProp;
                                        grandAggChild.LeftImageIndices = leaf;
                                        grandChild.Nodes.Add(grandAggChild);
                                    }
                                    aggNode.Nodes.Add(grandChild);
                                }
                            }
                        }
                    }
                }

                masterComps.SortColumn = "ComponentName";
                masterComps.Sort();

                IEnumerator mastCompEnum = masterComps.GetEnumerator();
                while (mastCompEnum.MoveNext())
                {
                    LRRMasterComponent mastComp = mastCompEnum.Current as LRRMasterComponent;
                    if (compType.ID == mastComp.ComponentID)
                    {
                        TreeNodeAdv child = new TreeNodeAdv();
                        child.Text = mastComp.ComponentName;
                        child.Tag = mastComp;
                        child.LeftImageIndices = folders;
                        tnav.Nodes.Add(child);
                    }
                }
                if (tnav.Nodes.Count > 0)
                {
                    treeViewAdv1.Nodes.Add(tnav);
                }
            }
            TreeNodeAdv tnHistDatas = new TreeNodeAdv();
            tnHistDatas.Text = "HistDatas";
            treeViewAdv1.Nodes.Add(tnHistDatas);
            foreach (LRRHistData histData in histDataWithFormulas)
            {
                TreeNodeAdv tnHistData = new TreeNodeAdv();
                tnHistData.Text = histData.HistTag;
                tnHistData.Tag = histData;
                if (histData.CustomFormulaID != null)
                {
                    if (histData.CustomFormulaObj.FormulaName != null && histData.CustomFormulaObj.FormulaName != "")
                    {
                        TreeNodeAdv tnHistDataCustomFormula = new TreeNodeAdv();
                        tnHistDataCustomFormula.Text = "[Custom Formula]";
                        tnHistDataCustomFormula.Tag = histData.CustomFormulaObj;
                        tnHistData.Nodes.Add(tnHistDataCustomFormula);
                    }
                }
                if (histData.FormulaID != null)
                {
                    if (histData.FormulaObj.FormulaName != null && histData.FormulaObj.FormulaName != "")
                    {
                        TreeNodeAdv tnHistDataNormalFormula = new TreeNodeAdv();
                        tnHistDataNormalFormula.Text = "[Normal Formula]";
                        tnHistDataNormalFormula.Tag = histData.FormulaObj;
                        tnHistData.Nodes.Add(tnHistDataNormalFormula);
                    }
                }
                if (histData.ConversionFormulaID != null)
                {
                    if (histData.ConversionFormulaObj.FormulaName != null && histData.ConversionFormulaObj.FormulaName != "")
                    {
                        TreeNodeAdv tnHistDataCoversionFormula = new TreeNodeAdv();
                        tnHistDataCoversionFormula.Text = "[Conversion Formula]";
                        tnHistDataCoversionFormula.Tag = histData.ConversionFormulaObj;
                        tnHistData.Nodes.Add(tnHistDataCoversionFormula);
                    }
                }
                tnHistDatas.Nodes.Add(tnHistData);
            }
            treeViewAdv1.ResumeLayout();
        }

        private void InitializeGrid(LRRFormulas formulas)
        {
            Symcon.LightRidge.PEAdvisor.GridUtilities.SyncGridUtilities.InitializeGridLookAndFeel(gridFormulas);
            gridFormulas.TableDescriptor.Columns.Clear();
            gridFormulas.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_FORMULANAME"), "FormulaName");
            gridFormulas.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_DESCRIPTION"), "FormulaDescription");
            gridFormulas.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_MODENAME"), "ModeID");
            this.gridFormulas.TopLevelGroupOptions.CaptionText = LRRHeaderText.GetHeaderString("SID_FORMULA") + " {RecordCount} " + LRRHeaderText.GetHeaderString("SID_ITEMS");
            if (gridFormulas.TableDescriptor.Columns[2].Name == LRRHeaderText.GetHeaderString("SID_MODENAME"))
            {
                gridFormulas.TableDescriptor.Columns[2].Appearance.AnyRecordFieldCell.CellType = "ComboBox";
                Syncfusion.Windows.Forms.Grid.Grouping.GridTableCellStyleInfo gtc = gridFormulas.TableDescriptor.Columns[2].Appearance.AnyRecordFieldCell;
                gtc.ShowButtons = Syncfusion.Windows.Forms.Grid.GridShowButtons.Hide;
                gtc.ReadOnly = true;
                gtc.DisplayMember = "ModeName";
                gtc.ValueMember = "ModeId";
                gtc.ReadOnly = true;
                gtc.DataSource = LRRModel.ChildModel.Modes;
                gtc.AutoSize = true;
            }
            gridFormulas.DataSource = null;
            gridFormulas.DataSource = formulas;
        }

        private void treeViewAdv1_AfterSelect(object sender, EventArgs e)
        {
            this.gridFormulas.DataSource = null;
            TreeNodeAdv tnSelectedNode = treeViewAdv1.SelectedNode;
            object selectedObj = tnSelectedNode.Tag;
            if (selectedObj != null)
            {
                if (selectedObj.GetType() == typeof(LRRSiteComponent))
                {
                    LRRSiteComponent lsc = selectedObj as LRRSiteComponent;
                    LRRFormulas formulas = new LRRFormulas();
                    lscpWithFormulas = siteCompPropsWithFormula[lsc.SiteComponentID];
                    foreach (LRRSiteComponentProperty lscp in lscpWithFormulas)
                    {
                        LRRFormulas lstformulas = LRRModel.FormulasObj.FilterFormulasBySourcePropertyID(lscp.ID);
                        foreach (LRRFormula formula in lstformulas)
                        {
                            formulas.Add(formula);
                        }
                    }
                    InitializeGrid(formulas);
                }
                if (selectedObj.GetType() == typeof(LRRAggregate))
                {
                    LRRAggregate aggregate = selectedObj as LRRAggregate;
                    LRRFormulas formulas = new LRRFormulas();
                    foreach (LRRAggregateProperty aggregateProp in aggregate.AggregateProperties)
                    {
                        LRRFormulas lstformulas = LRRModel.FormulasObj.FilterFormulasBySourcePropertyID(aggregateProp.ID);
                        foreach (LRRFormula formula in lstformulas)
                        {
                            formulas.Add(formula);
                        }
                    }
                    InitializeGrid(formulas);
                }
                if (selectedObj.GetType() == typeof(LRRSiteComponentProperty))
                {
                    LRRSiteComponentProperty lscp = selectedObj as LRRSiteComponentProperty;
                    LRRFormulas formulas = LRRModel.FormulasObj.FilterFormulasBySourcePropertyID(lscp.ID);
                    InitializeGrid(formulas);
                }
                else if (selectedObj.GetType() == typeof(LRRHistData))
                {
                    LRRHistData histDataObj = selectedObj as LRRHistData;
                    LRRFormula normalformula = histDataObj.FormulaObj as LRRFormula;
                    LRRFormula customformula = histDataObj.CustomFormulaObj;
                    LRRFormula conversionformula = histDataObj.ConversionFormulaObj;
                    LRRFormulas formulas = new LRRFormulas();
                    if (normalformula.FormulaName != null && normalformula.FormulaName != "")
                    {
                        formulas.Add(normalformula);
                    }
                    if (customformula.FormulaName != null && customformula.FormulaName != "")
                    {
                        formulas.Add(customformula);
                    }
                    if (conversionformula.FormulaName != null && conversionformula.FormulaName != "")
                    {
                        formulas.Add(conversionformula);
                    }
                    InitializeGrid(formulas);
                }
                else if (selectedObj.GetType() == typeof(LRRAggregateProperty))
                {
                    LRRAggregateProperty aggProp = selectedObj as LRRAggregateProperty;
                    LRRFormulas formulas = LRRModel.FormulasObj.FilterFormulasBySourcePropertyID(aggProp.ID);
                    InitializeGrid(formulas);
                }
                else if (selectedObj.GetType() == typeof(LRRFormula))
                {
                    LRRFormulas formulas = new LRRFormulas();
                    LRRFormula formula = selectedObj as LRRFormula;
                    formulas.Add(formula);
                    InitializeGrid(formulas);
                }
            }
        }

        private void gridFormulas_CurrentRecordContextChange(object sender, Syncfusion.Grouping.CurrentRecordContextChangeEventArgs e)
        {
            if (e.Record != null)
            {
                object selectedObj = e.Record.GetData();
                LRRFormula selectedFormulaObj = selectedObj as LRRFormula;
                string FormulaText = selectedFormulaObj.Expression;
                txtFormulaExpression.Text = FormulaText;
                int i = 0;
            }
        }

        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            
        }

        public void ClearFilter()
        {
            
        }

        public void ClearSort()
        {
            
        }

        public void Copy()
        {
            
        }

        public void CopyMasterProperties()
        {
            
        }

        public void CopyUOMTypes()
        {
            
        }

        public void Delete()
        {
            
        }

        public void DownLevel()
        {
            
        }

        public void Export()
        {
            
        }

        public void ImportToExcel()
        {
            
        }

        public void InsertNewRows()
        {
            
        }

        public void LeftColMovement()
        {
            
        }

        public void LeftLevel()
        {
            
        }

        public void OpenCompositeView()
        {
            
        }

        public void OpenQuery()
        {
            
        }

        public void OpenView()
        {
            
        }

        public void PageSetup()
        {
            
        }

        public TabControl ParentTabControl
        {
            set
            {
                _parentTabControl = value;
            }
            get { return _parentTabControl; }
        }

        public TabPage ParentTabPage
        {
            set
            {
                _parentTabPage = value;
            }
            get
            {
                return _parentTabPage;
            }
        }

        public void Paste()
        {
            
        }

        public bool PerformDirtyChk
        {
            get
            {
                // TODO:  Add SiteComponentEntry.PerformDirtyChk getter implementation
                return false;
            }
        }

        public void Print()
        {

        }

        public void PrintPreview()
        {

        }

        public void RightColMovement()
        {

        }

        public void RightLevel()
        {

        }

        public void Save()
        {

        }

        public void SelectionPaste()
        {

        }

        public void ShowComponentList()
        {

        }

        public void ShowPropertyList()
        {
         
        }

        public bool SupportsArrangeColumns()
        {

           return false;

        }

        public bool SupportsClearFilter()
        {
            return false;
        }

        public bool SupportsClearSort()
        {
            return false;
        }

        public bool SupportsCompositeView()
        {
            return false;
        }

        public bool SupportsCopy()
        {
            return false;
        }

        public bool SupportsCopyUOMTypes()
        {
            return false;
        }

        public bool SupportsDelete()
        {
            return false;
        }

        public bool SupportsExport()
        {
            return false;
        }

        public bool SupportsImportToExcel()
        {
            return false;
        }

        public bool SupportsInsertNewRows()
        {
            return false;
        }

        public bool SupportsLevel()
        {
            return false;
        }

        public bool SupportsMasterPropertiesCopy()
        {
            return false;
        }

        public bool SupportsPageSetup()
        {
            return false;
        }

        public bool SupportsPaste()
        {
            return false;
        }

        public bool SupportsPrint()
        {
            return false;
        }

        public bool SupportsRefresh()
        {
            return true;
        }

        public bool SupportsResultsFilter()
        {
            return false;
        }

        public bool SupportsSave()
        {
            return false;
        }

        public bool SupportsSelectionPaste()
        {
            return false;
        }

        public bool SupportsUndo()
        {
            return false;
        }

        public void TransposeView()
        {
            
        }

        public void Undo()
        {
            
        }

        public void UpLevel()
        {
            
        }

        public new void Refresh()
        {
            PopulateTreeView(); 
        }

        #endregion
    }
}