using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.HeaderDispatcher;

namespace Symcon.LightRidge.PEAdvisor.Global
{
    public partial class DeviationsAlertWindow : Form
    {
        public DeviationsAlertWindow()
        {
            InitializeComponent();

            this.Text = LRRHeaderText.GetHeaderString("SID_DEVIATION_ALERT_WINDOW");
            this.Load += new EventHandler(DeviationsAlertWindow_Load);
            this.Size= new Size( 910, 377);
        }

        #region Event Handlers

        /// <summary>
        /// Handles the Load event of the DeviationsAlertWindow control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void DeviationsAlertWindow_Load(object sender, EventArgs e)
        {
            
            gridControl1.ColCount = (4 + LRRModel.ProjectModes.Count);
            gridControl1.RowCount = 0;

            gridControl1[0, 0].Text = "";
            gridControl1[0, 1].Text = LRRHeaderText.GetHeaderString("SID_PLANTITEM");
            gridControl1.Cols.Size[1] = 250;
            gridControl1[0, 2].Text = LRRHeaderText.GetHeaderString("SID_PROPERTY");
            gridControl1.Cols.Size[2] = 250;
            gridControl1[0, 3].Text = LRRHeaderText.GetHeaderString("SID_UOM");

            gridControl1.Cols.Size[4] = 0;

            int iColCount = 5;
            foreach (LRRMode mode in LRRModel.ProjectModes)
            {
                gridControl1[0, iColCount].Text = mode.ModeName;
                
                iColCount++;
            }
            BusinessObjects.Utilities.PropertyValueValidator.GetInstance().DeviationCodeChanged += new PropertyChanged(DeviationsAlertWindow_DeviationCodeChanged);

            BusinessObjects.Utilities.PropertyValueValidator.GetInstance().DeviationCodeUnChanged += new PropertyChanged(DeviationsAlertWindow_DeviationCodeUnChanged);
            gridControl1.CellClick += new GridCellClickEventHandler(gridControl1_CellClick);

            foreach (LRRSiteComponentProperty scp in LRRModel.SiteComponentPropertiesObj)
            {
                bool check = false;
                foreach (LRRMode mode in LRRModel.ProjectModes)
                {
                    LRRSiteComponentPropertyValue scpValue = scp.GetPropertyValueForMode(mode.ModeId) as LRRSiteComponentPropertyValue;
                    if (scpValue != null && scpValue.PropertyValue != null)
                    {
                        if (scpValue.DeviationCode != Symcon.LightRidge.BusinessObjects.Utilities.ErrorCode.NONE)
                        {
                            check = true;
                            break;
                        }
                    }
                }
                if (check)
                {
                    AddAlert(scp);
                }
            }

        }

        /// <summary>
        /// Handles the CellClick event of the gridControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridCellClickEventArgs"/> instance containing the event data.</param>
        void gridControl1_CellClick(object sender, GridCellClickEventArgs e)
        {
            if (e.MouseEventArgs.Button == MouseButtons.Left
                && e.RowIndex == 0)
            {
                GridControl gridControl = sender as GridControl;
                gridControl.ResetVolatileData();
                gridControl.BeginUpdate();

                Cursor.Current = Cursors.WaitCursor;
                Cursor.Show();
                SortCol(e.ColIndex, gridControl);
                gridControl.EndUpdate();
                Cursor.Hide();
            }
        }

        /// <summary>
        /// Deviationses the alert window_ deviation code un changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="propertyName">Name of the property.</param>
        void DeviationsAlertWindow_DeviationCodeUnChanged(object sender, string propertyName)
        {
            if (this.InvokeRequired)
            {
                if (this.IsHandleCreated)
                {
                    AddAlertDel delAlertDelegate = new AddAlertDel(DelAlert);
                    try
                    {
                        this.Invoke(delAlertDelegate, new object[] { sender });
                    }
                    catch
                    {
                    }
                }
            }
            else
                DelAlert(sender as LRRSiteComponentProperty);
        }

        /// <summary>
        /// Deviationses the alert window_ deviation code changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="propertyName">Name of the property.</param>
        void DeviationsAlertWindow_DeviationCodeChanged(object sender, string propertyName)
        {
            if (this.InvokeRequired)
            {
                if (this.IsHandleCreated)
                {
                    AddAlertDel addAlertDelegate = new AddAlertDel(AddAlert);
                    try
                    {
                        this.Invoke(addAlertDelegate, new object[] { sender });
                    }
                    catch
                    {
                    }
                }
            }
            else
                AddAlert(sender as LRRSiteComponentProperty);
        }

        /// <summary>
        /// Handles the FormClosing event of the DeviationsAlertWindow control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void DeviationsAlertWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.UserClosing)
            {
                e.Cancel = true;
                this.Hide();
            }
        }

        
        #endregion

        #region Fields

        private ArrayList _arrIPropIDs = new ArrayList();
        private int _itemCount = 0;

        #endregion

        #region Delegates

        private delegate void AddAlertDel(LRRSiteComponentProperty scp);

        #endregion

        #region Public Methods

        /// <summary>
		/// This method applies sorting
		/// </summary>
		/// <param name="col"></param>
        public void SortCol(int col, GridControl gridControl)
        {
            System.ComponentModel.ListSortDirection dir = System.ComponentModel.ListSortDirection.Ascending;

            if (gridControl[0, col].HasTag)
            {
                if ((System.ComponentModel.ListSortDirection)gridControl[0, col].Tag == System.ComponentModel.ListSortDirection.Ascending)
                    dir = System.ComponentModel.ListSortDirection.Descending;
            }

            gridControl[0, col].Tag = dir;

            //int headerCount = gridControl.ColCount;
            gridControl.Data.SortByColumn(col, dir);
            gridControl.Refresh();
            _arrIPropIDs.Clear();
            for (int i = 1; i <= gridControl.RowCount; i++)
            {
                //if (gridControl[4, i].Text is string)
                //{
                    _arrIPropIDs.Add(gridControl[i,4].Text);
                //}
            }
        }

        /// <summary>
        /// Clears this instance.
        /// </summary>
        public void Clear()
        {
            _arrIPropIDs.Clear();
            _itemCount = 0;
            gridControl1.RowCount = 0;
        }

        /// <summary>
        /// Dels the alert.
        /// </summary>
        /// <param name="scp">The SCP.</param>
        public void DelAlert(LRRSiteComponentProperty scp)
        {
            if (_arrIPropIDs.Contains(scp.ID))
            {
                int toRemoveIndex = -1;
                for (int iCount = 0; iCount < _arrIPropIDs.Count; iCount++)
                {
                    if (_arrIPropIDs[iCount].Equals(scp.ID))
                    {
                        bool check = true;
                        foreach (LRRMode mode in LRRModel.ProjectModes)
                        {
                            LRRSiteComponentPropertyValue scpValue = scp.GetPropertyValueForMode(mode.ModeId) as LRRSiteComponentPropertyValue;
                            if (scpValue != null)
                            {
                                if (scpValue.DeviationCode != Symcon.LightRidge.BusinessObjects.Utilities.ErrorCode.NONE)
                                {
                                    check = false;
                                    break;
                                }
                            }
                        }
                        if (check)
                        {
                            gridControl1.Rows.RemoveRange(iCount + 1, iCount + 1);
                            toRemoveIndex = iCount;
                        }
                        else
                            AddAlert(scp);
                        break;
                    }
                }
                if (toRemoveIndex != -1)
                {
                    _arrIPropIDs.RemoveAt(toRemoveIndex);
                    _itemCount--;
                    //this.Show();
                }
            }
        }

        /// <summary>
        /// Adds the alert.
        /// </summary>
        /// <param name="scp">The SCP.</param>
        public void AddAlert(LRRSiteComponentProperty scp)
        {
            if (!_arrIPropIDs.Contains(scp.ID))
            {
                string siteCompName = LRRModel.SiteComponentsObj.GetSiteComponentByID(scp.ComponentID).Name;
                string scpName = scp.Name;

                _itemCount++;
                gridControl1.RowCount = _itemCount;

                gridControl1[_itemCount, 0].Text = "";
                gridControl1[_itemCount, 1].Text = siteCompName;
                gridControl1[_itemCount, 1].ReadOnly = true;
                gridControl1[_itemCount, 2].Text = scpName;
                gridControl1[_itemCount, 2].ReadOnly = true;
                gridControl1[_itemCount, 4].Text= scp.ID;

                string scpModelUOMID = scp.UOM;
                string scpUiUOMID = null;
                string scpUomTypeID = null;
                string localeID = LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID;
                try
                {



                    if (scpModelUOMID != "" && scpModelUOMID != null)
                    {

                        scpUomTypeID = scp.UOMTypeID;
                        scpUiUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localeID, scpUomTypeID);

                    }
                    //else
                    //{
                    //    int j = 5;
                    //}
                }
                catch
                {
                    scpUiUOMID = scpModelUOMID;
                }
                if (scpUiUOMID != null && scpUiUOMID != "")
                {
                    LRRUOM scpUom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(scpUiUOMID);
                    gridControl1[_itemCount, 3].Text = scpUom.Name;
                }
                gridControl1[_itemCount, 3].ReadOnly = true;

                int iColCount = 5;

                foreach (LRRMode mode in LRRModel.ProjectModes)
                {
                    LRRSiteComponentPropertyValue scpValue = scp.GetPropertyValueForMode(mode.ModeId) as LRRSiteComponentPropertyValue;
                    if (scpValue != null)
                    {
                        gridControl1[_itemCount, iColCount].ReadOnly = false;
                        IPropertyValue iPropValue = scpValue as IPropertyValue;
                        
                        string propertyValue = "";
                       
                        try
                        {
                            propertyValue = iPropValue.GetPropertyValue();
                            propertyValue = iPropValue.GetPropertyValueForUOM(scpModelUOMID, scpUiUOMID, propertyValue);
                        }
                        catch
                        { 
                        }
                        gridControl1[_itemCount, iColCount].Text = propertyValue;
                        SetColor(scpValue, gridControl1.Model[_itemCount, iColCount]);
                        
                        gridControl1[_itemCount, iColCount].ReadOnly = true;
                    }
                    iColCount++;    
                }

                _arrIPropIDs.Add(scp.ID);
            }
            else
            {
                for(int iCount=0;iCount<_arrIPropIDs.Count;iCount++)
                {
                    if (_arrIPropIDs[iCount].Equals(scp.ID))
                    {
                        int iColCount = 5;
                        
                        string localeID = LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID;
                        
                        gridControl1.Model.IgnoreReadOnly = true;



                        string scpModelUOMID = scp.UOM;
                        string scpUiUOMID = null;
                        string scpUomTypeID = null;

                        try
                        {

                            if (scpModelUOMID != "" && scpModelUOMID != null)
                            {
                                scpUomTypeID = scp.UOMTypeID;
                                scpUiUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localeID, scpUomTypeID);

                            }
                        }
                        catch
                        {
                            scpUiUOMID = scpModelUOMID;
                        }
                        if (scpUiUOMID != null && scpUiUOMID != "")
                        {
                            LRRUOM scpUom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(scpUiUOMID);
                            gridControl1[iCount + 1, 3].Text = scpUom.Name;
                        }

                        foreach (LRRMode mode in LRRModel.ProjectModes)
                        {
                            LRRSiteComponentPropertyValue scpValue = scp.GetPropertyValueForMode(mode.ModeId) as LRRSiteComponentPropertyValue;
                            if (scpValue != null)
                            {
                                IPropertyValue iPropValue = scpValue as IPropertyValue;
                                string propertyValue = "";
                                if (iPropValue != null)
                                {
                                    try
                                    {
                                        propertyValue = iPropValue.GetPropertyValue();
                                        propertyValue = iPropValue.GetPropertyValueForUOM(scpModelUOMID, scpUiUOMID, propertyValue);
                                    }
                                    catch
                                    { }

                                    gridControl1[iCount + 1, iColCount].Text = propertyValue;
                                    SetColor(scpValue, gridControl1.Model[iCount + 1, iColCount]);
                                }
                                iColCount++;
                            }
                        }
                        gridControl1.Model.IgnoreReadOnly = false;

                        break;
                    }
                }
            }
            //this.Show();

        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Method sets the fore/back/border colours for the grid cell
        /// </summary>
        /// <param name="iPropValue"></param>
        /// <param name="gsInfo"></param>
        private void SetColor(IPropertyValue iPropValue, GridStyleInfo gsInfo)
        {
            try
            {
                if (iPropValue != null)
                {
                    gsInfo.BackColor = iPropValue.GetBackColor();
                    gsInfo.TextColor = iPropValue.GetForeColor();

                    //Set Border Color and Border Thickness
                    Color borderColor = iPropValue.GetBorderColor();
                    int borderThickness = iPropValue.GetBorderThickness();
                    if (borderThickness == 2)
                    {	//gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid, borderColor,GridBorderWeight.Medium);
                        GridBorder grdBorder = new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                        gsInfo.Borders.Top = grdBorder;// new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                        gsInfo.Borders.Left = grdBorder;// new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                    }
                    else
                    {
                        gsInfo.Borders.ResetAll();
                        //gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid,IOSchemes.GRID_PROPVALUE_LINE_COLOR,GridBorderWeight.ExtraThin);
                    }
                    gsInfo.Font.Underline = iPropValue.GetUnderLine();
                }
            }
            catch
            { }
        }

        #endregion
    }
}