using System;
using System.Collections;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Styles;
using Symcon.LightRidge.BusinessObjects;   


namespace Symcon.LightRidge.PEAdvisor.Global
{
	/// <summary>
	/// Summary description for OverrideWndSortUtility.
	/// </summary>
	public class OverrideWndSortUtility :IComparer 
	{
		public OverrideWndSortUtility()
		{

		}

		#region Fields

			private string _sortCol ;
			private int   _sortOrder;


			#endregion

		#region Properties
	
		public  string SortColumn
		{
			get{return _sortCol;}set{_sortCol = value;}
		}
		public int SortingOrder
		{
			get{return _sortOrder;}set{_sortOrder = value;}
		}


		#endregion

		public int Compare(object x, object y)
		{
				return CheckSort(x,y,0);
		}
        /// <summary>
        /// Checks the sort.
        /// </summary>
        /// <param name="x">The x.</param>
        /// <param name="y">The y.</param>
        /// <param name="sortLevel">The sort level.</param>
        /// <returns></returns>
		public int CheckSort(object x,object y, int sortLevel)
		{
				LRROverride obj1 = (LRROverride)x;
				LRROverride obj2 = (LRROverride)y;

				//Compare based on the sorting order.
				int result = -2;
			if(sortLevel == 0)
			{
				if(SortingOrder == 0)
				{
					switch(SortColumn)
					{
						case "Mode" :
							LRRMode modeobj1 = LRRModel.Modes.GetModeByID(obj1.ModeID);     
							LRRMode modeobj2 = LRRModel.Modes.GetModeByID(obj2.ModeID);     
							result = modeobj1.ModeName.CompareTo(modeobj2.ModeName);    
							break;
						case "SiteComponents":
							LRRSiteComponent siteCompObj1 = LRRModel.SiteComponentsObj.GetSiteComponentByID(obj1.SiteComponentID);
							LRRSiteComponent siteCompObj2 = LRRModel.SiteComponentsObj.GetSiteComponentByID(obj2.SiteComponentID);
   							result = siteCompObj1.Name.CompareTo(siteCompObj2.Name);
							break;
						case "Attribute":
							LRRSiteComponentProperty propObj1 = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(obj1.SiteComponentPropertyID);    
							LRRSiteComponentProperty propObj2 = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(obj2.SiteComponentPropertyID);
							result = propObj1.Name.CompareTo(propObj2.Name);
							break;
						case "UOM":
							LRRUOM uomObj1 = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(obj1.UOM);
							LRRUOM uomObj2 = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(obj2.UOM);
    						result = uomObj1.Name.CompareTo(uomObj2.Name);
							break;
						case "DataConfidenceLevel":
							LRRDataConfidenceLevel dataConfedancelevelObj1 = LRRModel.ChildModel.DataConfidenceLevels.GetConfidenceLevelFromID(obj1.DataConfidenceLevelID);  
							LRRDataConfidenceLevel dataConfedancelevelObj2 = LRRModel.ChildModel.DataConfidenceLevels.GetConfidenceLevelFromID(obj2.DataConfidenceLevelID);  
							result = dataConfedancelevelObj1.ConfidenceLevel.CompareTo(dataConfedancelevelObj2.ConfidenceLevel);    
							break;
					}
				}
				if(SortingOrder == 1)
				{
					switch(SortColumn)
					{
						case "Mode" :
							LRRMode modeobj1 = LRRModel.Modes.GetModeByID(obj1.ModeID);     
							LRRMode modeobj2 = LRRModel.Modes.GetModeByID(obj2.ModeID);     
							result = modeobj2.ModeName.CompareTo(modeobj1.ModeName);    
							break;
						case "SiteComponents":
							LRRSiteComponent siteCompObj1 = LRRModel.SiteComponentsObj.GetSiteComponentByID(obj1.SiteComponentID);
							LRRSiteComponent siteCompObj2 = LRRModel.SiteComponentsObj.GetSiteComponentByID(obj2.SiteComponentID);
							result = siteCompObj2.Name.CompareTo(siteCompObj1.Name);
							break;
						case "Attribute":
							LRRSiteComponentProperty propObj1 = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(obj1.SiteComponentPropertyID);    
							LRRSiteComponentProperty propObj2 = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(obj2.SiteComponentPropertyID);
							result = propObj2.Name.CompareTo(propObj1.Name);
							break;
						case "UOM":
							LRRUOM uomObj1 = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(obj1.UOM);
							LRRUOM uomObj2 = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(obj2.UOM);
							result = uomObj2.Name.CompareTo(uomObj1.Name);
							break;
						case "DataConfidenceLevel":
							LRRDataConfidenceLevel dataConfedancelevelObj1 = LRRModel.ChildModel.DataConfidenceLevels.GetConfidenceLevelFromID(obj1.DataConfidenceLevelID);  
							LRRDataConfidenceLevel dataConfedancelevelObj2 = LRRModel.ChildModel.DataConfidenceLevels.GetConfidenceLevelFromID(obj2.DataConfidenceLevelID);  
							result = dataConfedancelevelObj2.ConfidenceLevel.CompareTo(dataConfedancelevelObj1.ConfidenceLevel);    
							break;
					}
				}
			}
			return result;
		}
			
	}
	
}

