using System;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;
using System.Collections;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Drawing;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// Summary description for GridGroupingPrint.
	/// </summary>
	public class GridGroupingPrint
	{
		public GridGroupingPrint()
		{
			//Get the App Object
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

			printDocument1 = new PrintDocument();
			printDocument1.PrintPage += new PrintPageEventHandler(printDocument1_PrintPage);
			
			//print preview Dialog
			printPreviewDialog1 = _lrrApp.PrintPreviewDialogObj;
			printPreviewDialog1.Document = printDocument1;

			//Print Dialog
			printDialog = _lrrApp.PrintDialogObj;
			printDialog.Document = printDocument1;
			
		}

        /// <summary>
        /// Prints the data.
        /// </summary>
        /// <param name="grid">The grid.</param>
        /// <param name="showPrintPreview">if set to <c>true</c> [show print preview].</param>
		public void PrintData(GridGroupingControl grid,bool showPrintPreview)
		{
			_gridControl = grid;
			
			try
			{
				if(grid.TableControl.Table.CurrentRecord == null)
				{
					Element eCurrentElement = grid.TableControl.Table.CurrentElement;
					GridNestedTable gnt = eCurrentElement as GridNestedTable;
					GridNestedTable gnt1 = gnt;
					GridTableDescriptor gtd = null;
					while(gnt1 != null && gnt1.ChildTable != null)
					{
						gnt = gnt1;
						gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
						gtd = gnt.ParentTable.ParentTableDescriptor;
					}

					GridChildTable childTable = gnt.ChildTable as GridChildTable;
					tableModel = childTable.ParentTable.TableModel;
					tableDescriptor = childTable.ParentTable.TableDescriptor;
					
					//Check if user has selected a range
					GridRangeInfoList ranges =tableModel.SelectedRanges;
					if(ranges.Count == 1 && ranges.ActiveRange.IsCells)
					{
						GetSelectedData(tableModel,showPrintPreview);
						return;
					}

				}
				else
				{
					tableModel = grid.TableModel;
					tableDescriptor = grid.TableDescriptor;
					
					//Check if user has selected a range
					GridRangeInfoList ranges =tableModel.SelectedRanges;
					if(ranges.Count == 1 && ranges.ActiveRange.IsCells)
					{
						GetSelectedData(tableModel,showPrintPreview);
						return;
					}
				}
				//Print the data
				printingData(tableModel,showPrintPreview);
			}
			catch{}
		}

		/// <summary>
		/// Get the selected data
		/// </summary>
		/// <param name="tableModel"></param>
		/// <param name="showPrintPreview"></param>
		private void GetSelectedData(GridTableModel tableModel,bool showPrintPreview)
		{
			GridRangeInfoList ranges = tableModel.SelectedRanges;
			GridRangeInfo range = ranges.ActiveRange;

			DataTable dtTable = new DataTable();

			//Get the columns
			for(int row = range.Top; row <= range.Bottom; ++row)
			{
				for(int col = range.Left; col <= range.Right; col++)
				{
					# region Setting Text
					string sColHeader = tableModel.GetColumnDescriptorAt(row,col).HeaderText;
					if(!dtTable.Columns.Contains(sColHeader))
					{
						dtTable.Columns.Add(new DataColumn(sColHeader, typeof(string)));
					}
					#endregion
				}
			}

			//Create the Header Row
			DataRow drHeaderRow = dtTable.NewRow();
			for(int i=0;i < dtTable.Columns.Count;i++)
			{
				drHeaderRow[i] = dtTable.Columns[i].ColumnName;
			}
			dtTable.Rows.Add(drHeaderRow);
			
			for(int row = range.Top; row <= range.Bottom; ++row)
			{
				DataRow dr = dtTable.NewRow();
				int l=0;
				for(int col = range.Left; col <= range.Right; col++)
				{
					# region Setting Text
					string text = tableModel[row, col].FormattedText.Replace("\n"," ");
					dr[l] = text;
					l++;
					#endregion
				}
				dtTable.Rows.Add(dr);
			}
			SetupGridPrinter(dtTable,showPrintPreview);
		}
		
		/// <summary>
		/// Prints the data
		/// </summary>
		/// <param name="tableModel"></param>
		/// <param name="grid"></param>
		private  void printingData(GridTableModel tableModel,bool showPrintPreview)
		{
			DataTable dtTable = new DataTable();

			//Get the columns
			int iColCount = tableModel.Table.TableDescriptor.VisibleColumns.Count;
			int colIndent = tableModel.Table.TableDescriptor.GetColumnIndentCount();
			int iCol = 0;
			while(iCol < iColCount)
			{
				string sColHeader = tableModel.Table.TableDescriptor.VisibleColumns[iCol].Name;
				if(!dtTable.Columns.Contains(sColHeader))
				{
					dtTable.Columns.Add(new DataColumn(sColHeader, typeof(string)));
				}
				iCol++;
			}
			
			int iGridColCount = tableModel.Table.TableDescriptor.VisibleColumns.Count;//.TableDescriptor.Relations.Count;
			for (int i = 0;i <= tableModel.RowCount+1;i++) 
			{
				DataRow dr = dtTable.NewRow();
				int l=0;
				for(int j = colIndent; j < colIndent + iGridColCount;j++) 
				{
					# region Setting Text
					GridTableCellStyleInfoIdentity tableCellInfo = tableModel[i, j].CellIdentity as GridTableCellStyleInfoIdentity;
					if(tableCellInfo != null && tableCellInfo.TableCellType == GridTableCellType.ColumnHeaderCell)
					{
						if(dtTable.Rows.Count > 0)
						{
							int iRowCount = dtTable.Rows.Count;
							ArrayList arrRowsToRemove = new ArrayList();
							for(int k=0; k < iRowCount;k++)
							{
								arrRowsToRemove.Add(dtTable.Rows[k]);
							}
							for(int iCount = 0; iCount < arrRowsToRemove.Count;iCount++)
							{
								dtTable.Rows.Remove((DataRow)arrRowsToRemove[iCount]);
							}
						}
					}
					
					string text = tableModel[i, j].FormattedText.Replace("\n"," ");
					dr[l] = text;
					l++;
					#endregion
				}
				dtTable.Rows.Add(dr);
			}
			SetupGridPrinter(dtTable,showPrintPreview);
		}

        /// <summary>
        /// Setups the grid printer.
        /// </summary>
        /// <param name="dtTable">The dt table.</param>
        /// <param name="showPrintPreview">if set to <c>true</c> [show print preview].</param>
		private void SetupGridPrinter(DataTable dtTable,bool showPrintPreview)
		{
			totalColCount = dtTable.Columns.Count;
			printDocument1.DefaultPageSettings = _lrrApp.PageSetupDialogObj.PageSettings;
			
			dataGridPrinter1 = new GroupingGridPrintData(_gridControl, tableDescriptor,printDocument1, dtTable);
			dataGridPrinter1.PageNumber = 1;
			dataGridPrinter1.RowCount = 0;

			if(showPrintPreview)
			{
				DialogResult dlg = printPreviewDialog1.ShowDialog();
			}
			else
			{
				if(printDialog.ShowDialog() == DialogResult.OK)
				{
					printDocument1.Print();
				}
			}
		}

        /// <summary>
        /// Pages the set up.
        /// </summary>
		public void PageSetUp()
		{
			_lrrApp.PageSetupDialogObj.Document = printDocument1;
			_lrrApp.PageSetupDialogObj.ShowDialog();
		}

        /// <summary>
        /// Handles the PrintPage event of the printDocument1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Drawing.Printing.PrintPageEventArgs"/> instance containing the event data.</param>
		private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
		{
			Graphics	g = e.Graphics;
			bool bLastPages = true;
			
			iTotalWidth = 0;
			for(int i=nPrevColCtr;i < totalColCount;i++)
			{
				int iWidth = tableDescriptor.Columns[nColCtr].Width;
				iTotalWidth = iTotalWidth + iWidth;
				if(iTotalWidth > dataGridPrinter1.PageWidth)
				{
					bool bContinue = dataGridPrinter1.DrawDataGrid(nPrevColCtr,nColCtr,g);
					e.HasMorePages = true;
					dataGridPrinter1.PageNumber++;
					if(!bContinue)
					{
						nPrevColCtr = nColCtr;
					}
					else
					{
						nColCtr = nPrevColCtr;
					}
					bLastPages = false;
					break;
				}

				nColCtr++;
			}

			if(bLastPages)
			{
				bool bContinue = dataGridPrinter1.DrawDataGrid(nPrevColCtr,totalColCount,g);
				if(bContinue)
				{
					e.HasMorePages = true;
					dataGridPrinter1.PageNumber++;
					nColCtr = nPrevColCtr;
				}
				else
				{
					bLastPages = false;
				}
			}
		}

		#region Fields
		private  PrintDocument printDocument1;
		private  PrintPreviewDialog printPreviewDialog1;
		private  PrintDialog printDialog;
		private  GroupingGridPrintData dataGridPrinter1 = null;
		private  GridGroupingControl	_gridControl;
		private  GridTableModel tableModel = null;
		private  GridTableDescriptor	tableDescriptor = null;
		private  PageSetupDialog pageSetUpDialog;
		private LRRApp _lrrApp;

		int nColCtr		= 0;
		int iTotalWidth = 0;
		int nPrevColCtr = 0;
		int totalColCount = 0;
		#endregion
	}
}
