using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Symcon.BusinessObjects;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Symcon.Utils.MessageLogger;

//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.


namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// Summary description for GridRecordStateFormatter.
	/// </summary>
	public class GridRecordStateFormatter
	{
		public GridRecordStateFormatter()
		{
			
		}

		#region Methods
        /// <summary>
        /// Wires the grid.
        /// </summary>
        /// <param name="grid">The grid.</param>
		public void WireGrid(GridDataBoundGrid grid)
		{
			if(this._grid != grid)
			{
				if(this._grid != null)
					UnwireGrid();
			
				this._grid = grid;
				this._grid.SmoothControlResize = false;
				this._grid.Model.QueryCellInfo+=new GridQueryCellInfoEventHandler(Model_QueryCellInfo);
			}
		}

		public void WireGrid(GridGroupingControl grid)
		{
			if(this._gridGrouping != grid)
			{
				if(this._gridGrouping != null)
					UnwireGrid();
			
				this._gridGrouping = grid;
				//this._gridGrouping.SmoothControlResize = false;
				this._gridGrouping.TableControl.Model.QueryCellInfo+=new GridQueryCellInfoEventHandler(Model_QueryCellInfo);
			}
		}

        /// <summary>
        /// Unwires the grid.
        /// </summary>
		public void UnwireGrid()
		{
			try
			{
				this._grid.Model.QueryCellInfo-=new GridQueryCellInfoEventHandler(Model_QueryCellInfo);
				this._gridGrouping.TableControl.Model.QueryCellInfo+=new GridQueryCellInfoEventHandler(Model_QueryCellInfo);
				this._gridGrouping = null;
				this._grid = null;

			}
			catch (Exception ex)
			{
				Logger.LogMessage(typeof(GridRecordStateFormatter),"UnwireGrid :"+ex.Message  ,Logger.Level.Debug);
			}
		}

		#endregion

		#region Properties

		public Color InsertColor
		{
			get{return _insertColor;}
			set
			{
				_insertColor = value;
			}
		}

		public Color UpdateColor
		{
			get{return _updateColor;}
			set
			{
				_updateColor = value;
			}
		}

		#endregion

		#region Fields

		private GridDataBoundGrid 						_grid = null;
		private GridGroupingControl						_gridGrouping = null;

		private Color									_insertColor = Color.Blue;
		private Color									_updateColor = Color.Orange;

		#endregion

        /// <summary>
        /// Handles the QueryCellInfo event of the Model control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridQueryCellInfoEventArgs"/> instance containing the event data.</param>
		private void Model_QueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
		{
			try
			{
				int dataSrcIndex = _grid.Binder.RowIndexToListManagerPosition(e.RowIndex);
				CustomCollection cc = _grid.Binder.DataSource as CustomCollection;
				BaseObject bo = cc[dataSrcIndex] as BaseObject;

				if(bo.ObjectState == Symcon.Globals.Enums.DBOperation.UPDATE)
				{
					e.Style.TextColor = System.Drawing.Color.Orange;
				}
				else if(bo.ObjectState == Symcon.Globals.Enums.DBOperation.INSERT)
				{
					e.Style.TextColor = System.Drawing.Color.Blue;
				}
			}
			catch (Exception ex)
			{
				Logger.LogMessage(typeof(GridRecordStateFormatter),"Model_QueryCellInfo :"+ex.Message  ,Logger.Level.Debug);
			}
		}
	}
}
