using System;
using System.Collections.Generic;
using System.Text;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using System.Windows.Forms;
using System.Data;

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
    public class GridTableFilterBarCellModel1 : GridTableFilterBarCellModel
    {
        public GridTableFilterBarCellModel1(Syncfusion.Windows.Forms.Grid.GridModel gm)
            : base(gm)
        {
        }

        /// <summary>
        /// Initializes a listbox with data binding information from a <see cref="T:Syncfusion.Windows.Forms.Grid.GridStyleInfo"/>
        /// object.
        /// </summary>
        /// <param name="listBox">The list box to be initialized with data binding information.</param>
        /// <param name="style">The style object with binding information.</param>
        /// <param name="exclusive">A place holder that indicates whether the list box is filled with an exclusive
        /// list of possible choices or if non-standard values are allowed.</param>
        public override void FillWithChoices(ListBox listBox, GridStyleInfo style, out bool exclusive)
        {
            exclusive = true;
            GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo)style;

            Symcon.LightRidge.BusinessObjects.LRRUOMs uoms = tableStyleInfo.DataSource as Symcon.LightRidge.BusinessObjects.LRRUOMs;


            listBox.Items.Clear();

            if (uoms != null)
            {
                listBox.Items.Add(SelectAllText);
                listBox.Items.Add(SelectCustomText);
                listBox.Items.Add(SelectEmptyText);

                foreach (Symcon.LightRidge.BusinessObjects.LRRUOM uom in uoms)
                {
                    listBox.Items.Add(uom.Name);
                }
            }
        }

        /// <summary>
        /// Select_s the without custom.
        /// </summary>
        /// <param name="tableCellIdentity">The table cell identity.</param>
        /// <param name="index">The index.</param>
        public void Select_WithoutCustom(GridTableCellStyleInfoIdentity tableCellIdentity, int index)
        {
            if (index >= 0)
            {
                if (index == 0)
                {
                    ResetFilterBar(tableCellIdentity);
                }
                else if (index == 1)
                {
                    SelectItem(tableCellIdentity, 0);
                }
                else
                {
                    SelectItem(tableCellIdentity, index - 1);
                }
            }
        }

        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new Cellrenderer(control, this);
        }
    }

    public class Cellrenderer : GridTableFilterBarCellRenderer
    {
        public Cellrenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
        }

        public new Symcon.LightRidge.PEAdvisor.GridUtilities.GridTableFilterBarCellModel1 Model
        {
            get
            {
                return (Symcon.LightRidge.PEAdvisor.GridUtilities.GridTableFilterBarCellModel1)base.Model;
            }
        }

        /// <summary>
        /// Handles the <see cref="E:System.Windows.Forms.Control.MouseUp"/> event of the list box.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">A <see cref="T:System.Windows.Forms.MouseEventArgs"/> with event data.</param>
        protected override void ListBoxMouseUp(object sender, MouseEventArgs e)
        {
            CurrentCell.CloseDropDown(PopupCloseType.Done);
            GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo)this.StyleInfo;
            GridTableCellStyleInfoIdentity tableCellIdentity = tableStyleInfo.TableCellIdentity;
            GridTableControl _grid = Grid as GridTableControl;

            foreach (RecordFilterDescriptor rfd in _grid.GroupingControl.TableDescriptor.RecordFilters)
            {
                if (rfd.Name == "InterfaceUnits" || rfd.Name == "HistUnits" || rfd.Name == "ModelUOM")
                {
                    _grid.GroupingControl.TableDescriptor.RecordFilters.Remove(rfd); 
                }
            }

            

            Model.Select_WithoutCustom(tableCellIdentity, this.ListBoxPart.SelectedIndex);

            String _FilterBarValue = this.ListBoxPart.SelectedItem.ToString();

                
            int _filed = _grid.TableDescriptor.ColIndexToField(ColIndex);
            string _columnName = _grid.TableDescriptor.Columns[_filed].Name;

            _grid.TableDescriptor.RecordFilters.Remove(_columnName);

            if (_FilterBarValue != "(All)" && _FilterBarValue != "Empty")
            {
                int index = this.ListBoxPart.SelectedIndex - 3;
                if (index >= 0)
                {
                    Symcon.LightRidge.BusinessObjects.LRRUOMs uoms = tableStyleInfo.DataSource as Symcon.LightRidge.BusinessObjects.LRRUOMs;
                    object selectedValue = uoms.GetUOMObjectFromName(_FilterBarValue).ID;
                    if (selectedValue != null)
                    {
                        _grid.GroupingControl.Engine.TableDescriptor.RecordFilters.Add(_columnName, FilterCompareOperator.Equals, selectedValue);
                    }
                }
            }
        }
    }
}
