using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Printing;
using System.Data;
using System.Collections;
using Symcon.Utils.MessageLogger;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;


//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{

	/// <summary>
	/// Summary description for PrintGrid.
	/// </summary>
	public class GroupingGridPrintData
	{

		private PrintDocument ThePrintDocument;
		private DataTable TheTable;
		private GridGroupingControl  TheDataGrid;
		private GridTableDescriptor tableDescriptor;

		public int RowCount = 0;  // current count of rows;
		public int initialRowCount = 0;
		private const int kVerticalCellLeeway = 10;
		public int PageNumber = 1;
		public ArrayList Lines = new ArrayList();

		public int PageWidth;
		int PageHeight;
		int TopMargin;
		int BottomMargin;

		private int startRowIndex;
		private int endRowIndex;

		private int LEFT_MARGIN = 10;
		private int RIGHT_MARGIN = 10;



        /// <summary>
        /// Initializes a new instance of the <see cref="GroupingGridPrintData"/> class.
        /// </summary>
        /// <param name="aGrid">A grid.</param>
        /// <param name="tableDesc">The table desc.</param>
        /// <param name="aPrintDocument">A print document.</param>
        /// <param name="aTable">A table.</param>
		public GroupingGridPrintData(GridGroupingControl aGrid, GridTableDescriptor tableDesc, PrintDocument aPrintDocument, DataTable aTable)
		{
			//
			// TODO: Add constructor logic here
			//
			TheDataGrid = aGrid;
			ThePrintDocument = aPrintDocument;
			TheTable = aTable;
			tableDescriptor = tableDesc;
			
			PageHeight = ThePrintDocument.DefaultPageSettings.PaperSize.Height;
			TopMargin = ThePrintDocument.DefaultPageSettings.Margins.Top;
			BottomMargin = ThePrintDocument.DefaultPageSettings.Margins.Bottom;
			LEFT_MARGIN = ThePrintDocument.DefaultPageSettings.Margins.Left;
			RIGHT_MARGIN = ThePrintDocument.DefaultPageSettings.Margins.Right;

			PageWidth = ThePrintDocument.DefaultPageSettings.PaperSize.Width - LEFT_MARGIN - RIGHT_MARGIN;

			endRowIndex = TheTable.Rows.Count;
			startRowIndex = 0;
		}

        /// <summary>
        /// Draws the grid.
        /// </summary>
        /// <param name="startColIndex">Start index of the col.</param>
        /// <param name="endColIndex">End index of the col.</param>
        /// <param name="strtRowIndex">Index of the STRT row.</param>
        /// <param name="endRowIndex">End index of the row.</param>
        /// <param name="g">The g.</param>
        /// <returns></returns>
		public bool DrawGrid(int startColIndex,int endColIndex,int strtRowIndex,int endRowIndex,Graphics g)
		{
			int lastRowBottom = TopMargin;

			SolidBrush ForeBrush = new SolidBrush(TheDataGrid.ForeColor);
			SolidBrush BackBrush = new SolidBrush(TheDataGrid.BackColor);
			SolidBrush AlternatingBackBrush = new SolidBrush(TheDataGrid.Appearance.AlternateRecordFieldCell.BackColor);

			Pen TheLinePen = new Pen(this.TheDataGrid.TableOptions.TreeLineBorder.Color, 1);

			StringFormat cellformat = new StringFormat();
			cellformat.Trimming = StringTrimming.EllipsisCharacter;

			int columnwidth = 0;

			RectangleF RowBounds  = new RectangleF(0, 0, 0, 0);
			RowCount = 0;
			int horzWidth = 0;

			for (int i = strtRowIndex; i < endRowIndex; i++)
			{
				DataRow dr = TheTable.Rows[i];
				int startxposition = LEFT_MARGIN;

				RowBounds.X = LEFT_MARGIN;
				RowBounds.Y = TopMargin + ((RowCount - initialRowCount)+1) * (TheDataGrid.Font.SizeInPoints  + kVerticalCellLeeway);
				RowBounds.Height = TheDataGrid.Font.SizeInPoints + kVerticalCellLeeway;
				RowBounds.Width = PageWidth;
				Lines.Add(RowBounds.Bottom);

				if (i%2 == 0)
				{
					g.FillRectangle(BackBrush, RowBounds);
				}
				else
				{
					g.FillRectangle(AlternatingBackBrush, RowBounds);
				}


				for (int j = startColIndex; j < endColIndex; j++)
				{
					//columnwidth = this.TheDataGrid.TableDescriptor.Columns[j].Width;
					columnwidth = tableDescriptor.Columns[j].Width;
					RectangleF cellbounds = new RectangleF(startxposition, 
						TopMargin + ((RowCount - initialRowCount) + 1) * (TheDataGrid.Font.SizeInPoints  + kVerticalCellLeeway),
						columnwidth, 
						TheDataGrid.Font.SizeInPoints + kVerticalCellLeeway);

						g.DrawString(dr[j].ToString(), TheDataGrid.Font, ForeBrush, cellbounds, cellformat);
						lastRowBottom = (int)cellbounds.Bottom;

					startxposition = startxposition + columnwidth;
					horzWidth = startxposition;
				}

				RowCount++;

				if (RowCount * (TheDataGrid.Font.SizeInPoints  + kVerticalCellLeeway) > (PageHeight) - (BottomMargin+TopMargin))
				{
					startRowIndex = i + 1;
					DrawHorizontalLines(g, Lines,startxposition);
					DrawVerticalGridLines(g, TheLinePen,endColIndex - startColIndex, lastRowBottom);
					return true;
				}
			}

			DrawHorizontalLines(g, Lines,horzWidth);
			DrawVerticalGridLines(g, TheLinePen,endColIndex - startColIndex, lastRowBottom);
			return false;
		}

        /// <summary>
        /// Draws the horizontal lines.
        /// </summary>
        /// <param name="g">The g.</param>
        /// <param name="lines">The lines.</param>
        /// <param name="horzWidth">Width of the horz.</param>
		void DrawHorizontalLines(Graphics g, ArrayList lines,int horzWidth)
		{
			Pen TheLinePen = new Pen(this.TheDataGrid.TableOptions.TreeLineBorder.Color);
			g.DrawLine(TheLinePen,LEFT_MARGIN, TopMargin,horzWidth,TopMargin);
			for (int i = 0;  i < lines.Count; i++)
			{
				g.DrawLine(TheLinePen, LEFT_MARGIN, (float)lines[i],horzWidth,(float)lines[i]);
			}
		}

        /// <summary>
        /// Draws the vertical grid lines.
        /// </summary>
        /// <param name="g">The g.</param>
        /// <param name="TheLinePen">The line pen.</param>
        /// <param name="iColCount">The i col count.</param>
        /// <param name="bottom">The bottom.</param>
		void DrawVerticalGridLines(Graphics g, Pen TheLinePen, /*int columnwidth*/int iColCount, int bottom)
		{
			g.DrawLine(TheLinePen, LEFT_MARGIN, 
						TopMargin,
						LEFT_MARGIN,
						bottom);
			int strtPos = 0;
			for (int k = 0; k < iColCount; k++)
			{
				//int columnwidth = this.TheDataGrid.TableDescriptor.Columns[k].Width;
				int columnwidth = this.tableDescriptor.Columns[k].Width;
				g.DrawLine(TheLinePen, LEFT_MARGIN + columnwidth + strtPos, 
					TopMargin,
					LEFT_MARGIN + columnwidth + strtPos,
					bottom);
				strtPos = strtPos + columnwidth;
			}

		}


        /// <summary>
        /// Draws the data grid.
        /// </summary>
        /// <param name="startColIndex">Start index of the col.</param>
        /// <param name="endColIndex">End index of the col.</param>
        /// <param name="g">The g.</param>
        /// <returns></returns>
		public bool DrawDataGrid(int startColIndex,int endColIndex,Graphics g)
		{
			try
			{
				int strtRowIndex = startRowIndex;
				bool bContinue = DrawGrid(startColIndex,endColIndex,strtRowIndex,endRowIndex,g);
				if(!bContinue)
				{
					startRowIndex = 0;
				}
				return bContinue;
			}
			catch (Exception ex)
			{
				Logger.LogMessage(typeof(GroupingGridPrintData),"DrawDataGrid :"+ex.Message  ,Logger.Level.Debug);
				return false;
			}
		}
	}
}
