using System;
using System.Collections.Generic;
using System.Text;
using Syncfusion.Windows.Forms.Grid.Grouping;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
    public class LRRGridGroupingControl : UserControl
    {

		#regionFields(15)

        private int _currentPage;
        private object _dataSource;
        private bool _enablePaging;
        private string _noDataText;
        private bool _noDataVisible;
        private int _range;
        private int _totalPages;
        private GridGroupingControl gridGroupingControl1;
        private LinkLabel lblFirst = new LinkLabel();
        private Label lblGoto = new Label();
        private LinkLabel lblLast = new LinkLabel();
        private LinkLabel lblNext = new LinkLabel();
        private LinkLabel lblPrev = new LinkLabel();
        
        private SplitContainer splitContainer1;
        private Label lblNoData;
        private Panel pnlNoData;

		#endregionFields

		#regionConstructors(1)

        public LRRGridGroupingControl()
        {
            _enablePaging = true;
            _currentPage = 1;
            _range = 20;
        }

		#endregionConstructors

		#regionProperties(7)

        public int CurrentPage
        {
            get { return _currentPage; }
            set { _currentPage = value; }
        }

        public object DataSource
        {
            get { return _dataSource; }
            set { _dataSource = value; }
        }

        public bool EnablePaging
        {
            get { return _enablePaging; }
            set { _enablePaging = value; }
        }

        public string NoDataText
        {
            get { return _noDataText; }
            set
            {
                _noDataText = value;
                lblNoData.Text = _noDataText;
                System.Drawing.Point center = GetCenter(gridGroupingControl1, pnlNoData.Size);
                pnlNoData.Left = center.X; pnlNoData.Top = center.Y;
            }
        }

        public bool NoDataVisible
        {
            get { return _noDataVisible; }
            set { _noDataVisible = value; pnlNoData.Visible = _noDataVisible; }
        }

        public int Range
        {
            get { return _range; }
            set { _range = value; }
        }

        public int TotalPages
        {
            get
            {
                return _totalPages;
            }
            set
            {
                _totalPages = value;
                if (TotalPages > 0)
                    pnlNoData.Visible = false;
                else
                    pnlNoData.Visible = true;
                PopulatePages();
            }
        }

		#endregionProperties

		#regionDelegatesandEvents(1)


		//Events(1)

        [Category("New"), Browsable(true)]
        public event EventHandler GoClick;


		#endregionDelegatesandEvents

		#regionMethods(8)


		//ProtectedMethods(1)

        protected void OnGoClick(EventArgs e)
        {
            if (GoClick != null)
            {
                GoClick(this, e);
            }
        }



		//PrivateMethods(7)

        private void dataGridView1_Resize(object sender, EventArgs e)
        {
            lblNoData.Width = gridGroupingControl1.Width;
            System.Drawing.Point center = GetCenter(gridGroupingControl1, pnlNoData.Size);
            pnlNoData.Left = center.X; pnlNoData.Top = center.Y;
        }

        private Point GetCenter(Control control, Size adjustmentSize)
        {
            //Declare x as a new Point
            Point pt = new Point();
            //Declare y as a new Point
            pt = new Point((control.Width - adjustmentSize.Width) / 2, (control.Height - adjustmentSize.Height) / 2);
            //Declare center as a String with the center points
            return pt;
        }

        private void ImpactGrid_Resize(object sender, EventArgs e)
        {
            PopulatePages();
        }

        private void InitializeComponent()
        {
            this.splitContainer1 = new System.Windows.Forms.SplitContainer();
            this.pnlNoData = new System.Windows.Forms.Panel();
            this.lblNoData = new System.Windows.Forms.Label();
            this.gridGroupingControl1 = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
            this.splitContainer1.Panel1.SuspendLayout();
            this.splitContainer1.SuspendLayout();
            this.pnlNoData.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.gridGroupingControl1)).BeginInit();
            this.SuspendLayout();
            // 
            // splitContainer1
            // 
            this.splitContainer1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.splitContainer1.FixedPanel = System.Windows.Forms.FixedPanel.Panel2;
            this.splitContainer1.IsSplitterFixed = true;
            this.splitContainer1.Location = new System.Drawing.Point(0, 0);
            this.splitContainer1.Name = "splitContainer1";
            this.splitContainer1.Orientation = System.Windows.Forms.Orientation.Horizontal;
            // 
            // splitContainer1.Panel1
            // 
            this.splitContainer1.Panel1.Controls.Add(this.pnlNoData);
            this.splitContainer1.Panel1.Controls.Add(this.gridGroupingControl1);
            this.splitContainer1.Panel1MinSize = 140;
            this.splitContainer1.Size = new System.Drawing.Size(308, 177);
            this.splitContainer1.SplitterDistance = 151;
            this.splitContainer1.SplitterWidth = 1;
            this.splitContainer1.TabIndex = 0;
            // 
            // pnlNoData
            // 
            this.pnlNoData.AutoSize = true;
            this.pnlNoData.BackColor = System.Drawing.Color.White;
            this.pnlNoData.Controls.Add(this.lblNoData);
            this.pnlNoData.Location = new System.Drawing.Point(59, 58);
            this.pnlNoData.Name = "pnlNoData";
            this.pnlNoData.Size = new System.Drawing.Size(204, 29);
            this.pnlNoData.TabIndex = 1;
            // 
            // lblNoData
            // 
            this.lblNoData.AutoSize = true;
            this.lblNoData.Dock = System.Windows.Forms.DockStyle.Fill;
            this.lblNoData.Location = new System.Drawing.Point(0, 0);
            this.lblNoData.Name = "lblNoData";
            this.lblNoData.Size = new System.Drawing.Size(35, 13);
            this.lblNoData.TabIndex = 0;
            this.lblNoData.Text = "label1";
            // 
            // gridGroupingControl1
            // 
            this.gridGroupingControl1.BackColor = System.Drawing.SystemColors.Window;
            this.gridGroupingControl1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.gridGroupingControl1.Location = new System.Drawing.Point(0, 0);
            this.gridGroupingControl1.Name = "gridGroupingControl1";
            this.gridGroupingControl1.Size = new System.Drawing.Size(308, 151);
            this.gridGroupingControl1.TabIndex = 0;
            this.gridGroupingControl1.Text = "gridGroupingControl1";
            this.gridGroupingControl1.VersionInfo = "3.302.0.0";
            // 
            // LRRGridGroupingControl
            // 
            this.Controls.Add(this.splitContainer1);
            this.Name = "LRRGridGroupingControl";
            this.Size = new System.Drawing.Size(308, 177);
            this.splitContainer1.Panel1.ResumeLayout(false);
            this.splitContainer1.Panel1.PerformLayout();
            this.splitContainer1.ResumeLayout(false);
            this.pnlNoData.ResumeLayout(false);
            this.pnlNoData.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.gridGroupingControl1)).EndInit();
            this.ResumeLayout(false);

        }

        private void InitializePagingControls()
        {
            this.lblGoto.Text = "Goto Page";
            this.lblFirst.Text = "First";
            this.lblPrev.Text = "Previous";
            this.lblNext.Text = "Next";
            this.lblLast.Text = "Last";
        }

        private void lbl_Click(object sender, EventArgs e)
        {
            LinkLabel lbl = sender as LinkLabel;
            foreach (Control item in splitContainer1.Panel2.Controls)
            {
                if (item.Text == CurrentPage.ToString())
                {
                    item.Enabled = true;
                }
            }
            CurrentPage = int.Parse(lbl.Tag.ToString());
            // lbl.Enabled = false;
            OnGoClick(e);
            this.splitContainer1.Panel2.Controls.Clear();
            PopulatePages();
        }

        private void PopulatePages()
        {
            //Check for Number of pages
            if (_totalPages > 0)
            {
                this.splitContainer1.Panel2.Controls.Clear();
                //Add Label with Text "Goto Page"
                this.lblGoto.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
                this.lblGoto.AutoSize = true;
                this.lblGoto.Location = new System.Drawing.Point(0, 5);
                this.lblGoto.Name = "lblGoto";
                this.lblGoto.Size = new System.Drawing.Size(58, 13);
                this.lblGoto.TabIndex = 23;
                this.lblGoto.Text = this.lblGoto.Text;
                this.splitContainer1.Panel2.Controls.Add(lblGoto);

                //starting location for rest of the items
                int xLocation = this.lblGoto.Width;


                int minRange = (_range % 2 == 0) ? (_range / 2) - 1 : (_range - 1) / 2;
                int maxRange = (_range % 2 == 0) ? minRange + 1 : minRange;
                int minPage = CurrentPage - minRange;
                int maxPage = CurrentPage + maxRange;
                minPage = (minPage < 1) ? 1 : minPage;
                maxPage = (maxPage < (minPage + _range - 1)) ? minPage + _range - 1 : maxPage;
                if (maxPage > _totalPages)
                {
                    minPage = (minPage > 1) ? _totalPages - _range + 1 : 1;
                    maxPage = _totalPages;
                }
                minPage = (minPage < 1) ? 1 : minPage;

                if ((CurrentPage > (_range - minRange)) && (_totalPages > _range))
                {
                    //LinkLabel lbl = new LinkLabel();
                    this.lblFirst.AutoSize = true;
                    this.splitContainer1.Panel2.Controls.Add(this.lblFirst);
                    this.lblFirst.Name = "lblFirst";
                    this.lblFirst.Text = this.lblFirst.Text;
                    this.lblFirst.Tag = 1;
                    this.lblFirst.Padding = new Padding(1, 0, 1, 0);
                    this.lblFirst.AutoSize = true;
                    this.lblFirst.Anchor = AnchorStyles.Top | AnchorStyles.Right;
                    this.lblFirst.Location = new Point(xLocation, lblGoto.Location.Y);//new Point(this.splitContainer1.Panel2.Width - ((_totalPages - page) * 15) - 5, label1.Location.Y);
                    this.lblFirst.Click -= new EventHandler(lbl_Click);
                    this.lblFirst.Click += new EventHandler(lbl_Click);
                    xLocation = xLocation + this.lblFirst.Width;
                }

                if (CurrentPage != 1)
                {
                    //LinkLabel lbl = new LinkLabel();
                    this.lblPrev.AutoSize = true;
                    this.splitContainer1.Panel2.Controls.Add(this.lblPrev);
                    this.lblPrev.Name = "lblPrev";
                    this.lblPrev.Text = this.lblPrev.Text;
                    this.lblPrev.Tag = CurrentPage - 1;
                    this.lblPrev.Padding = new Padding(1, 0, 1, 0);
                    this.lblPrev.AutoSize = true;
                    this.lblPrev.Anchor = AnchorStyles.Top | AnchorStyles.Right;
                    this.lblPrev.Location = new Point(xLocation, lblGoto.Location.Y);//new Point(this.splitContainer1.Panel2.Width - ((_totalPages - page) * 15) - 5, label1.Location.Y);
                    this.lblPrev.Click -= new EventHandler(lbl_Click);
                    this.lblPrev.Click += new EventHandler(lbl_Click);
                    xLocation = xLocation + this.lblPrev.Width;
                }

                for (int i = minPage; i <= maxPage; i++)
                {
                    if (i == CurrentPage)
                    {
                        LinkLabel lbl = new LinkLabel();
                        lbl.AutoSize = true;
                        this.splitContainer1.Panel2.Controls.Add(lbl);
                        lbl.Text = (i).ToString();
                        lbl.Tag = i;
                        lbl.Padding = new Padding(1, 0, 1, 0);
                        lbl.AutoSize = true;
                        lbl.Anchor = AnchorStyles.Top | AnchorStyles.Right;
                        lbl.Location = new Point(xLocation, lblGoto.Location.Y);//new Point(this.splitContainer1.Panel2.Width - ((_totalPages - page) * 15) - 5, label1.Location.Y);
                        lbl.Click -= new EventHandler(lbl_Click);
                        lbl.Click += new EventHandler(lbl_Click);
                        lbl.Enabled = false;
                        xLocation = xLocation + lbl.Width;
                    }
                    else
                    {
                        LinkLabel lbl = new LinkLabel();
                        lbl.AutoSize = true;
                        this.splitContainer1.Panel2.Controls.Add(lbl);
                        lbl.Text = (i).ToString();
                        lbl.Tag = i;
                        lbl.Padding = new Padding(1, 0, 1, 0);
                        lbl.AutoSize = true;
                        lbl.Anchor = AnchorStyles.Top | AnchorStyles.Right;
                        lbl.Location = new Point(xLocation, lblGoto.Location.Y);//new Point(this.splitContainer1.Panel2.Width - ((_totalPages - page) * 15) - 5, label1.Location.Y);
                        lbl.Click -= new EventHandler(lbl_Click);
                        lbl.Click += new EventHandler(lbl_Click);
                        xLocation = xLocation + lbl.Width;
                    }

                }

                if (CurrentPage < _totalPages)
                {
                    //LinkLabel lbl = new LinkLabel();
                    this.lblNext.AutoSize = true;
                    this.splitContainer1.Panel2.Controls.Add(this.lblNext);
                    this.lblNext.Name = "lblNext";
                    this.lblNext.Text = this.lblNext.Text;
                    this.lblNext.Tag = CurrentPage + 1;
                    this.lblNext.Padding = new Padding(1, 0, 1, 0);
                    this.lblNext.AutoSize = true;
                    this.lblNext.Anchor = AnchorStyles.Top | AnchorStyles.Right;
                    this.lblNext.Location = new Point(xLocation, lblGoto.Location.Y);//new Point(this.splitContainer1.Panel2.Width - ((_totalPages - page) * 15) - 5, label1.Location.Y);
                    this.lblNext.Click -= new EventHandler(lbl_Click);
                    this.lblNext.Click += new EventHandler(lbl_Click);
                    xLocation = xLocation + this.lblNext.Width;
                }


                if ((CurrentPage < (_totalPages - maxRange)) && (_totalPages > _range))
                {
                    //LinkLabel lbl = new LinkLabel();
                    this.lblLast.AutoSize = true;
                    this.splitContainer1.Panel2.Controls.Add(this.lblLast);
                    this.lblLast.Name = "lblLast";
                    this.lblLast.Text = this.lblLast.Text;
                    this.lblLast.Tag = _totalPages;
                    this.lblLast.Padding = new Padding(1, 0, 1, 0);
                    this.lblLast.AutoSize = true;
                    this.lblLast.Anchor = AnchorStyles.Top | AnchorStyles.Right;
                    this.lblLast.Location = new Point(xLocation, this.lblGoto.Location.Y);//new Point(this.splitContainer1.Panel2.Width - ((_totalPages - page) * 15) - 5, label1.Location.Y);
                    this.lblLast.Click -= new EventHandler(lbl_Click);
                    this.lblLast.Click += new EventHandler(lbl_Click);
                    xLocation = xLocation + this.lblLast.Width;
                }
            }
        }


		#endregionMethods


        public GridGroupingControl Grid
        {
            get { return gridGroupingControl1; }
            set { gridGroupingControl1 = value; }
        }
	

    }
}
