using System;
using System.Collections;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Tools;
 using System.Windows.Forms;
using Symcon.Utils.MessageDispatcher; 
using Symcon.Utils.MessageLogger;
namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// This class is used for Validations in Grid
	/// </summary>
	public class Validations
	{
		string[,] ColDetailsArry=null;
		string prevData;
	
		/// <summary>
		/// Pass the details of the colums to validate
		/// </summary>
		/// <param name="gridInstance">grid grouping Control to validate</param>
		/// <param name="colDetails">Two dimensional array to store the colNames & the field size </param>

		public  Validations(GridGroupingControl gridInstance,string[,] colDetails)
		{
			ColDetailsArry=colDetails;
			gridInstance.TableControlCurrentCellKeyPress+=new GridTableControlKeyPressEventHandler(gridInstance_TableControlCurrentCellKeyPress);
			gridInstance.TableControlCurrentCellKeyDown+=new GridTableControlKeyEventHandler(gridInstance_TableControlCurrentCellKeyDown);
		}



        /// <summary>
        /// Handles the TableControlCurrentCellKeyPress event of the gridInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlKeyPressEventArgs"/> instance containing the event data.</param>
		private  void gridInstance_TableControlCurrentCellKeyPress(object sender, GridTableControlKeyPressEventArgs e)
		{
			try
			{
				if(e.TableControl.CurrentCell.Renderer!=null)
				{
					if(!char.IsControl(e.Inner.KeyChar))
					{
						GridTableCellStyleInfo styleInfo = e.TableControl.Model[e.TableControl.CurrentCell.RowIndex,e.TableControl.CurrentCell.ColIndex];
						if(styleInfo.TableCellIdentity!=null)
						{
							if(styleInfo.TableCellIdentity!=null)
							{
								string colNameToValidate = styleInfo.TableCellIdentity.Column.Name;
								string colHeader = styleInfo.TableCellIdentity.Column.HeaderText;
								string data=e.TableControl.CurrentCell.Renderer.ControlText;
								int rowCount=ColDetailsArry.GetLength(0);
								int colCount=ColDetailsArry.Rank;
								for(int i=0;i<rowCount;i++)
								{
									string colName=ColDetailsArry[i,0];
									int fieldSize=int.Parse(ColDetailsArry[i,1],Symcon.BusinessObjects.SymConverter.Culture);
									if(colNameToValidate==colName)
									{
										if(data.Length>fieldSize-1)
										{
											string[] s = new string[2];
											s[0] = colHeader;
											s[1] = fieldSize.ToString(Symcon.BusinessObjects.SymConverter.Culture); 
											LRRMessagBox.ShowMessage("SID_FIELD_EXCEED","SID_LABEL",s,MessageBoxButtons.OK,MessageBoxIcon.Information);      
											e.Inner.Handled=true;	
											e.TableControl.CurrentCell.Renderer.ControlText = data;
										}
									}
								}
							}
						}
					}
					else 
					{
						if(e.Inner.KeyChar==(char)22)
						{
							GridTableCellStyleInfo styleInfo = e.TableControl.Model[e.TableControl.CurrentCell.RowIndex,e.TableControl.CurrentCell.ColIndex];
							if(styleInfo.TableCellIdentity!=null)
							{
								if(styleInfo.TableCellIdentity.Column!=null)
								{
									string colNameToValidate = styleInfo.TableCellIdentity.Column.Name;
									string colHeader = styleInfo.TableCellIdentity.Column.HeaderText;
									string data=e.TableControl.CurrentCell.Renderer.ControlText;
									int rowCount=ColDetailsArry.GetLength(0);
									int colCount=ColDetailsArry.Rank;
									for(int i=0;i<rowCount;i++)
									{
										string colName=ColDetailsArry[i,0];
										int fieldSize=int.Parse(ColDetailsArry[i,1],Symcon.BusinessObjects.SymConverter.Culture);
										if(colNameToValidate==colName)
										{
							
											if(data.Length>fieldSize)
											{
												string[] s = new string[2];
												s[0] = colHeader;
												s[1] = fieldSize.ToString(Symcon.BusinessObjects.SymConverter.Culture); 
												if(prevData=="")
													e.TableControl.CurrentCell.RejectChanges();
												else
													e.TableControl.CurrentCell.Renderer.ControlText =prevData;
												LRRMessagBox.ShowMessage("SID_FIELD_EXCEED","SID_LABEL",s,MessageBoxButtons.OK,MessageBoxIcon.Information);      
												e.Inner.Handled=true;	
												e.TableControl.CurrentCell.Activate(e.TableControl.CurrentCell.RowIndex,e.TableControl.CurrentCell.ColIndex);
											}
										}
									}						
								}
							}
						}
					}
				}
			}
			catch (Exception ex)
			{
				Logger.LogMessage(typeof(Validations),"TableControlCurrentCellKeyPress Event:"+ ex.Message  ,Logger.Level.Debug);
			}

		}

        /// <summary>
        /// Handles the TableControlCurrentCellKeyDown event of the gridInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlKeyEventArgs"/> instance containing the event data.</param>
		private void gridInstance_TableControlCurrentCellKeyDown(object sender, GridTableControlKeyEventArgs e)
		{
			try
			{
				if(e.Inner.KeyValue==17)//Control key
				{
					prevData=e.TableControl.CurrentCell.Renderer.ControlText;
				}
			}
			catch (Exception ex)
			{
				Logger.LogMessage(typeof(Validations),"TableControlCurrentCellKeyPress Event:"+ ex.Message  ,Logger.Level.Debug);
			}
		}

		
	}
}
