using System;
using System.Windows.Forms;
using System.Collections;
using System.Drawing;
using Syncfusion.Windows.Forms.Grid;

using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.Utils.Validator;
using Symcon.LightRidge.BusinessObjects.Utilities;
using Symcon.Utils.HeaderDispatcher; 
using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.PEAdvisor.GroupProperties;
using Symcon.LightRidge.PEAdvisor.IOScreen;
using Symcon.LightRidge.PEAdvisor.GridUtilities;

namespace Symcon.LightRidge.PEAdvisor.GroupProperties
{
	/// <summary>
	/// Summary description for GridDataArrayTransposeViewController.
	/// </summary>
    public class GridDataArrayTransposeViewController : IGroupGridFactory	
	{
		#region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="GridDataArrayTransposeViewController"/> class.
        /// </summary>
        public GridDataArrayTransposeViewController()
		{
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			_hashCollectionIndex.Add(typeof(LRRPropertyType),0);
			_hashCollectionIndex.Add(typeof(LRRSiteComponent),1);
		}
		#endregion

		#region IGroupGridFactory Members
		/// <summary>
		/// Creates the View
		/// </summary>
		/// <param name="custCollection"></param>
		public void LoadPanel(ArrayList custCollection,LRRSearchCriteriaItemGroupPropertiesUIUOMs srchCriteriaUIUOMs)
		{
			int rowCount = 0;
			int colCount = 0;

			//Split the collection according to data arrays
			object[,]  arrSplitComponents = SplitComponent(custCollection,out rowCount,out colCount);

			//Populate User Specifc UIUOMs
			this._lrrApp.DBManagerObj.PopulateCollection(_searchCriteriaUserSpecficUIUOMs);
			
			for(int iRow =0;iRow < rowCount;iRow++)
			{
				//Get the Col Count
				int iGridColCount = GetGridColCount(arrSplitComponents,iRow,1);
				
				//Get the Row Count
				int iGridRowCount = GetGridRowCount(arrSplitComponents,iRow,1);

				//Create Grid Instance,Panel,Register Events 
				GridControl gridControlInstance = new GridControl();
				gridControlInstance.Dock = DockStyle.Fill;
				Panel pnlGrid = new Panel();
				pnlGrid.Dock = DockStyle.Top;
				pnlGrid.Controls.Add(gridControlInstance);

				//Assign RowCount and ColCount
				gridControlInstance.RowCount = iGridRowCount;
				gridControlInstance.ColCount = iGridColCount;

				//Add the Panel to panel Collection
				_pnlCollection.Add(pnlGrid);
				
				//register grid events
				registerGridEvents(gridControlInstance);

				//Row/Col Freeze
				gridControlInstance.Rows.FreezeRange(FREEZE_ROW_START,FREEZE_ROW_END);
				gridControlInstance.Cols.FreezeRange(FREEZE_COL_START,FREEZE_COL_END);

				//Create a GridControl manager and attach to grid
				GridGroupPropertiesManager gridGroupPropsManager = new GridGroupPropertiesManager();

				gridGroupPropsManager.ForeColorChanged+=new Symcon.LightRidge.PEAdvisor.GroupProperties.GridGroupPropertiesManager.ForeColorChangesDel(gridManager_ForeColorChanged);
				gridGroupPropsManager.BackColorChanged+=new Symcon.LightRidge.PEAdvisor.GroupProperties.GridGroupPropertiesManager.BackColorChangesDel(gridManager_BackColorChanged);

				//Add to the Hash
				GridGroupPropsManagerHash.Add(gridControlInstance,gridGroupPropsManager);
				
				//Register the Binding Called Event
				gridGroupPropsManager.BindingManager.BindingCalled+=new Symcon.LightRidge.PEAdvisor.DataStore.LRRBindingManager.BindingCallDel(BindingManager_BindingCalled);

				//Render the grid
				createGrid(gridControlInstance,arrSplitComponents,iRow,gridGroupPropsManager,srchCriteriaUIUOMs);

				//Sort the Grid
				gridControlInstance.RaiseCellClick(PROPERTIES_COL,COMPONENT_ROW,new MouseEventArgs(MouseButtons.Left,1,1,1,1));
			}
            
            string userSettingsID = SearchQuery.SearchQuery.UserSettingsID;

            if (userSettingsID != null)
            {
                LRRUserSetting userSettings = new LRRUserSetting();
                _lrrApp.DBManagerObj.PopulateObject(userSettings, typeof(LRRUserSetting), " WHERE ID='" + userSettingsID + "'");

                if (userSettings != null)
                {
                    _gridPersistance.SetBytesToUserSettingsXml(userSettings.UserSettings);
                    _gridPersistance.GridCollection = GetPanelCollection();
                    _gridPersistance.LoadGridSettings(true, false);
                }
            }
		}
		
		/// <summary>
		/// Gets the panels containing the grids
		/// </summary>
		/// <returns></returns>
		public ArrayList GetPanelCollection()
		{
			return PanelCollection;
		}
		
		/// <summary>
		/// Hides the Modes
		/// </summary>
		public void HideModes()
		{
			Cursor.Current = Cursors.WaitCursor;
			Cursor.Show();
			
			//Check if Reference mode has changed
			foreach(LRRMode mode in LRRModel.ProjectModes)
			{
				if(mode.ArrListUpdatedColNames.Contains("Type"))
				{
					IEnumerator iEnum = GridGroupPropsManagerHash.GetEnumerator();
					while(iEnum.MoveNext())
					{
						DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
						GridControl gridControl =  dEntry.Key as GridControl;
						GridGroupPropertiesManager gridControlManager =  dEntry.Value as GridGroupPropertiesManager;
						ValidatePropertyValues(gridControl,gridControlManager);
					}
					break;
				}
			}

			GroupedViewUtilities.HideModes(GridGroupPropsManagerHash,DATA_COL_START);
			Cursor.Current = Cursors.Default;
		}
		
		/// <summary>
		/// Saves the query
		/// </summary>
		/// <param name="name"></param>
		/// <param name="desc"></param>
		/// <param name="groupLevel"></param>
		/// <param name="isPublished"></param>
		public void Save(string name,string desc,LRRSearchGroupLevel srchGroupLevel,bool isPublished)
		{
			GroupSearchCriteria searchQuery = this.SearchQuery;
			searchQuery.SearchQuery.Name = name;
			searchQuery.SearchQuery.Description = desc;
			searchQuery.SearchQuery.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            searchQuery.SearchQuery.TemplateClass = "Symcon.LightRidge.PEAdvisor.GroupProperties.GridDataArrayTransposeViewController";

			//UI UOMs
			LRRSearchCriteriaUserSpecificUIUOMs srchCriteriaUserSpecificUIUOMs = new LRRSearchCriteriaUserSpecificUIUOMs();
			LRRSearchCriteriaItemGroupPropertiesUIUOMs srchGrpPropsUIUOMs = new LRRSearchCriteriaItemGroupPropertiesUIUOMs();
			IDictionaryEnumerator iDictEnum = GridGroupPropsManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridGroupPropertiesManager gridControlManager = dEntry.Value as GridGroupPropertiesManager;
				string propertyType = gridControlManager.PropertyType;
				ArrayList rowHeaderList = gridControlManager.RowHeaderCellList;
                for (int i = 0; i < rowHeaderList.Count; i++)
				{
                    GroupRowHeaderCell rowHeaderCell = rowHeaderList[i] as GroupRowHeaderCell;
                    if (rowHeaderCell.UIUOMId != null && rowHeaderCell.UIUOMId.Trim() != "")
					{
						LRRSearchCriteriaItemGroupPropertiesUIUOM srchGrpPropsUIUOM = new LRRSearchCriteriaItemGroupPropertiesUIUOM();
						srchGrpPropsUIUOM.SearchCriteriaID = searchQuery.SearchQuery.ID;
                        srchGrpPropsUIUOM.ItemID = rowHeaderCell.ID;
						srchGrpPropsUIUOM.PropertyType = propertyType;
						srchGrpPropsUIUOMs.Add(srchGrpPropsUIUOM);

						//Save in the User Specific UIUOMs
						LRRSearchCriteriaUserSpecificUIUOM srchCriteriaUserSpecificUIUOM = new LRRSearchCriteriaUserSpecificUIUOM();
						srchCriteriaUserSpecificUIUOM.PropertyID = srchGrpPropsUIUOM.ID;
                        srchCriteriaUserSpecificUIUOM.UIUomID = rowHeaderCell.UIUOMId;
						srchCriteriaUserSpecificUIUOMs.Add(srchCriteriaUserSpecificUIUOM);
					}
				}
			}
			

			//Group Levels
			LRRSearchGroupLevel lrrLevel = new LRRSearchGroupLevel();
			lrrLevel.LevelID = searchQuery.SearchQuery.ID;

            lrrLevel.UserID = _lrrApp.DBManagerObj.AccessControlModel.UserID;

            IOScreen.GridPersistanceUtil gutil = _gridPersistance;
            gutil.GridCollection = _pnlCollection;
            gutil.SaveGridSettings(true, false);


            LRRUserSetting userSettings = new LRRUserSetting(); ;
            if (SearchQuery.SearchQuery.UserSettingsID == null)
            {
                SearchQuery.SearchQuery.UserSettingsID = userSettings.ID;
            }
            else
            {
                _lrrApp.DBManagerObj.PopulateObject(userSettings, typeof(LRRUserSetting), " WHERE ID='" + SearchQuery.SearchQuery.UserSettingsID + "'");
                userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                _lrrApp.DBManagerObj.CommitObject(userSettings);
            }

            userSettings.UserSettings = gutil.GetUserSettingsXmlBytes();
            userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            _lrrApp.DBManagerObj.CommitObject(userSettings, searchQuery.UserID);

			lrrLevel.Name = name;
			lrrLevel.Description = desc;
			lrrLevel.Type = "Query";
			lrrLevel.IsPublished = isPublished;

			LRRSearchGroupLevelLink lrrLvlLink = new LRRSearchGroupLevelLink();
			lrrLvlLink.ChildID = searchQuery.SearchQuery.ID;;
			lrrLvlLink.ParentID = srchGroupLevel.LevelID;
			lrrLvlLink.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;

			_lrrApp.DBManagerObj.CommitObject(searchQuery.SearchQuery,searchQuery.UserID);

			_lrrApp.DBManagerObj.CommitObject(lrrLevel,searchQuery.UserID);
			_lrrApp.DBManagerObj.CommitObject(lrrLvlLink,searchQuery.UserID);

			if(searchQuery.SearchCriteriaItems.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaItems,searchQuery.UserID);
			if(searchQuery.SearchCriteriaPropTypes.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaPropTypes,searchQuery.UserID);
			if(srchGrpPropsUIUOMs.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(srchGrpPropsUIUOMs);
			if(srchCriteriaUserSpecificUIUOMs.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(srchCriteriaUserSpecificUIUOMs);
		
		}
		
		/// <summary>
		/// Saves the query with the UIUOMs
		/// </summary>
		public void Save()
		{
			//UI UOMs
			LRRSearchCriteriaItemGroupPropertiesUIUOMs srchGrpPropsUIUOMs = this.SearchQuery.SearchCriteriaItemGroupPropertiesUIUOMs;
			IDictionaryEnumerator iDictEnum = GridGroupPropsManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridGroupPropertiesManager gridControlManager = dEntry.Value as GridGroupPropertiesManager;
				string propertyType = gridControlManager.PropertyType;
				ArrayList colHeaderList = gridControlManager.ColHeaderCellList;
				for(int i=0;i < colHeaderList.Count;i++)
				{
					GroupColHeaderCell colHeaderCell = colHeaderList[i] as GroupColHeaderCell;
					if(colHeaderCell.UIUOMId != null && colHeaderCell.UIUOMId.Trim() != "")
					{
						string itemID = colHeaderCell.ID;
						LRRSearchCriteriaItemGroupPropertiesUIUOM srchGrpPropsUIUOM = srchGrpPropsUIUOMs.GetUIUOMObject(itemID,null,propertyType);

						//Save in the User Specific UIUOMs
						LRRSearchCriteriaUserSpecificUIUOM srchCriteriaUserSpecificUIUOM = _searchCriteriaUserSpecficUIUOMs.GetUIUOMObjectByPropertyID(srchGrpPropsUIUOM.ID);
						if(srchCriteriaUserSpecificUIUOM != null)
						{
                            //Commented
                            //Based on Isseu No: 1794 Date 7/11/2008.
							//srchCriteriaUserSpecificUIUOM.UIUomID = colHeaderCell.UIUOMId;
						}
						else
						{
							srchCriteriaUserSpecificUIUOM = new LRRSearchCriteriaUserSpecificUIUOM();
							srchCriteriaUserSpecificUIUOM.PropertyID = srchGrpPropsUIUOM.ID;
                            //Commented
                            //Based on Isseu No: 1794 Date 7/11/2008.
                            //srchCriteriaUserSpecificUIUOM.UIUomID = colHeaderCell.UIUOMId;
							_searchCriteriaUserSpecficUIUOMs.Add(srchCriteriaUserSpecificUIUOM);
						}
					}
				}
			}
			if(_searchCriteriaUserSpecficUIUOMs.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(_searchCriteriaUserSpecficUIUOMs);
		}
		
		/// <summary>
		/// Exports the contents of the grid to an excel
		/// </summary>
		/// <returns></returns>
		public GridControl Export()
		{
			return GroupedViewUtilities.Export(GridGroupPropsManagerHash);
		}
		
		/// <summary>
		/// Prints the contents of the grid
		/// </summary>
		public void Print()
		{
			GroupedViewUtilities.Print(GridGroupPropsManagerHash);
		}
		
		/// <summary>
		/// Print Preview of the contents of the grid
		/// </summary>
		public void PrintPreview()
		{
			GroupedViewUtilities.PrintPreview(GridGroupPropsManagerHash);
		}
		
		/// <summary>
		/// Clears all the bindings
		/// </summary>
		public void ClearAllBindings()
		{
			IEnumerator iEnum = GridGroupPropsManagerHash.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
				GridGroupPropertiesManager gridControlManager =  dEntry.Value as GridGroupPropertiesManager;
				ClearBindings(gridControlManager);
			}
		}

		
		public GroupSearchCriteria SearchQuery 
		{
			get{return _searchCriteria;}
			set
			{
				_searchCriteria = value;
			}
		}
		
		/// <summary>
		/// Method changes the UOM Set
		/// </summary>
		/// <param name="measSysID"></param>
		public void ChangeUOMSet(string measSysID)
		{
			IDictionaryEnumerator iDictEnum = GridGroupPropsManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridGroupPropertiesManager gridControlManager = (GridGroupPropertiesManager)dEntry.Value;
				if(gridControl.CurrentCell.IsActive)
				{
                    for (int iRow = DATA_ROW_START; iRow < gridControl.RowCount + DATA_ROW_START; iRow++)
					{
						try
						{
							//Get the Col Data from Grid Data Manager
							ArrayList     rowHeaderList = gridControlManager.RowHeaderCellList;
							//Get the col data
                            GroupRowHeaderCell rowData = rowHeaderList[iRow - DATA_ROW_START] as GroupRowHeaderCell;

                            string uomTypeID = rowData.UomTypeID;
							string uiUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(measSysID,uomTypeID);
							rowData.UIUOMId = uiUOMID;

							//Add to UI UOM hash
							if(gridControlManager.UIUOMHash.ContainsKey(rowData.PropertyID))
							{
                                gridControlManager.UIUOMHash[rowData.PropertyID] = uiUOMID;
							}
							else
							{
                                gridControlManager.UIUOMHash.Add(rowData.PropertyID, uiUOMID);
							}
						
							string uomDisplayName = LRRModel.ChildModel.GetUOMNameFromID(uiUOMID);
                            gridControl.Model[iRow, UOM_COL].Text = uomDisplayName;
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridDataArrayTransposeViewController),"Exception in applying Changed UOM Set:- " + ex.Message,Logger.Level.Debug);
						}
					}
					InitializeGroupPropertyValues(gridControl,gridControlManager);
					break;
				}
			}
		}

		/// <summary>
		/// Method Unsubscribes the Reconnect Completed Event
		/// </summary>
		public void UnSubscribeReconnectCompleted()
		{
			this._lrrApp.ReconnectCompleted-=new EventHandler(_lrrApp_ReconnectCompleted);
		}

		/// <summary>
		/// Method clears the sorting
		/// </summary>
		public void ClearSort()
		{
			GroupedViewUtilities.ClearSort(GridGroupPropsManagerHash);
		}

		/// <summary>
		/// Method moves the range of columns to right
		/// </summary>
		public void RightColMovement()
		{
			Cursor.Current = Cursors.WaitCursor;
			Cursor.Show();

			IDictionaryEnumerator iDictEnum = GridGroupPropsManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					gridControl.ResetVolatileData();
					gridControl.BeginUpdate();
					GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
					GroupedViewUtilities.RightColMovement(gridControlManager,gridControl);
					InitializeGroupPropertyValues(gridControl,gridControlManager);
					gridControl.EndUpdate();
					break;
				}
			}

			Cursor.Hide();
		}

		/// <summary>
		/// Method moves the range of columns to left
		/// </summary>
		public void LeftColMovement()
		{
			Cursor.Current = Cursors.WaitCursor;
			Cursor.Show();

			IDictionaryEnumerator iDictEnum = GridGroupPropsManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					gridControl.ResetVolatileData();
					gridControl.BeginUpdate();
					GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
					GroupedViewUtilities.LeftColMovement(gridControlManager,gridControl,DATA_COL_START,HEADER_ROW);
					InitializeGroupPropertyValues(gridControl,gridControlManager);
					gridControl.EndUpdate();
					break;
				}
			}

			Cursor.Hide();
		}

        /// <summary>
        /// This method hides the rows on change of Properties window selected records
        /// </summary>
        /// <param name="arrayUnSelectedRecords"></param>
        public void HideComponents(ArrayList arrayUnSelectedRecords)
        {
            GroupedViewUtilities.HideRows(arrayUnSelectedRecords, GridGroupPropsManagerHash, DATA_ROW_START);
        }

        /// <summary>
        /// This method hides the rows on change of Components window selected records
        /// </summary>
        /// <param name="arrayUnSelectedRecords"></param>
        public void HideProperties(ArrayList arrayUnSelectedRecords)
        {
            UnSelectedRecords = arrayUnSelectedRecords;
            GroupedViewUtilities.HideColumns(UnSelectedRecords, GridGroupPropsManagerHash, DATA_COL_START);
        }
		#endregion

		#region Private Methods
		/// <summary>
		/// Gets the no. of columns in the grid
		/// </summary>
		/// <param name="objCollection"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>
		private int GetGridColCount(object[,] objCollection,int iRow,int iCol)
		{
            int nModesCol = LRRModel.ProjectModes.Count;
			CustomCollection custCollection = objCollection[iRow,iCol] as CustomCollection;

			//Get Property Type
			LRRPropertyType lrrPropType = objCollection[iRow,0] as LRRPropertyType;
			string propertyType = lrrPropType.ID;

			IEnumerator iEnum = custCollection.GetEnumerator();
			int iColCount = 0;
			bool flag = false;
			while(iEnum.MoveNext())
			{
				IComponent iComp = iEnum.Current as IComponent;
				CustomCollection propsCollection = iComp.GetProperties();
				IEnumerator iPropEnum = propsCollection.GetEnumerator();
				while(iPropEnum.MoveNext())
				{
					IProperty iProp = iPropEnum.Current as IProperty;
					if(iProp.GetPropertyType() == propertyType)
					{
                        iColCount = iProp.GetGroupProperties().Count;
						flag = true;
						break;
					}
				}
				if(flag)
					break;
			}
            iColCount = iColCount * nModesCol + DATA_COL_START - 1;
            return iColCount;
		}

		/// <summary>
		/// Gets the no. of rows in the grid
		/// </summary>
		/// <param name="objCollection"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>
		private int GetGridRowCount(object[,] objCollection,int iRow,int iCol)
		{
			

			Hashtable propHash = new Hashtable();
			CustomCollection custCollection = objCollection[iRow,iCol] as CustomCollection;

			//Get Property Type
			LRRPropertyType lrrPropertyType = objCollection[iRow,0] as LRRPropertyType;
			string propertyType = lrrPropertyType.ID;

			IEnumerator iEnum = custCollection.GetEnumerator();
			int iRowCount = 0;
			while(iEnum.MoveNext())
			{
				IComponent iComp = iEnum.Current as IComponent;
				CustomCollection propsCollection = iComp.GetProperties();
				//Loop through Properties Collection
				IEnumerator iPropEnum = propsCollection.GetEnumerator();
				while(iPropEnum.MoveNext())
				{
					IProperty iProp =iPropEnum.Current as IProperty;
					if(iProp.GetPropertyType() == propertyType)
					{
                        iRowCount = iRowCount + 1;
					}
				}
			}
            iRowCount = iRowCount + DATA_ROW_START - 1;
            return iRowCount;

		}

		/// <summary>
		/// This method registers all the grid events
		/// </summary>
		/// <param name="gridControlInstance"></param>
		private void registerGridEvents(GridControl gridControlInstance)
		{
			gridControlInstance.CurrentCellValidating +=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellValidating);
			gridControlInstance.CurrentCellStartEditing+=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellStartEditing);
			gridControlInstance.CurrentCellCloseDropDown +=new Syncfusion.Windows.Forms.PopupClosedEventHandler(gridControlInstance_CurrentCellCloseDropDown);
			
			//add a sort header to the CellModels.
			gridControlInstance.CellModels.Add("ColumnHeaderCell", new GridSortColumnHeaderCellModel(gridControlInstance.Model));

			//catch the click
			gridControlInstance.CellClick +=new GridCellClickEventHandler(gridControlInstance_CellClick);

			//Event for preventing hidden columns from expanding
			gridControlInstance.ResizingColumns+=new GridResizingColumnsEventHandler(gridControlInstance_ResizingColumns);

			//Event for Handling Current Cell Key down
			gridControlInstance.CurrentCellKeyDown+=new KeyEventHandler(gridControlInstance_CurrentCellKeyDown);
			
			//Event for Handling Current Cell Key Press
			gridControlInstance.CurrentCellKeyPress+=new KeyPressEventHandler(gridControlInstance_CurrentCellKeyPress);

            //Event for Handling Current Cell Accepted Changes
            gridControlInstance.CurrentCellAcceptedChanges +=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellAcceptedChanges);
			//Event for Painting Header text
			gridControlInstance.Paint+=new PaintEventHandler(gridControlInstance_Paint);

			//Event for KeyDown 
			gridControlInstance.KeyDown+=new KeyEventHandler(gridControlInstance_KeyDown);

            //Enabling command stack
            gridControlInstance.CommandStack.Enabled = true;

			//Register Run Completed Event
			_lrrApp.RunControllerObj.RunCycleCompleted+=new EventHandler(RunControllerObj_RunCycleCompleted);

			//Event for Reconnect
			this._lrrApp.ReconnectCompleted+=new EventHandler(_lrrApp_ReconnectCompleted);

			//Events for Row movement
			gridControlInstance.RowsMoving+=new GridRangeMovingEventHandler(gridControlInstance_RowsMoving);
			gridControlInstance.RowsMoved+=new GridRangeMovedEventHandler(gridControlInstance_RowsMoved);

			//Enabling/Disabling Left/Right Col Movement Buttons
			gridControlInstance.SelectionChanged+=new GridSelectionChangedEventHandler(gridControlInstance_SelectionChanged);

			//Event for Col Movement
			gridControlInstance.ColsMoving+=new GridRangeMovingEventHandler(gridControlInstance_ColsMoving);

            gridControlInstance.Model.ClipboardPaste += new Syncfusion.Windows.Forms.Grid.GridCutPasteEventHandler(Model_ClipboardPaste);

			//Context Menu for switching recording values
			ContextMenu contMenu = new ContextMenu();
			gridControlInstance.ContextMenu = contMenu;
			contMenu.Popup+=new EventHandler(contMenu_Popup);
		}

		/// <summary>
		/// This method renders the grid
		/// </summary>
		/// <param name="gridControlInstance"></param>
		/// <param name="compCollection"></param>
		/// <param name="iRow"></param>
		/// <param name="gridGroupPropsManager"></param>
		/// <param name="srchCriteriaUIUOMs"></param>
		private void createGrid(GridControl gridControlInstance,object[,] compCollection,int iRow,GridGroupPropertiesManager gridGroupPropsManager,LRRSearchCriteriaItemGroupPropertiesUIUOMs srchCriteriaUIUOMs)
		{
			gridControlInstance.ResetVolatileData();
			gridControlInstance.BeginUpdate();
			
			//Set the Header appearence and properties
			GroupedViewUtilities.InitializeGrid(gridControlInstance);

			//Create Row Header,Col Header Data Objects
			ArrayList rowManagerList = new ArrayList();
			ArrayList colHeaderCellList = new ArrayList();
			Hashtable RowHeader = new Hashtable();
			Hashtable ColHeader = new Hashtable();

			//Get Property Type
			LRRPropertyType lrrPropertyType = compCollection[iRow,0] as LRRPropertyType;
			string propertyType = lrrPropertyType.ID;

			//Assign to GridControlManager
			gridGroupPropsManager.RowHeaderCellList = rowManagerList;
			gridGroupPropsManager.ColHeaderCellList = colHeaderCellList;
			gridGroupPropsManager.RowHeader = RowHeader;
			gridGroupPropsManager.ColHeader = ColHeader;
			gridGroupPropsManager.PropertyType = propertyType;

			//Clear the Row/Col Headers
			ClearHeaders(gridGroupPropsManager);

			//Start Grid Rendering
			if(!RowHeader.Contains(HEADER_ROW))
				RowHeader.Add(HEADER_ROW,GroupPropertiesSchemes.ATTRIBUTE_HEADER_NAME);
			if(!ColHeader.Contains(HEADER_ROW))
				ColHeader.Add(HEADER_ROW,GroupPropertiesSchemes.ATTRIBUTE_HEADER_NAME);
			
			try
			{
				//Attributes header row
				gridControlInstance[COMPONENT_ROW,PROPERTIES_COL].Text = GroupPropertiesSchemes.ATTRIBUTE_HEADER_NAME;
				gridControlInstance[COMPONENT_ROW,PROPERTIES_COL].BackColor = GroupPropertiesSchemes.ATTRIBUTES_BACK_COLOR;
				gridControlInstance[COMPONENT_ROW + 1,PROPERTIES_COL].BackColor = GroupPropertiesSchemes.ATTRIBUTES_BACK_COLOR;
			
				//UOM Header Row
				gridControlInstance[UOM_ROW,UOM_COL].Text = GroupPropertiesSchemes.UOM_HEADER_NAME;
				gridControlInstance[UOM_ROW,UOM_COL].BackColor = GroupPropertiesSchemes.ATTRIBUTES_BACK_COLOR;
                gridControlInstance[UOM_ROW + 1, UOM_COL].BackColor = GroupPropertiesSchemes.ATTRIBUTES_BACK_COLOR;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GridDataArrayTransposeViewController),"Exception in rendering Header row:- " + ex.Message,Logger.Level.Debug);
			}
			
			int gridLastRowNum = DATA_ROW_START;
			int gridLastColNum = DATA_COL_START;

			//Create Row Headers,Column Headers
			createHeaders(gridControlInstance,(CustomCollection) compCollection[iRow,1],gridGroupPropsManager,ref gridLastRowNum,ref gridLastColNum,srchCriteriaUIUOMs);

			gridControlInstance.ResumeLayout();
			gridControlInstance.EndUpdate(true);
		}
		
		/// <summary>
		/// Creates the Headers for the grid
		/// </summary>
		/// <param name="gridControlInstance"></param>
		/// <param name="compCollection"></param>
		/// <param name="gridGroupPropsManager"></param>
		/// <param name="gridLastRowNum"></param>
		/// <param name="gridLastColNum"></param>
		/// <param name="srchCriteriaUIUOMs"></param>
        private void createHeaders(GridControl gridControlInstance, CustomCollection compCollection, GridGroupPropertiesManager gridGroupPropsManager, ref int gridLastRowNum, ref int gridLastColNum, LRRSearchCriteriaItemGroupPropertiesUIUOMs srchCriteriaUIUOMs)
        {
            //Create Row Header,Col Header Data Objects
            ArrayList rowManagerList = gridGroupPropsManager.RowHeaderCellList;
            ArrayList colHeaderCellList = gridGroupPropsManager.ColHeaderCellList;
            Hashtable RowHeader = gridGroupPropsManager.RowHeader;
            Hashtable ColHeader = gridGroupPropsManager.ColHeader;
            string propertyType = gridGroupPropsManager.PropertyType;

            //Start Grid Rendering
            int nModesCol = gridLastColNum;
            int nSiteCol = gridLastColNum;
            int nSitePropRow = gridLastRowNum;
            string ID = "";
            string name = "";
            string propID = "";
            string propName = "";
            int nModes = LRRModel.ProjectModes.Count;

            //Get Grp Property Type
            LRRPropertyType lrrPropertyType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(propertyType);
            string grpPropertyType = lrrPropertyType.PropertyType;
            gridGroupPropsManager.PropertyType = lrrPropertyType.ID;

            //Display Group Property Type
            gridControlInstance.ColWidths.SetSize(PROPERTIES_COL, GroupPropertiesSchemes.HEADER_COL_SIZE);

            //Display the data Array Name
            gridGroupPropsManager.HeaderText = grpPropertyType + ": 0 Items";

            //Loop through the Collection and render the grid
            IEnumerator iCompEnum = compCollection.GetEnumerator();
            while (iCompEnum.MoveNext())
            {
                IComponent iComp = iCompEnum.Current as IComponent;
                CustomCollection propsCollection = iComp.GetProperties();

                //Loop through Properties Collection
                IEnumerator iPropEnum = propsCollection.GetEnumerator();
                while (iPropEnum.MoveNext())
                {
                    IProperty iProp = iPropEnum.Current as IProperty;

                    if (iProp.GetPropertyType() != propertyType)
                        continue;
                    //Loop through Group Properties Collection
                    CustomCollection grpPropsCollection = iProp.GetGroupProperties();

                    //Display the data Array Name
                    gridGroupPropsManager.HeaderText = grpPropertyType + ": " + grpPropsCollection.Count + " Items";

                    IEnumerator iGroupPropEnum = grpPropsCollection.GetEnumerator();
                    while (iGroupPropEnum.MoveNext())
                    {
                        IProperty iGroupProp = iGroupPropEnum.Current as IProperty;
                        ID = iGroupProp.GetID();
                        name = iGroupProp.GetName();
                        if (ID == null)
                            ID = name;
                        if ((ID == null) && (name == null))
                        {
                            LRRMessagBox.ShowMessage("SID_BLANK_ERROR", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            continue;
                        }
                        if (!ColHeader.Contains(ID))
                        {
                            ColHeader.Add(ID, name);
                            try
                            {
                                gridControlInstance.CoveredRanges.Add(GridRangeInfo.Cells(PROPERTIES_ROW, gridLastColNum, PROPERTIES_ROW, gridLastColNum + nModes - 1));
                                gridControlInstance[PROPERTIES_ROW, gridLastColNum].Text = name;
                                gridControlInstance[PROPERTIES_ROW, gridLastColNum].BackColor = GroupPropertiesSchemes.ATTRIBUTES_BACK_COLOR;
                                gridControlInstance[PROPERTIES_ROW, gridLastColNum].CellTipText = name;
                                gridControlInstance[PROPERTIES_ROW, gridLastColNum].HorizontalAlignment = GridHorizontalAlignment.Left;
                                gridControlInstance[PROPERTIES_ROW, gridLastColNum].TextMargins.Left = 5;
                                //Loop through the Modes collection and render the grid
                                foreach (LRRMode lrrMode in LRRModel.ProjectModes)
                                {
                                    try
                                    {
                                        gridControlInstance[MODES_ROW, nModesCol].WrapText = false;
                                        gridControlInstance[MODES_ROW, nModesCol].AutoSize = true;
                                        gridControlInstance[MODES_ROW, nModesCol].Text = lrrMode.ModeDisplayName;
                                        gridControlInstance[MODES_ROW, nModesCol].BackColor = GroupPropertiesSchemes.GetModeBackColor(lrrMode.ModeId);
                                        gridControlInstance[MODES_ROW, nModesCol].TextColor = GroupPropertiesSchemes.GetModeTextColor(lrrMode.ModeId);
                                        gridControlInstance[MODES_ROW, nModesCol].HorizontalAlignment = GridHorizontalAlignment.Center;
                                        gridControlInstance[MODES_ROW, nModesCol].ReadOnly = true;
                                    }
                                    catch (Exception ex)
                                    {
                                        Logger.LogMessage(typeof(GridDataArrayTransposeViewController), "Exception in rendering Mode:- " + ex.Message, Logger.Level.Debug);
                                    }
                                    //Add Column Header Info
                                    GroupColHeaderCell colHeaderCell = new GroupColHeaderCell();
                                    colHeaderCell.Name = name;
                                    colHeaderCell.ID = ID;
                                    colHeaderCell.ModeID = lrrMode.ModeId;
                                    colHeaderCellList.Add(colHeaderCell);

                                    nModesCol = nModesCol + 1;

                                }

                            }
                            catch (Exception ex)
                            {
                                Logger.LogMessage(typeof(GridDataArrayTransposeViewController), "Exception in rendering Group Property :- " + ex.Message, Logger.Level.Debug);
                            }
                            gridLastColNum = nModesCol;
                        }
                    }
                    //Put Properties,Modes,Components in Col Header
                    propID = iProp.GetID();
                    propName = iProp.GetName();
                    if (!RowHeader.Contains(propID))
                    {
                        RowHeader.Add(propID, propName);
                        try
                        {
                            //gridControlInstance.CoveredRanges.Add(GridRangeInfo.Cells(PROPERTIES_ROW, nSitePropCol, PROPERTIES_ROW, nSitePropCol + nModes - 1));
                            gridControlInstance[nSitePropRow, PROPERTIES_COL].Text = propName;
                            gridControlInstance[nSitePropRow, PROPERTIES_COL].BackColor = GroupPropertiesSchemes.ATTRIBUTES_BACK_COLOR;
                            gridControlInstance[nSitePropRow, PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                        }
                        catch (Exception ex)
                        {
                            Logger.LogMessage(typeof(GridDataArrayTransposeViewController), "Exception in rendering Property:- " + ex.Message, Logger.Level.Debug);
                        }

                        //Get the UOMs
                        string uomTypeID = null;
                        uomTypeID = lrrPropertyType.UOMTypeID;

                        LRRMeasurementSystemDefinitions lrrMSDs = (LRRMeasurementSystemDefinitions)_lrrApp.LRRDataStoreObj.GetDataStoreObject(typeof(LRRMeasurementSystemDefinitions));
                        string uiUOMID = lrrMSDs.GetLocaleUOM(_lrrApp.AppLocaleID, uomTypeID);
                        string uomID = lrrMSDs.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID, uomTypeID);

                        if (srchCriteriaUIUOMs != null)
                        {
                            LRRSearchCriteriaItemGroupPropertiesUIUOM srchCriteriaUIUOM = srchCriteriaUIUOMs.GetUIUOMObject(iComp.GetID(), null, propertyType);
                            if (srchCriteriaUIUOM != null)
                            {
                                LRRSearchCriteriaUserSpecificUIUOM uiUOMObj = _searchCriteriaUserSpecficUIUOMs.GetUIUOMObjectByPropertyID(srchCriteriaUIUOM.ID);
                                if (uiUOMObj != null)
                                    uiUOMID = uiUOMObj.UIUomID;
                            }
                        }

                        //string newUIUOM = CommonViews.ChangedUIUOMDictionary.RetrieveUOM(this.GetType(), ((Symcon.LightRidge.BusinessObjects.LRRSiteComponentProperty)(iProp)).ComponentID);
                        //if (newUIUOM != null)
                        //    uiUOMID = newUIUOM;

                        if (uiUOMID == null || uiUOMID.Trim() == "")
                            uiUOMID = uomID;

                        

                            

                        //Render UOMs
                        LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomTypeID);

                        //Get the UI UOM
                        //Get UOM name from UOMID
                        string uomName = null;
                        string uiUOMName = null;
                        string uomDisplayName = null;
                        if (uomID != null && uomID.Trim() != "")
                        {
                            uomName = LRRModel.ChildModel.GetUOMNameFromID(uomID);
                        }
                        if (uiUOMID != null && uiUOMID.Trim() != "")
                        {
                            uiUOMName = LRRModel.ChildModel.GetUOMNameFromID(uiUOMID);
                        }
                        if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
                        {
                            uomDisplayName = uiUOMName;
                        }
                        else
                        {
                            uomDisplayName = uomName;
                        }

                        try
                        {
                            //gridControlInstance.CoveredRanges.Add(GridRangeInfo.Cells(UOM_ROW, nSitePropCol, UOM_ROW, nSitePropCol + nModes - 1));
                            gridControlInstance.Model[nSitePropRow,UOM_COL].Text = uomDisplayName;
                            gridControlInstance.Model[nSitePropRow, UOM_COL].CellType = "ComboBox";
                            gridControlInstance.Model[nSitePropRow, UOM_COL].DataSource = uoms;
                            uoms.SortColumn = "Name";
                            uoms.Sort();
                            gridControlInstance.Model[nSitePropRow, UOM_COL].DisplayMember = "Name";
                            gridControlInstance.Model[nSitePropRow, UOM_COL].ValueMember = "ID";
                            gridControlInstance.Model[nSitePropRow, UOM_COL].BackColor = GroupPropertiesSchemes.UOM_BACK_COLOR;
                        }
                        catch (Exception ex)
                        {
                            Logger.LogMessage(typeof(GridDataArrayTransposeViewController), "Exception in rendering UOM:- " + ex.Message, Logger.Level.Debug);
                        }
                        GroupRowHeaderCell rowHeaderCell = new GroupRowHeaderCell();
                        
                        rowHeaderCell.ID = iComp.GetID();
                        rowHeaderCell.PropertyID = iProp.GetID();
                        rowHeaderCell.UOMName = uomName;
                        rowHeaderCell.UIUOMId = uiUOMID;
                        rowHeaderCell.UomTypeID = uomTypeID;
                        rowManagerList.Add(rowHeaderCell);

                        //Put Component on Header Grid
                        ID = iComp.GetID() + "|" + propID;
                        name = iComp.GetName() + "(" + propName + ")";
                        rowHeaderCell.Name = name;
                        try
                        {
                            //gridControlInstance.CoveredRanges.Add(GridRangeInfo.Cells(COMPONENT_ROW, nSiteCol, COMPONENT_ROW, nSitePropCol - 1));
                            gridControlInstance[nSitePropRow, PROPERTIES_COL].Text = name;
                            gridControlInstance[nSitePropRow, PROPERTIES_COL].BackColor = GroupPropertiesSchemes.COMPONENT_BACK_COLOR;
                        }
                        catch (Exception ex)
                        {
                            Logger.LogMessage(typeof(GridDataArrayTransposeViewController), "Exception in rendering Component Name:- " + ex.Message, Logger.Level.Debug);
                        }
                        
                    }
                }


                nSitePropRow++;
            }
        }

		/// <summary>
		/// This method initializes the property values,displays it on grid,adds bindings
		/// </summary>
		/// <param name="gridControlInstance"></param>
		/// <param name="gridGroupPropsManager"></param>
		private void InitializeGroupPropertyValues(GridControl gridControlInstance,GridGroupPropertiesManager gridGroupPropsManager)
		{
			//Get the Modes Count
			int iModesCount = LRRModel.ProjectModes.Count;

			//Get the Row Data from Grid Data Manager
			ArrayList     rowHeaderCellList = gridGroupPropsManager.RowHeaderCellList;

			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridGroupPropsManager.ColHeaderCellList;
			
			//Get the Row Header from Grid Data Manager
			Hashtable		RowHeader = gridGroupPropsManager.RowHeader;

			//Get the Col Header from Grid Data Manager
			Hashtable		ColHeader = gridGroupPropsManager.ColHeader;
			
			//Get the Qualifier ID
			string qualifierID = gridGroupPropsManager.QualifierID;

			//Get the Property Type
			string propertyType = gridGroupPropsManager.PropertyType;

			//Clear the Bindings
			ClearBindings(gridGroupPropsManager);

			//Clear the cell Addresses
			gridGroupPropsManager.PropValueCellAddress.Clear();
	
			//Loop through the grid matrix,put property values,add bindings
			for(int iRow = DATA_ROW_START;iRow < RowHeader.Count + DATA_ROW_START - 1; iRow++)
			{
				for(int iCol = DATA_COL_START; iCol < (ColHeader.Count - DATA_COL_START + 2) * iModesCount + DATA_COL_START; iCol++)
				{
					try
					{
						//Get the row data
						GroupRowHeaderCell rowData = rowHeaderCellList[iRow - DATA_ROW_START] as GroupRowHeaderCell;
						

						//Get the column data
						GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;
                        string lookUpPropID = colData.ID;
						string modeID   = colData.ModeID;
						string siteCompID = rowData.ID;
						string siteCompPropID = rowData.PropertyID;
						string uiUOMID = rowData.UIUOMId;
					
						//check for EVVs,if yes then put an exclusive drop down
						object objProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
						if(objProp != null)
						{
							IProperty iProp = objProp as IProperty;
							string uomTypeID = iProp.GetUOMType();
							if(uomTypeID != null && uomTypeID.Trim() != "")
							{
								LRRUOMType lrrUOMType = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(uomTypeID);
								if(lrrUOMType != null && lrrUOMType.IsEVV && !LRRModel.ChildModel.ConstantEVVs.ContainsKey(uomTypeID))
								{
									LRREVVs evvs = LRRModel.ChildModel.GetEVVsFromUOMTypeID(uomTypeID);
									gridControlInstance.Model[iRow,iCol].CellType = "ComboBox";
									gridControlInstance.Model[iRow,iCol].DataSource = evvs;
									gridControlInstance.Model[iRow,iCol].DisplayMember = "EVV";
									gridControlInstance.Model[iRow,iCol].ValueMember = "EVV";
                                    gridControlInstance.Model[iRow, iCol].DropDownStyle = GridDropDownStyle.Exclusive;
								}
							}
						}

						//Get the group Property Value Object
						string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);
						object objPropValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(groupPropID,lookUpPropID,qualifierID,modeID);
					
						//Put Property Values in grid cells
						if(objPropValue != null)
						{
							IPropertyValue iPropValue = objPropValue as IPropertyValue;

							//Add Bindings
							GridStyleInfo gsInfo = gridControlInstance.Model[iRow,iCol];
							gridGroupPropsManager.BindingManager.AddBinding(objPropValue,"PropertyValue",gsInfo,"Text");

							//Add Colour Binding
							gridGroupPropsManager.PropValueCellAddress.Add(iPropValue.GetID(),new GridCellAddress(iRow,iCol));
							iPropValue.ForeColorChanged+=new EventHandler(gridGroupPropsManager.ForeColorChangedHandler);
							iPropValue.BackColorChanged+=new EventHandler(gridGroupPropsManager.BackColorChangedHandler);
						
							//Convert property value according to UIUOM 
							string propertyValue = iPropValue.GetPropertyValue();
							string actualPropertyValue = iPropValue.GetActualPropertyValue();
							string uomID = iPropValue.GetUOMID();
							if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
							{
								propertyValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,propertyValue);
								actualPropertyValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,actualPropertyValue);
							}

                            string lrrDataType = iPropValue.GetDataType();
                            if (lrrDataType == null || lrrDataType.Trim() == "")
                            {
                                propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                            }

                            gridControlInstance.Model[iRow, iCol].Text = propertyValue;
							gridControlInstance.Model[iRow,iCol].HorizontalAlignment = GridHorizontalAlignment.Right;
							gridControlInstance.Model[iRow,iCol].AutoSize = true;
                            gridControlInstance.Model[iRow, iCol].Trimming = StringTrimming.EllipsisCharacter;

							//						gridControlInstance.Model[iRow,iCol].BackColor = iPropValue.GetBackColor();
							//						gridControlInstance.Model[iRow,iCol].TextColor = iPropValue.GetForeColor();

							//Set Colour
							SetColor(iPropValue,gridControlInstance.Model[iRow,iCol]);

							SetCellToolTipText(iPropValue,gsInfo,uiUOMID);
						}
						else
						{
							gridControlInstance.Model[iRow,iCol].BackColor = Color.LightGray;
							gridControlInstance.Model[iRow,iCol].TextColor = Color.Black;
						}
					}
					catch(Exception ex)
					{
						Logger.LogMessage(typeof(GridDataArrayTransposeViewController),"Exception in rendering Data Array Value:- " + ex.Message,Logger.Level.Debug);
					}
				}
			}
		}

		/// <summary>
		/// Method Validates the property Values and sets the colors on the grid cells
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		private void ValidatePropertyValues(GridControl gridControl,GridGroupPropertiesManager gridControlManager)
		{
			gridControl.BeginUpdate();
			//Get the Modes Count
			int iModesCount = LRRModel.ProjectModes.Count;
			//Get the Row Data from Grid Data Manager
			ArrayList     rowHeaderCellList = gridControlManager.RowHeaderCellList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			//Get the Row Header from Grid Data Manager
			Hashtable	  RowHeader = gridControlManager.RowHeader;
			//Get the Col Header from Grid Data Manager
			Hashtable	  ColHeader = gridControlManager.ColHeader;

			string qualifierID = null;

			//Loop through the grid matrix,put property values,add bindings
			for(int iRow = DATA_ROW_START;iRow < RowHeader.Count + DATA_ROW_START - 1; iRow++)
			{
				for(int iCol = DATA_COL_START; iCol < (ColHeader.Count - DATA_COL_START + 1) * iModesCount + DATA_COL_START; iCol++)
				{
					try
					{
						//Get the row data
						GroupRowHeaderCell rowData = rowHeaderCellList[iRow - DATA_ROW_START] as GroupRowHeaderCell;
						string lookUpPropID = rowData.ID;

						//Get the column data
						GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;
						string modeID   = colData.ModeID;
						string siteCompID = colData.ID;
						string siteCompPropID = colData.PropertyID;
					
						//Get the group Property Value Object
						string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);
						object objPropValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(groupPropID,lookUpPropID,qualifierID,modeID);
					
						//Put Property Values in grid cells
						if(objPropValue != null)
						{
							IPropertyValue iPropValue = objPropValue as IPropertyValue;
							iPropValue.Validate();
							//Set Colour
							SetColor(iPropValue,gridControl.Model[iRow,iCol]);
						}
					}
					catch(Exception ex)
					{
						Logger.LogMessage(typeof(GridQualifierViewController),"Exception in rendering Group Property value:- " + ex.Message,Logger.Level.Debug);
					}
				}
			}
			gridControl.EndUpdate(true);
		}

		/// <summary>
		/// Sets the Cell Tool Tip Text
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="gsInfo"></param>
		/// <param name="uiUOMID"></param>
		private void SetCellToolTipText(IPropertyValue iPropValue,GridStyleInfo gsInfo,string uiUOMID)
		{
            gsInfo.CellTipText = null;
			string uomID = iPropValue.GetUOMID();
			string actualPropValue = iPropValue.GetActualPropertyValue();
			string siteCompPropID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).SiteComponentPropID;
            string histTagID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).HistTagID;

			LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
			LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
			string equipmentTag = siteComp.EquipmentTag;
            string histTag = null;
            histTag = histTagID == null || histTagID == "" ? null : LRRModel.HistDatas.GetHistDataFromHistTagUID(histTagID).HistTag;

			if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
			{
				actualPropValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,actualPropValue);
			}

            string strToolTip = actualPropValue;
            gsInfo.CellTipText = strToolTip;

			if(iPropValue.GetOverrideState())
			{
				string preOverrideValue = iPropValue.GetActualOverridedValue();
				if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
				{
					preOverrideValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,preOverrideValue);
				}
                
                string lrrDataType = iPropValue.GetDataType();
                //if (lrrDataType == null || lrrDataType.Trim() == "")
                //{
                //    actualPropValue = Utilities.FormatNumber(actualPropValue, _lrrApp);
                //    preOverrideValue = Utilities.FormatNumber(preOverrideValue, _lrrApp);
                //}
                strToolTip = LRRHeaderText.GetHeaderString("SID_PREOVERRIDEVALUE") + ":" + preOverrideValue + "\n" +
                 LRRHeaderText.GetHeaderString("SID_POSTOVERRIDEVALUE") + ":" + actualPropValue;
                if (string.IsNullOrEmpty(histTag))
                {
                    gsInfo.CellTipText = strToolTip;
                }
                else
                {
                    if (iPropValue.GetModeID() == "1")
                        gsInfo.CellTipText = strToolTip + "\n" +
                       LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
                    else
                        gsInfo.CellTipText = strToolTip;

                }

				
			}
			else
			{
                string lrrDataType = iPropValue.GetDataType();
                //if (lrrDataType == null || lrrDataType.Trim() == "")
                //{
                //    actualPropValue = Utilities.FormatNumber(actualPropValue, _lrrApp);
                //}
                if (!string.IsNullOrEmpty(histTag) && iPropValue.GetModeID() == "1")
                    gsInfo.CellTipText = strToolTip + "\n" + LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
            }
		}

		/// <summary>
		/// This method clears the row/col Headers collection
		/// </summary>
		private void ClearHeaders(GridGroupPropertiesManager gridGroupPropsManager)
		{
			gridGroupPropsManager.RowHeader.Clear();
			gridGroupPropsManager.ColHeader.Clear();
		}

		/// <summary>
		/// This method clears the bindings
		/// </summary>
		/// <param name="gridGroupPropsManager"></param>
		private void ClearBindings(GridGroupPropertiesManager gridGroupPropsManager)
		{
			if(gridGroupPropsManager.BindingManager != null)
			{
				gridGroupPropsManager.BindingManager.RemoveBinding();
			}
		}

		/// <summary>
		/// This method applies sorting
		/// </summary>
		/// <param name="col"></param>
		private void SortCol(int col,GridControl gridControl,GridGroupPropertiesManager gridGroupPropsManager)
		{
			if(gridControl.CurrentCell.IsEditing)
				gridControl.CurrentCell.EndEdit();
		
			GridStyleInfo style = new GridStyleInfo();
			style.Tag = "";

			//turn off the old sort arrow
			for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				if(col1 != col && gridControl[1, col1].CellType == "ColumnHeaderCell")
				{
					gridControl[1, col1].CellType = "Header";
					break;
				}

			//turn on new sort arrow
			gridControl[1, col].CellType = "ColumnHeaderCell";

			System.ComponentModel.ListSortDirection dir = System.ComponentModel.ListSortDirection.Ascending;
			if(gridControl[1, col].HasTag)
			{
				if((System.ComponentModel.ListSortDirection) gridControl[1, col].Tag == System.ComponentModel.ListSortDirection.Ascending)
					dir = System.ComponentModel.ListSortDirection.Descending;
			}

			gridControl[1, col].Tag = dir;
			
			int headerCount = 2;
			string colName = null;
			if(col == 1)
				colName = "Name";
			if(col == 2)
				colName = "UOMName";
			int direction = 0;
			if(dir == System.ComponentModel.ListSortDirection.Ascending)
				direction = 0;
			if(dir == System.ComponentModel.ListSortDirection.Descending)
				direction = 1;
			
			//Get the data from the grid manager
			Comparer   rowComparer = GroupedViewUtilities.RowComparer;
			gridControl.Data.SortByColumn(col,dir,headerCount);
			gridControl.Refresh();

			//Sort the Row List
			ArrayList rowManagerList = gridGroupPropsManager.RowHeaderCellList;
			rowComparer.SortColumn = colName;
			rowComparer.SortingOrder = direction;
			rowManagerList.Sort(rowComparer);
		}
		
		/// <summary>
		/// Splits the collection
		/// </summary>
		/// <param name="custCollection"></param>
		/// <param name="rowCount"></param>
		/// <param name="colCount"></param>
		/// <returns></returns>
		private object[,] SplitComponent(ArrayList custCollection,out int rowCount,out int colCount)
		{
			rowCount = 0;
			colCount = 0;

			ArrayList arrIndex = new ArrayList(SUPPORTED_COLLECTION_COUNT);
			for(int i=0;i < SUPPORTED_COLLECTION_COUNT;i++)
			{
				arrIndex.Add(null);
			}
			IEnumerator iCustEnum = custCollection.GetEnumerator();
			while(iCustEnum.MoveNext())
			{
				CustomCollection objCollection = iCustEnum.Current as CustomCollection;
				if(objCollection == null)
					continue;
				Type collectionType = objCollection.ItemType;
				if(_hashCollectionIndex[collectionType] != null)
				{
					int index = (int)_hashCollectionIndex[collectionType];
					arrIndex[index] = objCollection;
				}
			}
			
			object[,] arrSplitComponent = null;
			try
			{
				//Split based on Data Arrays
				LRRPropertyTypes lrrPropTypes = arrIndex[0] as LRRPropertyTypes;
				arrSplitComponent = new object[lrrPropTypes.Count,_hashCollectionIndex.Count];
				rowCount = lrrPropTypes.Count;
				colCount = _hashCollectionIndex.Count;

				int iRow = 0;
				IEnumerator iPropTypeEnum = lrrPropTypes.GetEnumerator();
				while(iPropTypeEnum.MoveNext())
				{
					LRRPropertyType lrrPropType = iPropTypeEnum.Current as LRRPropertyType;

					//Filter Site Components based on Property Type
					string sPropTypeID = lrrPropType.ID;
					LRRSiteComponents lrrSiteComponents = arrIndex[1] as LRRSiteComponents;
					LRRSiteComponents filteredSiteComps =  lrrSiteComponents.FilterSiteComponentsByPropertyType(sPropTypeID);
				
					//Assign to 2D-array
					arrSplitComponent[iRow,0] = lrrPropType;
					arrSplitComponent[iRow,1] = filteredSiteComps;
				
					iRow++;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GridDataArrayTransposeViewController),"Exception in splitting component :- " + ex.Message,Logger.Level.Debug);
			}
			
			return arrSplitComponent;
		}
		
		/// <summary>
		/// Gets the Group Property Value Object
		/// </summary>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>
		private object GetGroupPropertyValueObject(GridGroupPropertiesManager gridControlManager,int iRow,int iCol)
		{
			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.RowHeaderCellList;

			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the Qualifier ID
			string qualifierID = gridControlManager.QualifierID;

			//Get the Property Type
			string propertyType = gridControlManager.PropertyType;

			//Get the row data
			GroupRowHeaderCell rowData = rowSortList[iRow - DATA_ROW_START] as GroupRowHeaderCell;
			
			//Get the column data
			GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;
            string lookUpPropID = colData.ID;
			string modeID   = colData.ModeID;
			string siteCompPropID = rowData.PropertyID;
			string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);

			//Get the group Property Value Object
			object objPropValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(groupPropID,lookUpPropID,qualifierID,modeID);

			return objPropValue;
		}

        /// <summary>
        /// Property value changes
        /// </summary>
        /// <param name="gridControl"></param>
        /// <param name="gridControlManager"></param>
        /// <param name="iRow"></param>
        /// <param name="iCol"></param>
        /// <returns></returns>
        private bool changePropertyValue(GridControl gridControl, GridGroupPropertiesManager gridControlManager, int iRow, int iCol)
        {
            return changePropertyValue(gridControl, gridControlManager, iRow, iCol, null);
        }

        /// <summary>
        /// Property value changes
        /// </summary>
        /// <param name="gridControl"></param>
        /// <param name="gridControlManager"></param>
        /// <param name="iRow"></param>
        /// <param name="iCol"></param>
        /// <returns></returns>
        private bool changePropertyValue(GridControl gridControl, GridGroupPropertiesManager gridControlManager, int iRow, int iCol, string currentCellValue)
        {
            string currentValue = gridControl.CurrentCell.Renderer.ControlText;

            if (currentCellValue != null)
                currentValue = currentCellValue;

            currentValue = Utilities.ConvertCultureSpecficToPlainString(currentValue, _lrrApp);

			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.RowHeaderCellList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the row data
			GroupRowHeaderCell rowData = rowSortList[iRow - DATA_ROW_START] as GroupRowHeaderCell;
			
			
			//Get the Qualifier ID
			string qualifierID = gridControlManager.QualifierID;

			//Get the column data
			GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;
            string lookUpPropID = colData.ID;
			string modeID   = colData.ModeID;
			string siteCompPropID = rowData.PropertyID;
			string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);

			//Get the group Property Value Object
			object objPropValue = GetGroupPropertyValueObject(gridControlManager,iRow,iCol);
			if(objPropValue != null)
			{
				IPropertyValue iPropValue = objPropValue as IPropertyValue;

				//Get the UOMs
				string uomTypeID = null;
				string uomID = null;
				//Get the Property Object
//				object objProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
//				if(objProp != null)
//				{
//					IProperty iProp = objProp as IProperty;
//					uomTypeID = iProp.GetUOMType();
//					LRRMeasurementSystemDefinitions lrrMSDs = (LRRMeasurementSystemDefinitions)_lrrApp.LRRDataStoreObj.GetDataStoreObject(typeof(LRRMeasurementSystemDefinitions));
//					uomID = lrrMSDs.GetLocaleUOM(_lrrApp.AppLocaleID,uomTypeID);
//				}
//				string uomID = iPropValue.GetUOMID();
				//PropertyValue Validation
				/*if(currentValue != null && currentValue.Trim() != "")
				{
					if(!inputOutputValidation(iPropValue,currentValue))					
					{
						return false;
					}
				}*/
				uomID = iPropValue.GetUOMID();
				string newUOM = rowData.UIUOMId;
				string newpropValue = iPropValue.GetPropertyValueForUOM(newUOM,uomID,currentValue); // ConvertUOM(newUOM,uomID,currentValue);
				
				if(iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
				{
					bool prevOverrideState = iPropValue.GetOverrideState();
					iPropValue.SetOverrideState(newpropValue);
					bool currOverrideState = iPropValue.GetOverrideState();

					//Measurement Mode
					if(prevOverrideState != currOverrideState && modeID == "1")
					{
						_lrrApp.SwapGroupPropBindings(objPropValue,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}

					//Other Modes
					if(prevOverrideState != currOverrideState && modeID != "1")
					{
						_lrrApp.RemoveCascadedBindings(objPropValue,groupPropID,lookUpPropID,qualifierID,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}
					
					iPropValue.SetPropertyValue(newpropValue);
					
					PopulateGroupOverrides(iPropValue,groupPropID,qualifierID);

					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],newUOM);
				}
			}
			return true;
		}

		/// <summary>
		/// Apply this method while Deletion of PropertyValues for cascading.
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>

		private bool cascadeChangesToPropValues(GridControl gridControl,GridGroupPropertiesManager gridControlManager,int iRow,int iCol)
		{
			string currentValue = "";

			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.RowHeaderCellList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the row data
			GroupRowHeaderCell rowData = rowSortList[iRow - DATA_ROW_START] as GroupRowHeaderCell;
			string lookUpPropID = rowData.ID;
			
			//Get the Qualifier ID
			string qualifierID = gridControlManager.QualifierID;

			//Get the column data
			GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;
			string modeID   = colData.ModeID;
			string siteCompPropID = colData.PropertyID;
			string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);

			//Get the group Property Value Object
			object objPropValue = GetGroupPropertyValueObject(gridControlManager,iRow,iCol);
			if(objPropValue != null)
			{
				IPropertyValue iPropValue = objPropValue as IPropertyValue;

				//Get the UOMs
				string uomTypeID = null;
				string uomID = null;
				uomID = iPropValue.GetUOMID();
				string newUOM = colData.UIUOMId;
				string newpropValue = iPropValue.GetPropertyValueForUOM(newUOM,uomID,currentValue); // ConvertUOM(newUOM,uomID,currentValue);
				
				if(iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
				{
					bool prevOverrideState = iPropValue.GetOverrideState();
					iPropValue.SetOverrideState(newpropValue);
					bool currOverrideState = iPropValue.GetOverrideState();

					//Measurement Mode
					if(prevOverrideState != currOverrideState && modeID == "1")
					{
						_lrrApp.SwapGroupPropBindings(objPropValue,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}

					//Other Modes
					if(prevOverrideState != currOverrideState && modeID != "1")
					{
						_lrrApp.RemoveCascadedBindings(objPropValue,groupPropID,lookUpPropID,qualifierID,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}
					
					iPropValue.SetPropertyValue(newpropValue);
					
					PopulateGroupOverrides(iPropValue,groupPropID,qualifierID);

					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],newUOM);
				}
			}
			return true;
		}

		/// <summary>
		/// Changes property Values acording to UI UOM
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		private void changePropertyValueForUOM(GridControl gridControl,GridGroupPropertiesManager gridControlManager,int iRow,int iCol)
		{
			//Get the Row Data from Grid Data Manager
			ArrayList     rowHeaderCellList = gridControlManager.RowHeaderCellList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the Qualifier ID
			string qualifierID = gridControlManager.QualifierID;

			string propertyType = gridControlManager.PropertyType;

			//Get the old UOM and new UOM
			string newUOM = gridControl.CurrentCell.Renderer.ControlValue == null ? "" : gridControl.CurrentCell.Renderer.ControlValue.ToString();

			for(int iColIndex = DATA_COL_START; iColIndex <= gridControl.ColCount;iColIndex++)
			{
				//Get the row data
				GroupRowHeaderCell rowData = rowHeaderCellList[iRow - DATA_ROW_START] as GroupRowHeaderCell;
                string siteCompID = rowData.ID;
                string siteCompPropID = rowData.PropertyID;

				//Get the column data
				GroupColHeaderCell colData = colHeaderCellList[iColIndex - DATA_COL_START] as GroupColHeaderCell;
				string modeID   = colData.ModeID;
                string lookUpPropID = colData.ID;

				//Get the group Property Value Object
				string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);
				object objPropValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(groupPropID,lookUpPropID,qualifierID,modeID);

                //CommonViews.ChangedUIUOMDictionary.AddUOM(this.GetType(), rowData.ID.ToString(), newUOM);

				if(objPropValue != null)
				{
					IPropertyValue iPropValue = objPropValue as IPropertyValue;
					string uomID = iPropValue.GetUOMID();

					//Convert property value according to UIUOM 
					string propertyValue = iPropValue.GetPropertyValue();
					propertyValue = iPropValue.GetPropertyValueForUOM(uomID,newUOM,propertyValue);//ConvertUOM(uomID,newUOM,propertyValue);

                    string lrrDataType = iPropValue.GetDataType();
                    if (lrrDataType == null || lrrDataType.Trim() == "")
                    {
                        propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                    }
					gridControl.Model[iRow,iColIndex].Text = propertyValue;

					//Set Cell Tool Tip text
					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iColIndex],newUOM);

					//Set the UI UOM to Col
					colData.UIUOMId = newUOM;

					//Add to UI UOM hash
					if(gridControlManager.UIUOMHash.ContainsKey(rowData.PropertyID))
					{
                        gridControlManager.UIUOMHash[rowData.PropertyID] = newUOM;
					}
					else
					{
                        gridControlManager.UIUOMHash.Add(rowData.PropertyID, newUOM);
					}
					_lrrApp.LRRUIManagerObj.CheckTrendUOMChanged(iPropValue,newUOM);
				}
			}
		}

		/// <summary>
		/// Converts Property value from one UOM to another
		/// </summary>
		/// <param name="oldUOMValue"></param>
		/// <param name="newUOMValue"></param>
		/// <param name="propVal"></param>
		/// <returns></returns>
		/*
		private string ConvertUOM(string oldUOMValue,string newUOMValue,string propVal)
		{
			string newVal = "";
			if(propVal != null && propVal != "")
			{
				if(oldUOMValue == newUOMValue)
				{
					newVal = propVal;
				}
				else
				{
					if((oldUOMValue != null && oldUOMValue != "") &&
						(newUOMValue != null && newUOMValue != ""))
					{
												newVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(oldUOMValue,newUOMValue,propVal);
					}
				}
			}
			return newVal;
		}*/

		/// <summary>
		/// This method implements Input/Output Logics
		/// </summary>
		/// <param name="lrrSiteCompPropVal"></param>
		/// <param name="newVal"></param>
		private bool inputOutputValidation(IPropertyValue iPropValue,string newVal)
		{
			if(iPropValue != null)
			{
				if(iPropValue.GetDataType() != null && iPropValue.GetDataType().Trim() != "")
				{
					ErrorInfo errInfo = Validator.validateDataType(iPropValue.GetDataType(),newVal);
					if(errInfo.ErrorStatus)
					{
						MessageBox.Show(errInfo.ErrorMsg,errInfo.ErrorSource,MessageBoxButtons.OK,MessageBoxIcon.Warning);
						return false;
					}
				}
			}
			return true;
		}

		/// <summary>
		/// Populates Overrides Screen
		/// </summary>
		/// <param name="lrrSiteCompPropValue"></param>
		private void PopulateGroupOverrides(IPropertyValue iPropValue,string groupPropID,string qualifierID)
		{
			LRRGroupOverride lrrModelOverride = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(iPropValue.GetID());
			if(iPropValue.GetOverrideState())
			{
				if(lrrModelOverride != null)
				{
					LRRModel.ChildModel.GroupOverrides.Remove(lrrModelOverride);
				}
				LRRGroupOverride lrrGroupOverride = new LRRGroupOverride();

				string siteCompPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(groupPropID).SiteComponentPropID;
				string siteCompID = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID).ComponentID;
				
				string modeID = iPropValue.GetModeID();
				lrrGroupOverride.ModeID = modeID;
				lrrGroupOverride.SiteComponentPropertyID = siteCompPropID;
				lrrGroupOverride.SiteComponentID = siteCompID;

				//lrrGroupOverride.CurrentValue = iPropValue.GetPropertyValue();
				//Pre/Post Override Values
				lrrGroupOverride.PreOverrideValue = iPropValue.GetActualOverridedValue();
				lrrGroupOverride.PostOverrideValue = iPropValue.GetActualPropertyValue();

				lrrGroupOverride.UOM = iPropValue.GetUOMID();
				lrrGroupOverride.GroupPropertyID = groupPropID;
				lrrGroupOverride.SiteGroupPropertyQualifierValueID = iPropValue.GetID();
				lrrGroupOverride.QualifierID = qualifierID;
				
				//Assign Confidence Level
				string dataConfidenceLevelID = null;
				if(modeID == "1" || modeID == "2")
				{
					dataConfidenceLevelID = "3";
				}
				else
				{
					dataConfidenceLevelID = "1";
				}
				lrrGroupOverride.DataConfidenceLevelID = dataConfidenceLevelID;
				LRRModel.ChildModel.GroupOverrides.Add(lrrGroupOverride);
				
				iPropValue.DataConfidenceValueChanged+=new EventHandler(iPropValue_DataConfidenceValueChanged);
				iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
			}
			else
			{
				if(lrrModelOverride != null)
				{
					iPropValue.DataConfidenceValueChanged-=new EventHandler(iPropValue_DataConfidenceValueChanged);
					LRRModel.ChildModel.GroupOverrides.Remove(lrrModelOverride);
				}
			}
			//Update Overrides
//			_lrrApp.UpdateOverridePoints();
		}

		/// <summary>
		/// Method sets the fore/back/border colours for the grid cell
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="gsInfo"></param>
		private void SetColor(IPropertyValue iPropValue,GridStyleInfo gsInfo)
		{
			gsInfo.BackColor = iPropValue.GetBackColor();
			gsInfo.TextColor = iPropValue.GetForeColor();

			//Set Border Color and Border Thickness
			Color borderColor = iPropValue.GetBorderColor();
			int borderThickness = iPropValue.GetBorderThickness();
			if(borderThickness == 2)
            {	//gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid, borderColor,GridBorderWeight.Medium);
                GridBorder grdBorder = new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                gsInfo.Borders.Top = grdBorder;// new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                gsInfo.Borders.Left = grdBorder;// new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
            }
			else
				gsInfo.Borders.ResetAll();
			gsInfo.Font.Underline = iPropValue.GetUnderLine();
		}

		/// <summary>
		/// Sets State of the menu items
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="contMenu"></param>
		private void ProcessMenuItems(IPropertyValue iPropValue,ContextMenu contMenu)
		{
			bool overridedState = iPropValue.GetOverrideState();
			ErrorCode errorCode   = iPropValue.GetErrorCode();
			for(int i=0;i < contMenu.MenuItems.Count;i++)
			{
				MenuItem miItem = contMenu.MenuItems[i];
				switch(miItem.Index)
				{
					case 0:
						if(iPropValue != null)
							miItem.Enabled = iPropValue.GetIsTrended();
						else
							miItem.Enabled = false;
						break;
						//Override Values Menu
					case 1:
						miItem.Enabled = overridedState;
						break;
						//Uncertain Input Menu
					case 2:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.UNCERTAIN)) == (ErrorCode.UNCERTAIN) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
						//Trusted Input Menu
					case 3:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.TRUSTED)) == (ErrorCode.TRUSTED) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
						//Temp Trusted Input Menu
					case 4:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.TEMPTRUSTED)) == (ErrorCode.TEMPTRUSTED) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
						//Uncertain Override Menu
					case 5://3:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.UNCERTAIN)) == (ErrorCode.UNCERTAIN) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
						//Trusted Override Menu
					case 6://4:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.TRUSTED)) == (ErrorCode.TRUSTED) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
						//Temp Trusted Override Menu
					case 7://5:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.TEMPTRUSTED)) == (ErrorCode.TEMPTRUSTED) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
				}
			}
		}
		#endregion

		#region Constants
		private const int PROPERTIES_COL = 1;
		private const int PROPERTIES_ROW = 1;
		private const int MODES_ROW = 2;
		private const int COMPONENT_ROW = 1;
        private const int COMPONENT_COL = 2;
		private const int HEADER_ROW = 0;
		private const int MODE_SIZE = 100;
		private const int COL_SPAN = 5;
		private const int UOM_COL = 2;
		private const int UOM_ROW = 1;
		private const int DATA_COL_START = 3;
		private const int DATA_ROW_START = 3;

		private const int FREEZE_ROW_START = 0;
		private const int FREEZE_ROW_END = 1;
		private const int FREEZE_COL_START = 0;
		private const int FREEZE_COL_END= 1;

		private const int SUPPORTED_COLLECTION_COUNT = 2;
		#endregion

		#region Fields
		//Instance of App
		private LRRApp _lrrApp;

		//ArrayList for storing Panels
		private ArrayList _pnlCollection = new ArrayList();

		//Hash for stroing gridcontrol-gridManager key value pairs
		private Hashtable _gridGroupPropsManagerHash = new Hashtable();
		
		//Hash for Collection types
		private Hashtable _hashCollectionIndex = new Hashtable();
		
		//For saving Queries
		private GroupSearchCriteria _searchCriteria = new GroupSearchCriteria();

		//For UserSpecific UI UOM
		private LRRSearchCriteriaUserSpecificUIUOMs _searchCriteriaUserSpecficUIUOMs = new LRRSearchCriteriaUserSpecificUIUOMs();

        private GridPersistanceUtil _gridPersistance = new GridPersistanceUtil();

        //For Unselected Records
        private ArrayList _unSelectedRecords = new ArrayList();
		#endregion

		#region Properties
        /// <summary>
        /// Gets or sets the grid group props manager hash.
        /// </summary>
        /// <value>The grid group props manager hash.</value>
		private Hashtable GridGroupPropsManagerHash
		{
			get{return _gridGroupPropsManagerHash;}
			set
			{
				_gridGroupPropsManagerHash = value;
			}
		}
        /// <summary>
        /// Gets or sets the panel collection.
        /// </summary>
        /// <value>The panel collection.</value>
		private ArrayList PanelCollection
		{
			get{return _pnlCollection;}
			set
			{
				_pnlCollection = value;
			}
		}

        /// <summary>
        /// Gets or sets the un selected records.
        /// </summary>
        /// <value>The un selected records.</value>
        private ArrayList UnSelectedRecords
        {
            get { return _unSelectedRecords; }
            set { _unSelectedRecords = value; }
        }
		#endregion

		#region EventHandlers
		private void gridControlInstance_CurrentCellCloseDropDown(object sender, Syncfusion.Windows.Forms.PopupClosedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			gridControl.CurrentCell.ConfirmChanges();
			gridControl.Refresh();
		}

        /// <summary>
        /// Handles the CurrentCellValidating event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CurrentCellValidating(object sender, System.ComponentModel.CancelEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			//Get the Grid Data
			GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
			if(gridControlManager == null)
				return;

			//Check for UOM
			if(iCol == UOM_COL && iRow > UOM_ROW)
			{
				//Change the Property Values according to UOM
				changePropertyValueForUOM(gridControl,gridControlManager,iRow,iCol);
			}

			//Check for Property Value 
			if(iRow >= DATA_ROW_START && iCol >= DATA_COL_START)
			{
				bool result = changePropertyValue(gridControl,gridControlManager,iRow,iCol);
				if(!result)
				{
					gridControl.CurrentCell.ResetCurrentCellWithoutDeactivate();
					gridControl.CurrentCell.Refresh();
					e.Cancel = true;
				}
			}
		}

        /// <summary>
        /// Handles the CurrentCellAcceptedChanges event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        void gridControlInstance_CurrentCellAcceptedChanges(object sender, System.ComponentModel.CancelEventArgs e)
        {
            GridControl grid = sender as GridControl;
            if (!(grid[UOM_ROW, grid.CurrentCell.ColIndex].Text == IOSchemes.UOM_HEADER_NAME))
            {
                if (grid.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantDataArrayView))
                    GroupProperties.GroupedPlantDataArrayView.BPropertyValChanged = true;

                if (grid.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantPropertiesResultForm))
                    Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantPropertiesResultForm.BGroupedPlantPropertiesDataArrayResultFormChange = true;
            }
        }
        /// <summary>
        /// Handles the CurrentCellStartEditing event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CurrentCellStartEditing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridGroupPropsManagerHash[gridControl] == null)
			{
				e.Cancel = true;
				return;
			}
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

			if(iRow < DATA_ROW_START || iCol < DATA_COL_START)
				return;

			GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
			//Get the group Property Value Object
			object objPropValue = GetGroupPropertyValueObject(gridControlManager,iRow,iCol);
			if(objPropValue != null)
			{
				IPropertyValue iPropValue = objPropValue as IPropertyValue;
				if(!iPropValue.GetEditableState())
					e.Cancel = true;
			}
			else
			{
				e.Cancel = true;
			}
		}

        /// <summary>
        /// Handles the CellClick event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridCellClickEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CellClick(object sender, GridCellClickEventArgs e)
		{
			if(e.MouseEventArgs.Button == MouseButtons.Left
				&& e.RowIndex == 1 && (e.ColIndex == 1 || e.ColIndex == 2))
			{
				GridControl gridControl = sender as GridControl;
				gridControl.ResetVolatileData();
				gridControl.BeginUpdate();

				Cursor.Current = Cursors.WaitCursor;
				Cursor.Show();
				if(GridGroupPropsManagerHash[gridControl] != null)
				{
					GridGroupPropertiesManager gridGroupPropsManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
					SortCol(e.ColIndex,gridControl,gridGroupPropsManager);
					InitializeGroupPropertyValues(gridControl,gridGroupPropsManager);
				}
				gridControl.EndUpdate();
				Cursor.Hide();
			}
		}

        /// <summary>
        /// Handles the Popup event of the contMenu control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void contMenu_Popup(object sender, EventArgs e)
		{
			ContextMenu contMenu = sender as ContextMenu;
			contMenu.MenuItems.Clear();

			GridControl gridControl = contMenu.SourceControl as GridControl;
			if(gridControl == null)
				return;
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

            if (iRow >= DATA_ROW_START && iCol == COMPONENT_COL)
			{
				MenuItem miItem = new MenuItem(LRRHeaderText.GetHeaderString("SID_RECORDE_PROP_VALUE"),new EventHandler(recordValue));
				//				GetObjectData(gridControlManager,iRow + DATA_ROW_START - 1,iCol,out colData,out rowData);
				//				miItem.Checked = rowData.IsRecorded;
				contMenu.MenuItems.Add(miItem);

				MenuItem miItemTrend = new MenuItem(LRRHeaderText.GetHeaderString("SID_TREND_PROP_VALUE"),new EventHandler(trendValue));
				contMenu.MenuItems.Add(miItemTrend);
			}

			if((iRow >= DATA_ROW_START) && (iCol >= DATA_COL_START))
			{
				GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
				object objPropValue = GetGroupPropertyValueObject(gridControlManager,iRow,iCol);
				if(objPropValue != null)
				{
					IPropertyValue iPropValue = objPropValue as IPropertyValue;
					

					// Trend Menu
					MenuItem miItemTrend = new MenuItem(LRRHeaderText.GetHeaderString("SID_VIEW_TREND"));
					//miItemTrend.IsParent = true;
					foreach(string schemeName in UserControls.UserControlTrendOptions.HtSchemes.Keys)
					{
						MenuItem miItemTrendChild = new MenuItem(schemeName,new EventHandler(viewTrend));
						miItemTrend.MenuItems.Add(miItemTrendChild);
					}
					miItemTrend.Index = 0;//6;
					contMenu.MenuItems.Add(miItemTrend);

					if(!iPropValue.GetEditableState())
					{
						ProcessMenuItems(iPropValue,contMenu);
						return;
					}

					//Override Values Menu
					MenuItem miItem = new MenuItem(LRRHeaderText.GetHeaderString("SID_OVERRIDE_VAL_OFF"),new EventHandler(overrideValue));
					miItem.Index = 1;
					contMenu.MenuItems.Add(miItem);

					//Uncertain Input Menu
					MenuItem miItemUncertainInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_UNCERTAIN_INPUT"),new EventHandler(overrideValue));
					miItemUncertainInput.Index = 2;
					contMenu.MenuItems.Add(miItemUncertainInput);

					//Trusted Input Menu
					MenuItem miItemTrustedInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_TRUSTED_INPUT"),new EventHandler(overrideValue));
					miItemTrustedInput.Index = 3;
					contMenu.MenuItems.Add(miItemTrustedInput);

					//TempTrusted Input Menu
					MenuItem miItemTempTrustedInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_TEMP_TRUSTER_INPUT"),new EventHandler(overrideValue));
					miItemTempTrustedInput.Index = 4;
					contMenu.MenuItems.Add(miItemTempTrustedInput);

					//Uncertain Override Menu
					MenuItem miItemUncertainOverride = new MenuItem(LRRHeaderText.GetHeaderString("SID_UNCERTAIN_OVERRIDE"),new EventHandler(overrideValue));
					miItemUncertainOverride.Index = 5;//3;
					contMenu.MenuItems.Add(miItemUncertainOverride);

					//Trusted Override Menu
					MenuItem miItemTrustedOverride = new MenuItem(LRRHeaderText.GetHeaderString("SID_TRUSTED_OVERRIDE"),new EventHandler(overrideValue));
					miItemTrustedOverride.Index = 6;//4;
					contMenu.MenuItems.Add(miItemTrustedOverride);

					//TempTrusted Override Menu
					MenuItem miItemTempTrusted = new MenuItem(LRRHeaderText.GetHeaderString("SID_TEMP_TRUSTER_OVERRIDE"),new EventHandler(overrideValue));
					miItemTempTrusted.Index = 7;//5;
					contMenu.MenuItems.Add(miItemTempTrusted);

					ProcessMenuItems(iPropValue,contMenu);
				}

				MenuItem miOpenItem = new MenuItem(LRRHeaderText.GetHeaderString("SID_OPEN_PLANT_GRP_QUALIFIER_VALUES"),new EventHandler(OpenSiteComponentGroupProperty));
				miOpenItem.Index = 8;
				contMenu.MenuItems.Add(miOpenItem);
			}
		}

		public static ResultComponentsWindowGroupProp rsComponentsTrend = new ResultComponentsWindowGroupProp();
		bool isTrendEventSet = false;
        /// <summary>
        /// Trends the value.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		public void trendValue(object sender, EventArgs e)
		{
            MenuItem miItem = sender as MenuItem;
            GridControl gridControl = miItem.Parent.GetContextMenu().SourceControl as GridControl;
            if (GridGroupPropsManagerHash[gridControl] == null)
                return;

            int iRow = gridControl.CurrentCell.RowIndex;
            int iCol = gridControl.CurrentCell.ColIndex;

            GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
            //Get the Col Data from Grid Data Manager
            ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;
            ArrayList rowHeaderCellList = gridControlManager.RowHeaderCellList;
            LRRSiteComponents lsc = new LRRSiteComponents();
            LRRSiteComponentProperties lscp = new LRRSiteComponentProperties();
            int rowCount = gridControl.RowCount;
            ArrayList propertyId = new ArrayList();
            for (int i = DATA_ROW_START; i <= rowCount; i++)
            {
                GroupRowHeaderCell CompRowData = rowHeaderCellList[i - DATA_ROW_START] as GroupRowHeaderCell;
                string compRowCellID = CompRowData.ID;
                string compRowCellPropID = CompRowData.PropertyID;
                object Compobj = LRRModel.SiteComponentsObj.GetSiteComponentByID(compRowCellID);
                LRRSiteComponentProperty CompPropObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(compRowCellPropID);
                if (Compobj != null)
                {
                    if (!lsc.Contains(Compobj) && Compobj.GetType() == typeof(LRRSiteComponent))
                    {
                        lsc.Add(Compobj);
                    }

                }
                if (CompPropObj != null)
                {
                    if (!lscp.Contains(CompPropObj) && CompPropObj.GetType() == typeof(LRRSiteComponentProperty))
                    {
                        if (CompPropObj.PropertyID != null)
                        {
                            if (!propertyId.Contains(CompPropObj.PropertyID))
                            {
                                propertyId.Add(CompPropObj.PropertyID);
                                lscp.Add(CompPropObj);
                            }
                        }
                        else
                        {
                            lscp.Add(CompPropObj);
                        }
                    }
                }
            }

            LRRLookUpProperties llups = new LRRLookUpProperties();
            int colCount = gridControl.ColCount;
            for (int i = DATA_COL_START; i <= colCount; i++)
            {
                GroupColHeaderCell propColData = colHeaderCellList[i - DATA_COL_START] as GroupColHeaderCell;
                string compRowCellID = propColData.ID;
                object Compobj = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(compRowCellID);
                if (Compobj != null)
                {
                    if (!llups.Contains(Compobj) && Compobj.GetType() == typeof(LRRLookUpProperty))
                    {
                        llups.Add(Compobj);
                    }
                }
            }
			rsComponentsTrend.PropertyCollection = lscp;
			rsComponentsTrend.SiteComponents = lsc;
			rsComponentsTrend.LookUpProperties = llups;
			rsComponentsTrend.Text = "Trended Components Window :";
			if(!isTrendEventSet)
			{
				rsComponentsTrend.RecordedProperties+=new Symcon.LightRidge.PEAdvisor.GroupProperties.ResultComponentsWindowGroupProp.SelectedPropertiesChanged(rsComponents_TrendedProperties);
				isTrendEventSet = true;
			}
			//rsComponentsTrend.RecordedProperties+=new Symcon.LightRidge.PEAdvisor.IOScreen.ResultComponentsWindow.SelectedPropertiesChanged(rsComponents_TrendedProperties);
			rsComponentsTrend.BindGrid();
			rsComponentsTrend.TopMost = false;
			rsComponentsTrend.Owner = _lrrApp.MainFrame;
			rsComponentsTrend.Show();
		}

        /// <summary>
        /// Rses the components_ trended properties.
        /// </summary>
        /// <param name="e">The e.</param>
		private void rsComponents_TrendedProperties(RecordingEventArgsGroupProperties e)
		{
			for(int i =0 ;i < ((RecordingEventArgs)e).SelectedSiteComponents.Count;i++)
			{
				
				IComponent iComp = ((RecordingEventArgs)e).SelectedSiteComponents[i] as IComponent;  
				CustomCollection iPropCollection = iComp.GetProperties(); 
				IEnumerator iEnum = iPropCollection.GetEnumerator();
				LRRSiteComponentProperties lrrSCPToSave = new LRRSiteComponentProperties();
				while(iEnum.MoveNext())
				{
					IProperty iPropObj  = iEnum.Current as IProperty;
					string iPropID = iPropObj.GetMasterPropertyID(); 
					if(((RecordingEventArgs)e).SelectedProperties.Contains(iPropID)) 
					{
						CustomCollection iGroupPropCollection = iPropObj.GetGroupProperties(); 
						IEnumerator iEnumGP = iGroupPropCollection.GetEnumerator();
						LRRLookUpProperties lrrSCGPToSave = new LRRLookUpProperties();
						while(iEnumGP.MoveNext())
						{
							LRRSiteComponentGroupProperty gPropObj = iEnumGP.Current as LRRSiteComponentGroupProperty;
							string iPropertyValueID = gPropObj.LookUpPropertiesID;
							if(e.SelectedLookUpProperties.Contains(iPropertyValueID))
							{
								foreach(LRRMode mode in LRRModel.ProjectModes)
								{
									string modeId = mode.ModeId;
									LRRSiteGroupPropertyQualifierValue qalValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(gPropObj.GroupPropertyID,"",modeId);
									if(qalValue != null)
									{
										qalValue.IsTrended = true;
									}
								}
							}
							if(e.UnSelectedLookUpProperties.Contains(iPropertyValueID))
							{
								foreach(LRRMode mode in LRRModel.ProjectModes)
								{
									string modeId = mode.ModeId;
									LRRSiteGroupPropertyQualifierValue qalValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(gPropObj.GroupPropertyID,"",modeId);
									if(qalValue != null)
									{
										qalValue.IsTrended = false;
									}
								}
							}
						}
					}
				}
				_lrrApp.DBManagerObj.ExecuteBatchUpdate(lrrSCPToSave);
			}
		}

		public static ResultComponentsWindowGroupProp rsComponentsRecord = new ResultComponentsWindowGroupProp();
		bool isRecordEventSet = false;
        /// <summary>
        /// Records the value.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		public void recordValue(object sender, EventArgs e)
		{
			MenuItem miItem = sender as MenuItem;
			GridControl gridControl = miItem.Parent.GetContextMenu().SourceControl as GridControl;
			if(GridGroupPropsManagerHash[gridControl] == null)
				return;

			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

			GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			ArrayList     rowHeaderCellList = gridControlManager.RowHeaderCellList;
			LRRSiteComponents lsc = new LRRSiteComponents();
			LRRSiteComponentProperties lscp = new LRRSiteComponentProperties();
			int rowCount = gridControl.RowCount;
			ArrayList propertyId = new ArrayList();
			for(int i=DATA_ROW_START;i<=rowCount;i++)
			{
				GroupRowHeaderCell CompRowData = rowHeaderCellList[i - DATA_ROW_START] as GroupRowHeaderCell;
				string compRowCellID = CompRowData.ID;
				string compRowCellPropID = CompRowData.PropertyID;
				object Compobj = LRRModel.SiteComponentsObj.GetSiteComponentByID(compRowCellID);
				LRRSiteComponentProperty CompPropObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(compRowCellPropID);
				if(Compobj != null)
				{
					if(!lsc.Contains(Compobj) && Compobj.GetType()==typeof(LRRSiteComponent))
					{
						lsc.Add(Compobj);
					}
					
				}
				if(CompPropObj != null)
				{
					if(!lscp.Contains(CompPropObj) && CompPropObj.GetType()==typeof(LRRSiteComponentProperty))
					{
						if(CompPropObj.PropertyID != null)
						{
							if(!propertyId.Contains(CompPropObj.PropertyID))
							{
								propertyId.Add(CompPropObj.PropertyID); 
								lscp.Add(CompPropObj);
							}
						}
						else
						{
							lscp.Add(CompPropObj);
						}
					}
				}
			}

			LRRLookUpProperties llups = new LRRLookUpProperties();
			int colCount = gridControl.ColCount;
			for(int i=DATA_COL_START;i<=colCount;i++)
			{
				GroupColHeaderCell propColData = colHeaderCellList[i-DATA_COL_START] as GroupColHeaderCell;
				string compRowCellID = propColData.ID;
				object Compobj = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(compRowCellID);
				if(Compobj != null)
				{
					if(!llups.Contains(Compobj) && Compobj.GetType()==typeof(LRRLookUpProperty))
					{
						llups.Add(Compobj);
					}
				}
			}
			rsComponentsRecord.PropertyCollection = lscp;
			rsComponentsRecord.SiteComponents = lsc;
			rsComponentsRecord.LookUpProperties = llups;
			rsComponentsRecord.Text = "Recorded Components Window :";
			if(!isRecordEventSet)
			{
				rsComponentsRecord.RecordedProperties+=new Symcon.LightRidge.PEAdvisor.GroupProperties.ResultComponentsWindowGroupProp.SelectedPropertiesChanged(rsComponents_RecordedProperties);
				isRecordEventSet = true;
			}
			//rsComponentsTrend.RecordedProperties+=new Symcon.LightRidge.PEAdvisor.IOScreen.ResultComponentsWindow.SelectedPropertiesChanged(rsComponents_TrendedProperties);
			rsComponentsRecord.BindGrid();
			rsComponentsRecord.TopMost = false;
			rsComponentsRecord.Owner = _lrrApp.MainFrame;
			rsComponentsRecord.Show();
		}

        /// <summary>
        /// Rses the components_ recorded properties.
        /// </summary>
        /// <param name="e">The e.</param>
		private void rsComponents_RecordedProperties(RecordingEventArgsGroupProperties e)
		{
			for(int i =0 ;i < ((RecordingEventArgs)e).SelectedSiteComponents.Count;i++)
			{
				
				IComponent iComp = ((RecordingEventArgs)e).SelectedSiteComponents[i] as IComponent;  
				CustomCollection iPropCollection = iComp.GetProperties(); 
				IEnumerator iEnum = iPropCollection.GetEnumerator();
				LRRSiteComponentProperties lrrSCPToSave = new LRRSiteComponentProperties();
				while(iEnum.MoveNext())
				{
					IProperty iPropObj  = iEnum.Current as IProperty;
					string iPropID = iPropObj.GetMasterPropertyID(); 
					if(((RecordingEventArgs)e).SelectedProperties.Contains(iPropID)) 
					{
						CustomCollection iGroupPropCollection = iPropObj.GetGroupProperties(); 
						IEnumerator iEnumGP = iGroupPropCollection.GetEnumerator();
						LRRLookUpProperties lrrSCGPToSave = new LRRLookUpProperties();
						while(iEnumGP.MoveNext())
						{
							LRRSiteComponentGroupProperty gPropObj = iEnumGP.Current as LRRSiteComponentGroupProperty;
							string iPropertyValueID = gPropObj.LookUpPropertiesID;
							if(e.SelectedLookUpProperties.Contains(iPropertyValueID))
							{
								foreach(LRRMode mode in LRRModel.ProjectModes)
								{
									string modeId = mode.ModeId;
									LRRSiteGroupPropertyQualifierValue qalValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(gPropObj.GroupPropertyID,"",modeId);
									if(qalValue != null)
									{
										qalValue.IsTracked = true;
									}
								}
							}
							if(e.UnSelectedLookUpProperties.Contains(iPropertyValueID))
							{
								foreach(LRRMode mode in LRRModel.ProjectModes)
								{
									string modeId = mode.ModeId;
									LRRSiteGroupPropertyQualifierValue qalValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(gPropObj.GroupPropertyID,"",modeId);
									if(qalValue != null)
									{
										qalValue.IsTracked = false;
									}
								}
							}
						}
					}
				}
				_lrrApp.DBManagerObj.ExecuteBatchUpdate(lrrSCPToSave);
			}
		}

        /// <summary>
        /// Views the trend.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		public void viewTrend(object sender, EventArgs e)
		{
			MenuItem miItemChild = sender as MenuItem;
			string trendScheme = miItemChild.Text;
			MenuItem miItem = miItemChild.Parent as MenuItem;
			ContextMenu contMenu = miItem.GetContextMenu();
			GridControl gridControl = contMenu.SourceControl as GridControl;

			if(GridGroupPropsManagerHash[gridControl] == null)
				return;

			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;

			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.RowHeaderCellList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the row data
			GroupRowHeaderCell rowData = rowSortList[iRow - DATA_ROW_START] as GroupRowHeaderCell;
			string lookUpPropID = rowData.ID;
			
			//Get the Qualifier ID
			string qualifierID = gridControlManager.QualifierID;

			//Get the column data
			GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;
			string modeID   = colData.ModeID;
			string siteCompPropID = colData.PropertyID;
			string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);
			string uiUOMID = colData.UIUOMId;

			object objPropValue = GetGroupPropertyValueObject(gridControlManager,iRow,iCol);

			if(objPropValue != null)
			{
				IPropertyValue iPropValue =  objPropValue as IPropertyValue;
				_lrrApp.LRRUIManagerObj.ShowTrend(trendScheme,iPropValue,uiUOMID);
			}
		}

        /// <summary>
        /// Overrides the value.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void overrideValue(object sender, EventArgs e)
		{
			MenuItem miItem = sender as MenuItem;
			ContextMenu contMenu = miItem.GetContextMenu();
			GridControl gridControl = contMenu.SourceControl as GridControl;

			if(GridGroupPropsManagerHash[gridControl] == null)
				return;

			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;

			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.RowHeaderCellList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the row data
			GroupRowHeaderCell rowData = rowSortList[iRow - DATA_ROW_START] as GroupRowHeaderCell;
			
            string siteCompPropID = rowData.PropertyID;
            string uiUOMID = rowData.UIUOMId;
			//Get the Qualifier ID
			string qualifierID = gridControlManager.QualifierID;

			//Get the column data
			GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;
			string modeID   = colData.ModeID;
			string lookUpPropID =colData.ID;
			string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);
			

			object objPropValue = GetGroupPropertyValueObject(gridControlManager,iRow,iCol);

			if(objPropValue != null)
			{
				IPropertyValue iPropValue =  objPropValue as IPropertyValue;
				//Check for Menu item Index
				switch(miItem.Index)
				{
						//Override Value On/Off
					case 1:
						iPropValue.SetOverrideState(false);
						if(colData.ModeID == "1")
						{
							_lrrApp.SwapGroupPropBindings(objPropValue,true);
							iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
						}
						else
						{
							_lrrApp.RemoveCascadedBindings(objPropValue,groupPropID,lookUpPropID,qualifierID,false);
							iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
						}

						//Populate Overrides Collection
						PopulateGroupOverrides(iPropValue,groupPropID,qualifierID);
						
						//Set the Cell tool Tip
						SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],uiUOMID);
						break;
						//Uncertain Input
					case 2:
						iPropValue.SetTrustedState("0");
						break;
						//Trusted Input
					case 3:
						iPropValue.SetTrustedState("1");
						break;
						//Temp Trusted Input
					case 4:
						iPropValue.SetTrustedState("2");
						break;
						//Uncertain Override
					case 5://3:
						iPropValue.SetDataConfidenceLevelID("3");
						iPropValue.SetTrustedState("-1");
						break;
						//Trusted Override
					case 6://4:
						iPropValue.SetDataConfidenceLevelID("1");
						iPropValue.SetTrustedState("-1");
						break;
						//TempTrusted Override
					case 7://5:
						iPropValue.SetDataConfidenceLevelID("2");
						iPropValue.SetTrustedState("-1");
						break;
				}

				ProcessMenuItems(iPropValue,contMenu);
				//Set the Color
				SetColor(iPropValue,gridControl.Model[iRow,iCol]);
			}
		}

        /// <summary>
        /// Handles the DataConfidenceValueChanged event of the iPropValue control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void iPropValue_DataConfidenceValueChanged(object sender, EventArgs e)
		{
			IPropertyValue iPropValue = sender as IPropertyValue;
			string qualifierID = iPropValue.GetID();
			LRRGroupOverride lrrModelOverride = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(qualifierID);
			if(lrrModelOverride != null)
			{
				lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();		
				//				_lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);   
			}


			//Update Overrides
			//			_lrrApp.UpdateOverridePoints();
		}

        /// <summary>
        /// Opens the site component group property.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void OpenSiteComponentGroupProperty(object sender,EventArgs e)
		{
			MenuItem miItem = sender as MenuItem;
			GridControl gridControl = miItem.Parent.GetContextMenu().SourceControl as GridControl;

			if(GridGroupPropsManagerHash[gridControl] == null)
				return;

			int iRowIndex = gridControl.CurrentCell.RowIndex;
			int iColIndex = gridControl.CurrentCell.ColIndex;

			LRRSiteGroupPropertyQualifierValues objValues = new LRRSiteGroupPropertyQualifierValues();
			GridGroupPropertiesManager gridGrpPropsManager =  GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
			object objValue = GetGroupPropertyValueObject(gridGrpPropsManager,iRowIndex,iColIndex);
			
			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridGrpPropsManager.RowHeaderCellList;
			//Get the row data
			GroupRowHeaderCell rowData = rowSortList[iRowIndex - DATA_ROW_START] as GroupRowHeaderCell;
			string lookUpPropID = rowData.ID;

			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridGrpPropsManager.ColHeaderCellList;
			//Get the column data
			GroupColHeaderCell colData = colHeaderCellList[iColIndex - DATA_COL_START] as GroupColHeaderCell;
			string modeID   = colData.ModeID;
			string siteCompPropID = colData.PropertyID;

			if((objValue == null) ||(modeID == ""))
			{
				LRRMessagBox.ShowMessage("SID_QUALIFIER_NOTEXISTS","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Exclamation);    
				return;
			}
			objValues.Add(objValue);

			SiteComponentGroupProperty grpPropFrm = new SiteComponentGroupProperty(objValues);
			grpPropFrm.Text = grpPropFrm.Text + " - Mode: " + LRRModel.Modes.GetModeByID(modeID).ModeName;
			LRRFormulaDataStore.ModeID = modeID;
			
			//NidhiG GP Formula Implementation
			LRRFormulaDataStore.ShowGroupedAttributes = true;
			LRRSiteGroupPropertyQualifierValue objQValue = objValue as LRRSiteGroupPropertyQualifierValue;
			string sQualifierValueID = objQValue.SiteGroupPropertyQualifierValueID;

			if(LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sQualifierValueID) != null)
			{
				LRRFormulaDataStore.SourceObjectID= LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sQualifierValueID).SiteGroupPropertyQualifierValueID;
				LRRFormulaDataStore.SourcePropertyID= LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sQualifierValueID).GroupPropertyID;
				LRRFormulaDataStore.SourceLookupPropertyID= LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sQualifierValueID).LookUpPropertiesID;
				LRRFormulaDataStore.SourceLookupQualifierID= LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sQualifierValueID).LookUpQualifierID == null ? "" : LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sQualifierValueID).LookUpQualifierID;
			}
					

//			string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompPropID,lookUpPropID);
//			LRRFormulaDataStore.SourcePropertyID = groupPropID;
			grpPropFrm.ShowDialog();
		}

        /// <summary>
        /// Bindings the manager_ binding called.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="target">The target.</param>
		private void BindingManager_BindingCalled(object source, object target)
		{
			object src = source;
			object trg = target;
			GridStyleInfo gsInfo = target as GridStyleInfo;
			GridControl gridControl = gsInfo.GetActiveGridView() as GridControl;

			int iRow = gsInfo.CellIdentity.RowIndex;
			int iCol = gsInfo.CellIdentity.ColIndex;

			//Get the Grid Data
			if(GridGroupPropsManagerHash[gridControl] != null)
			{
				GridGroupPropertiesManager gridGrpPropsManager =  GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
				//Get the Row Data from Grid Data Manager
				ArrayList     rowHeaderCellList = gridGrpPropsManager.RowHeaderCellList;
				//Get the Col Data from Grid Data Manager
				ArrayList     colHeaderCellList = gridGrpPropsManager.ColHeaderCellList;

				//Get the column data
				GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;
				string siteCompPropID = colData.PropertyID;
				string uiUOMID = colData.UIUOMId;

				//Get the UOMs
				string uomID = null;
				//Get the Property Object
				object objValue = GetGroupPropertyValueObject(gridGrpPropsManager,iRow,iCol);
				if(objValue != null)
				{
					IPropertyValue iPropValue = objValue as IPropertyValue;
					//Convert property value according to UIUOM 
					string propertyValue = iPropValue.GetPropertyValue();
					uomID = iPropValue.GetUOMID();
					
					if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
					{
						//propertyValue = ConvertUOM(uomID,uiUOMID,propertyValue);
						propertyValue =  iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,propertyValue);
					}

                    string lrrDataType = iPropValue.GetDataType();
                    if (lrrDataType == null || lrrDataType.Trim() == "")
                    {
                        propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                    }
                    gsInfo.Text = propertyValue;

					//Set Colour
					SetColor(iPropValue,gridControl.Model[iRow,iCol]);

					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],uiUOMID);
				}
			}
		}

		private void gridControlInstance_ResizingColumns(object sender, GridResizingColumnsEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			bool result = GroupedViewUtilities.StopResizeHiddenColumns(gridControl,e.Columns.Left);
			if(result)
				e.Cancel = true;
		}

        /// <summary>
        /// Handles the CurrentCellKeyDown event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CurrentCellKeyDown(object sender, KeyEventArgs e)
		{
            GridControl gridControl = sender as GridControl;
            int iCol = gridControl.CurrentCell.ColIndex;

            Syncfusion.Windows.Forms.Grid.GridRangeInfoList rangeInfoList = new GridRangeInfoList();
            gridControl.Selections.GetSelectedRanges(out rangeInfoList, true);
            int iRow = rangeInfoList.ActiveRange.Top;

            if (((iRow < DATA_ROW_START || iCol < DATA_COL_START) && ((e.KeyCode == Keys.Delete || e.KeyCode == Keys.Back || (e.Control && e.KeyCode == Keys.V)))))
            {
                e.Handled = true;
                return;
            }
			//GroupedViewUtilities.HandleUOMEditingKeyDown(sender,e,UOM_ROW);
		}

        /// <summary>
        /// Handles the CurrentCellKeyPress event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyPressEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_CurrentCellKeyPress(object sender, KeyPressEventArgs e)
		{
			GroupedViewUtilities.HandleUOMEditingKeyPress(sender,e,UOM_ROW);
		}

        /// <summary>
        /// Handles the ForeColorChanged event of the gridManager control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Symcon.LightRidge.PEAdvisor.GroupProperties.ColorEventArgs"/> instance containing the event data.</param>
		private void gridManager_ForeColorChanged(object sender, ColorEventArgs e)
		{
			GridGroupPropertiesManager gridControlManager = sender as GridGroupPropertiesManager;
			IDictionaryEnumerator iDictEnum = GridGroupPropsManagerHash.GetEnumerator() as IDictionaryEnumerator;
			while(iDictEnum.MoveNext())
			{
				GridGroupPropertiesManager gcManager = iDictEnum.Value as GridGroupPropertiesManager;
				if(gcManager == gridControlManager)
				{
					GridControl gridControl = iDictEnum.Key as GridControl;
					IPropertyValue iPropValue = e.PropValueObj as IPropertyValue;
					gridControl.Model[e.Row,e.Col].TextColor = iPropValue.GetForeColor();
					break;
				}
			}
		}

        /// <summary>
        /// Handles the BackColorChanged event of the gridManager control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Symcon.LightRidge.PEAdvisor.GroupProperties.ColorEventArgs"/> instance containing the event data.</param>
		private void gridManager_BackColorChanged(object sender, ColorEventArgs e)
		{
			GridGroupPropertiesManager gridControlManager = sender as GridGroupPropertiesManager;
			IDictionaryEnumerator iDictEnum = GridGroupPropsManagerHash.GetEnumerator() as IDictionaryEnumerator;
			while(iDictEnum.MoveNext())
			{
				GridGroupPropertiesManager gcManager = iDictEnum.Value as GridGroupPropertiesManager;
				if(gcManager == gridControlManager)
				{
					GridControl gridControl = iDictEnum.Key as GridControl;
					IPropertyValue iPropValue = e.PropValueObj as IPropertyValue;
					gridControl.Model[e.Row,e.Col].BackColor = iPropValue.GetBackColor();
					break;
				}
			}
		}

        /// <summary>
        /// Handles the Paint event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.PaintEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_Paint(object sender, PaintEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridGroupPropsManagerHash[gridControl] != null)
			{
				GridGroupPropertiesManager gcManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
				GridStyleInfo styleInfo = gridControl.BaseStylesMap["Column Header"].StyleInfo;
				int x = gridControl.RowIndexToVScrollPixelPos(HEADER_ROW);
				int y = gridControl.ColIndexToHScrollPixelPos(PROPERTIES_COL - 1);
				int height = gridControl.RowHeights.GetSize(HEADER_ROW);
				StringFormat sFormat = new StringFormat();
				sFormat.Alignment = StringAlignment.Near;
				sFormat.LineAlignment = StringAlignment.Center;
				Graphics g = e.Graphics;
				int width = gridControl.Width;
				g.DrawString(gcManager.HeaderText,styleInfo.GdipFont,new SolidBrush(styleInfo.TextColor),new RectangleF(x,y,width,height),sFormat);
			}
		}

        /// <summary>
        /// Handles the ReconnectCompleted event of the _lrrApp control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void _lrrApp_ReconnectCompleted(object sender, EventArgs e)
		{
			IDictionaryEnumerator iDictEnum = GridGroupPropsManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				GridControl  gridControlInstance = (GridControl)iDictEnum.Key;
				GridGroupPropertiesManager  gridControlManager = (GridGroupPropertiesManager)iDictEnum.Value;
				InitializeGroupPropertyValues(gridControlInstance,gridControlManager);
			}
		}

		private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
		{
			//GroupedViewUtilities.ResizeToFit(GridGroupPropsManagerHash,DATA_ROW_START,DATA_COL_START);
		}

        /// <summary>
        /// Handles the RowsMoving event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridRangeMovingEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_RowsMoving(object sender, GridRangeMovingEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(e.Target < DATA_ROW_START || e.From < DATA_ROW_START)
			{
				e.Cancel = true;
				return;
			}
			//			for(int i=1;i<=gridControl.RowCount;i++)
			//			{
			//				if(gridControl[0,i].Text.Trim() != "")
			//				{
			//					GridPersistanceUtil._arrGridRowMove[0].Add(gridControl[0,i].Text);
			//					break;
			//				}
			//			}
			//			GridPersistanceUtil._arrGridRowMove[1].Add(e.From);
			//			GridPersistanceUtil._arrGridRowMove[2].Add(e.Count);
			//			GridPersistanceUtil._arrGridRowMove[3].Add(e.Target);
            _gridPersistance._arrGridRowMove[0].Add(e.From);
            _gridPersistance._arrGridRowMove[1].Add(e.Count);
            _gridPersistance._arrGridRowMove[2].Add(e.Target);
			if(GridGroupPropsManagerHash[gridControl] != null)
			{
				GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
				GroupedViewUtilities.RowMovement(e.From,e.Target,e.Count,gridControlManager.RowHeaderCellList,DATA_ROW_START);
			}
		}

        /// <summary>
        /// Handles the RowsMoved event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridRangeMovedEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_RowsMoved(object sender, GridRangeMovedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridGroupPropsManagerHash[gridControl] != null)
			{
				GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
				InitializeGroupPropertyValues(gridControl,gridControlManager);
				if(gridControl.CurrentCell.IsEditing)
					gridControl.CurrentCell.EndEdit();
				//turn off the sort arrow
				for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				{
					if(gridControl[1, col1].CellType == "ColumnHeaderCell")
					{
						gridControl[1, col1].CellType = "Header";
						break;
					}
				}
			}
		}

        /// <summary>
        /// Handles the SelectionChanged event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridSelectionChangedEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_SelectionChanged(object sender, GridSelectionChangedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(e.Range.Top == HEADER_ROW && e.Range.Left >= DATA_COL_START)
			{
				if(e.Reason != GridSelectionReason.MouseMove)
					return;
				int iModes = 0;

				//Get the Selected Modes
				ArrayList arrModesSelected = new ArrayList();
				foreach(LRRMode mode in LRRModel.ProjectModes)
				{
					if(mode.ViewSelection)
					{
						iModes++;
						arrModesSelected.Add(mode);
					}
				}
				
				string firstSelModeID = null;
				string lastSelModeID  = null;
				if(arrModesSelected.Count > 0)
				{
					LRRMode firstSelMode = arrModesSelected[0] as LRRMode;
					LRRMode lastSelMode  = arrModesSelected[arrModesSelected.Count - 1] as LRRMode;

					firstSelModeID = firstSelMode.ModeId;
					lastSelModeID = lastSelMode.ModeId;
				}
				
				//Check for Column Movement
				if(GridGroupPropsManagerHash[gridControl] != null)
				{
					GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
					ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;

					GroupColHeaderCell firstColHdrCell = colHeaderCellList[e.Range.Left - DATA_COL_START] as GroupColHeaderCell;
					GroupColHeaderCell lastColHdrCell = colHeaderCellList[e.Range.Right - DATA_COL_START] as GroupColHeaderCell;

					if(firstColHdrCell.ModeID == firstSelModeID && lastColHdrCell.ModeID == lastSelModeID)
					{
						LRRMode firstMode = LRRModel.ProjectModes[0];
						LRRMode lastMode = LRRModel.ProjectModes[LRRModel.ProjectModes.Count - 1];
						
						int leftIncr = Int32.Parse(firstSelModeID,Symcon.BusinessObjects.SymConverter.Culture) - Int32.Parse(firstMode.ModeId,Symcon.BusinessObjects.SymConverter.Culture);
						int rightIncr = Int32.Parse(lastMode.ModeId,Symcon.BusinessObjects.SymConverter.Culture ) - Int32.Parse(lastSelModeID,Symcon.BusinessObjects.SymConverter.Culture);
						int leftIndex = e.Range.Left - leftIncr;
						int rightIndex = e.Range.Right + rightIncr;
						
						gridControl.Selections.ChangeSelection(gridControl.Selections.Ranges.ActiveRange,GridRangeInfo.Cells(HEADER_ROW,leftIndex,gridControl.RowCount,rightIndex),false);

						_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = true;
						_lrrApp.MainFrame.BarItemRightColMovement.Enabled = true;	

						return;
					}
					else
					{
						_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = false;
						_lrrApp.MainFrame.BarItemRightColMovement.Enabled = false;
					}
				}
			}
			else
			{
				_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = false;
				_lrrApp.MainFrame.BarItemRightColMovement.Enabled = false;
			}
		}

        /// <summary>
        /// Handles the ColsMoving event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridRangeMovingEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_ColsMoving(object sender, GridRangeMovingEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridGroupPropsManagerHash[gridControl] != null)
			{
				GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
				GroupedViewUtilities.ColMovement(e.From,e.Target,e.Count,gridControlManager.ColHeaderCellList,DATA_COL_START);
			}
            _gridPersistance._arrGridColMove[0].Add(e.From);
            _gridPersistance._arrGridColMove[1].Add(e.Count);
            _gridPersistance._arrGridColMove[2].Add(e.Target);
		}

        private void Model_ClipboardPaste(object sender, Syncfusion.Windows.Forms.Grid.GridCutPasteEventArgs e)
        {
            GridModel gridControlModel = sender as GridModel;
            gridControlModel.CurrentCellInfo.GridView.CurrentCell.EndEdit();

            GridControl gridControl = gridControlModel.CurrentCellInfo.GridView as GridControl;

            if (GridGroupPropsManagerHash[gridControl] == null)
            {
                return;
            }

            GridGroupPropertiesManager gridControlManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;

            IDataObject data = Clipboard.GetDataObject();

            if (data.GetDataPresent(typeof(string)))
            {
                string s = (string)data.GetData(typeof(string));

                string[] sRows = s.Split(new string[] { "\r\n" }, StringSplitOptions.None);

                GridRangeInfoList rangeList;
                if (gridControl.Selections.GetSelectedRanges(out rangeList, true))
                {
                    foreach (GridRangeInfo range in rangeList)
                    {
                        int sRowIndex = 0;
                        for (int iRow = range.Top; iRow <= range.Bottom; iRow++)
                        {
                            if (sRowIndex < sRows.Length)
                            {
                                string sRow = sRows[sRowIndex++];
                                int sCellInRowIndex = 0;
                                string[] sCells = sRow.Split(new char[] { '\t' });
                                for (int iCol = range.Left; iCol <= range.Right; iCol++)
                                {
                                    if (sCellInRowIndex < sCells.Length)
                                    {
                                        string sCell = sCells[sCellInRowIndex++];

                                        if (iRow < DATA_ROW_START || iCol < DATA_COL_START)
                                            continue;

                                        //Get the Col Data from Grid Data Manager
                                        ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;

                                        //Get the column data
                                        GroupColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as GroupColHeaderCell;

                                        string modeId = colData.ModeID;
                                        LRRMode mode = LRRModel.Modes.GetModeByID(modeId);

                                        if (!mode.ViewSelection)
                                        {
                                            sCellInRowIndex--;
                                            continue;
                                        }

                                        //Get the group Property Value Object
                                        object objPropValue = GetGroupPropertyValueObject(gridControlManager, iRow, iCol);
                                        if (objPropValue != null)
                                        {
                                            IPropertyValue iPropValue = objPropValue as IPropertyValue;
                                            if (!iPropValue.GetEditableState())
                                                continue;
                                        }
                                        else
                                        {
                                            continue;
                                        }

                                        gridControl[iRow, iCol].Text = sCell;
                                        changePropertyValue(gridControl, gridControlManager, iRow, iCol, sCell);
                                    }
                                }
                            }
                        }
                    }
                    e.Result = true;
                    e.Handled = true;
                }
            }
        }

		#endregion

        /// <summary>
        /// Handles the KeyDown event of the gridControlInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
		private void gridControlInstance_KeyDown(object sender, KeyEventArgs e)
		{
			try
			{
				bool isDelete = true;
                GridControl gridControl = sender as GridControl;

                if (e.KeyCode == Keys.Delete && !gridControl.CurrentCell.IsEditing)
                {
					
					Syncfusion.Windows.Forms.Grid.GridRangeInfoList rangeInfoList = new GridRangeInfoList();
					gridControl.Selections.GetSelectedRanges(out rangeInfoList,true);
					int top = rangeInfoList.ActiveRange.Top;
					int bottom = rangeInfoList.ActiveRange.Bottom;
					int right = rangeInfoList.ActiveRange.Right;
					int left = rangeInfoList.ActiveRange.Left;
					GridGroupPropertiesManager gridGrpPropManager = GridGroupPropsManagerHash[gridControl] as GridGroupPropertiesManager;
					//Get the Row Data from the GridGroupPropManager.
					ArrayList rowSortList = gridGrpPropManager.RowHeaderCellList;
					//Get the Col Data from the GridGroupPropManager.
					ArrayList colSortList = gridGrpPropManager.ColHeaderCellList;
					if(GridGroupPropsManagerHash[gridControl] == null)
					{
						e.Handled = true;
						return;
					}
					for(int iRow=top;iRow<=bottom;iRow++)
					{
						for(int iCol=left;iCol<=right;iCol++)
						{
							GroupRowHeaderCell rowData = rowSortList[iRow- DATA_ROW_START] as GroupRowHeaderCell;
							string propertyType = rowData.GroupPropertyType;
							GroupColHeaderCell colData = colSortList[iCol - DATA_COL_START] as GroupColHeaderCell;
							object objValue = GetGroupPropertyValueObject(gridGrpPropManager,iRow,iCol);
							if(objValue != null)
							{
								IPropertyValue iPropValue = objValue as IPropertyValue;
								if(!iPropValue.GetEditableState())
								{
									isDelete = false;
									e.Handled = true;
									break;
								}
							}
						}
					}
					if(isDelete)
					{
						DialogResult dlgResult = LRRMessagBox.ShowMessage("SID_DELETE_VALUE","SID_LABEL",MessageBoxButtons.YesNo,MessageBoxIcon.Question);     
						if(dlgResult == DialogResult.No)
						{
							e.Handled = true;
						}
						else
						{
							for(int iRow=top;iRow<=bottom;iRow++)
							{
								for(int iCol=left;iCol<=right;iCol++)
								{
									if(iCol >= DATA_COL_START)
									{
										cascadeChangesToPropValues(gridControl,gridGrpPropManager,iRow,iCol);
									}
								}
							}
						}

					}
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRSiteComponentGroupProperties),ex.Message,Logger.Level.Debug); 
			}
		}
	}
}
