using System;
using System.Collections;

using  Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.BusinessObjects;

namespace Symcon.LightRidge.PEAdvisor.GroupProperties
{
	/// <summary>
	/// Summary description for GridGroupPropertiesManager.
	/// </summary>
	public class GridGroupPropertiesManager
	{
		public GridGroupPropertiesManager()
		{
		}

		#region Fields
		//ArrayList for storing row indexes
		private ArrayList _rowHeaderCellList = new ArrayList();

		//ArrayList for storing colHeader Cells
		private ArrayList _colHeaderCellList = new ArrayList();
		
		//BindingManager for storing bindings
		private LRRBindingManager _bindingManager = new LRRBindingManager();

		//Hash for Row Headers
		private Hashtable _rowHeader = new Hashtable();

		//Hash for Column Headers
		private Hashtable _colHeader = new Hashtable();

		//Qualifier ID
		private string _qualifierID;

		//Property Type
		private string _propertyType;

		//Plant Item ID
		private string _siteCompID;

		//Hash for storing cell address
		private Hashtable _propValueCellAddress = new Hashtable();

		//Hash for storing UI UOMs
		private Hashtable _uiUOMHash= new Hashtable();

		//Header Text
		private string _headerText;

		#endregion

		#region Properties
		public ArrayList RowHeaderCellList
		{
			get{return _rowHeaderCellList;}
			set
			{
				_rowHeaderCellList = value;
			}
		}

		public ArrayList ColHeaderCellList
		{
			get{return _colHeaderCellList;}
			set
			{
				_colHeaderCellList = value;
			}
		}

		public LRRBindingManager BindingManager
		{
			get{return _bindingManager;}
			set
			{
				_bindingManager = value;
			}
		}

		public Hashtable RowHeader
		{
			get{return _rowHeader;}
			set
			{
				_rowHeader = value;
			}
		}
		public Hashtable ColHeader
		{
			get{return _colHeader;}
			set
			{
				_colHeader = value;
			}
		}
		public string QualifierID
		{
			get{return _qualifierID;}
			set
			{
				_qualifierID = value;
			}
		}

		public string SiteComponentID
		{
			get{return _siteCompID;}
			set
			{
				_siteCompID = value;
			}
		}

		public string PropertyType
		{
			get{return _propertyType;}
			set
			{
				_propertyType = value;
			}
		}

		public Hashtable PropValueCellAddress
		{
			get{return _propValueCellAddress;}
			set
			{
				_propValueCellAddress = value;
			}
		}

		public Hashtable UIUOMHash
		{
			get{return _uiUOMHash;}
			set
			{
				_uiUOMHash = value;
			}
		}

		public string HeaderText
		{
			get{return _headerText;}
			set
			{
				_headerText = value;
			}
		}
		#endregion

		#region Methods
        /// <summary>
        /// Fores the color changed handler.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		public void ForeColorChangedHandler(object sender,EventArgs e)
		{
			IPropertyValue iPropValue = sender as IPropertyValue;
			string key = iPropValue.GetID();
			GridCellAddress gcAddress = this.PropValueCellAddress[key] as GridCellAddress;

			ColorEventArgs colorEventArgs = new ColorEventArgs();
			colorEventArgs.PropValueObj = iPropValue;
			colorEventArgs.Row = gcAddress.Row;
			colorEventArgs.Col = gcAddress.Col;
			
			if(ForeColorChanged != null)
				ForeColorChanged(this,colorEventArgs);
		}

        /// <summary>
        /// Backs the color changed handler.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		public void BackColorChangedHandler(object sender,EventArgs e)
		{
			IPropertyValue iPropValue = sender as IPropertyValue;
			string key = iPropValue.GetID();
			GridCellAddress gcAddress = this.PropValueCellAddress[key] as GridCellAddress;

			ColorEventArgs colorEventArgs = new ColorEventArgs();
			colorEventArgs.PropValueObj = iPropValue;
			colorEventArgs.Row = gcAddress.Row;
			colorEventArgs.Col = gcAddress.Col;
			
			if(BackColorChanged != null)
				BackColorChanged(this,colorEventArgs);
		}
		#endregion

		#region Event 
		public event ForeColorChangesDel ForeColorChanged;
		public event BackColorChangesDel BackColorChanged;
		#endregion

		public delegate void ForeColorChangesDel(object sender,ColorEventArgs e);
		public delegate void BackColorChangesDel(object sender,ColorEventArgs e);
	}

	public class GridCellAddress
	{
		public GridCellAddress(int row,int col)
		{
			_row = row;
			_col = col;
		}
		#region Properties
		public int Row
		{
			get{return _row;}
			set
			{
				_row = value;
			}
		}

		public int Col
		{
			get{return _col;}
			set
			{
				_col = value;
			}
		}
		#endregion
		#region Fields
		private int _row;
		private int _col;
		#endregion
	}
}
