using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Grid;

using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;

namespace Symcon.LightRidge.PEAdvisor.GroupProperties
{
	/// <summary>
	/// Summary description for GroupedViewUtilities.
	/// </summary>
	public class GroupedViewUtilities
	{
		static GroupedViewUtilities()
		{
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		}

		#region Methods

		/// <summary>
		/// Initializes the grid
		/// </summary>
		/// <param name="gridControlInstance"></param>
		public static void InitializeGrid(GridControl gridControlInstance)
		{
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.BackColor = GroupPropertiesSchemes.COLUMN_HEADER_BACK_COLOR;
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.TextColor = GroupPropertiesSchemes.COLUMN_HEADER_TEXT_COLOR;
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.CellType = "Static";
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.ReadOnly = true;

			gridControlInstance.BaseStylesMap["Row Header"].StyleInfo.BackColor = GroupPropertiesSchemes.ROW_HEADER_BACK_COLOR;

			gridControlInstance.Properties.RowHeaders = true;
			gridControlInstance.NumberedRowHeaders = false;
			gridControlInstance.AllowDragSelectedRows = true;
			gridControlInstance.BaseStylesMap["Row Header"].StyleInfo.Borders.Right = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.Dotted, System.Drawing.SystemColors.WindowFrame);
			gridControlInstance.ColWidths.SetSize(0, GroupPropertiesSchemes.ROW_HEADER_COL_SIZE);

			gridControlInstance.BaseStylesMap["Header"].StyleInfo.BackColor = GroupPropertiesSchemes.HEADER_BACK_COLOR;
			gridControlInstance.ColStyles[1].CellType = "Static";
			gridControlInstance.RowStyles[1].CellType = "Static";
			gridControlInstance.RowStyles[2].CellType = "Static";
			gridControlInstance.ColStyles[1].ReadOnly = true;
			gridControlInstance.RowStyles[0].ReadOnly = true;
			gridControlInstance.RowStyles[1].ReadOnly = true;
			gridControlInstance.RowStyles[2].ReadOnly = true;;

			gridControlInstance.Model.Options.DefaultGridBorderStyle = GridBorderStyle.Solid;
			gridControlInstance.Model.Options.ExcelLikeSelectionFrame = false;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.WrapText = true;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.Trimming = StringTrimming.None;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.AutoSize = true;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.HorizontalAlignment = GridHorizontalAlignment.Center;
			gridControlInstance.Properties.BackgroundColor = GroupPropertiesSchemes.PROPERTIES_BACK_COLOR;
			gridControlInstance.Properties.GridLineColor = GroupPropertiesSchemes.GRID_LINE_COLOR;
			gridControlInstance.DefaultGridBorderStyle = GridBorderStyle.Solid;

//			gridControlInstance.Properties.RowHeaders = false;
			gridControlInstance.Properties.Buttons3D = false;
			gridControlInstance.ColWidths.SetSize(1, GroupPropertiesSchemes.MODE_COL_SIZE);
		}

		/// <summary>
		/// Method hides the mode columns
		/// </summary>
		/// <param name="GridManagerHash"></param>
		public static void HideModes(Hashtable GridManagerHash,int colStart)
		{
			ArrayList arrModeSelected = new ArrayList();
			foreach(LRRMode mode in LRRModel.ProjectModes)
			{
				if(!mode.ViewSelection)
					arrModeSelected.Add(mode.ModeId);
			}
			
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				gridControl.ColHiddenEntries.Clear();
				GridGroupPropertiesManager gridControlManager = dEntry.Value as GridGroupPropertiesManager;
				ArrayList colList = gridControlManager.ColHeaderCellList;
				for(int i=0;i < colList.Count;i++)
				{
					GroupColHeaderCell colHeaderCell = colList[i] as GroupColHeaderCell;
					if(arrModeSelected.Contains(colHeaderCell.ModeID))
					{
						gridControl.ColHiddenEntries.Add(new GridColHidden(i + colStart));
					}
				}
				gridControl.Refresh();
			}
		}

		/// <summary>
		/// Prints the selected grid
		/// </summary>
		/// <param name="gridManagerHash"></param>
		public static void Print(Hashtable gridManagerHash)
		{
			IDictionaryEnumerator iDictEnum = gridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					GridControlPrint ggPrint = new GridControlPrint();
					ggPrint.PrintData(gridControl,false);
					break;
				}
			}
		}

        /// <summary>
        /// Hides the rows
        /// </summary>
        /// <param name="arrayUnSelectedRecords"></param>
        /// <param name="GridManagerHash"></param>
        /// <param name="rowStart"></param>
        public static void HideRows(ArrayList arrayUnSelectedRecords, Hashtable GridManagerHash, int rowStart)
        {
            IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
            while (iDictEnum.MoveNext())
            {
                DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
                GridControl gridControl = dEntry.Key as GridControl;
                GridGroupPropertiesManager gridGroupPropManager = dEntry.Value as GridGroupPropertiesManager;
                ArrayList rowList = gridGroupPropManager.RowHeaderCellList;
                gridControl.RowHiddenEntries.Clear();
                for (int i = 0; i < rowList.Count; i++)
                {
                    GroupRowHeaderCell sortManager = rowList[i] as GroupRowHeaderCell;
                    sortManager.IsVisible = true;
                    string rowID = string.Empty;
                    Form formName = gridControl.FindForm();
                    rowID = sortManager.ID;

                    if (formName != null && formName.GetType() == typeof(GroupedPlantPropertiesView))
                    {
                        Type typeOfForm = ((GroupedPlantPropertiesView)formName).gridView.GetType();
                        if (typeOfForm == typeof(GridPlantViewController))
                           rowID = sortManager.ID + "|" + gridGroupPropManager.SiteComponentID;
                        else if(typeOfForm == typeof(GridPlantTransposedViewController))
                           rowID = sortManager.QualifierID + "|" + sortManager.ID;
                        
                        else if(typeOfForm == typeof(GridQualifierViewController))
                            rowID = sortManager.ID + "|" + gridGroupPropManager.QualifierID;

                    }
                    if (formName != null && formName.GetType() == typeof(GroupedPlantDataArrayView))
                    {
                        Type typeOfForm = ((GroupedPlantDataArrayView)formName).gridView.GetType();
                        if (typeOfForm == typeof(GridDataArrayViewController))
                        {
                            rowID = sortManager.ID + "|" + gridGroupPropManager.PropertyType;
                        }
                    }
                    
                    if (arrayUnSelectedRecords.Contains(rowID))
                    {
                        sortManager.IsVisible = false;
                        gridControl.RowHiddenEntries.Add(new GridRowHidden(i + rowStart));
                    }
                }
                gridControl.Refresh();
            }
        }

        /// <summary>
        /// Hides the columns
        /// </summary>
        /// <param name="arrayUnSelectedRecords"></param>
        /// <param name="GridManagerHash"></param>
        public static void HideColumns(ArrayList arrayUnSelectedRecords, Hashtable GridManagerHash, int colStart)
        {
            ArrayList arrModeSelected = new ArrayList();
            foreach (LRRMode mode in LRRModel.ProjectModes)
            {
                if (!mode.ViewSelection)
                    arrModeSelected.Add(mode.ModeId);
            }
            IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
            while (iDictEnum.MoveNext())
            {
                DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
                GridControl gridControl = dEntry.Key as GridControl;
                GridGroupPropertiesManager gridGroupPropManager = dEntry.Value as GridGroupPropertiesManager;
                gridControl.ColHiddenEntries.Clear();
                ArrayList colList = gridGroupPropManager.ColHeaderCellList;
                for (int i = 0; i < colList.Count; i++)
                {
                    GroupColHeaderCell colHeaderCell = colList[i] as GroupColHeaderCell;
                    colHeaderCell.IsVisible = true;
                    string colID = colHeaderCell.ID;
                    Form formName = gridControl.FindForm();
                    
                    if (formName != null && formName.GetType() == typeof(GroupedPlantPropertiesView))
                    {
                        Type typeOfForm = ((GroupedPlantPropertiesView)formName).gridView.GetType();
                        if (typeOfForm == typeof(GridPlantViewController))
                        {
                            colID = colHeaderCell.QualifierID + "|" + colHeaderCell.ID;
                        }
                        else if (typeOfForm == typeof(GridPlantTransposedViewController))
                        {
                            colID = colHeaderCell.ID + "|" + gridGroupPropManager.SiteComponentID;
                        }
                        else if (typeOfForm == typeof(GroupProperties.GridQualifierTransposedViewController))
                        {
                            colID = colHeaderCell.ID + "|" + colHeaderCell.QualifierID;
                        }
                    }

                    if (formName != null && formName.GetType() == typeof(GroupedPlantDataArrayView))
                    {
                        Type typeOfForm = ((GroupedPlantDataArrayView)formName).gridView.GetType();
                        if (typeOfForm == typeof(GridDataArrayTransposeViewController))
                        {
                            colID = colHeaderCell.ID + "|" + gridGroupPropManager.PropertyType;
                        }
                    }
                 
                    if (arrayUnSelectedRecords.Contains(colID))
                    {
                        colHeaderCell.IsVisible = false;
                        gridControl.ColHiddenEntries.Add(new GridColHidden(i + colStart));
                    }
                    if (arrModeSelected.Contains(colHeaderCell.ModeID))
                    {
                        gridControl.ColHiddenEntries.Add(new GridColHidden(i + colStart));
                    }
                }
                gridControl.Refresh();
               
            }  
        }
		/// <summary>
		/// Print Preview of the selected grid
		/// </summary>
		/// <param name="gridManagerHash"></param>
		public static void PrintPreview(Hashtable gridManagerHash)
		{
			IDictionaryEnumerator iDictEnum = gridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					GridControlPrint ggPrint = new GridControlPrint();
					ggPrint.PrintData(gridControl,true);
					break;
				}
			}
		}

		/// <summary>
		/// Exports the selected grid
		/// </summary>
		/// <param name="gridManagerHash"></param>
		/// <returns></returns>
		public static GridControl Export(Hashtable gridManagerHash)
		{
			IDictionaryEnumerator iDictEnum = gridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					return gridControl;
				}
			}
			return null;
		}

		/// <summary>
		/// Method Prevents resize of hidden columns
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="col"></param>
		/// <returns></returns>
		public static bool StopResizeHiddenColumns(GridControl gridControl,int col)
		{
			GridColHiddenCollection gridColHiddenCollection = gridControl.ColHiddenEntries;
			GridColHiddenCollectionEnumerator gridEnum = gridColHiddenCollection.GetEnumerator();
			while(gridEnum.MoveNext())
			{
				GridColHidden gridColHidden = gridEnum.Current;
				if(gridColHidden.ColIndex == col)
				{
					return true;
				}
			}
			return false;
		}

		/// <summary>
		/// Method handles UOM editing
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		/// <param name="uomIndex"></param>
		public static void HandleUOMEditingKeyDown(object sender,KeyEventArgs e,int uomIndex)
		{
			//Check if the field is UOM field
			GridControl gridControl = sender as GridControl;
			int rowIndex = gridControl.CurrentCell.RowIndex;
			if(rowIndex == uomIndex)
			{
				e.Handled = true;
				return;
			}
		}

		/// <summary>
		/// Method handles UOM editing
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		/// <param name="uomIndex"></param>
		public static void HandleUOMEditingKeyPress(object sender,KeyPressEventArgs e,int uomIndex)
		{
			//Check if the field is UOM field
			GridControl gridControl = sender as GridControl;
			int rowIndex = gridControl.CurrentCell.RowIndex;
			if(rowIndex == uomIndex)
			{
				e.Handled = true;
				return;
			}
		}

		/// <summary>
		/// Resizes columns to fit
		/// </summary>
		/// <param name="GridManagerHash"></param>
		public static void ResizeToFit(Hashtable GridManagerHash,int DATA_ROW_START,int DATA_COL_START)
		{
			IEnumerator iEnum = GridManagerHash.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
				GridControl gridControl =  dEntry.Key as GridControl;
				gridControl.ColWidths.ResizeToFit(GridRangeInfo.Cells(DATA_ROW_START,DATA_COL_START,gridControl.RowCount,gridControl.ColCount),GridResizeToFitOptions.NoShrinkSize);
			}
		}

		/// <summary>
		/// Arranges Row List on Row Movement
		/// </summary>
		/// <param name="oldIndex"></param>
		/// <param name="newIndex"></param>
		/// <param name="countOFItems"></param>
		/// <param name="rowHeaderList"></param>
		/// <param name="rowStart"></param>
		public static void RowMovement(int oldIndex,int newIndex,int countOFItems,ArrayList rowHeaderList,int rowStart)
		{
			//Update the Row List
			int toMoveIndex = -1;
			int lastInsertIndex = -1;

			//Upward Movement
			if(oldIndex > newIndex)
			{
				toMoveIndex = oldIndex + (countOFItems - 1);
				lastInsertIndex = newIndex;
			}
			else//Downward Movement
			{
				toMoveIndex = oldIndex;
				lastInsertIndex = newIndex + countOFItems - 1;
			}

			for(int i=0;i<countOFItems;i++)
			{
				//Remove 
				GroupRowHeaderCell sortManager = (GroupRowHeaderCell)rowHeaderList[toMoveIndex - rowStart];
				//Update the new indexes
				rowHeaderList.RemoveAt(toMoveIndex - rowStart);
				rowHeaderList.Insert(lastInsertIndex - rowStart,sortManager);
			}
		}

		/// <summary>
		/// Method clears the sorting
		/// </summary>
		public static void ClearSort(Hashtable GridManagerHash)
		{
			IEnumerator iEnum = GridManagerHash.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
				GridControl gridControl =  dEntry.Key as GridControl;
				if(gridControl.CurrentCell.IsEditing)
					gridControl.CurrentCell.EndEdit();
				//turn off the sort arrow
				for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				{
					if(gridControl[1, col1].CellType == "ColumnHeaderCell")
					{
						gridControl[1, col1].CellType = "Header";
						break;
					}
				}
			}
		}

		/// <summary>
		/// Arranges Col List on Col Movement
		/// </summary>
		/// <param name="oldIndex"></param>
		/// <param name="newIndex"></param>
		/// <param name="countOFItems"></param>
		/// <param name="colList"></param>
		/// <param name="colStart"></param>
		public static void ColMovement(int oldIndex,int newIndex,int countOFItems,ArrayList colList,int colStart)
		{
			//Update the Row List
			int toMoveIndex = -1;
			int lastInsertIndex = -1;

			//Upward Movement
			if(oldIndex > newIndex)
			{
				toMoveIndex = oldIndex + (countOFItems - 1);
				lastInsertIndex = newIndex;
			}
			else//Downward Movement
			{
				toMoveIndex = oldIndex;
				lastInsertIndex = newIndex + countOFItems - 1;
			}

			for(int i=0;i<countOFItems;i++)
			{
				//Remove 
				GroupColHeaderCell colHeaderCell = (GroupColHeaderCell)colList[toMoveIndex - colStart];
				//Update the new indexes
				colList.RemoveAt(toMoveIndex - colStart);
				colList.Insert(lastInsertIndex - colStart,colHeaderCell);
			}
		}

		/// <summary>
		/// Method moves the range of columns to right
		/// </summary>
		/// <param name="gridControlManager"></param>
		/// <param name="gridControl"></param>
		public static void RightColMovement(GridGroupPropertiesManager gridControlManager,GridControl  gridControl)
		{
			GridRangeInfo rangeInfo = gridControl.Selections.Ranges.ActiveRange;
			int range = rangeInfo.Right - rangeInfo.Left;
			int right = rangeInfo.Right;
			int left = rangeInfo.Left;
			
			if(right + LRRModel.ProjectModes.Count > gridControl.ColCount)
				return;
			
			int target = left + LRRModel.ProjectModes.Count;
			gridControl.Cols.MoveRange(left,range + 1,target);

//			for(int i=1;i<=gridControl.ColCount;i++)
//			{
//				if(gridControl[0,i].Text.Trim() != "")
//				{
//					GridPersistanceUtil._arrGridColMove[0].Add(gridControl[0,i].Text);
//					break;
//				}
//			}
//			GridPersistanceUtil._arrGridColMove[1].Add(left);
//			GridPersistanceUtil._arrGridColMove[2].Add(range + 1);
//			GridPersistanceUtil._arrGridColMove[3].Add(target);
		}
		
		/// <summary>
		/// Method moves the range of columns to left
		/// </summary>
		public static void LeftColMovement(GridGroupPropertiesManager gridControlManager,GridControl  gridControl,int DATA_COL_START,int HEADER_ROW)
		{		
			GridRangeInfo rangeInfo = gridControl.Selections.Ranges.ActiveRange;
			int range = rangeInfo.Right - rangeInfo.Left;
			int right = rangeInfo.Right;
			int left = rangeInfo.Left;
			int target = left - LRRModel.ProjectModes.Count;
			if(target < DATA_COL_START || rangeInfo.Top != HEADER_ROW)
				return;

			gridControl.Cols.MoveRange(left,range + 1,target);

//			for(int i=1;i<=gridControl.ColCount;i++)
//			{
//				if(gridControl[0,i].Text.Trim() != "")
//				{
//					GridPersistanceUtil._arrGridColMove[0].Add(gridControl[0,i].Text);
//					break;
//				}
//			}
//			GridPersistanceUtil._arrGridColMove[1].Add(left);
//			GridPersistanceUtil._arrGridColMove[2].Add(range + 1);
//			GridPersistanceUtil._arrGridColMove[3].Add(target);
		}
		
		#endregion

		#region Fields
		private static LRRApp			_lrrApp;
		private static Comparer			_rowComparer = new Comparer();
		#endregion

		#region Properties
		public static Comparer RowComparer
		{
			get{return _rowComparer;}
			set
			{
				_rowComparer = value;
			}
		}
		#endregion
	}
}
