using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using Syncfusion.Windows.Forms.Grid;
using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects;
using System.Reflection;
using Symcon.Utils.HeaderDispatcher;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.MessageLogger;

namespace Symcon.LightRidge.PEAdvisor
{
    public partial class HistdataValidation : Form
    {
        public HistdataValidation(CustomCollection cc)
        {
            try
            {
                
                InitializeComponent();
                LRRWaitWindowUtility.ShowPogressIndicator(this.Text);
                _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
                _histDatas = LRRModel.HistDatas;
                foreach (LRRHistData histData in _histDatas)
                {
                    CheackForValidation(histData);
                }
                _histDatasValidateFalied.SortColumn = "HistTag";
                _histDatasValidateFalied.Sort();
                if (_levels != null)
                    _levels.Clear();

                _levels = new LRRLevels();
                _app.DBManagerObj.PopulateCollection(_levels, false);
                props = cc.ItemType.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
                int rowCount = getGridRowCount();
                //int colCount = getGridColCount(props);
                createGrid(rowCount, COL_COUNT);
                gridControl1.CellModels.Add("DropDownTree", new PlantLevelUserFormTreeControl(gridControl1.Model, new PlantLevelTreeControlStructure()));
                fillAbsoluteFiledList();
                initializeRowHeader();
                initializeColHeader();
                initializeData();
                initDataSource();
                gridControl1.Model.SaveCellInfo += new GridSaveCellInfoEventHandler(Model_SaveCellInfo);
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(HistdataValidation), "Validation Load: " + ex, Logger.Level.Debug);
            }
            finally
            {
                LRRWaitWindowUtility.EndProgressIndicator();
            }


        }

        void Model_SaveCellInfo(object sender, GridSaveCellInfoEventArgs e)
        {
            if (e.Style.CellType == "DropDownTree")
            {
                string valueMem = e.Style.CellValue.ToString();
                if (valueMem != "" && valueMem != null)
                {
                    GridModel grid = sender as GridModel;
                    string oldValueText = gridControl1.CurrentCell.Renderer.ControlValue == null ? "" : gridControl1.CurrentCell.Renderer.ControlValue.ToString();
                    LRRLevel lvl = LRRModel.ChildModel.Levels.getLevelByLevelID(valueMem);
                    e.Style.FormattedText = lvl.Name;
                    string newValue = lvl.LevelID;
                    string newValueText = lvl.Name;
                    int currentRow = e.RowIndex;
                    int currentCol = e.ColIndex;
                    ColomnHeaderInfo colInfo = ColInfo[currentRow] as ColomnHeaderInfo;
                    RowHeaderInfo rowInfo = RowInfo[currentCol] as RowHeaderInfo;
                    string keyField = colInfo.ID;
                    string Prop = rowInfo.PropertyName;
                    object record = _histHash[keyField] as object;
                    LRRHistData histDataObj = LRRModel.HistDatas.GetHistDataFromHistTagUID(keyField);

                    string oldValue = histDataObj.GetType().GetProperty(Prop).GetValue(record, null) == null ? "" : histDataObj.GetType().GetProperty(Prop).GetValue(record, null).ToString();
                    if (newValue != oldValue)
                    {
                        if (newValue != "")
                        {
                            e.Style.BackColor = Color.White;
                            histDataObj.GetType().GetProperty(Prop).SetValue(histDataObj, newValue, null);
                        }
                        else
                        {
                            e.Style.BackColor = Color.Blue;
                        }

                    }     
                }
            }
        }

        #region Fields

        private LRRHistDatas _histDatas = null;
        private LRRHistDatas _histDatasValidateFalied = new LRRHistDatas();
        private Hashtable _histHash = new Hashtable();
        //private Hashtable _histDataHash = new Hashtable();
        private PropertyInfo[] props;
        private ArrayList RowInfo = new ArrayList();
        private ArrayList absoluteFields = new ArrayList();
        private ArrayList ColInfo = new ArrayList();
        private int ROW_COUNT;
        private int COL_COUNT = 14;
        private bool _isFkField = false;
        private LRRLevels _levels;
        private LRRApp _app; //App Instance...

        private LRRProjectConfigurations _lrrProjConfigs;
        private LRRMeasurementSystemDefinitions _lrrMeasSystemDefs;
        #endregion

        #region Private Method

        private void fillAbsoluteFiledList()
        {
            absoluteFields.Add("CalibrationFactor");
            absoluteFields.Add("HistUnits");
            absoluteFields.Add("UOMTypeID");
            absoluteFields.Add("InterfaceUnits");
            absoluteFields.Add("PlantUnit");
            absoluteFields.Add("MeasType");
            absoluteFields.Add("OffConditionSubstituteValue");
            absoluteFields.Add("OnOffThresholdLowerLimit");
            absoluteFields.Add("OnOffThresholdUpperLimit");
            absoluteFields.Add("ServerID");
            absoluteFields.Add("ValidRangeLowerLimit");
            absoluteFields.Add("ValidRangeUpperLimit");
            absoluteFields.Add("FilterSelection");
        }

        private void createGrid(int rowCount, int colCount)
        {
            gridControl1.ColCount = COL_COUNT - 1;
            gridControl1.RowCount = rowCount;
            gridControl1.NumberedRowHeaders = false;
            gridControl1.ExcelLikeSelectionFrame = true;
            gridControl1.Properties.BackgroundColor = Color.White;
            gridControl1[0, 0].Text = "Hist Tag";
            RowHeaderInfo rowInfo = new RowHeaderInfo();
            rowInfo.PropertyName = "Properties";
            RowInfo.Add(rowInfo);
            ColomnHeaderInfo colInfo = new ColomnHeaderInfo();
            colInfo.ID = "1";
            colInfo.ObjectType = null;
            ColInfo.Add(colInfo);
            gridControl1.ColWidths[0] = 150;
            ROW_COUNT = gridControl1.RowCount;
        }

        private void initializeRowHeader()
        {
            int j = 1;
            for (int i = 0; i < _histDatasValidateFalied.Count; i++)
            {
                ColomnHeaderInfo colHeaderInfo = new ColomnHeaderInfo();
                colHeaderInfo.ID = _histDatasValidateFalied[i].HistTagUID;
                colHeaderInfo.ObjectType = _histDatasValidateFalied[i].TypeOfObject;
                gridControl1[j, 0].Text = _histDatasValidateFalied[i].HistTag;
                ColInfo.Add(colHeaderInfo);
                j++;
            }
        }

        private void initializeColHeader()
        {
            int j = 1;
            for (int i = 0; i < absoluteFields.Count; i++)
            {
                RowHeaderInfo rowHeaderInfo = new RowHeaderInfo();
                rowHeaderInfo.PropertyName = absoluteFields[i].ToString();
                gridControl1[0, j].Text = absoluteFields[i].ToString();
                gridControl1[0, j].WrapText = false;
                gridControl1.ColWidths[j] = 150;
                RowInfo.Add(rowHeaderInfo);
                j++;
            }
        }


        private void initializeData()
        {
            CustomCollection dataSource = null;
            string valueMember = "";
            string displayMenber = "";
            for (int i = 1; i < ColInfo.Count; i++)
            {
                ColomnHeaderInfo colInfo = ColInfo[i] as ColomnHeaderInfo;
                string histTagUID = colInfo.ID;
                for (int j = 1; j < RowInfo.Count; j++)
                {
                    RowHeaderInfo rowInfo = RowInfo[j] as RowHeaderInfo;
                    string propName = rowInfo.PropertyName;
                    for (int k = 0; k < props.Length; k++)
                    {
                        PropertyInfo prop = props[k];
                        if (prop.Name == propName)
                        {
                            SymForeignKeyFieldAttribute[] FKfield = (SymForeignKeyFieldAttribute[])
                            prop.GetCustomAttributes(typeof(SymForeignKeyFieldAttribute), false);
                            if (FKfield.Length > 0)
                            {
                                SymForeignKeyFieldAttribute sFkField = FKfield[0];
                                dataSource = LRRModel.GetObjectCopy(sFkField.CollectionType) as CustomCollection;
                                displayMenber = sFkField.DisplayMemeber;
                                valueMember = sFkField.ValueMemeber;
                                _isFkField = true;
                                break;
                            }
                            else
                            {
                                break;
                            }
                        }
                    }
                    LRRHistData histData1 = _histHash[histTagUID] as LRRHistData;
                    object histData = _histHash[histTagUID];
                    string sValue = histData.GetType().GetProperty(propName).GetValue(histData, null) == null ? "" : histData.GetType().GetProperty(propName).GetValue(histData, null).ToString();
                    if (sValue != "")
                    {
                        gridControl1[i, j].Text = sValue;
                        gridControl1[i, j].BackColor = Color.White;

                    }
                    else
                    {
                        gridControl1[i, j].Text = "";
                        gridControl1[i, j].BackColor = Color.LightBlue;
                    }
                    if (_isFkField)
                    {
                        rowInfo.IsFkField = true;
                        if (propName == "HistUnits" || propName == "InterfaceUnits")
                        {
                            string uomTypeID = histData1.UOMTypeID;
                            LRRUOMType uomType = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(uomTypeID);
                            if (uomType != null)
                            {
                                LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomType.UOMTypeID);
                                if (uoms.Count > 0)
                                {
                                    gridControl1.Model[i, j].CellType = "ComboBox";
                                    gridControl1.Model[i, j].DisplayMember = displayMenber;
                                    gridControl1.Model[i, j].ValueMember = valueMember;
                                    gridControl1.Model[i, j].DataSource = dataSource;
                                    gridControl1.Model[i, j].ExclusiveChoiceList = false;
                                    _isFkField = false;
                                }
                                else
                                {
                                    gridControl1[i, j].CellType = "ComboBox";
                                    gridControl1[i, j].Text = "No Uom(s) defined";
                                    gridControl1.Model[i, j].DisplayMember = displayMenber;
                                    gridControl1.Model[i, j].ValueMember = valueMember;
                                    gridControl1[i, j].TextColor = Color.Blue;
                                    gridControl1[i, j].BackColor = Color.White;
                                    gridControl1.Model[i, j].ExclusiveChoiceList = false;
                                    _isFkField = false;
                                }
                            }
                            else
                            {
                                gridControl1.Model[i, j].CellType = "ComboBox";
                                gridControl1.Model[i, j].DisplayMember = displayMenber;
                                gridControl1.Model[i, j].ValueMember = valueMember;
                                gridControl1.Model[i, j].DataSource = dataSource;
                                gridControl1.Model[i, j].ExclusiveChoiceList = false;
                                _isFkField = false;
                            }
                        }
                        else if (propName.ToUpper() == "PLANTUNIT")
                        {
                            gridControl1.Model[i, j].CellType = "DropDownTree";
                            string valueMem = gridControl1.Model[i, j].CellValue.ToString();
                            if (valueMem != "" && valueMem != null)
                            {
                                LRRLevel lvl = LRRModel.ChildModel.Levels.getLevelByLevelID(valueMem);
                                gridControl1.Model[i, j].FormattedText = lvl.Name;
                            }
                            gridControl1.Model[i, j].DisplayMember = displayMenber;
                            gridControl1.Model[i, j].ValueMember = valueMember;
                            gridControl1.Model[i, j].DataSource = dataSource;
                            _isFkField = false;

                        }
                        else
                        {
                            gridControl1.Model[i, j].CellType = "ComboBox";
                            gridControl1.Model[i, j].DisplayMember = displayMenber;
                            gridControl1.Model[i, j].ValueMember = valueMember;
                            gridControl1.Model[i, j].DataSource = dataSource;
                            gridControl1.Model[i, j].ExclusiveChoiceList = false;
                            _isFkField = false;
                        }
                    }
                    if (propName == "InterfaceUnits" || propName == "UOMTypeID")
                    {
                        gridControl1.Model[i, j].BackColor = Color.FromArgb(204, 255, 204);
                    }
                }
            }

        }

        private void initDataSource()
        {
            if (_lrrProjConfigs != null)
                _lrrProjConfigs.Clear();
            if (_lrrMeasSystemDefs != null)
                _lrrMeasSystemDefs.Clear();
            _lrrProjConfigs = new LRRProjectConfigurations();
            _app.DBManagerObj.PopulateCollection(_lrrProjConfigs, false);

            _lrrMeasSystemDefs = new LRRMeasurementSystemDefinitions();
            _app.DBManagerObj.PopulateCollection(_lrrMeasSystemDefs, false);
        }



        private int getGridRowCount()
        {
            int rowCount = _histDatasValidateFalied.Count;
            return rowCount;
        }

        private void CheackForValidation(LRRHistData lhd)
        {
            string retValue = "";
            string[] s = new string[1];
            s[0] = lhd.HistTag;
            if (lhd.CalibrationFactor == null || lhd.CalibrationFactor == "")
            {
                //LRRMessagBox.ShowMessage("SID_CALIFACTORNULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = "CalibrationFactor";
            }
            /*
            if (lhd.HistUnits == null || lhd.HistUnits == "")
            {
                //LRRMessagBox.ShowMessage("SID_HISTUNITNULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : HistUnits";
            }
             * */
            if (lhd.MeasType == null || lhd.MeasType == "")
            {
                //LRRMessagBox.ShowMessage("SID_MEASTYPENULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : MeasType";
            }
            if (lhd.OffConditionSubstituteValue == null || lhd.OffConditionSubstituteValue == "")
            {
                //LRRMessagBox.ShowMessage("SID_OFFCONDSUBVALUENULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : OffConditionSubstituteValue";
            }
            if (lhd.OnOffThresholdLowerLimit == null || lhd.OnOffThresholdLowerLimit == "")
            {
                //LRRMessagBox.ShowMessage("SID_THRESHOLDLOWER_NULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : OnOffThresholdLowerLimit";
            }
            if (lhd.OnOffThresholdUpperLimit == null || lhd.OnOffThresholdUpperLimit == "")
            {
                //LRRMessagBox.ShowMessage("SID_THRESHOLDUPPER_NULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : OnOffThresholdUpperLimit";
            }
            if (lhd.PlantUnit == null || lhd.PlantUnit == "")
            {
                //LRRMessagBox.ShowMessage("SID_PLANTUNITNULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : PlantUnit";
            }
            if (lhd.ServerID == null || lhd.ServerID == "")
            {
                //LRRMessagBox.ShowMessage("SID_SERVERIDNULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : ServerID";
            }
            if (lhd.ValidRangeLowerLimit == null || lhd.ValidRangeLowerLimit == "")
            {
                //LRRMessagBox.ShowMessage("SID_VALIDLOWER_NULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : ValidRangeLowerLimit";
            }
            if (lhd.ValidRangeUpperLimit == null || lhd.ValidRangeUpperLimit == "")
            {
                //LRRMessagBox.ShowMessage("SID_VALIDUPPER_NULL_HISTDATA", "SID_LABEL", s, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                retValue = retValue + " : ValidRangeUpperLimit";
            }
            if (lhd.UOMTypeID == null || lhd.UOMTypeID == "")
            {
                retValue = retValue + " : UomType";
            }
            if (lhd.UOMTypeID != null && lhd.UOMTypeID != "")
            {
                LRRUOMType uomType = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(lhd.UOMTypeID);
                if (uomType != null)
                {
                    if (!uomType.IsEVV)
                    {
                        LRRUOMs uom = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomType.UOMTypeID);
                        if (uom.Count > 0)
                        {
                            if (lhd.InterfaceUnits == null || lhd.InterfaceUnits.Trim() == "")
                            {
                                retValue = retValue + " : InterfaceUnits";
                            }
                        }
                    }
                }
            }
            if (retValue != "")
            {
                _histDatasValidateFalied.Add(lhd);
                _histHash.Add(lhd.HistTagUID, lhd);
            }
        }

        #endregion

        private void gridControl1_CurrentCellCloseDropDown(object sender, Syncfusion.Windows.Forms.PopupClosedEventArgs e)
        {
            object record = null;
            string Prop = "";
            string keyField = "";
            string oldValue = "";
            
            GridControl grid = sender as GridControl;
            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            ColomnHeaderInfo colInfo = ColInfo[currentRow] as ColomnHeaderInfo;
            RowHeaderInfo rowInfo = RowInfo[currentCol] as RowHeaderInfo;
            keyField = colInfo.ID;
            record = _histHash[keyField] as object;
            Prop = rowInfo.PropertyName;
            LRRHistData histDataObj = LRRModel.HistDatas.GetHistDataFromHistTagUID(keyField);
            string newTextValue = grid.CurrentCell.Renderer.ControlText.ToString();
            string newValue = grid.CurrentCell.Renderer.ControlValue == null ? "" : grid.CurrentCell.Renderer.ControlValue.ToString();
            if (Prop == "InterfaceUnits" || Prop == "HistUnits")
            {
                oldValue = histDataObj.ModelUOM;
            }

            if (Prop == "UOMTypeID")
            {
                oldValue = histDataObj.HistUOMTypeID;
            }

            if (newValue.Trim() != "" && newValue != null)
            {
                if (newValue != oldValue)
                {
                    histDataObj.GetType().GetProperty(Prop).SetValue(histDataObj, newValue, null);
                    if (record != null)
                    {
                        RelationalColumnConditions conditions = new RelationalColumnConditions();
                        conditions.ConditionApplyForFilterUOMbasedOnUomType(record, newValue, oldValue, Prop);
                        LRRProjectConfiguration lrrProjConfig = _lrrProjConfigs[0] as LRRProjectConfiguration;
                        string modelUOMID = _lrrMeasSystemDefs.GetLocaleUOM(lrrProjConfig.MeasurementSystemCEID, newValue);
                        string uiUOMID = _lrrMeasSystemDefs.GetLocaleUOM(lrrProjConfig.MeasurementSystemUserInterfaceID, newValue);
                        LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(uiUOMID);
                        LRRUOMs uomSource = LRRModel.ChildModel.GetUOMsFromUOMTypeID(newValue);
                        int interfaceColumn = 0;
                        int histUnitColumn = 0;
                        if (Prop == "UOMTypeID")
                        {
                            for (int i = 0; i < grid.ColCount; i++)
                            {
                                if (grid[0, i].Text == "InterfaceUnits")
                                {
                                    interfaceColumn = i;
                                    break;
                                }
                            }
                            for (int i = 0; i < grid.ColCount; i++)
                            {
                                if (grid[0, i].Text == "HistUnits")
                                {
                                    histUnitColumn = i;
                                    break;
                                }
                            }

                            if (histDataObj.MeasType != "C" && histDataObj.MeasType != "N")
                            {
                                grid[currentRow, histUnitColumn].CellValue = "";
                                grid[currentRow, histUnitColumn].FormattedText = "";

                                if (grid[currentRow, histUnitColumn].TextColor == Color.Blue)
                                {
                                    grid[currentRow, histUnitColumn].TextColor = Color.Black;
                                }

                                histDataObj.GetType().GetProperty("HistUOMTypeID").SetValue(histDataObj, newValue, null);
                            }

                            if (uom != null && (uiUOMID != null && uiUOMID.Trim() != ""))
                            {
                                grid[currentRow, interfaceColumn].CellValue = uiUOMID;
                                grid[currentRow, interfaceColumn].FormattedText = uom.Name;

                                if (grid[currentRow, interfaceColumn].TextColor == Color.Blue)
                                {
                                    grid[currentRow, interfaceColumn].TextColor = Color.Black;
                                }
                            }
                            else
                            {
                                grid[currentRow, interfaceColumn].CellValue = "";
                                grid[currentRow, interfaceColumn].FormattedText = "";
                            }
                        }
                        string uomTypeID = histDataObj.UOMTypeID;
                        string histUomTypeID = histDataObj.HistUOMTypeID; 
                        if (uomTypeID != null && uomTypeID != "")
                        {
                            LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomTypeID);
                            if (uoms.Count == 0)
                            {
                                gridControl1[currentRow, interfaceColumn].Text = "No Uom(s) defined";
                                gridControl1[currentRow, interfaceColumn].TextColor = Color.Blue;
                            }
                            else
                            {
                                gridControl1[currentRow, currentCol].Text = "";
                            }
                        }
                        if (histUomTypeID != null && histUomTypeID != "")
                        {
                            LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(histUomTypeID);
                            if (uoms.Count == 0)
                            {
                                gridControl1[currentRow, histUnitColumn].Text = "No Uom(s) defined";
                                gridControl1[currentRow, histUnitColumn].TextColor = Color.Blue;
                                gridControl1[currentRow, histUnitColumn].BackColor = Color.White;
                            }
                            else
                            {
                                gridControl1[currentRow, histUnitColumn].Text = "";
                                gridControl1[currentRow, currentCol].BackColor = Color.White;
                            }
                        }
                    }                    
                }
            }
        }

        private void gridControl1_CurrentCellValidated(object sender, EventArgs e)
        {
            GridControl grid = sender as GridControl;
            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            ColomnHeaderInfo colInfo = ColInfo[currentRow] as ColomnHeaderInfo;
            RowHeaderInfo rowInfo = RowInfo[currentCol] as RowHeaderInfo;
            string keyField = colInfo.ID;
            object record = _histHash[keyField] as object;
            string Prop = rowInfo.PropertyName;
            string oldValue = grid.Model[currentRow, currentCol].FormattedText;
            string newTextValue = grid.CurrentCell.Renderer.ControlText.ToString();
            string newValue = grid.CurrentCell.Renderer.ControlValue.ToString();
            if (newTextValue != oldValue)
            {
                LRRHistData histDataObj = LRRModel.HistDatas.GetHistDataFromHistTagUID(keyField);
                histDataObj.GetType().GetProperty(Prop).SetValue(histDataObj, newValue, null);

                if (newValue != "" && (Prop != "InterfaceUnits" && Prop != "UOMTypeID"))
                {
                    gridControl1[currentRow, currentCol].BackColor = Color.White;
                }
                else if (Prop == "InterfaceUnits" || Prop == "UOMTypeID")
                {
                    gridControl1[currentRow, currentCol].BackColor = Color.FromArgb(204, 255, 204);
                }
                
            }
        }

        private void gridControl1_CurrentCellShowingDropDown(object sender, GridCurrentCellShowingDropDownEventArgs e)
        {
            GridControl grid = sender as GridControl;
            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            ColomnHeaderInfo colInfo = ColInfo[currentRow] as ColomnHeaderInfo;
            RowHeaderInfo rowInfo = RowInfo[currentCol] as RowHeaderInfo;
            string prop = rowInfo.PropertyName;
            string keyField = colInfo.ID;
            object record = _histHash[keyField] as object;
            LRRHistData histData = LRRModel.HistDatas.GetHistDataFromHistTagUID(keyField);
            string sUOMType = histData.UOMTypeID;
            string histUomType = histData.HistUOMTypeID;
            //if (prop == "HistUnits" || prop == "InterfaceUnits")
            if (prop == "InterfaceUnits")
            {

                if (sUOMType.Trim() != "" && sUOMType != null )
                {
                    LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(sUOMType);
                    if (uoms != null)
                    {
                        grid[currentRow, currentCol].DataSource = uoms;
                        if (uoms.Count == 0)
                        {
                            LRRMessagBox.ShowMessage("SID_UOMNOTDEFINED", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                }
            }
            if (prop == "HistUnits")
            {

                if (histUomType.Trim() != "" && histUomType != null)
                {
                    LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(histUomType);
                    if (uoms != null)
                    {
                        grid[currentRow, currentCol].DataSource = uoms;
                        if (uoms.Count == 0)
                        {
                            LRRMessagBox.ShowMessage("SID_UOMNOTDEFINED", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
               }
           } 
 
        }

        private void gridControl1_CurrentCellValidating(object sender, CancelEventArgs e)
        {

            GridControl grid = sender as GridControl;
            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            ColomnHeaderInfo colInfo = ColInfo[currentRow] as ColomnHeaderInfo;
            RowHeaderInfo rowInfo = RowInfo[currentCol] as RowHeaderInfo;
            string keyField = colInfo.ID;
            object record = _histHash[keyField] as object;
            string Prop = rowInfo.PropertyName;
            string oldValue = grid.Model[currentRow, currentCol].FormattedText;
            string newTextValue = grid.CurrentCell.Renderer.ControlText.ToString();
            string newValue = grid.CurrentCell.Renderer.ControlValue == null ? "" : grid.CurrentCell.Renderer.ControlValue.ToString();
            if (newTextValue != oldValue)
            {
                LRRHistData histDataObj = LRRModel.HistDatas.GetHistDataFromHistTagUID(keyField);
                histDataObj.GetType().GetProperty(Prop).SetValue(histDataObj, newValue, null);
            }

        }

        void gridControl1_CurrentCellKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
          if (e.KeyData == System.Windows.Forms.Keys.Delete)
            {
                e.Handled = true;
            }
            
                
        }

        private void gridControl1_CurrentCellKeyPress(object sender, KeyPressEventArgs e)
        {
            GridControl g1 = sender as GridControl;
            if (g1[g1.CurrentCell.RowIndex, g1.CurrentCell.ColIndex].CellType == "ComboBox")
            {
                e.Handled = true;
            }
            
        }
    }
}