using System;
using System.Collections;

using Symcon.LightRidge.BusinessObjects;
using  Symcon.LightRidge.PEAdvisor.DataStore;

namespace Symcon.LightRidge.PEAdvisor.IOScreen
{
	/// <summary>
	/// Summary description for GridControlManager.
	/// </summary>
	public class GridControlManager
	{
		public GridControlManager()
		{

		}

		#region Fields
		//ArrayList for storing row indexes
		private ArrayList _sortManagerList = new ArrayList();

		//Hash for storing UI UOMs
		private Hashtable _uiUOMHash= new Hashtable();

		//Grid Data object
		private object[,] _gridData;

		//ArrayList for storing colHeader Cells
		private ArrayList _colHeaderCellList = new ArrayList();
		
		//BindingManager for storing bindings
		private LRRBindingManager _bindingManager = new LRRBindingManager();

		//Hash for Row Headers
		private Hashtable _rowHeader = new Hashtable();

		//Hash for Column Headers
		private Hashtable _colHeader = new Hashtable();

		//Hash for storing cell address
		private Hashtable _propValueCellAddress = new Hashtable();

		//Header Text
		private string _headerText;
		#endregion


		#region Methods
		public void ForeColorChangedHandler(object sender,EventArgs e)
		{
			IPropertyValue iPropValue = sender as IPropertyValue;
			string key = iPropValue.GetID();
			GridCellAddress gcAddress = this.PropValueCellAddress[key] as GridCellAddress;

			ColorEventArgs colorEventArgs = new ColorEventArgs();
			colorEventArgs.PropValueObj = iPropValue;
			colorEventArgs.Row = gcAddress.Row;
			colorEventArgs.Col = gcAddress.Col;
			
			if(ForeColorChanged != null)
				ForeColorChanged(this,colorEventArgs);
		}

		public void BackColorChangedHandler(object sender,EventArgs e)
		{
			IPropertyValue iPropValue = sender as IPropertyValue;
			string key = iPropValue.GetID();
			GridCellAddress gcAddress = this.PropValueCellAddress[key] as GridCellAddress;

			ColorEventArgs colorEventArgs = new ColorEventArgs();
			colorEventArgs.PropValueObj = iPropValue;
			colorEventArgs.Row = gcAddress.Row;
			colorEventArgs.Col = gcAddress.Col;
			
			if(BackColorChanged != null)
				BackColorChanged(this,colorEventArgs);
		}
		public void IsTrackChangedHandler(object sender,EventArgs e)
		{
			IProperty iPropObj = sender as IProperty;
			foreach(LRRMode mode in LRRModel.ProjectModes)
			{
				IPropertyValue iPropValueObj = iPropObj.GetPropertyValueForMode(mode.ModeId) as IPropertyValue; 
				if(iPropValueObj != null)
				{
					string key = iPropValueObj.GetID(); 
					GridCellAddress gcAddress = this.PropValueCellAddress[key] as GridCellAddress; 
					ColorEventArgs colorEventArgs = new ColorEventArgs();
					colorEventArgs.PropValueObj = iPropValueObj;
					colorEventArgs.Row = gcAddress.Row;
					colorEventArgs.Col = gcAddress.Col;   
					colorEventArgs.IsRecordedValue = iPropObj.GetIsTrackedValue();
                    colorEventArgs.IsTrendedValue = iPropObj.GetTrended(); 
					if(IsTrackChanged != null)
						IsTrackChanged(this,colorEventArgs);
                    if (IsTrendedChanged != null)
                        IsTrendedChanged(this, colorEventArgs);
				}
			}
		}

		#endregion

		#region Event 
		public event ForeColorChangesDel ForeColorChanged;
		public event BackColorChangesDel BackColorChanged;
		public event IsTrackChangesDel	 IsTrackChanged;
        public event IsTrendedChangesDel IsTrendedChanged;
		
		#endregion

		public delegate void ForeColorChangesDel(object sender,ColorEventArgs e);
		public delegate void BackColorChangesDel(object sender,ColorEventArgs e);
		public delegate void IsTrackChangesDel(object sender,ColorEventArgs e);
        public delegate void IsTrendedChangesDel(object sender, ColorEventArgs e);
		
		#region Properties
		public ArrayList SortManagerList
		{
			get{return _sortManagerList;}
			set
			{
				_sortManagerList = value;
			}
		}

		public Hashtable UIUOMHash
		{
			get{return _uiUOMHash;}
			set
			{
				_uiUOMHash = value;
			}
		}

		public object[,] GridData
		{
			get{return _gridData;}
			set
			{
				_gridData = value;
			}
		}

		public ArrayList ColHeaderCellList
		{
			get{return _colHeaderCellList;}
			set
			{
				_colHeaderCellList = value;
			}
		}

		public LRRBindingManager BindingManager
		{
			get{return _bindingManager;}
			set
			{
				_bindingManager = value;
			}
		}

		public Hashtable RowHeader
		{
			get{return _rowHeader;}
			set
			{
				_rowHeader = value;
			}
		}
		public Hashtable ColHeader
		{
			get{return _colHeader;}
			set
			{
				_colHeader = value;
			}
		}
		public Hashtable PropValueCellAddress
		{
			get{return _propValueCellAddress;}
			set
			{
				_propValueCellAddress = value;
			}
		}

		public string HeaderText
		{
			get{return _headerText;}
			set
			{
				_headerText = value;
			}
		}
		#endregion
	}

	
	public class GridCellAddress
	{
		public GridCellAddress(int row,int col)
		{
			_row = row;
			_col = col;
		}
		#region Properties
		public int Row
		{
			get{return _row;}
			set
			{
				_row = value;
			}
		}

		public int Col
		{
			get{return _col;}
			set
			{
				_col = value;
			}
		}
		#endregion

		#region Fields
		private int _row;
		private int _col;
		#endregion
	}
}
