using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;

using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.Utils.MessageLogger;
using Symcon.Database;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;
namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// File Dialog to open and Save Graphics
	/// </summary>
	public class FileDialog : System.Windows.Forms.Form
	{
		#region Fields

		private System.Windows.Forms.Label						lblFileName;
		private System.Windows.Forms.TextBox					txtFileName;
		private System.Windows.Forms.Label						lblDescriptin;
		private System.Windows.Forms.TextBox					txtDescription;
		private System.Windows.Forms.Button						btnSave;
		private System.Windows.Forms.Button						btnCancel;
		private LRRSavedViews									_lrrSavedViews;
		private LRRApp											_app;
		private System.Windows.Forms.ImageList					smallImageList;
		private System.ComponentModel.IContainer				components;
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl _gridSavedViews;
		private string											_fileID;
		private System.Windows.Forms.CheckBox					chkPublish;
		private System.Windows.Forms.Splitter					splitter1;
		private System.Windows.Forms.Splitter					splitter2;
		private System.Windows.Forms.Panel						pnlMain;
		private System.Windows.Forms.Panel						pnlTop;
		private System.Windows.Forms.Panel						pnlPublish;
		private System.Windows.Forms.Label						lblSelectPublishLevel;
		private GraphicsManagers.FileDialogMode				_dialogMode;
		private LRRSearchGroupLevels							_lrrSearchGroupLevels=null; //Levels Object
		private LRRSearchGroupLevelLinks						_lrrSearchGroupLevelLinks = null; //Level Links object
		private System.Windows.Forms.TreeView					trvLvls;
		//private LRRSearchGroupLevel								_selectedLevel = null;
		private	ArrayList										_selections = new ArrayList();
		private int												_countOfDelete = -1;
		private object											_selectedObject;
		private	bool											_deleteRecord = false;
        private SplitContainer splitContainer1;
        private SplitContainer splitContainer2;
		private GraphicsManagers.UserMode						_userMode;

		#endregion

		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="FileDialog"/> class.
        /// </summary>
		public FileDialog()
		{
			InitializeComponent();
			InternationalizationUtils.GetControlText(this,null);  
		}


        /// <summary>
        /// Initializes a new instance of the <see cref="FileDialog"/> class.
        /// </summary>
        /// <param name="mode">The mode.</param>
        /// <param name="userMode">The user mode.</param>
		public FileDialog(GraphicsManagers.FileDialogMode mode , GraphicsManagers.UserMode userMode)
		{
			_dialogMode = mode;
			_userMode = userMode;
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			if(mode == GraphicsManagers.FileDialogMode.Open)
			{
				InitializeComponent();
				InternationalizationUtils.GetControlText(this,null);  
				InitDataSource();
				chkPublish.Enabled = false;
				pnlPublish.Enabled = false;
				this.btnSave.Text = LRRHeaderText.GetHeaderString("SID_OPEN");
				this.lblFileName.Text = LRRHeaderText.GetHeaderString("SID_OPEN");;
				this.Text =LRRHeaderText.GetHeaderString("SID_OPEN");
				this.splitter1.Location.Offset(pnlPublish.Width,0);
			}
			else if(mode == GraphicsManagers.FileDialogMode.SaveAs)
			{
				InitializeComponent();
				InternationalizationUtils.GetControlText(this,null);  
				InitDataSource();
				//chkPublish.Enabled = true;
				chkPublish.Enabled = UIAccessController.UIACLController.IsUIEntityAccessible("EnablePublishing");
				this.btnSave.Text = LRRHeaderText.GetHeaderString("SID_SAVES");
				this.lblFileName.Text = LRRHeaderText.GetHeaderString("SID_SAVEAS");
				this.Text = LRRHeaderText.GetHeaderString("SID_SAVEAS");
				PopulateTreeView(trvLvls,_lrrSearchGroupLevels, _lrrSearchGroupLevelLinks);
			}
		}


		#endregion

		#region Overrides

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#endregion
	
		#region Windows Form Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FileDialog));
            this.pnlMain = new System.Windows.Forms.Panel();
            this.pnlTop = new System.Windows.Forms.Panel();
            this._gridSavedViews = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.pnlPublish = new System.Windows.Forms.Panel();
            this.splitter2 = new System.Windows.Forms.Splitter();
            this.lblSelectPublishLevel = new System.Windows.Forms.Label();
            this.trvLvls = new System.Windows.Forms.TreeView();
            this.chkPublish = new System.Windows.Forms.CheckBox();
            this.btnCancel = new System.Windows.Forms.Button();
            this.btnSave = new System.Windows.Forms.Button();
            this.txtDescription = new System.Windows.Forms.TextBox();
            this.lblDescriptin = new System.Windows.Forms.Label();
            this.txtFileName = new System.Windows.Forms.TextBox();
            this.lblFileName = new System.Windows.Forms.Label();
            this.smallImageList = new System.Windows.Forms.ImageList(this.components);
            this.splitContainer1 = new System.Windows.Forms.SplitContainer();
            this.splitContainer2 = new System.Windows.Forms.SplitContainer();
            this.pnlMain.SuspendLayout();
            this.pnlTop.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this._gridSavedViews)).BeginInit();
            this.pnlPublish.SuspendLayout();
            this.splitContainer1.Panel1.SuspendLayout();
            this.splitContainer1.Panel2.SuspendLayout();
            this.splitContainer1.SuspendLayout();
            this.splitContainer2.Panel1.SuspendLayout();
            this.splitContainer2.Panel2.SuspendLayout();
            this.splitContainer2.SuspendLayout();
            this.SuspendLayout();
            // 
            // pnlMain
            // 
            this.pnlMain.Controls.Add(this.splitContainer1);
            this.pnlMain.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlMain.Location = new System.Drawing.Point(0, 0);
            this.pnlMain.Name = "pnlMain";
            this.pnlMain.Size = new System.Drawing.Size(586, 335);
            this.pnlMain.TabIndex = 0;
            // 
            // pnlTop
            // 
            this.pnlTop.Controls.Add(this._gridSavedViews);
            this.pnlTop.Controls.Add(this.splitter1);
            this.pnlTop.Controls.Add(this.pnlPublish);
            this.pnlTop.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlTop.Location = new System.Drawing.Point(0, 0);
            this.pnlTop.Name = "pnlTop";
            this.pnlTop.Size = new System.Drawing.Size(586, 225);
            this.pnlTop.TabIndex = 10;
            // 
            // _gridSavedViews
            // 
            this._gridSavedViews.Appearance.TopLeftHeaderCell.Clickable = false;
            this._gridSavedViews.Appearance.TopLeftHeaderCell.Enabled = false;
            this._gridSavedViews.Appearance.TopLeftHeaderCell.MergeCell = Syncfusion.Windows.Forms.Grid.GridMergeCellDirection.ColumnsInRow;
            this._gridSavedViews.Appearance.TopLeftHeaderCell.ShowButtons = Syncfusion.Windows.Forms.Grid.GridShowButtons.ShowCurrentRow;
            this._gridSavedViews.BackColor = System.Drawing.SystemColors.Window;
            this._gridSavedViews.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this._gridSavedViews.Dock = System.Windows.Forms.DockStyle.Fill;
            this._gridSavedViews.Location = new System.Drawing.Point(0, 0);
            this._gridSavedViews.Name = "_gridSavedViews";
            this._gridSavedViews.Size = new System.Drawing.Size(383, 225);
            this._gridSavedViews.TabIndex = 8;
            this._gridSavedViews.TableDescriptor.AllowEdit = false;
            this._gridSavedViews.TableDescriptor.AllowNew = false;
            this._gridSavedViews.TableDescriptor.TopLevelGroupOptions.IsExpandedInitialValue = true;
            this._gridSavedViews.TableDescriptor.TopLevelGroupOptions.ShowCaptionPlusMinus = false;
            this._gridSavedViews.TableDescriptor.TopLevelGroupOptions.ShowColumnHeaders = true;
            this._gridSavedViews.TableDescriptor.TopLevelGroupOptions.ShowFilterBar = true;
            this._gridSavedViews.TableOptions.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)(((((((((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Row | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Column)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Table)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Cell)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Multiple)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Shift)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Keyboard)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.AlphaBlend)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.MixRangeType)));
            this._gridSavedViews.TableOptions.ListBoxSelectionCurrentCellOptions = Syncfusion.Windows.Forms.Grid.Grouping.GridListBoxSelectionCurrentCellOptions.None;
            this._gridSavedViews.TableOptions.ListBoxSelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
            this._gridSavedViews.TableOptions.ShowRowHeader = true;
            this._gridSavedViews.Text = "Saved Views";
            this._gridSavedViews.TopLevelGroupOptions.IsExpandedInitialValue = true;
            this._gridSavedViews.TopLevelGroupOptions.ShowCaptionPlusMinus = false;
            this._gridSavedViews.TopLevelGroupOptions.ShowColumnHeaders = true;
            this._gridSavedViews.TopLevelGroupOptions.ShowFilterBar = true;
            this._gridSavedViews.VersionInfo = "3.301.0.0";
            this._gridSavedViews.RecordDeleting += new Syncfusion.Grouping.RecordEventHandler(this.gridSavedViews_RecordDeleting);
            this._gridSavedViews.TableControlCellDoubleClick += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventHandler(this.gridSavedViews_TableControlCellDoubleClick);
            this._gridSavedViews.TableControlCellClick += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventHandler(this.gridSavedViews_TableControlCellClick);
            // 
            // splitter1
            // 
            this.splitter1.Dock = System.Windows.Forms.DockStyle.Right;
            this.splitter1.Location = new System.Drawing.Point(383, 0);
            this.splitter1.Name = "splitter1";
            this.splitter1.Size = new System.Drawing.Size(3, 225);
            this.splitter1.TabIndex = 11;
            this.splitter1.TabStop = false;
            // 
            // pnlPublish
            // 
            this.pnlPublish.Controls.Add(this.splitContainer2);
            this.pnlPublish.Controls.Add(this.splitter2);
            this.pnlPublish.Dock = System.Windows.Forms.DockStyle.Right;
            this.pnlPublish.Location = new System.Drawing.Point(386, 0);
            this.pnlPublish.Name = "pnlPublish";
            this.pnlPublish.Size = new System.Drawing.Size(200, 225);
            this.pnlPublish.TabIndex = 11;
            // 
            // splitter2
            // 
            this.splitter2.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.splitter2.Enabled = false;
            this.splitter2.Location = new System.Drawing.Point(0, 221);
            this.splitter2.Name = "splitter2";
            this.splitter2.Size = new System.Drawing.Size(200, 4);
            this.splitter2.TabIndex = 12;
            this.splitter2.TabStop = false;
            // 
            // lblSelectPublishLevel
            // 
            this.lblSelectPublishLevel.Dock = System.Windows.Forms.DockStyle.Fill;
            this.lblSelectPublishLevel.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.lblSelectPublishLevel.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblSelectPublishLevel.Location = new System.Drawing.Point(0, 0);
            this.lblSelectPublishLevel.Name = "lblSelectPublishLevel";
            this.lblSelectPublishLevel.Size = new System.Drawing.Size(200, 30);
            this.lblSelectPublishLevel.TabIndex = 11;
            this.lblSelectPublishLevel.Tag = "SID_SELECT_LEVEL";
            // 
            // trvLvls
            // 
            this.trvLvls.Dock = System.Windows.Forms.DockStyle.Fill;
            this.trvLvls.Location = new System.Drawing.Point(0, 0);
            this.trvLvls.Name = "trvLvls";
            this.trvLvls.Size = new System.Drawing.Size(200, 187);
            this.trvLvls.TabIndex = 10;
            // 
            // chkPublish
            // 
            this.chkPublish.Location = new System.Drawing.Point(18, 10);
            this.chkPublish.Name = "chkPublish";
            this.chkPublish.Size = new System.Drawing.Size(168, 16);
            this.chkPublish.TabIndex = 9;
            this.chkPublish.Tag = "SID_PUBLISH_TO_OTHER";
            // 
            // btnCancel
            // 
            this.btnCancel.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
            this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.btnCancel.Location = new System.Drawing.Point(485, 66);
            this.btnCancel.Name = "btnCancel";
            this.btnCancel.Size = new System.Drawing.Size(80, 24);
            this.btnCancel.TabIndex = 5;
            this.btnCancel.Tag = "SID_CANCEL";
            // 
            // btnSave
            // 
            this.btnSave.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
            this.btnSave.Location = new System.Drawing.Point(485, 34);
            this.btnSave.Name = "btnSave";
            this.btnSave.Size = new System.Drawing.Size(80, 24);
            this.btnSave.TabIndex = 4;
            this.btnSave.Tag = "SID_OK";
            this.btnSave.Click += new System.EventHandler(this.btnSave_Click);
            // 
            // txtDescription
            // 
            this.txtDescription.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
            this.txtDescription.Location = new System.Drawing.Point(101, 66);
            this.txtDescription.Multiline = true;
            this.txtDescription.Name = "txtDescription";
            this.txtDescription.Size = new System.Drawing.Size(376, 32);
            this.txtDescription.TabIndex = 3;
            // 
            // lblDescriptin
            // 
            this.lblDescriptin.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
            this.lblDescriptin.Location = new System.Drawing.Point(18, 66);
            this.lblDescriptin.Name = "lblDescriptin";
            this.lblDescriptin.Size = new System.Drawing.Size(72, 24);
            this.lblDescriptin.TabIndex = 2;
            this.lblDescriptin.Tag = "SID_DESCRIPTION";
            // 
            // txtFileName
            // 
            this.txtFileName.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
            this.txtFileName.Location = new System.Drawing.Point(101, 34);
            this.txtFileName.Name = "txtFileName";
            this.txtFileName.Size = new System.Drawing.Size(376, 20);
            this.txtFileName.TabIndex = 2;
            // 
            // lblFileName
            // 
            this.lblFileName.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
            this.lblFileName.Location = new System.Drawing.Point(18, 34);
            this.lblFileName.Name = "lblFileName";
            this.lblFileName.Size = new System.Drawing.Size(72, 24);
            this.lblFileName.TabIndex = 0;
            this.lblFileName.Tag = "SID_FILENAME";
            // 
            // smallImageList
            // 
            this.smallImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("smallImageList.ImageStream")));
            this.smallImageList.TransparentColor = System.Drawing.Color.Fuchsia;
            this.smallImageList.Images.SetKeyName(0, "");
            this.smallImageList.Images.SetKeyName(1, "");
            this.smallImageList.Images.SetKeyName(2, "");
            this.smallImageList.Images.SetKeyName(3, "");
            this.smallImageList.Images.SetKeyName(4, "");
            this.smallImageList.Images.SetKeyName(5, "");
            this.smallImageList.Images.SetKeyName(6, "");
            this.smallImageList.Images.SetKeyName(7, "");
            this.smallImageList.Images.SetKeyName(8, "");
            this.smallImageList.Images.SetKeyName(9, "");
            this.smallImageList.Images.SetKeyName(10, "");
            this.smallImageList.Images.SetKeyName(11, "");
            // 
            // splitContainer1
            // 
            this.splitContainer1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.splitContainer1.FixedPanel = System.Windows.Forms.FixedPanel.Panel2;
            this.splitContainer1.IsSplitterFixed = true;
            this.splitContainer1.Location = new System.Drawing.Point(0, 0);
            this.splitContainer1.Name = "splitContainer1";
            this.splitContainer1.Orientation = System.Windows.Forms.Orientation.Horizontal;
            // 
            // splitContainer1.Panel1
            // 
            this.splitContainer1.Panel1.Controls.Add(this.pnlTop);
            // 
            // splitContainer1.Panel2
            // 
            this.splitContainer1.Panel2.Controls.Add(this.chkPublish);
            this.splitContainer1.Panel2.Controls.Add(this.lblFileName);
            this.splitContainer1.Panel2.Controls.Add(this.btnCancel);
            this.splitContainer1.Panel2.Controls.Add(this.txtFileName);
            this.splitContainer1.Panel2.Controls.Add(this.btnSave);
            this.splitContainer1.Panel2.Controls.Add(this.lblDescriptin);
            this.splitContainer1.Panel2.Controls.Add(this.txtDescription);
            this.splitContainer1.Size = new System.Drawing.Size(586, 335);
            this.splitContainer1.SplitterDistance = 225;
            this.splitContainer1.TabIndex = 11;
            // 
            // splitContainer2
            // 
            this.splitContainer2.Dock = System.Windows.Forms.DockStyle.Fill;
            this.splitContainer2.FixedPanel = System.Windows.Forms.FixedPanel.Panel1;
            this.splitContainer2.IsSplitterFixed = true;
            this.splitContainer2.Location = new System.Drawing.Point(0, 0);
            this.splitContainer2.Name = "splitContainer2";
            this.splitContainer2.Orientation = System.Windows.Forms.Orientation.Horizontal;
            // 
            // splitContainer2.Panel1
            // 
            this.splitContainer2.Panel1.Controls.Add(this.lblSelectPublishLevel);
            // 
            // splitContainer2.Panel2
            // 
            this.splitContainer2.Panel2.Controls.Add(this.trvLvls);
            this.splitContainer2.Size = new System.Drawing.Size(200, 221);
            this.splitContainer2.SplitterDistance = 30;
            this.splitContainer2.TabIndex = 13;
            // 
            // FileDialog
            // 
            this.AcceptButton = this.btnSave;
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.CancelButton = this.btnCancel;
            this.ClientSize = new System.Drawing.Size(586, 335);
            this.Controls.Add(this.pnlMain);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.MinimumSize = new System.Drawing.Size(592, 352);
            this.Name = "FileDialog";
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Tag = "FileDialog_Graphics";
            this.Load += new System.EventHandler(this.FileDialog_Load);
            this.Closing += new System.ComponentModel.CancelEventHandler(this.FileDialog_Closing);
            this.pnlMain.ResumeLayout(false);
            this.pnlTop.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this._gridSavedViews)).EndInit();
            this.pnlPublish.ResumeLayout(false);
            this.splitContainer1.Panel1.ResumeLayout(false);
            this.splitContainer1.Panel2.ResumeLayout(false);
            this.splitContainer1.Panel2.PerformLayout();
            this.splitContainer1.ResumeLayout(false);
            this.splitContainer2.Panel1.ResumeLayout(false);
            this.splitContainer2.Panel2.ResumeLayout(false);
            this.splitContainer2.ResumeLayout(false);
            this.ResumeLayout(false);

		}
		

		
		#endregion
		
		#region Properties

		public string FileName
		{
			get
			{
				return txtFileName.Text;
			}
			set
			{
				txtFileName.Text = value;
			}
		}

		
		public string FileDescription
		{
			get{return txtDescription.Text;}
			set
			{
				if(txtDescription.Text !=null)
				{
					txtDescription.Text = value;
				}
				else txtDescription.Text = "";
			}
		}
		
		public string FileID
		{
			get
			{
				return _fileID;
			}
			set
			{
				_fileID = value;
			}
		}

		public object SelectedObject
		{
			get
			{
				return _selectedObject;
			}
			set
			{
				_selectedObject = value;
			}

		}


		public ArrayList SelectedRecords 
		{
			get
			{
				return _selections;
			}
		}
		

		public bool IsPublished
		{
			get
			{
				return chkPublish.Checked;
			}
		}


		public ArrayList Selections
		{
			get
			{
				return _selections;
			}
		}
	
		#endregion 

		#region Private Methods

        /// <summary>
        /// Inits the data source.
        /// </summary>
		private void InitDataSource()
		{
			_lrrSavedViews = LRRModel.SavedViews;
			//Initialize and populate the levels
			_lrrSearchGroupLevels = new LRRSearchGroupLevels();
			_app.DBManagerObj.PopulateCollection(_lrrSearchGroupLevels);
			//Initialize and populate the level links
			_lrrSearchGroupLevelLinks = new LRRSearchGroupLevelLinks();
			_app.DBManagerObj.PopulateCollection(_lrrSearchGroupLevelLinks);
		}


		/// <summary>
		/// 1. Initialises the Columns of the Grid using the type of the collection
		/// 2. Sets the Filter to the Grid.
		/// 3. Hooks up to the relative cell helper instance.
		/// </summary>
		private void InitializeGrid()
		{
			SyncGridUtilities.InitializeGridColumns(_gridSavedViews,_lrrSavedViews.ItemType,null);
			_gridSavedViews.DataSource = _lrrSavedViews;
			for(int i = 0; i<_gridSavedViews.TableDescriptor.Columns.Count; i++)
			{
				_gridSavedViews.TableDescriptor.Columns[i].AllowFilter = true;
				_gridSavedViews.TableDescriptor.Columns[i].Appearance.GetStyle(GridTableCellType.AnyRecordFieldCell).CellType = "Static";
				_gridSavedViews.TableDescriptor.Columns[i].Appearance.GetStyle(GridTableCellType.AlternateRecordFieldCell).BackColor = Color.White;
			}
			_gridSavedViews.TableDescriptor.Appearance.GetStyle(GridTableCellType.AlternateRecordFieldCell).AutoSize = true;
			_gridSavedViews.TopLevelGroupOptions.ShowFilterBar = true;
			_gridSavedViews.TopLevelGroupOptions.CaptionText = _lrrSavedViews.Count.ToString(Symcon.BusinessObjects.SymConverter.Culture) + " Views";
			
		}


        /// <summary>
        /// Gets the selected records.
        /// </summary>
		private void GetSelectedRecords()
		{
			int top,bottom;
			_selections.Clear();
			GridRangeInfoList list = _gridSavedViews.TableControl.Model.SelectedRanges;
			if(list.Count == 1)
			{
				GridRangeInfo range = list.ActiveRange;
				int selRows = range.Height;
				if(_gridSavedViews.TopLevelGroupOptions.ShowFilterBar == true)  
				{
					top  = range.Top-4;
					bottom = range.Bottom-4;
				}
				else
				{
					top = range.Top-3;
					bottom = range.Bottom-3;
				}
						
				for(int i=selRows;i>0;i--)
				{
					try
					{
						object obj = _gridSavedViews.TableModel.Table.Records[bottom].GetData();
						_selections.Add(_gridSavedViews.TableModel.Table.Records[bottom].GetData());
						bottom--;
					}
					catch(Exception ex)
					{
						Logger.LogMessage(typeof(FileDialog),"GetSelectedRecords :"+ex.Message  ,Logger.Level.Debug);
					}
				}
			}					
		}


		/// <summary>
		/// Validate The File Name
		/// </summary>
		/// <returns></returns>
		private bool ValidateFileName()
		{
			bool replaceFile = true;
			if(this.FileName != null)
			{
				this.FileID = null;
				foreach(LRRSavedView lrrsv in _lrrSavedViews)
				{
					if(lrrsv.SavedViewName == this.FileName)
					{
						string[] s=new string[1];
						s[0] = this.FileName;
						DialogResult result = LRRMessagBox.ShowMessage(this,"SID_SAVEAS_DIALOG","SID_LABEL",s,MessageBoxButtons.YesNo,MessageBoxIcon.Question);    
						if(result == DialogResult.Yes)
						{
							this.FileID = lrrsv.ID;
							replaceFile = true;
							break;
						}
						else
						{
							replaceFile = false;
							break;
						}
					}
				}
				if(this.FileID == null)
				{
					this.FileID = Guid.NewGuid().ToString();
				}
				return replaceFile;
			}
			else 
			{
				replaceFile = false;
				return replaceFile;
			}
		}

		/// <summary>
		/// Method populates the Tree
		/// </summary>
		/// <param name="treeView"></param>
		/// <param name="levels"></param>
		/// <param name="levelLinks"></param>
		/// <returns></returns>
		private bool PopulateTreeView(TreeView treeView,LRRSearchGroupLevels levels,LRRSearchGroupLevelLinks levelLinks)
		{
			treeView.Nodes.Clear();
			try
			{
				Hashtable				levelHash = new Hashtable();
				Hashtable				nodesHash = new Hashtable();
				Hashtable				childParent = new Hashtable();

				foreach(LRRSearchGroupLevel l in levels)
				{
					levelHash.Add(l.LevelID,l);
				}
				foreach(LRRSearchGroupLevelLink ll in levelLinks)
				{
					childParent.Add(ll.ChildID,ll.ParentID);

					if(nodesHash.ContainsKey(ll.ParentID))
					{
						TreeNode foundNode = nodesHash[ll.ParentID] as TreeNode;
						LRRSearchGroupLevel foundLevel = levelHash[ll.ChildID] as LRRSearchGroupLevel; 

						if(foundLevel.Type == "Level")
						{
							TreeNode childNode = new TreeNode(foundLevel.Name);
							childNode.Tag = foundLevel;
							foundNode.Nodes.Add(childNode);
						}
					}
					else
					{
						LRRSearchGroupLevel parentLevel = levelHash[ll.ParentID] as LRRSearchGroupLevel;
						LRRSearchGroupLevel childLevel = levelHash[ll.ChildID] as LRRSearchGroupLevel;

						if(childLevel.Type == "Level")
						{
							TreeNode parentNode = new TreeNode(parentLevel.Name);
							parentNode.Tag = parentLevel;
							TreeNode childNode = new TreeNode(childLevel.Name);
							childNode.Tag = childLevel;
							parentNode.Nodes.Add(childNode);
							nodesHash.Add(ll.ParentID,parentNode);
						}
					}
				}

				foreach(object k in childParent.Keys)
				{
					string childID = (string)k;
					string parentID = (string)childParent[k];
					TreeNode  childNode = null;
					TreeNode  parentNode = null;
					if(nodesHash.ContainsKey(childID))
					{
						childNode = nodesHash[childID] as TreeNode;
					}
					if(nodesHash.ContainsKey(parentID))
					{
						parentNode = nodesHash[parentID] as TreeNode;
					}
					if((parentNode != null) && (childNode != null))
					{
						int index = -1;
						foreach(TreeNode f in parentNode.Nodes)
						{
							if(f.Tag == childNode.Tag)
							{
								index = f.Index;
								break;
							}
						}
						if(index != -1)
							parentNode.Nodes[index] = childNode;
					}
				}
				TreeNode rootNode = null;
				if( nodesHash.Count > 0)
				{
					rootNode = nodesHash["0"] as TreeNode;
				}
				else
				{
					LRRSearchGroupLevel srchGroupLevel = levelHash["0"] as LRRSearchGroupLevel;
					rootNode = new TreeNode(srchGroupLevel.Name);
					rootNode.Tag = srchGroupLevel;
				}
				treeView.Nodes.Add(rootNode);
				treeView.Nodes[0].Expand();
				return true;

			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(SaveQueryForm),"PopulateTreeView :"+ex.Message  ,Logger.Level.Debug);
				return false;
			}
		}

		#endregion 

		#region Events

        /// <summary>
        /// Handles the Load event of the FileDialog control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void FileDialog_Load(object sender, EventArgs e)
		{
			InitializeGrid();
			ApplicationPersistanceUtil.ApplyGridSettings(_gridSavedViews, this as ILRRUIContainer, this.Tag);
		}


        /// <summary>
        /// Handles the TableControlCellDoubleClick event of the gridSavedViews control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs"/> instance containing the event data.</param>
		private void gridSavedViews_TableControlCellDoubleClick(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs e)
		{
			if(e.TableControl.CurrentCell.Renderer.GetType() == typeof(Syncfusion.Windows.Forms.Grid.GridStaticCellRenderer))
			{
				if(_dialogMode == GraphicsManagers.FileDialogMode.SaveAs)
				{
					bool update  = ValidateFileName();
					if(update)
					{
						this.DialogResult = DialogResult.OK;
					}
				}
				else
				{
					GetSelectedRecords();
					this.DialogResult = DialogResult.OK;
				}
			}
		}


        /// <summary>
        /// Handles the TableControlCellClick event of the gridSavedViews control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs"/> instance containing the event data.</param>
		private void gridSavedViews_TableControlCellClick(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs e)
		{
			CurrencyManager cm = (CurrencyManager)_gridSavedViews.BindingContext[this._gridSavedViews.DataSource];
			LRRSavedView lrrsv = (LRRSavedView)cm.Current;
			SelectedObject = cm.Current;
			txtFileName.Text = lrrsv.SavedViewName;;
			txtDescription.Text = lrrsv.SavedViewDescription;;
			_fileID = lrrsv.ID;
		}


        /// <summary>
        /// Handles the Click event of the btnSave control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void btnSave_Click(object sender, System.EventArgs e)
		{
			if(_dialogMode == GraphicsManagers.FileDialogMode.SaveAs)
			{
				if(txtFileName.Text.Trim() == "")
				{
					LRRMessagBox.ShowMessage(this,"SID_FILENAME_BLANK_ERROR","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Error);     
					return;
				}
				bool update  = ValidateFileName();
				if(update)
				{
					bool isCommitable = true;
					LRRSearchGroupLevel currentLevel;
					if(trvLvls.SelectedNode != null)
						currentLevel = trvLvls.SelectedNode.Tag as LRRSearchGroupLevel;
					else
						currentLevel = trvLvls.Nodes[0].Tag as LRRSearchGroupLevel;
					foreach(LRRSearchGroupLevelLink currentLvlLnk in _lrrSearchGroupLevelLinks)
					{
						if(currentLvlLnk.ParentID == currentLevel.LevelID && currentLvlLnk.ChildID == FileID)
						{
							isCommitable = false;
							break;
						}
					}
					if(isCommitable)
					{
						if(currentLevel != null)
						{
							LRRSearchGroupLevel lrrSearchGrpLvl = new LRRSearchGroupLevel();
							lrrSearchGrpLvl.LevelID = this.FileID;
							lrrSearchGrpLvl.Name = this.FileName;
							lrrSearchGrpLvl.Description = this.FileDescription;
							lrrSearchGrpLvl.IsPublished = this.IsPublished;
							lrrSearchGrpLvl.Type = "Graphics";
							_app.DBManagerObj.CommitObject(lrrSearchGrpLvl);
							LRRSearchGroupLevelLink lvlLink = new LRRSearchGroupLevelLink();
							lvlLink.ParentID = currentLevel.LevelID;
							lvlLink.ChildID = FileID;
							_app.DBManagerObj.CommitObject(lvlLink);

						}
					}
					this.DialogResult = DialogResult.OK;
				}
			}
			else
			{
				GetSelectedRecords();
				if(this.SelectedRecords.Count == 0)
				{
					LRRMessagBox.ShowMessage(this,"SID_VALIDFILE","SID_INVALID",MessageBoxButtons.OK,MessageBoxIcon.Information);  
					this.DialogResult = DialogResult.None;

				}
				else
				{
					this.DialogResult = DialogResult.OK;
				}
			}
		}

        /// <summary>
        /// Handles the RecordDeleting event of the gridSavedViews control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Grouping.RecordEventArgs"/> instance containing the event data.</param>
		private void gridSavedViews_RecordDeleting(object sender, Syncfusion.Grouping.RecordEventArgs e)
		{
			if(_userMode != GraphicsManagers.UserMode.IMPLEMENTOR)
			{
				e.Cancel = true;
				return;
			}
			
			if(_countOfDelete == -1)
			{
			
				//First Call
				LRRSavedView lrrsv = null;
				GetSelectedRecords();
				_countOfDelete = this.SelectedRecords.Count - 2;
				string str = "";
				for(int i = 0; i <this.SelectedRecords.Count;i++)
				{
					lrrsv  = this.SelectedRecords[i] as LRRSavedView;
					str = str + "\n " + lrrsv.SavedViewName;
				}
				string[] s = new string[1];
				s[0] = str ;
				DialogResult dlgResult = LRRMessagBox.ShowMessage(this,"SID_DELETE_CONFIRM","SID_LABEL",s,MessageBoxButtons.YesNo,MessageBoxIcon.Question,MessageBoxDefaultButton.Button2);      
				if(dlgResult == DialogResult.Yes)
				{
					for(int i = 0; i <this.SelectedRecords.Count;i++)
					{
						LRRSavedView lrrsvToDelete  = this.SelectedRecords[i] as LRRSavedView;
						lrrsvToDelete.ObjectState = DBOperation.DELETE;
						_app.DBManagerObj.CommitObject(lrrsvToDelete);
						_deleteRecord = true;
					
						foreach(LRRSearchGroupLevel lvlToDelete in _lrrSearchGroupLevels)
						{
							if(lvlToDelete.LevelID == lrrsvToDelete.ID)
							{
								lvlToDelete.ObjectState = DBOperation.DELETE;
								_app.DBManagerObj.CommitObject(lvlToDelete);
								break;
							}
						}
					}
				}
				else
				{
					e.Cancel = true;
					_deleteRecord = false;
				}
			}
			else
			{
				_countOfDelete--;
				if(!_deleteRecord)
					e.Cancel = true;
			}
		}


        /// <summary>
        /// Handles the Closing event of the FileDialog control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
		private void FileDialog_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			ApplicationPersistanceUtil.SaveGridSettings(_gridSavedViews, this as ILRRUIContainer, this.Tag);
		}

		#endregion
	}
}