using System;
using System.Drawing;
using Symcon.Lightridge.LRRUI.Controls;
using Symcon.Lightridge.LRRUI;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.BusinessObjects;
using System.Windows.Forms;
using Symcon.Utils.MessageLogger;
using Syncfusion.Windows.Forms.Diagram.Controls;
using Syncfusion.Windows.Forms.Diagram;
using Symcon.BusinessObjects;


namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for GraphicsManager.
	/// </summary>
	public class GraphicsManager
	{
		public GraphicsManager()
		{
			_labelControllerObj = new LabelController();
			_labelLookupObj = new LabelLookup();
			_canvasObj = new Canvas();
            _canvasObj.ScrollGranularity = .05f;
			_lrrBindingManger = new LRRBindingManager();
			_lrrBindingManger.BindingCalled+=new Symcon.LightRidge.PEAdvisor.DataStore.LRRBindingManager.BindingCallDel(_lrrBindingManger_BindingCalled);
			this.ID = Guid.NewGuid().ToString();
			_app = AppDomain.CurrentDomain.GetData("AppObject")	as LRRApp;
			WireCanvasEvents(_canvasObj);
		}

        public GraphicsManager(string fileID)
        {
            _labelControllerObj = new LabelController();
            _labelLookupObj = new LabelLookup();
            _canvasObj = new Canvas();
            _canvasObj.ScrollGranularity = .10f;
            LoadCanvas(fileID);
            _lrrBindingManger = new LRRBindingManager();
            _lrrBindingManger.BindingCalled += new Symcon.LightRidge.PEAdvisor.DataStore.LRRBindingManager.BindingCallDel(_lrrBindingManger_BindingCalled);
            this.ID = Guid.NewGuid().ToString();
            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            WireCanvasEvents(_canvasObj);
        }
		#region Public Properties 

		public string ID
		{
			get
			{
				return this._id;
			}
			set
			{
				_id = value;
			}

		}

		public Canvas CanvasObj
		{
			get{return _canvasObj;}
			set
			{
				_canvasObj = value;
			}
		}

		public LabelController LabelControllerObj
		{
			get{return _labelControllerObj;}
			set{_labelControllerObj = value;}
		}

		public LabelLookup LabelLookupObj
		{
			get{return _labelLookupObj;}
			set{_labelLookupObj = value;}
		}


		public LRRBindingManager LRRBindingManagerObj
		{
			get{return _lrrBindingManger;}
			set{_lrrBindingManger = value;}
		}

        public bool IsRunning
        {
            get { return _isRunning; }
            set { _isRunning = value; }
        }

		#endregion


		#region Public Methods

		public Canvas LoadCanvas(string fileID)
		{
			 GraphicsUtilities.LoadSavedView(fileID,CanvasObj);
			
			return CanvasObj;
		}

		public void BindSymbols(LRRModes selectedModes,GraphicsManagers.PropertiesMappingMode propertiesMappingMode,GraphicsManagers.UserMode userMode)
		{
			GraphicsUtilities.BindSymbols(this,selectedModes,propertiesMappingMode,userMode);
		}
        // bind symbols added by ashwin for handling custom formatting
        public void BindSymbols(LRRModes selectedModes, GraphicsManagers.PropertiesMappingMode propertiesMappingMode, GraphicsManagers.UserMode userMode,string firsttime)
        {
            GraphicsUtilities.BindSymbols(this, selectedModes, propertiesMappingMode, userMode, firsttime);
        }

		public void UpdateLabels(Symbol selectedSymbol, LRRModes selectedModes,CustomCollection selectedProperties,GraphicsManagers.PropertiesMappingMode propertiesMappingMode,GraphicsManagers.UserMode userMode,bool setEquipmentName)
		{
			
			if(selectedSymbol.GetType().Name == "Aggregates")
			{
				CustomCollection lrrAggregateProperties = selectedProperties as CustomCollection;
				GraphicsAggregatesUtilities.UpdateSymbolPropertiesMap(selectedSymbol,selectedModes,lrrAggregateProperties,this,propertiesMappingMode,userMode,this.CanvasObj.CreateGraphics());
			}
			else
			{
				CustomCollection lrrscps = selectedProperties as CustomCollection;
				GraphicsSiteComponentUtilities.UpdateSymbolPropertiesMap(selectedSymbol,selectedModes,lrrscps,this,propertiesMappingMode,userMode,this.CanvasObj.CreateGraphics(),setEquipmentName);
			}
		}

		#endregion

		#region Private Methods

	 	public void TerminateCanvas()
		{
			UnWireCanvasEvents(this.CanvasObj);
			
			this.LabelControllerObj = null;
			this._lrrBindingManger.BindingCalled-=new Symcon.LightRidge.PEAdvisor.DataStore.LRRBindingManager.BindingCallDel(_lrrBindingManger_BindingCalled);
			this.LRRBindingManagerObj = null;
		}


		private void UnWirePropertyValueEvent()
		{
			
		}

		private void WireCanvasEvents(Canvas Canvas)
		{
			CanvasObj.SelectionChanged +=new NodeCollectionEventHandler(CanvasObj_SelectionChanged);
			CanvasObj.ChildrenChangeComplete+=new NodeCollectionEventHandler(CanvasObj_ChildrenChangeComplete);
			CanvasObj.MapProperties+=new EventHandler(CanvasObj_MapProperties);
			CanvasObj.SelectDisplayUOM+=new EventHandler(CanvasObj_SelectDisplayUOM);
			CanvasObj.ViewTrend+=new EventHandler(CanvasObj_ViewTrend);
			CanvasObj.ShowAttributeDetails+=new EventHandler(CanvasObj_ShowAttributeDetails);
			CanvasObj.SetLabelsVisibility+=new EventHandler(CanvasObj_SetLabelsVisibility);
			CanvasObj.NodeDoubleClick+=new NodeMouseEventHandler(CanvasObj_NodeDoubleClick);
            CanvasObj.SelectedAttributeDetails += new EventHandler(CanvasObj_SelectedAttributeDetails);
            CanvasObj.MouseUp += new MouseEventHandler(CanvasObj_MouseUp);
			foreach(MenuItem mitem in CanvasObj.CanvasContextMenu.MenuItems)
			{
				if(mitem.Index == 1 || mitem.Index ==8 || mitem.Index ==9 || mitem.Index ==10 || mitem.Index >11)
					continue;
				else
					mitem.Click+=new EventHandler(OverRideValue_Click);
			}

			CanvasObj.CanvasContextMenu.Popup+=new EventHandler(CanvasContextMenu_Popup);
			GraphicsSiteComponentUtilities.LabelDoubleClick+=new NodeMouseEventHandler(GraphicsSiteComponentUtilities_LabelDoubleClick);
			GraphicsAggregatesUtilities.LabelDoubleClick+=new NodeMouseEventHandler(GraphicsSiteComponentUtilities_LabelDoubleClick);
			CanvasObj.NodeMouseEnter+=new NodeMouseEventHandler(Model_MouseEnter);
			CanvasObj.ChangeGridDimensions+=new EventHandler(CanvasObj_ChangeGridDimensions);
            CanvasObj.RemoveRow += new EventHandler(CanvasObj_RemoveRow);
            CanvasObj.RemoveColumn += new EventHandler(CanvasObj_RemoveColumn);
            CanvasObj.ResetGridCell += new EventHandler(CanvasObj_ResetGridCell);   
			 CanvasObj.ResetNode += new EventHandler(CanvasObj_ResetNode); 
			CanvasObj.SetCustomFormat+=new EventHandler(CanvasObj_SetCustomFormat);
            CanvasObj.SetCustomFormatGridCell += new EventHandler(CanvasObj_SetCustomFormatGridCell);
		}

        void CanvasObj_MouseUp(object sender, MouseEventArgs e)
        {
            if (MouseUp != null)
            {
                MouseUp(sender, e);
            }
        }

        
		private void UnWireCanvasEvents(Canvas Canvas)
		{
			CanvasObj.SelectionChanged -=new NodeCollectionEventHandler(CanvasObj_SelectionChanged);
			CanvasObj.ChildrenChangeComplete-=new NodeCollectionEventHandler(CanvasObj_ChildrenChangeComplete);
			CanvasObj.MapProperties-=new EventHandler(CanvasObj_MapProperties);
			CanvasObj.SelectDisplayUOM-=new EventHandler(CanvasObj_SelectDisplayUOM);
			CanvasObj.ShowAttributeDetails-=new EventHandler(CanvasObj_ShowAttributeDetails);
			CanvasObj.SetLabelsVisibility-=new EventHandler(CanvasObj_SetLabelsVisibility);
			CanvasObj.NodeDoubleClick-=new NodeMouseEventHandler(CanvasObj_NodeDoubleClick);
            CanvasObj.MouseUp -= new MouseEventHandler(CanvasObj_MouseUp);
			foreach(MenuItem mitem in CanvasObj.CanvasContextMenu.MenuItems)
			{
				if(mitem.Index == 7)
					continue;
				else
					mitem.Click-=new EventHandler(OverRideValue_Click);
			}
			CanvasObj.CanvasContextMenu.Popup-=new EventHandler(CanvasContextMenu_Popup);
			GraphicsSiteComponentUtilities.LabelDoubleClick-=new NodeMouseEventHandler(GraphicsSiteComponentUtilities_LabelDoubleClick);
			GraphicsAggregatesUtilities.LabelDoubleClick-=new NodeMouseEventHandler(GraphicsSiteComponentUtilities_LabelDoubleClick);
			CanvasObj.Model.MouseEnter-=new NodeMouseEventHandler(Model_MouseEnter);
			CanvasObj.ChangeGridDimensions-=new EventHandler(CanvasObj_ChangeGridDimensions);
            CanvasObj.RemoveRow -= new EventHandler(CanvasObj_RemoveRow);
            CanvasObj.RemoveColumn -= new EventHandler(CanvasObj_RemoveColumn);
            CanvasObj.ResetGridCell -= new EventHandler(CanvasObj_ResetGridCell);   
			   CanvasObj.ResetNode -= new EventHandler(CanvasObj_ResetNode);   

			CanvasObj.SetCustomFormat-=new EventHandler(CanvasObj_SetCustomFormat);
            CanvasObj.SetCustomFormatGridCell -= new EventHandler(CanvasObj_SetCustomFormatGridCell);
		}
		#endregion

		#region Fields
		
		private Canvas									_canvasObj;
		private LabelController							_labelControllerObj;
		private LabelLookup								_labelLookupObj;
		private	LRRBindingManager						_lrrBindingManger;
		private string									_id;
		private LRRApp									_app;
        private bool                                    _isRunning = false;
		public	event	NodeCollectionEventHandler		CanvasSelectionChanged;
		public	event	NodeCollectionEventHandler		ChildrenChangeComplete;
		public	event	EventHandler					MapProperties;
		public	event	EventHandler					ShowAttributeDetails;
        public  event   EventHandler                    SelectedAttributeDetails;
		public	event	EventHandler					SetLabelsVisibility;
		public	event	EventHandler					SelectDisplayUOM;
		public	event	EventHandler					ViewTrend;
		public	event	NodeMouseEventHandler			NodeDoubleClick;
		public	event	EventHandler					OverRideValueClick;
		public	event	Symcon.LightRidge.BusinessObjects.PropertyChanged	LabelPropertyValueChanged;
		public	event	NodeMouseEventHandler			LabelDoubleClick;
		public	event	NodeMouseEventHandler			MouseEnter;
		public	event	EventHandler					ChangeGridDimensions;
        public  event   EventHandler                    ResetGridCell;
		  public  event   EventHandler                    ResetNode;
        public  event   EventHandler                    RemoveRow;
        public  event   EventHandler                    RemoveColumn;
		public	event	EventHandler					SetCustomFormat;
        public  event   EventHandler                    SetCustomFormatGridCell;
        public  event   MouseEventHandler               MouseUp;
	
		#endregion

        
		public void PropertyValueChanged(object sender, string propertyName)
		{
			try
			{
				if(LabelPropertyValueChanged != null)
				{
					LabelPropertyValueChanged(sender,propertyName);
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GraphicsManagers),"PropertyValueChanged :"+ex.Message  ,Logger.Level.Debug);
			}
		}

		private void CanvasObj_SelectionChanged(object sender, NodeCollectionEventArgs evtArgs)
		{
			if(CanvasSelectionChanged != null)
			{
				CanvasSelectionChanged(sender,evtArgs);
			}
		}

		private void CanvasObj_ChildrenChangeComplete(object sender, NodeCollectionEventArgs evtArgs)
		{
			if(ChildrenChangeComplete != null)
			{
				ChildrenChangeComplete(sender,evtArgs);
			}
		}

		private void CanvasObj_NodeDoubleClick(object sender, NodeMouseEventArgs evtArgs)
		{
			if(NodeDoubleClick != null)
			{
				NodeDoubleClick(sender,evtArgs);
			}
		}

		private void OverRideValue_Click(object sender, EventArgs e)
		{
			if(OverRideValueClick != null)
			{
				OverRideValueClick(sender,e);
			}
		}

		private void _lrrBindingManger_BindingCalled(object source, object target)
		{
			if(source != null)
			{
				if(target.GetType() == typeof(GridCell))
				{
					GridCell lbl = target as GridCell;
					IPropertyValue lrspv = lbl.Tag as IPropertyValue;
					if(source.GetType() == typeof(string))
					{
											
						string newUom = lbl.UIUOMID;
						string oldUomID = lrspv.GetUOMID();
						string propVal = source.ToString();
                        string newPropValue = GraphicsUtilities.ChangeUOM(oldUomID, newUom, propVal, lrspv.GetDataType());
                        if (lrspv.GetDataType() == null)
                            newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
						lbl.Text = newPropValue;
						GraphicsUtilities.SetColor(lbl,lrspv);
                        if (!_isRunning)
                            this._canvasObj.Model.UpdateViews();
					}
					else
					{
						SymbolLabel smLbl = lbl as SymbolLabel;
						GraphicsUtilities.SetColor(smLbl,lrspv);
                        if (!_isRunning)
                            this._canvasObj.Model.UpdateViews();
					}
				}
                else if(target.GetType().GetInterface("IDashBoardControl") != null)
                {
                    Control objControl = target as Control;
                    ChangePropertyValue(objControl);
                }
                else if (target is LRRLabel)
                {
                    LRRLabel lbl = target as LRRLabel;
                    if (lbl.Tag.ToString().StartsWith("Symcon.LightRidge.DashBoardControlsLibrary"))
                    {
                        Control objControl = lbl.Tag as Control;
                        objControl.GetType().GetProperty("TargetValue").SetValue(objControl,source.ToString(),null);
                    }
                    else
                    {
                        IPropertyValue lrspv = lbl.Tag as IPropertyValue;
                        if (source.GetType() == typeof(string))
                        {
                            string propVal = source.ToString();
                            if (target.GetType() == typeof(LRRLabel))
                            {
                                LRRLabel uomLbl = GraphicsUtilities.GetUOMLabel(lbl);
                                if (uomLbl != null && uomLbl.Tag != null)
                                {
                                    string newUom = uomLbl.ID;
                                    LRRLabel parentLabel = GraphicsUtilities.GetParentLabel(uomLbl);
                                    IProperty prop = parentLabel.Tag as IProperty;
                                    string oldUomID = lrspv.GetUOMID();
                                    string newPropValue = GraphicsUtilities.ChangeUOM(oldUomID, newUom, propVal, lrspv.GetDataType());
                                    if (lrspv.GetDataType() == null)
                                        newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                    lbl.Text = newPropValue;
                                }
                                else
                                {
                                    lbl.Text = propVal;
                                }
                            }
                            if (lbl != null)
                            {
                                if (lrspv != null)
                                {
                                    GraphicsUtilities.SetColor(lbl, lrspv);
                                    GraphicsUtilities.ApplyDevationRule(LRRModel.ProjectModes, lbl.Parent as Symbol, lbl);
                                    if (!_isRunning)
                                    {
                                        this._canvasObj.Model.UpdateViews();
                                    }
                                }
                            }
                        }
                        else
                        {
                            GraphicsUtilities.SetColor(lbl, lrspv);
                            if (!_isRunning)
                            {
                                this._canvasObj.Model.UpdateViews();
                            }
                        }
                    }
                }
                else
                {
                    Symbol smbl = target as Symbol;
                    if (smbl != null)
                    {
                        string customFormatID = smbl.GetPropertyValue("CustomFormatID") as string;
                        IPropertyValue lrrscpv = null;
                        if (smbl.GetType() == typeof(Aggregates))
                        {
                            lrrscpv = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(customFormatID);
                        }
                        else
                        {
                            lrrscpv = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(customFormatID);
                        }
                        // scp and mode id part added and lssscpv commeting done by ashwin for custom formatting 
                        /*
                        if (lrrscpv != null && customFormatID != null && customFormatID == lrrscpv.GetID())
                        {
                            smbl.FillStyle.Color = lrrscpv.GetGraphicsBackColor();
                            smbl.LineStyle.LineColor = lrrscpv.GetForeColor();
                            if (!_isRunning)
                                this._canvasObj.Model.UpdateViews();
                        }
                         * */
                        if (customFormatID != null)
                        {
                            lrrscpv = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(customFormatID);
                            if (lrrscpv != null)
                            {
                                smbl.FillStyle.Color = lrrscpv.GetGraphicsBackColor();
                                smbl.LineStyle.LineColor = lrrscpv.GetForeColor();
                                if (!_isRunning)
                                    this._canvasObj.Model.UpdateViews();
                            }
                            else
                            {
                                string scp = smbl.GetPropertyValue("SCP") as string;
                                string modeID = smbl.GetPropertyValue("ModeID") as string;
                                if (scp != null && modeID != null)
                                {
                                    lrrscpv = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(scp, modeID);
                                    if (lrrscpv != null)
                                    {
                                        smbl.FillStyle.Color = lrrscpv.GetGraphicsBackColor();
                                        smbl.LineStyle.LineColor = lrrscpv.GetForeColor();
                                        if (!_isRunning)
                                            this._canvasObj.Model.UpdateViews();
                                    }
                                }
                            }
                        }
                    }
                }
			}
		}


        /// <summary>
        /// Binding for Dashboard controls.
        /// </summary>
        /// <param name="objControl"></param>
        /// <param name="sm"></param>

        public void ChangePropertyValue(object objControl)
        {
            try
            {
                object sourceObjectName = objControl.GetType().GetProperty("SourceObjectName").GetValue(objControl, null);
                object sourceObjectID = objControl.GetType().GetProperty("SourceObjectID").GetValue(objControl, null);
                object sourceObject = GetSourceObject((string)sourceObjectName, (string)sourceObjectID);
                if (sourceObject != null)
                {
                    LRRSiteComponentPropertyValue iPropValue = sourceObject as LRRSiteComponentPropertyValue;
                    string uomID = iPropValue.GetUOMID();
                    LRRMode mode = LRRModel.Modes.GetModeByID(iPropValue.GetModeID());
                    //Convert property value according to UIUOM 
                    string propertyValue = iPropValue.GetPropertyValue();
                    LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(iPropValue.SiteComponentPropertyID);
                    string uomTypeID = siteCompProp.UOMTypeID;
                    //Get the UI UOM
                    LRRMeasurementSystemDefinitions lrrMSDs = (LRRMeasurementSystemDefinitions)this._app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRMeasurementSystemDefinitions));
                    string uiUOMID = lrrMSDs.GetLocaleUOM(this._app.AppLocaleID, uomTypeID);

                    if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
                    {
                        propertyValue = iPropValue.GetPropertyValueForUOM(uomID, uiUOMID, propertyValue);
                    }

                    string lrrDataType = iPropValue.GetDataType();
                    if (lrrDataType == null || lrrDataType.Trim() == "")
                    {
                        propertyValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(propertyValue, this._app);
                    }
                    objControl.GetType().GetProperty("TargetValue").SetValue(objControl, propertyValue, null);
                    if (!_isRunning)
                        this._canvasObj.Model.UpdateViews();
                    //Symbol sm = this.CanvasObj.SelectionList.Last as Symbol;
                    //PropertyTable pt = sm.GetType().GetProperty("PT").GetValue(sm, null) as PropertyTable;
                    //pt.Properties.Add(new PropertySpec(siteCompProp.Name+"("+mode.ModeName+")",typeof(string),"Bound Properties",siteCompProp.Name+"("+mode.ModeName+")",propertyValue,"","",siteCompProp.PropertyID));
                    
                }
            }
            catch { }
        }


        public object GetSourceObject(string sourceObjectName, string sourceObjectID)
        {
            object retValue = null;
            switch (sourceObjectName)
            {
                case "LRRSiteComponentPropertyValue":
                    retValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(sourceObjectID);
                    break;
                case "LRRSiteGroupPropertyQualifierValueID":
                    retValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sourceObjectID);
                    break;
                case "LRRAggregateGroupedPropertyValueID":
                    retValue = LRRModel.AggregateGroupedPropertyValuesObj.GetAggregateGroupedPropertyValueFromID(sourceObjectID);
                    break;
                case "LRRAggregatePropertyValue":
                    retValue = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(sourceObjectID);
                    break;
                default:
                    break;
            }
            return retValue;
        }


		private void Model_MouseEnter(object sender, NodeMouseEventArgs evtArgs)
		{
			if(MouseEnter != null)
			{
				MouseEnter(sender,evtArgs);
			}
		}

		private void CanvasObj_MapProperties(object sender, EventArgs e)
		{
			if(MapProperties != null)
			{
				MapProperties(sender,e);
			}

		}

		private void CanvasObj_SelectDisplayUOM(object sender, EventArgs e)
		{
			if(SelectDisplayUOM != null)
			{
				SelectDisplayUOM(sender,e);
			}

		}

		private void CanvasObj_ShowAttributeDetails(object sender, EventArgs e)
		{
			if(ShowAttributeDetails != null)
			{
				ShowAttributeDetails(sender,e);
			}
		}

        private void CanvasObj_SelectedAttributeDetails(object sender, EventArgs e)
        {
            if (SelectedAttributeDetails  != null)
            {
                SelectedAttributeDetails(sender, e);
            }
        }

		private void GraphicsSiteComponentUtilities_LabelDoubleClick(object sender, NodeMouseEventArgs evtArgs)
		{
			if(LabelDoubleClick != null && this.CanvasObj != null)
			{
				LabelDoubleClick(sender,evtArgs);
			}

		}

		private void CanvasObj_ViewTrend(object sender, EventArgs e)
		{
			if(ViewTrend != null)
			{
				ViewTrend(sender,e);
			}
		}

		private void CanvasObj_SetLabelsVisibility(object sender, EventArgs e)
		{
			if(SetLabelsVisibility != null)
			{
				SetLabelsVisibility(sender,e);
			}
		}

		private void CanvasContextMenu_Popup(object sender, EventArgs e)
		{
			
			ContextMenu ctxMenu = sender as ContextMenu;
			MenuItem mItemViewTrend = ctxMenu.MenuItems[11];
			if(mItemViewTrend.Enabled)
			{
				mItemViewTrend.Enabled = false;
				mItemViewTrend.MenuItems.Clear();
			
				foreach(string schemeName in UserControls.UserControlTrendOptions.HtSchemes.Keys)
				{
					MenuItem miItemTrendChild = new MenuItem(schemeName,new EventHandler(ViewTrend));
					mItemViewTrend.MergeType = MenuMerge.Add;
					mItemViewTrend.MenuItems.Add(miItemTrendChild);
				}
				mItemViewTrend.Enabled = true;
			}
		}

		private void CanvasObj_ChangeGridDimensions(object sender, EventArgs e)
		{
			if(ChangeGridDimensions != null)
			{
				ChangeGridDimensions(sender,e);
			}
		}

        private void CanvasObj_RemoveRow(object sender, EventArgs e)
        {
            if (RemoveRow != null)
            {
                RemoveRow(sender, e);
            }
        }

        private void CanvasObj_RemoveColumn(object sender, EventArgs e)
        {
            if (RemoveColumn != null)
            {
                RemoveColumn(sender, e);
            }
        }

        private void CanvasObj_ResetGridCell(object sender, EventArgs e)
        {
            if (ResetGridCell != null)
            {
                ResetGridCell(sender, e);
            }
        }

		  private void CanvasObj_ResetNode(object sender, EventArgs e)
        {
            if (ResetNode!= null)
            {
                ResetNode(sender, e);
            }
        }

		private void CanvasObj_SetCustomFormat(object sender, EventArgs e)
		{
			if(SetCustomFormat!= null)
			{
				SetCustomFormat(sender,e);
			}

		}

        private void CanvasObj_SetCustomFormatGridCell(object sender, EventArgs e)
        {
            if (SetCustomFormatGridCell != null)
            {
                SetCustomFormatGridCell(sender, e);
            }

        }

        public void Dispose()
        {
            _labelControllerObj = null;
            _labelLookupObj = null;
            _canvasObj = null;
            _lrrBindingManger = null;
        }

    }
}