using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;
using System.IO;
using System.Resources;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters;
using System.Data;
using Symcon.Utils.MessageLogger;
using Symcon.Lightridge.LRRUI.Controls;
using Symcon.Lightridge.LRRUI;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.Options;

using Syncfusion.Windows.Forms.Diagram;
using System.Collections.Generic;

namespace Symcon.LightRidge.PEAdvisor
{
    ///	<summary>
    ///	Summary	description	for	GraphicsSiteComponentUtilities.
    ///	</summary>
    public class GraphicsSiteComponentUtilities
    {
        static GraphicsSiteComponentUtilities()
        {
            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            _lrrOptionsObj = _app.GetOptions() as LRROptions;
        }

        public static event Symcon.LightRidge.BusinessObjects.PropertyChanged LabelTextChanged;
        public static event NodeMouseEventHandler LabelDoubleClick;

        #region	Public Methods

        public static void UpdateSymbolPropertiesMap(Symbol smbl, LRRModes lrrModes, Symcon.BusinessObjects.CustomCollection mySelectedProperties, GraphicsManager gm, GraphicsManagers.PropertiesMappingMode propertiesMappingMode, GraphicsManagers.UserMode userMode, Graphics g)
        {
            ArrayList alst = new ArrayList();
            try
            {
                gm.CanvasObj.SuspendLayout();
                Canvas parent = gm.CanvasObj;
                LRRSiteComponent myLsc = GetSiteComponentForSymbol(smbl); //Selected Site component
                if (myLsc == null && propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                {
                    smbl.FillStyle.Color = Color.Black;
                    GraphicsUtilities.RemoveBoundProperties(smbl);
                    return;
                }
                LabelController myLabelController = gm.LabelControllerObj;
                Symbol mySymbol = smbl;	//symbol
                //bool setEquipmentName =false;
                /* //Set the Symbol				
                ISymbolPropertiesMapping isymbolMapping;
					
                if(mySymbol	== null)
                   return;
                //Set the interface based on symbol type
                if(mySymbol.GetType().Name == "Aggregates")
                   isymbolMapping = new AggregatesMapping();
                else
                   isymbolMapping = new SiteComponentMapping();

                if(mySymbol.GetPropertyValue("PlantItem") != null )//&&	smbl.GetPropertyValue("SiteItem").ToString() != "Select" )
                {
                   Symcon.Lightridge.LRRUI.PropertyTable pt =	mySymbol.GetPropertyValue("PT")	as Symcon.Lightridge.LRRUI.PropertyTable;
                   PropertiesMap propMap = new PropertiesMap(isymbolMapping,pt,mySymbol);
                   setEquipmentName = propMap.SetEquipmentName;
                }
                   */
                LRRModes myModes = lrrModes; //	slected	Modes
                if (mySymbol.GetType() == typeof(DashBoardControlProperties))
                {
                    SetBindingForDashBoardControls(mySymbol, lrrModes, mySelectedProperties, gm, propertiesMappingMode, userMode, g);
                    return;
                }
                Syncfusion.Windows.Forms.Diagram.LabelCollection myLabels = mySymbol.Labels; //	Labells	for	symbol
                //Remove all label Binding
                for (int i = mySymbol.Labels.Count; i > 0; i--)
                {
                    LRRLabel lrrLabel = mySymbol.Labels[i - 1] as LRRLabel;
                    if (userMode == GraphicsManagers.UserMode.OPERATOR)
                    {
                        lrrLabel.EditStyle.Enabled = false;
                        lrrLabel.ReadOnly = true;
                    }
                    if (lrrLabel.Tag != null && lrrLabel.Tag.GetType() == typeof(IPropertyValue))
                    {
                        gm.LRRBindingManagerObj.RemoveBinding(lrrLabel.Tag, "PropertyValue", mySymbol.Labels[i - 1], true);
                        gm.LRRBindingManagerObj.RemoveBinding(lrrLabel.Tag, "ErrCode", lrrLabel, true);
                    }
                }
                //Set the Header Label
                LRRLabel hsmblLbl = mySymbol.Labels[0] as LRRLabel;
                // hsmblLbl.Visible = true;
                //hsmblLbl.ReadOnly =	false;
                //TODO : use one method 
                if (myLsc != null)
                {
                    if (myLsc.EquipmentTag != hsmblLbl.Text)
                    {
                        hsmblLbl.Text = myLsc.Name;
                        hsmblLbl.Name = myLsc.Name;
                    }
                }
                
                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)//|| propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.REMAP)
                {

                    hsmblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[0];
                }
                hsmblLbl.Tag = myLsc;
                int count = 0;
                int lblvisibilityIndex = 0;
                //Stack	the	arraylist with three rows to hold the Labels
                for (int i = 0; i < 3; i++)
                {
                    alst.Add(null);
                }
                //Sort the Modes in	TypeID so they appear in order
                myModes.SortColumn = "ModeTypeID";
                myModes.Sort();
                //Iterate through Selected Modes and Setup the arraylist for each Mode
                foreach (LRRMode lrmd in myModes)
                {
                    int modeTypeId;
                    if (lrmd.ModeTypeID == null || lrmd.ModeTypeID == "")
                        continue;
                    modeTypeId = Int32.Parse(lrmd.ModeTypeID, Symcon.BusinessObjects.SymConverter.Culture) - 1;
                    alst[modeTypeId] = lrmd;
                }
                Font myFont = new Font(smbl.Labels[1].FontStyle.Family, smbl.Labels[1].FontStyle.Size);
                float offsetXPropName = -1f;
                float offsetXPropValue = -1f;
                float offsetXUOM = -1f;

                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)
                {
                    GetOffsetLength(mySelectedProperties, myModes, myFont, out	offsetXPropName, out	offsetXPropValue, out offsetXUOM, g);
                    if (offsetXPropName < 40)
                    {
                        offsetXPropName = 40;
                    }
                    else if (offsetXPropName > 100)
                    {
                        offsetXPropName = 100;
                    }
                    if (offsetXPropValue < 40)
                    {
                        offsetXPropValue = 40;
                    }
                    else if (offsetXPropValue > 100)
                    {
                        offsetXPropValue = 100;
                    }
                    if (offsetXUOM < 20)
                    {
                        offsetXUOM = 20;
                    }
                    else if (offsetXUOM > 100)
                    {
                        offsetXUOM = 100;
                    }
                    offsetXPropName = -offsetXPropName;
                    offsetXPropValue = -offsetXPropValue;
                    offsetXUOM = -offsetXUOM;
                }

                IEnumerator propsEnum = mySelectedProperties.GetEnumerator();
                while (propsEnum.MoveNext())
                {
                    LRRPlantItemProperty plProp = propsEnum.Current as LRRPlantItemProperty;

                    //Iterate through the selected Properties for Site Component
                    //1>Add	labels to the symbol
                    //2>Add	Binding	to each	Label
                    if (plProp != null)
                    {
                        InitializeLabels(plProp, count, mySymbol, alst, gm, propertiesMappingMode, mySelectedProperties, userMode, offsetXPropName, offsetXPropValue, offsetXUOM, ref lblvisibilityIndex);
                        if (plProp.IsExposed || userMode == GraphicsManagers.UserMode.IMPLEMENTOR)
                            AddtoPropertyGrid(plProp, alst, gm, mySymbol);
                        count++;
                        lblvisibilityIndex++;
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(GraphicsSiteComponentUtilities), "UpdateSymbolPropertiesMap :" + ex.Message, Logger.Level.Debug);
            }
            finally
            {
                alst.Clear();
                flagRemove = true;
                gm.CanvasObj.View.Update();
                gm.CanvasObj.ResumeLayout();
                _labelindex = 1;
            }
        }

        public static void UpdateSymbolPropertiesMap(Symbol smbl, LRRModes lrrModes, Symcon.BusinessObjects.CustomCollection mySelectedProperties, GraphicsManager gm, GraphicsManagers.PropertiesMappingMode propertiesMappingMode, GraphicsManagers.UserMode userMode, Graphics g, bool setEquipmentName)
        {
            ArrayList alst = new ArrayList();
            try
            {
                gm.CanvasObj.SuspendLayout();
                Canvas parent = gm.CanvasObj;
                LRRSiteComponent myLsc = GetSiteComponentForSymbol(smbl); //Selected Site component
                LabelController myLabelController = gm.LabelControllerObj;
                Symbol mySymbol = smbl;	//symbol

                LRRModes myModes = lrrModes; //	slected	Modes
                if (mySymbol.GetType() == typeof(DashBoardControlProperties))
                {
                    SetBindingForDashBoardControls(mySymbol, lrrModes, mySelectedProperties, gm, propertiesMappingMode, userMode, g);
                    return;
                }
                Syncfusion.Windows.Forms.Diagram.LabelCollection myLabels = mySymbol.Labels; //	Labells	for	symbol
                //Remove all label Binding
                for (int i = mySymbol.Labels.Count; i > 0; i--)
                {
                    LRRLabel lrrLabel = mySymbol.Labels[i - 1] as LRRLabel;
                    if (userMode == GraphicsManagers.UserMode.OPERATOR)
                    {
                        lrrLabel.EditStyle.Enabled = false;
                        lrrLabel.ReadOnly = true;
                    }
                    if (lrrLabel.Tag != null && lrrLabel.Tag.GetType() == typeof(IPropertyValue))
                    {
                        gm.LRRBindingManagerObj.RemoveBinding(lrrLabel.Tag, "PropertyValue", mySymbol.Labels[i - 1], true);
                        gm.LRRBindingManagerObj.RemoveBinding(lrrLabel.Tag, "ErrCode", lrrLabel, true);
                    }
                }
                //Set the Header Label
                LRRLabel hsmblLbl = mySymbol.Labels[0] as LRRLabel;

                // hsmblLbl.Visible = true;
                //hsmblLbl.ReadOnly =	false;

                if (setEquipmentName)
                {
                    hsmblLbl.Text = myLsc.Name;
                    hsmblLbl.Name = myLsc.Name;
                }
                else
                {
                    if (myLsc.EquipmentTag == null)
                    {
                        hsmblLbl.Text = "Equipment tag not available";

                    }
                    else
                    {
                        hsmblLbl.Text = myLsc.EquipmentTag;
                    }
                    hsmblLbl.Name = myLsc.EquipmentTag;
                }
                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)//|| propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.REMAP)
                {
                    hsmblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[0];


                }
                hsmblLbl.Tag = myLsc;

                int count = 0;
                int lblvisibilityIndex = 0;
                //Stack	the	arraylist with three rows to hold the Labels
                for (int i = 0; i < 3; i++)
                {
                    alst.Add(null);
                }
                //Sort the Modes in	TypeID so they appear in order
                myModes.SortColumn = "ModeTypeID";
                myModes.Sort();
                //Iterate through Selected Modes and Setup the arraylist for each Mode
                foreach (LRRMode lrmd in myModes)
                {
                    int modeTypeId;
                    if (lrmd.ModeTypeID == null || lrmd.ModeTypeID == "")
                        continue;
                    modeTypeId = Int32.Parse(lrmd.ModeTypeID, Symcon.BusinessObjects.SymConverter.Culture) - 1;
                    alst[modeTypeId] = lrmd;
                }
                Font myFont = new Font(smbl.Labels[1].FontStyle.Family, smbl.Labels[1].FontStyle.Size);
                float offsetXPropName = -1f;
                float offsetXPropValue = -1f;
                float offsetXUOM = -1f;

                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)
                {
                    GetOffsetLength(mySelectedProperties, myModes, myFont, out	offsetXPropName, out	offsetXPropValue, out offsetXUOM, g);
                    if (offsetXPropName < 40)
                    {
                        offsetXPropName = 40;
                    }
                    else if (offsetXPropName > 100)
                    {
                        offsetXPropName = 100;
                    }
                    if (offsetXPropValue < 40)
                    {
                        offsetXPropValue = 40;
                    }
                    else if (offsetXPropValue > 100)
                    {
                        offsetXPropValue = 100;
                    }
                    if (offsetXUOM < 20)
                    {
                        offsetXUOM = 20;
                    }
                    else if (offsetXUOM > 100)
                    {
                        offsetXUOM = 100;
                    }
                    offsetXPropName = -offsetXPropName;
                    offsetXPropValue = -offsetXPropValue;
                    offsetXUOM = -offsetXUOM;
                }

                IEnumerator propsEnum = mySelectedProperties.GetEnumerator();
                while (propsEnum.MoveNext())
                {
                    LRRPlantItemProperty plProp = propsEnum.Current as LRRPlantItemProperty;

                    //Iterate through the selected Properties for Site Component
                    //1>Add	labels to the symbol
                    //2>Add	Binding	to each	Label
                    if (plProp != null)
                    {
                        InitializeLabels(plProp, count, mySymbol, alst, gm, propertiesMappingMode, mySelectedProperties, userMode, offsetXPropName, offsetXPropValue, offsetXUOM, ref lblvisibilityIndex);
                        if (plProp.IsExposed || userMode == GraphicsManagers.UserMode.IMPLEMENTOR)
                            AddtoPropertyGrid(plProp, alst, gm, mySymbol);
                        count++;
                        lblvisibilityIndex++;
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(GraphicsSiteComponentUtilities), "UpdateSymbolPropertiesMap :" + ex.Message, Logger.Level.Debug);
            }
            finally
            {
                alst.Clear();
                flagRemove = true;
                gm.CanvasObj.View.Update();
                gm.CanvasObj.ResumeLayout();
                _labelindex = 1;
            }
        }

        private static void SetBindingForDashBoardControls(Symbol smbl, LRRModes lrrModes, Symcon.BusinessObjects.CustomCollection mySelectedProperties, GraphicsManager gm, GraphicsManagers.PropertiesMappingMode propertiesMappingMode, GraphicsManagers.UserMode userMode, Graphics g)
        {
            Symcon.Lightridge.LRRUI.Controls.DashBoardControlProperties baseObj = smbl as Symcon.Lightridge.LRRUI.Controls.DashBoardControlProperties;
            ControlNode cNode = smbl.Nodes.Last as ControlNode;
            Control objControl = cNode.Control;
            Symcon.LightRidge.PEAdvisor.DashBoard.BindingInfo bindingInfo = new Symcon.LightRidge.PEAdvisor.DashBoard.BindingInfo();
            string sourceObjectName = "";
            string sourceObjectID = "";
            string sourceProperty = "PropertyValue";
            LRRSiteComponentProperty lscp = null;
            LRRSiteComponentGroupProperty grpProp = null;
            LRRMode mode = null;
            Symcon.Lightridge.LRRUI.PropertyTable PT = baseObj.PT;
            string sourcePropertyID = "";
            string sourcePropertyModeId = "";
            PropertySpec boundPropSpec = null;
            string[] splitStrings = null;
            char[] ch1 = new char[1];
            ch1[0] = '(';
            foreach (PropertySpec ps in PT.Properties)
            {
                if (ps.Category == "Bound Properties")
                {
                    boundPropSpec = ps;
                    string sPropName = ps.Name;
                    sPropName = sPropName.Remove(sPropName.LastIndexOf(')'));
                    splitStrings = sPropName.Split(ch1);
                    sourcePropertyModeId = splitStrings[1].ToString();
                    bindingInfo.ModeName = sourcePropertyModeId;
                    sourcePropertyModeId = LRRModel.Modes.GetModeByName(sourcePropertyModeId).ModeId;
                    break;
                }
            }
            LRRPlantItemProperty plantItemProperty = mySelectedProperties[0] as LRRPlantItemProperty;
            if (plantItemProperty.GroupPropertyID == null || plantItemProperty.GroupPropertyID.Trim() == "")
            {
                sourcePropertyID = boundPropSpec.ObjectRef.ToString();
                lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(sourcePropertyID);
                bindingInfo.SelectedSiteComponentProperties.Add(lscp);
                bindingInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(sourcePropertyModeId));
                bindingInfo.SourceProperty = "PropertyValue";
                sourceObjectName = "LRRSiteComponentPropertyValue";
                bindingInfo.SourceObjectName = "LRRSiteComponentPropertyValue";
                LRRSiteComponentPropertyValue lscpv = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(sourcePropertyID, sourcePropertyModeId);
                bindingInfo.SourceObjectID = lscpv.ID;
                sourceObjectID = lscpv.ID;
                bindingInfo.SiteComponentProperty = lscp.Name;
                bindingInfo.SiteComponentName = baseObj.PlantItem.Name;
            }
            else
            {
                string currentLookupQualifierID = "";
                string currentGroupedPropertyID = "";
                if (boundPropSpec.ObjectRef.ToString().IndexOf('|') != -1)
                {
                    string[] ids = boundPropSpec.ObjectRef.ToString().Split("|".ToCharArray());
                    currentLookupQualifierID = ids[0];
                    currentGroupedPropertyID = ids[1];
                }
                string attName = boundPropSpec.Name;
                sourceObjectName = "LRRSiteGroupPropertyQualifierValueID";
                bindingInfo.SourceObjectName = "LRRSiteGroupPropertyQualifierValueID";
                bindingInfo.SourceProperty = "PropertyValue";
                LRRSiteGroupPropertyQualifierValue gpQv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(currentGroupedPropertyID, currentLookupQualifierID, sourcePropertyModeId);
                bindingInfo.SourceObjectID = gpQv.SiteGroupPropertyQualifierValueID;
                sourceObjectID = gpQv.SiteGroupPropertyQualifierValueID;
                grpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(currentGroupedPropertyID);
                bindingInfo.SelectedSiteCompGroups.Add(grpProp);
                bindingInfo.SiteComponentProperty = boundPropSpec.Name;
                bindingInfo.SiteComponentName = baseObj.PlantItem.Name;
                bindingInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(sourcePropertyModeId));
            }
            objControl.Tag = bindingInfo;
            object sourceObject = gm.GetSourceObject(sourceObjectName, sourceObjectID);
            LRRLabel lbl = smbl.Labels[0] as LRRLabel;
            lbl.Tag = objControl;
            gm.LRRBindingManagerObj.AddBinding(sourceObject, "PropertyValue", boundPropSpec, "DefaultValue", true);
            gm.LRRBindingManagerObj.AddBinding(sourceObject, "PropertyValue", lbl, "Text", true);
            mode = LRRModel.ChildModel.Modes.GetModeByID(sourcePropertyModeId);
            InitializeLablesForDashBoard(objControl, smbl, lscp, grpProp, mode);
            /*string sourceObjectName = bindInfo.SourceObjectName;
            string sourceObjectID = bindInfo.SourceObjectID;
            string sourceProperty = bindInfo.SourceProperty;
            string propetyValue = "";
            LRRSiteComponentProperty lscp = null;
            LRRMode mode = null;*/

            int i = 0;
        }

        public static void InitializeLablesForDashBoard(Control objControl, Symbol smbl, LRRSiteComponentProperty lscp, LRRSiteComponentGroupProperty lscgp, LRRMode mode)
        {
            string siteComponent = "";
            string siteComponentPropName = "";
            string modeName = "";
            string myUOMName = "";
            string modelUOMID = "";
            string uiUOMIDID = "";
            string parentPropID = "";
            LRRUOMType uomTypeObj = null;
            LRRLabel lblSiteComp = smbl.Labels[4] as LRRLabel;
            LRRLabel lblMode = smbl.Labels[6] as LRRLabel;
            LRRLabel lblUom = smbl.Labels[9] as LRRLabel;
            LRRLabel lblParent = smbl.Labels[1] as LRRLabel;
            LRRLabel lblPropValue = smbl.Labels[2] as LRRLabel;

            object siteComp = objControl.GetType().GetProperty("SourceItem").GetValue(objControl, null);
            if (lscp != null)
            {
                parentPropID = lscp.ID;
                uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(lscp.UOMTypeID);
                LRRSiteComponentPropertyValue lscpv = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(parentPropID, mode.ModeId);
                lblPropValue.Tag = lscpv;
                lblPropValue.ID = lscpv.ID;
                lblPropValue.ParentID = parentPropID;
                lblPropValue.Name = lscp.Name + "(" + mode.ModeName + ")";
            }
            else if (lscgp != null)
            {
                parentPropID = lscgp.GroupPropertyID;
                LRRPropertyType propType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(lscgp.PropertyID);
                string UomTypeID = propType.UOMTypeID;
                uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(UomTypeID);
                LRRSiteGroupPropertyQualifierValue gpValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValueFromGroupPropIDandModeID(lscgp.GroupPropertyID, mode.ModeId);
                lblPropValue.Tag = gpValue;
                lblPropValue.ID = gpValue.SiteGroupPropertyQualifierValueID;
                lblPropValue.ParentID = parentPropID;

            }
            if (uomTypeObj != null)
            {
                string modelID = LRRModel.MeasurementSystems.GetCurrentModelUOM().ID;
                string localID = _app.AppLocaleID;
                modelUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(modelID, uomTypeObj.UOMTypeID);
                uiUOMIDID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localID, uomTypeObj.UOMTypeID);
            }
            if (siteComp != null)
            {
                siteComponent = siteComp.ToString();
            }
            if (mode != null)
            {
                modeName = mode.ModeDisplayName;
            }
            object siteCompProp = objControl.GetType().GetProperty("Attribute").GetValue(objControl, null);
            if (siteCompProp != null)
            {
                siteComponentPropName = siteCompProp.ToString();
            }

            //Parent Label.
            lblParent.ParentID = parentPropID;
            lblParent.ID = parentPropID;
            lblParent.IsParent = true;
            if (lscgp != null)
            {
                lblParent.Tag = lscgp;
            }
            else
            {
                lblParent.Tag = lscp;
            }

            Graphics g = objControl.CreateGraphics();
            //SiteComponent  Label
            lblSiteComp.Text = siteComponent;
            lblSiteComp.Visible = true;
            SizeF sizeSiteComponent = g.MeasureString(siteComponent, objControl.Font);
            if (sizeSiteComponent.Width > 150)
            {
                lblSiteComp.Size = new SizeF(150, lblSiteComp.Height);
            }
            else
            {
                lblSiteComp.Size = new SizeF(sizeSiteComponent.Width + 15, lblSiteComp.Height);
            }

            //Mode Label
            SizeF sizeMode = g.MeasureString(modeName, objControl.Font);
            if (sizeMode.Width > 150)
            {
                lblMode.Size = new SizeF(150, lblMode.Height);
            }
            else
            {
                lblMode.Size = new SizeF(sizeMode.Width + 5, lblMode.Height);
            }
            lblMode.Text = modeName;
            lblMode.Visible = true;

            //UOM Label
            lblUom.Tag = uomTypeObj;
            lblUom.Visible = true;

            myUOMName = GraphicsUtilities.GetUomName(uiUOMIDID);
            SizeF sizeUOM = g.MeasureString(myUOMName, objControl.Font);
            if (sizeUOM.Width > 150)
            {
                lblUom.Size = new SizeF(150, lblUom.Height);
            }
            else
            {
                lblUom.Size = new SizeF(sizeUOM.Width + 5, lblUom.Height);
            }
            lblUom.Text = myUOMName;
            lblUom.Name = "UOM" + "(" + myUOMName + ")-" + siteComponentPropName;
            lblUom.ID = uiUOMIDID;
            lblUom.IsUOM = true;
            lblUom.ParentID = parentPropID;
            lblUom.LabelDoubleClick -= new NodeMouseEventHandler(lblUom_LabelDoubleClick);
            lblUom.LabelDoubleClick += new NodeMouseEventHandler(lblUom_LabelDoubleClick);
        }

        public static void lblUom_LabelDoubleClick(object sender, NodeMouseEventArgs evtArgs)
        {
            LRRLabel lbl = evtArgs.Node as LRRLabel;
            if (lbl.ID != null && lbl.IsUOM && lbl.Tag != null)
            {
                LRRUOMType lrrUOMType = lbl.Tag as LRRUOMType;
                if (lrrUOMType != null)
                    GraphicsUtilities.ChangeUOMClick(lbl, lrrUOMType.UOMTypeID);
            }
        }


        private static void GetOffsetLength(Symcon.BusinessObjects.CustomCollection mySelectedProperties, LRRModes mySelectedModes, Font myFont, out float offsetXPropName, out float offsetXPropValue, out float offsetXUOM, Graphics gphs)
        {
            offsetXPropName = -1;
            offsetXPropValue = -1;
            offsetXUOM = -1;
            SizeF stringSize;
            IEnumerator propsEnum = mySelectedProperties.GetEnumerator();
            LRRUOMType uomTypeObject = null;
            while (propsEnum.MoveNext())
            {
                LRRPlantItemProperty plProp = propsEnum.Current as LRRPlantItemProperty;
                IProperty myProp = null;
                if (plProp.SiteGroupPropertyQualifierValueID == null && plProp.ID != null)
                {
                    myProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(plProp.ID);
                }
                else if (plProp.ID != null)
                {
                    myProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(plProp.GroupPropertyID);
                }
                //Set The UOM Type
                if (myProp != null)
                {
                    if (myProp.GetPropertyType() == "1")
                    {
                        //For Simple Properties
                        if (myProp.GetUOMType() != null)
                        {
                            uomTypeObject = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(myProp.GetUOMType());
                        }
                    }
                    else
                    {
                        //For Grouped Properties
                        if (plProp.LookupQualifierID != null && plProp.LookupQualifierID.Trim() != "")
                        {
                            LRRLookUpQualifier lrrLookUpQualifier = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(plProp.LookupQualifierID);
                            if (lrrLookUpQualifier != null)
                                uomTypeObject = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(lrrLookUpQualifier.UOMTypeID);
                        }
                        //For Data Arrays
                        else
                        {
                            LRRPropertyType lrrPropType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(myProp.GetPropertyType());
                            if (lrrPropType != null)
                                uomTypeObject = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(lrrPropType.UOMTypeID);
                        }
                    }
                }
                //IProperty myProp = propsEnum.Current as IProperty;
                if (myProp != null)
                {
                    stringSize = gphs.MeasureString(plProp.Name, myFont);
                    if (stringSize.Width > offsetXPropName)
                    {
                        offsetXPropName = stringSize.Width;
                    }
                    foreach (LRRMode lrrmode in mySelectedModes)
                    {
                        //string myPropValue = myProp.GetValueForPropertyMode(lrrmode.ModeId);
                        List<IPropertyValue> lscpv = new List<IPropertyValue>();
                        if (myProp.GetPropertyType() == "1")
                        {
                            lscpv = LRRModel.PropertyValuesObj.GetPropValuesByPropID(myProp.GetID());
                        }
                        else
                        {
                            if (plProp.LookupQualifierID != null && plProp.LookupQualifierID.Trim() != "")
                                lscpv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValuesFromGrpPropIdAndLkupQualId(plProp.GroupPropertyID, plProp.LookupQualifierID);
                            else
                                lscpv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValuesinArrayFromGroupPropID(plProp.GroupPropertyID);
                        }
                        if (lscpv != null)
                        {
                            IEnumerator iEnum = lscpv.GetEnumerator();
                            while (iEnum.MoveNext())
                            {
                                IPropertyValue lscpValue = iEnum.Current as IPropertyValue;
                                if (lscpValue.GetModeID() == lrrmode.ModeId)
                                {
                                    stringSize = gphs.MeasureString(lscpValue.GetPropertyValue(), myFont);
                                    if (stringSize.Width > offsetXPropValue)
                                    {
                                        offsetXPropValue = stringSize.Width;
                                    }
                                }
                            }
                        }
                        else if (myProp.GetName() == "NullValue")
                        {
                            IPropertyValue myPropValue = myProp.GetPropertyValueForMode(lrrmode.ModeId) as IPropertyValue;
                            if (myPropValue == null)
                                continue;
                            stringSize = gphs.MeasureString(myPropValue.GetPropertyValue(), myFont);
                            if (stringSize.Width > offsetXPropValue)
                            {
                                offsetXPropValue = stringSize.Width;
                            }
                        }

                    }
                    string uiUOMID = null;
                    if (uomTypeObject != null)
                        uiUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_app.AppLocaleID, uomTypeObject.UOMTypeID);
                    //string UOMId =  myProp.GetUOM();
                    if (uiUOMID != null && uiUOMID.Trim() != "")
                    {
                        string UOMName = LRRModel.ChildModel.GetUOMNameFromID(uiUOMID);
                        stringSize = gphs.MeasureString(UOMName, myFont);
                        if (stringSize.Width > offsetXUOM)
                        {
                            offsetXUOM = stringSize.Width;
                        }
                    }
                }
            }
        }

        #endregion

        #region	Private	Methods

        private static void AddtoPropertyGrid(LRRPlantItemProperty lrcp, ArrayList myModes, GraphicsManager gm, Symbol mySymbol)
        {
            if (lrcp.ID == null && lrcp.Name != "NullValue")
            {
                GraphicsUtilities.RemoveBoundProperties(mySymbol);
                return;
            }
            //Return if Property not Exposed or Property is Null
            if (lrcp.Name == "NullValue")
            {
                return;
            }

            IProperty myProp;
            if (lrcp.GroupPropertyID == null)
            {
                myProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrcp.ID);
            }
            else
            {
                myProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrcp.GroupPropertyID);
            }

            PropertyTable myPT = GetPropertyTableForsymbol(mySymbol);
            ArrayList arryList = new ArrayList();
            LabelController myLabelController = gm.LabelControllerObj;
            LRRBindingManager lrrBindingManager = gm.LRRBindingManagerObj;

            //Remove al	Bound Properties from the Property Table
            if (flagRemove)
            {
                for (int i = myPT.Properties.Count - 1; i > 0; i--)
                {
                    PropertySpec ps = myPT.Properties[i] as PropertySpec;
                    if (ps.Category == "Bound Properties")
                    {
                        myPT.Properties.Remove(ps);
                    }
                }
                flagRemove = false;
            }

            try
            {
                LRRLabel uomLabel = null;
                //				if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.OPEN)
                //				{
                uomLabel = GraphicsUtilities.GetUOMLabelFromParentID(mySymbol, lrcp.SerializedID);
                //				}
                foreach (LRRMode myMode in myModes)
                {
                    if (myMode != null)
                    {
                        //LRRSiteComponentPropertyValue lrcpv	= lrcp.GetValueObjectForPropertyMode(myMode.ModeId);
                        IPropertyValue lrcpv = null;
                        List<IPropertyValue> lscpv = new List<IPropertyValue>();
                        if (myProp.GetPropertyType() == "1")
                        {
                            lscpv = LRRModel.PropertyValuesObj.GetPropValuesByPropID(myProp.GetID());
                        }
                        else
                        {
                            if (lrcp.LookupQualifierID != null && lrcp.LookupQualifierID.Trim() != "")
                                lscpv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValuesFromGrpPropIdAndLkupQualId(lrcp.GroupPropertyID, lrcp.LookupQualifierID);
                            else
                                lscpv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValuesinArrayFromGroupPropID(lrcp.GroupPropertyID);
                        }
                        if (lscpv != null)
                        {
                            IEnumerator iEnum = lscpv.GetEnumerator();
                            while (iEnum.MoveNext())
                            {
                                IPropertyValue lscpValue = iEnum.Current as IPropertyValue;
                                if (lscpValue.GetModeID() == myMode.ModeId)
                                {
                                    lrcpv = lscpValue;
                                }
                            }
                        }
                        foreach (DictionaryEntry obj in myLabelController.SymbolObjectBinding)
                        {
                            //Associate	the	key	object to Labellookup Type
                            LabelLookup lblLookup = obj.Key as LabelLookup;
                            if (myLabelController.SymbolObjectBinding.ContainsKey(lblLookup.PropertyName))
                            {
                                PropertySpec ps = myLabelController.SymbolsCollection[lblLookup] as PropertySpec;
                                myLabelController.SymbolsCollection.Remove(lblLookup);
                                arryList.Add(lblLookup);
                            }
                        }
                        for (int i = 0; i < arryList.Count; i++)
                        {
                            myLabelController.SymbolObjectBinding.Remove(arryList[i]);
                        }
                        arryList.Clear();
                    }
                }
                foreach (LRRMode myMode in myModes)
                {
                    if (myMode != null)
                    {
                        LabelLookup lblLookup = new LabelLookup();
                        lblLookup.SymbolID = mySymbol.GetPropertyValue("ID").ToString();
                        lblLookup.PropertyName = myProp.GetName() + "(" + myMode.ModeName + ")";
                        List<IPropertyValue> lscpv = new List<IPropertyValue>();
                        if (myProp.GetPropertyType() == "1")
                        {
                            lscpv = LRRModel.PropertyValuesObj.GetPropValuesByPropID(myProp.GetID());
                        }
                        else
                        {
                            if (lrcp.LookupQualifierID != null && lrcp.LookupQualifierID.Trim() != "")
                                lscpv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValuesFromGrpPropIdAndLkupQualId(lrcp.GroupPropertyID, lrcp.LookupQualifierID);
                            else
                                lscpv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValuesinArrayFromGroupPropID(lrcp.GroupPropertyID);
                        }
                        IPropertyValue lscpValue = null;
                        if (lscpv != null)
                        {
                            IEnumerator iEnum = lscpv.GetEnumerator();
                            while (iEnum.MoveNext())
                            {
                                IPropertyValue currentVal = iEnum.Current as IPropertyValue;
                                if (currentVal.GetModeID() == myMode.ModeId)
                                {
                                    if (uomLabel != null)
                                    {
                                        string modelUomID = currentVal.GetUOMID();//  LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                        lblLookup.PropertyValue = currentVal.GetPropertyValueForUOM(modelUomID, uomLabel.ID, currentVal.GetPropertyValue());
                                    }
                                    else
                                    {
                                        string modelUomID = currentVal.GetUOMID();//LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                        lblLookup.PropertyValue = currentVal.GetPropertyValueForUOM(modelUomID, myProp.GetUIUOMID(), currentVal.GetPropertyValue());
                                    }
                                    lscpValue = currentVal;
                                }
                            }
                        }
                        lblLookup.PropertyMode = myMode.ModeId;
                        lblLookup.PropertyID = myProp.GetID();
                        IPropertyValue lrcpv = lscpValue; //lrcp.GetValueObjectForPropertyMode(myMode.ModeId);
                        if (lrcpv != null && !myLabelController.SymbolsCollection.ContainsKey(lblLookup))
                        {
                            int propIndex = -1;
                            if (uomLabel != null)
                            {
                                string modelUomID = lrcpv.GetUOMID();//LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                LRRUOMType uomTypeObj = uomLabel.Tag as LRRUOMType;
                                if (!uomTypeObj.IsEVV)
                                {
                                    string propValue = lrcpv.GetPropertyValueForUOM(modelUomID, uomLabel.ID, lrcpv.GetPropertyValue());
                                    if (lrcpv.GetDataType() == null)
                                        propValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(propValue, _app);
                                    propIndex = myPT.Properties.Add(new PropertySpec(lrcp.Name + "(" + myMode.ModeName + ")", typeof(String), "Bound Properties", myProp.GetName(), propValue, "", typeof(String), lrcp.SerializedID));
                                }
                                else
                                {
                                    string propValue = lrcpv.GetPropertyValueForUOM(modelUomID, uomLabel.ID, lrcpv.GetPropertyValue());
                                    propIndex = myPT.Properties.Add(new PropertySpec(lrcp.Name + "(" + myMode.ModeName + ")", typeof(String), "Bound Properties", myProp.GetName(), propValue, typeof(Symcon.Lightridge.LRRUI.Controls.EVVTypeEditor), "", lrcp.SerializedID));
                                }
                            }
                            Attribute[] attrib = new Attribute[1];
                            ReadOnlyAttribute readonlyattrib;
                            readonlyattrib = new ReadOnlyAttribute(!lrcpv.GetEditableState());
                            attrib[0] = readonlyattrib;
                            myPT.Properties[propIndex].AttributesNew = attrib;
                            PropertySpec ps = myPT.Properties[propIndex] as PropertySpec;
                            myLabelController.SymbolsCollection.Add(lblLookup, ps);
                            lrcpv.PropertyValueChanged += new PropertyChanged(gm.PropertyValueChanged);
                        }
                        myLabelController.SymbolObjectBinding.Add(lblLookup, mySymbol);
                    }
                }
                gm.CanvasObj.View.Refresh();
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(GraphicsSiteComponentUtilities), "AddtoPropertyGrid :" + ex.Message, Logger.Level.Debug);
            }
        }

        ///	<summary>
        ///	Create a matrix	for	the	selected Property and add label	
        ///	at defined positions
        ///	00(Nothing)					01(PropValue 1)		02(Nothing)
        ///	10(PropName)				11(PropValue 2)		12(UOM)
        ///	20(Property	Description)	21(PropValue 3)		22(Nothing)
        ///	</summary>
        ///	<param name="myProp">Selected Property</param>
        ///	<param name="count">Selected Property Index</param>
        ///	<param name="mySymbol">Symbol</param>
        ///	<param name="myModes">Selected Modes</param>
        private static void InitializeLabels(LRRPlantItemProperty plProp, int count, Symbol mySymbol, ArrayList myModes, GraphicsManager gm, GraphicsManagers.PropertiesMappingMode propertiesMappingMode, Symcon.BusinessObjects.CustomCollection selectedProps, GraphicsManagers.UserMode userMode, float offsetXPropName, float offsetXPropValue, float offsetXUOM, ref int lblVsbltyIdx)
        {
            LRRMode compare1Mode = null;
            IPropertyValue compare1PropValueObj = null;
            LRRMode refMode = null;
            IPropertyValue refPropValueObj = null;
            LRRMode compare2Mode = null;
            IPropertyValue compare2PropValueObj = null;
            bool isCompare1Visible = true;
            bool isCompare2Visible = true;
            double currentTolerence = 0;
            double currentRefValue = 0;
            double currentCompare1Value = 0;
            double currentCompare2Value = 0;
            double minValue = 0;
            double maxValue = 0;
            LRRBindingManager lrrBindingManager = gm.LRRBindingManagerObj;
            int _maxRow = 3;
            int _maxCol = 3;
            IProperty myProp = null;
            LRRUOMType uomTypeObj = null;
            string modelUOMID = null;
            string uiUOMIDID = null;

            if (plProp.ID != null)
            {
                if (plProp.GroupPropertyID == null && plProp.ID != null)
                {
                    myProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(plProp.ID);
                }
                else if (plProp.GroupPropertyID != null)
                {
                    myProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(plProp.GroupPropertyID);
                }
                //Set the UOMType Object
                if (myProp.GetPropertyType() == "1")
                {
                    //For Simple Properties
                    if (myProp.GetUOMType() != null)
                    {
                        uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(myProp.GetUOMType());
                    }
                }
                else
                {
                    //For Grouped Properties
                    if (plProp.LookupQualifierID != null && plProp.LookupQualifierID.Trim() != "")
                    {
                        LRRLookUpQualifier lrrLookUpQualifier = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(plProp.LookupQualifierID);
                        if (lrrLookUpQualifier != null)
                            uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(lrrLookUpQualifier.UOMTypeID);
                    }
                    //For Data Arrays
                    else
                    {
                        LRRPropertyType lrrPropType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(myProp.GetPropertyType());
                        if (lrrPropType != null)
                            uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(lrrPropType.UOMTypeID);
                    }
                }
                //Set Model and UI UOM id's
                if (uomTypeObj != null)
                {
                    string modelID = LRRModel.MeasurementSystems.GetCurrentModelUOM().ID;
                    string localID = _app.AppLocaleID;
                    modelUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(modelID, uomTypeObj.UOMTypeID);
                    uiUOMIDID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localID, uomTypeObj.UOMTypeID);
                }
                //Set Modes and Modes Value 
                foreach (LRRMode mode in myModes)
                {
                    if (mode != null)
                    {
                        //Set Compare1ValueObject
                        if (mode.ModeTypeID == "1")
                        {
                            compare1Mode = mode;
                            IPropertyValue iPropVal = SetValueObjects(mode, ref currentCompare1Value, uomTypeObj, myProp, plProp);
                            compare1PropValueObj = iPropVal;
                        }
                        //Set RefValueObject
                        else if (mode.ModeTypeID == "2")
                        {
                            refMode = mode;
                            IPropertyValue iPropVal = SetValueObjects(mode, ref currentRefValue, uomTypeObj, myProp, plProp);
                            refPropValueObj = iPropVal;
                        }
                        //Set Compare2ValueObject
                        else if (mode.ModeTypeID == "3")
                        {
                            compare2Mode = mode;
                            IPropertyValue iPropVal = SetValueObjects(mode, ref currentCompare2Value, uomTypeObj, myProp, plProp);
                            compare2PropValueObj = iPropVal;
                        }
                    }
                }

                //Set Min tolerance	and	Max	tolerance Value
                if (uomTypeObj != null && uomTypeObj.Tolerance != null && !uomTypeObj.IsEVV)
                {
                    currentTolerence = (double)Symcon.BusinessObjects.SymExcelConverter.Convert(typeof(string), uomTypeObj.Tolerance, typeof(double));//Decimal.Parse(uomTypeObj.Tolerance);
                    minValue = currentRefValue - (currentRefValue * currentTolerence);
                    maxValue = currentRefValue + (currentRefValue * currentTolerence);
                }

                //Set Visibility variables based on	tolerance
                if (!_lrrOptionsObj.ShowLabels)
                {
                    if (uomTypeObj != null && !uomTypeObj.IsEVV)
                    {
                        if ((currentCompare1Value >= maxValue || currentCompare1Value <= minValue) && currentRefValue != 0)
                        {
                            isCompare1Visible = true;
                        }
                        else
                        {
                            isCompare1Visible = false;
                        }

                        if ((currentCompare2Value >= maxValue || currentCompare2Value <= minValue) && currentRefValue != 0)
                        {
                            isCompare2Visible = true;
                        }
                        else
                        {
                            isCompare2Visible = false;
                        }
                    }
                }
            }
            //ArrayList arrprops = mySymbol.GetPropertyValue("ArryIndexedProps") as ArrayList;
            //Check	fo nullValue or IsExposed for Property Object
            if (plProp.Name == "NullValue" || (!plProp.IsExposed && userMode != GraphicsManagers.UserMode.IMPLEMENTOR))
            {
                for (int i = 0; i < ((mySymbol.Labels.Count - 2) / 4); i++)
                {
                    LRRLabel lbl = mySymbol.Labels[_labelindex] as LRRLabel;
                    lbl.Text = "";
                    lbl.ParentID = "";
                    lbl.BackgroundStyle.Color = Color.Transparent;
                    lbl.FillStyle.Color = Color.Black;
                    _labelindex++;
                }
                lblVsbltyIdx--;
                return;
            }

            LRRLabel uomLabel = null;

            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN || propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NONE)
            {
                uomLabel = GraphicsUtilities.GetUOMLabel(mySymbol.Labels[_labelindex] as LRRLabel);
                LRRUOMs lrrUOMs = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomTypeObj.UOMTypeID);
                bool flagUOMTypeMisMatch = true;
                foreach (LRRUOM lrrUOM in lrrUOMs)
                {
                    if (uomLabel.ID == lrrUOM.ID)
                    {
                        flagUOMTypeMisMatch = false;
                        break;
                    }
                }

                if (uomLabel != null && flagUOMTypeMisMatch)
                {
                    //Change in Persisted UOM in case of change in UOMTYPE in Master Attributes
                    string myUOMName = GraphicsUtilities.GetUomName(uiUOMIDID);
                    //string myUOMName = GetUomName(refPropValueObj.GetUOMID());
                    uomLabel.Text = myUOMName;
                    uomLabel.Name = "UOM" + "(" + myUOMName + ")-" + myProp.GetName();
                    uomLabel.ParentID = plProp.SerializedID;
                    uomLabel.ID = uiUOMIDID;
                    uomLabel.IsUOM = true;
                }
            }

            for (int i = 0; i < _maxRow; i++)
            {
                for (int j = 0; j < _maxCol; j++)
                {
                    string str = (string)i.ToString(Symcon.BusinessObjects.SymConverter.Culture) + j.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                    //-------------------------------------------------
                    //|	00(Nothing)				01(PropValue 1)		02(Nothing)
                    //|	10(PropName)			11(PropValue 2)		12(UOM)
                    //|	20(Property	Descp.)		21(PropValue 3)		22(Flag)
                    //-------------------------------------------------
                    switch (str)
                    {
                        case "00":
                            {
                                break;
                            }
                        case "01":
                            {
                                LRRLabel smblLbl = mySymbol.Labels[_labelindex] as LRRLabel;
                                smblLbl.Visible = true;
                                float symbolHeight = smblLbl.Height;
                                _labelindex++;

                                if (compare1PropValueObj != null)
                                {
                                    if (smblLbl.Tag != null)
                                    {
                                        gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "PropertyValue", smblLbl, true);
                                        gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "ErrCode", smblLbl, true);
                                    }
                                    lrrBindingManager.AddBinding(compare1PropValueObj, "PropertyValue", smblLbl, "Text");

                                    //Add Colour Binding
                                    lrrBindingManager.AddBinding(compare1PropValueObj, "ErrCode", smblLbl, "LabelColorCode");

                                    if (uomLabel != null)
                                    {
                                        //string uomid = uomTypeObj.UOMTypeID;// LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                        string newPropValue = compare1PropValueObj.GetPropertyValueForUOM(modelUOMID, uomLabel.ID, compare1PropValueObj.GetPropertyValue());
                                        if (compare1PropValueObj.GetDataType() == null)
                                            newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                        smblLbl.Text = newPropValue;//compare1PropValueObj.GetPropertyValueForUOM(modelUOMID,uomLabel.ID,compare1PropValueObj.GetPropertyValue());
                                    }
                                    else
                                    {
                                        //string uomid = compare1PropValueObj.GetUOMID();//LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                        string newPropValue = compare1PropValueObj.GetPropertyValueForUOM(modelUOMID, uiUOMIDID, compare1PropValueObj.GetPropertyValue());
                                        if (compare1PropValueObj.GetDataType() == null)
                                            newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                        smblLbl.Text = newPropValue;//compare1PropValueObj.GetPropertyValueForUOM(modelUOMID,uiUOMIDID,compare1PropValueObj.GetPropertyValue());
                                    }
                                    smblLbl.ID = compare1PropValueObj.GetID();
                                    smblLbl.Tag = compare1PropValueObj;
                                    smblLbl.IsCompare1 = true;
                                    if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)
                                    {
                                        smblLbl.OffsetX = offsetXPropValue + offsetXUOM - offsetXPropValue / 2 /*-offsetXUOM/2*/;
                                        smblLbl.Width = -offsetXPropValue;
                                    }
                                    smblLbl.BackgroundStyle.Color = compare1PropValueObj.GetBackColor();
                                    smblLbl.FillStyle.Color = compare1PropValueObj.GetForeColor();
                                    GraphicsUtilities.SetColor(smblLbl, compare1PropValueObj);
                                    smblLbl.Name = plProp.Name + "(" + compare1Mode.ModeName + ")";
                                    smblLbl.Visible = isCompare1Visible;
                                }
                                else
                                {
                                    smblLbl.Text = "";
                                    smblLbl.ID = "";
                                    smblLbl.Tag = null;
                                }
                                smblLbl.HorizontalAlignment = StringAlignment.Far;
                                smblLbl.ParentID = plProp.SerializedID;
                                //	smblLbl.TextChanged+=new PropertyChanged(smblLbl_TextChanged);
                                smblLbl.LabelDoubleClick += new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                InitLabelLookFeel(smblLbl);
                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                {
                                    smblLbl.Height = symbolHeight;
                                }
                                break;
                            }
                        case "02":
                            {
                                LRRLabel smblLbl;
                                if (mySymbol.Labels.Count < 22 + (selectedProps.Count * 2))
                                {
                                    smblLbl = new LRRLabel();
                                    smblLbl.Anchor = BoxPosition.TopLeft;
                                    mySymbol.Labels.Insert(_labelindex, smblLbl);
                                    smblLbl.EditStyle.Enabled = true;
                                    smblLbl.EditStyle.AllowResize = false;

                                    if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)
                                    {
                                        smblLbl.OffsetX = offsetXUOM - offsetXUOM / 2;
                                    }
                                    smblLbl.HorizontalAlignment = StringAlignment.Center;
                                    smblLbl.VerticalAlignment = StringAlignment.Far;
                                    smblLbl.Width = 10;
                                    smblLbl.Height = 10;
                                    smblLbl.Translate(mySymbol.Labels[_labelindex - 1].X + mySymbol.Labels[_labelindex - 1].Width + smblLbl.Width - smblLbl.X, mySymbol.Labels[_labelindex - 1].Y - smblLbl.Y);
                                    smblLbl.LabelDoubleClick += new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                }
                                else
                                {
                                    smblLbl = mySymbol.Labels[_labelindex] as LRRLabel;
                                }
                                float symbolHeight = smblLbl.Height;
                                smblLbl.ParentID = plProp.SerializedID;
                                _labelindex++;
                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                {
                                    smblLbl.Height = symbolHeight;
                                }
                                break;
                            }
                        case "10":
                            {
                                LRRLabel smblLbl = mySymbol.Labels[_labelindex] as LRRLabel;
                                float symbolHeight = smblLbl.Height;
                                _labelindex++;

                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL && smblLbl.ParentID != plProp.SerializedID)
                                {
                                    smblLbl.Text = plProp.Name;
                                    InitLabelLookFeel(smblLbl);

                                    smblLbl.OffsetX = (offsetXPropName + offsetXPropValue + offsetXUOM) - offsetXPropName / 2;
                                    smblLbl.Width = -offsetXPropName;
                                    smblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[(lblVsbltyIdx * 2) + 1];
                                }
                                else if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.REMAP)
                                {
                                 
                                    smblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[(lblVsbltyIdx * 2) + 1];
                                }
                                smblLbl.Text = plProp.Name;
                                smblLbl.ParentID = plProp.SerializedID;
                                smblLbl.ID = myProp.GetID();
                                smblLbl.IsParent = true;
                                smblLbl.Tag = myProp;
                                //if (userMode == GraphicsManagers.UserMode.IMPLEMENTOR)
                                //    smblLbl.ReadOnly = false;
                                smblLbl.LabelDoubleClick += new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                {
                                    smblLbl.Height = symbolHeight;
                                }
                                break;
                            }
                        case "11":
                            {
                                LRRLabel smblLbl = mySymbol.Labels[_labelindex] as LRRLabel;
                                float symbolHeight = smblLbl.Height;
                                smblLbl.Visible = true;
                                _labelindex++;
                                if (refPropValueObj != null)
                                {
                                    if (smblLbl.Tag != null)
                                    {
                                        gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "PropertyValue", smblLbl, true);
                                        gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "ErrCode", smblLbl, true);
                                    }
                                    lrrBindingManager.AddBinding(refPropValueObj, "PropertyValue", smblLbl, "Text");

                                    //Add Colour Binding
                                    lrrBindingManager.AddBinding(refPropValueObj, "ErrCode", smblLbl, "LabelColorCode");

                                    if (uomLabel != null)
                                    {
                                        string newPropValue = refPropValueObj.GetPropertyValueForUOM(modelUOMID, uomLabel.ID, refPropValueObj.GetPropertyValue());
                                        if (refPropValueObj.GetDataType() == null)
                                            newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                        //string uomid = refPropValueObj.GetUOMID();// LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                        smblLbl.Text = newPropValue;// refPropValueObj.GetPropertyValueForUOM(modelUOMID, uomLabel.ID, refPropValueObj.GetPropertyValue());
                                    }
                                    else
                                    {
                                        //string uomid = refPropValueObj.GetUOMID();// LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                        string newPropValue = refPropValueObj.GetPropertyValueForUOM(modelUOMID, uiUOMIDID, refPropValueObj.GetPropertyValue());
                                        if (refPropValueObj.GetDataType() == null)
                                            newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                        smblLbl.Text = newPropValue;// refPropValueObj.GetPropertyValueForUOM(modelUOMID, uiUOMIDID, refPropValueObj.GetPropertyValue());
                                    }
                                    smblLbl.ID = refPropValueObj.GetID();
                                    smblLbl.Tag = refPropValueObj;
                                    smblLbl.IsRefrence = true;
                                    smblLbl.BackgroundStyle.Color = refPropValueObj.GetBackColor();
                                    smblLbl.FillStyle.Color = refPropValueObj.GetForeColor();
                                    GraphicsUtilities.SetColor(smblLbl, refPropValueObj);
                                    if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)
                                    {
                                        smblLbl.OffsetX = (offsetXPropValue + offsetXUOM) - offsetXPropValue / 2;
                                        smblLbl.Width = -offsetXPropValue;
                                    }
                                    if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                    {
                                        smblLbl.Height = symbolHeight;
                                    }
                                    smblLbl.Name = plProp.Name + "(" + refMode.ModeName + ")";
                                }
                                else
                                {
                                    smblLbl.Text = "";
                                    smblLbl.ID = "";
                                    smblLbl.Tag = null;
                                }

                                smblLbl.ParentID = plProp.SerializedID;
                                //	smblLbl.TextChanged+=new PropertyChanged(smblLbl_TextChanged);
                                smblLbl.LabelDoubleClick += new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                InitLabelLookFeel(smblLbl);
                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                {
                                    smblLbl.Height = symbolHeight;
                                }
                                break;
                            }
                        case "12":
                            {
                                LRRLabel smblLbl = mySymbol.Labels[_labelindex] as LRRLabel;
                                float symbolHeight = smblLbl.Height;
                                _labelindex++;
                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN && smblLbl.ID != "" && smblLbl.ID != null)
                                {
                                    smblLbl.Tag = uomTypeObj;//.ChildModel.UOMs.GetUOMObjectFromID(smblLbl.ID);
                                    smblLbl.LabelDoubleClick += new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                    smblLbl.Height = symbolHeight;
                                }
                                else if (propertiesMappingMode != GraphicsManagers.PropertiesMappingMode.NONE)
                                {
                                    smblLbl.LabelDoubleClick -= new NodeMouseEventHandler(smblLbl_LabelDoubleClick);

                                    // true;
                                    string myUOMName = GraphicsUtilities.GetUomName(uiUOMIDID);
                                    //string myUOMName = GetUomName(refPropValueObj.GetUOMID());
                                    smblLbl.Text = myUOMName;
                                    smblLbl.Name = "UOM" + "(" + myUOMName + ")-" + myProp.GetName();
                                    smblLbl.ParentID = plProp.SerializedID;
                                    smblLbl.ID = uiUOMIDID;
                                    smblLbl.IsUOM = true;
                                    //smblLbl.BackgroundStyle.Color = refPropValueObj.GetBackColor();
                                    //smblLbl.FillStyle.Color =	compare2PropValueObj.GetForeColor();
                                    if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)
                                    {
                                        smblLbl.OffsetX = offsetXUOM - offsetXUOM / 2;
                                        smblLbl.Width = -offsetXUOM;
                                    }
                                    smblLbl.Tag = uomTypeObj;//LRRModel.ChildModel.UOMs.GetUOMObjectFromID(uiUOMIDID);
                                    smblLbl.LabelDoubleClick += new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                    InitLabelLookFeel(smblLbl);

                                }
                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL || propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.REMAP)//&& smblLbl.ParentID != plProp.SerializedID)
                                {
                                    smblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[(lblVsbltyIdx * 2) + 2];
                                }
                                //else if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.REMAP)
                                //{
                                //    smblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[(lblVsbltyIdx * 2) + 2];
                                //}
                                break;
                            }
                        case "21":
                            {
                                LRRLabel smblLbl = mySymbol.Labels[_labelindex] as LRRLabel;
                                smblLbl.Visible = true;
                                float symbolHeight = smblLbl.Height;
                                _labelindex++;

                                if (compare2PropValueObj != null)
                                {
                                    if (smblLbl.Tag != null)
                                    {
                                        gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "PropertyValue", smblLbl, true);
                                        gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "ErrCode", smblLbl, true);
                                    }
                                    lrrBindingManager.AddBinding(compare2PropValueObj, "PropertyValue", smblLbl, "Text");

                                    //Add Colour Binding
                                    lrrBindingManager.AddBinding(compare2PropValueObj, "ErrCode", smblLbl, "LabelColorCode");

                                    if (uomLabel != null)
                                    {
                                        string newPropValue = compare2PropValueObj.GetPropertyValueForUOM(modelUOMID, uomLabel.ID, compare2PropValueObj.GetPropertyValue());
                                        if (compare2PropValueObj.GetDataType() == null)
                                            newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                        //string uomid = compare2PropValueObj.GetUOMID();// LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                        smblLbl.Text = newPropValue;// compare2PropValueObj.GetPropertyValueForUOM(modelUOMID, uomLabel.ID, compare2PropValueObj.GetPropertyValue());
                                    }
                                    else
                                    {
                                        string newPropValue = compare2PropValueObj.GetPropertyValueForUOM(modelUOMID, uiUOMIDID, compare2PropValueObj.GetPropertyValue());
                                        if (compare2PropValueObj.GetDataType() == null)
                                            newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                        //string uomid = compare2PropValueObj.GetUOMID();// LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,myProp.GetUOMType());
                                        smblLbl.Text = newPropValue;//compare2PropValueObj.GetPropertyValueForUOM(modelUOMID,uiUOMIDID,compare2PropValueObj.GetPropertyValue());
                                    }
                                    smblLbl.ID = compare2PropValueObj.GetID();
                                    smblLbl.IsCompare2 = true;
                                    if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)
                                    {
                                        smblLbl.OffsetX = offsetXPropValue + offsetXUOM - offsetXPropValue / 2 /* - offsetXUOM/2*/;
                                        smblLbl.Width = -offsetXPropValue;
                                    }
                                    smblLbl.Tag = compare2PropValueObj;
                                    smblLbl.BackgroundStyle.Color = compare2PropValueObj.GetBackColor();
                                    smblLbl.FillStyle.Color = compare2PropValueObj.GetForeColor();
                                    GraphicsUtilities.SetColor(smblLbl, compare2PropValueObj);
                                    smblLbl.Name = plProp.Name + "(" + compare2Mode.ModeName + ")";
                                    smblLbl.Visible = isCompare2Visible;
                                }
                                else
                                {
                                    smblLbl.Text = "";
                                    smblLbl.ID = "";
                                    smblLbl.Tag = null;
                                    
                                }
                                smblLbl.ParentID = plProp.SerializedID;
                                //	smblLbl.TextChanged+=new PropertyChanged(smblLbl_TextChanged);
                                smblLbl.LabelDoubleClick += new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                InitLabelLookFeel(smblLbl);
                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                {
                                    smblLbl.Height = symbolHeight;
                                }
                                break;
                            }

                        case "22":
                            {
                                LRRLabel smblLbl;
                                if (mySymbol.Labels.Count < 22 + (selectedProps.Count * 2))
                                {
                                    smblLbl = new LRRLabel();
                                    smblLbl.Anchor = BoxPosition.TopLeft;
                                    mySymbol.Labels.Insert(_labelindex, smblLbl);
                                    smblLbl.EditStyle.Enabled = true;
                                    smblLbl.EditStyle.AllowResize = false;
                                    smblLbl.HorizontalAlignment = StringAlignment.Center;
                                    smblLbl.VerticalAlignment = StringAlignment.Far;
                                    smblLbl.Width = 10;
                                    smblLbl.Height = 10;
                                    smblLbl.LabelDoubleClick += new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                    smblLbl.Translate(mySymbol.Labels[_labelindex - 1].X + mySymbol.Labels[_labelindex - 1].Width + smblLbl.Width - smblLbl.X, mySymbol.Labels[_labelindex - 1].Y - smblLbl.Y);
                                }
                                else
                                {
                                    smblLbl = mySymbol.Labels[_labelindex] as LRRLabel;
                                }
                                float symbolHeight = smblLbl.Height;
                                _labelindex++;
                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                {
                                    smblLbl.Height = symbolHeight;
                                }
                                break;
                            }

                        default:
                            {
                                break;
                            }
                    }
                }
            }
        }



        ///	<summary>
        ///	Given a	PropertyTable of a symbol returns the selected Properties 
        ///	</summary>
        ///	<param name="pt"></param>
        ///	<returns>Hashtable</returns>
        public static Symcon.BusinessObjects.CustomCollection GetSelectedSiteComponentProperties(PropertyTable pt, Symbol smbl)
        {
            Hashtable hshSelectedProps = new Hashtable();
            LRRPlantItemProperties selectedProperties = new LRRPlantItemProperties();// = new LRRSiteComponentProperties();
            try
            {
                ArrayList arryPropIndex = smbl.GetPropertyValue("ArryIndexedProps") as ArrayList;
                for (int j = 0; j < arryPropIndex.Count; j++)
                {
                    if (arryPropIndex[j].ToString() == "Null")
                    {
                        LRRPlantItemProperty lrrscp = new LRRPlantItemProperty();
                        lrrscp.Name = "NullValue";
                        selectedProperties.Add(lrrscp);
                    }
                    else
                    {
                        PropertyTable myPT = pt;
                        ArrayList arrToRemovePropSpecs = new ArrayList();

                        Symcon.Lightridge.LRRUI.PropertyBag.PropertySpecCollection pscl = pt.Properties;
                        foreach (PropertySpec ps in pscl)
                        {
                            if (ps.ObjectRef != null && hshSelectedProps.ContainsKey(ps.ObjectRef.ToString()))
                                continue;
                            //Ashwin
                            if (ps.Name == "PlantItem")
                            {
                                if (ps.ObjectRef != null && ps.ObjectRef.GetType().GetProperty("PlantItem") != null)
                                {
                                    SiteItem siteItem = ps.ObjectRef.GetType().GetProperty("PlantItem").GetValue(ps.ObjectRef, null) as SiteItem;
                                    LRRSiteComponent lsc =  LRRModel.SiteComponentsObj.GetSiteComponentByID(siteItem.ID);
                                    if (lsc != null)
                                    {
                                        string str = LRRModel.ChildModel.GetPlantLevelName(lsc.PlantUnit);

                                        string name = lsc.Name;
                                        if (str != null && str.Trim() != "")
                                            name = name + "[" + str + " ]";
                                        else
                                            name = name + "[ Level not defined ]";

                                        name = name + "[" + lsc.EquipmentTag + "]";

                                        siteItem.Name = name;
                                    }
                                }
                            }
                            if (ps.Category == "Bound Properties")
                            {
                                LRRSiteComponentProperty lrrcp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(ps.ObjectRef.ToString());
                                if (lrrcp != null && !selectedProperties.Contains(lrrcp))
                                {
                                    LRRMasterProperty lrrmcp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(lrrcp.PropertyID);
                                    if (lrrmcp.IsExposed)
                                    {
                                        LRRPlantItemProperty propertyObj = new LRRPlantItemProperty();
                                        propertyObj.ID = lrrcp.ID;
                                        propertyObj.PropertyID = lrrcp.ID;
                                        propertyObj.Name = lrrcp.Name;
                                        propertyObj.Description = lrrcp.Description;
                                        propertyObj.SerializedID = lrrcp.ID;
                                        selectedProperties.Add(propertyObj);
                                        propertyObj.IsExposed = true;
                                        hshSelectedProps.Add(ps.ObjectRef.ToString(), propertyObj);
                                        propertyObj = null;
                                    }
                                    else
                                    {
                                        LRRPlantItemProperty propertyObj = new LRRPlantItemProperty();
                                        propertyObj.ID = lrrcp.ID;
                                        propertyObj.PropertyID = lrrcp.ID;
                                        propertyObj.SerializedID = lrrcp.ID;
                                        propertyObj.Name = lrrcp.Name;
                                        propertyObj.Description = lrrcp.Description;
                                        propertyObj.IsExposed = false;
                                        selectedProperties.Add(propertyObj);
                                        hshSelectedProps.Add(ps.ObjectRef.ToString(), propertyObj);
                                        propertyObj = null;
                                    }
                                }
                                else
                                {
                                    //string sGroupPropertyID = "";
                                    string masterPropID = "";

                                    string currentLookupQualifierID = "";
                                    string currentGroupedPropertyID = "";

                                    if (ps.ObjectRef.ToString().IndexOf('|') != -1)
                                    {
                                        string[] ids = ps.ObjectRef.ToString().Split("|".ToCharArray());
                                        currentLookupQualifierID = ids[0];
                                        currentGroupedPropertyID = ids[1];
                                    }
                                    else if (ps.ObjectRef.ToString().Trim() != "")
                                    {
                                        //arrToRemovePropSpecs = new ArrayList();
                                        //pscl.Remove(ps);
                                        arrToRemovePropSpecs.Add(ps);
                                        continue;
                                    }
                                    else
                                    {
                                        continue;
                                    }
                                    //LRRSiteGroupPropertyQualifierValue grPropValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(ps.ObjectRef.ToString());
                                    //							//sGroupPropertyID = currentGroupedPropertyID;
                                    //LRRModel.SiteGroupPropertyQualifierValuesObj.
                                    LRRSiteComponentGroupProperty grpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(currentGroupedPropertyID);
                                    if (grpProp != null)
                                    {
                                        masterPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(currentGroupedPropertyID).MasterPropID;
                                        if (LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(masterPropID).IsExposed)
                                        {
                                            LRRPlantItemProperty propertyObj1 = new LRRPlantItemProperty();
                                            string sQualifierID = "";
                                            string sGroupPropertyName = "";
                                            string scpName = "";
                                            string gpAttributeName = "";
                                            string sQualifierName = "";
                                            propertyObj1.ID = currentGroupedPropertyID;
                                            propertyObj1.GroupPropertyID = currentGroupedPropertyID;
                                            //propertyObj1.SiteGroupPropertyQualifierValueID = grPropValue.SiteGroupPropertyQualifierValueID;
                                            propertyObj1.LookupQualifierID = currentLookupQualifierID; ;// grPropValue.LookUpQualifierID == null ? null : grPropValue.LookUpQualifierID;
                                            //propertyObj1.LookupPropertiesID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(sGroupPropertyID).LookUpPropertiesID;
                                            propertyObj1.PropertyID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(currentGroupedPropertyID).SiteComponentPropID;
                                            propertyObj1.IsExposed = true;
                                            scpName = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(propertyObj1.PropertyID).Name;
                                            gpAttributeName = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(propertyObj1.GroupPropertyID).AttributeName;
                                            if (currentLookupQualifierID != null && currentLookupQualifierID.Trim() != "") //--Its a case of data array
                                            {
                                                sQualifierID = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(propertyObj1.LookupQualifierID).QualifierID;
                                                sQualifierName = LRRModel.Qualifiers.getQualifierObjectForQualifierID(sQualifierID).Name;
                                                sGroupPropertyName = scpName + " / " + gpAttributeName + " / " + sQualifierName + " [ Grouped ]";
                                            }
                                            else
                                            {
                                                sGroupPropertyName = scpName + " / " + gpAttributeName + " [ Data Array ]";
                                            }
                                            propertyObj1.Name = sGroupPropertyName;

                                            propertyObj1.SerializedID = propertyObj1.LookupQualifierID + "|" + propertyObj1.GroupPropertyID;
                                            selectedProperties.Add(propertyObj1);
                                            hshSelectedProps.Add(ps.ObjectRef.ToString(), propertyObj1);
                                            propertyObj1 = null;
                                        }
                                        else
                                        {
                                            LRRPlantItemProperty propertyObj1 = new LRRPlantItemProperty();
                                            string sQualifierID = "";
                                            string sGroupPropertyName = "";
                                            string scpName = "";
                                            string gpAttributeName = "";
                                            string sQualifierName = "";
                                            propertyObj1.ID = currentGroupedPropertyID;
                                            propertyObj1.GroupPropertyID = currentGroupedPropertyID;
                                            //propertyObj1.SiteGroupPropertyQualifierValueID = grPropValue.SiteGroupPropertyQualifierValueID;
                                            propertyObj1.LookupQualifierID = currentLookupQualifierID;// grPropValue.LookUpQualifierID == null ? null : grPropValue.LookUpQualifierID;
                                            //propertyObj1.LookupPropertiesID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(sGroupPropertyID).LookUpPropertiesID;
                                            propertyObj1.PropertyID = grpProp.SiteComponentPropID;//LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(currentGroupedPropertyID).SiteComponentPropID;
                                            propertyObj1.IsExposed = false;
                                            scpName = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(propertyObj1.PropertyID).Name;
                                            gpAttributeName = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(propertyObj1.GroupPropertyID).AttributeName;
                                            if (currentLookupQualifierID == null) //--Its a case of data array
                                            {
                                                sGroupPropertyName = scpName + " / " + gpAttributeName + " [ Data Array ]";
                                            }
                                            else
                                            {
                                                sQualifierID = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(propertyObj1.LookupQualifierID).QualifierID;
                                                sQualifierName = LRRModel.Qualifiers.getQualifierObjectForQualifierID(sQualifierID).Name;
                                                sGroupPropertyName = scpName + " / " + gpAttributeName + " / " + sQualifierName + " [ Grouped ]";
                                            }
                                            propertyObj1.Name = sGroupPropertyName;
                                            propertyObj1.SerializedID = propertyObj1.LookupQualifierID + "|" + propertyObj1.GroupPropertyID;
                                            selectedProperties.Add(propertyObj1);
                                            hshSelectedProps.Add(ps.ObjectRef.ToString(), propertyObj1);
                                            propertyObj1 = null;
                                        }
                                    }
                                    else
                                    {
                                        arrToRemovePropSpecs.Add(ps);
                                    }

                                }
                            }
                        }
                        foreach (PropertySpec var in arrToRemovePropSpecs)
                        {

                            foreach (LRRLabel lbl in smbl.Labels)
                            {
                                if (lbl.ParentID == var.ObjectRef.ToString())
                                {
                                    lbl.Text = "";
                                    lbl.ID = "";
                                    lbl.Tag = null;
                                    continue;
                                }
                            }
                            pscl.Remove(var);
                        }
                    }
                }
                return selectedProperties;
            }
            catch (Exception ex)
            {

                Logger.LogMessage(typeof(GraphicsSiteComponentUtilities), "GetSelectedSiteComponentProperties :" + ex.Message, Logger.Level.Debug);
                return selectedProperties;

            }
        }

        private static IPropertyValue SetValueObjects(LRRMode mode, ref double currentCompareValue, LRRUOMType uomTypeObj, IProperty myProp, LRRPlantItemProperty plProp)
        {
            Symcon.LightRidge.BusinessObjects.IPropertyValue iPropVal = null;
            if (myProp.GetPropertyType() == "1")
            {
                iPropVal = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(myProp.GetID(), mode.ModeId);
            }
            else
            {
                if (plProp.LookupQualifierID != null && plProp.LookupQualifierID.Trim() != "")
                {
                    iPropVal = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(plProp.GroupPropertyID, plProp.LookupQualifierID, mode.ModeId);
                }
                else
                {
                    iPropVal = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValueFromGroupPropIDandModeID(plProp.GroupPropertyID, mode.ModeId);
                }
            }
            if (iPropVal != null)
            {
                if (uomTypeObj != null && !uomTypeObj.IsEVV)
                    currentCompareValue = (double)Symcon.BusinessObjects.SymExcelConverter.Convert(typeof(string), iPropVal.GetPropertyValue(), typeof(double));//Decimal.Parse(compare1PropValueObj.PropertyValue);

            }
            return iPropVal;
        }

        private static PropertyTable GetPropertyTableForsymbol(Symbol symbol)
        {
            PropertyTable pt = symbol.GetPropertyValue("PT") as PropertyTable;
            return pt;
        }

        ///	<summary>
        ///	Retrives the site components for a symbol give site	Component name
        ///	</summary>
        ///	<param name="symbol"></param>
        ///	<returns></returns>
        private static LRRSiteComponent GetSiteComponentForSymbol(Symbol symbol)
        {
            string siteItemId = GraphicsUtilities.GetSiteItemIdForSymbol(symbol);
            Symcon.LightRidge.BusinessObjects.LRRSiteComponents lscs = LRRModel.SiteComponentsObj;
            LRRSiteComponent lsc = lscs.GetSiteComponentByID(siteItemId);//Get the Site	Item	By Name
            return lsc;
        }

        ///	<summary>
        ///	Set	The	General	Properties For the Label
        ///	</summary>
        ///	<param name="lbl"></param>
        private static void InitLabelLookFeel(SymbolLabel lbl)
        {
        }

        #endregion

        #region	Fields

        private static LRRApp _app;
        private static LRROptions _lrrOptionsObj;

        private static int _labelindex = 1;
        private static bool flagRemove = true;

        #endregion

        public static void SetOverRideValue(LRRLabel lbl, string oldValue)
        {
            LRRLabel parentLabel = GraphicsUtilities.GetParentLabel(lbl);
            IProperty iProp = null;
            if (parentLabel != null)
            {
                iProp = parentLabel.Tag as IProperty;
            }
            else
            {

            }
            if (iProp != null)
            {

                if (iProp.GetPropertyType() == "1")
                {
                    SetSiteCompPropOveride(lbl, oldValue);
                }
                else
                {
                    SetGroupPropOverride(lbl, oldValue);
                }
            }
        }

        public static void SetOverRideValue(Control objControl, string oldValue)
        {
            IProperty iProp = null;
            Symcon.LightRidge.PEAdvisor.DashBoard.BindingInfo bindingInfo = objControl.Tag as Symcon.LightRidge.PEAdvisor.DashBoard.BindingInfo;
            if (bindingInfo.SourceObjectName == "LRRSiteGroupPropertyQualifierValueID")
            {
                LRRSiteGroupPropertyQualifierValue lsgpqv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(bindingInfo.SourceObjectID);
                LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lsgpqv.SiteComponentPropID);
                SetSiteCompPropOveride(lsgpqv, lscp, oldValue);
            }
            else
            {
                LRRSiteComponentPropertyValue lscpv = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(bindingInfo.SourceObjectID);
                LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                SetSiteCompPropOveride(lscpv, lscp, oldValue);
            }
        }

        private static void SetSiteCompPropOveride(LRRSiteComponentPropertyValue lscpv, LRRSiteComponentProperty lscp, string oldValue)
        {
            IProperty iProp = lscp as IProperty;
            IPropertyValue iPropValue = lscpv as IPropertyValue;
            string modeID = iPropValue.GetModeID();
            //string uomID = iProp.GetUOMType();
            string uomTypeID = iProp.GetUOMType();
            string uomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_app.AppLocaleID, uomTypeID);//iPropValue.GetUOMID();//
            string newUomID = "";
            string newpropValue = "";
            newpropValue = oldValue;

            if (iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
            {
                bool prevOverrideState = iPropValue.GetOverrideState();
                iPropValue.SetOverrideState(newpropValue);
                bool currOverrideState = iPropValue.GetOverrideState();

                //Measurement Mode
                if (prevOverrideState != currOverrideState && modeID == "1")
                {
                    iPropValue.SetTrustedState("-1");
                    _app.SwapBindings(iPropValue, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                //Other Modes
                if (prevOverrideState != currOverrideState && modeID != "1")
                {
                    iPropValue.SetTrustedState("-1");
                    _app.RemoveCascadedBindings(iPropValue, typeof(LRRSiteComponent), lscp.ID, lscp.ComponentID, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                iPropValue.SetPropertyValue(newpropValue);
                if (lscp != null)
                {
                    PopulateOverrides(iPropValue, lscp.ID, lscp.ComponentID, modeID, uomID);
                }
            }
        }

        private static void SetSiteCompPropOveride(LRRSiteGroupPropertyQualifierValue lsgpqv, LRRSiteComponentProperty lscp, string oldValue)
        {
            IProperty iProp = lscp as IProperty;
            IPropertyValue iPropValue = lsgpqv as IPropertyValue;
            string modeID = iPropValue.GetModeID();
            //string uomID = iProp.GetUOMType();
            string uomTypeID = iProp.GetUOMType();
            string uomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_app.AppLocaleID, uomTypeID);//iPropValue.GetUOMID();//
            string newUomID = "";
            string newpropValue = "";
            newpropValue = oldValue;

            if (iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
            {
                bool prevOverrideState = iPropValue.GetOverrideState();
                iPropValue.SetOverrideState(newpropValue);
                bool currOverrideState = iPropValue.GetOverrideState();

                //Measurement Mode
                if (prevOverrideState != currOverrideState && modeID == "1")
                {
                    iPropValue.SetTrustedState("-1");
                    _app.SwapBindings(iPropValue, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                //Other Modes
                if (prevOverrideState != currOverrideState && modeID != "1")
                {
                    iPropValue.SetTrustedState("-1");
                    _app.RemoveCascadedBindings(iPropValue, typeof(LRRSiteComponent), lscp.ID, lscp.ComponentID, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                iPropValue.SetPropertyValue(newpropValue);
                if (lscp != null)
                {
                    PopulateOverrides(iPropValue, lscp.ID, lscp.ComponentID, modeID, uomID);
                }
            }
        }

        private static void SetSiteCompPropOveride(LRRLabel lbl, string oldValue)
        {
            Symbol smbl = lbl.Parent as Symbol;
            LRRLabel hdrLbl = smbl.Labels[0] as LRRLabel;
            LRRSiteComponent lsc = hdrLbl.Tag as LRRSiteComponent;
            LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
            IPropertyValue iPropValue = lbl.Tag as IPropertyValue;
            IProperty iProp = parentLbl.Tag as IProperty;

            string modeID = iPropValue.GetModeID();

            //string uomID = iProp.GetUOMType();
            string uomTypeID = iProp.GetUOMType();
            string uomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_app.AppLocaleID, uomTypeID);//iPropValue.GetUOMID();//
            LRRLabel Uomlbl = GraphicsUtilities.GetUOMLabel(lbl);
            string newUomID = "";
            string newpropValue = "";
            if (Uomlbl != null && Uomlbl.Tag != null)
            {
                //newUomID	= LRRModel.ChildModel.GetUOMIDFromName(Uomlbl.Text);

                newUomID = Uomlbl.ID;
                newpropValue = GraphicsUtilities.ChangeUOM(newUomID, uomID, oldValue, iPropValue.GetDataType());
            }


            if (iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
            {
                bool prevOverrideState = iPropValue.GetOverrideState();
                iPropValue.SetOverrideState(newpropValue);
                bool currOverrideState = iPropValue.GetOverrideState();

                //Measurement Mode
                if (prevOverrideState != currOverrideState && modeID == "1")
                {
                    iPropValue.SetTrustedState("-1");
                    _app.SwapBindings(lbl.Tag, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                //Other Modes
                if (prevOverrideState != currOverrideState && modeID != "1")
                {
                    iPropValue.SetTrustedState("-1");
                    _app.RemoveCascadedBindings(lbl.Tag, typeof(LRRSiteComponent), parentLbl.ID, lsc.SiteComponentID, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                iPropValue.SetPropertyValue(newpropValue);
                if (parentLbl.Tag != null)
                {
                    PopulateOverrides(iPropValue, parentLbl.ID, lsc.SiteComponentID, modeID, uomID);
                }
            }
        }

        private static void SetGroupPropOverride(LRRLabel lbl, string oldValue)
        {
            LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
            LRRLabel uomlabel = GraphicsUtilities.GetUOMLabel(lbl);
            IProperty lrrScgp = parentLbl.Tag as IProperty;
            IPropertyValue iPropValue = lbl.Tag as IPropertyValue;
            LRRSiteGroupPropertyQualifierValue lrscgpVal = iPropValue as LRRSiteGroupPropertyQualifierValue;
            string groupPropID = lrscgpVal.GroupPropertyID;
            string qualifierID = lrscgpVal.QualifierID;
            string lookUpPropID = lrscgpVal.LookUpPropertiesID;


            LRRLookUpQualifier lkUpQualifier = LRRModel.LookUpQualifiers.GetLookUpQualifierForPropTypeAndQualifierID(qualifierID, lrscgpVal.PropertyID);
            string uomTypeID = null;
            string uomID = null;
            if (lrrScgp != null)
            {
                uomTypeID = lrrScgp.GetUOMType();
                LRRMeasurementSystemDefinitions lrrMSDs = (LRRMeasurementSystemDefinitions)_app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRMeasurementSystemDefinitions));
                uomID = iPropValue.GetUOMID();// LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,lrrScgp.GetUOMType());
            }

            //LRRUOM uomObj = uomlabel.Tag as LRRUOM;
            string newUOM = "";

            //if(uomObj!= null)
            newUOM = uomlabel.ID;
            string newpropValue = GraphicsUtilities.ChangeUOM(newUOM, uomID, oldValue, lrscgpVal.LRRDataType);

            if (iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
            {
                bool prevOverrideState = iPropValue.GetOverrideState();
                iPropValue.SetOverrideState(newpropValue);
                bool currOverrideState = iPropValue.GetOverrideState();

                //Measurement Mode
                if (prevOverrideState != currOverrideState && iPropValue.GetModeID() == "1")
                {
                    _app.SwapGroupPropBindings(lbl.Tag, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                //Other Modes
                if (prevOverrideState != currOverrideState && iPropValue.GetModeID() != "1")
                {
                    _app.RemoveCascadedBindings(lbl.Tag, groupPropID, lookUpPropID, qualifierID, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                iPropValue.SetPropertyValue(newpropValue);

                PopulateGroupOverrides(iPropValue, groupPropID, qualifierID);

            }
        }


        /// <summary>
        /// Populates Overrides Screen
        /// </summary>
        /// <param name="lrrSiteCompPropValue"></param>
        private static void PopulateOverrides(IPropertyValue iPropValue, string propertyID, string compID, string modeID, string sNewUOM)
        {
            LRROverride lrrModelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(propertyID, modeID);
            if (iPropValue.GetOverrideState())
            {
                if (lrrModelOverride != null)
                {
                    sNewUOM = lrrModelOverride.UOM;
                    LRRModel.ChildModel.Overrides.Remove(lrrModelOverride);
                }
                LRROverride lrrOverride = new LRROverride();
                lrrOverride.ModeID = modeID;
                lrrOverride.SiteComponentPropertyID = propertyID;
                lrrOverride.SiteComponentID = compID;

                //Pre/Post Override Values
                string modelUOM = iPropValue.GetUOMID();
                lrrOverride.PreOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualOverridedValue());
                lrrOverride.PostOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualPropertyValue());

                lrrOverride.UOM = sNewUOM;
                //Assign Confidence Level
                string dataConfidenceLevelID = null;
                if (modeID == "1" || modeID == "2")
                {
                    dataConfidenceLevelID = "3";
                }
                else
                {
                    dataConfidenceLevelID = "1";
                }
                lrrOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                LRRModel.ChildModel.Overrides.Add(lrrOverride);

                iPropValue.DataConfidenceValueChanged += new EventHandler(iPropValue_DataConfidenceValueChanged);

                iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
            }
            else
            {
                if (lrrModelOverride != null)
                {
                    iPropValue.DataConfidenceValueChanged -= new EventHandler(iPropValue_DataConfidenceValueChanged);
                    LRRModel.ChildModel.Overrides.Remove(lrrModelOverride);
                }
            }
            //Update Overrides
            _app.UpdateOverridePoints();
        }

        private static void PopulateGroupOverrides(IPropertyValue iPropValue, string groupPropID, string qualifierID)
        {
            LRRGroupOverride lrrModelOverride = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(iPropValue.GetID());
            if (iPropValue.GetOverrideState())
            {
                if (lrrModelOverride != null)
                {
                    LRRModel.ChildModel.GroupOverrides.Remove(lrrModelOverride);
                }
                LRRGroupOverride lrrGroupOverride = new LRRGroupOverride();

                string siteCompPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(groupPropID).SiteComponentPropID;
                string siteCompID = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID).ComponentID;

                string modeID = iPropValue.GetModeID();
                lrrGroupOverride.ModeID = modeID;
                lrrGroupOverride.SiteComponentPropertyID = siteCompPropID;
                lrrGroupOverride.SiteComponentID = siteCompID;

                //				lrrGroupOverride.CurrentValue = iPropValue.GetPropertyValue();
                //Pre/Post Override Values
                lrrGroupOverride.PreOverrideValue = iPropValue.GetActualOverridedValue();
                lrrGroupOverride.PostOverrideValue = iPropValue.GetActualPropertyValue();

                lrrGroupOverride.UOM = iPropValue.GetUOMID();
                lrrGroupOverride.GroupPropertyID = groupPropID;
                lrrGroupOverride.SiteGroupPropertyQualifierValueID = iPropValue.GetID();
                lrrGroupOverride.QualifierID = qualifierID;

                //Assign Confidence Level
                string dataConfidenceLevelID = null;
                if (modeID == "1" || modeID == "2")
                {
                    dataConfidenceLevelID = "3";
                }
                else
                {
                    dataConfidenceLevelID = "1";
                }
                lrrGroupOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                LRRModel.ChildModel.GroupOverrides.Add(lrrGroupOverride);

                iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
            }
            else
            {
                if (lrrModelOverride != null)
                {
                    LRRModel.ChildModel.GroupOverrides.Remove(lrrModelOverride);
                }
            }
        }

        private static void smblLbl_LabelDoubleClick(object sender, NodeMouseEventArgs evtArgs)
        {
            LRRLabel lbl = evtArgs.Node as LRRLabel;
            if (lbl != null)
            {
                try
                {
                    if (lbl.ID != null && lbl.IsUOM && lbl.Tag != null)
                    {
                        LRRUOMType lrrUOMType = lbl.Tag as LRRUOMType;
                        if (lrrUOMType != null)
                            GraphicsUtilities.ChangeUOMClick(lbl, lrrUOMType.UOMTypeID);
                    }
                    else
                    {
                        if (LabelDoubleClick != null)
                        {
                            LabelDoubleClick(sender, evtArgs);
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.LogMessage(typeof(ImplementerScreen), "gms_LabelDoubleClick :" + ex.Message, Logger.Level.Debug);
                }
            }
        }

        public static void OnOverrideValueClickForSiteComponent(MenuItem miItem, IPropertyValue iPropValue, LRRSiteComponentProperty lrrScp)
        {
            //			LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
            //			LRRSiteComponentProperty lrrScp = parentLbl.Tag as LRRSiteComponentProperty;
            if (lrrScp != null)
            {
                switch (miItem.Index)
                {
                    //Override Value On/Off
                    case 0:
                        iPropValue.SetOverrideState(false);
                        if (iPropValue.GetModeID() == "1")
                        {
                            _app.SwapBindings(iPropValue, false);
                            iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
                        }
                        else
                        {
                            _app.RemoveCascadedBindings(iPropValue, typeof(LRRSiteComponent), lrrScp.PropertyID, lrrScp.ComponentID, false);
                            iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
                        }
                        //Populate Overrides Collection
                        IProperty iProp = lrrScp as IProperty;
                        string propID = iProp.GetID();
                        string uomTypeID = iProp.GetUOMType();
                        string uomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_app.AppLocaleID, uomTypeID);//iPropValue.GetUOMID();//
                        PopulateOverrides(iPropValue, propID, lrrScp.ComponentID, iPropValue.GetModeID(), uomID);//rowData.UOMID);
                        break;
                    //Uncertain Input
                    case 2:
                        iPropValue.SetTrustedState("0");
                        break;
                    //Trusted Input
                    case 3:
                        iPropValue.SetTrustedState("1");
                        break;
                    //Temp Trusted Input
                    case 4:
                        iPropValue.SetTrustedState("2");
                        break;
                    //Uncertain Override
                    case 5://3:
                        iPropValue.SetDataConfidenceLevelID("3");
                        iPropValue.SetTrustedState("-1");
                        break;
                    //Trusted Override
                    case 6://4:
                        iPropValue.SetDataConfidenceLevelID("1");
                        iPropValue.SetTrustedState("-1");
                        break;
                    //TempTrusted Override
                    case 7://5:
                        iPropValue.SetDataConfidenceLevelID("2");
                        iPropValue.SetTrustedState("-1");
                        break;
                }
            }
        }

        public static void OnOverrideValueClickForGroupProp(MenuItem miItem, IPropertyValue iPropValue)
        {
            //LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
            //LRRSiteComponentGroupProperty lrrScgp = parentLbl.Tag as LRRSiteComponentGroupProperty;
            LRRSiteGroupPropertyQualifierValue lrscgpVal = iPropValue as LRRSiteGroupPropertyQualifierValue;
            string groupPropID = lrscgpVal.GroupPropertyID;
            string qualifierID = lrscgpVal.QualifierID;
            string lookUpPropID = lrscgpVal.LookUpPropertiesID;

            //Check for Menu item Index
            switch (miItem.Index)
            {
                //Override Value On/Off
                case 0:
                    iPropValue.SetOverrideState(false);
                    if (iPropValue.GetModeID() == "1")
                    {
                        _app.SwapGroupPropBindings(iPropValue, false);
                        iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
                    }
                    else
                    {
                        _app.RemoveCascadedBindings(iPropValue, groupPropID, lookUpPropID, qualifierID, false);
                        iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
                    }
                    PopulateGroupOverrides(iPropValue, groupPropID, qualifierID);
                    break;
                //Uncertain Input
                case 2:
                    iPropValue.SetTrustedState("0");
                    break;
                //Trusted Input
                case 3:
                    iPropValue.SetTrustedState("1");
                    break;
                //Temp Trusted Input
                case 4:
                    iPropValue.SetTrustedState("2");
                    break;
                //Uncertain Override
                case 5://3:
                    iPropValue.SetDataConfidenceLevelID("3");
                    iPropValue.SetTrustedState("-1");
                    break;
                //Trusted Override
                case 6://4:
                    iPropValue.SetDataConfidenceLevelID("1");
                    iPropValue.SetTrustedState("-1");
                    break;
                //TempTrusted Override
                case 7://5:
                    iPropValue.SetDataConfidenceLevelID("2");
                    iPropValue.SetTrustedState("-1");
                    break;
            }
        }


        private static void iPropValue_DataConfidenceValueChanged(object sender, EventArgs e)
        {
            GraphicsUtilities.OnDataConfidenceValueChanged(sender, e);
        }
    }
}
