using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;
using System.IO;
using System.Resources;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters;
using System.Data;
using System.Drawing.Printing;
using Symcon.Utils.MessageLogger;
using Symcon.Lightridge.LRRUI;
using Symcon.Lightridge.LRRUI.Controls;
using Symcon.LightRidge.BusinessObjects.TypeConverters;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.BusinessObjects;
using System.Text.RegularExpressions;

using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Tools.XPMenus;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.PEAdvisor.ScreenManagement;

//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor
{
	public class OperatorsScreen : System.Windows.Forms.Form,Symcon.BusinessObjects.ILRRUIContainer, Symcon.BusinessObjects.ILRRUISpecificContainer
	{

		#region Fields

		private System.ComponentModel.IContainer components;
		private Syncfusion.Windows.Forms.Tools.XPMenus.ChildFrameBarManager childFrameBarManager1;
		private Syncfusion.Windows.Forms.Tools.XPMenus.ParentBarItem parentBarItemFileOptions;
		private Syncfusion.Windows.Forms.Tools.XPMenus.BarItem bItemOpen;	
		private LabelController labelcontroller;
		private ResourceManager			resStringMgr;
		private FileDialog						fileDialog;
		private LRRApp _app;
		private LRRSavedViews lrrSavedViews;
		private LRRModes lrrModes;
		private Syncfusion.Windows.Forms.Diagram.SymbolPalette objectBrowserPalette;
		private TabControl _parentTabControl;
		private TabPage				_parentTabPage;
        private GraphicsManagers gms = null;
		private bool _isFormClosing = false;
		private ToolTip _toolTip = new ToolTip();
		private EventHandler idleEventHandler;
		private delegate void OpenFile(string fileID);
		OpenFile _openFile;
		private DockingManager dockingManager1;
		private DockingClientPanel dcPanel;
		private Panel pnlCanvasHeader;
		private Syncfusion.Windows.Forms.ButtonAdv btnMoveLeft;
		private Button btnClose;
		private Syncfusion.Windows.Forms.ButtonAdv btnMoveRight;
		private TabControlAdv tabControlAdv1;
		private ImageList smallImageList;
		private Panel pnlQueryView;
		private Syncfusion.Windows.Forms.Diagram.Controls.OverviewControl overViewNavigation;
		private BarItem biPrintZooming;
		private ParentBarItem parentBarItemSession;
		private BarItem bItemPropertyDetails;
		private BarItem bItemClose;
		private ImageList imageList1;
		private BarItem barItemNavigation;
		private ParentBarItem parentBarItem1;
		private ImageList CanvasToolbarImages;
		private ToolStripButton toolStripButton1;
		private Splitter splitter1;
		private ToolStripPanel toolStripPanel1;
		private ToolStrip toolStrip2;
		private ToolStripButton toolStripBtnOpen;
		private ToolStrip toolStrip1;
		private ToolStripButton toolStripBtnSelect;
		private ToolStripButton toolStripBtnPan;
		private ToolStripButton toolStripBtnShowGrid;
		private ToolStripButton toolStripBtnSnapToGrid;
		private ToolStripButton toolStripBtnZoom;
		private ToolStripComboBox toolStripCmbMagnify;
		private ToolStripPanel BottomToolStripPanel;
		private ToolStripPanel TopToolStripPanel;
		private ToolStripPanel RightToolStripPanel;
		private ToolStripPanel LeftToolStripPanel;
		private ToolStripContentPanel ContentPanel;
		private bool _implementerPanAndZoom = false;
		private ArrayList magnifyValues;
		private Syncfusion.Windows.Forms.Tools.XPMenus.Bar barToolBar;
		#endregion

		#region Constructor
		
		public OperatorsScreen()
		{
            // hookup events called by ashwin
            gms = new GraphicsManagers();
            //gms.HookUpEvents();
			this.objectBrowserPalette = new SymbolPalette();
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			lrrModes = LRRModel.ProjectModes;
			InitializeComponent(); 		
			//sortMagnifyValues();
			sortCmbValues(this.toolStripCmbMagnify);
			InternationalizationUtils.GetControlText(this,this.components.Components);
			_app.MainFrame.MainFrameBarManager.ChildManagers.Add(this.childFrameBarManager1);
			labelcontroller = new LabelController();
			this.resStringMgr = new ResourceManager("Symcon.LightRidge.PEAdvisor.ImplementerGraphics.Strings", typeof(OperatorsScreen).Assembly);
			this.tabControlAdv1.SelectedIndexChanged+=new EventHandler(this.tabControlAdv1_SelectedIndexChanged);
			this.Closing+=new CancelEventHandler(this.OperatorsScreen_Closing);
			this.Closed+=new EventHandler(OperatorsScreen_Closed);
			this.Activated+=new EventHandler(OperatorsScreen_Activated);
			this.Deactivate+=new EventHandler(OperatorsScreen_Deactivate); 
			lrrSavedViews = LRRModel.SavedViews;
			pnlCanvasHeader.Hide();
			tabControlAdv1.ThemesEnabled = false;
			gms.ApplyModes();
			WireGmsEvents(gms);
			idleEventHandler = new EventHandler(OnIdle);
			Application.Idle+= idleEventHandler;
			_toolTip.InitialDelay = 500;
			_toolTip.ReshowDelay = 500;
			_toolTip.AutoPopDelay = 5000;
			_openFile = new OpenFile(OnOpenFile);
            this.tabControlAdv1.ShowScroll = true;
		}


		#endregion

		#region Overrides
		
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					UnWireGmsEvents(gms);
                    Application.Idle -= idleEventHandler;
					components.Dispose();
                    gms.Dispose();
                    gms = null;
					//gms = null;
				}
			}
			base.Dispose( disposing );
		}
		
		#endregion

		#region Windows Form Designer generated code
	
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(OperatorsScreen));
			this.CanvasToolbarImages = new System.Windows.Forms.ImageList(this.components);
			this.smallImageList = new System.Windows.Forms.ImageList(this.components);
			this.dockingManager1 = new Syncfusion.Windows.Forms.Tools.DockingManager(this.components);
			this.childFrameBarManager1 = new Syncfusion.Windows.Forms.Tools.XPMenus.ChildFrameBarManager(this.components, this);
			this.overViewNavigation = new Syncfusion.Windows.Forms.Diagram.Controls.OverviewControl();
			this.parentBarItemFileOptions = new Syncfusion.Windows.Forms.Tools.XPMenus.ParentBarItem();
			this.barToolBar = new Syncfusion.Windows.Forms.Tools.XPMenus.Bar(this.childFrameBarManager1, "SID_TOOLBARS");
			this.pnlQueryView = new System.Windows.Forms.Panel();
			this.bItemOpen = new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem();
			this.biPrintZooming = new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem();
			this.pnlCanvasHeader = new System.Windows.Forms.Panel();
			this.btnMoveLeft = new Syncfusion.Windows.Forms.ButtonAdv();
			this.btnMoveRight = new Syncfusion.Windows.Forms.ButtonAdv();
			this.btnClose = new System.Windows.Forms.Button();
			this.parentBarItemSession = new Syncfusion.Windows.Forms.Tools.XPMenus.ParentBarItem();
			this.bItemPropertyDetails = new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem();
			this.bItemClose = new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem();
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			this.tabControlAdv1 = new Syncfusion.Windows.Forms.Tools.TabControlAdv();
			this.dcPanel = new Syncfusion.Windows.Forms.Tools.DockingClientPanel();
			this.barItemNavigation = new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem();
			this.parentBarItem1 = new Syncfusion.Windows.Forms.Tools.XPMenus.ParentBarItem();
			this.BottomToolStripPanel = new System.Windows.Forms.ToolStripPanel();
			this.TopToolStripPanel = new System.Windows.Forms.ToolStripPanel();
			this.RightToolStripPanel = new System.Windows.Forms.ToolStripPanel();
			this.LeftToolStripPanel = new System.Windows.Forms.ToolStripPanel();
			this.ContentPanel = new System.Windows.Forms.ToolStripContentPanel();
			this.toolStripPanel1 = new System.Windows.Forms.ToolStripPanel();
			this.toolStrip2 = new System.Windows.Forms.ToolStrip();
			this.toolStripBtnOpen = new System.Windows.Forms.ToolStripButton();
			this.toolStrip1 = new System.Windows.Forms.ToolStrip();
			this.toolStripBtnSelect = new System.Windows.Forms.ToolStripButton();
			this.toolStripBtnPan = new System.Windows.Forms.ToolStripButton();
			this.toolStripBtnShowGrid = new System.Windows.Forms.ToolStripButton();
			this.toolStripBtnSnapToGrid = new System.Windows.Forms.ToolStripButton();
			this.toolStripBtnZoom = new System.Windows.Forms.ToolStripButton();
			this.toolStripCmbMagnify = new System.Windows.Forms.ToolStripComboBox();
			this.splitter1 = new System.Windows.Forms.Splitter();
			((System.ComponentModel.ISupportInitialize)(this.dockingManager1)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.childFrameBarManager1)).BeginInit();
			this.pnlCanvasHeader.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.tabControlAdv1)).BeginInit();
			this.dcPanel.SuspendLayout();
			this.toolStripPanel1.SuspendLayout();
			this.toolStrip2.SuspendLayout();
			this.toolStrip1.SuspendLayout();
			this.SuspendLayout();
			// 
			// CanvasToolbarImages
			// 
			this.CanvasToolbarImages.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("CanvasToolbarImages.ImageStream")));
			this.CanvasToolbarImages.TransparentColor = System.Drawing.Color.Fuchsia;
			this.CanvasToolbarImages.Images.SetKeyName(0, "");
			this.CanvasToolbarImages.Images.SetKeyName(1, "");
			this.CanvasToolbarImages.Images.SetKeyName(2, "");
			this.CanvasToolbarImages.Images.SetKeyName(3, "");
			this.CanvasToolbarImages.Images.SetKeyName(4, "");
			this.CanvasToolbarImages.Images.SetKeyName(5, "");
			this.CanvasToolbarImages.Images.SetKeyName(6, "");
			this.CanvasToolbarImages.Images.SetKeyName(7, "");
			this.CanvasToolbarImages.Images.SetKeyName(8, "");
			this.CanvasToolbarImages.Images.SetKeyName(9, "");
			this.CanvasToolbarImages.Images.SetKeyName(10, "");
			this.CanvasToolbarImages.Images.SetKeyName(11, "");
			this.CanvasToolbarImages.Images.SetKeyName(12, "");
			this.CanvasToolbarImages.Images.SetKeyName(13, "");
			this.CanvasToolbarImages.Images.SetKeyName(14, "");
			this.CanvasToolbarImages.Images.SetKeyName(15, "");
			this.CanvasToolbarImages.Images.SetKeyName(16, "");
			this.CanvasToolbarImages.Images.SetKeyName(17, "");
			this.CanvasToolbarImages.Images.SetKeyName(18, "");
			this.CanvasToolbarImages.Images.SetKeyName(19, "");
			this.CanvasToolbarImages.Images.SetKeyName(20, "");
			this.CanvasToolbarImages.Images.SetKeyName(21, "");
			this.CanvasToolbarImages.Images.SetKeyName(22, "");
			this.CanvasToolbarImages.Images.SetKeyName(23, "");
			this.CanvasToolbarImages.Images.SetKeyName(24, "");
			this.CanvasToolbarImages.Images.SetKeyName(25, "");
			this.CanvasToolbarImages.Images.SetKeyName(26, "");
			this.CanvasToolbarImages.Images.SetKeyName(27, "");
			this.CanvasToolbarImages.Images.SetKeyName(28, "");
			this.CanvasToolbarImages.Images.SetKeyName(29, "");
			this.CanvasToolbarImages.Images.SetKeyName(30, "");
			this.CanvasToolbarImages.Images.SetKeyName(31, "");
			this.CanvasToolbarImages.Images.SetKeyName(32, "");
			this.CanvasToolbarImages.Images.SetKeyName(33, "");
			this.CanvasToolbarImages.Images.SetKeyName(34, "");
			this.CanvasToolbarImages.Images.SetKeyName(35, "");
			this.CanvasToolbarImages.Images.SetKeyName(36, "");
			this.CanvasToolbarImages.Images.SetKeyName(37, "");
			this.CanvasToolbarImages.Images.SetKeyName(38, "");
			this.CanvasToolbarImages.Images.SetKeyName(39, "");
			this.CanvasToolbarImages.Images.SetKeyName(40, "");
			this.CanvasToolbarImages.Images.SetKeyName(41, "");
			this.CanvasToolbarImages.Images.SetKeyName(42, "");
			this.CanvasToolbarImages.Images.SetKeyName(43, "");
			this.CanvasToolbarImages.Images.SetKeyName(44, "");
			this.CanvasToolbarImages.Images.SetKeyName(45, "");
			this.CanvasToolbarImages.Images.SetKeyName(46, "");
			this.CanvasToolbarImages.Images.SetKeyName(47, "");
			this.CanvasToolbarImages.Images.SetKeyName(48, "");
			this.CanvasToolbarImages.Images.SetKeyName(49, "");
			this.CanvasToolbarImages.Images.SetKeyName(50, "");
			this.CanvasToolbarImages.Images.SetKeyName(51, "");
			this.CanvasToolbarImages.Images.SetKeyName(52, "");
			this.CanvasToolbarImages.Images.SetKeyName(53, "");
			this.CanvasToolbarImages.Images.SetKeyName(54, "");
			this.CanvasToolbarImages.Images.SetKeyName(55, "");
			this.CanvasToolbarImages.Images.SetKeyName(56, "");
			this.CanvasToolbarImages.Images.SetKeyName(57, "");
			this.CanvasToolbarImages.Images.SetKeyName(58, "");
			this.CanvasToolbarImages.Images.SetKeyName(59, "");
			// 
			// smallImageList
			// 
			this.smallImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("smallImageList.ImageStream")));
			this.smallImageList.TransparentColor = System.Drawing.Color.Fuchsia;
			this.smallImageList.Images.SetKeyName(0, "");
			this.smallImageList.Images.SetKeyName(1, "");
			this.smallImageList.Images.SetKeyName(2, "");
			this.smallImageList.Images.SetKeyName(3, "");
			this.smallImageList.Images.SetKeyName(4, "");
			this.smallImageList.Images.SetKeyName(5, "");
			this.smallImageList.Images.SetKeyName(6, "");
			this.smallImageList.Images.SetKeyName(7, "");
			this.smallImageList.Images.SetKeyName(8, "");
			this.smallImageList.Images.SetKeyName(9, "");
			this.smallImageList.Images.SetKeyName(10, "");
			this.smallImageList.Images.SetKeyName(11, "");
			// 
			// barToolBar
			// 
			this.barToolBar.BarName = "SID_TOOLBARS";
			this.barToolBar.BarStyle = ((Syncfusion.Windows.Forms.Tools.XPMenus.BarStyle)((Syncfusion.Windows.Forms.Tools.XPMenus.BarStyle.IsMainMenu | Syncfusion.Windows.Forms.Tools.XPMenus.BarStyle.Visible)));
			this.barToolBar.Caption = "";
			this.barToolBar.Items.AddRange(new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem[] {
																									this.parentBarItem1,
																									this.parentBarItemSession,
																									this.parentBarItemFileOptions});
			this.barToolBar.Manager = this.childFrameBarManager1;
			// 
			// dockingManager1
			// 
			this.dockingManager1.AutoHideInterval = 200;
			this.dockingManager1.DockLayoutStream = ((System.IO.MemoryStream)(resources.GetObject("dockingManager1.DockLayoutStream")));
			this.dockingManager1.DockTabFont = new System.Drawing.Font("Tahoma", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World);
			this.dockingManager1.HostControl = this;
			this.dockingManager1.PaintBorders = false;
			this.dockingManager1.PersistState = true;
			this.dockingManager1.ShowToolTips = true;
			this.dockingManager1.SplitterWidth = 1;
			this.dockingManager1.ThemesEnabled = true;
			this.dockingManager1.SetDockLabel(this.overViewNavigation, "Pan And Zoom");
			this.dockingManager1.SetDockLabel(this.pnlQueryView, "Query View");
			// 
			// overViewNavigation
			// 
			this.overViewNavigation.BackColor = System.Drawing.SystemColors.AppWorkspace;
			this.dockingManager1.SetEnableDocking(this.overViewNavigation, true);
			this.overViewNavigation.ForeColor = System.Drawing.Color.Red;
			this.overViewNavigation.Location = new System.Drawing.Point(1, 1);
			this.overViewNavigation.Name = "overViewNavigation";
			this.overViewNavigation.Size = new System.Drawing.Size(175, 100);
			this.overViewNavigation.TabIndex = 1;
			// 

			// 
			// parentBarItemFileOptions
			// 
			this.parentBarItemFileOptions.CategoryIndex = 0;
			this.parentBarItemFileOptions.ID = "Graphics";
			this.parentBarItemFileOptions.Items.AddRange(new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem[] {
																												  this.bItemOpen,
																												this.bItemClose});
			this.parentBarItemFileOptions.Tag = "SID_GRPAHICS";
			// 
			// childFrameBarManager1
			// 
			this.childFrameBarManager1.BarPositionInfo = ((System.IO.MemoryStream)(resources.GetObject("childFrameBarManager1.BarPositionInfo")));
			//this.childFrameBarManager1.Bars.Add(this.barOptions);
			//this.childFrameBarManager1.Bars.Add(this.barFileOptions);
			this.childFrameBarManager1.Bars.Add(this.barToolBar);
			this.childFrameBarManager1.Categories.Add("File Menu");
			this.childFrameBarManager1.Categories.Add("View");
			this.childFrameBarManager1.Categories.Add("ToolBar");
			this.childFrameBarManager1.CategoriesToIgnoreInCustDialog.AddRange(new int[] {
																							 8});
			this.childFrameBarManager1.CurrentBaseFormType = "";
			this.childFrameBarManager1.DisabledImageList = null;
			this.childFrameBarManager1.DisabledLargeImageList = null;
			this.childFrameBarManager1.Form = this;
			this.childFrameBarManager1.ImageList = null;
			this.childFrameBarManager1.Items.AddRange(new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem[] {
																											   this.parentBarItemFileOptions,
																											   this.bItemOpen,
																											   this.parentBarItem1,
																											   this.biPrintZooming,
																											   this.barItemNavigation, 
																											   this.bItemClose,	 
																											   this.bItemPropertyDetails,
																											   this.parentBarItemSession});
			this.childFrameBarManager1.LargeImageList = null;
			this.childFrameBarManager1.Style = Syncfusion.Windows.Forms.VisualStyle.Office2003;

			// 
			// bItemClose
			// 
			this.bItemClose.CategoryIndex = 0;
			this.bItemClose.ID = "Close";
			this.bItemClose.Text = "";
			this.bItemClose.Tag = "SID_CLOSE";
			this.bItemClose.Click += new EventHandler(bItemClose_Click);
			// 
			// bItemOpen
			// 
			this.bItemOpen.CategoryIndex = 0;
			this.bItemOpen.ID = "&Open";
			this.bItemOpen.ImageIndex = 1;
			this.bItemOpen.ImageList = this.smallImageList;
			this.bItemOpen.Tag = "SID_OPEN";
			this.bItemOpen.Click += new System.EventHandler(this.bItemOpen_Click);

			// 
			// pnlQueryView
			// 
			this.pnlQueryView.AutoScroll = true;
			this.pnlQueryView.BackColor = System.Drawing.SystemColors.ControlLightLight;
			this.dockingManager1.SetEnableDocking(this.pnlQueryView, true);
			this.pnlQueryView.Location = new System.Drawing.Point(1, 1);
			this.pnlQueryView.Name = "Query View";
			this.pnlQueryView.Size = new System.Drawing.Size(412, 168);
			this.pnlQueryView.TabIndex = 101;

			// 
			// biPrintZooming
			// 
			this.biPrintZooming.CategoryIndex = 0;
			this.biPrintZooming.ID = "Print Zooming";
			this.biPrintZooming.ImageList = this.smallImageList;
			this.biPrintZooming.Tag = "SID_PRIENT_ZOOMING";
			// 
			// pnlCanvasHeader
			// 
			this.pnlCanvasHeader.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.pnlCanvasHeader.BackColor = System.Drawing.Color.Transparent;
			this.pnlCanvasHeader.Controls.Add(this.btnMoveLeft);
			this.pnlCanvasHeader.Controls.Add(this.btnMoveRight);
			this.pnlCanvasHeader.Controls.Add(this.btnClose);
			this.pnlCanvasHeader.ForeColor = System.Drawing.Color.Transparent;
			this.pnlCanvasHeader.Location = new System.Drawing.Point(456, 0);
			this.pnlCanvasHeader.Name = "pnlCanvasHeader";
			this.pnlCanvasHeader.Size = new System.Drawing.Size(48, 24);
			this.pnlCanvasHeader.TabIndex = 16;
			// 
			// btnMoveLeft
			// 
			this.btnMoveLeft.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnMoveLeft.Appearance = Syncfusion.Windows.Forms.ButtonAppearance.Office2003;
			this.btnMoveLeft.BackColor = System.Drawing.Color.Transparent;
			this.btnMoveLeft.ButtonType = Syncfusion.Windows.Forms.Tools.ButtonTypes.Left;
			this.btnMoveLeft.ComboEditBackColor = System.Drawing.Color.Empty;
			this.btnMoveLeft.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.btnMoveLeft.ForeColor = System.Drawing.Color.Transparent;
			this.btnMoveLeft.IsMouseDown = false;
			this.btnMoveLeft.Location = new System.Drawing.Point(0, 0);
			this.btnMoveLeft.Name = "btnMoveLeft";
			this.btnMoveLeft.Size = new System.Drawing.Size(16, 24);
			this.btnMoveLeft.TabIndex = 18;
			this.btnMoveLeft.UseVisualStyle = true;
			this.btnMoveLeft.UseVisualStyleBackColor = false;
			this.btnMoveLeft.Click += new System.EventHandler(this.btnMoveLeft_Click);
			// 
			// btnMoveRight
			// 
			this.btnMoveRight.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnMoveRight.Appearance = Syncfusion.Windows.Forms.ButtonAppearance.Office2003;
			this.btnMoveRight.BackColor = System.Drawing.Color.Transparent;
			this.btnMoveRight.ButtonType = Syncfusion.Windows.Forms.Tools.ButtonTypes.Right;
			this.btnMoveRight.ComboEditBackColor = System.Drawing.Color.Empty;
			this.btnMoveRight.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.btnMoveRight.ForeColor = System.Drawing.Color.Transparent;
			this.btnMoveRight.IsMouseDown = false;
			this.btnMoveRight.Location = new System.Drawing.Point(16, 0);
			this.btnMoveRight.Name = "btnMoveRight";
			this.btnMoveRight.Size = new System.Drawing.Size(16, 24);
			this.btnMoveRight.TabIndex = 17;
			this.btnMoveRight.UseVisualStyle = true;
			this.btnMoveRight.UseVisualStyleBackColor = false;
			this.btnMoveRight.Click += new System.EventHandler(this.btnMoveRight_Click);
			// 
			// btnClose
			// 
			this.btnClose.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnClose.BackColor = System.Drawing.Color.Transparent;
			this.btnClose.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.btnClose.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.btnClose.ForeColor = System.Drawing.Color.Black;
			this.btnClose.Location = new System.Drawing.Point(32, 0);
			this.btnClose.Name = "btnClose";
			this.btnClose.Size = new System.Drawing.Size(16, 24);
			this.btnClose.TabIndex = 10;
			this.btnClose.TabStop = false;
			this.btnClose.Text = "X";
			this.btnClose.UseVisualStyleBackColor = false;
			this.btnClose.Click += new System.EventHandler(this.bItemClose_Click);
			// 
			// parentBarItemSession
			// 
			this.parentBarItemSession.CategoryIndex = 0;
			this.parentBarItemSession.ID = "&Session";
			this.parentBarItemSession.Items.AddRange(new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem[] {
            this.biPrintZooming});
			this.parentBarItemSession.Tag = "1";
			// 
			// bItemPropertyDetails
			// 
			this.bItemPropertyDetails.CategoryIndex = 2;
			this.bItemPropertyDetails.ID = "Property Details";
			this.bItemPropertyDetails.Tag = "SID_ATT_DETL_WND";
			this.bItemPropertyDetails.Click += new System.EventHandler(this.bItemPropertyDetails_Click);
			// 
			// bItemClose
			// 
			this.bItemClose.CategoryIndex = 0;
			this.bItemClose.ID = "Close";
			this.bItemClose.Tag = "SID_CLOSE";
			// 
			// imageList1
			// 
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Fuchsia;
			this.imageList1.Images.SetKeyName(0, "");
			this.imageList1.Images.SetKeyName(1, "");
			this.imageList1.Images.SetKeyName(2, "");
			this.imageList1.Images.SetKeyName(3, "");
			this.imageList1.Images.SetKeyName(4, "");
			this.imageList1.Images.SetKeyName(5, "");
			this.imageList1.Images.SetKeyName(6, "");
			this.imageList1.Images.SetKeyName(7, "");
			this.imageList1.Images.SetKeyName(8, "");
			this.imageList1.Images.SetKeyName(9, "");
			this.imageList1.Images.SetKeyName(10, "");
			this.imageList1.Images.SetKeyName(11, "");
			this.imageList1.Images.SetKeyName(12, "");
			this.imageList1.Images.SetKeyName(13, "");
			this.imageList1.Images.SetKeyName(14, "");
			this.imageList1.Images.SetKeyName(15, "");
			this.imageList1.Images.SetKeyName(16, "");
			this.imageList1.Images.SetKeyName(17, "");
			this.imageList1.Images.SetKeyName(18, "");
			this.imageList1.Images.SetKeyName(19, "");
			this.imageList1.Images.SetKeyName(20, "");
			this.imageList1.Images.SetKeyName(21, "");
			this.imageList1.Images.SetKeyName(22, "");
			this.imageList1.Images.SetKeyName(23, "");
			this.imageList1.Images.SetKeyName(24, "");
			this.imageList1.Images.SetKeyName(25, "");
			this.imageList1.Images.SetKeyName(26, "");
			this.imageList1.Images.SetKeyName(27, "");
			this.imageList1.Images.SetKeyName(28, "");
			this.imageList1.Images.SetKeyName(29, "");
			this.imageList1.Images.SetKeyName(30, "");
			this.imageList1.Images.SetKeyName(31, "");
			// 
			// tabControlAdv1
			// 
			this.tabControlAdv1.ActiveTabColor = System.Drawing.Color.LightGoldenrodYellow;
			this.tabControlAdv1.ActiveTabFont = new System.Drawing.Font("Tahoma", 8F, System.Drawing.FontStyle.Bold);
			this.tabControlAdv1.BackColor = System.Drawing.SystemColors.AppWorkspace;
			this.tabControlAdv1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.tabControlAdv1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.tabControlAdv1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Bold);
			this.tabControlAdv1.ImageList = this.smallImageList;
			this.tabControlAdv1.Location = new System.Drawing.Point(0, 0);
			this.tabControlAdv1.Name = "tabControlAdv1";
			this.tabControlAdv1.Padding = new System.Drawing.Point(4, 1);
			this.tabControlAdv1.Size = new System.Drawing.Size(504, 480);
			this.tabControlAdv1.TabIndex = 10;
			this.tabControlAdv1.TabPanelBackColor = System.Drawing.Color.White;
			this.tabControlAdv1.ThemesEnabled = true;
			this.tabControlAdv1.UserMoveTabs = true;
			this.tabControlAdv1.ControlAdded += new System.Windows.Forms.ControlEventHandler(this.tabControlAdv1_ControlAdded);
			this.tabControlAdv1.SelectedIndexChanged += new System.EventHandler(this.tabControlAdv1_SelectedIndexChanged);
			// 
			// dcPanel
			// 
			this.dcPanel.BackColor = System.Drawing.Color.Transparent;
			this.dcPanel.Controls.Add(this.pnlCanvasHeader);
			this.dcPanel.Controls.Add(this.tabControlAdv1);
			this.dcPanel.Location = new System.Drawing.Point(197, 62);
			this.dcPanel.Name = "dcPanel";
			this.dcPanel.Size = new System.Drawing.Size(504, 480);
			this.dcPanel.TabIndex = 103;
			// 
			// barItemNavigation
			// 
			this.barItemNavigation.CategoryIndex = 2;
			this.barItemNavigation.ID = "Na&vigation Pane";
			this.barItemNavigation.Shortcut = System.Windows.Forms.Shortcut.CtrlShiftN;
			this.barItemNavigation.Tag = "SID_PAN_ZOOM_WND";
			this.barItemNavigation.Click += new System.EventHandler(this.barItem1_Click);
			// 
			// parentBarItem1
			// 
			this.parentBarItem1.CategoryIndex = 2;
			this.parentBarItem1.ID = "&View";
			this.parentBarItem1.Items.AddRange(new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem[] {
            this.barItemNavigation,
            this.bItemPropertyDetails});
			this.parentBarItem1.Tag = "10";
			// 
			// BottomToolStripPanel
			// 
			this.BottomToolStripPanel.Location = new System.Drawing.Point(0, 0);
			this.BottomToolStripPanel.Name = "BottomToolStripPanel";
			this.BottomToolStripPanel.Orientation = System.Windows.Forms.Orientation.Horizontal;
			this.BottomToolStripPanel.RowMargin = new System.Windows.Forms.Padding(3, 0, 0, 0);
			this.BottomToolStripPanel.Size = new System.Drawing.Size(0, 0);
			// 
			// TopToolStripPanel
			// 
			this.TopToolStripPanel.BackColor = System.Drawing.SystemColors.InactiveCaptionText;
			this.TopToolStripPanel.Location = new System.Drawing.Point(0, 0);
			this.TopToolStripPanel.Name = "TopToolStripPanel";
			this.TopToolStripPanel.Orientation = System.Windows.Forms.Orientation.Horizontal;
			this.TopToolStripPanel.RowMargin = new System.Windows.Forms.Padding(3, 0, 0, 0);
			this.TopToolStripPanel.Size = new System.Drawing.Size(0, 0);
			// 
			// RightToolStripPanel
			// 
			this.RightToolStripPanel.Location = new System.Drawing.Point(0, 0);
			this.RightToolStripPanel.Name = "RightToolStripPanel";
			this.RightToolStripPanel.Orientation = System.Windows.Forms.Orientation.Horizontal;
			this.RightToolStripPanel.RowMargin = new System.Windows.Forms.Padding(3, 0, 0, 0);
			this.RightToolStripPanel.Size = new System.Drawing.Size(0, 0);
			// 
			// LeftToolStripPanel
			// 
			this.LeftToolStripPanel.Location = new System.Drawing.Point(0, 0);
			this.LeftToolStripPanel.Name = "LeftToolStripPanel";
			this.LeftToolStripPanel.Orientation = System.Windows.Forms.Orientation.Horizontal;
			this.LeftToolStripPanel.RowMargin = new System.Windows.Forms.Padding(3, 0, 0, 0);
			this.LeftToolStripPanel.Size = new System.Drawing.Size(0, 0);
			// 
			// ContentPanel
			// 
			this.ContentPanel.Size = new System.Drawing.Size(800, 4);
			// 
			// toolStripPanel1
			// 
			this.toolStripPanel1.BackColor = System.Drawing.SystemColors.InactiveCaptionText;
			this.toolStripPanel1.Controls.Add(this.toolStrip2);
			this.toolStripPanel1.Controls.Add(this.toolStrip1);
			this.toolStripPanel1.Dock = System.Windows.Forms.DockStyle.Top;
			this.toolStripPanel1.Location = new System.Drawing.Point(0, 0);
			this.toolStripPanel1.Name = "toolStripPanel1";
			this.toolStripPanel1.Orientation = System.Windows.Forms.Orientation.Horizontal;
			this.toolStripPanel1.RowMargin = new System.Windows.Forms.Padding(3, 0, 0, 0);
			this.toolStripPanel1.Size = new System.Drawing.Size(800, 25);
			// 
			// toolStrip2
			// 
			this.toolStrip2.AccessibleRole = System.Windows.Forms.AccessibleRole.MenuBar;
			this.toolStrip2.AllowItemReorder = true;
			this.toolStrip2.BackColor = System.Drawing.SystemColors.InactiveCaptionText;
			this.toolStrip2.Dock = System.Windows.Forms.DockStyle.None;
			this.toolStrip2.ImageList = this.smallImageList;
			this.toolStrip2.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.toolStripBtnOpen});
			this.toolStrip2.LayoutStyle = System.Windows.Forms.ToolStripLayoutStyle.HorizontalStackWithOverflow;
			this.toolStrip2.Location = new System.Drawing.Point(25, 0);
			this.toolStrip2.Name = "toolStrip2";
			this.toolStrip2.RenderMode = System.Windows.Forms.ToolStripRenderMode.Professional;
			this.toolStrip2.Size = new System.Drawing.Size(35, 25);
			this.toolStrip2.TabIndex = 1;
			// 
			// toolStripBtnOpen
			// 
			this.toolStripBtnOpen.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
			this.toolStripBtnOpen.ImageIndex = 1;
			this.toolStripBtnOpen.Name = "toolStripBtnOpen";
			this.toolStripBtnOpen.Size = new System.Drawing.Size(23, 22);
			this.toolStripBtnOpen.Text = "toolStripBtnOpen";
			this.toolStripBtnOpen.Click += new System.EventHandler(this.toolStripBtnOpen_Click);
			// 
			// toolStrip1
			// 
			this.toolStrip1.AccessibleRole = System.Windows.Forms.AccessibleRole.MenuBar;
			this.toolStrip1.AllowItemReorder = true;
			this.toolStrip1.BackColor = System.Drawing.SystemColors.InactiveCaptionText;
			this.toolStrip1.Dock = System.Windows.Forms.DockStyle.None;
			this.toolStrip1.ImageList = this.CanvasToolbarImages;
			this.toolStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.toolStripBtnSelect,
            this.toolStripBtnPan,
            this.toolStripBtnShowGrid,
            this.toolStripBtnSnapToGrid,
            this.toolStripBtnZoom,
            this.toolStripCmbMagnify});
			this.toolStrip1.Location = new System.Drawing.Point(101, 0);
			this.toolStrip1.Margin = new System.Windows.Forms.Padding(2);
			this.toolStrip1.Name = "toolStrip1";
			this.toolStrip1.RenderMode = System.Windows.Forms.ToolStripRenderMode.Professional;
			this.toolStrip1.Size = new System.Drawing.Size(250, 25);
			this.toolStrip1.TabIndex = 0;
			// 
			// toolStripBtnSelect
			// 
			this.toolStripBtnSelect.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
			this.toolStripBtnSelect.ImageIndex = 0;
			this.toolStripBtnSelect.Name = "toolStripBtnSelect";
			this.toolStripBtnSelect.Size = new System.Drawing.Size(23, 22);
			this.toolStripBtnSelect.Tag = "SelectTool";
			this.toolStripBtnSelect.Text = "SelectTool";
			this.toolStripBtnSelect.ToolTipText = "Select";
			this.toolStripBtnSelect.Click += new System.EventHandler(this.ToolActivate);
			// 
			// toolStripBtnPan
			// 
			this.toolStripBtnPan.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
			this.toolStripBtnPan.ImageIndex = 16;
			this.toolStripBtnPan.Name = "toolStripBtnPan";
			this.toolStripBtnPan.Size = new System.Drawing.Size(23, 22);
			this.toolStripBtnPan.Tag = "PanTool";
			this.toolStripBtnPan.Text = "PanTool";
			this.toolStripBtnPan.ToolTipText = "Pan";
			this.toolStripBtnPan.Click += new System.EventHandler(this.ToolActivate);
			// 
			// toolStripBtnShowGrid
			// 
			this.toolStripBtnShowGrid.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
			this.toolStripBtnShowGrid.ImageIndex = 18;
			this.toolStripBtnShowGrid.Name = "toolStripBtnShowGrid";
			this.toolStripBtnShowGrid.Size = new System.Drawing.Size(23, 22);
			this.toolStripBtnShowGrid.Tag = "Show Grid";
			this.toolStripBtnShowGrid.Text = "Show Grid";
			this.toolStripBtnShowGrid.Click += new System.EventHandler(this.toolStripBtnShowGrid_Click);
			// 
			// toolStripBtnSnapToGrid
			// 
			this.toolStripBtnSnapToGrid.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
			this.toolStripBtnSnapToGrid.ImageIndex = 19;
			this.toolStripBtnSnapToGrid.Name = "toolStripBtnSnapToGrid";
			this.toolStripBtnSnapToGrid.Size = new System.Drawing.Size(23, 22);
			this.toolStripBtnSnapToGrid.Tag = "Snap To Grid";
			this.toolStripBtnSnapToGrid.Text = "Snap To Grid";
			this.toolStripBtnSnapToGrid.Click += new System.EventHandler(this.toolStripBtnSnapToGrid_Click);
			// 
			// toolStripBtnZoom
			// 
			this.toolStripBtnZoom.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
			this.toolStripBtnZoom.ImageIndex = 17;
			this.toolStripBtnZoom.Name = "toolStripBtnZoom";
			this.toolStripBtnZoom.Size = new System.Drawing.Size(23, 22);
			this.toolStripBtnZoom.Tag = "ZoomTool";
			this.toolStripBtnZoom.Text = "ZoomTool";
			this.toolStripBtnZoom.ToolTipText = "Zoom";
			this.toolStripBtnZoom.Click += new System.EventHandler(this.ToolActivate);
			// 
			// toolStripCmbMagnify
			// 
			this.toolStripCmbMagnify.AutoCompleteSource = System.Windows.Forms.AutoCompleteSource.CustomSource;
			//this.toolStripCmbMagnify.DropDownHeight = 110;
			//this.toolStripCmbMagnify.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			//this.toolStripCmbMagnify.DropDownWidth = 100;
			this.toolStripCmbMagnify.FlatStyle = System.Windows.Forms.FlatStyle.Standard;
			this.toolStripCmbMagnify.IntegralHeight = false;
			this.toolStripCmbMagnify.Items.AddRange(new object[] {
            "100%",
            "150%",
            "200%",
            "25%",
            "300%",
            "400%",
            "50%",
            "500%",
            "75%"});
			this.toolStripCmbMagnify.Name = "toolStripCmbMagnify";
			this.toolStripCmbMagnify.Size = new System.Drawing.Size(121, 23);
			this.toolStripCmbMagnify.Text = "100%";
			this.toolStripCmbMagnify.SelectedIndexChanged += new System.EventHandler(this.toolStripCmbMagnify_SelectedIndexChanged);
			this.toolStripCmbMagnify.Click += new System.EventHandler(this.toolStripCmbMagnify_Click);
			this.toolStripCmbMagnify.LostFocus += new EventHandler(toolStripCmbMagnify_LostFocus);
			// 
			// splitter1
			// 
			this.splitter1.Dock = System.Windows.Forms.DockStyle.Top;
			this.splitter1.Location = new System.Drawing.Point(0, 25);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(800, 3);
			this.splitter1.TabIndex = 107;
			this.splitter1.TabStop = false;
			// 
			// OperatorsScreen
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.InactiveCaptionText;
			this.ClientSize = new System.Drawing.Size(800, 510);
			this.Controls.Add(this.splitter1);
			this.Controls.Add(this.toolStripPanel1);
			this.Controls.Add(this.dcPanel);
			this.Controls.Add(this.pnlQueryView);
			this.ForeColor = System.Drawing.SystemColors.ControlText;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "OperatorsScreen";
			this.Tag = "SID_OPERATOR";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			((System.ComponentModel.ISupportInitialize)(this.dockingManager1)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.childFrameBarManager1)).EndInit();
			this.pnlCanvasHeader.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.tabControlAdv1)).EndInit();
			this.dcPanel.ResumeLayout(false);
			this.toolStripPanel1.ResumeLayout(false);
			this.toolStripPanel1.PerformLayout();
			this.toolStrip2.ResumeLayout(false);
			this.toolStrip2.PerformLayout();
			this.toolStrip1.ResumeLayout(false);
			this.toolStrip1.PerformLayout();
			this.ResumeLayout(false);
			this.PerformLayout();

		}
		#endregion

		#region Events
		
		
		private void canvas1_ChildrenChangeComplete(object sender, Syncfusion.Windows.Forms.Diagram.NodeCollectionEventArgs evtArgs)
		{
			try
			{
				//Check if the node type is a sybol and if true
				//associate properties to it.
				if(evtArgs.Node.GetType().BaseType == typeof(Symbol) )
				{
					
					//Symbol symbol = new Symbol();
					Symbol symbol = evtArgs.Node as Symbol;
					Type bi = evtArgs.Node.GetType();

					
					ArrayList aryList = new ArrayList();

					//Assign the selected object's PropertyTable to the propertygrid
					//propertyEditor1.PropertyGrid.SelectedObject = bi.GetProperty("PT").GetValue(evtArgs.Node, null);
									
					//Update the BusinesObject Collection 
					//Based on the changetype
					switch(evtArgs.ChangeType)
					{
						case(CollectionExChangeType.Clear):
						{
							foreach(object obj in labelcontroller.SymbolObjectBinding)
							{
								LabelLookup lblLookup = obj as LabelLookup;
								if(lblLookup != null && lblLookup.SymbolID == symbol.GetPropertyValue("ID").ToString())
									aryList.Add(lblLookup);
							}
							for(int i = 0;i<aryList.Count; i++)
							{
								labelcontroller.SymbolObjectBinding.Remove(aryList[i]);
							}
							aryList.Clear();
							break;
						}
					
						case(CollectionExChangeType.Insert):
						{
							if(labelcontroller.BusinessObjectCollection.ContainsKey(symbol.GetPropertyValue("ID")))
								symbol.SetPropertyValue("ID", Guid.NewGuid().ToString());
							labelcontroller.BusinessObjectCollection.Add(symbol.GetPropertyValue("ID"), null);
							break;
						}

						case(CollectionExChangeType.Remove):
						{
							
							labelcontroller.BusinessObjectCollection.Remove(symbol.GetPropertyValue("ID"));
							foreach(object obj in labelcontroller.SymbolObjectBinding)
							{
								LabelLookup lblLookup = obj as LabelLookup;
								if(lblLookup != null && lblLookup.SymbolID == symbol.GetPropertyValue("ID").ToString())
									aryList.Add(lblLookup);
							}
							for(int i = 0;i<aryList.Count; i++)
							{
								labelcontroller.SymbolObjectBinding.Remove(aryList[i]);
							}
							aryList.Clear();
							break;
						}

						case(CollectionExChangeType.Set):
						{
							LRRMessagBox.ShowMessage("SID_SET","SID_LABEL",MessageBoxButtons.OK);    	
							break;
						}
					}
				}
				else if(evtArgs.Node.GetType() == typeof(Syncfusion.Windows.Forms.Diagram.Link))
				{
					if(evtArgs.ChangeType ==CollectionExChangeType.Insert)
					{
						//propertyEditor1.PropertyGrid.SelectedObject = evtArgs.Node;
					}
				}
				else
				{
					//propertyEditor1.PropertyGrid.SelectedObject = null;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(OperatorsScreen),"canvas1_ChildrenChangeComplete Event:"+ex.Message  ,Logger.Level.Debug);
			}
		}


		private void bItemOpen_Click(object sender, System.EventArgs e)
		{
			fileDialog = new FileDialog(GraphicsManagers.FileDialogMode.Open, GraphicsManagers.UserMode.OPERATOR);
			if (fileDialog.ShowDialog(this) == DialogResult.OK)
			{
				for (int i = 0; i < fileDialog.SelectedRecords.Count; i++)
				{
					LRRSavedView selectedView = fileDialog.SelectedRecords[i] as LRRSavedView;
					//LRRSavedView selectedView = obj as LRRSavedView;
					OpenFiles(selectedView.ID);
				}
			}
		}

		
		private void ToolActivate(object sender, System.EventArgs e)
		{
			
			//Syncfusion.Windows.Forms.Tools.XPMenus.BarItem barItem = sender as Syncfusion.Windows.Forms.Tools.XPMenus.BarItem;
			ToolStripButton barItem = sender as ToolStripButton;

			try
			{
				Canvas selectedCanvas = gms.ActiveManager.CanvasObj;
				if (barItem != null)
				{
					string toolName = (string) barItem.Tag;
					Tool tool = selectedCanvas.Controller.GetTool(toolName);
					if (tool != null)
					{
						if (tool.Active)
						{
							selectedCanvas.Controller.DeactivateTool(tool);
							
						}
						else
						{
							selectedCanvas.Controller.ActivateTool(tool);
						}
					}
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(OperatorsScreen),"ToolActivate:"+ex.Message  ,Logger.Level.Debug);
			}
		}

		
		//private void cmbBItemMagnification_Click(object sender, System.EventArgs e)
		//{
		//   if(this.gms.ActiveManager != null)
		//      this.gms.ActiveManager.CanvasObj.Zoom(this.cmbBItemMagnification.TextBoxValue);
		//}


		//private void bItemShowGrid_Click(object sender, System.EventArgs e)
		//{
		//   if(this.gms.ActiveManager != null)
		//   {
		//      this.bItemShowGrid.Checked = !this.bItemShowGrid.Checked;
		//      this.gms.ActiveManager.CanvasObj.View.Grid.Visible = this.bItemShowGrid.Checked;
		//   }
		//}

		
		//private void bItemSnapToGrid_Click(object sender, System.EventArgs e)
		//{
		//   if(this.gms.ActiveManager != null)
		//   {
		//      this.bItemSnapToGrid.Checked = !this.bItemSnapToGrid.Checked ;
		//      this.gms.ActiveManager.CanvasObj.View.Grid.SnapToGrid =	this.bItemSnapToGrid.Checked;
		//   }
		//}

		
		private void tabControlAdv1_ControlAdded(object sender, ControlEventArgs e)
		{
			if(e.Control.GetType() != typeof(Syncfusion.Windows.Forms.ScrollButtons))
			{
				Canvas newCanvas = gms.ActiveManager.CanvasObj;
				try
				{
					this.overViewNavigation.Model= newCanvas.Model;
					this.overViewNavigation.View = newCanvas.View;
					UpdateMagnificationComboBox();
					//this.bItemShowGrid.Checked = newCanvas.View.Grid.Visible;					
					//this.bItemSnapToGrid.Checked = newCanvas.View.Grid.SnapToGrid;
					this.toolStripBtnShowGrid.Checked = newCanvas.View.Grid.Visible;
					this.toolStripBtnSnapToGrid.Checked = newCanvas.View.Grid.SnapToGrid;
					if(this.tabControlAdv1.TabCount == 1)
					{
						pnlCanvasHeader.Show();
						tabControlAdv1.ThemesEnabled = true;
					}
					newCanvas.View.MagnificationChanged +=new ViewMagnificationEventHandler(View_MagnificationChanged);
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(OperatorsScreen),"ActivateLinkTools:"+ex.Message  ,Logger.Level.Debug);
				}
			}
		}

	
		private void barItem1_Click(object sender, System.EventArgs e)
		{
			if(this.barItemNavigation.Checked)
			{
				this.dockingManager1.SetDockVisibility(this.overViewNavigation,	false);
				this.barItemNavigation.Checked = false;
			}
			else
			{
				this.dockingManager1.SetDockVisibility(this.overViewNavigation,	true);
				this.barItemNavigation.Checked = true;
			}
		}

		
		public void SetPanAndZoomVisibility(bool flag)
		{
			this.dockingManager1.SetDockVisibility(this.overViewNavigation,	flag);
		}

		
		public bool GetPanAndZoomVisibility()
		{
			return this.barItemNavigation.Checked;
		}

		#endregion

		#region Protected Methods
//		protected Link CreateDirectedLink(PointF[] pts)
//		{
//			try
//			{
//				Link link = new Link(pts);
//				EndPointDecoratorModel decoratorMdl = Syncfusion.Windows.Forms.Diagram.Global.EndPointDecoratorPalette["Filled Arrow"];
//				if (decoratorMdl != null)
//				{
//					link.EndPoints.LastEndPointDecorator = decoratorMdl.CreateInstance();
//				
//				}
//				return link;
//			}
//			catch(Exception ex)
//			{
//				Logger.LogMessage(typeof(OperatorsScreen),"CreateDirectedLink:"+ex.Message   + "Source :"+ex.Source,Logger.Level.Debug);
//				return null;
//			}
//		}

//		protected Link CreateLink(PointF[] pts)
//		{
//			Link link = new Link(pts);
//			//link.EndPoints.LastEndPoint = new FilledArrowDecorator();
//			return link;
//		}

//		protected Link CreateOrthogonalLink(PointF[] pts)
//		{
//			Link link = new Link(Link.Shapes.OrthogonalLine, pts);
//			link.LineStyle.LineColor = Color.Navy;
//			link.LineStyle.LineWidth = 5;
//			link.LineStyle.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor; 
//			return link;
//		}
		
//		protected Link CreateArcLink(PointF[] pts)
//		{
//			Link link = new Link(Link.Shapes.Arc, pts);
//			return link;
//		}

//		protected Link CreateDirectedOrthogonalLink(PointF[] pts)
//		{
//			Link link = new Link(Link.Shapes.OrthogonalLine, pts);
//			EndPointDecoratorModel decoratorMdl = Syncfusion.Windows.Forms.Diagram.Global.EndPointDecoratorPalette["Filled Arrow"];
//			if (decoratorMdl != null)
//			{
//				link.EndPoints.LastEndPointDecorator = decoratorMdl.CreateInstance();
//			}
//			return link;
//		}
		#endregion

		#region Private Methods
	
		public void OpenFiles(string fileID)
		{
			object[] objArray = new object[1];
			objArray[0] = fileID;
			_app.MainFrame.BeginInvoke(_openFile,objArray);
			
		}


	
		///	<summary>
		///	Occurs when	the	canvas is being	closed
		///	</summary>
		///	<returns></returns>
		private	bool CanvasClosing()
		{
		
			if(gms.ActiveManager != null)
			{
				Canvas activeCanvas	= gms.ActiveManager.CanvasObj;
				CloseTabPage(activeCanvas);
			
			}
				return true;
		}


		/// <summary>
		/// Unhook Events and clear Opend view Hashtable Entry 
		/// </summary>
		/// <param name="activeCanvas"></param>
		private void CloseTabPage(Canvas activeCanvas)
		{
			gms.ActiveManager.CanvasObj.View.MagnificationChanged-=new ViewMagnificationEventHandler(View_MagnificationChanged);
			if(activeCanvas.FileID != null && gms.OpenedViews.ContainsKey(activeCanvas.FileID))
			{
				gms.OpenedViews.Remove(activeCanvas.FileID);
				
			}
			gms.ActiveManager.LRRBindingManagerObj.RemoveBinding();
			gms.RemoveGraphicsManager(gms.ActiveManager.ID);

			
			if(!_isFormClosing)
				this.tabControlAdv1.SelectedTab.Dispose();
			if(this.tabControlAdv1.TabCount	== 0)
			{
				pnlCanvasHeader.Hide();
				tabControlAdv1.ThemesEnabled = false;
				_app.LRRUIManagerObj.HidePropertyDetailsDialog();
				
			}
		}


		/// <summary>
		/// To update the Magnification ComboBox based on the magnification 
		/// </summary>
		private void UpdateMagnificationComboBox()
		{
			Canvas activeCanvas = gms.ActiveManager.CanvasObj;
			System.Drawing.Size szMag = activeCanvas.View.Magnification;
			if (this.toolStripCmbMagnify.ComboBox.SelectedItem != null)
			{
				this.toolStripCmbMagnify.ComboBox.SelectedItem = szMag.Width + "%";
				if (this.toolStripCmbMagnify.ComboBox.SelectedItem.ToString() != szMag.Width + "%")
				{
					if (!(toolStripCmbMagnify.ComboBox.Items.Contains(szMag.Width + "%")))
					{
						this.toolStripCmbMagnify.ComboBox.Items.Add(szMag.Width + "%");
						this.toolStripCmbMagnify.ComboBox.SelectedItem = szMag.Width + "%";
					}
				}
			}
			else if (this.toolStripCmbMagnify.ComboBox.Text != null)
			{
				if (!(toolStripCmbMagnify.ComboBox.Items.Contains(szMag.Width + "%")))
				{
					this.toolStripCmbMagnify.ComboBox.Items.Add(szMag.Width + "%");
					this.toolStripCmbMagnify.ComboBox.SelectedItem = szMag.Width + "%"; 				
					sortCmbValues(this.toolStripCmbMagnify);
				}
				else
				{
					this.toolStripCmbMagnify.ComboBox.SelectedItem = szMag.Width + "%";
				}
			}
			activeCanvas.View.Origin = new PointF(0, 0);
			//sortCmbValues(this.toolStripCmbMagnify);
		}


		private void WireGmsEvents(GraphicsManagers gms)
		{
			this.gms.CanvasSelectionChanged+=new NodeCollectionEventHandler(gms_CanvasSelectionChanged);
			this.gms.MouseEnter+=new NodeMouseEventHandler(gms_MouseEnter);
			this.gms.ShowAttributeDetails +=new EventHandler(gms_ShowAttributeDetails);
//			this.gms.SetLabelsVisibility+=new EventHandler(gms_SetLabelsVisibility);
			this.gms.SelectDisplayUOM +=new EventHandler(gms_SelectDisplayUOM);
			this.gms.ViewTrend+=new EventHandler(gms_ViewTrend);
			this.gms.LabelDoubleClick+=new NodeMouseEventHandler(gms_LabelDoubleClick);
			this.gms.OverrideValueClick+=new EventHandler(gms_OverrideValueClick);
			this.gms.NodeDoubleClick+=new NodeMouseEventHandler(gms_NodeDoubleClick);
		}

		private void UnWireGmsEvents(GraphicsManagers gms)
		{
			this.gms.CanvasSelectionChanged-=new NodeCollectionEventHandler(gms_CanvasSelectionChanged);
			this.gms.MouseEnter-=new NodeMouseEventHandler(gms_MouseEnter);
			this.gms.ShowAttributeDetails -=new EventHandler(gms_ShowAttributeDetails);
//			this.gms.SetLabelsVisibility-=new EventHandler(gms_SetLabelsVisibility);
			this.gms.SelectDisplayUOM -=new EventHandler(gms_SelectDisplayUOM);
			this.gms.LabelDoubleClick-=new NodeMouseEventHandler(gms_LabelDoubleClick);
			this.gms.OverrideValueClick-=new EventHandler(gms_OverrideValueClick);
			this.gms.ViewTrend-=new EventHandler(gms_ViewTrend);
			this.gms.NodeDoubleClick-=new NodeMouseEventHandler(gms_NodeDoubleClick);
		}

		#endregion
	
		#region File Menu


		private void btnClose_Click(object sender, System.EventArgs e)
		{
			CanvasClosing();
//			pnlPropDetails.Controls.Clear();

		}

		private void tabControlAdv1_SelectedIndexChanged(object sender, EventArgs e)
		{
			TabControlAdv tabControl = sender as TabControlAdv;
			if(tabControl.SelectedTab != null)
			{
				gms.SetActiveGraphicsManager(this.tabControlAdv1.SelectedTab.Tag as String);
				Canvas selectedCanvas =	gms.ActiveManager.CanvasObj;//tabControl.SelectedTab.Controls[0]as Canvas;
				UpdateMagnificationComboBox();
				this.overViewNavigation.Model= selectedCanvas.Model;
				this.overViewNavigation.View = selectedCanvas.View;
				//this.bItemShowGrid.Checked = selectedCanvas.View.Grid.Visible;				
				//this.bItemSnapToGrid.Checked = selectedCanvas.View.Grid.SnapToGrid;
				this.toolStripBtnShowGrid.Checked = selectedCanvas.View.Grid.Visible;
				this.toolStripBtnSnapToGrid.Checked = selectedCanvas.View.Grid.SnapToGrid;
			}
		}


		private void OperatorsScreen_Closing(object sender, CancelEventArgs e)
		{
			_isFormClosing = true;
            ApplicationPersistanceUtil.SaveOperatorsCanvas(this);
			for(int i = tabControlAdv1.TabCount; i>0; i--)
			{
				tabControlAdv1.SelectedIndex  = i-1;
				bool closeWindow = CanvasClosing();
				if(!closeWindow)
					e.Cancel = true;
				else
				{
					if(gms.GridView != null)
					{
						Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridFactory = gms.GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
						if(iGridFactory != null)
						{
							iGridFactory.ClearAllBindings();
							iGridFactory.UnSubscribeReconnectCompleted();
						}
						else
						{
							Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory iGrpGridFactory = gms.GridView as Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory;
							if(iGridFactory != null)
							{
								iGrpGridFactory.ClearAllBindings();
								iGrpGridFactory.UnSubscribeReconnectCompleted();
							}
						}
					}
					_app.LRRUIManagerObj.HidePropertyDetailsDialog();
					_app.MainFrame.Activate();
				}
			}
          
		
		}

	
		#endregion

		#region Enumerator

		public enum	fileDialogMode
		{
			Open,
			SaveAs
		}

		#endregion

		/// <summary>
		/// Gets Canvas Data from the DB when opening saved View
		/// </summary>
		/// <param name="fileID"></param>
		/// <returns></returns>
		public LRRSavedViewData GetCanvasData(string fileID)
		{
			try
			{
				string whereClause =  " WHERE ID = '"+ fileID + "'"; // Where Clause for Querying the DB
				LRRSavedViewDatas lrrSavedViewDatas = new LRRSavedViewDatas(); 
				_app.DBManagerObj.PopulateCollection(lrrSavedViewDatas,whereClause);//Fetch Saved View data using FileID
				LRRSavedViewData myobj = lrrSavedViewDatas[0];
				return myobj;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(OperatorsScreen),"GetCanvasData:"+ex.Message  ,Logger.Level.Debug);
				return null;
			}
		}

		public TabControlAdv GetTabControl()
		{
			return this.tabControlAdv1;
		}


		private ArrayList alst = new ArrayList();

		#region ILRRUIContainer Members

		public new void Refresh()
		{
		}

		public void Save()
		{
		}

		public void CopyMasterProperties()
		{
		}

		public void Paste()
		{
		}

		public void Copy()
		{
		}

		public void Print()
		{
			if(gms.ActiveManager != null && gms.ActiveManager.CanvasObj != null)
			{
				Canvas activeDiagram = gms.ActiveManager.CanvasObj;
				System.Drawing.Printing.PrintDocument printDoc = activeDiagram.CreatePrintDocument();
				PrintDialog printDlg = new PrintDialog();
				printDlg.Document = printDoc;
				printDlg.AllowSomePages = true;
				
				if (printDlg.ShowDialog(this) == DialogResult.OK)
				{
					printDoc.PrinterSettings = printDlg.PrinterSettings;
					printDoc.Print();
				}
			}
		}

		public void Delete()
		{
		}

		public void ArrangeColumns()
		{
		}

		public void ClearFilter()
		{
		}

		public void InsertNewRows()
		{
		}

		public void ImportToExcel()
		{
		}

		public void Export()
		{
		}

		public void SelectionPaste()
		{
		}

		public void RightLevel()
		{
		}

		public void LeftLevel()
		{
		}

		public void UpLevel()
		{
		}

		public void DownLevel()
		{
		}

		public bool SupportsMasterPropertiesCopy()
		{
			return false;
		}

		public bool SupportsCopy()
		{
			return false;
		}

		public bool SupportsPaste()
		{
			return false;
		}

		public bool SupportsExport()
		{
			return false;
		}

		public bool SupportsSave()
		{
			return false;
		}

		public bool SupportsRefresh()
		{
			return false;
		}

		public bool SupportsDelete()
		{
			return false;
		}

		public bool SupportsPrint()
		{
			return true;
		}

		public bool SupportsArrangeColumns()
		{
			return false;
		}

		public bool SupportsClearFilter()
		{
			return false;
		}

		public bool SupportsInsertNewRows()
		{
			return false;
		}

		public bool SupportsImportToExcel()
		{
			return false;
		}

		public bool SupportsSelectionPaste()
		{
			return false;
		}

		public bool SupportsLevel()
		{
			return false;
		}

		private void bItemPropertyDetails_Click(object sender, System.EventArgs e)
		{
			if(this.bItemPropertyDetails.Checked)
			{
				_app.LRRUIManagerObj.HidePropertyDetailsDialog();
				this.bItemPropertyDetails.Checked =	false;
			}
			else
			{
				_app.LRRUIManagerObj.ShowPropertyDetailsDialog();
				if(gms.ActiveManager != null) 
				_app.LRRUIManagerObj.UpdatePropertyDetailsPanel(gms);
				this.bItemPropertyDetails.Checked =	true;
			}
		}


		public TabControl	ParentTabControl
		{
			set
			{
				_parentTabControl = value;
			}
			get{return _parentTabControl;}
		}
	
		public TabPage ParentTabPage
		{
			set
			{
				_parentTabPage = value;
			}
			get
			{
				return _parentTabPage;
			}
		}
		#endregion

		#region ILRRUIContainer Members

		public void ClearSort()
		{
			// TODO:  Add OperatorsScreen.ClearSort implementation
		}

		public bool SupportsClearSort()
		{
			// TODO:  Add OperatorsScreen.SupportsClearSort implementation
			return false;
		}

		#endregion

		private void biPrintZooming_Click(object sender, System.EventArgs e)
		{
			this.PrintZooming();
		
		}


		#region Printing

		public void PageSetup()
		{
			if(gms.ActiveManager != null && gms.ActiveManager.CanvasObj != null)
			{
				Canvas activeDiagram = gms.ActiveManager.CanvasObj;

				PageSetupDialog dlg = new PageSetupDialog();

				// Made to make values more user friendly 
				activeDiagram.View.ConvertPageMargins(false);
				dlg.PageSettings = activeDiagram.Model.PrintData.PageSettings;
				
				if (dlg.ShowDialog(this) == DialogResult.OK)
				{
					activeDiagram.Model.PrintData.PageSettings = dlg.PageSettings;
					activeDiagram.View.RefreshPageSettings();
					activeDiagram.View.Update(true);
				}
				else
				{
					// Need to convert backwards
					activeDiagram.View.ConvertPageMargins(true);	
				}
			}
		}



		public void PrintPreview()
		{
			if(gms.ActiveManager != null && gms.ActiveManager.CanvasObj != null)
			{
				Canvas activeDiagram = gms.ActiveManager.CanvasObj;
				System.Drawing.Printing.PrintDocument printDoc = activeDiagram.CreatePrintDocument();
				PrintPreviewDialog printPreviewDlg = new PrintPreviewDialog();

				printDoc.PrinterSettings.FromPage = 0;
				printDoc.PrinterSettings.ToPage = 0;
				printDoc.PrinterSettings.PrintRange = PrintRange.AllPages;
       
				printPreviewDlg.Document = printDoc;
				printPreviewDlg.ShowDialog(this);
			}
		}

		private void PrintZooming()
		{
			if(gms.ActiveManager != null && gms.ActiveManager.CanvasObj != null)
			{
				Canvas activeDiagram = gms.ActiveManager.CanvasObj;
				PrintZoomingDialog dlg = new PrintZoomingDialog();
				float i = (activeDiagram.Model.PrintData.PrintingZoom * 100);
				dlg.comboBox1.Text = i.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
				dlg.SheetsAcross = activeDiagram.Model.PrintData.SheetsAcross.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				dlg.SheetsDown = activeDiagram.Model.PrintData.SheetsDown.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				if ( activeDiagram.Model.PrintData.UsePrintingZoom )
					dlg.radio1.Checked = true;
				else
				{
					dlg.radio2.Checked = true;
				}
				if ( dlg.ShowDialog(this) == DialogResult.OK )
				{
					if ( dlg.radio1.Checked )
					{
						activeDiagram.Model.PrintData.UsePrintingZoom = true; 
						activeDiagram.Model.PrintData.PrintingZoom = float.Parse(Regex.Replace(dlg.comboBox1.Text, "%", ""),Symcon.BusinessObjects.SymConverter.Culture);
					}
					else
					{
						activeDiagram.Model.PrintData.UsePrintingZoom = false;
						activeDiagram.Model.PrintData.SheetsAcross = int.Parse(dlg.SheetsAcross,Symcon.BusinessObjects.SymConverter.Culture);
						activeDiagram.Model.PrintData.SheetsDown = int.Parse(dlg.SheetsDown,Symcon.BusinessObjects.SymConverter.Culture);
					}
				}
			}
		}

		#endregion

		#region ILRRUIContainer Members

		public void CopyUOMTypes()
		{
		
		}

		public bool SupportsCopyUOMTypes()
		{
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void ShowPropertyList()
		{
			// TODO:  Add OperatorsScreen.ShowPropertyList implementation
		}

		public bool SupportsResultsFilter()
		{
			// TODO:  Add OperatorsScreen.SupportsResultsFilter implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void ShowComponentList()
		{
			// TODO:  Add OperatorsScreen.ShowComponentList implementation
		}

		#endregion

		private void gms_CanvasSelectionChanged(object sender, NodeCollectionEventArgs evtArgs)
		{
//			if(evtArgs.ChangeType == CollectionExChangeType.Insert)
//			{
//				if(evtArgs.Node.GetType() !=	typeof(Syncfusion.Windows.Forms.Diagram.SymbolLabel) &&	 gms.SelectedModes.Count > 0 &&	_app.LRRUIManagerObj.IsPropertyDialogVisible)
//					_app.LRRUIManagerObj.UpdatePropertyDetailsPanel(gms);
//			}
			gms.ActiveManager.CanvasObj.Model.UpdateViews();
		}

//		private void gms_LabelTextChanged(object sender, string propertyName)
//		{
//			//LRRLabel lbl = sender as LRRLabel;
//			//GraphicsUtilities.ApplyDevationRule(gms.SelectedModes,lbl.Parent as Symbol,lbl);
//		}
//
//		private void gms_LabelPropertyValueChanged(object sender, string propertyName)
//		{
//			//this.gms.ActiveManager.CanvasObj.Model.UpdateViews();
//
//		}

		private void OnIdle(object sender, EventArgs e)
        {
			this.bItemPropertyDetails.Checked = _app.LRRUIManagerObj.IsPropertyDialogVisible;
			if(gms.ActiveManager != null)
			{
				biPrintZooming.Enabled			= true;
				//bItemPan.Enabled				= true;
				toolStripBtnPan.Enabled			= true;	  
				toolStripBtnShowGrid.Enabled	= true;
				toolStripBtnSnapToGrid.Enabled = true;
				//bItemZoom.Enabled					= true;
				toolStripBtnZoom.Enabled = true;
				//cmbBItemMagnification.Enabled	= true;	  
				toolStripCmbMagnify.Enabled = true;
				toolStripBtnSelect.Enabled		= true;
				bItemClose.Enabled				= true;
                if (this.tabControlAdv1.Renderer != null)
                {
                    btnMoveRight.Enabled = this.tabControlAdv1.Renderer.CanScrollRight;
                    btnMoveLeft.Enabled = this.tabControlAdv1.Renderer.CanScrollLeft;
                }
			}
			else
			{
				biPrintZooming.Enabled			= false;	   
				toolStripBtnPan.Enabled			= false;	  
				toolStripBtnSnapToGrid.Enabled = false;
				toolStripBtnShowGrid.Enabled = false;	 
				toolStripBtnZoom.Enabled = false;   
				//toolStripCmbMagnify.Enabled = false; 
				toolStripBtnSelect.Enabled = false;
				bItemClose.Enabled				= false;

			}
		
		}
		#region ILRRUIContainer Members

		public void LeftColMovement()
		{
			// TODO:  Add OperatorsScreen.LeftColMovement implementation
		}

		public void RightColMovement()
		{
			// TODO:  Add OperatorsScreen.RightColMovement implementation
		}

		#endregion

		private void gms_MouseEnter(object sender, NodeMouseEventArgs evtArgs)
		{
			GraphicsUtilities.ActivateToolTip(evtArgs,_toolTip,gms);
		}


		#region ILRRUIContainer Members

		public void Undo()
		{
			// TODO:  Add OperatorsScreen.Undo implementation
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add OperatorsScreen.SupportsPageSetup implementation
			return true;
		}

		public bool SupportsUndo()
		{
			// TODO:  Add OperatorsScreen.SupportsUndo implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void OpenCompositeView()
		{
			// TODO:  Add OperatorsScreen.OpenCompositeView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add OperatorsScreen.TransposeView implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add OperatorsScreen.OpenQuery implementation
		}

		public void OpenView()
		{
			// TODO:  Add OperatorsScreen.OpenView implementation
		}

		public bool SupportsOpenCompositeView()
		{
			// TODO:  Add OperatorsScreen.SupportsOpenCompositeView implementation
			return false;
		}

		public bool SupportsTransposeView()
		{
			// TODO:  Add OperatorsScreen.SupportsTransposeView implementation
			return false;
		}

		public bool SupportsOpenQuery()
		{
			// TODO:  Add OperatorsScreen.SupportsOpenQuery implementation
			return false;
		}

		public bool SupportsOpenView()
		{
			// TODO:  Add OperatorsScreen.SupportsOpenView implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public bool SupportsCompositeView()
		{
			// TODO:  Add OperatorsScreen.SupportsCompositeView implementation
			return false;
		}

		#endregion

		private void gms_ShowAttributeDetails(object sender, EventArgs e)
		{
			try
			{
				if(!this.bItemPropertyDetails.Checked)
				{
					_app.LRRUIManagerObj.ShowPropertyDetailsDialog();
					this.bItemPropertyDetails.Checked =	true;
				}
				_app.LRRUIManagerObj.UpdatePropertyDetailsPanel(gms);
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(OperatorsScreen),"gms_LabelDoubleClick :"+ex.Message  ,Logger.Level.Debug);

			}
		}

		private void gms_SelectDisplayUOM(object sender, EventArgs e)
		{
			if(gms.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
			{
				GraphicsUtilities.OnUOMChangeForGridCell(gms.ActiveManager.CanvasObj.Controller.NodesHit.Last,null);

			}
			else
			{

				LRRLabel selectedLbl = gms.ActiveManager.CanvasObj.Controller.NodesHit.Last as LRRLabel;

				Symbol smbl = selectedLbl.Parent as Symbol;
				if(selectedLbl != null)
				{
					foreach(LRRLabel lbl in smbl.Labels)
					{
						if(lbl.IsUOM && lbl.ParentID == selectedLbl.ParentID)
						{
							if(lbl.ID != null && lbl.Tag != null)
							{
								LRRUOMType lrrUOMType = lbl.Tag as LRRUOMType;
								if(lrrUOMType != null)
									GraphicsUtilities.ChangeUOMClick(lbl,lrrUOMType.UOMTypeID);
								break;
							}
						}
					}
				}
			}
			gms.ActiveManager.CanvasObj.Model.UpdateViews();
		}

		private void btnMoveLeft_Click(object sender, System.EventArgs e)
        {
            this.tabControlAdv1.Renderer.Scroll(ScrollIncrement.Tab, ScrollDirection.Left);
            //if(this.tabControlAdv1.SelectedIndex !=	-1)
            //    this.tabControlAdv1.SelectedIndex =	this.tabControlAdv1.SelectedIndex -1;
		}

		private void btnMoveRight_Click(object sender, System.EventArgs e)
        {
            this.tabControlAdv1.Renderer.Scroll(ScrollIncrement.Tab, ScrollDirection.Right);
            
            //if(this.tabControlAdv1.SelectedIndex +1	< this.tabControlAdv1.TabCount)
            //    this.tabControlAdv1.SelectedIndex =	this.tabControlAdv1.SelectedIndex +1;
		}
		#region ILRRUIContainer Members

		public bool PerformDirtyChk
		{
			get
			{
				// TODO:  Add OperatorsScreen.PerformDirtyChk getter implementation
				return true;
			}
		}

		#endregion

		private void OnOpenFile(string fileID)
		{
			if(fileID != null && gms.OpenedViews.ContainsKey(fileID))
			{
				foreach(TabPageAdv tp in tabControlAdv1.TabPages)
				{
					Canvas openCanvas =	tp.Controls[0] as Canvas;
					if(openCanvas.FileID ==	fileID)
					{
						tabControlAdv1.SelectedTab = tp;
						break;
					}
				}
			}
			else
			{                          // Writes the BLOB to a file (*.bmp).
				try
				{
									
					LRRSavedView lrsv =	lrrSavedViews.GetSavedViewFromID(fileID);
					if(lrsv == null)
						return;
                    gms.CreateGraphicsManager(fileID);
                    Canvas canvas = gms.ActiveManager.CanvasObj;
                    canvas.Model.BeginUpdate();
					//lrrSavedViews.Clear();
					//_app.DBManagerObj.PopulateCollection(lrrSavedViews);

                    //_app.ProcessCanvasForFlagging(canvas) added by ashwin
                    _app.ProcessCanvasForFlagging(canvas);

					TabPageAdv tabPage = new TabPageAdv();
					tabPage.Controls.Add(canvas);
					tabPage.Tag = gms.ActiveManager.ID;
                    tabPage.BackColor = Color.LightBlue;
                    tabPage.BackColor = Color.Black;
					tabControlAdv1.Controls.Add(tabPage);
					canvas.FileDescription = lrsv.SavedViewDescription;
					if(lrsv.SavedViewDescription != null && lrsv.SavedViewDescription.Trim() != "")
						tabPage.ToolTipText = lrsv.SavedViewDescription;
					tabControlAdv1.SelectedTab = tabPage;
					canvas.FileName	= lrsv.SavedViewName;
					canvas.FileID = lrsv.ID	 ;
					tabPage.Text = canvas.FileName;
					gms.OpenedViews.Add(lrsv.ID, canvas);
					gms.ActiveManager.BindSymbols(gms.SelectedModes,GraphicsManagers.PropertiesMappingMode.OPEN,GraphicsManagers.UserMode.OPERATOR);

					canvas.Model.Modified =	false;
					canvas.View.HorizontalRuler.Visible = false;
					canvas.View.VerticalRuler.Visible = false;
					canvas.Model.Modified =	false;
					//canvas.Model.UpdateViews();
					//					canvas.Enabled = false;
					//					canvas.DisableScrollWindow = true;

                    canvas.Model.EndUpdate(true);

				}

				catch(Exception ex)
				{
					Logger.LogMessage(typeof(OperatorsScreen),"OpenFiles:"+ex.Message  ,Logger.Level.Debug);
				}

			}

		}

		private void bItemClose_Click(object sender, EventArgs e)
		{
			CanvasClosing();
		}

		private void LoadQuery(LRRSearchCriteria lrrsc)
		{
			if(gms.GridView != null)
			{
				Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridFactory = gms.GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
				if(iGridFactory != null)
				{
					iGridFactory.ClearAllBindings();
					iGridFactory.UnSubscribeReconnectCompleted();
				}
				else
				{
					Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory iGrpGridFactory = gms.GridView as Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory;
					if(iGridFactory != null)
					{
						iGrpGridFactory.ClearAllBindings();
						iGrpGridFactory.UnSubscribeReconnectCompleted();
					}
				}
			}
			pnlQueryView.Controls.Clear();

			LRRModes lrrModes = LRRModel.ProjectModes;
			
			SearchRetrievalUtility searchUtil = new SearchRetrievalUtility();
			LRRSiteComponents lrrSiteComponents = searchUtil.PopulateSiteComponents(lrrsc.ID);
		
			//Aggregates
			LRRAggregates lrrResultAggregates = searchUtil.PopulateAggregates(lrrsc.ID);
	
			string sSource = lrrsc.Source == null ? "" : lrrsc.Source.ToString(Symcon.BusinessObjects.SymConverter.Culture);
			if(sSource == "I/O")
			{
				LRRSearchCriteria lrrSearchCriteria =  lrrsc;

				string templateClass = lrrSearchCriteria.TemplateClass; 
				switch(templateClass)
				{
					case "Symcon.LightRidge.PEAdvisor.GridSimpleView":
						templateClass = "Symcon.LightRidge.PEAdvisor.IOScreen.GridSimpleViewController";
						break;
					case "Symcon.LightRidge.PEAdvisor.GridTransposedView":
						templateClass = "Symcon.LightRidge.PEAdvisor.IOScreen.GridTransposedViewController";
						break;
					case "Symcon.LightRidge.PEAdvisor.GridSimpleGroupedModeView":
						templateClass = "Symcon.LightRidge.PEAdvisor.IOScreen.GridSimpleGroupedModeViewController";
						break;
					case "Symcon.LightRidge.PEAdvisor.GridTransposedGroupedModeView":
						templateClass = "Symcon.LightRidge.PEAdvisor.IOScreen.GridTransposedGroupedModeViewController";
						break;
					default :
						templateClass = templateClass;
						break;
				}
				
				Type gridType = Type.GetType(templateClass);
				object gridView = Activator.CreateInstance(gridType);
				gms.GridView = gridView;
				Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory objGrid = gridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
				ArrayList custCollection = new ArrayList();
				custCollection.Add(lrrSiteComponents);
				custCollection.Add(lrrResultAggregates);
				objGrid.SearchQuery.SearchQuery.UserSettingsID = lrrSearchCriteria.UserSettingsID;

				LRRSearchCriteriaItemProperties  srchCriteriaItemProps = new LRRSearchCriteriaItemProperties();
				this._app.DBManagerObj.PopulateCollection(srchCriteriaItemProps," where SearchCriteriaID = '" + lrrSearchCriteria.ID + "'");

				objGrid.LoadPanel(custCollection,srchCriteriaItemProps);
				ArrayList arrList = objGrid.GetPanelCollection();
				if(arrList.Count > 0)
				{
					SetPanel(arrList);
				}
				if(!dockingManager1.GetDockVisibility(pnlQueryView))
					dockingManager1.SetDockVisibility(pnlQueryView,true);
				gms.HideColumns();

			}
			else
			{
				LRRSearchCriteria lrrSearchCriteria =  lrrsc;
				Type gridType = Type.GetType(lrrSearchCriteria.TemplateClass);
				object gridView = Activator.CreateInstance(gridType);
				gms.GridView = gridView;
				Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory objGrid = gridView as Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory;
				LRRLookUpProperties objLookUpProperties = new LRRLookUpProperties();
				LRRLookUpQualifiers objLookUpQualifiers = new LRRLookUpQualifiers();
				LRRSearchCriteriaPropertyTypes lrrSearchCriteriaPropertyTypes = new LRRSearchCriteriaPropertyTypes();
				_app.DBManagerObj.PopulateCollection(lrrSearchCriteriaPropertyTypes," WHERE SearchCriteriaID = '" + lrrsc.ID + "'",false);
				LRRQualifiers objQualifiers = new LRRQualifiers();
				LRRSiteComponents objModelComponent = LRRModel.SiteComponentsObj;
				LRRSiteComponents objSiteCompCollection	 = new LRRSiteComponents();
				string qualIDs = "";
				
				////////////				LRRSearchCriteriaQualifiers lrrSrchCriteriaQualifiers = new LRRSearchCriteriaQualifiers();
				////////////                _app.DBManagerObj.PopulateCollection(lrrSrchCriteriaQualifiers,false);
				////////////				LRRSearchCriteriaQualifiers lrrSrchCriteriaQuals = lrrSrchCriteriaQualifiers.FilterBySearchCriteriaID(lrrsc.ID); 
				////////////
				////////////				foreach(LRRSearchCriteriaQualifier lrrSearchCriteriaQual in lrrSrchCriteriaQuals)
				////////////				{
				////////////					if( qualIDs=="")
				////////////					{
				////////////						qualIDs="'" + lrrSearchCriteriaQual.QualifierID + "'";
				////////////					}
				////////////					else
				////////////						qualIDs=qualIDs + ",'" + lrrSearchCriteriaQual.QualifierID + "'";
				////////////				
				////////////				}
				////////////				_app.DBManagerObj.PopulateCollection(objLookUpQualifiers," WHERE LookupQualifierID IN (" + qualIDs + ")",false);
				////////////
				////////////				foreach(LRRSiteComponent objSiteComponent in lrrSiteComponents)
				////////////				{
				////////////					LRRSiteComponent objModelSiteComponent = objModelComponent.GetSiteComponentByID(objSiteComponent.SiteComponentID);
				////////////					objSiteCompCollection.Add(objModelSiteComponent);
				////////////				}
				////////////
				////////////				ArrayList custCollection = new ArrayList();
				////////////				custCollection.Add(objSiteCompCollection);
				////////////				custCollection.Add(objLookUpQualifiers);
				///

				ArrayList custCollection = new ArrayList();
				foreach(LRRSiteComponent objSiteComponent in lrrSiteComponents)
				{
					LRRSiteComponent objModelSiteComponent = objModelComponent.GetSiteComponentByID(objSiteComponent.SiteComponentID);
					objSiteCompCollection.Add(objModelSiteComponent);
				}
				//LRRSearchCriteriaPropertyTypes srchPropTypes = _lrrSearchCriteriaPropertyTypes.FilterBySearchCriteriaID(lrrSearchCriteria.ID);
				LRRSearchCriteriaPropertyTypes srchPropTypes =lrrSearchCriteriaPropertyTypes;
				if(srchPropTypes.Count > 0)
				{
					LRRSearchCriteriaPropertyType srchPropType = srchPropTypes[0];
					LRRPropertyType propTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(srchPropType.PropertyType);
					if(propTypeObj != null && propTypeObj.IsDataArray)
					{
						LRRPropertyTypes objPropTypes = new LRRPropertyTypes();
						foreach(LRRSearchCriteriaPropertyType srchCriteriaPropType in srchPropTypes)
						{
							LRRPropertyType lrrPropTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(srchCriteriaPropType.PropertyType);
							objPropTypes.Add(lrrPropTypeObj);
						}
						custCollection.Add(objPropTypes);
						custCollection.Add(objSiteCompCollection);
					}
					else
					{
						LRRSearchCriteriaQualifiers lrrSrchCriteriaQualifiers = new LRRSearchCriteriaQualifiers();
						_app.DBManagerObj.PopulateCollection(lrrSrchCriteriaQualifiers,false);
						LRRSearchCriteriaQualifiers lrrSrchCriteriaQuals = lrrSrchCriteriaQualifiers.FilterBySearchCriteriaID(lrrsc.ID); 
						foreach(LRRSearchCriteriaQualifier lrrSearchCriteriaQual in lrrSrchCriteriaQuals)
						{
							if( qualIDs=="")
							{
								qualIDs="'" + lrrSearchCriteriaQual.QualifierID + "'";
							}
							else
								qualIDs=qualIDs + ",'" + lrrSearchCriteriaQual.QualifierID + "'";
				
						}

						_app.DBManagerObj.PopulateCollection(objLookUpQualifiers," WHERE LookupQualifierID IN (" + qualIDs + ")",false);
						custCollection.Add(objSiteCompCollection);
						custCollection.Add(objLookUpQualifiers);
					}
					
				}

				LRRSearchCriteriaItemGroupPropertiesUIUOMs srchCriteriaUIUOMs = null;
				srchCriteriaUIUOMs = new LRRSearchCriteriaItemGroupPropertiesUIUOMs();
				this._app.DBManagerObj.PopulateCollection(srchCriteriaUIUOMs," where SearchCriteriaID = '" + lrrSearchCriteria.ID + "'");

				objGrid.LoadPanel(custCollection,srchCriteriaUIUOMs);//Change to UIUOMs collection
				ArrayList arrList = objGrid.GetPanelCollection();
				if(arrList.Count > 0)
				{
					SetPanel(arrList);
				}
				if(!dockingManager1.GetDockVisibility(pnlQueryView))
					dockingManager1.SetDockVisibility(pnlQueryView,true);
			}
		}

		private	void SetPanel(ArrayList	pnlCollection)
		{
			int heightOfEach = pnlQueryView.Height/pnlCollection.Count;
			int splitCnt = 0;
			for(int i=pnlCollection.Count-1;i>=0;i--)
			{
				Panel panel = pnlCollection[i] as Panel;
				panel.Height = heightOfEach;
				pnlQueryView.Controls.Add(panel);
				if(splitCnt < pnlCollection.Count - 1)
				{
					Splitter splitGrid = new Splitter();
					splitGrid.Dock = DockStyle.Top;
					pnlQueryView.Controls.Add(splitGrid);
					splitCnt++;
				}
			}
			if(pnlQueryView.Controls.Count > 0)
				pnlQueryView.Controls[0].Dock = DockStyle.Fill;
		}

		private void OperatorsScreen_Activated(object sender, EventArgs e)
		{
			if(gms.ActiveManager != null && gms.ActiveManager.CanvasObj != null && gms.ActiveManager.CanvasObj.Model != null)
				gms.ActiveManager.CanvasObj.Model.UpdateViews();
			if(_implementerPanAndZoom)
				this.SetPanAndZoomVisibility(_implementerPanAndZoom); 

		}

		private void OperatorsScreen_Closed(object sender, EventArgs e)
		{
			_app.MainFrame.Refresh();
			_app.MainFrame.Activate();
		}
		#region ILRRUISpecificContainer Members

		public void ChangeUOMSet()
		{
			if(gms.ActiveManager != null)
			{
				ChangeUOMSetDefinitionForm chngUOMSetDefn = new ChangeUOMSetDefinitionForm();
				DialogResult dlgResult = chngUOMSetDefn.ShowDialog(this);
				if(dlgResult == DialogResult.OK && chngUOMSetDefn.MeasSysID != null)
				{
					gms.ChangeUOMSetDefinationForSelectedSymbols(chngUOMSetDefn.MeasSysID);
					gms.ActiveManager.CanvasObj.Model.UpdateViews();
				}
			}
		}

		public bool SupportsChangeUOMSet()
		{
			// TODO:  Add OperatorsScreen.SupportsChangeUOMSet implementation
			return true;
		}

		#endregion

		private void gms_LabelDoubleClick(object sender, NodeMouseEventArgs evtArgs)
		{
			if(this.bItemPropertyDetails.Checked)
			{
				//_app.LRRUIManagerObj.HidePropertyDetailsDialog();
				//this.barItem2.Checked =	false;
			}
			else
			{
				_app.LRRUIManagerObj.ShowPropertyDetailsDialog();
				_app.LRRUIManagerObj.UpdatePropertyDetailsPanel(gms);
				this.bItemPropertyDetails.Checked =	true;
			}
		}

		private void gms_OverrideValueClick(object sender, EventArgs e)
		{
			if(gms.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
			{
				GridCell lbl =  gms.ActiveManager.CanvasObj.Controller.NodesHit.Last  as GridCell;
				object objPropValue = lbl.Tag ;
				IPropertyValue ipropVal =  lbl.Tag as  IPropertyValue;
				MenuItem miItem = sender as MenuItem;
				if(objPropValue != null)
				{
					LRRSiteComponentPropertyValue lrrPropVal = ipropVal as LRRSiteComponentPropertyValue;
					if(lrrPropVal != null)
					{
						LRRSiteComponentProperty lrrScp =  LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lbl.PropertyID);
						GraphicsSiteComponentUtilities.OnOverrideValueClickForSiteComponent(miItem,ipropVal,lrrScp);
						GraphicsUtilities.SetColor(lbl,ipropVal);
						gms.ActiveManager.CanvasObj.Model.UpdateViews();
					}
					else
					{
						GraphicsSiteComponentUtilities.OnOverrideValueClickForGroupProp(miItem,ipropVal);
						GraphicsUtilities.SetColor(lbl,ipropVal);
						gms.ActiveManager.CanvasObj.Model.UpdateViews();
					}
				}
			}
			else
			{
			
				LRRLabel lbl =  gms.ActiveManager.CanvasObj.Controller.NodesHit.Last  as LRRLabel;
				object objPropValue = lbl.Tag ;
				IPropertyValue ipropVal =  lbl.Tag as  IPropertyValue;
				MenuItem miItem = sender as MenuItem;

				if(objPropValue != null)
				{
					LRRSiteComponentPropertyValue lrrPropVal = ipropVal as LRRSiteComponentPropertyValue;
					if(lrrPropVal != null)
					{
						LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
						LRRSiteComponentProperty lrrScp = parentLbl.Tag as LRRSiteComponentProperty;
						GraphicsSiteComponentUtilities.OnOverrideValueClickForSiteComponent(miItem,ipropVal,lrrScp);
					}
					else
					{
						GraphicsSiteComponentUtilities.OnOverrideValueClickForGroupProp(miItem,ipropVal);
					}
				}
			}
		}

		private void gms_ViewTrend(object sender, EventArgs e)
		{
			MenuItem mItemChild = sender as MenuItem;
            if (mItemChild != null)
            {
                string trendScheme = mItemChild.Text;

                if (gms.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() == typeof(GridCell))
                {
                    GridCell gCell = gms.ActiveManager.CanvasObj.Controller.NodesHit.Last as GridCell;
                    IProperty iProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(gCell.PropertyID);
                    _app.LRRUIManagerObj.ShowTrend(trendScheme, iProp, gCell.ModeID, gCell.UIUOMID);
                }
                else
                {
                    LRRLabel selectedLbl = gms.ActiveManager.CanvasObj.Controller.NodesHit.Last as LRRLabel;
                    GraphicsUtilities.OnVeiwTrend(trendScheme, selectedLbl);
                }
            }
		}

		private void gms_MagnificationChanged(object sender, ViewMagnificationEventArgs evtArgs)
		{
			UpdateMagnificationComboBox();
		}


		/// <summary>
		/// Handles the NodeDoubleClick event of the gms control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="evtArgs">The <see cref="Syncfusion.Windows.Forms.Diagram.NodeMouseEventArgs"/> instance containing the event data.</param>
		private void gms_NodeDoubleClick(object sender, NodeMouseEventArgs evtArgs)
		{
			if (gms.ActiveManager.CanvasObj.Controller.NodesHit.Last.GetType() != typeof(LRRLabel))
			{
				if (evtArgs.Node.GetType().BaseType == typeof(Symbol))
				{
					GraphicalLink gphLink = (evtArgs.Node.GetType().GetProperty("Link").GetValue(evtArgs.Node, null)) as GraphicalLink;
					if (gphLink != null)
					{
						OpenLink(gphLink);

					}
				}
			}
			evtArgs.ContinueOperation = true;
		}




		private void View_MagnificationChanged(object sender, ViewMagnificationEventArgs evtArgs)
		{
			UpdateMagnificationComboBox();
		}

		private void OperatorsScreen_Deactivate(object sender, EventArgs e)
		{
			_implementerPanAndZoom = this.GetPanAndZoomVisibility();
			this.SetPanAndZoomVisibility(false); 
		}

		private void toolStripBtnPan_Click(object sender, EventArgs e)
		{

		}

		private void toolStripBtnShowGrid_Click(object sender, EventArgs e)
		{
			if (this.gms.ActiveManager != null)
			{
				//this.bItemShowGrid.Checked = !this.bItemShowGrid.Checked;
				this.toolStripBtnShowGrid.Checked = !this.toolStripBtnShowGrid.Checked;
				this.gms.ActiveManager.CanvasObj.View.Grid.Visible = this.toolStripBtnShowGrid.Checked;//this.bItemShowGrid.Checked;
			}
		}

		private void toolStripBtnSnapToGrid_Click(object sender, EventArgs e)
		{
			if (this.gms.ActiveManager != null)
			{
				//this.bItemSnapToGrid.Checked = !this.bItemSnapToGrid.Checked;
				this.toolStripBtnSnapToGrid.Checked = !this.toolStripBtnSnapToGrid.Checked;
				this.gms.ActiveManager.CanvasObj.View.Grid.SnapToGrid = this.toolStripBtnSnapToGrid.Checked;//this.bItemSnapToGrid.Checked;
			}
		}

	

		private void toolStripCmbMagnify_Click(object sender, EventArgs e)
		{
			//if (this.gms.ActiveManager != null)
			//   this.gms.ActiveManager.CanvasObj.Zoom(this.toolStripCmbMagnify.SelectedItem.ToString());//this.cmbBItemMagnification.TextBoxValue);
			//sortMagnifyValues();
		}

		private void toolStripCmbMagnify_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (this.gms.ActiveManager != null)
				this.gms.ActiveManager.CanvasObj.Zoom(this.toolStripCmbMagnify.ComboBox.SelectedItem.ToString());//this.cmbBItemMagnification.TextBoxValue);
			//sortMagnifyValues(); 
		}

		private void toolStripBtnOpen_Click(object sender, EventArgs e)
		{
			fileDialog = new FileDialog(GraphicsManagers.FileDialogMode.Open, GraphicsManagers.UserMode.OPERATOR);
			if (fileDialog.ShowDialog(this) == DialogResult.OK)
			{
				for (int i = 0; i < fileDialog.SelectedRecords.Count; i++)
				{
					LRRSavedView selectedView = fileDialog.SelectedRecords[i] as LRRSavedView;
					//LRRSavedView selectedView = obj as LRRSavedView;
					OpenFiles(selectedView.ID);
				}
			}
		}

		private void sortMagnifyValues()
		{
			string selectedVal = toolStripCmbMagnify.SelectedItem.ToString();
  			 
			ArrayList tempList = new ArrayList();
			for (int i = 0; i < toolStripCmbMagnify.Items.Count; i++)
			{
				string val = toolStripCmbMagnify.Items[i].ToString();
				string[] temp = val.Split('%');

				tempList.Add(Convert.ToInt32(temp[0]));

			}
			tempList.Sort();
			object[] tempobj = new object[tempList.Count];
			for (int j = 0; j < tempList.Count; j++)
			{
				tempobj.SetValue((tempList[j] + "%"), j); 
			}

			toolStripCmbMagnify.Items.Clear();
			toolStripCmbMagnify.Items.AddRange(tempobj);

			toolStripCmbMagnify.ComboBox.SelectedItem = selectedVal;
			 
		}

		/// <summary>
		/// Opens the link.
		/// </summary>
		/// <param name="gphLink">The GPH link.</param>
		private void OpenLink(GraphicalLink gphLink)
		{
			string fileID = gphLink.ID;

			switch (gphLink.LinkTypeID)
			{
				case 1:
					OpenFiles(fileID);
					break;
				case 2:
					LRRSearchCriteria lrrsc = LRRModel.ChildModel.SearchCriterias.GetSearchCriteriaByID(fileID);
					LoadQuery(lrrsc);
					break;
				case 3:
					LRRUnifiedViewSearchCrtiteria lrrunifiedViewSearchCriteria = LRRModel.ChildModel.UnifiedViewSearchCrtiterias.GetUnifiedViewByID(fileID);
					LRRViewManager.OpenUnifiedView(lrrunifiedViewSearchCriteria);
					break;
				case 4:
					LRRCompositeView compView = LRRModel.ChildModel.CompositeViews.GetCompositeViewFromID(fileID);
					LRRViewManager.OpenCompositeView(compView);
					break;
				case 5:
					LRRExcelView excelView = LRRModel.ChildModel.ExcelViews.GetExcelViewFromID(fileID);
					LRRViewManager.OpenExcelView(excelView);
					break;
				case 6:
					LRRMultiAttributeTrendInfo multiTrendInfo = LRRModel.ChildModel.MultiTrendInfosObj.GetMultiTrendFromID(fileID);
					LRRViewManager.OpenMultiTrend(multiTrendInfo);
					break;
				default:
					break;
			}
		}

		/// <summary>
		/// Sorts the CMB values.
		/// </summary>
		/// <param name="toolStripCmb">The tool strip CMB.</param>
		private void sortCmbValues(ToolStripComboBox toolStripCmb)
		{
			string selectedVal = toolStripCmb.ComboBox.SelectedItem.ToString();

			ArrayList tempList = new ArrayList();
			for (int i = 0; i < toolStripCmb.Items.Count; i++)
			{
				string val = toolStripCmb.Items[i].ToString();
				string[] temp = val.Split('%');

				tempList.Add(Convert.ToInt32(temp[0]));

			}
			tempList.Sort();
			object[] tempobj = new object[tempList.Count];
			for (int j = 0; j < tempList.Count; j++)
			{
				tempobj.SetValue((tempList[j] + "%"), j);
			}

			toolStripCmb.Items.Clear();
			toolStripCmb.Items.AddRange(tempobj);

			toolStripCmb.ComboBox.SelectedItem = selectedVal;

		}
		/// <summary>
		/// Handles the LostFocus event of the toolStripCmbMagnify control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		void toolStripCmbMagnify_LostFocus(object sender, EventArgs e)
		{
			if (this.gms.ActiveManager != null)
			{
				if (toolStripCmbMagnify.SelectedItem != null)
				{
					this.gms.ActiveManager.CanvasObj.Zoom(this.toolStripCmbMagnify.SelectedItem.ToString());
				}
				else if (toolStripCmbMagnify.Text != null && this.toolStripCmbMagnify.Text != "")
				{
					int orignalVal = gms.ActiveManager.CanvasObj.View.Magnification.Height;
					int newValue = 0;
					int.TryParse(toolStripCmbMagnify.Text, out newValue);
					if (newValue == 0 || newValue > 1000 || newValue < 10)
					{
						newValue = orignalVal;
					}

					this.gms.ActiveManager.CanvasObj.Zoom(this.toolStripCmbMagnify.Text.ToString());
				}
			}

		}		
	}
}
