using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Symcon.LightRidge.Options;
using Symcon.Utils.XMLUtils;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.Utils.MessageLogger;

namespace Symcon.LightRidge.PEAdvisor
{
	public enum MethodExecutionMode
	{
		UI,
		WORKER
	}
	public delegate void ExecuteMethodWithWaitDel(object sorceObject,string methodName,object[] arguments,MethodExecutionMode execMode);
	
	/// <summary>
	/// Summary description for LRRWaitWindowUtility.
	/// </summary>
	public class LRRWaitWindowUtility
	{
		

		static LRRWaitWindowUtility()
		{
			lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
	
		}

		# region method

		public static void Initialize(Form mainWindow)
		{
			try
			{
				_mainFrame = mainWindow;
				_isInitialized = true;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRWaitWindowUtility),"Initialize:"+ex.Message  ,Logger.Level.Debug);

				_isInitialized = false;
			}
		}
	
		public static void ExecuteMethodWithWait(object sorceObject,string methodName,object[] arguments,MethodExecutionMode execMode)
		{
			
			try
			{
				if(_isInitialized == false)
				{
					throw new Exception("Initialized not called!");
				}

				string s = lrrApp.LRRUIManagerObj.WindowName(Convert.ToInt16(arguments[0],Symcon.BusinessObjects.SymConverter.Culture));
				ShowPogressIndicator(s);
                				
				if(_mainFrame.InvokeRequired)
				{
					object[] args = new object[4];
					args[0] = sorceObject;
					args[1] = methodName;
					args[2] = arguments;
					args[3] = execMode;
					
					_mainFrame.Invoke(new ExecuteMethodWithWaitDel(ExecuteMethodWithWait),args);
				}

				//_pw = new ProgressWindow(s);
				
				 
				object[] args1 = new object[4];
				args1[0] = sorceObject;
				args1[1] = methodName;
				args1[2] = arguments;
				args1[3] = execMode;
			
				if(execMode == MethodExecutionMode.UI)
				{
					_mainFrame.Invoke(new ExecuteMethodWithWaitDel(InternalExecuteMethodWithWait),args1);
									
				}
				else
				{
					ExecuteMethodWithWaitDel del = new ExecuteMethodWithWaitDel(InternalExecuteMethodWithWait);
					del.BeginInvoke(sorceObject,methodName,arguments,execMode,null,null);
				}
				
				//_pw.Visible = false;
				//_pw.ShowDialog(); 
						
				//sorceObject.GetType().GetMethod(methodName).Invoke(sorceObject,System.Reflection.BindingFlags.Public,null,arguments,System.Globalization.CultureInfo.CurrentCulture);
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRWaitWindowUtility),"ExecuteMethodWithWait:"+ex.Message  ,Logger.Level.Debug);
				return;
			}

		}

//		public delegate void RefreshWindowDel(object obj);
//
//		private static void TimerMethod(Object obj)
//		{
//			object[] args = new object[1];
//			args[0] = obj;
//			_mainFrame.BeginInvoke(new RefreshWindowDel(RefreshWindow),args);
//			
//		}
//
//		private static void RefreshWindow(object obj)
//		{
//			ProgressWindow pw = obj as ProgressWindow;
//			Application.DoEvents();
//			pw.Refresh();
//		}

		/// <summary>
		/// This method is used to show the ProgressIndicator while the Forms are loading
		/// </summary>
		/// <param name="screenName">Name of the screen that is loaded</param>
		internal static void ShowPogressIndicator(string screenName)
		{
			p1=new System.Diagnostics.Process();
			p1.StartInfo.WorkingDirectory = lrrApp.AppPath;;
			p1.StartInfo.FileName="ProgressIndicator.exe";
            p1.StartInfo.Arguments ="\"" + screenName + "\"";
			p1.Start();
			p1.Refresh();
		}

        internal static void EndProgressIndicator()
        {
            try
            {
                p1.Kill();
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(LRRWaitWindowUtility), ex.Message, Logger.Level.Debug);
            }
        }
		
		#endregion

		#region Private method

		private static void InternalExecuteMethodWithWait(object sourceObject,string methodName,object[] arguments,MethodExecutionMode execMode)
		{
			//sourceObject.GetType().GetMethod(methodName) 
			
			//_pw.Refresh();
			Type[] typs = new Type[1];
			typs[0] = typeof(int);
			sourceObject.GetType().GetMethod(methodName,typs).Invoke(sourceObject,arguments);
			//_pw.Refresh();
			//_pw.Close();
			//_inProgress = false;
			try
			{
				p1.Kill();
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRWaitWindowUtility),ex.Message  ,Logger.Level.Debug);
			}

			
		}

		private static void CallBackResult(IAsyncResult result)
		{
			ExecuteMethodWithWaitDel del = result.AsyncState as ExecuteMethodWithWaitDel;
			del.EndInvoke(result);
		}
		
		#endregion

		
		#region Fields
		
		
		private static Form				_mainFrame;
		private static bool				_isInitialized = false;
		private static ProgressWindow	_pw;
		private  static LRRApp           lrrApp;
		private	static System.Diagnostics.Process	p1;
		private static 	bool			_inProgress;


		#endregion

		
	}
}
