using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using System.Reflection;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Activation;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Http;
using System.Runtime.Remoting.Channels.Tcp;
using System.Globalization;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Lifetime;


using System.Collections.Specialized;

using System.Security.Permissions;
using Symcon.LightRidge.MessageService;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using System.Collections.Generic;

namespace Symcon.LightRidge.PEAdvisor
{
	public class CallbackClass : Symcon.LightRidge.MessageService.MessageComponentDelegatableObject
	{
		private LRRApp _mainApp = null;
		private delegate void PropertyValueChangedCallback(string propertyId, string propertyValue,string modeID);
		private delegate void GroupedPropertyValueChangedCallback(string lookupQualifierID,string groupPropID,string modeID, string groupPropVal);
        private delegate void ShowAlertCallback(string msg);
		PropertyValueChangedCallback _propValueChangedCallback;
		GroupedPropertyValueChangedCallback _groupedPropValueChangedCallback;
        ShowAlertCallback _showAlertCallback;
		public CallbackClass (LRRApp _app)
		{ 
			_mainApp = _app; 
			_propValueChangedCallback = new PropertyValueChangedCallback(OnPropertyValueChangedCallback);
			_groupedPropValueChangedCallback = new GroupedPropertyValueChangedCallback(OnGroupedPropertyValueUpdateCallback);
            _showAlertCallback = new ShowAlertCallback(OnShowAlertCallback);
		}


		/// <summary>
		/// Called at the Client Machine to UPDATE when either 
		/// CalibrationFactor,PreFilterUserOverride or PostFilterUserOverride 
		/// is changed at another Remote Location
		/// </summary>
		/// <param name="histTagUid">HistTag</param>
		/// <param name="calibrationFactor">CalibrationFactor value</param>
		/// <param name="preFilterUserOverride">PreFilterUserOverride value</param>
		/// <param name="postFilterUserOverride">PostFilterUserOverride value</param>
		protected override void InternalCalibrationFactorUpdateCallback(string histTagUid,string calibrationFactor, string preFilterUserOverride,string postFilterUserOverride)
		{
			LRRHistData lrrHistData = ((LRRHistData)LRRModel.HistDatas[histTagUid]);
			bool updated = false;
			string message = "HistTag: ";
			message = message + lrrHistData.HistTag;

			if(lrrHistData.CalibrationFactor != calibrationFactor)
			{
				lrrHistData.CalibrationFactor = calibrationFactor;
				message = message + " CalibrationFactor = " + lrrHistData.CalibrationFactor;
 				updated = true;
                Symcon.Utils.Alert.Messages.LRRAlert.AddAlert(Symcon.Globals.Enums.MessageType.MSgUpdate, "Calibration factor changed for hist tag " + lrrHistData.HistTag);
			}
			if(lrrHistData.PreFilterUserOverride != preFilterUserOverride)
			{
				lrrHistData.PreFilterUserOverride = preFilterUserOverride;
				message = message + " PreFilterUserOverride = " + lrrHistData.PreFilterUserOverride;
				updated = true;
                Symcon.Utils.Alert.Messages.LRRAlert.AddAlert(Symcon.Globals.Enums.MessageType.MSgUpdate, "Pre Filter User Override changed for hist tag " + lrrHistData.HistTag);
			}
			if(lrrHistData.PostFilterUserOverride != postFilterUserOverride)
			{
				lrrHistData.PostFilterUserOverride = postFilterUserOverride;
				message = message + " PostFilterUserOverride = " + lrrHistData.PostFilterUserOverride;
				updated = true;
                Symcon.Utils.Alert.Messages.LRRAlert.AddAlert(Symcon.Globals.Enums.MessageType.MSgUpdate, "Post Filter User Override changed for hist tag " + lrrHistData.HistTag);
			}
			if(updated)
			{
				lrrHistData.ObjectState = Symcon.Globals.Enums.DBOperation.UNCHANGED;
				//Symcon.Utils.Alert.Messages.LRRAlert.AddAlert(Symcon.CCLibrary.Controls.MessageType.MSgUpdate,message);
				updated = false;
			}
		}

		/// <summary>
		/// Called at the Client Machine to UPDATE when
		/// GroupPropertyQualifierValue is changed at 
		/// another Remote Location
		/// </summary>
		/// <param name="groupPropValId"></param>
		/// <param name="groupPropVal"></param>
		protected override void InternalGroupedPropertyValueUpdateCallback(string lookupQualifierID,string groupPropID,string modeID, string groupPropVal)
		{
			object[] objarray = new object[]{lookupQualifierID,groupPropID,modeID, groupPropVal};
			_mainApp.MainFrame.BeginInvoke(_groupedPropValueChangedCallback,objarray);
		}

		/// <summary>
		/// Called at the Client Machine to UPDATE AND SAVE when SiteComponentPropertyValue
		/// is changed at another Remote Location
		/// </summary>
		/// <param name="propertyId"> SiteComponentPropertyID</param>
		/// <param name="propertyValue">value</param>
		/// <param name="modeID">Mode ID of SiteComponentPropertyValue</param>
		protected override void InternalPropertyValueChangedUpdateCallback(string propertyId, string propertyValue,string modeID)
		{
			object[] objarray = new object[]{propertyId, propertyValue,modeID};
			_mainApp.MainFrame.BeginInvoke(_propValueChangedCallback,objarray);
		}

		protected void OnPropertyValueChangedCallback(string propertyId, string propertyValue,string modeID)
		{
		
			try
			{
				string message = "Plant Attribute: ";
				LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(propertyId);
				LRRSiteComponentPropertyValue siteCompPropVal = siteCompProp.GetPropertyValueForMode(modeID) as LRRSiteComponentPropertyValue;
				if(siteCompPropVal != null)
				{
					if(siteCompPropVal.PropertyValue != propertyValue)
					{
						siteCompPropVal.PropertyValue = propertyValue;
						//_mainApp.DBManagerObj.CommitObject(siteCompPropVal);
						message = message + siteCompProp.Name;
						message = message + "(" + LRRModel.Modes.GetModeByID(modeID).ModeDisplayName + ")";
						message = message + " Value = " + propertyValue;
						//	Symcon.Utils.Alert.Messages.LRRAlert.AddAlert(Symcon.CCLibrary.Controls.MessageType.MSgUpdate,message);
				
					}
				}
			}
			catch
			{

			}
		}

        protected void OnShowAlertCallback(string msg)
        {
            Symcon.Utils.Alert.Messages.LRRAlert.AddAlert(Symcon.Globals.Enums.MessageType.MSgUpdate, msg + " Please restart PE-Advisor for changes to reflect.");
        }

        protected override void InternalRaiseAlertCallback(string msg)
        {
            object[] objarray = new object[] { msg };
            _mainApp.MainFrame.BeginInvoke(_showAlertCallback, objarray);
        }

		protected void OnGroupedPropertyValueUpdateCallback(string lookupQualifierID,string groupPropID,string modeID, string groupPropVal)
		{
			try
			{
				string message = "Group Property: ";
                List<IPropertyValue> arrGrpPropQualval = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValuesFromGrpPropIdAndLkupQualId(groupPropID, lookupQualifierID);//GetQualifierValue(groupPropID,lookupQualifierID,modeID);

				if(arrGrpPropQualval != null)
				{
					foreach(LRRSiteGroupPropertyQualifierValue grpPropQualifierValue in arrGrpPropQualval)
					{
						if(grpPropQualifierValue != null && grpPropQualifierValue.ModeID == modeID)
						{
							if(grpPropQualifierValue.PropertyValue != groupPropVal)
							{
								LRRModel.SiteGroupPropertyQualifierValuesObj.WriteQualifierValueForPropertyMode(modeID,groupPropVal);
								LRRSiteComponentGroupProperty grpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(groupPropID);
								LRRSiteComponentProperty lrrSiteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(grpProp.SiteComponentPropID);
								LRRLookUpQualifier lrrLookUpQualifier = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookupQualifierID);
								LRRQualifier lrrQualifier = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLookUpQualifier.QualifierID);
								message = message + lrrSiteCompProp.Name + "\\";
								message = message + lrrQualifier.Name + "\\";
								message = message + LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(groupPropID).AttributeName;
								message = message + "(" + LRRModel.Modes.GetModeByID(modeID).ModeDisplayName + ")";
								message = message + " Value = " + groupPropVal;

								grpPropQualifierValue.ObjectState = Symcon.Globals.Enums.DBOperation.UNCHANGED;
								//Symcon.Utils.Alert.Messages.LRRAlert.AddAlert(Symcon.CCLibrary.Controls.MessageType.MSgUpdate,message);
							}
						}
					}
				}
			}
			catch
			{
			}
		}

		protected override void InternalCommitUpdate()
		{
			try
			{
				LRRSiteGroupPropertyQualifierValues siteCompGrpPropQualVals =LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();
				LRRSiteComponentPropertyValues siteCompPropVals =  LRRModel.PropertyValuesObj.GetChanges(); 
				if(siteCompGrpPropQualVals.Count >0)
				{
					this._mainApp.DBManagerObj.BeginTransaction();
					_mainApp.DBManagerObj.ExecuteBatchUpdate(siteCompGrpPropQualVals);
					this._mainApp.DBManagerObj.CommitTransaction();
					siteCompGrpPropQualVals.RefreshCollection();
				}
				if(siteCompPropVals.Count >0)
				{
					this._mainApp.DBManagerObj.BeginTransaction();
					_mainApp.DBManagerObj.ExecuteBatchUpdate(siteCompPropVals);
					this._mainApp.DBManagerObj.CommitTransaction();
					siteCompPropVals.RefreshCollection();
				}
			}
			catch
			{
			}
		}

		public override object InitializeLifetimeService()
		{
			return null;
		}

        
    }
}
