using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.HeaderDispatcher;
using Symcon.LightRidge.PEAdvisor.UserControls;
using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects.Utilities;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Text;

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for Trend.
	/// </summary>
	public class MultiTrend : System.Windows.Forms.Form, ILRRUIContainer
	{
		private System.Windows.Forms.Panel panelMultiTrend;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public MultiTrend(string trendName)
		{
			if(_lrrApp == null)
				_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

			LRRMultiAttributeTrendInfo trendInfo = new LRRMultiAttributeTrendInfo();

			_lrrApp.DBManagerObj.PopulateObject(trendInfo,typeof(LRRMultiAttributeTrendInfo)," where Name='" + trendName + "'");

			
			
			if(trendInfo != null)
			{
				MemoryStream memoryStream = new MemoryStream (trendInfo.TrendInfo);
				XmlSerializer xs = new XmlSerializer(typeof(TemplateRows));
				TemplateRows templateRows = xs.Deserialize(memoryStream) as TemplateRows;
				memoryStream.Close();

				InitForm(templateRows,trendInfo);
			}
				
		}

		public MultiTrend(TemplateRows templateRows, LRRMultiAttributeTrendInfo trendInfo)
		{
			InitForm(templateRows,trendInfo);
		}

		#region Fields
		
		private TabPage _tabPage;
		private TabControl _parentTabControl;
		private LRRApp _lrrApp = null;
		private TemplateRows _templateRows = null;
		private Panel _pnlSettings = new Panel();
		private Hashtable _htTemplatesRows = new Hashtable();
        private Hashtable _htTrends = new Hashtable();
		private LRRMultiAttributeTrendInfo _trendInfo = null;
        private string _xUOM = null;
        private string _yUOM = null;
		
		#endregion

		#region Delegates

		private delegate void RefreshTrend();
		#endregion

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			//_lrrApp.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(MultiTrend));
            this.panelMultiTrend = new System.Windows.Forms.Panel();
            this.SuspendLayout();
            // 
            // panelMultiTrend
            // 
            this.panelMultiTrend.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panelMultiTrend.Location = new System.Drawing.Point(0, 0);
            this.panelMultiTrend.Name = "panelMultiTrend";
            this.panelMultiTrend.Size = new System.Drawing.Size(743, 466);
            this.panelMultiTrend.TabIndex = 0;
            // 
            // MultiTrend
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(743, 466);
            this.Controls.Add(this.panelMultiTrend);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Name = "MultiTrend";
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "SID_TREND";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
            this.Load += new System.EventHandler(this.MultiTrend_Load);
            this.ResumeLayout(false);

		}
		#endregion

		#region Private Methods

		private void InitForm(TemplateRows templateRows, LRRMultiAttributeTrendInfo trendInfo)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			
			if(_lrrApp == null)
				_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

			_templateRows = templateRows;
			_trendInfo = trendInfo;
			LoadPanelSettings();
			LoadTrends();
			this.Text = trendInfo.Name;
			this.Resize +=new EventHandler(MultiTrend_Resize);
            

			//_lrrApp.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);
		}

        private void LoadPanelSettings()
		{
			int noOfRows = _templateRows.TrendSettings.Length;
			int totalHeight = 0;
			_htTemplatesRows.Clear();
            if (_templateRows.IsXYGraph)
            {
                for (int i = 1; i <= noOfRows; i++)
                {
                    TrendSetting trendSetting = _templateRows.TrendSettings[i - 1];
                    UserControlTrendTemplateRows templateRow = new UserControlTrendTemplateRows();
                    _htTemplatesRows.Add(i, templateRow);
                    templateRow.Dock = DockStyle.Top;
                    templateRow.labelID.Text = "Row " + (i).ToString() + ":";
                    templateRow.checkBoxLog.Checked = trendSetting.IsXLog;
                    templateRow.checkBoxLog.CheckedChanged += new EventHandler(checkBoxLog_CheckedChanged);
                    //UserControls.UserControlTrendTemplates.SelectItem(templateRow.comboBoxSeriesInteriorColor,trendSetting.SeriesInteriorColor.ToString());
                    try
                    {
                        templateRow.labelSeriesColor.BackColor = Color.FromArgb(int.Parse(trendSetting.SeriesInteriorColor));
                    }
                    catch
                    {
                        try
                        {
                            templateRow.labelSeriesColor.BackColor = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), trendSetting.SeriesInteriorColor));
                        }
                        catch { }
                    }
                    UserControls.UserControlTrendTemplates.SelectItem(templateRow.comboBoxSeriesStyle, trendSetting.SeriesType.ToString());
                    try
                    {
                        templateRow.labelSymbolColor.BackColor = Color.FromArgb(int.Parse(trendSetting.SymbolColor));
                    }
                    catch
                    {
                        try
                        {
                            templateRow.labelSymbolColor.BackColor = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), trendSetting.SymbolColor));
                        }
                        catch { }
                    }
                    //templateRow.comboBoxSymbolColor.SelectedIndexChanged += new EventHandler(comboBoxSymbolColor_SelectedIndexChanged);
                    UserControls.UserControlTrendTemplates.SelectItem(templateRow.comboBoxSymbolShape, trendSetting.SymbolShape.ToString()); ;
                    totalHeight += templateRow.Height;
                }
            }
            else
            {
                for (int i = 1; i <= noOfRows; i++)
                {
                    TrendSetting trendSetting = _templateRows.TrendSettings[i - 1];
                    UserControlTrendTemplateRowsWithUOM templateRow = new UserControlTrendTemplateRowsWithUOM();
                    _htTemplatesRows.Add(i, templateRow);
                    templateRow.Dock = DockStyle.Top;
                    templateRow.labelID.Text = "Row " + (i).ToString() + ":";

                    templateRow.checkBoxLog.Checked = trendSetting.IsXLog;
                    templateRow.checkBoxLog.CheckedChanged += new EventHandler(checkBoxLog_CheckedChanged);
                    //UserControls.UserControlTrendTemplates.SelectItem(templateRow.comboBoxSeriesInteriorColor,trendSetting.SeriesInteriorColor.ToString());
                    try
                    {
                        templateRow.labelSeriesColor.BackColor = Color.FromArgb(int.Parse(trendSetting.SeriesInteriorColor));
                    }
                    catch
                    {
                        try
                        {
                            templateRow.labelSeriesColor.BackColor = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), trendSetting.SeriesInteriorColor));
                        }
                        catch { }
                    }
                    UserControls.UserControlTrendTemplates.SelectItem(templateRow.comboBoxSeriesStyle, trendSetting.SeriesType.ToString());
                    try
                    {
                        templateRow.labelSymbolColor.BackColor = Color.FromArgb(int.Parse(trendSetting.SymbolColor));
                    }
                    catch
                    {
                        try
                        {
                            templateRow.labelSymbolColor.BackColor = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), trendSetting.SymbolColor));
                        }
                        catch { }
                    }
                    //templateRow.comboBoxSymbolColor.SelectedIndexChanged += new EventHandler(comboBoxSymbolColor_SelectedIndexChanged);
                    UserControls.UserControlTrendTemplates.SelectItem(templateRow.comboBoxSymbolShape, trendSetting.SymbolShape.ToString()); ;
                    totalHeight += templateRow.Height;
                }
            }
			
			_pnlSettings.Controls.Clear();
			_pnlSettings.Dock = DockStyle.Top;
			
			for(int i= noOfRows;i>=1;i--)
			{
				_pnlSettings.Controls.Add(_htTemplatesRows[i] as Control);
			}

            if (_templateRows.IsXYGraph)
            {
                UserControlTrendOptionsRowsHeaders tempHeaders = new UserControlTrendOptionsRowsHeaders();

                tempHeaders.Dock = DockStyle.Top;
                totalHeight += tempHeaders.Height;
                _pnlSettings.Controls.Add(tempHeaders);
            }
            else
            {
                UserControlTrendOptionsRowsHeadersWithUOM tempHeaders = new UserControlTrendOptionsRowsHeadersWithUOM();

                tempHeaders.Dock = DockStyle.Top;
                totalHeight += tempHeaders.Height;
                _pnlSettings.Controls.Add(tempHeaders);
            }
			_pnlSettings.Height = totalHeight;
		}

		private void LoadTrends()
		{
			int noOfRows = _templateRows.TrendSettings.Length;
            if (_templateRows.IsXYGraph)
            {
                _htTrends.Clear();
                noOfRows = 1;
                UserControlTrendXY ucTrend = new UserControlTrendXY();
                ucTrend.InitTrend(_templateRows);
                ucTrend.Height = this.panelMultiTrend.Height / 2;
                ucTrend.Dock = DockStyle.Top;
                
                ucTrend.Tag = _templateRows;
                _htTrends.Add(0, ucTrend);

                TrendSetting trendSetting = _templateRows.TrendSettings[0];

                try
                {
                    if(_xUOM!=null)
                    {
                        try
                        {
                            ucTrend.comboBoxXUOM.SelectedValue = trendSetting.UOM = _xUOM;
                        }
                        catch
                        { }
                    }
                    else if (trendSetting.UOM != null)
                    {
                        try
                        {
                            ucTrend.comboBoxXUOM.SelectedValue = trendSetting.UOM;
                            _xUOM = trendSetting.UOM;
                        }
                        catch
                        {
                        }
                    }
                    
                    if (_yUOM != null)
                    {
                        try
                        {
                            ucTrend.comboBoxYUOM.SelectedValue = trendSetting.YUOM = _yUOM;
                        }
                        catch
                        {
                        }
                    }
                    else if (trendSetting.YUOM != null)
                    {
                        try
                        {
                            ucTrend.comboBoxYUOM.SelectedValue = trendSetting.YUOM;
                            _yUOM = trendSetting.YUOM;
                        }
                        catch
                        {
                        }
                    }
                }
                catch
                { }
                ucTrend.comboBoxXUOM.SelectedIndexChanged += new EventHandler(comboBoxXUOM_SelectedIndexChanged);
                ucTrend.comboBoxYUOM.SelectedIndexChanged += new EventHandler(comboBoxYUOM_SelectedIndexChanged);
            }
            else if (_templateRows.SingleGraph)
            {
                _htTrends.Clear();
                noOfRows = 1;
                UserControlMultiTrendSingleGraph ucTrend = new UserControlMultiTrendSingleGraph();
                ucTrend.InitTrend(_templateRows,_htTemplatesRows);
                ucTrend.Height = this.panelMultiTrend.Height / 2;
                ucTrend.Dock = DockStyle.Top;

                ucTrend.Tag = _templateRows;
                _htTrends.Add(0, ucTrend);
            }
            else
            {
                _htTrends.Clear();
                for (int i = noOfRows - 1; i >= 0; i--)
                {
                    string propValueID = _templateRows.TrendSettings[i].SchemeName;
                    UserControlTrendTemplateRowsWithUOM tempRow = _htTemplatesRows[i + 1] as UserControlTrendTemplateRowsWithUOM;
                    if (LRRModel.PropertyValuesObj.FindBySiteComponentPropID(propValueID) != null)
                    {
                        if (!_templateRows.TrendSettings[i].IsXLog)
                        {
                            UserControlTrend ucTrend = new UserControlTrend();

                            
                            ucTrend.InitTrend(_templateRows.TrendSettings[i],tempRow.comboBoxUOM);

                            ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                            ucTrend.Dock = DockStyle.Top;

                            ucTrend.Tag = _templateRows.TrendSettings[i];
                            _htTrends.Add(i, ucTrend);
                            if (_templateRows.TrendSettings[i].UOM != null)
                            {
                                try
                                {
                                    tempRow.comboBoxUOM.SelectedValue = _templateRows.TrendSettings[i].UOM;
                                }
                                catch
                                { }
                            }
                            tempRow.comboBoxUOM.SelectedIndexChanged +=new EventHandler(comboBoxUOM_SelectedIndexChanged);
                        }
                        else
                        {
                            UserControlTrendLog ucTrend = new UserControlTrendLog();
                            ucTrend.InitTrend(_templateRows.TrendSettings[i], tempRow.comboBoxUOM);
                            ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                            ucTrend.Dock = DockStyle.Top;

                            ucTrend.Tag = _templateRows.TrendSettings[i];
                            _htTrends.Add(i, ucTrend);
                            if (_templateRows.TrendSettings[i].UOM != null)
                            {
                                try
                                {
                                    tempRow.comboBoxUOM.SelectedValue = _templateRows.TrendSettings[i].UOM;
                                }
                                catch
                                { }
                            }
                            tempRow.comboBoxUOM.SelectedIndexChanged += new EventHandler(comboBoxUOM_SelectedIndexChanged);
                        }
                    }
                    else
                    {
                        if (!_templateRows.TrendSettings[i].IsXLog)
                        {
                            UserControlGroupedTrend ucTrend = new UserControlGroupedTrend();
                           
                            (ucTrend as IGroupedTrend).InitTrend(_templateRows.TrendSettings[i], tempRow.comboBoxUOM);
                            ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                            ucTrend.Dock = DockStyle.Top;

                            ucTrend.Tag = _templateRows.TrendSettings[i];
                            _htTrends.Add(i, ucTrend);
                            if (_templateRows.TrendSettings[i].UOM != null)
                            {
                                try
                                {
                                    tempRow.comboBoxUOM.SelectedValue = _templateRows.TrendSettings[i].UOM;
                                }
                                catch
                                { }
                            }
                            tempRow.comboBoxUOM.SelectedIndexChanged += new EventHandler(comboBoxUOM_SelectedIndexChanged);
                        }
                        else
                        {
                            UserControlGroupedTrendLog ucTrend = new UserControlGroupedTrendLog();
                            (ucTrend as IGroupedTrend).InitTrend(_templateRows.TrendSettings[i], tempRow.comboBoxUOM);
                            ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                            ucTrend.Dock = DockStyle.Top;
                            ucTrend.Tag = _templateRows.TrendSettings[i];
                            _htTrends.Add(i, ucTrend);
                            if (_templateRows.TrendSettings[i].UOM != null)
                            {
                                try
                                {
                                    tempRow.comboBoxUOM.SelectedValue = _templateRows.TrendSettings[i].UOM;
                                }
                                catch
                                { }
                            }
                            tempRow.comboBoxUOM.SelectedIndexChanged += new EventHandler(comboBoxUOM_SelectedIndexChanged);
                        }
                    }
                }
            }
            Control[] ctls = new Control[noOfRows];
            for (int j = 0; j < noOfRows; j++)
            {
                ctls[j] = _htTrends[j] as Control;
            }
            this.panelMultiTrend.SuspendLayout();
            this.panelMultiTrend.Controls.Clear();
            panelMultiTrend.Controls.Add(_pnlSettings);

            for (int j = noOfRows - 1; j >= 0; j--)
            {
                this.panelMultiTrend.Controls.Add(ctls[j]);
            }
            this.panelMultiTrend.ResumeLayout();
            MultiTrend_Resize(null, null);
		}

        private void ReplaceControl(Control ctl, int index)
        {
            int noOfRows = this.panelMultiTrend.Controls.Count;

            Control[] ctls = new Control[noOfRows];
            for (int j = 0; j < noOfRows; j++)
            {
                ctls[j] = this.panelMultiTrend.Controls[j];
            }
            ctls[noOfRows - index] = ctl;

            this.panelMultiTrend.Controls.Clear();

            for (int j = 0; j < noOfRows; j++)
            {
                this.panelMultiTrend.Controls.Add(ctls[j]);
            }
        }
		#endregion

		#region Events

        void comboBoxYUOM_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                ComboBox combo = sender as ComboBox;
                TrendSetting trendSetting = _templateRows.TrendSettings[0];
                trendSetting.YUOM = _yUOM = combo.SelectedValue.ToString();
            }
            catch
            { }
        }
        void comboBoxUOM_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combo = sender as ComboBox;
            for (int i = 0; i < _htTemplatesRows.Count; i++)
            {
                UserControlTrendTemplateRowsWithUOM tempRow = _htTemplatesRows[i+1] as UserControlTrendTemplateRowsWithUOM;
                ComboBox combo2 = tempRow.comboBoxUOM;
                if (combo == combo2)
                {
                    TrendSetting trendSetting = _templateRows.TrendSettings[i];
                    trendSetting.UOM = combo.SelectedValue.ToString();
                    break;
                }
            }
        }
        void comboBoxXUOM_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                ComboBox combo = sender as ComboBox;
                TrendSetting trendSetting = _templateRows.TrendSettings[0];
                trendSetting.UOM = _xUOM = combo.SelectedValue.ToString();
                
            }
            catch
            { 
            }
        }

        //void comboBoxSymbolColor_SelectedIndexChanged(object sender, EventArgs e)
        //{
        //    ComboBox cmbSymbolColor = sender as ComboBox;

        //    int noOfRows = _htTemplatesRows.Count;

        //    for (int i = 1; i < (noOfRows + 1); i++)
        //    {
        //        UserControlTrendTemplateRows templateRow = _htTemplatesRows[i] as UserControlTrendTemplateRows;
        //        if (templateRow.comboBoxSymbolColor == cmbSymbolColor)
        //        {
        //            Control ctl = _htTrends[i - 1] as Control;
        //            if (ctl is UserControlTrend)
        //            {
        //                UserControlTrend ucTrend = ctl as UserControlTrend;
        //                ucTrend.SymbolColor = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), cmbSymbolColor.SelectedItem.ToString()));
        //            }
        //        }
        //    }
        //}

		private void MultiTrend_Resize(object sender, EventArgs e)
		{
			int noOfRows = this.panelMultiTrend.Controls.Count;
			for(int i=noOfRows-1;i>=1;i--)
			{
				Control ctl = this.panelMultiTrend.Controls[i] as Control;
				ctl.Height = (this.ClientSize.Height -_pnlSettings.Height)/(noOfRows-1);
			}
		}

        void checkBoxLog_CheckedChanged(object sender, EventArgs e)
        {
            if (!_templateRows.SingleGraph && !_templateRows.IsXYGraph)
            {
                CheckBox chkLogLinear = sender as CheckBox;

                int noOfRows = _htTemplatesRows.Count;

                if (_templateRows.IsXYGraph)
                {
                    for (int i = 1; i < (noOfRows + 1); i++)
                    {
                        UserControlTrendTemplateRows templateRow = _htTemplatesRows[i] as UserControlTrendTemplateRows;
                        if (templateRow.checkBoxLog == chkLogLinear)
                        {
                            Control ctl = _htTrends[i - 1] as Control;
                            TrendSetting trendSetting = ctl.Tag as TrendSetting;
                            if (chkLogLinear.Checked)
                            {
                                if (ctl is UserControlTrend)
                                {
                                    UserControlTrendLog ucTrend = new UserControlTrendLog();
                                    ucTrend.InitTrend(trendSetting,null);
                                    ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                                    ucTrend.Dock = DockStyle.Top;
                                    ReplaceControl(ucTrend, i);
                                    ucTrend.Tag = trendSetting;
                                    _htTrends[i - 1] = ucTrend;
                                }
                                else if (ctl is UserControlGroupedTrend)
                                {
                                    UserControlGroupedTrendLog ucTrend = new UserControlGroupedTrendLog();
                                    (ucTrend as IGroupedTrend).InitTrend(trendSetting,null);
                                    ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                                    ucTrend.Dock = DockStyle.Top;
                                    ReplaceControl(ucTrend, i);
                                    ucTrend.Tag = trendSetting;
                                    _htTrends[i - 1] = ucTrend;
                                }
                            }
                            else
                            {
                                if (ctl is UserControlTrendLog)
                                {
                                    UserControlTrend ucTrend = new UserControlTrend();
                                    ucTrend.InitTrend(trendSetting,null);
                                    ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                                    ucTrend.Dock = DockStyle.Top;
                                    ReplaceControl(ucTrend, i);
                                    ucTrend.Tag = trendSetting;
                                    _htTrends[i - 1] = ucTrend;
                                }
                                else if (ctl is UserControlGroupedTrendLog)
                                {
                                    UserControlGroupedTrend ucTrend = new UserControlGroupedTrend();
                                    (ucTrend as IGroupedTrend).InitTrend(trendSetting,null);
                                    ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                                    ucTrend.Dock = DockStyle.Top;
                                    ReplaceControl(ucTrend, i);
                                    ucTrend.Tag = trendSetting;
                                    _htTrends[i - 1] = ucTrend;
                                }
                            }
                        }
                    }
                }
                else
                {
                    for (int i = 1; i < (noOfRows + 1); i++)
                    {
                        UserControlTrendTemplateRowsWithUOM templateRow = _htTemplatesRows[i] as UserControlTrendTemplateRowsWithUOM;
                        if (templateRow.checkBoxLog == chkLogLinear)
                        {
                            Control ctl = _htTrends[i - 1] as Control;
                            TrendSetting trendSetting = ctl.Tag as TrendSetting;
                            if (chkLogLinear.Checked)
                            {
                                if (ctl is UserControlTrend)
                                {
                                    UserControlTrendLog ucTrend = new UserControlTrendLog();
                                    ucTrend.InitTrend(trendSetting,templateRow.comboBoxUOM);
                                    ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                                    ucTrend.Dock = DockStyle.Top;
                                    ReplaceControl(ucTrend, i);
                                    ucTrend.Tag = trendSetting;
                                    _htTrends[i - 1] = ucTrend;
                                }
                                else if (ctl is UserControlGroupedTrend)
                                {
                                    UserControlGroupedTrendLog ucTrend = new UserControlGroupedTrendLog();
                                    (ucTrend as IGroupedTrend).InitTrend(trendSetting, templateRow.comboBoxUOM);
                                    ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                                    ucTrend.Dock = DockStyle.Top;
                                    ReplaceControl(ucTrend, i);
                                    ucTrend.Tag = trendSetting;
                                    _htTrends[i - 1] = ucTrend;
                                }
                            }
                            else
                            {
                                if (ctl is UserControlTrendLog)
                                {
                                    UserControlTrend ucTrend = new UserControlTrend();
                                    ucTrend.InitTrend(trendSetting, templateRow.comboBoxUOM);
                                    ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                                    ucTrend.Dock = DockStyle.Top;
                                    ReplaceControl(ucTrend, i);
                                    ucTrend.Tag = trendSetting;
                                    _htTrends[i - 1] = ucTrend;
                                }
                                else if (ctl is UserControlGroupedTrendLog)
                                {
                                    UserControlGroupedTrend ucTrend = new UserControlGroupedTrend();
                                    (ucTrend as IGroupedTrend).InitTrend(trendSetting, templateRow.comboBoxUOM);
                                    ucTrend.Height = this.panelMultiTrend.Height / (noOfRows + 1);
                                    ucTrend.Dock = DockStyle.Top;
                                    ReplaceControl(ucTrend, i);
                                    ucTrend.Tag = trendSetting;
                                    _htTrends[i - 1] = ucTrend;
                                }
                            }
                        }
                    }
                }
            }
        }

        
        private void MultiTrend_Load(object sender, EventArgs e)
        {
            
        }

		
		private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
		{
			RefreshTrend refreshTrendDel = new RefreshTrend(Refresh);
			_lrrApp.MainFrame.Invoke(refreshTrendDel,null);
		}

		#endregion

		#region ILRRUIContainer Members

		public void LeftColMovement()
		{
			// TODO:  Add MultiTrend.LeftColMovement implementation
		}

		public void ClearSort()
		{
			// TODO:  Add MultiTrend.ClearSort implementation
		}

		public void RightLevel()
		{
			// TODO:  Add MultiTrend.RightLevel implementation
		}

		public void UpLevel()
		{
			// TODO:  Add MultiTrend.UpLevel implementation
		}

		public bool SupportsImportToExcel()
		{
			// TODO:  Add MultiTrend.SupportsImportToExcel implementation
			return false;
		}

		public bool SupportsLevel()
		{
			// TODO:  Add MultiTrend.SupportsLevel implementation
			return false;
		}

		public bool PerformDirtyChk
		{
			get
			{
				// TODO:  Add MultiTrend.PerformDirtyChk getter implementation
				return false;
			}
		}

		public bool SupportsRefresh()
		{
			return true;
		}

		public void Undo()
		{
			// TODO:  Add MultiTrend.Undo implementation
		}

		public bool SupportsCopy()
		{
			// TODO:  Add MultiTrend.SupportsCopy implementation
			return false;
		}

		public void SelectionPaste()
		{
			// TODO:  Add MultiTrend.SelectionPaste implementation
		}

		public TabPage ParentTabPage
		{
			get
			{
				return _tabPage;
			}
			set
			{
				_tabPage = value;
			}
		}

		public void PageSetup()
		{
			// TODO:  Add MultiTrend.PageSetup implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add MultiTrend.OpenQuery implementation
		}

		public void Copy()
		{
			// TODO:  Add MultiTrend.Copy implementation
		}

		public bool SupportsInsertNewRows()
		{
			// TODO:  Add MultiTrend.SupportsInsertNewRows implementation
			return false;
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add MultiTrend.SupportsPageSetup implementation
			return false;
		}

		public void ShowPropertyList()
		{
			// TODO:  Add MultiTrend.ShowPropertyList implementation
		}

		public bool SupportsCompositeView()
		{
			// TODO:  Add MultiTrend.SupportsCompositeView implementation
			return false;
		}

		public bool SupportsClearSort()
		{
			// TODO:  Add MultiTrend.SupportsClearSort implementation
			return false;
		}

		public void RightColMovement()
		{
			// TODO:  Add MultiTrend.RightColMovement implementation
		}

		public void LeftLevel()
		{
			// TODO:  Add MultiTrend.LeftLevel implementation
		}

		public bool SupportsClearFilter()
		{
			// TODO:  Add MultiTrend.SupportsClearFilter implementation
			return false;
		}

		public bool SupportsDelete()
		{
			// TODO:  Add MultiTrend.SupportsDelete implementation
			return false;
		}

		public void PrintPreview()
		{
			// TODO:  Add MultiTrend.PrintPreview implementation
		}

		public bool SupportsResultsFilter()
		{
			// TODO:  Add MultiTrend.SupportsResultsFilter implementation
			return false;
		}

		public bool SupportsPaste()
		{
			// TODO:  Add MultiTrend.SupportsPaste implementation
			return false;
		}

		public void Print()
		{
			// TODO:  Add MultiTrend.Print implementation
		}

		public bool SupportsArrangeColumns()
		{
			// TODO:  Add MultiTrend.SupportsArrangeColumns implementation
			return false;
		}

		public bool SupportsPrint()
		{
			// TODO:  Add MultiTrend.SupportsPrint implementation
			return false;
		}

		public void OpenView()
		{
			// TODO:  Add MultiTrend.OpenView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add MultiTrend.TransposeView implementation
		}

		public void ArrangeColumns()
		{
			// TODO:  Add MultiTrend.ArrangeColumns implementation
		}

		public void Export()
		{
			// TODO:  Add MultiTrend.Export implementation
		}

		public bool SupportsMasterPropertiesCopy()
		{
			// TODO:  Add MultiTrend.SupportsMasterPropertiesCopy implementation
			return false;
		}

		public void ShowComponentList()
		{
			// TODO:  Add MultiTrend.ShowComponentList implementation
		}

		public void ClearFilter()
		{
			// TODO:  Add MultiTrend.ClearFilter implementation
		}

		public void OpenCompositeView()
		{
			// TODO:  Add MultiTrend.OpenCompositeView implementation
		}

		public void Delete()
		{
			// TODO:  Add MultiTrend.Delete implementation
		}

		public bool SupportsUndo()
		{
			// TODO:  Add MultiTrend.SupportsUndo implementation
			return false;
		}

		public bool SupportsExport()
		{
			// TODO:  Add MultiTrend.SupportsExport implementation
			return false;
		}

		public void CopyUOMTypes()
		{
			// TODO:  Add MultiTrend.CopyUOMTypes implementation
		}

		public void ImportToExcel()
		{
			// TODO:  Add MultiTrend.ImportToExcel implementation
		}

		public void Refresh()
		{
            LRRMultiAttributeTrendInfo trendInfo = _trendInfo;

            int noOfRows = _pnlSettings.Controls.Count - 1;
            for (int i = 0; i < noOfRows; i++)
            {
                TrendSetting trendSetting = _templateRows.TrendSettings[i];
                //trendSetting.SchemeName = (i + 1).ToString();

                if (_templateRows.IsXYGraph)
                {
                    UserControlTrendTemplateRows templateRow = _htTemplatesRows[i + 1] as UserControlTrendTemplateRows;

                    trendSetting.SeriesInteriorColor = templateRow.labelSeriesColor.BackColor.ToArgb().ToString();
                    trendSetting.SeriesType = templateRow.comboBoxSeriesStyle.SelectedItem.ToString();
                    trendSetting.SymbolColor = templateRow.labelSymbolColor.BackColor.ToArgb().ToString();
                    trendSetting.SymbolShape = templateRow.comboBoxSymbolShape.SelectedItem.ToString();
                    trendSetting.IsXLog = templateRow.checkBoxLog.Checked;
                }
                else
                {
                    UserControlTrendTemplateRowsWithUOM templateRow = _htTemplatesRows[i + 1] as UserControlTrendTemplateRowsWithUOM;

                    trendSetting.SeriesInteriorColor = templateRow.labelSeriesColor.BackColor.ToArgb().ToString();
                    trendSetting.SeriesType = templateRow.comboBoxSeriesStyle.SelectedItem.ToString();
                    trendSetting.SymbolColor = templateRow.labelSymbolColor.BackColor.ToArgb().ToString();
                    trendSetting.SymbolShape = templateRow.comboBoxSymbolShape.SelectedItem.ToString();
                    trendSetting.IsXLog = templateRow.checkBoxLog.Checked;
                }
            }

			LoadTrends();
		}

		public bool SupportsSelectionPaste()
		{
			// TODO:  Add MultiTrend.SupportsSelectionPaste implementation
			return false;
		}

		public void DownLevel()
		{
			// TODO:  Add MultiTrend.DownLevel implementation
		}

		public void Paste()
		{
			// TODO:  Add MultiTrend.Paste implementation
		}

		public bool SupportsSave()
		{
			return true;
		}

		public void InsertNewRows()
		{
			// TODO:  Add MultiTrend.InsertNewRows implementation
		}

		public void CopyMasterProperties()
		{
			// TODO:  Add MultiTrend.CopyMasterProperties implementation
		}

		public void Save()
		{
			LRRMultiAttributeTrendInfo trendInfo = _trendInfo;

			int noOfRows = _pnlSettings.Controls.Count -1;
			for(int i=0;i<noOfRows;i++)
			{
				TrendSetting trendSetting = _templateRows.TrendSettings[i];
				//trendSetting.SchemeName = (i + 1).ToString();

                if (_templateRows.IsXYGraph)
                {
                    UserControlTrendTemplateRows templateRow = _htTemplatesRows[i + 1] as UserControlTrendTemplateRows;

                    trendSetting.SeriesInteriorColor = templateRow.labelSeriesColor.BackColor.ToArgb().ToString();
                    trendSetting.SeriesType = templateRow.comboBoxSeriesStyle.SelectedItem.ToString();
                    trendSetting.SymbolColor = templateRow.labelSymbolColor.BackColor.ToArgb().ToString();
                    trendSetting.SymbolShape = templateRow.comboBoxSymbolShape.SelectedItem.ToString();
                    trendSetting.IsXLog = templateRow.checkBoxLog.Checked;
                }
                else
                {
                    UserControlTrendTemplateRowsWithUOM templateRow = _htTemplatesRows[i + 1] as UserControlTrendTemplateRowsWithUOM;

                    trendSetting.SeriesInteriorColor = templateRow.labelSeriesColor.BackColor.ToArgb().ToString();
                    trendSetting.SeriesType = templateRow.comboBoxSeriesStyle.SelectedItem.ToString();
                    trendSetting.SymbolColor = templateRow.labelSymbolColor.BackColor.ToArgb().ToString();
                    trendSetting.SymbolShape = templateRow.comboBoxSymbolShape.SelectedItem.ToString();
                    trendSetting.IsXLog = templateRow.checkBoxLog.Checked;
                }
			}

			MemoryStream memoryStream = new MemoryStream();
			XmlSerializer xs = new XmlSerializer(typeof(TemplateRows));
			XmlTextWriter xmlTextWriter = new XmlTextWriter(memoryStream, Encoding.UTF8);

			xs.Serialize(xmlTextWriter, _templateRows);
			memoryStream = (MemoryStream)xmlTextWriter.BaseStream;
			byte[] buffer = memoryStream.ToArray();

			trendInfo.TrendInfo = buffer;

			
			trendInfo.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
			_lrrApp.DBManagerObj.CommitObject(trendInfo);
			trendInfo.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
			
			_lrrApp.DBManagerObj.CommitObject(trendInfo);
            LoadTrends();
		}

		public TabControl ParentTabControl
		{
			get
			{
				return _parentTabControl;
			}
			set
			{
				_parentTabControl = value;
			}
		}

		public bool SupportsCopyUOMTypes()
		{
			// TODO:  Add MultiTrend.SupportsCopyUOMTypes implementation
			return false;
		}

		#endregion

       
	}
}
