using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Tools.XPMenus;

using Symcon.Database;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using Symcon.Utils.HeaderDispatcher;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms;



namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for PlantItemSingleView.
	/// </summary>
	public class PlantItemSingleView : System.Windows.Forms.Form
	{
		#region Private Fields

			private System.ComponentModel.IContainer								components;
			private LRRSiteComponents												_siteComponents; //Instance for the site components 
			private LRRSiteComponentProperties										_siteCompProperties; //Instance for the site components properties
			private LRRModes														_modes; //Instance for the modes
			private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl		plantItemSingleViewGrid; //Instance for the properties...
			private LRRApp															_app;
			bool needRefresh = false;
			
		
		#endregion

		#region Public Methods
			public PlantItemSingleView()
			{
				//
				// Required for Windows Form Designer support
				//
				InitializeComponent();
				_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
				InitDataSource();
				
			}
		#endregion

		#region Private Methods

			/// <summary>
			/// This Method initializes the data source.
			/// </summary>
			private void InitDataSource()
			{
				_siteComponents  = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRSiteComponents)) as LRRSiteComponents;
				_siteCompProperties = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRSiteComponentProperties)) as LRRSiteComponentProperties;
				_modes = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRModes)) as LRRModes;

			}

			private void InitializePlantItemSingleViewGrid()
			{
				
				SyncGridUtilities.InitializeGridLookAndFeel(plantItemSingleViewGrid);
				
				
				plantItemSingleViewGrid.TableControl.CurrentCellActivated += new EventHandler(TableControl_CurrentCellActivated);
				plantItemSingleViewGrid.ShowGroupDropArea = true;

				
				//Set the relation, so that we get all the component properties as a child table
				// for the corresponding component.
				//Register all the collection bases which will participate in Parent Child Relationship
							
				plantItemSingleViewGrid.Engine.SourceListSet.Add("	",_siteComponents);
				plantItemSingleViewGrid.Engine.SourceListSet.Add("ComponentProperties",_siteCompProperties);
				//plantItemSingleViewGrid.Engine.SourceListSet.Add("Modes",_modes);
				
				//The following codes define the relations and adds to the parent table
				GridRelationDescriptor parentToChildRelationDescriptor = new GridRelationDescriptor();
				parentToChildRelationDescriptor.ChildTableName = "ComponentProperties"; 		
				parentToChildRelationDescriptor.RelationKeys.Add("SiteComponentID", "ComponentID");
				parentToChildRelationDescriptor.RelationKind = RelationKind.RelatedMasterDetails;
				
				parentToChildRelationDescriptor.ChildTableDescriptor.Appearance.GroupCaptionCell.BackColor = Color.DarkGoldenrod;
				parentToChildRelationDescriptor.ChildTableDescriptor.Appearance.GroupCaptionCell.TextColor = Color.White;
				parentToChildRelationDescriptor.ChildTableDescriptor.TopLevelGroupOptions.CaptionText = LRRHeaderText.GetHeaderString("SID_PROP") +" {RecordCount} "+ LRRHeaderText.GetHeaderString("SID_ITEMS");  
				plantItemSingleViewGrid.ShowRelationFields = ShowRelationFields.ShowDisplayFieldsOnly;
				
				SyncGridUtilities.InitializeGridColumns(plantItemSingleViewGrid,_siteComponents.ItemType,null);

				// Add relation to ParentTable 
				plantItemSingleViewGrid.TableDescriptor.Relations.Add(parentToChildRelationDescriptor);

				plantItemSingleViewGrid.NestedTableGroupOptions.ShowFilterBar = true;
				//gridSiteComponent.Appearance.NestedTableRowHeaderCell.BackColor = Color.Red;
				plantItemSingleViewGrid.NestedTableGroupOptions.ShowAddNewRecordBeforeDetails = false;
				plantItemSingleViewGrid.NestedTableGroupOptions.ShowAddNewRecordAfterDetails = true;
				plantItemSingleViewGrid.Engine.SourceListSet.Add("ComponentProperties",_siteCompProperties);

				//The following codes define the relations and adds to the parent table
//				GridRelationDescriptor ChildToSubChildRelationDescriptor = new GridRelationDescriptor();
//				ChildToSubChildRelationDescriptor.ChildTableName = "Modes"; 		
//				ChildToSubChildRelationDescriptor.RelationKind = RelationKind.RelatedMasterDetails;
//				ChildToSubChildRelationDescriptor.RelationKeys.Add("ModeID", "ModeId");
//				
//				// Add relation to ParentTable 
//				parentToChildRelationDescriptor.ChildTableDescriptor.Relations.Add(ChildToSubChildRelationDescriptor);
//				
				plantItemSingleViewGrid.DataSource = _siteComponents;
					
				// Add all the three table descriptors manually to the group area.
				AddGroupDropAreas(plantItemSingleViewGrid.Table);
				SetTableAsReadOnly(plantItemSingleViewGrid.Table);

				//plantItemSingleViewGrid.GetTableControl("Modes").CurrentCellActivated += new EventHandler(TableControl_CurrentCellActivated);
				
				//plantItemSingleViewGrid.GetTableDescriptor("Modes").Columns.Remove("IsSelected");
				//plantItemSingleViewGrid.GetTableDescriptor("ComponentProperties").Columns.Remove("IsSelected");
				
				//plantItemSingleViewGrid.FilterRuntimeProperties = true;
		

//				plantItemSingleViewGrid.PropertyChanged += new DescriptorPropertyChangedEventHandler(grid_PropertyChanged);
//				plantItemSingleViewGrid.TableDescriptor.PropertyChanged += new DescriptorPropertyChangedEventHandler(TableDescriptor_PropertyChanged);
//				plantItemSingleViewGrid.Engine.SourceListChanged += new EventHandler(Engine_SourceListChanged);
				
				
			}
		

			private void grid_PropertyChanged(object sender, DescriptorPropertyChangedEventArgs e)
			{
//				GridGroupingControl grid = sender as GridGroupingControl;
//				if (grid != null)
//					grid.CurrentCell.ScrollInView();
//				
//				//RefreshGroupGrid(sender);
								
				
			}

			private void TableDescriptor_PropertyChanged(object sender, DescriptorPropertyChangedEventArgs e)
			{
				RefreshGroupGrid(sender);
			}

			private void Engine_SourceListChanged(object sender, EventArgs e)
			{
				RefreshGroupGrid(sender);
			}

			private void RefreshGroupGrid(object sender)
			{
				if (plantItemSingleViewGrid != null && plantItemSingleViewGrid.Parent != null)
				{
					plantItemSingleViewGrid.Refresh();
				}
				//.ScrollInView();
			}

			private void AddGroupDropAreas(GridTable table)
			{
				Syncfusion.Grouping.RelationDescriptor rd = table.TableDescriptor.ParentRelation;
				if (rd != null && rd.RelationKind != Syncfusion.Grouping.RelationKind.RelatedMasterDetails)
					return;

				foreach(GridTable tbl in table.RelatedTables)
				{
					plantItemSingleViewGrid.AddGroupDropArea(tbl);
					AddGroupDropAreas(tbl);
					SetTableAsReadOnly(tbl);
				}
			}


			private void SetTableAsReadOnly(GridTable table)
			{
				table.TableDescriptor.AllowNew = false;
				table.TableDescriptor.AllowEdit = false;
				table.TableDescriptor.AllowRemove = false;

				
			}


			private void TableControl_CurrentCellActivated(object sender, EventArgs e)
			{
				GridControlBase grid = sender as GridControlBase;
				if (grid != null)
					grid.CurrentCell.ScrollInView();
			}
		#endregion

		#region Protected Methods
			/// <summary>
			/// Clean up any resources being used.
			/// </summary>
			protected override void Dispose( bool disposing )
			{
				if( disposing )
				{
					if(components != null)
					{
						components.Dispose();
					}
				}
				base.Dispose( disposing );
			}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(PlantItemSingleView));
			this.plantItemSingleViewGrid = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			((System.ComponentModel.ISupportInitialize)(this.plantItemSingleViewGrid)).BeginInit();
			this.SuspendLayout();
			// 
			// plantItemSingleViewGrid
			// 
			this.plantItemSingleViewGrid.AllowDrop = true;
			this.plantItemSingleViewGrid.BackColor = System.Drawing.SystemColors.Window;
			this.plantItemSingleViewGrid.Location = new System.Drawing.Point(16, 24);
			this.plantItemSingleViewGrid.Name = "plantItemSingleViewGrid";
			this.plantItemSingleViewGrid.Size = new System.Drawing.Size(936, 700);
			this.plantItemSingleViewGrid.TabIndex = 0;
			this.plantItemSingleViewGrid.Text = "plantItemSingleViewGrid";
			this.plantItemSingleViewGrid.VersionInfo = "3.301.0.0";
			// 
			// PlantItemSingleView
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(968, 742);
			this.Controls.Add(this.plantItemSingleViewGrid);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "PlantItemSingleView";
			this.Tag = "SID_PLANTSINGLEVIEW";
			this.Load += new System.EventHandler(this.PlantItemSingleView_Load);
			((System.ComponentModel.ISupportInitialize)(this.plantItemSingleViewGrid)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void PlantItemSingleView_Load(object sender, System.EventArgs e)
		{
			InitializePlantItemSingleViewGrid();
		}
	}
}
