using System;
using System.Windows.Forms;
using System.Drawing;
using Symcon.Utils.MessageLogger;

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// 
	/// </summary>
	public class PrintManager
	{
		private PrintData printData;
		private System.Drawing.Printing.PrintDocument docToPrint;
		private System.Windows.Forms.PrintDialog printDialog = null;
//		private CustomPrintController printController = null;
		private Form parent = null;
		int rowPrintCount = 0;

		public PrintManager(Form Parent, PrintData pd)
		{
			this.printDialog = new System.Windows.Forms.PrintDialog();
        	this.printData = pd;
			this.docToPrint = new System.Drawing.Printing.PrintDocument();
			parent = Parent;
			// 
			// TODO: Add constructor logic here
			//
		}

		public void Print()
		{
			printDialog.AllowSomePages = false;
			// Show the help button.
			printDialog.ShowHelp = false;
			printDialog.Document = docToPrint;
			
			try
			{
				DialogResult result = printDialog.ShowDialog();

				// If the result is OK then print the document.
				if (result == DialogResult.OK)
				{
					rowPrintCount = 0;
					//				printController = new CustomPrintController(parent, parent.Text);
					//				docToPrint.PrintController = printController;
					docToPrint.PrintPage += new System.Drawing.Printing.PrintPageEventHandler
						(this.document_PrintPage);

					docToPrint.Print();						
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(PrintManager),"Print:"+ex.Message  ,Logger.Level.Debug);
				return;
			}

		}

		private void document_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs ev) 
		{
			int count = 0;
			float linesPerPage = 0;
			float leftMargin = ev.MarginBounds.Left;
			float topMargin = ev.MarginBounds.Top;
			float rightMargin = ev.MarginBounds.Right;
			float yPos = topMargin;
			float headerHeight = 0;
			float maxValueWidth = 0;
		
			if( (printData == null) || (printData.PrintDataTableObj == null) )
				return;

			// first draw the header of the document.
			ev.Graphics.DrawString(printData.HeaderName, printData.HeaderPrintFont, Brushes.Black, leftMargin, yPos, new StringFormat());
			
			// calculate the header height needed for each page
			headerHeight = printData.HeaderPrintFont.GetHeight(ev.Graphics) + 20;
			
			PrintRow[] rows = printData.PrintDataTableObj.Rows;

			if(rows.Length > 0)
			{
				// Calculate the number of lines per page and maximum width for the value 
				// (2nd column)
				linesPerPage = ( ev.MarginBounds.Height - headerHeight )/ rows[0].LabelPrintFont.GetHeight(ev.Graphics);
				maxValueWidth = ( rightMargin - leftMargin - rows[0].LabelWidth - 15.0f);				
			}

			yPos = topMargin + headerHeight;
			
			// Traverse thru each row for printing. Here drawing is done twice for each 
			// row(label and value)
			while(rowPrintCount < rows.Length)
			{
				PrintRow row = rows[rowPrintCount];

				// draw the label first as it is of fixed width.
				ev.Graphics.DrawString( row.LabelName, row.LabelPrintFont, Brushes.Black, leftMargin, yPos );
				
				// calculate the max size of the "value". 
				// If it exceeds the right margin, calculate the extra lines.
				SizeF sizeF = ev.Graphics.MeasureString(row.ValueName, row.ValuePrintFont );
				int extraLineNo = (int)( sizeF.Width / maxValueWidth );
				
				// if line width is more than right margin, draw the extra lines down.
				if(extraLineNo > 0)
				{
					count += extraLineNo;
					float rectHeight = (extraLineNo+1)*row.ValuePrintFont.GetHeight(ev.Graphics);
					RectangleF rectF = new RectangleF( leftMargin , yPos, maxValueWidth + 25.0f, rectHeight);
					ev.Graphics.DrawString( row.ValueName, row.ValuePrintFont, Brushes.Black, rectF );
				}
				else
				{
					ev.Graphics.DrawString(row.ValueName, row.ValuePrintFont, Brushes.Black, leftMargin , yPos);
				}

				rowPrintCount++;
				count++;
				
				// Calculate the Y position of the page after drawing a row.
				yPos = topMargin + headerHeight + (count * row.LabelPrintFont.GetHeight(ev.Graphics));
				
				// If more lines exist, print another page.
				if( count > linesPerPage )
				{
					ev.HasMorePages = true;
					break;
				}
				else
					ev.HasMorePages = false;
			}			

		}

	}
}
