using System;
using System.Collections;
using System.ComponentModel;
using System.Text;

using Syncfusion.Collections;
using Syncfusion.Collections.BinaryTree;
using Syncfusion.Diagnostics;
using Syncfusion.Grouping;

using Symcon.LightRidge.BusinessObjects;

using ISummary = Syncfusion.Collections.BinaryTree.ITreeTableSummary;

namespace Symcon.LightRidge.PEAdvisor
{

		/// <summary>
		/// Simple summary example
		/// </summary>
		public sealed class PropertyNameSummary : SummaryBase
		{
			string _name;

			//public static readonly PropertyNameSummary Empty = new PropertyNameSummary(0);
		
			/// <summary>
			/// Assign this CreateSummaryDelegate handler method to SummaryDescriptor.CreateSummaryMethod 
			/// </summary>
			/// <param name="sd"></param>
			/// <param name="record"></param>
			/// <returns></returns>
			public static ISummary CreateSummaryMethod(SummaryDescriptor sd, Record record)
			{
				if(record != null && record.Kind != DisplayElementKind.AddNewRecord)
				{
					object obj = record.GetData();
					LRRHistPropCompPropMapping histPropCompPropMapping = obj as LRRHistPropCompPropMapping;
					string propValID = histPropCompPropMapping.SiteComponentPropValID;
					string name = "";
					if(propValID != null && propValID.Trim() != "")
					{
						string propID = histPropCompPropMapping.SiteComponentPropID;
						LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(propID);
						name = siteCompProp.Name;
					}
					else
					{
						LRRSiteComponentGroupProperty siteCompGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(histPropCompPropMapping.GroupPropertyID);
						LRRSiteGroupPropertyQualifierValue qualifierObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(histPropCompPropMapping.GroupPropertyValID);
						if(qualifierObj.QualifierID != null && qualifierObj.QualifierID != "")
						{
							string sQualifierName = LRRModel.Qualifiers.getQualifierObjectForQualifierID(qualifierObj.QualifierID).Name;

							LRRSiteComponentProperty siteCompProp =  LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompGrpProp.SiteComponentPropID);
							name = siteCompProp.Name + " / " + siteCompGrpProp.AttributeName + " / " + sQualifierName + " [ Grouped ]";
						}
					}
					return new PropertyNameSummary(name);
				}
				else
					return new PropertyNameSummary("");
			}

			/// <summary>
			/// Initializes a new summary object.
			/// </summary>
			/// <param name="total"></param>
			public PropertyNameSummary(string name)
			{
				_name= name;
			}

			/// <summary>
			/// The running total of this summary
			/// </summary>
			public string Name
			{
				get
				{
					return _name;
				}
				set
				{
					_name = value;
				}
			}

			/// <summary>
			/// Combines this summary information with another objects summary and returns a new object.  
			/// </summary>
			/// <param name="other"></param>
			/// <returns></returns>
			/// <remarks>
			/// This method must honor the immutable characteristics of summary objects and return 
			/// a new summary object instead of modifying an existing summary object.
			/// </remarks>
			public override SummaryBase Combine(SummaryBase other)
			{
				return Combine((PropertyNameSummary) other);
			}

			/// <summary>
			/// Combines this summary information with another objects summary and returns a new object.  
			/// </summary>
			/// <param name="other"></param>
			/// <returns></returns>
			/// <remarks>
			/// This method must honor the immutable characteristics of summary objects and return 
			/// a new summary object instead of modifying an existing summary object.
			/// </remarks>
			public PropertyNameSummary Combine(PropertyNameSummary other)
			{
				// Summary objects are immutable. That means properties cannot be modified for an 
				// existing object. Instead every time a change is made a new object must be created (just like 
				// System.String). 
				//
				// This allows following optimization: return existing summary object if either one of the values is 0. -->
				if (other.Name == "")
					return this;
				else if (this.Name == "")
					return other;
					// <-- end of optimization
				else
					return new PropertyNameSummary(this.Name);
			}

			/// <override/>
			public override string ToString()
			{
				return String.Format("Attribute :- {0}", Name);
			}
		}
}
