using System;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.Database;

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for RecordingValuesManager.
	/// </summary>
	public class RecordingValuesManager
	{

		public RecordingValuesManager()
		{	
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp; //Get the App Object..
		}

		#region Methods
		/// <summary>
		/// Method loads the Recording Values DB Info
		/// </summary>
		public void Initialize()
		{
			DBInfo dbInfo =  _lrrApp.GetOptions().RecordingValuesConnectionObj.GetActiveRecordingValuesDBInfo();
			if(dbInfo != null)
			{
                if (dbInfo.DatabaseType == Symcon.Globals.Enums.DBType.ACCESS)
                {
                    bool fileExists = System.IO.File.Exists(dbInfo.DatabaseFilePath);
                    if (!fileExists)
                    {
                        return;
                    }
                }
				try
				{
					this.RecordingValuesDBManagerObj = new DBManager(dbInfo);
				}
				catch
				{
					return;
				}
				bool test = this.RecordingValuesDBManagerObj.TestConnection(dbInfo);
				if(test)
				{
					//Enable Export recorded values
					_lrrApp.MainFrame.SetExportRecordedValues(true);
				}
				else
				{
					//Disable Export recorded values
					_lrrApp.MainFrame.SetExportRecordedValues(false);
				}

                if (test)
                {
                    _lrrApp.MainFrame.SetRunSetManager(true);
                }
			}
			else
			{
				//Disable Export recorded values
				_lrrApp.MainFrame.SetExportRecordedValues(false);
			}
		}
		
		/// <summary>
		/// Method commits the collection
		/// </summary>
		/// <param name="objCollection"></param>
		public void CommitCollection(object objCollection)
		{
			this.RecordingValuesDBManagerObj.CommitCollection(objCollection);
		}
		
		/// <summary>
		/// Method Commits the object
		/// </summary>
		/// <param name="dataObject"></param>
		public void CommitObject(object dataObject)
		{
			this.RecordingValuesDBManagerObj.CommitObject(dataObject);
		}

		/// <summary>
		/// Method Executes Batch Insert
		/// </summary>
		/// <param name="objCollection"></param>
		public void ExecuteBatchInsert(object objCollection)
		{
			this.RecordingValuesDBManagerObj.ExecuteBatchInsert(objCollection);
		}

		/// <summary>
		/// Method Executes Batch Update
		/// </summary>
		/// <param name="objCollection"></param>
		public void ExecuteBatchUpdate(object objCollection)
		{
			this.RecordingValuesDBManagerObj.ExecuteBatchUpdate(objCollection);
		}

		/// <summary>
		/// Method Executes Batch Delete
		/// </summary>
		/// <param name="objCollection"></param>
		public void ExecuteBatchDelete(object objCollection)
		{
			this.RecordingValuesDBManagerObj.ExecuteBatchDelete(objCollection);
		}

		/// <summary>
		/// Begins the Transaction
		/// </summary>
		public void BeginTransaction()
		{
			this.RecordingValuesDBManagerObj.BeginTransaction();
		}

		/// <summary>
		/// Commit the Transaction
		/// </summary>
		public void CommitTransaction()
		{
			this.RecordingValuesDBManagerObj.CommitTransaction();
		}


		
		/// <summary>
		/// Method Populates the collection
		/// </summary>
		/// <param name="objCollection"></param>
		/// <returns></returns>
		public object PopulateCollection(object objCollection)
		{
            if (this.RecordingValuesDBManagerObj != null)
            {
                return this.RecordingValuesDBManagerObj.PopulateCollection(objCollection);
            }
            else
            {
                return null;
            }
		}

		/// <summary>
		/// Method Populates the collection
		/// </summary>
		/// <param name="objCollection"></param>
		/// <returns></returns>
		public object PopulateCollection(object objCollection,string whereClause)
		{
            if (this.RecordingValuesDBManagerObj != null)
            {
                return this.RecordingValuesDBManagerObj.PopulateCollection(objCollection, whereClause);
            }
            else
            {
                return null;
            }
			//return this.RecordingValuesDBManagerObj.PopulateCollection(objCollection,whereClause);
		}
		
		/// <summary>
		/// Method checks for RecordingValues DB
		/// </summary>
		/// <returns></returns>
		public bool ExistsRecordingDB()
		{
			if(_lrrApp.GetOptions().RecordingValuesConnectionObj != null)
			{
				DBInfo dbInfo =  _lrrApp.GetOptions().RecordingValuesConnectionObj.GetActiveRecordingValuesDBInfo();
                
				if(dbInfo != null)
				{
                    if (dbInfo.DatabaseType == Symcon.Globals.Enums.DBType.ACCESS)
                    {
                        bool fileExists = System.IO.File.Exists(dbInfo.DatabaseFilePath);
                        if (!fileExists)
                        {
                            return false;
                        }
                    }

					if(this.RecordingValuesDBManagerObj == null)
					{
						try
						{
							this.RecordingValuesDBManagerObj = new DBManager(dbInfo);
						}
						catch
						{
							return false;
						}
					}
					bool test = this.RecordingValuesDBManagerObj.TestConnection(dbInfo);
					return test;
				}
			}
			return false;
		}



		#endregion

		#region Fields
		private LRRApp _lrrApp;
		private LRRRecordingValuesConnections _recordingValuesConns = new LRRRecordingValuesConnections();
		private DBManager					  _recordingValuesDBManager;
		//Run Record Activity Details
		private string						_runRecordID;
		private string						_runRecordName;
		private string						_runRecordDesc;
		#endregion

		#region Properties
		public DBManager	RecordingValuesDBManagerObj
		{
			get{return _recordingValuesDBManager;}
			set
			{
				_recordingValuesDBManager = value;
			}
		}
		public string RunRecordID
		{
			get{return _runRecordID;}
			set
			{
				_runRecordID = value;
			}
		}
		public string RunRecordName
		{
			get{return _runRecordName;}
			set
			{
				_runRecordName = value;
			}
		}
		public string RunRecordDesc
		{
			get{return _runRecordDesc;}
			set
			{
				_runRecordDesc = value;
			}
		}
		#endregion
	}
}
