using System;
using System.Collections;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Styles;

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for ResultsList.
	/// </summary>

	public class ResultsList
	{
		public ResultsList()
		{
		}
		
		#region Properties
		public string ID
		{
			get{return _id;}
			set
			{
				_id = value;
			}
		}
		public string Name
		{
			get{return _name;}
			set
			{
				_name = value;
			}
		}

		public string UOMName
		{
			get{return _uomName;}
			set
			{
				_uomName = value;
			}
		}
		#endregion
		#region Fields
		private string _name;
		private string _uomName;
		public string _id;
		#endregion
	}

	public class Comparer : IComparer
	{

		/// <summary>
		/// Column(public property of the class) to be sorted.
		/// </summary>
		public string SortColumn
		{
			get{return sortcolumn;}set{sortcolumn = value;}
		}
		
		/// <summary>
		/// Sorting order.
		/// </summary>
		public int SortingOrder
		{
			get{return sortingOrder;}set{sortingOrder = value;}
		}
		#region Fields
		public string sortcolumn = "";
		public int sortingOrder = 0;
		#endregion

		#region Methods
		/// <summary>
		/// Compare interface implementation
		/// </summary>
		/// <param name="x">Object 1</param>
		/// <param name="y">Object 2</param>
		/// <returns>Result of comparison</returns>
		public int Compare(object x, object y)
		{
			return CheckSort(x,y,0);
		}

		public int CheckSort(object x,object y, int sortLevel)
		{
			ResultsList obj1 = (ResultsList)x;
			ResultsList obj2 = (ResultsList)y;

			//Compare based on the sorting order.
			int result = -2;
			if(sortLevel == 0)
			{
				if(sortingOrder == 0)
				{
					switch(sortcolumn)
					{
						case "Name" :
							result = obj1.Name.CompareTo(obj2.Name);
							break;
						case "UOMName":
							result = obj1.UOMName.CompareTo(obj2.UOMName);
							break;
					}
				}
				if(sortingOrder == 1)
				{
					switch(sortcolumn)
					{
						case "Name" :
							result =  obj2.Name.CompareTo(obj1.Name);
							break;
						case "UOMName":
							result = obj2.UOMName.CompareTo(obj1.UOMName);
							break;
					}
				}
			}

			return result;
		}
		#endregion
	}

	public class GridControlComparer : IComparer
	{
		
		public GridControlComparer(GridControl gridControl)
		{
			this._gridControl = gridControl;
		}
		/// <summary>
		/// Column(public property of the class) to be sorted.
		/// </summary>
		public string SortColumn
		{
			get{return sortcolumn;}set{sortcolumn = value;}
		}
		
		/// <summary>
		/// Sorting order.
		/// </summary>
		public int SortingOrder
		{
			get{return sortingOrder;}set{sortingOrder = value;}
		}

		#region Fields
		public string sortcolumn = "";
		public int sortingOrder = 0;
		private GridControl _gridControl;
		#endregion

		#region Methods
		/// <summary>
		/// Compare interface implementation
		/// </summary>
		/// <param name="x">Object 1</param>
		/// <param name="y">Object 2</param>
		/// <returns>Result of comparison</returns>
		public int Compare(object x, object y)
		{
			GridStyleInfoStore obj1 = (GridStyleInfoStore)x;
			GridStyleInfoStore obj2 = (GridStyleInfoStore)y;

			StyleInfoProperty sip1 = obj1.FindStyleInfoProperty("CellValue");
			StyleInfoProperty sip2 = obj2.FindStyleInfoProperty("CellValue");
			string s1 = obj1.GetValue(sip1) == null ? "" : obj1.GetValue(sip1).ToString();
			string s2 = obj2.GetValue(sip2) == null ? "" : obj2.GetValue(sip2).ToString();

			//Compare based on the sorting order.
			int result = s1.CompareTo(s2);
			return result;
		}


		#endregion
	}

}
