using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Diagnostics;

using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;

using Symcon.Utils.MessageLogger;
using Symcon.Database;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;
using Symcon.LightRidge.PEAdvisor.IOScreen;
using Symcon.LightRidge.PEAdvisor.Global;

namespace Symcon.LightRidge.PEAdvisor
{
    public partial class RollUpValues : Form , ILRRUIContainer 
    {
        public RollUpValues()
        {
            InitializeComponent();
            TreeViewUtility.WiredTreeView(trvLevels, null);
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitDataSource();
            if (_levels.Count > 0)
            {
                InitializeTreeView(trvLevels, _levels, _lrrLevelLinks);
                trvLevels.TreeViewNodeSorter = new TreeNodeSorter();
                trvLevels.Sort();
            }
        }

        private bool InitializeTreeView(TreeView treeView, LRRLevels levels, LRRLevelLinks levelLinks)
        {
            treeView.Nodes.Clear();
            try
            {
                Hashtable _levelHash = new Hashtable();
                Hashtable _nodesHash = new Hashtable();
                Hashtable _childParent = new Hashtable();

                foreach (LRRLevel l in levels)
                {
                    _levelHash.Add(l.LevelID, l);
                }
                if (levelLinks.Count == 0 && levels.Count == 1)
                {
                    //In here only when we have single entry on Level and No entry in Level Links
                    LRRLevel lonerLevel = _levelHash["0"] as LRRLevel;
                    if (lonerLevel != null)
                    {
                        TreeNode singleRootNode = new TreeNode(lonerLevel.Name);
                        singleRootNode.Tag = lonerLevel;
                        treeView.Nodes.Add(singleRootNode);
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
                foreach (LRRLevelLink ll in levelLinks)
                {
                    _childParent.Add(ll.ChildID, ll.ParentID);

                    if (_nodesHash.ContainsKey(ll.ParentID))
                    {
                        TreeNode foundNode = _nodesHash[ll.ParentID] as TreeNode;
                        LRRLevel foundLevel = _levelHash[ll.ChildID] as LRRLevel;
                        TreeNode childNode = new TreeNode(foundLevel.Name);
                        childNode.Tag = foundLevel;
                        foundNode.Nodes.Add(childNode);
                    }
                    else
                    {
                        LRRLevel parentLevel = _levelHash[ll.ParentID] as LRRLevel;
                        LRRLevel childLevel = _levelHash[ll.ChildID] as LRRLevel;
                        TreeNode parentNode = new TreeNode(parentLevel.Name);
                        parentNode.Tag = parentLevel;
                        TreeNode childNode = new TreeNode(childLevel.Name);
                        childNode.Tag = childLevel;
                        parentNode.Nodes.Add(childNode);
                        _nodesHash.Add(ll.ParentID, parentNode);
                    }
                }

                foreach (object k in _childParent.Keys)
                {
                    string childID = (string)k;
                    string parentID = (string)_childParent[k];
                    TreeNode childNode = null;
                    TreeNode parentNode = null;
                    if (_nodesHash.ContainsKey(childID))
                    {
                        childNode = _nodesHash[childID] as TreeNode;
                    }
                    if (_nodesHash.ContainsKey(parentID))
                    {
                        parentNode = _nodesHash[parentID] as TreeNode;
                    }
                    if ((parentNode != null) && (childNode != null))
                    {
                        int index = -1;
                        foreach (TreeNode f in parentNode.Nodes)
                        {
                            if (f.Tag == childNode.Tag)
                            {
                                index = f.Index;
                                break;
                            }
                        }
                        if (index != -1)
                            parentNode.Nodes[index] = childNode;
                    }
                }

                TreeNode rootNode = _nodesHash["0"] as TreeNode;
                rootNode.ExpandAll();
                treeView.Nodes.Add(rootNode);
                return true;
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(PlantLevels), "PopulateLevels:" + ex.Message, Logger.Level.Debug);
                return false;
            }
        }
        private void InitDataSource()
        {
            _levels = LRRModel.ChildModel.Levels;
            _rollUpValues = LRRModel.RollUpValues; 
            _lrrLevelLinks = new LRRLevelLinks();
            _lrrApp.DBManagerObj.PopulateCollection(_lrrLevelLinks);
            _rollUpTypes = LRRModel.RollUpTypes;
        }

        #region Fields
        private Hashtable gridManagerHash = new Hashtable();
        private LRRLevels _levels;
        private LRRLevelLinks _lrrLevelLinks;
        private LRRRollUpTypes _rollUpTypes;
        private LRRApp _lrrApp;
        private LRRRollUPValues _rollUpValues;
        private TreeNode prevSelNode = null;
        private TabControl _parentTabControl;
        private TabPage _parentTabPage;
        private ArrayList selectedLevels = new ArrayList();

            
        #endregion


        private void trvLevels_AfterSelect(object sender, TreeViewEventArgs e)
        {
            //if (prevSelNode != null)
            //{
            //    prevSelNode.ForeColor = Color.Black;
            //}
            ////e.Node.ForeColor = Color.Blue;
            prevSelNode = e.Node;
            LRRLevel level = prevSelNode.Tag as LRRLevel;
            LRRRollUPValues rollUpValues = _rollUpValues.GetRollUpValuesCollByLevel(level.LevelID);
        }

        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            
        }

        public void ClearFilter()
        {
            
        }

        public void ClearSort()
        {
            
        }

        public void Copy()
        {
            
        }

        public void CopyMasterProperties()
        {
            
        }

        public void CopyUOMTypes()
        {
            
        }

        public void Delete()
        {
            
        }

        public void DownLevel()
        {
            
        }

        public void Export()
        {
            IOScreenUtilities.Export(gridManagerHash);  
        }

        public void ImportToExcel()
        {
            
        }

        public void InsertNewRows()
        {
        }

        public void LeftColMovement()
        {
        }

        public void LeftLevel()
        {
        }

        public void OpenCompositeView()
        {
        }

        public void OpenQuery()
        {
        }

        public void OpenView()
        {
        }

        public void PageSetup()
        {
        }

        public TabControl ParentTabControl
        {
            set
            {
                _parentTabControl = value;
            }
            get { return _parentTabControl; }
        }

        public TabPage ParentTabPage
        {
            set
            {
                _parentTabPage = value;
            }
            get
            {
                return _parentTabPage;
            }
        }

        public void Paste()
        {
        }

        public bool PerformDirtyChk
        {
            get
            {
                // TODO:  Add MasterPlantItems.PerformDirtyChk getter implementation
                return true;
            }
        }

        public void Print()
        {
            IOScreenUtilities.Print(gridManagerHash); 
        }

        public void PrintPreview()
        {
            IOScreenUtilities.PrintPreview(gridManagerHash);
        }

        public void RightColMovement()
        {
            
        }

        public void RightLevel()
        {
            
        }

        public void Save()
        {
            LRREffectivePrices effectivePrices = LRRModel.EffectivePrices.GetChanges();
            if (effectivePrices.Count > 0)
            {
                _lrrApp.DBManagerObj.CommitCollection(effectivePrices);
                LRRModel.EffectivePrices.RefreshCollection();
                LRRMessagBox.ShowMessage("SID_SAVEDINFO", true);
            }
            else
            {
                LRRMessagBox.ShowMessage("SID_NODATASAVED", true);
            }
        }

        public void SelectionPaste()
        {
           
        }

        public void ShowComponentList()
        {
           
        }

        public void ShowPropertyList()
        {

        }

        public bool SupportsArrangeColumns()
        {
            return false;
        }

        public bool SupportsClearFilter()
        {
            return false;
        }

        public bool SupportsClearSort()
        {
            return false;
        }

        public bool SupportsCompositeView()
        {
            return false;
        }

        public bool SupportsCopy()
        {
            return false;
        }

        public bool SupportsCopyUOMTypes()
        {
            return false;
        }

        public bool SupportsDelete()
        {
            return false;
        }

        public bool SupportsExport()
        {
            return false;
        }

        public bool SupportsImportToExcel()
        {
            return false;
        }

        public bool SupportsInsertNewRows()
        {
            return false;
        }

        public bool SupportsLevel()
        {
            return false;
        }

        public bool SupportsMasterPropertiesCopy()
        {
            return false;
        }

        public bool SupportsPageSetup()
        {
            return false;
        }

        public bool SupportsPaste()
        {
            return false;
        }

        public bool SupportsPrint()
        {
            return false;
        }

        public bool SupportsRefresh()
        {
            return false;
        }

        public bool SupportsResultsFilter()
        {
            return false;
        }

        public bool SupportsSave()
        {
            return true;
        }

        public bool SupportsSelectionPaste()
        {
            return false;
        }

        public bool SupportsUndo()
        {
            return false;
        }

        public void TransposeView()
        {
        }

        public void Undo()
        {
        }

        public void UpLevel()
        {
        }
        #endregion

        private Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory iGridView;

        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                LRRWaitWindowUtility.ShowPogressIndicator(LRRHeaderText.GetHeaderString("SID_EXECUTING"));
                this.panelMain.Controls.Clear();
                int nModesCol = 0;
                foreach (LRRMode lrrMode in LRRModel.ProjectModes)//_lrrModes)
                {
                    if (lrrMode.ViewSelection)
                        nModesCol++;
                }
                if (nModesCol == 0)
                {
                    LRRMessagBox.ShowMessage("SID_SELECT_MODE", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _lrrApp.LRRUIManagerObj.ShowModesDialog();
                    return;
                }

                GridControl gridControl1 = new GridControl();
                gridControl1.ResumeLayout();
                ragisterGridEvents(gridControl1);
                gridControl1.Dock = DockStyle.Fill;
                this.panelMain.Controls.Add(gridControl1);
                IOScreen.IOScreenUtilities.InitializeGrid(gridControl1);
                //Get the Col Count
                int iGridColCount = (selectedLevels.Count * LRRModel.ProjectModes.Count) + 2;
                LRRLevels levelsCollection = new LRRLevels();
                for (int i = 0; i < selectedLevels.Count; i++)
                {
                    LRRLevel level = selectedLevels[i] as LRRLevel;
                    levelsCollection.Add(level);
                }
                //Get the Row Count
                int iGridRowCount = _rollUpTypes.Count + 2;
                gridControl1.RowCount = iGridRowCount;
                gridControl1.ColCount = iGridColCount;
                object[,] arrLevelData = new object[1, 1];
                arrLevelData[0, 0] = levelsCollection;

                //Row/Col Freeze
                gridControl1.Rows.FreezeRange(FREEZE_ROW_START, FREEZE_ROW_END);
                gridControl1.Cols.FreezeRange(FREEZE_COL_START, FREEZE_COL_END);

                GridControlManager gridManager = new GridControlManager();
                gridManager.GridData = arrLevelData;
                gridManagerHash.Add(gridControl1, gridManager);

                //Create Row Header,Col Header Data Objects
                ArrayList rowSortManagerList = new ArrayList();
                ArrayList colHeaderCellList = new ArrayList();
                Hashtable RowHeader = new Hashtable();
                Hashtable ColHeader = new Hashtable();

                //Assign to GridControlManager
                gridManager.SortManagerList = rowSortManagerList;
                gridManager.ColHeaderCellList = colHeaderCellList;
                gridManager.RowHeader = RowHeader;
                gridManager.ColHeader = ColHeader;

                ClearHeaders(gridManager);

                int gridLastRowNum = 3;
                int gridLastColNum = 3;

                CreateHeader(gridControl1, gridManager, ref gridLastRowNum, ref gridLastColNum);

                HideDefauleModeCol(gridControl1, gridManager, ref gridLastRowNum, ref gridLastColNum);

                PopulteRollUpValues(gridControl1, gridManager, ref gridLastRowNum, ref gridLastColNum);
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(LRRRollUPValues), "ExecuteClick: " + ex, Logger.Level.Debug);
            }
            finally
            {
                LRRWaitWindowUtility.EndProgressIndicator();
            }
        }

        private void ragisterGridEvents(GridControl grid)
        {
            grid.CurrentCellCloseDropDown += new PopupClosedEventHandler(grid_CurrentCellCloseDropDown);
            grid.CurrentCellValidating += new CancelEventHandler(grid_CurrentCellValidating);
            grid.CurrentCellStartEditing += new CancelEventHandler(grid_CurrentCellStartEditing); 
        }

        void grid_CurrentCellStartEditing(object sender, CancelEventArgs e)
        {
            GridControl gridControl = sender as GridControl;
            int iRow = gridControl.CurrentCell.RowIndex;
            int iCol = gridControl.CurrentCell.ColIndex;
            GridControlManager gridControlManager = gridManagerHash[gridControl] as GridControlManager;
            if (gridControlManager == null)
                return;
            if (iCol != UOM_COL)
            {
                e.Cancel = true;
                return;
            }
        }
        void grid_CurrentCellValidating(object sender, CancelEventArgs e)
        {
            GridControl gridControl = sender as GridControl;
            int iRow = gridControl.CurrentCell.RowIndex;
            int iCol = gridControl.CurrentCell.ColIndex;
            //Get the Grid Data
            GridControlManager gridControlManager = gridManagerHash[gridControl] as GridControlManager;
            if (gridControlManager == null)
                return;
            //Check for UOM
            if (iRow > UOM_ROW && iCol == UOM_COL)
            {
                //Change the Property Values according to UOM
                changePropertyValueForUOM(gridControl, gridControlManager, iRow, iCol);
            }
        }

        private void changePropertyValueForUOM(GridControl gridControl, GridControlManager gridControlManager, int iRow, int iCol)
        {
            //Get the Row Data from Grid Data Manager
            ArrayList rowSortList = gridControlManager.SortManagerList;
            //Get the Col Data from Grid Data Manager
            ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;

            //Get the old UOM and new UOM
            string newUOM = gridControl.CurrentCell.Renderer.ControlValue == null ? "" : gridControl.CurrentCell.Renderer.ControlValue.ToString();

            //Get the row data
            IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
            string rowCellID = rowData.ID;
            string uomID     = rowData.UOMID;

            for (int iColIndex = DATA_COL_START; iColIndex <= gridControl.ColCount; iColIndex++)
            {
                //Get the column data
                ColHeaderCell colData = colHeaderCellList[iColIndex - DATA_COL_START] as ColHeaderCell;
                Type objectType = colData.ObjectType;
                string modeID = colData.ModeID;
                string colCellID = colData.ID;
                //Get the Property Object
                LRRRollUpValue rollUpValue = LRRModel.RollUpValues.GetRollUpValueObjByLevelAndRollUpTypeAndMode(colCellID, rowCellID, modeID);
                if (rollUpValue != null)
                {
                    string oldValue = gridControl.Model[iRow, iColIndex].Text;
                    string value = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(uomID, newUOM, oldValue,null);
                    gridControl.Model[iRow, iColIndex].Text = value;
                    rowData.UOMID = newUOM;
                    rowData.UOMDisplayName = LRRModel.ChildModel.GetUOMNameFromID(newUOM);
                }
            }
        }

        void grid_CurrentCellCloseDropDown(object sender, PopupClosedEventArgs e)
        {
            GridControl gridControl = sender as GridControl;
            gridControl.CurrentCell.ConfirmChanges();
            gridControl.Refresh();
        }

        private void PopulteRollUpValues(GridControl gridControl1, GridControlManager gridControlManager, ref int gridLastRowNum, ref int gridLastColNum)
        {
            //Get the Modes Count
            int iModesCount = LRRModel.ProjectModes.Count;

            //Get the Row Data from Grid Data Manager
            ArrayList rowSortList = gridControlManager.SortManagerList;

            //Get the Col Data from Grid Data Manager
            ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;

            //Get the Row Header from Grid Data Manager
            Hashtable RowHeader = gridControlManager.RowHeader;

            //Get the Col Header from Grid Data Manager
            Hashtable ColHeader = gridControlManager.ColHeader;

            //Clear the Bindings
           // ClearBindings(GridControlManager);

            //Clear the cell Addresses
            //GridControlManager.PropValueCellAddress.Clear();

            for (int iRow = DATA_ROW_START; iRow <= RowHeader.Count + DATA_ROW_START - 1; iRow++)
            {
                for (int iCol = DATA_COL_START; iCol < (ColHeader.Count - DATA_COL_START + 1) * iModesCount + DATA_COL_START; iCol++)
                {
                    try
                    {
                        //Get the row data
                        IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
                        string rowCellID = rowData.ID;
                        string uiUOMID = rowData.UOMID;

                        //Get the column data
                        ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
                        Type objectType = colData.ObjectType;
                        string modeID = colData.ModeID;
                        string colCellID = colData.ID;
                        LRRRollUpValue rollUpValue = LRRModel.RollUpValues.GetRollUpValueObjByLevelAndRollUpTypeAndMode(colCellID, rowCellID, modeID);
                        if (rollUpValue != null)
                        {
                            gridControl1.Model[iRow, iCol].Text = rollUpValue.RollUpValue;
                            GridStyleInfo gsInfo = gridControl1.Model[iRow, iCol];
                            gridControlManager.BindingManager.AddBinding(rollUpValue,"RollUpValue",gsInfo,"Text");
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.LogMessage(typeof(GridSimpleViewController), "Exception in rendering SiteComponentProperty Value:- " + ex.Message, Logger.Level.Debug);
                    }
                }
               
            }

        }

        private void ClearBindings(GridControlManager GridControlManager)
        {
         
        }

        private void HideDefauleModeCol(GridControl gridControl1, GridControlManager gridManager, ref int gridLastRowNum, ref int gridLastColNum)
        {
            ArrayList rowSortManagerList = gridManager.SortManagerList;
            ArrayList colHeaderCellList = gridManager.ColHeaderCellList;
            Hashtable RowHeader = gridManager.RowHeader;
            Hashtable ColHeader = gridManager.ColHeader;
            for (int i = 0; i < colHeaderCellList.Count; i++)
            {
                ColHeaderCell colHeaderCell = colHeaderCellList[i] as ColHeaderCell;
                if (colHeaderCell.ModeID == "0")
                {
                    colHeaderCell.IsVisible = false;
                    gridControl1.ColHiddenEntries.Add(new GridColHidden(i + DATA_COL_START));
                }
            }
            int j = 0;
        }

        private void CreateHeader(GridControl gridControl1, GridControlManager gridManager, ref int gridLastRowNum, ref int gridLastColNum)
        {
            gridControl1[MODES_ROW, ROLLUP_TYPE_COL].BackColor = Color.LightGray;
            gridControl1[MODES_ROW, UOM_COL].BackColor = Color.LightGray;

            ArrayList rowSortManagerList = gridManager.SortManagerList;
            ArrayList colHeaderCellList = gridManager.ColHeaderCellList;
            Hashtable RowHeader = gridManager.RowHeader;
            Hashtable ColHeader = gridManager.ColHeader;
            int nModesCol = gridLastColNum;
            int nModes = LRRModel.ProjectModes.Count;
            string ID = "";
            string name = "";
            //RollUp Type rendering
            gridControl1[1, ROLLUP_TYPE_COL].Text = LRRHeaderText.GetHeaderString("SID_ROLL_UP_TYPE");
            gridControl1[1, ROLLUP_TYPE_COL].BackColor = Color.LightGray;
            gridControl1[1, UOM_ROW].Text = LRRHeaderText.GetHeaderString("SID_UOM");
            gridControl1[1, UOM_ROW].BackColor = Color.LightGray;
            ColHeader.Add(0, LRRHeaderText.GetHeaderString("SID_ROLL_UP_TYPE"));
            ColHeader.Add(1, LRRHeaderText.GetHeaderString("SID_UOM"));
            for (int i = 0; i < _rollUpTypes.Count; i++)
            {
                LRRRollUpType rollUpType = _rollUpTypes[i] as LRRRollUpType;
                name = rollUpType.RollUpType;
                IOSortManager sortManager = new IOSortManager();
                ID = rollUpType.ID;
                sortManager.ID = ID;
                sortManager.Name = name;
                string uomID = rollUpType.UOM;
                sortManager.UOMTypeId = rollUpType.UOMTypeID;
                sortManager.ObjectType = rollUpType.GetType();
                string UomName = "";
                if (uomID != null && uomID.Trim() != "")
                {
                    UomName = LRRModel.ChildModel.GetUOMNameFromID(uomID);
                    sortManager.UOMName = UomName;
                    sortManager.UOMID = uomID;
                }
                if (!RowHeader.Contains(ID))
                {
                    rowSortManagerList.Add(sortManager);
                }
                RowHeader.Add(ID, name);

                gridControl1[gridLastRowNum, ROLLUP_TYPE_COL].Text = name;
                gridControl1[gridLastRowNum, ROLLUP_TYPE_COL].BackColor = Color.LightGray;
                gridControl1[gridLastRowNum, ROLLUP_TYPE_COL].ReadOnly = true;
                gridControl1[gridLastRowNum, ROLLUP_TYPE_COL].CellTipText = name;
                gridControl1[gridLastRowNum, ROLLUP_TYPE_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                gridControl1[gridLastRowNum, ROLLUP_TYPE_COL].TextMargins.Left = 5;

                //UOM Col.
                LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(rollUpType.UOMTypeID);
                //Get UOM name from UOMID
                string uomName = "";
                string uomDisplayName = "";
                if (uomID != null && uomID.Trim() != "")
                {
                    uomName = LRRModel.ChildModel.GetUOMNameFromID(uomID);
                }
                else
                {
                    uomDisplayName = uomName;
                }

                try
                {
                    sortManager.UOMDisplayName = uomName;
                    gridControl1.Model[gridLastRowNum, UOM_COL].Text = uomName;
                    gridControl1.Model[gridLastRowNum, UOM_COL].CellType = "ComboBox";
                    gridControl1.Model[gridLastRowNum, UOM_COL].DataSource = uoms;
                    uoms.SortColumn = "Name";
                    uoms.Sort();
                    gridControl1.Model[gridLastRowNum, UOM_COL].DisplayMember = "Name";
                    gridControl1.Model[gridLastRowNum, UOM_COL].ValueMember = "ID";
                    gridControl1.Model[gridLastRowNum, UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;
                    gridControl1.Model[gridLastRowNum, UOM_COL].HorizontalAlignment = GridHorizontalAlignment.Left;

                    if (!gridManager.UIUOMHash.ContainsKey(sortManager.ID))
                        gridManager.UIUOMHash.Add(sortManager.ID, sortManager.UIUOMId);
                }
                catch (Exception ex)
                {
                    Logger.LogMessage(typeof(GridSimpleViewController), "Exception in rendering UOM:- " + ex.Message, Logger.Level.Debug);
                }
                gridLastRowNum++;
            }
            //Put Levels on grid cells
            LRRLevels selectedLevels = gridManager.GridData[0, 0] as LRRLevels;
            foreach (LRRLevel level in selectedLevels)
            {
                ID = level.LevelID;
                name = level.Name;
                if (!ColHeader.Contains(ID))
                {
                    ColHeader.Add(ID, name);
                    gridControl1.CoveredRanges.Add(GridRangeInfo.Cells(COMPONENT_ROW, gridLastColNum, COMPONENT_ROW, gridLastColNum + nModes - 1));
                    gridControl1[COMPONENT_ROW, gridLastColNum].Text = name;
                    gridControl1[COMPONENT_ROW, gridLastColNum].CellTipText = name;
                    //Loop through the Modes collection and render the grid
					foreach(LRRMode lrrMode in LRRModel.ProjectModes)
					{
						try
						{
                            gridControl1.ColWidths.SetSize(nModesCol, IOSchemes.MODE_COL_SIZE);
                            gridControl1[MODES_ROW, nModesCol].WrapText = false;
                            gridControl1[MODES_ROW, nModesCol].AutoSize = true;
                            gridControl1[MODES_ROW, nModesCol].Text = lrrMode.ModeDisplayName;
                            gridControl1[MODES_ROW, nModesCol].CellTipText = lrrMode.ModeName;
                            gridControl1[MODES_ROW, nModesCol].BackColor = IOSchemes.GetModeBackColor(lrrMode.ModeId);
                            gridControl1[MODES_ROW, nModesCol].TextColor = IOSchemes.GetModeTextColor(lrrMode.ModeId);
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridSimpleViewController),"Exception in rendering Mode:- " + ex.Message,Logger.Level.Debug);
						}

						//Add ColHeader Info
						ColHeaderCell colHeaderCell = new ColHeaderCell();
						colHeaderCell.ID = ID;
						colHeaderCell.ModeID = lrrMode.ModeId;
                        colHeaderCell.ObjectType = level.GetType(); 
 						colHeaderCellList.Add(colHeaderCell);

						nModesCol=nModesCol+1;
					}
                    gridControl1[COMPONENT_ROW, gridLastColNum].BackColor = IOSchemes.COMPONENT_BACK_COLOR;
					gridLastColNum = gridLastColNum + 1;
				}
				gridLastColNum = nModesCol;
			}
        }

        private void ClearHeaders(GridControlManager gridManager)
        {
            gridManager.RowHeader.Clear();
            gridManager.ColHeader.Clear();
        }

        #region Constants

        private const int ROLLUP_TYPE_COL = 1;
        private const int MODES_ROW = 2;
        private const int COMPONENT_ROW = 1;
        private const int HEADER_ROW = 0;
        private const int COL_SPAN = 2;
        private const int UOM_COL = 2;
        private const int UOM_ROW = 2;
        private const int DATA_COL_START = 3;
        private const int DATA_ROW_START = 3;

        private const int FREEZE_ROW_START = 0;
        private const int FREEZE_ROW_END = 1;
        private const int FREEZE_COL_START = 0;
        private const int FREEZE_COL_END = 1;

        #endregion

        private void trvLevels_AfterCheck(object sender, TreeViewEventArgs e)
        {
            LRRLevel level = e.Node.Tag as LRRLevel;
            if (e.Node.Checked)
                selectedLevels.Add(level);
            else
                selectedLevels.Remove(level);
        }
    }
}