using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.ExcelWrapper;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
	/// <summary>
	/// Summary description for UserControlChart.
	/// </summary>
	public class UserControlChart : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.PictureBox pictureBoxChart;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public UserControlChart(string chartID)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitializeComponent call

			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			_chartID = chartID;

			_lrrApp.DBManagerObj.PopulateObject(_chart,typeof(LRRChart)," where ID='" + _chartID + "'");
			_lrrApp.SymExcelWrapperObj.OpenWorkBook(_chart.WorkBookName);
			_img = _lrrApp.SymExcelWrapperObj.GetChart(_chart.WorkBookName, _chart.SheetName, _chart.ChartName);
			this.BackColor = Color.White;
			if(_img == null)
				throw new Exception();

			pictureBoxChart.SizeMode = PictureBoxSizeMode.StretchImage;
			pictureBoxChart.Image = _img;
			LoadChart();
			_lrrApp.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);
			this.Resize += new EventHandler(Chart_Resize);
			this.Text = _chart.DisplayName;
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			_lrrApp.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.pictureBoxChart = new System.Windows.Forms.PictureBox();
			this.SuspendLayout();
			// 
			// pictureBoxChart
			// 
			this.pictureBoxChart.Location = new System.Drawing.Point(-320, -76);
			this.pictureBoxChart.Name = "pictureBoxChart";
			this.pictureBoxChart.Size = new System.Drawing.Size(1104, 512);
			this.pictureBoxChart.TabIndex = 1;
			this.pictureBoxChart.TabStop = false;
			// 
			// UserControlChart
			// 
			this.Controls.Add(this.pictureBoxChart);
			this.Name = "UserControlChart";
			this.Size = new System.Drawing.Size(464, 360);
			this.Tag = "SID_CHART";
			this.ResumeLayout(false);

		}
		#endregion

		#region Fields

		private LRRApp	_lrrApp;
		private string	_chartID;
		private LRRChart _chart = new LRRChart();
		private Image _img;

		#endregion

		#region Public Methods

		public string GetDisplayName()
		{
			return _chart.DisplayName;
		}

		#endregion

		#region Private Methods

		private void RefreshImage()
		{
			_img = _lrrApp.SymExcelWrapperObj.GetChart(_chart.WorkBookName, _chart.SheetName, _chart.ChartName);
			pictureBoxChart.Image = _img;
		}
		private void LoadChart()
		{
			//this.Controls.Clear();
			//			lblDisplayName.Text = "Chart Name: " + _chart.DisplayName;
			//			lblDesc.Text = "Description: " + _chart.Description;
			//			lblDesc.Dock = DockStyle.Top;
			//			lblDisplayName.Dock = DockStyle.Top;
			//pictureBoxChart.SizeMode = PictureBoxSizeMode.StretchImage;
			//pictureBoxChart.Image = _img;
			//pictureBoxChart.Dock = DockStyle.Fill;
			
			int width, height;
			float scale, xscale, yscale;
			
			xscale = (float)this.Width/(float)_img.Width;
			yscale = (float)this.Height/(float)_img.Height;

			scale = Math.Min(xscale,yscale);

			width = (int)(_img.Width * scale);
			height = (int)(_img.Height * scale);

			pictureBoxChart.Width = width;
			pictureBoxChart.Height = height;
			pictureBoxChart.Location = new Point(Math.Max((this.Width-width)/2,0),Math.Max((this.Height - height)/2,0));
			//this.Controls.Add(pictureBoxChart);
			//			this.Controls.Add(lblDesc);
			//			this.Controls.Add(lblDisplayName);
		}

		#endregion

		#region Event Handlers

		private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
		{
			RefreshViewDel refViewDel = new RefreshViewDel(RefreshView);
			_lrrApp.MainFrame.BeginInvoke(refViewDel);
		}

		private delegate void RefreshViewDel();
		private void RefreshView()
		{
			RefreshImage();
		}

		private void Chart_Resize(object sender, EventArgs e)
		{
			LoadChart();
		}

		#endregion
	}
}
