using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Diagnostics;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.ExcelWrapper;
using Symcon.Utils.MessageLogger;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
	/// <summary>
	/// Summary description for UserControlCharts.
	/// </summary>
	public class UserControlCharts : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.Panel pnlMainChartItems;
		private System.Windows.Forms.Panel pnlForCharts;
		private System.Windows.Forms.Label lblDesc;
		private System.Windows.Forms.Label lblDisplayName;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public UserControlCharts(string chartItemID)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			if(chartItemID.IndexOf("ChartItem")!=-1)
				chartItemID = chartItemID.Replace("ChartItem","");
			_chartItemID = chartItemID;

			_lrrApp.DBManagerObj.PopulateObject(_chartItem,typeof(LRRChartItem)," where ID='" + _chartItemID + "'");
            LRRChartDefinitions chartDefinitions = new LRRChartDefinitions();
            
            //_lrrApp.DBManagerObj.PopulateCollection(chartDefinitions, " where ChartItemID='" + _chartItemID + "' ORDER BY StartRow, StartCol", false);
           
            _lrrApp.DBManagerObj.PopulateCollection(chartDefinitions, " where ChartItemID='" + _chartItemID + "'", false);
            
            // two level sorting

            // First find the max
            int maxStartRow = 0;
            int maxStartCol = 0;

            foreach (LRRChartDefinition chartDefinition in chartDefinitions)
            {
                if (chartDefinition.StartRow > maxStartRow)
                    maxStartRow = chartDefinition.StartRow;

                if (chartDefinition.StartCol > maxStartCol)
                    maxStartCol = chartDefinition.StartCol;

            }
			
            //Now Sorting
            for (int k = 0; k <= maxStartRow; k++)
            {
                for (int j = 0; j <= maxStartCol; j++)
                {
                    foreach (LRRChartDefinition chartDefinition in chartDefinitions)
                    {
                        if((chartDefinition.StartRow == k) && (chartDefinition.StartCol == j))
                        {
                            _chartDefinitions.Add(chartDefinition);
                        }
                    }
                }
            }


                _imgs = new ChartImage[_chartDefinitions.Count];
			int i=0;

			// Code added for testing.
			//			_lrrApp.SymExcelWrapperObj.Terminate();
			//			_lrrApp.SymExcelWrapperObj = new Symcon.LightRidge.ExcelWrapper.SymExcelWrapper(Application.StartupPath);

			foreach(LRRChartDefinition chartDef in _chartDefinitions)
			{
				_imgs[i] = new ChartImage();
				LRRChart chart = new LRRChart();
				_lrrApp.DBManagerObj.PopulateObject(chart,typeof(LRRChart)," where ID='" + chartDef.ChartID + "' and DataType='ChartData'");
				_lrrApp.SymExcelWrapperObj.OpenWorkBook(chart.WorkBookName);
				_imgs[i].img = _lrrApp.SymExcelWrapperObj.GetChart(chart.WorkBookName, chart.SheetName, chart.ChartName);
				if(_imgs[i].img == null)
					throw new Exception("Could not get chart image.");
				_imgs[i].name = chart.ChartName + ":" + chartDef.StartRow + "," + chartDef.StartCol;
				//				if(_imgs[i].img == null)
				//					throw new Exception();
				_imgs[i].startRow = chartDef.StartRow;
				_imgs[i].startCol = chartDef.StartCol;
				_imgs[i].rowSpan = chartDef.RowSpan;
				_imgs[i].colSpan = chartDef.ColSpan;
				_imgs[i].workBookName = chart.WorkBookName;
				_imgs[i].sheetName = chart.SheetName;
				_imgs[i].chartName = chart.ChartName;
				i++;
			}
			for(int j=(_chartDefinitions.Count -1);j>=0;j--)
			{
				LRRChartDefinition chartDef = _chartDefinitions[j];
				if((_maxStartRow +_maxStartColSpan) < (chartDef.StartRow + chartDef.ColSpan))
				{
					_maxStartRow = chartDef.StartRow;
					_maxStartColSpan = chartDef.ColSpan;
				}
				if((_maxStartRow +_maxStartColSpan) == (chartDef.StartRow + chartDef.ColSpan))
				{
					if(chartDef.StartRow < _maxStartRow)
					{
						_maxStartRow = chartDef.StartRow;
						_maxStartColSpan = chartDef.ColSpan;
					}
				}
				if((_maxStartCol + _maxStartRowSpan) < (chartDef.StartCol + chartDef.RowSpan))
				{
					_maxStartCol = chartDef.StartCol;
					_maxStartRowSpan = chartDef.RowSpan;
				}
				if((_maxStartCol + _maxStartRowSpan) == (chartDef.StartCol + chartDef.RowSpan))
				{
					if(chartDef.StartCol < _maxStartCol)
					{
						_maxStartCol = chartDef.StartCol;
						_maxStartRowSpan = chartDef.RowSpan;
					}
				}
			}
			// Matrix of chart images to be filled by charts or dummy for blank rows.
			ChartImage[][] _arrCI = new ChartImage[_maxStartCol + _maxStartRowSpan][];
			for(int j=0;j<_arrCI.Length;j++)
			{
				_arrCI[j] = new ChartImage[_maxStartRow + _maxStartColSpan];
			}
			for(int l=0;l<_imgs.Length;l++)
			{
				ChartImage cimg = _imgs[l];
				for(int j=cimg.startRow;j<(cimg.startRow+cimg.colSpan);j++)
				{
					for(int k=cimg.startCol;k<(cimg.startCol+cimg.rowSpan);k++)
					{
						_arrCI[k][j] = cimg;
					}
				}
			}
			for(int k=0;k<_arrCI[0].Length;k++)
			{
				if(_arrCI[0][k] == null)
				{
					bool isRowEmpty = true;
					for(int j=0;j<_arrCI.Length;j++)
					{
						if(!(_arrCI[j][k]==null))
							isRowEmpty = false;
					}
					if(isRowEmpty)
					{
						ChartImage dummyImage = new ChartImage();
						dummyImage.isDummy = true;
						dummyImage.startRow = k;
						dummyImage.startCol = 0;
						dummyImage.rowSpan = _arrCI[0].Length;
						dummyImage.colSpan = 1;
						for(int j=0;j<_arrCI.Length;j++)
						{
							_arrCI[j][k] = dummyImage;
						}
					}
				}
			}
			for(int k=0;k<_arrCI[0].Length;k++)
			{
				for(int j=0;j<_arrCI.Length;j++)
				{
					if(_arrCI[j][k]==null)
					{
						ChartImage dummyImage = new ChartImage();
						dummyImage.isDummy = true;
						dummyImage.startRow = k;
						dummyImage.startCol = j;
						dummyImage.name = "Dummy:" + k.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "," + j.ToString(Symcon.BusinessObjects.SymConverter.Culture);
						dummyImage.rowSpan = 1;
						dummyImage.colSpan = 1;
						_arrCI[j][k] = dummyImage;
					}
				}
			}
			ArrayList arrImages = new ArrayList();
			for(int k=0;k<_arrCI.Length;k++)
			{
				for(int j=0;j<_arrCI[0].Length;j++)
				{
					if(_arrCI[k][j]!=null)
					{
						if(!arrImages.Contains(_arrCI[k][j]))
						{
							arrImages.Add(_arrCI[k][j]);
						}
					}
				}
			}
			_imgs = new ChartImage[arrImages.Count];
			for(int j=0;j<arrImages.Count;j++)
			{
				_imgs[j] = arrImages[j] as ChartImage;
			}
			this.BackColor = Color.White;
			this.Text = _chartItem.Name;
			_lrrApp.DBManagerObj.PopulateObject(_size,typeof(LRRChartItemsCellSize)," where ID='1'");
			this.LoadCharts(pnlMainChartItems.Width);
			pnlForCharts.Controls.Add(_mainPanel);
			_mainPanel.AutoScroll = true;
			this._mainPanel.Dock = DockStyle.Fill;
			
			this.lblDisplayName.Text = "Name: " + _chartItem.Name;
			this.lblDesc.Text = "Description: " + _chartItem.Description;
			this.lblDisplayName.Dock = DockStyle.Top;
			this.lblDesc.Dock = DockStyle.Top;
			
			pnlMainChartItems.Controls.Add(lblDesc);
			pnlMainChartItems.Controls.Add(lblDisplayName);

			this.AutoScroll = true;
			this.Load += new EventHandler(UserControlCharts_Load);
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			_lrrApp.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.pnlMainChartItems = new System.Windows.Forms.Panel();
			this.pnlForCharts = new System.Windows.Forms.Panel();
			this.lblDesc = new System.Windows.Forms.Label();
			this.lblDisplayName = new System.Windows.Forms.Label();
			this.pnlMainChartItems.SuspendLayout();
			this.SuspendLayout();
			// 
			// pnlMainChartItems
			// 
			this.pnlMainChartItems.AutoScroll = true;
			this.pnlMainChartItems.Controls.Add(this.pnlForCharts);
			this.pnlMainChartItems.Controls.Add(this.lblDesc);
			this.pnlMainChartItems.Controls.Add(this.lblDisplayName);
			this.pnlMainChartItems.Location = new System.Drawing.Point(0, 0);
			this.pnlMainChartItems.Name = "pnlMainChartItems";
			this.pnlMainChartItems.Size = new System.Drawing.Size(952, 504);
			this.pnlMainChartItems.TabIndex = 0;
			// 
			// pnlForCharts
			// 
			this.pnlForCharts.AutoScroll = true;
			this.pnlForCharts.Dock = System.Windows.Forms.DockStyle.Fill;
			this.pnlForCharts.Location = new System.Drawing.Point(0, 80);
			this.pnlForCharts.Name = "pnlForCharts";
			this.pnlForCharts.Size = new System.Drawing.Size(952, 424);
			this.pnlForCharts.TabIndex = 11;
			// 
			// lblDesc
			// 
			this.lblDesc.Dock = System.Windows.Forms.DockStyle.Top;
			this.lblDesc.Location = new System.Drawing.Point(0, 40);
			this.lblDesc.Name = "lblDesc";
			this.lblDesc.Size = new System.Drawing.Size(952, 40);
			this.lblDesc.TabIndex = 10;
			// 
			// lblDisplayName
			// 
			this.lblDisplayName.Dock = System.Windows.Forms.DockStyle.Top;
			this.lblDisplayName.Location = new System.Drawing.Point(0, 0);
			this.lblDisplayName.Name = "lblDisplayName";
			this.lblDisplayName.Size = new System.Drawing.Size(952, 40);
			this.lblDisplayName.TabIndex = 9;
			// 
			// UserControlCharts
			// 
			this.Controls.Add(this.pnlMainChartItems);
			this.Name = "UserControlCharts";
			this.Size = new System.Drawing.Size(952, 504);
			this.pnlMainChartItems.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region Fields

		private LRRApp	_lrrApp;
		private string _chartItemID;
		private LRRChartItem _chart = new LRRChartItem();
		private ChartImage[] _imgs;
		private LRRChartDefinitions _chartDefinitions = new LRRChartDefinitions();
		private ArrayList _arrSplittersBound = new ArrayList();
		private LRRChartItem _chartItem = new LRRChartItem();
		private int _maxStartRow = 0;
		private int _maxStartCol = 0;
		private int _maxStartRowSpan = 0;
		private int _maxStartColSpan = 0;
		private Panel _mainPanel = new Panel();
		private LRRChartItemsCellSize _size = new LRRChartItemsCellSize();

		#endregion

		#region Private Methods

		private void RefreshImages()
		{
			foreach(ChartImage ci in _imgs)
			{
				try
				{
					if(ci.workBookName != null)
					{
						ci.img = _lrrApp.SymExcelWrapperObj.GetChart(ci.workBookName, ci.sheetName, ci.chartName);
						ci.pbImageContainer.Image = ci.img;
					}
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(UserControlCharts),ex.Message  ,Logger.Level.Debug);
				}
			}
		}
		/// <summary>
		/// Add all charts in the current's rows's coloumn
		/// </summary>
		/// <param name="panel">The panel to which the charts are to be placed</param>
		/// <param name="currentRowColSpan">Current row's coloumn span</param>
		/// <param name="cimg">Chart Image whose all the charts in the given row's coloumn is to be added.</param>
		/// <returns></returns>
		private int AddAllChartsInCurrentCol(Panel panel, int currentRowColSpan, ChartImage cimg)
		{
			ChartImage lastCimg = null;
			Splitter splitter3 = new Splitter();
			Panel panel3 = new Panel();
			int maxRowSpan = 0;
			int totalHeight = 0;
			for(int i=(_imgs.Length-1);i>=0; i--)
			{
				ChartImage cimg2 = _imgs[i];
				if(cimg2.startCol == cimg.startCol && cimg2.startRow>=cimg.startRow && cimg2.startRow<(cimg.startRow + currentRowColSpan))
				{
					if(cimg2.rowSpan>maxRowSpan)
						maxRowSpan = cimg2.rowSpan;
				}
			}
			panel3.Width = maxRowSpan * _size.Width;
			panel3.Name = "PanelRowsCurrentColContainer:" + cimg.startRow + "," + cimg.startCol;
			panel.Width = panel3.Width;
			panel3.Dock = DockStyle.Left;
			splitter3.Dock = DockStyle.Left;
			for(int i=(_imgs.Length-1);i>=0; i--)
			{
				ChartImage cimg2 = _imgs[i];
				if(cimg2.startCol == cimg.startCol && cimg2.startRow>=cimg.startRow && cimg2.startRow <(cimg.startRow + currentRowColSpan))
				{
					//					if(i<(_imgs.Length-1))
					//					{
					//							
					//						if(_imgs[i+1].startCol == cimg.startCol)
					//						{
					//							int spanDiff = (cimg.startRow + cimg.colSpan) -_imgs[i+1].startRow;
					//							if(spanDiff>0)
					//							{
					//								// Panel added at the middle
					//								Splitter splitterDummyRowDivider = new Splitter();
					//								splitterDummyRowDivider.Dock = DockStyle.Top;
					//								panel3.Controls.Add(splitterDummyRowDivider);
					//								Panel pnlDummy = new Panel();
					//								pnlDummy.Height = spanDiff * _size.Height-3;
					//								totalHeight += spanDiff * _size.Height;
					//								pnlDummy.Dock = DockStyle.Top;
					//								panel3.Controls.Add(pnlDummy);
					//							}
					//						}
					//					}

					Splitter splitter2 = new Splitter();
					splitter2.Dock = DockStyle.Top;
					panel3.Controls.Add(splitter2);
					Panel panel2 = new Panel();
					panel2.Dock = DockStyle.Top;
					panel2.Height = cimg2.colSpan * _size.Height - 3;
					totalHeight +=cimg2.colSpan * _size.Height;
	
					if(!cimg2.isDummy)
					{
						PictureBox pBox = new PictureBox();
						pBox.Image = cimg2.img;
						cimg2.pbImageContainer = pBox;
						pBox.SizeMode = PictureBoxSizeMode.StretchImage;
						panel2.Controls.Add(pBox);
						pBox.Dock = DockStyle.Fill;
						pBox.Show();
					}
					lastCimg = cimg2;
					panel2.Name = "Panel:" + cimg2.name;
					panel3.Controls.Add(panel2);
				}
				panel.Controls.Add(splitter3);
				panel.Controls.Add(panel3);
			}
			if(lastCimg!=null && lastCimg.startRow>cimg.startRow)
			{
				// Panel added at the top
				Splitter splitterDummyColDivider = new Splitter();
				splitterDummyColDivider.Dock = DockStyle.Top;
				panel3.Controls.Add(splitterDummyColDivider);
				Panel pnlDummy = new Panel();
				pnlDummy.Name = "PanelDummyAddedAtTheTop:" + cimg.startRow.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "," + lastCimg.startRow.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				pnlDummy.Dock=DockStyle.Top;
				pnlDummy.Height = (lastCimg.startRow - cimg.startRow)*_size.Height-3;
				totalHeight += (lastCimg.startRow - cimg.startRow)*_size.Height;
				panel3.Controls.Add(pnlDummy);
			}
			//			if(totalHeight<(currentRowColSpan *_size.Height))
			//			{
			//				// Panel added at the bottom
			//				Panel pnlDummy = new Panel();
			//				pnlDummy.Dock=DockStyle.Bottom;
			//				pnlDummy.Height = (currentRowColSpan *_size.Height) - totalHeight;
			//				panel3.Controls.Add(pnlDummy);
			//			}
			return maxRowSpan * _size.Width + 3;
		}
		/// <summary>
		/// Loads the charts to the screen
		/// </summary>
		/// <param name="windowWidth">Width of the window</param>
		private void LoadCharts(int windowWidth)
		{
			int currentRowColSpan=0;
			int prevRowColSpan = 0;
			int checkRowColSpan = 0;
			for(int i=_maxStartRow;i>=0;i-=prevRowColSpan)
			{
				Panel panelRow = null;
				bool isLast = true;
				int totalWidth=0;
				int currentRowHeight = 0;
				ChartImage lastCimg = null;
				for(int j=(_imgs.Length-1);j>=0;j--)
				{
					// Getting the current row height and current row coloumn span
					ChartImage cimg = _imgs[j];
					if(cimg.startRow == i)
					{
						if((cimg.colSpan*_size.Height -3 )>currentRowHeight || cimg.colSpan>currentRowColSpan)
						{
							currentRowHeight = cimg.colSpan*_size.Height -3;
							currentRowColSpan = cimg.colSpan;
						}
					}
				}
				
				for(int j=(_imgs.Length-1);j>=0;j--)
				{
					ChartImage cimg = _imgs[j];	
					if(cimg.startRow == i)
					{
						if(j>0)
						{
							// If the chart is already added continue.
							if(cimg.startCol == _imgs[j-1].startCol && cimg.startRow < (_imgs[j-1].startRow + _imgs[j-1].colSpan))
								continue;
						}
						//						if(j<(_imgs.Length-1))
						//						{
						//							if(_imgs[j+1].startRow == cimg.startRow)
						//							{
						//								int spanDiff = _imgs[j+1].startCol - (cimg.startCol + cimg.rowSpan);
						//								if(spanDiff>0)
						//								{
						//									// Panel added at the middle
						//									Splitter splitterDummyColDivider = new Splitter();
						//									splitterDummyColDivider.Dock = DockStyle.Left;
						//									panelRow.Controls.Add(splitterDummyColDivider);
						//									Panel pnlDummy = new Panel();
						//									pnlDummy.Width = spanDiff * _size.Width-3;
						//									totalWidth+=spanDiff * _size.Width;
						//									pnlDummy.Dock = DockStyle.Left;
						//									panelRow.Controls.Add(pnlDummy);
						//									ChartImage dummyChartImage = new ChartImage();
						//									dummyChartImage.colSpan = spanDiff;
						//									dummyChartImage.rowSpan =pnlDummy.Height/_size.Height;
						//									dummyChartImage.startRow = cimg.startRow;
						//									dummyChartImage.startCol = cimg.startCol + cimg.rowSpan;
						//									AddAllChartsInCurrentCol(pnlDummy,currentRowColSpan,dummyChartImage);
						//								}
						//							}
						//						}
						
						if(panelRow==null)
						{
							// Add the row's panel
							Splitter splitterBottomRow = new Splitter();
							splitterBottomRow.Dock = DockStyle.Top;
							_mainPanel.Controls.Add(splitterBottomRow);
							panelRow = new Panel();
							panelRow.Name = "PanelRow:" + i.ToString(Symcon.BusinessObjects.SymConverter.Culture);
							panelRow.Dock = DockStyle.Top;
							panelRow.Height = currentRowHeight;
							_mainPanel.Controls.Add(panelRow);
						}

						Splitter splitterColDivider = new Splitter();
						if(isLast)
							splitterColDivider.Dock = DockStyle.Right;
						else
							splitterColDivider.Dock = DockStyle.Left;
						panelRow.Controls.Add(splitterColDivider);
						Panel panel = new Panel();
						panel.Width = cimg.rowSpan * _size.Width-3;
						panel.Name = "PanelCol:" + cimg.name;

						totalWidth += AddAllChartsInCurrentCol(panel,currentRowColSpan,cimg);
						if(isLast)
						{
							panel.Dock = DockStyle.Fill;
							isLast = false;
						}
						else
							panel.Dock = DockStyle.Left;
						panelRow.Controls.Add(panel);
						lastCimg = cimg;
					}	
				}
				
				for(int k=0;k<i;k+=checkRowColSpan)
				{
					for(int j=(_imgs.Length-1);j>=0;j--)
					{
						ChartImage cimg = _imgs[j];
						if(cimg.startRow == k)
						{
							if(cimg.colSpan>checkRowColSpan)
							{
								checkRowColSpan = cimg.colSpan;
							}
						}
					}
					for(int j=(_imgs.Length-1);j>=0;j--)
					{
						ChartImage cimg = _imgs[j];
						if(cimg.startRow == k && lastCimg!=null)
						{
							if((cimg.startRow + cimg.colSpan)==lastCimg.startRow)
							{
								if(cimg.colSpan>prevRowColSpan)
								{
									prevRowColSpan = cimg.colSpan;
								}
							}
						}
						if(lastCimg == null)
						{
							if(cimg.startRow == k && (cimg.startRow + cimg.colSpan)==_maxStartRow)
							{
								if(cimg.colSpan>prevRowColSpan)
								{
									prevRowColSpan = cimg.colSpan;
								}
							}
						}
					}
				}
				if(i==0)
					prevRowColSpan=1;
				if(lastCimg!=null && lastCimg.startCol>0)
				{
					// Panel added at the start
					Splitter splitterDummyColDivider = new Splitter();
					splitterDummyColDivider.Dock = DockStyle.Left;
					panelRow.Controls.Add(splitterDummyColDivider);
					Panel pnlDummy = new Panel();
					pnlDummy.Dock=DockStyle.Left;
					pnlDummy.Width = lastCimg.startCol*_size.Width-3;
					pnlDummy.Name = "PanelDummyRowStart:" + i;
					totalWidth+=lastCimg.startCol * _size.Width;
					panelRow.Controls.Add(pnlDummy);
				}
				if(totalWidth<windowWidth && panelRow!=null)
				{
					// Panel added at the end
					Panel pnlDummy = new Panel();
					pnlDummy.Dock=DockStyle.Right;
					pnlDummy.Width = windowWidth - totalWidth;
					pnlDummy.Name = "PanelDummyRowLast:" + i;
					panelRow.Controls.Add(pnlDummy);
				}
			}
		}

		#endregion

		#region Event Handlers

		private void UserControlCharts_Load(object sender, EventArgs e)
		{
			_lrrApp.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);
		}

		private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
		{
			RefreshViewDel refViewDel = new RefreshViewDel(RefreshView);
			_lrrApp.MainFrame.BeginInvoke(refViewDel);
		}

		private delegate void RefreshViewDel();
		private void RefreshView()
		{
			RefreshImages();
		}
		#endregion
	}
	/// <summary>
	/// Chart Image class for holding the detais of a chart. If the chart is dummy there is no image for it.
	/// </summary>
	class ChartImage
	{
		public Image img;
		public string name;
		public string workBookName;
		public string sheetName;
		public string chartName;
		public PictureBox pbImageContainer;
		public int startRow;
		public int startCol;
		public int rowSpan;
		public int colSpan;
		public bool isDummy = false;
	}
}
