using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Data;
using System.Diagnostics;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.XMLUtils;
using Symcon.Utils.Cryptography;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using System.Collections.Specialized;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.Options;
using Symcon.LightRidge.Options.DBOptions;
using System.IO;
using System.Text.RegularExpressions;
using Symcom.Evaluator;
using System.Xml;
using System.Xml.Serialization;
using Symcon.Globals;
using Symcon.Utils.HeaderDispatcher;
//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
	/// <summary>
	/// Summary description for UserControlDataBase.
	/// </summary>
	public class UserControlDataBase : System.Windows.Forms.UserControl,ILRRUIContainer
	{
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridLRRDatabase;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private LRRApp							_app;
		private LRROptions				_lrrOptionsObj; //Local copy of the Options Object...
		private TabControl					_parentTabControl;
		private GridExcelFormatter	_gridExcelFormatter = new GridExcelFormatter(); 
		private TabPage						_parentTabPage;
		public UserControlDataBase()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			_gridExcelFormatter.WireGrid(this.gridLRRDatabase);  
			_lrrOptionsObj = _app.GetOptions() as LRROptions;
			_lrrOptionsObj.DataBaseConnectionInfosObj.RefreshCollection();

			SyncGridUtilities.InitializeGridLookAndFeel(gridLRRDatabase);
			gridLRRDatabase.TableModel.Model.CellModels.Add("EllipsisText", new  EllipsisCellModel(gridLRRDatabase.TableModel.Model));
			gridLRRDatabase.DataSource = _lrrOptionsObj.DataBaseConnectionInfosObj;	
			gridLRRDatabase.TableDescriptor.Columns.Clear();
			gridLRRDatabase.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_DBENAME"),"DatabaseName");
			GridColumnDescriptor gcd = new GridColumnDescriptor();
			gcd.HeaderText = LRRHeaderText.GetHeaderString("SID_DBTYPE");
			gcd.MappingName	= "DatabaseTypeStr";
			GridTableCellStyleInfo style = gcd.Appearance.GetStyle(GridTableCellType.AnyRecordFieldCell);
			style.CellType	= "ComboBox";
			StringCollection types = new StringCollection();
			int i;
			for(i=0;i<_lrrOptionsObj.DBTypes.Count;i++)
			{
				types.Add(_lrrOptionsObj.DBTypes[i].ToString());
			}
			style.ChoiceList = types;
			gridLRRDatabase.TableDescriptor.Columns.Add(gcd);
			//			gridLRRDatabase.TableDescriptor.Columns.Add("Database Type","DatabaseTypeStr");
			gridLRRDatabase.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_DBEPATH"),"DatabaseFilePath");
			gridLRRDatabase.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_PORTNO"),"PortNumber");
			gridLRRDatabase.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_SERVERNAME"),"ServerName");
			gridLRRDatabase.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_USERNAME"),"DatabaseUser");
			gridLRRDatabase.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_PASSWARD"),"DatabasePWD");
			
//			this.gridLRRDatabase.TableDescriptor.Columns["Password"].Appearance.AnyRecordFieldCell.CellType = "OriginalTextBox";
//			this.gridLRRDatabase.TableDescriptor.Columns["Password"].Appearance.AnyRecordFieldCell.PasswordChar = '*';
//			
			this.gridLRRDatabase.TableDescriptor.Columns[gridLRRDatabase.TableDescriptor.Columns.IndexOf(LRRHeaderText.GetHeaderString("SID_DBEPATH"))].Appearance.AddNewRecordFieldCell.CellType ="TextBox";
			this.gridLRRDatabase.TableDescriptor.Columns[gridLRRDatabase.TableDescriptor.Columns.IndexOf(LRRHeaderText.GetHeaderString("SID_DBEPATH"))].Appearance.AnyRecordFieldCell.ReadOnly=true;
			//			this.gridLRRDatabase.TableDescriptor.Columns[gridLRRDatabase.TableDescriptor.Columns.IndexOf("Database File Path")].Appearance.AnyRecordFieldCell.ShowButtons=GridShowButtons.ShowCurrentCell;
			
			this.gridLRRDatabase.TableDescriptor.QueryCellStyleInfo += new GridTableCellStyleInfoEventHandler(TableDescriptor_QueryCellStyleInfo);
			
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.gridLRRDatabase = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			((System.ComponentModel.ISupportInitialize)(this.gridLRRDatabase)).BeginInit();
			this.SuspendLayout();
			// 
			// gridLRRDatabase
			// 
			this.gridLRRDatabase.BackColor = System.Drawing.SystemColors.Window;
			this.gridLRRDatabase.Dock = System.Windows.Forms.DockStyle.Fill;
			this.gridLRRDatabase.Location = new System.Drawing.Point(0, 0);
			this.gridLRRDatabase.Name = "gridLRRDatabase";
			this.gridLRRDatabase.Size = new System.Drawing.Size(880, 368);
			this.gridLRRDatabase.TabIndex = 1;
			this.gridLRRDatabase.TableOptions.AllowDragColumns = false;
			this.gridLRRDatabase.TableOptions.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Any | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.MixRangeType)));
			this.gridLRRDatabase.TableOptions.AllowSortColumns = true;
			this.gridLRRDatabase.TableOptions.ListBoxSelectionColorOptions = Syncfusion.Windows.Forms.Grid.Grouping.GridListBoxSelectionColorOptions.None;
			this.gridLRRDatabase.TableOptions.ListBoxSelectionCurrentCellOptions = Syncfusion.Windows.Forms.Grid.Grouping.GridListBoxSelectionCurrentCellOptions.None;
			this.gridLRRDatabase.TableOptions.ListBoxSelectionMode = System.Windows.Forms.SelectionMode.None;
			this.gridLRRDatabase.Text = "gridGroupingControl1";
			this.gridLRRDatabase.TopLevelGroupOptions.CaptionText = "LRR Database Information: {RecordCount} Items";
			this.gridLRRDatabase.TopLevelGroupOptions.IsExpandedInitialValue = true;
			this.gridLRRDatabase.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;
			this.gridLRRDatabase.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
			this.gridLRRDatabase.TopLevelGroupOptions.ShowCaptionPlusMinus = false;
			this.gridLRRDatabase.TopLevelGroupOptions.ShowColumnHeaders = true;
			this.gridLRRDatabase.VersionInfo = "3.301.0.0";
			this.gridLRRDatabase.TableControlCellButtonClicked += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellButtonClickedEventHandler(this.gridLRRDatabase_TableControlCellButtonClicked);
			// 
			// UserControlDataBase
			// 
			this.Controls.Add(this.gridLRRDatabase);
			this.Name = "UserControlDataBase";
			this.Size = new System.Drawing.Size(880, 368);
			this.Tag = "SID_USDB";
			((System.ComponentModel.ISupportInitialize)(this.gridLRRDatabase)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion
		private void TableDescriptor_QueryCellStyleInfo(object sender, GridTableCellStyleInfoEventArgs e)
		{

			if(e.TableCellIdentity.Column != null && e.TableCellIdentity.Column.MappingName == "DatabaseFilePath" && e.TableCellIdentity.DisplayElement is GridRecordRow)
			{
				Symcon.LightRidge.Options.DBOptions.DBInfo dr  =e.TableCellIdentity.DisplayElement.ParentRecord.GetData() as Symcon.LightRidge.Options.DBOptions.DBInfo;
				if(dr != null)
				{
					if(dr.DatabaseTypeStr.Equals("ACCESS"))
					{
						e.Style.CellType = "EllipsisText";
						e.Handled = true;
					}
				}
			}	
			
		}
		#region ILRRUIContainer Members

		public new void Refresh()
		{
			
		}

		public void Save()
		{
		    if(_lrrOptionsObj.DataBaseConnectionInfosObj.IsChanged)
			{
				EnCryptyPassword(_lrrOptionsObj);
				SGTXmlSerializer.SerializeAsXML(_app.OptionsFile,typeof(LRROptions),_lrrOptionsObj);
				//Decrypt the password information
				DecryptyPassword(_lrrOptionsObj);
				OptionsUserControl.saved	=true;
				_lrrOptionsObj.DataBaseConnectionInfosObj.RefreshCollection();
			}
		}

		public void EnCryptyPassword(LRROptions lrrOptionsObj)
		{
			int nRow = 0;
			int nTotal = 0;
			DBConnectionInfos objDBConnectionInfos = lrrOptionsObj.DataBaseConnectionInfosObj;
			nTotal = objDBConnectionInfos.Count;
			for(nRow = 0; nRow <nTotal;nRow++)
			{
				string sPWD = objDBConnectionInfos[nRow].DatabasePWD == null ? "" :  objDBConnectionInfos[nRow].DatabasePWD.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				if(sPWD != "")
				{
					sPWD = CryptographyUtils.EncryptedPassword(sPWD);
					objDBConnectionInfos[nRow].DatabasePWD = sPWD;
				}
			}
			
		}
		private void DecryptyPassword(LRROptions lrrOptionsObj)
		{
			int nRow = 0;
			int nTotal = 0;
			DBConnectionInfos objDBConnectionInfos = lrrOptionsObj.DataBaseConnectionInfosObj;
			nTotal = objDBConnectionInfos.Count;
			for(nRow = 0; nRow <nTotal;nRow++)
			{
				string sPWD = objDBConnectionInfos[nRow].DatabasePWD == null ? "" :  objDBConnectionInfos[nRow].DatabasePWD.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				if(sPWD != "")
				{
					sPWD = CryptographyUtils.DecryptPassword(sPWD);
					objDBConnectionInfos[nRow].DatabasePWD = sPWD;
				}
			}
			
		}
		public void CopyMasterProperties()
		{
		
		}

		public void Paste()
		{
			// TODO:  Add ComplexEquip.Paste implementation
		}

		public void Copy()
		{
			// TODO:  Add ComplexEquip.Copy implementation
		}

		public void Print()
		{
			// TODO:  Add ComplexEquip.Print implementation
			if(gridLRRDatabase.TableControl.CurrentCell.IsActive)
			{
				GridGroupingPrint ggPrint = new GridGroupingPrint();
				ggPrint.PrintData(gridLRRDatabase,false);
			}
		}

		public void Delete()
		{
			// TODO:  Add ComplexEquip.Delete implementation
		}

		public void ArrangeColumns()
		{
			// TODO:  Add ComplexEquip.ArrangeColumns implementation
		}

		public void ClearFilter()
		{
			
		}
		public void ImportToExcel()
		{
		}
		public bool SupportsSelectionPaste()
		{
			return true;
		}

		public void SelectionPaste()
		{
		}

		public  void RightLevel()
		{

		}
		public  void LeftLevel()
		{

		}
		public  void UpLevel()
		{

		}
		public  void DownLevel()
		{

		}

		public bool SupportsCopy()
		{
			// TODO:  Add ComplexEquip.SupportsCopy implementation
			return false;
		}

		public bool SupportsPaste()
		{
			// TODO:  Add ComplexEquip.SupportsPaste implementation
			return true;
		}

		public bool SupportsExport()
		{
			// TODO:  Add ComplexEquip.SupportsExport implementation
			return true;
		}

		public bool SupportsSave()
		{
			// TODO:  Add ComplexEquip.SupportsSave implementation
			return true;
		}

		public bool SupportsRefresh()
		{
			// TODO:  Add ComplexEquip.SupportsRefresh implementation
			return true;
		}

		public bool SupportsDelete()
		{
			// TODO:  Add ComplexEquip.SupportsDelete implementation
			return false;
		}

		public bool SupportsPrint()
		{
			// TODO:  Add ComplexEquip.SupportsPrint implementation
			return true;
		}

		public bool SupportsArrangeColumns()
		{
			// TODO:  Add ComplexEquip.SupportsArrangeColumns implementation
			return false;
		}

		public bool SupportsClearFilter()
		{
			// TODO:  Add ComplexEquip.SupportsClearFilter implementation
			return true;
		}
		public bool SupportsMasterPropertiesCopy()
		{
			return false;
		}
		public bool SupportsInsertNewRows()
		{
			// TODO:  Add MasterProperties.SupportsClearFilter implementation
			return true;
		}

		public bool SupportsImportToExcel()
		{
			return true;
		}

		public bool SupportsLevel()
		{
			return false;
		}

		public void InsertNewRows()
		{
			
		}

		public void Export()
		{
			
		}

		private void gridLRRDatabase_TableControlCellButtonClicked(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellButtonClickedEventArgs e)
		{
			if(e.TableControl.Table.TableModel.Model[e.Inner.RowIndex,e.Inner.ColIndex].CellType=="EllipsisText")
			{
				//gridLRRDatabase.TableControl.GetCellRenderer(e.Inner.RowIndex,e.Inner.ColIndex)
				gridLRRDatabase.TableControl.GetCellRenderer(e.Inner.RowIndex,e.Inner.ColIndex).CurrentCell.Activate(e.Inner.RowIndex,e.Inner.ColIndex);
				OpenFileDialog flDialog = new OpenFileDialog();
				flDialog.Filter ="*.mdb |";
				if(flDialog.ShowDialog() == DialogResult.OK)
				{
					string fName=Path.GetFullPath(flDialog.FileName);
					//gridLRRDatabase.TableControl.CurrentCell.Renderer.ControlText=fName;
					//gridLRRDatabase.TableControl.Model[e.Inner.RowIndex,e.Inner.ColIndex];/*.ControlText = fName;*/
					GridTableCellStyleInfo gtsif = gridLRRDatabase.TableControl.Model[e.Inner.RowIndex,e.Inner.ColIndex];
					gtsif.ApplyText(fName);
					//gtsif.Control.Text = fName;
				}
			}
		}

	
		public TabControl	ParentTabControl
		{
			set
			{
				_parentTabControl = value;
			}
			get{return _parentTabControl;}
		}
	
		public TabPage ParentTabPage
		{
			set
			{
				_parentTabPage = value;
			}
			get
			{
				return _parentTabPage;
			}
		}
		#endregion

		#region ILRRUIContainer Members

		public void ClearSort()
		{
			SyncGridUtilities.ClearSort(gridLRRDatabase);
		}

		public bool SupportsClearSort()
		{
			// TODO:  Add UserControlDataBaseConnectionInfos.SupportsClearSort implementation
			return true;
		}

		#endregion
		
		#region ILRRUIContainer Members

		public void CopyUOMTypes()
		{
		
		}

		public bool SupportsCopyUOMTypes()
		{
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void PrintPreview()
		{
			// TODO:  Add UserControlDataBase.PrintPreview implementation
			if(gridLRRDatabase.TableControl.CurrentCell.IsActive)
			{
				GridGroupingPrint ggPrint = new GridGroupingPrint();
				ggPrint.PrintData(gridLRRDatabase,true);
			}

		}

		#endregion

		#region ILRRUIContainer Members

		public void ShowPropertyList()
		{
			// TODO:  Add UserControlDataBase.ShowPropertyList implementation
		}

		public void ShowComponentList()
		{
			// TODO:  Add UserControlDataBase.ShowComponentList implementation
		}

		public bool SupportsResultsFilter()
		{
			// TODO:  Add UserControlDataBase.SupportsResultsFilter implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void LeftColMovement()
		{
			// TODO:  Add UserControlDataBase.LeftColMovement implementation
		}

		public void RightColMovement()
		{
			// TODO:  Add UserControlDataBase.RightColMovement implementation
		}

		#endregion

		#region ILRRUIContainer Members

		public void PageSetup()
		{
			GridGroupingPrint ggPrint = new GridGroupingPrint();
			ggPrint.PageSetUp();
		}

		public void Undo()
		{
			// TODO:  Add UserControlDataBase.Undo implementation
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add UserControlDataBase.SupportsPageSetup implementation
			return true;
		}

		public bool SupportsUndo()
		{
			// TODO:  Add UserControlDataBase.SupportsUndo implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void OpenCompositeView()
		{
			// TODO:  Add UserControlDataBase.OpenCompositeView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add UserControlDataBase.TransposeView implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add UserControlDataBase.OpenQuery implementation
		}

		public void OpenView()
		{
			// TODO:  Add UserControlDataBase.OpenView implementation
		}

		public bool SupportsOpenCompositeView()
		{
			// TODO:  Add UserControlDataBase.SupportsOpenCompositeView implementation
			return false;
		}

		public bool SupportsTransposeView()
		{
			// TODO:  Add UserControlDataBase.SupportsTransposeView implementation
			return false;
		}

		public bool SupportsOpenQuery()
		{
			// TODO:  Add UserControlDataBase.SupportsOpenQuery implementation
			return false;
		}

		public bool SupportsOpenView()
		{
			// TODO:  Add UserControlDataBase.SupportsOpenView implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public bool SupportsCompositeView()
		{
			// TODO:  Add UserControlDataBase.SupportsCompositeView implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public bool PerformDirtyChk
		{
			get
			{
				// TODO:  Add UserControlDataBase.PerformDirtyChk getter implementation
				return true;
			}
		}

		#endregion
	}
}
