using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Syncfusion.Windows.Forms.Grid;
using Symcon.Globals.Enums;
using Symcon.BusinessObjects;
using Syncfusion.Grouping;
using System.Collections.Generic;
using System.Reflection;
using Symcon.LightRidge.HistorianConnect;
using System.Text;
using Symcon.Utils.HeaderDispatcher;
using Symcom.Utils;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
    /// <summary>
    /// Summary description for UserControlDataHistorian.
    /// </summary>
    public class UserControlDataHistorian : System.Windows.Forms.UserControl, ILRRUIContainer
    {
        private Panel panel1;
        private CheckBox chkUseSimulator;
        private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridOnlineAdeprotInfo;
        private Button btnConnectOnlineAdeptor;
        private Panel pnlGrid;
        private Panel pnlButton;

        /// <summary> 
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;

        public UserControlDataHistorian()
        {
            // This call is required by the Windows.Forms Form Designer.
            InitializeComponent();
            _onlineAdaptorConnections = LRRModel.ChildModel.OnlineAdaptorConnections;
            // TODO: Add any initialization after the InitializeComponent call
            InternationalizationUtils.GetControlText(this, null);
            
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitializeOnlineAdeptorGrid();
            if (_onlineAdaptorConnections.Count > 0)
            {
                LRROnlineAdaptorConnection onlineConn = _onlineAdaptorConnections[0];
                if (onlineConn != null)
                {
                    this.chkUseSimulator.Checked = onlineConn.UseSimulator;
                    _lrrApp.GetOptions().UseSimulator = onlineConn.UseSimulator;
                }
            }
            this.chkUseSimulator.Checked = _lrrApp.GetOptions().UseSimulator;
            gridOnlineAdeprotInfo.QueryCellStyleInfo += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableCellStyleInfoEventHandler(gridOnlineAdeprotInfo_QueryCellStyleInfo);
        }

        void gridOnlineAdeprotInfo_QueryCellStyleInfo(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableCellStyleInfoEventArgs e)
        {
            if (e.TableCellIdentity.RowIndex < 0)
                return;
            if (e.TableCellIdentity.Table.TableModel.GetColumnDescriptorAt(e.TableCellIdentity.RowIndex, e.TableCellIdentity.ColIndex) == null)
                return;
            if (e.TableCellIdentity.Table.TableModel.GetColumnDescriptorAt(e.TableCellIdentity.RowIndex, e.TableCellIdentity.ColIndex).HeaderText == LRRHeaderText.GetHeaderString("SID_CONNECTED") && e.TableCellIdentity.RowIndex > 2)
            {
                string displaymember = e.TableCellIdentity.DisplayElement.GetRecord().GetValue("IsConnected") == null ? "" : e.TableCellIdentity.DisplayElement.GetRecord().GetValue("IsConnected").ToString();
                if (displaymember.ToUpper() == "TRUE")
                {
                    e.Style.BackColor = Color.Green;
                    e.Style.Text = "Connected";
                }
                else if (displaymember.ToUpper() == "FALSE")
                {
                    e.Style.BackColor = Color.Red;
                    e.Style.Text = "Disconnected";
                }
            }
        }

        private void InitializeOnlineAdeptorGrid()
        {
            SyncGridUtilities.InitializeGridLookAndFeel(gridOnlineAdeprotInfo);
            Hashtable fkData = new Hashtable();
            Dictionary<string, SymForeignKeyFieldAttribute> fkColumns;
            Dictionary<string, SymChildDataAttribute> childColumns;
            Symcon.LightRidge.BusinessObjects.Utilities.LRRBusinessObjectUtilities.GetObjectInfo(_onlineAdaptorConnections.ItemType, out fkColumns, out childColumns);
            foreach (object obj in fkColumns.Keys)
            {
                string propName = (string)obj;
                SymForeignKeyFieldAttribute fkAttr = fkColumns[propName] as SymForeignKeyFieldAttribute;
                object collectionObject = Activator.CreateInstance(fkAttr.CollectionType);
                _lrrApp.DBManagerObj.PopulateCollection(collectionObject, false);
                fkData.Add(propName, collectionObject);
            }
            SyncGridUtilities.InitializeGridColumns(gridOnlineAdeprotInfo, _onlineAdaptorConnections.ItemType, null, fkData);
            gridOnlineAdeprotInfo.DataSource = _onlineAdaptorConnections;
            gridOnlineAdeprotInfo.SourceListSet.Add("OnlineAdaptorCollection", _onlineAdaptorConnections);

            gridOnlineAdeprotInfo.TableModel.Model.CellModels.Add("EllipsisText", new EllipsisCellModel(gridOnlineAdeprotInfo.TableModel.Model));
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["DatabaseFilePath"].Appearance.AddNewRecordFieldCell.CellType = "TextBox";
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["DatabaseFilePath"].Appearance.AnyRecordFieldCell.CellType = "EllipsisText";
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["DatabaseType"].Appearance.AnyRecordFieldCell.CellType = "ComboBox";
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["DatabaseType"].Appearance.AnyRecordFieldCell.DataSource = _lrrApp.GetOptions().DBTypes;
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["DatabaseFilePath"].Appearance.AnyRecordFieldCell.ShowButtons = GridShowButtons.ShowCurrentCell;
            this.gridOnlineAdeprotInfo.TableOptions.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Any | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.MixRangeType)));
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["PlantTimeZone"].Appearance.AnyRecordFieldCell.CellType = "ComboBox";
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["PlantTimeZone"].Appearance.AnyRecordFieldCell.DataSource = DateUtils.GetTimeZones();
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["HistorianTimeZone"].Appearance.AnyRecordFieldCell.CellType = "ComboBox";
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["HistorianTimeZone"].Appearance.AnyRecordFieldCell.DataSource = DateUtils.GetTimeZones();
            this.gridOnlineAdeprotInfo.TableDescriptor.Columns["PlantTimeZone"].PropertyChanged += new DescriptorPropertyChangedEventHandler(UserControlDataHistorian_PropertyChanged);
        }

        void UserControlDataHistorian_PropertyChanged(object sender, DescriptorPropertyChangedEventArgs e)
        {
            
        }


        private LRRApp _lrrApp;
        private LRRHistorianConnections _historianConnectionObj;
        private LRROnlineAdaptorConnections _onlineAdaptorConnections;

        /// <summary> 
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #region Component Designer generated code
        /// <summary> 
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.panel1 = new System.Windows.Forms.Panel();
            this.chkUseSimulator = new System.Windows.Forms.CheckBox();
            this.btnConnectOnlineAdeptor = new System.Windows.Forms.Button();
            this.gridOnlineAdeprotInfo = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
            this.pnlGrid = new System.Windows.Forms.Panel();
            this.pnlButton = new System.Windows.Forms.Panel();
            this.panel1.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.gridOnlineAdeprotInfo)).BeginInit();
            this.pnlGrid.SuspendLayout();
            this.pnlButton.SuspendLayout();
            this.SuspendLayout();
            // 
            // panel1
            // 
            this.panel1.BackColor = System.Drawing.SystemColors.ActiveCaptionText;
            this.panel1.Controls.Add(this.chkUseSimulator);
            this.panel1.Dock = System.Windows.Forms.DockStyle.Top;
            this.panel1.Location = new System.Drawing.Point(0, 0);
            this.panel1.Name = "panel1";
            this.panel1.Size = new System.Drawing.Size(832, 36);
            this.panel1.TabIndex = 0;
            // 
            // chkUseSimulator
            // 
            this.chkUseSimulator.AutoSize = true;
            this.chkUseSimulator.Location = new System.Drawing.Point(4, 9);
            this.chkUseSimulator.Name = "chkUseSimulator";
            this.chkUseSimulator.Size = new System.Drawing.Size(15, 14);
            this.chkUseSimulator.TabIndex = 0;
            this.chkUseSimulator.Tag = "SID_USES_SIMULATOR";
            this.chkUseSimulator.UseVisualStyleBackColor = true;
            this.chkUseSimulator.CheckedChanged += new System.EventHandler(this.chkUseSimulator_CheckedChanged);
            // 
            // btnConnectOnlineAdeptor
            // 
            this.btnConnectOnlineAdeptor.Location = new System.Drawing.Point(4, 6);
            this.btnConnectOnlineAdeptor.Name = "btnConnectOnlineAdeptor";
            this.btnConnectOnlineAdeptor.Size = new System.Drawing.Size(75, 23);
            this.btnConnectOnlineAdeptor.TabIndex = 0;
            this.btnConnectOnlineAdeptor.Tag = "SID_CONNECT";
            this.btnConnectOnlineAdeptor.UseVisualStyleBackColor = true;
            this.btnConnectOnlineAdeptor.Click += new System.EventHandler(this.btnConnectOnlineAdeptor_Click);
            // 
            // gridOnlineAdeprotInfo
            // 
            this.gridOnlineAdeprotInfo.BackColor = System.Drawing.SystemColors.ActiveCaptionText;
            this.gridOnlineAdeprotInfo.Dock = System.Windows.Forms.DockStyle.Fill;
            this.gridOnlineAdeprotInfo.Location = new System.Drawing.Point(0, 0);
            this.gridOnlineAdeprotInfo.Name = "gridOnlineAdeprotInfo";
            this.gridOnlineAdeprotInfo.Size = new System.Drawing.Size(832, 338);
            this.gridOnlineAdeprotInfo.TabIndex = 1;
            this.gridOnlineAdeprotInfo.TableOptions.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)(((((((((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Row | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Column)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Table)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Cell)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Multiple)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Shift)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Keyboard)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.AlphaBlend)
                        | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.MixRangeType)));
            this.gridOnlineAdeprotInfo.Text = "gridGroupingControl1";
            this.gridOnlineAdeprotInfo.TopLevelGroupOptions.CaptionText = "Online Adaptor Information : {RecordCount} Items";
            this.gridOnlineAdeprotInfo.TopLevelGroupOptions.IsExpandedInitialValue = true;
            this.gridOnlineAdeprotInfo.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;
            this.gridOnlineAdeprotInfo.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
            this.gridOnlineAdeprotInfo.TopLevelGroupOptions.ShowCaptionPlusMinus = false;
            this.gridOnlineAdeprotInfo.TopLevelGroupOptions.ShowColumnHeaders = true;
            this.gridOnlineAdeprotInfo.VersionInfo = "3.301.0.0";
            this.gridOnlineAdeprotInfo.TableControlPrepareViewStyleInfo += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPrepareViewStyleInfoEventHandler(this.gridOnlineAdeprotInfo_TableControlPrepareViewStyleInfo);
            // 
            // pnlGrid
            // 
            this.pnlGrid.Controls.Add(this.gridOnlineAdeprotInfo);
            this.pnlGrid.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlGrid.Location = new System.Drawing.Point(0, 36);
            this.pnlGrid.Name = "pnlGrid";
            this.pnlGrid.Size = new System.Drawing.Size(832, 338);
            this.pnlGrid.TabIndex = 3;
            // 
            // pnlButton
            // 
            this.pnlButton.Controls.Add(this.btnConnectOnlineAdeptor);
            this.pnlButton.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.pnlButton.Location = new System.Drawing.Point(0, 374);
            this.pnlButton.Name = "pnlButton";
            this.pnlButton.Size = new System.Drawing.Size(832, 34);
            this.pnlButton.TabIndex = 2;
            // 
            // UserControlDataHistorian
            // 
            this.Controls.Add(this.pnlGrid);
            this.Controls.Add(this.pnlButton);
            this.Controls.Add(this.panel1);
            this.Name = "UserControlDataHistorian";
            this.Size = new System.Drawing.Size(832, 408);
            this.panel1.ResumeLayout(false);
            this.panel1.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.gridOnlineAdeprotInfo)).EndInit();
            this.pnlGrid.ResumeLayout(false);
            this.pnlButton.ResumeLayout(false);
            this.ResumeLayout(false);

        }
        #endregion

        private void chkUseSimulator_CheckedChanged(object sender, EventArgs e)
        {
            if (this.chkUseSimulator.Checked)
            {
                _lrrApp.GetOptions().UseSimulator = true;
                SetUI(false);
            }
            else
            {
                _lrrApp.GetOptions().UseSimulator = false;
                SetUI(true);
            }
        }

        private void SetUI(bool useSimulator)
        {
            this.btnConnectOnlineAdeptor.Enabled = useSimulator;
        }

        private void gridHistConnectionInfo_TableControlPrepareViewStyleInfo(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPrepareViewStyleInfoEventArgs e)
        {
            if (this.chkUseSimulator.Checked)
            {
                e.Inner.Style.ReadOnly = true;
            }
            else
            {
                e.Inner.Style.ReadOnly = false;
            }
        }

        private void gridOnlineAdeprotInfo_TableControlPrepareViewStyleInfo(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPrepareViewStyleInfoEventArgs e)
        {
            if (this.chkUseSimulator.Checked)
            {
                e.Inner.Style.ReadOnly = true;
            }
            else
            {
                e.Inner.Style.ReadOnly = false;
            }
        }

        private void gridHistConnectionInfo_TableControlPrepareViewStyleInfo_1(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPrepareViewStyleInfoEventArgs e)
        {
            if (this.chkUseSimulator.Checked)
            {
                e.Inner.Style.ReadOnly = true;
            }
            else
            {
                e.Inner.Style.ReadOnly = false;
            }
        }
        
        private void btnConnectOnlineAdeptor_Click(object sender, EventArgs e)
        {
            try
            {
                bool isInitialized = InitalizeOnlineConnection();
                if (isInitialized)
                {
                    MessageBox.Show("Connection Sucessful");
                }
                else
                {
                    MessageBox.Show("Connection Falied, Please Check the information Entered");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Connection Falied, Please Check the information Entered");
            }
        }

        private bool InitalizeOnlineConnection()
        {
            bool isConnected = false;
            LRRApp app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            Record currntRecord = this.gridOnlineAdeprotInfo.TableControl.Table.CurrentRecord;
            Dictionary<string, IHistorian> loadedAssemblies = app.LoadedOnLineHistorianAssemblies;
            if (currntRecord == null)
            {
                MessageBox.Show("Please select one record.");
            }
            else
            {
                LRROnlineAdaptorConnection Obj = currntRecord.GetData() as LRROnlineAdaptorConnection;
                LRROnlineAdaptorConnection onlineAdeptorObj = _onlineAdaptorConnections.GetOnlineAdaptorByID(Obj.ConnectID);  
                if (onlineAdeptorObj != null)
                {
                    LRRHistTagSourceInfos sourceInfoColl = LRRModel.ChildModel.TagSourceInfos;
                    string typeofAdapter = sourceInfoColl.GetSourceInfoByID(onlineAdeptorObj.SourceType).TypeName;
                    string assemblyName = sourceInfoColl.GetSourceInfoByID(onlineAdeptorObj.SourceType).AssemblyName;
                    if (loadedAssemblies.ContainsKey(onlineAdeptorObj.ConnectID))
                    {
                        if (DialogResult.Yes != MessageBox.Show(this, "Do You wish to Reset Previous Connection", "Connection Already Exists", MessageBoxButtons.YesNo))
                        {
                            return true;
                        }
                        else
                        {
                            IHistorian iHist = loadedAssemblies[onlineAdeptorObj.ConnectID];
                            iHist.Initailize(onlineAdeptorObj, Application.StartupPath);
                            if (iHist.Initialized)
                            {
                                onlineAdeptorObj.IsConnected = "TRUE";
                                isConnected = true;
                            }
                            else
                            {
                                onlineAdeptorObj.IsConnected = "FALSE";
                            }
                        }
                    }
                    else
                    {
                        Assembly asm = Assembly.LoadFrom(Application.StartupPath + "\\" +assemblyName);
                        IHistorian iHist = (IHistorian)Activator.CreateInstance(asm.GetType(typeofAdapter));
                        iHist.Initailize(onlineAdeptorObj,Application.StartupPath);
                        if (iHist.Initialized)
                        {
                            isConnected = true;
                            onlineAdeptorObj.IsConnected = "TRUE";
                            loadedAssemblies.Add(onlineAdeptorObj.ConnectID, iHist);
                        }
                        else
                        {
                            onlineAdeptorObj.IsConnected = "FALSE";
                        }
                    }
                }
            }
            return isConnected;
        }

        private void InitalizeOnlineConnection(LRROnlineAdaptorConnection onlineAdeptorObj)
        {
            bool isConnected = false;
            LRRApp app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            Dictionary<string, IHistorian> loadedAssemblies = app.LoadedOnLineHistorianAssemblies;
            LRRHistTagSourceInfos sourceInfoColl = LRRModel.ChildModel.TagSourceInfos;
            string typeofAdapter = sourceInfoColl.GetSourceInfoByID(onlineAdeptorObj.SourceType).TypeName;
            string assemblyName = sourceInfoColl.GetSourceInfoByID(onlineAdeptorObj.SourceType).AssemblyName;
            if (loadedAssemblies.ContainsKey(onlineAdeptorObj.ConnectID))
            {
                IHistorian iHist = loadedAssemblies[onlineAdeptorObj.ConnectID];
                iHist.Initailize(onlineAdeptorObj, Application.StartupPath);
                if (iHist.Initialized)
                {
                    onlineAdeptorObj.IsConnected = "TRUE";
                }
                else
                {
                    onlineAdeptorObj.IsConnected = "FALSE";
                }
            }
            else
            {
                Assembly asm = Assembly.LoadFrom(Application.StartupPath + "\\" + assemblyName);
                IHistorian iHist = (IHistorian)Activator.CreateInstance(asm.GetType(typeofAdapter));
                iHist.Initailize(onlineAdeptorObj, Application.StartupPath);
                if (iHist.Initialized)
                {
                    onlineAdeptorObj.IsConnected = "TRUE";
                    loadedAssemblies.Add(onlineAdeptorObj.ConnectID, iHist);
                }
                else
                {
                    onlineAdeptorObj.IsConnected = "FALSE";
                }
            }
        }


        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Copy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyMasterProperties()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Delete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DownLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Export()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void InsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenQuery()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public TabControl ParentTabControl
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        public TabPage ParentTabPage
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        public void Paste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool PerformDirtyChk
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public void Print()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PrintPreview()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Save()
        {
            bool isSaved = false;
            _lrrApp.GetOptions().UseSimulator = this.chkUseSimulator.Checked;  
            foreach (LRROnlineAdaptorConnection onlineAdaptorConn in _onlineAdaptorConnections)
            {
                if (onlineAdaptorConn.UseSimulator != _lrrApp.GetOptions().UseSimulator)
                {
                    onlineAdaptorConn.UseSimulator = _lrrApp.GetOptions().UseSimulator;
                }
            }
            if (_onlineAdaptorConnections.GetChanges().Count > 0)
            {
                isSaved = true;
                foreach (LRROnlineAdaptorConnection onlineConnObj in _onlineAdaptorConnections)
                {
                    if (onlineConnObj.ObjectState == DBOperation.UPDATE || onlineConnObj.ObjectState == DBOperation.INSERT)
                    {
                        InitalizeOnlineConnection(onlineConnObj);
                    }
                }
                _lrrApp.DBManagerObj.CommitCollection(_onlineAdaptorConnections.GetChanges());
                _onlineAdaptorConnections.RefreshCollection();
            }
            OptionsUserControl.saved = isSaved;
        }

        public void SelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowComponentList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowPropertyList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsCopy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsCopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsDelete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsExport()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsInsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsMasterPropertiesCopy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsPageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsPrint()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsRefresh()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsResultsFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsSave()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsSelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsUndo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void TransposeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Undo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}
