using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.HeaderDispatcher;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
    public partial class UserControlExcelNamedRangeConfig : UserControl
    {
        #region Constructor

        public UserControlExcelNamedRangeConfig()
        {
            InitializeComponent();

            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
        }

        #endregion

        #region Fields

        private LRRApp _app = null;

        #endregion

        #region Public Methods

        public LRRChart AddNewExcelView()
        {
            if (LRRModel.ChildModel.ExcelCharts.GetChartFromDisplayName(textBoxDisplayName.Text) != null)
            {
                textBoxDisplayName.Text = "";
                throw new DuplicateNameException(LRRHeaderText.GetHeaderString("SID_EXCEL_VIEW_EXISTS"));
            }

            LRRExcelView lrrExcelView = new LRRExcelView();
            LRRChart lrrChart = new LRRChart();

            lrrExcelView.ViewID = lrrChart.ID;
            lrrExcelView.ViewType = "Misc";

            UpdateChartObject(lrrChart);

            lrrChart.DataType = "TabularData";

            lrrExcelView.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            lrrChart.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;

            LRRModel.ChildModel.ExcelCharts.Add(lrrChart);
            LRRModel.ChildModel.ExcelViews.Add(lrrExcelView);

            return lrrChart;
        }

        public void Reset()
        {
            textBoxDisplayName.ReadOnly = false;

            textBoxWorkBookName.Text = string.Empty;
            labelBackColor.BackColor = Color.RoyalBlue;
            textBoxChartName.Text = string.Empty;
            textBoxDisplayName.Text = string.Empty;
            textBoxDescription.Text = string.Empty;
        }

        public void PopulateChart(LRRChart lrrChart)
        {
            textBoxWorkBookName.Text = lrrChart.WorkBookName;
            
            labelBackColor.BackColor = Color.RoyalBlue;

            try
            {
                string[] srgb = lrrChart.SheetName.Split(new char[] { ',' });
                if (srgb.Length == 3)
                {
                    labelBackColor.BackColor = Color.FromArgb(255, int.Parse(srgb[0].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(), Symcon.BusinessObjects.SymConverter.Culture), int.Parse(srgb[1].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(), Symcon.BusinessObjects.SymConverter.Culture), int.Parse(srgb[2].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(), Symcon.BusinessObjects.SymConverter.Culture));
                }
            }
            catch
            { 
            
            }

            textBoxChartName.Text = lrrChart.ChartName;
            
            textBoxDisplayName.ReadOnly = false;
            textBoxDisplayName.Text = lrrChart.DisplayName;
            textBoxDisplayName.ReadOnly = true;

            textBoxDescription.Text = lrrChart.Description;
        }

        public void UpdateChartObject(LRRChart lrrChart)
        {
            lrrChart.WorkBookName = textBoxWorkBookName.Text;
            lrrChart.SheetName = labelBackColor.BackColor.R.ToString() + "," + labelBackColor.BackColor.G.ToString() + "," + labelBackColor.BackColor.B.ToString();
            lrrChart.ChartName = textBoxChartName.Text;
            lrrChart.DisplayName = textBoxDisplayName.Text;
            lrrChart.Description = textBoxDescription.Text;
        }

        #endregion

        #region Event Handlers

        private void buttonBackColor_Click(object sender, EventArgs e)
        {
            if (colorDialog1.ShowDialog() == DialogResult.OK)
            {
                labelBackColor.BackColor = colorDialog1.Color;
            }
        }

        #endregion
    }
}
