using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Symcon.LightRidge.PEAdvisor;
using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
    public partial class UserControlExcelViewConfig : UserControl, ILRRUIContainer
    {
        #region Constructor

        public UserControlExcelViewConfig()
        {
            InitializeComponent();

            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

            InternationalizationUtils.GetControlText(this, null);
            InternationalizationUtils.GetControlText(_userControlCharSheetsConfig1, null);
            InternationalizationUtils.GetControlText(_userControlExcelNamedRangeConfig1, null);

            LRRModel.ChildModel.ExcelCharts.SortColumn = "ID";
            LRRModel.ChildModel.ExcelCharts.Sort();

            BindCombo();

            buttonDelete.Enabled = false;

            comboBoxExcelViews.SelectedIndexChanged += new EventHandler(comboBoxExcelViews_SelectedIndexChanged);

        }

        #endregion

        #region Fields

        private LRRApp _app = null;
        private UserControlChartSheetsConfig _userControlCharSheetsConfig1 = new UserControlChartSheetsConfig();
        private UserControlExcelNamedRangeConfig _userControlExcelNamedRangeConfig1 = new UserControlExcelNamedRangeConfig();

        #endregion

        #region Private Methods

        private void BindCombo()
        {
            comboBoxExcelViews.DataSource = LRRModel.ChildModel.ExcelCharts;
            comboBoxExcelViews.DisplayMember = "DisplayName";
            comboBoxExcelViews.ValueMember = "ID";
        }

        #endregion

        #region Event Handlers

        private void comboBoxExcelViews_SelectedIndexChanged(object sender, EventArgs e)
        {
            LRRChart lrrChart = comboBoxExcelViews.SelectedItem as LRRChart;

            if (lrrChart.ID == "0") // Case of new Excel View
            {
                buttonDelete.Enabled = false;

                userControlChartsConfig1.Reset();
                _userControlCharSheetsConfig1.Reset();
                _userControlExcelNamedRangeConfig1.Reset();

                radioButtonChart.Enabled = true;
                radioButtonChartSheet.Enabled = true;
                radioButtonNameRange.Enabled = true;
            }
            else // Case of existing Excel View
            {
                buttonDelete.Enabled = true;

                radioButtonChart.Enabled = false;
                radioButtonChartSheet.Enabled = false;
                radioButtonNameRange.Enabled = false;

                if (lrrChart.DataType == "ChartData") // Case of Chart or Chart Sheet
                {
                    if ((lrrChart.SheetName == null) || (lrrChart.SheetName.Trim() == string.Empty)) // Case of Chart Sheet
                    {
                        _userControlCharSheetsConfig1.PopulateChart(lrrChart);
                        radioButtonChartSheet.Checked = true;
                    }
                    else // Case of normal chart
                    {
                        userControlChartsConfig1.PopulateChart(lrrChart);
                        radioButtonChart.Checked = true;
                    }
                }
                else if (lrrChart.DataType == "TabularData") // Case of Named Range
                {
                    _userControlExcelNamedRangeConfig1.PopulateChart(lrrChart);
                    radioButtonNameRange.Checked = true;
                }

            }
        }

        private void radioButtonChart_CheckedChanged(object sender, EventArgs e)
        {
            bool isChart = radioButtonChart.Checked;

            if (isChart)
            {
                radioButtonChartSheet.Checked = false;
                radioButtonNameRange.Checked = false;

                panelUCContainer.Controls.Clear();
                userControlChartsConfig1.Dock = DockStyle.Top;
                panelUCContainer.Controls.Add(userControlChartsConfig1);
            }
        }

        private void radioButtonChartSheet_CheckedChanged(object sender, EventArgs e)
        {
            bool isChartSheet = radioButtonChartSheet.Checked;

            if (isChartSheet)
            {
                radioButtonChart.Checked = false;
                radioButtonNameRange.Checked = false;

                panelUCContainer.Controls.Clear();
                _userControlCharSheetsConfig1.Dock = DockStyle.Top;
                panelUCContainer.Controls.Add(_userControlCharSheetsConfig1);
            }
        }

        private void radioButtonNameRange_CheckedChanged(object sender, EventArgs e)
        {
            bool isNameRange = radioButtonNameRange.Checked;

            if (isNameRange)
            {
                radioButtonChartSheet.Checked = false;
                radioButtonChart.Checked = false;

                panelUCContainer.Controls.Clear();
                userControlChartsConfig1.Dock = DockStyle.Top;
                panelUCContainer.Controls.Add(_userControlExcelNamedRangeConfig1);
            }
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            if (LRRMessagBox.ShowMessage(this, "SID_EXCEL_VIEW_DELETE_CONFIRM", "SID_WARNING", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                LRRChart lrrChart = comboBoxExcelViews.SelectedItem as LRRChart;

                LRRExcelView lrrExcelView = LRRModel.ChildModel.ExcelViews.GetExcelViewFromViewID(lrrChart.ID);

                if (lrrChart.DataType == "ChartData")
                {
                    _app.MainFrame.RemoveChartMenuItem(lrrExcelView.ID);
                }
                else if (lrrChart.DataType == "TabularData")
                {
                    _app.MainFrame.RemoveMiscMenuItem(lrrExcelView.ID);
                }

                LRRModel.ChildModel.ExcelViews.Remove(lrrExcelView);
                LRRModel.ChildModel.ExcelCharts.Remove(lrrChart);

                lrrChart.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                lrrExcelView.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;

                _app.DBManagerObj.CommitObject(lrrChart);
                _app.DBManagerObj.CommitObject(lrrExcelView);

                lrrChart = comboBoxExcelViews.SelectedItem as LRRChart;

                comboBoxExcelViews_SelectedIndexChanged(lrrChart, new EventArgs());
            }
        }

        #endregion

        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Copy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyMasterProperties()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Delete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DownLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Export()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void InsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenQuery()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public TabControl ParentTabControl
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        public TabPage ParentTabPage
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        public void Paste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool PerformDirtyChk
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public void Print()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PrintPreview()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Save()
        {
            try
            {
                LRRChart lrrChart = comboBoxExcelViews.SelectedItem as LRRChart;

                if (lrrChart.ID == "0") // Case of new Excel View
                {
                    if (radioButtonChart.Checked) // Case of Chart
                    {
                        lrrChart = userControlChartsConfig1.AddNewExcelView();
                        _app.MainFrame.AddChartMenuItem(lrrChart);
                    }
                    else if (radioButtonChartSheet.Checked) // Case of Chart Sheet
                    {
                        lrrChart = _userControlCharSheetsConfig1.AddNewExcelView();
                        _app.MainFrame.AddChartMenuItem(lrrChart);
                    }
                    else if (radioButtonNameRange.Checked) // Case of Name Range
                    {
                        lrrChart = _userControlExcelNamedRangeConfig1.AddNewExcelView();
                        _app.MainFrame.AddMiscMenuItem(lrrChart);
                    }

                    comboBoxExcelViews.SelectedItem = lrrChart;
                }
                else
                {
                    if (radioButtonChart.Checked) // Case of Chart
                    {
                        userControlChartsConfig1.UpdateChartObject(lrrChart);
                    }
                    else if (radioButtonChartSheet.Checked) // Case of Chart Sheet
                    {
                        _userControlCharSheetsConfig1.UpdateChartObject(lrrChart);
                    }
                    else if (radioButtonNameRange.Checked) // Case of Name Range
                    {
                        _userControlExcelNamedRangeConfig1.UpdateChartObject(lrrChart);
                    }
                }

                LRRCharts lrrCharts = LRRModel.ChildModel.ExcelCharts.GetChanges();
                LRRExcelViews lrrExcelViews = LRRModel.ChildModel.ExcelViews.GetChanges();

                if (lrrCharts.Count > 0)
                {
                    _app.DBManagerObj.CommitCollection(lrrCharts);
                }

                if (lrrCharts.Count > 0)
                {
                    _app.DBManagerObj.CommitCollection(lrrExcelViews);
                }

                LRRModel.ChildModel.ExcelCharts.RefreshCollection();
                LRRModel.ChildModel.ExcelViews.RefreshCollection();
            }
            catch (DuplicateNameException ex)
            {
                MessageBox.Show(this, ex.Message, LRRHeaderText.GetHeaderString("SID_ERROR"), MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void SelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowComponentList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowPropertyList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsCopy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsCopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsDelete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsExport()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsInsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsMasterPropertiesCopy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsPageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsPrint()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsRefresh()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsResultsFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsSave()
        {
            return true;
        }

        public bool SupportsSelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsUndo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void TransposeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Undo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}
