using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.Options;
using Symcon.Utils.HeaderDispatcher; 
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;  

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
	/// <summary>
	/// Summary description for UserControlHistorianTextCollection.
	/// </summary>
	public class UserControlHistorianTextCollection : System.Windows.Forms.UserControl,ILRRUIContainer
	{
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridHistTexts;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private GridExcelFormatter   _gridExcelFormatter = new GridExcelFormatter();

		public UserControlHistorianTextCollection()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitializeComponent call
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			_lrrOptionsObj = _app.GetOptions() as LRROptions;
			_gridExcelFormatter.WireGrid(this.gridHistTexts);
			InitDataSource();
			BindGridHistTexts();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.gridHistTexts = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			((System.ComponentModel.ISupportInitialize)(this.gridHistTexts)).BeginInit();
			this.SuspendLayout();
			// 
			// gridHistTexts
			// 
			this.gridHistTexts.BackColor = System.Drawing.SystemColors.Window;
			this.gridHistTexts.Location = new System.Drawing.Point(128, 72);
			this.gridHistTexts.Name = "gridHistTexts";
			this.gridHistTexts.Size = new System.Drawing.Size(352, 224);
			this.gridHistTexts.TabIndex = 0;
			this.gridHistTexts.TableOptions.AllowSelection = Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Row;
			this.gridHistTexts.Text = "gridGroupingControl1";
			this.gridHistTexts.VersionInfo = "3.301.0.0";
			this.gridHistTexts.RecordDeleting += new Syncfusion.Grouping.RecordEventHandler(this.gridHistTexts_RecordDeleting);
			// 
			// UserControlHistorianTextCollection
			// 
			this.Controls.Add(this.gridHistTexts);
			this.Name = "UserControlHistorianTextCollection";
			this.Size = new System.Drawing.Size(632, 408);
			this.Tag = "SID_HISTTEXT";
			((System.ComponentModel.ISupportInitialize)(this.gridHistTexts)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		#region Fields

		private LRRApp						_app;
		private HistorianTexts				_histTexts;
		public LRROptions					_lrrOptionsObj;

		#endregion

		#region Private Methods

		private void InitDataSource()
		{
			_histTexts = new HistorianTexts();
			foreach(string histText in _lrrOptionsObj.HistTextCollection)
			{
				HistorianText histTextObj = new HistorianText();
				histTextObj.HistText = histText;
				_histTexts.Add(histTextObj);
			}
			_histTexts.RefreshCollection();
		}
		
		private void BindGridHistTexts()
		{
			gridHistTexts.SuspendLayout();
			Symcon.LightRidge.PEAdvisor.GridUtilities.SyncGridUtilities.InitializeGridLookAndFeel(gridHistTexts);

			gridHistTexts.TableDescriptor.Columns.Clear();
			gridHistTexts.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_HIST_TEXT"),"HistText");

			gridHistTexts.TopLevelGroupOptions.CaptionText = LRRHeaderText.GetHeaderString("SID_HISTORIAN_TEXT") + "{RecordCount}" + LRRHeaderText.GetHeaderString("SID_ITEMS");   
			gridHistTexts.TableDescriptor.Columns[LRRHeaderText.GetHeaderString("SID_HIST_TEXT")].AllowFilter	= true;

			gridHistTexts.NestedTableGroupOptions.ShowFilterBar = true;
			gridHistTexts.TableDescriptor.AllowRemove = true;
			gridHistTexts.TableDescriptor.AllowNew = true;
			gridHistTexts.TableDescriptor.AllowEdit = true;

			gridHistTexts.DataSource = _histTexts;
			this.gridHistTexts.TableOptions.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Any | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.MixRangeType)));
			gridHistTexts.ResumeLayout(false);
		}
		#endregion

		#region ILRRUIContainer Members

		public new void Refresh()
		{
			InitDataSource();
			BindGridHistTexts();
		}

		public void Save()
		{
			if(_histTexts.GetChanges().Count>0)
			{
				_lrrOptionsObj.HistTextCollection = new string[_histTexts.Count];
				int i=0;
				ArrayList arrHistTexts = new ArrayList();
				foreach(HistorianText histText in _histTexts)
				{
					_lrrOptionsObj.HistTextCollection[i] = histText.HistText;
					arrHistTexts.Add(histText.HistText);
					i++;
				}
				_histTexts.RefreshCollection();
				Symcon.LightRidge.HistorianConnect.HistorianUtilities.ArrHistTexts = arrHistTexts;
				OptionsUserControl.saved = true;
				BindGridHistTexts();
			}
		}

		public void CopyMasterProperties()
		{
		
		}

		public void Paste()
		{
			// TODO:  Add UserControlHistorianTextCollection.Paste implementation
		}

		public void Copy()
		{
			// TODO:  Add UserControlHistorianTextCollection.Copy implementation
		}

		public void Print()
		{
			// TODO:  Add UserControlHistorianTextCollection.Print implementation
			if(gridHistTexts.TableControl.CurrentCell.IsActive)
			{
				GridGroupingPrint ggPrint = new GridGroupingPrint();
				ggPrint.PrintData(gridHistTexts,false);
			}
		}

		public void Delete()
		{
			// TODO:  Add UserControlHistorianTextCollection.Delete implementation
		}

		public void ArrangeColumns()
		{
			// TODO:  Add UserControlHistorianTextCollection.ArrangeColumns implementation
		}

		public void ClearFilter()
		{
			
		}
		public void ImportToExcel()
		{
		}
		public bool SupportsSelectionPaste()
		{
			return true;
		}

		public void SelectionPaste()
		{
		}

		public  void RightLevel()
		{

		}
		public  void LeftLevel()
		{

		}
		public  void UpLevel()
		{

		}
		public  void DownLevel()
		{

		}

		public bool SupportsCopy()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsCopy implementation
			return false;
		}

		public bool SupportsPaste()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsPaste implementation
			return true;
		}

		public bool SupportsExport()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsExport implementation
			return true;
		}

		public bool SupportsSave()
		{
			// TODO:  Add ComplexEquip.SupportsSave implementation
			return true;
		}

		public bool SupportsRefresh()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsRefresh implementation
			return true;
		}

		public bool SupportsDelete()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsDelete implementation
			return false;
		}

		public bool SupportsPrint()
		{
			// TODO:  Add ComplexEquip.SupportsPrint implementation
			return true;
		}

		public bool SupportsArrangeColumns()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsArrangeColumns implementation
			return false;
		}

		public bool SupportsClearFilter()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsClearFilter implementation
			return true;
		}
		public bool SupportsMasterPropertiesCopy()
		{
			return false;
		}
		public bool SupportsInsertNewRows()
		{
			return true;
		}

		public bool SupportsImportToExcel()
		{
			return true;
		}

		public bool SupportsLevel()
		{
			return false;
		}

		public void InsertNewRows()
		{
			
		}

		public void Export()
		{
			
		}
	
		public TabControl	ParentTabControl
		{
			set
			{
				
			}
			get{return null;}
		}
	
		public TabPage ParentTabPage
		{
			set
			{
			}
			get
			{
				return null;
			}
		}
		#endregion

		#region ILRRUIContainer Members

		public void ClearSort()
		{
			// TODO:  Add UserControlHistorianTextCollection.ClearSort implementation
			this.gridHistTexts.TableDescriptor.SortedColumns.Clear();
		}

		public bool SupportsClearSort()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsClearSort implementation
			return true;
		}

		#endregion

		#region ILRRUIContainer Members

		public void CopyUOMTypes()
		{
		
		}

		public bool SupportsCopyUOMTypes()
		{
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void PrintPreview()
		{
			// TODO:  Add UserControlHistorianTextCollection.PrintPreview implementation
			if(gridHistTexts.TableControl.CurrentCell.IsActive)
			{
				GridGroupingPrint ggPrint = new GridGroupingPrint();
				ggPrint.PrintData(gridHistTexts,true);
			}
		}

		#endregion

		#region ILRRUIContainer Members

		public void ShowPropertyList()
		{
			// TODO:  Add UserControlHistorianTextCollection.ShowPropertyList implementation
		}

		public void ShowComponentList()
		{
			// TODO:  Add UserControlHistorianTextCollection.ShowComponentList implementation
		}

		public bool SupportsResultsFilter()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsResultsFilter implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void LeftColMovement()
		{
			// TODO:  Add UserControlHistorianTextCollection.LeftColMovement implementation
		}

		public void RightColMovement()
		{
			// TODO:  Add UserControlHistorianTextCollection.RightColMovement implementation
		}

		#endregion

		#region ILRRUIContainer Members

		public void PageSetup()
		{
			GridGroupingPrint ggPrint = new GridGroupingPrint();
			ggPrint.PageSetUp();
		}

		public void Undo()
		{
			// TODO:  Add UserControlHistorianTextCollection.Undo implementation
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsPageSetup implementation
			return true;
		}

		public bool SupportsUndo()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsUndo implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public void OpenCompositeView()
		{
			// TODO:  Add UserControlHistorianTextCollection.OpenCompositeView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add UserControlHistorianTextCollection.TransposeView implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add UserControlHistorianTextCollection.OpenQuery implementation
		}

		public void OpenView()
		{
			// TODO:  Add UserControlHistorianTextCollection.OpenView implementation
		}

		public bool SupportsOpenCompositeView()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsOpenCompositeView implementation
			return false;
		}

		public bool SupportsTransposeView()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsTransposeView implementation
			return false;
		}

		public bool SupportsOpenQuery()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsOpenQuery implementation
			return false;
		}

		public bool SupportsOpenView()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsOpenView implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public bool SupportsCompositeView()
		{
			// TODO:  Add UserControlHistorianTextCollection.SupportsCompositeView implementation
			return false;
		}

		#endregion

		protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
		{
			if(keyData == Keys.Delete)
			{
				_isDeletePress = true;
			}
			return base.ProcessCmdKey (ref msg, keyData);
		}

		private bool _isDeletePress = false;
		private bool _isRecordeDel = false;
		private void gridHistTexts_RecordDeleting(object sender, Syncfusion.Grouping.RecordEventArgs e)
		{
			if(_isDeletePress)
			{
				DialogResult _dlgResult = LRRMessagBox.ShowMessage("SID_DELETE_RECORD","SID_LABEL",MessageBoxButtons.YesNo,MessageBoxIcon.Question);     
				if(_dlgResult == DialogResult.Yes)
				{
					_isRecordeDel = true; 
				}
				else
				{
					_isRecordeDel = false; 
				}
				_isDeletePress = false;
			}
			if(!_isRecordeDel)
			{
				e.Cancel = true;
				return;
			}
		}

		#region ILRRUIContainer Members

		public bool PerformDirtyChk
		{
			get
			{
				// TODO:  Add UserControlHistorianTextCollection.PerformDirtyChk getter implementation
				return true;
			}
		}

		#endregion
	}
	public class HistorianText:BaseObject
	{
		#region Properties

		public string HistText
		{
			get { return _histText; }
			set 
			{ 
				_histText = value; 
				SetDirtyFlag("HistText");
			}
		}

		#endregion

		#region Public Methods

		public override void CopyRecord(IDataRecord drec)
		{
			if(drec["HistText"] != DBNull.Value)
				this.HistText = (string)drec["HistText"];
		}

		public override void SetData(System.Data.IDataParameterCollection IParams)
		{
			foreach(System.Data.IDataParameter iParam in IParams)
			{
				string paramName = iParam.ParameterName.Replace("@","");
				switch(paramName)
				{
					case "HistText":
						iParam.Value = this.HistText;
						break;
				}
				if(iParam.Value == null)
					iParam.Value = DBNull.Value;
			}
		}

		#endregion

		#region Fields

		private string _histText;

		#endregion
	}
	public class HistorianTexts:CustomCollection
	{
		#region Constructor

		public HistorianTexts()
		{
			this.ItemType = typeof(HistorianText);
		}

		#endregion

		#region Public Methods

		public new HistorianText this[int index] 
		{
			get 
			{
				return (HistorianText)(base[index]);
			}
			set 
			{
				base[index] = value;
			}
		}
		public new HistorianTexts GetChanges()
		{
			return (HistorianTexts) base.GetChanges();
		}

		#endregion
	}
}
