using System;
using System.Collections;

namespace Symcon.LightRidge.PEAdvisor.CommonViews
{
	/// <summary>
	/// Summary description for CellHeader.
	/// </summary>
	public class CellHeader
	{
		public CellHeader()
		{
		}

		#region Properties
        /// <summary>
        /// Gets or sets the search criteria obj.
        /// </summary>
        /// <value>The search criteria obj.</value>
		public SearchCriteria SearchCriteriaObj
		{
			get{return _searchCriteria;}
			set
			{
				_searchCriteria = value;
			}
		}

        /// <summary>
        /// Gets or sets the mode ID.
        /// </summary>
        /// <value>The mode ID.</value>
		public string ModeID
		{
			get{return _modeID;}
			set
			{
				_modeID = value;
			}
		}

        /// <summary>
        /// Gets or sets the UIUOM id.
        /// </summary>
        /// <value>The UIUOM id.</value>
		public string UIUOMId
		{
			get{return _uiUOMID;}
			set
			{
				_uiUOMID = value;
			}
		}

        /// <summary>
        /// Gets or sets the UOM id.
        /// </summary>
        /// <value>The UOM id.</value>
		public string UOMId
		{
			get{return _UOMID;}
			set
			{
				_UOMID = value;
			}
		}

        /// <summary>
        /// Gets or sets the display name of the UOM.
        /// </summary>
        /// <value>The display name of the UOM.</value>
		public string UOMDisplayName
		{
			get{return _uomDisplayName;}
			set
			{
				_uomDisplayName = value;
			}
		}

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
		public string Name
		{
			get{return _name;}
			set
			{
				_name = value;
			}
		}

        /// <summary>
        /// Gets or sets the ID.
        /// </summary>
        /// <value>The ID.</value>
		public string ID
		{
			get{return _id;}
			set
			{
				_id = value;
			}
		}

        /// <summary>
        /// Gets or sets the UOM type ID.
        /// </summary>
        /// <value>The UOM type ID.</value>
		public string UOMTypeID
		{
			get{return _uomTypeID;}
			set
			{
				_uomTypeID = value;
			}
		}

        /// <summary>
        /// Gets or sets a value indicating whether this instance is visible.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is visible; otherwise, <c>false</c>.
        /// </value>
		public bool IsVisible
		{
			get{return _isVisible;}
			set
			{
				_isVisible = value;
			}
		}

		#endregion

		#region Fields
		private SearchCriteria _searchCriteria;
		private string _modeID;
		private string _uiUOMID;
		private string _UOMID;
		private string _uomDisplayName;
		private string _name;
		private string _id;
		private string _uomTypeID;
		private bool _isVisible = true;
		#endregion
	}

   
	public class Comparer : IComparer
	{

        /// <summary>
        /// Column(public property of the class) to be sorted.
        /// </summary>
        /// <value>The sort column.</value>
		public string SortColumn
		{
			get{return sortcolumn;}set{sortcolumn = value;}
		}
		
		/// <summary>
		/// Sorting order.
		/// </summary>
		public int SortingOrder
		{
			get{return sortingOrder;}set{sortingOrder = value;}
		}
		#region Fields
		public string sortcolumn = "";
		public int sortingOrder = 0;
		#endregion

		#region Methods
		/// <summary>
		/// Compare interface implementation
		/// </summary>
		/// <param name="x">Object 1</param>
		/// <param name="y">Object 2</param>
		/// <returns>Result of comparison</returns>
		public int Compare(object x, object y)
		{
			return CheckSort(x,y,0);
		}

        /// <summary>
        /// Checks the sort.
        /// </summary>
        /// <param name="x">The x.</param>
        /// <param name="y">The y.</param>
        /// <param name="sortLevel">The sort level.</param>
        /// <returns></returns>
		public int CheckSort(object x,object y, int sortLevel)
		{
			CellHeader obj1 = (CellHeader)x;
			CellHeader obj2 = (CellHeader)y;

			//Compare based on the sorting order.
			int result = -2;
			if(sortLevel == 0)
			{
				if(sortingOrder == 0)
				{
					switch(sortcolumn)
					{
						case "Name" :
							result = obj1.Name.CompareTo(obj2.Name);
							break;
						case "UOMName":
							result = obj1.UOMDisplayName.CompareTo(obj2.UOMDisplayName);
							break;
					}
				}
				if(sortingOrder == 1)
				{
					switch(sortcolumn)
					{
						case "Name" :
							result =  obj2.Name.CompareTo(obj1.Name);
							break;
						case "UOMName":
							result = obj2.UOMDisplayName.CompareTo(obj1.UOMDisplayName);
							break;
					}
				}
			}

			return result;
		}
		#endregion
	}
}
