using System;
using System.Collections;

using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;

namespace Symcon.LightRidge.PEAdvisor.CommonViews
{
	/// <summary>
	/// Summary description for CommonViewUtilities.
	/// </summary>
	public class CommonViewUtilities
	{
		public CommonViewUtilities()
		{
		}
		
		#region Methods
        /// <summary>
        /// Gets the propeties.
        /// </summary>
        /// <param name="arrListProps">The arr list props.</param>
        /// <returns></returns>
		public static LRRResultProperties GetPropeties(ArrayList arrListProps)
		{
			LRRResultProperties lrrResultProps = new LRRResultProperties();
			for(int i=0;i < arrListProps.Count;i++)
			{
				SearchCriteria criteria = arrListProps[i] as SearchCriteria;
				ArrayList arrCondition = criteria.ArrListSearchCondition;
				if(criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
				{
					SearchCondition condition = arrCondition[0] as SearchCondition;
					LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition.Value);
					string name = null;
					string id = null;
					if(masterProp != null)
					{
						name = "[S] - " + masterProp.Name;
						id = masterProp.PropID;
					}
					else
					{
						LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(condition.Value);
						name = "[S] - " + siteCompProp.Name;
						id = siteCompProp.ID;
                        
					}
					LRRResultProperty lrrResultProp = new LRRResultProperty();
					lrrResultProp.ID = id;
					lrrResultProp.Name = name;
					lrrResultProps.Add(lrrResultProp);
				}
				if(criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
				{
					string lkUpPropertyID = null;
					string lookUpQualifierID = null;
					string name = null;
					string id = null;
					string masterPropID = null;
					for(int k=0; k < arrCondition.Count;k++)
					{
						SearchCondition condition1 = arrCondition[k] as SearchCondition;
						if(condition1.Name == "LookUpPropertiesID")
							lkUpPropertyID = condition1.Value;
						if(condition1.Name == "LookUpQualifierID")
							lookUpQualifierID = condition1.Value;
						if(condition1.Name == "MasterPropID")
							masterPropID = condition1.Value;
					}
					
					string masterPropName = null;
					LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(masterPropID);
					if(masterProp != null)
						masterPropName = masterProp.Name;

					LRRLookUpProperty lkupProp = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(lkUpPropertyID);
					if(lookUpQualifierID != null)
					{
						LRRLookUpQualifier lrrLkUpQual = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
						if(lrrLkUpQual != null)
						{
							LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);
							name = "[GP] - " + masterPropName + "/" + qual.Name + "/" + lkupProp.Name;
                            
							id = qual.QualifierID + "|" + lkupProp.LookUpPropertiesID;
						}
					}
					else
					{
						name = "[DA] - " + masterPropName + "/" + lkupProp.Name;
						id = masterPropID + "|" + lkupProp.LookUpPropertiesID;
					}
					LRRResultProperty lrrResultProp = new LRRResultProperty();
					lrrResultProp.ID = id;
					lrrResultProp.Name = name;
					lrrResultProps.Add(lrrResultProp);
				}
                if (criteria.ObjType == typeof(LRRAggregatePropertyValue))
                {
                    SearchCondition condition = arrCondition[0] as SearchCondition;
                    LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition.Value);
                    string name = null;
                    string id = null;
                    if (masterProp != null)
                    {
                        name = "[S] - " + masterProp.Name;
                        id = masterProp.PropID;
                    }
                    else
                    {
                        LRRAggregateProperty aggProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(condition.Value);
                        name = "[S] - " + aggProp.Name;
                        id = aggProp.ID;
                    }
                    LRRResultProperty lrrResultProp = new LRRResultProperty();
                    lrrResultProp.ID = id;
                    lrrResultProp.Name = name;
                    lrrResultProps.Add(lrrResultProp);
                }
			}
			return lrrResultProps;
		}
		
		/// <summary>
		/// Method gets the result Properties
		/// </summary>
		/// <param name="arrListSiteComps"></param>
		/// <returns></returns>
		public static LRRResultProperties GetSiteComponents(ArrayList arrListSiteComps)
		{
			LRRResultProperties lrrResultProps = new LRRResultProperties();
			for(int i=0;i < arrListSiteComps.Count;i++)
			{
				SearchCriteria criteria = arrListSiteComps[i] as SearchCriteria;
				ArrayList arrCondition = criteria.ArrListSearchCondition;
                if (criteria.ObjType == typeof(LRRSiteComponent))
                {
                    for (int j = 0; j < arrCondition.Count; j++)
                    {
                        SearchCondition condition = arrCondition[j] as SearchCondition;
                        LRRSiteComponent siteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(condition.Value);
                        string sMasterCompType = siteComponent.MasterComponentType;
                        LRRMasterComponent masterComp = LRRModel.Model.GetMasterComponentByID(sMasterCompType);

                        LRRResultProperty lrrResultProp = new LRRResultProperty();
                        lrrResultProp.ID = siteComponent.SiteComponentID;
                        lrrResultProp.Name = siteComponent.Name + "(" + masterComp.ComponentName + ")";
                        lrrResultProps.Add(lrrResultProp);
                        
                    }
                }
                else if (criteria.ObjType == typeof(LRRAggregate))
                {
                    for (int j = 0; j < arrCondition.Count; j++)
                    {
                        SearchCondition condition = arrCondition[j] as SearchCondition;
                        LRRAggregate aggregate = LRRModel.AggregatesObj.GetAggregateFromID(condition.Value);
                        string sMasterCompType = aggregate.EquipmentTypeID;
                        LRRMasterComponent masterComp = LRRModel.Model.GetMasterComponentByID(sMasterCompType);

                        LRRResultProperty lrrResultProp = new LRRResultProperty();
                        lrrResultProp.ID = aggregate.AggregateID;
                        lrrResultProp.Name = aggregate.Name + "(" + masterComp.ComponentName + ")";
                        lrrResultProps.Add(lrrResultProp);
                    }
                }

			}

			return lrrResultProps;
		}

        /// <summary>
        /// Gets the collection.
        /// </summary>
        /// <param name="unifiedSearchCriteriaID">The unified search criteria ID.</param>
        /// <param name="arrXCollection">The arr X collection.</param>
        /// <param name="arrYCollection">The arr Y collection.</param>
		public static void GetCollection(string unifiedSearchCriteriaID ,out ArrayList arrXCollection,out ArrayList arrYCollection)
		{
			arrXCollection = new ArrayList();
			arrYCollection = new ArrayList();

			LRRUnifiedViewSearchCrtiteria lrrUniSrchCriteria = LRRModel.ChildModel.UnifiedViewSearchCrtiterias.GetUnifiedViewByID(unifiedSearchCriteriaID);

			//Populate Items and Item Condition
			LRRUnifiedViewSearchItems lrrSrchItems = new LRRUnifiedViewSearchItems();
			_lrrApp.DBManagerObj.PopulateCollection(lrrSrchItems," where SearchCriteriaID = '" + unifiedSearchCriteriaID + "'");

			LRRUnifiedViewSearchItemConditions lrrSrchItemConditions = new LRRUnifiedViewSearchItemConditions();
			_lrrApp.DBManagerObj.PopulateCollection(lrrSrchItemConditions," where SearchCriteriaID = '" + unifiedSearchCriteriaID + "'");

			IEnumerator iItemEnum = lrrSrchItems.GetEnumerator();
			while(iItemEnum.MoveNext())
			{
				LRRUnifiedViewSearchItem lrrSrchItem = iItemEnum.Current as LRRUnifiedViewSearchItem;
				LRRUnifiedViewSearchItemConditions lrrSrchItemConds = lrrSrchItemConditions.GetItemConditionsByItemID(lrrSrchItem.ID);
				
				SearchCriteria criteria = new SearchCriteria();
				criteria.ObjType = GetType(lrrSrchItem.Type);
				
				ArrayList arrConds = new ArrayList();
				IEnumerator iEnum = lrrSrchItemConds.GetEnumerator();
				while(iEnum.MoveNext())
				{
					LRRUnifiedViewSearchItemCondition itemCond = iEnum.Current as LRRUnifiedViewSearchItemCondition;
					SearchCondition condition = new SearchCondition();
					condition.Name = itemCond.Name;
					condition.Value = itemCond.ConditionValue;
					condition.Operator = itemCond.Oper;
                    if (criteria.ObjType == typeof(LRRSiteComponent) || criteria.ObjType == typeof(LRRAggregate))
                    {
                        if (criteria.ObjType == typeof(LRRSiteComponent))
                        {
                            condition.ObjType = typeof(LRRSiteComponent);
                        }
                        else
                        {
                            condition.ObjType = typeof(LRRAggregate);
                        }
                    }
                    else
                    {
                        if (condition.Name == "ID")
                        {
                            condition.ObjType = typeof(LRRAggregatePropertyValue);
                        }
                        else if(condition.Name == "SiteComponentPropertyID")
                        {
                            condition.ObjType = typeof(LRRSiteComponentPropertyValue);
                        }
                    }
					arrConds.Add(condition);
				}
				criteria.ArrListSearchCondition = arrConds;

				if(criteria.ObjType == typeof(LRRSiteComponent) || criteria.ObjType == typeof(LRRAggregate))
				{
					arrXCollection.Add(criteria);
				}
				else
				{
					arrYCollection.Add(criteria);
				}
			}
		}

        /// <summary>
        /// Gets the type.
        /// </summary>
        /// <param name="sType">Type of the s.</param>
        /// <returns></returns>
		public static Type GetType(string sType)
		{
			Type retType = null;
			switch(sType)
			{
				case "Symcon.LightRidge.BusinessObjects.LRRSiteComponent":
					retType = typeof(LRRSiteComponent);
					break;
				case "Symcon.LightRidge.BusinessObjects.LRRSiteComponentPropertyValue":
					retType = typeof(LRRSiteComponentPropertyValue);
					break;
				case "Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue":
					retType = typeof(LRRSiteGroupPropertyQualifierValue);
					break;
                case "Symcon.LightRidge.BusinessObjects.LRRAggregate":
                    retType = typeof(LRRAggregate);
                    break;
			}
			return retType;
		}

		#endregion

		#region Fields
		private static LRRApp _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		#endregion
	}
}
