using System;
using System.Collections;	

using  Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.BusinessObjects;

namespace Symcon.LightRidge.PEAdvisor.CommonViews
{
	/// <summary>
	/// Summary description for GridControlManager.
	/// </summary>
	public class GridControlManager
	{
		public GridControlManager()
		{
		}

		#region Fields

			//ArrayList for storing row indexes
		private ArrayList _sortManagerList = new ArrayList();

			//ArrayList for storing colHeader Cells
		private ArrayList _colHeaderCellList = new ArrayList();

		//ArrayList for Row Headers
		private ArrayList _rowHeader = new ArrayList();

		//ArrayList for Column Headers
		private ArrayList _colHeader = new ArrayList();

		//ArrayList for Row List
		private ArrayList _rowList = new ArrayList();

		//ArrayList for Column List
		private ArrayList _colList = new ArrayList();

		//BindingManager for storing bindings
		private LRRBindingManager _bindingManager = new LRRBindingManager();

		
		//Hash for storing UI UOMs
		private Hashtable _uiUOMHash= new Hashtable();

        //Hash for storing cell address
        private Hashtable _propValueCellAddress = new Hashtable();

		//Header Text
		private string _headerText;
		#endregion

		#region Properties

        /// <summary>
        /// Gets or sets the sort manager list.
        /// </summary>
        /// <value>The sort manager list.</value>
		public ArrayList SortManagerList
		{
			get{return _sortManagerList;}
			set
			{
				_sortManagerList = value;
			}
		}

        /// <summary>
        /// Gets or sets the row header.
        /// </summary>
        /// <value>The row header.</value>
		public ArrayList RowHeader
		{
			get{return _rowHeader;}
			set
			{
				_rowHeader = value;
			}
		}
        /// <summary>
        /// Gets or sets the col header.
        /// </summary>
        /// <value>The col header.</value>
		public ArrayList ColHeader
		{
			get{return _colHeader;}
			set
			{
				_colHeader = value;
			}
		}

        /// <summary>
        /// Gets or sets the row list.
        /// </summary>
        /// <value>The row list.</value>
		public ArrayList RowList
		{
			get{return _rowList;}
			set
			{
				_rowList = value;
			}
		}
        /// <summary>
        /// Gets or sets the col list.
        /// </summary>
        /// <value>The col list.</value>
		public ArrayList ColList
		{
			get{return _colList;}
			set
			{
				_colList = value;
			}
		}

        /// <summary>
        /// Gets or sets the binding manager.
        /// </summary>
        /// <value>The binding manager.</value>
		public LRRBindingManager BindingManager
		{
			get{return _bindingManager;}
			set
			{
				_bindingManager = value;
			}
		}

        /// <summary>
        /// Gets or sets the header text.
        /// </summary>
        /// <value>The header text.</value>
		public string HeaderText
		{
			get{return _headerText;}
			set
			{
				_headerText = value;
			}
		}

        /// <summary>
        /// Gets or sets the UIUOM hash.
        /// </summary>
        /// <value>The UIUOM hash.</value>
		public Hashtable UIUOMHash
		{
			get{return _uiUOMHash;}
			set
			{
				_uiUOMHash = value;
			}
		}
        /// <summary>
        /// Gets or sets the col header cell list.
        /// </summary>
        /// <value>The col header cell list.</value>
		public ArrayList ColHeaderCellList
		{
			get{return _colHeaderCellList;}
			set
			{
				_colHeaderCellList = value;
			}
		}

        /// <summary>
        /// Gets or sets the prop value cell address.
        /// </summary>
        /// <value>The prop value cell address.</value>
        public Hashtable PropValueCellAddress
        {
            get { return _propValueCellAddress; }
            set
            {
                _propValueCellAddress = value;
            }
        }

		#endregion

        #region Methods
        /// <summary>
        /// Fores the color changed handler.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        public void ForeColorChangedHandler(object sender, EventArgs e)
        {
            IPropertyValue iPropValue = sender as IPropertyValue;
            string key = iPropValue.GetID();
            GridCellAddress gcAddress = this.PropValueCellAddress[key] as GridCellAddress;

            ColorEventArgs colorEventArgs = new ColorEventArgs();
            colorEventArgs.PropValueObj = iPropValue;
            colorEventArgs.Row = gcAddress.Row;
            colorEventArgs.Col = gcAddress.Col;

            if (ForeColorChanged != null)
                ForeColorChanged(this, colorEventArgs);
        }

        /// <summary>
        /// Backs the color changed handler.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        public void BackColorChangedHandler(object sender, EventArgs e)
        {
            IPropertyValue iPropValue = sender as IPropertyValue;
            string key = iPropValue.GetID();
            GridCellAddress gcAddress = this.PropValueCellAddress[key] as GridCellAddress;

            ColorEventArgs colorEventArgs = new ColorEventArgs();
            colorEventArgs.PropValueObj = iPropValue;
            colorEventArgs.Row = gcAddress.Row;
            colorEventArgs.Col = gcAddress.Col;

            if (BackColorChanged != null)
                BackColorChanged(this, colorEventArgs);
        }
        #endregion

        #region Event
        public event ForeColorChangesDel ForeColorChanged;
        public event BackColorChangesDel BackColorChanged;
        #endregion

        public delegate void ForeColorChangesDel(object sender, ColorEventArgs e);
        public delegate void BackColorChangesDel(object sender, ColorEventArgs e);

	}


    public class GridCellAddress
    {
        public GridCellAddress(int row, int col)
        {
            _row = row;
            _col = col;
        }
        #region Properties
        /// <summary>
        /// Gets or sets the row.
        /// </summary>
        /// <value>The row.</value>
        public int Row
        {
            get { return _row; }
            set
            {
                _row = value;
            }
        }

        /// <summary>
        /// Gets or sets the col.
        /// </summary>
        /// <value>The col.</value>
        public int Col
        {
            get { return _col; }
            set
            {
                _col = value;
            }
        }
        #endregion
        #region Fields
        private int _row;
        private int _col;
        #endregion
    }
}
