using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Symcon.Database;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;
using Symcon.Utils.MessageLogger;
using System.Collections.Generic;

namespace Symcon.LightRidge.PEAdvisor.CommonViews
{
	/// <summary>
	/// Summary description for SavedViewForm.
	/// </summary>
	public class SavedViewForm : System.Windows.Forms.Form,ILRRUIContainer,ILRRUISpecificContainer
    {
        private System.Windows.Forms.TreeView trvMasterProps;
		private System.Windows.Forms.Button btnExec;
		private System.Windows.Forms.Panel pnlGrid;
		private System.Windows.Forms.Panel pnlTreeControl;
		private System.Windows.Forms.Panel pnlButton;
		private System.Windows.Forms.Panel pnlLeft;
		private System.Windows.Forms.Splitter splBottom;
		private System.Windows.Forms.Splitter splVertical;
        private Button btnTranspose;
        private Button btnSaveView;
        private Panel pnlQuerySelector;
        private Panel pnlTreeView;
        private Label lblOpenQuery;
        private ComboBox cmbQuerySelector;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

        #region Constructor
        public SavedViewForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            Symcon.LightRidge.PEAdvisor.Global.TreeViewUtility.WiredTreeView(trvMasterProps, null);
			InternationalizationUtils.GetControlText(this,null);
			InitDataSource();
			InitializeTree();
            InitializeQuerySelector();

			LRRModel.ProjectModes.ModesChanged+=new EventHandler(ProjectModes_ModesChanged);

			Syncfusion.Windows.Forms.Tools.XPMenus.Bar resultsFilter = _lrrApp.LRRUIManagerObj.MainFrameObj.BarResultsFilter;
			_lrrApp.LRRUIManagerObj.MainFrameObj.MainFrameBarManager.GetCommandBarManager().SetBarVisibility(resultsFilter,false);

			_resultComponentsWindow = new ResultWindow();
			_resultPropertiesWindow = new ResultWindow();
        }

        #endregion

        #region Methods
        private void InitializeQuerySelector()
        {
            cmbQuerySelector.DataSource = _lrrUnifiedSrchCriterias;
            cmbQuerySelector.DisplayMember = "Name";
            cmbQuerySelector.ValueMember = "ID";
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
            LRRModel.ProjectModes.ModesChanged -= new EventHandler(ProjectModes_ModesChanged);
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(SavedViewForm));
            this.trvMasterProps = new System.Windows.Forms.TreeView();
            this.btnExec = new System.Windows.Forms.Button();
            this.pnlTreeControl = new System.Windows.Forms.Panel();
            this.pnlTreeView = new System.Windows.Forms.Panel();
            this.pnlQuerySelector = new System.Windows.Forms.Panel();
            this.cmbQuerySelector = new System.Windows.Forms.ComboBox();
            this.lblOpenQuery = new System.Windows.Forms.Label();
            this.pnlGrid = new System.Windows.Forms.Panel();
            this.pnlButton = new System.Windows.Forms.Panel();
            this.btnSaveView = new System.Windows.Forms.Button();
            this.btnTranspose = new System.Windows.Forms.Button();
            this.pnlLeft = new System.Windows.Forms.Panel();
            this.splBottom = new System.Windows.Forms.Splitter();
            this.splVertical = new System.Windows.Forms.Splitter();
            this.pnlTreeControl.SuspendLayout();
            this.pnlTreeView.SuspendLayout();
            this.pnlQuerySelector.SuspendLayout();
            this.pnlButton.SuspendLayout();
            this.pnlLeft.SuspendLayout();
            this.SuspendLayout();
            // 
            // trvMasterProps
            // 
            this.trvMasterProps.CheckBoxes = true;
            this.trvMasterProps.Dock = System.Windows.Forms.DockStyle.Fill;
            this.trvMasterProps.Location = new System.Drawing.Point(0, 0);
            this.trvMasterProps.Name = "trvMasterProps";
            this.trvMasterProps.Size = new System.Drawing.Size(320, 383);
            this.trvMasterProps.TabIndex = 0;
            this.trvMasterProps.AfterCheck += new System.Windows.Forms.TreeViewEventHandler(this.trvMasterProps_AfterCheck);
            // 
            // btnExec
            // 
            this.btnExec.Location = new System.Drawing.Point(93, 16);
            this.btnExec.Name = "btnExec";
            this.btnExec.Size = new System.Drawing.Size(75, 23);
            this.btnExec.TabIndex = 1;
            this.btnExec.Tag = "SID_EXECUTE";
            this.btnExec.Click += new System.EventHandler(this.btnExec_Click);
            // 
            // pnlTreeControl
            // 
            this.pnlTreeControl.Controls.Add(this.pnlTreeView);
            this.pnlTreeControl.Controls.Add(this.pnlQuerySelector);
            this.pnlTreeControl.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlTreeControl.Location = new System.Drawing.Point(0, 0);
            this.pnlTreeControl.Name = "pnlTreeControl";
            this.pnlTreeControl.Size = new System.Drawing.Size(320, 414);
            this.pnlTreeControl.TabIndex = 2;
            // 
            // pnlTreeView
            // 
            this.pnlTreeView.Controls.Add(this.trvMasterProps);
            this.pnlTreeView.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlTreeView.Location = new System.Drawing.Point(0, 31);
            this.pnlTreeView.Name = "pnlTreeView";
            this.pnlTreeView.Size = new System.Drawing.Size(320, 383);
            this.pnlTreeView.TabIndex = 1;
            // 
            // pnlQuerySelector
            // 
            this.pnlQuerySelector.Controls.Add(this.cmbQuerySelector);
            this.pnlQuerySelector.Controls.Add(this.lblOpenQuery);
            this.pnlQuerySelector.Dock = System.Windows.Forms.DockStyle.Top;
            this.pnlQuerySelector.Location = new System.Drawing.Point(0, 0);
            this.pnlQuerySelector.Name = "pnlQuerySelector";
            this.pnlQuerySelector.Size = new System.Drawing.Size(320, 31);
            this.pnlQuerySelector.TabIndex = 0;
            // 
            // cmbQuerySelector
            // 
            this.cmbQuerySelector.FormattingEnabled = true;
            this.cmbQuerySelector.Location = new System.Drawing.Point(111, 5);
            this.cmbQuerySelector.Name = "cmbQuerySelector";
            this.cmbQuerySelector.Size = new System.Drawing.Size(161, 21);
            this.cmbQuerySelector.TabIndex = 0;
            this.cmbQuerySelector.SelectedIndexChanged += new System.EventHandler(this.cmbQuerySelector_SelectedIndexChanged);
            this.cmbQuerySelector.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.cmbQuerySelector_KeyPress);
            this.cmbQuerySelector.DropDown += new System.EventHandler(this.cmbQuerySelector_DropDown);
            // 
            // lblOpenQuery
            // 
            this.lblOpenQuery.AutoSize = true;
            this.lblOpenQuery.Location = new System.Drawing.Point(12, 8);
            this.lblOpenQuery.Name = "lblOpenQuery";
            this.lblOpenQuery.Size = new System.Drawing.Size(0, 13);
            this.lblOpenQuery.TabIndex = 1;
            this.lblOpenQuery.Tag = "SID_OPEN/NEW_QUERY";
            // 
            // pnlGrid
            // 
            this.pnlGrid.Dock = System.Windows.Forms.DockStyle.Right;
            this.pnlGrid.Location = new System.Drawing.Point(320, 0);
            this.pnlGrid.Name = "pnlGrid";
            this.pnlGrid.Size = new System.Drawing.Size(592, 462);
            this.pnlGrid.TabIndex = 3;
            // 
            // pnlButton
            // 
            this.pnlButton.Controls.Add(this.btnSaveView);
            this.pnlButton.Controls.Add(this.btnTranspose);
            this.pnlButton.Controls.Add(this.btnExec);
            this.pnlButton.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.pnlButton.Location = new System.Drawing.Point(0, 414);
            this.pnlButton.Name = "pnlButton";
            this.pnlButton.Size = new System.Drawing.Size(320, 48);
            this.pnlButton.TabIndex = 4;
            // 
            // btnSaveView
            // 
            this.btnSaveView.Location = new System.Drawing.Point(12, 16);
            this.btnSaveView.Name = "btnSaveView";
            this.btnSaveView.Size = new System.Drawing.Size(75, 23);
            this.btnSaveView.TabIndex = 3;
            this.btnSaveView.Tag = "SID_SAVE_VIEW";
            this.btnSaveView.UseVisualStyleBackColor = true;
            this.btnSaveView.Click += new System.EventHandler(this.btnSaveView_Click);
            // 
            // btnTranspose
            // 
            this.btnTranspose.Enabled = false;
            this.btnTranspose.Location = new System.Drawing.Point(174, 16);
            this.btnTranspose.Name = "btnTranspose";
            this.btnTranspose.Size = new System.Drawing.Size(75, 23);
            this.btnTranspose.TabIndex = 2;
            this.btnTranspose.Tag = "SID_TRANSPOSE";
            this.btnTranspose.UseVisualStyleBackColor = true;
            this.btnTranspose.Click += new System.EventHandler(this.btnTranspose_Click);
            // 
            // pnlLeft
            // 
            this.pnlLeft.Controls.Add(this.splBottom);
            this.pnlLeft.Controls.Add(this.pnlTreeControl);
            this.pnlLeft.Controls.Add(this.pnlButton);
            this.pnlLeft.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlLeft.Location = new System.Drawing.Point(0, 0);
            this.pnlLeft.Name = "pnlLeft";
            this.pnlLeft.Size = new System.Drawing.Size(320, 462);
            this.pnlLeft.TabIndex = 6;
            // 
            // splBottom
            // 
            this.splBottom.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.splBottom.Enabled = false;
            this.splBottom.Location = new System.Drawing.Point(0, 412);
            this.splBottom.Name = "splBottom";
            this.splBottom.Size = new System.Drawing.Size(320, 2);
            this.splBottom.TabIndex = 5;
            this.splBottom.TabStop = false;
            // 
            // splVertical
            // 
            this.splVertical.BackColor = System.Drawing.SystemColors.Highlight;
            this.splVertical.Dock = System.Windows.Forms.DockStyle.Right;
            this.splVertical.Location = new System.Drawing.Point(315, 0);
            this.splVertical.Name = "splVertical";
            this.splVertical.Size = new System.Drawing.Size(5, 462);
            this.splVertical.TabIndex = 7;
            this.splVertical.TabStop = false;
            this.splVertical.DoubleClick += new System.EventHandler(this.splitter2_DoubleClick);
            // 
            // SavedViewForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(912, 462);
            this.Controls.Add(this.splVertical);
            this.Controls.Add(this.pnlLeft);
            this.Controls.Add(this.pnlGrid);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Name = "SavedViewForm";
            this.Tag = "SID_UNIFIED_VIEWS";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
            this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.SavedViewForm_FormClosing);
            this.pnlTreeControl.ResumeLayout(false);
            this.pnlTreeView.ResumeLayout(false);
            this.pnlQuerySelector.ResumeLayout(false);
            this.pnlQuerySelector.PerformLayout();
            this.pnlButton.ResumeLayout(false);
            this.pnlLeft.ResumeLayout(false);
            this.ResumeLayout(false);

		}
		#endregion

        /// <summary>
        /// Inits the data source.
        /// </summary>
		private void InitDataSource()
		{
			_lrrMasterComps = LRRModel.Model;
			_lrrPropertyTypes = LRRModel.ChildModel.PropertyTypes;

			_lrrCompTypes = new LRRComponentTypes();
			_lrrApp.DBManagerObj.PopulateCollection(_lrrCompTypes);
            //_lrrUnifiedSrchCriterias = LRRModel.ChildModel.UnifiedViewSearchCrtiterias;
            _lrrApp.DBManagerObj.PopulateCollection(_lrrUnifiedSrchCriterias);
            _lrrUnifiedSrchCriterias.SortColumn = "Name";
            _lrrUnifiedSrchCriterias.Sort();     
		}

        /// <summary>
        /// Initializes the tree.
        /// </summary>
		private void InitializeTree()
		{
			TreeNode root = new TreeNode(LRRHeaderText.GetHeaderString("SID_MASTERPLANTITEMTYPE"));
			IEnumerator compTypeEnum = _lrrCompTypes.GetEnumerator();

			while(compTypeEnum.MoveNext())
			{
				LRRComponentType lrrCompType = compTypeEnum.Current as LRRComponentType;

				//Create Component Type Node
                TreeNode compTypeNode = new TreeNode(lrrCompType.Name);
				
				//Filter Master Components on master Component Type
                LRRMasterComponents masterComps = new LRRMasterComponents();
				LRRMasterComponents lrrMasterComps = _lrrMasterComps.FilterMasterComponentsByComponentTypeID(lrrCompType.ID);
                if (lrrMasterComps.Count > 0)
                {
                    root.Nodes.Add(compTypeNode);
                }
				IEnumerator iMasterCompEnum = lrrMasterComps.GetEnumerator();
				while(iMasterCompEnum.MoveNext())
				{
					LRRMasterComponent lrrMasterComp = iMasterCompEnum.Current as LRRMasterComponent;

                    //Get Site Components
                    LRRSiteComponents lrrSiteComps = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(lrrMasterComp.ID);
                    lrrSiteComps.SortColumn = "Name";
                    lrrSiteComps.Sort();

                    LRRAggregates lrrAggregates = LRRModel.AggregatesObj.GetAggregatesByMasterComponentID(lrrMasterComp.ID);
                    lrrAggregates.SortColumn = "Name";
                    lrrAggregates.Sort();

                    if (lrrSiteComps.Count > 0 || lrrAggregates.Count > 0)
                    {
                        //Create Master Component Node
                        TreeNode masterCompNode = new TreeNode(lrrMasterComp.ComponentName);
                        compTypeNode.Nodes.Add(masterCompNode);

                        if (lrrSiteComps.Count > 0)
                        {
                            TreeNode siteNode = new TreeNode(LRRHeaderText.GetHeaderString("SID_PLANT_ITEMS"));
                            masterCompNode.Nodes.Add(siteNode);

                            IEnumerator iSiteCompEnum = lrrSiteComps.GetEnumerator();
                            while (iSiteCompEnum.MoveNext())
                            {
                                LRRSiteComponent lrrSiteComp = iSiteCompEnum.Current as LRRSiteComponent;

                                //Create Site Component Node
                                TreeNode siteCompNode = new TreeNode(lrrSiteComp.Name);

                                SearchCondition condition = GetConditionObj("SiteComponentID", lrrSiteComp.SiteComponentID, "=",typeof(LRRSiteComponent));
                                SearchCriteria criteria = new SearchCriteria();
                                criteria.ObjType = typeof(LRRSiteComponent);
                                ArrayList arrList = new ArrayList();
                                arrList.Add(condition);
                                criteria.ArrListSearchCondition = arrList;

                                siteCompNode.Tag = criteria;
                                siteNode.Nodes.Add(siteCompNode);

                                LRRSiteComponentProperties lrrSiteCompProps = lrrSiteComp.Properties.FilterPropertiesByPropertyID(null);
                                IEnumerator iSitePropEnum = lrrSiteCompProps.GetEnumerator();
                                while (iSitePropEnum.MoveNext())
                                {
                                    LRRSiteComponentProperty lrrSiteCompProp = iSitePropEnum.Current as LRRSiteComponentProperty;

                                    //Create Site Comp Property Node
                                    TreeNode siteCompPropNode = new TreeNode(lrrSiteCompProp.Name);

                                    SearchCondition condition1 = GetConditionObj("SiteComponentPropertyID", lrrSiteCompProp.ID, "=",typeof(LRRSiteComponentProperty));
                                    SearchCriteria criteria1 = new SearchCriteria();
                                    criteria1.ObjType = typeof(LRRSiteComponentPropertyValue);
                                    ArrayList arrList1 = new ArrayList();
                                    arrList1.Add(condition1);
                                    criteria1.ArrListSearchCondition = arrList1;

                                    siteCompPropNode.Tag = criteria1;
                                    siteCompNode.Nodes.Add(siteCompPropNode);
                                }
                            }
                        }

                        if (lrrAggregates.Count > 0)
                        {
                            TreeNode aggCompNode = new TreeNode(LRRHeaderText.GetHeaderString("SID_AGGREGATES"));
                            masterCompNode.Nodes.Add(aggCompNode);

                            IEnumerator iAggEnum = lrrAggregates.GetEnumerator();
                            while (iAggEnum.MoveNext())
                            {
                                LRRAggregate lrrAggregate = iAggEnum.Current as LRRAggregate;

                                //Create Site Component Node
                                TreeNode aggNode = new TreeNode(lrrAggregate.Name);

                                SearchCondition condition = GetConditionObj("AggregateID", lrrAggregate.AggregateID , "=",typeof(LRRAggregate));
                                SearchCriteria criteria = new SearchCriteria();
                                criteria.ObjType = typeof(LRRAggregate);
                                ArrayList arrList = new ArrayList();
                                arrList.Add(condition);
                                criteria.ArrListSearchCondition = arrList;

                                aggNode.Tag = criteria;
                                aggCompNode.Nodes.Add(aggNode);

                                LRRAggregateProperties lrrAggProps = lrrAggregate.AggregateProperties.FilterPropertiesByMasterProperty(null); 
                                IEnumerator iAggPropEnum = lrrAggProps.GetEnumerator();
                                while (iAggPropEnum.MoveNext())
                                {
                                    LRRAggregateProperty lrrAggProp = iAggPropEnum.Current as LRRAggregateProperty;

                                    //Create Site Comp Property Node
                                    TreeNode aggPropNode = new TreeNode(lrrAggProp.Name);

                                    SearchCondition condition1 = GetConditionObj("ID", lrrAggProp.ID, "=",typeof(LRRAggregateProperty));
                                    SearchCriteria criteria1 = new SearchCriteria();
                                    criteria1.ObjType = typeof(LRRAggregatePropertyValue);
                                    ArrayList arrList1 = new ArrayList();
                                    arrList1.Add(condition1);
                                    criteria1.ArrListSearchCondition = arrList1;

                                    aggPropNode.Tag = criteria1;
                                    aggNode.Nodes.Add(aggPropNode);
                                }
                            }
                        }

                        //Master Properties
                        LRRMasterProperties lrrMasterProps = LRRModel.MasterComponentPropertiesObj.GetMasterPropertiesByType(lrrMasterComp.ID);

                        TreeNode masterNode = null;
                        if (lrrMasterProps.Count > 0)
                        {
                            masterNode = new TreeNode(LRRHeaderText.GetHeaderString("SID_MASTER_PLANT_ATTRIBUTES"));
                            masterCompNode.Nodes.Add(masterNode);
                                                 
                        }

                        LRRMasterProperties lrrMasterSimpleProps = new LRRMasterProperties();
                        LRRMasterProperties lrrMasterGroupedProps = new LRRMasterProperties();
                        LRRMasterProperties lrrMasterAggregateProps = new LRRMasterProperties();
                        LRRMasterProperties lrrMasterAggregateGrpProps = new LRRMasterProperties();

                        IEnumerator iPropTypeEnum = _lrrPropertyTypes.GetEnumerator();
                        while (iPropTypeEnum.MoveNext())
                        {
                            LRRPropertyType lrrPropType = iPropTypeEnum.Current as LRRPropertyType;
                            LRRMasterProperties lrrMasterProperties = lrrMasterProps.GetMasterPropertiesByPropType(lrrPropType.ID);
                            IEnumerator iMasterPropEnum = lrrMasterProperties.GetEnumerator();
                            while (iMasterPropEnum.MoveNext())
                            {
                                LRRMasterProperty lrrMasterProp = iMasterPropEnum.Current as LRRMasterProperty;
                                bool isExistsInAllSiteComp = ExistsInAllSiteComponetn(lrrMasterProp);
                                if (isExistsInAllSiteComp)
                                {
                                    //Create Master Property Node(MasterProperties --> BFWFlow)
                                    if (lrrMasterProp.Type == "1")
                                    {
                                        lrrMasterSimpleProps.Add(lrrMasterProp);
                                        LRRAggregateProperties aggProps = LRRModel.AggregatePropertiesObj.FilterPropertiesByMasterProperty(lrrMasterProp.PropID);
                                        if (aggProps != null)
                                        {
                                            if (aggProps.Count > 0)
                                            {
                                                lrrMasterAggregateProps.Add(lrrMasterProp);
                                            }
                                        }
                                    }
                                    else
                                    {
                                        lrrMasterGroupedProps.Add(lrrMasterProp);
                                        LRRAggregateGroupedProperties aggregateGroupProps = LRRModel.AggregateGroupedPropertiesObj.FilterGroupPropertiesByPropertyType(lrrMasterProp.Type);
                                        if (aggregateGroupProps != null)
                                        {
                                            if (aggregateGroupProps.Count > 0)
                                            {
                                                lrrMasterAggregateGrpProps.Add(lrrMasterProp);
                                            }
                                        }

                                    }
                                }
                            }
                        }

                        lrrMasterGroupedProps.SortColumn = "Name";
                        lrrMasterGroupedProps.Sort();
                        IEnumerator iMasterGroupPropEnum = lrrMasterGroupedProps.GetEnumerator();
                        while (iMasterGroupPropEnum.MoveNext())
                        {
                            LRRMasterProperty lrrMasterProp = iMasterGroupPropEnum.Current as LRRMasterProperty;
                            LRRPropertyType lrrPropType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(lrrMasterProp.Type);
                            //Property Type Node (MasterProperties --> BFWFlow-->Air Components)
                            TreeNode childNode1 = new TreeNode(lrrMasterProp.Name + "-" + lrrPropType.PropertyType);
                            masterNode.Nodes.Add(childNode1);

                            //Check for Data Array
                            if (!lrrPropType.IsDataArray)
                            {
                                LRRLookUpQualifiers lrrLkUpQuals = LRRModel.LookUpQualifiers.getLookUpQualifiersForPropType(lrrPropType.ID);

                                LRRQualifiers lrrQualifiers = new LRRQualifiers();
                                IEnumerator iLkUpEnum1 = lrrLkUpQuals.GetEnumerator();
                                while (iLkUpEnum1.MoveNext())
                                {
                                    LRRLookUpQualifier lrrLkUpQual1 = iLkUpEnum1.Current as LRRLookUpQualifier;
                                    LRRQualifier qual1 = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual1.QualifierID);
                                    lrrQualifiers.Add(qual1);
                                }
                                lrrQualifiers.SortColumn = "Name";
                                lrrQualifiers.Sort();

                                IEnumerator iQualEnum = lrrQualifiers.GetEnumerator();
                                while (iQualEnum.MoveNext())
                                {
                                    LRRQualifier qual = iQualEnum.Current as LRRQualifier;
                                    LRRLookUpQualifier lrrLkUpQual = LRRModel.LookUpQualifiers.GetLookUpQualifierForPropTypeAndQualifierID(qual.QualifierID, lrrPropType.ID);

                                    //Qualifier Node (MasterProperties --> BFWFlow-->Air Components --> Mol Fraction)
                                    TreeNode childNode2 = new TreeNode(qual.Name);
                                    childNode1.Nodes.Add(childNode2);

                                    //Get the Group properties
                                    LRRLookUpProperties lrrLkUpProps = LRRModel.LookUpProperties.getLookUpPropertiesForPropTypeID(lrrPropType.ID);
                                    lrrLkUpProps.SortColumn = "Name";
                                    lrrLkUpProps.Sort();

                                    IEnumerator iLkUpPropEnum = lrrLkUpProps.GetEnumerator();
                                    while (iLkUpPropEnum.MoveNext())
                                    {
                                        LRRLookUpProperty lrrLkUpProp = iLkUpPropEnum.Current as LRRLookUpProperty;

                                        //Grp Property Node (MasterProperties --> BFWFlow-->Air Components --> Mol Fraction --> Ar)
                                        TreeNode childNode3 = new TreeNode(lrrLkUpProp.Name);
                                        
                                        SearchCriteria criteria = new SearchCriteria();
                                        criteria.ObjType = typeof(LRRSiteGroupPropertyQualifierValue);
                                        ArrayList arrConditions = new ArrayList();
                                        SearchCondition condition1 = GetConditionObj("LookUpPropertiesID", lrrLkUpProp.LookUpPropertiesID, "=", typeof(LRRSiteGroupPropertyQualifierValue));
                                        arrConditions.Add(condition1);
                                        SearchCondition condition2 = GetConditionObj("LookUpQualifierID", lrrLkUpQual.LookUpQualifierID, "=", typeof(LRRSiteGroupPropertyQualifierValue));
                                        arrConditions.Add(condition2);
                                        SearchCondition condition3 = GetConditionObj("MasterPropID", lrrMasterProp.PropID, "=", typeof(LRRSiteGroupPropertyQualifierValue));
                                        arrConditions.Add(condition3);
                                        IEnumerator iMasterAggregateGrpPropEnum = lrrMasterAggregateGrpProps.GetEnumerator();
                                        while (iMasterAggregateGrpPropEnum.MoveNext())
                                        {
                                            LRRMasterProperty lrrAggGrpProp = iMasterAggregateGrpPropEnum.Current as LRRMasterProperty;
                                            if (lrrAggGrpProp.PropID == lrrMasterProp.PropID)
                                            {
                                                SearchCondition conditionAggGp1 = GetConditionObj("LookUpPropertiesID", lrrLkUpProp.LookUpPropertiesID, "=", typeof(LRRAggregateGroupedPropertyValue));
                                                arrConditions.Add(conditionAggGp1);
                                                SearchCondition conditionAggGp2 = GetConditionObj("LookUpQualifierID", lrrLkUpQual.LookUpQualifierID, "=", typeof(LRRAggregateGroupedPropertyValue));
                                                arrConditions.Add(conditionAggGp2);
                                                SearchCondition conditionAggGp3 = GetConditionObj("MasterPropID", lrrMasterProp.PropID, "=", typeof(LRRAggregateGroupedPropertyValue));
                                                arrConditions.Add(conditionAggGp3);
                                            }
                                        }
                                        criteria.ArrListSearchCondition = arrConditions;
                                        childNode3.Tag = criteria;
                                        childNode2.Nodes.Add(childNode3);
                                    }
                                }
                            }
                            else
                            {
                                //Get the Data Arrays
                                LRRLookUpProperties lrrLkUpProps = LRRModel.LookUpProperties.getLookUpPropertiesForPropTypeID(lrrPropType.ID);
                                lrrLkUpProps.SortColumn = "Name";
                                lrrLkUpProps.Sort();
                                LRRMasterProperty lrrAggGrpProp = null;
                                IEnumerator iLkUpPropEnum = lrrLkUpProps.GetEnumerator();
                                while (iLkUpPropEnum.MoveNext())
                                {
                                    LRRLookUpProperty lrrLkUpProp = iLkUpPropEnum.Current as LRRLookUpProperty;

                                    //Grp Property Node (MasterProperties --> BFWFlow-->Air Components --> C00)
                                    TreeNode childNode2 = new TreeNode(lrrLkUpProp.Name);

                                    SearchCriteria criteria = new SearchCriteria();
                                    criteria.ObjType = typeof(LRRSiteGroupPropertyQualifierValue);
                                    ArrayList arrConditions = new ArrayList();
                                    SearchCondition conditionAggGp1 = null;
                                    SearchCondition conditionAggGp2 = null;
                                    SearchCondition condition1 = GetConditionObj("LookUpPropertiesID", lrrLkUpProp.LookUpPropertiesID, "=", typeof(LRRSiteGroupPropertyQualifierValue));
                                    arrConditions.Add(condition1);
                                    SearchCondition condition2 = GetConditionObj("MasterPropID", lrrMasterProp.PropID, "=", typeof(LRRSiteGroupPropertyQualifierValue));
                                    arrConditions.Add(condition2);
                                    IEnumerator iMasterAggregateGrpPropEnum = lrrMasterAggregateGrpProps.GetEnumerator();
                                    while (iMasterAggregateGrpPropEnum.MoveNext())
                                    {
                                        lrrAggGrpProp = iMasterAggregateGrpPropEnum.Current as LRRMasterProperty;
                                        if (lrrAggGrpProp.PropID == lrrMasterProp.PropID)
                                        {
                                            conditionAggGp1 = GetConditionObj("LookUpPropertiesID", lrrLkUpProp.LookUpPropertiesID, "=", typeof(LRRAggregateGroupedPropertyValue));
                                            arrConditions.Add(conditionAggGp1);
                                            conditionAggGp2 = GetConditionObj("MasterPropID", lrrMasterProp.PropID, "=", typeof(LRRAggregateGroupedPropertyValue));
                                            arrConditions.Add(conditionAggGp2);
                                        }
                                    }
                                    criteria.ArrListSearchCondition = arrConditions;
                                    childNode2.Tag = criteria;
                                    childNode1.Nodes.Add(childNode2);
                                }
                            }
                        }

                        lrrMasterSimpleProps.SortColumn = "Name";
                        lrrMasterSimpleProps.Sort();
                        IEnumerator iMasterSimplePropEnum = lrrMasterSimpleProps.GetEnumerator();
                        while (iMasterSimplePropEnum.MoveNext())
                        {
                            LRRMasterProperty lrrMasterProp = iMasterSimplePropEnum.Current as LRRMasterProperty;
                            LRRMasterProperty lrrAggMasterProp = null;
                            SearchCondition conditionAgg1 = null;
                            IEnumerator iMasterAggregatePropEnum = lrrMasterAggregateProps.GetEnumerator();
                            while (iMasterAggregatePropEnum.MoveNext())
                            {
                                lrrAggMasterProp = iMasterAggregatePropEnum.Current as LRRMasterProperty;
                                if (lrrAggMasterProp.PropID == lrrMasterProp.PropID)
                                {
                                    conditionAgg1 = GetConditionObj("ID", lrrMasterProp.PropID, "=", typeof(LRRAggregatePropertyValue));
                                    break;
                                }
                            }
                            TreeNode masterPropNode = new TreeNode(lrrMasterProp.Name);
                            SearchCondition condition1 = GetConditionObj("SiteComponentPropertyID", lrrMasterProp.PropID, "=",typeof(LRRSiteComponentPropertyValue));
                            SearchCriteria criteria1 = new SearchCriteria();
                            criteria1.ObjType = typeof(LRRSiteComponentPropertyValue);
                            ArrayList arrList1 = new ArrayList();
                            arrList1.Add(condition1);
                            if(conditionAgg1 != null)
                                arrList1.Add(conditionAgg1);
                            criteria1.ArrListSearchCondition = arrList1;
                            masterPropNode.Tag = criteria1;
                            masterNode.Nodes.Add(masterPropNode);
                        }
                    }
				}

                masterComps.SortColumn = "ComponentName";
                masterComps.Sort();

                IEnumerator mastCompEnum = masterComps.GetEnumerator();
                while (mastCompEnum.MoveNext())
                {
                    LRRMasterComponent mastComp = mastCompEnum.Current as LRRMasterComponent;
                    TreeNode child = new TreeNode();
                    child.Text = mastComp.ComponentName;
                    compTypeNode.Nodes.Add(child);
                }
			}
			trvMasterProps.Nodes.Add(root);
            trvMasterProps.Sort();
		}


        /// <summary>
        /// Existses the in all site componetn.
        /// </summary>
        /// <param name="lrrmasteProp">The lrrmaste prop.</param>
        /// <returns></returns>
        private bool ExistsInAllSiteComponetn(LRRMasterProperty lrrmasteProp)
        {
            LRRMasterComponent masterComp = LRRModel.Model.GetMasterComponentByID(lrrmasteProp.ComponentID);
            LRRSiteComponents lrrSiteComponents = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(masterComp.ID);
            foreach (LRRSiteComponent lsc in lrrSiteComponents)
            {
                foreach (LRRSiteComponentProperty lscp in lsc.Properties)
                {
                    if (lscp.PropertyID == lrrmasteProp.PropID)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// Gets the condition obj.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="val">The val.</param>
        /// <param name="oper">The oper.</param>
        /// <param name="objectType">Type of the object.</param>
        /// <returns></returns>
		private SearchCondition GetConditionObj(string name,string val,string oper,Type objectType)
		{
			SearchCondition condition = new SearchCondition();
			condition.Name = name;
			condition.Value = val;
			condition.Operator = oper;
            condition.ObjType = objectType; 
			return condition;
		}

        /// <summary>
        /// Gets all child nodes.
        /// </summary>
        /// <param name="trvNode">The TRV node.</param>
		private void GetAllChildNodes(TreeNode trvNode)
		{
			for(int i=0;i < trvNode.Nodes.Count;i++)
			{
				TreeNode trvChildNode = trvNode.Nodes[i];
				//trvChildNode.Checked = trvNode.Checked;
				if(trvChildNode.Nodes != null && trvChildNode.Nodes.Count > 0)
				{
					GetAllChildNodes(trvChildNode);
				}
				else
				{
					CheckUncheckListItems(trvNode);
				}
			}
		}

        /// <summary>
        /// Checks the uncheck list items.
        /// </summary>
        /// <param name="trvNode">The TRV node.</param>
		private void CheckUncheckListItems(TreeNode trvNode)
		{
			SearchCriteria criteria = trvNode.Tag as SearchCriteria;
			if(trvNode.Checked)
			{
                if(!_arrCollection.Contains(criteria))
                    _arrCollection.Add(criteria);
			}
			else
			{
				if(_arrCollection.Contains(criteria))
					_arrCollection.Remove(criteria);
			}
		}

        /// <summary>
        /// Sets the panel.
        /// </summary>
        /// <param name="pnlCollection">The PNL collection.</param>
		private void SetPanel(ArrayList pnlCollection)
		{
			if(pnlCollection.Count == 0)
				return;
			int heightOfEach = this.pnlGrid.Height/pnlCollection.Count;
			int splitCnt = 0;
			for(int i= pnlCollection.Count-1;i>=0;i--)
			{
				Panel panel = pnlCollection[i] as Panel;
				panel.Height = heightOfEach;
				this.pnlGrid.Controls.Add(panel);
				if(splitCnt < pnlCollection.Count - 1)
				{
					Splitter splitGrid = new Splitter();
					splitGrid.Dock = DockStyle.Top;
					this.pnlGrid.Controls.Add(splitGrid);
					splitCnt++;
				}
			}
			if(this.pnlGrid.Controls.Count > 0)
				this.pnlGrid.Controls[0].Dock = DockStyle.Fill;
		}

		#region Fields
		private LRRApp _lrrApp;
		private LRRMasterComponents _lrrMasterComps = new LRRMasterComponents();
		private LRRPropertyTypes _lrrPropertyTypes;
		private LRRComponentTypes _lrrCompTypes;
		private ArrayList _arrCollection = new ArrayList();
        private LRRUnifiedViewSearchCrtiterias _lrrUnifiedSrchCriterias = new LRRUnifiedViewSearchCrtiterias();
		private TabControl			_parentTabControl;
		private TabPage				_parentTabPage;
		private IUnifiedGridFactory _gridView;
		private ResultWindow		_resultComponentsWindow;
		private ResultWindow		_resultPropertiesWindow;
        private bool _isTransposed = false;
        public bool IsChanged = false;
        public static bool BPropertyValChanged = false;

        // added by ashwin

        public Hashtable _scSCPHas = new Hashtable();
		#endregion

        /// <summary>
        /// Handles the AfterCheck event of the trvMasterProps control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.TreeViewEventArgs"/> instance containing the event data.</param>
		private void trvMasterProps_AfterCheck(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
            //IsChanged = true;
			TreeNode trvNode = e.Node;
			if(trvNode.Nodes != null && trvNode.Nodes.Count > 0)
			{
				GetAllChildNodes(trvNode);
			}
			else
			{
				CheckUncheckListItems(trvNode);
			}
		}

        /// <summary>
        /// Handles the Click event of the btnExec control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void btnExec_Click(object sender, System.EventArgs e)
		{
			//Split arrayList into x and y
            try
            {
                ArrayList arrXCollection = new ArrayList();
                ArrayList arrYCollection = new ArrayList();
                
                for (int i = 0; i < _arrCollection.Count; i++)
                {
                    SearchCriteria criteria = _arrCollection[i] as SearchCriteria;
                    if (criteria != null)
                    {
                        if (criteria.ObjType == typeof(LRRSiteComponent) || criteria.ObjType == typeof(LRRAggregate))
                        {
                            arrXCollection.Add(criteria);
                        }
                        else
                        {
                            arrYCollection.Add(criteria);
                        }
                    }
                }

                if (arrXCollection.Count == 0)
                {
                    LRRMessagBox.ShowMessage("SID_ATLEASTSITE", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }

                if (arrYCollection.Count == 0)
                {
                    LRRMessagBox.ShowMessage("SID_ATLEASTPROP", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
                LRRUnifiedViewSearchCrtiteria cmbSearchItem = null;

                IUnifiedGridFactory oldGridView = _gridView;
                if (cmbQuerySelector != null)
                {
                    cmbSearchItem = cmbQuerySelector.SelectedItem != null ? cmbQuerySelector.SelectedItem as LRRUnifiedViewSearchCrtiteria : null;
                    if (cmbSearchItem != null && cmbSearchItem.TemplateClass == "Symcon.LightRidge.PEAdvisor.CommonViews.GridTransposedSavedViewController")
                    {
                        _gridView = new GridTransposedSavedViewController();
                        _gridView.SearchQueryID = cmbSearchItem.ID;
                        _isTransposed = true;
                    }
                    else
                    {
                        _gridView = new GridSavedViewController();
                        _gridView.SearchQueryID = cmbSearchItem != null ? cmbSearchItem.ID : null;
                        _isTransposed = false;
                    }
                }
                else
                {
                    _gridView = new GridSavedViewController();
                    _gridView.SearchQueryID = cmbSearchItem != null ? cmbSearchItem.ID : null;
                    _isTransposed = false;
                }

                if (oldGridView != null)
                {
                    oldGridView.ClearAllBindings();
                    oldGridView.UnSubscribeReconnectCompleted();
                }

                //_isTransposed = false;
                LRRWaitWindowUtility.ShowPogressIndicator(LRRHeaderText.GetHeaderString(""));
                Panel pnl = _gridView.LoadPanel(arrXCollection, arrYCollection);
                this.pnlGrid.Controls.Clear();
                SetPanel(_gridView.GetPanelCollection());
                _gridView.HideModes();

                //Show the Components Window
                LRRResultProperties resultComponents = CommonViewUtilities.GetSiteComponents(arrXCollection);
                _resultComponentsWindow.PropertyCollection = resultComponents;
                _resultComponentsWindow.Text = "Result Components Window";
                _resultComponentsWindow.BindGrid();
                _resultComponentsWindow.SelectedProperties += new Symcon.LightRidge.PEAdvisor.CommonViews.ResultWindow.SelectedPropertiesChanged(_resultComponentsWindow_SelectedProperties);
                _resultComponentsWindow.Owner = this;

                //Show the Properties Window
                LRRResultProperties resultProperties = CommonViewUtilities.GetPropeties(arrYCollection);
                _resultPropertiesWindow.PropertyCollection = resultProperties;
                _resultPropertiesWindow.Text = "Result Attributes Window";
                _resultPropertiesWindow.BindGrid();
                _resultPropertiesWindow.SelectedProperties += new Symcon.LightRidge.PEAdvisor.CommonViews.ResultWindow.SelectedPropertiesChanged(_resultPropertiesWindow_SelectedProperties);
                _resultPropertiesWindow.Owner = this;

                this.btnTranspose.Enabled = true;
                IsChanged = true;
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(SavedViewForm), "Execute Click: " + ex, Logger.Level.Debug);
            }
            finally
            {
                LRRWaitWindowUtility.EndProgressIndicator();
            }
		}


        /// <summary>
        /// Handles the Click event of the btnTranspose control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnTranspose_Click(object sender, EventArgs e)
        {
            
            if (_isTransposed)
                _isTransposed = false;
            else
                _isTransposed = true;

            //Clear All Bindings			
            if (_gridView != null)
            {
                _gridView.ClearAllBindings();
                _gridView.UnSubscribeReconnectCompleted();
            }

            if (_isTransposed)
            {
                _gridView = new GridTransposedSavedViewController();
            }
            else
            {
                _gridView = new GridSavedViewController();
            }

            //Split arrayList into x and y
            ArrayList arrXCollection = new ArrayList();
            ArrayList arrYCollection = new ArrayList();
            for (int i = 0; i < _arrCollection.Count; i++)
            {
                SearchCriteria criteria = _arrCollection[i] as SearchCriteria;
                if (criteria != null)
                {
                    if (criteria.ObjType == typeof(LRRSiteComponent) || criteria.ObjType == typeof(LRRAggregate))
                    {
                        arrXCollection.Add(criteria);
                    }
                    else
                    {
                        arrYCollection.Add(criteria);
                    }
                }
            }

            Panel pnl = _gridView.LoadPanel(arrXCollection, arrYCollection);
            this.pnlGrid.Controls.Clear();
            SetPanel(_gridView.GetPanelCollection());
            _gridView.HideModes();
        }

        /// <summary>
        /// Handles the DoubleClick event of the splitter2 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void splitter2_DoubleClick(object sender, System.EventArgs e)
		{
			if(pnlLeft.Visible)
			{
				pnlLeft.Visible = false;
				pnlGrid.Dock = DockStyle.Fill;
				splVertical.Dock = DockStyle.Left;
			}
			else if(!pnlLeft.Visible)
			{
				pnlLeft.Visible = true;
				pnlGrid.Dock = DockStyle.Right;
				splVertical.Dock = DockStyle.Right;
			}
		}

        /// <summary>
        /// Handles the ModesChanged event of the ProjectModes control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void ProjectModes_ModesChanged(object sender, EventArgs e)
		{
            if(_gridView != null)
			    _gridView.HideModes();
		}

        /// <summary>
        /// _results the components window_ selected properties.
        /// </summary>
        /// <param name="arrayUnSelectedRecords">The array un selected records.</param>
		private void _resultComponentsWindow_SelectedProperties(ArrayList arrayUnSelectedRecords)
		{
			_gridView.HideModes();
			_gridView.HideComponents(arrayUnSelectedRecords);
		}

        /// <summary>
        /// _results the properties window_ selected properties.
        /// </summary>
        /// <param name="arrayUnSelectedRecords">The array un selected records.</param>
		private void _resultPropertiesWindow_SelectedProperties(ArrayList arrayUnSelectedRecords)
		{
			_gridView.HideProperties(arrayUnSelectedRecords);
		}
		#region ILRRUIContainer Members

        /// <summary>
        /// Forces the control to invalidate its client area and immediately redraw itself and any child controls.
        /// </summary>
        /// <PermissionSet><IPermission class="System.Security.Permissions.EnvironmentPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Unrestricted="true"/><IPermission class="System.Security.Permissions.FileIOPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Unrestricted="true"/><IPermission class="System.Security.Permissions.SecurityPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Flags="UnmanagedCode, ControlEvidence"/><IPermission class="System.Diagnostics.PerformanceCounterPermission, System, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Unrestricted="true"/></PermissionSet>
		public void Refresh()
		{
			// TODO:  Add SavedViewForm.Refresh implementation
		}

        /// <summary>
        /// Saves this instance.
        /// </summary>
		public void Save()
        {
            //Save the Property Values & QualifierValues
            LRRSiteComponentPropertyValues toSaveSiteCompPropValues = LRRModel.PropertyValuesObj.GetChanges();
            //LRRAggregatePropertyValues toSaveAggrPropValues = LRRModel.AggregatePropertyValuesObj.GetChanges();
            LRRSiteGroupPropertyQualifierValues toSaveSiteGrpQualifierValues = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();
            if (toSaveSiteCompPropValues.Count > 0 || toSaveSiteGrpQualifierValues.Count > 0)
            {
                //DialogResult dlgResult = MessageBox.Show(this,"Do you want to save the property values?","Question",MessageBoxButtons.YesNo,MessageBoxIcon.Question); 
                if (toSaveSiteCompPropValues.Count > 0)
                {
                    _lrrApp.DBManagerObj.CommitCollection(toSaveSiteCompPropValues);
                    LRRModel.PropertyValuesObj.RefreshCollection();

                }
                if (toSaveSiteGrpQualifierValues.Count > 0)
                {
                    _lrrApp.DBManagerObj.CommitCollection(toSaveSiteGrpQualifierValues);
                    LRRModel.SiteGroupPropertyQualifierValuesObj.RefreshCollection();
                }
                //LRRMessagBox.ShowMessage("SID_PROP_VALUE_SAVE", true);
                LRRMessagBox.ShowMessage("SID_SAVEDINFO", true);
            }
            else
            {
                LRRMessagBox.ShowMessage("SID_NODATASAVED", true);
            }
            BPropertyValChanged = false;
		}

		public void CopyMasterProperties()
		{
			// TODO:  Add SavedViewForm.CopyMasterProperties implementation
		}

		public void Paste()
		{
			// TODO:  Add SavedViewForm.Paste implementation
		}

		public void Copy()
		{
			// TODO:  Add SavedViewForm.Copy implementation
		}

		public void Print()
		{
			if(_gridView != null)
			{
				_gridView.Print();
			}
		}

		public void PrintPreview()
		{
			if(_gridView != null)
			{
				_gridView.PrintPreview();
			}
		}

		public void PageSetup()
		{
			GridControlPrint gcPrint = new GridControlPrint();
			gcPrint.PageSetUp();
		}

		public void Undo()
		{
			// TODO:  Add SavedViewForm.Undo implementation
		}

		public void Delete()
		{
			// TODO:  Add SavedViewForm.Delete implementation
		}

		public void ArrangeColumns()
		{
			// TODO:  Add SavedViewForm.ArrangeColumns implementation
		}

		public void ClearFilter()
		{
			// TODO:  Add SavedViewForm.ClearFilter implementation
		}

		public void InsertNewRows()
		{
			// TODO:  Add SavedViewForm.InsertNewRows implementation
		}

		public void ImportToExcel()
		{
			// TODO:  Add SavedViewForm.ImportToExcel implementation
		}

		public void Export()
		{
			if(_gridView.Export() != null)
			{
				ExportToExcel excelExpForm = new ExportToExcel(_gridView.Export());
				excelExpForm.ShowDialog(this);
			}
		}

		public void SelectionPaste()
		{
			// TODO:  Add SavedViewForm.SelectionPaste implementation
		}

		public void RightLevel()
		{
			// TODO:  Add SavedViewForm.RightLevel implementation
		}

		public void LeftLevel()
		{
			// TODO:  Add SavedViewForm.LeftLevel implementation
		}

		public void UpLevel()
		{
			// TODO:  Add SavedViewForm.UpLevel implementation
		}

		public void DownLevel()
		{
			// TODO:  Add SavedViewForm.DownLevel implementation
		}

		public void ClearSort()
		{
			// TODO:  Add SavedViewForm.ClearSort implementation
		}

		public void CopyUOMTypes()
		{
			// TODO:  Add SavedViewForm.CopyUOMTypes implementation
		}

		public void ShowPropertyList()
		{
			if(_resultPropertiesWindow != null)
			{
				_resultPropertiesWindow.IsWindowOpen = true;
				_resultPropertiesWindow.Show();
			}
		}

		public void ShowComponentList()
		{
			if(_resultComponentsWindow != null)
			{
				_resultComponentsWindow.IsWindowOpen = true;
				_resultComponentsWindow.Show();
			}
		}

		public void LeftColMovement()
		{
			if(_gridView != null)
				_gridView.LeftColMovement();
		}

		public void RightColMovement()
		{
			if(_gridView != null)
				_gridView.RightColMovement();
		}

		public void OpenCompositeView()
		{
			// TODO:  Add SavedViewForm.OpenCompositeView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add SavedViewForm.TransposeView implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add SavedViewForm.OpenQuery implementation
		}

		public void OpenView()
		{
			// TODO:  Add SavedViewForm.OpenView implementation
		}

		public bool SupportsMasterPropertiesCopy()
		{
			// TODO:  Add SavedViewForm.SupportsMasterPropertiesCopy implementation
			return false;
		}

		public bool SupportsCopy()
		{
			// TODO:  Add SavedViewForm.SupportsCopy implementation
			return false;
		}

		public bool SupportsPaste()
		{
			// TODO:  Add SavedViewForm.SupportsPaste implementation
			return false;
		}

		public bool SupportsExport()
		{
			return true;
		}

		public bool SupportsSave()
		{
			return true;
		}

		public bool SupportsRefresh()
		{
			// TODO:  Add SavedViewForm.SupportsRefresh implementation
			return false;
		}

		public bool SupportsDelete()
		{
			// TODO:  Add SavedViewForm.SupportsDelete implementation
			return false;
		}

		public bool SupportsPrint()
		{
			// TODO:  Add SavedViewForm.SupportsPrint implementation
			return true;
		}

		public bool SupportsArrangeColumns()
		{
			// TODO:  Add SavedViewForm.SupportsArrangeColumns implementation
			return false;
		}

		public bool SupportsClearFilter()
		{
			// TODO:  Add SavedViewForm.SupportsClearFilter implementation
			return false;
		}

		public bool SupportsInsertNewRows()
		{
			// TODO:  Add SavedViewForm.SupportsInsertNewRows implementation
			return false;
		}

		public bool SupportsImportToExcel()
		{
			// TODO:  Add SavedViewForm.SupportsImportToExcel implementation
			return false;
		}

		public bool SupportsSelectionPaste()
		{
			// TODO:  Add SavedViewForm.SupportsSelectionPaste implementation
			return false;
		}

		public bool SupportsLevel()
		{
			// TODO:  Add SavedViewForm.SupportsLevel implementation
			return false;
		}

		public bool SupportsClearSort()
		{
			// TODO:  Add SavedViewForm.SupportsClearSort implementation
			return false;
		}

		public bool SupportsCopyUOMTypes()
		{
			// TODO:  Add SavedViewForm.SupportsCopyUOMTypes implementation
			return false;
		}

		public bool SupportsResultsFilter()
		{
			// TODO:  Add SavedViewForm.SupportsResultsFilter implementation
			return true;
		}

		public bool SupportsCompositeView()
		{
			// TODO:  Add SavedViewForm.SupportsCompositeView implementation
			return false;
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add SavedViewForm.SupportsPageSetup implementation
			return true;
		}

		public bool SupportsUndo()
		{
			// TODO:  Add SavedViewForm.SupportsUndo implementation
			return false;
		}

		public TabControl	ParentTabControl
		{
			set
			{
				_parentTabControl = value;
			}
			get{return _parentTabControl;}
		}
	
		public TabPage ParentTabPage
		{
			set
			{
				_parentTabPage = value;
			}
			get
			{
				return _parentTabPage;
			}
		}

		public bool PerformDirtyChk
		{
			get
			{
				return true;
			}
		}

        public void CheckForChanges(Form frm,bool fromRun)
        {


            GridCheckForViewChanges.CheckForViewChanges(IsChanged, _gridView, null, frm, null, null, false, null, null, null, null, fromRun);
            //Save the Property Value
            /*if (_isChanged)
            {
                
                DialogResult k =   LRRMessagBox.ShowMessage("SID_SAVE_QUERY","SID_LABEL",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
                if (k == DialogResult.Yes)
                {
                    SaveQueryForm saveQueryForm = new SaveQueryForm(null, "Query");
                    InternationalizationUtils.GetControlText(saveQueryForm, null);
                    DialogResult result = saveQueryForm.ShowDialog();
                    if (result == DialogResult.OK)
                    {

                        _gridView.Save(saveQueryForm.queryName, saveQueryForm.queryDesc, saveQueryForm.SelectedLevel.LevelID);
                        LRRMessagBox.ShowMessage("SID_SAVE_VIEW_SUCCESSFULLY", true);
                        _isChanged = false;
                    }
                }
            }

            //Save the Property Values
            LRRSiteComponentPropertyValues toSaveSiteCompPropValues = LRRModel.PropertyValuesObj.GetChanges();
            LRRAggregatePropertyValues toSaveAggrPropValues = LRRModel.AggregatePropertyValuesObj.GetChanges();
            LRRSiteGroupPropertyQualifierValues toSaveGPValues = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();
            if ((toSaveSiteCompPropValues.Count > 0) || (toSaveAggrPropValues.Count > 0) || (toSaveGPValues.Count > 0))
            {
                //DialogResult dlgResult = MessageBox.Show(this,"Do you want to save the property values?","Question",MessageBoxButtons.YesNo,MessageBoxIcon.Question); 
                DialogResult dlgResult = LRRMessagBox.ShowMessage(this, "SID_SAVE_PROPVALUE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dlgResult == DialogResult.Yes)
                {
                    Save();
                    _isChanged = false;
                }
                else
                {
                    foreach (LRRSiteComponentPropertyValue lscpv in toSaveSiteCompPropValues)
                    {
                        LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                        LRRSiteComponentPropertyValue lscpvChanged = LRRModel.GetPropertyValueObject(typeof(LRRSiteComponent), lscpv.ModeID, lscp.ID, lscp.ComponentID) as LRRSiteComponentPropertyValue;
                        lscpvChanged.RollBackChanges(true);
                    }
                    foreach (LRRSiteGroupPropertyQualifierValue lscgpv in toSaveGPValues)
                    {
                        LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscgpv.SiteComponentPropID);
                        if (siteCompProp != null)
                        {
                            string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompProp.ID, lscgpv.LookUpPropertiesID);
                            LRRLookUpQualifier lkUpQualObj = null;
                            if (lscgpv.LookUpQualifierID != null && lscgpv.LookUpQualifierID.Trim() != "")
                                lkUpQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lscgpv.LookUpQualifierID);
                            string qualID = lkUpQualObj == null ? null : lkUpQualObj.QualifierID;
                            LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp, lscgpv.LookUpPropertiesID, lscgpv.QualifierID, lscgpv.ModeID);
                            if (qualValueObj != null)
                            {
                                qualValueObj.RollBackChanges(true);
                            }
                        }
                    }
                    foreach (LRRAggregatePropertyValue aggPropVal in toSaveAggrPropValues)
                    {
                        aggPropVal.RollBackChanges(true);
                    }
                }
            }*/
        }

		#endregion

		#region ILRRUISpecificContainer Members

		public void ChangeUOMSet()
		{
			if(_gridView != null)
			{
				ChangeUOMSetDefinitionForm chngUOMSetDefn = new ChangeUOMSetDefinitionForm();
				DialogResult dlgResult = chngUOMSetDefn.ShowDialog();
				if(dlgResult == DialogResult.OK && chngUOMSetDefn.MeasSysID != null)
				{
					_gridView.ChangeUOMSet(chngUOMSetDefn.MeasSysID);
				}
			}
			
		}

		public bool SupportsChangeUOMSet()
		{			
			return true;
		}

		#endregion

        /// <summary>
        /// Gets all tree levels.
        /// </summary>
        /// <param name="currentNode">The current node.</param>
        /// <param name="parentNodeList">The parent node list.</param>
        private void GetAllTreeLevels(TreeNode currentNode, ref List<TreeNode> parentNodeList)
        {
            if (currentNode != null)
            {
                if (currentNode.Nodes.Count > 0)
                {
                    foreach (TreeNode node in currentNode.Nodes)
                    {
                        parentNodeList.Add(node);
                        GetAllTreeLevels(node, ref parentNodeList);
                    }
                }
                else
                    foreach (TreeNode node in currentNode.Nodes)
                    {
                        parentNodeList.Add(node);
                    }
            }
        }

        #endregion

        #region Events
        /// <summary>
        /// Handles the DropDown event of the cmbQuerySelector control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void cmbQuerySelector_DropDown(object sender, System.EventArgs e)
        {
            ComboBox senderComboBox = (ComboBox)sender;
            int width = senderComboBox.DropDownWidth;
            Graphics g = senderComboBox.CreateGraphics();
            Font font = senderComboBox.Font;
            int vertScrollBarWidth =
                (senderComboBox.Items.Count > senderComboBox.MaxDropDownItems)
                ? SystemInformation.VerticalScrollBarWidth : 0;

            int newWidth;
            foreach (LRRUnifiedViewSearchCrtiteria uvSearchCriteria in ((ComboBox)sender).Items)
            {
                string s = uvSearchCriteria.Name;
                newWidth = (int)g.MeasureString(s, font).Width
                    + vertScrollBarWidth;
                if (width < newWidth)
                {
                    width = newWidth;
                }
            }
            senderComboBox.DropDownWidth = width;
        }

        /// <summary>
        /// Handles the Click event of the btnSaveView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnSaveView_Click(object sender, EventArgs e)
        {
            if (_gridView == null)
            {
                MessageBox.Show("Please execute the query before saving!");
                return;
            }

            string queryName = cmbQuerySelector.Text;
            SaveQueryForm saveQueryForm = new SaveQueryForm(queryName, "UnifiedView");
            InternationalizationUtils.GetControlText(saveQueryForm, null);
            DialogResult result = saveQueryForm.ShowDialog();
            if (result == DialogResult.OK)
            {
                _gridView.Save(saveQueryForm.queryName, saveQueryForm.queryDesc, saveQueryForm.SelectedLevel.LevelID);
                //LRRMessagBox.ShowMessage("SID_SAVE_VIEW_SUCCESSFULLY", true);
                IsChanged = false;
                _lrrUnifiedSrchCriterias = LRRModel.ChildModel.UnifiedViewSearchCrtiterias;
                _lrrUnifiedSrchCriterias.SortColumn = "Name";
                _lrrUnifiedSrchCriterias.Sort();
                cmbQuerySelector.DataSource = _lrrUnifiedSrchCriterias;
                if(_gridView.SearchQueryID != null)
                    cmbQuerySelector.SelectedItem = _lrrUnifiedSrchCriterias.GetUnifiedViewByID(_gridView.SearchQueryID);
                
            }
        }

        /// <summary>
        /// Handles the FormClosing event of the SavedViewForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void SavedViewForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            //ChangedUIUOMDictionary.Clear(this.GetType());
            Form frm = sender as Form;
            CheckForChanges(frm,false);
            IsChanged = false;
            BPropertyValChanged = false;

            if (_gridView != null)
            {
                _gridView.ClearAllBindings();
                _gridView.UnSubscribeReconnectCompleted();
            }

            /*if (_isChanged)
            {
                DialogResult k =   LRRMessagBox.ShowMessage("SID_SAVE_QUERY","SID_LABEL",MessageBoxButtons.YesNo,MessageBoxIcon.Question);
                if (k == DialogResult.Yes)
                {
                    SaveQueryForm saveQueryForm = new SaveQueryForm(null, "Query");
                    InternationalizationUtils.GetControlText(saveQueryForm, null);
                    DialogResult result = saveQueryForm.ShowDialog();
                    if (result == DialogResult.OK)
                    {
                        _gridView.Save(saveQueryForm.queryName, saveQueryForm.queryDesc, saveQueryForm.SelectedLevel.LevelID);
                        LRRMessagBox.ShowMessage("SID_SAVE_VIEW_SUCCESSFULLY", true);
                    }
                }
            }

            //Save the Property Values
            LRRSiteComponentPropertyValues toSaveSiteCompPropValues = LRRModel.PropertyValuesObj.GetChanges();
            LRRAggregatePropertyValues toSaveAggrPropValues = LRRModel.AggregatePropertyValuesObj.GetChanges();
            LRRSiteGroupPropertyQualifierValues toSaveGPValues = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();
            if ((toSaveSiteCompPropValues.Count > 0) || (toSaveAggrPropValues.Count > 0) || (toSaveGPValues.Count > 0))
            {
                //DialogResult dlgResult = MessageBox.Show(this,"Do you want to save the property values?","Question",MessageBoxButtons.YesNo,MessageBoxIcon.Question); 
                DialogResult dlgResult = LRRMessagBox.ShowMessage(this, "SID_SAVE_PROPVALUE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dlgResult == DialogResult.Yes)
                {
                    Save();
                }
                else
                {
                    foreach (LRRSiteComponentPropertyValue lscpv in toSaveSiteCompPropValues)
                    {
                        LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                        LRRSiteComponentPropertyValue lscpvChanged = LRRModel.GetPropertyValueObject(typeof(LRRSiteComponent), lscpv.ModeID, lscp.ID, lscp.ComponentID) as LRRSiteComponentPropertyValue;
                        lscpvChanged.RollBackChanges(true); 
                    }
                    foreach (LRRSiteGroupPropertyQualifierValue lscgpv in toSaveGPValues)
                    {
                        LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscgpv.SiteComponentPropID); 
                        if (siteCompProp != null)
                        {
                            string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompProp.ID, lscgpv.LookUpPropertiesID);
                            LRRLookUpQualifier lkUpQualObj = null;
                            if (lscgpv.LookUpQualifierID != null && lscgpv.LookUpQualifierID.Trim() != "")
                                lkUpQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lscgpv.LookUpQualifierID);
                            string qualID = lkUpQualObj == null ? null : lkUpQualObj.QualifierID;
                            LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp, lscgpv.LookUpPropertiesID, lscgpv.QualifierID, lscgpv.ModeID);
                            if (qualValueObj != null)
                            {
                                qualValueObj.RollBackChanges(true);
                            }
                        }
                    }
                    foreach (LRRAggregatePropertyValue aggPropVal in toSaveAggrPropValues)
                    {
                        aggPropVal.RollBackChanges(true);
                    }
                }
            }*/
        }
        /// <summary>
        /// Handles the SelectedIndexChanged event of the cmbQuerySelector control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void cmbQuerySelector_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                _arrCollection.Clear();
                trvMasterProps.BeginUpdate();
                TreeNode rootNode = trvMasterProps.Nodes[0];
                trvMasterProps.SelectedNode = trvMasterProps.Nodes[0];
                LRRUnifiedViewSearchCrtiteria uvSearchCriteria = cmbQuerySelector.SelectedItem as LRRUnifiedViewSearchCrtiteria;

                Global.TreeViewUtility.UncheckNodes(rootNode);
                Symcon.LightRidge.PEAdvisor.Global.TreeViewUtility.TreeViewColorConversion(rootNode);
                if (uvSearchCriteria != null || uvSearchCriteria.Name != "New Query")
                {

                    List<TreeNode> parentNodeList = new List<TreeNode>();
                    List<LRRSiteComponent> lstSitCompToSelect = new List<LRRSiteComponent>();
                    List<LRRAggregate> lstAggregateToSelect = new List<LRRAggregate>();
                    List<LRRMasterProperty> lstMasterPropsToSelect = new List<LRRMasterProperty>();
                    List<LRRAggregatePropertyValue> lstAggreGatePropertyValueToSelect = new List<LRRAggregatePropertyValue>();
                    List<LRRLookUpProperty> lstLookUpPropertyToSelect = new List<LRRLookUpProperty>();
                    List<SearchCriteria> lstSearchCriteria = new List<SearchCriteria>();

                    string unifiedSearchCriteriaID = uvSearchCriteria.ID;
                    LRRUnifiedViewSearchItems lrrSrchItems = new LRRUnifiedViewSearchItems();

                    //Getting the selected values according to searchCriteriaID
                    _lrrApp.DBManagerObj.PopulateCollection(lrrSrchItems, " where SearchCriteriaID = '" + unifiedSearchCriteriaID + "'");

                    LRRUnifiedViewSearchItemConditions lrrSrchItemConditions = new LRRUnifiedViewSearchItemConditions();
                    _lrrApp.DBManagerObj.PopulateCollection(lrrSrchItemConditions, " where SearchCriteriaID = '" + unifiedSearchCriteriaID + "'");

                    IEnumerator iItemEnum = lrrSrchItems.GetEnumerator();
                    while (iItemEnum.MoveNext())
                    {
                        int lrrSearchItemsCount = 1;

                        LRRUnifiedViewSearchItem lrrSrchItem = iItemEnum.Current as LRRUnifiedViewSearchItem;
                        LRRUnifiedViewSearchItemConditions lrrSrchItemConds = lrrSrchItemConditions.GetItemConditionsByItemID(lrrSrchItem.ID);

                        SearchCriteria criteria = new SearchCriteria();
                        criteria.ObjType = CommonViewUtilities.GetType(lrrSrchItem.Type);

                        IEnumerator iEnum = lrrSrchItemConds.GetEnumerator();

                        while (iEnum.MoveNext())
                        {
                            LRRUnifiedViewSearchItemCondition itemCond = iEnum.Current as LRRUnifiedViewSearchItemCondition;
                            SearchCondition condition = new SearchCondition();
                            condition.Name = itemCond.Name;
                            condition.Value = itemCond.ConditionValue;
                            condition.Operator = itemCond.Oper;


                            if (criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
                            {
                                condition.ObjType = typeof(LRRSiteGroupPropertyQualifierValue);

                                criteria.ArrListSearchCondition.Add(condition);

                                if (lrrSearchItemsCount == lrrSrchItemConds.Count)
                                {
                                    lstSearchCriteria.Add(criteria);
                                }
                                lrrSearchItemsCount++;

                            }

                            if (criteria.ObjType == typeof(LRRSiteComponent) || criteria.ObjType == typeof(LRRAggregate))
                            {
                                if (criteria.ObjType == typeof(LRRSiteComponent))
                                {
                                    condition.ObjType = typeof(LRRSiteComponent);
                                    string siteCompID = condition.Value.ToString();
                                    LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompID);
                                    if (siteComp != null)
                                    {
                                        lstSitCompToSelect.Add(siteComp);
                                    }
                                }
                                else
                                {
                                    condition.ObjType = typeof(LRRAggregate);
                                    string aggregateID = condition.Value.ToString();
                                    LRRAggregate aggregate = LRRModel.AggregatesObj.GetAggregateFromID(aggregateID);
                                    if (aggregate != null)
                                    {
                                        lstAggregateToSelect.Add(aggregate);
                                    }
                                }
                            }
                            else
                            {
                                if (condition.Name == "ID")
                                {
                                    condition.ObjType = typeof(LRRAggregatePropertyValue);
                                    string aggPropertyID = condition.Value.ToString();
                                    LRRAggregatePropertyValue aggregatePropertyValue = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(aggPropertyID);
                                    if (aggregatePropertyValue != null)
                                    {
                                        lstAggreGatePropertyValueToSelect.Add(aggregatePropertyValue);
                                    }
                                }
                                else if (condition.Name == "SiteComponentPropertyID")
                                {
                                    condition.ObjType = typeof(LRRSiteComponentPropertyValue);
                                    string siteCompPropID = condition.Value.ToString();
                                    LRRMasterProperty masterProperty = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(siteCompPropID);
                                    if (masterProperty != null)
                                    {
                                        lstMasterPropsToSelect.Add(masterProperty);
                                    }
                                }
                            }
                        }
                    }

                    parentNodeList.Add(trvMasterProps.SelectedNode);
                    GetAllTreeLevels(trvMasterProps.SelectedNode, ref parentNodeList);
                    for (int i = trvMasterProps.SelectedNode.Index; i < trvMasterProps.GetNodeCount(true); i++)
                    {
                        string lkUpPropertyID = null;
                        string lookUpQualifierID = null;
                        string masterPropID = null;
                        string lstLookUpPropID = null;
                        string lstLookUpQualifierID = null;
                        string lstMasterPropID = null;

                        SearchCriteria srchCriteria = parentNodeList[i].Tag as SearchCriteria;
                        TreeNode levelNode = parentNodeList[i] as TreeNode;

                        string nodeName = levelNode.Text;

                        //Selecting the nodes according to its Tag ID's 
                        if ((srchCriteria != null && srchCriteria.ArrListSearchCondition != null) && srchCriteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
                        {
                            foreach (SearchCondition searchCondition in srchCriteria.ArrListSearchCondition)
                            {
                                if (searchCondition.Name == "LookUpPropertiesID")
                                    lkUpPropertyID = searchCondition.Value;
                                if (searchCondition.Name == "MasterPropID")
                                    masterPropID = searchCondition.Value;
                                if (searchCondition.Name == "LookUpQualifierID")
                                    lookUpQualifierID = searchCondition.Value;
                            }

                            IEnumerator iSearchEnum = lstSearchCriteria.GetEnumerator();
                            while (iSearchEnum.MoveNext())
                            {
                                SearchCriteria criteria = (SearchCriteria)iSearchEnum.Current;
                                foreach (SearchCondition srchCondition in criteria.ArrListSearchCondition)
                                {
                                    if (srchCondition.Name == "LookUpPropertiesID")
                                        lstLookUpPropID = srchCondition.Value;
                                    if (srchCondition.Name == "MasterPropID")
                                        lstMasterPropID = srchCondition.Value;
                                    if (srchCondition.Name == "LookUpQualifierID")
                                        lstLookUpQualifierID = srchCondition.Value;
                                }
                                if (lstLookUpPropID == lkUpPropertyID && lstMasterPropID == masterPropID && lstLookUpQualifierID == lookUpQualifierID)
                                {
                                    levelNode.Checked = true;
                                    Global.TreeViewUtility.CheckNodes(levelNode);
                                    Symcon.LightRidge.PEAdvisor.Global.TreeViewUtility.TreeViewColorConversion(levelNode);
                                }
                                lstLookUpPropID = null;
                                lstLookUpQualifierID = null;
                                lstMasterPropID = null;
                            }
                            lkUpPropertyID = null;
                            lookUpQualifierID = null;
                            masterPropID = null;

                        }

                        else if (srchCriteria != null && srchCriteria.ArrListSearchCondition != null)
                        {
                            foreach (SearchCondition searchCondition in srchCriteria.ArrListSearchCondition)
                            {
                                if (searchCondition.ObjType == typeof(LRRSiteComponent))
                                {
                                    foreach (LRRSiteComponent siteComponent in lstSitCompToSelect)
                                    {
                                        if (siteComponent.Name == nodeName && siteComponent.SiteComponentID == searchCondition.Value)
                                        {
                                            levelNode.Checked = true;
                                            Global.TreeViewUtility.CheckNodes(levelNode);
                                            Symcon.LightRidge.PEAdvisor.Global.TreeViewUtility.TreeViewColorConversion(levelNode);
                                        }
                                    }
                                }

                                else if (searchCondition.ObjType == typeof(LRRSiteComponentPropertyValue))
                                {
                                    foreach (LRRMasterProperty masterProperty in lstMasterPropsToSelect)
                                    {
                                        if (masterProperty.Name == nodeName && masterProperty.PropID == searchCondition.Value)
                                        {
                                            levelNode.Checked = true;
                                            Global.TreeViewUtility.CheckNodes(levelNode);
                                            Symcon.LightRidge.PEAdvisor.Global.TreeViewUtility.TreeViewColorConversion(levelNode);
                                        }

                                    }
                                }

                                else if (searchCondition.ObjType == typeof(LRRAggregate))
                                {
                                    foreach (LRRAggregate aggregate in lstAggregateToSelect)
                                    {
                                        if (aggregate.Name == nodeName && aggregate.AggregateID == searchCondition.Value)
                                        {
                                            levelNode.Checked = true;
                                            Global.TreeViewUtility.CheckNodes(levelNode);
                                            Symcon.LightRidge.PEAdvisor.Global.TreeViewUtility.TreeViewColorConversion(levelNode);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                trvMasterProps.EndUpdate();
            }
            catch (Exception ex)
            {
                trvMasterProps.EndUpdate();
                Logger.LogMessage(typeof(SavedViewForm), "DropDown IndexChanged Click: " + ex, Logger.Level.Debug);
            }
        }

        /// <summary>
        /// Handles the KeyPress event of the cmbQuerySelector control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyPressEventArgs"/> instance containing the event data.</param>
        private void cmbQuerySelector_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }
        #endregion
    }
}
