using System;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.BusinessObjects;

namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
    public class DashBoardCreatorFactory
    {
        public DashBoardCreatorFactory()
        {
        }

        /// <summary>
        /// Method creates the dashboard
        /// </summary>
        /// <param name="dashBoardXml"></param>
        /// <returns></returns>
        public static TableLayoutPanel CreateDashBoard(XmlDashBoard dashBoardXml)
        {
            LoadControlNames();

            TableLayoutPanel layoutpanel = new TableLayoutPanel();
            layoutpanel.RowCount = dashBoardXml.Rows;
            layoutpanel.ColumnCount = dashBoardXml.Cols;

            float dColSize = 100.0f / layoutpanel.ColumnCount;
            float dRowSize = 100.0f / layoutpanel.RowCount;

            int iIndex = 0;
            for (int iRow = 0; iRow < layoutpanel.RowCount; iRow++)
            {
                for (int iCol = 0; iCol < layoutpanel.ColumnCount; iCol++)
                {
                    Cell cell = dashBoardXml.Cells[iIndex] as Cell;
                    string controlName = cell.ControlName;
                    if (_hashControls.ContainsKey(controlName))
                        controlName = _hashControls[controlName] as string;

                    try
                    {
                        Type objType = _assembly.GetType(controlName);
                        Control objControl = Activator.CreateInstance(objType) as Control;
                        layoutpanel.Controls.Add(objControl,iCol,iRow);
                        layoutpanel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, dColSize));
                        layoutpanel.RowStyles.Add(new RowStyle(SizeType.Percent, dRowSize));
                        objControl.Dock = DockStyle.Fill;
                        objControl.Click += new EventHandler(objControl_Click);

                        //Bind the Control
                        BindingInfo bindInfo = cell.BindInfo;
                        string sourceObjectName = bindInfo.SourceObjectName;
                        string sourceObjectID = bindInfo.SourceObjectID;
                        string sourceProperty = bindInfo.SourceProperty;

                        object sourceObject = GetSourceObject(sourceObjectName, sourceObjectID);
                        _lrrBindingManager.AddBinding(sourceObject, sourceProperty, objControl, "TargetValue");
                        _lrrBindingManager.AddBinding(cell,"MaxLimit", objControl, "MaxValue");
                        _lrrBindingManager.AddBinding(cell, "MinLimit", objControl, "MinValue");
                        _lrrBindingManager.AddBinding(cell.BindInfo, "SiteComponentName", objControl, "SourceItem");
                        _lrrBindingManager.AddBinding(cell.BindInfo, "SiteComponentProperty", objControl, "Attribute");
                        _lrrBindingManager.AddBinding(cell.BindInfo, "ModeName", objControl, "ModeName");
                    }
                    catch { }

                    iIndex++;
                }
            }
            return layoutpanel;
        }

        private static void objControl_Click(object sender, EventArgs e)
        {
            SelectedCtrl = sender as Control;   
        }

        /// <summary>
        /// Method gets the source object
        /// </summary>
        /// <param name="sourceObjectName"></param>
        /// <param name="sourceObjectID"></param>
        /// <returns></returns>
        private static object GetSourceObject(string sourceObjectName, string sourceObjectID)
        {
            object retValue = null;
            switch (sourceObjectName)
            {
                case "LRRSiteComponentPropertyValue":
                    retValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(sourceObjectID);
                    break;
                case "LRRSiteGroupPropertyQualifierValueID":
                    retValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sourceObjectID);
                    break;
                default:
                    break;
            }
            return retValue;
        }

        /// <summary>
        /// Method loads the control Names
        /// </summary>
        private static void LoadControlNames()
        {
            _hashControls.Clear();
            _assembly = System.Reflection.Assembly.Load("Symcon.LightRidge.DashBoardControlsLibrary");
            Type[] types = _assembly.GetTypes();
            for (int i = 0; i < types.Length; i++)
            {
                Type type = types[i];
                if (type.Namespace == "Symcon.LightRidge.DashBoardControlsLibrary")
                {
                    if (!_hashControls.ContainsKey(type.Name))
                        _hashControls.Add(type.Name, type.FullName);
                }
            }
        }

        #region Fields
        private static LRRBindingManager _lrrBindingManager = new LRRBindingManager();
        private static Hashtable _hashControls = new Hashtable();
        private static Assembly _assembly = null;

        private static Control _selectedCtrl = null;
        public static Control SelectedCtrl
        {
            get { return DashBoardCreatorFactory._selectedCtrl; }
            set { DashBoardCreatorFactory._selectedCtrl = value; }
        }
        #endregion
    }
}
