using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Diagram;

using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.IOScreen;
using Symcon.LightRidge.DashBoardControlsLibrary;

namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
	/// <summary>
	/// Summary description for PropertyDetailsDialog.
	/// </summary>
	public class AttributeDetailsDialog : System.Windows.Forms.Form
	{
		//private System.Windows.Forms.Panel pnlPropDetails;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;


		public AttributeDetailsDialog()
		{
			InitializeComponent();
			InternationalizationUtils.GetControlText(this,null); 
			LRRModel.ProjectModes.ModesChanged+=new EventHandler(ProjectModes_ModesChanged);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
					LRRModel.ProjectModes.ModesChanged-=new EventHandler(ProjectModes_ModesChanged);
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(AttributeDetailsDialog));
            this.lblInitMsg = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // lblInitMsg
            // 
            this.lblInitMsg.Location = new System.Drawing.Point(40, 104);
            this.lblInitMsg.Name = "lblInitMsg";
            this.lblInitMsg.Size = new System.Drawing.Size(216, 24);
            this.lblInitMsg.TabIndex = 0;
            this.lblInitMsg.Tag = "SID_SELECT_OBJ";
            // 
            // AttributeDetailsDialog
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.BackColor = System.Drawing.SystemColors.Window;
            this.ClientSize = new System.Drawing.Size(292, 266);
            this.Controls.Add(this.lblInitMsg);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.MaximizeBox = false;
            this.Name = "AttributeDetailsDialog";
            this.ShowInTaskbar = false;
            this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Show;
            this.Tag = "SID_ATT_DETL";
            this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.AttributeDetailsDialog_FormClosing);
            this.ResumeLayout(false);

		}
		#endregion

		private System.Windows.Forms.Label lblInitMsg;

		private object _gridView;


		public object GridView
		{
			get { return _gridView;}
			set
			{
				_gridView = value;
			}
		}

		public void ClearControls()
		{
			if(this.Controls.Count > 0)
			{
				this.Controls.Clear();
			}
		}

		public void UpdatePropertyDetailsPanel(AttributeDetailsEventArgs e)
		{
			this.SuspendLayout();
			if(this.GridView != null)
			{
				IGridFactory iGridFactory = this.GridView as IGridFactory;
				iGridFactory.ClearAllBindings();
				iGridFactory.UnSubscribeReconnectCompleted();
			}
			this.ClearControls();

            string siteCompPropValID = e.SiteCompPropValID;
            LRRSiteComponentPropertyValue siteCompPropValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(siteCompPropValID);
            if(siteCompPropValue != null)
            {
                LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropValue.SiteComponentPropertyID);
                if(siteCompProp != null)
                {
                    LRRSiteComponent lrrSiteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
                    if(lrrSiteComponent != null)
                    {
                        LRRSiteComponentProperties lrrSiteCompProps = LRRModel.SiteComponentPropertiesObj.GetAllSiteComponentPropertiesByComponentID(lrrSiteComponent.SiteComponentID);
                        lrrSiteComponent.Properties = lrrSiteCompProps;

                        LRRSiteComponents lrrSiteComponents = new LRRSiteComponents();
                        lrrSiteComponents.Add(lrrSiteComponent);
                        ArrayList custCollectionList = new ArrayList();
			            custCollectionList.Add(new LRRAggregates());
			            custCollectionList.Add(lrrSiteComponents);
			            GridSimpleViewController gridView = new GridSimpleViewController();
			            gridView.LoadPanel(custCollectionList,null);
			            SetPanel(gridView.GetPanelCollection());
			            _gridView = gridView;
			            gridView.HideModes();
                    }
                }
            }
			this.ResumeLayout(true);
			Cursor.Current = Cursors.Default;
		}


		
		private	void SetPanel(ArrayList	pnlCollection)
		{
			int heightOfEach = this.Height/pnlCollection.Count;
			int splitCnt = 0;
			for(int i=pnlCollection.Count-1;i>=0;i--)
			{
				Panel panel = pnlCollection[i] as Panel;
				panel.Height = heightOfEach;
				this.Controls.Add(panel);
				if(splitCnt < pnlCollection.Count - 1)
				{
					Splitter splitGrid = new Splitter();
					splitGrid.Dock = DockStyle.Top;
					this.Controls.Add(splitGrid);
					splitCnt++;
				}
			}
			if(this.Controls.Count > 0)
				this.Controls[0].Dock = DockStyle.Fill;
		}

		private void ProjectModes_ModesChanged(object sender, EventArgs e)
		{
			Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory igridFactory = this.GridView as Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
			if(igridFactory != null)
				igridFactory.HideModes();
			else
			{
				Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory igrpGridFactory = this.GridView as Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory;
				if(igrpGridFactory != null)
					igrpGridFactory.HideModes();
			}
		}

        private void AttributeDetailsDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
			if(this.GridView != null)
			{
				IGridFactory iGridFactory = this.GridView as IGridFactory;
				iGridFactory.ClearAllBindings();
				iGridFactory.UnSubscribeReconnectCompleted();
			}
			this.Controls.Clear();
			this.Controls.Add(lblInitMsg);
			this.Hide();
        }
	}
}
