using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Utils.HeaderDispatcher;
using Symcon.Utils.MessageDispatcher;

namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
    public partial class BindTree : Form
    {
        public BindTree()
        {
            InitializeComponent();
            InternationalizationUtils.GetControlText(this, null); 
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitDataSource();
            InitializeTreeView();
        }

        public BindTree(LRRSiteComponent siteComponent)
        {
            InitializeComponent();
            InternationalizationUtils.GetControlText(this, null);
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitDataSource(siteComponent);
            InitializeTreeView(siteComponent, _siteComponentProperties);
        }

        public BindTree(LRRAggregate aggregate)
        {
            InitializeComponent();
            InternationalizationUtils.GetControlText(this, null);
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitDataSource(aggregate);
            InitializeTreeView(aggregate, _aggregateProperties);
        }


        #region Methods

        private void InitializeTreeView(LRRAggregate aggregate, LRRAggregateProperties _aggregateProperties)
        {
            TreeNode root = new TreeNode(aggregate.Name);
            foreach (LRRAggregateProperty aggProp in _aggregateProperties)
            {
                //Check if Property is Simple
                LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(aggProp.MasterPropertyID);   
                if (masterProp.Type == "1")
                {
                    TreeNode aggPropNode = new TreeNode(aggProp.Name);
                    root.Nodes.Add(aggPropNode);

                    for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                    {
                        LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                        LRRAggregatePropertyValue aggPropValue = aggProp.GetValueObjectForPropertyMode(lrrMode.ModeId);
                        if (aggPropValue != null)
                        {
                            BindingInfo bindInfo = new BindingInfo();
                            bindInfo.SourceObjectName = "LRRAggregatePropertyValue";
                            bindInfo.SelectedSiteComponentProperties.Add(aggProp);
                            bindInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(aggPropValue.ModeID));
                            bindInfo.SourceObjectID = aggPropValue.ID;
                            bindInfo.SourceProperty = "PropertyValue";
                            bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(aggPropValue.ModeID).ModeDisplayName;
                            bindInfo.SiteComponentProperty = aggProp.Name;
                            bindInfo.SiteComponentName = aggregate.Name;
                            TreeNode aggPropValueNode = new TreeNode(lrrMode.ModeName);
                            aggPropValueNode.Tag = bindInfo;
                            aggPropNode.Nodes.Add(aggPropValueNode);
                        }
                    }
                }
                else
                {
                    if (masterProp.Type != null && masterProp.Type != "1")
                    {
                        LRRPropertyType lrrPropType = _lrrPropertyTypes.getPropertyTypeForID(masterProp.Type);

                        //Property Type Node (SiteComponentProperty --> BFWFlow-->Air Components)
                        TreeNode childNode1 = new TreeNode(aggProp.Name + "-" + lrrPropType.PropertyType);
                        root.Nodes.Add(childNode1);

                        if (!lrrPropType.IsDataArray)
                        {
                            LRRLookUpQualifiers lrrLkUpQuals = LRRModel.LookUpQualifiers.getLookUpQualifiersForPropType(lrrPropType.ID);
                            IEnumerator iLkUpEnum = lrrLkUpQuals.GetEnumerator();
                            while (iLkUpEnum.MoveNext())
                            {
                                LRRLookUpQualifier lrrLkUpQual = iLkUpEnum.Current as LRRLookUpQualifier;
                                LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);

                                //Qualifier Node (MasterProperties --> BFWFlow-->Air Components --> Mol Fraction)
                                TreeNode childNode2 = new TreeNode(qual.Name);
                                childNode1.Nodes.Add(childNode2);

                                //Get the Group properties
                                LRRLookUpProperties lrrLkUpProps = LRRModel.LookUpProperties.getLookUpPropertiesForPropTypeID(lrrPropType.ID);
                                IEnumerator iLkUpPropEnum = lrrLkUpProps.GetEnumerator();
                                while (iLkUpPropEnum.MoveNext())
                                {
                                    LRRLookUpProperty lrrLkUpProp = iLkUpPropEnum.Current as LRRLookUpProperty;

                                    //Grp Property Node (MasterProperties --> BFWFlow-->Air Components --> Mol Fraction --> Ar)
                                    TreeNode childNode3 = new TreeNode(lrrLkUpProp.Name);
                                    childNode2.Nodes.Add(childNode3);

                                    for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                                    {
                                        LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                                        string aggreGrpPropID = LRRModel.AggregateGroupedPropertiesObj.GetAggregateGroupPropID(aggProp.ID, lrrLkUpProp.LookUpPropertiesID);
                                        LRRAggregateGroupedProperty aggGrpProp = LRRModel.AggregateGroupedPropertiesObj.GetAggregateGroupPropFromID(aggreGrpPropID);
                                        LRRAggregateGroupedPropertyValue aggGrpPropValueObj = LRRModel.AggregateGroupedPropertyValuesObj.GetGroupedValueObjectForPropertyMode(aggreGrpPropID, lrrLkUpProp.LookUpPropertiesID, qual.QualifierID, lrrMode.ModeId);
                                        if (aggGrpPropValueObj != null)
                                        {
                                            BindingInfo bindInfo = new BindingInfo();
                                            bindInfo.SourceObjectName = "LRRAggregateGroupedPropertyValueID";
                                            bindInfo.SourceObjectID = aggGrpPropValueObj.AggregateGroupedPropertyValueID;
                                            bindInfo.SourceProperty = "PropertyValue";
                                            bindInfo.SelectedSiteCompGroups.Add(aggGrpProp);
                                            bindInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(aggGrpPropValueObj.ModeID));
                                            bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(aggGrpPropValueObj.ModeID).ModeDisplayName;
                                            bindInfo.SiteComponentProperty = aggProp.Name + " / " + lrrLkUpProp.Name + " / " + LRRModel.Qualifiers.getQualifierObjectForQualifierID(aggGrpPropValueObj.QualifierID).Name + LRRMessages.GetMessageString("SID_GROUP_MAPPING");
                                            bindInfo.SiteComponentName = aggregate.Name;
                                            TreeNode aggPropValueNode = new TreeNode(lrrMode.ModeName);
                                            aggPropValueNode.Tag = bindInfo;
                                            childNode3.Nodes.Add(aggPropValueNode);
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            //Get the Data Arrays
                            LRRLookUpProperties lrrLkUpProps = LRRModel.LookUpProperties.getLookUpPropertiesForPropTypeID(lrrPropType.ID);
                            IEnumerator iLkUpPropEnum = lrrLkUpProps.GetEnumerator();
                            while (iLkUpPropEnum.MoveNext())
                            {
                                LRRLookUpProperty lrrLkUpProp = iLkUpPropEnum.Current as LRRLookUpProperty;

                                //Grp Property Node (MasterProperties --> BFWFlow-->Air Components --> C00)
                                TreeNode childNode2 = new TreeNode(lrrLkUpProp.Name);
                                childNode1.Nodes.Add(childNode2);

                                for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                                {
                                    LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                                    string aggGrpPropID = LRRModel.AggregateGroupedPropertiesObj.GetAggregateGroupPropID(aggProp.ID, lrrLkUpProp.LookUpPropertiesID);
                                    LRRAggregateGroupedPropertyValue aggGrpValueObj = LRRModel.AggregateGroupedPropertyValuesObj.GetGroupedValueObjectForPropertyMode(aggGrpPropID, lrrLkUpProp.LookUpPropertiesID, null, lrrMode.ModeId);
                                    if (aggGrpValueObj != null)
                                    {
                                        BindingInfo bindInfo = new BindingInfo();
                                        bindInfo.SourceObjectName = "LRRAggregateGroupedPropertyValueID";
                                        bindInfo.SourceObjectID = aggGrpValueObj.AggregateGroupedPropertyValueID;
                                        bindInfo.SourceProperty = "PropertyValue";
                                        bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(aggGrpValueObj.ModeID).ModeDisplayName;
                                        bindInfo.SiteComponentProperty = aggProp.Name + "/" + lrrLkUpProp.Name + LRRMessages.GetMessageString("SID_DATA_ARRAY_MAPPING");
                                        bindInfo.SiteComponentName = aggregate.Name;
                                        TreeNode aggPropValueNode = new TreeNode(lrrMode.ModeName);
                                        aggPropValueNode.Tag = bindInfo;
                                        childNode2.Nodes.Add(aggPropValueNode);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            trvView.Nodes.Add(root);
            trvView.Sort();
        }
     

        private void InitializeTreeView(LRRSiteComponent siteComponent, LRRSiteComponentProperties _siteComponentProperties)
        {
            TreeNode root = new TreeNode(siteComponent.Name);
            foreach (LRRSiteComponentProperty lrrSiteCompProp in _siteComponentProperties)
            {
                //Check if Property is Simple
                if (lrrSiteCompProp.Type == "1")
                {
                    TreeNode siteCompPropNode = new TreeNode(lrrSiteCompProp.Name);
                    root.Nodes.Add(siteCompPropNode);

                    for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                    {
                        LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                        LRRSiteComponentPropertyValue lrrSiteCompPropValue = lrrSiteCompProp.GetValueObjectForPropertyMode(lrrMode.ModeId);
                        if (lrrSiteCompPropValue != null)
                        {
                            BindingInfo bindInfo = new BindingInfo();
                            bindInfo.SourceObjectName = "LRRSiteComponentPropertyValue";
                            bindInfo.SelectedSiteComponentProperties.Add(lrrSiteCompProp);
                            bindInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(lrrSiteCompPropValue.ModeID));
                            bindInfo.SourceObjectID = lrrSiteCompPropValue.ID;
                            bindInfo.SourceProperty = "PropertyValue";
                            bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(lrrSiteCompPropValue.ModeID).ModeDisplayName;
                            bindInfo.SiteComponentProperty = lrrSiteCompProp.Name;
                            bindInfo.SiteComponentName = siteComponent.Name;
                            TreeNode siteCompPropValueNode = new TreeNode(lrrMode.ModeName);
                            siteCompPropValueNode.Tag = bindInfo;
                            siteCompPropNode.Nodes.Add(siteCompPropValueNode);
                        }
                    }
                }
                else
                {
                    if (lrrSiteCompProp.Type != null && lrrSiteCompProp.Type != "1")
                    {
                        LRRPropertyType lrrPropType = _lrrPropertyTypes.getPropertyTypeForID(lrrSiteCompProp.Type);

                        //Property Type Node (SiteComponentProperty --> BFWFlow-->Air Components)
                        TreeNode childNode1 = new TreeNode(lrrSiteCompProp.Name + "-" + lrrPropType.PropertyType);
                        root.Nodes.Add(childNode1);

                        if (!lrrPropType.IsDataArray)
                        {
                            LRRLookUpQualifiers lrrLkUpQuals = LRRModel.LookUpQualifiers.getLookUpQualifiersForPropType(lrrPropType.ID);
                            IEnumerator iLkUpEnum = lrrLkUpQuals.GetEnumerator();
                            while (iLkUpEnum.MoveNext())
                            {
                                LRRLookUpQualifier lrrLkUpQual = iLkUpEnum.Current as LRRLookUpQualifier;
                                LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);

                                //Qualifier Node (MasterProperties --> BFWFlow-->Air Components --> Mol Fraction)
                                TreeNode childNode2 = new TreeNode(qual.Name);
                                childNode1.Nodes.Add(childNode2);

                                //Get the Group properties
                                LRRLookUpProperties lrrLkUpProps = LRRModel.LookUpProperties.getLookUpPropertiesForPropTypeID(lrrPropType.ID);
                                IEnumerator iLkUpPropEnum = lrrLkUpProps.GetEnumerator();
                                while (iLkUpPropEnum.MoveNext())
                                {
                                    LRRLookUpProperty lrrLkUpProp = iLkUpPropEnum.Current as LRRLookUpProperty;

                                    //Grp Property Node (MasterProperties --> BFWFlow-->Air Components --> Mol Fraction --> Ar)
                                    TreeNode childNode3 = new TreeNode(lrrLkUpProp.Name);
                                    childNode2.Nodes.Add(childNode3);

                                    for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                                    {
                                        LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                                        string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lrrSiteCompProp.ID, lrrLkUpProp.LookUpPropertiesID);
                                        LRRSiteComponentGroupProperty lscgp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(siteGrpProp);
                                        LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp, lrrLkUpProp.LookUpPropertiesID, qual.QualifierID, lrrMode.ModeId);
                                        if (qualValueObj != null)
                                        {
                                            BindingInfo bindInfo = new BindingInfo();
                                            bindInfo.SourceObjectName = "LRRSiteGroupPropertyQualifierValueID";
                                            bindInfo.SourceObjectID = qualValueObj.SiteGroupPropertyQualifierValueID;
                                            bindInfo.SourceProperty = "PropertyValue";
                                            bindInfo.SelectedSiteCompGroups.Add(lscgp);
                                            bindInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(qualValueObj.ModeID));
                                            bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(qualValueObj.ModeID).ModeDisplayName;
                                            bindInfo.SiteComponentProperty = lrrSiteCompProp.Name + " / " + lrrLkUpProp.Name + " / " + LRRModel.Qualifiers.getQualifierObjectForQualifierID(qualValueObj.QualifierID).Name + LRRMessages.GetMessageString("SID_GROUP_MAPPING");
                                            bindInfo.SiteComponentName = siteComponent.Name;
                                            TreeNode siteCompPropValueNode = new TreeNode(lrrMode.ModeName);
                                            siteCompPropValueNode.Tag = bindInfo;
                                            childNode3.Nodes.Add(siteCompPropValueNode);
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            //Get the Data Arrays
                            LRRLookUpProperties lrrLkUpProps = LRRModel.LookUpProperties.getLookUpPropertiesForPropTypeID(lrrPropType.ID);
                            IEnumerator iLkUpPropEnum = lrrLkUpProps.GetEnumerator();
                            while (iLkUpPropEnum.MoveNext())
                            {
                                LRRLookUpProperty lrrLkUpProp = iLkUpPropEnum.Current as LRRLookUpProperty;

                                //Grp Property Node (MasterProperties --> BFWFlow-->Air Components --> C00)
                                TreeNode childNode2 = new TreeNode(lrrLkUpProp.Name);
                                childNode1.Nodes.Add(childNode2);

                                for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                                {
                                    LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                                    string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lrrSiteCompProp.ID, lrrLkUpProp.LookUpPropertiesID);
                                    LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp, lrrLkUpProp.LookUpPropertiesID, null, lrrMode.ModeId);
                                    if (qualValueObj != null)
                                    {
                                        BindingInfo bindInfo = new BindingInfo();
                                        bindInfo.SourceObjectName = "LRRSiteGroupPropertyQualifierValueID";
                                        bindInfo.SourceObjectID = qualValueObj.SiteGroupPropertyQualifierValueID;
                                        bindInfo.SourceProperty = "PropertyValue";
                                        bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(qualValueObj.ModeID).ModeDisplayName;
                                        bindInfo.SiteComponentProperty = lrrSiteCompProp.Name + "/" + lrrLkUpProp.Name + LRRMessages.GetMessageString("SID_DATA_ARRAY_MAPPING");
                                        bindInfo.SiteComponentName = siteComponent.Name;
                                        TreeNode siteCompPropValueNode = new TreeNode(lrrMode.ModeName);
                                        siteCompPropValueNode.Tag = bindInfo;
                                        childNode2.Nodes.Add(siteCompPropValueNode);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            trvView.Nodes.Add(root);
            trvView.Sort();
        }
       
        private void InitializeTreeView()
        {
            TreeNode root = new TreeNode(LRRHeaderText.GetHeaderString("SID_MASTERPLANTITEMTYPE"));
            IEnumerator compTypeEnum = _lrrCompTypes.GetEnumerator();

            while (compTypeEnum.MoveNext())
            {
                LRRComponentType lrrCompType = compTypeEnum.Current as LRRComponentType;

                //Create Component Type Node
                TreeNode compTypeNode = new TreeNode(lrrCompType.Name);
                root.Nodes.Add(compTypeNode);

                //Filter Master Components on master Component Type
                LRRMasterComponents lrrMasterComps = _lrrMasterComps.FilterMasterComponentsByComponentTypeID(lrrCompType.ID);
                IEnumerator iMasterCompEnum = lrrMasterComps.GetEnumerator();
                while (iMasterCompEnum.MoveNext())
                {
                    LRRMasterComponent lrrMasterComp = iMasterCompEnum.Current as LRRMasterComponent;

                    //Create Master Component Node
                    TreeNode masterCompNode = new TreeNode(lrrMasterComp.ComponentName);
                    compTypeNode.Nodes.Add(masterCompNode);

                    //Get Site Components
                    LRRSiteComponents lrrSiteComps = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(lrrMasterComp.ID);
                    IEnumerator iSiteCompEnum = lrrSiteComps.GetEnumerator();
                    while (iSiteCompEnum.MoveNext())
                    {
                        LRRSiteComponent lrrSiteComp = iSiteCompEnum.Current as LRRSiteComponent;

                        //Create Site Component Node
                        TreeNode siteCompNode = new TreeNode(lrrSiteComp.Name);
                        masterCompNode.Nodes.Add(siteCompNode);

                        LRRSiteComponentProperties lrrSiteCompProps = lrrSiteComp.Properties;
                        IEnumerator iSitePropEnum = lrrSiteCompProps.GetEnumerator();
                        while (iSitePropEnum.MoveNext())
                        {
                            LRRSiteComponentProperty lrrSiteCompProp = iSitePropEnum.Current as LRRSiteComponentProperty;

                            //Check if Property is Simple
                            if (lrrSiteCompProp.Type == "1")
                            {
                                TreeNode siteCompPropNode = new TreeNode(lrrSiteCompProp.Name);
                                siteCompNode.Nodes.Add(siteCompPropNode);

                                for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                                {
                                    LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                                    LRRSiteComponentPropertyValue lrrSiteCompPropValue = lrrSiteCompProp.GetValueObjectForPropertyMode(lrrMode.ModeId);
                                    if (lrrSiteCompPropValue != null)
                                    {
                                        BindingInfo bindInfo = new BindingInfo();
                                        bindInfo.SourceObjectName = "LRRSiteComponentPropertyValue";
                                        bindInfo.SelectedSiteComponentProperties.Add(lrrSiteCompProp);
                                        bindInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(lrrSiteCompPropValue.ModeID)); 
                                        bindInfo.SourceObjectID = lrrSiteCompPropValue.ID;
                                        bindInfo.SourceProperty = "PropertyValue";
                                        bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(lrrSiteCompPropValue.ModeID).ModeDisplayName;
                                        bindInfo.SiteComponentProperty = lrrSiteCompProp.Name;
                                        bindInfo.SiteComponentName = lrrSiteComp.Name;
                                        TreeNode siteCompPropValueNode = new TreeNode(lrrMode.ModeName);
                                        siteCompPropValueNode.Tag = bindInfo;
                                        siteCompPropNode.Nodes.Add(siteCompPropValueNode);
                                    }
                                }
                            }

                            if (lrrSiteCompProp.Type != null && lrrSiteCompProp.Type != "1")
                            {
                                LRRPropertyType lrrPropType = _lrrPropertyTypes.getPropertyTypeForID(lrrSiteCompProp.Type);

                                //Property Type Node (SiteComponentProperty --> BFWFlow-->Air Components)
                                TreeNode childNode1 = new TreeNode(lrrSiteCompProp.Name + "-" + lrrPropType.PropertyType);
                                siteCompNode.Nodes.Add(childNode1);

                                if (!lrrPropType.IsDataArray)
                                {
                                    LRRLookUpQualifiers lrrLkUpQuals = LRRModel.LookUpQualifiers.getLookUpQualifiersForPropType(lrrPropType.ID);
                                    IEnumerator iLkUpEnum = lrrLkUpQuals.GetEnumerator();
                                    while (iLkUpEnum.MoveNext())
                                    {
                                        LRRLookUpQualifier lrrLkUpQual = iLkUpEnum.Current as LRRLookUpQualifier;
                                        LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);

                                        //Qualifier Node (MasterProperties --> BFWFlow-->Air Components --> Mol Fraction)
                                        TreeNode childNode2 = new TreeNode(qual.Name);
                                        childNode1.Nodes.Add(childNode2);

                                        //Get the Group properties
                                        LRRLookUpProperties lrrLkUpProps = LRRModel.LookUpProperties.getLookUpPropertiesForPropTypeID(lrrPropType.ID);
                                        IEnumerator iLkUpPropEnum = lrrLkUpProps.GetEnumerator();
                                        while (iLkUpPropEnum.MoveNext())
                                        {
                                            LRRLookUpProperty lrrLkUpProp = iLkUpPropEnum.Current as LRRLookUpProperty;

                                            //Grp Property Node (MasterProperties --> BFWFlow-->Air Components --> Mol Fraction --> Ar)
                                            TreeNode childNode3 = new TreeNode(lrrLkUpProp.Name);
                                            childNode2.Nodes.Add(childNode3);

                                            for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                                            {
                                                LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                                                string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lrrSiteCompProp.ID, lrrLkUpProp.LookUpPropertiesID);
                                                LRRSiteComponentGroupProperty lscgp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID( siteGrpProp);
                                                LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp, lrrLkUpProp.LookUpPropertiesID, qual.QualifierID, lrrMode.ModeId);
                                                if (qualValueObj != null)
                                                {
                                                    BindingInfo bindInfo = new BindingInfo();
                                                    bindInfo.SourceObjectName = "LRRSiteGroupPropertyQualifierValueID";
                                                    bindInfo.SourceObjectID = qualValueObj.SiteGroupPropertyQualifierValueID;
                                                    bindInfo.SourceProperty = "PropertyValue";
                                                    bindInfo.SelectedSiteCompGroups.Add(lscgp);
                                                    bindInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(qualValueObj.ModeID));
                                                    bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(qualValueObj.ModeID).ModeDisplayName;
                                                    bindInfo.SiteComponentProperty = lrrSiteCompProp.Name + " / " + lrrLkUpProp.Name + " / " + LRRModel.Qualifiers.getQualifierObjectForQualifierID(qualValueObj.QualifierID).Name + LRRMessages.GetMessageString("SID_GROUP_MAPPING");
                                                    bindInfo.SiteComponentName = lrrSiteComp.Name;
                                                    TreeNode siteCompPropValueNode = new TreeNode(lrrMode.ModeName);
                                                    siteCompPropValueNode.Tag = bindInfo;
                                                    childNode3.Nodes.Add(siteCompPropValueNode);
                                                }
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    //Get the Data Arrays
                                    LRRLookUpProperties lrrLkUpProps = LRRModel.LookUpProperties.getLookUpPropertiesForPropTypeID(lrrPropType.ID);
                                    IEnumerator iLkUpPropEnum = lrrLkUpProps.GetEnumerator();
                                    while (iLkUpPropEnum.MoveNext())
                                    {
                                        LRRLookUpProperty lrrLkUpProp = iLkUpPropEnum.Current as LRRLookUpProperty;

                                        //Grp Property Node (MasterProperties --> BFWFlow-->Air Components --> C00)
                                        TreeNode childNode2 = new TreeNode(lrrLkUpProp.Name);
                                        childNode1.Nodes.Add(childNode2);

                                        for (int i = 1; i < LRRModel.ProjectModes.Count; i++)
                                        {
                                            LRRMode lrrMode = LRRModel.ProjectModes[i] as LRRMode;
                                            string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lrrSiteCompProp.ID, lrrLkUpProp.LookUpPropertiesID);
                                            LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp, lrrLkUpProp.LookUpPropertiesID,null, lrrMode.ModeId);
                                            if (qualValueObj != null)
                                            {
                                                BindingInfo bindInfo = new BindingInfo();
                                                bindInfo.SourceObjectName = "LRRSiteGroupPropertyQualifierValueID";
                                                bindInfo.SourceObjectID = qualValueObj.SiteGroupPropertyQualifierValueID;
                                                bindInfo.SourceProperty = "PropertyValue";
                                                bindInfo.ModeName = LRRModel.ProjectModes.GetModeByID(qualValueObj.ModeID).ModeDisplayName;
                                                bindInfo.SiteComponentProperty = lrrSiteCompProp.Name + "/" + lrrLkUpProp.Name + LRRMessages.GetMessageString("SID_DATA_ARRAY_MAPPING"); 
                                                bindInfo.SiteComponentName = lrrSiteComp.Name;
                                                TreeNode siteCompPropValueNode = new TreeNode(lrrMode.ModeName);
                                                siteCompPropValueNode.Tag = bindInfo;
                                                childNode2.Nodes.Add(siteCompPropValueNode);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            trvView.Nodes.Add(root);

            trvView.Sort();
        }

        /// <summary>
        /// Method Initializes the data source
        /// </summary>
        private void InitDataSource()
        {
            _lrrMasterComps = LRRModel.Model;
            _lrrPropertyTypes = LRRModel.ChildModel.PropertyTypes;

            _lrrCompTypes = new LRRComponentTypes();
            _lrrApp.DBManagerObj.PopulateCollection(_lrrCompTypes);
        }

        private void InitDataSource(LRRAggregate aggregate)
        {
            _aggregateProperties = LRRModel.AggregatePropertiesObj.FilterPropertiesByAggregate(aggregate.AggregateID);

            _lrrMasterComps = LRRModel.Model;
            _lrrPropertyTypes = LRRModel.ChildModel.PropertyTypes;

            _lrrCompTypes = new LRRComponentTypes();
            _lrrApp.DBManagerObj.PopulateCollection(_lrrCompTypes);
   
        }

        private void InitDataSource(LRRSiteComponent siteComponent)
        {
            _siteComponentProperties = LRRModel.SiteComponentPropertiesObj.GetAllSiteComponentPropertiesByComponentID(siteComponent.SiteComponentID);

            _lrrMasterComps = LRRModel.Model;
            _lrrPropertyTypes = LRRModel.ChildModel.PropertyTypes;

            _lrrCompTypes = new LRRComponentTypes();
            _lrrApp.DBManagerObj.PopulateCollection(_lrrCompTypes);
        }

        #endregion

        #region fields
        private LRRApp _lrrApp;
        private LRRMasterComponents _lrrMasterComps = new LRRMasterComponents();
        private LRRPropertyTypes _lrrPropertyTypes;
        private LRRComponentTypes _lrrCompTypes;
        private LRRSiteComponentProperties _siteComponentProperties;
        private LRRAggregateProperties _aggregateProperties;
        private BindingInfo _bindInfo;
        #endregion

        #region Properties
        public BindingInfo BindInfo
        {
            get { return _bindInfo; }
            set
            {
                _bindInfo = value;
            }
        }

        #endregion

        #region Event Handlers
        private void btnOk_Click(object sender, EventArgs e)
        {
            TreeNode trvNode = this.trvView.SelectedNode;
            if (trvNode != null)
            {
                BindingInfo bindInfo = trvNode.Tag as BindingInfo;
                if (bindInfo != null)
                {
                    this.BindInfo = bindInfo;
                }
            }
        }
        #endregion

    }
}