using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Reflection;
using System.Windows.Forms;
using System.IO;
using System.Resources;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters;
using Syncfusion.Windows.Forms.Diagram;
//using Syncfusion.Windows.Forms.Diagram.Controls;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageLogger;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.DashBoardControlsLibrary;
using Symcon.LightRidge.DashBoardControlsLibrary.Controls;

namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
    public partial class DashboardCanvas : Form,ILRRUIContainer
    {
        public DashboardCanvas()
        {
            InitializeComponent();

            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

            _app.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
            _app.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);
            this.diagram1.Model.EventSink.NodeCollectionChanged += new CollectionExEventHandler(diagram1_ChildrenChangeComplete);
            this.diagram1.EventSink.NodeDoubleClick += new NodeMouseEventHandler(diagram1_NodeDoubleClick);

            _attrDetDialog = new AttributeDetailsDialog();
        }
        
        #region Event Handlers


        void barItem1_Click(object sender, System.EventArgs e)
        {
            OpenDashBoardForm openDashBoardForm = new OpenDashBoardForm();
            if (openDashBoardForm.ShowDialog() == DialogResult.OK)
            {
                if (openDashBoardForm.SelectedDashBoardView != null)
                {
                    _lrrDashBoardView = openDashBoardForm.SelectedDashBoardView;
                    InitDataSource();
                    OpenDashBoard();
                }
            }
        }

        private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
        {
            this.diagram1.UpdateView();
        }

        private void diagram1_ChildrenChangeComplete(CollectionExEventArgs evtArgs)
        {
            //Check	if the node	type is	a symbol	and	if true
			//associate	properties to it.
            if(evtArgs.Element != null)
            {
                Node node = evtArgs.Element as Node;            
                switch (evtArgs.ChangeType)
                {
                    case (CollectionExChangeType.Insert):
                        ControlNode ctrlNode = node as ControlNode;
                        ctrlNode.HostingControl.GetType().GetEvent("AddBindingCalled").AddEventHandler(ctrlNode.HostingControl, new EventHandler(AddBindingCalled));
                        ctrlNode.HostingControl.GetType().GetEvent("AttributeDetailsCalled", BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public).AddEventHandler(ctrlNode.HostingControl, System.Delegate.CreateDelegate(typeof(Symcon.LightRidge.DashBoardControlsLibrary.AttributeDetailsDel), this, "AttributeDetailsCalled"));
                        ctrlNode.HostingControl.GetType().GetEvent("DoubleClick").AddEventHandler(ctrlNode.HostingControl, new EventHandler(DoubleClickHandler));
                        break;
                }
            }
        }

        private void AddBindingCalled(object sender, EventArgs e)
        {
            Control objControl = sender as Control;
            BindTree modelTree = new BindTree();
            DialogResult dlgResult = modelTree.ShowDialog();
            if (dlgResult == DialogResult.OK)
            {
                BindingInfo bindInfo = modelTree.BindInfo;
                if (bindInfo != null)
                {
                    string sourceObjectName = bindInfo.SourceObjectName;
                    string sourceObjectID = bindInfo.SourceObjectID;
                    string sourceProperty = bindInfo.SourceProperty;

                    object sourceObject = GetSourceObject(sourceObjectName, sourceObjectID);

                    try
                    {
                        ClearBindings(sourceObject, objControl,sourceProperty);
                        _lrrBindingManager.BindingCalled +=new LRRBindingManager.BindingCallDel(_lrrBindingManager_BindingCalled);
                        _lrrBindingManager.AddBinding(sourceObject, sourceProperty, objControl, "TargetValue",true);
                        _lrrBindingManager.AddBinding(bindInfo, "SiteComponentName", objControl, "SourceItem");
                        _lrrBindingManager.AddBinding(bindInfo, "SiteComponentProperty", objControl, "Attribute");
                        _lrrBindingManager.AddBinding(bindInfo, "ModeName", objControl, "ModeName");
                        _lrrBindingManager.AddBinding(bindInfo, "SourceObjectName", objControl, "SourceObjectName");
                        _lrrBindingManager.AddBinding(bindInfo, "SourceObjectID", objControl, "SourceObjectID");
                        _lrrBindingManager.AddBinding(bindInfo, "SourceProperty", objControl, "SourceProperty");

                        ChangePropertyValue(objControl);
                    }
                    catch { }
                }
            }
        }

        private void _lrrBindingManager_BindingCalled(object source, object target)
        {
            try
            {
                Control objControl = target as Control;
                ChangePropertyValue(objControl);
            }
            catch{}
        }

        private void ChangePropertyValue(Control objControl)
        {
            try
            {
                object sourceObjectName = objControl.GetType().GetProperty("SourceObjectName").GetValue(objControl,null);
                object sourceObjectID = objControl.GetType().GetProperty("SourceObjectID").GetValue(objControl,null);
                object sourceObject = GetSourceObject((string)sourceObjectName, (string)sourceObjectID);
                if(sourceObject != null)
                {
				    LRRSiteComponentPropertyValue iPropValue = sourceObject as LRRSiteComponentPropertyValue;
				    string uomID = iPropValue.GetUOMID();
				    //Convert property value according to UIUOM 
				    string propertyValue = iPropValue.GetPropertyValue();
                    LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(iPropValue.ID);
                    string uomTypeID = siteCompProp.UOMTypeID;
					//Get the UI UOM
					LRRMeasurementSystemDefinitions lrrMSDs = (LRRMeasurementSystemDefinitions)this._app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRMeasurementSystemDefinitions));
					string uiUOMID = lrrMSDs.GetLocaleUOM(this._app.AppLocaleID,uomTypeID);

				    if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
				    {
					    propertyValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,propertyValue);
				    }

                    string lrrDataType = iPropValue.GetDataType();
                    if (lrrDataType == null || lrrDataType.Trim() == "")
                    {
                        propertyValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(propertyValue, this._app);
                    }
                    objControl.GetType().GetProperty("TargetValue").SetValue(objControl,propertyValue, null);
                }
            }
            catch{}
        }

        private void AttributeDetailsCalled(object sender, Symcon.LightRidge.DashBoardControlsLibrary.AttributeDetailsEventArgs e)
        {
            _attrDetDialog.UpdatePropertyDetailsPanel(e);
            _attrDetDialog.Owner = this;
            _attrDetDialog.Show();
            _attrDetDialog.WindowState = FormWindowState.Normal;
        }
        
        private void DoubleClickHandler(object sender, EventArgs e)
        {
            Group symbol = this.diagram1.View.SelectionList[0] as Group;
            ControlNode ctrlNode = symbol.Nodes.First as ControlNode;
            DashboardLink dhLink = (ctrlNode.HostingControl.GetType().GetProperty("Link").GetValue(ctrlNode.HostingControl, null)) as DashboardLink;
			if(dhLink != null )
			{
                int linkTypeID = dhLink.LinkTypeID;
				string viewID =	dhLink.ID;
                Symcon.LightRidge.PEAdvisor.ScreenManagement.ScreenListView listView = new Symcon.LightRidge.PEAdvisor.ScreenManagement.ScreenListView();
                listView.OpenViews(linkTypeID,viewID);
			}
        }

        void diagram1_NodeDoubleClick(NodeMouseEventArgs evtArgs)
        {
            if(evtArgs.Node.GetType().BaseType == typeof(Group))
			{
                Group symbol = evtArgs.Node as Group;
                ControlNode ctrlNode = symbol.Nodes.First as ControlNode;
                DashboardLink dhLink = (ctrlNode.HostingControl.GetType().GetProperty("Link").GetValue(ctrlNode.HostingControl, null)) as DashboardLink;
				if(dhLink != null )
				{
                    int linkTypeID = dhLink.LinkTypeID;
					string viewID =	dhLink.ID;
                    Symcon.LightRidge.PEAdvisor.ScreenManagement.ScreenListView listView = new Symcon.LightRidge.PEAdvisor.ScreenManagement.ScreenListView();
                    listView.OpenViews(linkTypeID,viewID);
				}
			}
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (keyData == Keys.Delete)
            {
                if (this.diagram1.View.SelectionList.Count > 0)
                {
                    DialogResult dlgResult = LRRMessagBox.ShowMessage("SID_DELETE_CONTROL", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dlgResult == DialogResult.Yes)
                    {
                        Group symbol = this.diagram1.View.SelectionList[0] as Group;
                        ControlNode ctrlNode = symbol.Nodes.First as ControlNode;
                        this.diagram1.Controls.Remove(ctrlNode.HostingControl);
                        this.diagram1.Controller.Delete();
                        this.diagram1.Refresh();
                    }
                }
            }
            return base.ProcessCmdKey(ref msg, keyData);
        }

        #endregion

        #region Methods
        /// <summary>
        /// Method gets the source object
        /// </summary>
        /// <param Name1="sourceObjectName"></param>
        /// <param Name1="sourceObjectID"></param>
        /// <returns></returns>
        private object GetSourceObject(string sourceObjectName, string sourceObjectID)
        {
            object retValue = null;
            switch (sourceObjectName)
            {
                case "LRRSiteComponentPropertyValue":
                    retValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(sourceObjectID);
                    break;
                case "LRRSiteGroupPropertyQualifierValueID":
                    retValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(sourceObjectID);
                    break;
                default:
                    break;
            }
            return retValue;
        }
       
        /// <summary>
        /// Method Initializes the Data Source
        /// </summary>
        private void InitDataSource()
        {
            _lrrDashBoardViewData = new LRRDashBoardViewData();
            _app.DBManagerObj.PopulateObject(_lrrDashBoardViewData, typeof(LRRDashBoardViewData), " where ID = '" + _lrrDashBoardView.ID + "'");
        }

        /// <summary>
        /// Method opens the dashboard
        /// </summary>
        private void OpenDashBoard()
        {
            LRRDashBoardViewData lrrDashBoardViewData = _lrrDashBoardViewData;
            byte[] byteArray = lrrDashBoardViewData.ViewBlob;
            if (byteArray == null)
                return;
            MemoryStream ms = null;
            try
            {
                ms = new MemoryStream(byteArray);
                if (ms != null)
                {
                    this.diagram1.LoadBinary(ms);
                    ms.Close();
                    byteArray = null;
                }
            }
            catch (Exception ex)
            {
                if (ms != null)
                {
                    ms.Close();
                }
            }
        }

        /// <summary>
        /// Method saves the dashboard
        /// </summary>
        /// <param name="name"></param>
        /// <param name="desc"></param>
        private void SaveFile(string name,string desc)
        {
            MemoryStream ms = null;
            BinaryReader br = null;
            try
            {
                ms = new MemoryStream();
                BinaryFormatter formatter = new BinaryFormatter();
                formatter.Binder = Syncfusion.Runtime.Serialization.AppStateSerializer.CustomBinder;
                formatter.AssemblyFormat = FormatterAssemblyStyle.Simple;
                //changes
                DiagramDocument doc = new DiagramDocument(this.diagram1.Model, this.diagram1.View);
                formatter.Serialize(ms, doc);
                ms.Position = 0;
                br = new BinaryReader(ms);
                byte[] fileData = br.ReadBytes((int)ms.Length);
                LRRDashBoardView lrrsv = new LRRDashBoardView();
                lrrsv.SavedViewName = name;
                lrrsv.SavedViewDescription = desc;
                LRRDashBoardViewData lrsvd = new LRRDashBoardViewData();
                lrsvd.ID = lrrsv.ID;
                lrsvd.ViewBlob = fileData;
                _app.DBManagerObj.CommitObject(lrrsv);
                _app.DBManagerObj.CommitObject(lrsvd);
                this.diagram1.Model.Modified = false;
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(ImplementerScreen), "DashBoard Files :" + ex.Message, Logger.Level.Debug);
            }
            finally
            {
                if (br != null)
                    br.Close();
                if (ms != null)
                    ms.Close();
                ms = null;
                br = null;
            }
        }

        /// <summary>
        /// Method clears all the bindings of the control
        /// </summary>
        /// <param name="source"></param>
        /// <param name="target"></param>
        /// <param name="sourceProperty"></param>
        private void ClearBindings(object source, object target, string sourceProperty)
        {
            _lrrBindingManager.RemoveBinding(source, sourceProperty, target);
            _lrrBindingManager.RemoveBinding(source, "SiteComponentName", target);
            _lrrBindingManager.RemoveBinding(source, "SiteComponentProperty", target);
            _lrrBindingManager.RemoveBinding(source, "ModeName", target);
            _lrrBindingManager.RemoveBinding(source, "SourceObjectName", target);
            _lrrBindingManager.RemoveBinding(source, "SourceObjectID", target);
            _lrrBindingManager.RemoveBinding(source, "SourceProperty", target);
        }
        #endregion

        #region Fields
        private TabControl _parentTabControl;
        private TabPage _parentTabPage;
        private static LRRBindingManager _lrrBindingManager = new LRRBindingManager();
        private LRRApp _app;
        private AttributeDetailsDialog _attrDetDialog;
        private LRRDashBoardViewData _lrrDashBoardViewData;
        private LRRDashBoardView _lrrDashBoardView;
        #endregion

        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Copy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyMasterProperties()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Delete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DownLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Export()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void InsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenQuery()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public TabControl ParentTabControl
        {
            set
            {
                _parentTabControl = value;
            }
            get { return _parentTabControl; }
        }

        public TabPage ParentTabPage
        {
            set
            {
                _parentTabPage = value;
            }
            get
            {
                return _parentTabPage;
            }
        }

        public void Paste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool PerformDirtyChk
        {
            get { return false; }
        }

        public void Print()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PrintPreview()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public new void Refresh()
        {
            this.diagram1.UpdateView();
        }

        public void Save()
        {
            if (_lrrDashBoardView == null)
            {
                SaveDataSetForm saveForm = new SaveDataSetForm();
                DialogResult dlgResult = saveForm.ShowDialog();
                if (dlgResult == DialogResult.OK)
                {
                    SaveFile(saveForm.Name, saveForm.Desc);
                    LRRMessagBox.ShowMessage("SID_SAVEDINFO", true);
                }
            }
            else
            {
                MemoryStream ms = null;
                BinaryReader br = null;
                try
                {
                    ms = new MemoryStream();
                    BinaryFormatter formatter = new BinaryFormatter();
                    formatter.Binder = Syncfusion.Runtime.Serialization.AppStateSerializer.CustomBinder;
                    formatter.AssemblyFormat = FormatterAssemblyStyle.Simple;
                    DiagramDocument doc = new DiagramDocument(this.diagram1.Model, this.diagram1.View);
                    formatter.Serialize(ms, doc);
                    ms.Position = 0;
                    br = new BinaryReader(ms);
                    byte[] fileData = br.ReadBytes((int)ms.Length);

                    _lrrDashBoardViewData.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                    _app.DBManagerObj.CommitObject(_lrrDashBoardViewData);

                    _lrrDashBoardViewData.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
                    _lrrDashBoardViewData.ID = _lrrDashBoardView.ID;
                    _lrrDashBoardViewData.ViewBlob = fileData;
                    _app.DBManagerObj.CommitObject(_lrrDashBoardViewData);
                    
                    LRRMessagBox.ShowMessage("SID_SAVEDINFO", true);

                }
                catch (Exception ex)
                {
                    Logger.LogMessage(typeof(ImplementerScreen), "DashBoard Files :" + ex.Message, Logger.Level.Debug);
                }
                finally
                {
                    if (br != null)
                        br.Close();
                    if (ms != null)
                        ms.Close();
                    ms = null;
                    br = null;
                }
            }
        }

        public void SelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowComponentList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowPropertyList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsArrangeColumns()
        {
            return false;
        }

        public bool SupportsClearFilter()
        {
            return false;
        }

        public bool SupportsClearSort()
        {
            return false;
        }

        public bool SupportsCompositeView()
        {
            return false;
        }

        public bool SupportsCopy()
        {
            return false;
        }

        public bool SupportsCopyUOMTypes()
        {
            return false;
        }

        public bool SupportsDelete()
        {
            return false;
        }

        public bool SupportsExport()
        {
            return false;
        }

        public bool SupportsImportToExcel()
        {
            return false;
        }

        public bool SupportsInsertNewRows()
        {
            return false;
        }

        public bool SupportsLevel()
        {
            return false;
        }

        public bool SupportsMasterPropertiesCopy()
        {
            return false;
        }

        public bool SupportsPageSetup()
        {
            return false;
        }

        public bool SupportsPaste()
        {
            return false;
        }

        public bool SupportsPrint()
        {
            return false;
        }

        public bool SupportsRefresh()
        {
            return true;
        }

        public bool SupportsResultsFilter()
        {
            return false;
        }

        public bool SupportsSave()
        {
            return true;
        }

        public bool SupportsSelectionPaste()
        {
            return false;
        }

        public bool SupportsUndo()
        {
            return false;
        }

        public void TransposeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Undo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

    }
}