using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Symcon.LightRidge.PEAdvisor;
using Symcon.BusinessObjects;
using Symcon.LightRidge.DashBoardControlsLibrary;
using Symcon.LightRidge.DashBoardControlsLibrary.Controls;

namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
    public partial class DashBoardForm : Form,ILRRUIContainer
    {
        public DashBoardForm()
        {
            InitializeComponent();
        }

        public DashBoardForm(XmlDashBoard dashBoardXML)
        {
            InitializeComponent();

            _dashBoardXML = dashBoardXML;
            _ctrlPropertyForm = new ControlPropertyForm();
            _ctrlPropertyForm.ControlPropertiesChanged += new ControlPropertyForm.ControlPropertiesDel(_ctrlPropertyForm_ControlPropertiesChanged);

            InternationalizationUtils.GetControlText(this, null);
            CreateDashBoard(dashBoardXML);
        }

        /// <summary>
        /// Method creates the Dash Board
        /// </summary>
        /// <param name="dashBoardXML"></param>
        public void CreateDashBoard(XmlDashBoard dashBoardXML)
        {
            _tableLayoutPanel = DashBoardCreatorFactory.CreateDashBoard(dashBoardXML);
            this.Controls.Add(_tableLayoutPanel);
            _tableLayoutPanel.Dock = DockStyle.Fill;
            //TableLookAndFeel(tableLayoutPanel, dashBoardXML.Rows, dashBoardXML.Cols);

            //Context Menu for showing property grid
            ContextMenu contMenu = new ContextMenu();
            _tableLayoutPanel.ContextMenu = contMenu;
            contMenu.Popup += new EventHandler(contMenu_Popup);
        }

        private void contMenu_Popup(object sender, EventArgs e)
        {
            Control selectedCtrl = DashBoardCreatorFactory.SelectedCtrl;
            if (selectedCtrl != null)
            {
                ContextMenu contMenu = sender as ContextMenu;
                contMenu.MenuItems.Clear();
                MenuItem miItem = new MenuItem("Show Properties",new EventHandler(ShowProperties_Click));
                contMenu.MenuItems.Add(miItem);
            }
        }

        private void ShowProperties_Click(object sender, EventArgs e)
        {
            Control selectedCtrl = DashBoardCreatorFactory.SelectedCtrl;
            if (selectedCtrl != null)
            {
                int row = _tableLayoutPanel.GetRow(selectedCtrl);
                int col = _tableLayoutPanel.GetColumn(selectedCtrl);
                _ctrlPropertyForm.CellControl = _dashBoardXML.Cells[row + col];

                _ctrlPropertyForm.IDashBoardCtrl = selectedCtrl as IDashBoardControl;
                _ctrlPropertyForm.BindControl();
                _ctrlPropertyForm.ShowDialog();
            }
        }


        private void _ctrlPropertyForm_ControlPropertiesChanged(object sender, ControlPropertiesEventArgs e)
        {
            Control selectedCtrl = DashBoardCreatorFactory.SelectedCtrl;

            ControlProperties ctrlProps = e.CtrlProperties;
            float height = ctrlProps.Height;
            float width = ctrlProps.Width;

            if (height != 0.0f && width != 0.0f)
            {
                int row = _tableLayoutPanel.GetRow(selectedCtrl);
                int col = _tableLayoutPanel.GetColumn(selectedCtrl);

                RowStyle rowStyle = _tableLayoutPanel.RowStyles[row];
                rowStyle.Height = height;

                ColumnStyle colStyle = _tableLayoutPanel.ColumnStyles[col];
                colStyle.Width = width;
            }

            DashBoardCreatorFactory.SelectedCtrl = null;
        }

        private void TableLookAndFeel( TableLayoutPanel tableLayoutPanel,int iRows, int iCols)
        {
            TableLayoutRowStyleCollection rowStyles = tableLayoutPanel.RowStyles;
            int rowHeight = 100 / iRows;
            foreach (RowStyle style in rowStyles)
            {
                style.SizeType = SizeType.AutoSize;
                style.Height = rowHeight;
            }

            TableLayoutColumnStyleCollection colStyles = tableLayoutPanel.ColumnStyles;
            int colHeight = 100 / iCols;
            foreach (ColumnStyle style in colStyles)
            {
                style.SizeType = SizeType.AutoSize;
                style.Width = colHeight;
            }
        }

        #region Fields
        private ControlPropertyForm _ctrlPropertyForm;
        private TableLayoutPanel _tableLayoutPanel;
        private XmlDashBoard _dashBoardXML;
        #endregion

        #region ILRRUIContainer Members

        private TabControl _parentTabControl;
        private TabPage _parentTabPage;

        public void ArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Copy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyMasterProperties()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Delete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DownLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Export()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void InsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenQuery()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public TabControl ParentTabControl
        {
            get
            {
                return _parentTabControl;
            }
            set
            {
                _parentTabControl = value;
            }
        }

        public TabPage ParentTabPage
        {
            get
            {
                return _parentTabPage;
            }
            set
            {
                _parentTabPage = value;
            }
        }

        public void Paste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool PerformDirtyChk
        {
            get { return false; }
        }

        public void Print()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PrintPreview()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Save()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void SelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowComponentList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowPropertyList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsArrangeColumns()
        {
            return false;
        }

        public bool SupportsClearFilter()
        {
            return false;
        }

        public bool SupportsClearSort()
        {
            return false;
        }

        public bool SupportsCompositeView()
        {
            return false;
        }

        public bool SupportsCopy()
        {
            return false;
        }

        public bool SupportsCopyUOMTypes()
        {
            return false;
        }

        public bool SupportsDelete()
        {
            return false;
        }

        public bool SupportsExport()
        {
            return false;
        }

        public bool SupportsImportToExcel()
        {
            return false;
        }

        public bool SupportsInsertNewRows()
        {
            return false;
        }

        public bool SupportsLevel()
        {
            return false;
        }

        public bool SupportsMasterPropertiesCopy()
        {
            return false;
        }

        public bool SupportsPageSetup()
        {
            return false;
        }

        public bool SupportsPaste()
        {
            return false;
        }

        public bool SupportsPrint()
        {
            return false;
        }

        public bool SupportsRefresh()
        {
            return false;
        }

        public bool SupportsResultsFilter()
        {
            return false;
        }

        public bool SupportsSave()
        {
            return false;
        }

        public bool SupportsSelectionPaste()
        {
            return false;
        }

        public bool SupportsUndo()
        {
            return false;
        }

        public void TransposeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Undo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}