using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Symcon.LightRidge.PEAdvisor;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.HeaderDispatcher;
using Symcon.Globals.Enums;

namespace Symcon.LightRidge.PEAdvisor.DashBoard
{
    public partial class OpenDashBoardForm : Form
    {
        public OpenDashBoardForm()
        {
            InitializeComponent();
            InternationalizationUtils.GetControlText(this, null); 
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

            InitDataSource();
            BindGrid();
        }

        #region Private Methods
        private void InitDataSource()
        {
            if (_lrrDashBoardViews != null)
                _lrrDashBoardViews.Clear();

            _lrrDashBoardViews = new LRRDashBoardViews();
            _lrrApp.DBManagerObj.PopulateCollection(_lrrDashBoardViews);
        }

        private void BindGrid()
        {
            //Suspend the Layout of the Grid...
            this.gridDashBoards.SuspendLayout();

            SyncGridUtilities.InitializeGridLookAndFeel(gridDashBoards);

            //Populate the columns for the grid object
            SyncGridUtilities.InitializeGridColumns(gridDashBoards, _lrrDashBoardViews.ItemType, null, true, null);

            this.gridDashBoards.Appearance.AnyHeaderCell.Font.Bold = true;
            this.gridDashBoards.Appearance.AnyHeaderCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.InactiveCaptionText);
            this.gridDashBoards.BackColor = System.Drawing.SystemColors.Window;
            this.gridDashBoards.TableDescriptor.Appearance.AlternateRecordFieldCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.InactiveCaptionText);
            this.gridDashBoards.TableDescriptor.Appearance.AnyHeaderCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.ControlLight);
            this.gridDashBoards.TableDescriptor.Appearance.GroupCaptionCell.Font.Bold = true;
            this.gridDashBoards.TableDescriptor.Appearance.GroupCaptionCell.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.PatternStyle.None, System.Drawing.SystemColors.WindowText, System.Drawing.Color.RoyalBlue);
            this.gridDashBoards.TableDescriptor.TopLevelGroupOptions.CaptionText = LRRHeaderText.GetHeaderString("SID_DASHBOARD") + ":" + " {RecordCount} " + LRRHeaderText.GetHeaderString("SID_ITEMS");
            this.gridDashBoards.TableDescriptor.Appearance.GroupCaptionCell.TextColor = System.Drawing.Color.White;

            this.gridDashBoards.TableDescriptor.AllowNew = false;
            this.gridDashBoards.TableDescriptor.AllowEdit = false;
            this.gridDashBoards.TableDescriptor.AllowRemove = true;

            this.gridDashBoards.SourceListSet.Add("Parent", _lrrDashBoardViews);

            this.gridDashBoards.TableDescriptor.Relations.Clear();
            this.gridDashBoards.DataSource = null;
            this.gridDashBoards.DataSource = _lrrDashBoardViews;

            this.gridDashBoards.ResumeLayout(true);

            this.gridDashBoards.TableDescriptor.RecordDeleting += new Syncfusion.Grouping.RecordEventHandler(TableDescriptor_RecordDeleting);
        }

        #endregion

        #region Fields/Properties
        private LRRDashBoardViews _lrrDashBoardViews;
        private LRRApp _lrrApp;
        private LRRDashBoardView _selectedDashBoardView = null;
        public LRRDashBoardView SelectedDashBoardView
        {
            get { return _selectedDashBoardView; }
            set { _selectedDashBoardView = value; }
        }

        #endregion

        #region EventHandlers
        private void gridDashBoards_TableControlCellClick(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs e)
        {
            CurrencyManager cm = (CurrencyManager)this.gridDashBoards.BindingContext[this.gridDashBoards.DataSource];
            this.SelectedDashBoardView = cm.Current as LRRDashBoardView;
        }

        private void TableDescriptor_RecordDeleting(object sender, Syncfusion.Grouping.RecordEventArgs e)
        {
            CurrencyManager cm = (CurrencyManager)this.gridDashBoards.BindingContext[this.gridDashBoards.DataSource];
            LRRDashBoardView lrrDashBoardView = cm.Current as LRRDashBoardView;
            if (lrrDashBoardView != null)
            {
                if (_isDeletePress)
                {
                    DialogResult dlgResult = LRRMessagBox.ShowMessage("SID_DELETE_RECORD", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dlgResult == DialogResult.Yes)
                    {
                        _isRecordeDel = true;
                    }
                    else
                    {
                        _isRecordeDel = false;
                    }
                    _isDeletePress = false;
                }
                if (_isRecordeDel)
                {
                    lrrDashBoardView.ObjectState = DBOperation.DELETE;
                    _lrrApp.DBManagerObj.CommitObject(lrrDashBoardView);
                    this.SelectedDashBoardView = null;
                }
                else
                {
                    e.Cancel = true;
                    return;
                }
            }
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (keyData == Keys.Delete)
            {
                _isDeletePress = true;
            }
            return base.ProcessCmdKey(ref msg, keyData);
        }

        private bool _isDeletePress = false;
        private bool _isRecordeDel = false;
        #endregion
    }
}