using System;
using System.IO;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Symcon.Utils.MessageDispatcher;   

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for DataBaseConnectionInfosForm.
	/// </summary>
	public class DataBaseConnectionInfosUserControl: System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.Panel pnlFull;
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridInfo;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public DataBaseConnectionInfosUserControl()
		{
			InitializeComponent();
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		}

		public DataBaseConnectionInfosUserControl(CustomCollection custCollection)
		{
			InitializeComponent();
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			CustCollection = custCollection;
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.pnlFull = new System.Windows.Forms.Panel();
			this.gridInfo = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			this.pnlFull.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.gridInfo)).BeginInit();
			this.SuspendLayout();
			// 
			// pnlFull
			// 
			this.pnlFull.Controls.Add(this.gridInfo);
			this.pnlFull.Dock = System.Windows.Forms.DockStyle.Fill;
			this.pnlFull.Location = new System.Drawing.Point(0, 0);
			this.pnlFull.Name = "pnlFull";
			this.pnlFull.Size = new System.Drawing.Size(592, 392);
			this.pnlFull.TabIndex = 0;
			// 
			// gridInfo
			// 
			this.gridInfo.BackColor = System.Drawing.SystemColors.Window;
			this.gridInfo.Dock = System.Windows.Forms.DockStyle.Fill;
			this.gridInfo.Location = new System.Drawing.Point(0, 0);
			this.gridInfo.Name = "gridInfo";
			this.gridInfo.Size = new System.Drawing.Size(592, 392);
			this.gridInfo.TabIndex = 0;
			this.gridInfo.TopLevelGroupOptions.IsExpandedInitialValue = true;
			this.gridInfo.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;
			this.gridInfo.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
			this.gridInfo.TopLevelGroupOptions.ShowCaptionPlusMinus = false;
			this.gridInfo.TopLevelGroupOptions.ShowColumnHeaders = true;
			this.gridInfo.VersionInfo = "3.301.0.0";
			this.gridInfo.TableControlCellButtonClicked += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellButtonClickedEventHandler(this.gridInfo_TableControlCellButtonClicked);
			this.gridInfo.TableControlCheckBoxClick += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventHandler(this.gridInfo_TableControlCheckBoxClick);
			// 
			// DataBaseConnectionInfosUserControl
			// 
			this.Controls.Add(this.pnlFull);
			this.Name = "DataBaseConnectionInfosUserControl";
			this.Size = new System.Drawing.Size(592, 392);
			this.pnlFull.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.gridInfo)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		#region Methods
		public void BindGrid()
		{
			SyncGridUtilities.InitializeGridLookAndFeel(gridInfo);
			CustomCollection custCollection = CustCollection as CustomCollection;
			SyncGridUtilities.InitializeGridColumns(gridInfo,custCollection.ItemType,null,true,null);

			gridInfo.TableModel.Model.CellModels.Add("EllipsisText", new  EllipsisCellModel(gridInfo.TableModel.Model));
			this.gridInfo.TableDescriptor.Columns[1].Appearance.AddNewRecordFieldCell.CellType ="TextBox";
			this.gridInfo.TableDescriptor.Columns[1].Appearance.AnyRecordFieldCell.CellType ="EllipsisText";
			this.gridInfo.TableDescriptor.Columns[2].Appearance.AnyRecordFieldCell.CellType ="ComboBox";
			this.gridInfo.TableDescriptor.Columns[2].Appearance.AnyRecordFieldCell.DataSource = _lrrApp.GetOptions().DBTypes;
			this.gridInfo.TableDescriptor.Columns[1].Appearance.AnyRecordFieldCell.ShowButtons=GridShowButtons.ShowCurrentCell;
			this.gridInfo.TableDescriptor.Columns[5].Appearance.AnyRecordFieldCell.CellType ="CheckBox";
			this.gridInfo.TableDescriptor.Columns[5].Appearance.AnyRecordFieldCell.HorizontalAlignment=GridHorizontalAlignment.Center;

			gridInfo.DataSource = custCollection;
	
		}

		public void Save()
		{
			CustomCollection custCollection = CustCollection as CustomCollection;
			if(custCollection.GetChanges().Count > 0)
			{
				_lrrApp.DBManagerObj.CommitCollection(custCollection.GetChanges());
				LRRMessagBox.ShowMessage("SID_SAVEDINFO",true); 
			}
		}
		#endregion

		#region Properties
		public object CustCollection
		{
			get{ return _custCollection;}
			set
			{
				_custCollection = value;
			}
		}
		#endregion

		#region Fields
		private object _custCollection;
		private LRRApp _lrrApp;
		#endregion

		private void gridInfo_TableControlCellButtonClicked(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellButtonClickedEventArgs e)
		{
			if(e.TableControl.Table.TableModel.Model[e.Inner.RowIndex,e.Inner.ColIndex].CellType=="EllipsisText")
			{
				OpenFileDialog flDialog = new OpenFileDialog();
                flDialog.Filter = "*.mdb |";
				if(flDialog.ShowDialog() == DialogResult.OK)
				{
					string fName=Path.GetFullPath(flDialog.FileName);
					GridTableCellStyleInfo gtsif = gridInfo.TableControl.Model[e.Inner.RowIndex,e.Inner.ColIndex];
					gtsif.ApplyText(fName);
				}
			}
		}

		private void gridInfo_TableControlCheckBoxClick(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs e)
		{
//			object current = gridInfo.CurrencyManager.Current;
//			CustomCollection custCollection = CustCollection as CustomCollection;
//			for(int i=0;i < custCollection.Count;i++)
//			{
//				object bo = custCollection[i];
//				if(bo.Equals(current))
//					continue;
//				if(bo.GetType().GetProperty("IsActive") != null)
//				{
//					bo.GetType().GetProperty("IsActive").SetValue(bo,false,null);
//				}
//			}
//			for(int i = 4; i < gridInfo.Table.TableModel.RowCount;i++)
//			{
//				if(i == e.Inner.RowIndex)
//					continue;
//
//				GridTableCellStyleInfo gtsif = gridInfo.TableControl.Model[i,e.Inner.ColIndex];
//				gtsif.CellValue = false;
//			}
//			gridInfo.Refresh();
		}
	}
}
