using System;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.ExcelWrapper;
using Symcon.Utils.MessageLogger;


namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for LRRExecutionEngine.
	/// </summary>
	public class LRRExecutionEngine
	{
		public LRRExecutionEngine(SymExcelWrapper excelWrapper)
		{
			_symExcelWrapper = excelWrapper;
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		}

		public bool OpenAllWorkBooks()
		{
			try
			{
				foreach(LRRExecutionEngineModel model in LRREEModel.EEModel)
				{
					_symExcelWrapper.OpenWorkBook(model.WorkBookName); 
				}
				return true;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(ExecutionEngine),ex.Message  ,Logger.Level.Debug);
				return false;
			}
			
		}

		/*public bool SaveAllWorkBooks()
		{
			try
			{
				foreach(LRRExecutionEngineModel model in LRREEModel.EEModel)
				{
					_symExcelWrapper.SaveWorkBook(model.WorkBookName); 
				}
				return true;
			}
			catch
			{
				return false;
			}
		}*/

		/// <summary>
		/// This method is used to write the information
		/// to the excel execution engine...
		/// takes the INPUT's for the cycle
		/// </summary>
		/// <returns></returns>
		public bool ExecuteWriteCycle()
		{
			try
			{
				LRRExecutionEngineModels models = LRREEModel.EEModel;
				LRRMasterComponents comps = LRRModel.Model;
				foreach(LRRExecutionEngineModel model in models)
				{
					foreach(LRRExecutionEngineModelEquipment modelEquip in model.ModelEquipments)
					{
						foreach(LRRExecutionEngineModelEquipmentSection modelEquipSec in modelEquip.ModelEquipmentSectionObj)
						{
							if(modelEquipSec.SectionType == "OUTPUT")
							{
								continue;
							}
							LRRMasterComponent masterComp =  comps.GetMasterComponentByID(modelEquip.MasterCompID);
							if(masterComp == null)
							{
								//Added by Nidhig on Johan's Request for incorrect Ranges
								string msg = "WorkBook Name: " + model.WorkBookName  + "Range Name: " + modelEquipSec.RangeName  + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
								Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
								continue;
							}
							
							int i=0;
							int totalCountItems = masterComp.SiteComponentsObj.Count;
							masterComp.SiteComponentsObj.SortColumn = "InternalSortIndex";
							masterComp.SiteComponentsObj.SortingOrder = Symcon.BusinessObjects.GenericComparer.SortOrder.Ascending;
							masterComp.SiteComponentsObj.Sort();

							for(int counter=0;counter< totalCountItems;counter++)
							{
								LRRSiteComponent lsc = masterComp.SiteComponentsObj[counter];

								//Check for Plant Effective Time
								bool isPET = CheckForEffectivePlantTime(lsc.EffectivePlantTime);
									
								//foreach(LRRSiteComponent lsc in masterComp.SiteComponentsObj)
								//{
								if(modelEquipSec.Orientation == "R")
								{
									foreach(object obj in modelEquipSec.EquipmentModelData.Arrtrs)
									{
										if((string)obj == "Name")
										{
											int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Name"];
											int rowIndex = i;
											string val = lsc.Name;
											try
											{
												modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
											}
											catch(Exception ex)
											{
												string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
												Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + ex.Message + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
											}
										}
										if((string)obj == "PlantUnit")
										{
											int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["PlantUnit"];
											int rowIndex = i;
											string val = LRRModel.ChildModel.GetPlantLevelName(lsc.PlantUnit);
											try
											{
												modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
											}
											catch(Exception ex)
											{
												string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
												Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + ex.Message + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
											}
										}
										try
										{
											if((string)obj == "Tag")
											{
												int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Tag"];
												int rowIndex = i;
												string val = lsc.EquipmentTag;
												try
												{
													modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
												}
												catch(Exception ex)
												{
													string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
													Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + ex.Message + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
												}
											}
										}
										catch
										{

										}
										//									if((string)obj == "Tag")
										//									{
										//										int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Tag"];
										//										int rowIndex = i;
										//										string val = lsc.PlantMeterTag;
										//										modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = val;
										//									}
									}

									foreach(object obj in modelEquipSec.EquipmentModelData.Properties)
									{
										LRRSiteComponentProperty lscp = lsc.GetPropertyFromMasterPropID((string)obj);

										
										string valueToWrite = "";
										if(lscp != null)
										{
											//Check for Plant Effective Time
											if(isPET)
												isPET = CheckForEffectivePlantTime(lscp.EffectivePlantTime);

											LRRSiteComponentPropertyValue lscpv =  lscp.GetValueObjectForPropertyMode(modelEquipSec.ModeID);
											if(lscpv != null) //Added NULL check by Nidhig
											{
												//Check for Plant Effective Time
												if(isPET)
													isPET = CheckForEffectivePlantTime(lscpv.EffectivePlantTime);
											}
											if(modelEquipSec.IsOverridenValueOnly == true)
											{
												//LRRSiteComponentPropertyValue lscpv =  lscp.GetValueObjectForPropertyMode(modelEquipSec.ModeID);
												if(lscpv != null)
												{
													if(lscpv.IsOverrided)
													{
														//valueToWrite = lscpv.ActualOverridedValue;
														if(isPET)
															valueToWrite = lscpv.ActualPropertyValue;
														else
															valueToWrite = PET_MODEL_CONSTANT;

													}
													else
													{
														if(isPET)
															valueToWrite = "";
														else
															valueToWrite = PET_MODEL_CONSTANT;
													}
												}
											}
											else
											{	
												if(lscpv != null)
												{
													if(lscpv.ModeID != "2")
													{
														if(lscpv.IsOverrided)
														{
															if(isPET)
															{
																valueToWrite = lscpv.ActualOverridedValue;
																//valueToWrite = lscpv.ActualPropertyValue;
															}
															else
																valueToWrite = PET_MODEL_CONSTANT;
														}
														else
														{
															if(isPET)
															{
																//valueToWrite = lscp.GetActualValueForPropertyMode(modelEquipSec.ModeID);
																valueToWrite = lscpv.ActualPropertyValue;
															}
															else
																valueToWrite = PET_MODEL_CONSTANT;
														}
													}
													else
													{
														if(isPET)
														{
															//valueToWrite = lscp.GetActualValueForPropertyMode(modelEquipSec.ModeID);
															valueToWrite = lscpv.ActualPropertyValue;
														}
														else
															valueToWrite = PET_MODEL_CONSTANT;
													}
												}
												//valueToWrite = lscp.GetActualValueForPropertyMode(modelEquipSec.ModeID);
											}
											if(lscp != null)
											{
												int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID];
												int rowIndex = i;
												try
												{
													object retValue;
													if(Symcon.Utils.Validator.Validator.CanConvertToDouble(valueToWrite,out retValue))
													{
														modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = retValue;
													}
													else
													{
														modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)valueToWrite;
													}
													//modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = valueToWrite;
												}
												catch(Exception ex)
												{
													string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
													Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + ex.Message + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
												}
											}
										}
										else
										{
											string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
											Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- "  + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
										}
									}
								}
								else
								{

									foreach(object obj in modelEquipSec.EquipmentModelData.Arrtrs)
									{
										if((string)obj == "Name")
										{
											int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["Name"]*/i;
											int rowIndex = /*i*/(int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Name"];
											string val = lsc.Name;
											try
											{
												modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
											}
											catch(Exception ex)
											{
												string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
												Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + ex.Message + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
											}
										}
										if((string)obj == "PlantUnit")
										{
											int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["PlantUnit"]*/i;
											int rowIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["PlantUnit"]/*i*/;
											string val = LRRModel.ChildModel.GetPlantLevelName(lsc.PlantUnit);
											try
											{
												modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
											}
											catch(Exception ex)
											{
												string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
												Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + ex.Message + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
											}
										}
										try
										{
											if((string)obj == "Tag")
											{
												int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["Name"]*/i;
												int rowIndex = /*i*/(int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Tag"];
												string val = lsc.EquipmentTag;
												try
												{
													modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
												}
												catch(Exception ex)
												{
													string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
													Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + ex.Message + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
												}
											}
										}
										catch
										{

										}
										//									if((string)obj == "Tag")
										//									{
										//										int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["Tag"]*/i;
										//										int rowIndex = /*i*/(int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Tag"];
										//										string val = lsc.PlantMeterTag;
										//										modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = val;
										//									}
									}

									foreach(object obj in modelEquipSec.EquipmentModelData.Properties)
									{
										LRRSiteComponentProperty lscp = lsc.GetPropertyFromMasterPropID((string)obj);
										//Check for Plant Effective Time
										
										string valueToWrite = "";
										if(lscp != null)
										{
											if(isPET)
												isPET = CheckForEffectivePlantTime(lscp.EffectivePlantTime);

											LRRSiteComponentPropertyValue lscpv =  lscp.GetValueObjectForPropertyMode(modelEquipSec.ModeID);
											//Check for Plant Effective Time
											if(isPET)
												isPET = CheckForEffectivePlantTime(lscpv.EffectivePlantTime);
											
											if(modelEquipSec.IsOverridenValueOnly == true)
											{
												/*LRRSiteComponentPropertyValue lscpv =  lscp.GetValueObjectForPropertyMode(modelEquipSec.ModeID);
												if(lscpv.IsOverrided)
												{
													valueToWrite = lscpv.ActualOverridedValue;
												}
												else
												{
													valueToWrite = "";
												}*/

												if(lscpv != null)
												{
													if(lscpv.IsOverrided)
													{
														if(isPET)
														{
															//valueToWrite = lscpv.ActualOverridedValue;
															valueToWrite = lscpv.ActualPropertyValue;
														}
														else
															valueToWrite = PET_MODEL_CONSTANT;

													}
													else
													{
														if(isPET)
															valueToWrite = "";
														else
															valueToWrite = PET_MODEL_CONSTANT;
													}
												}
											}
											else
											{
												if(lscpv != null)
												{
													if(lscpv.IsOverrided)
													{
														if(isPET)
														{
															valueToWrite = lscpv.ActualOverridedValue;
															//valueToWrite = lscpv.ActualPropertyValue;
														}
														else
															valueToWrite = PET_MODEL_CONSTANT;
													}
													else
													{
														if(isPET)
														{
															//valueToWrite = lscp.GetActualValueForPropertyMode(modelEquipSec.ModeID);
															valueToWrite = lscpv.ActualPropertyValue;
														}
														else
															valueToWrite = PET_MODEL_CONSTANT;
	
													}
												}
												//valueToWrite = lscp.GetActualValueForPropertyMode(modelEquipSec.ModeID);
											}
											if(lscp != null)
											{
												int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID]*/i;
												int rowIndex = /*i*/(int)modelEquipSec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID];

												try
												{
													object retValue;
													if(Symcon.Utils.Validator.Validator.CanConvertToDouble(valueToWrite,out retValue))
													{
														modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = retValue;
													}
													else
													{
														modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)valueToWrite;
													}
													//modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = valueToWrite;
												}
												catch(Exception ex)
												{
													string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
													Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + ex.Message + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
												}
											}
										}
										else
										{
											string msg = "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
											Symcon.Utils.MessageLogger.Logger.LogMessage("WRITE ERROR IN " + msg + "ACTUAL Message -- " + "-------------------------\r\n",Symcon.Utils.MessageLogger.Logger.Level.Debug);
										}
									}

								}
								i++;
							}

							//Check if the Read from Engine flag is true . If so read the values from the named range and apply if to the current data
							//to be written to engine.
							//*******************************************************
							if(_app.GetOptions().ReadInputsFromEngine)
							{
								object[,] copyData = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,modelEquipSec.RangeName,modelEquipSec.Orientation);
								if(copyData.Length == modelEquipSec.EquipmentModelData.Data.Length) //ChBeck if the Src and Destination Lengths Match...
								{
									//Write the data to the excel sheets...
									_symExcelWrapper.CopyObjectArrayData(copyData,modelEquipSec.EquipmentModelData.Data,modelEquipSec.RangeName,model.WorkBookName);
									_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,modelEquipSec.RangeName,modelEquipSec.Orientation,modelEquipSec.EquipmentModelData.Data);
									ExecuteInternalRead(modelEquipSec,modelEquip.MasterCompID,model.WorkBookName);
								}
								else 
								{
									//Should not be here....
									Symcon.Utils.MessageLogger.Logger.LogMessage("ERROR IN Extract Data From Engine Array Size Mismatch -- \r\n NamedRange: " + modelEquipSec.RangeName + "\r\n WorkBook:" + model.WorkBookName + "\r\n ----------",Symcon.Utils.MessageLogger.Logger.Level.Debug);
									_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,modelEquipSec.RangeName,modelEquipSec.Orientation,modelEquipSec.EquipmentModelData.Data);
								}
							}
							else
							{
								_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,modelEquipSec.RangeName,modelEquipSec.Orientation,modelEquipSec.EquipmentModelData.Data);
							}

						
	
							//_symExcelWrapper.ExecuteMacro(model.WorkBookName,"Recalc");
						}

					

					}
				
					//Write for the Grouped Properties

					foreach(LRRModelEngineGroupedPropSection sec in model.GroupedPropertiesSections)
					{
						if(sec.Direction == "I" && sec.Orientation == "C")
						{
							object[,] dataArray = sec.Data;
							for(int i=0;i<sec.Columns.Count;i++)
							{   
								string siteComponentID = "";
								siteComponentID =  sec.Columns[i].ToString();
								if(siteComponentID != "")
								{
							
									LRRSiteComponent lsc =  LRRModel.SiteComponentsObj.GetSiteComponentByID(siteComponentID);
									if(lsc == null)
									{
										Logger.LogMessage("ERROR IN WRITE CYCLE "+ sec.NamedRange,Logger.Level.Debug);
										continue;
									}

									//Check for Plant Effective Time
									bool isGPPET = CheckForEffectivePlantTime(lsc.EffectivePlantTime);

									for(int k = 0;k<sec.Rows.Count;k++)
									{
										string constituentID = sec.Rows[k].ToString();

										LRRSiteComponentProperties lscprops = lsc.Properties.FilterPropertiesByPropertyType(sec.GroupPropertyID);
									
										if(lscprops.Count > 0)
										{
											LRRSiteComponentProperty lscp =  lscprops[0];
											//Check for Plant Effective Time
											if(isGPPET)
												isGPPET = CheckForEffectivePlantTime(lscp.EffectivePlantTime);

											string siteCompID = lscp.ID;
											string siteGroupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lscp.ID,constituentID); //Argon

											//LRRSiteComponentGroupProperties lscgp = lscp.GroupPpties.FilterGroupPropertiesForLkUpQualifierID(sec.LookupQualifierID);
											//LRRSiteComponentGroupProperty gp = lscgp.GetSiteCompGroupPropFromID(constituentID);
											if(siteGroupPropID != null && siteGroupPropID != "")
											{
												string val = LRRModel.SiteGroupPropertyQualifierValuesObj.GetActualQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode);

												//Check for Effective Plant Time
												LRRSiteGroupPropertyQualifierValue qualValObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(siteGroupPropID,sec.LookupQualifierID,sec.Mode);

												if(qualValObj == null)
												{
													Logger.LogMessage("ERROR IN WRITE CYCLE "+ sec.NamedRange,Logger.Level.Debug);
													continue;
												}

												if(isGPPET)
													isGPPET = CheckForEffectivePlantTime(qualValObj.EffectivePlantTime);
												string val2 = val;

												object retValue;

												if(Symcon.Utils.Validator.Validator.CanConvertToDouble(val,out retValue))
												{
													if(isGPPET)
														dataArray[k,i] = /*Double.Parse(val)*/ retValue;
													else
														dataArray[k,i] = PET_MODEL_CONSTANT;
												}
												else
												{
													if(isGPPET)
														dataArray[k,i] = (object)val;
													else
														dataArray[k,i] = PET_MODEL_CONSTANT;
												}
											
												//dataArray[k,i] = val;
												//Check for nulll....
												//string val = gp.GroupPptiesQualifierValues.GetQualifierValueForPropertyMode(sec.Mode);
											}
											else
											{
												int gg = 900;
											}
										}

									}
								}
						
							}

							//long hh =  dataArray.GetLongLength(0);
							if(_app.GetOptions().ReadInputsFromEngine) //Check if Values need to be read from Engine
							{
								object[,] copyData = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,sec.NamedRange,"R/C");
								if(copyData.Length == sec.Data.Length) //Check if the Src and Destination Lengths Match...
								{
									_symExcelWrapper.CopyObjectArrayData(copyData,sec.Data,sec.NamedRange,model.WorkBookName);
									_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,"R/C",sec.Data);
									ExecuteInternalReadGroupedProperties(sec,sec.Data);
								}
								else
								{
									//Should not be here....
									Symcon.Utils.MessageLogger.Logger.LogMessage("ERROR IN Extract Data From Engine Array Size Mismatch -- \r\n NamedRange: " + sec.NamedRange + "\r\n WorkBook:" + model.WorkBookName + "\r\n ----------",Symcon.Utils.MessageLogger.Logger.Level.Debug);
									_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,"R/C",sec.Data);
								}
							}
							else
							{
								_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,"R/C",sec.Data);
							}
						}
						else if (sec.Direction == "I" && sec.Orientation == "R")
						{
							object[,] dataArray = sec.Data;

							for(int i=0;i<sec.Rows.Count;i++)
							{   
								string siteComponentID = "";
								siteComponentID =  sec.Rows[i].ToString();
								if(siteComponentID != "")
								{
							
									LRRSiteComponent lsc =  LRRModel.SiteComponentsObj.GetSiteComponentByID(siteComponentID);

									if(lsc == null)
									{
										Logger.LogMessage("ERROR IN WRITE CYCLE "+ sec.NamedRange,Logger.Level.Debug);
										continue;
									}
									
									//Check for Plant Effective Time
									bool isGPPET = CheckForEffectivePlantTime(lsc.EffectivePlantTime);
									for(int k = 0;k<sec.Columns.Count;k++)
									{
										string constituentID = sec.Columns[k].ToString();

										LRRSiteComponentProperties lscprops = lsc.Properties.FilterPropertiesByPropertyType(sec.GroupPropertyID);
            
										if(lscprops.Count > 0)
										{
											LRRSiteComponentProperty lscp =  lscprops[0];
											//Check for Plant Effective Time
											if(isGPPET)
												isGPPET = CheckForEffectivePlantTime(lscp.EffectivePlantTime);

											string siteCompID = lscp.ID;
											string siteGroupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lscp.ID,constituentID); //Argon

											//LRRSiteComponentGroupProperties lscgp = lscp.GroupPpties.FilterGroupPropertiesForLkUpQualifierID(sec.LookupQualifierID);
											//LRRSiteComponentGroupProperty gp = lscgp.GetSiteCompGroupPropFromID(constituentID);
											if(siteGroupPropID != null && siteGroupPropID != "")
											{
												string val = LRRModel.SiteGroupPropertyQualifierValuesObj.GetActualQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode);
												//Check for Effective Plant Time
												LRRSiteGroupPropertyQualifierValue qualValObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(siteGroupPropID,sec.LookupQualifierID,sec.Mode);

												if(qualValObj == null)
												{
													Logger.LogMessage("ERROR IN WRITE CYCLE "+ sec.NamedRange,Logger.Level.Debug);
													continue;
												}

												if(isGPPET)
													isGPPET = CheckForEffectivePlantTime(qualValObj.EffectivePlantTime);

												string val2 = val;
												object retValue;
												if(Symcon.Utils.Validator.Validator.CanConvertToDouble(val,out retValue))
												{
													if(isGPPET)
														dataArray[i,k] = retValue;
													else
														dataArray[i,k] = PET_MODEL_CONSTANT;
												}
												else
												{
													if(isGPPET)
														dataArray[i,k] = (object)val;
													else
														dataArray[i,k] = PET_MODEL_CONSTANT;
												}
											
												//dataArray[i,k] = val;
												//Check for nulll....
												//string val = gp.GroupPptiesQualifierValues.GetQualifierValueForPropertyMode(sec.Mode);
											}
											else
											{
												int gg = 900;
											}
										}

									}
								}
						
							}

							_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,"R/C",sec.Data);
						}
					}





					//For Data Arrays
					foreach(LRRModelEngineDataArrayPropSection sec in model.DataArrayPropSections)
					{
						if(sec.Direction == "I" && sec.Orientation == "C")
						{
							object[,] dataArray = sec.Data;
						
							LRRSiteComponents lrrSiteComps = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(sec.ComponentTypeID);
							int k = 0;
							foreach(LRRSiteComponent lrrSComp in lrrSiteComps)
							{
								//Check for Plant Effective Time
								bool isDAPET = CheckForEffectivePlantTime(lrrSComp.EffectivePlantTime);

								foreach(LRRSiteComponentProperty lrrsitecompProp in lrrSComp.Properties)
								{
									if(lrrsitecompProp.Type == sec.PropertyTypeID)
									{
										if(isDAPET)
											isDAPET = CheckForEffectivePlantTime(lrrsitecompProp.EffectivePlantTime);
										for(int i=0;i<sec.Headers.Count;i++)
										{
											string lookUpPropertiesID = sec.Headers[i].ToString();
											LRRSiteGroupPropertyQualifierValue val = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForModeSiteCompAndLookUpProp(lrrsitecompProp.ID,lookUpPropertiesID,sec.Mode);
											if(isDAPET)
												isDAPET = CheckForEffectivePlantTime(val.EffectivePlantTime);

											if(isDAPET)
												dataArray[i,k] = (object)val.PropertyValue;
											else
												dataArray[i,k] = PET_MODEL_CONSTANT;
										}
									}
								}
								k++;
							}
							//Check if the Read from Engine flag is true . If so read the values from the named range and apply if to the current data
							//to be written to engine.
							//*******************************************************
							if(_app.GetOptions().ReadInputsFromEngine)
							{
								object[,] copyData = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,sec.NamedRange,sec.Orientation);
								if(copyData.Length == sec.Data.Length) //ChBeck if the Src and Destination Lengths Match...
								{
									//Write the data to the excel sheets...
									_symExcelWrapper.CopyObjectArrayData(copyData,sec.Data,sec.NamedRange,model.WorkBookName);
									_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,sec.Orientation,sec.Data);
									ExecuteInternalReadDataArrays(sec,sec.Data);
								}
								else 
								{
									//Should not be here....
									Symcon.Utils.MessageLogger.Logger.LogMessage("ERROR IN Extract Data From Engine Array Size Mismatch -- \r\n NamedRange: " + sec.NamedRange+ "\r\n WorkBook:" + model.WorkBookName + "\r\n ----------",Symcon.Utils.MessageLogger.Logger.Level.Debug);
									_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,sec.Orientation,sec.Data);
								}
							}
							else
							{
								_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,sec.Orientation,sec.Data);
							}
						}
						else if (sec.Direction == "I" && sec.Orientation == "R")
						{
							object[,] dataArray = sec.Data;
						
							LRRSiteComponents lrrSiteComps = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(sec.ComponentTypeID);
							int k = 0;
							foreach(LRRSiteComponent lrrSComp in lrrSiteComps)
							{
								//Check for Plant Effective Time
								bool isDAPET = CheckForEffectivePlantTime(lrrSComp.EffectivePlantTime);

								foreach(LRRSiteComponentProperty lrrsitecompProp in lrrSComp.Properties)
								{
									if(lrrsitecompProp.Type == sec.PropertyTypeID)
									{
										if(isDAPET)
											isDAPET = CheckForEffectivePlantTime(lrrsitecompProp.EffectivePlantTime);
										for(int i=0;i<sec.Headers.Count;i++)
										{
											string lookUpPropertiesID = sec.Headers[i].ToString();
											LRRSiteGroupPropertyQualifierValue val = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForModeSiteCompAndLookUpProp(lrrsitecompProp.ID,lookUpPropertiesID,sec.Mode);
											if(isDAPET)
												isDAPET = CheckForEffectivePlantTime(val.EffectivePlantTime);

											if(isDAPET)
												dataArray[k,i] = (object)val.ActualPropertyValue;
											else
												dataArray[k,i] = PET_MODEL_CONSTANT;
										}
									}
								}
								k++;
							}
							if(_app.GetOptions().ReadInputsFromEngine)
							{
								object[,] copyData = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,sec.NamedRange,sec.Orientation);
								if(copyData.Length == sec.Data.Length) //ChBeck if the Src and Destination Lengths Match...
								{
									//Write the data to the excel sheets...
									_symExcelWrapper.CopyObjectArrayData(copyData,sec.Data,sec.NamedRange,model.WorkBookName);
									_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,sec.Orientation,sec.Data);
									ExecuteInternalReadDataArrays(sec,sec.Data);
								}
								else 
								{
									//Should not be here....
									Symcon.Utils.MessageLogger.Logger.LogMessage("ERROR IN Extract Data From Engine Array Size Mismatch -- \r\n NamedRange: " + sec.NamedRange+ "\r\n WorkBook:" + model.WorkBookName + "\r\n ----------",Symcon.Utils.MessageLogger.Logger.Level.Debug);
									_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,sec.Orientation,sec.Data);
								}
							}
							else
							{
								_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,sec.Orientation,sec.Data);
							}
						}
					}

				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage("ERROR IN WRITE CYCLE "+ ex.Message,Logger.Level.Debug);
			}
			 WriteSingleCellValues();
			if(_app.GetOptions().ReadInputsFromEngine == true)
			{
				_app.GetOptions().ReadInputsFromEngine = false;
				_app.SetReadInputFromEngineState(false);
			}
			return true;
		}

		/// <summary>
		/// This method is used to read the information
		/// to the model from the execution engine...
		/// takes the OUTPUT's for the cycle
		/// </summary>
		/// <returns></returns>
		public bool ExecuteReadCycle()
		{
			LRRExecutionEngineModels models = LRREEModel.EEModel;
			LRRMasterComponents comps = LRRModel.Model;

			//Write the data to the excel sheets...
			foreach(LRRExecutionEngineModel model in models)
			{
				foreach(LRRExecutionEngineModelEquipment modelEquip in model.ModelEquipments)
				{
					foreach(LRRExecutionEngineModelEquipmentSection modelEquipSec in modelEquip.ModelEquipmentSectionObj)
					{
						if(modelEquipSec.SectionType == "INPUT")
						{
							continue;
						}

						modelEquipSec.EquipmentModelData.Data = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,modelEquipSec.RangeName,modelEquipSec.Orientation);

						LRRMasterComponent masterComp =  comps.GetMasterComponentByID(modelEquip.MasterCompID);
						if(masterComp == null)
						{
							//Added by Nidhig on Johan's Request for incorrect Ranges
							string msg = "WorkBook Name: " + model.WorkBookName  + "Range Name: " + modelEquipSec.RangeName  + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
							Symcon.Utils.MessageLogger.Logger.LogMessage("READ ERROR IN " + msg ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
							continue;
						}

						int i=0;
						foreach(LRRSiteComponent lsc in masterComp.SiteComponentsObj)
						{
							//Check for Plant Effective Time
							bool isPET = CheckForEffectivePlantTime(lsc.EffectivePlantTime);

							if(modelEquipSec.Orientation == "R")
							{
								foreach(object obj in modelEquipSec.EquipmentModelData.Arrtrs)
								{
									if((string)obj == "Name")
									{
										int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Name"];
										int rowIndex = i;
										string val = lsc.Name;
										modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
									}
									if((string)obj == "PlantUnit")
									{
										int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["PlantUnit"];
										int rowIndex = i;
										string val = lsc.PlantUnit;
										modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
									}
								}

								foreach(object obj in modelEquipSec.EquipmentModelData.Properties)
								{
									LRRSiteComponentProperty lscp = lsc.GetPropertyFromMasterPropID((string)obj);
									string valueToWrite = "";
									if(lscp != null)
									{
										//Check for Plant Effective Time
										if(isPET)
											isPET = CheckForEffectivePlantTime(lscp.EffectivePlantTime);

										valueToWrite = lscp.GetValueForPropertyMode(modelEquipSec.ModeID);
										if(lscp != null)
										{
											int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID];
											int rowIndex = i;
											LRRSiteComponentPropertyValues propValues = lscp.Values;
											if(propValues != null)
											{
												foreach(LRRSiteComponentPropertyValue pv in propValues)
												{
													if(pv.ModeID == modelEquipSec.ModeID)
													{
														//If property value is not overrided
														if(!pv.IsOverrided)
														{
															if(isPET)
																isPET = CheckForEffectivePlantTime(pv.EffectivePlantTime);

															if(isPET)
																pv.PropertyValue = modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex].ToString();
															else
																pv.PropertyValue =PET_UI_CONSTANT;
														}
													}
												}
											}
											//modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = valueToWrite;
										}
									}
									else
									{
										//Added by Nidhi to provide feedback on error
										string msg = "WorkBook Name: " + model.WorkBookName  + "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
										Symcon.Utils.MessageLogger.Logger.LogMessage("READ ERROR IN " + msg ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
									}
								}
							}
							else
							{

								foreach(object obj in modelEquipSec.EquipmentModelData.Arrtrs)
								{
									if((string)obj == "Name")
									{
										int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["Name"]*/i;
										int rowIndex = /*i*/(int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Name"];
										string val = lsc.Name;
										modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
									}
									if((string)obj == "PlantUnit")
									{
										int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["PlantUnit"]*/i;
										int rowIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["PlantUnit"]/*i*/;
										string val = lsc.PlantUnit;
										modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
									}
								}

								foreach(object obj in modelEquipSec.EquipmentModelData.Properties)
								{
									LRRSiteComponentProperty lscp = lsc.GetPropertyFromMasterPropID((string)obj);
									string valueToWrite = "";
									if(lscp != null)
									{
										//Check for Plant Effective Time
										if(isPET)
											isPET = CheckForEffectivePlantTime(lscp.EffectivePlantTime);

										int colIndex = i;
										int rowIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID];
										LRRSiteComponentPropertyValues propValues = lscp.Values;
										if(propValues != null)
										{
											foreach(LRRSiteComponentPropertyValue pv in propValues)
											{
												if(pv.ModeID == modelEquipSec.ModeID)
												{
													//If property value is not overrided
													if(!pv.IsOverrided)
													{
														if(isPET)
															isPET = CheckForEffectivePlantTime(pv.EffectivePlantTime);

														if(isPET)
															pv.PropertyValue = modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex].ToString();
														else
															pv.PropertyValue =PET_UI_CONSTANT;
													}
												}
											}
										}

										//actual
										/*valueToWrite = lscp.GetValueForPropertyMode(modelEquipSec.ModeID);
										if(lscp != null)
										{
											int colIndex = i;
											int rowIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID];
											modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = valueToWrite;
										}*/
									}
									else
									{
										//Added by Nidhi to provide feedback on error
										string msg = "WorkBook Name: " + model.WorkBookName  + "Range Name: " + modelEquipSec.RangeName + "\r\n" + "Orientation:" + modelEquipSec.Orientation + "\r\n";
										Symcon.Utils.MessageLogger.Logger.LogMessage("READ ERROR IN " + msg ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
									}
								}

							}
							i++;
						}

						
					}

				}

				//Grouped properties read.... 
				
				foreach(LRRModelEngineGroupedPropSection sec in model.GroupedPropertiesSections)
				{
					if(sec.Direction == "O" && sec.Orientation == "C")
					{
						object[,] dataArray = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,sec.NamedRange,"R/C");
						for(int i=0;i<sec.Columns.Count;i++)
						{   
							string siteComponentID = "";
							siteComponentID =  sec.Columns[i].ToString();
							if(siteComponentID != "")
							{
							
								LRRSiteComponent lsc =  LRRModel.SiteComponentsObj.GetSiteComponentByID(siteComponentID);
								//Check for Plant Effective Time
								if(lsc == null)
								{
									Logger.LogMessage("ERROR IN WRITE CYCLE "+ sec.NamedRange,Logger.Level.Debug);
									continue;
								}

								bool isGPPET = CheckForEffectivePlantTime(lsc.EffectivePlantTime);
								for(int k = 0;k<sec.Rows.Count;k++)
								{
									string constituentID = sec.Rows[k].ToString();

									LRRSiteComponentProperties lscprops = lsc.Properties.FilterPropertiesByPropertyType(sec.GroupPropertyID);

									if(lscprops.Count > 0)
									{
										LRRSiteComponentProperty lscp =  lscprops[0];

										if(isGPPET)
											isGPPET = CheckForEffectivePlantTime(lscp.EffectivePlantTime);

										string siteCompID = lscp.ID;
										string siteGroupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lscp.ID,constituentID); //Argon
										if(siteGroupPropID != null && siteGroupPropID != "")
										{
											string val;// = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode);
											 val = dataArray[k,i].ToString();
											LRRSiteGroupPropertyQualifierValue qualValObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(siteGroupPropID,sec.LookupQualifierID,sec.Mode);

											//If prop Value is Not Overrided
											if(qualValObj != null && !qualValObj.IsOverrided)
											{
												if(isGPPET)
													isGPPET = CheckForEffectivePlantTime(qualValObj.EffectivePlantTime);

												if(isGPPET)
													LRRModel.SiteGroupPropertyQualifierValuesObj.SetQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode,val);
												else
													LRRModel.SiteGroupPropertyQualifierValuesObj.SetQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode,PET_UI_CONSTANT);
											}
										}
										else
										{
											int gg = 900;
										}
									}

								}
							}
						
						}
					}
					else if (sec.Direction == "O" && sec.Orientation == "R")
					{
						object[,] dataArray = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,sec.NamedRange,"R/C");
						
						for(int i=0;i<sec.Rows.Count;i++)
						{   
							string siteComponentID = "";
							siteComponentID =  sec.Rows[i].ToString();
							if(siteComponentID != "")
							{
							
								LRRSiteComponent lsc =  LRRModel.SiteComponentsObj.GetSiteComponentByID(siteComponentID);

								if(lsc == null)
								{
									Logger.LogMessage("ERROR IN WRITE CYCLE "+ sec.NamedRange,Logger.Level.Debug);
									continue;
								}
								if(lsc != null) //Added NULL Check by NidhiG
								{
									//Check for Plant Effective Time
									bool isGPPET = CheckForEffectivePlantTime(lsc.EffectivePlantTime);

									for(int k = 0;k<sec.Columns.Count;k++)
									{
										string constituentID = sec.Columns[k].ToString();

										LRRSiteComponentProperties lscprops = lsc.Properties.FilterPropertiesByPropertyType(sec.GroupPropertyID);
            
										if(lscprops.Count > 0)
										{
											LRRSiteComponentProperty lscp =  lscprops[0];
											if(isGPPET)
												isGPPET = CheckForEffectivePlantTime(lscp.EffectivePlantTime);

											string siteCompID = lscp.ID;
											string siteGroupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lscp.ID,constituentID); //Argon

											if(siteGroupPropID != null && siteGroupPropID != "")
											{
												string val;// = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode);
												val = dataArray[i,k].ToString();
												LRRSiteGroupPropertyQualifierValue qualValObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(siteGroupPropID,sec.LookupQualifierID,sec.Mode);

												//If prop Value is Not Overrided
												if(qualValObj != null && !qualValObj.IsOverrided)
												{
													if(isGPPET)
														isGPPET = CheckForEffectivePlantTime(qualValObj.EffectivePlantTime);

													if(isGPPET)
														LRRModel.SiteGroupPropertyQualifierValuesObj.SetQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode,val);
													else
														LRRModel.SiteGroupPropertyQualifierValuesObj.SetQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode,PET_UI_CONSTANT);
												}
											}
											else
											{
												int gg = 900;
											}
										}

									}
								}
							}
						
						}
						//_symExcelWrapper.PutDataIntoNamedRange(model.WorkBookName,sec.NamedRange,"R/C",sec.Data);
					}
				}


				// For Data Arrays.... 
				foreach(LRRModelEngineDataArrayPropSection sec in model.DataArrayPropSections)
				{
					if(sec.Direction == "O" && sec.Orientation == "C")
					{
						object[,] dataArray = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,sec.NamedRange,sec.Orientation);
						
						LRRSiteComponents lrrSiteComps = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(sec.ComponentTypeID);
						int k = 0;
						foreach(LRRSiteComponent lrrSComp in lrrSiteComps)
						{
							//Check for PET
							bool isDAPET = CheckForEffectivePlantTime(lrrSComp.EffectivePlantTime);

							foreach(LRRSiteComponentProperty lrrsitecompProp in lrrSComp.Properties)
							{
								if(lrrsitecompProp.Type == sec.PropertyTypeID)
								{
									if(isDAPET)
										isDAPET = CheckForEffectivePlantTime(lrrsitecompProp.EffectivePlantTime);

									for(int i=0;i<sec.Headers.Count;i++)
									{
										string lookUpPropertiesID = sec.Headers[i].ToString();
										LRRSiteGroupPropertyQualifierValue val = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForModeSiteCompAndLookUpProp(lrrsitecompProp.ID,lookUpPropertiesID,sec.Mode);
										//If property value is not overrided
										if(!val.IsOverrided)
										{
											if(isDAPET)
												isDAPET = CheckForEffectivePlantTime(val.EffectivePlantTime);

											if(isDAPET)
												val.PropertyValue = dataArray[i,k].ToString();
											else
												val.PropertyValue = PET_UI_CONSTANT;
											}
									}
								}
							}
							k++;
						}
					}
					else if (sec.Direction == "O" && sec.Orientation == "R")
					{
						object[,] dataArray = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(model.WorkBookName,sec.NamedRange,sec.Orientation);
						
						LRRSiteComponents lrrSiteComps = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(sec.ComponentTypeID);
						int k = 0;
						foreach(LRRSiteComponent lrrSComp in lrrSiteComps)
						{
							//Check for PET
							bool isDAPET = CheckForEffectivePlantTime(lrrSComp.EffectivePlantTime);

							foreach(LRRSiteComponentProperty lrrsitecompProp in lrrSComp.Properties)
							{
								if(lrrsitecompProp.Type == sec.PropertyTypeID)
								{
									if(isDAPET)
										isDAPET = CheckForEffectivePlantTime(lrrsitecompProp.EffectivePlantTime);

									for(int i=0;i<sec.Headers.Count;i++)
									{
										string lookUpPropertiesID = sec.Headers[i].ToString();
										LRRSiteGroupPropertyQualifierValue val = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForModeSiteCompAndLookUpProp(lrrsitecompProp.ID,lookUpPropertiesID,sec.Mode);
										//If property value is not overrided
										if(!val.IsOverrided)
										{
											if(isDAPET)
												isDAPET = CheckForEffectivePlantTime(val.EffectivePlantTime);

											if(isDAPET)
												val.PropertyValue = dataArray[k,i].ToString();
											else
												val.PropertyValue = PET_UI_CONSTANT;
										}
									}
								}
							}
							k++;
						}
					}
				}
			}
			ReadSingleCellValues();
			return true;
		}

		private void ExecuteInternalRead(LRRExecutionEngineModelEquipmentSection modelEquipSec,string MasterCompID,string WorkBookName)
		{
			modelEquipSec.EquipmentModelData.Data = _symExcelWrapper.GetDataFromNamedRangeAsObjectArray(WorkBookName,modelEquipSec.RangeName,modelEquipSec.Orientation);

			LRRMasterComponent masterComp =  LRRModel.Model.GetMasterComponentByID(MasterCompID);
			if(masterComp == null)
				return;
			int i=0;
			int totalCountItems = masterComp.SiteComponentsObj.Count;
			masterComp.SiteComponentsObj.SortColumn = "InternalSortIndex";
			masterComp.SiteComponentsObj.SortingOrder = Symcon.BusinessObjects.GenericComparer.SortOrder.Ascending;
			masterComp.SiteComponentsObj.Sort();
			for(int counter=0;counter< totalCountItems;counter++)
			{
				 LRRSiteComponent lsc = masterComp.SiteComponentsObj[counter];
				if(modelEquipSec.Orientation == "R")
				{
					foreach(object obj in modelEquipSec.EquipmentModelData.Arrtrs)
					{
						if((string)obj == "Name")
						{
							int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Name"];
							int rowIndex = i;
							string val = lsc.Name;
							modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
						}
						if((string)obj == "PlantUnit")
						{
							int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["PlantUnit"];
							int rowIndex = i;
							string val = lsc.PlantUnit;
							modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
						}
						//									if((string)obj == "Tag")
						//									{
						//										int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Tag"];
						//										int rowIndex = i;
						//										string val = lsc.PlantMeterTag;
						//										modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = val;
						//									}
					}

					foreach(object obj in modelEquipSec.EquipmentModelData.Properties)
					{
						LRRSiteComponentProperty lscp = lsc.GetPropertyFromMasterPropID((string)obj);
						string valueToWrite = "";
						if(lscp != null)
						{
							valueToWrite = lscp.GetValueForPropertyMode(modelEquipSec.ModeID);
							if(lscp != null)
							{
								int colIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID];
								int rowIndex = i;
								LRRSiteComponentPropertyValues propValues = lscp.Values;
								if(propValues != null)
								{
									foreach(LRRSiteComponentPropertyValue pv in propValues)
									{
										if(pv.ModeID == modelEquipSec.ModeID)
										{
											if(!pv.IsOverrided)
											{
												if(modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] != null)
												{
													pv.PropertyValue = modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex].ToString();
												}
												else
												{
													pv.PropertyValue = "";
												}
											}
										}
									}
								}
								//modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = valueToWrite;
							}
						}
					}
				}
				else
				{

					foreach(object obj in modelEquipSec.EquipmentModelData.Arrtrs)
					{
						if((string)obj == "Name")
						{
							int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["Name"]*/i;
							int rowIndex = /*i*/(int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Name"];
							string val = lsc.Name;
							modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
						}
						if((string)obj == "PlantUnit")
						{
							int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["PlantUnit"]*/i;
							int rowIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex["PlantUnit"]/*i*/;
							string val = lsc.PlantUnit;
							modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = (object)val;
						}
						//									if((string)obj == "Tag")
						//									{
						//										int colIndex = /*(int)sec.EquipmentModelData.PropertyIDIndex["Tag"]*/i;
						//										int rowIndex = /*i*/(int)modelEquipSec.EquipmentModelData.PropertyIDIndex["Tag"];
						//										string val = lsc.PlantMeterTag;
						//										modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = val;
						//									}
					}

					foreach(object obj in modelEquipSec.EquipmentModelData.Properties)
					{
						LRRSiteComponentProperty lscp = lsc.GetPropertyFromMasterPropID((string)obj);
						string valueToWrite = "";
						if(lscp != null)
						{
							int colIndex = i;
							int rowIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID];
							LRRSiteComponentPropertyValues propValues = lscp.Values;
							if(propValues != null)
							{
								foreach(LRRSiteComponentPropertyValue pv in propValues)
								{
									if(pv.ModeID == modelEquipSec.ModeID)
									{
										if(!pv.IsOverrided)
										{
											if(modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] != null)
											{
												pv.PropertyValue = modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex].ToString();
											}
											else
											{
												pv.PropertyValue = "";
											}
										}
									}
								}
							}

							//actual
							/*valueToWrite = lscp.GetValueForPropertyMode(modelEquipSec.ModeID);
										if(lscp != null)
										{
											int colIndex = i;
											int rowIndex = (int)modelEquipSec.EquipmentModelData.PropertyIDIndex[lscp.PropertyID];
											modelEquipSec.EquipmentModelData.Data[rowIndex,colIndex] = valueToWrite;
										}*/
						}
					}

				}
				i++;
			}

						
		}


		private void ExecuteInternalReadGroupedProperties(LRRModelEngineGroupedPropSection sec,object[,] dataArray)
		{
			if(sec.Direction == "I" && sec.Orientation == "C")
			{
				//string[,] dataArray = _symExcelWrapper.GetDataFromNamedRange(model.WorkBookName,sec.NamedRange);
				for(int i=0;i<sec.Columns.Count;i++)
				{   
					string siteComponentID = "";
					siteComponentID =  sec.Columns[i].ToString();
					if(siteComponentID != "")
					{
							
						LRRSiteComponent lsc =  LRRModel.SiteComponentsObj.GetSiteComponentByID(siteComponentID);
						if(lsc != null)//Added NULL Check by NidhiG
						{
							for(int k = 0;k<sec.Rows.Count;k++)
							{
								string constituentID = sec.Rows[k].ToString();

								LRRSiteComponentProperties lscprops = lsc.Properties.FilterPropertiesByPropertyType(sec.GroupPropertyID);
            
								if(lscprops.Count > 0)
								{
									LRRSiteComponentProperty lscp =  lscprops[0];

									try
									{
										string siteCompID = lscp.ID;
										string siteGroupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(lscp.ID,constituentID); //Argon

										//LRRSiteComponentGroupProperties lscgp = lscp.GroupPpties.FilterGroupPropertiesForLkUpQualifierID(sec.LookupQualifierID);
										//LRRSiteComponentGroupProperty gp = lscgp.GetSiteCompGroupPropFromID(constituentID);
										if(siteGroupPropID != null && siteGroupPropID != "")
										{
											string val;// = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode);
											//string val2 = val;
											if(dataArray[k,i] != null)
											{
												val = dataArray[k,i].ToString();
											}
											else 
											{
												val = "";
											}
											LRRModel.SiteGroupPropertyQualifierValuesObj.SetQualifierValue(siteGroupPropID,sec.LookupQualifierID,sec.Mode,val);

											//Check for nulll....
											//string val = gp.GroupPptiesQualifierValues.GetQualifierValueForPropertyMode(sec.Mode);
										}
										else
										{
											int gg = 900;
										}
									}
									catch(Exception ex)
									{
										Logger.LogMessage(typeof(ExecutionEngine),ex.Message  ,Logger.Level.Debug);
									}
								}

							}
						}
					}
						
				}
			}
		}

		/// <summary>
		/// Internal Read for Data arrays
		/// </summary>
		/// <param name="sec">Object Array</param>
		/// <param name="dataArray">LRRModelEngineDataArrayPropSection</param>
		private void ExecuteInternalReadDataArrays(LRRModelEngineDataArrayPropSection sec,object[,] dataArray)
		{
				if(sec.Direction == "I" && sec.Orientation == "C")
				{
					LRRSiteComponents lrrSiteComps = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(sec.ComponentTypeID);
					int k = 0;
					foreach(LRRSiteComponent lrrSComp in lrrSiteComps)
					{
						foreach(LRRSiteComponentProperty lrrsitecompProp in lrrSComp.Properties)
						{
							if(lrrsitecompProp.Type == sec.PropertyTypeID)
							{
								for(int i=0;i<sec.Headers.Count;i++)
								{
									string lookUpPropertiesID = sec.Headers[i].ToString();
									LRRSiteGroupPropertyQualifierValue val = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForModeSiteCompAndLookUpProp(lrrsitecompProp.ID,lookUpPropertiesID,sec.Mode);
									if(val != null) //Added NULL Check by NidhiG
									{
										if(dataArray[i,k] != null)
										{
											//If property value is not overrided
											if(!val.IsOverrided)
											{
												val.PropertyValue = dataArray[i,k].ToString();
											}
										}
										else
										{
											//If property value is not overrided
											if(!val.IsOverrided)
											{
												val.PropertyValue = "";
											}
										}
									}
								}
							}
						}
						k++;
					}
				}
				else if (sec.Direction == "I" && sec.Orientation == "R")
				{
					LRRSiteComponents lrrSiteComps = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(sec.ComponentTypeID);
					int k = 0;
					foreach(LRRSiteComponent lrrSComp in lrrSiteComps)
					{
						foreach(LRRSiteComponentProperty lrrsitecompProp in lrrSComp.Properties)
						{
							if(lrrsitecompProp.Type == sec.PropertyTypeID)
							{
								for(int i=0;i<sec.Headers.Count;i++)
								{
									string lookUpPropertiesID = sec.Headers[i].ToString();
									LRRSiteGroupPropertyQualifierValue val = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForModeSiteCompAndLookUpProp(lrrsitecompProp.ID,lookUpPropertiesID,sec.Mode);
									if(val != null) //Added NULL Check by NidhiG
									{
										if(dataArray[k,i] != null)
										{
											//If property value is not overrided
											if(!val.IsOverrided)
											{
												val.PropertyValue = dataArray[k,i].ToString();
											}
										}
										else
										{
											//If property value is not overrided
											if(!val.IsOverrided)
											{
												val.PropertyValue = "";
											}
										}
									}
								}
							}
						}
						k++;
					}
				}
		}



		public void WriteSingleCellValues()
		{
			try
			{
				LRRExecutionEngineModels models = LRREEModel.EEModel;
				foreach(LRRExecutionEngineModel m in models)
				{
					foreach(CellValueSource cvs in m.InputCellRanges.Values)
					{
						try
						{
							if(cvs.Direction == "I")
							{
								string siteCompID = cvs.SourceItemID;
								string siteCompPropID = cvs.SourcePropertyValueID;
								string mode = cvs.Mode;
								string val ="";

								//Check for Plant Effective Time
								LRRSiteComponent siteComp = null;
								if(siteCompID != null)
									siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompID);
								bool isPET = true;
								LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
								if(siteComp != null)
									isPET = CheckForEffectivePlantTime(siteComp.EffectivePlantTime);
								else
								{
									if(siteCompProp != null) //Added null check by NidhiG
									{
										string compID = siteCompProp.ComponentID;
										siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(compID);
									}
									if(siteComp != null)
										isPET = CheckForEffectivePlantTime(siteComp.EffectivePlantTime);
								}
								if(siteCompProp != null) //Added null check by NidhiG
								{
									if(isPET)
										isPET = CheckForEffectivePlantTime(siteCompProp.EffectivePlantTime);
								}

								LRRSiteComponentProperty lrrscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);

								if(lrrscp == null)
								{
									Symcon.Utils.MessageLogger.Logger.LogMessage("Error in Write Cycle" + cvs.NamedRange ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
									continue;
								}

								LRRSiteComponentPropertyValue lrrscpv = lrrscp.GetValueObjectForPropertyMode(mode);

								if(lrrscpv == null)
								{
									Symcon.Utils.MessageLogger.Logger.LogMessage("Error in Write Cycle" + cvs.NamedRange ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
									continue;
								}

								if(lrrscpv != null && isPET)
									isPET = CheckForEffectivePlantTime(lrrscpv.EffectivePlantTime);

								if(cvs.IsOverridenValueOnly)
								{
								
									//LRRSiteComponentPropertyValue lrrscpv = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID).GetValueObjectForPropertyMode(mode);
									if(lrrscpv == null)
									{
										Symcon.Utils.MessageLogger.Logger.LogMessage("Error in Write Cycle" + cvs.NamedRange ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
										continue;
									}
									if(lrrscpv.IsOverrided)
									{
										if(isPET)
										{
											//val = lrrscpv.ActualOverridedValue;
											val = lrrscpv.ActualPropertyValue;
										}
										else
											val = PET_MODEL_CONSTANT;
									}
									else
									{
										if(isPET)
											val = "";
										else
											val = PET_MODEL_CONSTANT;
									}
								}
								else
								{
									//val = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID).GetActualValueForPropertyMode(mode);/*,val[0,0]);*/
									if(lrrscpv == null) //Added NULL Check by NidhiG
									{
										Symcon.Utils.MessageLogger.Logger.LogMessage("Error in Write Cycle" + cvs.NamedRange ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
										continue;
									}
									if(lrrscpv.ModeID != "2")
									{
										if(lrrscpv.IsOverrided)
										{
											if(isPET)
												val = lrrscpv.ActualOverridedValue;
											else
												val = PET_MODEL_CONSTANT;
										}
										else
										{
											if(isPET)
												val = lrrscpv.ActualPropertyValue;
											else
												val = PET_MODEL_CONSTANT;
										}
									}
									else
									{
										if(isPET)
											val = lrrscpv.ActualPropertyValue;
										else
											val = PET_MODEL_CONSTANT;
									}
								}
								object[,] data = new object[1,1];
								object retValue;
								if(Symcon.Utils.Validator.Validator.CanConvertToDouble(val,out retValue))
								{
									data[0,0] = /*Double.Parse(val)*/ retValue;
								}
								else
								{
									data[0,0] = (object)val;
								}
								if(_app.GetOptions().ReadInputsFromEngine) //Code for Reading the values from the Engine...
								{
									string engineVal = _symExcelWrapper.GetDataFromCell(m.WorkBookName,cvs.NamedRange);
									string currentVal;
									bool toWrite = false;
									if(data[0,0] != null)
									{
										currentVal = data[0,0].ToString();
									}
									else
									{
										currentVal = "";
									}
									if(currentVal == null || currentVal == "")
									{
										if(engineVal != null && engineVal != "")
										{
											data[0,0] = engineVal;
											toWrite = true;
										}
									}
									_symExcelWrapper.PutDataToCell(m.WorkBookName,cvs.NamedRange,data);
									if(toWrite == true)
									{
										ReadInternalSingleCellValues(cvs,engineVal);
									}
								}
								else
								{
									_symExcelWrapper.PutDataToCell(m.WorkBookName,cvs.NamedRange,data);
								}
							}
							else
							{
								continue;
							}
						}
						catch
						{
							Symcon.Utils.MessageLogger.Logger.LogMessage("Error in Write Cycle" + cvs.NamedRange ,Symcon.Utils.MessageLogger.Logger.Level.Debug);

						}

					}
				}
			}
			catch
			{
				Symcon.Utils.MessageLogger.Logger.LogMessage("Error in Write Cycle",Symcon.Utils.MessageLogger.Logger.Level.Debug);
			}
			/*if(System.IO.File.Exists(_symExcelWrapper.ExcelAppPath + @"\" + LRREEModel.ModelConfigs.GetValueForParam("StartupWB") + ".xls"))
			{
				object[] parameters = new object[1];
				parameters[0] = _app.ActiveRunControlSet;
				_symExcelWrapper.ExecuteMacro(LRREEModel.ModelConfigs.GetValueForParam("StartupWB"),LRREEModel.ModelConfigs.GetValueForParam("EntryPoint"),parameters);
			}  */
		}

		public void ExecuteEntryPointMethod()
		{
			if(System.IO.File.Exists(_symExcelWrapper.ExcelAppPath + @"\" + LRREEModel.ModelConfigs.GetValueForParam("StartupWB")))
			{
				object[] parameters = new object[1];
				parameters[0] = _app.ActiveRunControlSet;
				_symExcelWrapper.ExecuteMacro(LRREEModel.ModelConfigs.GetValueForParam("StartupWB"),LRREEModel.ModelConfigs.GetValueForParam("EntryPoint"),parameters);
			}
		}

		public void ReadSingleCellValues()
		{
			try
			{
				LRRExecutionEngineModels models = LRREEModel.EEModel;
				foreach(LRRExecutionEngineModel m in models)
				{
					foreach(CellValueSource cvs in m.OutputCellRanges.Values)
					{
						if(cvs.Direction == "O")
						{
							string siteCompID = cvs.SourceItemID;
							string siteCompPropID = cvs.SourcePropertyValueID;
							string mode = cvs.Mode;
							string data = _symExcelWrapper.GetDataFromCell(m.WorkBookName,cvs.NamedRange);
							
							//Check for Plant Effective Time
							LRRSiteComponent siteComp = null;
							if(siteCompID != null)
								siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompID);
							
							bool isPET = true;
							LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
							if(siteComp != null)
								isPET = CheckForEffectivePlantTime(siteComp.EffectivePlantTime);
							else
							{
								if(siteCompProp == null)
								{
									Symcon.Utils.MessageLogger.Logger.LogMessage("Error in Read Cycle",Symcon.Utils.MessageLogger.Logger.Level.Error);
									continue;
								}
								string compID = siteCompProp.ComponentID;
								siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(compID);
								if(siteComp != null)
									isPET = CheckForEffectivePlantTime(siteComp.EffectivePlantTime);
							}

							if(isPET)
								isPET = CheckForEffectivePlantTime(siteCompProp.EffectivePlantTime);
							LRRSiteComponentPropertyValue propValueObj = siteCompProp.GetValueObjectForPropertyMode(mode);
							
							//If property Value is not overrided
							if(propValueObj != null && !propValueObj.IsOverrided)
							{
								if(isPET)
									isPET = CheckForEffectivePlantTime(propValueObj.EffectivePlantTime);

								if(isPET)
									LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID).WriteValueForPropertyMode(mode,data);/*,val[0,0]);*/
								else
									LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID).WriteValueForPropertyMode(mode,PET_UI_CONSTANT);
							}
						}
						else
						{
							continue;
						}
					}
				}
			}
			catch
			{
				Symcon.Utils.MessageLogger.Logger.LogMessage("Error in Read Cycle",Symcon.Utils.MessageLogger.Logger.Level.Error);
			}
		}

		public void ReadInternalSingleCellValues(CellValueSource cvs,string data)
		{
			if(cvs.Direction == "I")
			{
				string siteCompID = cvs.SourceItemID;
				string siteCompPropID = cvs.SourcePropertyValueID;
				string mode = cvs.Mode;
				LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
				if(siteCompProp != null)
				{
					LRRSiteComponentPropertyValue propValueObj = siteCompProp.GetValueObjectForPropertyMode(mode);
					if(propValueObj != null)
					{
						if(!propValueObj.IsOverrided)
						{
							LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID).WriteValueForPropertyMode(mode,data);/*,val[0,0]);*/
						}
					}
				}
			}
		}

		private bool CheckForEffectivePlantTime(DateTime effectivePlantTime)
		{
			DateTime startTime = _app.ActiveRunControlSet.DataWindowObj.StartTime;
			TimeSpan ts = startTime.Subtract(effectivePlantTime);
			if(ts.Ticks > 0)
				return true;
			else
				return false;
		}
		
		private SymExcelWrapper _symExcelWrapper;
		private LRRApp			_app;

		#region Constants
		private const string PET_UI_CONSTANT = "N/E";
		private const string PET_MODEL_CONSTANT = "";

		#endregion
	}
}
