using System;
using System.Data;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.Utils.Validator;
using Symcon.LightRidge.BusinessObjects.Utilities;
using Symcon.Utils.HeaderDispatcher; 
using Symcon.LightRidge.PEAdvisor.GridUtilities;
namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for UOMTypeAnalysis.
	/// </summary>
	public class UOMTypeAnalysis : System.Windows.Forms.Form
	{
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridUOMTypes;
		private System.Windows.Forms.Button btnApply;
		private System.Windows.Forms.Button btnCancel;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Panel pnlGrid;
		private System.Windows.Forms.Splitter splitter1;

		#region Private Fields
		private DataTable _dtUOMTypeTable = new DataTable();
		private Hashtable _htSiteComponentPropsToUpdate = new Hashtable();
		private LRRApp _lrrApp;
		#endregion

		public UOMTypeAnalysis()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			BindGrid();
			
		}
		private void BindGrid()
		{
			string msg = "Started... \r\n";
			Symcon.Utils.MessageLogger.Logger.LogMessage("UOM Type Analysis : " + msg ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
						
			_dtUOMTypeTable.Columns.Add(new DataColumn("Master Property [Master Component]", typeof(string)));
			_dtUOMTypeTable.Columns.Add(new DataColumn("Master UOM Type", typeof(string)));
			_dtUOMTypeTable.Columns.Add(new DataColumn("Plant Item Attribute [Plant Item] ", typeof(string)));
			_dtUOMTypeTable.Columns.Add(new DataColumn("Plant UOM Type", typeof(string)));
			
			//Step 1: get the UOM type of Master Properties.
			//Step 2: Compare the UOM type of Site Component Properties.
			//Step 3: Log the mismatch.
			LRRMasterProperties lrrMasterProps = LRRModel.MasterComponentPropertiesObj;
			foreach(LRRMasterProperty masterProp in lrrMasterProps)
			{
				string sUOMType = masterProp.MasterPropertyUOMTypeID == null ? "" : masterProp.MasterPropertyUOMTypeID;
				string sMasterPropertyID = masterProp.PropID == null ? "" : masterProp.PropID;
				LRRSiteComponentProperties lrrSiteProps = LRRModel.SiteComponentPropertiesObj.FilterPropertiesByPropertyID(sMasterPropertyID);
				foreach(LRRSiteComponentProperty siteProp in lrrSiteProps)
				{
					string sSiteUOMType = siteProp.UOMTypeID == null ? "" : siteProp.UOMTypeID;
					if(sSiteUOMType != sUOMType)
					{
						DataRow drDetailsRow = _dtUOMTypeTable.NewRow();
						string sMasterComponent = LRRModel.Model.GetMasterComponentByID(masterProp.ComponentID).ComponentName;
						string sMasterProperty = masterProp.Name;

						string sMasterUOMTypeName = sUOMType == "" ? "" : LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(sUOMType).Name;
						string sSiteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteProp.ComponentID).Name;
						string sSiteProperty = siteProp.Name;
						string sSiteUOMTypeName = sSiteUOMType == "" ? "" : LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(sSiteUOMType).Name;

						drDetailsRow[0] = sMasterProperty + " [ " + sMasterComponent + "]";
						drDetailsRow[1] = sMasterUOMTypeName;
						drDetailsRow[2] = sSiteProperty + " [" + sSiteComponent + "]";
						drDetailsRow[3] = sSiteUOMTypeName;

						_dtUOMTypeTable.Rows.Add(drDetailsRow);
						msg = "Master Property : " + sMasterProperty  + " [" + sMasterComponent  + "]; UOM type: " + sMasterUOMTypeName + " mismatches UOM type: " + sSiteUOMTypeName + "; for plant item attribute:  " + sSiteProperty +  " [" + sSiteComponent  + "] \r\n";
						Symcon.Utils.MessageLogger.Logger.LogMessage("UOM Type Analysis : " + msg ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
						if(!_htSiteComponentPropsToUpdate.Contains(siteProp.ID))
							_htSiteComponentPropsToUpdate.Add(siteProp.ID,sUOMType);	
					}
				}
			}
		
			gridUOMTypes.DataSource = _dtUOMTypeTable;
		
			gridUOMTypes.Engine.RecordAsDisplayElements = true;

			SyncGridUtilities.InitializeGridLookAndFeel(gridUOMTypes);

			msg = "Completed. \r\n";
			Symcon.Utils.MessageLogger.Logger.LogMessage("UOM Type Analysis : " + msg ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
			
		}
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(UOMTypeAnalysis));
			this.gridUOMTypes = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			this.btnApply = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.panel1 = new System.Windows.Forms.Panel();
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.pnlGrid = new System.Windows.Forms.Panel();
			((System.ComponentModel.ISupportInitialize)(this.gridUOMTypes)).BeginInit();
			this.panel1.SuspendLayout();
			this.pnlGrid.SuspendLayout();
			this.SuspendLayout();
			// 
			// gridUOMTypes
			// 
			this.gridUOMTypes.BackColor = System.Drawing.SystemColors.Window;
			this.gridUOMTypes.Dock = System.Windows.Forms.DockStyle.Fill;
			this.gridUOMTypes.Location = new System.Drawing.Point(0, 0);
			this.gridUOMTypes.Name = "gridUOMTypes";
			this.gridUOMTypes.Size = new System.Drawing.Size(592, 392);
			this.gridUOMTypes.TabIndex = 1;
			this.gridUOMTypes.TableOptions.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Any | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.MixRangeType)));
			this.gridUOMTypes.Text = "gridGroupingControl1";
			this.gridUOMTypes.TopLevelGroupOptions.CaptionText = "UOM Types Analysis: {RecordCount} Items";
			this.gridUOMTypes.TopLevelGroupOptions.IsExpandedInitialValue = true;
			this.gridUOMTypes.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;
			this.gridUOMTypes.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
			this.gridUOMTypes.TopLevelGroupOptions.ShowCaptionPlusMinus = false;
			this.gridUOMTypes.TopLevelGroupOptions.ShowColumnHeaders = true;
			this.gridUOMTypes.TopLevelGroupOptions.ShowFilterBar = true;
			this.gridUOMTypes.VersionInfo = "3.301.0.0";
			// 
			// btnApply
			// 
			this.btnApply.Location = new System.Drawing.Point(264, 400);
			this.btnApply.Name = "btnApply";
			this.btnApply.Size = new System.Drawing.Size(232, 32);
			this.btnApply.TabIndex = 2;
			this.btnApply.Text = "&Apply Master UOM Type to Plant Item(s)";
			this.btnApply.Click += new System.EventHandler(this.btnApply_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.Location = new System.Drawing.Point(504, 400);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(75, 32);
			this.btnCancel.TabIndex = 3;
			this.btnCancel.Text = "&Cancel";
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// panel1
			// 
			this.panel1.Controls.Add(this.splitter1);
			this.panel1.Controls.Add(this.pnlGrid);
			this.panel1.Controls.Add(this.btnApply);
			this.panel1.Controls.Add(this.btnCancel);
			this.panel1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panel1.Location = new System.Drawing.Point(0, 0);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(592, 446);
			this.panel1.TabIndex = 4;
			// 
			// splitter1
			// 
			this.splitter1.Dock = System.Windows.Forms.DockStyle.Top;
			this.splitter1.Location = new System.Drawing.Point(0, 392);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(592, 2);
			this.splitter1.TabIndex = 5;
			this.splitter1.TabStop = false;
			// 
			// pnlGrid
			// 
			this.pnlGrid.Controls.Add(this.gridUOMTypes);
			this.pnlGrid.Dock = System.Windows.Forms.DockStyle.Top;
			this.pnlGrid.Location = new System.Drawing.Point(0, 0);
			this.pnlGrid.Name = "pnlGrid";
			this.pnlGrid.Size = new System.Drawing.Size(592, 392);
			this.pnlGrid.TabIndex = 4;
			// 
			// UOMTypeAnalysis
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.AutoScroll = true;
			this.ClientSize = new System.Drawing.Size(592, 446);
			this.Controls.Add(this.panel1);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "UOMTypeAnalysis";
			this.Text = "UOM Type Analysis";
			this.Load += new System.EventHandler(this.UOMTypeAnalysis_Load);
			((System.ComponentModel.ISupportInitialize)(this.gridUOMTypes)).EndInit();
			this.panel1.ResumeLayout(false);
			this.pnlGrid.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void btnCancel_Click(object sender, System.EventArgs e)
		{
		
		}

		private void btnApply_Click(object sender, System.EventArgs e)
		{
			LRRMessagBox.ShowMessage("SID_UOMTYPE_UPDATION_STARTED","SID_LABEL",MessageBoxButtons.OK);
			foreach(DictionaryEntry	entry in _htSiteComponentPropsToUpdate)
			{
				string sSiteComponentID = (string)entry.Key;
				string sUOMTypeID = (string)entry.Value;
				LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(sSiteComponentID).UOMTypeID = sUOMTypeID;
			}
			if(LRRModel.SiteComponentPropertiesObj.GetChanges().Count >0)
			{
				_lrrApp.DBManagerObj.CommitCollection(LRRModel.SiteComponentPropertiesObj.GetChanges());
			}
			LRRMessagBox.ShowMessage("SID_UOMTYPE_UPDATION_COMPLETED","SID_LABEL",MessageBoxButtons.OK);
		}

		private void UOMTypeAnalysis_Load(object sender, System.EventArgs e)
		{
			if(_htSiteComponentPropsToUpdate.Count == 0)
			{
				LRRMessagBox.ShowMessage("SID_NO_MISMATCH","SID_LABEL",MessageBoxButtons.OK);
				this.Close();
			}
		}
	}
}
