using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.BusinessObjects; 

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for FinancialOverview.
	/// </summary>
	public class FinancialOverview : System.Windows.Forms.Form,ILRRUIContainer
	{
		private System.ComponentModel.IContainer components;

		public FinancialOverview(string miscScreenId)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			LRRChart chart = new LRRChart();
			_lrrApp.DBManagerObj.PopulateObject(chart,typeof(LRRChart), " where ID='" + miscScreenId + "' and DataType='TabularData'");
			if(chart.WorkBookName!=null)
			{
				_workbook = chart.WorkBookName;
				_lrrApp.SymExcelWrapperObj.OpenWorkBook(_workbook);
				_nameRange = chart.ChartName;
				this.Text = chart.DisplayName;
				try
				{
					string[] srgb = chart.SheetName.Split(new char[] {','});
					if(srgb.Length == 3)
					{
						this.BackColor = Color.FromArgb(255,int.Parse(srgb[0].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(),Symcon.BusinessObjects.SymConverter.Culture),int.Parse(srgb[1].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(),Symcon.BusinessObjects.SymConverter.Culture),int.Parse(srgb[2].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(),Symcon.BusinessObjects.SymConverter.Culture));
					}
					else
					{
						throw new Exception("Invalid number of colors in rgb string, should be 3.");
					}
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(FinancialOverview),"Invalid color string in db(table: Charts, field:SheetName, ID:" + chart.ID + "):" + ex.Message,Logger.Level.Debug);
				}
				RefreshImage();
				pictureBox1.Height = _img.Height;
				pictureBox1.Width = _img.Width;
			}
			else
			{
				throw new Exception("Cannot populate chart object");
			}
			pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;
			pictureBox1.Dock = DockStyle.None;
			this.Resize += new EventHandler(FinancialOverview_SizeChanged);
//			if(!ApplicationPersistanceUtil.DisablePersistance)
//				pictureBox1.SizeChanged += new EventHandler(pictureBox1_SizeChanged);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.pictureBox1 = new System.Windows.Forms.PictureBox();
			this.panelZoom = new System.Windows.Forms.Panel();
			this.labelZoom = new System.Windows.Forms.Label();
			this.comboBoxZoom = new System.Windows.Forms.ComboBox();
			this.panelZoomContainer = new System.Windows.Forms.Panel();
			this.buttonZoom = new System.Windows.Forms.Button();
			this.panelImage = new System.Windows.Forms.Panel();
			this.panelZoom.SuspendLayout();
			this.panelZoomContainer.SuspendLayout();
			this.panelImage.SuspendLayout();
			this.SuspendLayout();
			// 
			// pictureBox1
			// 
			this.pictureBox1.Location = new System.Drawing.Point(104, 40);
			this.pictureBox1.Name = "pictureBox1";
			this.pictureBox1.Size = new System.Drawing.Size(432, 216);
			this.pictureBox1.TabIndex = 0;
			this.pictureBox1.TabStop = false;
			// 
			// panelZoom
			// 
			this.panelZoom.Controls.Add(this.labelZoom);
			this.panelZoom.Controls.Add(this.comboBoxZoom);
			this.panelZoom.Location = new System.Drawing.Point(0, 8);
			this.panelZoom.Name = "panelZoom";
			this.panelZoom.Size = new System.Drawing.Size(120, 24);
			this.panelZoom.TabIndex = 4;
			// 
			// labelZoom
			// 
			this.labelZoom.Location = new System.Drawing.Point(8, 0);
			this.labelZoom.Name = "labelZoom";
			this.labelZoom.Size = new System.Drawing.Size(40, 16);
			this.labelZoom.TabIndex = 1;
			this.labelZoom.Tag = "SID_ZOOM";
			// 
			// comboBoxZoom
			// 
			this.comboBoxZoom.Items.AddRange(new object[] {
															  "25%",
															  "50%",
															  "75%",
															  "100%",
															  "150%",
															  "200%",
															  "300%",
															  "400%",
															  "500%"});
			this.comboBoxZoom.Location = new System.Drawing.Point(56, 0);
			this.comboBoxZoom.Name = "comboBoxZoom";
			this.comboBoxZoom.Size = new System.Drawing.Size(56, 21);
			this.comboBoxZoom.TabIndex = 0;
			this.comboBoxZoom.Text = "100%";
			this.comboBoxZoom.SelectedValueChanged += new System.EventHandler(this.buttonZoom_Click);
			// 
			// panelZoomContainer
			// 
			this.panelZoomContainer.Controls.Add(this.buttonZoom);
			this.panelZoomContainer.Controls.Add(this.panelZoom);
			this.panelZoomContainer.Dock = System.Windows.Forms.DockStyle.Top;
			this.panelZoomContainer.Location = new System.Drawing.Point(0, 0);
			this.panelZoomContainer.Name = "panelZoomContainer";
			this.panelZoomContainer.Size = new System.Drawing.Size(752, 40);
			this.panelZoomContainer.TabIndex = 5;
			// 
			// buttonZoom
			// 
			this.buttonZoom.Location = new System.Drawing.Point(168, 48);
			this.buttonZoom.Name = "buttonZoom";
			this.buttonZoom.TabIndex = 5;
			this.buttonZoom.Text = "Zoom";
			this.buttonZoom.Click += new System.EventHandler(this.buttonZoom_Click);
			// 
			// panelImage
			// 
			this.panelImage.AutoScroll = true;
			this.panelImage.BackColor = System.Drawing.SystemColors.ActiveCaptionText;
			this.panelImage.Controls.Add(this.pictureBox1);
			this.panelImage.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panelImage.Location = new System.Drawing.Point(0, 40);
			this.panelImage.Name = "panelImage";
			this.panelImage.Size = new System.Drawing.Size(752, 566);
			this.panelImage.TabIndex = 6;
			// 
			// FinancialOverview
			// 
			this.AcceptButton = this.buttonZoom;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(752, 606);
			this.Controls.Add(this.panelImage);
			this.Controls.Add(this.panelZoomContainer);
			this.Name = "FinancialOverview";
			this.Tag = "SID_FINANCIALOVERVIEW";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.Closing += new System.ComponentModel.CancelEventHandler(this.FinancialOverview_Closing);
			this.Load += new System.EventHandler(this.FinancialOverview_Load);
			this.panelZoom.ResumeLayout(false);
			this.panelZoomContainer.ResumeLayout(false);
			this.panelImage.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region Fields

		private LRRApp	_lrrApp;
		private string _workbook = null;
		private System.Windows.Forms.PictureBox pictureBox1;
		private string _nameRange = null;
		private int _width = -1;
		private int _height = -1;
		private Image _img;
		private TabPage _parentTabPage;
		private System.Windows.Forms.Panel panelZoom;
		private System.Windows.Forms.ComboBox comboBoxZoom;
		private System.Windows.Forms.Label labelZoom;
		private System.Windows.Forms.Panel panelZoomContainer;
		private System.Windows.Forms.Panel panelImage;
		private System.Windows.Forms.Button buttonZoom;
		private TabControl _parentTabControl;
		private int _lastZoom = 100;
		private float _initZoom = 1.0f;
		
		#endregion

		private void FinancialOverview_Load(object sender, System.EventArgs e)
		{
			_initZoom = Math.Min((this.Width - SystemInformation.VerticalScrollBarWidth)/(float)_img.Width ,((this.Height - panelZoomContainer.Height - SystemInformation.HorizontalScrollBarHeight)/(float)_img.Height));
			_width = pictureBox1.Width;
			_height = pictureBox1.Height;
			int nZoom = (int)((_width*100.0)/(_img.Width) + 0.5);
			if(nZoom == 100)
			{
				nZoom = (int)((_height*100.0)/(_img.Height) + 0.5);
				_width = ((_width * nZoom)/100);
			}
			else
			{
				_height = ((_height * nZoom)/100);
			}
			_width = (int)(_width *_initZoom);
			_height = (int)(_height* _initZoom);
			ShowImage();
			comboBoxZoom.Text = nZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
			_lastZoom = nZoom;
			_lrrApp.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);
		}

		#region Public Methods

		public void RefreshImage()
		{
			try
			{
				_img = _lrrApp.SymExcelWrapperObj.GetDataFromNamedRangeAsImage(_workbook,_nameRange);
				pictureBox1.Image = _img;
			}
			catch(Exception ex)
			{
			//	System.Diagnostics.Debug.Assert(false,ex.Message,ex.StackTrace);
				Logger.LogMessage(typeof(FinancialOverview),"Could not load image:" + ex.Message ,Logger.Level.Debug);
			}
		}

		public void ShowImage()
		{
			if(_width == -1)
				_width = (int)(_img.Width * _initZoom);

			if(_height == -1)
				_height = (int)(_img.Height * _initZoom);
			
			
			pictureBox1.Height = _height;
			pictureBox1.Width = _width;
			pictureBox1.Location = new Point(Math.Max((this.Width - _width)/2,0),Math.Max((this.panelImage.Height - _height - SystemInformation.HorizontalScrollBarHeight)/2,0));
		}
		#endregion

		#region ILRRUIContainer Members

		public void Refresh()
		{
			// TODO:  Add FinancialOverview.Refresh implementation
			ShowImage();
		}

		public void Save()
		{
			// TODO:  Add FinancialOverview.Save implementation
		}

		public void CopyMasterProperties()
		{
			// TODO:  Add FinancialOverview.CopyMasterProperties implementation
		}

		public void Paste()
		{
			// TODO:  Add FinancialOverview.Paste implementation
		}

		public void Copy()
		{
			// TODO:  Add FinancialOverview.Copy implementation
		}

		public void Print()
		{
			// TODO:  Add FinancialOverview.Print implementation
		}

		public void PrintPreview()
		{
			// TODO:  Add FinancialOverview.PrintPreview implementation
		}

		public void Delete()
		{
			// TODO:  Add FinancialOverview.Delete implementation
		}

		public void ArrangeColumns()
		{
			// TODO:  Add FinancialOverview.ArrangeColumns implementation
		}

		public void ClearFilter()
		{
			// TODO:  Add FinancialOverview.ClearFilter implementation
		}

		public void InsertNewRows()
		{
			// TODO:  Add FinancialOverview.InsertNewRows implementation
		}

		public void ImportToExcel()
		{
			// TODO:  Add FinancialOverview.ImportToExcel implementation
		}

		public void Export()
		{
			// TODO:  Add FinancialOverview.Export implementation
		}

		public void SelectionPaste()
		{
			// TODO:  Add FinancialOverview.SelectionPaste implementation
		}

		public void RightLevel()
		{
			// TODO:  Add FinancialOverview.RightLevel implementation
		}

		public void LeftLevel()
		{
			// TODO:  Add FinancialOverview.LeftLevel implementation
		}

		public void UpLevel()
		{
			// TODO:  Add FinancialOverview.UpLevel implementation
		}

		public void DownLevel()
		{
			// TODO:  Add FinancialOverview.DownLevel implementation
		}

		public void ClearSort()
		{
			// TODO:  Add FinancialOverview.ClearSort implementation
		}

		public void CopyUOMTypes()
		{
			// TODO:  Add FinancialOverview.CopyUOMTypes implementation
		}

		public void ShowPropertyList()
		{
			// TODO:  Add FinancialOverview.ShowPropertyList implementation
		}

		public void ShowComponentList()
		{
			// TODO:  Add FinancialOverview.ShowComponentList implementation
		}

		public bool SupportsMasterPropertiesCopy()
		{
			// TODO:  Add FinancialOverview.SupportsMasterPropertiesCopy implementation
			return false;
		}

		public bool SupportsCopy()
		{
			// TODO:  Add FinancialOverview.SupportsCopy implementation
			return false;
		}

		public bool SupportsPaste()
		{
			// TODO:  Add FinancialOverview.SupportsPaste implementation
			return false;
		}

		public bool SupportsExport()
		{
			// TODO:  Add FinancialOverview.SupportsExport implementation
			return false;
		}

		public bool SupportsSave()
		{
			// TODO:  Add FinancialOverview.SupportsSave implementation
			return false;
		}

		public bool SupportsRefresh()
		{
			// TODO:  Add FinancialOverview.SupportsRefresh implementation
			return true;
		}

		public bool SupportsDelete()
		{
			// TODO:  Add FinancialOverview.SupportsDelete implementation
			return false;
		}

		public bool SupportsPrint()
		{
			// TODO:  Add FinancialOverview.SupportsPrint implementation
			return false;
		}

		public bool SupportsArrangeColumns()
		{
			// TODO:  Add FinancialOverview.SupportsArrangeColumns implementation
			return false;
		}

		public bool SupportsClearFilter()
		{
			// TODO:  Add FinancialOverview.SupportsClearFilter implementation
			return false;
		}

		public bool SupportsInsertNewRows()
		{
			// TODO:  Add FinancialOverview.SupportsInsertNewRows implementation
			return false;
		}

		public bool SupportsImportToExcel()
		{
			// TODO:  Add FinancialOverview.SupportsImportToExcel implementation
			return false;
		}

		public bool SupportsSelectionPaste()
		{
			// TODO:  Add FinancialOverview.SupportsSelectionPaste implementation
			return false;
		}

		public bool SupportsLevel()
		{
			// TODO:  Add FinancialOverview.SupportsLevel implementation
			return false;
		}

		public bool SupportsClearSort()
		{
			// TODO:  Add FinancialOverview.SupportsClearSort implementation
			return false;
		}

		public bool SupportsCopyUOMTypes()
		{
			// TODO:  Add FinancialOverview.SupportsCopyUOMTypes implementation
			return false;
		}

		public bool SupportsResultsFilter()
		{
			// TODO:  Add FinancialOverview.SupportsResultsFilter implementation
			return false;
		}

		public TabControl ParentTabControl
		{
			get
			{
				// TODO:  Add FinancialOverview.ParentTabControl getter implementation
				return _parentTabControl;
			}
			set
			{
				// TODO:  Add FinancialOverview.ParentTabControl setter implementation
				_parentTabControl = value;
			}
		}

		public TabPage ParentTabPage
		{
			get
			{
				// TODO:  Add FinancialOverview.ParentTabPage getter implementation
				return _parentTabPage;
			}
			set
			{
				// TODO:  Add FinancialOverview.ParentTabPage setter implementation
				_parentTabPage = value;
			}
		}
		public 	void LeftColMovement()
		{
		}
		public void RightColMovement()
		{
		}
		#endregion

		private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
		{
			RefreshImage();
		}

		private void FinancialOverview_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			_lrrApp.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
		}

		private void FinancialOverview_SizeChanged(object sender, EventArgs e)
		{
			ShowImage();
		}

//		private void pictureBox1_SizeChanged(object sender, EventArgs e)
//		{
//			pictureBox1.SizeChanged -= new EventHandler(pictureBox1_SizeChanged);
////			_initZoom = Math.Min((this.Width/(float)_img.Width),((this.Height - panelZoomContainer.Height)/(float)_img.Height));
//			_width = (int)(pictureBox1.Width * _initZoom);
//			_height = (int)(pictureBox1.Height * _initZoom);
//			int nZoom = (int)((_width*100.0)/(_img.Width * _initZoom * _initZoom) + 0.5);
//			if(nZoom == 100)
//			{
//				nZoom = (int)((_height*100.0)/_img.Height + 0.5);
//				_width = (int)((_width * nZoom * _initZoom)/100);
//			}
//			else
//			{
//				_height = (int)((_height * nZoom *_initZoom)/100);
//			}
//			ShowImage();
//			comboBoxZoom.Text = nZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
//			_lastZoom = nZoom;
//			
//		}
		#region ILRRUIContainer Members

		public void PageSetup()
		{
			GridGroupingPrint ggPrint = new GridGroupingPrint();
			ggPrint.PageSetUp();
		}

		public void Undo()
		{
			// TODO:  Add FinancialOverview.Undo implementation
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add FinancialOverview.SupportsPageSetup implementation
			return true;
		}

		public bool SupportsUndo()
		{
			// TODO:  Add FinancialOverview.SupportsUndo implementation
			return false;
		}

		#endregion


		private void buttonZoom_Click(object sender, System.EventArgs e)
		{
			try
			{
				double dblZoom = double.Parse(comboBoxZoom.Text.Replace("%","").Trim(),Symcon.BusinessObjects.SymConverter.Culture);
				int nZoom = (int)dblZoom;
				comboBoxZoom.Text = nZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
				_width = (int)((_img.Width * nZoom *_initZoom)/100);
				_height = (int)((_img.Height * nZoom * _initZoom)/100);
				_lastZoom = nZoom;
				ShowImage();
				comboBoxZoom.SelectAll();
			}
			catch(Exception ex)
			{
				comboBoxZoom.Text = _lastZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
				Logger.LogMessage(typeof(FinancialOverview),"buttonZoom_Click Event:" + ex.Message,Logger.Level.Debug);
			}
		}
		#region ILRRUIContainer Members

		public void OpenCompositeView()
		{
			// TODO:  Add FinancialOverview.OpenCompositeView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add FinancialOverview.TransposeView implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add FinancialOverview.OpenQuery implementation
		}

		public void OpenView()
		{
			// TODO:  Add FinancialOverview.OpenView implementation
		}

		public bool SupportsOpenCompositeView()
		{
			// TODO:  Add FinancialOverview.SupportsOpenCompositeView implementation
			return false;
		}

		public bool SupportsTransposeView()
		{
			// TODO:  Add FinancialOverview.SupportsTransposeView implementation
			return false;
		}

		public bool SupportsOpenQuery()
		{
			// TODO:  Add FinancialOverview.SupportsOpenQuery implementation
			return false;
		}

		public bool SupportsOpenView()
		{
			// TODO:  Add FinancialOverview.SupportsOpenView implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public bool SupportsCompositeView()
		{
			// TODO:  Add FinancialOverview.SupportsCompositeView implementation
			return false;
		}

		#endregion

		#region ILRRUIContainer Members

		public bool PerformDirtyChk
		{
			get
			{
				// TODO:  Add FinancialOverview.PerformDirtyChk getter implementation
				return true;
			}
		}

		#endregion
	}

//	public class LRRImagePanel: Panel
//	{
//		public Image img;
//		public float fZoom = 1.0f;
//		
////		protected override void OnPaint(PaintEventArgs e)
////		{
////			Graphics g = e.Graphics;
////			int destWidth = (int)(img.Width * nZoom);
////			int destHeight = (int)(img.Height * nZoom);
////			g.DrawImage(img,0,0,destWidth,destHeight);
////			g.ResetTransform();
////			this.Width = destWidth;
////			this.Height = destHeight;
////		}
//
////		public override void Zoom(int nZoom)
////		{
////			Graphics g = this.CreateGraphics();
////
////			fZoom = nZoom/100.0f;
////			g.ScaleTransform(fZoom);
////			g.ResetTransform();
////		}
//	}
}
