using System;
using System.Drawing;
using System.Collections;
using System.Diagnostics;
using System.Reflection;
using System.ComponentModel;
using System.Windows.Forms;

using Symcon.Utils.MessageLogger;
using Symcon.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.PEAdvisor.Global;
using Symcon.Utils.HeaderDispatcher;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for RelationalColumnConditions.
	/// </summary>
	public class RelationalColumnConditions
	{
		public RelationalColumnConditions()
		{
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			if(_lrrProjConfigs != null)
				_lrrProjConfigs.Clear();
			_lrrProjConfigs = new LRRProjectConfigurations();
			_app.DBManagerObj.PopulateCollection(_lrrProjConfigs,false);
			if(_lrrMeasSystemDefs != null)
				_lrrMeasSystemDefs.Clear();
			_lrrMeasSystemDefs = new LRRMeasurementSystemDefinitions();
			_app.DBManagerObj.PopulateCollection(_lrrMeasSystemDefs,false);


		}

		#region Fields

		private LRRProjectConfigurations _lrrProjConfigs; 
		private LRRMeasurementSystemDefinitions	 _lrrMeasSystemDefs;
		private LRRApp							_app;

		#endregion

        /// <summary>
        /// Conditions the type of the apply for filter UO mbased on uom.
        /// </summary>
        /// <param name="recordObject">The record object.</param>
        /// <param name="newValue">The new value.</param>
        /// <param name="oldValue">The old value.</param>
        /// <param name="propName">Name of the prop.</param>
		public void ConditionApplyForFilterUOMbasedOnUomType(object recordObject,string newValue,string oldValue,string propName)
		{
			if(propName == "UOMTypeID")
			{
				LRRProjectConfiguration lrrProjConfig = _lrrProjConfigs[0] as LRRProjectConfiguration;
				string modelUOMID = _lrrMeasSystemDefs.GetLocaleUOM(lrrProjConfig.MeasurementSystemCEID,newValue);
				string uiUOMID = _lrrMeasSystemDefs.GetLocaleUOM(lrrProjConfig.MeasurementSystemUserInterfaceID,newValue);
				if(modelUOMID != null && modelUOMID.Trim() != "")
				{
					recordObject.GetType().GetProperty("ModelUOM").SetValue(recordObject,modelUOMID,null);
				}
				else
				{
					recordObject.GetType().GetProperty("ModelUOM").SetValue(recordObject,"",null);
				}
				recordObject.GetType().GetProperty("InterfaceUnits").SetValue(recordObject,uiUOMID,null);
				recordObject.GetType().GetProperty("HistUnits").SetValue(recordObject,"",null);
				recordObject.GetType().GetProperty("InterfaceValue").SetValue(recordObject,"",null);
			}
			else if(propName == "InterfaceUnits")
			{
				ConvertUOM(recordObject,oldValue,newValue);				
			}
		}

        /// <summary>
        /// Converts the UOM.
        /// </summary>
        /// <param name="recordObj">The record obj.</param>
        /// <param name="oldUom">The old uom.</param>
        /// <param name="newUom">The new uom.</param>
		private void ConvertUOM(object recordObj,string oldUom,string newUom)
		{
			string val = recordObj.GetType().GetProperty("InterfaceValue").GetValue(recordObj,null) == null ? "" : recordObj.GetType().GetProperty("InterfaceValue").GetValue(recordObj,null).ToString();
			string val2 = recordObj.GetType().GetProperty("PostFilterUserOverride").GetValue(recordObj,null) == null ? "" : recordObj.GetType().GetProperty("PostFilterUserOverride").GetValue(recordObj,null).ToString();
			string val3 = recordObj.GetType().GetProperty("InterfaceUnits").GetValue(recordObj,null)== null ? "" : recordObj.GetType().GetProperty("InterfaceUnits").GetValue(recordObj,null).ToString(); 
			recordObj.GetType().GetProperty("InterfaceUnits").SetValue(recordObj,newUom,null);
			if(!(val2 == null || val2 == ""))
			{
				if(val3 != newUom)
				{
					string PostOVFilterVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(val3,newUom,val2,false,null).ToString();
					recordObj.GetType().GetProperty("PostFilterUserOverride").SetValue(recordObj,PostOVFilterVal,null);
				}
			}
			if(val == null || val == "")
			{
                oldUom = recordObj.GetType().GetProperty("UnderlyingInterfaceUnits").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("UnderlyingInterfaceUnits").GetValue(recordObj, null).ToString();///histData.UnderlyingInterfaceUnits;
                val = recordObj.GetType().GetProperty("UnderlyingInterfaceValue").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("UnderlyingInterfaceValue").GetValue(recordObj, null).ToString();
                if (oldUom == newUom)
				{
                    recordObj.GetType().GetProperty("InterfaceValue").SetValue(recordObj, val, null);
					return;
				}
			}
			if((val != null && val != "")
				&& (oldUom != null && oldUom != "")
				&& (newUom != null && newUom != ""))
				//&& )
			{
				if(oldUom != newUom)
				{
                    string newVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(oldUom, newUom, val, null);
					recordObj.GetType().GetProperty("DisplayInterfaceValue").SetValue(recordObj,newVal,null);
				}
				else
				{
					string displayInterfaceValue = recordObj.GetType().GetProperty("InterfaceValue").GetValue(recordObj,null)==null ? "" : recordObj.GetType().GetProperty("InterfaceValue").GetValue(recordObj,null).ToString(); 
					recordObj.GetType().GetProperty("DisplayInterfaceValue").SetValue(recordObj,displayInterfaceValue,null);
				}
			}
		}
	}
}
