using System;
using System.Drawing;
using System.Collections;
using System.Diagnostics;
using System.Reflection;
using System.ComponentModel;
using System.Windows.Forms;

using Symcon.Utils.MessageLogger;
using Symcon.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.PEAdvisor.Global;
using Symcon.Utils.HeaderDispatcher;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;


namespace Symcon.LightRidge.PEAdvisor
{
    /// <summary>
    /// Summary description for TransposedView.
    /// </summary>
    public class TransPosedView : System.Windows.Forms.Form
    {
        private Syncfusion.Windows.Forms.Grid.GridControl gridControl1;
        private System.Windows.Forms.Panel panel1;
        private System.Windows.Forms.Panel panel2;
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;

        /// <summary>
        /// Initializes a new instance of the <see cref="TransPosedView"/> class.
        /// </summary>
        /// <param name="cc">The cc.</param>
        /// <param name="grid">The grid.</param>
        /// <param name="bAddNewRecord">if set to <c>true</c> [b add new record].</param>
        public TransPosedView(CustomCollection cc, GridGroupingControl grid, bool bAddNewRecord)
        {
            if (cc != null)
            {
                InitializeComponent();
                _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
                InternationalizationUtils.GetControlText(this, null);
                cCollection = cc;
                gridGroupControl = grid;
                lrrUoms = LRRModel.ChildModel.UOMs;
                GridRangeInfoList list = grid.TableModel.SelectedRanges;
                props = cCollection.ItemType.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
                //CreateGrid 
                if (bAddNewRecord == true)
                    createGrid(getGridRowCount(props), 1);
                else
                    createGrid(getGridRowCount(props), getGridColCount(list));
                gridControl1.CellModels.Add("EllipsisText", new EllipsisCellModel(gridControl1.Model));
                gridControl1.CellModels.Add("DropDownTree", new PlantLevelUserFormTreeControl(gridControl1.Model, new PlantLevelTreeControlStructure()));
                //Initialize Grid with colomnHeader and Values.
                initializeGrid(props, grid, bAddNewRecord);
                initializeReadOnlyProperties(grid,bAddNewRecord);
                initDataSource();
                gridControl1.CellModels.Model.SaveCellInfo += new GridSaveCellInfoEventHandler(Model_SaveCellInfo);
                

                
                
                
            }
            else
            {
                MessageBox.Show("Atleast Select one record for edit");
            }
        }

        /// <summary>
        /// Inits the data source.
        /// </summary>
        private void initDataSource()
        {
            if (_lrrProjConfigs != null)
                _lrrProjConfigs.Clear();
            if (_lrrMeasSystemDefs != null)
                _lrrMeasSystemDefs.Clear();
            _lrrProjConfigs = new LRRProjectConfigurations();
            _app.DBManagerObj.PopulateCollection(_lrrProjConfigs, false);

            _lrrMeasSystemDefs = new LRRMeasurementSystemDefinitions();
            _app.DBManagerObj.PopulateCollection(_lrrMeasSystemDefs, false);
        }

        /// <summary>
        /// Handles the SaveCellInfo event of the Model control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridSaveCellInfoEventArgs"/> instance containing the event data.</param>
        void Model_SaveCellInfo(object sender, GridSaveCellInfoEventArgs e)
        {
            if(e.Style.CellType == "DropDownTree")
            {
                object record = null;
                string Prop = "";
                string keyField = "";
                string valueMem = e.Style.CellValue.ToString();
                string newValue = "";
                string newValueText = "";

            
                if (valueMem != "" && valueMem != null)
                {
                    ColomnHeaderInfo colInfo = ColInfo[e.ColIndex] as ColomnHeaderInfo;
                    RowHeaderInfo rowInfo = RowInfo[e.RowIndex] as RowHeaderInfo;
                    keyField = colInfo.ID;
                    Prop = rowInfo.PropertyName;
                    record = recordHash[keyField] as object;
                    string oldValueText = gridControl1.CurrentCell.Renderer.ControlValue.ToString();
                    LRRLevel lvl = LRRModel.ChildModel.Levels.getLevelByLevelID(valueMem);
                    e.Style.FormattedText = lvl.Name;
                    string oldValue = record.GetType().GetProperty(Prop).GetValue(record, null) == null ? "" : record.GetType().GetProperty(Prop).GetValue(record, null).ToString();
                    if (lvl.LevelID != null)
                    {
                        newValue = lvl.LevelID;
                        newValueText = lvl.Name;
                    }

                    if (newValue != oldValue)
                    {
                        if(newValue != null && newValue.Trim() != "")
                            record.GetType().GetProperty(Prop).SetValue(record, newValue, null);
                    }
                }                 
            }
        }

       
        #region Private Methods

        /// <summary>
        /// Gets the selected range.
        /// </summary>
        /// <param name="grid">The grid.</param>
        /// <param name="isParentGrid">if set to <c>true</c> [is parent grid].</param>
        /// <returns></returns>
        private GridRangeInfoList getSelectedRange(GridGroupingControl grid, out bool isParentGrid)
        {
            GridRangeInfoList returnList;
            if (grid.TableControl.Table.CurrentRecord == null)
            {
                Element eCurrentInnerElement = grid.Table.GetInnerMostCurrentElement();
                Element eCurrentElement = grid.TableControl.Table.CurrentElement;
                GridTable gt = eCurrentInnerElement.ParentTable as GridTable;
                GridNestedTable gnt = eCurrentElement as GridNestedTable;
                GridNestedTable gnt1 = gnt;
                GridTableDescriptor gtd = null;
                //Loop throgh we get the childlevel grid data.
                while (gnt1 != null && gnt1.ChildTable != null)
                {
                    gnt = gnt1;
                    gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
                    gtd = gnt.ParentTable.ParentTableDescriptor;
                }
                GridChildTable gt1 = gnt.ChildTable;
                returnList = gt.TableModel.SelectedRanges;
                isParentGrid = false;

            }
            else
            {
                returnList = grid.TableModel.SelectedRanges;
                isParentGrid = true;
            }
            return returnList;

        }

        /// <summary>
        /// Updates the read only properties.
        /// </summary>
        /// <param name="recordObj">The record obj.</param>
        /// <param name="col">The col.</param>
        private void updateReadOnlyProperties(object recordObj, int col)
        {
            for (int i = 0; i < RowInfo.Count; i++)
            {
                RowHeaderInfo rInfo = RowInfo[i] as RowHeaderInfo;
                string propName = rInfo.PropertyName;
                if (propName == "ModelUOM")
                {
                    string modelUOM = recordObj.GetType().GetProperty("ModelUOM").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("ModelUOM").GetValue(recordObj, null).ToString();
                    this.gridControl1[i, col].Text = modelUOM;
                }
                else if (propName == "UnderlyingInterfaceValue")
                {
                    string internalInterfaceValue = recordObj.GetType().GetProperty("UnderlyingInterfaceValue").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("UnderlyingInterfaceValue").GetValue(recordObj, null).ToString();
                    this.gridControl1[i, col].Text = internalInterfaceValue;
                }
                else if (propName == "DisplayInterfaceValue")
                {
                    string interfaceValue = recordObj.GetType().GetProperty("DisplayInterfaceValue").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("DisplayInterfaceValue").GetValue(recordObj, null).ToString();
                    this.gridControl1[i, col].Text = interfaceValue;
                }
            }
        }


        /// <summary>
        /// Initializes the read only properties.
        /// </summary>
        /// <param name="grid">The grid.</param>
        /// <param name="bAddNewRecord">if set to <c>true</c> [b add new record].</param>
        private void initializeReadOnlyProperties(GridGroupingControl grid, bool bAddNewRecord)
        {
            try
            {
                bool isParentGrid = false;
                int top, bottom;
                GridRangeInfoList list = getSelectedRange(grid, out isParentGrid);
                if (list.Info != "")
                {
                    int rowCount = ROW_COUNT;
                    this.gridControl1.RowCount = rowCount + 3;
                    this.gridControl1[rowCount + 1, 0].Text = "ModelUOM";
                    RowHeaderInfo rowInfoModelUom = new RowHeaderInfo();
                    rowInfoModelUom.PropertyName = "ModelUOM";
                    RowInfo.Add(rowInfoModelUom);
                    this.gridControl1[rowCount + 2, 0].Text = "Interface Value";
                    RowHeaderInfo rowInfoInterFaceValue = new RowHeaderInfo();
                    rowInfoInterFaceValue.PropertyName = "DisplayInterfaceValue";
                    RowInfo.Add(rowInfoInterFaceValue);
                    this.gridControl1[rowCount + 3, 0].Text = "InternalInterfaceValue";
                    RowHeaderInfo rowInfoInternalInterfaceValue = new RowHeaderInfo();
                    rowInfoInternalInterfaceValue.PropertyName = "UnderlyingInterfaceValue";
                    RowInfo.Add(rowInfoInternalInterfaceValue);
                    int k = 1;
                    for (int l = 0; l < list.Count; l++)
                    {
                        GridRangeInfo range = list[l];
                        if (isParentGrid)
                        {
                            if (grid.TopLevelGroupOptions.ShowFilterBar == true)
                            {
                                top = range.Top - 4;
                                bottom = range.Bottom - 4;
                            }
                            else
                            {
                                top = range.Top - 3;
                                bottom = range.Bottom - 3;
                            }
                            int selRows = range.Height;
                            for (int m = 0; m < selRows; m++)
                            {
                                object recordObj = grid.Table.Records[top].GetData();
                                string modelUOM = recordObj.GetType().GetProperty("ModelUOM").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("ModelUOM").GetValue(recordObj, null).ToString();
                                string interfaceValue = recordObj.GetType().GetProperty("DisplayInterfaceValue").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("DisplayInterfaceValue").GetValue(recordObj, null).ToString();
                                string internalInterfaceValue = recordObj.GetType().GetProperty("UnderlyingInterfaceValue").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("UnderlyingInterfaceValue").GetValue(recordObj, null).ToString();


                                //Model UOM
                                this.gridControl1[rowCount + 1, k].CellType = "ComboBox";
                                this.gridControl1[rowCount + 1, k].ShowButtons = Syncfusion.Windows.Forms.Grid.GridShowButtons.Hide;
                                this.gridControl1[rowCount + 1, k].Text = modelUOM;
                                this.gridControl1[rowCount + 1, k].DataSource = lrrUoms;
                                this.gridControl1[rowCount + 1, k].DisplayMember = "Name";
                                this.gridControl1[rowCount + 1, k].ValueMember = "ID";
                                //this.gridControl1[rowCount + 1, k].BackColor = Color.Gray;
                                this.gridControl1[rowCount + 1, k].ReadOnly = true;

                                //Interface Value
                                this.gridControl1[rowCount + 2, k].Text = interfaceValue;
                                //this.gridControl1[rowCount + 2, k].BackColor = Color.Gray;
                                this.gridControl1[rowCount + 2, k].ReadOnly = true;


                                //Internal Interface Value
                                this.gridControl1[rowCount + 3, k].Text = internalInterfaceValue;
                                //this.gridControl1[rowCount + 3, k].BackColor = Color.Gray;
                                this.gridControl1[rowCount + 3, k].ReadOnly = true;
                                k++;
                                top++;
                            }
                        }
                    }
                }
                else if (list.Info == "" && bAddNewRecord == true)
                {
                    int rowCount = ROW_COUNT;
                    this.gridControl1.RowCount = rowCount + 3;
                    this.gridControl1[rowCount + 1, 0].Text = "ModelUOM";
                    RowHeaderInfo rowInfoModelUom = new RowHeaderInfo();
                    rowInfoModelUom.PropertyName = "ModelUOM";
                    RowInfo.Add(rowInfoModelUom);
                    this.gridControl1[rowCount + 2, 0].Text = "Interface Value";
                    RowHeaderInfo rowInfoInterFaceValue = new RowHeaderInfo();
                    rowInfoInterFaceValue.PropertyName = "DisplayInterfaceValue";
                    RowInfo.Add(rowInfoInterFaceValue);
                    this.gridControl1[rowCount + 3, 0].Text = "InternalInterfaceValue";
                    RowHeaderInfo rowInfoInternalInterfaceValue = new RowHeaderInfo();
                    rowInfoInternalInterfaceValue.PropertyName = "UnderlyingInterfaceValue";
                    RowInfo.Add(rowInfoInternalInterfaceValue);

                    object recordObj = grid.Table.CurrentRecord.GetData();
                    string modelUOM = recordObj.GetType().GetProperty("ModelUOM").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("ModelUOM").GetValue(recordObj, null).ToString();
                    string interfaceValue = recordObj.GetType().GetProperty("DisplayInterfaceValue").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("DisplayInterfaceValue").GetValue(recordObj, null).ToString();
                    string internalInterfaceValue = recordObj.GetType().GetProperty("UnderlyingInterfaceValue").GetValue(recordObj, null) == null ? "" : recordObj.GetType().GetProperty("UnderlyingInterfaceValue").GetValue(recordObj, null).ToString();

                    for (int i = 0; i <= this.gridControl1.RowCount; i++)
                    {
                        if (this.gridControl1[i, 0].Text == "ModelUOM")
                        {
                            this.gridControl1[i, 1].CellType = "ComboBox";
                            this.gridControl1[i, 1].ShowButtons = Syncfusion.Windows.Forms.Grid.GridShowButtons.Hide;
                            this.gridControl1[i, 1].Text = modelUOM;
                            this.gridControl1[i, 1].DataSource = lrrUoms;
                            this.gridControl1[i, 1].DisplayMember = "Name";
                            this.gridControl1[i, 1].ValueMember = "ID";
                            //this.gridControl1[i, 1].BackColor = Color.Gray;
                            this.gridControl1[i, 1].ReadOnly = true;
                        }
                        if (this.gridControl1[i, 0].Text == "Interface Value")
                        {
                            this.gridControl1[i, 1].Text = interfaceValue;
                            //this.gridControl1[i, 1].BackColor = Color.Gray;
                            this.gridControl1[i, 1].ReadOnly = true;
                        }
                        if (this.gridControl1[i, 0].Text == "InternalInterfaceValue")
                        {
                            this.gridControl1[i, 1].Text = internalInterfaceValue;
                            //this.gridControl1[i, 1].BackColor = Color.Gray;
                            this.gridControl1[i, 1].ReadOnly = true;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.Assert(false, ex.Message);
            }

        }
        /// <summary>
        /// Initializes the grid.
        /// </summary>
        /// <param name="props">The props.</param>
        /// <param name="grid">The grid.</param>
        /// <param name="bAddNewRecord">if set to <c>true</c> [b add new record].</param>
        private void initializeGrid(PropertyInfo[] props, GridGroupingControl grid, bool bAddNewRecord)
        {
            try
            {
                string displayMenber = "";
                string valueMember = "";
                int top, bottom;
                string primeryKey = "";
                _isFkField = false;
                bool isParentGrid = false;
                CustomCollection dataSource = null;
                GridRangeInfoList list = getSelectedRange(grid, out isParentGrid);
                if (list.Info != "")
                {
                    int j = 0;
                    _showForm = true;
                    for (int i = 0; i < props.Length; i++)
                    {
                        PropertyInfo prop = props[i];

                        SymKeyFieldAttribute[] PKfield = (SymKeyFieldAttribute[])
                            prop.GetCustomAttributes(typeof(SymKeyFieldAttribute), false);
                        if (PKfield.Length > 0)
                        {
                            primeryKey = prop.Name;
                        }

                        SymForeignKeyFieldAttribute[] FKfield = (SymForeignKeyFieldAttribute[])
                            prop.GetCustomAttributes(typeof(SymForeignKeyFieldAttribute), false);
                        if (FKfield.Length > 0)
                        {
                            SymForeignKeyFieldAttribute sFkField = FKfield[0];
                            dataSource = LRRModel.GetObjectCopy(sFkField.CollectionType) as CustomCollection;
                            displayMenber = sFkField.DisplayMemeber;
                            valueMember = sFkField.ValueMemeber;
                            _isFkField = true;
                        }
                        SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
                            prop.GetCustomAttributes(typeof(SymDataFieldAttribute),
                            false);
                        SymViewFieldAttribute[] SymViewfield = (SymViewFieldAttribute[])
                            prop.GetCustomAttributes(typeof(SymViewFieldAttribute),
                            false);

                        if (field.Length > 0)
                        {
                            SymDataFieldAttribute sDFA = field[0];
                            if (sDFA.IsVisible)
                            {
                                j++;
                                int k = 1;
                                gridControl1[j, 0].Text = LRRHeaderText.GetHeaderString(sDFA.DisplayName);
                                RowHeaderInfo rowInfo = new RowHeaderInfo();
                                rowInfo.PropertyName = sDFA.PropName;
                                if (_isFkField)
                                {
                                    rowInfo.IsFkField = true;
                                }
                                RowInfo.Add(rowInfo);
                                for (int l = 0; l < list.Count; l++)
                                {
                                    GridRangeInfo range = list[l];
                                    int selRows = range.Height;
                                    top = range.Top;
                                    bottom = range.Bottom;
                                    if (isParentGrid)
                                    {
                                        if (grid.TopLevelGroupOptions.ShowFilterBar == true)
                                        {
                                            top = range.Top - 4;
                                            bottom = range.Bottom - 4;
                                        }
                                        else
                                        {
                                            top = range.Top - 3;
                                            bottom = range.Bottom - 3;
                                        }

                                        for (int m = 0; m < selRows; m++)
                                        {
                                            ColomnHeaderInfo colInfo = new ColomnHeaderInfo();
                                            object recordObject = grid.Table.Records[top].GetData();
                                            string pkFieldValue = recordObject.GetType().GetProperty(primeryKey).GetValue(recordObject, null) == null ? "" : recordObject.GetType().GetProperty(primeryKey).GetValue(recordObject, null).ToString();
                                            if (!recordHash.Contains(pkFieldValue))
                                                recordHash.Add(pkFieldValue, recordObject);
                                            colInfo.ID = pkFieldValue;
                                            colInfo.ObjectType = cCollection.GetType();
                                            ColInfo.Add(colInfo);
                                            top++;
                                            if (recordObject != null)
                                            {
                                                string Value = recordObject.GetType().GetProperty(sDFA.PropName).GetValue(recordObject, null) == null ? "" : recordObject.GetType().GetProperty(sDFA.PropName).GetValue(recordObject, null).ToString();
                                                if (Value != null)
                                                {
                                                    gridControl1[j, k].Text = Value;
                                                }
                                                else
                                                {
                                                    gridControl1[j, k].Text = " ";
                                                }
                                                if (sDFA.PropName == "PlantUnit")
                                                {
                                                    gridControl1.Model[j, k].CellType = "DropDownTree";
                                                    string valueMem = gridControl1.Model[j, k].CellValue.ToString();
                                                    if (valueMem != "" && valueMem != null)
                                                    {
                                                        LRRLevel lvl = LRRModel.ChildModel.Levels.getLevelByLevelID(valueMem);
                                                        gridControl1.Model[j, k].FormattedText = lvl.Name;
                                                    }
                                                    gridControl1.Model[j, k].DisplayMember = displayMenber;
                                                    gridControl1.Model[j, k].ValueMember = valueMember;
                                                    gridControl1.Model[j, k].DataSource = dataSource;
                                                }
                                                else if (sDFA.PropName == "ErrorStatus")
                                                {
                                                    gridControl1.Model[j, k].CellType = "TextBox";
                                                    gridControl1[j, k].Text = Value;
                                                }
                                                else if (sDFA.DisplayType == Symcon.Globals.Enums.DisplayTypes.LIST)
                                                {
                                                    gridControl1.Model[j, k].CellType = "ComboBox";
                                                    gridControl1.Model[j, k].DisplayMember = displayMenber;
                                                    gridControl1.Model[j, k].ValueMember = valueMember;
                                                    gridControl1.Model[j, k].DataSource = dataSource;
                                                }

                                            }
                                            k++;
                                        }
                                    }
                                }
                            }
                        }
                        else if (SymViewfield.Length > 0)
                        {
                            SymViewFieldAttribute sVFA = SymViewfield[0];
                            if (sVFA.IsVisible)
                            {
                                j++;
                                int k = 1;
                                gridControl1[j, 0].Text = LRRHeaderText.GetHeaderString(sVFA.DisplayName);
                                RowHeaderInfo rowInfo = new RowHeaderInfo();
                                rowInfo.PropertyName = sVFA.PropName;
                                if (_isFkField)
                                {
                                    rowInfo.IsFkField = true;
                                }
                                RowInfo.Add(rowInfo);
                                for (int l = 0; l < list.Count; l++)
                                {
                                    GridRangeInfo range = list[l];
                                    int selRows = range.Height;
                                    top = range.Top;
                                    bottom = range.Bottom;
                                    if (isParentGrid)
                                    {
                                        if (grid.TopLevelGroupOptions.ShowFilterBar == true)
                                        {
                                            top = range.Top - 4;
                                            bottom = range.Bottom - 4;
                                        }
                                        else
                                        {
                                            top = range.Top - 3;
                                            bottom = range.Bottom - 3;
                                        }
                                        for (int m = 0; m < selRows; m++)
                                        {
                                            ColomnHeaderInfo colInfo = new ColomnHeaderInfo();
                                            object recordObject = grid.Table.Records[top].GetData();
                                            string pkFieldValue = recordObject.GetType().GetProperty(primeryKey).GetValue(recordObject, null) == null ? "" : recordObject.GetType().GetProperty(primeryKey).GetValue(recordObject, null).ToString();
                                            if (!recordHash.Contains(pkFieldValue))
                                                recordHash.Add(pkFieldValue, recordObject);
                                            colInfo.ID = pkFieldValue;
                                            colInfo.ObjectType = cCollection.GetType();
                                            ColInfo.Add(colInfo);
                                            top++;
                                            if (recordObject != null)
                                            {
                                                string Value = recordObject.GetType().GetProperty(sVFA.PropName).GetValue(recordObject, null) == null ? "" : recordObject.GetType().GetProperty(sVFA.PropName).GetValue(recordObject, null).ToString();
                                                if (Value != null)
                                                {
                                                    gridControl1.Model[j, k].CellType = "TextBox";
                                                    gridControl1[j, k].Text = Value;
                                                }
                                                else
                                                {
                                                    gridControl1[j, k].Text = " ";
                                                }
                                                if (sVFA.DisplayType == Symcon.Globals.Enums.DisplayTypes.CUSTOM)
                                                {
                                                    gridControl1.Model[j, k].CellType = "EllipsisText";
                                                }
                                            }
                                            k++;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                else if (bAddNewRecord == true && list.Info == "")
                {
                    int j = 0;
                    _showForm = true;
                    for (int i = 0; i < props.Length; i++)
                    {
                        PropertyInfo prop = props[i];

                        SymKeyFieldAttribute[] PKfield = (SymKeyFieldAttribute[])
                            prop.GetCustomAttributes(typeof(SymKeyFieldAttribute), false);
                        if (PKfield.Length > 0)
                        {
                            primeryKey = prop.Name;
                        }

                        SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
                          prop.GetCustomAttributes(typeof(SymDataFieldAttribute),
                          false);

                        SymForeignKeyFieldAttribute[] FKfield = (SymForeignKeyFieldAttribute[])
                            prop.GetCustomAttributes(typeof(SymForeignKeyFieldAttribute), false);

                        if (FKfield.Length > 0)
                        {
                            SymForeignKeyFieldAttribute sFkField = FKfield[0];
                            dataSource = LRRModel.GetObjectCopy(sFkField.CollectionType) as CustomCollection;
                            displayMenber = sFkField.DisplayMemeber;
                            valueMember = sFkField.ValueMemeber;
                            _isFkField = true;
                        }

                        SymViewFieldAttribute[] SymViewfield = (SymViewFieldAttribute[])
                            prop.GetCustomAttributes(typeof(SymViewFieldAttribute),
                            false);

                        if (field.Length > 0)
                        {
                            SymDataFieldAttribute sDFA = field[0];
                            if (sDFA.IsVisible)
                            {
                                j++;
                                int k = 1;
                                gridControl1[j, 0].Text = LRRHeaderText.GetHeaderString(sDFA.DisplayName);
                                RowHeaderInfo rowInfo = new RowHeaderInfo();
                                rowInfo.PropertyName = sDFA.PropName;
                                if (_isFkField)
                                {
                                    rowInfo.IsFkField = true;
                                }
                                RowInfo.Add(rowInfo);

                                ColomnHeaderInfo colInfo = new ColomnHeaderInfo();
                                object recordObject = grid.Table.CurrentRecord.GetData();
                                string pkFieldValue = recordObject.GetType().GetProperty(primeryKey).GetValue(recordObject, null) == null ? "" : recordObject.GetType().GetProperty(primeryKey).GetValue(recordObject, null).ToString();
                                if (!recordHash.Contains(pkFieldValue))
                                    recordHash.Add(pkFieldValue, recordObject);
                                colInfo.ID = pkFieldValue;
                                colInfo.ObjectType = cCollection.GetType();
                                ColInfo.Add(colInfo);
                                if (recordObject != null)
                                {
                                    string Value = recordObject.GetType().GetProperty(sDFA.PropName).GetValue(recordObject, null) == null ? "" : recordObject.GetType().GetProperty(sDFA.PropName).GetValue(recordObject, null).ToString();
                                    if (Value != null)
                                    {
                                        gridControl1[j, k].Text = Value;
                                    }
                                    else
                                    {
                                        gridControl1[j, k].Text = " ";
                                    }
                                    if (sDFA.PropName == "PlantUnit")
                                    {
                                        gridControl1.Model[j, k].CellType = "DropDownTree";
                                        string valueMem = gridControl1.Model[j, k].CellValue.ToString();
                                        if (valueMem != "" && valueMem != null)
                                        {
                                            LRRLevel lvl = LRRModel.ChildModel.Levels.getLevelByLevelID(valueMem);
                                            gridControl1.Model[j, k].FormattedText = lvl.Name;
                                        }
                                        gridControl1.Model[j, k].DisplayMember = displayMenber;
                                        gridControl1.Model[j, k].ValueMember = valueMember;
                                        gridControl1.Model[j, k].DataSource = dataSource;
                                    }
                                    else if (sDFA.PropName == "ErrorStatus")
                                    {
                                        gridControl1.Model[j, k].CellType = "TextBox";
                                        gridControl1[j, k].Text = Value;
                                    }
                                    else if (sDFA.DisplayType == Symcon.Globals.Enums.DisplayTypes.LIST)
                                    {
                                        gridControl1.Model[j, k].CellType = "ComboBox";
                                        gridControl1.Model[j, k].DisplayMember = displayMenber;
                                        gridControl1.Model[j, k].ValueMember = valueMember;
                                        if (sDFA.PropName == "InterfaceUnits" || sDFA.PropName == "HistUnits")
                                        {
                                            gridControl1[j, k].DataSource = null;
                                        }
                                        else
                                            gridControl1.Model[j, k].DataSource = dataSource;
                                    }
                                }
                            }
                        }
                        else if (SymViewfield.Length > 0)
                        {
                            SymViewFieldAttribute sVFA = SymViewfield[0];
                            if (sVFA.IsVisible)
                            {
                                j++;
                                int k = 1;
                                gridControl1[j, 0].Text = LRRHeaderText.GetHeaderString(sVFA.DisplayName);
                                RowHeaderInfo rowInfo = new RowHeaderInfo();
                                rowInfo.PropertyName = sVFA.PropName;
                                if (_isFkField)
                                {
                                    rowInfo.IsFkField = true;
                                }
                                RowInfo.Add(rowInfo);       
                                ColomnHeaderInfo colInfo = new ColomnHeaderInfo();
                                object recordObject = grid.Table.CurrentRecord.GetData();
                                string pkFieldValue = recordObject.GetType().GetProperty(primeryKey).GetValue(recordObject, null) == null ? "" : recordObject.GetType().GetProperty(primeryKey).GetValue(recordObject, null).ToString();
                                if (!recordHash.Contains(pkFieldValue))
                                    recordHash.Add(pkFieldValue, recordObject);
                                colInfo.ID = pkFieldValue;
                                colInfo.ObjectType = cCollection.GetType();
                                ColInfo.Add(colInfo);
                                if (recordObject != null)
                                {
                                    string Value = recordObject.GetType().GetProperty(sVFA.PropName).GetValue(recordObject, null) == null ? "" : recordObject.GetType().GetProperty(sVFA.PropName).GetValue(recordObject, null).ToString();
                                    if (Value != null)
                                    {
                                        gridControl1.Model[j, k].CellType = "TextBox";
                                        gridControl1[j, k].Text = Value;
                                    }
                                    else
                                    {
                                        gridControl1[j, k].Text = " ";
                                    }
                                    if (sVFA.DisplayType == Symcon.Globals.Enums.DisplayTypes.CUSTOM)
                                    {
                                        gridControl1.Model[j, k].CellType = "EllipsisText";
                                    }
                                }                                        
                            }
                                    
                        }
                                
                    }
                           
                }
                else
                {
                    _showForm = false;
                }
            }
            catch (Exception ex)
            {
                Debug.Assert(false, ex.Message);
            }
        }
        /// <summary>
        /// Creates the grid.
        /// </summary>
        /// <param name="rowCount">The row count.</param>
        /// <param name="colCount">The col count.</param>
        private void createGrid(int rowCount, int colCount)
        {
            gridControl1.ColCount = colCount;
            gridControl1.RowCount = rowCount;
            gridControl1.NumberedRowHeaders = false;
            gridControl1.ExcelLikeSelectionFrame = true;
            gridControl1.Properties.BackgroundColor = Color.White;
            gridControl1[0, 0].Text = "Coloumn Header";
            RowHeaderInfo rowInfo = new RowHeaderInfo();
            rowInfo.PropertyName = "Properties";
            RowInfo.Add(rowInfo);
            ColomnHeaderInfo colInfo = new ColomnHeaderInfo();
            colInfo.ID = "1";
            colInfo.ObjectType = null;
            ColInfo.Add(colInfo);
            gridControl1.ColWidths[0] = 150;
            for (int i = 1; i <= gridControl1.ColCount; i++)
            {
                gridControl1[0, i].Text = "Values";
                gridControl1.ColWidths[i] = 175;
            }
            ROW_COUNT = gridControl1.RowCount;
        }
        /// <summary>
        /// Gets the grid col count.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <returns></returns>
        private int getGridColCount(GridRangeInfoList list)
        {
            int colCount = 0;
            for (int i = 0; i < list.Count; i++)
            {
                colCount = colCount + list[0].Height;
            }
            return colCount;
        }
        /// <summary>
        /// Gets the grid row count.
        /// </summary>
        /// <param name="props">The props.</param>
        /// <returns></returns>
        private int getGridRowCount(PropertyInfo[] props)
        {
            int length = 0;
            for (int i = 0; i < props.Length; i++)
            {
                PropertyInfo prop = props[i];
                SymDataFieldAttribute[] symDatafield = (SymDataFieldAttribute[])
                    prop.GetCustomAttributes(typeof(SymDataFieldAttribute),
                    false);
                SymViewFieldAttribute[] symViewfield = (SymViewFieldAttribute[])
                    prop.GetCustomAttributes(typeof(SymViewFieldAttribute),
                    false);
                if (symDatafield.Length > 0)
                {
                    SymDataFieldAttribute sDFA = symDatafield[0];
                    if (sDFA.IsVisible)
                    {
                        length++;
                    }
                }
                else if (symViewfield.Length > 0)
                {
                    SymViewFieldAttribute sVFA = symViewfield[0];
                    if (sVFA.IsVisible)
                    {
                        length++;
                    }
                }

            }
            return length;
        }

        #endregion

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }


        public bool ShowForm
        {
            get { return _showForm; }
            set
            {
                _showForm = value;
            }
        }

        #region Fields

        private CustomCollection cCollection;
        private bool _isFkField;
        private GridGroupingControl gridGroupControl;
        private ArrayList RowInfo = new ArrayList();
        private ArrayList ColInfo = new ArrayList();
        private Hashtable recordHash = new Hashtable();
        private GridRangeInfoList list;
        private PropertyInfo[] props;
        private LRRUOMs lrrUoms;
        private int ROW_COUNT;
        private bool _showForm;
        private LRRApp _app;


        private LRRProjectConfigurations _lrrProjConfigs;
        private LRRMeasurementSystemDefinitions _lrrMeasSystemDefs;

        #endregion

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.gridControl1 = new Syncfusion.Windows.Forms.Grid.GridControl();
            this.panel1 = new System.Windows.Forms.Panel();
            this.panel2 = new System.Windows.Forms.Panel();
            ((System.ComponentModel.ISupportInitialize)(this.gridControl1)).BeginInit();
            this.panel2.SuspendLayout();
            this.SuspendLayout();
            // 
            // gridControl1
            // 
            this.gridControl1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.gridControl1.Location = new System.Drawing.Point(0, 0);
            this.gridControl1.Name = "gridControl1";
            this.gridControl1.Size = new System.Drawing.Size(448, 210);
            this.gridControl1.SmartSizeBox = false;
            this.gridControl1.TabIndex = 0;
            this.gridControl1.Text = "gridControl1";
            this.gridControl1.CellClick += new Syncfusion.Windows.Forms.Grid.GridCellClickEventHandler(this.gridControl1_CellClick);
            this.gridControl1.CurrentCellCloseDropDown += new Syncfusion.Windows.Forms.PopupClosedEventHandler(this.gridControl1_CurrentCellCloseDropDown);
            this.gridControl1.CurrentCellValidating += new System.ComponentModel.CancelEventHandler(this.gridControl1_CurrentCellValidating);
            this.gridControl1.CurrentCellShowingDropDown += new Syncfusion.Windows.Forms.Grid.GridCurrentCellShowingDropDownEventHandler(this.gridControl1_CurrentCellShowingDropDown);
            this.gridControl1.CurrentCellValidated += new System.EventHandler(this.gridControl1_CurrentCellValidated);
            this.gridControl1.CurrentCellEditingComplete += new System.EventHandler(this.gridControl1_CurrentCellEditingComplete);
            this.gridControl1.CellButtonClicked += new Syncfusion.Windows.Forms.Grid.GridCellButtonClickedEventHandler(this.gridControl1_CellButtonClicked);
            this.gridControl1.CurrentCellKeyPress += new KeyPressEventHandler(gridControl1_CurrentCellKeyPress);
            this.gridControl1.CurrentCellKeyDown += new KeyEventHandler(gridControl1_CurrentCellKeyDown);
            // 
            // panel1
            // 
            this.panel1.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.panel1.Location = new System.Drawing.Point(0, 210);
            this.panel1.Name = "panel1";
            this.panel1.Size = new System.Drawing.Size(448, 56);
            this.panel1.TabIndex = 1;
            // 
            // panel2
            // 
            this.panel2.Controls.Add(this.gridControl1);
            this.panel2.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panel2.Location = new System.Drawing.Point(0, 0);
            this.panel2.Name = "panel2";
            this.panel2.Size = new System.Drawing.Size(448, 210);
            this.panel2.TabIndex = 2;
            // 
            // TransPosedView
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(448, 266);
            this.Controls.Add(this.panel2);
            this.Controls.Add(this.panel1);
            this.Name = "TransPosedView";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Tag = "SID_TRANSPOSED_VIEW";
            ((System.ComponentModel.ISupportInitialize)(this.gridControl1)).EndInit();
            this.panel2.ResumeLayout(false);
            this.ResumeLayout(false);

        }


        #endregion

        /// <summary>
        /// Handles the CurrentCellCloseDropDown event of the gridControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.PopupClosedEventArgs"/> instance containing the event data.</param>
        private void gridControl1_CurrentCellCloseDropDown(object sender, Syncfusion.Windows.Forms.PopupClosedEventArgs e)
        {
            object record = null;
            string Prop = "";
            string keyField = "";
            GridControl grid = sender as GridControl;
            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            string oldValue = grid.Model[currentRow, currentCol].Text;
            string newTextValue = grid.CurrentCell.Renderer.ControlText.ToString();
            string newValue = grid.CurrentCell.Renderer.ControlValue.ToString();
            if (newValue != null && newValue.Trim() != "")
            {
                if (newTextValue != oldValue)
                {
                    ColomnHeaderInfo colInfo = ColInfo[currentCol] as ColomnHeaderInfo;
                    RowHeaderInfo rowInfo = RowInfo[currentRow] as RowHeaderInfo;
                    keyField = colInfo.ID;
                    record = recordHash[keyField] as object;
                    LRRHistData histDataObj = record as LRRHistData;
                    Prop = rowInfo.PropertyName;
                    record.GetType().GetProperty(Prop).SetValue(record, newValue, null);
                    if (record != null)
                    {
                        RelationalColumnConditions conditions = new RelationalColumnConditions();
                        conditions.ConditionApplyForFilterUOMbasedOnUomType(record, newValue, oldValue, Prop);
                    }

                    if (Prop == "HistUOMTypeID")
                    {
                        int histUnitRow = 0;
                        int histUOMTypeID = 0;
                        for (int i = 0; i < grid.RowCount; i++)
                        {
                            if (grid[i, 0].Text == "Historian UOM")
                            {
                                histUnitRow = i;
                                break;
                            }
                        }
                        for (int i = 0; i < grid.RowCount; i++)
                        {
                            if (grid[i, 0].Text == "Historian UOM Type")
                            {
                                histUOMTypeID = i;
                                break;
                            }
                        }
                        grid[histUOMTypeID, currentCol].CellValue = newValue;
                        grid[histUOMTypeID, currentCol].FormattedText = newTextValue;
                        LRRProjectConfiguration lrrProjConfig = _lrrProjConfigs[0] as LRRProjectConfiguration;
                        string modelUOMID = _lrrMeasSystemDefs.GetLocaleUOM(lrrProjConfig.MeasurementSystemCEID, newValue);
                        string uiUOMID = _lrrMeasSystemDefs.GetLocaleUOM(lrrProjConfig.MeasurementSystemUserInterfaceID, newValue);
                        LRRUOM uomObj = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(uiUOMID);
                        LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(newValue);

                        grid[histUnitRow, currentCol].CellValue = "";
                        grid[histUnitRow, currentCol].FormattedText = "";
                        if (uoms != null)
                            grid[histUnitRow, currentCol].DataSource = uoms;
                    }
                    else if (Prop == "UOMTypeID")
                    {
                        int histUnitRow = 0;
                        int histUOMTypeID = 0;
                        int uomID = 0;
                        int uomTypeRow = 0;
                        for (int i = 0; i < grid.RowCount; i++)
                        {
                            if (grid[i, 0].Text == "Historian UOM")
                            {
                                histUnitRow = i;
                                break;
                            }
                        }
                        for (int i = 0; i < grid.RowCount; i++)
                        {
                            if (grid[i, 0].Text == "Historian UOM Type")
                            {
                                histUOMTypeID = i;
                                break;
                            }
                        }
                        for (int i = 0; i < grid.RowCount; i++)
                        {
                            if (grid[i, 0].Text == "Interface UOM")
                            {
                                uomID = i;
                                break;
                            }
                        }
                        for (int i = 0; i < grid.RowCount; i++)
                        {
                            if (grid[i, 0].Text == "UOM Type")
                            {
                                uomTypeRow = i;
                                break;

                            }
                        }
                        LRRProjectConfiguration lrrProjConfig = _lrrProjConfigs[0] as LRRProjectConfiguration;
                        string modelUOMID = _lrrMeasSystemDefs.GetLocaleUOM(lrrProjConfig.MeasurementSystemCEID, newValue);
                        string uiUOMID = _lrrMeasSystemDefs.GetLocaleUOM(lrrProjConfig.MeasurementSystemUserInterfaceID, newValue);
                        LRRUOM uomObj = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(uiUOMID);
                        LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(newValue);

                        if (newValue != null && newValue.Trim() != "")
                        {
                            grid[uomTypeRow, currentCol].CellValue = newValue;
                            grid[uomTypeRow, currentCol].FormattedText = newTextValue;
                        }

                        if (uiUOMID != null && uiUOMID.Trim() != "")
                            grid[uomID, currentCol].CellValue = uiUOMID;
                        if(uomObj != null)    
                            grid[uomID, currentCol].FormattedText = uomObj.Name;
                        else
                        {
                            grid[uomID, currentCol].CellValue = "";
                            grid[uomID, currentCol].FormattedText = "";
                        }
                        if (uoms != null)
                            grid[uomID, currentCol].DataSource = uoms;



                        if ((histDataObj.MeasType != "C" && histDataObj.MeasType != "N") && (histDataObj.MeasType != null && histDataObj.MeasType != ""))
                        {
                            record.GetType().GetProperty("HistUOMTypeID").SetValue(record, newValue, null);
 
                            grid[histUOMTypeID, currentCol].CellValue = newValue;
                            grid[histUOMTypeID, currentCol].FormattedText = newTextValue;

                            grid[histUnitRow, currentCol].CellValue = "";
                            grid[histUnitRow, currentCol].FormattedText = "";

                            if (uoms != null)
                                grid[histUnitRow, currentCol].DataSource = uoms;
                        }
                    }
                    //initializeGrid(props, gridGroupControl);
                    record = recordHash[keyField] as object;
                    updateReadOnlyProperties(record, currentCol);
                }
            }
        }

        private void gridControl1_CurrentCellConfirmChangesFailed(object sender, System.EventArgs e)
        {



        }

        /// <summary>
        /// Handles the CurrentCellValidating event of the gridControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void gridControl1_CurrentCellValidating(object sender, System.ComponentModel.CancelEventArgs e)
        {
            
            GridControl grid = sender as GridControl;
            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            if (grid.Model[currentRow, currentCol].CellType != "ComboBox")
            {
                ColomnHeaderInfo colInfo = ColInfo[currentCol] as ColomnHeaderInfo;
                RowHeaderInfo rowInfo = RowInfo[currentRow] as RowHeaderInfo;
                string keyField = colInfo.ID;
                object record = recordHash[keyField] as object;
                string Prop = rowInfo.PropertyName;
                string oldValue = grid.Model[currentRow, currentCol].FormattedText;
                string newValue = grid.Model[currentRow, currentCol].CellModel.GetActiveText(currentRow, currentCol) == null ? "" : grid.Model[currentRow, currentCol].CellModel.GetActiveText(currentRow, currentCol).ToString();
                if (newValue != oldValue)
                {
                    record.GetType().GetProperty(Prop).SetValue(record, newValue, null);
                }
            }
        }

        private void gridControl1_CurrentCellEditingComplete(object sender, System.EventArgs e)
        {

        }

        /// <summary>
        /// Handles the CurrentCellValidated event of the gridControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void gridControl1_CurrentCellValidated(object sender, System.EventArgs e)
        {
            GridControl grid = sender as GridControl;
            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            if (grid.Model[currentRow, currentCol].CellType != "ComboBox")
            {
                ColomnHeaderInfo colInfo = ColInfo[currentCol] as ColomnHeaderInfo;
                RowHeaderInfo rowInfo = RowInfo[currentRow] as RowHeaderInfo;
                string keyField = colInfo.ID;
                object record = recordHash[keyField] as object;
                string Prop = rowInfo.PropertyName;
                string oldValue = grid.Model[currentRow, currentCol].FormattedText;
                string newValue = grid.Model[currentRow, currentCol].CellModel.GetActiveText(currentRow, currentCol).ToString();
                if (newValue != oldValue)
                {
                    record.GetType().GetProperty(Prop).SetValue(record, newValue, null);
                }
            }
        }

        /// <summary>
        /// Handles the CurrentCellShowingDropDown event of the gridControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridCurrentCellShowingDropDownEventArgs"/> instance containing the event data.</param>
        private void gridControl1_CurrentCellShowingDropDown(object sender, Syncfusion.Windows.Forms.Grid.GridCurrentCellShowingDropDownEventArgs e)
        {
            GridControl grid = sender as GridControl;
            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            ColomnHeaderInfo colInfo = ColInfo[currentCol] as ColomnHeaderInfo;
            RowHeaderInfo rowInfo = RowInfo[currentRow] as RowHeaderInfo;
            string prop = rowInfo.PropertyName;
            string keyField = colInfo.ID;
            object record = recordHash[keyField] as object;
            LRRHistData histDataObj = LRRModel.HistDatas.GetHistDataFromHistTagUID(keyField);
            string sUOMType = histDataObj.UOMTypeID;
            string sHistUomType = histDataObj.HistUOMTypeID;
            if (histDataObj != null)
            {
                if (prop == "InterfaceUnits")
                {
                    if (sUOMType != "" && sUOMType != null)
                    {
                        LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(sUOMType);
                        if (uoms.Count == 0)
                        {
                            LRRMessagBox.ShowMessage("SID_UOMNOTDEFINED", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            e.Cancel = true;
                        }
                    }
                    else
                    {
                        LRRMessagBox.ShowMessage("SID_UOMNOTDEFINED", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        e.Cancel = true;
                    }
                }
                if (prop == "HistUnits")
                {
                    if (sHistUomType != "" && sHistUomType != null)
                    {
                        LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(sHistUomType);
                        if (uoms.Count == 0)
                        {
                            LRRMessagBox.ShowMessage("SID_UOMNOTDEFINED", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            e.Cancel = true;
                        }
                    }
                    else
                    {
                        LRRMessagBox.ShowMessage("SID_UOMNOTDEFINED", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        e.Cancel = true;
                    }
                }
                if (prop == "HistUOMTypeID")
                {
                    if ((histDataObj.MeasType != "C" && histDataObj.MeasType != "N") && (histDataObj.MeasType != null && histDataObj.MeasType != ""))
                    {
                        e.Cancel = true;
                    }
                }
            }
        }

        /// <summary>
        /// Handles the CellButtonClicked event of the gridControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridCellButtonClickedEventArgs"/> instance containing the event data.</param>
        private void gridControl1_CellButtonClicked(object sender, Syncfusion.Windows.Forms.Grid.GridCellButtonClickedEventArgs e)
        {
            int currentRow = e.RowIndex;
            int currentCol = e.ColIndex;
            ColomnHeaderInfo colInfo = ColInfo[currentCol] as ColomnHeaderInfo;
            RowHeaderInfo rowInfo = RowInfo[currentRow] as RowHeaderInfo;
            string prop = rowInfo.PropertyName;
            string keyField = colInfo.ID;
            object record = recordHash[keyField] as object;
            if (prop == "FormulaObj" || prop == "ConversionFormulaObj" || prop == "CustomFormulaObj")
            {
                LRRFormula formula = null;
                if (prop == "FormulaObj")
                {
                    formula = record.GetType().GetProperty("FormulaObj").GetValue(record, null) as LRRFormula;
                    Symcon.LightRidge.BusinessObjects.TypeEditors.FormulaWindow fw = new Symcon.LightRidge.BusinessObjects.TypeEditors.FormulaWindow(formula);
                    fw.ShowDialog();
                }
                else if (prop == "ConversionFormulaObj")
                {
                    formula = record.GetType().GetProperty("ConversionFormulaObj").GetValue(record, null) as LRRFormula;
                    Symcon.LightRidge.BusinessObjects.TypeEditors.ConversionFormulaWindow cfw = new Symcon.LightRidge.BusinessObjects.TypeEditors.ConversionFormulaWindow(formula);
                    cfw.ShowDialog();
                }
                else
                {
                    formula = record.GetType().GetProperty("CustomFormulaObj").GetValue(record, null) as LRRFormula;
                    Symcon.LightRidge.BusinessObjects.TypeEditors.CustomFormulaWindow customFW = new Symcon.LightRidge.BusinessObjects.TypeEditors.CustomFormulaWindow(formula);
                    customFW.ShowDialog();
                }
                //initializeGrid(props, gridGroupControl);
                record = recordHash[keyField] as object;
                if (record != null)
                {
                    if (prop == "FormulaObj")
                    {                        
                        string Value = record.GetType().GetProperty("FormulaObj").GetValue(record, null) == null ? "" : record.GetType().GetProperty("FormulaObj").GetValue(record, null).ToString();
                        gridControl1[currentRow, currentCol].Text = Value;                        
                    }
                    if (prop == "ConversionFormulaObj")
                    {                        
                        string Value = record.GetType().GetProperty("ConversionFormulaObj").GetValue(record, null) == null ? "" : record.GetType().GetProperty("ConversionFormulaObj").GetValue(record, null).ToString();
                        gridControl1[currentRow, currentCol].Text = Value;
                    }
                    if (prop == "CustomFormulaObj")
                    {
                        string Value = record.GetType().GetProperty("CustomFormulaObj").GetValue(record, null) == null ? "" : record.GetType().GetProperty("CustomFormulaObj").GetValue(record, null).ToString();
                        gridControl1[currentRow, currentCol].Text = Value;
                    }
                    updateReadOnlyProperties(record, currentCol);
                }
            }
        }

        private void gridControl1_CellClick(object sender, Syncfusion.Windows.Forms.Grid.GridCellClickEventArgs e)
        {
           
        }
        /// <summary>
        /// Handles the CurrentCellKeyDown event of the gridControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
        void gridControl1_CurrentCellKeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == System.Windows.Forms.Keys.Delete)
                e.Handled = true;
        }
        /// <summary>
        /// Handles the CurrentCellKeyPress event of the gridControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyPressEventArgs"/> instance containing the event data.</param>
        void gridControl1_CurrentCellKeyPress(object sender, KeyPressEventArgs e)
        {
            GridControl grid = sender as GridControl;
            if (grid[grid.CurrentCell.RowIndex, grid.CurrentCell.ColIndex].CellType == "ComboBox")
            {
                e.Handled = true;
            }

            int currentRow = grid.Model.CurrentCellInfo.RowIndex;
            int currentCol = grid.Model.CurrentCellInfo.ColIndex;
            ColomnHeaderInfo colInfo = ColInfo[currentCol] as ColomnHeaderInfo;
            RowHeaderInfo rowInfo = RowInfo[currentRow] as RowHeaderInfo;
            string prop = rowInfo.PropertyName;
            if (prop == "FormulaObj" || prop == "ConversionFormulaObj" || prop == "CustomFormulaObj")
            {
                e.Handled = true;
            }
        }
    }
}
