using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Tools;
using System.Drawing;
using System.Collections;
using Symcon.Utils.MessageLogger;

namespace Symcon.LightRidge.PEAdvisor.Global
{
    /// <summary>
    /// Cares for selection/deselection and color change of TreeView nodes which contains CheckBoxes
    /// </summary>
    class TreeViewUtility
    {
        //public static bool checkLoop = false;
        /// <summary>
        /// Initializes a new instance of the <see cref="TreeViewUtility"/> class.
        /// </summary>
        public TreeViewUtility()
        {
        }

        /// <summary>
        /// Wireds the tree view.
        /// </summary>
        /// <param name="winTreeView">The win tree view.</param>
        /// <param name="syncTreeView">The sync tree view.</param>
        public static void WiredTreeView(TreeView winTreeView, TreeViewAdv syncTreeView)
        {
            if (winTreeView != null && winTreeView.GetType() == typeof(TreeView))
                winTreeView.AfterCheck += new TreeViewEventHandler(winTreeView_AfterCheck);


            else if (syncTreeView != null && syncTreeView.GetType() == typeof(TreeViewAdv))
                syncTreeView.AfterCheck += new TreeNodeAdvEventHandler(syncTreeView_AfterCheck);
        }
        
        /// <summary>
        /// Handles the AfterCheck event of the winTreeView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.TreeViewEventArgs"/> instance containing the event data.</param>
        static void winTreeView_AfterCheck(object sender, TreeViewEventArgs e)
        {
            try
            {
                TreeView trView = sender as TreeView;
                trView.BeginUpdate();

                TreeNode winTreeNode = e.Node;
                if (e.Action != TreeViewAction.Unknown && winTreeNode.Checked == true)
                {
                    CheckNodes(winTreeNode);
                }
                else if (e.Action != TreeViewAction.Unknown && winTreeNode.Checked == false)
                {
                    UncheckNodes(winTreeNode);
                }
                if (e.Action != TreeViewAction.Unknown)
                    TreeViewColorConversion(winTreeNode);

                trView.EndUpdate();
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(TreeView), "TreeOperation:" + ex.Message, Logger.Level.Debug);
            }
        }


        /// <summary>
        /// Handles the AfterCheck event of the syncTreeView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Tools.TreeNodeAdvEventArgs"/> instance containing the event data.</param>
        static void syncTreeView_AfterCheck(object sender, TreeNodeAdvEventArgs e)
        {        
            //Event handler for syncfusion TreeView
        }

        /// <summary>
        /// Checks all child nodes.
        /// </summary>
        /// <param name="winTreeNode">The win tree node.</param>
        public static void CheckNodes(TreeNode winTreeNode)
        {
            List<TreeNode> parentNodeList = new List<TreeNode>();
            GetParentList(winTreeNode, ref parentNodeList);
            foreach (TreeNode tnNode in parentNodeList)
            {
                if (AllChildCheck(tnNode))
                {
                    if (tnNode.Checked != winTreeNode.Checked)
                    {
                        tnNode.Checked = winTreeNode.Checked;
                    }
                }
            }
            foreach (TreeNode node in winTreeNode.Nodes)
            {
              node.Checked = winTreeNode.Checked;
              if(node.Nodes.Count > 0)
              {
                 // If the current node has child nodes, call the CheckAllChildsNodes method recursively.
                  CheckNodes(node);
              }
                
            }

        }

        /// <summary>
        /// Unchecks nodes.
        /// </summary>
        /// <param name="winTreeNode">The win tree node.</param>
        public static void UncheckNodes(TreeNode winTreeNode)
        {
            List<TreeNode> parentNodeList = new List<TreeNode>();
            GetParentList(winTreeNode, ref parentNodeList);
            foreach (TreeNode tnNode in parentNodeList)
            {
                if (tnNode.Checked != winTreeNode.Checked)
                {
                    tnNode.Checked = winTreeNode.Checked;
                }
            }
            foreach (TreeNode node in winTreeNode.Nodes)
            {
                node.Checked = winTreeNode.Checked;
                if (node.Nodes.Count > 0)
                {
                    // If the current node has child nodes, call the UncheckAllNodes method recursively.
                    UncheckNodes(node);
                }
            }
        }

        /// <summary>
        /// converts Treeview node color
        /// </summary>
        /// <param name="winTreeNode">The win tree node.</param>
        public static void TreeViewColorConversion(TreeNode winTreeNode)
        {
            if (winTreeNode.Checked == true)
            {
                winTreeNode.ForeColor = Color.Blue;
                List<TreeNode> parentNodeList = new List<TreeNode>();
                GetParentList(winTreeNode, ref parentNodeList);
                if (parentNodeList.Count != 0)
                {
                    foreach (TreeNode tn in parentNodeList)
                    {
                        tn.ForeColor = Color.Blue;
                        if (tn.Nodes.Count > 0)
                        {
                            ChangeColorForChecked(tn);
                        }
                    }
                }
                else
                {
                    ChangeColorForChecked(winTreeNode);
                }
            }
            else
            {
                winTreeNode.ForeColor = Color.Black;
                List<TreeNode> parentNodeList = new List<TreeNode>();
                GetParentList(winTreeNode, ref parentNodeList);
                if (winTreeNode.Nodes.Count > 0)
                {
                    ChangeColorForUnchecked(winTreeNode);

                }
                foreach(TreeNode tn in parentNodeList)
                {
                    bool boolVal = false;

                    if (!CheckForChildCheck(tn,ref boolVal))
                        tn.ForeColor = Color.Black;
                    else
                    {
                        return;
                    }
                }
            }
        }

        public static void TreeViewColorConversionOnLoad(TreeNode winTreeNode)
        {
            if (winTreeNode.Checked == true)
            {
                winTreeNode.ForeColor = Color.Blue;
                List<TreeNode> parentNodeList = new List<TreeNode>();
                GetParentList(winTreeNode, ref parentNodeList);
                if (parentNodeList.Count != 0)
                {
                    foreach (TreeNode tn in parentNodeList)
                    {
                        tn.ForeColor = Color.Blue;
                        if (tn.Nodes.Count > 0)
                        {
                            ChangeColorForChecked(tn);
                        }
                    }
                }
                else
                {
                    ChangeColorForChecked(winTreeNode);
                }
            }
            else
            {
                //winTreeNode.ForeColor = Color.Black;
                List<TreeNode> parentNodeList = new List<TreeNode>();
                GetParentList(winTreeNode, ref parentNodeList);
                if (winTreeNode.Nodes.Count > 0)
                {
                    ChangeColorForUnchecked(winTreeNode);
                    ChangeColorForChecked(winTreeNode);
                }
                foreach (TreeNode tn in parentNodeList)
                {
                    bool boolVal = false;

                    if (!CheckForChildCheck(tn, ref boolVal))
                        tn.ForeColor = Color.Black;
                    if (boolVal == true && tn.ForeColor == Color.Black)
                        tn.ForeColor = Color.Blue;

                    else
                    {
                        return;
                    }
                }
            }
        }
        public static void ChangeColorForChecked(TreeNode tn)
        {
            foreach (TreeNode trNode in tn.Nodes)
            {
                if (trNode.Checked == true)
                {
                    trNode.ForeColor = Color.Blue;
                    if (trNode.Nodes.Count > 0)
                        ChangeColorForChecked(trNode);
                }

              
            }
        }
        public static void ChangeColorForUnchecked(TreeNode tn)
        {
            foreach (TreeNode trNode in tn.Nodes)
            {
                if (trNode.Checked == false)
                {
                    trNode.ForeColor = Color.Black;
                    if (trNode.Nodes.Count > 0)
                        ChangeColorForUnchecked(trNode);
                }
            }
        }

        private static bool CheckForChildCheck(TreeNode tn,ref bool boolVal)
        {

            foreach (TreeNode trNode in tn.Nodes)
            {
                if (boolVal == true)
                    return boolVal;
                if (trNode.Nodes.Count == 0)
                {
                    if (trNode.Checked == true)
                    {
                        boolVal = true;
                        return boolVal;
                    }
                }
                else
                {
                     CheckForChildCheck(trNode,ref boolVal);
                }
                    
                    
            }
            return boolVal;
        }
        /// <summary>
        /// Gets the parent list.
        /// </summary>
        /// <param name="trNode">The tr node.</param>
        /// <param name="parentNodeList">The parent node list.</param>
        private static void GetParentList(TreeNode trNode,ref List<TreeNode> parentNodeList)
        {
            if (trNode.Parent != null)
            {
                TreeNode trParent = trNode.Parent;
                parentNodeList.Add(trParent);
                GetParentList(trParent, ref parentNodeList);
            }
        }

        /// <summary>
        /// Anies the child checked.
        /// </summary>
        /// <param name="tn">The tn.</param>
        /// <returns></returns>
        private static bool AnyChildChecked(TreeNode tn)
        {
            bool retVal = false;
            if (tn != null)
            {
                foreach (TreeNode tnNode in tn.Nodes)
                {
                    if (tnNode.Checked)
                    {
                        retVal = true;
                        return retVal;
                    }
                }
            }
            return retVal;
             
        }

        /// <summary>
        /// Alls the child check.
        /// </summary>
        /// <param name="tn">The tn.</param>
        /// <returns></returns>
        private static bool AllChildCheck(TreeNode tn)
        {
            bool retVal = true;
            foreach (TreeNode tnNode in tn.Nodes)
            {
                if (!tnNode.Checked)
                {
                    retVal = false;
                    return retVal;
                }
            }
            return retVal;
        }
    }
}
