using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using Symcon.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.LightRidge.BusinessObjects;

using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Symcon.LightRidge.HistorianConnect;
using Syncfusion.Grouping;


namespace Symcon.LightRidge.PEAdvisor
{
    public partial class TrendedPropertiesDialog : Form 
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="TrendedPropertiesDialog"/> class.
        /// </summary>
        public TrendedPropertiesDialog()
        {
            InitializeComponent();
            InternationalizationUtils.GetControlText(this, null);
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            this.Load += new EventHandler(TrendedPropertiesDialog_Load);
            gridTrendedProperties.SizeChanged += new EventHandler(gridTrendedProperties_SizeChanged);
            gridTrendedProperties.TableControl.MouseUp += new MouseEventHandler(TableControl_MouseUp);
        }

        /// <summary>
        /// Handles the Load event of the TrendedPropertiesDialog control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void TrendedPropertiesDialog_Load(object sender, EventArgs e)
        {
            InitializeTrendPropsGrid();
        }


        /// <summary>
        /// Initializes the trend props grid.
        /// </summary>
        private void InitializeTrendPropsGrid()
        {
            //lrrTrendedProps = LRRModel.TrendedProperties;

            lrrTrendedProps = new LRRTrendedProperties();

            _lrrUserSpecificTrendedProps = LRRModel.UserSpecificTrendedProperties;

            Hashtable RelativeWidths = new Hashtable();
            Hashtable columnForDisplay = new Hashtable();
            //Suspend the Layout of	the	Grid...
            gridTrendedProperties.SuspendLayout();
            SyncGridUtilities.InitializeGridLookAndFeel(gridTrendedProperties);
            //
            //			
            columnForDisplay.Add("Name", null);
            columnForDisplay.Add("IsTracked", null);
            columnForDisplay.Add("SiteComponentID",null);

            gridTrendedProperties.Engine.SourceListSet.Add("Parent", _lrrUserSpecificTrendedProps);
            gridTrendedProperties.Engine.SourceListSet.Add(LRRModel.SiteComponentsObj);
            //Populate the columns for the grid	object
            //SyncGridUtilities.InitializeGridColumns(gridTrendedProperties, lrrTrendedProps.ItemType, null, true, columnForDisplay);
            SyncGridUtilities.InitializeGridColumns(gridTrendedProperties, _lrrUserSpecificTrendedProps.ItemType, null, true, columnForDisplay);
            
            gridTrendedProperties.TableDescriptor.Columns["SiteComponentID_Name"].ReadOnly = true;
            gridTrendedProperties.TableDescriptor.Columns["SiteComponentID_Name"].Appearance.AnyRecordFieldCell.ExclusiveChoiceList = false;
            gridTrendedProperties.TableDescriptor.Columns["SiteComponentID_Name"].Width = 225;
            gridTrendedProperties.TableDescriptor.Columns["SiteComponentID_Name"].Appearance.AnyRecordFieldCell.ShowButtons = GridShowButtons.Hide;

            gridTrendedProperties.TopLevelGroupOptions.CaptionText = Symcon.Utils.HeaderDispatcher.LRRHeaderText.GetHeaderString("SID_TRENDED_PROPS") + " {RecordCount} " + Symcon.Utils.HeaderDispatcher.LRRHeaderText.GetHeaderString("SID_ITEMS");  


            //			GridColumnDescriptor gbc = new GridColumnDescriptor();
            //			GridStyleInfo gsi = gbc.Appearance.GetStyle(GridTableCellType.AnyRecordFieldCell);
            //			gsi.BaseStyle = "Standard";
            //			gsi.AutoSize = true;
            //			gsi.CellType = "ComboBox";
            //			gsi.VerticalScrollbar = true;
            //
            //			gsi.DisplayMember = "Name";
            //			gsi.ValueMember = "SiteComponentID";
            //			gsi.DataSource = LRRModel.SiteComponentsObj;
            //			gsi.ShowButtons = GridShowButtons.Hide;
            //			gbc.HeaderText = "Item Name";
            //			gbc.MappingName = "ComponentID";
            //			GridTableDescriptor gtc = gridRecordedValues.TableDescriptor;
            //			gtc.Columns.Add(gbc);


            RelativeWidths.Add("SiteComponentID_Name", 30);
            RelativeWidths.Add("Name", 30);
            RelativeWidths.Add("IsTracked", 15);
            SyncGridUtilities.InitializeGridColumnsWidth(gridTrendedProperties, RelativeWidths);

            //Set the data store 
            gridTrendedProperties.DataSource = _lrrUserSpecificTrendedProps;

            gridTrendedProperties.TableDescriptor.AllowNew = false;
            gridTrendedProperties.TableDescriptor.AllowRemove = false;
            gridTrendedProperties.TableDescriptor.AllowEdit = true;

            gridTrendedProperties.TableDescriptor.Columns["Name"].ReadOnly = true;
            gridTrendedProperties.TableDescriptor.Columns["Name"].Width = 161;

            //gridRecordedValues.TableDescriptor.Columns["IsTracked"].Width = 20;
            
           
            gridTrendedProperties.TopLevelGroupOptions.ShowFilterBar = true;;
           // gridRecordedValues_SizeChanged(gridRecordedValues, new EventArgs());
            gridTrendedProperties.ResumeLayout();
            _unBound.WireGrid(gridTrendedProperties);
            /*
            object[] objArray = new object[lrrTrendedProps.Count];
            int k = 0;
            foreach (LRRTrendedProperty  _trendProps in lrrTrendedProps)
            {
                if (_trendProps.IsTrended)
                {
                    objArray[k] = _trendProps;
                    k++;
                }
            }
             * */
            object[] objArray = new object[_lrrUserSpecificTrendedProps.Count];
            int k = 0;
            foreach (LRRUserSpecificTrendedProperty userSpTrProp in _lrrUserSpecificTrendedProps)
            {
                objArray[k] = userSpTrProp;
                k++;
            }
            _unBound.SetSelectedRecords(objArray);

        }
        protected override void OnClosing(CancelEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
            _isClose = true;
            _lrrApp.MainFrame.Activate();
            _lrrApp.LRRUIManagerObj.IsTrendedPropsWindowActive = false;
			   if(_lrrApp.LRRUIManagerObj.MainFrameObj.TabbedMDIChildList.TabCount==0)
				  {
					  _lrrApp.LRRUIManagerObj.MainFrameObj.tlbExport.Enabled=false;
				  }
				  else
				  {
					 _lrrApp.LRRUIManagerObj.MainFrameObj.tlbExport.Enabled=true;
				  }

        }

        protected override void OnActivated(EventArgs e)
        {
            /*
            if (_isClose)
            {
                object[] _recObj = new object[lrrTrendedProps.Count];
                if (lrrTrendedProps.Count > 0)
                {
                    for (int i = 0; i < lrrTrendedProps.Count; i++)
                    {
                        _recObj[i] = lrrTrendedProps[i];
                    }
                }
                _unBound.SetSelectedRecords(_recObj, true);

            }
			*/
            if (_isClose)
            {
                object[] trendedObj = new object[_lrrUserSpecificTrendedProps.Count];
                if (_lrrUserSpecificTrendedProps.Count > 0)
                {
                    for (int i = 0; i < _lrrUserSpecificTrendedProps.Count; i++)
                    {
                        trendedObj[i] = _lrrUserSpecificTrendedProps[i];
                    }
                    _unBound.SetSelectedRecords(trendedObj, true);
                }
            }

        }

        /// <summary>
        /// Raises the <see cref="E:System.Windows.Forms.Form.Deactivate"></see> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"></see> that contains the event data.</param>
        protected override void OnDeactivate(EventArgs e)
        {
            /*
            if (this.Visible == false)
            {
                IEnumerator iEnum = lrrTrendedProps.GetEnumerator();
                LRRSiteComponentProperties selectedSCPs = new LRRSiteComponentProperties();
                while (iEnum.MoveNext())
                {
                    LRRTrendedProperty trendProps = iEnum.Current as LRRTrendedProperty;
                    foreach (LRRSiteComponentProperty siteCompProperty in LRRModel.SiteComponentPropertiesObj)
                    {
                        if (siteCompProperty.ArrListUpdatedColNames.Contains("IsTrended"))
                        {
                            selectedSCPs.Add(siteCompProperty);
                        }
                    }
                }
                _lrrApp.DBManagerObj.ExecuteBatchUpdate(selectedSCPs);
            }
            */
            LRRSiteComponentProperties selectedSCPs = new LRRSiteComponentProperties();
            LRRSiteComponentProperties scps = LRRModel.SiteComponentPropertiesObj.GetChanges();
            foreach (LRRSiteComponentProperty scp in scps)
            {
                if (scp.ArrListUpdatedColNames.Contains("IsTrended"))
                {
                    selectedSCPs.Add(scp);
                }
            }
            if (removedUserSpecificTrendedProps.Count > 0)
            {
                _lrrApp.DBManagerObj.CommitCollection(removedUserSpecificTrendedProps);
            }
            if (selectedSCPs.Count != 0)
            {
                _lrrApp.DBManagerObj.ExecuteBatchUpdate(selectedSCPs);
            }

            base.OnDeactivate(e);
        }

        #region Fields

        private CustomCollection _lrrTrendedVals;
        private LRRApp _lrrApp;
        private LRRTrendedProperties  lrrTrendedProps;
        private GridUnBoundColumn _unBound = new GridUnBoundColumn();
        private bool _isClose = false;
        private LRRUserSpecificTrendedProperties _lrrUserSpecificTrendedProps;
        #endregion



        #region Grid Events
        /// <summary>
        /// Handles the MouseUp event of the TableControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        void TableControl_MouseUp(object sender, MouseEventArgs e)
        {
            int row, col;
            GridTableControl gridTableControl = sender as GridTableControl;
            gridTableControl.PointToRowCol(new Point(e.X, e.Y), out row, out col);
            GridTableCellStyleInfo style = gridTableControl.Model[row, col];

            IMouseController controller;
            gridTableControl.MouseControllerDispatcher.HitTest(new Point(e.X, e.Y), e.Button, e.Clicks, out controller);

            if (controller != null && controller.Name == "DragGroupHeader" && style.CellType == "CheckBox")
            {
                gridTableControl.GetCellRenderer(row, col).RaiseMouseUp(row, col, e);
            }
        }

        /// <summary>
        /// Handles the SizeChanged event of the gridTrendedProperties control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void gridTrendedProperties_SizeChanged(object sender, EventArgs e)
        {
            int totalColsWidth = 0;

            foreach (GridColumnDescriptor gcd in this.gridTrendedProperties.TableDescriptor.Columns)
            {
                totalColsWidth += gcd.Width;
            }

            float nPercent = (float)(this.gridTrendedProperties.Width - (5 + (new VScrollBar()).Width)) / (float)totalColsWidth;

            foreach (GridColumnDescriptor gcd in this.gridTrendedProperties.TableDescriptor.Columns)
            {
                gcd.Width = (int)(gcd.Width * nPercent + 0.5);
                if (gcd.Width == 0)
                    gcd.Width = 1;
            } 
        }

        /// <summary>
        /// Handles the TableControlQueryAllowSortColumn event of the gridTrendedProperties control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridQueryAllowSortColumnEventArgs"/> instance containing the event data.</param>
        private void gridTrendedProperties_TableControlQueryAllowSortColumn(object sender, GridQueryAllowSortColumnEventArgs e)
        {
            if (e.Column.Name == "Select")
            {
                e.AllowSort = false;
            }
        }
        int row, col;
        bool CheckBoxValue = true;
        /// <summary>
        /// Handles the QueryCellStyleInfo event of the gridTrendedProperties control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableCellStyleInfoEventArgs"/> instance containing the event data.</param>
        private void gridTrendedProperties_QueryCellStyleInfo(object sender, GridTableCellStyleInfoEventArgs e)
        {
            e.Style.TableCellIdentity.Table.TableDescriptor.ColumnToRowColIndex("Select", out row, out col);
            if (e.TableCellIdentity.TableCellType == GridTableCellType.ColumnHeaderCell && e.TableCellIdentity.ColIndex == col + 1)
            {
                e.Style.CellType = "CheckBox";
                e.Style.ReadOnly = false;
                e.Style.CellAppearance = GridCellAppearance.Raised;
                e.Style.CellValue = CheckBoxValue;
                e.Style.CellValueType = typeof(bool);
                e.Style.Enabled = true;
            }
            e.Handled = true;
        }

        /// <summary>
        /// Handles the SaveCellText event of the gridTrendedProperties control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridCellTextEventArgs"/> instance containing the event data.</param>
        LRRUserSpecificTrendedProperties removedUserSpecificTrendedProps = new LRRUserSpecificTrendedProperties();
        private void gridTrendedProperties_SaveCellText(object sender, GridCellTextEventArgs e)
        {
            DialogResult dr = MessageBox.Show("This will reset all the trended attributes. Would you like to continue?", " ", MessageBoxButtons.YesNo);
            if (dr.ToString() == "Yes")
            {
                // user specific part added by ashwin
                GridTableCellStyleInfo style = (GridTableCellStyleInfo)e.Style;
                if (style.TableCellIdentity.TableCellType == GridTableCellType.ColumnHeaderCell && e.Style.CellType == "CheckBox")
                {
                    this.CheckBoxValue = bool.Parse(e.Text);
                    foreach (LRRUserSpecificTrendedProperty uspT in _lrrUserSpecificTrendedProps)
                    {
                        LRRUserSpecificTrendedProperty modelUsp = LRRModel.UserSpecificTrendedProperties.GetUserSpecificTrendedPropertyByUIDAndSCPID(_lrrApp.UID, uspT.SiteComponentPropertyID);
                        if (modelUsp != null)
                        {
                            modelUsp.IsTracked = false;
                            modelUsp.UID = _lrrApp.UID;
                            modelUsp.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                            removedUserSpecificTrendedProps.Add(modelUsp);
                        }
                    }
                    if (removedUserSpecificTrendedProps.Count > 0)
                    {
                        _lrrApp.DBManagerObj.CommitCollection(removedUserSpecificTrendedProps);
                    }
                    foreach (LRRUserSpecificTrendedProperty uspT in removedUserSpecificTrendedProps)
                    {
                        _lrrUserSpecificTrendedProps.Remove(uspT);
                    }
                    /*
                    
                    IEnumerator iEnum = lrrTrendedProps.GetEnumerator();
                    ArrayList selectedRecords = new ArrayList();
                    while (iEnum.MoveNext())
                    {
                        LRRTrendedProperty _trendProps = iEnum.Current as LRRTrendedProperty;
                        if (_trendProps != null)
                        {
                            selectedRecords.Add(_trendProps);
                        }
                    }
                    object[] objSelectedRecords = new object[selectedRecords.Count];
                    for (int i = 0; i < selectedRecords.Count; i++)
                    {
                        LRRTrendedProperty trendProp = selectedRecords[i] as LRRTrendedProperty;
                        trendProp.IsTrended = false;
                    }
                    //_unBound.SetSelectedRecords(objSelectedRecords,CheckBoxValue);  
                    gridTrendedProperties.Refresh();
                    e.Handled = true;
                     * */
                }
                else
                {
                    GridGroupingControl _grid = sender as GridGroupingControl;
                    LRRUserSpecificTrendedProperty userTrProp = _grid.CurrencyManager.Current as LRRUserSpecificTrendedProperty;
                    if (userTrProp != null)
                    {
                        LRRUserSpecificTrendedProperty modelUserTrProp = LRRModel.UserSpecificTrendedProperties.GetUserSpecificTrendedPropertyByUIDAndSCPID(_lrrApp.UID, userTrProp.SiteComponentPropertyID);
                        if (modelUserTrProp != null)
                        {
                            modelUserTrProp.IsTracked = false;
                            modelUserTrProp.UID = _lrrApp.UID;
                            modelUserTrProp.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                            removedUserSpecificTrendedProps.Add(modelUserTrProp);
                            LRRModel.UserSpecificTrendedProperties.Remove(modelUserTrProp);
                        }
                    }
                }
            }
        }


        #endregion

        //private void viewTrendToolStripMenuItem_Click(object sender, EventArgs e)
        //{
        //    Element eCurrentElement = gridTrendedProperties.TableControl.Table.CurrentElement;
        //    GridRecord gnt = eCurrentElement as GridRecord;
        //    LRRTrendedProperty lrrTrendedProp = eCurrentElement.GetRecord().GetData() as LRRTrendedProperty;
        //    if (lrrTrendedProp != null)
        //    {
                
        //    }
        //}s
    }
}