using System;
using System.Reflection;
using System.Collections;
using System.Diagnostics;
using Symcon.Utils.MessageLogger;
using Symcon.BusinessObjects;
using Syncfusion.Windows.Forms.Grid;


//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// GridCellStyleHelper is a utilitily class
	/// which is used to format the cells of the
	/// grid at runtime.The relative cells of
	/// the grid get Manipulated here.
	/// </summary>
	public class GridRelatedCellStyleHelper
	{
		/// <summary>
		/// Constructor takes the type to extract the 
		/// properties of the target type of object
		/// </summary>
		/// <param name="type">Type of the object to which it has to hook up
		/// e.g LRRMasterProperty to hook up for the UOM Field depending
		/// on the UOM Type.
		/// </param>
		public GridRelatedCellStyleHelper()
		{
			
		}

		/// <summary>
		/// Wires the grid control to change the
		/// style of the cells at Runtime.
		/// </summary>
		/// <param name="grid">Intsance of the Grid</param>
		/// <param name="type"></param>
		/// <returns>true if success</returns>
		public bool WireGrid(GridDataBoundGrid grid,Type type)
		{
			try
			{
				if(this._grid != grid)
				{
					if(this._grid != null)
						UnWireGrid();

					this._grid = grid;

					_startRowIndex = _grid.Model.Rows.HeaderCount;

					StoreTypeInfoHash(type);

					StoreColumnsInfo();
					
					//Map to the Query Cell info event of the grid..
					this._grid.Model.QueryCellInfo+=new GridQueryCellInfoEventHandler(Model_QueryCellInfo);
					//Hook up to the event when the cols have been moved by the user or pogramatically...
					this._grid.Model.ColsMoved+=new GridRangeMovedEventHandler(Model_ColsMoved);
					return true;
				}
				else
				{
					return false;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GridRelatedCellStyleHelper),"WireGrid:"+ex.Message  ,Logger.Level.Debug);
				UnWireGrid();
				return false;
			}

		}

		/// <summary>
		/// Unwires the grid and hooked up events
		/// </summary>
		/// <returns>true if success</returns>
		public bool UnWireGrid()
		{
			try
			{
				//unMap to the Query Cell info event of the grid..
				this._grid.Model.QueryCellInfo-=new GridQueryCellInfoEventHandler(Model_QueryCellInfo);
				//UnHook up to the event when the cols have been moved by the user or pogramatically...
				this._grid.Model.ColsMoved-=new GridRangeMovedEventHandler(Model_ColsMoved);
				this._grid = null;
				return true;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GridRelatedCellStyleHelper),"UnWireGrid:"+ex.Message  ,Logger.Level.Debug);
				
				return false;
			}
		}

		/// <summary>
		/// Stores the hash for the name to column mapping
		/// </summary>
		private void StoreColumnsInfo()
		{
			if(_grid == null)
				return;
			_allColIndexInfo.Clear();
			_relativeFieldNamedIndex.Clear();
			//Get the grid Data Bound Collection...
			for(int i=0;i<_grid.GridBoundColumns.Count;i++)
			{
				if(_relativeFieldInfo.ContainsKey(_grid.GridBoundColumns[i].MappingName)) //This is a relative column
				{
					_relativeFieldNamedIndex.Add(i + 1,_grid.GridBoundColumns[i].MappingName); //Add it to the has to look up for the relative field
				}
				_allColIndexInfo.Add(_grid.GridBoundColumns[i].MappingName,i + 1);
			}
		}

        /// <summary>
        /// Stores the type info hash.
        /// </summary>
        /// <param name="type">The type.</param>
		private void StoreTypeInfoHash(Type type)
		{
			_relativeFieldInfo.Clear();
			//Get Meta Data for the type the instance is associated with..
			//get All the properties...
			PropertyInfo[] properties = type.GetProperties();
			foreach (PropertyInfo property in properties)
			{
				SymRelativeDataAttribute[] field = (SymRelativeDataAttribute[])
					property.GetCustomAttributes(typeof(SymRelativeDataAttribute),
					false);
				if(field.Length > 0) //Found Attribute
				{
					SymRelativeDataAttribute sRDA = field[0];
					_relativeFieldInfo.Add(property.Name,sRDA);
				}
			}
		}

		#region Event Handlers

        /// <summary>
        /// Change the data source of the related cells..
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridQueryCellInfoEventArgs"/> instance containing the event data.</param>
		private void Model_QueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
		{
			if(e.RowIndex > _startRowIndex)
			{
				try
				{
					if(_relativeFieldNamedIndex.ContainsKey(e.ColIndex)) 
					{
						SymRelativeDataAttribute sRDA = _relativeFieldInfo[_relativeFieldNamedIndex[e.ColIndex]] as SymRelativeDataAttribute;
						CustomCollection cust  = Symcon.LightRidge.PEAdvisor.DataStore.LRRDataStore.GetDataStoreObjectCopy(sRDA.TargetObjectType) as CustomCollection;
						cust.ItemFilter = "";
						try
						{
							if(_grid[e.RowIndex,Int32.Parse(_allColIndexInfo[sRDA.SourcePropertyName].ToString(),Symcon.BusinessObjects.SymConverter.Culture)].CellValue.ToString() != "")
							{
								//_grid.Model[e.RowIndex,Int32.Parse(_allColIndexInfo[sRDA.SourcePropertyName].ToString(Symcon.BusinessObjects.SymConverter.Culture))].CellValue = null;
								cust.ItemFilter = sRDA.SourcePropertyName + " = " + _grid[e.RowIndex,Int32.Parse(_allColIndexInfo[sRDA.SourcePropertyName].ToString(),Symcon.BusinessObjects.SymConverter.Culture)].CellValue.ToString();
								e.Style.DataSource = cust;
								e.Style.DisplayMember = sRDA.DisplayMemberName;
								e.Style.ValueMember = sRDA.ValueMemberName;
								//e.Style.ResetDisplayMember();
								e.Handled = true;
							}
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridRelatedCellStyleHelper),"Model_QueryCellInfo:"+ex.Message,Logger.Level.Debug);
						}
					}
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(GridRelatedCellStyleHelper),"Model_QueryCellInfo:"+ex.Message,Logger.Level.Debug);
				}
			}
		}

		/// <summary>
		/// Event handler for the movement of the cols on the grid...
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Model_ColsMoved(object sender, GridRangeMovedEventArgs e)
		{
			StoreColumnsInfo();
		}

		#endregion

		#region Fields
		private GridDataBoundGrid 			_grid = null;
		private Hashtable					_relativeFieldInfo = new Hashtable();
		private Hashtable					_relativeFieldNamedIndex = new Hashtable();
		private Hashtable					_allColIndexInfo = new Hashtable();
		private int							_startRowIndex = -1;

		#endregion
	}
}
