using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.PEAdvisor.CommonViews;
using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.PEAdvisor.GroupProperties;
using Symcon.LightRidge.PEAdvisor.IOScreen;
using Symcon.Utils.HeaderDispatcher;
using Syncfusion.Windows.Forms.Tools;


namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
    class GridCheckForViewChanges
    {

        /// <summary>
        /// This Method Checks For View Changes Before Form Close and Run.
        /// </summary>
        public static void CheckForViewChanges(bool isChanged, IUnifiedGridFactory gridView, IGroupGridFactory gridViewForGPP, Form frm, IGridFactory ioGridView, ResultPropertiesWindow rsProperties, bool isLastQuerySaved, ComboBox cmbOpen,DockingManager dockingManager1,Panel pnlFilterSection, LRRApp lrrApp,bool fromRun)
        {
            GridCheckForViewChanges objGridCheckForViewChanges = new GridCheckForViewChanges();
            string formName = frm.Text;

            if (formName == LRRHeaderText.GetHeaderString("SID_UNIFIED_VIEWS"))
            {
                objGridCheckForViewChanges.internalCheckForViewChanges(isChanged, gridView, frm,fromRun);
            }
            else if (formName == LRRHeaderText.GetHeaderString("300") || formName == LRRHeaderText.GetHeaderString("400"))
            {
                objGridCheckForViewChanges.internalCheckForViewChanges(isChanged, frm, gridViewForGPP, fromRun);
            }
            else if (formName == LRRHeaderText.GetHeaderString("200"))
            {
                objGridCheckForViewChanges.internalCheckForViewChanges(isChanged, ioGridView, frm, rsProperties, isLastQuerySaved, cmbOpen, dockingManager1, pnlFilterSection, lrrApp, fromRun);
            }

        }

        /// <summary>
        /// Internals the check for view changes.
        /// </summary>
        /// <param name="isChanged">if set to <c>true</c> [is changed].</param>
        /// <param name="gridView">The grid view.</param>
        /// <param name="frm">The FRM.</param>
        private void internalCheckForViewChanges(bool isChanged, IUnifiedGridFactory gridView, Form frm,bool fromRun)
        {
            Symcon.LightRidge.PEAdvisor.CommonViews.SavedViewForm objSavedViewForm = frm as Symcon.LightRidge.PEAdvisor.CommonViews.SavedViewForm;
            if (isChanged)
            {
                if (!fromRun)
                {
                    DialogResult k = LRRMessagBox.ShowMessage("SID_SAVE_QUERY", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (k == DialogResult.Yes)
                    {
                        SaveQueryForm saveQueryForm = new SaveQueryForm(null, "Query");
                        InternationalizationUtils.GetControlText(saveQueryForm, null);
                        DialogResult result = saveQueryForm.ShowDialog();
                        if (result == DialogResult.OK)
                        {

                            gridView.Save(saveQueryForm.queryName, saveQueryForm.queryDesc, saveQueryForm.SelectedLevel.LevelID);
                            LRRMessagBox.ShowMessage("SID_SAVE_VIEW_SUCCESSFULLY", true);
                            isChanged = false;
                        }
                    }
                }
            }
            LRRSiteComponentPropertyValues toSaveSiteCompPropValues = LRRModel.PropertyValuesObj.GetChanges();
            LRRAggregatePropertyValues toSaveAggrPropValues = LRRModel.AggregatePropertyValuesObj.GetChanges();
            LRRSiteGroupPropertyQualifierValues toSaveGPValues = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();

            // rollback part added by ashwin
            LRROverrides toSavePVOverrides = LRRModel.ChildModel.Overrides.GetChanges();
            LRRGroupOverrides toSaveQVOverrides = LRRModel.ChildModel.GroupOverrides.GetChanges();
            if ((toSaveSiteCompPropValues.Count > 0) || (toSaveAggrPropValues.Count > 0) || (toSaveGPValues.Count > 0))
            {
                DialogResult dlgResult = LRRMessagBox.ShowMessage(frm, "SID_SAVE_PROPVALUE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dlgResult == DialogResult.Yes)
                {
                    objSavedViewForm.Save();
                }
                else
                {
                    foreach (LRRSiteComponentPropertyValue lscpv in toSaveSiteCompPropValues)
                    {
                        LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                        LRRSiteComponentPropertyValue lscpvChanged = LRRModel.GetPropertyValueObject(typeof(LRRSiteComponent), lscpv.ModeID, lscp.ID, lscp.ComponentID) as LRRSiteComponentPropertyValue;
                        lscpvChanged.RollBackChanges(true);
                        // start of changes by ashwin
                        LRROverride modelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(lscpv.SiteComponentPropertyID, lscpv.ModeID);
                        if (modelOverride != null)
                        {
                            modelOverride.RollBackChanges(true);
                            LRRModel.ChildModel.Overrides.Remove(modelOverride);
                        }
                        // end of changes by ashwin
                    }
                    // refreshcollection for property values and overides added by ashwin
                    LRRModel.PropertyValuesObj.RefreshCollection();
                    LRRModel.ChildModel.Overrides.RefreshCollection();

                    foreach (LRRSiteGroupPropertyQualifierValue lscgpv in toSaveGPValues)
                    {
                        LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscgpv.SiteComponentPropID);
                        if (siteCompProp != null)
                        {
                            string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompProp.ID, lscgpv.LookUpPropertiesID);
                            LRRLookUpQualifier lkUpQualObj = null;
                            if (lscgpv.LookUpQualifierID != null && lscgpv.LookUpQualifierID.Trim() != "")
                                lkUpQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lscgpv.LookUpQualifierID);
                            string qualID = lkUpQualObj == null ? null : lkUpQualObj.QualifierID;
                            LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp, lscgpv.LookUpPropertiesID, lscgpv.QualifierID, lscgpv.ModeID);
                            if (qualValueObj != null)
                            {
                                qualValueObj.RollBackChanges(true);
                            }
                            // start of changes by ashwin
                            LRRGroupOverride modelOverride = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(lscgpv.SiteGroupPropertyQualifierValueID);
                            if (modelOverride != null)
                            {
                                lscgpv.SetOverrideState(false);
                                lscgpv.SetPropertyValue(lscgpv.GetActualOverridedValue());
                                modelOverride.RollBackChanges(true);
                                LRRModel.ChildModel.GroupOverrides.Remove(modelOverride);
                            }
                            // end of changes by ashwin

                        }
                    }
                    // refreshcollection for Qualifier values and overides added by ashwin
                    LRRModel.SiteGroupPropertyQualifierValuesObj.RefreshCollection();
                    LRRModel.ChildModel.GroupOverrides.RefreshCollection();

                    foreach (LRRAggregatePropertyValue aggPropVal in toSaveAggrPropValues)
                    {
                        aggPropVal.RollBackChanges(true);
                    }
                    // rollback part for overrides added by ashwin
                    /*
                    foreach (LRROverride overriObj in toSavePVOverrides)
                    {
                        LRROverride modelOverrideObj = LRRModel.ChildModel.Overrides.GetOverrideObjByID(overriObj.ID);
                        if (modelOverrideObj != null)
                        {
                            LRRModel.ChildModel.Overrides.Remove(modelOverrideObj);
                            modelOverrideObj.RollBackChanges(true);
                        }
                    }
                    foreach (LRRGroupOverride groupOverride in toSaveQVOverrides)
                    {
                        LRRGroupOverride modelQVOverride = LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(groupOverride.SiteGroupPropertyQualifierValueID);
                        if (modelQVOverride != null)
                        {
                            LRRModel.ChildModel.GroupOverrides.Remove(modelQVOverride);
                            modelQVOverride.RollBackChanges(true);
                        }
                    }
                    */
                }
            }
        }
        /// <summary>
        /// Internals the check for view changes.
        /// </summary>
        /// <param name="isChanged">if set to <c>true</c> [is changed].</param>
        /// <param name="frm">The FRM.</param>
        /// <param name="gridView">The grid view.</param>
        private void internalCheckForViewChanges(bool isChanged,Form frm, IGroupGridFactory gridView,bool fromRun)
        {
            if (frm.Text == "Plant Item Attributes (Data Arrays)")
            {
                GroupedPlantDataArrayView groupedPDA = frm as GroupedPlantDataArrayView;
                if (gridView != null)
                {
                    gridView.ClearAllBindings();
                    gridView.UnSubscribeReconnectCompleted();
                }


                if (isChanged)
                {
                    if (!fromRun)
                    {
                        DialogResult k = LRRMessagBox.ShowMessage("SID_SAVE_QUERY", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (k == DialogResult.Yes)
                        {
                            groupedPDA.SaveQuery();
                        }
                    }
                }

                // Save For Property Values
                LRRSiteGroupPropertyQualifierValues propertyQualifierValues = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();

                if (propertyQualifierValues.Count > 0)
                {
                    DialogResult dlgResult = LRRMessagBox.ShowMessage(groupedPDA, "SID_SAVE_PROPVALUE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dlgResult == DialogResult.Yes)
                    {
                        groupedPDA.Save();
                    }
                    else
                    {
                        foreach (LRRSiteGroupPropertyQualifierValue propertyQualifierValue in propertyQualifierValues)
                        {
                            LRRSiteComponentProperty objProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(propertyQualifierValue.SiteComponentPropID);
                            string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(propertyQualifierValue.SiteComponentPropID, propertyQualifierValue.LookUpPropertiesID);
                            LRRSiteGroupPropertyQualifierValue objPropValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(groupPropID, propertyQualifierValue.LookUpPropertiesID, propertyQualifierValue.QualifierID, propertyQualifierValue.ModeID);
                            objPropValue.RollBackChanges(true);
                        }
                    }
                }
            }
            else
            {
                GroupedPlantPropertiesView objGPPV = frm as GroupedPlantPropertiesView;
                //Clear All Bindings
                if (gridView != null)
                {
                    gridView.ClearAllBindings();
                    gridView.UnSubscribeReconnectCompleted();
                }

                if (isChanged)
                {
                    if (!fromRun)
                    {
                        DialogResult k = LRRMessagBox.ShowMessage("SID_SAVE_QUERY", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (k == DialogResult.Yes)
                        {
                            objGPPV.SaveQuery();
                        }
                    }
                }

                // Save For Property Values
                LRRSiteGroupPropertyQualifierValues propertyQualifierValues = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges();

                if (propertyQualifierValues.Count > 0)
                {
                    DialogResult dlgResult = LRRMessagBox.ShowMessage(objGPPV, "SID_SAVE_PROPVALUE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dlgResult == DialogResult.Yes)
                    {
                        objGPPV.Save();
                    }
                    else
                    {
                        foreach (LRRSiteGroupPropertyQualifierValue propertyQualifierValue in propertyQualifierValues)
                        {
                            LRRSiteComponentProperty objProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(propertyQualifierValue.SiteComponentPropID);
                            string groupPropID = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(propertyQualifierValue.SiteComponentPropID, propertyQualifierValue.LookUpPropertiesID);
                            LRRSiteGroupPropertyQualifierValue objPropValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(groupPropID, propertyQualifierValue.LookUpPropertiesID, propertyQualifierValue.QualifierID, propertyQualifierValue.ModeID);
                            objPropValue.RollBackChanges(true);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Internals the check for view changes.
        /// </summary>
        /// <param name="isChanged">if set to <c>true</c> [is changed].</param>
        /// <param name="iGridView">The i grid view.</param>
        /// <param name="frm">The FRM.</param>
        /// <param name="rsProperties">The rs properties.</param>
        /// <param name="isLastQuerySaved">if set to <c>true</c> [is last query saved].</param>
        /// <param name="cmbOpen">The CMB open.</param>
        /// <param name="dockingManager1">The docking manager1.</param>
        /// <param name="pnlFilterSection">The PNL filter section.</param>
        /// <param name="lrrApp">The LRR app.</param>
        private void internalCheckForViewChanges(bool isChanged, IGridFactory iGridView, Form frm, ResultPropertiesWindow rsProperties, bool isLastQuerySaved, ComboBox cmbOpen, DockingManager dockingManager1, Panel pnlFilterSection, LRRApp lrrApp,bool fromRun)
        {
            InputOutputViews objIOViews = frm as InputOutputViews;
            if(iGridView != null)
			{
				iGridView.ClearAllBindings();
				iGridView.UnSubscribeReconnectCompleted();
			}
			rsProperties.Dispose();

            if (isChanged)
			{
                if (!fromRun)
                {
                    DialogResult k = LRRMessagBox.ShowMessage("SID_SAVE_QUERY", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (k == DialogResult.Yes)
                    {
                        objIOViews.SaveQuery();
                    }
                }
                
			}
			
			//Save the Property Values
			LRRSiteComponentPropertyValues toSaveSiteCompPropValues = LRRModel.PropertyValuesObj.GetChanges();
			LRRAggregatePropertyValues toSaveAggrPropValues = LRRModel.AggregatePropertyValuesObj.GetChanges();
			if(toSaveSiteCompPropValues.Count > 0)
			{
				DialogResult dlgResult = LRRMessagBox.ShowMessage(objIOViews,"SID_SAVE_PROPVALUE","SID_LABEL",MessageBoxButtons.YesNo,MessageBoxIcon.Question);      
				if(dlgResult == DialogResult.Yes)  
				{
                    lrrApp.DBManagerObj.CommitCollection(toSaveSiteCompPropValues);
                    LRRModel.PropertyValuesObj.RefreshCollection();
					if(toSaveAggrPropValues.Count > 0)
                        lrrApp.DBManagerObj.CommitCollection(toSaveAggrPropValues);
                    LRRMessagBox.ShowMessage("SID_SAVEDINFO", true);
				}
                else if (dlgResult == DialogResult.No)
                {
                    foreach(LRRSiteComponentPropertyValue lscpv in toSaveSiteCompPropValues)
                    {
                        LRRSiteComponentProperty lscp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lscpv.SiteComponentPropertyID);
                        LRRSiteComponentPropertyValue lscpvObj = LRRModel.GetPropertyValueObject(typeof(LRRSiteComponent), lscpv.ModeID, lscpv.SiteComponentPropertyID, lscp.ComponentID) as LRRSiteComponentPropertyValue; 
                        lscpvObj.RollBackChanges(true);
                    }
                }
			}

            
			if(!isLastQuerySaved)
                LRRApplicationSettings.SSelectedQuery = cmbOpen.SelectedIndex;

            dockingManager1.SetAutoHideMode(pnlFilterSection, true);
            dockingManager1.SaveDockState();
            
            ApplicationPersistanceUtil.SaveGridSettings(pnlFilterSection, objIOViews, 200);
        }
    }
}
