using System;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;
using System.Collections;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Drawing;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// Summary description for GridControlPrint.
	/// </summary>
	public class GridControlPrint
	{
        /// <summary>
        /// Initializes a new instance of the <see cref="GridControlPrint"/> class.
        /// </summary>
		public GridControlPrint()
		{
			//Get the App Object
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

			timeWnd = _lrrApp.TimeWindowObj;
			if(timeWnd != null)
			{
				string _refValue = "[" + timeWnd.ModeSlotsObj.RefMode.StartTime + " To " + timeWnd.ModeSlotsObj.RefMode.EndTime + "]";  
				string _cmpr1Value = "[" + timeWnd.ModeSlotsObj.Compare1Mode.StartTime + " To " + timeWnd.ModeSlotsObj.Compare1Mode.EndTime + "]";  
				string _cmpr2Value = "[" + timeWnd.ModeSlotsObj.Compare2Mode.StartTime + " To " + timeWnd.ModeSlotsObj.Compare2Mode.EndTime + "]";  
				footer = "Ref "+ _refValue +"\n"+"Cmpr1 "+_cmpr1Value +"\n"+"Cmpr2 "+ _cmpr2Value;
			}
			printDocument1 = new PrintDocument();
			printDocument1.PrintPage += new PrintPageEventHandler(printDocument1_PrintPage);

			printPreviewDialog1 = _lrrApp.PrintPreviewDialogObj;
			printPreviewDialog1.Document = printDocument1;

			printDialog1 = _lrrApp.PrintDialogObj;
			printDialog1.Document = printDocument1;
			
		}

        /// <summary>
        /// Prints the data.
        /// </summary>
        /// <param name="grid">The grid.</param>
        /// <param name="showPrintPreview">if set to <c>true</c> [show print preview].</param>
		public void PrintData(GridControl grid,bool showPrintPreview)
		{
			_gridControl = grid;

			GridRangeInfoList ranges = _gridControl.Selections.Ranges;
			if(ranges.Count == 1 && ranges.ActiveRange.IsCells)
			{
				GetSelectedData(showPrintPreview);
				return;
			}

			//Print the data
			printingData(showPrintPreview);
		}

		/// <summary>
		/// Get the selected data
		/// </summary>
		/// <param name="tableModel"></param>
		/// <param name="showPrintPreview"></param>
		private void GetSelectedData(bool showPrintPreview)
		{
			GridRangeInfoList ranges = _gridControl.Selections.Ranges;
			GridRangeInfo range = ranges.ActiveRange;

			DataTable dtTable = new DataTable();

			//Get the columns
			for(int col = range.Left; col <= range.Right; col++)
			{
				dtTable.Columns.Add(new DataColumn());
			}

			//Create the Header Row
//			DataRow drHeaderRow = dtTable.NewRow();
//			for(int col = range.Left; col <= range.Right; col++)
//			{
//				string text = _gridControl[1, col].FormattedText.Replace("\n"," ");
//				drHeaderRow[i] = text;
//			}
//			dtTable.Rows.Add(drHeaderRow);
			
			for(int row = range.Top; row <= range.Bottom; ++row)
			{
				DataRow dr = dtTable.NewRow();
				int l=0;
				for(int col = range.Left; col <= range.Right; col++)
				{
					# region Setting Text
					string text = _gridControl[row, col].FormattedText.Replace("\n"," ");
					dr[l] = text;
					l++;
					#endregion
				}
				dtTable.Rows.Add(dr);
			}
			SetupGridPrinter(dtTable,showPrintPreview);
		}
		
		/// <summary>
		/// Prints the data
		/// </summary>
		/// <param name="tableModel"></param>
		/// <param name="grid"></param>
		private  void printingData(bool showPrintPreview)
		{
			DataTable dtTable = new DataTable();

			for(int k=0;k < _gridControl.ColCount;k++)
			{
				dtTable.Columns.Add(new DataColumn());
			}

			for (int i = 1;i <= _gridControl.RowCount;i++) 
			{
				DataRow dr = dtTable.NewRow();
				int l = 0;
				for(int j = 1; j <= _gridControl.ColCount;j++) 
				{
					# region Setting Text
					string text = _gridControl[i, j].FormattedText.Replace("\n"," ");
					dr[l] = text;
					l++;
					#endregion
				}
				dtTable.Rows.Add(dr);
			}

			SetupGridPrinter(dtTable,showPrintPreview);
		}

        /// <summary>
        /// Setups the grid printer.
        /// </summary>
        /// <param name="dtTable">The dt table.</param>
        /// <param name="showPrintPreview">if set to <c>true</c> [show print preview].</param>
		private void SetupGridPrinter(DataTable dtTable,bool showPrintPreview)
		{
			printDocument1.DefaultPageSettings = _lrrApp.PageSetupDialogObj.PageSettings;

			dataGridPrinter1 = new GridControlPrintData(_gridControl, printDocument1, dtTable, footer);
			dataGridPrinter1.PageNumber = 1;
			dataGridPrinter1.RowCount = 0;

			totalColCount = dtTable.Columns.Count;

			if(showPrintPreview)
			{
				DialogResult dlg = printPreviewDialog1.ShowDialog();
			}
			else
			{
				if(printDialog1.ShowDialog() == DialogResult.OK)
				{
					printDocument1.Print();
				}
			}
			
		}

        /// <summary>
        /// Pages the set up.
        /// </summary>
		public void PageSetUp()
		{
			_lrrApp.PageSetupDialogObj.Document = printDocument1;
			_lrrApp.PageSetupDialogObj.ShowDialog();
		}

        /// <summary>
        /// Handles the PrintPage event of the printDocument1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Drawing.Printing.PrintPageEventArgs"/> instance containing the event data.</param>
		private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
		{
			Graphics	g = e.Graphics;
			bool bLastPages = true;
			
			iTotalWidth = 0;
			for(int i=nPrevColCtr;i < totalColCount;i++)
			{
				int iWidth = _gridControl.ColWidths.GetSize(nColCtr);
				iTotalWidth = iTotalWidth + iWidth;
				if(iTotalWidth > dataGridPrinter1.PageWidth)
				{
					bool bContinue = dataGridPrinter1.DrawDataGrid(nPrevColCtr,nColCtr,g);
					e.HasMorePages = true;
					dataGridPrinter1.PageNumber++;
					if(!bContinue)
					{
						nPrevColCtr = nColCtr;
					}
					else
					{
						nColCtr = nPrevColCtr;
					}
					bLastPages = false;
					break;
				}

				nColCtr++;
			}

			if(bLastPages)
			{
				bool bContinue = dataGridPrinter1.DrawDataGrid(nPrevColCtr,totalColCount,g);
				if(bContinue)
				{
					e.HasMorePages = true;
					dataGridPrinter1.PageNumber++;
					nColCtr = nPrevColCtr;
				}
				else
				{
					bLastPages = false;
				}
			}
		}

		#region Fields
		private  PrintDocument printDocument1;
		private  PrintPreviewDialog printPreviewDialog1;
		private  GridControlPrintData dataGridPrinter1 = null;
		private  GridControl	_gridControl;
		private  PrintDialog printDialog1;
		private  PageSetupDialog pageSetUpDialog;
		private TimeWindow timeWnd;
		private LRRApp _lrrApp;
		private string footer = null;

		int nColCtr		= 0;
		int iTotalWidth = 0;
		int nPrevColCtr = 0;
		int totalColCount = 0;
		#endregion
	}
}
