using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Printing;
using System.Data;
using System.Collections;
using Symcon.Utils.MessageLogger;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;


//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{

	/// <summary>
	/// Summary description for GridControlPrintData.
	/// </summary>
	public class GridControlPrintData
	{

		private PrintDocument ThePrintDocument;
		private DataTable TheTable;
		private GridControl  TheDataGrid;

		public int RowCount = 0;  // current count of rows;
		public int initialRowCount = 0;
		private const int kVerticalCellLeeway = 10;
		public int PageNumber = 1;
		public ArrayList Lines = new ArrayList();

		public int PageWidth;
		int PageHeight;
		int TopMargin;
		int BottomMargin;
		float[] HeaderBottom = new float[4];
		string _sFooterString;

		private int startRowIndex;
		private int endRowIndex;

		private int LEFT_MARGIN = 10;
		private int RIGHT_MARGIN = 10;



        /// <summary>
        /// Initializes a new instance of the <see cref="GridControlPrintData"/> class.
        /// </summary>
        /// <param name="aGrid">A grid.</param>
        /// <param name="aPrintDocument">A print document.</param>
        /// <param name="aTable">A table.</param>
        /// <param name="footerString">The footer string.</param>
		public GridControlPrintData(GridControl aGrid, PrintDocument aPrintDocument, DataTable aTable,string footerString)
		{
			//
			// TODO: Add constructor logic here
			//
			TheDataGrid = aGrid;
			ThePrintDocument = aPrintDocument;
			TheTable = aTable;

			
			PageHeight = ThePrintDocument.DefaultPageSettings.PaperSize.Height;
			TopMargin = ThePrintDocument.DefaultPageSettings.Margins.Top;
			BottomMargin = ThePrintDocument.DefaultPageSettings.Margins.Bottom;
			LEFT_MARGIN = ThePrintDocument.DefaultPageSettings.Margins.Left;
			RIGHT_MARGIN = ThePrintDocument.DefaultPageSettings.Margins.Right;
			_sFooterString=footerString;

			PageWidth = ThePrintDocument.DefaultPageSettings.PaperSize.Width  - LEFT_MARGIN - RIGHT_MARGIN;

			endRowIndex = TheTable.Rows.Count;
			startRowIndex = 0;
		}

        /// <summary>
        /// Draws the grid.
        /// </summary>
        /// <param name="startColIndex">Start index of the col.</param>
        /// <param name="endColIndex">End index of the col.</param>
        /// <param name="strtRowIndex">Index of the STRT row.</param>
        /// <param name="endRowIndex">End index of the row.</param>
        /// <param name="g">The g.</param>
        /// <returns></returns>
		public bool DrawGrid(int startColIndex,int endColIndex,int strtRowIndex,int endRowIndex,Graphics g)
		{
			int lastRowBottom = TopMargin;

			SolidBrush ForeBrush = new SolidBrush(TheDataGrid.ForeColor);
			SolidBrush BackBrush = new SolidBrush(TheDataGrid.BackColor);
			SolidBrush AlternatingBackBrush = new SolidBrush(TheDataGrid.BackColor);

			Pen TheLinePen = new Pen(Color.Black, 1);
			StringFormat cellformat = new StringFormat();
			cellformat.Trimming = StringTrimming.EllipsisCharacter;

			int totalColWidth = PageWidth*(endColIndex - startColIndex)/TheDataGrid.ColCount;
			int totalGridColWidth = 0;

			for (int j = startColIndex; j < endColIndex; j++)
			{
				totalGridColWidth += TheDataGrid.ColWidths[j+1];
			}

			int[] columnwidth = new int[endColIndex - startColIndex + 1];

			RectangleF RowBounds  = new RectangleF(0, 0, 0, 0);
			RowCount = 0;

			for (int i = strtRowIndex; i < endRowIndex; i++)
			{
				DataRow dr = TheTable.Rows[i];
				int startxposition = /*TheDataGrid.Location.X + */LEFT_MARGIN;

				RowBounds.X = /*TheDataGrid.Location.X + */LEFT_MARGIN;
				RowBounds.Y = /*TheDataGrid.Location.Y + */TopMargin + ((RowCount - initialRowCount)+1) * (TheDataGrid.Font.SizeInPoints  + kVerticalCellLeeway);
				RowBounds.Height = TheDataGrid.Font.SizeInPoints + kVerticalCellLeeway;
				RowBounds.Width = PageWidth;
				
				if(i-startRowIndex < 4)
					HeaderBottom[i-startRowIndex] = RowBounds.Bottom;

				Lines.Add(RowBounds.Bottom);

				if (i%2 == 0)
				{
					g.FillRectangle(BackBrush, RowBounds);
				}
				else
				{
					g.FillRectangle(AlternatingBackBrush, RowBounds);
				}


				for (int j = startColIndex; j < endColIndex; j++)
				{
					
					columnwidth[j - startColIndex] = TheDataGrid.ColWidths[j+1]*PageWidth/totalGridColWidth;
					
					
					GridRangeInfo range;
					TheDataGrid.CoveredRanges.Find(i+1,j+1,out range);

					int rangeDiff = range.Right - range.Left;
					RectangleF cellbounds = new RectangleF(startxposition, 
						/*TheDataGrid.Location.Y + */TopMargin + ((RowCount - initialRowCount) + 1) * (TheDataGrid.Font.SizeInPoints  + kVerticalCellLeeway),
						columnwidth[j - startColIndex] *(1 + rangeDiff), 
						TheDataGrid.Font.SizeInPoints + kVerticalCellLeeway);
									

//					if (startxposition + columnwidth <= PageWidth)
//					{
					
			
					if(rangeDiff == 0)
					{
						if(columnwidth[j - startColIndex]>0)
						{
							g.DrawString(dr[j].ToString(), TheDataGrid.Font, ForeBrush, cellbounds, cellformat);
						}
					}
					else
					{
						g.DrawString(dr[j].ToString(), TheDataGrid.Font, ForeBrush, cellbounds, cellformat);
					}
					
					lastRowBottom = (int)cellbounds.Bottom;
//					}

					startxposition = startxposition + columnwidth[j - startColIndex];
				}

				RowCount++;

				if (RowCount * (TheDataGrid.Font.SizeInPoints  + kVerticalCellLeeway) > (PageHeight) - (BottomMargin+TopMargin))
				{
					startRowIndex = i + 1;
					DrawHorizontalLines(g, Lines);
					DrawVerticalGridLines(g, TheLinePen, columnwidth, lastRowBottom,startColIndex);
					return true;
				}
			}

			DrawHorizontalLines(g, Lines);
			DrawVerticalGridLines(g, TheLinePen, columnwidth, lastRowBottom,startColIndex);
			return false;
		}

        /// <summary>
        /// Draws the horizontal lines.
        /// </summary>
        /// <param name="g">The g.</param>
        /// <param name="lines">The lines.</param>
		void DrawHorizontalLines(Graphics g, ArrayList lines)
		{
			Pen TheLinePen = new Pen(Color.Black);/*(TheDataGrid.GridLineColor, 1);*/

//			if (TheDataGrid.GridLineStyle == DataGridLineStyle.None)
//				return;
			
			g.DrawLine(TheLinePen, /*TheDataGrid.Location.X*/LEFT_MARGIN, TopMargin, PageWidth + RIGHT_MARGIN, TopMargin);
			for (int i = 0;  i < lines.Count; i++)
			{
				g.DrawLine(TheLinePen, LEFT_MARGIN/*TheDataGrid.Location.X*/, (float)lines[i], PageWidth + RIGHT_MARGIN, (float)lines[i]);
			}
		}

        /// <summary>
        /// Draws the vertical grid lines.
        /// </summary>
        /// <param name="g">The g.</param>
        /// <param name="TheLinePen">The line pen.</param>
        /// <param name="columnwidth">The columnwidth.</param>
        /// <param name="bottom">The bottom.</param>
        /// <param name="startColIndex">Start index of the col.</param>
		void DrawVerticalGridLines(Graphics g, Pen TheLinePen, int[] columnwidth, int bottom, int startColIndex)
		{
//			if (TheDataGrid.Appearance.AnyRecordFieldCell.GridLineStyle == DataGridLineStyle.None)
//				return;

			int iColCount = columnwidth.Length;

			int currentXLeft=0;

			for (int k = 0; k < iColCount/*TheTable.Columns.Count*/;)
			{
				
				if(k == iColCount - 1)
				{
					g.DrawLine(TheLinePen,PageWidth + RIGHT_MARGIN,TopMargin,PageWidth + RIGHT_MARGIN,bottom);
					currentXLeft += columnwidth[k];
					k++;
				}
				else
				{
					GridRangeInfo[] range = new GridRangeInfo[4];
					TheDataGrid.CoveredRanges.Find(1,k+startColIndex+1,out range[0]);
					TheDataGrid.CoveredRanges.Find(2,k+startColIndex+1,out range[1]);
					TheDataGrid.CoveredRanges.Find(3,k+startColIndex+1,out range[2]);
					TheDataGrid.CoveredRanges.Find(4,k+startColIndex+1,out range[3]);

					int[] rangeDiff = new int[4];
					rangeDiff[0] = range[0].Right - range[0].Left;
					rangeDiff[1] = range[1].Right - range[1].Left;
					rangeDiff[2] = range[2].Right - range[2].Left;
					rangeDiff[3] = range[3].Right - range[3].Left;

					if(rangeDiff[0] == 0)
					{
						g.DrawLine(TheLinePen, LEFT_MARGIN/*TheDataGrid.Location.X */+ currentXLeft, 
							/*TheDataGrid.Location.Y + */TopMargin,
							/*TheDataGrid.Location.X*/LEFT_MARGIN + currentXLeft,
							bottom);
						currentXLeft += columnwidth[k];
						k++;
					}
					else
					{
						g.DrawLine(TheLinePen, LEFT_MARGIN/*TheDataGrid.Location.X */+ currentXLeft, 
							/*TheDataGrid.Location.Y + */TopMargin,
							/*TheDataGrid.Location.X*/LEFT_MARGIN + currentXLeft,
							bottom);
						currentXLeft += columnwidth[k];
						k++;
						while((k+startColIndex)<range[0].Right &&  k < iColCount)
						{
							if(k == iColCount - 1)
							{
								g.DrawLine(TheLinePen,PageWidth + RIGHT_MARGIN,TopMargin,PageWidth + RIGHT_MARGIN,bottom);
							}
							else
							{
								if(rangeDiff[3] == 0 && HeaderBottom[1]!=0)
								{
									g.DrawLine(TheLinePen, LEFT_MARGIN/*TheDataGrid.Location.X */+ currentXLeft, 
										/*TheDataGrid.Location.Y + */HeaderBottom[0],
										/*TheDataGrid.Location.X*/LEFT_MARGIN + currentXLeft,
										HeaderBottom[1]);
								}
								if(rangeDiff[2] == 0 && HeaderBottom[2]!=0)
								{
									g.DrawLine(TheLinePen, LEFT_MARGIN/*TheDataGrid.Location.X */+ currentXLeft, 
										/*TheDataGrid.Location.Y + */HeaderBottom[1],
										/*TheDataGrid.Location.X*/LEFT_MARGIN + currentXLeft,
										HeaderBottom[2]);
								}
								if(rangeDiff[1] == 0 && HeaderBottom[3]!=0)
								{
									g.DrawLine(TheLinePen, LEFT_MARGIN/*TheDataGrid.Location.X */+ currentXLeft, 
										/*TheDataGrid.Location.Y + */HeaderBottom[2],
										/*TheDataGrid.Location.X*/LEFT_MARGIN + currentXLeft,
										HeaderBottom[3]);
								}
								if(TheDataGrid.RowCount>3)
									g.DrawLine(TheLinePen, LEFT_MARGIN/*TheDataGrid.Location.X */+ currentXLeft, 
									/*TheDataGrid.Location.Y + */HeaderBottom[3],
									/*TheDataGrid.Location.X*/LEFT_MARGIN + currentXLeft,
									bottom);
							}
							currentXLeft += columnwidth[k];
							k++;
						}
					}
				}
				
				
			}

		}

        /// <summary>
        /// Drows the footer.
        /// </summary>
        /// <param name="g">The g.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <param name="BottomMargine">The bottom margine.</param>
        /// <param name="footerString">The footer string.</param>
		void DrowFooter(Graphics g,int pageSize,int BottomMargine,string footerString)
		{
			System.Drawing.Font font = new Font("Arial",11);
			SolidBrush ForeBrush = new SolidBrush(TheDataGrid.ForeColor);
			StringFormat sf = new StringFormat(); 
			sf.Alignment = System.Drawing.StringAlignment.Center; 
			sf.LineAlignment = System.Drawing.StringAlignment.Far;    
			RectangleF rectf = new RectangleF(LEFT_MARGIN,ThePrintDocument.DefaultPageSettings.Bounds.Height - BottomMargine,PageWidth,BottomMargine - 10);
			//g.DrawRectangle(new Pen(Color.Red,2),rect); //For Debugging
			g.DrawString(footerString,font,ForeBrush,rectf,sf);    
		}
        /// <summary>
        /// Draws the data grid.
        /// </summary>
        /// <param name="startColIndex">Start index of the col.</param>
        /// <param name="endColIndex">End index of the col.</param>
        /// <param name="g">The g.</param>
        /// <returns></returns>
		public bool DrawDataGrid(int startColIndex,int endColIndex,Graphics g)
		{
			try
			{
				int strtRowIndex = startRowIndex;
				bool bContinue = DrawGrid(startColIndex,endColIndex,strtRowIndex,endRowIndex,g);
				DrowFooter(g,PageHeight,BottomMargin,_sFooterString);
				if(!bContinue)
				{
					startRowIndex = 0;
				}
				return bContinue;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GridControlPrintData),"DrawDataGrid:"+ex.Message,Logger.Level.Debug);
				return false;
			}
		}
	}
}
