using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Symcon.BusinessObjects;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.BusinessObjects;


//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// Summary description for GridExcelFormatter.
	/// </summary>
	public class GridExcelFormatter
	{
		public GridExcelFormatter()
		{

		}

		#region Methods
        /// <summary>
        /// Wires the grid.
        /// </summary>
        /// <param name="grid">The grid.</param>
		public void WireGrid(GridGroupingControl grid)
		{
			if(this._gridGrouping != grid)
			{
				if(this._gridGrouping != null)
					UnwireGrid();
			
				this._gridGrouping = grid;
				this._gridGrouping.TableModel.ClipboardPaste+=new GridCutPasteEventHandler(TableModel_ClipboardPaste);
				this._gridGrouping.TableControlCellClick+=new GridTableControlCellClickEventHandler(_gridGrouping_TableControlCellClick);
				this._gridGrouping.CurrentRecordContextChange+=new CurrentRecordContextChangeEventHandler(_gridGrouping_CurrentRecordContextChange);  				this._gridGrouping.TableControlCurrentCellMoved+=new GridTableControlCurrentCellMovedEventHandler(_gridGrouping_TableControlCurrentCellMoved);
				this._gridGrouping.RecordValueChanged +=new RecordValueChangedEventHandler(_gridGrouping_RecordValueChanged);
				this._gridGrouping.TableControlCurrentCellMoved+=new GridTableControlCurrentCellMovedEventHandler(_gridGrouping_TableControlCurrentCellMoved);
				this._gridGrouping.TableControlCurrentCellCloseDropDown +=new GridTableControlPopupClosedEventHandler(_gridGrouping_TableControlCurrentCellCloseDropDown);	
				
				
				

				int iRelationsCount = this._gridGrouping.TableDescriptor.Relations.Count;
				for(int i=0; i < iRelationsCount;i++)
				{
					GridRelationDescriptor gRelationDesc =  this._gridGrouping.TableDescriptor.Relations[i];
					if(gRelationDesc.RelationKind == RelationKind.RelatedMasterDetails)
					{
						try
						{
							GridTableModel tableModel = this._gridGrouping.GetTableModel(gRelationDesc.Name);
							if(tableModel != null)
							{
								tableModel.ClipboardPaste+=new GridCutPasteEventHandler(TableModel_ClipboardPaste);
								
							}
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridExcelFormatter),"WireGrid:"+ex.Message,Logger.Level.Debug);
						}
						registerTableModelCopyPaste(gRelationDesc);
					}
				}
			}
		}

        /// <summary>
        /// Handles the TableControlCurrentCellCloseDropDown event of the _gridGrouping control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPopupClosedEventArgs"/> instance containing the event data.</param>
		void _gridGrouping_TableControlCurrentCellCloseDropDown(object sender, GridTableControlPopupClosedEventArgs e)
		{

				if(e.TableControl.Table.CurrentRecord != null)
					e.TableControl.Table.CurrentRecord.EndEdit();
 
		}

        /// <summary>
        /// Handles the RecordValueChanged event of the _gridGrouping control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Grouping.RecordValueChangedEventArgs"/> instance containing the event data.</param>
		void _gridGrouping_RecordValueChanged(object sender, RecordValueChangedEventArgs e)
		{
		 
			//this event get fired when ever there is a change in the grid
			GridGroupingControl ggc = sender as GridGroupingControl;
			Form frm =  ggc.FindForm();
			
			//string gridScreenName=((Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl)(sender)).DataSource.GetType().Name;
			string gridScreenName=frm.Text;
			if(gridScreenName!="Options")
			 {	
				Object obj = (e.Record.GetData().GetType());
				obj = e.Record.GetData();
				if(GridChangedRecordLists.GetInstance().Count>0)
				{
						IEnumerator ieNumChangedRecordLists = GridChangedRecordLists.GetInstance().GetEnumerator();

						bool screenRecordExists = false;
						while(ieNumChangedRecordLists.MoveNext())
						{
						 string recordScreenName=((GridChangedRecord)(ieNumChangedRecordLists.Current)).ScreenName.ToString();
							if(gridScreenName==recordScreenName)
							{
								screenRecordExists=true;
								if(((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList!=null)
								{
									if(!((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList.Contains(obj))							
										((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList.Add(obj);	
								}
								else
								{
									((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList=new ArrayList();	
									((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList.Add(obj);	
								}
								
							}
						} 
					if(!screenRecordExists)
					{
						GridChangedRecord gridChangedRecord = new GridChangedRecord();
						gridChangedRecord.ScreenName=gridScreenName;
						gridChangedRecord.DataTypeStored=obj.GetType().ToString();
						gridChangedRecord.ChangedRecordList.Add(obj);
						GridChangedRecordLists.GetInstance().Add(gridChangedRecord);

					}
				}
				else
				{
					GridChangedRecord gridChangedRecord = new GridChangedRecord();
					gridChangedRecord.ScreenName=gridScreenName;
					gridChangedRecord.DataTypeStored=obj.GetType().ToString();
					gridChangedRecord.ChangedRecordList.Add(obj);
					GridChangedRecordLists.GetInstance().Add(gridChangedRecord);
				}  
				
			} 

		}

        /// <summary>
        /// Unwires the grid.
        /// </summary>
		public void UnwireGrid()
		{
			try
			{
				if(this._gridGrouping == null)
					return;
				this._gridGrouping.CurrentRecordContextChange-=new CurrentRecordContextChangeEventHandler(_gridGrouping_CurrentRecordContextChange);   
				this._gridGrouping.TableModel.ClipboardPaste-=new GridCutPasteEventHandler(TableModel_ClipboardPaste);
				this._gridGrouping.TableControlCellClick-=new GridTableControlCellClickEventHandler(_gridGrouping_TableControlCellClick);
				int iRelationsCount = this._gridGrouping.TableDescriptor.Relations.Count;
				for(int i=0; i < iRelationsCount;i++)
				{
					GridRelationDescriptor gRelationDesc =  this._gridGrouping.TableDescriptor.Relations[i];
					if(gRelationDesc.RelationKind == RelationKind.RelatedMasterDetails)
					{
						try
						{
							GridTableModel tableModel = this._gridGrouping.GetTableModel(gRelationDesc.Name);
							if(tableModel != null)
							{
								tableModel.ClipboardPaste-=new GridCutPasteEventHandler(TableModel_ClipboardPaste);
								
								
							}
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridExcelFormatter),"UnWireGrid:"+ex.Message,Logger.Level.Debug);
						}
						unRegisterTableModelCopyPaste(gRelationDesc);
					}
				}

				this._gridGrouping = null;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GridExcelFormatter),"UnWireGrid:"+ex.Message,Logger.Level.Debug);
			}
		}
        /// <summary>
        /// Registers the table model copy paste.
        /// </summary>
        /// <param name="grd">The GRD.</param>
		private void registerTableModelCopyPaste(GridRelationDescriptor grd)
		{
			int iChildCount = grd.ChildTableDescriptor.Relations.Count;
			for(int i=0;i < iChildCount;i++)
			{
				GridRelationDescriptor gRelationDesc =  grd.ChildTableDescriptor.Relations[i];
				if(gRelationDesc.RelationKind == RelationKind.RelatedMasterDetails)
				{
					try
					{
						GridTableModel tableModel = this._gridGrouping.GetTableModel(gRelationDesc.Name);
						if(tableModel != null)
						{
							tableModel.ClipboardPaste+=new GridCutPasteEventHandler(TableModel_ClipboardPaste);
						}
					}
					catch(Exception ex)
					{
						Logger.LogMessage(typeof(GridExcelFormatter),"registerTableModelCopyPaste Event:"+ex.Message,Logger.Level.Debug);
					}
					registerTableModelCopyPaste(gRelationDesc);
				}
			}
		}
        /// <summary>
        /// Uns the register table model copy paste.
        /// </summary>
        /// <param name="grd">The GRD.</param>
		private void unRegisterTableModelCopyPaste(GridRelationDescriptor grd)
		{
			int iChildCount = grd.ChildTableDescriptor.Relations.Count;
			for(int i=0;i < iChildCount;i++)
			{
				GridRelationDescriptor gRelationDesc =  grd.ChildTableDescriptor.Relations[i];
				if(gRelationDesc.RelationKind == RelationKind.RelatedMasterDetails)
				{
					try
					{
						GridTableModel tableModel = this._gridGrouping.GetTableModel(gRelationDesc.Name);
						if(tableModel != null)
						{
							tableModel.ClipboardPaste-=new GridCutPasteEventHandler(TableModel_ClipboardPaste);
						}
					}
					catch(Exception ex)
					{
						Logger.LogMessage(typeof(GridExcelFormatter),"unRegisterTableModelCopyPaste Event:"+ex.Message,Logger.Level.Debug);
					}
					registerTableModelCopyPaste(gRelationDesc);
				}
			}
		}
		#endregion

		#region Fields
		private GridGroupingControl						_gridGrouping = null;
		#endregion

		#region Event Handlers
        /// <summary>
        /// Handles the ClipboardPaste event of the TableModel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.GridCutPasteEventArgs"/> instance containing the event data.</param>
		private void TableModel_ClipboardPaste(object sender, GridCutPasteEventArgs e)
		{
			IDataObject obj = Clipboard.GetDataObject();
			string text = (string)obj.GetData(DataFormats.Text);
			string[] dataObjRows = text == null ? new string[0] : text.Split(new char[]{'\n'});
			string[] dataObjCols = text == null ? new string[0] : dataObjRows[0].Split(new char[]{'\t'});
			DialogResult dlgResult = DialogResult.OK;

			GridTableModel tableModel = null;
			
			if(_gridGrouping.TableControl.Table.CurrentRecord == null)
			{
				Element eCurrentElement = _gridGrouping.TableControl.Table.CurrentElement;
				GridNestedTable gnt = eCurrentElement as GridNestedTable;
				GridNestedTable gnt1 = gnt;
				GridTableDescriptor gtd = null;
				while(gnt1 != null && gnt1.ChildTable != null)
				{
					gnt = gnt1;
					gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
					gtd = gnt.ParentTable.ParentTableDescriptor;
				}

				GridChildTable childTable = gnt.ChildTable as GridChildTable;
				tableModel = childTable.ParentTable.TableModel;
			}
			else
			{
				tableModel = _gridGrouping.TableModel;
			}

			if(tableModel.Selections.GetSelectedRows(false,true).ActiveRange.Height < dataObjRows.Length - 1)
			{
				e.Handled= true;
				dlgResult = LRRMessagBox.ShowMessage("SID_INSERT_ROWSOPTION","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Error);  
				return;
			}
			else if(tableModel.Selections.GetSelectedCols(false,true).ActiveRange.Width < dataObjCols.Length)
			{
				e.Handled= true;
				dlgResult = LRRMessagBox.ShowMessage("SID_SELECTE_MORECOLUMN","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Error);   
				return;
			}
		}
        /// <summary>
        /// Handles the TableControlCellClick event of the _gridGrouping control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs"/> instance containing the event data.</param>
		private void _gridGrouping_TableControlCellClick(object sender, GridTableControlCellClickEventArgs e)
		{
			// Check for CTRL - Key to do multi-column sorting.....
			if(((Control.ModifierKeys & Keys.Control) != 0))
			{
				// Cancel the default sorting...
				e.Inner.Cancel = true;

				GridTableDescriptor tableDescriptor = e.TableControl.TableDescriptor;
				if(e.TableControl.TableDescriptor.Name == "PropertyValues")
				{
					e.Inner.Cancel = true;
					return;
				}
				GridColumnDescriptor column = e.TableControl.Table.GetColumnDescriptorAt(e.Inner.RowIndex, e.Inner.ColIndex);
				string col = column.Name;
				
				if(tableDescriptor.SortedColumns.Contains(col))
				{
					if(tableDescriptor.SortedColumns[col].SortDirection == ListSortDirection.Ascending)
					{
						tableDescriptor.SortedColumns[col].SortDirection = ListSortDirection.Descending;
					}
					else
					{
						tableDescriptor.SortedColumns[col].SortDirection = ListSortDirection.Ascending;
					}
				}
				else
				{
					tableDescriptor.SortedColumns.Add(col, ListSortDirection.Ascending);
				}
			}
		}

		bool lastAddNewRow = false;
        /// <summary>
        /// Handles the CurrentRecordContextChange event of the _gridGrouping control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Grouping.CurrentRecordContextChangeEventArgs"/> instance containing the event data.</param>
		private void _gridGrouping_CurrentRecordContextChange(object sender, CurrentRecordContextChangeEventArgs e)
		{
			try
			{
				GridTableModel tableModel  = this._gridGrouping.TableModel as GridTableModel; 
				GridTable table = tableModel.Table as GridTable;   
				int iRow = this._gridGrouping.TableControl.CurrentCell.RowIndex;
				int iCOl = this._gridGrouping.TableControl.CurrentCell.ColIndex;  
//				this._gridGrouping.TableModel.Selections.Clear(); 
				this._gridGrouping.TableModel.Table.SelectedRecords.Clear();     
				Record _rec = e.Record as Record;
				if((e.Action == CurrentRecordAction.EndEditCalled) &&  (e.Record.Kind == DisplayElementKind.AddNewRecord))
				{
					_rec.SetCurrent();
					lastAddNewRow = true;
				
				}
				else if(lastAddNewRow && e.Action == CurrentRecordAction.EndEditComplete)
				{
					if(this._gridGrouping.TableControl.Table.CurrentRecord == null)
					{
						Element eCurrentInnerElement = _gridGrouping.Table.GetInnerMostCurrentElement();
						Element eCurrentElement = _gridGrouping.TableControl.Table.CurrentElement;
						if(eCurrentElement != null)
						{
							GridTable gt = eCurrentInnerElement.ParentTable as GridTable;
							GridNestedTable gnt = eCurrentElement as GridNestedTable;
							GridNestedTable gnt1 = gnt;
							GridTableDescriptor gtd = null;
							while(gnt1 != null && gnt1.ChildTable != null)
							{
								gnt = gnt1;
								gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
								gtd = gnt.ParentTable.ParentTableDescriptor;
							}
							GridChildTable gt1 = gnt.ChildTable;
							gt.TableModel.Selections.Clear();
							int row = gt.DisplayElements.IndexOf(gt.CurrentRecord);    
							gt.TableModel.Selections.SelectRange(GridRangeInfo.Row(row),true); 
							this._gridGrouping.TableControl.ScrollCellInView(iRow,iCOl);  
							lastAddNewRow = false;
						}
						else
						{
							_rec.SetCurrent(); 
							_rec.SetSelected(true);
						}
					}
					else
					{
						int row = table.DisplayElements.IndexOf(table.CurrentRecord);
						this._gridGrouping.TableModel.Selections.Clear(); 
						this._gridGrouping.TableModel.Selections.SelectRange(GridRangeInfo.Row(row),true);      
						this._gridGrouping.TableModel.Table.SelectedRecords.Add(_rec);     
						this._gridGrouping.TableModel.ScrollCellInView(GridRangeInfo.Row(row),GridScrollCurrentCellReason.Any); 
						lastAddNewRow = false;
					}
				}
				else if(e.Action == CurrentRecordAction.EndEditComplete && e.Record.Kind == DisplayElementKind.Record)
				{
					if(this._gridGrouping.TableControl.Table.CurrentRecord == null)
					{
						Element eCurrentInnerElement = _gridGrouping.Table.GetInnerMostCurrentElement();
						Element eCurrentElement = _gridGrouping.TableControl.Table.CurrentElement;
						if(eCurrentElement != null)
						{
							GridTable gt = eCurrentInnerElement.ParentTable as GridTable;
							GridNestedTable gnt = eCurrentElement as GridNestedTable;
							GridNestedTable gnt1 = gnt;
							GridTableDescriptor gtd = null;
							while(gnt1 != null && gnt1.ChildTable != null)
							{
								gnt = gnt1;
								gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
								gtd = gnt.ParentTable.ParentTableDescriptor;
							}
							GridChildTable gt1 = gnt.ChildTable;
							gt.TableModel.Selections.Clear();
							int row = gt.DisplayElements.IndexOf(gt.CurrentRecord);    
							gt.TableModel.Selections.SelectRange(GridRangeInfo.Row(row),true); 
							gt.TableModel.ScrollCellInView(GridRangeInfo.Row(row),GridScrollCurrentCellReason.Any);      
							//lastAddNewRow = false;
						}
						else
						{
							_rec.SetCurrent(); 
							_rec.SetSelected(true);
						}
					}
					else
					{
						GridRangeInfoList rangeList = new GridRangeInfoList(); 
						int row = table.DisplayElements.IndexOf(table.CurrentRecord);
						this._gridGrouping.TableModel.Selections.Clear(); 
						this._gridGrouping.TableModel.Selections.SelectRange(GridRangeInfo.Row(row),true);      
						this._gridGrouping.TableModel.Table.SelectedRecords.Add(_rec);     
						this._gridGrouping.TableModel.ScrollCellInView(GridRangeInfo.Row(row),GridScrollCurrentCellReason.Any);     
						//lastAddNewRow = false;
					}
				}				
			}
			catch (Exception ex)
			{
				Logger.LogMessage(typeof(GridExcelFormatter),"_gridGrouping_CurrentRecordContextChange :"+ex.Message  ,Logger.Level.Debug);
			}
		}
		
		private void _gridGrouping_TableControlCurrentCellMoved(object sender, GridTableControlCurrentCellMovedEventArgs e)
		{
			/*if(this._gridGrouping.TableControl.Table.CurrentRecord != null)
			{
				this._gridGrouping.TableModel.Selections.Clear();    
				this._gridGrouping.TableControl.Table.CurrentRecord.SetCurrent();
				this._gridGrouping.TableControl.Table.CurrentRecord.SetSelected(true);      
				int row = e.TableControl.CurrentCell.RowIndex;
				this._gridGrouping.TableModel.Selections.SelectRange(GridRangeInfo.Row(row),true);   
				this._gridGrouping.TableModel.Table.SelectedRecords.Add(e.TableControl.Table.CurrentRecord);
				this._gridGrouping.TableModel.ScrollCellInView(GridRangeInfo.Row(row),GridScrollCurrentCellReason.Any);  
			}
			else
			{
				Element eCurrentInnerElement = _gridGrouping.Table.GetInnerMostCurrentElement();
				Element eCurrentElement = _gridGrouping.TableControl.Table.CurrentElement;
				if(eCurrentElement != null)
				{
					GridTable gt = eCurrentInnerElement.ParentTable as GridTable;
					GridNestedTable gnt = eCurrentElement as GridNestedTable;
					GridNestedTable gnt1 = gnt;
					GridTableDescriptor gtd = null;
					while(gnt1 != null && gnt1.ChildTable != null)
					{
						gnt = gnt1;
						gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
						gtd = gnt.ParentTable.ParentTableDescriptor;
					}
					GridChildTable gt1 = gnt.ChildTable;
					gt.TableModel.Selections.Clear();
					if(gt.CurrentRecord != null)
					{
						int row = gt.DisplayElements.IndexOf(gt.CurrentRecord);    
						gt.TableModel.Selections.SelectRange(GridRangeInfo.Row(row),true);   
						gt.TableModel.Table.SelectedRecords.Add(gt.CurrentRecord);
						gt.TableModel.ScrollCellInView(GridRangeInfo.Row(row),GridScrollCurrentCellReason.Any);  
					}
				}
			}*/
  		}
		#endregion

	}
}
