using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Reflection;
using Symcon.BusinessObjects;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Symcon.Globals; 
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.MessageLogger;
using Symcon.Utils.HeaderDispatcher;


namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// Summary description for GridTransposedView.
	/// </summary>
	public class GridTransposedView
	{
		public GridTransposedView()
		{
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		}

        /// <summary>
        /// Applies the transposed view.
        /// </summary>
        /// <param name="gridInstance">The grid instance.</param>
        /// <param name="bAddNewRecord">if set to <c>true</c> [b add new record].</param>
		public static void ApplyTransposedView(GridGroupingControl gridInstance, bool bAddNewRecord)
		{
            try
            {
                LRRWaitWindowUtility.ShowPogressIndicator(LRRHeaderText.GetHeaderString("SID_TRANSPOSED_VIEW"));
                GridGroupingControl gc = gridInstance as GridGroupingControl;
                //object recordObj = gc.TableControl.Table.CurrentRecord.GetData(); 
                if (gc.TableControl.Table.CurrentRecord == null)
                {
                    GetDataSource(gc);
                    CustomCollection cc = sDataSource as CustomCollection;
                    TransPosedView transPosedView = new TransPosedView(cc, gc, bAddNewRecord);
                    if (transPosedView.ShowForm)
                    {
                        transPosedView.WindowState = System.Windows.Forms.FormWindowState.Normal;
                        LRRWaitWindowUtility.EndProgressIndicator();
                        transPosedView.ShowDialog();
                    }
                }
                else
                {
                    IList dataSource = gc.Engine.Table.SourceList;
                    CustomCollection cc = dataSource as CustomCollection;
                    object recordObject = gc.TableControl.Table.CurrentRecord.GetData();
                    TransPosedView transPosedView = new TransPosedView(cc, gc, bAddNewRecord);
                    if (transPosedView.ShowForm)
                    {
                        transPosedView.WindowState = System.Windows.Forms.FormWindowState.Normal;
                        LRRWaitWindowUtility.EndProgressIndicator();
                        transPosedView.ShowDialog();
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(GridTransposedView), "ApplyTransposedView: " + ex, Logger.Level.Debug);
            }
            finally
            {
                LRRWaitWindowUtility.EndProgressIndicator();
            }
		}
        /// <summary>
        /// Gets the data source.
        /// </summary>
        /// <param name="gridGroupControl">The grid group control.</param>
		public static void GetDataSource(GridGroupingControl gridGroupControl)
		{
			Element eCurrentElement = gridGroupControl.TableControl.Table.CurrentElement;
			if(eCurrentElement != null)
			{
				string pkField = null;
				GridNestedTable gnt = eCurrentElement as GridNestedTable;
				GridNestedTable gnt1 = gnt;
				GridTableDescriptor gtd = null;
				while(gnt1 != null && gnt1.ChildTable != null)
				{
					gnt = gnt1;
					gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
					gtd = gnt.ParentTable.ParentTableDescriptor;
				}

				string sRelation = gnt.ChildTable.ParentTableDescriptor.Name;
				relation = gtd.Relations[sRelation].RelationKeys[0].ChildKeyFieldName;
				pkField = gtd.Relations[sRelation].RelationKeys[0].ParentKeyFieldName;
				parentID = gnt.ParentRecord.GetValue(pkField).ToString();
				string sChild = gtd.Relations[sRelation].ChildTableName;
				//sDataSource = gridGroupControl.Engine.SourceListSet[sChild].List;
			}
		}

		#region Private Methods

		
		


		#endregion

		#region Fields

		public static IList sDataSource = null;
		public static string relation = null;
		public static string parentID = null;
		private static LRRApp  _lrrApp;

		#endregion
	}
}
