using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Symcon.BusinessObjects;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using System.Reflection;
using Symcon.Utils.HeaderDispatcher;   


//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// Class for handling Unbound column
	/// </summary>
	public class GridUnBoundColumn
	{
		Hashtable checkboxvalues=new Hashtable();
		string[] pKeys=new string[]{};
		public GridUnBoundColumn()
		{
			
		}


		#region Methods
        /// <summary>
        /// This method is used to add an UnBound Column of type CheckBox
        /// </summary>
        /// <param name="gridGroupingControl">The grid grouping control.</param>
		public  void WireGrid(GridGroupingControl gridGroupingControl)
		{
			gridGroupingControl.TableDescriptor.UnboundFields.Clear();
			gridGroupingControl.TableDescriptor.UnboundFields.Add(LRRHeaderText.GetHeaderString("SID_SELECT"));
			gridGroupingControl.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_SELECT"));
			gridGroupingControl.TableDescriptor.Columns[LRRHeaderText.GetHeaderString("SID_SELECT")].Appearance.AnyRecordFieldCell.CellType="CheckBox";
			gridGroupingControl.TableDescriptor.Columns[LRRHeaderText.GetHeaderString("SID_SELECT")].Appearance.AnyRecordFieldCell.HorizontalAlignment=GridHorizontalAlignment.Center;
			//events for saving the checkbox values entered.
			gridGroupingControl.TableDescriptor.QueryValue += new FieldValueEventHandler(unboundField_QueryValue); // Routine that queries for the value
			gridGroupingControl.TableDescriptor.SaveValue += new FieldValueEventHandler(unboundField_SaveValue); // Routine that queries for the value
			gridGroupingControl.TableControlQueryAllowSortColumn+=new GridQueryAllowSortColumnEventHandler(gridGroupingControl_TableControlQueryAllowSortColumn);
		}

        /// <summary>
        /// Wires the grid.
        /// </summary>
        /// <param name="gridGroupingControl">The grid grouping control.</param>
        /// <param name="ChileTable">The chile table.</param>
        public void WireGrid(GridGroupingControl gridGroupingControl,string ChileTable)
        {
            GridTableDescriptor tableDescriptor = gridGroupingControl.GetTableDescriptor(ChileTable);
            tableDescriptor.UnboundFields.Add(LRRHeaderText.GetHeaderString("SID_SELECT"));
            tableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_SELECT"));
            tableDescriptor.Columns[LRRHeaderText.GetHeaderString("SID_SELECT")].Appearance.AnyRecordFieldCell.CellType = "CheckBox";
            tableDescriptor.Columns[LRRHeaderText.GetHeaderString("SID_SELECT")].Appearance.AnyRecordFieldCell.HorizontalAlignment = GridHorizontalAlignment.Center;
            //events for saving the checkbox values entered.
            tableDescriptor.QueryValue += new FieldValueEventHandler(tableDescriptor_QueryValue);
            tableDescriptor.SaveValue += new FieldValueEventHandler(tableDescriptor_SaveValue); 
            gridGroupingControl.TableControlQueryAllowSortColumn += new GridQueryAllowSortColumnEventHandler(gridGroupingControl_TableControlQueryAllowSortColumn);
		
			
        }

        /// <summary>
        /// Handles the SaveValue event of the tableDescriptor control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Grouping.FieldValueEventArgs"/> instance containing the event data.</param>
        void tableDescriptor_SaveValue(object sender, FieldValueEventArgs e)
        {
            if (e.Field.Name == LRRHeaderText.GetHeaderString("SID_SELECT"))
            {
                checkboxvalues[e.Record.GetData()] = e.Value;
                this.LastRecord = e.Record.GetData();
            }
        }

        /// <summary>
        /// Handles the QueryValue event of the tableDescriptor control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Grouping.FieldValueEventArgs"/> instance containing the event data.</param>
        void tableDescriptor_QueryValue(object sender, FieldValueEventArgs e)
        {
            if (e.Field.Name == LRRHeaderText.GetHeaderString("SID_SELECT"))
            {
                object value = checkboxvalues[e.Record.GetData()];
                if (value != null)
                {
                    e.Value = value;
                }
            }
        }

        /// <summary>
        /// This Method is used to get the selected records
        /// </summary>
        /// <returns></returns>
		public object[] GetSelectedRecords()
		{
			IDictionaryEnumerator Enumerator;
			ArrayList arlist = new ArrayList();
			Enumerator = checkboxvalues.GetEnumerator();
			while (Enumerator.MoveNext())
			{
				bool value = Enumerator.Value.ToString()==true.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				if(value)
				{
					arlist.Add(Enumerator.Key);
				}
			}
			object[] obj=new object[arlist.Count];
			for(int i=0;i<arlist.Count;i++)
			{
				obj[i] = arlist[i];
			}
			//Array.Copy(arlist,obj,arlist.Count);
			return obj;
		}

		/// <summary>
		/// This Method is used to get the selected records
		/// </summary>
		/// <returns></returns>
		public object[] GetSelectedRecords(string sortBy)
		{
			IDictionaryEnumerator Enumerator;
			ArrayList arlist = new ArrayList();
			Enumerator = checkboxvalues.GetEnumerator();
			while (Enumerator.MoveNext())
			{
				bool value = Enumerator.Value.ToString() == true.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				if (value)
				{
					arlist.Add(Enumerator.Key);
				}
			}
			arlist.Sort(new Symcon.BusinessObjects.ObjectPropertyComparer(sortBy));
			object[] obj = new object[arlist.Count];
			for (int i = 0; i < arlist.Count; i++)
			{
				obj[i] = arlist[i];
			}
			//Array.Copy(arlist,obj,arlist.Count);
			return obj;
		}

		/// <summary>
		/// This Method is used to get the unselected records 
		/// </summary>
		public object[] GetUnSelectedRecords()
		{
			IDictionaryEnumerator Enumerator;
			ArrayList arlist = new ArrayList();
			Enumerator = checkboxvalues.GetEnumerator();
			while (Enumerator.MoveNext())
			{
				bool value = Enumerator.Value.ToString()== false.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				if(value)
				{
					arlist.Add(Enumerator.Key);
				}
			}
			object[] obj=new object[arlist.Count];
			for(int i=0;i<arlist.Count;i++)
			{
				obj[i] = arlist[i];
			}
			//Array.Copy(arlist,obj,arlist.Count);
			return obj;
		}


		/// <summary>
		/// This Method is used to get the primarykey values of the selected records
		/// </summary>
		public string[] GetPrimaryKeys()
		{
			ArrayList arlist = new ArrayList();

			IDictionaryEnumerator Enumerator;
			Enumerator = checkboxvalues.GetEnumerator();
			while (Enumerator.MoveNext())
			{
				bool value = Enumerator.Value.ToString()==true.ToString(Symcon.BusinessObjects.SymConverter.Culture);
				if(value)
				{
					Type objSelect = Enumerator.Key.GetType(); 
				
					SymDataTableAttribute [] dTable = (SymDataTableAttribute[])objSelect.GetCustomAttributes(typeof(SymDataTableAttribute),true);
						
					if(dTable.Length > 0)
					{

						PropertyInfo[] properties = objSelect.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public);
						IEnumerator iPropEnum = properties.GetEnumerator();
						while (iPropEnum.MoveNext())
						{
							PropertyInfo property = (PropertyInfo)iPropEnum.Current;
							SymKeyFieldAttribute[] pkeyfield = (SymKeyFieldAttribute[])property.GetCustomAttributes(typeof(SymKeyFieldAttribute),false);
							if(pkeyfield.Length > 0)
							{
								if(property.GetValue(Enumerator.Key,null) == null)
									continue;

								if(property.PropertyType == typeof(string))
								{
									arlist.Add(property.GetValue(Enumerator.Key,null));
								}
							}
						}
					}
				}
			}
			string[] strArray=new string[arlist.Count];
			for(int i=0;i<arlist.Count;i++)
			{
				strArray[i] = arlist[i].ToString();
			}
			return strArray;
		}

		/// <summary>
		/// This method is used to set the records
		/// </summary>
		/// <param name="obj">
		/// 
		/// </param>
		public void SetSelectedRecords(object[] obj)
		{
			checkboxvalues.Clear();
			IEnumerator Enumerator;
			Enumerator = obj.GetEnumerator();
			while (Enumerator.MoveNext())
			{
				if(Enumerator.Current!=null)
					checkboxvalues[Enumerator.Current]=true;
			}
		}

        /// <summary>
        /// Sets the selected records.
        /// </summary>
        /// <param name="obj">The obj.</param>
        /// <param name="checkBoxValue">if set to <c>true</c> [check box value].</param>
		public void SetSelectedRecords(object[] obj,bool checkBoxValue)
		{
			checkboxvalues.Clear();
			IEnumerator Enumerator;
			Enumerator = obj.GetEnumerator();
			while (Enumerator.MoveNext())
			{
				if(Enumerator.Current!=null)
					checkboxvalues[Enumerator.Current]=checkBoxValue;
			}
		}
		/// <summary>
		/// This method is used to unset the records
		/// </summary>
		/// <param name="obj">
		/// 
		/// </param>
		public void ClearSelectedRecords()
		{
			checkboxvalues.Clear();
			
		}
		
		/// <summary>
		/// THis method sets/unsets a record;
		/// </summary>
		/// <param name="obj"></param>
		/// <param name="checkState"></param>
		public void SetRecord(object obj,bool checkState)
		{
			checkboxvalues[obj] = checkState;
		}
		#endregion


		#region EventHandlers

		//Getting the checkBox values in a hashtable.
        /// <summary>
        /// Handles the QueryValue event of the unboundField control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Grouping.FieldValueEventArgs"/> instance containing the event data.</param>
		private void unboundField_QueryValue(object sender, FieldValueEventArgs e)
		{
			if (e.Field.Name == LRRHeaderText.GetHeaderString("SID_SELECT") )
			{
				object value = checkboxvalues[e.Record.GetData()];
				if (value != null)
				{
					e.Value=value;
				}
			}
		}

		//Setting the checkbox values from the hashtable.
        /// <summary>
        /// Handles the SaveValue event of the unboundField control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Grouping.FieldValueEventArgs"/> instance containing the event data.</param>
		private void unboundField_SaveValue(object sender, FieldValueEventArgs e)
		{
			if (e.Field.Name == LRRHeaderText.GetHeaderString("SID_SELECT"))
			{
				checkboxvalues[e.Record.GetData()] = e.Value;
				this.LastRecord = e.Record.GetData();
			}
		}
        /// <summary>
        /// Handles the TableControlQueryAllowSortColumn event of the gridGroupingControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridQueryAllowSortColumnEventArgs"/> instance containing the event data.</param>
		private void gridGroupingControl_TableControlQueryAllowSortColumn(object sender, GridQueryAllowSortColumnEventArgs e)
		{
			if(e.Column.Name==LRRHeaderText.GetHeaderString("SID_SELECT"))
			{
				if(AllowSorting==false)
					e.AllowSort=false;
				else
					e.AllowSort=true;
			}
		}
		#endregion

		#region Fields
		private object _lastRecord;
		private bool	_allowSort=false;
		#endregion

		#region Properties
		public object LastRecord
		{
			get {return  _lastRecord;}
			set
			{
				_lastRecord = value;
			}
		}

		public bool AllowSorting
		{
			get
			{
				return _allowSort;
			}
			set
			{
				_allowSort=value;
			}
		}
		#endregion

	
	}
}
