using System;
using System.Reflection;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using Symcon.Utils.MessageLogger;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Tools;

using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using Symcon.LightRidge.PEAdvisor.DataStore;
using System.Globalization;


//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.



namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// Summary description for LRRGridControl.
	/// </summary>
	public class LRRGridControl : Syncfusion.Windows.Forms.Grid.GridDataBoundGrid 
	{
		public LRRGridControl()
		{
			this.AllowDragSelectedCols = true;
			this.AllowResizeToFit = false;
			this.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.Dock = System.Windows.Forms.DockStyle.Fill;
			this.HScrollPixel = true;
			this.Location = new System.Drawing.Point(0, 31);
			this.MinResizeColSize = 20;
			this.MinResizeRowSize = 10;
			this.Name = "";
			this.OptimizeInsertRemoveCells = true;
			this.Properties.BackgroundColor = System.Drawing.SystemColors.ActiveCaptionText;
			this.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.GrayWhenLostFocus;
			this.Size = new System.Drawing.Size(879, 310);
			this.SmartSizeBox = false;
			this.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.DoubleClick;
			this.TabIndex = 18;
			this.Text = "gridDataBoundGrid2";
			this.UseListChangedEvent = true;
			this.VScrollPixel = true;

		}

        /// <summary>
        /// Initializes the grid columns.
        /// </summary>
        /// <param name="type">The type.</param>
		public void InitializeGridColumns(Type type)
		{
			try
			{
				PropertyInfo[] properties = type.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public);
				ArrayList propList = new ArrayList();
				Hashtable referencedColumns = new Hashtable();
				foreach (PropertyInfo property in properties)
				{
					SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
						property.GetCustomAttributes(typeof(SymDataFieldAttribute),
						false);
					if (field.Length > 0)
					{
						SymDataFieldAttribute sDFA = field[0];
						if(sDFA.IsVisible)
							propList.Add(sDFA);
					}

					SymForeignKeyFieldAttribute[] fkField = (SymForeignKeyFieldAttribute[])
						property.GetCustomAttributes(typeof(SymForeignKeyFieldAttribute),
						false);

					if(fkField.Length > 0)
					{
						//referencedColumns
						SymForeignKeyFieldAttribute sFKeyFA = fkField[0];
						referencedColumns.Add(property.Name,sFKeyFA);
					}
					
				}

				propList.Sort();

				//gridInstance.GridBoundColumns.Clear();

				for(int i=0;i<propList.Count;i++)
				{
					SymDataFieldAttribute dFA = propList[i] as SymDataFieldAttribute;
					GridBoundColumn gbc = new GridBoundColumn();
					
					GridStyleInfo gsi = new GridStyleInfo();
					gsi.BaseStyle = "Standard";
					gsi.AutoSize = true;
					
					if(dFA.DisplayEdit == DisplayEditTypes.READONLY)
					{
						gsi.ReadOnly = true;
						gsi.BackColor = Color.LightGray;
					}
							
					if(dFA.DisplayType == DisplayTypes.LIST)
					{
						gsi.CellType = "GridListControl";
						gsi.VerticalScrollbar = true;
						//gsi.HasVerticalScrollbar = true;
						if(referencedColumns.ContainsKey(dFA.PropName))
						{
							SymForeignKeyFieldAttribute sFKeyFld = (SymForeignKeyFieldAttribute)referencedColumns[dFA.PropName];

							try
							{
								object collectionObj  = LRRDataStore.GetDataStoreObjectCopy(sFKeyFld.CollectionType);
								gsi.DisplayMember = sFKeyFld.DisplayMemeber;
								gsi.ValueMember = sFKeyFld.ValueMemeber;
								gsi.DataSource = collectionObj;
								gsi.ExclusiveChoiceList = true;
							}
							catch(Exception ex)
							{
												Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);
							}

						}
					}
					else if(dFA.DisplayType == DisplayTypes.TEXT)
					{
						if(dFA.PropType !=null)
							gsi.CellValueType = dFA.PropType;
						else
						{
							gsi.CellType = "TextBox";
						}
					}
					else if(dFA.DisplayType == DisplayTypes.BOOL)
					{
						//gsi.CellType = "DropDown";
						gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
						gsi.VerticalAlignment = GridVerticalAlignment.Middle;
						gsi.CellValueType = typeof(bool);
					}
					else if(dFA.DisplayType == DisplayTypes.DATETIME)
					{
						gsi.CellType = "MonthCalendar";
						/*DateTimePickerAdv dtPAdv = new DateTimePickerAdv();
						dtPAdv.Culture = new System.Globalization.CultureInfo("en-US");
						dtPAdv.CustomFormat = "HH:m , dd MM yyyy";
						dtPAdv.ShowCheckBox = false;
						dtPAdv.Format = System.Windows.Forms.DateTimePickerFormat.Custom;
						gsi.Control = dtPAdv;*/
						gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
						gsi.VerticalAlignment = GridVerticalAlignment.Middle;
						gsi.Format = "HH:m , dd MM yyyy";;
					}
			
					gbc.HeaderText = dFA.DisplayName;
					gbc.MappingName = dFA.PropName;
					gbc.StyleInfo = gsi;

					//gridInstance.GridBoundColumns.Add(gbc);

					//gridInstance.Model.ColWidths[i + 1] = (int)(gridInstance.Width * dFA.ColWidthPercantage) - (int)(System.Windows.Forms.SystemInformation.VerticalScrollBarWidth/(propList.Count - 1));
					
				}
				
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);
			}
		}
	}
}
