using System;
using System.Drawing;
using System.Diagnostics;
using System.Reflection;
using System.Collections;
using Symcon.Utils.MessageLogger;
using Syncfusion.Windows.Forms.Grid;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Tools;

using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
//using Symcon.LightRidge.PEAdvisor.DataStore;
using System.Globalization;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.HeaderDispatcher; 
using Symcon.Utils.MessageDispatcher;
using System.Collections.Generic;
//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// Summary description for SyncGridUtilities.
	/// </summary>
	public class SyncGridUtilities
	{
		//Added by Resmi For ACL
		private static GridTableDescriptor						_gridTableDescriptor=null;
		private static GridGroupingControl						_gridGroupingControl=null;
		private static GridTableDescriptor						_gridTableDescriptor1=null;
		private static GridGroupingControl						_gridGroupingControl1=null;
        private static ArrayList                                baseObjects = new ArrayList();
		//ACL Changes ends here
		private static object _oCollection;
		public SyncGridUtilities()
		{
			
		}

		#region GridBoundControl Utilities

		/// <summary>
		/// This functions is used to prepare the 
		/// filter string from the header of the grid.
		/// It check if there is a Extra Header 
		/// row for the Grid Manipulated the 
		/// Grid Bound Columns and
		/// returns the filter string.
		/// ******This function could be moved to a utility class going forward.
		/// </summary>
		/// <param name="gridInstance"></param>
		/// <returns>filter string to be applied on the datasource.</returns>
		public static string PrepareFilterString(GridDataBoundGrid gridInstance)
		{

			try
			{
				string filterString = ""; //Return value holding the filter string to the data source
				if(gridInstance.Model.Rows.HeaderCount == 1) //Identify if the grid Instance has a header
				{
					//Loop through the Columns and prepare the filter string for the data..
					
					int currentCellColIndex = -1;
					//First Get the Current Cell Value..
					GridCurrentCell gridCurCell = gridInstance.CurrentCell;
					if(gridCurCell.RowIndex == 1)
					{
						//The cursor is on the first row so get the colindex and the 
						//text for the cell
						currentCellColIndex = gridCurCell.ColIndex;

						//Support for only Combo box and text styles..
						if(gridCurCell.Renderer.StyleInfo.CellType == "ComboBox")
						{
							if((gridCurCell.Renderer.GetCellValue() != null) && (gridCurCell.Renderer.GetCellValue().ToString() != ""))
							{
								filterString = filterString + gridInstance.GridBoundColumns[currentCellColIndex - 1].MappingName + " = " + gridCurCell.Renderer.GetCellValue().ToString();
							}
						}
						else
						{
							if((gridCurCell.Renderer.GetDisplayText() != null) && (gridCurCell.Renderer.GetDisplayText() != ""))
							{

								filterString = filterString + gridInstance.GridBoundColumns[currentCellColIndex - 1].MappingName + " = " + gridCurCell.Renderer.GetCellValue().ToString();
							}
						}
					}

					//Run through all the rest of the columsn of the grid...
					for(int i=0;i<=gridInstance.GridBoundColumns.Count;i++)
					{
						if(i == currentCellColIndex)
							continue;

						if(gridInstance[1,i].CellType == "ComboBox")
						{
							if((gridInstance[1,i].CellValue != null) && (gridInstance[1,i].CellValue.ToString() != ""))
							{
								if(filterString == "")
								{
									filterString = filterString + gridInstance.GridBoundColumns[i - 1].MappingName + " = " + gridInstance[1,i].CellValue.ToString();
								}
								else
								{
									filterString = filterString + " AND " + gridInstance.GridBoundColumns[i - 1].MappingName + " = " + gridInstance[1,i].CellValue.ToString();
								}
							}
						}
						else
						{
							if((gridInstance[1,i].CellValue.ToString() != null) && (gridInstance[1,i].CellValue.ToString() != ""))
							{
								if(filterString == "")
								{
									filterString = filterString + gridInstance.GridBoundColumns[i - 1].MappingName + " = " + gridInstance[1,i].CellValue.ToString();
								}
								else
								{
									filterString = filterString + " AND " + gridInstance.GridBoundColumns[i - 1].MappingName + " = " + gridInstance[1,i].CellValue.ToString();
								}
							}
						}
					}
					return filterString;
				}
				else
				{
					return filterString;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(SyncGridUtilities),"PrepareFilterString :"+ex.Message  ,Logger.Level.Debug);
				return "";
			}
		}


		/// <summary>
		/// This method is used to set the filter on the grid control..
		/// </summary>
		public static void SetFilterHeader(GridDataBoundGrid gridInstance)
		{
			try
			{
				/*GridStyleInfo fixedColumnStyle = new GridStyleInfo();
				fixedColumnStyle.BaseStyle = "Standard";
				fixedColumnStyle.CellType = "Header";
				//fixedColumnStyle.BackColor = Color.LightBlue;
				for(int i=1;i<gridInstance.Model.ColCount + 1;i++)
				{
					gridInstance.Model.ChangeCells(GridRangeInfo.Cell(0,i),fixedColumnStyle);
				}*/

				GridStyleInfo style = new GridStyleInfo();
				style.BaseStyle = "Column Header";
				style.CellType = "TextBox";
				style.BackColor = Color.Khaki;
				//style.ImageList = imageList1;
				style.ImageIndex = 0;
				gridInstance.Model.Rows.HeaderCount = 1;
				gridInstance.Model.Rows.FrozenCount = 1;
				
				for(int i=1;i<gridInstance.Model.ColCount + 1;i++)
				{
					//Get the Target style from the Grid Bound Columns..
					GridStyleInfo targetStyle = gridInstance.GridBoundColumns[i-1].StyleInfo;

					style.DataSource = targetStyle.DataSource;
					if(targetStyle.CellType == "CheckBox")
					{
						style.ReadOnly = true;
						style.CellType = "TextBox";
						style.BackColor = Color.Gray;
					}
					else
					{
						style.CellType = targetStyle.CellType;
					}
					style.DisplayMember = targetStyle.DisplayMember;
					style.ValueMember = targetStyle.ValueMember;
					gridInstance.Model.ChangeCells(GridRangeInfo.Cell(1,i),style);
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(SyncGridUtilities),"SetFilterHeader :"+ex.Message  ,Logger.Level.Debug);

			}


		}
        private static Dictionary<Type, PropertyInfo[]> _propInfoForType = new Dictionary<Type, PropertyInfo[]>();
		/// <summary>
		/// This method is used to set the columns of the Grid Control
		/// depending on the type of object shown by the grid..
		/// </summary>
		/// <param name="gridInstance">Instance of the grid to operate on</param>
		/// <param name="type">the Type of the object to show the columns for.</param>
		public static void InitializeGridColumns(GridDataBoundGrid gridInstance,Type type,Hashtable relativeColumnsData)
		{
			try
			{
				if(relativeColumnsData != null)
					relativeColumnsData.Clear();

                PropertyInfo[] properties = null;// type.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
                if(_propInfoForType.ContainsKey(type))
                {
                    properties = _propInfoForType[type];
                }
                else
                {
                    properties = type.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
                }
				ArrayList propList = new ArrayList();
				Hashtable referencedColumns = new Hashtable();
				foreach (PropertyInfo property in properties)
				{
					SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
						property.GetCustomAttributes(typeof(SymDataFieldAttribute),
						false);
					if (field.Length > 0)
					{
						SymDataFieldAttribute sDFA = field[0];
						if(sDFA.IsVisible)
							propList.Add(sDFA);
					}

					SymForeignKeyFieldAttribute[] fkField = (SymForeignKeyFieldAttribute[])
						property.GetCustomAttributes(typeof(SymForeignKeyFieldAttribute),
						false);

					if(fkField.Length > 0)
					{
						//referencedColumns
						SymForeignKeyFieldAttribute sFKeyFA = fkField[0];
						referencedColumns.Add(property.Name,sFKeyFA);
					}
					if(relativeColumnsData != null)
					{
						SymRelativeDataAttribute[] relativeField = (SymRelativeDataAttribute[])
							property.GetCustomAttributes(typeof(SymRelativeDataAttribute),
							false);

						if(relativeField.Length > 0)
						{
							SymRelativeDataAttribute sRelativeData = relativeField[0];
							relativeColumnsData.Add(property.Name,sRelativeData);
						}
					}

				}

				propList.Sort();

				gridInstance.GridBoundColumns.Clear();

				for(int i=0;i<propList.Count;i++)
				{
					SymDataFieldAttribute dFA = propList[i] as SymDataFieldAttribute;
					GridBoundColumn gbc = new GridBoundColumn();
					
					GridStyleInfo gsi = new GridStyleInfo();
					gsi.BaseStyle = "Standard";
					gsi.AutoSize = true;
					
					if(dFA.DisplayEdit == DisplayEditTypes.READONLY)
					{
						gsi.ReadOnly = true;
						gsi.BackColor = Color.LightGray;
					}
							
					if(dFA.DisplayType == DisplayTypes.LIST)
					{
						gsi.CellType = "ComboBox";
						gsi.VerticalScrollbar = true;
						//LRRGridControl lgc = new LRRGridControl();
						//gsi.Control = lgc;

						//gsi.HasVerticalScrollbar = true;
						if(referencedColumns.ContainsKey(dFA.PropName))
						{
							SymForeignKeyFieldAttribute sFKeyFld = (SymForeignKeyFieldAttribute)referencedColumns[dFA.PropName];

							try
							{
								object collectionObj  = LRRModel.GetObjectCopy(sFKeyFld.CollectionType);// LRRDataStore.GetDataStoreObjectCopy();								
								gsi.DisplayMember = sFKeyFld.DisplayMemeber;
								gsi.ValueMember = sFKeyFld.ValueMemeber;
								gsi.DataSource = collectionObj;
								gsi.ExclusiveChoiceList = true;
								//lgc.InitializeGridColumns(sFKeyFld.CollectionType);
							}
							catch(Exception ex)
							{
								Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);

							}


						}
					}
					else if(dFA.DisplayType == DisplayTypes.TEXT)
					{
						if(dFA.PropType !=null)
							gsi.CellValueType = dFA.PropType;
						else
						{
							gsi.CellType = "TextBox";
						}
					}
					else if(dFA.DisplayType == DisplayTypes.BOOL)
					{
						//gsi.CellType = "DropDown";
						gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
						gsi.VerticalAlignment = GridVerticalAlignment.Middle;
						gsi.CellValueType = typeof(bool);
					}
					else if(dFA.DisplayType == DisplayTypes.TABLE)
					{
						gsi.CellType = "GridListControl";
						if(referencedColumns.ContainsKey(dFA.PropName))
						{
							SymForeignKeyFieldAttribute sFKeyFld = (SymForeignKeyFieldAttribute)referencedColumns[dFA.PropName];

							try
							{
								object collectionObj  = LRRModel.GetObjectCopy(sFKeyFld.CollectionType);
								gsi.DisplayMember = sFKeyFld.DisplayMemeber;
								gsi.ValueMember = sFKeyFld.ValueMemeber;
								gsi.DataSource = collectionObj;
								//gsi.ExclusiveChoiceList = true;
								//lgc.InitializeGridColumns(sFKeyFld.CollectionType);
							}
							catch(Exception ex)
							{
								Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);

							}


						}
					}
					else if(dFA.DisplayType == DisplayTypes.DATETIME)
					{
						gsi.CellType = "MonthCalendar";
						/*DateTimePickerAdv dtPAdv = new DateTimePickerAdv();
						dtPAdv.Culture = new System.Globalization.CultureInfo("en-US");
						dtPAdv.CustomFormat = "HH:m , dd MM yyyy";
						dtPAdv.ShowCheckBox = false;
						dtPAdv.Format = System.Windows.Forms.DateTimePickerFormat.Custom;
						gsi.Control = dtPAdv;*/
						gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
						gsi.VerticalAlignment = GridVerticalAlignment.Middle;
						gsi.Format = "HH:m , dd MM yyyy";;
					}
			
					//gbc.HeaderText = dFA.DisplayName;
					gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);  

					gbc.MappingName = dFA.PropName;
					gbc.StyleInfo = gsi;

					gridInstance.GridBoundColumns.Add(gbc);

					gridInstance.Model.ColWidths[i + 1] = (int)(gridInstance.Width * dFA.ColWidthPercantage) - (int)(System.Windows.Forms.SystemInformation.VerticalScrollBarWidth/(propList.Count - 1));
					
				}
				
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);

			}

		}


        ///// <summary>
        ///// This method is used to set the columns of the Grid Control
        ///// depending on the type of object shown by the grid..
        ///// </summary>
        ///// <param name="gridInstance">Instance of the grid to operate on</param>
        ///// <param name="type">the Type of the object to show the columns for.</param>
        //public static void InitializeGridColumns(GridDataBoundGrid gridInstance,Type type,Hashtable relativeColumnsData,bool useIsSelected)
        //{
        //    try
        //    {
        //        if(relativeColumnsData != null)
        //            relativeColumnsData.Clear();

        //        PropertyInfo[] properties;
        //        if(!useIsSelected)
        //        {
        //            properties = type.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public);
        //        }
        //        else
        //        {
        //            properties = type.GetProperties();
        //        }

        //        ArrayList propList = new ArrayList();
        //        Hashtable referencedColumns = new Hashtable();
        //        foreach (PropertyInfo property in properties)
        //        {
        //            SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
        //                property.GetCustomAttributes(typeof(SymDataFieldAttribute),
        //                true);
        //            if (field.Length > 0)
        //            {
        //                SymDataFieldAttribute sDFA = field[0];
        //                if(sDFA.IsVisible)
        //                    propList.Add(sDFA);
        //            }

        //            SymForeignKeyFieldAttribute[] fkField = (SymForeignKeyFieldAttribute[])
        //                property.GetCustomAttributes(typeof(SymForeignKeyFieldAttribute),
        //                false);

        //            if(fkField.Length > 0)
        //            {
        //                //referencedColumns
        //                SymForeignKeyFieldAttribute sFKeyFA = fkField[0];
        //                referencedColumns.Add(property.Name,sFKeyFA);
        //            }
        //            if(relativeColumnsData != null)
        //            {
        //                SymRelativeDataAttribute[] relativeField = (SymRelativeDataAttribute[])
        //                    property.GetCustomAttributes(typeof(SymRelativeDataAttribute),
        //                    false);

        //                if(relativeField.Length > 0)
        //                {
        //                    SymRelativeDataAttribute sRelativeData = relativeField[0];
        //                    relativeColumnsData.Add(property.Name,sRelativeData);
        //                }
        //            }

        //        }

        //        propList.Sort();

        //        gridInstance.GridBoundColumns.Clear();

        //        for(int i=0;i<propList.Count;i++)
        //        {
        //            SymDataFieldAttribute dFA = propList[i] as SymDataFieldAttribute;
        //            GridBoundColumn gbc = new GridBoundColumn();
					
        //            GridStyleInfo gsi = new GridStyleInfo();
        //            gsi.BaseStyle = "Standard";
        //            gsi.AutoSize = true;
        //            gsi.ReadOnly = true;
					
        //            if(dFA.DisplayEdit == DisplayEditTypes.READONLY)
        //            {
        //                gsi.ReadOnly = true;
        //                gsi.BackColor = Color.LightGray;
        //            }
							
        //            if(dFA.DisplayType == DisplayTypes.LIST)
        //            {
        //                gsi.CellType = "ComboBox";
        //                if(referencedColumns.ContainsKey(dFA.PropName))
        //                {
        //                    SymForeignKeyFieldAttribute sFKeyFld = (SymForeignKeyFieldAttribute)referencedColumns[dFA.PropName];

        //                    try
        //                    {
        //                        object collectionObj  = LRRModel.GetObjectCopy(sFKeyFld.CollectionType);
        //                        gsi.DisplayMember = sFKeyFld.DisplayMemeber;
        //                        gsi.ValueMember = sFKeyFld.ValueMemeber;
        //                        gsi.DataSource = collectionObj;
        //                        gsi.ExclusiveChoiceList = true;
        //                    }
        //                    catch(Exception ex)
        //                    {
        //                        Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);

        //                    }

        //                }
        //            }
        //            else if(dFA.DisplayType == DisplayTypes.TEXT)
        //            {
        //                if(dFA.PropType !=null)
        //                    gsi.CellValueType = dFA.PropType;
        //                else
        //                {
        //                    gsi.CellType = "TextBox";
        //                }
        //            }
        //            else if(dFA.DisplayType == DisplayTypes.BOOL)
        //            {
        //                if(i==0)
        //                    gsi.ReadOnly = false;
        //                //gsi.CellType = "DropDown";
        //                gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
        //                gsi.VerticalAlignment = GridVerticalAlignment.Middle;
        //                gsi.CellValueType = typeof(bool);
        //            }
        //            else if(dFA.DisplayType == DisplayTypes.DATETIME)
        //            {
        //                gsi.CellType = "MonthCalendar";
        //                /*DateTimePickerAdv dtPAdv = new DateTimePickerAdv();
        //                dtPAdv.Culture = new System.Globalization.CultureInfo("en-US");
        //                dtPAdv.CustomFormat = "HH:m , dd MM yyyy";
        //                dtPAdv.ShowCheckBox = false;
        //                dtPAdv.Format = System.Windows.Forms.DateTimePickerFormat.Custom;
        //                gsi.Control = dtPAdv;*/
        //                gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
        //                gsi.VerticalAlignment = GridVerticalAlignment.Middle;
        //                gsi.Format = "HH:m , dd MM yyyy";;
        //            }
			
        //            //gbc.HeaderText = dFA.DisplayName;
        //            gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);  
        //            gbc.MappingName = dFA.PropName;
        //            gbc.StyleInfo = gsi;

        //            gridInstance.GridBoundColumns.Add(gbc);

        //            gridInstance.Model.ColWidths[i + 1] = (int)(gridInstance.Width * dFA.ColWidthPercantage) - (int)(System.Windows.Forms.SystemInformation.VerticalScrollBarWidth/(propList.Count - 1));
					
        //        }
				
        //    }
        //    catch(Exception ex)
        //    {
        //        Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);

        //    }

        //}

		/// <summary>
		/// This method is used to set the columns of the Grid Control
		/// depending on the type of object shown by the grid..
		/// </summary>
		/// <param name="gridInstance">Instance of the grid to operate on</param>
		/// <param name="type">the Type of the object to show the columns for.</param>
        //public static void InitializeGridColumns(GridDataBoundGrid gridInstance,Type type,Hashtable relativeColumnsData,bool useIsSelected,Hashtable selectedFields)
        //{
        //    try
        //    {
        //        if(relativeColumnsData != null)
        //            relativeColumnsData.Clear();

        //        PropertyInfo[] properties;
        //        if(!useIsSelected)
        //        {
        //            properties = type.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public);
        //        }
        //        else
        //        {
        //            properties = type.GetProperties();
        //        }

        //        ArrayList propList = new ArrayList();
        //        Hashtable referencedColumns = new Hashtable();
        //        foreach (PropertyInfo property in properties)
        //        {
        //            if(!selectedFields.ContainsKey(property.Name))
        //                continue;
        //            SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
        //                property.GetCustomAttributes(typeof(SymDataFieldAttribute),
        //                true);
        //            if (field.Length > 0)
        //            {
        //                SymDataFieldAttribute sDFA = field[0];
        //                if(sDFA.IsVisible)
        //                    propList.Add(sDFA);
        //            }

        //            SymForeignKeyFieldAttribute[] fkField = (SymForeignKeyFieldAttribute[])
        //                property.GetCustomAttributes(typeof(SymForeignKeyFieldAttribute),
        //                false);

        //            if(fkField.Length > 0)
        //            {
        //                //referencedColumns
        //                SymForeignKeyFieldAttribute sFKeyFA = fkField[0];
        //                referencedColumns.Add(property.Name,sFKeyFA);
        //            }
        //            if(relativeColumnsData != null)
        //            {
        //                SymRelativeDataAttribute[] relativeField = (SymRelativeDataAttribute[])
        //                    property.GetCustomAttributes(typeof(SymRelativeDataAttribute),
        //                    false);

        //                if(relativeField.Length > 0)
        //                {
        //                    SymRelativeDataAttribute sRelativeData = relativeField[0];
        //                    relativeColumnsData.Add(property.Name,sRelativeData);
        //                }
        //            }

        //        }

        //        propList.Sort();

        //        gridInstance.GridBoundColumns.Clear();

        //        for(int i=0;i<propList.Count;i++)
        //        {
        //            SymDataFieldAttribute dFA = propList[i] as SymDataFieldAttribute;
        //            GridBoundColumn gbc = new GridBoundColumn();
					
        //            GridStyleInfo gsi = new GridStyleInfo();
        //            gsi.BaseStyle = "Standard";
        //            gsi.AutoSize = true;
        //            gsi.ReadOnly = true;
					
        //            if(dFA.DisplayEdit == DisplayEditTypes.READONLY)
        //            {
        //                gsi.ReadOnly = true;
        //                gsi.BackColor = Color.LightGray;
        //            }
							
        //            if(dFA.DisplayType == DisplayTypes.LIST)
        //            {
        //                gsi.CellType = "ComboBox";
        //                if(referencedColumns.ContainsKey(dFA.PropName))
        //                {
        //                    SymForeignKeyFieldAttribute sFKeyFld = (SymForeignKeyFieldAttribute)referencedColumns[dFA.PropName];

        //                    try
        //                    {
        //                        object collectionObj  = LRRModel.GetObjectCopy(sFKeyFld.CollectionType);
        //                        gsi.DisplayMember = sFKeyFld.DisplayMemeber;
        //                        gsi.ValueMember = sFKeyFld.ValueMemeber;
        //                        gsi.DataSource = collectionObj;
        //                        gsi.ExclusiveChoiceList = true;
        //                    }
        //                    catch(Exception ex)
        //                    {
        //                        Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);

        //                    }

        //                }
        //            }
        //            else if(dFA.DisplayType == DisplayTypes.TEXT)
        //            {
        //                if(dFA.PropType !=null)
        //                    gsi.CellValueType = dFA.PropType;
        //                else
        //                {
        //                    gsi.CellType = "TextBox";
        //                }
        //            }
        //            else if(dFA.DisplayType == DisplayTypes.BOOL)
        //            {
        //                if(i==0)
        //                    gsi.ReadOnly = false;
        //                //gsi.CellType = "DropDown";
        //                gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
        //                gsi.VerticalAlignment = GridVerticalAlignment.Middle;
        //                gsi.CellValueType = typeof(bool);
        //            }
        //            else if(dFA.DisplayType == DisplayTypes.DATETIME)
        //            {
        //                gsi.CellType = "MonthCalendar";
        //                /*DateTimePickerAdv dtPAdv = new DateTimePickerAdv();
        //                dtPAdv.Culture = new System.Globalization.CultureInfo("en-US");
        //                dtPAdv.CustomFormat = "HH:m , dd MM yyyy";
        //                dtPAdv.ShowCheckBox = false;
        //                dtPAdv.Format = System.Windows.Forms.DateTimePickerFormat.Custom;
        //                gsi.Control = dtPAdv;*/
        //                gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
        //                gsi.VerticalAlignment = GridVerticalAlignment.Middle;
        //                gsi.Format = "HH:m , dd MM yyyy";;
        //            }
			
        //            //gbc.HeaderText = dFA.DisplayName;
        //            gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);  
        //            gbc.MappingName = dFA.PropName;
        //            gbc.StyleInfo = gsi;

        //            gridInstance.GridBoundColumns.Add(gbc);

        //            gridInstance.Model.ColWidths[i + 1] = (int)(gridInstance.Width * dFA.ColWidthPercantage) - (int)(System.Windows.Forms.SystemInformation.VerticalScrollBarWidth/(propList.Count - 1));
					
        //        }
				
        //    }
        //    catch(Exception ex)
        //    {
        //        Logger.LogMessage(typeof(SyncGridUtilities),"InitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);

        //    }

        //}


		#region Private Methods

		#endregion

		#endregion

		#region GridGroupingControl Utilities
		
		public static IList sDataSource = null;
		public static string relation = null;
		public static string parentID = null;
		
		/// <summary>
		/// This Method is Used to Initilize the grid Columns.
		/// </summary>
		/// <param name="gridInstance">Instance of the Grid Grouping Control</param>
		/// <param name="type">The type of the Object to show the mapping names</param>
		/// <param name="relativeColumnsData">Hash Used to get the relative columns data e.gUOM and UOMType</param>
		public static void InitializeGridColumns(GridGroupingControl gridInstance,Type type,Hashtable relativeColumnsData)
		{
			internalInitializeGridColumns(gridInstance,gridInstance.TableDescriptor,type,relativeColumnsData,false,null,null,false);
		}

		public static void InitializeGridColumns(GridGroupingControl gridInstance,Type type,Hashtable relativeColumnsData,Hashtable foreignKeysColumnData)
		{
			internalInitializeGridColumns(gridInstance,gridInstance.TableDescriptor,type,relativeColumnsData,true,foreignKeysColumnData,null,false);
		}

		/// <summary>
		/// This method is used to set the columns of the Grid Control
		/// depending on the type of object shown by the grid..
		/// </summary>
		/// <param name="gridInstance">Instance of the grid to operate on</param>
		/// <param name="type">the Type of the object to show the columns for.</param>
		public static void InitializeGridColumns(GridGroupingControl gridInstance,GridTableDescriptor  childTableDescriptor,Type type,Hashtable relativeColumnsData)
		{
			internalInitializeGridColumns(gridInstance,childTableDescriptor,type,relativeColumnsData,false,null,null,false);
		}

		public static void InitializeGridColumns(GridGroupingControl gridInstance,GridTableDescriptor  childTableDescriptor,Type type,Hashtable relativeColumnsData,Hashtable foreignKeysColumnData)
		{
			internalInitializeGridColumns(gridInstance,childTableDescriptor,type,relativeColumnsData,true,foreignKeysColumnData,null,false);
		}


		/// <summary>
		/// This method is used to set the columns of the Grid Control
		/// depending on the type of object shown by the grid..
		/// </summary>
		/// <param name="gridInstance">Instance of the grid to operate on</param>
		/// <param name="type">the Type of the object to show the columns for.</param>
		public static void InitializeGridColumns(GridGroupingControl gridInstance,Type type,Hashtable relativeColumnsData,bool restorePreviousColumnWidths)
		{
			internalInitializeGridColumns(gridInstance,gridInstance.TableDescriptor,type,relativeColumnsData,restorePreviousColumnWidths,null,null,false);
		}

		/// <summary>
		/// This method is used to set the columns of the Grid Control
		/// depending on the type of object shown by the grid..
		/// </summary>
		/// <param name="gridInstance">Instance of the grid to operate on</param>
		/// <param name="type">the Type of the object to show the columns for.</param>
		/// To be used for displaying selective columns.
		public static void InitializeGridColumns(GridGroupingControl gridInstance,Type type,Hashtable relativeColumnsData,bool useIsSelected,Hashtable selectedFields)
		{
			internalInitializeGridColumns(gridInstance,gridInstance.TableDescriptor,type,relativeColumnsData,false,null,selectedFields,true);
		}
        /// <summary>
        /// Initializes the grid columns.
        /// </summary>
        /// <param name="gridInstance">The grid instance.</param>
        /// <param name="type">The type.</param>
        /// <param name="relativeColumnsData">The relative columns data.</param>
        /// <param name="useIsSelected">if set to <c>true</c> [use is selected].</param>
        /// <param name="selectedFields">The selected fields.</param>
        /// <param name="foreignKeysColumnData">The foreign keys column data.</param>
		public static void InitializeGridColumns(GridGroupingControl gridInstance,Type type,Hashtable relativeColumnsData,bool useIsSelected,Hashtable selectedFields,Hashtable foreignKeysColumnData)
		{
			internalInitializeGridColumns(gridInstance,gridInstance.TableDescriptor,type,relativeColumnsData,false,foreignKeysColumnData,selectedFields,true);
		}

        /// <summary>
        /// Initializes the grid columns.
        /// </summary>
        /// <param name="gridInstance">The grid instance.</param>
        /// <param name="mainTd">The main td.</param>
        /// <param name="type">The type.</param>
        /// <param name="relativeColumnsData">The relative columns data.</param>
        /// <param name="restorePreviousColumnWidths">if set to <c>true</c> [restore previous column widths].</param>
        /// <param name="foreignKeysColumnData">The foreign keys column data.</param>
        /// <param name="columnsToDisplay">The columns to display.</param>
        /// <param name="blnDisplayIsSelected">if set to <c>true</c> [BLN display is selected].</param>
		public static void InitializeGridColumns(GridGroupingControl gridInstance,GridTableDescriptor  mainTd,Type type,Hashtable relativeColumnsData,bool restorePreviousColumnWidths,Hashtable foreignKeysColumnData,Hashtable columnsToDisplay,bool blnDisplayIsSelected)
		{
			internalInitializeGridColumns(gridInstance,mainTd,type,relativeColumnsData,restorePreviousColumnWidths,foreignKeysColumnData,columnsToDisplay,blnDisplayIsSelected);
		}

		/*public static void InitializeGridColumns(GridGroupingControl gridInstance,Type type,Hashtable relativeColumnsData,bool useIsSelected)
		{
			internalInitializeGridColumns(gridInstance,gridInstance.TableDescriptor,type,relativeColumnsData,true,null,null,useIsSelected);
		}*/
		
		/// <summary>
		/// This Method is used to Initilize the Look and feel of the Grid for the gridGrouping Control
		/// </summary>
		/// <param name="gridGroupControl">instance of the Grid Crouping Control</param>
		public static void InitializeGridLookAndFeel(GridGroupingControl gridGroupControl)
		{

			gridGroupControl.Appearance.AnyHeaderCell.Font.Bold = true;
			gridGroupControl.Appearance.AnyHeaderCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.InactiveCaptionText);
			gridGroupControl.BackColor = System.Drawing.SystemColors.Window;
			gridGroupControl.Dock = System.Windows.Forms.DockStyle.Fill;
			gridGroupControl.Location = new System.Drawing.Point(0, 0);
			gridGroupControl.Size = new System.Drawing.Size(784, 144);
			gridGroupControl.TableDescriptor.Appearance.AlternateRecordFieldCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.InactiveCaptionText);
			gridGroupControl.TableDescriptor.Appearance.AnyHeaderCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.ControlLight);
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.Font.Bold = true;
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.PatternStyle.None, System.Drawing.SystemColors.WindowText, System.Drawing.Color.RoyalBlue);
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.ShowButtons = Syncfusion.Windows.Forms.Grid.GridShowButtons.Show;
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.TextColor = System.Drawing.Color.White;
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.TextMargins.Left = 2;
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.VerticalAlignment = Syncfusion.Windows.Forms.Grid.GridVerticalAlignment.Middle;
			gridGroupControl.TableDescriptor.Appearance.FilterBarCell.BackColor = Color.FromArgb(255,255,139)/*(245,202,115)*/;     
			gridGroupControl.TableOptions.CaptionRowHeight = 20;
			gridGroupControl.TableOptions.ColumnHeaderRowHeight = 30;
			gridGroupControl.Text = "gridGroupingControl1";
			gridGroupControl.ThemesEnabled = false;
			gridGroupControl.TopLevelGroupOptions.IsExpandedInitialValue = true;
			gridGroupControl.VersionInfo = "3.301.0.0";

		}
		/// <summary>
		/// This Method is used to Initilize the Look and feel of the Grid for the gridGrouping Control
		/// </summary>
		/// <param name="gridGroupControl">instance of the Grid Crouping Control</param>
		public static void InitializeMasterGridLookAndFeel(GridGroupingControl gridGroupControl)
		{

			gridGroupControl.Appearance.AnyHeaderCell.Font.Bold = true;
			gridGroupControl.Appearance.AnyHeaderCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.Color.LightSteelBlue);
			gridGroupControl.BackColor = System.Drawing.SystemColors.Window;
			gridGroupControl.Dock = System.Windows.Forms.DockStyle.Fill;
			gridGroupControl.Location = new System.Drawing.Point(0, 0);
			gridGroupControl.Size = new System.Drawing.Size(784, 144);
			gridGroupControl.TableDescriptor.Appearance.AlternateRecordFieldCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.Color.LightSteelBlue);
			gridGroupControl.TableDescriptor.Appearance.AnyHeaderCell.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.ControlLight);
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.Font.Bold = true;
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.PatternStyle.None, System.Drawing.SystemColors.WindowText, System.Drawing.Color.RoyalBlue);
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.ShowButtons = Syncfusion.Windows.Forms.Grid.GridShowButtons.Show;
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.TextColor = System.Drawing.Color.White;
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.TextMargins.Left = 2;
			gridGroupControl.TableDescriptor.Appearance.GroupCaptionCell.VerticalAlignment = Syncfusion.Windows.Forms.Grid.GridVerticalAlignment.Middle;
			gridGroupControl.TableDescriptor.Appearance.FilterBarCell.BackColor = Color.FromArgb(255,255,139);     
			gridGroupControl.TableOptions.CaptionRowHeight = 20;
			gridGroupControl.TableOptions.ColumnHeaderRowHeight = 30;
			gridGroupControl.Text = "gridGroupingControl1";
			gridGroupControl.ThemesEnabled = false;
			gridGroupControl.TopLevelGroupOptions.IsExpandedInitialValue = true;
			gridGroupControl.VersionInfo = "3.301.0.0";

		}
		/// <summary>
		/// This method is used to get the datasource of the grid
		/// in which the data from excel has to be imported
		/// </summary>
		/// <param name="gridGroupControl">Instance of the grid to operate on</param>
        public static void ImportFromExcel(GridGroupingControl gridGroupControl)
        {
            ArrayList arrDataSource = new ArrayList();
            if (gridGroupControl.TableControl.CurrentCell.IsActive)
            {
                if (gridGroupControl.TableControl.Table.CurrentRecord == null)
                {
                    GetDataSource(gridGroupControl);
                    arrDataSource.Add(sDataSource as CustomCollection);
                    ExcelMapping emForm = new ExcelMapping(arrDataSource, parentID, relation);
                    emForm.ShowDialog();
                }
                else
                {
                    IList dataSource = gridGroupControl.Engine.Table.SourceList;
                    arrDataSource.Add(dataSource);
                    ExcelMapping emForm = new ExcelMapping(arrDataSource);
                    emForm.ShowDialog();
                }
            }

        }


	/// <summary>
		/// This method is used to insert new rows
		/// in the grid
		/// </summary>
		/// <param name="gridGroupControl">Instance of the grid to operate on</param>
		/// <param name="iNumofRows">Number of rows to insert</param>
		public static void InsertNewRows(GridGroupingControl gridGroupControl,int iNumofRows )
		{
			if(gridGroupControl.TableControl.CurrentCell.IsActive)
			{
				if(gridGroupControl.TableControl.Table.CurrentRecord == null)
				{
					GetDataSource(gridGroupControl);
					for(int i = 0; i < iNumofRows;i++)
					{
						CustomCollection cc = sDataSource as CustomCollection;
						object obj = Activator.CreateInstance(cc.ItemType);
						obj.GetType().GetProperty(relation).SetValue(obj,parentID,null);
						cc.Add(obj);
						gridGroupControl.Refresh();
					}
				}
				else
				{
					for(int i = 0; i < iNumofRows;i++)
					{
						IList dataSource = gridGroupControl.Engine.Table.SourceList;
						CustomCollection cc = dataSource as CustomCollection;
						object obj = Activator.CreateInstance(cc.ItemType);
						cc.Add(obj);
						gridGroupControl.Refresh();
					}
				}
			}
			else
			{
				return;
			}
		}
		
		/// <summary>
		/// This method is used to get the datasource
		/// of the grid
		/// </summary>
		/// <param name="gridGroupControl">Instance of the grid to operate on</param>
		public static void GetDataSource(GridGroupingControl gridGroupControl)
		{
			Element eCurrentElement = gridGroupControl.TableControl.Table.CurrentElement;
			if(eCurrentElement != null)
			{
				string pkField = null;
				GridNestedTable gnt = eCurrentElement as GridNestedTable;
				GridNestedTable gnt1 = gnt;
				GridTableDescriptor gtd = null;
				while(gnt1 != null && gnt1.ChildTable != null)
				{
					gnt = gnt1;
					gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
					gtd = gnt.ParentTable.ParentTableDescriptor;
				}

				string sRelation = gnt.ChildTable.ParentTableDescriptor.Name;
				relation = gtd.Relations[sRelation].RelationKeys[0].ChildKeyFieldName;
				pkField = gtd.Relations[sRelation].RelationKeys[0].ParentKeyFieldName;
				parentID = gnt.ParentRecord.GetValue(pkField).ToString();
				string sChild = gtd.Relations[sRelation].ChildTableName;
				//sDataSource = gridGroupControl.Engine.SourceListSet[sChild].List;
			}
		}

		
		
		/// <summary>
		/// This methos is used for the selection paste on the grid...
		/// </summary>
		/// <param name="gridGroupControl"></param>
		public static void PasteSelection(GridGroupingControl gridGroupingControl)
		{
			try
			{
				if(gridGroupingControl.TableControl.CurrentCell.IsActive)
				{
					int iRow = gridGroupingControl.TableControl.CurrentCell.RowIndex;
					int iCol = gridGroupingControl.TableControl.CurrentCell.ColIndex;
					bool _isContain = false;
					object pasteValue = null;
					BaseObject obj1 = null;
					if(gridGroupingControl.TableControl.Table.CurrentRecord == null)
					{
						//Get Data Source.
						GetDataSource(gridGroupingControl);  
						CustomCollection cc = sDataSource as CustomCollection;
						PropertyInfo[] props = cc.ItemType.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);      
				
						Element eCurrentInnerElement = gridGroupingControl.Table.GetInnerMostCurrentElement();
						Element eCurrentElement = gridGroupingControl.TableControl.Table.CurrentElement;
						Element eParentElement = eCurrentElement.ParentElement;
						GridTable gt = eCurrentInnerElement.ParentTable as GridTable;

						if(gt != null)
						{
							GridRangeInfoList ranges = gt.TableModel.SelectedRanges;
							if(ranges.Count == 1 && ranges.ActiveRange.IsCells)
							{
								GridRangeInfo range = ranges.ActiveRange;
								for(int col = range.Left; col <= range.Right; ++col)
								{
									string prop = gt.TableModel.GetColumnDescriptorAt(range.Top, col).MappingName;  
									
									if(prop.IndexOf("_") != -1)
										_isContain = true;
									else
										_isContain = false;
									if(_isContain)
									{
										string[] res = prop.Split(new char[]{'_'}); 
										prop = res[0];
										obj1 = gt.TableModel.GetDisplayElementAt(range.Top).GetData() as BaseObject;     
										if(obj1 != null)
											pasteValue = obj1.GetType().GetProperty(prop).GetValue(obj1,null);      
									}
									else
									{
										for(int i =0;i< props.Length ;i++)
										{
											PropertyInfo property = props[i];
											SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
												property.GetCustomAttributes(typeof(SymDataFieldAttribute),
												false);
											if(field.Length > 0)
											{
												SymDataFieldAttribute sDFA = field[0];
												if(sDFA.PropName == prop)
												{
													if(sDFA.IsMandatory)
													{
														return;
													}
												}
											}
										}
										obj1 = gt.TableModel.GetDisplayElementAt(range.Top).GetData() as BaseObject;     
										if(obj1 != null)
											pasteValue = obj1.GetType().GetProperty(prop).GetValue(obj1,null);  
									}
									ArrayList selectedRecord = new ArrayList();
									for(int row = range.Top + 1; row <= range.Bottom; row++)
									{
										try
										{
											BaseObject obj = gt.TableModel.GetDisplayElementAt(row).GetData() as BaseObject;
											selectedRecord.Add(obj); 
										}
										catch(Exception ex)
										{
											Logger.LogMessage(typeof(SyncGridUtilities),"PasteSelection :"+ex.Message  ,Logger.Level.Debug);
										}
									}
									for(int i=0 ;i<selectedRecord.Count;i++)
									{
										BaseObject baseObj = selectedRecord[i] as BaseObject;
										baseObj.GetType().GetProperty( prop ).SetValue(baseObj,pasteValue,null); 
									}
								}
							}
						}
					}
					else
					{
						IList dataSource = gridGroupingControl.Engine.Table.SourceList;
						CustomCollection cc = dataSource as CustomCollection;
						PropertyInfo[] props = cc.ItemType.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);      

						string prop =gridGroupingControl.TableModel.GetColumnDescriptorAt(iRow,iCol).MappingName;  
						GridRangeInfoList ranges = gridGroupingControl.TableControl.Model.SelectedRanges;
						if(ranges.Count == 1 && ranges.ActiveRange.IsCells)
						{
							GridRangeInfo range = ranges.ActiveRange;
							for(int col = range.Left; col <= range.Right; ++col)
							{
								string  sType = gridGroupingControl.TableControl.Model[range.Top, col].CellType;
								if(prop.IndexOf("_") != -1)
									_isContain = true;
								else
									_isContain = false;
								if(_isContain)
								{
									string[] res = prop.Split(new char[]{'_'}); 
									prop = res[0];
									obj1 = gridGroupingControl.TableControl.Model.GetDisplayElementAt(range.Top).GetData() as BaseObject;     
									if(obj1 != null)
										pasteValue = obj1.GetType().GetProperty(prop).GetValue(obj1,null);     
								}
								else
								{
									for(int i =0;i< props.Length ;i++)
									{
										PropertyInfo property = props[i];
										SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
											property.GetCustomAttributes(typeof(SymDataFieldAttribute),
											false);
										if(field.Length > 0)
										{
											SymDataFieldAttribute sDFA = field[0];
											if(sDFA.PropName == prop)
											{
												if(sDFA.IsMandatory)
												{
													return;
												}
											}
										}
									}
									obj1 = gridGroupingControl.TableControl.Model.GetDisplayElementAt(range.Top).GetData() as BaseObject;     
									if(obj1 != null)
										pasteValue = obj1.GetType().GetProperty(prop).GetValue(obj1,null);  
								}
								ArrayList selectedRecord = new ArrayList();
								for(int row = range.Top + 1; row <= range.Bottom; row++)
								{
									try
									{
										BaseObject obj = gridGroupingControl.TableControl.Model.GetDisplayElementAt(row).GetData() as BaseObject;
										selectedRecord.Add(obj); 
									}
									catch(Exception ex)
									{
										Logger.LogMessage(typeof(SyncGridUtilities),"PasteSelection :"+ex.Message  ,Logger.Level.Debug);
									}
								}
								for(int i=0 ;i<selectedRecord.Count;i++)
								{
									BaseObject baseObj = selectedRecord[i] as BaseObject;
									baseObj.GetType().GetProperty( prop ).SetValue(baseObj,pasteValue,null); 
								}
							}
						}
					}
					gridGroupingControl.Refresh();
				}
			}
			catch
			{

			}
		}
		/// <summary>
		/// This method the clears the sorting from the grid
		/// </summary>
		/// <param name="gridGroupControl"></param>
		public static void ClearSort(GridGroupingControl gridGroupControl)
		{
			gridGroupControl.TableDescriptor.SortedColumns.Clear();
			int iChildCount = gridGroupControl.TableDescriptor.Relations.Count;
			for(int i=0;i < iChildCount;i++)
			{
				GridRelationDescriptor grd = gridGroupControl.TableDescriptor.Relations[i];
				if(grd.RelationKind == RelationKind.RelatedMasterDetails)
				{
					ClearChildSort(grd);
				}
			}
		}
		
		/// <summary>
		/// Recursive method for Clearing Child Grid Sorting
		/// </summary>
		/// <param name="grd"></param>
		private static void ClearChildSort(GridRelationDescriptor grd)
		{
			grd.ChildTableDescriptor.SortedColumns.Clear();
			int iChildCount = grd.ChildTableDescriptor.Relations.Count;
			for(int i=0;i < iChildCount;i++)
			{
				GridRelationDescriptor grd1 = grd.ChildTableDescriptor.Relations[i];
				if(grd1.RelationKind == RelationKind.RelatedMasterDetails)
				{
					ClearChildSort(grd1);
				}
			}
		}
		/// <summary>
		/// This method clears the Filter applied to  the grid
		/// </summary>
		/// <param name="gridGroupControl"></param>
		public static void ClearFilter(GridGroupingControl gridGroupControl)
		{
			gridGroupControl.TableDescriptor.RecordFilters.Clear();
			int iChildCount = gridGroupControl.TableDescriptor.Relations.Count;
			for(int i=0;i < iChildCount;i++)
			{
				GridRelationDescriptor grd = gridGroupControl.TableDescriptor.Relations[i];
				if(grd.RelationKind == RelationKind.RelatedMasterDetails)
				{
					ClearChildFilter(grd);
				}
			}
		}
		
		/// <summary>
		/// Recursive method for Clearing Filter applied to  the Childgrid
		/// </summary>
		/// <param name="grd"></param>
		private static void ClearChildFilter(GridRelationDescriptor grd)
		{
			grd.ChildTableDescriptor.RecordFilters.Clear();
			int iChildCount = grd.ChildTableDescriptor.Relations.Count;
			for(int i=0;i < iChildCount;i++)
			{
				GridRelationDescriptor grd1 = grd.ChildTableDescriptor.Relations[i];
				if(grd1.RelationKind == RelationKind.RelatedMasterDetails)
				{
					ClearChildFilter(grd1);
				}
			}
		}

		/// <summary>
		/// Takes one argument gridInstance -- Object of a grid from which data to be deleted.
		/// </summary>
		/// <param name="gridInstance"></param>
		public static void DeleteRecords(GridGroupingControl gridInstance)
		{
			//top and bottom variable stored the top & bottom value for selected range.
			int top,bottom; 
			if(gridInstance.TableControl.TableDescriptor.AllowRemove == true)
			{
				if(gridInstance.TableControl.CurrentCell.IsActive)
				{
					if(gridInstance.TableControl.Table.CurrentRecord == null)
					{
						//method called for getting datasorce for the corresponding gridInstance
						GetDataSource(gridInstance);  
						//Delete Data From Child Level Grid.
						CustomCollection cc = sDataSource as CustomCollection;
						Element eCurrentInnerElement = gridInstance.Table.GetInnerMostCurrentElement();
						Element eCurrentElement = gridInstance.TableControl.Table.CurrentElement;
						GridTable gt = eCurrentInnerElement.ParentTable as GridTable;
						GridNestedTable gnt = eCurrentElement as GridNestedTable;
						GridNestedTable gnt1 = gnt;
						GridTableDescriptor gtd = null;
						//Loop throgh we get the childlevel grid data.
						while(gnt1 != null && gnt1.ChildTable != null)
						{
							gnt = gnt1;
							gnt1 = gnt.ChildTable.ParentTable.CurrentElement as GridNestedTable;
							gtd = gnt.ParentTable.ParentTableDescriptor;
						}
						GridChildTable gt1 = gnt.ChildTable;
						GridRangeInfoList list = gt.TableModel.SelectedRanges;  
						if(gt1.ParentTableDescriptor.AllowRemove == true)
						{
							if(list.Count == 1)
							{
								GridRangeInfo range = list.ActiveRange;
								int selRows = range.Height;
								if(gridInstance.NestedTableGroupOptions.ShowFilterBar == true)  
								{
									top  = range.Top-3;
									bottom = range.Bottom-3;
								}
								else
								{
									top = range.Top-2;
									bottom = range.Bottom-2;
								}
								DialogResult _dlgResult = LRRMessagBox.ShowMessage("SID_DELETE_RECORD","SID_LABEL",MessageBoxButtons.YesNo,MessageBoxIcon.Question);     
								if(_dlgResult == DialogResult.Yes)
								{
									for(int i=selRows;i>0;i--)
									{
										object obj = gt1.Records[bottom].GetData();
										cc.Remove(obj); 
										bottom--;
									}
									list.Clear(); 
								}
								else
								{
									return;
								}
							}
						}
					}
					else
					{
						//Delete Data From ParantGrid.
						IList dataSource = gridInstance.Engine.Table.SourceList;
						CustomCollection cc = dataSource as CustomCollection;
						GridRangeInfoList list = gridInstance.TableControl.Model.SelectedRanges;
						if(list.Count == 1)
						{
							GridRangeInfo range = list.ActiveRange;
							int selRows = range.Height;
							if(gridInstance.TopLevelGroupOptions.ShowFilterBar == true)  
							{
								top  = range.Top-4;
								bottom = range.Bottom-4;
							}
							else
							{
								top = range.Top-3;
								bottom = range.Bottom-3;
							}
							DialogResult _dlgResult = LRRMessagBox.ShowMessage("SID_DELETE_RECORD","SID_LABEL",MessageBoxButtons.YesNo,MessageBoxIcon.Question);     
							if(_dlgResult == DialogResult.Yes)
							{
								for(int i=selRows;i>0;i--)
								{
									try
									{
										object obj = gridInstance.TableModel.Table.Records[bottom].GetData();
										cc.Remove(obj); 
										bottom--;
									}
									catch
									{
									}
								}
								list.Clear();  
							}
							else
							{
								return;
							}
						}					
					}
				}
			}
		}

        /// <summary>
        /// RollBack Changes. Undo Functionality.
        /// </summary>
        /// <param name="gridGroupingControl"></param>
        public static void RollBackChanges(GridGroupingControl gridGroupingControl)
        {
            TimeComparer timeComparer = new TimeComparer();
            foreach (SourceListSetEntry sse in gridGroupingControl.Engine.SourceListSet)
            {
                CustomCollection cc = sse.List as CustomCollection;
                foreach (BaseObject obj in cc)
                {
                    if (obj.ObjectState != DBOperation.UNCHANGED)
                    {
                        if (obj.UpdatedRecordStack.Count > 0)
                        {
                            object updatedObject = obj.UpdatedRecordStack.Peek();
                            baseObjects.Add(updatedObject);
                        }
                    }
                }
                foreach (BaseObject obj in cc.GetChanges())
                {
                    if (obj.ObjectState == DBOperation.DELETE)
                    {
                        if (obj.UpdatedRecordStack.Count > 0)
                        {
                            object updatedObject = obj.UpdatedRecordStack.Peek();
                            baseObjects.Add(updatedObject);
                        }
                    }
                }
            }
            if (baseObjects.Count > 0)
            {
                baseObjects.Sort(timeComparer);
                UpdatedObject updatedObj = baseObjects[0] as UpdatedObject;
                BaseObject obj = updatedObj.BsObject as BaseObject;
                if (obj.ObjectState == DBOperation.DELETE || obj.ObjectState == DBOperation.INSERT)
                {
                    CustomCollection cc = updatedObj.Collection as CustomCollection;
                    cc.RollBackChanges(obj);
                }
                else
                {
                    obj.RollBackChanges();
						 //remove the obj from GridChangedRecordLists
						 if(GridChangedRecordLists.GetInstance().Count>0)
						 {
							 IEnumerator ieNumChangedRecordLists = GridChangedRecordLists.GetInstance().GetEnumerator();
							 while(ieNumChangedRecordLists.MoveNext())
								{ 
									 string dataType=((GridChangedRecord)(ieNumChangedRecordLists.Current)).DataTypeStored;
									 if(obj.GetType().ToString()==dataType)
									 {
										 if(((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList!=null)
											{
											 if(!((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList.Contains(obj))						
													((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList.Remove(obj);
											}

									 }
								}
						 }

                }
                baseObjects.Clear();
             	//gridGroupingControl.Refresh();
            }
        }

        /// <summary>
        /// Rolls the back changes.
        /// </summary>
        /// <param name="gridGroupingControl">The grid grouping control.</param>
        /// <param name="isRollBackAll">if set to <c>true</c> [is roll back all].</param>
        public static void RollBackChanges(GridGroupingControl gridGroupingControl,bool isRollBackAll)
        {
            TimeComparer timeComparer = new TimeComparer();
            foreach (SourceListSetEntry sse in gridGroupingControl.Engine.SourceListSet)
            {
                CustomCollection cc = sse.List as CustomCollection;
                foreach (BaseObject obj in cc)
                {
                    if (obj.ObjectState != DBOperation.UNCHANGED)
                    {
                        if (obj.UpdatedRecordStack.Count > 0)
                        {
                            object updatedObject = obj.UpdatedRecordStack.Peek();
                            baseObjects.Add(updatedObject);
                        }
                    }
                }
                foreach (BaseObject obj in cc.GetChanges())
                {
                    if (obj.ObjectState == DBOperation.DELETE)
                    {
                        if (obj.UpdatedRecordStack.Count > 0)
                        {
                            object updatedObject = obj.UpdatedRecordStack.Peek();
                            baseObjects.Add(updatedObject);
                        }
                    }
                }
            }
            if (baseObjects.Count > 0)
            {
                baseObjects.Sort(timeComparer);
                foreach (UpdatedObject updatedObj in baseObjects)
                {
                    BaseObject obj = updatedObj.BsObject as BaseObject;
                    if (obj.ObjectState == DBOperation.DELETE || obj.ObjectState == DBOperation.INSERT)
                    {
                        CustomCollection cc = updatedObj.Collection as CustomCollection;
                        cc.RollBackChanges(obj);
                    }
                    else
                    {
                        obj.RollBackChanges(isRollBackAll);
                    }
                }
                baseObjects.Clear();
            }
        }



		#region Private Methods

		/// <summary>
		/// This method is used for the overloaded call for the InitializeGridColumns
		/// </summary>
		/// <param name="gridInstance">Instance of the Grid</param>
		/// <param name="mainTd">The Table Descriptor</param>
		/// <param name="type">The type of the object to get the property info</param>
		/// <param name="relativeColumnsData">returned relative columns data</param>
		/// <param name="restorePreviousColumnWidths">flag to restore column width</param>
		private static void internalInitializeGridColumns(GridGroupingControl gridInstance,GridTableDescriptor  mainTd,Type type,Hashtable relativeColumnsData,bool restorePreviousColumnWidths,Hashtable foreignKeysColumnData,Hashtable columnsToDisplay,bool blnDisplayIsSelected)
		{
			try
			{
				if(relativeColumnsData != null)
					relativeColumnsData.Clear();

				PropertyInfo[] properties;

				if(columnsToDisplay != null)    
				{
					properties = type.GetProperties();
				}
				else
				{
					properties = type.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public);
				}

				ArrayList propList = new ArrayList();
				Hashtable referencedColumns = new Hashtable();

				Hashtable colWidthHash = new Hashtable();

				foreach (PropertyInfo property in properties)
				{
					if(columnsToDisplay != null)
					{
						if(!columnsToDisplay.ContainsKey(property.Name))
							continue;
					}
					SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
						property.GetCustomAttributes(typeof(SymDataFieldAttribute),
						false);
					if (field.Length > 0)
					{
						SymDataFieldAttribute sDFA = field[0];
						if(sDFA.IsVisible)
							propList.Add(sDFA);
					}
					SymViewFieldAttribute[] viewField = (SymViewFieldAttribute[])
						property.GetCustomAttributes(typeof(SymViewFieldAttribute),
						false);
					if (viewField.Length > 0)
					{
						SymViewFieldAttribute sVFA = viewField[0];
						SymDataFieldAttribute sDFA = new SymDataFieldAttribute(sVFA.ColumnName,sVFA.PropName,sVFA.DisplayName,sVFA.IsVisible,sVFA.DisplayIndex,sVFA.DisplayType,sVFA.DisplayEdit,sVFA.ColWidthPercantage,sVFA.PropType);
						if(sDFA.IsVisible)
							propList.Add(sDFA);
					}


					SymForeignKeyFieldAttribute[] fkField = (SymForeignKeyFieldAttribute[])
						property.GetCustomAttributes(typeof(SymForeignKeyFieldAttribute),
						false);

					if(fkField.Length > 0)
					{
						//referencedColumns
						SymForeignKeyFieldAttribute sFKeyFA = fkField[0];
						referencedColumns.Add(property.Name,sFKeyFA);
					}
					if(relativeColumnsData != null)
					{
						SymRelativeDataAttribute[] relativeField = (SymRelativeDataAttribute[])
							property.GetCustomAttributes(typeof(SymRelativeDataAttribute),
							false);

						if(relativeField.Length > 0)
						{
							SymRelativeDataAttribute sRelativeData = relativeField[0];
							relativeColumnsData.Add(property.Name,sRelativeData);
						}
					}

				}

				propList.Sort();

				if(restorePreviousColumnWidths)
				{
					foreach(GridColumnDescriptor oldColWidth in mainTd.Columns)
					{
						colWidthHash.Add(oldColWidth.HeaderText,oldColWidth.Width); 
					}
				}

				mainTd.Columns.Clear();

				for(int i=0;i<propList.Count;i++)
				{
					SymDataFieldAttribute dFA = propList[i] as SymDataFieldAttribute;
					GridColumnDescriptor  gbc = new GridColumnDescriptor();
					GridStyleInfo gsi = gbc.Appearance.GetStyle(GridTableCellType.AnyRecordFieldCell);
					gsi.BaseStyle = "Standard";
					gsi.AutoSize = true;
					gsi.AllowEnter = false;
					
					if(dFA.DisplayEdit == DisplayEditTypes.READONLY)
					{
						gsi.ReadOnly = true;
						gsi.BackColor = Color.LightGray;
					}
							
					if(dFA.DisplayType == DisplayTypes.LIST)
					{
						if(referencedColumns.ContainsKey(dFA.PropName))
						{
							SymForeignKeyFieldAttribute sFKeyFld = (SymForeignKeyFieldAttribute)referencedColumns[dFA.PropName];
							try
							{
								object collectionObj;
								if(foreignKeysColumnData == null)
								{
									collectionObj  = LRRModel.GetObjectCopy(sFKeyFld.CollectionType);
								}
								else
								{
									collectionObj = foreignKeysColumnData[dFA.PropName];
								}
								if(sFKeyFld.ShowSimple)
								{
									gsi.CellType = "ComboBox";
									gsi.VerticalScrollbar = true;
								
									gsi.DisplayMember = sFKeyFld.DisplayMemeber;
									gsi.ValueMember = sFKeyFld.ValueMemeber;
									gsi.DataSource = collectionObj;
									gsi.ExclusiveChoiceList = true;

									gbc.MappingName = dFA.PropName;
									gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);
									//gbc.HeaderText = dFA.DisplayName;
									gbc.AllowFilter = true;
									mainTd.Columns.Add(gbc);
									
								}
								else
								{
									
									IList tmp = collectionObj as IList;
									gridInstance.Engine.SourceListSet.Add(dFA.PropName,tmp);
									GridRelationDescriptor childReltion = new GridRelationDescriptor(dFA.PropName);
									mainTd.Relations.Add(childReltion);
									childReltion.Name = dFA.PropName;
									childReltion.RelationKind = RelationKind.ForeignKeyReference;
									childReltion.ChildTableName = dFA.PropName;  // SourceListSet name for lookup
									childReltion.RelationKeys.Add(dFA.PropName,sFKeyFld.ValueMemeber);
									childReltion.ChildTableDescriptor.Appearance.AlternateRecordFieldCell.BackColor = Color.Beige;
									childReltion.ChildTableDescriptor.TopLevelGroupOptions.ShowFilterBar = true;
									childReltion.ChildTableDescriptor.VisibleColumns.Clear();

									if(sFKeyFld.DisplayMembers != null)
									{
										//Check for sort By PK
										if(sFKeyFld.SortByID)
										{
											CustomCollection custCollection = tmp as CustomCollection;
											
											Type custType = custCollection.ItemType;
											//Find PK for this collection
											string primaryKey = "";
											PropertyInfo[] propInfos = custType.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly| BindingFlags.Public);
											IEnumerator iPropEnum = propInfos.GetEnumerator();
											while (iPropEnum.MoveNext())
											{
												PropertyInfo propInfo = (PropertyInfo)iPropEnum.Current;
												SymKeyFieldAttribute[] keyfield = (SymKeyFieldAttribute[])propInfo.GetCustomAttributes(typeof(SymKeyFieldAttribute),
													true);
												if(keyfield.Length > 0)
												{
													primaryKey = propInfo.Name;
													break;
												}
											}

											//Sort By PK
											custCollection.SortColumn = primaryKey;
											custCollection.Sort();
										}

										for(int j=0;j<sFKeyFld.DisplayMembers.Length;j++)
										{
											childReltion.ChildTableDescriptor.VisibleColumns.Add(sFKeyFld.DisplayMembers[j]);
											childReltion.ChildTableDescriptor.Columns[sFKeyFld.DisplayMembers[j]].HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);   /*" " + sFKeyFld.DisplayMembers[j] + " ";*/
											childReltion.ChildTableDescriptor.AllowEdit = false;
											childReltion.ChildTableDescriptor.AllowNew = false;  // Make pencil icon disappear, users can't modify.
											childReltion.ChildTableDescriptor.TopLevelGroupOptions.ShowFilterBar = true;
											if(!sFKeyFld.SortByID)
											{
												childReltion.ChildTableDescriptor.SortedColumns.Add(sFKeyFld.DisplayMembers[j]);//Vi
											}
										}
										GridColumnDescriptor tmpGBC= new GridColumnDescriptor();
										tmpGBC.MappingName = dFA.PropName + "_" + sFKeyFld.DisplayMembers[0];
										tmpGBC.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);
										tmpGBC.AllowFilter = true;

										tmpGBC.Appearance.AnyRecordFieldCell.DropDownStyle = GridDropDownStyle.Exclusive;

										mainTd.Columns.Add(tmpGBC);
									}
									else
									{
										GridVisibleColumnDescriptor gvcd = new GridVisibleColumnDescriptor();
										childReltion.ChildTableDescriptor.VisibleColumns.Add(sFKeyFld.DisplayMemeber);
										childReltion.ChildTableDescriptor.Columns[sFKeyFld.DisplayMemeber].HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);     
										childReltion.ChildTableDescriptor.AllowEdit = false;
										childReltion.ChildTableDescriptor.AllowNew = false;
										childReltion.ChildTableDescriptor.TopLevelGroupOptions.ShowFilterBar = true;
										childReltion.ChildTableDescriptor.ResetSortedColumns();
										childReltion.ChildTableDescriptor.SortedColumns.Add(sFKeyFld.DisplayMemeber);

										gbc.MappingName = dFA.PropName + "_" + sFKeyFld.DisplayMemeber;
										gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);
										//gbc.HeaderText = dFA.DisplayName;
										gbc.AllowFilter = true;

										gbc.Appearance.AnyRecordFieldCell.DropDownStyle = GridDropDownStyle.Exclusive;
										gbc.Appearance.AnyRecordFieldCell.ExclusiveChoiceList = true;

										
										mainTd.Columns.Add(gbc);

									}
								}
							}
							catch(Exception ex)
							{
								Logger.LogMessage(typeof(SyncGridUtilities),"internalInitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);

							}

						}
//						else
//						{
//							gsi.CellType = "ComboBox";
//							gsi.VerticalScrollbar = true;
//								
//							gsi.DisplayMember = sFKeyFld.DisplayMemeber;
//							gsi.ValueMember = sFKeyFld.ValueMemeber;
//							gsi.DataSource = collectionObj;
//							gsi.ExclusiveChoiceList = true;
//
//							gbc.MappingName = dFA.PropName;
//							gbc.HeaderText = dFA.DisplayName;
//							gbc.AllowFilter = true;
//							mainTd.Columns.Add(gbc);
//						}
					}
					else if(dFA.DisplayType == DisplayTypes.TEXT)
					{
						gsi.CellType = "TextBox";
						gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);
						//gbc.HeaderText = dFA.DisplayName;
					}
					else if(dFA.DisplayType == DisplayTypes.CUSTOM)
					{
						gsi.CellValueType = dFA.PropType;
						gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);
						//gbc.HeaderText = dFA.DisplayName;
					}
					else if(dFA.DisplayType == DisplayTypes.BOOL)
					{
						gsi.HorizontalAlignment = GridHorizontalAlignment.Center;
						gsi.VerticalAlignment = GridVerticalAlignment.Middle;
						if(dFA.DisplayName != " ")
						{
							gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);
						}
						else
						{
							gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.PropName);
						}
						//gbc.HeaderText = dFA.DisplayName;

						if (blnDisplayIsSelected == true)
						{
							gsi.CellType = "CheckBox";
							gsi.CellValueType = typeof(bool);
						}

					}
					else if(dFA.DisplayType == DisplayTypes.DATETIME)
					{
						gsi.CellType  = "MonthCalendar";
						gsi.CellValueType = DateTime.Now.GetType();
						gsi.Format = Symcon.BusinessObjects.SymConverter.LRR_DATE_TIME_FORMAT;
						gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);
						//gbc.HeaderText = dFA.DisplayName;
					}

					if(referencedColumns.ContainsKey(dFA.PropName))
					{
						gbc.AllowFilter = true;
					}
					else
					{
						gbc.MappingName = dFA.PropName;
//						try
//						{
							if(gsi.ToString().IndexOf("LRRFormula") !=-1)
							{
								gbc.AllowFilter = false;
								gbc.Appearance.FilterBarCell.AllowEnter = false;
						
							}
							else
							{
								gbc.AllowFilter = true;
								gbc.Appearance.FilterBarCell.AllowEnter = true;
								if(dFA.DisplayName != " ")
								{
									gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.DisplayName);
								}
								else
								{
									gbc.HeaderText = LRRHeaderText.GetHeaderString(dFA.PropName);
								}
							}

//							if(gsi.CellValueType == typeof(LRRFormula))
//							{
//								gbc.AllowFilter = false;
//								gbc.Appearance.FilterBarCell.AllowEnter = false;
//						
//							}
//						}
//						catch
//						{
//							gbc.AllowFilter = true;
//							gbc.Appearance.FilterBarCell.AllowEnter = true;
//						}

						mainTd.Columns.Add(gbc);
					}

					foreach(GridColumnDescriptor newColWidth in mainTd.Columns)
					{
						if(colWidthHash.ContainsKey(newColWidth.HeaderText))
						{
							newColWidth.Width = (int)colWidthHash[newColWidth.HeaderText];
						}
					}
				}
				if(!restorePreviousColumnWidths)
					InitializeGridColumnsWidth(gridInstance);
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(SyncGridUtilities),"internalInitializeGridColumns :"+ex.Message  ,Logger.Level.Debug);
			}
			gridInstance.TableControlCellClick+=new GridTableControlCellClickEventHandler(gridInstance_TableControlCellClick);
		}


		#endregion

		#endregion

		#region gridSizing

		public static void InitializeGridColumnsWidth(GridGroupingControl gridInstance)
		{
			int nCols = gridInstance.TableDescriptor.Columns.Count + 1;
			int nRelWidthForCol;
			int nWidthForGrid;
			int nWidthForCol;
			int nColCtr = 0;
			nRelWidthForCol = Math.DivRem(100,nCols,out nRelWidthForCol);
			nWidthForGrid= Math.DivRem(gridInstance.Size.Width,100,out nWidthForGrid);
			nWidthForCol = nWidthForGrid * nRelWidthForCol;

			foreach(GridColumnDescriptor newColWidth in gridInstance.TableDescriptor.Columns)
			{
				if (gridInstance.TableControl.GetColWidth(nColCtr) <nWidthForCol)
					newColWidth.Width = nWidthForCol;
				nColCtr++;
			}
		}
		public static void InitializeGridColumnsWidth(GridGroupingControl gridInstance,Hashtable RelativeWidths )
		{
			int nCols = gridInstance.TableDescriptor.Columns.Count;
			int nRelWidthForCol;
			int nWidthForGrid;
			int nWidthForCol;
			string sColumnName;

			nWidthForGrid= Math.DivRem(gridInstance.Size.Width ,100,out nWidthForGrid);
			
			foreach(GridColumnDescriptor newColWidth in gridInstance.TableDescriptor.Columns)
			{
				sColumnName = newColWidth.GetName();
				if(RelativeWidths.Contains(sColumnName))
				{
					nRelWidthForCol = (int)RelativeWidths[sColumnName];
					nWidthForCol = nWidthForGrid * nRelWidthForCol;
					newColWidth.Width = nWidthForCol;
				}

				//colWidthHash.Add(oldColWidth.HeaderText,oldColWidth.Width); 
			}
		}
		public static void InitializeGridColumnsWidth(GridGroupingControl gridInstance,int minWidth)
		{
			int nCols = gridInstance.TableDescriptor.Columns.Count + 1;
			int nRelWidthForCol;
			int nWidthForGrid;
			int nWidthForCol;
			nRelWidthForCol = Math.DivRem(100,nCols,out nRelWidthForCol);
			nWidthForGrid= Math.DivRem(gridInstance.Size.Width,100,out nWidthForGrid);
			nWidthForCol = nWidthForGrid * nRelWidthForCol;
			foreach(GridColumnDescriptor newColWidth in gridInstance.TableDescriptor.Columns)
			{
				newColWidth.Width = nWidthForCol * minWidth;
			}
		}
		#endregion


		/// <summary>
		///  Method for getting the records which are ReadOnly
		/// </summary>
		/// <param name="gridGroupControl"></param>
		/// <param name="gridTableDescriptor"></param>
	
		public static void ApplyViewStyleOnGrid(GridGroupingControl gridGroupControl,GridTableDescriptor gridTableDescriptor)
		{
			_gridGroupingControl=gridGroupControl;
			_gridTableDescriptor=gridTableDescriptor;
			gridGroupControl.QueryCellStyleInfo+=new GridTableCellStyleInfoEventHandler(gridGroupControl_QueryCellStyleInfo);
		}
        /// <summary>
        /// Handles the QueryCellStyleInfo event of the gridGroupControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableCellStyleInfoEventArgs"/> instance containing the event data.</param>
		private static void gridGroupControl_QueryCellStyleInfo(object sender, GridTableCellStyleInfoEventArgs e)
		{
			if(e.TableCellIdentity.RowIndex!=-1) 
			{
				if(e.TableCellIdentity.DisplayElement is GridRecordRow)
				{
					BaseObject obj  = e.TableCellIdentity.DisplayElement.ParentRecord.GetData() as BaseObject; 
					if(obj!=null)
					{
						if(obj.ReadOnlyAccess==true)
						{
							if(e.TableCellIdentity.ColIndex > 0)
							{
								if(e.Style.CellType != "FilterBarCell" && e.TableCellIdentity.TableCellType != GridTableCellType.AddNewRecordFieldCell &&  e.TableCellIdentity.TableCellType != GridTableCellType.ColumnHeaderCell && e.TableCellIdentity.TableCellType != GridTableCellType.GroupCaptionCell)
								{
									e.Style.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.PatternStyle.None, System.Drawing.SystemColors.WindowText, System.Drawing.Color.LightGray);
									e.Style.ReadOnly=true;
								}
							}
						}
						
					}
				}
			}
		}

        /// <summary>
        /// Handles the TableControlCellClick event of the gridInstance control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellClickEventArgs"/> instance containing the event data.</param>
		private static void gridInstance_TableControlCellClick(object sender, GridTableControlCellClickEventArgs e)
		{
			GridTableCellStyleInfo styleInfo = e.TableControl.Model[e.Inner.RowIndex,e.Inner.ColIndex];
			
			if(styleInfo.CellValueType == typeof(LRRFormula))
			{
				e.Inner.Cancel=true;
			}
            //else if(styleInfo.TableCellIdentity.Column != null)
            //{
            //    if((styleInfo.TableCellIdentity.Column.HeaderText == LRRHeaderText.GetHeaderString("SID_INTERFACEUOM")) 
            //        || (styleInfo.TableCellIdentity.Column.HeaderText == LRRHeaderText.GetHeaderString("SID_HISTUOM")))  
            //    {
            //        //e.Inner.Cancel = true;
            //    }
            //}
		}
	}
}
