using System;
using System.Drawing;
using System.Diagnostics;
using System.Reflection;
using System.Collections;
using Symcon.Utils.MessageLogger;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Tools;
using System.Windows.Forms;  
using Symcon.LightRidge.BusinessObjects;
using Symcon.Globals.Enums;     
using Symcon.BusinessObjects;
using Symcom.Evaluator;
using Symcon.Scripting;
using Symcon.Utils.MessageDispatcher;

namespace Symcon.LightRidge.PEAdvisor.GridUtilities
{
	/// <summary>
	/// This static method is used for checking duplicates in database.
	/// </summary>
	public class Utilities
	{ 
		/// <summary>
		/// 
		/// </summary>
		/// <param name="targetObject"></param>custom class object
		/// <param name="propName"></param>property name for which duplicate data needs to be checked
		/// <param name="parentKey"></param>parent key field.
		/// <returns></returns> true if duplicate will found else return false 
		/// 
		public static bool CheckDuplicate(object targetObject,string propName,string parentKey)
		{
			CustomCollection cc = targetObject as CustomCollection;
			CustomCollection  gridModified = cc.Clone() as CustomCollection;
			object obj = Activator.CreateInstance(cc.ItemType);
			string duplicateCheckFor= null;
			string parentfield = null;
			string pKfield = null;
			//gives all the properties of the bse class stored in Propertyinfo[]
			PropertyInfo[] props = cc.ItemType.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);      
			// gets the fields according to the arduments passing by the User.
			for(int i=0;i<props.Length;i++)
			{
				PropertyInfo prop = props[i];
				SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
					prop.GetCustomAttributes(typeof(SymDataFieldAttribute),
					false);
				if(field.Length > 0)
				{
					SymDataFieldAttribute sDFA = field[0];
					if(sDFA.PropName == propName)
					{
						duplicateCheckFor = sDFA.PropName;
					}
					else
					{
						if(sDFA.PropName == parentKey)
						{
							parentfield = sDFA.PropName;
						}
					}
				}
				// Getting a primery key fields..
				SymKeyFieldAttribute[] PKfield = (SymKeyFieldAttribute[])
					prop.GetCustomAttributes(typeof(SymKeyFieldAttribute),false);
				if(PKfield.Length > 0)
				{
					
					pKfield = prop.Name;
		 
				}

			}
			CustomCollection changes = gridModified.GetChanges();
			int change = changes.Count;  
			if(changes.Count != 0)
			{
				int nLRRDuplicate = 0;
				string[] chkDuplicate = new string[change];
				string[] PKId = new string[change];
				string[] parentKId = new string[change];
				string sFilter;
				foreach(BaseObject duplicate in changes)
				{
					if(duplicate.ObjectState != DBOperation.DELETE)
					{
						chkDuplicate[nLRRDuplicate] = duplicate.GetType().GetProperty(duplicateCheckFor).GetValue(duplicate,null)==null?"" :duplicate.GetType().GetProperty(duplicateCheckFor).GetValue(duplicate,null).ToString().Trim();
						PKId[nLRRDuplicate] = duplicate.GetType().GetProperty(pKfield).GetValue(duplicate,null)==null? "" :duplicate.GetType().GetProperty(pKfield).GetValue(duplicate,null).ToString();
						parentKId[nLRRDuplicate] = duplicate.GetType().GetProperty(parentfield).GetValue(duplicate,null)==null? "" :duplicate.GetType().GetProperty(parentfield).GetValue(duplicate,null).ToString();       
						nLRRDuplicate++;
						for(int j=0;j<nLRRDuplicate;j++)
						{
							//filter string which will check for the duplicate base on primerykey field for given database.
							sFilter = duplicateCheckFor +" = '"+ chkDuplicate[j] +"' and "+parentfield+" =  '"+parentKId[j]+"' " + "and " + pKfield + " != '"+PKId[j]+"' "; 
							if(chkDuplicate[j] == "")
							{
								LRRMessagBox.ShowMessage("SID_BLANKROWS","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Information);    
								return true;
							}
							gridModified.ItemFilter = sFilter;
							if(gridModified.Count > 0)
							{
								MessageBox.Show("Duplicate attribute name - '"+chkDuplicate[j]+"' exists. Can not continue with save operation!");
								gridModified = null;
								return true;
							}
						}

					}
				}
			}
			gridModified = null;
			return false;
		}


        /// <summary>
        /// Checks the duplicate.
        /// </summary>
        /// <param name="targetObject">The target object.</param>
        /// <param name="propName">Name of the prop.</param>
        /// <returns></returns>
		public static bool CheckDuplicate(object targetObject,string propName)
		{
			CustomCollection cc = targetObject as CustomCollection;
			CustomCollection  gridModified = cc.Clone() as CustomCollection;
			object obj = Activator.CreateInstance(cc.ItemType);
			string duplicateCheckFor= null;
			string pKfield = null;
			//gives all the properties of the bse class stored in Propertyinfo[]
			PropertyInfo[] props = cc.ItemType.GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);      
			// gets the fields according to the arduments passing by the User.
			for(int i=0;i<props.Length;i++)
			{
				PropertyInfo prop = props[i];
				SymDataFieldAttribute[] field = (SymDataFieldAttribute[])
					prop.GetCustomAttributes(typeof(SymDataFieldAttribute),
					false);
				if(field.Length > 0)
				{
					SymDataFieldAttribute sDFA = field[0];
					if(sDFA.PropName == propName)
					{
						duplicateCheckFor = sDFA.PropName;
					}
				}
				// Getting a primery key fields..
				SymKeyFieldAttribute[] PKfield = (SymKeyFieldAttribute[])
					prop.GetCustomAttributes(typeof(SymKeyFieldAttribute),false);
				if(PKfield.Length > 0)
				{
					
					pKfield = prop.Name;
		 
				}

			}
			CustomCollection changes = gridModified.GetChanges();
			int change = changes.Count;  
			if(changes.Count != 0)
			{
				int nLRRDuplicate = 0;
				string[] chkDuplicate = new string[change];
				string[] PKId = new string[change];
				string sFilter;
				foreach(BaseObject duplicate in changes)
				{
					if(duplicate.ObjectState != DBOperation.DELETE)
					{
						chkDuplicate[nLRRDuplicate] = duplicate.GetType().GetProperty(duplicateCheckFor).GetValue(duplicate,null) == null ? "" : duplicate.GetType().GetProperty(duplicateCheckFor).GetValue(duplicate,null).ToString().Trim();
						PKId[nLRRDuplicate] = duplicate.GetType().GetProperty(pKfield).GetValue(duplicate,null).ToString();
						nLRRDuplicate++;
						for(int j=0;j<nLRRDuplicate;j++)
						{
							//filter string which will check for the duplicate base on primerykey field for given database.
							sFilter = duplicateCheckFor +" = '"+ chkDuplicate[j] +"' and " + pKfield + " != '"+PKId[j]+"' ";
							if(chkDuplicate[j] == "")
							{
								LRRMessagBox.ShowMessage("SID_BLANKROWS","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Information);    
								return true;
							}
							gridModified.ItemFilter = sFilter;
							if(gridModified.Count > 0)
							{
								MessageBox.Show("Duplicate attribute name - '"+chkDuplicate[j]+"' exists. Can not continue with save operation!");
								gridModified = null;
								return true;
							}
						}
					}
				}
			}
			gridModified = null;
			return false;
		}

        /// <summary>
        /// Formats the number.
        /// </summary>
        /// <param name="stringToConvert">The string to convert.</param>
        /// <param name="_app">The _app.</param>
        /// <returns></returns>
        public static string FormatNumber(string stringToConvert,LRRApp _app)
        {
            string returnString = stringToConvert;
            System.Globalization.CultureInfo ci = new System.Globalization.CultureInfo(_app.CurrentNumericFormat);
            double result;
            bool isNumeric = double.TryParse(stringToConvert, System.Globalization.NumberStyles.Number, ci,out result);
            if (isNumeric)
            {
                if (_app.CurrentNumericFormat != null && _app.CurrentNumericFormat.Trim() != "")
                {

                    //int x = Int32.Parse(stringToConvert);
                    returnString = result.ToString("N", ci.NumberFormat);
                }
            }
            return returnString;
        }

        /// <summary>
        /// Converts the culture specfic to plain string.
        /// </summary>
        /// <param name="stringToConvert">The string to convert.</param>
        /// <param name="_app">The _app.</param>
        /// <returns></returns>
        public static string ConvertCultureSpecficToPlainString(string stringToConvert, LRRApp _app)
        {
            string returnString = stringToConvert;
            System.Globalization.CultureInfo ci = new System.Globalization.CultureInfo(_app.CurrentNumericFormat);
            double result;
            
            bool isNumeric = double.TryParse(stringToConvert, System.Globalization.NumberStyles.Number, ci, out result);
            if(isNumeric)
                returnString = result.ToString();
            return returnString;
 
        }

//		public static string Calculate(LRRFormulaParams lrrFormulaParams,string formula)
//		{
//			LRRApp _app;
//			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
//			Hashtable htFormulaParam = new Hashtable();
//			foreach(LRRFormulaParam lrrFormulaParam in lrrFormulaParams)
//			{
//				htFormulaParam.Add(lrrFormulaParam.ParamName,lrrFormulaParam);
//			}
//
//			ExecutionQueue eq = Evaluator.ParseIt(formula);
//			ExecutionQueue newEq = new ExecutionQueue(); 
//			ExecutionItem ei;
//			while(eq.Count > 0)
//			{
//				ei = eq.Dequeue();
//				switch(ei.itemType)
//				{
//					case ItemType.itString:
//						if(htFormulaParam.Contains(ei.itemString))
//						{
//							LRRFormulaParam lrrFormula = htFormulaParam[ei.itemString] as LRRFormulaParam;
//							string curVal = "";
//							if(lrrFormula.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
//							{
//								if(LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID)!= null)
//									curVal = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID).GetValueForPropertyMode(lrrFormula.ModeID);
//							}
//							if(lrrFormula.SourceCollectioName == typeof(LRRHistData).ToString())
//							{
//								//LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
//								//Populate from Model
//								LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
//								LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceOBJID);
//								curVal = lrrhistData.SpotValue;
//							}
//
//							curVal = curVal == "" ? "0" : curVal;
//							newEq.Enqueue(new ExecutionItem(ItemType.itDigit,curVal));
//						}
//						break;
//					default:
//						newEq.Enqueue(ei);
//						break;
//				}
//			}
//
//			Variant var  =  Evaluator.CalcIt(newEq);
//			string val =  var.oValue.ToString();
//			return val;
//		}

        /// <summary>
        /// Calculates the formula marked for pre calc
        /// </summary>
		public void InitiatePreCalc()
		{
			try
			{
				LRRFormulas lrrFormulas = LRRModel.FormulasObj.FilterFormulasByPreCalc();
				foreach(LRRFormula lrrFormula in lrrFormulas)
				{
					LRRFormulaParams lrrFormulaParams = lrrFormula.Params;
					string formula = lrrFormula.Expression;
					if(lrrFormula.IsFormula)
					{
						string val = Calculate(lrrFormulaParams,formula);

						if(lrrFormula.SourceCollectionName == typeof(LRRSiteComponentPropertyValue).ToString())
						{
							LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID).PropertyValue = val;
						}
						if(lrrFormula.SourceCollectionName == typeof(LRRHistData).ToString())
						{
							//LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
							//Populate from Model
							LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
							LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
							lrrhistData.InterfaceValue = val;
						}
						//Aggregates
						if(lrrFormula.SourceCollectionName == typeof(LRRAggregatePropertyValue).ToString())
						{
							//Populate from Model
							if(LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID) != null)
								LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID).PropertyValue = val;
						}
					}
					else
					{
						ExceuteScript(lrrFormulaParams,formula,lrrFormula);
					}
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(Utilities),"InitiatePreCalc :"+ex.Message  ,Logger.Level.Debug);
			}

		}

		
		/// <summary>
		/// Calculates the formula marked for post calc
		/// </summary>
		public void InitiatePostCalc()
		{
			try
			{
				LRRFormulas lrrFormulas = LRRModel.FormulasObj.FilterFormulasByPostCalc();
				foreach(LRRFormula lrrFormula in lrrFormulas)
				{

					try
					{
						LRRFormulaParams lrrFormulaParams = lrrFormula.Params;
						string formula = lrrFormula.Expression;
						if(lrrFormula.IsFormula)
						{
							string val = "";
							if(formula != null && formula != "" &&	lrrFormulaParams != null)
							{
								val = Calculate(lrrFormulaParams,formula);
							}

							if(lrrFormula.SourceCollectionName == typeof(LRRSiteComponentPropertyValue).ToString())
							{
								LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID).PropertyValue = val;
							}
							if(lrrFormula.SourceCollectionName == typeof(LRRHistData).ToString())
							{
								//LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
								//Populate from Model
								LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
								LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
								lrrhistData.InterfaceValue= val;

							}
							//Aggregates
							if(lrrFormula.SourceCollectionName == typeof(LRRAggregatePropertyValue).ToString())
							{
								//Populate from Model
								if(LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID) != null)
									LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID).PropertyValue = val;
							}

						}
						else
						{
							ExceuteScript(lrrFormulaParams,formula,lrrFormula);
						}
					}
					catch
					{
						//Symcon.Utils.MessageLogger.Logger.LogMessage("Error executing function",Symcon.Utils.MessageLogger.Level.Debug);
						Symcon.Utils.MessageLogger.Logger.LogMessage("Error Executing Function for " + lrrFormula.FormulaName  ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
					}
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(Utilities),"InitiatePostCalc :"+ex.Message  ,Logger.Level.Debug);
			}
		}


		/// <summary>
		/// Executes the script & updates the result
		/// </summary>
		public void ExceuteScript(LRRFormulaParams lrrFormulaParams,string script,LRRFormula lrrFormula)
		{
			try
			{
				string rootHostName = "MyScriptingHost";
				string rootMoniker = string.Format("{0}://{1}", rootHostName, Guid.NewGuid().ToString());
				VsaScriptingHost host = new VsaScriptingHost(VsaScriptingLanguages.JScript,@"MyScriptingHost",rootMoniker,@"Scripting",true);

				bool flag = true;

				foreach(LRRFormulaParam param in lrrFormulaParams)
				{
					if(param.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
					{
						if(LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(param.SourceOBJID)!= null)
						{
							LRRSiteComponentPropertyValue lrrSiteCompPropVal = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(param.SourceOBJID).GetValueObjectForPropertyMode(param.ModeID);
							if(lrrSiteCompPropVal != null)
							{
								host.CreateGlobalItem(param.ParamName,lrrSiteCompPropVal,false);
							}
							LRRSiteComponentPropertyValue siteCompPropVal = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);
							if(siteCompPropVal != null && flag)
							{
								host.CreateGlobalItem("RESULT",siteCompPropVal,false);
								flag = false;
							}
						}
					}
					if(param.SourceCollectioName == typeof(LRRHistData).ToString())
					{
						LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
						LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(param.SourceOBJID);
						if(lrrhistData != null)
						{
							host.CreateGlobalItem(param.ParamName,lrrhistData,false);
						}
						LRRHistData histData = LRRModel.HistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
						if(histData != null && flag)
						{
							host.CreateGlobalItem("RESULT",histData,false);
							flag = false;
						}
					}

					//Constants
					if(param.SourceCollectioName == typeof(LRRConstant).ToString())
					{
						LRRConstant lrrConstant = new LRRConstant();
						lrrConstant.Value = param.CurrentValue;
						host.CreateGlobalItem(param.ParamName,lrrConstant,false);
					}
				}
				host.Execute(@script);
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(Utilities),"ExceuteScript :"+ex.Message  ,Logger.Level.Debug);
			}
		}


		/// <summary>
		/// Executes the formula & updates the result
		/// </summary>
		public string Calculate(LRRFormulaParams lrrFormulaParams,string formula)
		{
			Hashtable htFormulaParam = new Hashtable();
			foreach(LRRFormulaParam lrrFormulaParam in lrrFormulaParams)
			{
				htFormulaParam.Add(lrrFormulaParam.ParamName,lrrFormulaParam);
			}

			//--Inline Operator--
			string[] inlineOperator = formula.Split(new char[]{'?'});
			if(inlineOperator.Length > 1)
			{
				string exp = inlineOperator[0].ToString(Symcon.BusinessObjects.SymConverter.Culture);
				string[] param = exp.Split(new char[]{'=','>','<'});
				int greaterThanIndex = exp.IndexOf('>');
				int lesserThanIndex  = exp.IndexOf('<');
				int equalToIndex	 = exp.IndexOf('=');
				string oper	= null;
				if(greaterThanIndex != -1)
				{
					oper = ">";
				}
				else if(lesserThanIndex != -1)
				{
					oper = "<";
				}
				else if(equalToIndex != -1)
				{
					oper = "=";
				}

				if(param.Length > 1)
				{
					string param1 = param[0].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim();
					string param2 = param[1].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim();
					string finalValue = null;
					if(htFormulaParam.Contains(param1))
					{
						string[] result = inlineOperator[1].Split(new char[]{':'});

						try
						{
							//Get the current Value
							LRRFormulaParam lrrFormulaParam1 = htFormulaParam[param1] as LRRFormulaParam;
							string curVal = "";
							if(lrrFormulaParam1.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
							{
								if(lrrFormulaParam1.SourceOBJID == null || lrrFormulaParam1.SourceOBJID.Trim() == "")
								{
									curVal = lrrFormulaParam1.CurrentValue;
								}
								else
								{
									if(LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormulaParam1.SourceOBJID)!= null)
										curVal = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormulaParam1.SourceOBJID).GetValueForPropertyMode(lrrFormulaParam1.ModeID);
								}
							}
							else if(lrrFormulaParam1.SourceCollectioName == typeof(LRRHistData).ToString())
							{
								if(lrrFormulaParam1.SourceOBJID == null || lrrFormulaParam1.SourceOBJID.Trim() == "")
								{
									curVal = lrrFormulaParam1.CurrentValue;
								}
								else
								{
									//LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
									//Populate from Model
									LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
									LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormulaParam1.SourceOBJID);
									curVal = lrrhistData.InterfaceValue;
								}
							}
								//If constant
							else
							{
								curVal = lrrFormulaParam1.CurrentValue;
							}
							
							if(curVal == null || curVal.Trim() == "")
								curVal = "0";

							//check for operator
							switch(oper)
							{
								case ">" :
									double cmp1= Convert.ToDouble(curVal,Symcon.BusinessObjects.SymConverter.Culture);
									double cmp2= Convert.ToDouble(param2,Symcon.BusinessObjects.SymConverter.Culture);
									if(cmp1 > cmp2)
									{
										finalValue = result[0].ToString(Symcon.BusinessObjects.SymConverter.Culture);
									}
									else
									{
										finalValue = result[1].ToString(Symcon.BusinessObjects.SymConverter.Culture);
									}
									break;
								case "<" :
									double cmp3= Convert.ToDouble(curVal,Symcon.BusinessObjects.SymConverter.Culture);
									double cmp4= Convert.ToDouble(param2,Symcon.BusinessObjects.SymConverter.Culture);
									if(cmp3 < cmp4)
									{
										finalValue = result[0].ToString(Symcon.BusinessObjects.SymConverter.Culture);
									}
									else
									{
										finalValue = result[1].ToString(Symcon.BusinessObjects.SymConverter.Culture);
									}
									break;
								case "=" :
									if(curVal == param2)
									{
										finalValue = result[0].ToString(Symcon.BusinessObjects.SymConverter.Culture);
									}
									else
									{
										finalValue = result[1].ToString(Symcon.BusinessObjects.SymConverter.Culture);
									}
									break;
								default :
									break;

							}
						}
						catch(Exception ex)
						{
							
						Logger.LogMessage(typeof(Utilities),"Calculate :"+ex.Message  ,Logger.Level.Debug);
						}
						return finalValue;
					}
				}
			}
			//--End--

            return CalculateFunctionsExpression(formula, htFormulaParam);

            //ExecutionQueue eq = Evaluator.ParseIt(formula);
            //ExecutionQueue newEq = new ExecutionQueue(); 
            //ExecutionItem ei;
            //while(eq.Count > 0)
            //{
            //    ei = eq.Dequeue();
            //    switch(ei.ItemType)
            //    {
            //        case ItemType.itString:
            //            if(htFormulaParam.Contains(ei.ItemString))
            //            {
            //                LRRFormulaParam lrrFormula = htFormulaParam[ei.ItemString] as LRRFormulaParam;
            //                string curVal = "";
            //                if(lrrFormula.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
            //                {
            //                    if(lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
            //                    {
            //                        curVal = lrrFormula.CurrentValue;
            //                    }
            //                    else
            //                    {
            //                        if(LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID)!= null)
            //                            curVal = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID).GetValueForPropertyMode(lrrFormula.ModeID);
            //                    }
            //                }
            //                else if(lrrFormula.SourceCollectioName == typeof(LRRHistData).ToString())
            //                {
            //                    if(lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
            //                    {
            //                        curVal = lrrFormula.CurrentValue;
            //                    }
            //                    else
            //                    {
            //                        //LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
            //                        //Populate from Model
            //                        LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
            //                        LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceOBJID);
            //                        curVal = lrrhistData.InterfaceValue;
            //                    }
            //                }
            //                    //If constant
            //                else
            //                {
            //                    curVal = lrrFormula.CurrentValue;
            //                }
							
            //                if(curVal == null || curVal.Trim() == "")
            //                    curVal = "0";

            //                newEq.Enqueue(new ExecutionItem(ItemType.itDigit,curVal));
            //            }
            //            break;
            //        default:
            //            newEq.Enqueue(ei);
            //            break;
            //    }
            //}

            //Variant var  =  Evaluator.CalcIt(newEq);
            //string val =  var.oValue.ToString();
            //return val;
		}

        private string CalculateFunctionsExpression(string expression, Hashtable htFormulaParam)
        {
            ExecutionQueue eq = Evaluator.ParseIt(expression);
            ExecutionQueue newEq = new ExecutionQueue();
            ExecutionItem ei;

            while (eq.Count > 0)
            {
                ei = eq.Dequeue();
                switch (ei.ItemType)
                {
                    case ItemType.itString:
                        if (htFormulaParam.Contains(ei.ItemString))
                        {
                            LRRFormulaParam lrrFormula = htFormulaParam[ei.ItemString] as LRRFormulaParam;
                            string curVal = "";
                            if (lrrFormula.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
                            {
                                if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                {
                                    curVal = lrrFormula.CurrentValue;
                                }
                                else
                                {
                                    if (LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID) != null)
                                        curVal = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID).GetValueForPropertyMode(lrrFormula.ModeID);
                                }
                            }
                            else if (lrrFormula.SourceCollectioName == typeof(LRRHistData).ToString())
                            {
                                if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                {
                                    curVal = lrrFormula.CurrentValue;
                                }
                                else
                                {
                                    //LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
                                    //Populate from Model
                                    LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
                                    LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceOBJID);
                                    curVal = lrrhistData.InterfaceValue;
                                }
                            }
                            //If constant
                            else
                            {
                                curVal = lrrFormula.CurrentValue;
                            }

                            if (curVal == null || curVal.Trim() == "")
                                curVal = "0";

                            double res;
                            if (double.TryParse(curVal, out res))
                                newEq.Enqueue(new ExecutionItem(ItemType.itDigit, curVal));
                            else
                                newEq.Enqueue(new ExecutionItem(ItemType.itString, curVal));
                        }
                        else
                        {
                            newEq.Enqueue(ei);
                        }
                        //else
                        //{ 
                        //    int i=5;i
                        //}
                        break;
                    case ItemType.itFunction:
                        System.Collections.Specialized.StringCollection itParamsNew = new System.Collections.Specialized.StringCollection();
                        foreach (string itParam in ei.ItemParams)
                        {
                            itParamsNew.Add(CalculateFunctionsExpression(itParam, htFormulaParam));
                        }
                        ei.ItemParams = itParamsNew;
                        newEq.Enqueue(ei);
                        break;
                    default:
                        newEq.Enqueue(ei);
                        break;
                }
            }

            Variant var = Evaluator.CalcIt(newEq);
            return var.oValue.ToString();
        }

		/// <summary>
		/// Calculate the formula values.
		/// </summary>
		/// <param name="htAggregateFormulas"></param>
		public void InitiateAggregateFormulaCalculation(ArrayList arAggregateFormulas,Symcon.LightRidge.PEAdvisor.LRRApp _app)
		{
			for(int itemCtr=0; itemCtr < arAggregateFormulas.Count;itemCtr++)
			{
				string sFormulaID = arAggregateFormulas[itemCtr].ToString();
				LRRFormula lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(sFormulaID);
				LRRFormulaParams lrrFormulaParams = lrrFormula.Params;
				try
				{
					if(lrrFormula.SourceCollectionName.Trim() != typeof(LRRAggregatePropertyValue).ToString())
						continue;
					
					string formula = lrrFormula.Expression;
					
					if(lrrFormula.IsFormula)
					{
						string val = "";
						
						if(formula != null && formula != "" &&	lrrFormulaParams != null)
						{
							val =_app.RunControllerObj.Calculate(lrrFormulaParams,formula);
							
							//Aggregates
							if(lrrFormula.SourceCollectionName.Trim() == typeof(LRRAggregatePropertyValue).ToString())
							{
								//Populate from Model
								if(LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID) != null)
									LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID).PropertyValue = val;
							}
						
		
						}
					}
					else
					{
						if(!lrrFormula.IsCustomFormula)
						{
							string val = "";

						
							_app.RunControllerObj.ExceuteScript(lrrFormulaParams,formula,lrrFormula);
							
							//Aggregates
							if(lrrFormula.SourceCollectionName.Trim() == typeof(LRRAggregatePropertyValue).ToString())
							{
								//Populate from Model
								if(LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID) != null)
									LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID).PropertyValue = val;
							}
							
						}
					}
				}
				catch
				{
					Symcon.Utils.MessageLogger.Logger.LogMessage("Aggregate Formula: Error Executing Formula for " + lrrFormula.FormulaName  ,Symcon.Utils.MessageLogger.Logger.Level.Debug);
				}
			}
		}	
	}
	public delegate void ModesChangedEventhandler(LRRModes modes);
	
}
