using System;
using System.Drawing;

using Symcon.LightRidge.PEAdvisor;
using Symcon.ColorSchemes;

namespace Symcon.LightRidge.PEAdvisor.GroupProperties
{
	/// <summary>
	/// Summary description for GroupPropertiesSchemes.
	/// </summary>
	public class GroupPropertiesSchemes
	{
		static GroupPropertiesSchemes()
		{
			//Get the colors from XML file
			InitializeScheme();
		}

		#region Constants
		public static Color HEADER_BACK_COLOR;// = Color.RoyalBlue;
		public static Color COLUMN_HEADER_BACK_COLOR;// = Color.RoyalBlue;
		public static Color COLUMN_HEADER_TEXT_COLOR;//= SystemColors.ActiveCaptionText;
		public static Color ROW_HEADER_BACK_COLOR;// = Color.SkyBlue;
		public static Color ATTRIBUTES_BACK_COLOR;// = Color.SkyBlue;
		public static Color PROPERTIES_BACK_COLOR;// = Color.White;
		public static Color GRID_LINE_COLOR;// = SystemColors.ActiveCaption;
		public static Color GRID_ROW_COLOR;// = Color.SkyBlue;
		public static Color DEFAULT_MODE_COLOR;// = Color.LightGray;
		public static Color MEAS_MODE_COLOR;// = Color.SkyBlue;
		public static Color SS_MODE_COLOR;// = Color.Yellow;
		public static Color SCENARIO_MODE_COLOR;// = Color.LightCyan;
		public static Color MODE_TEXT_COLOR;//= Color.Black;
		public static Color COMPONENT_BACK_COLOR;// = Color.SkyBlue;
		public static Color	UOM_BACK_COLOR;// = Color.LightCyan;
		public static Color	RECORDED_VALUE_COLOR;// = Color.LightYellow;
		public static Color	NORMAL_BACK_COLOR ;//= Color.White;
		public static Color GRP_PROPERTY_BACK_COLOR ;//= Color.AliceBlue;

		public static string ATTRIBUTE_HEADER_NAME = "Attributes [Grouped]";
		public static string COMPONENT_HEADER_NAME = "Components";
		public static string GRP_PROPERTY_HEADER_NAME = "Type ";
		public static string UOM_HEADER_NAME = "UOM";
		public static int MODE_COL_SIZE = 150;
		public static int HEADER_COL_SIZE = 200;
		public static int ROW_HEADER_COL_SIZE = 10;
		#endregion

		#region Methods

		/// <summary>
		/// //Get the colors from XML file
		/// </summary>
		public static void InitializeScheme()
		{
			LRRApp _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			ColorScheme colorScheme = _app.LRRUIManagerObj.ColorSchemeObj;

			//Header Colors
			HEADER_BACK_COLOR = colorScheme.GroupPropertiesColors.HeaderBackColor;
			COLUMN_HEADER_BACK_COLOR = colorScheme.GroupPropertiesColors.ColumnHeaderBackColor;
			COLUMN_HEADER_TEXT_COLOR = colorScheme.GroupPropertiesColors.ColumnHeaderTextColor;
			ROW_HEADER_BACK_COLOR = colorScheme.GroupPropertiesColors.RowHeaderBackColor;
			ATTRIBUTES_BACK_COLOR = colorScheme.GroupPropertiesColors.AttributesBackColor;
			PROPERTIES_BACK_COLOR = colorScheme.GroupPropertiesColors.PropertiesBackColor;
			GRID_LINE_COLOR = colorScheme.GroupPropertiesColors.GridLineColor;
			GRID_ROW_COLOR = colorScheme.GroupPropertiesColors.GridRowColor;
			COMPONENT_BACK_COLOR = colorScheme.GroupPropertiesColors.ComponentBackColor;
			UOM_BACK_COLOR = colorScheme.GroupPropertiesColors.UomBackColor;
			
			GRP_PROPERTY_BACK_COLOR  = colorScheme.GroupPropertiesColors.GrpPropertyBackColor;
			NORMAL_BACK_COLOR = colorScheme.GroupPropertiesColors.NormalBackColor;
//			GRID_BACK_COLOR = colorScheme.GroupPropertiesColors.GridRowColor;

			//Mode Colors
			MEAS_MODE_COLOR = colorScheme.GroupPropertiesColors.MeasModeColor;
			SS_MODE_COLOR  = colorScheme.GroupPropertiesColors.SsModeColor;
			SCENARIO_MODE_COLOR = colorScheme.GroupPropertiesColors.ScenarioModeColor;
			DEFAULT_MODE_COLOR = colorScheme.GroupPropertiesColors.DefaultModeColor;
			MODE_TEXT_COLOR = colorScheme.GroupPropertiesColors.ModeTextColor;
		}

		/// <summary>
		/// Gets the Mode Back color for the given Mode
		/// </summary>
		/// <param name="modeID"></param>
		/// <returns></returns>
		public static Color GetModeBackColor(string modeID)
		{
			Color color = GroupPropertiesSchemes.DEFAULT_MODE_COLOR;
			if(modeID == "1")
				color = GroupPropertiesSchemes.MEAS_MODE_COLOR;
			if(modeID == "2")
				color = GroupPropertiesSchemes.SS_MODE_COLOR;
			if(modeID == "3")
				color = GroupPropertiesSchemes.SCENARIO_MODE_COLOR;

			return color;
		}
		
		/// <summary>
		/// Gets the Mode Text Color for the given Mode
		/// </summary>
		/// <param name="modeID"></param>
		/// <returns></returns>
		public static Color GetModeTextColor(string modeID)
		{
			return GroupPropertiesSchemes.MODE_TEXT_COLOR;
		}
		#endregion
	}
}
