using System;
using System.Windows.Forms;
using System.Collections;
using System.Drawing;
using Syncfusion.Windows.Forms.Grid;

using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.Utils.Validator;
using Symcon.LightRidge.BusinessObjects.Utilities;
using Symcon.Utils.HeaderDispatcher; 
using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.LightRidge.PEAdvisor.CommonViews;

namespace Symcon.LightRidge.PEAdvisor.IOScreen
{
	/// <summary>
	/// Summary description for GridSimpleViewController.
	/// </summary>
	public class GridSimpleViewController : IGridFactory		
	{
		public GridSimpleViewController()
		{
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
             _gutil = new IOScreen.GridPersistanceUtil();
		}

		#region IGridFactory Members
		
		/// <summary>
		/// This method loads the panel for the simpleview.Responsible for creating the grid
		/// </summary>
		/// <param name="custCollection"></param>
		/// <param name="srchCriteriaItemProperties"></param>
        public void LoadPanel(ArrayList custCollection, LRRSearchCriteriaItemProperties srchCriteriaItemProperties)
        {
            int rowCount = 0;
            int colCount = 0;

            //Split the collection according to master Component type
            object[,] arrSplitComponents = IOScreenUtilities.SplitComponents(custCollection, out rowCount, out colCount);

            //Populate User Specifc UIUOMs
            this._lrrApp.DBManagerObj.PopulateCollection(_searchCriteriaUserSpecficUIUOMs);

            for (int iRow = 0; iRow < rowCount; iRow++)
            {
                //Get the Col Count
                int iGridColCount = GetGridColCount(arrSplitComponents, iRow, colCount);

                //Get the Row Count
                int iGridRowCount = GetGridRowCount(arrSplitComponents, iRow, colCount);

                //Create Grid Instance,Panel,Register Events 
                GridControl gridControlInstance = new GridControl();
                gridControlInstance.Dock = DockStyle.Fill;
                Panel pnlGrid = new Panel();
                pnlGrid.Dock = DockStyle.Top;
                pnlGrid.Controls.Add(gridControlInstance);

                //Assign RowCount and ColCount
                gridControlInstance.RowCount = iGridRowCount;
                gridControlInstance.ColCount = iGridColCount;

                //Add the Panel to panel Collection
                _pnlCollection.Add(pnlGrid);

                //register grid events
                registerGridEvents(gridControlInstance);

                //Disable the Left/Right Col Buttons
                _lrrApp.MainFrame.BarItemLeftColMovement.Enabled = false;
                _lrrApp.MainFrame.BarItemRightColMovement.Enabled = false;

                //Row/Col Freeze
                gridControlInstance.Rows.FreezeRange(FREEZE_ROW_START, FREEZE_ROW_END);
                gridControlInstance.Cols.FreezeRange(FREEZE_COL_START, FREEZE_COL_END);

                //Create a GridControl manager and attach to grid
                GridControlManager gridManager = new GridControlManager();
                gridManager.GridData = arrSplitComponents;

                gridManager.ForeColorChanged += new Symcon.LightRidge.PEAdvisor.IOScreen.GridControlManager.ForeColorChangesDel(gridManager_ForeColorChanged);
                gridManager.BackColorChanged += new Symcon.LightRidge.PEAdvisor.IOScreen.GridControlManager.BackColorChangesDel(gridManager_BackColorChanged);
                gridManager.IsTrackChanged += new Symcon.LightRidge.PEAdvisor.IOScreen.GridControlManager.IsTrackChangesDel(gridManager_IsTrackChanged);
                gridManager.IsTrendedChanged += new GridControlManager.IsTrendedChangesDel(gridManager_IsTrendedChanged);

                //Add to the Hash
                GridManagerHash.Add(gridControlInstance, gridManager);

                //Register the Binding Called Event
                gridManager.BindingManager.BindingCalled += new Symcon.LightRidge.PEAdvisor.DataStore.LRRBindingManager.BindingCallDel(bindingManager_BindingCalled);

                //Render the grid
                createGrid(gridControlInstance, arrSplitComponents, iRow, gridManager, srchCriteriaItemProperties);

                //Recorded Notification 
                _commentMouseControler = new CommentMouseController(gridControlInstance);
                //_commentMouseControler.ContextMenuEnabled = true;
                gridControlInstance.MouseControllerDispatcher.Add(_commentMouseControler);

                //Sort the Grid
                gridControlInstance.RaiseCellClick(PROPERTIES_COL, COMPONENT_ROW, new MouseEventArgs(MouseButtons.Left, 1, 1, 1, 1));

                //Register Run Completed Event
                //_lrrApp.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
                //_lrrApp.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);
            }



            if (_searchCriteria.SearchQuery.UserSettingsID != null)
            {
                LRRUserSetting userSettings = new LRRUserSetting();
                _lrrApp.DBManagerObj.PopulateObject(userSettings, typeof(LRRUserSetting), " WHERE ID='" + _searchCriteria.SearchQuery.UserSettingsID + "'");

                if (userSettings != null)
                {
                    _gridPersistance.SetBytesToUserSettingsXml(userSettings.UserSettings);
                    _gridPersistance.GridCollection = GetPanelCollection();
                    GridPersistance.LoadGridSettings(this.GetType().ToString());
                }
            }


            //Check forPersistance
            //if (ApplicationPersistanceUtil.HtUserQueries.ContainsKey(_searchCriteria.SearchQuery.ID))
            //{
            //    GridPersistance.GctlSetting = ApplicationPersistanceUtil.HtUserQueries[_searchCriteria.SearchQuery.ID] as LRRGridControlSetting;
            //    GridPersistance.GridCollection = GetPanelCollection();
            //    GridPersistance.LoadGridSettings(this.GetType().ToString());
            //}
        }
		
		/// <summary>
		/// Returns the panels contianing the grids
		/// </summary>
		/// <returns></returns>
		public System.Collections.ArrayList GetPanelCollection()
		{
			return PanelCollection;
		}
		
		/// <summary>
		/// Saves the query
		/// </summary>
		/// <param name="name"></param>
		/// <param name="desc"></param>
		/// <param name="srchGroupLevel"></param>
		public 	void Save(string name,string desc,LRRSearchGroupLevel srchGroupLevel,bool isPublished)
		{
			//Persist the grid Settings
			//SearchQuery.SearchQuery.SizeXml = GridPersistance.GetSizeXmlBytes();
			SearchQuery.SearchQuery.TemplateClass = this.GetType().ToString();

			//IOScreenUtilities.SaveQuery(name,desc,srchGroupLevel,GridManagerHash,SearchQuery,isPublished);

            //IOScreen.GridPersistanceUtil gutil = new IOScreen.GridPersistanceUtil();
            _gutil.GridCollection = _pnlCollection;
            _gutil.SaveGridSettings(false, false);

            string searchID = SearchQuery.SearchQuery.ID;
            
            LRRUserSetting userSettings = new LRRUserSetting();
            if (SearchQuery.SearchQuery.UserSettingsID == null)
            {
                SearchQuery.SearchQuery.UserSettingsID = userSettings.ID;
            }
            else
            {
                _lrrApp.DBManagerObj.PopulateObject(userSettings, typeof(LRRUserSetting), " WHERE ID='" + SearchQuery.SearchQuery.UserSettingsID + "'");
                userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                _lrrApp.DBManagerObj.CommitObject(userSettings);
            }
            
            string userID = "";
            if (SearchQuery.SearchQuery.UserID == "" || SearchQuery.SearchQuery.UserID == null)
            {
                userID = _lrrApp.DBManagerObj.AccessControlModel.UserID;
            }
            else
            {
                userID = SearchQuery.SearchQuery.UserID;
            }
            
            userSettings.UserSettings = _gutil.GetUserSettingsXmlBytes();
            userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            _lrrApp.DBManagerObj.CommitObject(userSettings, userID);

            IOScreenUtilities.SaveQuery(name, desc, srchGroupLevel, GridManagerHash, SearchQuery, isPublished);

            //if (ApplicationPersistanceUtil.HtUserQueries.ContainsKey(searchID))
            //    ApplicationPersistanceUtil.HtUserQueries[searchID] = gutil.GctlSetting;
            //else
            //    ApplicationPersistanceUtil.HtUserQueries.Add(searchID, gutil.GctlSetting);
		}
		
		/// <summary>
		/// Saves the Query
		/// </summary>
		public void Save()
		{
			string searchID = this.SearchQuery.SearchQuery.ID;
			//IOScreenUtilities.Save(searchID,GridManagerHash,false);

			//IOScreen.GridPersistanceUtil gutil = new IOScreen.GridPersistanceUtil();
			_gutil.GridCollection = _pnlCollection;
			_gutil.SaveGridSettings(false,false);

            LRRUserSetting userSettings = new LRRUserSetting(); ;
            if (SearchQuery.SearchQuery.UserSettingsID == null)
            {
                SearchQuery.SearchQuery.UserSettingsID = userSettings.ID;
            }
            else
            {
                _lrrApp.DBManagerObj.PopulateObject(userSettings, typeof(LRRUserSetting), " WHERE ID='" + SearchQuery.SearchQuery.UserSettingsID + "'");
                userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
                _lrrApp.DBManagerObj.CommitObject(userSettings);
            }

            string userID = "";
            if (SearchQuery.SearchQuery.UserID == "" || SearchQuery.SearchQuery.UserID == null)
            {
                userID = _lrrApp.DBManagerObj.AccessControlModel.UserID;
            }
            else
            {
                userID = SearchQuery.SearchQuery.UserID;
            }

            userSettings.UserSettings = _gutil.GetUserSettingsXmlBytes();
            userSettings.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            _lrrApp.DBManagerObj.CommitObject(userSettings, userID);

            IOScreenUtilities.Save(searchID, GridManagerHash, false);

            //if(ApplicationPersistanceUtil.HtUserQueries.ContainsKey(searchID))
            //    ApplicationPersistanceUtil.HtUserQueries[searchID] = gutil.GctlSetting;
            //else
            //    ApplicationPersistanceUtil.HtUserQueries.Add(searchID,gutil.GctlSetting);
		}

		/// <summary>
		/// Exports the selected grid
		/// </summary>
		/// <returns></returns>
		public GridControl Export()
		{
			return IOScreenUtilities.Export(GridManagerHash);
		}

		/// <summary>
		/// Prints the selected grid
		/// </summary>
		public void Print()
		{
			IOScreenUtilities.Print(GridManagerHash);
		}

		/// <summary>
		/// Print preview of the selected grid
		/// </summary>
		public void PrintPreview()
		{
			IOScreenUtilities.PrintPreview(GridManagerHash);
		}

		public SearchCriteria SearchQuery 
		{
			get{return _searchCriteria;}
			set
			{
				_searchCriteria = value;
			}
		}

		/// <summary>
		/// This method hides the rows on change of Properties window selected records
		/// </summary>
		/// <param name="arrayUnSelectedRecords"></param>
		public void HideComponents(ArrayList arrayUnSelectedRecords)
		{
            UnSelectedRecords = arrayUnSelectedRecords;
            HideModes();
		}
		
		/// <summary>
		/// This method hides the rows on change of Components window selected records
		/// </summary>
		/// <param name="arrayUnSelectedRecords"></param>
		public void HideProperties(ArrayList arrayUnSelectedRecords)
		{
			IOScreenUtilities.HideRows(arrayUnSelectedRecords,GridManagerHash,DATA_ROW_START);
		}
		
		/// <summary>
		/// Method hides the modes columns
		/// </summary>
		public void HideModes()
		{
			Cursor.Current = Cursors.WaitCursor;
			Cursor.Show();
			
			//Check if Reference mode has changed
			foreach(LRRMode mode in LRRModel.ProjectModes)
			{
				if(mode.ArrListUpdatedColNames.Contains("Type"))
				{
					IEnumerator iEnum = GridManagerHash.GetEnumerator();
					while(iEnum.MoveNext())
					{
						DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
						GridControl gridControl =  dEntry.Key as GridControl;
						GridControlManager gridControlManager =  dEntry.Value as GridControlManager;
						ValidatePropertyValues(gridControl,gridControlManager);
					}
					break;
				}
			}

			IOScreenUtilities.HideColumns(UnSelectedRecords,GridManagerHash,DATA_COL_START);
			IOScreenUtilities.HideModes(GridManagerHash,DATA_COL_START);
			Cursor.Current = Cursors.Default;
		}

		/// <summary>
		/// Method Validates the property Values and sets the colors on the grid cells
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		private void ValidatePropertyValues(GridControl gridControl,GridControlManager gridControlManager)
		{
			gridControl.BeginUpdate();
			//Get the Modes Count
			int iModesCount = LRRModel.ProjectModes.Count;
			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.SortManagerList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			//Get the Row Header from Grid Data Manager
			Hashtable		RowHeader = gridControlManager.RowHeader;
			//Get the Col Header from Grid Data Manager
			Hashtable		ColHeader = gridControlManager.ColHeader;

			//Loop through the grid matrix,put property values,add bindings
			for(int iRow = DATA_ROW_START;iRow < RowHeader.Count + DATA_ROW_START - 1; iRow++)
			{
				for(int iCol = DATA_COL_START; iCol < (ColHeader.Count - DATA_COL_START + 1) * iModesCount + DATA_COL_START; iCol++)
				{
					//Get the row data
					IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
					string rowCellID = rowData.ID;
					//Get the column data
					ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
					Type objectType = colData.ObjectType;
					string modeID   = colData.ModeID;
					string colCellID = colData.ID;

					//Get the Property Value Object
					object objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);
				
					//Put Property Values in grid cells
					if(objPropValue != null)
					{
						IPropertyValue iPropValue = objPropValue as IPropertyValue;
						iPropValue.Validate();
						SetColor(iPropValue,gridControl.Model[iRow,iCol]);
					}
				}
			}
			gridControl.EndUpdate(true);
		}

		/// <summary>
		/// This method clears the bindings
		/// </summary>
		public void ClearAllBindings()
		{
			IEnumerator iEnum = GridManagerHash.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
				GridControlManager gridControlManager =  dEntry.Value as GridControlManager;
				ClearBindings(gridControlManager);
			}
		}
		
		/// <summary>
		/// Method clears the sorting
		/// </summary>
		public void ClearSort()
		{
			IOScreenUtilities.ClearSort(GridManagerHash);
		}
		
		/// <summary>
		/// Method moves the range of columns to right
		/// </summary>
		public void RightColMovement()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;

				if(gridControl.CurrentCell.IsActive)
				{
					GridRangeInfo rangeInfo = gridControl.Selections.Ranges.ActiveRange;
					int range = rangeInfo.Right - rangeInfo.Left;
					int right = rangeInfo.Right;
					int left = rangeInfo.Left;
					
					if(right + LRRModel.ProjectModes.Count > gridControl.ColCount)
						return;

					gridControl.ResetVolatileData();
					gridControl.BeginUpdate();
					Cursor.Current = Cursors.WaitCursor;
					Cursor.Show();
					
					int target = left + LRRModel.ProjectModes.Count;
					gridControl.Cols.MoveRange(left,range + 1,target);
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;

                    //for(int i=1;i<=gridControl.ColCount;i++)
                    //{
                    //    if(gridControl[0,i].Text.Trim() != "")
                    //    {
							//GridPersistanceUtil._arrGridColMove[0].Add(gridControl[0,1].Text);
                    //        break;
                    //    }
                    //}
					_gutil._arrGridColMove[0].Add(left);
                    _gutil._arrGridColMove[1].Add(range + 1);
                    _gutil._arrGridColMove[2].Add(target);

					InitializePropertyValues(gridControl,gridControlManager);

					gridControl.EndUpdate();
					Cursor.Hide();
					break;
				}
			}
		}
		
		/// <summary>
		/// Method moves the range of columns to left
		/// </summary>
		public void LeftColMovement()
		{		
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;

				if(gridControl.CurrentCell.IsActive)
				{
					GridRangeInfo rangeInfo = gridControl.Selections.Ranges.ActiveRange;
					int range = rangeInfo.Right - rangeInfo.Left;
					int right = rangeInfo.Right;
					int left = rangeInfo.Left;
					int target = left - LRRModel.ProjectModes.Count;
					if(target < DATA_COL_START || rangeInfo.Top != HEADER_ROW)
						return;

					gridControl.ResetVolatileData();
					gridControl.BeginUpdate();

					Cursor.Current = Cursors.WaitCursor;
					Cursor.Show();

					gridControl.Cols.MoveRange(left,range + 1,target);
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;

                    //for(int i=1;i<=gridControl.ColCount;i++)
                    //{
                    //    if(gridControl[0,i].Text.Trim() != "")
                    //    {
							//GridPersistanceUtil._arrGridColMove[0].Add(gridControl[0,1].Text);
                    //        break;
                    //    }
                    //}
                    _gutil._arrGridColMove[0].Add(left);
                    _gutil._arrGridColMove[1].Add(range + 1);
                    _gutil._arrGridColMove[2].Add(target);

					InitializePropertyValues(gridControl,gridControlManager);

					gridControl.EndUpdate();
					Cursor.Hide();
					break;
				}
			}
		}
		
		/// <summary>
		/// Method changes the UOM Set
		/// </summary>
		/// <param name="measSysID"></param>
		public void ChangeUOMSet(string measSysID)
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = (GridControlManager)dEntry.Value;
				if(gridControl.CurrentCell.IsActive)
				{
					for(int iRow = UOM_ROW;iRow < gridControl.RowCount;iRow ++)
					{
						try
						{
							//Get the Row Data from Grid Data Manager
							ArrayList     rowSortList = gridControlManager.SortManagerList;
							//Get the row data
							IOSortManager rowData = rowSortList[iRow - UOM_ROW] as IOSortManager;
						
							string uomTypeID = rowData.UOMTypeId;
							string uiUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(measSysID,uomTypeID);
							rowData.UIUOMId = uiUOMID;

							//Add to UI UOM hash
							if(gridControlManager.UIUOMHash.ContainsKey(rowData.ID))
							{
								gridControlManager.UIUOMHash[rowData.ID] = uiUOMID;
							}
							else
							{
								gridControlManager.UIUOMHash.Add(rowData.ID,uiUOMID);
							}
						
							string uomDisplayName = LRRModel.ChildModel.GetUOMNameFromID(uiUOMID);
							gridControl.Model[iRow + 1,UOM_COL].Text = uomDisplayName;
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridSimpleViewController),"Exception in changing UOMSet:- " + ex.Message,Logger.Level.Debug);
						}
					}
					InitializePropertyValues(gridControl,gridControlManager);
					break;
				}
			}
		}
		
		/// <summary>
		/// Method Unsubscribes the Reconnect Completed Event
		/// </summary>
		public void UnSubscribeReconnectCompleted()
		{
			this._lrrApp.ReconnectCompleted-=new EventHandler(_lrrApp_ReconnectCompleted);
		}

        /// <summary>
        /// Method resets trending/Recording display flag
        /// </summary>
        /// <param name="displayFlag"></param>
        public void ResetRecordingTrendingDisplayFlag(bool displayFlag)
        {
            IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator() as IDictionaryEnumerator;
            while (iDictEnum.MoveNext())
            {
                GridControlManager gcManager = iDictEnum.Value as GridControlManager;
                GridControl gridControl = iDictEnum.Key as GridControl;
                ResetDisplayFlag(gridControl, gcManager, displayFlag);
                _recTrendDisplayFlag = displayFlag;
            }
        }

		#endregion

		#region Private Methods
		/// <summary>
		/// This method gets the total no. of rows in the grid
		/// </summary>
		/// <param name="objCollection"></param>
		/// <param name="colCount"></param>
		/// <returns></returns>
		private int GetGridRowCount(object[,] objCollection,int iRow,int colCount)
		{
			int iGridRowCount = 0;
			Hashtable propHash = new Hashtable();
			for(int i=0;i < colCount;i++)
			{
				CustomCollection custCollection = objCollection[iRow,i] as CustomCollection;
				IEnumerator iEnum = custCollection.GetEnumerator();
				while(iEnum.MoveNext())
				{
					IComponent iComp = iEnum.Current as IComponent;
					CustomCollection propsCollection = iComp.GetProperties();
					IEnumerator iPropEnum = propsCollection.GetEnumerator();
					while(iPropEnum.MoveNext())
					{
						IProperty iProp = iPropEnum.Current as IProperty;
						string id = iProp.GetMasterPropertyID();
						if(id == null || id.Trim() == "")
							id = iProp.GetID();
						if(id != null && id.Trim() !="" && !propHash.ContainsKey(id))
							propHash.Add(id,i);
					}
				}
			}
			iGridRowCount = propHash.Count + 2;
			return iGridRowCount;
		}
		
		/// <summary>
		/// This method gets the total no. of columns in the grid
		/// </summary>
		/// <param name="objCollection"></param>
		/// <param name="colCount"></param>
		/// <returns></returns>
		private int GetGridColCount(object[,] objCollection,int iRow,int colCount)
		{
			int iGridColCount = 0;
			for(int i=0;i < colCount;i++)
			{
				CustomCollection custCollection = objCollection[iRow,i] as CustomCollection;
				iGridColCount = iGridColCount + custCollection.Count;
			}
			iGridColCount = iGridColCount * LRRModel.ProjectModes.Count + 2;
			return iGridColCount;
		}

		/// <summary>
		/// This method registers all the grid events
		/// </summary>
		/// <param name="gridControlInstance"></param>
		private void registerGridEvents(GridControl gridControlInstance)
		{
			gridControlInstance.CurrentCellCloseDropDown +=new Syncfusion.Windows.Forms.PopupClosedEventHandler(gridControlInstance_CurrentCellCloseDropDown);
			gridControlInstance.CurrentCellValidating +=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellValidating);
			gridControlInstance.CurrentCellStartEditing+=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellStartEditing);
			
					
			//add a sort header to the CellModels.
			gridControlInstance.CellModels.Add("ColumnHeaderCell", new GridSortColumnHeaderCellModel(gridControlInstance.Model));

			//catch the click
			gridControlInstance.CellClick +=new GridCellClickEventHandler(gridControlInstance_CellClick);

			//Event for row movement
			gridControlInstance.RowsMoving+=new GridRangeMovingEventHandler(gridControlInstance_RowsMoving);
			gridControlInstance.RowsMoved+=new GridRangeMovedEventHandler(gridControlInstance_RowsMoved);

			//Event for Col Movement
			gridControlInstance.ColsMoving+=new GridRangeMovingEventHandler(gridControlInstance_ColsMoving);

			//Event for preventing hidden columns from expanding
			gridControlInstance.ResizingColumns+=new GridResizingColumnsEventHandler(gridControlInstance_ResizingColumns);
			
			//Enabling/Disabling Left/Right Col Movement Buttons
			gridControlInstance.SelectionChanged+=new GridSelectionChangedEventHandler(gridControlInstance_SelectionChanged);
			
			//Event for Handling Key down
			gridControlInstance.KeyDown+=new KeyEventHandler(gridControlInstance_KeyDown);
			
			//Event for Handling Current Cell Key down
			gridControlInstance.CurrentCellKeyDown+=new KeyEventHandler(gridControlInstance_CurrentCellKeyDown);
			
			//Event for Handling Current Cell Key Press
			gridControlInstance.CurrentCellKeyPress+=new KeyPressEventHandler(gridControlInstance_CurrentCellKeyPress);

            //Enabling command stack
            gridControlInstance.CommandStack.Enabled = true;

			//Event for Painting Header text
			gridControlInstance.Paint+=new PaintEventHandler(gridControlInstance_Paint);

            //Event for after a cell accepted changes
            gridControlInstance.CurrentCellAcceptedChanges += new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellAcceptedChanges);

            gridControlInstance.Model.ClipboardPaste += new Syncfusion.Windows.Forms.Grid.GridCutPasteEventHandler(Model_ClipboardPaste);

			//Event for Reconnect
			this._lrrApp.ReconnectCompleted+=new EventHandler(_lrrApp_ReconnectCompleted);

			//Context Menu for switching recording values
			ContextMenu contMenu = new ContextMenu();
			gridControlInstance.ContextMenu = contMenu;
			contMenu.Popup+=new EventHandler(contMenu_Popup);
		}

        void gridControlInstance_CurrentCellAcceptedChanges(object sender, System.ComponentModel.CancelEventArgs e)
        {
            GridControl grid = sender as GridControl;
            if (!(grid[COMPONENT_ROW, grid.CurrentCell.ColIndex].Text == IOSchemes.UOM_HEADER_NAME))
            {
                if (grid.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.IOScreen.InputOutputViews))
                {
                    IOScreen.InputOutputViews.BPropertyValChanged = true;
                }
                else if (grid.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.IOScreen.ResultForm))
                {
                    IOScreen.ResultForm.BIOViewsResultFormChange = true;
                }
            }
        }
		
		/// <summary>
		/// This method renders the grid
		/// </summary>
		/// <param name="gridControlInstance"></param>
		/// <param name="compCollection"></param>
		/// <param name="iRow"></param>
		/// <param name="gridManager"></param>
		/// <param name="srchCriteriaItemProperties"></param>
		private void createGrid(GridControl gridControlInstance,object[,] compCollection,int iRow,GridControlManager gridManager,LRRSearchCriteriaItemProperties srchCriteriaItemProperties)
		{
			gridControlInstance.ResetVolatileData();
			gridControlInstance.BeginUpdate();
			
			//Set the Header appearence and properties
			IOScreenUtilities.InitializeGrid(gridControlInstance);

			//Create Row Header,Col Header Data Objects
			ArrayList rowSortManagerList = new ArrayList();
			ArrayList colHeaderCellList = new ArrayList();
			Hashtable RowHeader = new Hashtable();
			Hashtable ColHeader = new Hashtable();

			//Assign to GridControlManager
			gridManager.SortManagerList = rowSortManagerList;
			gridManager.ColHeaderCellList = colHeaderCellList;
			gridManager.RowHeader = RowHeader;
			gridManager.ColHeader = ColHeader;

			//Clear the Row/Col Headers
			ClearHeaders(gridManager);

			//Start Grid Rendering
			if(!RowHeader.Contains(HEADER_ROW))
				RowHeader.Add(HEADER_ROW,IOSchemes.ATTRIBUTE_HEADER_NAME);
			
			//Attributes header row
			try
			{
				gridControlInstance[COMPONENT_ROW,PROPERTIES_COL].Text = IOSchemes.ATTRIBUTE_HEADER_NAME;
				gridControlInstance[COMPONENT_ROW,PROPERTIES_COL].BackColor = IOSchemes.ATTRIBUTES_BACK_COLOR;
				gridControlInstance[COMPONENT_ROW,PROPERTIES_COL + 1].BackColor = IOSchemes.ATTRIBUTES_BACK_COLOR;
				gridControlInstance[COMPONENT_ROW + 1,PROPERTIES_COL].BackColor = IOSchemes.ATTRIBUTES_BACK_COLOR;
				gridControlInstance[COMPONENT_ROW + 1,PROPERTIES_COL + 1].BackColor = IOSchemes.ATTRIBUTES_BACK_COLOR;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GridSimpleViewController),"Exception in rendering Attribute Header Row:- " + ex.Message,Logger.Level.Debug);
			}
			
			//UOM Header Row
			try
			{
				gridControlInstance[COMPONENT_ROW,UOM_COL].Text = IOSchemes.UOM_HEADER_NAME;
				gridControlInstance[COMPONENT_ROW,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;
				gridControlInstance[COMPONENT_ROW + 1,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GridSimpleViewController),"Exception in rendering UOM Header Row:- " + ex.Message,Logger.Level.Debug);
			}
			
			int gridLastRowNum = 3;
			int gridLastColNum = 3;
			for(int iCol=0;iCol <= compCollection.GetUpperBound(1);iCol++)
			{
				//Create Row Headers,Column Headers
				createHeaders(gridControlInstance,(CustomCollection) compCollection[iRow,iCol],gridManager,ref gridLastRowNum,ref gridLastColNum,srchCriteriaItemProperties);
			}

            //this.commentMouseController1 = new ExcelTip.CommentMouseController(gridControlInstance);
            //this.commentMouseController1.ContextMenuEnabled = true;
            //gridControlInstance.MouseControllerDispatcher.Add(commentMouseController1);
			gridControlInstance.ResumeLayout();
			gridControlInstance.EndUpdate(true);
		}
		
		/// <summary>
		/// This method creates the row/col headers for the grid
		/// </summary>
		/// <param name="gridControlInstance"></param>
		/// <param name="compCollection"></param>
		/// <param name="gridManager"></param>
		/// <param name="gridLastRowNum"></param>
		/// <param name="gridLastColNum"></param>
		/// <param name="srchCriteriaItemProperties"></param>
		private void createHeaders(GridControl gridControlInstance,CustomCollection compCollection,GridControlManager gridManager,ref int gridLastRowNum,ref int gridLastColNum,LRRSearchCriteriaItemProperties srchCriteriaItemProperties)
		{
			//Create Row Header,Col Header Data Objects
			ArrayList rowSortManagerList = gridManager.SortManagerList;
			ArrayList       colHeaderCellList = gridManager.ColHeaderCellList;
			Hashtable		RowHeader = gridManager.RowHeader;
			Hashtable		ColHeader = gridManager.ColHeader;
			
			int nModesCol = gridLastColNum;
			string ID = "";
			string name ="";
			int nModes = LRRModel.ProjectModes.Count;
            string desc = LRRHeaderText.GetHeaderString("SID_ATTRIBUTEDESC");

			//Loop through the Collection and render the grid
			IEnumerator iCompEnum = compCollection.GetEnumerator();
			while(iCompEnum.MoveNext())
			{
				IComponent iComp = iCompEnum.Current as IComponent;
				CustomCollection propsCollection =  iComp.GetProperties();
			
				//Get the Master Component name for Header Display
				LRRMasterComponent masterComp = LRRModel.Model.GetMasterComponentByID(iComp.GetMasterComponentID());
				gridControlInstance[HEADER_ROW,PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
                gridControlInstance[HEADER_ROW, PROPERTIES_COL].CellTipText = desc;
				gridManager.HeaderText = masterComp.ComponentName + ": " + compCollection.Count + " Items";

				//Loop through Properties collection and render the grid
				IEnumerator iPropEnum = propsCollection.GetEnumerator();
				while(iPropEnum.MoveNext())
				{
					IProperty iProp = iPropEnum.Current as IProperty;
					ID = iProp.GetMasterPropertyID();
					name = iProp.GetName();

					//if masterPropertyID is null, then get ID --> sitespecific property
					if(ID == null || ID.Trim() == "")
					{
						ID = iProp.GetID();
					}
					if((ID == null) && (name == null))
					{
						LRRMessagBox.ShowMessage("SID_BLANK_ERROR","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Information);  
						continue;
					}
					
					//Put Property Name on Grid Cells
					if(!RowHeader.Contains(ID))
					{
						//Add to Row Data manager
						IOSortManager sortManager = new IOSortManager();
						sortManager.Name = name;
						sortManager.ID = ID;

						//Get UOMID from property
						string uomTypeID = iProp.GetUOMType();
						string uomID = iProp.GetUOM();
						string uiUOMID = iProp.GetUIUOMID();
                        


						if(srchCriteriaItemProperties != null)
						{
							LRRSearchCriteriaItemProperty srchCriteriaItemProperty = srchCriteriaItemProperties.GetItemPropertyFromPropertyID(ID);
							if(srchCriteriaItemProperty != null)
							{
								LRRSearchCriteriaUserSpecificUIUOM  uiUOMObj = _searchCriteriaUserSpecficUIUOMs.GetUIUOMObjectByPropertyID(srchCriteriaItemProperty.ID);
								if(uiUOMObj != null)
									uiUOMID = uiUOMObj.UIUomID;
							}
						}

                        //string newUIUOM = ChangedUIUOMDictionary.RetrieveUOM(this.GetType(), ID);
                        //if (newUIUOM != null)
                        //    uiUOMID = newUIUOM;

						if(uomID != null && uomID.Trim() != "")
							sortManager.UOMName = LRRModel.ChildModel.GetUOMNameFromID(uomID);
						else
							sortManager.UOMName = "";
						sortManager.UOMID = uomID;
						sortManager.UIUOMId = uiUOMID;
						sortManager.UOMTypeId = uomTypeID;
						rowSortManagerList.Add(sortManager);
						
						try
						{
							RowHeader.Add(ID,name);
							gridControlInstance[gridLastRowNum,PROPERTIES_COL].Text = name;
                            gridControlInstance[gridLastRowNum, PROPERTIES_COL].CellTipText = desc;
							gridControlInstance[gridLastRowNum,PROPERTIES_COL].BackColor = IOSchemes.GRID_ROW_COLOR;
							gridControlInstance[gridLastRowNum,PROPERTIES_COL].CellTipText = name;
							gridControlInstance[gridLastRowNum,PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
							gridControlInstance[gridLastRowNum,PROPERTIES_COL].TextMargins.Left = 5;
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridSimpleViewController),"Exception in rendering Properties Row:- " + ex.Message,Logger.Level.Debug);
						}

						//Put UOMs on grid cells
						LRRUOMs	uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomTypeID);

						//Get UOM name from UOMID
						string uomName = "";
						string uiUOMName = "";
						string uomDisplayName = "";
						if(uomID != null && uomID.Trim() != "")
						{
							uomName = LRRModel.ChildModel.GetUOMNameFromID(uomID);
						}
						if(uiUOMID != null && uiUOMID.Trim() != "")
						{
							uiUOMName = LRRModel.ChildModel.GetUOMNameFromID(uiUOMID);
						}
						if(uiUOMID!=null && uiUOMID.Trim()!= "" && uiUOMID != uomID)
						{
							uomDisplayName = uiUOMName;
						}
						else
						{
							uomDisplayName = uomName;
						}
						
						try
						{
							sortManager.UOMDisplayName = uomDisplayName;
							gridControlInstance.Model[gridLastRowNum,UOM_COL].Text = uomDisplayName;
							gridControlInstance.Model[gridLastRowNum,UOM_COL].CellType = "ComboBox";
							gridControlInstance.Model[gridLastRowNum,UOM_COL].DataSource = uoms;
							uoms.SortColumn="Name";
							uoms.Sort();
							gridControlInstance.Model[gridLastRowNum,UOM_COL].DisplayMember = "Name";
							gridControlInstance.Model[gridLastRowNum,UOM_COL].ValueMember = "ID";
							gridControlInstance.Model[gridLastRowNum,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;
							gridControlInstance.Model[gridLastRowNum,UOM_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
							
							if(!gridManager.UIUOMHash.ContainsKey(sortManager.ID))
								gridManager.UIUOMHash.Add(sortManager.ID,sortManager.UIUOMId);
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridSimpleViewController),"Exception in rendering UOM:- " + ex.Message,Logger.Level.Debug);
						}
						gridLastRowNum++;
					}
				}

				//Put Components on grid cells
				ID = iComp.GetID();
				name = iComp.GetName();
				if(!ColHeader.Contains(ID))
				{
					ColHeader.Add(ID,name);
					gridControlInstance.CoveredRanges.Add(GridRangeInfo.Cells(COMPONENT_ROW,gridLastColNum,COMPONENT_ROW,gridLastColNum+nModes-1));
					gridControlInstance[COMPONENT_ROW,gridLastColNum].Text = name;
					gridControlInstance[COMPONENT_ROW,gridLastColNum].CellTipText = name;

					//Loop through the Modes collection and render the grid
					foreach(LRRMode lrrMode in LRRModel.ProjectModes)
					{
						try
						{
							gridControlInstance.ColWidths.SetSize(nModesCol,IOSchemes.MODE_COL_SIZE);
							gridControlInstance[MODES_ROW,nModesCol].WrapText = false;
							gridControlInstance[MODES_ROW,nModesCol].AutoSize = true;
							gridControlInstance[MODES_ROW,nModesCol].Text = lrrMode.ModeDisplayName;
							gridControlInstance[MODES_ROW,nModesCol].CellTipText = lrrMode.ModeName;
							gridControlInstance[MODES_ROW,nModesCol].BackColor = IOSchemes.GetModeBackColor(lrrMode.ModeId);
							gridControlInstance[MODES_ROW,nModesCol].TextColor = IOSchemes.GetModeTextColor(lrrMode.ModeId);
						}
						catch(Exception ex)
						{
							Logger.LogMessage(typeof(GridSimpleViewController),"Exception in rendering Mode:- " + ex.Message,Logger.Level.Debug);
						}

						//Add ColHeader Info
						ColHeaderCell colHeaderCell = new ColHeaderCell();
						colHeaderCell.ID = ID;
						colHeaderCell.ModeID = lrrMode.ModeId;
						colHeaderCell.ObjectType = iComp.GetObjectType();
						colHeaderCellList.Add(colHeaderCell);

						nModesCol=nModesCol+1;
					}
					gridControlInstance[COMPONENT_ROW,gridLastColNum].BackColor = IOSchemes.COMPONENT_BACK_COLOR;
					gridLastColNum = gridLastColNum + 1;
				}
				gridLastColNum = nModesCol;
			}
			if(!ColHeader.Contains(HEADER_ROW))
				ColHeader.Add(HEADER_ROW,IOSchemes.ATTRIBUTE_HEADER_NAME);
			if(!ColHeader.Contains(HEADER_ROW + 1))
				ColHeader.Add(HEADER_ROW + 1,IOSchemes.UOM_HEADER_NAME);
		}
		
		/// <summary>
		/// Method creates the row Headers
		/// </summary>
		/// <param name="gridControlInstance"></param>
		/// <param name="gridControlManager"></param>
		private void createRowHeaders(GridControl gridControlInstance,GridControlManager gridControlManager)
		{
			ArrayList rowSortManagerList = gridControlManager.SortManagerList;
			
			for(int iRow = DATA_ROW_START;iRow < rowSortManagerList.Count + DATA_ROW_START;iRow++)
			{
				IOSortManager rowManager = rowSortManagerList[iRow - DATA_ROW_START] as IOSortManager;
				gridControlInstance[iRow,PROPERTIES_COL].Text = rowManager.Name;
				gridControlInstance[iRow,PROPERTIES_COL].BackColor = IOSchemes.GRID_ROW_COLOR;
				gridControlInstance[iRow,PROPERTIES_COL].CellTipText = rowManager.Name;
				gridControlInstance[iRow,PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
				gridControlInstance[iRow,PROPERTIES_COL].TextMargins.Left = 5;
				

				LRRUOMs	uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(rowManager.UOMTypeId);
				gridControlInstance.Model[iRow,UOM_COL].Text = rowManager.UOMDisplayName;
				gridControlInstance.Model[iRow,UOM_COL].CellType = "ComboBox";
				gridControlInstance.Model[iRow,UOM_COL].DataSource = uoms;
				uoms.SortColumn="Name";
				uoms.Sort();
				gridControlInstance.Model[iRow,UOM_COL].DisplayMember = "Name";
				gridControlInstance.Model[iRow,UOM_COL].ValueMember = "ID";
				gridControlInstance.Model[iRow,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;
				gridControlInstance.Model[iRow,UOM_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
			}
		}

		/// <summary>
		/// This method initializes the property values,displays it on grid,adds bindings
		/// </summary>
		/// <param name="gridControlInstance"></param>
		/// <param name="gridControlManager"></param>
		private void InitializePropertyValues(GridControl gridControlInstance,GridControlManager gridControlManager)
		{
			//Get the Modes Count
			int iModesCount = LRRModel.ProjectModes.Count;

			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.SortManagerList;

			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			
			//Get the Row Header from Grid Data Manager
			Hashtable		RowHeader = gridControlManager.RowHeader;

			//Get the Col Header from Grid Data Manager
			Hashtable		ColHeader = gridControlManager.ColHeader;

			//Clear the Bindings
			ClearBindings(gridControlManager);
			
			//Clear the cell Addresses
			gridControlManager.PropValueCellAddress.Clear();

			//Loop through the grid matrix,put property values,add bindings
			for(int iRow = DATA_ROW_START;iRow < RowHeader.Count + DATA_ROW_START - 1; iRow++)
			{
				for(int iCol = DATA_COL_START; iCol < (ColHeader.Count - DATA_COL_START + 1) * iModesCount + DATA_COL_START; iCol++)
				{
					try
					{
						//Get the row data
						IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
						string rowCellID = rowData.ID;
						string uiUOMID   = rowData.UIUOMId;
						//					string uomID     = rowData.UOMID;

						//Get the column data
						ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
						Type objectType = colData.ObjectType;
						string modeID   = colData.ModeID;
						string colCellID = colData.ID;
					
						object objProp = LRRModel.GetPropertyObject(objectType,rowCellID,colCellID);
						if(objProp != null)
						{
							//Get the Property Object
							IProperty iProp = objProp as IProperty;
							iProp.IsTrackedChanged+=new EventHandler(gridControlManager.IsTrackChangedHandler);
                            iProp.IsTrendedChanged += new EventHandler(gridControlManager.IsTrackChangedHandler); 
						
							object objPropValue = null;
							//If Mode is true
							if(iProp.GetModeDefinition())
							{
								//Get the Property Value Object
								objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);
							}
							else
							{
								//Get the Property Value Object for Default Mode
								objPropValue = LRRModel.GetPropertyValueObject(objectType,"0",rowCellID,colCellID);
							}
							//Put Property Values in grid cells
							if(objPropValue != null)
							{
								IPropertyValue iPropValue = objPropValue as IPropertyValue;
								string uomID = iPropValue.GetUOMID();

								//check for EVVs,if yes then put an exclusive drop down
								if(iPropValue.GetEditableState())
								{
									string uomTypeID = iProp.GetUOMType();
									if(uomTypeID != null && uomTypeID.Trim() != "")
									{
										LRRUOMType lrrUOMType = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(uomTypeID);
										if(lrrUOMType != null && lrrUOMType.IsEVV && !LRRModel.ChildModel.ConstantEVVs.ContainsKey(uomTypeID))
										{
											LRREVVs evvs = LRRModel.ChildModel.GetEVVsFromUOMTypeID(uomTypeID);
											gridControlInstance.Model[iRow,iCol].CellType = "ComboBox";
											gridControlInstance.Model[iRow,iCol].DataSource = evvs;
											gridControlInstance.Model[iRow,iCol].DisplayMember = "EVV";
											gridControlInstance.Model[iRow,iCol].ValueMember = "EVV";
                                            gridControlInstance.Model[iRow, iCol].DropDownStyle = GridDropDownStyle.Exclusive;
										}
									}
								}
							
								GridStyleInfo gsInfo = gridControlInstance.Model[iRow,iCol];
								if(iProp.GetModeDefinition() || modeID == "0")
								{
									//Add Bindings
									gridControlManager.BindingManager.AddBinding(objPropValue,"PropertyValue",gsInfo,"Text");

									//Add Colour Binding
									gridControlManager.PropValueCellAddress.Add(iPropValue.GetID(),new GridCellAddress(iRow,iCol));
									iPropValue.ForeColorChanged+=new EventHandler(gridControlManager.ForeColorChangedHandler);
									iPropValue.BackColorChanged+=new EventHandler(gridControlManager.BackColorChangedHandler);

									if(modeID == "0")
										gsInfo.Changed+=new Syncfusion.Styles.StyleChangedEventHandler(gsInfo_Changed);
								
									//Set Colour
									SetColor(iPropValue,gridControlInstance.Model[iRow,iCol]);
									SetRecordedNotification(iPropValue,gridControlInstance.Model[iRow,iCol]);
								}
								else
								{
									gridControlInstance.Model[iRow,iCol].BackColor = IOSchemes.DEFAULT_MODE_COLOR;
									gridControlInstance.Model[iRow,iCol].TextColor = IOSchemes.MODE_TEXT_COLOR;
								}

								//Convert property value according to UIUOM 
								string propertyValue = iPropValue.GetPropertyValue();
								string actualPropertyValue = iPropValue.GetActualPropertyValue();
								if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
								{
									propertyValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,propertyValue);
									actualPropertyValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,actualPropertyValue);
								}

                                string lrrDataType = iPropValue.GetDataType();
                                if (lrrDataType == null || lrrDataType.Trim() == "")
                                {
                                    propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                                }
								gridControlInstance.Model[iRow,iCol].Text = propertyValue;
					
								//Set the Cell tool Tip
								SetCellToolTipText(iPropValue,gridControlInstance.Model[iRow,iCol],uiUOMID);

								gridControlInstance.Model[iRow,iCol].HorizontalAlignment = GridHorizontalAlignment.Right;
								gridControlInstance.Model[iRow,iCol].AutoSize = true;
                                gridControlInstance.Model[iRow, iCol].Trimming = StringTrimming.EllipsisCharacter;
							}
							else
							{
								gridControlInstance.Model[iRow,iCol].BackColor = IOSchemes.GRID_ROW_COLOR;
								gridControlInstance.Model[iRow,iCol].Text = "";
								gridControlInstance.Model[iRow,iCol].CellTipText = "";
								_style = new GridExcelTipStyleProperties(gridControlInstance.Model[iRow,iCol]);
								_style.ResetExcelTipText();
							}
						}
						else
						{
							gridControlInstance.Model[iRow,iCol].BackColor = IOSchemes.GRID_ROW_COLOR;
							gridControlInstance.Model[iRow,iCol].Text = "";
							gridControlInstance.Model[iRow,iCol].CellTipText = "";
							_style = new GridExcelTipStyleProperties(gridControlInstance.Model[iRow,iCol]);
							_style.ResetExcelTipText();
						}
					}
					catch(Exception ex)
					{
						Logger.LogMessage(typeof(GridSimpleViewController),"Exception in rendering SiteComponentProperty Value:- " + ex.Message,Logger.Level.Debug);
					}
				}
			}
			
			//Resize to fit
			gridControlInstance.ColWidths.ResizeToFit(GridRangeInfo.Cells(DATA_ROW_START,DATA_COL_START,gridControlInstance.RowCount,gridControlInstance.ColCount),GridResizeToFitOptions.NoShrinkSize);
		}
		
		/// <summary>
		/// This method clears the row/col Headers collection
		/// </summary>
		private void ClearHeaders(GridControlManager gridManager)
		{
			gridManager.RowHeader.Clear();
			gridManager.ColHeader.Clear();
		}
		
		/// <summary>
		/// This method clears the bindings
		/// </summary>
		/// <param name="gridControlManager"></param>
		private void ClearBindings(GridControlManager gridControlManager)
		{
			if(gridControlManager.BindingManager != null)
			{
				gridControlManager.BindingManager.RemoveBinding();
				
			}
		}

		/// <summary>
		/// This method applies sorting
		/// </summary>
		/// <param name="col"></param>
		public void SortCol(int col,GridControl gridControl,GridControlManager gridControlManager)
		{
			if(gridControl.CurrentCell.IsEditing)
				gridControl.CurrentCell.EndEdit();
		
			GridStyleInfo style = new GridStyleInfo();
			style.Tag = "";

			//turn off the old sort arrow
			for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				if(col1 != col && gridControl[1, col1].CellType == "ColumnHeaderCell")
				{
					gridControl[1, col1].CellType = "Header";
					break;
				}

			//turn on new sort arrow
			gridControl[1, col].CellType = "ColumnHeaderCell";

			System.ComponentModel.ListSortDirection dir = System.ComponentModel.ListSortDirection.Ascending;
			if(gridControl[1, col].HasTag)
			{
				if((System.ComponentModel.ListSortDirection) gridControl[1, col].Tag == System.ComponentModel.ListSortDirection.Ascending)
					dir = System.ComponentModel.ListSortDirection.Descending;
			}

			gridControl[1, col].Tag = dir;
			
			int headerCount = 2;
			string colName = null;
			if(col == 1)
				colName = "Name";
			if(col == 2)
				colName = "UOMName";
			int direction = 0;
			if(dir == System.ComponentModel.ListSortDirection.Ascending)
				direction = 0;
			if(dir == System.ComponentModel.ListSortDirection.Descending)
				direction = 1;
			
			//Get the data from the grid manager
			//Sort By Name
			if(col == 1)
			{
				gridControl.Data.SortByColumn(col,dir,headerCount);
				gridControl.Refresh();
			}

			//Sort the Row List
			Comparer   ioComparer = IOScreenUtilities.IOComparer;
			ArrayList sortManagerList = gridControlManager.SortManagerList;
			ioComparer.SortColumn = colName;
			ioComparer.SortingOrder = direction;
			sortManagerList.Sort(ioComparer);
			
			//Sort By UOM
			if(col == 2)
			{
				createRowHeaders(gridControl,gridControlManager);
				gridControl.Refresh();
			}
		}
		
		/// <summary>
		/// Changes property Values acording to UI UOM
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		private void changePropertyValueForUOM(GridControl gridControl,GridControlManager gridControlManager,int iRow,int iCol)
		{
			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.SortManagerList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the old UOM and new UOM
			string newUOM = gridControl.CurrentCell.Renderer.ControlValue == null ? "" : gridControl.CurrentCell.Renderer.ControlValue.ToString();

			//Get the row data
			IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
			string rowCellID = rowData.ID;
//			string uomID     = rowData.UOMID;

            //ChangedUIUOMDictionary.AddUOM(this.GetType(), rowData.ID.ToString(), newUOM);
			
			for(int iColIndex =DATA_COL_START; iColIndex <= gridControl.ColCount;iColIndex++)
			{
				//Get the column data
				ColHeaderCell colData = colHeaderCellList[iColIndex - DATA_COL_START] as ColHeaderCell;
				Type objectType = colData.ObjectType;
				string modeID   = colData.ModeID;
				string colCellID = colData.ID;
				
				//Get the Property Object
				object objProp = LRRModel.GetPropertyObject(objectType,rowCellID,colCellID);
				object objPropValue = null;
				if(objProp != null)
				{
					IProperty iProp = objProp as IProperty;
					//Get the Property Value Object
					if(iProp.GetModeDefinition())
					{
						objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);
					}
					else
					{
						objPropValue = LRRModel.GetPropertyValueObject(objectType,"0",rowCellID,colCellID);
					}
					_lrrApp.LRRUIManagerObj.CheckTrendUOMChanged(iProp,modeID,newUOM);
				}

				if(objPropValue != null)
				{
					IPropertyValue iPropValue = objPropValue as IPropertyValue;
					string uomID = iPropValue.GetUOMID();

					//Convert property value according to UIUOM 
					string propertyValue = iPropValue.GetPropertyValue();
					propertyValue = iPropValue.GetPropertyValueForUOM(uomID,newUOM,propertyValue);
                    
                    string lrrDataType = iPropValue.GetDataType();
                    if (lrrDataType == null || lrrDataType.Trim() == "")
                    {
                        propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                    }
					gridControl.Model[iRow,iColIndex].Text = propertyValue;

					//Set Cell Tool Tip text
					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iColIndex],newUOM);

					//Set the UI UOM to Sortmanager Row 
					rowData.UIUOMId = newUOM;

					//Set the Display UOM
					rowData.UOMDisplayName = LRRModel.ChildModel.GetUOMNameFromID(newUOM);

					//Add to UI UOM hash
					if(gridControlManager.UIUOMHash.ContainsKey(rowData.ID))
					{
						gridControlManager.UIUOMHash[rowData.ID] = newUOM;
					}
					else
					{
						gridControlManager.UIUOMHash.Add(rowData.ID,newUOM);
					}
				}
			}
		}
		
        /// <summary>
		/// Property value changes
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>
        private bool changePropertyValue(GridControl gridControl, GridControlManager gridControlManager, int iRow, int iCol)
        {
            return changePropertyValue(gridControl, gridControlManager, iRow, iCol, null);
        }

		/// <summary>
		/// Property value changes
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>
		private bool changePropertyValue(GridControl gridControl,GridControlManager gridControlManager,int iRow,int iCol, string currentCellValue)
		{
			string currentValue = gridControl.CurrentCell.Renderer.ControlText;

            if (currentCellValue != null)
                currentValue = currentCellValue;

            currentValue = Utilities.ConvertCultureSpecficToPlainString(currentValue, _lrrApp);

			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.SortManagerList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the row data
			IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
			string rowCellID = rowData.ID;
			//Get the column data
			ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
			Type objectType = colData.ObjectType;
			string modeID   = colData.ModeID;
			string colCellID = colData.ID;
			
			//Get the Property Value Object
			object objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);
			if(objPropValue != null)
			{
				IPropertyValue iPropValue = objPropValue as IPropertyValue;
				string uomID = iPropValue.GetUOMID();
				string localeID = LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID;
				string uiUomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localeID,rowData.UOMTypeId);    
				//PropertyValue Validation
				/*if(currentValue != null && currentValue.Trim() != "")
				{
					if(!inputOutputValidation(iPropValue,currentValue))					
					{
						return false;
					}
				}*/

				string newUOM = rowData.UIUOMId;
				string newpropValue = iPropValue.GetPropertyValueForUOM(newUOM,uomID,currentValue);
				
				if(iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
				{
					bool prevOverrideState = iPropValue.GetOverrideState();
					iPropValue.SetOverrideState(newpropValue);
					bool currOverrideState = iPropValue.GetOverrideState();

					//Measurement Mode
					if(prevOverrideState != currOverrideState && modeID == "1")
					{
						iPropValue.SetTrustedState("-1");
						_lrrApp.SwapBindings(objPropValue,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}

					//Other Modes
					if(prevOverrideState != currOverrideState && modeID != "1")
					{
						iPropValue.SetTrustedState("-1");
						_lrrApp.RemoveCascadedBindings(objPropValue,objectType,rowCellID,colCellID,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}
					
					iPropValue.SetPropertyValue(newpropValue);
					//Set the Cell tool Tip
					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],newUOM);
					
					//Get the Property Object
					object objProp = LRRModel.GetPropertyObject(objectType,rowCellID,colCellID);
					if(objProp != null)
					{
						//Populate Overrides Collection	
						IProperty iProp = objProp as IProperty;
						string propID = iProp.GetID();
                        PopulateOverrides(iPropValue, propID, colCellID, modeID, uiUomID);
					}
				}
			}
			return true;
		}

		/// <summary>
		/// Apply this method when deletion apply on property values.
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>

		private bool cascadeChangesToPropValues(GridControl gridControl,GridControlManager gridControlManager,int iRow,int iCol)
		{
			string currentValue = "";
			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.SortManagerList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;

			//Get the row data
			IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
			string rowCellID = rowData.ID;
			//			string uomID     = rowData.UOMID;

			//Get the column data
			ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
			Type objectType = colData.ObjectType;
			string modeID   = colData.ModeID;
			string colCellID = colData.ID;

			//Get the Property Value Object
			object objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);
			if(objPropValue != null)
			{
				IPropertyValue iPropValue = objPropValue as IPropertyValue;
				string uomID     = iPropValue.GetUOMID();
				string newUOM = rowData.UIUOMId;
				string newpropValue = iPropValue.GetPropertyValueForUOM(newUOM,uomID,currentValue);

				if(iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
				{
					bool prevOverrideState = iPropValue.GetOverrideState();
					iPropValue.SetOverrideState(newpropValue);
					bool currOverrideState = iPropValue.GetOverrideState();

					//Measurement Mode
					if(prevOverrideState != currOverrideState && modeID == "1")
					{
						iPropValue.SetTrustedState("-1");
						_lrrApp.SwapBindings(objPropValue,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}

					//Other Modes
					if(prevOverrideState != currOverrideState && modeID != "1")
					{
						iPropValue.SetTrustedState("-1");
						_lrrApp.RemoveCascadedBindings(objPropValue,objectType,rowCellID,colCellID,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}
					
					iPropValue.SetPropertyValue(newpropValue);
					//Set the Cell tool Tip
					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],newUOM);
					//Populate Overrides Collection
					object objProp = LRRModel.GetPropertyObject(objectType,rowCellID,colCellID);
					if(objProp != null)
					{
						IProperty iProp = objProp as IProperty;
						string propID = iProp.GetID();
						PopulateOverrides(iPropValue,propID,colCellID,modeID,uomID);
					}

				}
				//				SetColor(iRowIndex,iColIndex);
			}
			return true;
		}

		/// <summary>
		/// This method implements Input/Output Logics
		/// </summary>
		/// <param name="lrrSiteCompPropVal"></param>
		/// <param name="newVal"></param>
		private bool inputOutputValidation(IPropertyValue iPropValue,string newVal)
		{
			if(iPropValue != null)
			{
				if(iPropValue.GetDataType() != null && iPropValue.GetDataType().Trim() != "")
				{
					ErrorInfo errInfo = Validator.validateDataType(iPropValue.GetDataType(),newVal);
					if(errInfo.ErrorStatus)
					{
						MessageBox.Show(errInfo.ErrorMsg,errInfo.ErrorSource,MessageBoxButtons.OK,MessageBoxIcon.Warning);
						return false;
					}
				}
			}
			return true;
		}
		
		/// <summary>
		/// Populates Overrides Screen
		/// </summary>
		/// <param name="lrrSiteCompPropValue"></param>
		private void PopulateOverrides(IPropertyValue iPropValue,string propertyID,string compID,string modeID, string sNewUOM)
		{
            if(iPropValue.GetType() == typeof(LRRAggregatePropertyValue))
            {
                LRRAggregateOverride lrrModelOverride = LRRModel.ChildModel.AggregateOverrides.GetOverrideObjectByAggPropertyAndModeID(propertyID,modeID);
			    if(iPropValue.GetOverrideState())
			    {
				    if(lrrModelOverride != null)
				    {
					    sNewUOM = lrrModelOverride.UOM;
                        LRRModel.ChildModel.AggregateOverrides.Remove(lrrModelOverride);
				    }
                    LRRAggregateOverride lrraggOverride = new LRRAggregateOverride();
                    lrraggOverride.ModeID = modeID;
                    lrraggOverride.AggregatePropertyID = propertyID;
                    lrraggOverride.AggregateID = compID;

				    //Pre/Post Override Values
				    string modelUOM = iPropValue.GetUOMID();
                    lrraggOverride.PreOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualOverridedValue());
                    lrraggOverride.PostOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualPropertyValue());

                    lrraggOverride.UOM = sNewUOM;
				    //Assign Confidence Level
				    string dataConfidenceLevelID = null;
				    if(modeID == "1" || modeID == "2")
				    {
					    dataConfidenceLevelID = "3";
				    }
				    else
				    {
					    dataConfidenceLevelID = "1";
				    }
                    lrraggOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                    LRRModel.ChildModel.AggregateOverrides.Add(lrraggOverride);

				    iPropValue.DataConfidenceValueChanged+=new EventHandler(iPropValue_DataConfidenceValueChanged);
    				
				    iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
			    }
			    else
			    {
				    if(lrrModelOverride != null)
				    {
					    iPropValue.DataConfidenceValueChanged-=new EventHandler(iPropValue_DataConfidenceValueChanged);
					    LRRModel.ChildModel.AggregateOverrides.Remove(lrrModelOverride);
				    }
			    }
			    //Update Overrides
			    _lrrApp.UpdateAggregateOverridePoints();
            }
						 
            else
            {
			    LRROverride lrrModelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(propertyID,modeID);
			    if(iPropValue.GetOverrideState())
			    {
				    if(lrrModelOverride != null)
				    {
					    sNewUOM = lrrModelOverride.UOM;
					    LRRModel.ChildModel.Overrides.Remove(lrrModelOverride);
				    }
				    LRROverride lrrOverride = new LRROverride();
				    lrrOverride.ModeID = modeID;
				    lrrOverride.SiteComponentPropertyID = propertyID;
				    lrrOverride.SiteComponentID = compID;

				    //Pre/Post Override Values
				    string modelUOM = iPropValue.GetUOMID();
				    lrrOverride.PreOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM,sNewUOM,iPropValue.GetActualOverridedValue());
				    lrrOverride.PostOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM,sNewUOM,iPropValue.GetActualPropertyValue());

				    lrrOverride.UOM = sNewUOM;
				    //Assign Confidence Level
				    string dataConfidenceLevelID = null;
				    if(modeID == "1" || modeID == "2")
				    {
					    dataConfidenceLevelID = "3";
				    }
				    else
				    {
					    dataConfidenceLevelID = "1";
				    }
				    lrrOverride.DataConfidenceLevelID = dataConfidenceLevelID;
				    LRRModel.ChildModel.Overrides.Add(lrrOverride);

				    iPropValue.DataConfidenceValueChanged+=new EventHandler(iPropValue_DataConfidenceValueChanged);
    				
				    iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
			    }
			    else
			    {
				    if(lrrModelOverride != null)
				    {
					    iPropValue.DataConfidenceValueChanged-=new EventHandler(iPropValue_DataConfidenceValueChanged);
					    LRRModel.ChildModel.Overrides.Remove(lrrModelOverride);
				    }
			    }
			    //Update Overrides
			    _lrrApp.UpdateOverridePoints();
            }
		}
		
		/// <summary>
		/// Gets the data for a given cell
		/// </summary>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <param name="colData"></param>
		/// <param name="rowData"></param>
		private void GetObjectData(GridControlManager gridControlManager,int iRow,int iCol,out ColHeaderCell colData,out IOSortManager rowData)
		{
			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.SortManagerList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			//Get the row data
			rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
			//Get the column data
			colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
		}
		
		/// <summary>
		/// Gets the list of grids
		/// </summary>
		/// <returns></returns>
		private ArrayList GetGridList()
		{
			ArrayList gridList = new ArrayList();
			IEnumerator iEnum = GridManagerHash.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
				GridControl gridControl = dEntry.Key as GridControl;
				gridList.Add(gridControl);
			}
			return gridList;
		}
		
		/// <summary>
		/// Sets State of the menu items
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="contMenu"></param>
		private void ProcessMenuItems(IProperty iProp,IPropertyValue iPropValue,ContextMenu contMenu)
		{
			bool overridedState = iPropValue.GetOverrideState();
			ErrorCode errorCode   = iPropValue.GetErrorCode();
			for(int i=0;i < contMenu.MenuItems.Count;i++)
			{
				MenuItem miItem = contMenu.MenuItems[i];
				switch(miItem.Index)
				{
						//Trend Values Menu
					case 0:
						if(iProp != null)
							miItem.Enabled = iProp.GetTrended();
						else
							miItem.Enabled = false;
						break;
					//Override Values Menu
					case 1:
						miItem.Enabled = overridedState;
						break;
					//Uncertain Input Menu
					case 2:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.UNCERTAIN)) == (ErrorCode.UNCERTAIN) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Trusted Input Menu
					case 3:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.TRUSTED)) == (ErrorCode.TRUSTED) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Temp Trusted Input Menu
					case 4:
						miItem.Enabled = !overridedState;
						if((errorCode & (ErrorCode.TEMPTRUSTED)) == (ErrorCode.TEMPTRUSTED) && !overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Uncertain Override Menu
					case 5://3:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.UNCERTAIN)) == (ErrorCode.UNCERTAIN) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Trusted Override Menu
					case 6://4:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.TRUSTED)) == (ErrorCode.TRUSTED) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
					//Temp Trusted Override Menu
					case 7://5:
						miItem.Enabled = overridedState;
						if((errorCode & (ErrorCode.TEMPTRUSTED)) == (ErrorCode.TEMPTRUSTED) && overridedState)
						{
							miItem.Checked = true;
						}
						break;
				}
			}
		}
		
		/// <summary>
		/// Method sets the fore/back/border colours for the grid cell
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="gsInfo"></param>
		private void SetColor(IPropertyValue iPropValue,GridStyleInfo gsInfo)
		{
			gsInfo.BackColor = iPropValue.GetBackColor();
			gsInfo.TextColor = iPropValue.GetForeColor();

			//Set Border Color and Border Thickness
			Color borderColor = iPropValue.GetBorderColor();
			int borderThickness = iPropValue.GetBorderThickness();
			if(borderThickness == 2)
            {	//gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid, borderColor,GridBorderWeight.Medium);
                GridBorder grdBorder = new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                gsInfo.Borders.Top = grdBorder;// new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
                gsInfo.Borders.Left = grdBorder;// new GridBorder(GridBorderStyle.Solid, borderColor, GridBorderWeight.Medium);
            }
			else
			{
				gsInfo.Borders.ResetAll();
				//gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid,IOSchemes.GRID_PROPVALUE_LINE_COLOR,GridBorderWeight.ExtraThin);
			}
			gsInfo.Font.Underline = iPropValue.GetUnderLine();
		}
		/// <summary>
		/// Sets the Notification for IsRecorded values
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="gsInfo"></param>
		/// 
		private void SetRecordedNotification(IPropertyValue iPropValue,GridStyleInfo gsInfo)
		{
			_style = new GridExcelTipStyleProperties(gsInfo);
            if(iPropValue.GetIsRecorded() && iPropValue.GetIsTrended())
            {
                _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_RECORDED_AND_TRENDED");
            }
            else if(iPropValue.GetIsRecorded())
			{
                _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_RECORDED");
			}
			else if(iPropValue.GetIsTrended())
			{
                _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_TRENDED");
			}
			else
			{
				_style.ResetExcelTipText();
			}
		}
		
		/// <summary>
		/// Sets the Cell Tool Tip Text
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="gsInfo"></param>
		/// <param name="uiUOMID"></param>
		private void SetCellToolTipText(IPropertyValue iPropValue,GridStyleInfo gsInfo,string uiUOMID)
		{
            /*
            gsInfo.CellTipText = null;
			string uomID = iPropValue.GetUOMID();
			string actualPropValue = iPropValue.GetActualPropertyValue();

			string siteCompPropID = iPropValue.GetPropertyID();
			LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
			LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
			string equipmentTag = siteComp.EquipmentTag;
            string histTag = null;
            LRRHistPropCompPropMapping histComProp = LRRModel.HistPropMaps.GetHistMapObjFromValID(siteCompPropID);
            histTag = histComProp == null ? null : histComProp.HistTag;

			if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
			{
				actualPropValue = iPropValue.GetPropertyValueForUOM(uomID, uiUOMID, actualPropValue);
			}

            string strToolTip = actualPropValue;
            gsInfo.CellTipText = strToolTip;

			if (iPropValue.GetOverrideState())
			{
				string preOverrideValue = iPropValue.GetActualOverridedValue();
				if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
				{
					preOverrideValue = iPropValue.GetPropertyValueForUOM(uomID, uiUOMID, preOverrideValue);
				}

				string lrrDataType = iPropValue.GetDataType();
                //if (lrrDataType == null || lrrDataType.Trim() == "")
                //{
                //    actualPropValue = Utilities.FormatNumber(actualPropValue, _lrrApp);
                //    preOverrideValue = Utilities.FormatNumber(preOverrideValue, _lrrApp);
                //}
                strToolTip = LRRHeaderText.GetHeaderString("SID_PREOVERRIDEVALUE") + ":" + preOverrideValue + "\n" +
                        LRRHeaderText.GetHeaderString("SID_POSTOVERRIDEVALUE") + ":" + actualPropValue;
                if (string.IsNullOrEmpty(histTag))
				{
                    gsInfo.CellTipText = strToolTip;

				}
				else
				{
                    if (iPropValue.GetModeID() == "1")
                    {
                        gsInfo.CellTipText = strToolTip + "\n" +
                        LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
                    }
                    else
                    {
                        gsInfo.CellTipText = strToolTip;
                    }

				}
                //gsInfo.CellTipText = LRRHeaderText.GetHeaderString("SID_PREOVERRIDEVALUE") + ":" + preOverrideValue + "\n" +
                //    LRRHeaderText.GetHeaderString("SID_POSTOVERRIDEVALUE") + ":" + actualPropValue + "\n" +
                //    LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
			}
			else
			{
				string lrrDataType = iPropValue.GetDataType();
                //if (lrrDataType == null || lrrDataType.Trim() == "")
                //{
                //    actualPropValue = Utilities.FormatNumber(actualPropValue, _lrrApp);
                //}
				//gsInfo.CellTipText = actualPropValue;
                if (!string.IsNullOrEmpty(histTag) && iPropValue.GetModeID() == "1")
                    gsInfo.CellTipText = strToolTip + "\n" + LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
			}
              */
            gsInfo.CellTipText = null;
            string uomID = iPropValue.GetUOMID();
            string actualPropValue = iPropValue.GetActualPropertyValue();
            //string equipmentTag = null;
            string histTag = null;

            if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
            {
                actualPropValue = ConvertUOM(uomID, uiUOMID, actualPropValue, iPropValue.GetDataType());

            }

            string strToolTip = actualPropValue;
            gsInfo.CellTipText = strToolTip;

            if (iPropValue.GetType().Name == "LRRSiteGroupPropertyQualifierValue")
            {
                string siteCompPropID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).SiteComponentPropID;
                string histTagID = ((Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue)(iPropValue)).HistTagID;
                LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
                LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
                //equipmentTag = siteComp.EquipmentTag;
                histTag = histTagID == null || histTagID == "" ? null : LRRModel.HistDatas.GetHistDataFromHistTagUID(histTagID).HistTag;
            }
            else if (iPropValue.GetType().Name == "LRRSiteComponentPropertyValue")
            {
                string siteCompPropID = ((Symcon.LightRidge.BusinessObjects.LRRSiteComponentPropertyValue)(iPropValue)).SiteComponentPropertyID;
                LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
                LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
                //equipmentTag = siteComp.EquipmentTag;
                LRRHistPropCompPropMapping histComProp = LRRModel.HistPropMaps.GetHistMapObjFromValID(siteCompPropID);
                histTag = histComProp == null ? null : histComProp.HistTag;
            }
            else if (iPropValue.GetType().Name == "LRRAggregatePropertyValue")
            {
                histTag = null;
            }

            if (iPropValue.GetOverrideState())
            {
                string preOverrideValue = iPropValue.GetActualOverridedValue();
                if (uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
                {
                    preOverrideValue = ConvertUOM(uomID, uiUOMID, preOverrideValue, iPropValue.GetDataType());
                }

                string lrrDataType = iPropValue.GetDataType();
                //if (lrrDataType == null || lrrDataType.Trim() == "")
                //{
                //    actualPropValue = Utilities.FormatNumber(actualPropValue, _lrrApp);
                //    preOverrideValue = Utilities.FormatNumber(preOverrideValue, _lrrApp);
                //}
                strToolTip = "Pre-Override Value :" + preOverrideValue + "\n" +
              "Post-Override Value :" + actualPropValue;

                if (string.IsNullOrEmpty(histTag))
                {
                    gsInfo.CellTipText = strToolTip;
                }

                else
                {
                    if (iPropValue.GetModeID() == "1")
                    {
                        gsInfo.CellTipText = strToolTip + "\n" +
                                LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
                    }
                    else
                        gsInfo.CellTipText = strToolTip;

                }
            }
            else
            {
                string lrrDataType = iPropValue.GetDataType();
                if (lrrDataType == null || lrrDataType.Trim() == "")
                {
                    actualPropValue = Utilities.FormatNumber(actualPropValue, _lrrApp);
                }
                if (!string.IsNullOrEmpty(histTag) && iPropValue.GetModeID() == "1")

                    gsInfo.CellTipText = strToolTip + "\n" + LRRHeaderText.GetHeaderString("SID_HISTTAG") + ":" + histTag;
            }
		}

        private string ConvertUOM(string oldUOMValue, string newUOMValue, string propVal, string lrrDataTypeID)
        {
            string newVal = "";
            if (propVal != null && propVal != "")
            {
                if (oldUOMValue == newUOMValue)
                {
                    newVal = propVal;
                }
                else
                {
                    if ((oldUOMValue != null && oldUOMValue != "") &&
                        (newUOMValue != null && newUOMValue != ""))
                    {
                        newVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(oldUOMValue, newUOMValue, propVal, lrrDataTypeID);
                    }
                }
            }
            return newVal;
        }

        /// <summary>
        /// This method initializes resets the display flag
        /// </summary>
        /// <param name="gridControlInstance"></param>
        /// <param name="gridControlManager"></param>
        private void ResetDisplayFlag(GridControl gridControlInstance, GridControlManager gridControlManager,bool displayFlag)
        {
            //Get the Modes Count
            int iModesCount = LRRModel.ProjectModes.Count;

            //Get the Row Data from Grid Data Manager
            ArrayList rowSortList = gridControlManager.SortManagerList;

            //Get the Col Data from Grid Data Manager
            ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;

            //Get the Row Header from Grid Data Manager
            Hashtable RowHeader = gridControlManager.RowHeader;

            //Get the Col Header from Grid Data Manager
            Hashtable ColHeader = gridControlManager.ColHeader;

            //Loop through the grid matrix,put property values,add bindings
            for (int iRow = DATA_ROW_START; iRow < RowHeader.Count + DATA_ROW_START - 1; iRow++)
            {
                for (int iCol = DATA_COL_START; iCol < (ColHeader.Count - DATA_COL_START + 1) * iModesCount + DATA_COL_START; iCol++)
                {
                    try
                    {
                        //Get the row data
                        IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
                        string rowCellID = rowData.ID;

                        //Get the column data
                        ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
                        Type objectType = colData.ObjectType;
                        string modeID = colData.ModeID;
                        string colCellID = colData.ID;

                        object objPropValue = null;

                        //Get the Property Value Object
                        objPropValue = LRRModel.GetPropertyValueObject(objectType, modeID, rowCellID, colCellID);

                        if (objPropValue != null)
                        {
                            IPropertyValue iPropValue = objPropValue as IPropertyValue;
                            _style = new GridExcelTipStyleProperties(gridControlInstance[iRow, iCol]);

                            if (displayFlag)
                            {

                                if (iPropValue.GetIsRecorded() && iPropValue.GetIsTrended())
                                {
                                    _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_RECORDED_AND_TRENDED");
                                }
                                else if (iPropValue.GetIsTrended())
                                {
                                    _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_TRENDED");
                                }
                                else if (iPropValue.GetIsRecorded())
                                {
                                    _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_RECORDED");
                                }
                                else
                                {
                                    _style.ResetExcelTipText();
                                }
                            }
                            else
                            {
                                _style.ResetExcelTipText();
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.LogMessage(typeof(GridSimpleViewController), "Exception in display recording/trending flag:- " + ex.Message, Logger.Level.Debug);
                    }
                }
            }
        }
		#endregion

		#region Fields
		//Instance of App
		private LRRApp _lrrApp;

		//ArrayList for storing Panels
		private ArrayList _pnlCollection = new ArrayList();

		//Hash for stroing gridcontrol-gridManager key value pairs
		private Hashtable _gridManagerHash = new Hashtable();

		//For saving Queries
		private SearchCriteria _searchCriteria = new SearchCriteria();

		//For Unselected Records
		private ArrayList	  _unSelectedRecords = new ArrayList();

		//Grid Peristance
		private GridPersistanceUtil _gridPersistance = new GridPersistanceUtil();

		//For IsTrack Notification 
		private CommentMouseController _commentMouseControler;
		private GridExcelTipStyleProperties _style; 

		//For UserSpecific UI UOM
		private LRRSearchCriteriaUserSpecificUIUOMs _searchCriteriaUserSpecficUIUOMs = new LRRSearchCriteriaUserSpecificUIUOMs();

        //// For Excel Tip
        //private ExcelTip.CommentMouseController commentMouseController1;

        //Recording/Trending Display Flag
        private bool _recTrendDisplayFlag = true;

        private IOScreen.GridPersistanceUtil _gutil = null;

		#endregion

		#region Constants

		private const int PROPERTIES_COL = 1;
		private const int MODES_ROW = 2;
		private const int COMPONENT_ROW = 1;
		private const int HEADER_ROW = 0;
		private const int COL_SPAN = 2;
		private const int UOM_COL = 2;
		private const int UOM_ROW = 2;
		private const int DATA_COL_START = 3;
		private const int DATA_ROW_START = 3;

		private const int FREEZE_ROW_START = 0;
		private const int FREEZE_ROW_END = 1;
		private const int FREEZE_COL_START = 0;
		private const int FREEZE_COL_END= 1;

		#endregion
		
		#region Properties
		private Hashtable GridManagerHash
		{
			get{return _gridManagerHash;}
			set
			{
				_gridManagerHash = value;
			}
		}
		private ArrayList PanelCollection
		{
			get{return _pnlCollection;}
			set
			{
				_pnlCollection = value;
			}
		}
		private ArrayList UnSelectedRecords
		{
			get{return _unSelectedRecords;}
			set
			{
				_unSelectedRecords = value;
			}
		}
		private GridPersistanceUtil GridPersistance
		{
			get{return _gridPersistance;}
			set
			{
				_gridPersistance = value;
			}
		}
		#endregion

		#region EventHandlers
		private void gridControlInstance_CurrentCellCloseDropDown(object sender, Syncfusion.Windows.Forms.PopupClosedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			gridControl.CurrentCell.ConfirmChanges();
			gridControl.Refresh();
		}

		private void gridControlInstance_CurrentCellValidating(object sender, System.ComponentModel.CancelEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

			//Get the Grid Data
			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			if(gridControlManager == null)
				return;
			//Check for UOM
			if(iRow > UOM_ROW && iCol == UOM_COL)
			{
				//Change the Property Values according to UOM
				changePropertyValueForUOM(gridControl,gridControlManager,iRow,iCol);
			}

			//Check for Property Value 
			if(iRow >= DATA_ROW_START && iCol >= DATA_COL_START)
			{
				bool result = changePropertyValue(gridControl,gridControlManager,iRow,iCol);
				if(!result)
				{
					gridControl.CurrentCell.ResetCurrentCellWithoutDeactivate();
					gridControl.CurrentCell.Refresh();
					e.Cancel = true;
				}
			}
		}

		private void gridControlInstance_CurrentCellStartEditing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] == null)
			{
				e.Cancel = true;
				return;
			}
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

			if(iRow < DATA_ROW_START || iCol < DATA_COL_START)
				return;

			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			//Get the Row Data from Grid Data Manager
			ArrayList     rowSortList = gridControlManager.SortManagerList;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			//Get the row data
			IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
			string rowCellID = rowData.ID;
			//Get the column data
			ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
			Type objectType = colData.ObjectType;
			string modeID   = colData.ModeID;
			string colCellID = colData.ID;

			//Get the Property Value Object
			object objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);
			if(objPropValue != null)
			{
				IPropertyValue iPropValue = objPropValue as IPropertyValue;
				if(!iPropValue.GetEditableState())
					e.Cancel = true;
			}
			else
			{
				e.Cancel = true;
			}
				
		}

		private void gridControlInstance_CellClick(object sender, GridCellClickEventArgs e)
		{
			if(e.MouseEventArgs.Button == MouseButtons.Left
				&& e.RowIndex == 1 && (e.ColIndex == 1 || e.ColIndex == 2))
			{
				GridControl gridControl = sender as GridControl;
				gridControl.ResetVolatileData();
				gridControl.BeginUpdate();

				Cursor.Current = Cursors.WaitCursor;
				Cursor.Show();
				if(GridManagerHash[gridControl] != null)
				{
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
					SortCol(e.ColIndex,gridControl,gridControlManager);
					InitializePropertyValues(gridControl,gridControlManager);

					//Hide Rows
					ArrayList rowList = gridControlManager.SortManagerList;
					gridControl.RowHiddenEntries.Clear();
					for(int i=0;i < rowList.Count;i++)
					{
						IOSortManager sortManager = rowList[i] as IOSortManager;
						if(!sortManager.IsVisible)
						{
							gridControl.RowHiddenEntries.Add(new GridRowHidden(i + DATA_ROW_START));
						}
					}
				}
				gridControl.EndUpdate();
				Cursor.Hide();
			}
		}

		
		private void bindingManager_BindingCalled(object source, object target)
		{
			object src = source;
			object trg = target;
			GridStyleInfo gsInfo = target as GridStyleInfo;
			GridControl gridControl = gsInfo.GetActiveGridView() as GridControl;

			int iRow = gsInfo.CellIdentity.RowIndex;
			int iCol = gsInfo.CellIdentity.ColIndex;

			//Get the Grid Data
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
				
				//Get the row data
				IOSortManager rowData = new IOSortManager();
				//Get the column data
				ColHeaderCell colData = new ColHeaderCell();

				GetObjectData(gridControlManager,iRow,iCol,out colData,out rowData);
				object objProp = LRRModel.GetPropertyObject(colData.ObjectType,rowData.ID,colData.ID);
				
				//Get the row data
				string rowCellID = rowData.ID;
				string uiUOMID   = rowData.UIUOMId;
//				string uomID     = rowData.UOMID;

				//Get the column data
				Type objectType = colData.ObjectType;
				string modeID   = colData.ModeID;
				string colCellID = colData.ID;

				//Get the Property Value Object
				object objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);

				//Convert and put Property Values in grid cells
				if(objPropValue != null)
				{
					IPropertyValue iPropValue = objPropValue as IPropertyValue;
					string uomID = iPropValue.GetUOMID();
					//Convert property value according to UIUOM 
					string propertyValue = iPropValue.GetPropertyValue();
					if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
					{
						propertyValue = iPropValue.GetPropertyValueForUOM(uomID,uiUOMID,propertyValue);
					}
//					gridControl.Model[iRow,iCol].Text = propertyValue;
                    string lrrDataType = iPropValue.GetDataType();
                    if (lrrDataType == null || lrrDataType.Trim() == "")
                    {
                        propertyValue = Utilities.FormatNumber(propertyValue, _lrrApp);
                    }
                    gsInfo.Text = propertyValue;
					SetCellToolTipText(iPropValue,gsInfo,uiUOMID);
				
					//Set the Color
					SetColor(iPropValue,gridControl.Model[iRow,iCol]);
				}
			}
		}

		private void gridControlInstance_RowsMoving(object sender, GridRangeMovingEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(e.Target < DATA_ROW_START || e.From < DATA_ROW_START)
			{
				e.Cancel = true;
				return;
			}
            //for(int i=1;i<=gridControl.RowCount;i++)
            //{
            //    if(gridControl[0,i].Text.Trim() != "")
            //    {
					//GridPersistanceUtil._arrGridRowMove[0].Add(gridControl[0,1].Text);
                    //break;
            //    }
            //}
			_gutil._arrGridRowMove[0].Add(e.From);
            _gutil._arrGridRowMove[1].Add(e.Count);
            _gutil._arrGridRowMove[2].Add(e.Target);
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
				IOScreenUtilities.RowMovement(e.From,e.Target,e.Count,gridControlManager.SortManagerList,DATA_ROW_START);
			}
		}

		private void gridControlInstance_RowsMoved(object sender, GridRangeMovedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
				InitializePropertyValues(gridControl,gridControlManager);
				if(gridControl.CurrentCell.IsEditing)
					gridControl.CurrentCell.EndEdit();
				//turn off the sort arrow
				for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				{
					if(gridControl[1, col1].CellType == "ColumnHeaderCell")
					{
						gridControl[1, col1].CellType = "Header";
						break;
					}
				}
			}
		}
		
		private void contMenu_Popup(object sender, EventArgs e)
		{
			ContextMenu contMenu = sender as ContextMenu;
			contMenu.MenuItems.Clear();

			GridControl gridControl = contMenu.SourceControl as GridControl;
			if(gridControl == null)
				return;
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			ColHeaderCell colData = new ColHeaderCell();
			IOSortManager rowData = new IOSortManager();

			//Get the GridData
			if(GridManagerHash[gridControl] == null)
			   return;
			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			
			//Record Properties Menu
			//if(iCol == PROPERTIES_COL && iRow > DATA_ROW_START - 1)
			if(iRow == COMPONENT_ROW && iCol > PROPERTIES_COL && iCol > UOM_COL)
			{
				MenuItem miItem = new MenuItem(LRRHeaderText.GetHeaderString("SID_RECORDE_PROP_VALUE"),new EventHandler(recordValue));
//				GetObjectData(gridControlManager,iRow + DATA_ROW_START - 1,iCol,out colData,out rowData);
//				miItem.Checked = rowData.IsRecorded;
				contMenu.MenuItems.Add(miItem);

				MenuItem miItemTrend = new MenuItem(LRRHeaderText.GetHeaderString("SID_TREND_PROP_VALUE"),new EventHandler(trendValue));
				contMenu.MenuItems.Add(miItemTrend);
			}

			//Property Values Overrides Menu
			if(iRow >= DATA_ROW_START && iCol >= DATA_COL_START)
			{
				//Get the GridData
				GetObjectData(gridControlManager,iRow,iCol,out colData,out rowData);
				//Get the Property Value Object
				object objPropValue = LRRModel.GetPropertyValueObject(colData.ObjectType,colData.ModeID,rowData.ID,colData.ID);

				if(objPropValue != null)
				{
					IProperty iProp = LRRModel.GetPropertyObject(colData.ObjectType,rowData.ID,colData.ID) as IProperty;
					IPropertyValue iPropValue = objPropValue as IPropertyValue;

//					if(iPropValue.GetPropertyValue() != null && iPropValue.GetPropertyValue() != "")
//					{
						MenuItem miItemTrend = new MenuItem(LRRHeaderText.GetHeaderString("SID_VIEW_TREND"));
						//miItemTrend.IsParent = true;
						foreach(string schemeName in UserControls.UserControlTrendOptions.HtSchemes.Keys)
						{
							MenuItem miItemTrendChild = new MenuItem(schemeName,new EventHandler(viewTrend));
							miItemTrend.MenuItems.Add(miItemTrendChild);
						}
						miItemTrend.Index = 0;//6;
						contMenu.MenuItems.Add(miItemTrend);
//					}

					if(!iPropValue.GetEditableState())
					{
						ProcessMenuItems(iProp,iPropValue,contMenu);
						return;
					}

					//Override Values Menu
					MenuItem miItem = new MenuItem(LRRHeaderText.GetHeaderString("SID_OVERRIDE_VAL_OFF"),new EventHandler(overrideValue));
					miItem.Index = 1;
					contMenu.MenuItems.Add(miItem);

					//Uncertain Input Menu
					MenuItem miItemUncertainInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_UNCERTAIN_INPUT"),new EventHandler(overrideValue));
					miItemUncertainInput.Index = 2;
					contMenu.MenuItems.Add(miItemUncertainInput);

					//Trusted Input Menu
					MenuItem miItemTrustedInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_TRUSTED_INPUT"),new EventHandler(overrideValue));
					miItemTrustedInput.Index = 3;
					contMenu.MenuItems.Add(miItemTrustedInput);

					//TempTrusted Input Menu
					MenuItem miItemTempTrustedInput = new MenuItem(LRRHeaderText.GetHeaderString("SID_TEMP_TRUSTER_INPUT"),new EventHandler(overrideValue));
					miItemTempTrustedInput.Index = 4;
					contMenu.MenuItems.Add(miItemTempTrustedInput);

					//Uncertain Override Menu
					MenuItem miItemUncertainOverride = new MenuItem(LRRHeaderText.GetHeaderString("SID_UNCERTAIN_OVERRIDE"),new EventHandler(overrideValue));
					miItemUncertainOverride.Index = 5;//3;
					contMenu.MenuItems.Add(miItemUncertainOverride);

					//Trusted Override Menu
					MenuItem miItemTrustedOverride = new MenuItem(LRRHeaderText.GetHeaderString("SID_TRUSTED_OVERRIDE"),new EventHandler(overrideValue));
					miItemTrustedOverride.Index = 6;//4;
					contMenu.MenuItems.Add(miItemTrustedOverride);

					//TempTrusted Override Menu
					MenuItem miItemTempTrusted = new MenuItem(LRRHeaderText.GetHeaderString("SID_TEMP_TRUSTER_OVERRIDE"),new EventHandler(overrideValue));
					miItemTempTrusted.Index = 6;//5;
					contMenu.MenuItems.Add(miItemTempTrusted);

					ProcessMenuItems(iProp,iPropValue,contMenu);
				}
			}
		}

		public void viewTrend(object sender, EventArgs e)
		{
			MenuItem miItemChild = sender as MenuItem;
			string trendScheme = miItemChild.Text;
			MenuItem miItem = miItemChild.Parent as MenuItem;
			ContextMenu contMenu = miItem.GetContextMenu();
			GridControl gridControl = contMenu.SourceControl as GridControl;

			if(GridManagerHash[gridControl] == null)
				return;

			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			ColHeaderCell colData = new ColHeaderCell();
			IOSortManager rowData = new IOSortManager();

			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			GetObjectData(gridControlManager,iRow,iCol,out colData,out rowData);

			//Get the Property
			//IProperty iProp = LRRModel.GetPropertyValueObject(colData.ObjectType,colData.ModeID,rowData.ID,colData.ID);
			IProperty iProp = LRRModel.GetPropertyObject(colData.ObjectType,rowData.ID,colData.ID) as IProperty;

			if(iProp != null)
			{
				_lrrApp.LRRUIManagerObj.ShowTrend(trendScheme,iProp,colData.ModeID,rowData.UIUOMId);
			}
		}

		public void overrideValue(object sender, EventArgs e)
		{
			MenuItem miItem = sender as MenuItem;
			ContextMenu contMenu = miItem.GetContextMenu();
			GridControl gridControl = contMenu.SourceControl as GridControl;

			if(GridManagerHash[gridControl] == null)
				return;

			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			ColHeaderCell colData = new ColHeaderCell();
			IOSortManager rowData = new IOSortManager();

			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			GetObjectData(gridControlManager,iRow,iCol,out colData,out rowData);
			//Get the Property Value Object
			object objPropValue = LRRModel.GetPropertyValueObject(colData.ObjectType,colData.ModeID,rowData.ID,colData.ID);

			if(objPropValue != null)
			{
				IPropertyValue iPropValue =  objPropValue as IPropertyValue;
				IProperty iProp = null;

				//Check for Menu item Index
				switch(miItem.Index)
				{
					//Override Value On/Off
					case 1:
						iPropValue.SetOverrideState(false);
						if(colData.ModeID == "1")
						{
							_lrrApp.SwapBindings(objPropValue,false);
							iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
						}
						else
						{
							_lrrApp.RemoveCascadedBindings(objPropValue,colData.ObjectType,rowData.ID,colData.ID,false);
							iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
						}

						//Populate Overrides Collection
						object objProp = LRRModel.GetPropertyObject(colData.ObjectType,rowData.ID,colData.ID);
						if(objProp != null)
						{
							iProp = objProp as IProperty;
							string propID = iProp.GetID();
							PopulateOverrides(iPropValue,propID,colData.ID,colData.ModeID,iPropValue.GetUOMID());//rowData.UOMID);
						}
						
						//Set the Cell tool Tip
						SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],rowData.UIUOMId);
                        if (gridControl.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.IOScreen.InputOutputViews))
                        {
                            InputOutputViews.BPropertyValChanged = true;
                        }
                        else if (gridControl.Parent.FindForm().GetType() == typeof(Symcon.LightRidge.PEAdvisor.IOScreen.ResultForm))
                        {
                            ResultForm.BIOViewsResultFormChange = true;
                        }
						break;
					//Uncertain Input
					case 2:
						iPropValue.SetTrustedState("0");
						break;
					//Trusted Input
					case 3:
						iPropValue.SetTrustedState("1");
						break;
					//Temp Trusted Input
					case 4:
						iPropValue.SetTrustedState("2");
						break;
					//Uncertain Override
					case 5://3:
						iPropValue.SetDataConfidenceLevelID("3");
						iPropValue.SetTrustedState("-1");
						break;
					//Trusted Override
					case 6://4:
						iPropValue.SetDataConfidenceLevelID("1");
						iPropValue.SetTrustedState("-1");
						break;
					//TempTrusted Override
					case 7://5:
						iPropValue.SetDataConfidenceLevelID("2");
						iPropValue.SetTrustedState("-1");
						break;
				}

				ProcessMenuItems(iProp,iPropValue,contMenu);
				//Set the Color
				SetColor(iPropValue,gridControl.Model[iRow,iCol]);
			}
		}

        public static ResultComponentsWindow rsComponents;
            //= new ResultComponentsWindow();
		bool isRecordEventSet = false;
        public void recordValue(object sender, EventArgs e)
		{
            rsComponents = new ResultComponentsWindow(false);
			MenuItem miItem = sender as MenuItem;
			GridControl gridControl = miItem.Parent.GetContextMenu().SourceControl as GridControl;
			if(GridManagerHash[gridControl] == null)
				return;

			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			LRRSiteComponents lsc = new LRRSiteComponents();
			int colCount = gridControl.ColCount;
			for(int i=DATA_COL_START;i<=colCount;i++)
			{
				ColHeaderCell CompcolData = colHeaderCellList[i - DATA_COL_START] as ColHeaderCell;
				Type ComobjectType = CompcolData.ObjectType;
				string compcolCellID = CompcolData.ID;
				object Compobj = LRRModel.GetComponentObject(ComobjectType,compcolCellID);
				if(Compobj != null)
				{
					if(!lsc.Contains(Compobj) && Compobj.GetType()==typeof(LRRSiteComponent))
					{
						lsc.Add(Compobj);
					}
				}
			}
			ArrayList    rowHeaderCellList = gridControlManager.SortManagerList;
			LRRSiteComponentProperties siteComProps = new LRRSiteComponentProperties();
			ArrayList propertyId = new ArrayList();
			int rowCount = gridControl.RowCount;

			foreach(LRRSiteComponent siteComp in lsc)
			{
				for(int i=DATA_ROW_START;i<=rowCount;i++)
				{
					IOSortManager propRowData = rowHeaderCellList[i-DATA_ROW_START] as IOSortManager;
					Type _propObjectType = propRowData.ObjectType;
					string _propRowCellID = propRowData.ID;
					LRRSiteComponentProperty propObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromMasterPropertyID(siteComp.SiteComponentID,_propRowCellID); 
					if(propObj != null)
					{                                 
						//LRRSiteComponentProperty lscp = propObj as LRRSiteComponentProperty; 
						if(propObj.PropertyID != null)
						{
							if(!propertyId.Contains(propObj.PropertyID))
							{
								propertyId.Add(propObj.PropertyID); 
								siteComProps.Add(propObj);
							}
						}
						else
						{
							siteComProps.Add(propObj);
						}
					}
				}
			}
			//Show the Properties Window
			rsComponents.PropertyCollection = siteComProps;
			rsComponents.SiteComponents = lsc;
			rsComponents.Text = "Recorded Components Window :";
			if(!isRecordEventSet)
			{
				rsComponents.RecordedProperties+=new Symcon.LightRidge.PEAdvisor.IOScreen.ResultComponentsWindow.SelectedPropertiesChanged(rsComponents_RecordedProperties); 
				isRecordEventSet = true;
			}
			rsComponents.BindGrid();
			rsComponents.TopMost = false;
			rsComponents.Owner = _lrrApp.MainFrame;
			rsComponents.Show();
		}
	
		private void iPropValue_DataConfidenceValueChanged(object sender, EventArgs e)
		{
            IPropertyValue iPropValue = sender as IPropertyValue;
            string modeID = iPropValue.GetModeID();
            string propertyID = iPropValue.GetPropertyID();
            if (iPropValue.GetType() == typeof(LRRAggregatePropertyValue))
            {
                LRRAggregateOverride lrrModelOverride = LRRModel.ChildModel.AggregateOverrides.GetOverrideObjectByAggPropertyAndModeID(propertyID, modeID);
                if (lrrModelOverride != null)
                {
                    lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();
                    _lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);
                }
                //Update Overrides
                _lrrApp.UpdateAggregateOverridePoints();
            }
				else if(iPropValue.GetType() == typeof(LRRAggregateGroupedPropertyValue))
				{
					LRRAggGroupOverride lrrModelOverride =LRRModel.ChildModel.GroupAggOverrides.GetOverrideObjectByAggPropertyAndModeID(propertyID, modeID);
					if (lrrModelOverride != null)
                {
                    lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();
                    _lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);
                }
                //Update Overrides
                _lrrApp.UpdataGrpAggOverridePoints();

				}
				else if(iPropValue.GetType() == typeof(LRRSiteComponentGroupPropertyValue))
				{
					LRRGroupOverride  lrrModelOverride =LRRModel.ChildModel.GroupOverrides.GetOverrideObjectByValueID(propertyID);
					if (lrrModelOverride != null)
                {
                    lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();
                    _lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);
                }
                //Update Overrides
                _lrrApp.UpdateGrpOverridePoints();
				}
				

            else
            {
                LRROverride lrrModelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(propertyID, modeID);
                if (lrrModelOverride != null)
                {
                    lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();
                    _lrrApp.OverridesWindow.SetDataConfidanceLevel(lrrModelOverride.ID);
                }
                //Update Overrides
                _lrrApp.UpdateOverridePoints();
            }
			
		}

		private void gridManager_ForeColorChanged(object sender, ColorEventArgs e)
		{
			GridControlManager gridControlManager = sender as GridControlManager;
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator() as IDictionaryEnumerator;
			while(iDictEnum.MoveNext())
			{
				GridControlManager gcManager = iDictEnum.Value as GridControlManager;
				if(gcManager == gridControlManager)
				{
					GridControl gridControl = iDictEnum.Key as GridControl;
					IPropertyValue iPropValue = e.PropValueObj as IPropertyValue;
					gridControl.Model[e.Row,e.Col].TextColor = iPropValue.GetForeColor();
					break;
				}
			}
		}

		private void gridManager_BackColorChanged(object sender, ColorEventArgs e)
		{
			GridControlManager gridControlManager = sender as GridControlManager;
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator() as IDictionaryEnumerator;
			while(iDictEnum.MoveNext())
			{
				GridControlManager gcManager = iDictEnum.Value as GridControlManager;
				if(gcManager == gridControlManager)
				{
					GridControl gridControl = iDictEnum.Key as GridControl;
					IPropertyValue iPropValue = e.PropValueObj as IPropertyValue;
					gridControl.Model[e.Row,e.Col].BackColor = iPropValue.GetBackColor();
					break;
				}
			}
		}

		private void gridControlInstance_ResizingColumns(object sender, GridResizingColumnsEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			bool result = IOScreenUtilities.StopResizeHiddenColumns(gridControl,e.Columns.Left);
			if(result)
				e.Cancel = true;
		}

		private void gridControlInstance_SelectionChanged(object sender, GridSelectionChangedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(e.Range.Top == HEADER_ROW && e.Range.Left >= DATA_COL_START)
			{
				if(e.Reason != GridSelectionReason.MouseMove)
					return;
				int iModes = 0;

				//Get the Selected Modes
				ArrayList arrModesSelected = new ArrayList();
				foreach(LRRMode mode in LRRModel.ProjectModes)
				{
					if(mode.ViewSelection)
					{
						iModes++;
						arrModesSelected.Add(mode);
					}
				}
				
				string firstSelModeID = null;
				string lastSelModeID  = null;
				if(arrModesSelected.Count > 0)
				{
					LRRMode firstSelMode = arrModesSelected[0] as LRRMode;
					LRRMode lastSelMode  = arrModesSelected[arrModesSelected.Count - 1] as LRRMode;

					firstSelModeID = firstSelMode.ModeId;
					lastSelModeID = lastSelMode.ModeId;
				}
				
				//Check for Column Movement
				if(GridManagerHash[gridControl] != null)
				{
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
					ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;

					ColHeaderCell firstColHdrCell = colHeaderCellList[e.Range.Left - DATA_COL_START] as ColHeaderCell;
					ColHeaderCell lastColHdrCell = colHeaderCellList[e.Range.Right - DATA_COL_START] as ColHeaderCell;

					if(firstColHdrCell.ModeID == firstSelModeID && lastColHdrCell.ModeID == lastSelModeID)
					{
						LRRMode firstMode = LRRModel.ProjectModes[0];
						LRRMode lastMode = LRRModel.ProjectModes[LRRModel.ProjectModes.Count - 1];
						
						int leftIncr = Int32.Parse(firstSelModeID,Symcon.BusinessObjects.SymConverter.Culture) - Int32.Parse(firstMode.ModeId,Symcon.BusinessObjects.SymConverter.Culture);
						int rightIncr = Int32.Parse(lastMode.ModeId,Symcon.BusinessObjects.SymConverter.Culture ) - Int32.Parse(lastSelModeID,Symcon.BusinessObjects.SymConverter.Culture);
						int leftIndex = e.Range.Left - leftIncr;
						int rightIndex = e.Range.Right + rightIncr;
						
						gridControl.Selections.ChangeSelection(gridControl.Selections.Ranges.ActiveRange,GridRangeInfo.Cells(HEADER_ROW,leftIndex,gridControl.RowCount,rightIndex),false);

						_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = true;
						_lrrApp.MainFrame.BarItemRightColMovement.Enabled = true;	

						return;
					}
					else
					{
						_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = false;
						_lrrApp.MainFrame.BarItemRightColMovement.Enabled = false;
					}
				}
			}
			else
			{
				_lrrApp.MainFrame.BarItemLeftColMovement.Enabled = false;
				_lrrApp.MainFrame.BarItemRightColMovement.Enabled = false;
			}
		}

        public static ResultComponentsWindow rsComponentsTrend; 
            //= new ResultComponentsWindow();
		private bool isTrendEventSet = false;
		public void trendValue(object sender, EventArgs e)
		{
            rsComponentsTrend = new ResultComponentsWindow(true);
			MenuItem miItem = sender as MenuItem;
			GridControl gridControl = miItem.Parent.GetContextMenu().SourceControl as GridControl;
			if(GridManagerHash[gridControl] == null)
				return;

			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			//Get the Col Data from Grid Data Manager
			ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
			LRRSiteComponents lsc = new LRRSiteComponents();
			int colCount = gridControl.ColCount;
			for(int i=DATA_COL_START;i<=colCount;i++)
			{
				ColHeaderCell CompcolData = colHeaderCellList[i - DATA_COL_START] as ColHeaderCell;
				Type ComobjectType = CompcolData.ObjectType;
				string compcolCellID = CompcolData.ID;
				object Compobj = LRRModel.GetComponentObject(ComobjectType,compcolCellID);
				if(Compobj != null)
				{
					if(!lsc.Contains(Compobj) && Compobj.GetType()==typeof(LRRSiteComponent))
					{
						lsc.Add(Compobj);
					}
				}
			}
			ArrayList    rowHeaderCellList = gridControlManager.SortManagerList;
			LRRSiteComponentProperties siteComProps = new LRRSiteComponentProperties();
			ArrayList propertyId = new ArrayList();
			int rowCount = gridControl.RowCount;
			foreach(LRRSiteComponent siteComp in lsc)
			{
				for(int i=DATA_ROW_START;i<=rowCount;i++)
				{
					IOSortManager propRowData = rowHeaderCellList[i-DATA_ROW_START] as IOSortManager;
					Type _propObjectType = propRowData.ObjectType;
					string _propRowCellID = propRowData.ID;
					LRRSiteComponentProperty propObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromMasterPropertyID(siteComp.SiteComponentID,_propRowCellID); 
					if(propObj != null)
					{                                 
						//LRRSiteComponentProperty lscp = propObj as LRRSiteComponentProperty; 
						if(propObj.PropertyID != null)
						{
							if(!propertyId.Contains(propObj.PropertyID))
							{
								propertyId.Add(propObj.PropertyID); 
								siteComProps.Add(propObj);
							}
						}
						else
						{
							siteComProps.Add(propObj);
						}
					}
				}
			}
			//Show the Properties Window
			rsComponentsTrend.PropertyCollection = siteComProps;
			rsComponentsTrend.SiteComponents = lsc;
			rsComponentsTrend.Text = "Trended Components Window :";
			if(!isTrendEventSet)
			{
				rsComponentsTrend.RecordedProperties+=new Symcon.LightRidge.PEAdvisor.IOScreen.ResultComponentsWindow.SelectedPropertiesChanged(rsComponents_TrendedProperties);
				isTrendEventSet = true;
			}
			rsComponentsTrend.BindGrid();
			rsComponentsTrend.TopMost = false;
			rsComponentsTrend.Owner = _lrrApp.MainFrame;
			rsComponentsTrend.Show();
			
		}

		private void gridControlInstance_ColsMoving(object sender, GridRangeMovingEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
				IOScreenUtilities.ColMovement(e.From,e.Target,e.Count,gridControlManager.ColHeaderCellList,DATA_COL_START);
			}
            _gridPersistance._arrGridColMove[0].Add(e.From);
            _gridPersistance._arrGridColMove[1].Add(e.Count);
            _gridPersistance._arrGridColMove[2].Add(e.Target);
		}

		private void gridControlInstance_KeyDown(object sender, KeyEventArgs e)
		{
			try
			{
				bool isDelete = true;
                GridControl gridControl = sender as GridControl;

                if (e.KeyCode == Keys.Delete && !gridControl.CurrentCell.IsEditing)
                {
					
					Syncfusion.Windows.Forms.Grid.GridRangeInfoList rangeInfoList = new GridRangeInfoList();
					gridControl.Selections.GetSelectedRanges(out rangeInfoList,true);
					int top = rangeInfoList.ActiveRange.Top;
					int bottom = rangeInfoList.ActiveRange.Bottom;
					int right = rangeInfoList.ActiveRange.Right;
					int left = rangeInfoList.ActiveRange.Left;
					if(top < DATA_ROW_START || right <= DATA_COL_START)
						return;
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
					//Get the Row Data from Grid Data Manager
					ArrayList     rowSortList = gridControlManager.SortManagerList;
					//Get the Col Data from Grid Data Manager
					ArrayList     colHeaderCellList = gridControlManager.ColHeaderCellList;
					//Get the row data
					if(GridManagerHash[gridControl] == null)
					{
						e.Handled = true;
						return;
					}
					for(int iRow=top;iRow<=bottom;iRow++)
					{
						for(int iCol=left;iCol<=right;iCol++)
						{
							IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
							string rowCellID = rowData.ID;
							//Get the column data
							ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
							Type objectType = colData.ObjectType;
							string modeID   = colData.ModeID;
							string colCellID = colData.ID;

							//Get the Property Value Object
							object objPropValue = LRRModel.GetPropertyValueObject(objectType,modeID,rowCellID,colCellID);
							if(objPropValue != null)
							{
								IPropertyValue iPropValue = objPropValue as IPropertyValue;
								if(!iPropValue.GetEditableState())
								{
									isDelete = false;
									e.Handled = true;
									break;
								}
							}
						}
					}
					if(isDelete)
					{
						DialogResult dlgResult = LRRMessagBox.ShowMessage("SID_DELETE_VALUE","SID_LABEL",MessageBoxButtons.YesNo,MessageBoxIcon.Question);     
						if(dlgResult == DialogResult.No)
						{
							e.Handled = true;
						}
						else
						{
							for(int iRow=top;iRow<=bottom;iRow++)
							{
								for(int iCol=left;iCol<=right;iCol++)
								{
									cascadeChangesToPropValues(gridControl,gridControlManager,iRow,iCol);
								}
							}
						}
					}
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(IOScreenUtilities),ex.Message,Logger.Level.Debug); 
			}
		}
		private void gridControlInstance_CurrentCellKeyDown(object sender, KeyEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			int iRow = gridControl.CurrentCell.RowIndex;
            Syncfusion.Windows.Forms.Grid.GridRangeInfoList rangeInfoList = new GridRangeInfoList();
            gridControl.Selections.GetSelectedRanges(out rangeInfoList, true);
            int iCol = rangeInfoList.ActiveRange.Left;
			if(((iRow < DATA_ROW_START || iCol < DATA_COL_START) && ((e.KeyCode == Keys.Delete || e.KeyCode == Keys.Back || (e.Control && e.KeyCode == Keys.V)))))
			{
				e.Handled = true;
				return;
			}
            //if(e.Shift)
            //    IOScreenUtilities.HandleUOMEditingKeyDown(sender,e,UOM_COL,false);
		}

		private void gridControlInstance_CurrentCellKeyPress(object sender, KeyPressEventArgs e)
		{
			IOScreenUtilities.HandleUOMEditingKeyPress(sender,e,UOM_COL,false);
		}

		private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
		{
			//IOScreenUtilities.ResizeToFit(GridManagerHash,DATA_ROW_START,DATA_COL_START);
		}

		private void gsInfo_Changed(object sender, Syncfusion.Styles.StyleChangedEventArgs e)
		{
			GridStyleInfo gsInfo = sender as GridStyleInfo;
			string text = gsInfo.Text;

			GridControl gridControl = gsInfo.GetActiveGridView() as GridControl;

			int iRow = gsInfo.CellIdentity.RowIndex;
			int iCol = gsInfo.CellIdentity.ColIndex;
			
			//Get the Grid Data
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
				
				//Get the row data
				IOSortManager rowData = new IOSortManager();
				//Get the column data
				ColHeaderCell colData = new ColHeaderCell();

				GetObjectData(gridControlManager,iRow,iCol,out colData,out rowData);
				object objDefaultProp = LRRModel.GetPropertyObject(colData.ObjectType,rowData.ID,colData.ID);
				if(objDefaultProp != null)
				{
					IProperty iDefaultProp = objDefaultProp as IProperty;
					string propID = iDefaultProp.GetID();
					for(int i= DATA_COL_START;i < gridControl.ColCount + 1;i++)
					{
						//Get the row data
						IOSortManager rowData1 = new IOSortManager();
						//Get the column data
						ColHeaderCell colData1 = new ColHeaderCell();

						GetObjectData(gridControlManager,iRow,i,out colData1,out rowData1);
						object objProp = LRRModel.GetPropertyObject(colData1.ObjectType,rowData1.ID,colData1.ID);
						if(objProp != null)
						{	
							IProperty iProp = objProp as IProperty;
							string propertyID = iProp.GetID();
							if(propertyID == propID && i != iCol)
							{
								gridControl.Model[iRow,i].Text = text;
							}
						}
					}
				}
			}
		}
		
		private void gridControlInstance_Paint(object sender, PaintEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] != null)
			{
				GridControlManager gcManager = GridManagerHash[gridControl] as GridControlManager;
				GridStyleInfo styleInfo = gridControl.BaseStylesMap["Column Header"].StyleInfo;
				int x = gridControl.RowIndexToVScrollPixelPos(HEADER_ROW);
				int y = gridControl.ColIndexToHScrollPixelPos(PROPERTIES_COL - 1);
				int height = gridControl.RowHeights.GetSize(HEADER_ROW);
				StringFormat sFormat = new StringFormat();
				sFormat.Alignment = StringAlignment.Near;
				sFormat.LineAlignment = StringAlignment.Center;
				Graphics g = e.Graphics;
				int width = gridControl.Width;
				g.DrawString(gcManager.HeaderText,styleInfo.GdipFont,new SolidBrush(styleInfo.TextColor),new RectangleF(x,y,width,height),sFormat);
			}
		}	

		private void _lrrApp_ReconnectCompleted(object sender, EventArgs e)
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				GridControl  gridControlInstance = (GridControl)iDictEnum.Key;
				GridControlManager  gridControlManager = (GridControlManager)iDictEnum.Value;
				InitializePropertyValues(gridControlInstance,gridControlManager);
			}
		}

        private void Model_ClipboardPaste(object sender, Syncfusion.Windows.Forms.Grid.GridCutPasteEventArgs e)
        {
            GridModel gridControlModel = sender as GridModel;
            gridControlModel.CurrentCellInfo.GridView.CurrentCell.EndEdit();

            GridControl gridControl = gridControlModel.CurrentCellInfo.GridView as GridControl;

            if (GridManagerHash[gridControl] == null)
            {
                return;
            }

            GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;

            IDataObject data = Clipboard.GetDataObject();

            if (data.GetDataPresent(typeof(string)))
            {
                string s = (string)data.GetData(typeof(string));

                string[] sRows = s.Split(new string[] { "\r\n" }, StringSplitOptions.None);

                GridRangeInfoList rangeList;
                if (gridControl.Selections.GetSelectedRanges(out rangeList, true))
                {
                    foreach (GridRangeInfo range in rangeList)
                    {
                        int sRowIndex = 0;
                        for (int iRow = range.Top; iRow <= range.Bottom; iRow++)
                        {
                            if (sRowIndex < sRows.Length)
                            {
                                string sRow = sRows[sRowIndex++];
                                int sCellInRowIndex = 0;
                                string[] sCells = sRow.Split(new char[] { '\t' });
                                for (int iCol = range.Left; iCol <= range.Right; iCol++)
                                {
                                    if (sCellInRowIndex < sCells.Length)
                                    {
                                        string sCell = sCells[sCellInRowIndex++];

                                        if (iRow < DATA_ROW_START || iCol < DATA_COL_START)
                                            continue;

                                        //Get the Row Data from Grid Data Manager
                                        ArrayList rowSortList = gridControlManager.SortManagerList;
                                        //Get the Col Data from Grid Data Manager
                                        ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;

                                        //Get the row data
                                        IOSortManager rowData = rowSortList[iRow - DATA_ROW_START] as IOSortManager;
                                        string rowCellID = rowData.ID;
                                        //Get the column data
                                        ColHeaderCell colData = colHeaderCellList[iCol - DATA_COL_START] as ColHeaderCell;
                                        Type objectType = colData.ObjectType;
                                        string colCellID = colData.ID;

                                        string modeId = colData.ModeID;
                                        LRRMode mode = LRRModel.Modes.GetModeByID(modeId);

                                        if (!mode.ViewSelection)
                                        {
                                            sCellInRowIndex--;
                                            continue;
                                        }

                                        //Get the Property Value Object
                                        object objPropValue = LRRModel.GetPropertyValueObject(objectType, modeId, rowCellID, colCellID);
                                        if (objPropValue != null)
                                        {
                                            IPropertyValue iPropValue = objPropValue as IPropertyValue;
                                            if (!iPropValue.GetEditableState())
                                                continue;
                                        }
                                        else
                                        {
                                            continue;
                                        }

                                        gridControl[iRow, iCol].Text = sCell;
                                        changePropertyValue(gridControl, gridControlManager, iRow, iCol, sCell);
                                    }
                                }
                            }
                        }
                    }
                    e.Result = true;
                    e.Handled = true;
                }
            }
        }

		#endregion

		#region RecordedProperties Events
		private void rsComponents_RecordedProperties(RecordingEventArgs e)
		{
			for(int i =0 ;i < e.SelectedSiteComponents.Count;i++)
			{
				
				IComponent iComp = e.SelectedSiteComponents[i] as IComponent;  
				CustomCollection iPropCollection = iComp.GetProperties(); 
				IEnumerator iEnum = iPropCollection.GetEnumerator();
				LRRSiteComponentProperties lrrSCPToSave = new LRRSiteComponentProperties();
				while(iEnum.MoveNext())
				{
					IProperty iPropObj  = iEnum.Current as IProperty;
					string iPropID = iPropObj.GetMasterPropertyID(); 
					if(e.SelectedProperties.Contains(iPropID)) 
					{
						iPropObj.SetRecorded(true);
						lrrSCPToSave.Add(iPropObj);
					}
					else if(e.UnSelectedSiteProperties.Contains(iPropID)) 
					{
						iPropObj.SetRecorded(false);
						lrrSCPToSave.Add(iPropObj);
					}
				}
				_lrrApp.DBManagerObj.ExecuteBatchUpdate(lrrSCPToSave);
			}
		}
		
		private void rsComponents_TrendedProperties(RecordingEventArgs e)
		{
			for(int i =0 ;i < e.SelectedSiteComponents.Count;i++)
			{
				
				IComponent iComp = e.SelectedSiteComponents[i] as IComponent;  
				CustomCollection iPropCollection = iComp.GetProperties(); 
				IEnumerator iEnum = iPropCollection.GetEnumerator();
				LRRSiteComponentProperties lrrSCPToSave = new LRRSiteComponentProperties();
				while(iEnum.MoveNext())
				{
					IProperty iPropObj  = iEnum.Current as IProperty;
					string iPropID = iPropObj.GetMasterPropertyID(); 
					if(e.SelectedProperties.Contains(iPropID)) 
					{
						iPropObj.SetTrended(true);
						lrrSCPToSave.Add(iPropObj);
					}
					else if(e.UnSelectedSiteProperties.Contains(iPropID)) 
					{
						iPropObj.SetTrended(false);
						lrrSCPToSave.Add(iPropObj);
					}
				}
				_lrrApp.DBManagerObj.ExecuteBatchUpdate(lrrSCPToSave);
			}
		}

		private void gridManager_IsTrackChanged(object sender, ColorEventArgs e)
		{
            if (!_recTrendDisplayFlag)
                return;

			GridControlManager gridControlManager = sender as GridControlManager;
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator() as IDictionaryEnumerator;
			while(iDictEnum.MoveNext())
			{
				GridControlManager gcManager = iDictEnum.Value as GridControlManager;
				if(gcManager == gridControlManager)
				{
					GridControl gridControl = iDictEnum.Key as GridControl;
					IPropertyValue iPropValue = e.PropValueObj as IPropertyValue;
					_style = new GridExcelTipStyleProperties(gridControl[e.Row,e.Col]);

                    if (e.IsRecordedValue && iPropValue.GetIsTrended())
                    {
                        _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_RECORDED_AND_TRENDED");
                    }
                    else if (iPropValue.GetIsTrended())
                    {
                        _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_TRENDED");
                    }
					else if(e.IsRecordedValue)
					{
                        _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_RECORDED");
					}
					else
					{
						_style.ResetExcelTipText();
					}
                	break;
				}
			}
        }

        void gridManager_IsTrendedChanged(object sender, ColorEventArgs e)
        {
            if (!_recTrendDisplayFlag)
                return;

            GridControlManager gridControlManager = sender as GridControlManager;
            IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator() as IDictionaryEnumerator;
            while (iDictEnum.MoveNext())
            {
                GridControlManager gcManager = iDictEnum.Value as GridControlManager;
                if (gcManager == gridControlManager)
                {
                    GridControl gridControl = iDictEnum.Key as GridControl;
                    IPropertyValue iPropValue = e.PropValueObj as IPropertyValue;
                    _style = new GridExcelTipStyleProperties(gridControl[e.Row, e.Col]);

                    if (e.IsTrendedValue && iPropValue.GetIsRecorded())
                    {
                        _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_RECORDED_AND_TRENDED");
                    }
                    else if (e.IsTrendedValue)
                    {
                        _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_TRENDED");
                    }
                    else if (iPropValue.GetIsRecorded())
                    {
                        _style.ExcelTipText = LRRHeaderText.GetHeaderString("SID_RECORDED");
                    }
                    else
                    {
                        _style.ResetExcelTipText();
                    }
                    break;
                }
            }
        }

		#endregion

	}
}
