using System;
using System.Drawing;

using Symcon.LightRidge.PEAdvisor;
using Symcon.ColorSchemes;
using Symcon.Utils.HeaderDispatcher;  

namespace Symcon.LightRidge.PEAdvisor.IOScreen
{
	/// <summary>
	/// Summary description for IOSchemes.
	/// </summary>
	public class IOSchemes
	{
		static IOSchemes()
		{
			//Get the colors from XML file
			InitializeScheme();


		}


		#region Constants
		public static Color HEADER_BACK_COLOR;//Color.DarkBlue;
		public static Color COLUMN_HEADER_BACK_COLOR;// = Color.DarkBlue;
		public static Color COLUMN_HEADER_TEXT_COLOR;// = SystemColors.ActiveCaptionText;
		public static Color ROW_HEADER_BACK_COLOR;// = Color.FromArgb(192,192,192);//Color.LightGray
		public static Color ATTRIBUTES_BACK_COLOR;// = Color.FromArgb(192,192,192);//Color.LightGray
		public static Color PROPERTIES_BACK_COLOR;// = Color.FromArgb(192,192,192);//Color.LightGray
		public static Color GRID_LINE_COLOR;// = Color.FromArgb(0,0,0);//SystemColors.ActiveCaption;
		public static Color GRID_PROPVALUE_LINE_COLOR;// = Color.FromArgb(128,128,128);//SystemColors.ActiveCaption;
		public static Color GRID_ROW_COLOR;// = Color.FromArgb(192,192,192);//Color.LightGray
		public static Color GRID_BACK_COLOR;// = Color.FromArgb(150,150,150);
		public static Color DEFAULT_MODE_COLOR;// = Color.LightGreen;//Color.LightGray;
		public static Color MEAS_MODE_COLOR;// = Color.FromArgb(255,204,153);//Tan;
		public static Color SS_MODE_COLOR;// = Color.FromArgb(255,255,153);//Color.LightYellow;
		public static Color SCENARIO_MODE_COLOR;// = Color.FromArgb(204,255,255);//Color.LightCyan;
		public static Color MODE_TEXT_COLOR;// = Color.Black;
		public static Color COMPONENT_BACK_COLOR;// = Color.FromArgb(192,192,192);//Color.LightGray;
		public static Color	UOM_BACK_COLOR;// = Color.FromArgb(192,192,192);//Color.LightGray;
		public static Color	RECORDED_VALUE_COLOR;// = Color.LightYellow;
		public static Color	NORMAL_BACK_COLOR;// = Color.FromArgb(150,150,150);//Color.White;

		public static string ATTRIBUTE_HEADER_NAME = LRRHeaderText.GetHeaderString("SID_ATTRIBUTES");
		public static string COMPONENT_HEADER_NAME = LRRHeaderText.GetHeaderString("SID_COMP");
        public static string PLANT_ITEM_HEADER = LRRHeaderText.GetHeaderString("SID_PLANT_ITEMS");
		public static string UOM_HEADER_NAME = LRRHeaderText.GetHeaderString("SID_UOM");
		public static int MODE_COL_SIZE = 50;
		public static int HEADER_COL_SIZE = 10;
		public static int ROW_HEADER_COL_SIZE = 150;
		#endregion

		#region Methods
		
		/// <summary>
		/// //Get the colors from XML file
		/// </summary>
		public static void InitializeScheme()
		{
			LRRApp _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			ColorScheme colorScheme = _app.LRRUIManagerObj.ColorSchemeObj;

			//Header Colors
			HEADER_BACK_COLOR = colorScheme.Search.HeaderBackColor;
			COLUMN_HEADER_BACK_COLOR = colorScheme.Search.ColumnHeaderBackColor;
			COLUMN_HEADER_TEXT_COLOR = colorScheme.Search.ColumnHeaderTextColor;
			ROW_HEADER_BACK_COLOR = colorScheme.Search.RowHeaderBackColor;
			ATTRIBUTES_BACK_COLOR = colorScheme.Search.AttributesBackColor;
			PROPERTIES_BACK_COLOR = colorScheme.Search.PropertiesBackColor;
			GRID_LINE_COLOR = colorScheme.Search.GridLineColor;
			GRID_ROW_COLOR = colorScheme.Search.GridRowColor;
			GRID_PROPVALUE_LINE_COLOR = colorScheme.Search.GridPropValueLineColor;
			COMPONENT_BACK_COLOR = colorScheme.Search.ComponentBackColor;
			UOM_BACK_COLOR = colorScheme.Search.UomBackColor;
			
			RECORDED_VALUE_COLOR = colorScheme.Search.RecordedBackColor;
			NORMAL_BACK_COLOR = colorScheme.Search.NormalBackColor;
			GRID_BACK_COLOR = colorScheme.Search.GridRowColor;

			//Mode Colors
			MEAS_MODE_COLOR = colorScheme.Search.MeasModeColor;
			SS_MODE_COLOR  = colorScheme.Search.SsModeColor;
			SCENARIO_MODE_COLOR = colorScheme.Search.ScenarioModeColor;
			DEFAULT_MODE_COLOR = colorScheme.Search.DefaultModeColor;
			MODE_TEXT_COLOR = colorScheme.Search.ModeTextColor;
		}

		/// <summary>
		/// Gets the Mode Back color for the given Mode
		/// </summary>
		/// <param name="modeID"></param>
		/// <returns></returns>
		public static Color GetModeBackColor(string modeID)
		{
			Color color = IOSchemes.DEFAULT_MODE_COLOR;
			if(modeID == "1")
				color = IOSchemes.MEAS_MODE_COLOR;
			if(modeID == "2")
				color = IOSchemes.SS_MODE_COLOR;
			if(modeID == "3")
				color = IOSchemes.SCENARIO_MODE_COLOR;

			return color;
		}
		
		/// <summary>
		/// Gets the Mode Text Color for the given Mode
		/// </summary>
		/// <param name="modeID"></param>
		/// <returns></returns>
		public static Color GetModeTextColor(string modeID)
		{
			return IOSchemes.MODE_TEXT_COLOR;
		}
		#endregion

	}
}
