using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Grid;

using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.GridUtilities;

namespace Symcon.LightRidge.PEAdvisor.IOScreen
{
	/// <summary>
	/// Summary description for IOScreenUtilities.
	/// </summary>
	public class IOScreenUtilities
	{
		static IOScreenUtilities()
		{
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			hashCollectionIndex.Add(typeof(LRRSiteComponent),0);
			hashCollectionIndex.Add(typeof(LRRAggregate),1);
		}
		
		#region Methods
		/// <summary>
		/// Method for splitting the components according to MasterComponent Type
		/// </summary>
		/// <param name="custCollection"></param>
		/// <param name="rowCount"></param>
		/// <param name="colCount"></param>
		/// <returns></returns>
		public static object[,] SplitComponents(ArrayList custCollection,out int rowCount,out int colCount)
		{
			rowCount = 0;
			colCount = 0;

			ArrayList arrIndex = new ArrayList(SUPPORTED_COLLECTION_COUNT);
			for(int i=0;i < SUPPORTED_COLLECTION_COUNT;i++)
			{
				arrIndex.Add(null);
			}
			IEnumerator iCustEnum = custCollection.GetEnumerator();
			while(iCustEnum.MoveNext())
			{
				CustomCollection objCollection = iCustEnum.Current as CustomCollection;
				if(objCollection == null)
					continue;
				Type collectionType = objCollection.ItemType;
				if(hashCollectionIndex[collectionType] != null)
				{
					int index = (int)hashCollectionIndex[collectionType];
					arrIndex[index] = objCollection;
				}
			}
			
			//Split the Collections based on Master Component Type
			if(arrIndex == null && arrIndex.Count == 0)
				return null;

			Hashtable hashMasterCompType = new Hashtable();
			int nCtr = 0;
			for(int i=0;i < arrIndex.Count;i++)
			{
				CustomCollection objCollection = arrIndex[i] as CustomCollection;
				//check Collection,if it is siteComponent,Aggregates
				switch(i)
				{
					//If SiteComponents
					case 0:
						LRRSiteComponents siteComponents = objCollection as LRRSiteComponents;
						foreach(LRRSiteComponent siteCompObj in siteComponents)
						{
							string sMasterCompType = siteCompObj.MasterComponentType == null ? "" : siteCompObj.MasterComponentType.ToString(Symcon.BusinessObjects.SymConverter.Culture);
							if(!hashMasterCompType.ContainsValue(sMasterCompType))
							{
								hashMasterCompType.Add(nCtr,siteCompObj.MasterComponentType.ToString(Symcon.BusinessObjects.SymConverter.Culture));
								nCtr ++ ;
							}
						}
						break;
					//If Aggregates
					case 1:
						LRRAggregates aggregates = objCollection as LRRAggregates;
						foreach(LRRAggregate aggrObj in aggregates)
						{
							string sMasterCompType = aggrObj.EquipmentTypeID == null ? "" : aggrObj.EquipmentTypeID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
							if(!hashMasterCompType.ContainsValue(sMasterCompType))
							{
								hashMasterCompType.Add(nCtr,sMasterCompType);
								nCtr ++ ;
							}
						}
						break;
					default:
						break;
				}
			}

			object[,] arrSplitComponent = new object[nCtr,hashCollectionIndex.Count];
			rowCount = nCtr;
			colCount = hashCollectionIndex.Count;
			int iCount = 0;
			while(iCount < nCtr)
			{
				string sMasterComponentType = hashMasterCompType[iCount] .ToString();

				for(int i=0;i < arrIndex.Count;i++)
				{
					CustomCollection objCollection = arrIndex[i] as CustomCollection;
					//check Collection,if it is siteComponent,Aggregates
					switch(i)
					{
						//If SiteComponents
						case 0:
							LRRSiteComponents siteComponents = objCollection as LRRSiteComponents;
							LRRSiteComponents siteComps =  siteComponents.FilterSiteComponentsByMasterComponentID(sMasterComponentType);
							arrSplitComponent[iCount,0] = siteComps;
							break;
						//If Aggreagtes
						case 1:
							LRRAggregates aggregates = objCollection as LRRAggregates;
							LRRAggregates lrrAggregates = aggregates.FilterAggregatesByMasterComponentType(sMasterComponentType);
							arrSplitComponent[iCount,1] = lrrAggregates;
							break;
						default:
							break;
					}
				}
				iCount ++;
			}

			return arrSplitComponent;
		}
		
		/// <summary>
		/// Initializes the grid
		/// </summary>
		/// <param name="gridControlInstance"></param>
		public static void InitializeGrid(GridControl gridControlInstance)
		{
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.BackColor = IOSchemes.COLUMN_HEADER_BACK_COLOR;
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.TextColor = IOSchemes.COLUMN_HEADER_TEXT_COLOR;
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.CellType = "Static";
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.ReadOnly = true;

			gridControlInstance.BaseStylesMap["Row Header"].StyleInfo.BackColor = IOSchemes.ROW_HEADER_BACK_COLOR;

			gridControlInstance.Properties.RowHeaders = true;
			gridControlInstance.NumberedRowHeaders = false;
			gridControlInstance.AllowDragSelectedRows = true;
			gridControlInstance.BaseStylesMap["Row Header"].StyleInfo.Borders.Right = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.Solid, System.Drawing.SystemColors.WindowFrame);
			gridControlInstance.ColWidths.SetSize(0, IOSchemes.HEADER_COL_SIZE);

			gridControlInstance.BaseStylesMap["Header"].StyleInfo.BackColor = IOSchemes.HEADER_BACK_COLOR;
			gridControlInstance.ColStyles[1].CellType = "Static";
			gridControlInstance.RowStyles[1].CellType = "Static";
			gridControlInstance.RowStyles[2].CellType = "Static";
			gridControlInstance.ColStyles[1].ReadOnly = true;
			gridControlInstance.RowStyles[0].ReadOnly = true;
			gridControlInstance.RowStyles[1].ReadOnly = true;
			gridControlInstance.RowStyles[2].ReadOnly = true;;

			gridControlInstance.Model.Options.DefaultGridBorderStyle = GridBorderStyle.Solid;
			gridControlInstance.Model.Options.ExcelLikeSelectionFrame = false;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.WrapText = true;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.Trimming = StringTrimming.None;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.AutoSize = true;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.HorizontalAlignment = GridHorizontalAlignment.Center;
			gridControlInstance.Properties.BackgroundColor = IOSchemes.GRID_BACK_COLOR;
			gridControlInstance.Properties.GridLineColor = IOSchemes.GRID_LINE_COLOR;
			gridControlInstance.DefaultGridBorderStyle = GridBorderStyle.Solid;

			//			gridControlInstance.Properties.RowHeaders = false;
			gridControlInstance.Properties.Buttons3D = false;
			gridControlInstance.ColWidths.SetSize(1, IOSchemes.ROW_HEADER_COL_SIZE);
		}
		
		/// <summary>
		/// Exports the selected grid
		/// </summary>
		/// <param name="gridManagerHash"></param>
		/// <returns></returns>
		public static GridControl Export(Hashtable gridManagerHash)
		{
			IDictionaryEnumerator iDictEnum = gridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					return gridControl;
				}
			}
			return null;
		}

		/// <summary>
		/// Prints the selected grid
		/// </summary>
		/// <param name="gridManagerHash"></param>
		public static void Print(Hashtable gridManagerHash)
		{
			IDictionaryEnumerator iDictEnum = gridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					GridControlPrint ggPrint = new GridControlPrint();
					ggPrint.PrintData(gridControl,false);
					break;
				}
			}
		}

		/// <summary>
		/// Print Preview of the selected grid
		/// </summary>
		/// <param name="gridManagerHash"></param>
		public static void PrintPreview(Hashtable gridManagerHash)
		{
			IDictionaryEnumerator iDictEnum = gridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					GridControlPrint ggPrint = new GridControlPrint();
					ggPrint.PrintData(gridControl,true);
					break;
				}
			}
		}
		
		/// <summary>
		/// Hides the rows
		/// </summary>
		/// <param name="arrayUnSelectedRecords"></param>
		/// <param name="GridManagerHash"></param>
		/// <param name="rowStart"></param>
		public static void HideRows(ArrayList arrayUnSelectedRecords,Hashtable GridManagerHash,int rowStart)
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;
				ArrayList rowList = gridControlManager.SortManagerList;
				gridControl.RowHiddenEntries.Clear();
				for(int i=0;i < rowList.Count;i++)
				{
					IOSortManager sortManager = rowList[i] as IOSortManager;
					sortManager.IsVisible = true;
					if(arrayUnSelectedRecords.Contains(sortManager.ID))
					{
						sortManager.IsVisible = false;
						gridControl.RowHiddenEntries.Add(new GridRowHidden(i + rowStart));
					}
				}
                gridControl.Refresh();
			}
		}
		
		/// <summary>
		/// Hides the columns
		/// </summary>
		/// <param name="arrayUnSelectedRecords"></param>
		/// <param name="GridManagerHash"></param>
		public static void HideColumns(ArrayList arrayUnSelectedRecords,Hashtable GridManagerHash,int colStart)
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;
				gridControl.ColHiddenEntries.Clear();
				ArrayList colList = gridControlManager.ColHeaderCellList;
				for(int i=0;i < colList.Count;i++)
				{
					ColHeaderCell colHeaderCell = colList[i] as ColHeaderCell;
					colHeaderCell.IsVisible = true;
					if(arrayUnSelectedRecords.Contains(colHeaderCell.ID))
					{
						colHeaderCell.IsVisible = false;
						gridControl.ColHiddenEntries.Add(new GridColHidden(i + colStart));
					}
				}
			}
		}
		
		/// <summary>
		/// Method Prevents resize of hidden columns
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="col"></param>
		/// <returns></returns>
		public static bool StopResizeHiddenColumns(GridControl gridControl,int col)
		{
			GridColHiddenCollection gridColHiddenCollection = gridControl.ColHiddenEntries;
			GridColHiddenCollectionEnumerator gridEnum = gridColHiddenCollection.GetEnumerator();
			while(gridEnum.MoveNext())
			{
				GridColHidden gridColHidden = gridEnum.Current;
				if(gridColHidden.ColIndex == col)
				{
					return true;
				}
			}
			return false;
		}

		/// <summary>
		/// Method hides the mode columns
		/// </summary>
		/// <param name="GridManagerHash"></param>
		public static void HideModes(Hashtable GridManagerHash,int colStart)
		{
			ArrayList arrModeSelected = new ArrayList();
			foreach(LRRMode mode in LRRModel.ProjectModes)
			{
				if(!mode.ViewSelection)
					arrModeSelected.Add(mode.ModeId);
			}
			
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;
				ArrayList colList = gridControlManager.ColHeaderCellList;
				for(int i=0;i < colList.Count;i++)
				{
					ColHeaderCell colHeaderCell = colList[i] as ColHeaderCell;
					if(arrModeSelected.Contains(colHeaderCell.ModeID))
					{
						gridControl.ColHiddenEntries.Add(new GridColHidden(i + colStart));
					}
				}
				gridControl.Refresh();
			}
		}
		
		/// <summary>
		/// Saves the query
		/// </summary>
		/// <param name="name"></param>
		/// <param name="desc"></param>
		/// <param name="srchGroupLevel"></param>
		/// <param name="GridManagerHash"></param>
		/// <param name="searchQuery"></param>
		public static void SaveQuery(string name,string desc,LRRSearchGroupLevel srchGroupLevel,Hashtable GridManagerHash,SearchCriteria searchQuery,bool isPublished)
		{
			//Item propeties
			LRRSearchCriteriaItemProperties srchItemProps = searchQuery.SearchCriteriaItemProps;
			LRRSearchCriteriaUserSpecificUIUOMs srchCriteriaUserSpecificUIUOMs = new LRRSearchCriteriaUserSpecificUIUOMs();
			IEnumerator iEnum = srchItemProps.GetEnumerator();
			while(iEnum.MoveNext())
			{
				LRRSearchCriteriaItemProperty srchItemProperty = iEnum.Current as LRRSearchCriteriaItemProperty;
				IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
				while(iDictEnum.MoveNext())
				{
					DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
					GridControlManager gridControlManager = dEntry.Value as GridControlManager;
					if(gridControlManager.UIUOMHash.ContainsKey(srchItemProperty.ItemPropertyID))
					{
//						srchItemProperty.UIUomID = (string)gridControlManager.UIUOMHash[srchItemProperty.ItemPropertyID];
						//Save in the User Specific UIUOMs
						LRRSearchCriteriaUserSpecificUIUOM srchCriteriaUserSpecificUIUOM = new LRRSearchCriteriaUserSpecificUIUOM();
						srchCriteriaUserSpecificUIUOM.PropertyID = srchItemProperty.ID;
						srchCriteriaUserSpecificUIUOM.UIUomID = (string)gridControlManager.UIUOMHash[srchItemProperty.ItemPropertyID];
						srchCriteriaUserSpecificUIUOM.IsVisible = true;
						srchCriteriaUserSpecificUIUOMs.Add(srchCriteriaUserSpecificUIUOM);
					}
				}
			}
			searchQuery.SearchQuery.Name = name;
			searchQuery.SearchQuery.Description = desc;
			searchQuery.SearchQuery.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
			
			//Group Levels
			LRRSearchGroupLevel lrrLevel = new LRRSearchGroupLevel();
			lrrLevel.LevelID = searchQuery.SearchQuery.ID;
			lrrLevel.Name = name;
			lrrLevel.Description = desc;
			lrrLevel.Type = "Query";
			lrrLevel.IsPublished = isPublished;

			LRRSearchGroupLevelLink lrrLvlLink = new LRRSearchGroupLevelLink();
			lrrLvlLink.ChildID = searchQuery.SearchQuery.ID;;
			lrrLvlLink.ParentID = srchGroupLevel.LevelID;
			lrrLvlLink.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;

			_lrrApp.DBManagerObj.CommitObject(searchQuery.SearchQuery,searchQuery.UserID);

			_lrrApp.DBManagerObj.CommitObject(lrrLevel,searchQuery.UserID);

			_lrrApp.DBManagerObj.CommitObject(lrrLvlLink,searchQuery.UserID);

			if(searchQuery.SearchCriteriaModes.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaModes,searchQuery.UserID);
			if(searchQuery.SearchCriteriaLevels.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaLevels,searchQuery.UserID);
			if(searchQuery.SearchCriteriaItems.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaItems,searchQuery.UserID);
			if(searchQuery.SearchCriteriaItemProps.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaItemProps,searchQuery.UserID);
			//Aggregates
			if(searchQuery.SearchCriteriaAggrItems.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaAggrItems,searchQuery.UserID);
			//Commit Master Items and Comp Types
			if(searchQuery.SearchCriteriaMasterItems.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaMasterItems,searchQuery.UserID);
			if(searchQuery.SearchCriteriaCompTypes.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(searchQuery.SearchCriteriaCompTypes,searchQuery.UserID);
			if(srchCriteriaUserSpecificUIUOMs.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(srchCriteriaUserSpecificUIUOMs);
		}
		
		/// <summary>
		/// Method saves the Item Properties with the changed UOMs,Items and Item Properties
		/// </summary>
		/// <param name="searchID"></param>
		/// <param name="GridManagerHash"></param>
		/// <param name="isTransposed"></param>
		public static void Save(string searchID,Hashtable GridManagerHash,bool isTransposed)
		{
			LRRSearchCriteriaItemProperties srchItemProps = new LRRSearchCriteriaItemProperties();
			_lrrApp.DBManagerObj.PopulateCollection(srchItemProps," where SearchCriteriaID = '" + searchID + "'");

			LRRSearchCriteriaUserSpecificUIUOMs srchCriteriaUserSpecificUIUOMs = new LRRSearchCriteriaUserSpecificUIUOMs();
			_lrrApp.DBManagerObj.PopulateCollection(srchCriteriaUserSpecificUIUOMs);
			
			//Populate Items
			LRRSearchCriteriaItems srchItems = new LRRSearchCriteriaItems();
			_lrrApp.DBManagerObj.PopulateCollection(srchItems," where SearchCriteriaID = '" + searchID + "'");

			//Populate User Specific Items
			LRRSearchCriteriaUserSpecificItems  srchCriteriaUserSpecificItems = new LRRSearchCriteriaUserSpecificItems();
			_lrrApp.DBManagerObj.PopulateCollection(srchCriteriaUserSpecificItems);

			IEnumerator iEnum = srchItemProps.GetEnumerator();
			while(iEnum.MoveNext())
			{
				LRRSearchCriteriaItemProperty srchItemProperty = iEnum.Current as LRRSearchCriteriaItemProperty;
				IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
				while(iDictEnum.MoveNext())
				{
					DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
					GridControlManager gridControlManager = dEntry.Value as GridControlManager;
					if(gridControlManager.UIUOMHash.ContainsKey(srchItemProperty.ItemPropertyID))
					{
						//Save in the User Specific UIUOMs
						LRRSearchCriteriaUserSpecificUIUOM srchCriteriaUserSpecificUIUOM = srchCriteriaUserSpecificUIUOMs.GetUIUOMObjectByPropertyID(srchItemProperty.ID);
						if(srchCriteriaUserSpecificUIUOM != null)
						{
                            //Commented
                            //Based on Isseu No: 1794 Date 7/11/2008.
							//srchCriteriaUserSpecificUIUOM.UIUomID = (string)gridControlManager.UIUOMHash[srchItemProperty.ItemPropertyID];
							if(isTransposed)
								srchCriteriaUserSpecificUIUOM.IsVisible = IsColumnVisible(srchItemProperty.ItemPropertyID,GridManagerHash);
							else
								srchCriteriaUserSpecificUIUOM.IsVisible = IsRowVisible(srchItemProperty.ItemPropertyID,GridManagerHash);
						}
						else
						{
							srchCriteriaUserSpecificUIUOM = new LRRSearchCriteriaUserSpecificUIUOM();
							srchCriteriaUserSpecificUIUOM.PropertyID = srchItemProperty.ID;
							if(isTransposed)
								srchCriteriaUserSpecificUIUOM.IsVisible = IsColumnVisible(srchItemProperty.ItemPropertyID,GridManagerHash);
							else
								srchCriteriaUserSpecificUIUOM.IsVisible = IsRowVisible(srchItemProperty.ItemPropertyID,GridManagerHash);
							srchCriteriaUserSpecificUIUOM.UIUomID = (string)gridControlManager.UIUOMHash[srchItemProperty.ItemPropertyID];
							srchCriteriaUserSpecificUIUOMs.Add(srchCriteriaUserSpecificUIUOM);
						}
					}
				}
			}

			//Update User Specific Items
			IEnumerator iItemEnum = srchItems.GetEnumerator();
			while(iItemEnum.MoveNext())
			{
				LRRSearchCriteriaItem srchItem = iItemEnum.Current as LRRSearchCriteriaItem;
				LRRSearchCriteriaUserSpecificItem srchUserSpecificItem = srchCriteriaUserSpecificItems.GetUIItemObjectByItemID(srchItem.ID);
				if(srchUserSpecificItem != null)
				{
					if(isTransposed)
						srchUserSpecificItem.IsVisible = IsRowVisible(srchItem.ItemID,GridManagerHash);
					else
						srchUserSpecificItem.IsVisible = IsColumnVisible(srchItem.ItemID,GridManagerHash);
				}
				else
				{
					srchUserSpecificItem = new LRRSearchCriteriaUserSpecificItem();
					srchUserSpecificItem.ItemID = srchItem.ID;
					if(isTransposed)
						srchUserSpecificItem.IsVisible = IsRowVisible(srchItem.ItemID,GridManagerHash);
					else
						srchUserSpecificItem.IsVisible = IsColumnVisible(srchItem.ItemID,GridManagerHash);
					srchCriteriaUserSpecificItems.Add(srchUserSpecificItem);
				}
			}
            if (srchCriteriaUserSpecificItems.Count > 0)
                _lrrApp.DBManagerObj.CommitCollection(srchCriteriaUserSpecificItems);
			
            if(srchCriteriaUserSpecificUIUOMs.Count > 0)
				_lrrApp.DBManagerObj.CommitCollection(srchCriteriaUserSpecificUIUOMs);
			
		}
		
		/// <summary>
		/// Method finds if the column is Visible
		/// </summary>
		/// <param name="itemID"></param>
		/// <param name="GridManagerHash"></param>
		/// <returns></returns>
		private static bool IsColumnVisible(string itemID,Hashtable GridManagerHash)
		{
			IDictionaryEnumerator iGridDictEnum = GridManagerHash.GetEnumerator();
			while(iGridDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iGridDictEnum.Current;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;
				ArrayList colHeaderCellList = gridControlManager.ColHeaderCellList;
				for(int i=0;i < colHeaderCellList.Count; i++)
				{
					ColHeaderCell colHeaderCell = colHeaderCellList[i] as ColHeaderCell;
					if(colHeaderCell.ID == itemID)
					{
						return colHeaderCell.IsVisible;
					}
				}
			}
			return false;
		}
		
		/// <summary>
		/// Method Finds if Row is Visible
		/// </summary>
		/// <param name="itemID"></param>
		/// <param name="GridManagerHash"></param>
		/// <returns></returns>
		private static bool IsRowVisible(string itemID,Hashtable GridManagerHash)
		{
			IDictionaryEnumerator iGridDictEnum = GridManagerHash.GetEnumerator();
			while(iGridDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iGridDictEnum.Current;
				GridControlManager gridControlManager = dEntry.Value as GridControlManager;
				ArrayList rowHeaderCellList = gridControlManager.SortManagerList;
				for(int i=0;i < rowHeaderCellList.Count; i++)
				{
					IOSortManager rowHeaderCell = rowHeaderCellList[i] as IOSortManager;
					if(rowHeaderCell.ID == itemID)
					{
						return rowHeaderCell.IsVisible;
					}
				}
			}
			return false;
		}


		/// <summary>
		/// Arranges Row List on Row Movement
		/// </summary>
		/// <param name="oldIndex"></param>
		/// <param name="newIndex"></param>
		/// <param name="countOFItems"></param>
		/// <param name="sortManagerList"></param>
		/// <param name="rowStart"></param>
		public static void RowMovement(int oldIndex,int newIndex,int countOFItems,ArrayList sortManagerList,int rowStart)
		{
			//Update the Row List
			int toMoveIndex = -1;
			int lastInsertIndex = -1;

			//Upward Movement
			if(oldIndex > newIndex)
			{
				toMoveIndex = oldIndex + (countOFItems - 1);
				lastInsertIndex = newIndex;
			}
			else//Downward Movement
			{
				toMoveIndex = oldIndex;
				lastInsertIndex = newIndex + countOFItems - 1;
			}

			for(int i=0;i<countOFItems;i++)
			{
				//Remove 
				IOSortManager sortManager = (IOSortManager)sortManagerList[toMoveIndex - rowStart];
				//Update the new indexes
				sortManagerList.RemoveAt(toMoveIndex - rowStart);
				sortManagerList.Insert(lastInsertIndex - rowStart,sortManager);
			}
		}

		/// <summary>
		/// Arranges Col List on Col Movement
		/// </summary>
		/// <param name="oldIndex"></param>
		/// <param name="newIndex"></param>
		/// <param name="countOFItems"></param>
		/// <param name="sortManagerList"></param>
		/// <param name="rowStart"></param>
		public static void ColMovement(int oldIndex,int newIndex,int countOFItems,ArrayList colList,int colStart)
		{
			//Update the Row List
			int toMoveIndex = -1;
			int lastInsertIndex = -1;

			//Upward Movement
			if(oldIndex > newIndex)
			{
				toMoveIndex = oldIndex + (countOFItems - 1);
				lastInsertIndex = newIndex;
			}
			else//Downward Movement
			{
				toMoveIndex = oldIndex;
				lastInsertIndex = newIndex + countOFItems - 1;
			}

			for(int i=0;i<countOFItems;i++)
			{
				//Remove 
				ColHeaderCell colHeaderCell = (ColHeaderCell)colList[toMoveIndex - colStart];
				//Update the new indexes
				colList.RemoveAt(toMoveIndex - colStart);
				colList.Insert(lastInsertIndex - colStart,colHeaderCell);
			}
		}

		/// <summary>
		/// Method clears the sorting
		/// </summary>
		public static void ClearSort(Hashtable GridManagerHash)
		{
			IEnumerator iEnum = GridManagerHash.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
				GridControl gridControl =  dEntry.Key as GridControl;
				if(gridControl.CurrentCell.IsEditing)
					gridControl.CurrentCell.EndEdit();
				//turn off the sort arrow
				for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				{
					if(gridControl[1, col1].CellType == "ColumnHeaderCell")
					{
						gridControl[1, col1].CellType = "Header";
						break;
					}
				}
			}
		}
		
		/// <summary>
		/// Resizes columns to fit
		/// </summary>
		/// <param name="GridManagerHash"></param>
		public static void ResizeToFit(Hashtable GridManagerHash,int DATA_ROW_START,int DATA_COL_START)
		{
			IEnumerator iEnum = GridManagerHash.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iEnum.Current;
				GridControl gridControl =  dEntry.Key as GridControl;
				gridControl.ColWidths.ResizeToFit(GridRangeInfo.Cells(DATA_ROW_START,DATA_COL_START,gridControl.RowCount,gridControl.ColCount),GridResizeToFitOptions.NoShrinkSize);
			}
		}
		
		/// <summary>
		/// Method handles UOM editing
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		/// <param name="uomIndex"></param>
		/// <param name="isTransposed"></param>
		public static void HandleUOMEditingKeyDown(object sender,KeyEventArgs e,int uomIndex,bool isTransposed)
		{
			//Check if the field is UOM field
			GridControl gridControl = sender as GridControl;
			int colIndex = gridControl.CurrentCell.ColIndex;
			int rowIndex = gridControl.CurrentCell.RowIndex;
			if(colIndex == uomIndex && !isTransposed)
			{
				e.Handled = true;
				return;
			}
			if(rowIndex == uomIndex && isTransposed)
			{
				e.Handled = true;
				return;
			}
		}

		/// <summary>
		/// Method handles UOM editing
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		/// <param name="uomIndex"></param>
		/// <param name="isTransposed"></param>
		public static void HandleUOMEditingKeyPress(object sender,KeyPressEventArgs e,int uomIndex,bool isTransposed)
		{
			//Check if the field is UOM field
			GridControl gridControl = sender as GridControl;
			int colIndex = gridControl.CurrentCell.ColIndex;
			int rowIndex = gridControl.CurrentCell.RowIndex;
			if(colIndex == uomIndex && !isTransposed)
			{
				e.Handled = true;
				return;
			}
			if(rowIndex == uomIndex && isTransposed)
			{
				e.Handled = true;
				return;
			}
		}
		#endregion

		#region Fields
		private static int SUPPORTED_COLLECTION_COUNT = 2;
		private static Hashtable		 hashCollectionIndex = new Hashtable();
		private static Comparer			_ioComparer = new Comparer();
		private static GridControlComparer _gridControlComparer = new GridControlComparer();
		private static LRRApp			_lrrApp;
		#endregion

		#region Properties
		public static Comparer IOComparer
		{
			get{return _ioComparer;}
			set
			{
				_ioComparer = value;
			}
		}

		public static GridControlComparer GridComparer
		{
			get{return _gridControlComparer;}
			set
			{
				_gridControlComparer = value;
			}
		}

		#endregion
	}
}
