using System;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Lightridge.LRRUI;
using System.Collections;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.MessageLogger;

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for AggregatesMapping.
	/// </summary>
	public class AggregatesMapping : ISymbolPropertiesMapping
	{

		#region Fields

		private LRRPlantItemProperties _selectedProps = new LRRPlantItemProperties();

		#endregion

		public AggregatesMapping()
		{

		}
	
		#region ISymbolPropertiesMapping Members

        /// <summary>
        /// Retrives the Properties Selected for a symbol in hashtable
        /// </summary>
        /// <param name="pt">PropertyTable for the Symbol</param>
        /// <param name="customCollection"></param>
        /// <returns>
        /// The hashtable containing the selected Properties for a symbol,Key:PropertyId,Value:Property Object
        /// </returns>
		public System.Collections.Hashtable GetSelectedProperties(PropertyTable pt,CustomCollection customCollection)
		{

			PropertyTable myPT = pt;
			Hashtable hashSelectedSiteCompProps = new Hashtable();

			Symcon.Lightridge.LRRUI.PropertyBag.PropertySpecCollection pscl = pt.Properties;
			foreach(PropertySpec ps in pscl)
			{
				if(ps.Category == "Bound Properties")
				{
					//LRRPlantItemProperty propertyObj=new LRRPlantItemProperty();
					LRRAggregateProperty lrrcp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(ps.ObjectRef.ToString());

					if(lrrcp != null)
					{
						IEnumerator enumerator = customCollection.GetEnumerator();
						while(enumerator.MoveNext())
						{
							LRRPlantItemProperty lrrplprop = enumerator.Current as LRRPlantItemProperty; 
							if(lrrplprop.ID == lrrcp.ID && lrrplprop.Name == lrrcp.Name)
							{
								if(!hashSelectedSiteCompProps.ContainsKey(lrrplprop.ID))
								{
									hashSelectedSiteCompProps.Add(lrrplprop.ID,lrrplprop);
									break;
								}
							}
						}
					}
				}
			}

			return hashSelectedSiteCompProps;
			
		}


        /// <summary>
        /// Fills the Property Holder Array with selected Properties and "NullValue" for the remaining
        /// </summary>
        /// <param name="collection">Selected Properties Collection</param>
        /// <param name="arryPropIndex">Property Holder Array associated with the symbol</param>
		public void AddSelection(object collection,ArrayList arryPropIndex)
		{
			LRRPlantItemProperties siteCompProps =  collection as LRRPlantItemProperties;
			if(arryPropIndex != null)
			{
				for(int j = 0;j<arryPropIndex.Count;j++)
				{
					if(arryPropIndex[j].ToString() == "Null")
					{
						LRRPlantItemProperty lrrscp = new LRRPlantItemProperty(); 
						lrrscp.Name = "NullValue";
						this._selectedProps.Add(lrrscp);
					}
					else
					{
						foreach(LRRPlantItemProperty lrrplprop in siteCompProps)
						{
							if(lrrplprop.ID == arryPropIndex[j].ToString())
							{
								this._selectedProps.Add(lrrplprop);
								break;
							}
						}
					}
				}
			}
		}


        /// <summary>
        /// Retrives Selected Properties
        /// </summary>
        /// <value></value>
		public CustomCollection SelectedProperties
		{
			get
			{
				return _selectedProps;
			}

		}


        /// <summary>
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="arryPropIndex"></param>
		public void UpdateSelection(object obj, ArrayList arryPropIndex)
		{
			LRRPlantItemProperty propertyObj=null;
			try
			{
				propertyObj =	obj	as LRRPlantItemProperty;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(SiteCompPropHistorianMapping),"MappingWithoutPET:"+ex.Message  ,Logger.Level.Debug);
				return;
			}

			if(arryPropIndex.Contains(propertyObj.ID))
			{
				int index = arryPropIndex.IndexOf(propertyObj.ID);
				arryPropIndex[index] = "Null";
				_selectedProps.RemoveAt(index);
				LRRPlantItemProperty lrrscp = new LRRPlantItemProperty();
				lrrscp.Name = "NullValue";
				_selectedProps.Insert(index,lrrscp);
			}
			else
			{
				for(int i=0;i < arryPropIndex.Count;i++)
				{
					if(arryPropIndex[i].ToString() == "Null")
					{
						arryPropIndex[i] = propertyObj.ID;
						_selectedProps.RemoveAt(i);
						_selectedProps.Insert(i,propertyObj);
						break;
					}
				}
			}
		}


        /// <summary>
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
		public string GetSelectedPropertyNameByIndex(int index)
		{
			return _selectedProps[index].Name;
		}


        /// <summary>
        /// Gets the data source.
        /// </summary>
        /// <param name="siteItemID">The site item ID.</param>
        /// <returns></returns>
		public LRRPlantItemProperties GetDataSource(string siteItemID)
		{
			Symcon.LightRidge.BusinessObjects.LRRAggregates	lrrAggregates = LRRModel.AggregatesObj;
			LRRAggregate lrrAggregate = lrrAggregates.GetAggregateFromID(siteItemID);
			LRRPlantItemProperties propertiesObj= new LRRPlantItemProperties();

			if(lrrAggregate != null)
			{
				//prop = LRRModel.SiteComponentPropertiesObj.GetSiteComponentPropertiesByComponentID(lsc.SiteComponentID);

				foreach(LRRAggregateProperty ex in lrrAggregate.AggregateProperties)
				{
					if(ex.ModeID)
					{
						LRRPlantItemProperty propertyObj=new LRRPlantItemProperty();
                        propertyObj.ID = ex.ID;
                        propertyObj.PropertyID = ex.ID;
                        propertyObj.Name = ex.Name;
                        propertyObj.Description = ex.Description;
                        propertyObj.SerializedID = ex.ID;
                        if (ex.ConditionalFormatXML != null)
                            propertyObj.CondFormatsObj = ex.ConditionalFormatsObj;
                        propertiesObj.Add(propertyObj);
                        propertyObj = null;
					}
				}
			}
			propertiesObj.SortColumn = "Name";
			propertiesObj.Sort();
			return propertiesObj;
		}

		
		#endregion
	}
}
