using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;
using System.IO;
using System.Resources;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters;
using System.Data.OleDb;

using Symcon.Lightridge.LRRUI;
using Symcon.Lightridge.LRRUI.Controls;
using Symcon.LightRidge.BusinessObjects.TypeConverters;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Utils.HeaderDispatcher;   

using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Tools.XPMenus;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;
using System.Collections.Generic;


namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for GridCellMappingDialog.
	/// </summary>
	public class GridCellMappingDialog : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Panel pnlMain;
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Button btnCncl;
		private System.Windows.Forms.Panel pnlBottom;
		private System.Windows.Forms.Splitter spltrBottom;
		private System.Windows.Forms.Panel pnlTop;
		private ISymbolPropertiesMapping _myISymbolMapping;
		private string _siteItemID = null;
		private System.Windows.Forms.Panel pnlGrid;
		private System.Windows.Forms.Splitter splitter1;
		private System.Windows.Forms.Panel pnlLeft;
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridSiteCompProps;
		private System.Windows.Forms.GroupBox groupBox1;
		private GridCell myGridCell = null;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public GridCellMappingDialog(ISymbolPropertiesMapping iSymbolMapping,PropertyTable pt, GridCell cell)
		{
			_myISymbolMapping = iSymbolMapping;
			myGridCell = cell;
			SiteItem si = cell.GetPropertyContainerByName("PlantItem") as SiteItem;
			_siteItemID = si.ID;
			CustomCollection customCollection = iSymbolMapping.GetDataSource(_siteItemID);
		
			InitializeComponent();
			gridSiteCompProps.TableDescriptor.Relations.Clear();
			gridSiteCompProps.DataSource = null;
			gridSiteCompProps.DataSource = customCollection;
			gridSiteCompProps.TableDescriptor.Columns.Clear();
			gridSiteCompProps.TableDescriptor.Relations.Clear();
			gridSiteCompProps.DataSource = null;
			gridSiteCompProps.DataSource = customCollection;
			gridSiteCompProps.TableDescriptor.Columns.Clear();
			gridSiteCompProps.TableDescriptor.Columns.Add("Name");
			gridSiteCompProps.TableDescriptor.Columns.Add("Description");
			gridSiteCompProps.TableDescriptor.Columns["Name"].Appearance.AnyRecordFieldCell.ReadOnly = true;
			gridSiteCompProps.TableDescriptor.Columns["Name"].HeaderText = LRRHeaderText.GetHeaderString("SID_NAME");
			gridSiteCompProps.TableDescriptor.Columns["Description"].HeaderText = LRRHeaderText.GetHeaderString("SID_DESCRIPTION");
			gridSiteCompProps.CurrentRecordContextChange+=new Syncfusion.Grouping.CurrentRecordContextChangeEventHandler(gridSiteCompProps_CurrentRecordContextChange);
		}

        public GridCellMappingDialog(ISymbolPropertiesMapping iSymbolMapping, PropertyTable pt, GridCell cell,bool isReMap)
        {
            _myISymbolMapping = iSymbolMapping;
            myGridCell = cell;
            SiteItem si = cell.GetPropertyContainerByName("PlantItem") as SiteItem;

            _siteItemID = si.ID;
            CustomCollection customCollection = iSymbolMapping.GetDataSource(_siteItemID);

            InitializeComponent();
            gridSiteCompProps.TableDescriptor.Relations.Clear();
            gridSiteCompProps.DataSource = null;
            gridSiteCompProps.DataSource = customCollection;
            gridSiteCompProps.TableDescriptor.Columns.Clear();
            gridSiteCompProps.TableDescriptor.Relations.Clear();
            gridSiteCompProps.DataSource = null;
            gridSiteCompProps.DataSource = customCollection;
            gridSiteCompProps.TableDescriptor.Columns.Clear();
            gridSiteCompProps.TableDescriptor.Columns.Add("Name");
            gridSiteCompProps.TableDescriptor.Columns.Add("Description");
            gridSiteCompProps.TableDescriptor.Columns["Name"].Appearance.AnyRecordFieldCell.ReadOnly = true;
            gridSiteCompProps.TableDescriptor.Columns["Name"].HeaderText = LRRHeaderText.GetHeaderString("SID_NAME");
            gridSiteCompProps.TableDescriptor.Columns["Description"].HeaderText = LRRHeaderText.GetHeaderString("SID_DESCRIPTION");
            gridSiteCompProps.CurrentRecordContextChange += new Syncfusion.Grouping.CurrentRecordContextChangeEventHandler(gridSiteCompProps_CurrentRecordContextChange);
            SetCurrentMappingActive(cell);
        }

        private void SetCurrentMappingActive(GridCell cell)
        {
            IPropertyValue iPropValue = cell.Tag as IPropertyValue;
            string propID = iPropValue.GetPropertyID();
            foreach (Syncfusion.Grouping.Record rec in this.gridSiteCompProps.Table.Records)
            {
                string recPropID = rec.GetData().GetType().GetProperty("PropertyID").GetValue(rec.GetData(), null).ToString();
                if (recPropID == propID)
                {
                    rec.SetCurrent();
                    rec.SetSelected(true);
                    break;
                }
            }

            int i = 0;

            foreach (Control ctl in this.groupBox1.Controls)
            {
                if(ctl.GetType() == typeof(RadioButton))
                {
                    RadioButton rb = ctl as RadioButton;
                    if (rb.Tag != null)
                    {
                        LRRMode mode = rb.Tag as LRRMode;
                        if (mode.ModeId == cell.ModeID)
                        {
                            rb.Checked = true;
                            break;
                        }
                    }
                }
            }
            
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.pnlMain = new System.Windows.Forms.Panel();
            this.pnlTop = new System.Windows.Forms.Panel();
            this.pnlLeft = new System.Windows.Forms.Panel();
            this.groupBox1 = new System.Windows.Forms.GroupBox();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.pnlGrid = new System.Windows.Forms.Panel();
            this.gridSiteCompProps = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
            this.spltrBottom = new System.Windows.Forms.Splitter();
            this.pnlBottom = new System.Windows.Forms.Panel();
            this.btnCncl = new System.Windows.Forms.Button();
            this.btnOK = new System.Windows.Forms.Button();
            this.pnlMain.SuspendLayout();
            this.pnlTop.SuspendLayout();
            this.pnlLeft.SuspendLayout();
            this.pnlGrid.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.gridSiteCompProps)).BeginInit();
            this.pnlBottom.SuspendLayout();
            this.SuspendLayout();
            // 
            // pnlMain
            // 
            this.pnlMain.Controls.Add(this.pnlTop);
            this.pnlMain.Controls.Add(this.spltrBottom);
            this.pnlMain.Controls.Add(this.pnlBottom);
            this.pnlMain.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlMain.Location = new System.Drawing.Point(0, 0);
            this.pnlMain.Name = "pnlMain";
            this.pnlMain.Size = new System.Drawing.Size(729, 344);
            this.pnlMain.TabIndex = 0;
            // 
            // pnlTop
            // 
            this.pnlTop.Controls.Add(this.pnlLeft);
            this.pnlTop.Controls.Add(this.splitter1);
            this.pnlTop.Controls.Add(this.pnlGrid);
            this.pnlTop.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlTop.Location = new System.Drawing.Point(0, 0);
            this.pnlTop.Name = "pnlTop";
            this.pnlTop.Size = new System.Drawing.Size(729, 293);
            this.pnlTop.TabIndex = 2;
            // 
            // pnlLeft
            // 
            this.pnlLeft.Controls.Add(this.groupBox1);
            this.pnlLeft.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlLeft.Location = new System.Drawing.Point(474, 0);
            this.pnlLeft.Name = "pnlLeft";
            this.pnlLeft.Size = new System.Drawing.Size(255, 293);
            this.pnlLeft.TabIndex = 2;
            // 
            // groupBox1
            // 
            this.groupBox1.Location = new System.Drawing.Point(24, 16);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(216, 256);
            this.groupBox1.TabIndex = 0;
            this.groupBox1.TabStop = false;
            this.groupBox1.Text = "Modes";
            // 
            // splitter1
            // 
            this.splitter1.Location = new System.Drawing.Point(471, 0);
            this.splitter1.Name = "splitter1";
            this.splitter1.Size = new System.Drawing.Size(3, 293);
            this.splitter1.TabIndex = 1;
            this.splitter1.TabStop = false;
            // 
            // pnlGrid
            // 
            this.pnlGrid.Controls.Add(this.gridSiteCompProps);
            this.pnlGrid.Dock = System.Windows.Forms.DockStyle.Left;
            this.pnlGrid.Location = new System.Drawing.Point(0, 0);
            this.pnlGrid.Name = "pnlGrid";
            this.pnlGrid.Size = new System.Drawing.Size(471, 293);
            this.pnlGrid.TabIndex = 0;
            // 
            // gridSiteCompProps
            // 
            this.gridSiteCompProps.BackColor = System.Drawing.SystemColors.Window;
            this.gridSiteCompProps.CausesValidation = false;
            this.gridSiteCompProps.Dock = System.Windows.Forms.DockStyle.Fill;
            this.gridSiteCompProps.Location = new System.Drawing.Point(0, 0);
            this.gridSiteCompProps.Name = "gridSiteCompProps";
            this.gridSiteCompProps.Size = new System.Drawing.Size(471, 293);
            this.gridSiteCompProps.TabIndex = 0;
            this.gridSiteCompProps.TableDescriptor.AllowEdit = false;
            this.gridSiteCompProps.TableDescriptor.AllowNew = false;
            this.gridSiteCompProps.TableDescriptor.AllowRemove = false;
            this.gridSiteCompProps.Text = "gridGroupingControl1";
            this.gridSiteCompProps.TopLevelGroupOptions.CaptionText = "Attributes: {RecordCount} Items";
            this.gridSiteCompProps.TopLevelGroupOptions.IsExpandedInitialValue = true;
            this.gridSiteCompProps.TopLevelGroupOptions.ShowCaptionPlusMinus = false;
            this.gridSiteCompProps.TopLevelGroupOptions.ShowColumnHeaders = true;
            this.gridSiteCompProps.VersionInfo = "3.301.0.0";
            // 
            // spltrBottom
            // 
            this.spltrBottom.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.spltrBottom.Enabled = false;
            this.spltrBottom.Location = new System.Drawing.Point(0, 293);
            this.spltrBottom.Name = "spltrBottom";
            this.spltrBottom.Size = new System.Drawing.Size(729, 3);
            this.spltrBottom.TabIndex = 1;
            this.spltrBottom.TabStop = false;
            // 
            // pnlBottom
            // 
            this.pnlBottom.BackColor = System.Drawing.SystemColors.Control;
            this.pnlBottom.Controls.Add(this.btnCncl);
            this.pnlBottom.Controls.Add(this.btnOK);
            this.pnlBottom.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.pnlBottom.Location = new System.Drawing.Point(0, 296);
            this.pnlBottom.Name = "pnlBottom";
            this.pnlBottom.Size = new System.Drawing.Size(729, 48);
            this.pnlBottom.TabIndex = 0;
            // 
            // btnCncl
            // 
            this.btnCncl.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.btnCncl.Location = new System.Drawing.Point(551, 13);
            this.btnCncl.Name = "btnCncl";
            this.btnCncl.Size = new System.Drawing.Size(75, 23);
            this.btnCncl.TabIndex = 1;
            this.btnCncl.Text = "Cancel";
            // 
            // btnOK
            // 
            this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.btnOK.Location = new System.Drawing.Point(639, 13);
            this.btnOK.Name = "btnOK";
            this.btnOK.Size = new System.Drawing.Size(75, 23);
            this.btnOK.TabIndex = 0;
            this.btnOK.Text = "OK";
            this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
            // 
            // GridCellMappingDialog
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(729, 344);
            this.Controls.Add(this.pnlMain);
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "GridCellMappingDialog";
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Cell Mapping Dialog";
            this.pnlMain.ResumeLayout(false);
            this.pnlTop.ResumeLayout(false);
            this.pnlLeft.ResumeLayout(false);
            this.pnlGrid.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.gridSiteCompProps)).EndInit();
            this.pnlBottom.ResumeLayout(false);
            this.ResumeLayout(false);

		}
		#endregion

		private void gridSiteCompProps_CurrentRecordContextChange(object sender, Syncfusion.Grouping.CurrentRecordContextChangeEventArgs e)
		{
			if(e.Action == Syncfusion.Grouping.CurrentRecordAction.NavigateComplete)
			{
                gridSiteCompProps.Table.SelectedRecords.Clear();
				LRRPlantItemProperty plProp =  gridSiteCompProps.CurrencyManager.Current as LRRPlantItemProperty;
				IProperty						myProp = null;
                lscpv = new List<IPropertyValue>();
				if(plProp.ID != null)
				{
					if(plProp.GroupPropertyID == null && plProp.ID != null)
					{
						if(_myISymbolMapping is Symcon.LightRidge.PEAdvisor.AggregatesMapping)
						{
							myProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(plProp.ID);
						}
						else
						{
							myProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(plProp.ID);
						}
					}
					else if(plProp.GroupPropertyID != null)
					{
						myProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(plProp.GroupPropertyID);
					}
				}
				if(_myISymbolMapping is  Symcon.LightRidge.PEAdvisor.AggregatesMapping)
				{
					lscpv = LRRModel.AggregatePropertyValuesObj.GetPropValuesByPropID(plProp.ID);   
				}
				else
				{
					if(myProp.GetPropertyType() == "1")
					{
						lscpv = LRRModel.PropertyValuesObj.GetPropValuesByPropID(myProp.GetID());     
					}
					else
					{
						if(plProp.LookupQualifierID != null && plProp.LookupQualifierID.Trim() != "")
							lscpv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValuesFromGrpPropIdAndLkupQualId(plProp.GroupPropertyID,plProp.LookupQualifierID);
						else
							lscpv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifierValuesinArrayFromGroupPropID(plProp.GroupPropertyID);
					}
				}
				if(lscpv != null)
				{
					groupBox1.Controls.Clear();
					int yPos = 30;
					lscpv.Sort(_propertyValuesSortOnMode);
					
					foreach(IPropertyValue ipropVal in lscpv)
					{
						LRRMode lrrmode = LRRModel.ProjectModes.GetModeByID(ipropVal.GetModeID());
						RadioButton rb = new RadioButton();
						rb.Text = lrrmode.ModeName;
						rb.Tag = lrrmode;
						groupBox1.Controls.Add(rb);
						rb.Location = new Point(20,yPos);
						yPos = yPos+30;
					}
				}
			_selectedItem = myProp;	
			}
		}



        public List<IPropertyValue> SelectedValues
		{
			get
			{
				return lscpv;
			}
		}

        private List<IPropertyValue> lscpv = new List<IPropertyValue>();

		private LRRMode _selectedMode;
		private IProperty _selectedItem;
        private PropertyValuesSorter<IPropertyValue> _propertyValuesSortOnMode = new PropertyValuesSorter<IPropertyValue>();


		private void btnOK_Click(object sender, System.EventArgs e)
		{
			bool isSelected = false;
			foreach(RadioButton rb in groupBox1.Controls)
			{
				if(rb.Checked)
				{
					LRRPlantItemProperty plProp  = this.gridSiteCompProps.CurrencyManager.Current as LRRPlantItemProperty;
					//string uiUOMID = "";
					//string modelUOMID = "";
					GraphicsUtilities.GetUIUOMIDForProperty(this.SelectedItem, plProp,ref _uiUOMID, ref _uomTypeID);

					this._selectedMode = rb.Tag as LRRMode;
					_iPropVal = this.SelectedItem.GetPropertyValueForMode(_selectedMode.ModeId) as IPropertyValue;
					isSelected = true;
					break;
				}
			}
			if(!isSelected || lscpv.Count == 0)
			{
				this.DialogResult = DialogResult.None;
			}
		}
	
		public LRRMode SelectedMode
		{
			get
			{
				return _selectedMode;
			}
		}

		public IProperty SelectedItem
		{
			get
			{
				return _selectedItem;
			}
			set
			{
				_selectedItem = value;
			}
		}


		private IPropertyValue _iPropVal;

		public IPropertyValue PropertyValue
		{
			get
			{
				return _iPropVal;
			}
		}

		private string _uiUOMID;

		public string UIUOMID
		{
			get
			{
				return _uiUOMID;
			}
		}


		private string _uomTypeID;

		public string UOMTypeID
		{
			get
			{
				return _uomTypeID;
			}
		}
	}

    public class PropertyValuesSorter<T> : IComparer<T>
	{
        #region IComparer<T> Members

        public int Compare(T x, T y)
        {
            IPropertyValue iPropVal1 = x as IPropertyValue;
            IPropertyValue iPropVal2 = y as IPropertyValue;
            int val1 = System.Convert.ToInt32(iPropVal1.GetModeID());
            int val2 = System.Convert.ToInt32(iPropVal2.GetModeID());
            return val1.CompareTo(val2);
        }

        #endregion
    }

}
