using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Diagnostics;

using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;

using Symcon.Utils.MessageLogger;
using Symcon.Database;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;

namespace Symcon.LightRidge.PEAdvisor
{
    public partial class RollUpValues : Form , ILRRUIContainer 
    {
        public RollUpValues()
        {
            InitializeComponent();
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitDataSource();
            BindGrid();
            if (_levels.Count > 0)
            {
                InitializeTreeView(trvLevels, _levels, _lrrLevelLinks);
                trvLevels.TreeViewNodeSorter = new TreeNodeSorter();
                trvLevels.Sort();
            }
        }

        private bool InitializeTreeView(TreeView treeView, LRRLevels levels, LRRLevelLinks levelLinks)
        {
            treeView.Nodes.Clear();
            try
            {
                Hashtable _levelHash = new Hashtable();
                Hashtable _nodesHash = new Hashtable();
                Hashtable _childParent = new Hashtable();

                foreach (LRRLevel l in levels)
                {
                    _levelHash.Add(l.LevelID, l);
                }
                if (levelLinks.Count == 0 && levels.Count == 1)
                {
                    //In here only when we have single entry on Level and No entry in Level Links
                    LRRLevel lonerLevel = _levelHash["0"] as LRRLevel;
                    if (lonerLevel != null)
                    {
                        TreeNode singleRootNode = new TreeNode(lonerLevel.Name);
                        singleRootNode.Tag = lonerLevel;
                        treeView.Nodes.Add(singleRootNode);
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
                foreach (LRRLevelLink ll in levelLinks)
                {
                    _childParent.Add(ll.ChildID, ll.ParentID);

                    if (_nodesHash.ContainsKey(ll.ParentID))
                    {
                        TreeNode foundNode = _nodesHash[ll.ParentID] as TreeNode;
                        LRRLevel foundLevel = _levelHash[ll.ChildID] as LRRLevel;
                        TreeNode childNode = new TreeNode(foundLevel.Name);
                        childNode.Tag = foundLevel;
                        foundNode.Nodes.Add(childNode);
                    }
                    else
                    {
                        LRRLevel parentLevel = _levelHash[ll.ParentID] as LRRLevel;
                        LRRLevel childLevel = _levelHash[ll.ChildID] as LRRLevel;
                        TreeNode parentNode = new TreeNode(parentLevel.Name);
                        parentNode.Tag = parentLevel;
                        TreeNode childNode = new TreeNode(childLevel.Name);
                        childNode.Tag = childLevel;
                        parentNode.Nodes.Add(childNode);
                        _nodesHash.Add(ll.ParentID, parentNode);
                    }
                }

                foreach (object k in _childParent.Keys)
                {
                    string childID = (string)k;
                    string parentID = (string)_childParent[k];
                    TreeNode childNode = null;
                    TreeNode parentNode = null;
                    if (_nodesHash.ContainsKey(childID))
                    {
                        childNode = _nodesHash[childID] as TreeNode;
                    }
                    if (_nodesHash.ContainsKey(parentID))
                    {
                        parentNode = _nodesHash[parentID] as TreeNode;
                    }
                    if ((parentNode != null) && (childNode != null))
                    {
                        int index = -1;
                        foreach (TreeNode f in parentNode.Nodes)
                        {
                            if (f.Tag == childNode.Tag)
                            {
                                index = f.Index;
                                break;
                            }
                        }
                        if (index != -1)
                            parentNode.Nodes[index] = childNode;
                    }
                }

                TreeNode rootNode = _nodesHash["0"] as TreeNode;
                rootNode.ExpandAll();
                treeView.Nodes.Add(rootNode);
                return true;
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(PlantLevels), "PopulateLevels:" + ex.Message, Logger.Level.Debug);
                return false;
            }
        }

        private void BindGrid()
        {
            Hashtable fkColumns;
            Hashtable childColumns;
            Symcon.LightRidge.BusinessObjects.Utilities.LRRBusinessObjectUtilities.GetObjectInfo(_rollUpValues.ItemType, out fkColumns, out childColumns);
            Hashtable fkData = new Hashtable();
            foreach (object obj in fkColumns.Keys)
            {
                string propName = (string)obj;
                SymForeignKeyFieldAttribute fkAttr = fkColumns[obj] as SymForeignKeyFieldAttribute;
                object collectionObject = Activator.CreateInstance(fkAttr.CollectionType);
                _lrrApp.DBManagerObj.PopulateCollection(collectionObject, false);
                fkData.Add(propName, collectionObject);
            }
            SyncGridUtilities.InitializeGridLookAndFeel(gridRollUpValues);
            SyncGridUtilities.InitializeGridColumns(gridRollUpValues, typeof(LRRRollUpValue), null, fkData);
            SetGridColumnStyles();
        }

        private void BindGrid(LRRRollUPValues rollUpValues)
        {
            gridRollUpValues.Engine.SourceListSet.Clear();
            gridRollUpValues.DataSource = null;
            gridRollUpValues.TableDescriptor.Relations.Clear();
            Hashtable fkColumns;
            Hashtable childColumns;
            Symcon.LightRidge.BusinessObjects.Utilities.LRRBusinessObjectUtilities.GetObjectInfo(_rollUpValues.ItemType, out fkColumns, out childColumns);
            Hashtable fkData = new Hashtable();
            foreach (object obj in fkColumns.Keys)
            {
                string propName = (string)obj;
                SymForeignKeyFieldAttribute fkAttr = fkColumns[obj] as SymForeignKeyFieldAttribute;
                object collectionObject = Activator.CreateInstance(fkAttr.CollectionType);
                _lrrApp.DBManagerObj.PopulateCollection(collectionObject, false);
                fkData.Add(propName, collectionObject);
            }
            SyncGridUtilities.InitializeGridLookAndFeel(gridRollUpValues);
            SyncGridUtilities.InitializeGridColumns(gridRollUpValues, typeof(LRRRollUpValue), null, fkData);
            gridRollUpValues.DataSource = rollUpValues;
            gridRollUpValues.Engine.SourceListSet.Add(rollUpValues);
            SetGridColumnStyles();
        }

        private void SetGridColumnStyles()
        {
            gridRollUpValues.TableDescriptor.TopLevelGroupOptions.ShowFilterBar = false;
            foreach (GridColumnDescriptor gcd in gridRollUpValues.TableDescriptor.Columns)
            {
                gcd.ReadOnly = true;
                gcd.Appearance.AnyRecordFieldCell.ShowButtons = Syncfusion.Windows.Forms.Grid.GridShowButtons.Hide;
                gcd.Appearance.AnyRecordFieldCell.ExclusiveChoiceList = false;
            }
        }


        /*public void RefreshView(LRRRollUPValues rollUpValues)
        {
            //Get the Current selected level
            try
            {
                gridRollUpValues.SuspendLayout();
                //ApplicationPersistanceUtil.SaveGridSettings(gridRollUpValues, this, this.Tag);
                gridRollUpValues.Engine.SourceListSet.Clear();
                gridRollUpValues.DataSource = null;
                gridRollUpValues.TableDescriptor.Relations.Clear();
                Hashtable fkColumns;
                Hashtable childColumns;
                Symcon.LightRidge.BusinessObjects.Utilities.LRRBusinessObjectUtilities.GetObjectInfo(_rollUpValues.ItemType, out fkColumns, out childColumns);
                Hashtable fkData = new Hashtable();
                foreach (object obj in fkColumns.Keys)
                {
                    string propName = (string)obj;
                    SymForeignKeyFieldAttribute fkAttr = fkColumns[obj] as SymForeignKeyFieldAttribute;
                    object collectionObject = Activator.CreateInstance(fkAttr.CollectionType);
                    _lrrApp.DBManagerObj.PopulateCollection(collectionObject, false);
                    fkData.Add(propName, collectionObject);
                }
                SyncGridUtilities.InitializeGridLookAndFeel(gridRollUpValues);
                SyncGridUtilities.InitializeGridColumns(gridRollUpValues, typeof(LRRRollUpValue), null, fkData);
                gridRollUpValues.DataSource = rollUpValues;
                gridRollUpValues.Engine.SourceListSet.Add(rollUpValues);
                SetGridColumnStyles();
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(PlantLevels), "RefreshView:" + ex.Message, Logger.Level.Debug);
                this.Cursor = Cursors.Default;
            }
        }*/
        private void InitDataSource()
        {
            _levels = LRRModel.ChildModel.Levels;
            _rollUpValues = LRRModel.RollUpValues; 
            _lrrLevelLinks = new LRRLevelLinks();
            _lrrApp.DBManagerObj.PopulateCollection(_lrrLevelLinks);
        }

        #region Fields
        private LRRLevels _levels;
        private LRRLevelLinks _lrrLevelLinks;
        private LRRApp _lrrApp;
        private LRRRollUPValues _rollUpValues;
        private TreeNode prevSelNode = null;
        private TabControl _parentTabControl;
        private TabPage _parentTabPage;
        #endregion


        private void trvLevels_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (prevSelNode != null)
            {
                prevSelNode.ForeColor = Color.Black;
            }
            e.Node.ForeColor = Color.Blue;
            prevSelNode = e.Node;
            LRRLevel level = prevSelNode.Tag as LRRLevel;
            LRRRollUPValues rollUpValues = _rollUpValues.GetRollUpValuesCollByLevel(level.LevelID);
            BindGrid(rollUpValues);
        }

        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Copy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyMasterProperties()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Delete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DownLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Export()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void InsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenQuery()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public TabControl ParentTabControl
        {
            set
            {
                _parentTabControl = value;
            }
            get { return _parentTabControl; }
        }

        public TabPage ParentTabPage
        {
            set
            {
                _parentTabPage = value;
            }
            get
            {
                return _parentTabPage;
            }
        }

        public void Paste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool PerformDirtyChk
        {
            get
            {
                // TODO:  Add MasterPlantItems.PerformDirtyChk getter implementation
                return true;
            }
        }

        public void Print()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PrintPreview()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Save()
        {
            LRREffectivePrices effectivePrices = LRRModel.EffectivePrices.GetChanges();
            if (effectivePrices.Count > 0)
            {
                _lrrApp.DBManagerObj.CommitCollection(effectivePrices);
                LRRModel.EffectivePrices.RefreshCollection();
                LRRMessagBox.ShowMessage("SID_SAVEDINFO", true);
            }
            else
            {
                LRRMessagBox.ShowMessage("SID_NODATASAVED", true);
            }
        }

        public void SelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowComponentList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowPropertyList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsArrangeColumns()
        {
            return false;
        }

        public bool SupportsClearFilter()
        {
            return false;
        }

        public bool SupportsClearSort()
        {
            return false;
        }

        public bool SupportsCompositeView()
        {
            return false;
        }

        public bool SupportsCopy()
        {
            return false;
        }

        public bool SupportsCopyUOMTypes()
        {
            return false;
        }

        public bool SupportsDelete()
        {
            return false;
        }

        public bool SupportsExport()
        {
            return false;
        }

        public bool SupportsImportToExcel()
        {
            return false;
        }

        public bool SupportsInsertNewRows()
        {
            return false;
        }

        public bool SupportsLevel()
        {
            return false;
        }

        public bool SupportsMasterPropertiesCopy()
        {
            return false;
        }

        public bool SupportsPageSetup()
        {
            return false;
        }

        public bool SupportsPaste()
        {
            return false;
        }

        public bool SupportsPrint()
        {
            return false;
        }

        public bool SupportsRefresh()
        {
            return false;
        }

        public bool SupportsResultsFilter()
        {
            return false;
        }

        public bool SupportsSave()
        {
            return true;
        }

        public bool SupportsSelectionPaste()
        {
            return false;
        }

        public bool SupportsUndo()
        {
            return false;
        }

        public void TransposeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Undo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }
        #endregion
    }
}