using System;
using System.Collections;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.PEAdvisor;
using Symcon.Utils.XMLUtils;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Tools;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;   
using Symcon.ColorSchemes;
using Symcon.BusinessObjects; 
using Symcon.LightRidge.PEAdvisor.GridUtilities;

//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.


namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for LRRUIManager.
	/// </summary>
	public class LRRUIManager
	{
		public delegate void ShowRunStatusDialogDel();
        public delegate void ShowRunStatusDialogWithParamsDel(string startTime,string endTime,string dataWindowSize);
        public delegate void RollBackChangesDel(Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl ggc);   

		public LRRUIManager()
		{
			_windowStore = new Hashtable();
			//Get the App Object
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

			_colorScheme = Schemes.GetActiveColorScheme(System.Windows.Forms.Application.StartupPath + "\\config\\ColorSchemes.xml");
			
		}
		#region Constructor 
		public LRRUIManager(PEMainWindow mainFrame)
		{
			_mainFrame = mainFrame;
			_mainFrame.MdiChildActivate+=new EventHandler(_mainFrame_MdiChildActivate);
			_windowStore = new Hashtable();
			_mainFrame.TabbedMDIChildList.SelectedIndexChanged+=new EventHandler(TabbedMDIChildList_SelectedIndexChanged);
			_runStatusDialog = new RunStatusDialog();
			_runStatusDialog.Owner = _mainFrame;
			_propertyDetailsDialog.Owner = _mainFrame;
			
			_dataQuality.Owner =_mainFrame;
			_timeWindow.Owner = _mainFrame;
			//_recordedPropsWindow.Owner = _mainFrame;
			//_messageSubscriptionDialog.Owner = _mainFrame;
            //_trendedPropsDialog.Owner = _mainFrame;

			_overridesWindow.Owner = mainFrame;
			//Get the App Object
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			_app.TimeWindowObj = _timeWindow;

			_app.OverridesWindow = _overridesWindow;

			_colorScheme = Schemes.GetActiveColorScheme(System.Windows.Forms.Application.StartupPath + "\\config\\ColorSchemes.xml");
			_runStatusDialog.Closing+=new System.ComponentModel.CancelEventHandler(_runStatusDialog_Closing);
		}
		#endregion

		#region Methods
		public Form GetGraphicsForm()
		{
			if(_windowStore.ContainsKey(160))
			{
				return _windowStore[160] as Form;
			}
			else if(_windowStore.ContainsKey(270))
			{
				return _windowStore[270] as Form;
			}
			else 
			{
				return null;
			}
		}

		public ImplementerScreen GetImplementerScreen()
		{
			if(_windowStore.ContainsKey(160))
			{
				return _windowStore[160] as ImplementerScreen;
			}
			else
			{
				return null;
			}
		}

		public OperatorsScreen GetOperatorScreen()
		{
			if(_windowStore.ContainsKey(270))
			{
				return _windowStore[270] as OperatorsScreen;
			}
			else
			{
				return null;
			}
		}

		public bool ShowWindow(int id)
		{
			try
			{
				if(_windowStore.ContainsKey(id))
				{
					Form frm =_windowStore[id] as Form;
					if(frm.WindowState == System.Windows.Forms.FormWindowState.Minimized)
						frm.WindowState = System.Windows.Forms.FormWindowState.Normal;
					frm.Select();
					return true;
				}
				else
				{
					Form frm = CreateForm(id);
					frm.Load+=new EventHandler(frm_Load);
					frm.Closed+=new EventHandler(frm_Closed);
					frm.Closing+=new System.ComponentModel.CancelEventHandler(frm_Closing);
					frm.Activated+=new EventHandler(frm_Activated);

					ILRRUIContainer cont = frm as ILRRUIContainer;
					//Add a Tabpage and enables it
					TabPage tp = new TabPage();
					tp.Parent = _mainFrame.TabbedMDIChildList;
					string frmText  = LRRHeaderText.GetHeaderString(frm.Tag.ToString());
					if(frmText.StartsWith("&"))
					{
						string s = frmText.Remove(0,1);
						frm.Text = s;
					}
					else
					{
						frm.Text = frmText;
					}
					tp.Text = frm.Text;
					tp.Show();
					//child Form will now hold a reference value to a tabpage
					cont.ParentTabPage = tp;

					_windowStore.Add(id,frm);
					frm.MdiParent = _mainFrame;
					frm.Show();
					return true;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRUIManager),"ShowWindow:"+ex.Message  ,Logger.Level.Debug);

				return false;
			}
		}

		public bool ShowWindow(string guid,Type type,object[] parameters)
		{
			try
			{
				if(_windowStore.ContainsKey(guid))
				{
					Form frm =_windowStore[guid] as Form;
					if(frm.WindowState == System.Windows.Forms.FormWindowState.Minimized)
						frm.WindowState = System.Windows.Forms.FormWindowState.Normal;
					frm.Select();
					return true;
				}
				else
				{
					Form frm = CreateForm(type,parameters);
					if(frm == null)
					{
						return false;
					}
					frm.Load+=new EventHandler(frm_Load);
					frm.Closed+=new EventHandler(frm_Closed);
					frm.Closing+=new System.ComponentModel.CancelEventHandler(frm_Closing);
					frm.Activated+=new EventHandler(frm_Activated);

					ILRRUIContainer cont = frm as ILRRUIContainer;
					//Add a Tabpage and enables it
					TabPage tp = new TabPage();
					tp.Parent = _mainFrame.TabbedMDIChildList;
					tp.Text = frm.Text;
					tp.Show();
					//child Form will now hold a reference value to a tabpage
					cont.ParentTabPage = tp;

					_windowStore.Add(guid,frm);
					frm.MdiParent = _mainFrame;
					frm.Tag = guid;
					frm.Show();
                    frm.Text = tp.Text;
					return true;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRUIManager),"ShowWindow:"+ex.Message  ,Logger.Level.Debug);

				return false;
			}
		}
		public bool ShowWindow(int id,ArrayList arrParams)//(int id,string ID)
		{
			try
			{
				if(_windowStore.ContainsKey(id))
				{
					Form frm =_windowStore[id] as Form;
					if(id!=310)
						frm.WindowState = System.Windows.Forms.FormWindowState.Maximized;
					else
					{
						frm.Show();
						frm.WindowState = System.Windows.Forms.FormWindowState.Normal;
					}
					frm.Select();
					return true;
				}
				else
				{
					Form frm = CreateForm(id,arrParams);//CreateForm(id,ID);
					frm.Load+=new EventHandler(frm_Load);
					frm.Closed+=new EventHandler(frm_Closed);
					frm.Closing+=new System.ComponentModel.CancelEventHandler(frm_Closing);
					frm.Activated+=new EventHandler(frm_Activated);

					if(id!=310)
					{
						ILRRUIContainer cont = frm as ILRRUIContainer;
						//Add a Tabpage and enables it
						TabPage tp = new TabPage();
						tp.Parent = _mainFrame.TabbedMDIChildList;
						string frmText  = LRRHeaderText.GetHeaderString(frm.Tag.ToString());
						if(frmText.StartsWith("&"))
						{
							string s = frmText.Remove(0,1);
							frm.Text = s;
						}
						else
						{
							frm.Text = frmText;
						}
						tp.Text = frm.Text;
						tp.Show();
						//child Form will now hold a reference value to a tabpage
						cont.ParentTabPage = tp;
						frm.MdiParent = _mainFrame;
						
					}
					else
					{
						string frmText  = LRRHeaderText.GetHeaderString(frm.Tag.ToString());
						frm.Text = frmText;
						frm.Owner = _mainFrame;
					}
					_windowStore.Add(id,frm);
					frm.Show();
					return true;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRUIManager),"ShowWindow:"+ex.Message  ,Logger.Level.Debug);

				return false;
			}
		}
		
		/// <summary>
		/// This form shows multiple windows for the same form
		/// </summary>
		/// <param name="id"></param>
		/// <param name="arrParams"></param>
		/// <param name="showMultipleForms"></param>
		/// <returns></returns>
		public bool ShowWindow(int id,ArrayList arrParams,string name,bool showMultipleForms)
		{
			try
			{
				if(_windowStore.ContainsKey(id + name))
				{
					Form frm =_windowStore[id + name] as Form;
					frm.WindowState = System.Windows.Forms.FormWindowState.Maximized;
					frm.Select();
					return true;
				}
				else
				{
					Form frm = CreateForm(id,arrParams);
					frm.Load+=new EventHandler(frm_Load);
					frm.Closed+=new EventHandler(frm_Closed);
					frm.Closing+=new System.ComponentModel.CancelEventHandler(frm_Closing);
					frm.Activated+=new EventHandler(frm_Activated);
					frm.WindowState = System.Windows.Forms.FormWindowState.Maximized;

					ILRRUIContainer cont = frm as ILRRUIContainer;
					//Add a Tabpage and enable it
					TabPage tp = new TabPage();
					tp.Parent = _mainFrame.TabbedMDIChildList;
					tp.Text = frm.Text + "-[" + name + "]";
					tp.Show();
					//child Form will now hold a reference value to a tabpage
					cont.ParentTabPage = tp;
					frm.MdiParent = _mainFrame;
					_windowStore.Add(id + name,frm);
					frm.Show();
					return true;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRUIManager),"ShowWindow:"+ex.Message  ,Logger.Level.Debug);
				return false;
			}
		}

        public bool RunStatusWindowVisible = false;

        public void UpdateStartAndEndTimeOnRunStatusDialog(string startTime, string endTime, string dataWindowSize)
        {
            //_runStatusDialog.
        }

		public void ShowRunStatusDialog()
		{
			if(_mainFrame.InvokeRequired)
			{
				_mainFrame.Invoke(new ShowRunStatusDialogDel(this.ShowRunStatusDialog),null);
			}
			else
			{
				if(_runStatusDialog == null || _runStatusDialog.IsDisposed)
				{
					_runStatusDialog = new RunStatusDialog();
					_runStatusDialog.Closing+=new System.ComponentModel.CancelEventHandler(_runStatusDialog_Closing);
					_runStatusDialog.Owner = _mainFrame;
				}

			_runStatusDialog.ResetProgressBar();
			
			 if(_app.GetOptions().ShowRunStatus)
				{
					_runStatusDialog.Owner = _mainFrame; 
				_runStatusDialog.Show();
                RunStatusWindowVisible = true;
					//_mainFrame.Activate();
				}
			}
				 
		}

        public void ShowRunStatusDialog(string startTime,string endTime,string dataWindowSize)
        {
            if (_mainFrame.InvokeRequired)
            {
                _mainFrame.Invoke(new ShowRunStatusDialogWithParamsDel(this.ShowRunStatusDialog),new object[]{startTime,endTime,dataWindowSize});
            }
            else
            {
                if (_runStatusDialog == null || _runStatusDialog.IsDisposed)
                {
                    _runStatusDialog = new RunStatusDialog();
                    _runStatusDialog.Closing += new System.ComponentModel.CancelEventHandler(_runStatusDialog_Closing);
                    _runStatusDialog.Owner = _mainFrame;

                }

                _runStatusDialog.ResetProgressBar();

                if (_app.GetOptions().ShowRunStatus)
                {
                    _runStatusDialog.Owner = _mainFrame;
                    _runStatusDialog.SetContentOnDialog(startTime, endTime, dataWindowSize);
                    _runStatusDialog.Show();
                    RunStatusWindowVisible = true;
                    //_mainFrame.Activate();
                }
            }

        }

		public void HideRunControlPanelDialog()
		{
			_runControlPanelDialog.Hide();
		}
		public void SetMessageOnRunStatusDialog(string message)
		{
			if(_runStatusDialog != null)
				_runStatusDialog.SetMessage(message);
		}
		public void HideRunStatusDialog()
		{
			_runStatusDialog.Hide();
		}


		public void InitializeModesDialog()
		{
			_modesDialog.InitializeModes();
		}


		public void CheckTrendUOMChanged(IProperty iProp, string modeID, string currentUOM)
		{
			foreach(string trendScheme in _htFrmTrends.Keys)
			{
				Form frm = _htFrmTrends[trendScheme] as Form;
				if(frm is Trend)
				{
					Trend frmTrend = _htFrmTrends[trendScheme] as Trend;
					if(frmTrend == null || (frmTrend as Form).IsDisposed)
						continue;
					else
					{
						if(frmTrend.IsTrendUOMChanged(trendScheme,iProp, modeID, currentUOM))
						{
							ShowTrend(trendScheme,iProp, modeID, currentUOM);
						}
					}
				}
			}
		}

		public void CheckTrendUOMChanged(IPropertyValue iPropValue, string currentUOM)
		{
			foreach(string trendScheme in _htFrmTrends.Keys)
			{
				Form frm = _htFrmTrends[trendScheme] as Form;
				if(frm is TrendGrouped)
				{
					TrendGrouped frmTrend = frm as TrendGrouped;
					if(frmTrend == null || (frmTrend as Form).IsDisposed)
						continue;
					else
					{
						if(frmTrend.IsTrendUOMChanged(trendScheme,iPropValue, currentUOM))
						{
							ShowTrend(trendScheme,iPropValue, currentUOM);
						}
					}
				}
			}
		}

		public void ShowTrend(string trendScheme,IProperty iProp, string modeID, string currentUOM)
		{
			try
			{
				Trend frmTrend = null;

				if(_htFrmTrends.ContainsKey(trendScheme))
				{
					frmTrend = _htFrmTrends[trendScheme] as Trend;
				}
				if(frmTrend == null || frmTrend.IsDisposed)
				{
					frmTrend = new Trend();
					frmTrend.Text += "(" + trendScheme + ")";
					frmTrend.Closing += new System.ComponentModel.CancelEventHandler(frmTrend_Closing);
					if(!_htFrmTrends.ContainsKey(trendScheme))
						_htFrmTrends.Add(trendScheme,frmTrend);
					else
						_htFrmTrends[trendScheme] = frmTrend;
				}
					
				frmTrend.LoadChart(trendScheme,iProp,modeID, currentUOM);
					
				if(!frmTrend._hasError)
				{
					frmTrend.Show();
					frmTrend.Activate();
				}
				else
					frmTrend.Dispose();
			}
			catch
			{
			}
		}

		public void ShowTrend(string trendScheme,IPropertyValue iPropValue, string currentUOM)
		{
			try
			{
				TrendGrouped frmTrend = null;

				if(_htFrmTrends.ContainsKey(trendScheme))
				{
					Form frm = _htFrmTrends[trendScheme] as Form;
					if(frm is TrendGrouped)
					{
						frmTrend = frm as TrendGrouped;
					}
				}
				if(frmTrend == null || frmTrend.IsDisposed)
				{
					frmTrend = new TrendGrouped();
					frmTrend.Text += "(" + trendScheme + ")";
					frmTrend.Closing += new System.ComponentModel.CancelEventHandler(frmTrend_Closing);
					if(!_htFrmTrends.ContainsKey(trendScheme))
						_htFrmTrends.Add(trendScheme,frmTrend);
					else
						_htFrmTrends[trendScheme] = frmTrend;
				}
					
				frmTrend.LoadChart(trendScheme,iPropValue, currentUOM);
					
				if(!frmTrend._hasError)
				{
					frmTrend.Show();
					frmTrend.Activate();
				}
				else
					frmTrend.Dispose();
			}
			catch
			{
			}
		}

		public void ShowModesDialog()
		{
            if (_modesDialog == null)
            {
                _modesDialog = new ModesDialog();
                _modesDialog.Owner = _mainFrame;
                InitializeModesDialog();
            }
			_modesDialog.Show();
			_modesDialog.WindowState = FormWindowState.Normal;
		}

		public void ShowPropertyDetailsDialog()
		{
			_propertyDetailsDialog.Show();
			_propertyDetailsDialog.WindowState = FormWindowState.Normal;
		}

		public void UpdatePropertyDetailsPanel(GraphicsManagers gms)
		{
			_propertyDetailsDialog.UpdatePropertyDetailsPanel(gms);

		}

		public bool IsPropertyDialogVisible
		{
			get
			{
				return _propertyDetailsDialog.Visible ;
			}
		}

		public void HideModesDialog()
		{
			_modesDialog.Hide();
		}

		public void HidePropertyDetailsDialog()
		{
			_propertyDetailsDialog.Hide();
		}

		public void ShowRecordedPropsDialog()
		{
            if (_recordedPropsWindow == null)
            {
                _recordedPropsWindow = new RecordedProperties();
                _recordedPropsWindow.Owner = _mainFrame;
            }
            else
			_mainFrame.tlbExport.Enabled=true;
			_recordedPropsWindow.Show();
			_recordedPropsWindow.WindowState = FormWindowState.Normal;
            IsRecordedPropsWindowActive = true;
		}

        public void ShowTrendedPropsDialog()
        {
            if (_trendedPropsDialog == null)
            {
                _trendedPropsDialog = new TrendedPropertiesDialog();
                _trendedPropsDialog.Owner = _mainFrame;
            }		
			   _mainFrame.tlbExport.Enabled=true;
            _trendedPropsDialog.Show();
            _trendedPropsDialog.WindowState = FormWindowState.Normal;
            IsTrendedPropsWindowActive = true;
			
        }

		public void HideRecordedPropsDialog()
		{
			_recordedPropsWindow.Hide();
            IsRecordedPropsWindowActive = false;
		}

        public void HideTrendedPropsDialog()
        {
            _trendedPropsDialog.Hide();
            IsTrendedPropsWindowActive = false;
			  
        }

		public void ShowMessageSubscriptionDialog()
		{
            if (_messageSubscriptionDialog == null)
            {
                _messageSubscriptionDialog = new MessageSubscriptionDialog();
                _messageSubscriptionDialog.Owner = _mainFrame;
            }
			_messageSubscriptionDialog.Show();
		}

		/// <summary>
		/// This method enables/disables the stop button depending on the 
		/// run condition
		/// </summary>
		/// <param name="enabled"></param>
		
		public void InitializeDataQualityWindow()
		{
            _dataQuality.InitializeDataQualityGrid();
		}
		public void RefreshDataQualityGrid(bool bInitialize)
		{
			_dataQuality.RefreshDataQualityGrid(bInitialize);
		}
		public void ShowDataQualityWindow()
		{
            _dataQuality.RefreshDataQualityGrid(false);
			_dataQuality.Show();
			_dataQuality.WindowState = FormWindowState.Normal;
		}

		public void HideDataQualityWindow()
		{
			_dataQuality.Hide();
		}



		public void ShowTimeWindow()
		{
			_timeWindow.Show();
			_timeWindow.WindowState = FormWindowState.Normal;
		}

		public void HideTimeWindow()
		{
			_timeWindow.Hide();
		}

		public void ShowOverridesWindow()
		{
			_overridesWindow.Show();
			_overridesWindow.WindowState = FormWindowState.Normal;
		}

		public void HideOverridesWindow()
		{
			_overridesWindow.Hide();
		}


		public string WindowName(int id)
		{
			try
			{
				switch(id)
				{
					case 100:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
                       // frmText= "Master Plant Items...";
						break;
					case 120:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Master Properties...";
						break;
					case 130:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Plant Items And Attributes...";
						break;
					case 140:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Historian Mappings...";
						break;
					case 150:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Data Historian Points...";
						break;
					case 160:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Implementer Screen...";
						break;
					case 170:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Plant Item Connectors...";
						break;
					case 180:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Master Plant Levels...";
						break;
					case 190:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Plant Aggregates...";
						break;
					case 200:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "Search and Result...";
						break;
					case 210:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frm = new PlantItemSingleView();
						break;
					case 220:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
					//	frmText= "Plant Complex Equipments...";
						break;
					case 230:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText= "User Permissions...";
						break;
					case 270:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText = "Operators Screen...";
						break;
					case 290:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText = "Calc Engine Settings";
						break;
					case 300:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText = "Plant Item Attributes(Grouped)";
						break;
					case 320:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText = "Event Log";
						break;
					case 350:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText = "Plant Item And Grouped Attributes";
						break;
					case 360:
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//frmText = "Application Log";
						break;
					case 400:
						//Plant Item Attributes(Data Arrays)
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						break;
					case 410:
						//Saved Views
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						break;
					case 420:
						//Plant Item Grouped DataArrays
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						break;
					case 430:
						//Plant Item Grouped Aggregates
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						break;
					case 440:
						//Dyanmic Saved Case
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						break;
					case 450:
						//Manual Entry for Historian Points
						frmText=LRRHeaderText.GetHeaderString(id.ToString());
						//"Manual Entry for Historian Points...";
						break;
                    case 480:
                      frmText = LRRHeaderText.GetHeaderString(id.ToString());
                      break;
                    case 490:
                      frmText = LRRHeaderText.GetHeaderString("SID_PLANT_INDICES");
                      break;
                    case 500:
                      frmText = LRRHeaderText.GetHeaderString("SID_UNIFIED_VIEWS");
                      break;
                    case 600:
                      frmText = LRRHeaderText.GetHeaderString("SID_DASHBOARD_WIZARD");
                      break;
					case 520:
						//Write Access for Historian Points
						//frmText=LRRHeaderText.GetHeaderString(id.ToString());
						frmText = LRRHeaderText.GetHeaderString("SID_WRITE_ACCESS_HIST_POINTS");
						break;
					case 530:
						//Write Access for Historian Points mappings
						frmText = LRRHeaderText.GetHeaderString("SID_WRITE_ACCESS_HIST_MAPPINGS");
						break;
                    case 610:
                        frmText = LRRHeaderText.GetHeaderString("SID_DASHBOARD_WIZARD");
                        break;
                    case 630:
                        frmText = LRRHeaderText.GetHeaderString("SID_PLANT_LVL_ROLLUP_VALUES");
                        break;
							case 640:
								frmText = LRRHeaderText.GetHeaderString("SID_REPORTS");
								break;
					default:
						frmText = null;
						break;
				}
				return(frmText); 
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRUIManager),"WindowName:"+ex.Message  ,Logger.Level.Debug);
				return null;
			}
		}

		public void SetStopEnabled(bool enabled)
		{
			if(_mainFrame != null)
				_mainFrame.SetStopEnabled(enabled);

			if(_runStatusDialog != null)
				_runStatusDialog.SetStopEnabled(enabled);
		}

		public void SetPauseSate(bool enabled)
		{
			if(_runStatusDialog != null)
				_runStatusDialog.SetPauseEnabled(enabled);
		}


		public void SetRunningState(bool isRunning)
		{
			if(_runStatusDialog != null)
				_runStatusDialog.SetRunningState(isRunning);
		}

		public Options  GetOptionsWindow()
		{
			if(_windowStore.ContainsKey(260))
			{
				return (Options)_windowStore[260];
			}
			else
			{
				return null;
			}
		}

		public bool IsSchemeInUse(string trendScheme)
		{
			return _htFrmTrends.ContainsKey(trendScheme);
		}

		#endregion

		#region Properties

		public PEMainWindow MainFrameObj
		{
			get{return _mainFrame;}
		}
		public bool PerformDataDirtyCheck
		{
			get{return _blnPerformDataDirtyCheck;}
			set
			{
				_blnPerformDataDirtyCheck = value;
			}

		}
		public DataQuality DataQualityObj
		{
			get{return _dataQuality;}
		}

		public ColorScheme ColorSchemeObj
		{
			get{return _colorScheme;}
		}
		#endregion

		#region Private Methods

		/// <summary>
		/// This method is a factory to create
		/// form for the application.
		/// </summary>
		/// <param name="id">ID of the form which needs to be created.</param>
		/// <returns></returns>
		private Form CreateForm(int id)
		{
			try
			{

				Form frm;
				switch(id)
				{
					case 100:
						//frm = new MasterComponent();
						frm = new MasterPlantItems();
						break;
					case 130:
						frm = new SiteComponentEntry();
						break;
					case 140:
						frm = new SiteCompPropHistorianMapping();
						break;
					case 150:
						frm = new DataHistorianPoints();
						break;
					case 160:
						frm = new ImplementerScreen();
						break;
					case 170:
						frm = new PlantItemConnectors();
						break;
					case 180:
						frm = new PlantLevels();
						break;
					case 190:
						frm = new AggregateDetails();
						break;
					case 200:
						frm = new Symcon.LightRidge.PEAdvisor.IOScreen.InputOutputViews();
                    	break;
					case 210:
						frm = new PlantItemSingleView();
						break;
					case 220:
						frm = new ComplexEquipment();
						break;
					case 230:
						frm=new HistTagUserPermissions();
						break;
					case 260:
						frm=new Options("");
						break;
					case 270:
						frm = new OperatorsScreen();
						break;
					case 280:
						frm = new CompositeView();
						break;
					case 290:
						frm = new ExecutionEngineModel();
						break;
					case 300:
//						frm = new GroupedPlantPropertiesView();
						frm = new Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantPropertiesView();
						break;
					case 320:
						frm = new EventLogger();
						break;
					case 350:
						frm = new GroupedPlantItems();
						break;
					case 360:
						frm = new LoggerLog();
						break;
					case 400:
						frm = new Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantDataArrayView();
						break;
					case 410:
						frm = new Symcon.LightRidge.PEAdvisor.CommonViews.SavedViewForm();
						break;
					case 420:
						frm = new Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantDataArrays();
						break;
					case 430:
						frm = new Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedAggregates();
						break;
					case 440:
						frm = new Symcon.LightRidge.PEAdvisor.SavedViews.DynamicSiteComponentEntry();
						break;
					case 450:
						frm = new ManualEntryForHistPoints();
						break;
                    case 480:
                        frm = new MultiTrendForm();
                        break;
                    case 490:
                        frm = new EffectivePricesMapping();
                        break;
                    case 500:
						frm = new Symcon.LightRidge.PEAdvisor.CommonViews.SavedViewForm();
                        break;
                    case 600:
                        frm = new Symcon.LightRidge.PEAdvisor.DashBoard.DashBoardForm();
                        break;
					case 520:
						frm = new UserDataHistorianPoints();
						break;
					case 530:
						frm = new PropHistorianSiteCompMapping();
						break;
                    case 610:
                        frm = new Symcon.LightRidge.PEAdvisor.DashBoard.DashboardCanvas();
                        break;
                    case 620:
                        frm = new Symcon.LightRidge.PEAdvisor.DashBoard.DashBoardViewer();
                        break;
                    case 630:
                        frm = new RollUpValues();
                        break;
					case 640:
							frm= new Symcon.LightRidge.PEAdvisor.Reports.LRRReportViewer();
							break;
                    case 650:
                        frm = new FormulaScriptBrowser();
                        break;
					default:
						frm = null;
						break;
				}
				if(frm!=null)
					frm.Tag = id;
				return frm;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRUIManager),"CreateForm:"+ex.Message  ,Logger.Level.Debug);
				return null;
			}
		}

		private Form CreateForm(Type type,object[] parameters)
		{
			try
			{
				Form frm = (Form)Activator.CreateInstance(type,parameters);
				return frm;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRUIManager),"Could not load the form:"+ex.Message  ,Logger.Level.Debug);
				bool flag = true;
				if(ex.InnerException != null)
				{
					if(ex.InnerException.Message == "Log file not found.")
					{
						flag = false;
					}
				}
				if(flag)
				{
					MessageBox.Show("Could not load the form.");
				}
                return null;
			}
		}
		
		private Form CreateForm(int id,ArrayList arrParams)
		{
			try
			{

				Form frm;
				switch(id)
				{
					case 100:
						//frm = new MasterComponent();
						frm = new MasterPlantItems();
						break;
					case 130:
						frm = new SiteComponentEntry();
						break;
					case 140:
						frm = new SiteCompPropHistorianMapping();
						break;
					case 150:
						frm = new DataHistorianPoints();
						break;
					case 160:
						frm = new ImplementerScreen();
						break;
					case 170:
						frm = new PlantItemConnectors();
						break;
					case 180:
						frm = new PlantLevels();
						break;
					case 190:
						frm = new AggregateDetails();
						break;
					case 200:
						frm = new Symcon.LightRidge.PEAdvisor.IOScreen.InputOutputViews();
						break;
					case 210:
						frm = new PlantItemSingleView();
						break;
					case 220:
						frm = new ComplexEquipment();
						break;
					case 230:
						frm=new HistTagUserPermissions();
						break;
					case 240:
						string ID = arrParams[0].ToString();
						frm=new SiteComponentEntry(ID);
						break;
					case 250:
						 Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory gridView = arrParams[0] as  Symcon.LightRidge.PEAdvisor.IOScreen.IGridFactory;
						 Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow rsPropertiesWindow = arrParams[1] as  Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow;
						 Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow rsComponentsWindow = arrParams[2] as  Symcon.LightRidge.PEAdvisor.IOScreen.ResultPropertiesWindow;
						frm=new Symcon.LightRidge.PEAdvisor.IOScreen.ResultForm(gridView,rsPropertiesWindow,rsComponentsWindow);
						break;
					case 260:
						frm=new Options("");
						break;
					case 270:
						frm = new OperatorsScreen();
						break;
					case 280:
						frm = new CompositeView();
						break;
					case 301:
						//frm = new GroupedPlantPropertiesView();
						Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory grpPropertyGridView = arrParams[0] as  Symcon.LightRidge.PEAdvisor.GroupProperties.IGroupGridFactory;
						frm=new Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantPropertiesResultForm(grpPropertyGridView);
						break;
					case 310:
						Symcon.LightRidge.Runcontrol.RunOptions runOptions = arrParams[0] as Symcon.LightRidge.Runcontrol.RunOptions;
						TimeWindow timeWindow = arrParams[1] as TimeWindow;
						frm = new RunControlPanelDialog(runOptions,timeWindow);
						_runControlPanelDialog = frm as RunControlPanelDialog;
						break;
					case 320:
						frm = new EventLogger();
						break;
					case 350:
						frm = new GroupedPlantItems();
						break;
					case 440:
						frm = new Symcon.LightRidge.PEAdvisor.SavedViews.DynamicSiteComponentEntry();
						break;
					case 450:
						frm = new ManualEntryForHistPoints();
						break;
					
              case 480:
                  frm = new MultiTrendForm();
                  break;
              case 490:
                  frm = new EffectivePricesMapping ();
                  break;
              case 500:
                  Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory unifiedGridView = arrParams[0] as Symcon.LightRidge.PEAdvisor.CommonViews.IUnifiedGridFactory;
						LRRResultProperties resultComponents = arrParams[1] as LRRResultProperties;
						LRRResultProperties resultAttributes = arrParams[2] as LRRResultProperties;
						frm = new Symcon.LightRidge.PEAdvisor.CommonViews.UnifiedViewResultForm(unifiedGridView,resultComponents,resultAttributes);
						break;
                    case 600:
                        Symcon.LightRidge.PEAdvisor.DashBoard.XmlDashBoard dashBoardXml = arrParams[0] as Symcon.LightRidge.PEAdvisor.DashBoard.XmlDashBoard;
                        frm = new Symcon.LightRidge.PEAdvisor.DashBoard.DashBoardForm(dashBoardXml);
                        break;
						case 520:
							frm = new UserDataHistorianPoints();
							break;
						case 530:
							frm = new PropHistorianSiteCompMapping();
							break;
                        case 620:
                            LRRDashBoardView lrrDashBoardView = arrParams[0] as LRRDashBoardView;
                            frm = new Symcon.LightRidge.PEAdvisor.DashBoard.DashBoardViewer(lrrDashBoardView);
                            break;
                        case 630:
                            frm = new RollUpValues();
                            break;
                         case 640:
                        frm = new Symcon.LightRidge.PEAdvisor.Reports.LRRReportViewer();
                        break;
                    case 650:
                        frm = new FormulaScriptBrowser();
                        break;
					default:
						frm = null;
						break;
				}
				if(frm!=null)
					frm.Tag = id;
				return frm;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(LRRUIManager),"CreateForm:"+ex.Message  ,Logger.Level.Debug);
				return null;
			}
		}

		#endregion

		#region Fields
		public PEMainWindow			_mainFrame; //Instance for the mainFrmae of the Application
		public Hashtable			_windowStore; //Intsnce to store the windows...
		public ArrayList			_windowList = new ArrayList();
        public bool IsRecordedPropsWindowActive = false;
        public bool IsTrendedPropsWindowActive = false;
		private bool				_blnPerformDataDirtyCheck = true;
		private bool				_isCancel = false;
		public int					indexValue;
		public  string				frmText;
		private int					_lastActiveIndex;
		private RunStatusDialog		_runStatusDialog;
		private RunControlPanelDialog	_runControlPanelDialog;
		private ModesDialog			_modesDialog = null;
		private Hashtable				_htFrmTrends = new Hashtable();
		private PropertyDetailsDialog _propertyDetailsDialog = new PropertyDetailsDialog();

		private DataQuality			_dataQuality = new DataQuality();
		private TimeWindow			_timeWindow  = new TimeWindow();	

		//Overrides Window
		private OverridesWindow		_overridesWindow = new OverridesWindow();
		public RecordedProperties      _recordedPropsWindow = null;
		private MessageSubscriptionDialog _messageSubscriptionDialog = null;
        public TrendedPropertiesDialog  _trendedPropsDialog = null;
        
		private LRRApp				_app;

		private ColorScheme		_colorScheme;
		#endregion

		#region Constants

		public const int MASTER_COMPONENT_WINDOW_ID = 100;
		public const int MASTER_PROPERTY_WINDOW_ID = 120;
		public const int SITE_COMPONENT_WINDOW = 130;
		public const int SITE_COMP_DATA_HIST_MAP_WINDOW = 140;
		public const int DATA_HIST_POINT_WINDOW = 150;
		public const int IMPLEMENTOR_GRAPHICS_WINDOW = 160;
		public const int SITE_COMPONENT_CONNECTORS = 170;
		public const int SITE_LEVELS = 180;
		public const int SITE_AGGREGATES = 190;
		public const int FILTER_SCREEN = 200;
		public const int IO_SCREEN = 210;
		public const int COMPLEX_EQUIPMENT = 220;
		public const int HISTTAG_USER_PERMISSIONS = 230;
		public const int SINGEL_SITE_COMPONENT_WINDOW = 240;
		public const int OPTIONS_WINDOW = 260;
		public const int CALC_ENGINE_SEQUENCE = 250;
		public const int GROUPED_PLANT_ITEM_ATTRIBUTES_WINDOW = 300;
		public const int GROUPED_PLANT_ITEM_ATTRIBUTES_R_WINDOW = 301;
		public const int RUN_CONTROL_PANEL_DIALOG_WINDOW = 310;
		public const int LOGGER_LOG_WINDOW=360;
		#endregion

		#region Event Handlers
		private void frm_Load(object sender, EventArgs e)
		{
			Form frm = sender as Form;
			Control ctl = sender as Control;
			ILRRUIContainer cont = frm as ILRRUIContainer;
			_windowList.Add(sender);
			Symcom.Utils.HelpUtility.HelpUtils.ApplyHelpSettings(frm);

			InternationalizationUtils.GetControlText(ctl,null); 
			
			if(frm.GetType() != typeof(CompositeView))
				ApplicationPersistanceUtil.ApplyGridSettings(frm,cont, frm.Tag);

			if(frm.GetType() == typeof(ImplementerScreen))
			{
				ApplicationPersistanceUtil.LoadCanvas(frm);
			}
			if(frm.GetType() == typeof(OperatorsScreen))
			{
				ApplicationPersistanceUtil.LoadOperatorsCanvas(frm);
			}
		}

		private void frm_Closed(object sender, EventArgs e)
		{
			if(_windowStore.ContainsValue(sender))
			{
				object found = null;
				foreach(object obj in _windowStore.Keys)
				{
					if(_windowStore[obj].Equals(sender))
					{
						found = obj;
						break;
					}
				}

				if(found != null)
				{
					ILRRUIContainer cont = sender as ILRRUIContainer;
					if(cont.ParentTabPage != null)
						cont.ParentTabPage.Dispose();
					if(!_mainFrame.TabbedMDIChildList.HasChildren)
					{
						_mainFrame.TabbedMDIChildList.Visible = false;
					}
					_windowList.Remove(_windowStore[found]);
					_windowStore.Remove(found);
				}
				GC.Collect();
			}
		}

		private void frm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			//e.Cancel = false;
			Form frm = sender as Form;
			ILRRUIContainer cont = frm as ILRRUIContainer;
			if(cont.PerformDirtyChk == true) 
			{
				foreach(Control ctl in frm.Controls)
				{
					try
					{
                        if (!CheckForClose(ctl, cont, frm))
                        {
                            e.Cancel = true;
                            _isCancel = false;
                        }
					}
					catch(Exception ex)
					{
						Logger.LogMessage(typeof(LRRUIManager),"frm_Closing Event:"+ex.Message  ,Logger.Level.Debug);
					}
				}
                _bChangeAlertFlashed = false;
			}
			if(e.Cancel != true)
			{
                if(frm.GetType() != typeof(CompositeView))
                    ApplicationPersistanceUtil.SaveGridSettings(frm,cont, frm.Tag);

                //if(frm.GetType() == typeof(ImplementerScreen))
                //{
                //    ApplicationPersistanceUtil.SaveCanvas(frm);
                //}
                //if(frm.GetType() == typeof(OperatorsScreen))
                //{
                //    ApplicationPersistanceUtil.SaveOperatorsCanvas(frm);
                //}
			}
		}

		private bool CheckForClose(Control ctl,ILRRUIContainer container)
		{
			if(ctl.GetType() == typeof(Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl))
			{
				
				ILRRUIContainer form = container;
				if(form.ParentTabPage == null)
					return true;
				string formName = form.ParentTabPage.Text;  
				if(GridChangedRecordLists.GetInstance().Count>0)
				{
					IEnumerator ieNumChangedRecordLists = GridChangedRecordLists.GetInstance().GetEnumerator();
					while(ieNumChangedRecordLists.MoveNext())
					{
						string recordScreenName=((GridChangedRecord)(ieNumChangedRecordLists.Current)).ScreenName.ToString();
						if(formName==recordScreenName)
						{
							if(((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList!=null)
							{
									Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl ggc = ctl as Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl;
									Syncfusion.Grouping.SourceListSetEnumerator slse = ggc.Engine.SourceListSet.GetEnumerator();
									while(slse.MoveNext())
									{
										form.ParentTabPage.BringToFront();   
										Syncfusion.Grouping.SourceListSetEntry ent = slse.Current;
										Symcon.BusinessObjects.CustomCollection cc = ent.List as Symcon.BusinessObjects.CustomCollection;
										if(cc!=null)
										{
											if(cc.GetChanges().Count != 0)
											{
												DialogResult k =LRRMessagBox.ShowMessage("SID_SAVEINFO_QUESTION","SID_LABEL",MessageBoxButtons.YesNoCancel,MessageBoxIcon.Question);      
												//DialogResult k=MessageBox.Show("Unsaved Data for '"+formName+"'! \n would you like to save the changes?","Question",MessageBoxButtons.YesNoCancel,MessageBoxIcon.Question);
												if(k == DialogResult.Cancel)
												{
													_isCancel  = true;
													return false;
												}
												else if(k == DialogResult.No)
												{
															  _isCancel = false;
															  RollBackChangesDel rollBackDel = new RollBackChangesDel(RollBackAllChanges);
															  IAsyncResult result = rollBackDel.BeginInvoke(ggc, null, null);
															  while (!result.IsCompleted)
															  {
																	Application.DoEvents();
																	System.Threading.Thread.Sleep(10);
															  }
															  rollBackDel.EndInvoke(result);
													GridChangedRecordLists.GetInstance().Remove(formName);
												}
												else
												{
													container.Save();
													GridChangedRecordLists.GetInstance().Remove(formName);
													_isCancel = false;
												}
												return true;
											}
										}
									}
							}
						}
					}
				}
		
			}
			foreach(Control cc in ctl.Controls)
			{
				CheckForClose(cc,container);
			}
			if(_isCancel == true)
				return false;
			else 
				return true;
		}

        private bool _bChangeAlertFlashed = false;
		private bool CheckForClose(Control ctl,ILRRUIContainer container,Form frm)
		{
			if(ctl.GetType() == typeof(Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl))
			{
				ILRRUIContainer form = container;
				if(form.ParentTabPage == null)
					return true;
				string formName = form.ParentTabPage.Text;  
				if(GridChangedRecordLists.GetInstance().Count>0)
				{
					IEnumerator ieNumChangedRecordLists = GridChangedRecordLists.GetInstance().GetEnumerator();
					while(ieNumChangedRecordLists.MoveNext())
					{
						string recordScreenName=((GridChangedRecord)(ieNumChangedRecordLists.Current)).ScreenName.ToString();
						if(formName==recordScreenName)
						{
                            if (((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList != null && ((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList.Count != 0)
							{
                                bool bRecordChanged = false;
                                foreach (BaseObject gridChangeRcd in ((GridChangedRecord)(ieNumChangedRecordLists.Current)).ChangedRecordList)
                                {
                                    if (gridChangeRcd.ObjectState != Symcon.Globals.Enums.DBOperation.UNCHANGED)
                                    {
                                        bRecordChanged = true;
                                    }
                                }
								Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl ggc = ctl as Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl;
								Syncfusion.Grouping.SourceListSetEnumerator slse = ggc.Engine.SourceListSet.GetEnumerator();
								while(slse.MoveNext())
								{
									Syncfusion.Grouping.SourceListSetEntry ent = slse.Current;
									Symcon.BusinessObjects.CustomCollection cc = ent.List as Symcon.BusinessObjects.CustomCollection;

                                    if (cc != null && cc.GetChanges().Count != 0 && _bChangeAlertFlashed == false)
									{
                                        _bChangeAlertFlashed = true;
                                        OperateOnUserResponse(frm, formName, container, ggc);
									}
                                    //else if (bRecordChanged == true && _bChangeAlertFlashed == false)
                                    //{
                                    //    _bChangeAlertFlashed = true;
                                    //    OperateOnUserResponse(frm, formName, container, ggc);
                                    //}
								} 
							}
						}
					}
				}
			}
			foreach(Control cc in ctl.Controls)
			{
				CheckForClose(cc,container,frm);
			}
			if(_isCancel == true)
				return false;
			else 
				return true;
		}


        /// <summary>
        /// Give the notification for saving changes on form closing.
        /// </summary>
        /// <param name="frm"></param>
        /// <param name="formName"></param>
        /// <param name="container"></param>
        /// <param name="ggc"></param>
        /// <returns></returns>
        private bool OperateOnUserResponse(Form frm,string formName,ILRRUIContainer container, GridGroupingControl ggc)
        {

            frm.Activate();
            string[] s = new string[1];
            s[0] = formName;
            DialogResult k = LRRMessagBox.ShowMessage("SID_UNSAVED_FORM", "SID_LABEL", s, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button3);

            if (k == DialogResult.Cancel)
            {
                _isCancel = true;
                return false;
            }
            else if (k == DialogResult.No)
            {
                _isCancel = false;
                RollBackChangesDel rollBackDel = new RollBackChangesDel(RollBackAllChanges);
                IAsyncResult result = rollBackDel.BeginInvoke(ggc, null, null);
                while (!result.IsCompleted)
                {
                    Application.DoEvents();
                    System.Threading.Thread.Sleep(10);
                }
                ggc.Refresh();
                rollBackDel.EndInvoke(result);
                //call function to remove the items in the GridChangedRecordLists 
                GridChangedRecordLists.GetInstance().Remove(formName);

            }
            else
            {
                container.Save();
                GridChangedRecordLists.GetInstance().Remove(formName);
                //call function to remove the items saved in the GridChangedRecordLists 
                _isCancel = false;
            }
            return true;        
        }
		 
        private void RollBackAllChanges(Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl ggc)
        {
            Symcon.LightRidge.PEAdvisor.GridUtilities.SyncGridUtilities.RollBackChanges(ggc,true);
        }


//		public void SaveGridSettings(Control ctl,ILRRUIContainer container, object formID)
//		{
//			if(formID == null)
//				return;
//			if(ctl.GetType() == typeof(Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl))
//			{
//				Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridInstance = ctl as Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl;
//				int nCols = 0;
//				int nColCtr = 0;
//				
//				LRRApplicationSettings appSettings = new LRRApplicationSettings(); //LRRGridSetting gridSetting = new LRRGridSetting();
//				LRRApplicationSettings appSettings2 = new LRRApplicationSettings(); //LRRGridSetting gridSetting2 = new LRRGridSetting();
//				
//				int iCount = 0;
//
//				GridTableDescriptor gtd = gridInstance.TableDescriptor;
//				GridTableDescriptor gtd2 = gridInstance.TableDescriptor;
//				int iCounter = gtd.Relations.Count;
//				string baseName=null;
//				OpenedWindow ow = new OpenedWindow();
//				OpenedWindow ow2 = new OpenedWindow();
//				while(iCounter > -1)
//				{
//					
//					int iRelation = gtd.Relations.Count;
//					nCols = gtd.Columns.Count + 1;
//					string hh=null;
//					bool flag = false;
//					string sConfigFilePath = Application.StartupPath + @"\config\ApplicationSettings.xml";
//					if(System.IO.File.Exists(sConfigFilePath))
//					{
//						appSettings = SGTXmlSerializer.DeserializeXML(sConfigFilePath,typeof(LRRApplicationSettings)) as LRRApplicationSettings;
//				
//						if(appSettings == null)
//							return;
//						ow = appSettings.GetOpenedWindowByID(formID.ToString());
//						if(ow == null)
//						{
//							int newId = 0;
//							if(appSettings.openedWindows !=null)
//							{
//								newId = appSettings.openedWindows.Length;
//								appSettings2.openedWindows = new OpenedWindow[newId + 1];
//								appSettings.openedWindows.CopyTo(appSettings2.openedWindows,0);
//							}
//							else
//								appSettings2.openedWindows = new OpenedWindow[1];
//							appSettings2.openedWindows[newId] = new OpenedWindow();
//							appSettings2.openedWindows[newId].WindowID = formID.ToString();
//							appSettings2.openedWindows[newId].Index = 0;
//							appSettings2.openedCanvas = appSettings.openedCanvas;
//							appSettings = appSettings2;
//							ow = appSettings2.openedWindows[newId];
//						}
//						baseName="";
//						baseName=	gtd.Name;
//						hh=gridInstance.Name+"_"+ baseName;
//						if(ow.gridInstance != null)
//						{	
//							for(int j=0;j < ow.gridInstance.Length;j++)
//							{
//								if(hh.Equals(ow.gridInstance[j].GridName))
//								{
//									flag = true;
//									Grid gridControl = new Grid();
//									gridControl.GridName = hh;//gtd.Name;
//									gridControl.GridColumns = new Columns[nCols];
//									gridControl.GridSortedColumns = new SortedColumns[gtd.SortedColumns.Count];
//									gridControl.Filter = new RecordFilterDescriptor[gtd.RecordFilters.Count];
//									nColCtr = 0;
//									int nSortColCtr = 0;
//									int nFilterCtr = 0;
//
//									foreach(GridColumnDescriptor newColWidth in gtd.Columns)
//									{
//										Columns col = new Columns();
//										col.Name =  newColWidth.Name;
//										col.Width= newColWidth.Width;
//										col.Header=newColWidth.HeaderText;
//										gridControl.GridColumns[nColCtr] = col;
//										nColCtr++;
//									}
//									foreach(RecordFilterDescriptor rfd in gtd.RecordFilters)
//									{
//										gridControl.Filter[nFilterCtr] = rfd;
//										nFilterCtr++;
//									}
//									foreach(SortColumnDescriptor scd in gtd.SortedColumns)
//									{
//										SortedColumns scol = new SortedColumns();
//										scol.Name = gtd.SortedColumns[nSortColCtr].Name;
//										scol.SortDirection = gtd.SortedColumns[nSortColCtr].SortDirection;
//										gridControl.GridSortedColumns[nSortColCtr] = scol;
//										nSortColCtr++;
//									}
//									ow.gridInstance[j] = gridControl;
//								}
//							}
//						}
//						if(!flag)
//						{
//							Grid gridControl = new Grid();
//							gridControl.GridName = hh;//gtd.Name;
//							gridControl.GridColumns = new Columns[nCols];
//							gridControl.GridSortedColumns = new SortedColumns[gtd.SortedColumns.Count];
//							gridControl.Filter = new RecordFilterDescriptor[gtd.RecordFilters.Count];
//							nColCtr = 0;
//							int nSortColCtr = 0;
//							int nFilterCtr = 0;
//
//							foreach(GridColumnDescriptor newColWidth in gtd.Columns)
//							{
//								Columns col = new Columns();
//								col.Name =  newColWidth.Name;
//								col.Width= newColWidth.Width;
//								col.Header=newColWidth.HeaderText;
//								gridControl.GridColumns[nColCtr] = col;
//								nColCtr++;
//							}
//							foreach(RecordFilterDescriptor rfd in gtd.RecordFilters)
//							{
//								gridControl.Filter[nFilterCtr] = rfd;
//								nFilterCtr++;
//							}
//							foreach(SortColumnDescriptor scd in gtd.SortedColumns)
//							{
//								SortedColumns scol = new SortedColumns();
//								scol.Name = scd.Name;
//								scol.SortDirection = scd.SortDirection;
//								gridControl.GridSortedColumns[nSortColCtr] = scol;
//								nSortColCtr++;
//							}
//							if(ow.gridInstance == null)
//							{
//								ow2.gridInstance = new Grid[1];
//								ow2.gridInstance[0] = gridControl;
//							}
//							else
//							{
//								ow2.gridInstance = new Grid[ow.gridInstance.Length + 1];
//								ow.gridInstance.CopyTo(ow2.gridInstance,0);
//								ow2.gridInstance[ow.gridInstance.Length] = gridControl;
//							}
//							ow2.WindowID = ow.WindowID;
//							ow2.Index = 0;
//							appSettings.SetOpenWindow(ow2);
//
//							System.IO.File.Delete(sConfigFilePath);
//							SGTXmlSerializer.SerializeAsXML(sConfigFilePath,typeof(LRRApplicationSettings),appSettings,true);
//						}
//						else
//						{
//							System.IO.File.Delete(sConfigFilePath);
//							SGTXmlSerializer.SerializeAsXML(sConfigFilePath,typeof(LRRApplicationSettings),appSettings,true);
//						}
//					}
//					else
//					{
//						ow.gridInstance = new Grid[1];
//						Grid gridControl = new Grid();
//
//						gridControl.GridName =gridInstance.Name+"_"+ gtd.Name;
//						gridControl.GridColumns = new Columns[nCols];
//						gridControl.GridSortedColumns = new SortedColumns[gtd.SortedColumns.Count];
//						gridControl.Filter = new RecordFilterDescriptor[gtd.RecordFilters.Count];
//						int nSortColCtr = 0;
//						int nFilterCtr = 0;
//
//						foreach(GridColumnDescriptor newColWidth in gtd.Columns)
//						{
//							Columns col = new Columns();
//							col.Name =  newColWidth.Name;
//							col.Width= newColWidth.Width;
//							col.Header=newColWidth.HeaderText;
//							gridControl.GridColumns[nColCtr] = col;
//							nColCtr++;
//						}
//						foreach(RecordFilterDescriptor rfd in gtd.RecordFilters)
//						{
//							gridControl.Filter[nFilterCtr] = rfd;
//							nFilterCtr++;
//						}
//						foreach(SortColumnDescriptor scd in gtd.SortedColumns)
//						{
//							SortedColumns scol = new SortedColumns();
//							scol.Name = gtd.SortedColumns[nSortColCtr].Name;
//							scol.SortDirection = gtd.SortedColumns[nSortColCtr].SortDirection;
//							gridControl.GridSortedColumns[nSortColCtr] = scol;
//							nSortColCtr++;
//						}
//						ow.gridInstance[0] = gridControl;
//						ow.WindowID = formID.ToString();
//						ow.Index = 0;
//						appSettings.openedWindows = new OpenedWindow[1];
//						appSettings.openedWindows[0] = ow;
//						System.IO.File.Delete(sConfigFilePath);
//						SGTXmlSerializer.SerializeAsXML(sConfigFilePath,typeof(LRRApplicationSettings),appSettings,true);
//					}
//
//				child:
//					try
//					{
//						if(gtd.Relations.Count > iCount)
//						{
//							GridTableDescriptor gtd1 =  gtd.Relations[iCount].ChildTableDescriptor;
//							iRelation = gtd1.Relations.Count;
//						}
//						else
//						{
//							gridInstance.TableDescriptor.Relations.Clear();
//							return;
//						}
//					}
//					catch(Exception ex)
//					{
//						gridInstance.TableDescriptor.Relations.Clear();
//						return;
//					}
//					
//					if(iRelation == 0)
//					{
//						iCount ++;
//					
//						goto child;
//					}
//					else
//					{
//						gtd.Name="";
//						gtd =  gtd.Relations[iCount].ChildTableDescriptor;
//						iCounter = gtd.Relations.Count;
//					}
//				}
//			}
//			foreach(Control cc in ctl.Controls)
//			{
//				SaveGridSettings(cc,container,formID);
//			}
//			
//		}

//		public void ApplyGridSettings(Control ctl,ILRRUIContainer container, object formID)
//		{
//			if(ctl.GetType() == typeof(Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl))
//			{
//				string sConfigFilePath = Application.StartupPath + @"\config\ApplicationSettings.xml";
//				Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridInstance = ctl as Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl;
//				LRRApplicationSettings appSettings = SGTXmlSerializer.DeserializeXML(sConfigFilePath,typeof(LRRApplicationSettings)) as LRRApplicationSettings;
//				GridTableDescriptor gtd = gridInstance.TableDescriptor;
//				GridTableDescriptor gtd2 = gridInstance.TableDescriptor;
//				string hh=null;
//				int iChildGrid = gtd.Relations.Count;
//				int iCount = 0;
//				if(appSettings == null)
//					return;
//
//				OpenedWindow ow = appSettings.GetOpenedWindowByID(formID.ToString());
//				if(ow == null)
//					return;
//				if(ow.gridInstance == null)
//					return;
//				while(iChildGrid > -1)
//				{
//				
//					string baseName=null;
//					baseName=	gtd.Name;
//					
//				//	gtd.Name=gridInstance.Name+"_"+ baseName;
//					 hh=gridInstance.Name+"_"+ baseName;
//					
//						for(int j=0;j < ow.gridInstance.Length;j++)
//						{
//							
//							if(hh.Equals(ow.gridInstance[j].GridName))
//							{
//								int nColCtr = 0;
//								Columns[] col = ow.gridInstance[j].GridColumns;
//								SortedColumns[] scols = ow.gridInstance[j].GridSortedColumns;
//								Syncfusion.Grouping.RecordFilterDescriptor[] filters = ow.gridInstance[j].Filter;
//
//								try
//								{
//									if(col!=null)
//									{
//										for(int i=0;i < col.Length; i++)
//										{
//											int colIndex = 0;
//											foreach(GridColumnDescriptor newColWidth in gtd.Columns)
//											{
//												if(newColWidth.Name.Equals(col[i].Name))
//												{
//													newColWidth.Width = col[i].Width;
//													gtd.Columns.Move(colIndex,i);
//												}
//												colIndex++;
//												nColCtr++;
//											}
//										}
//									}
//									if(filters!=null)
//									{
//										gtd.RecordFilters.Clear();
//										foreach(RecordFilterDescriptor rfd in filters)
//										{
//											gtd.RecordFilters.Add(rfd);
//										}
//									}
//									if(scols!=null)
//									{
//										gtd.SortedColumns.Clear();
//										foreach(SortedColumns scol in scols)
//										{
//											gtd.SortedColumns.Add(scol.Name,scol.SortDirection);
//										}
//									}
//								}
//								catch(Exception ex)
//								{
//									System.Diagnostics.Debug.Assert(false,ex.Message,ex.StackTrace);
//								}
//								
//							}
//							gtd.Name=baseName;
//						}
//					child:
//						try
//						{
//							if(gtd.Relations.Count > iCount)
//							{
//								GridTableDescriptor gtd1 =  gtd.Relations[iCount].ChildTableDescriptor;
//								iChildGrid = gtd1.Relations.Count;
//							}
//							else
//							{
//								return;
//							}
//						}
//						catch(Exception ex)
//						{
//							return;
//						}
//					
//						if(iChildGrid == 0)
//						{
//							iCount ++;
//					
//							goto child;
//						}
//						else
//						{
//							gtd =  gtd.Relations[iCount].ChildTableDescriptor;
//							iChildGrid = gtd.Relations.Count;
//						}
//					}
//			}
//			foreach(Control cc in ctl.Controls)
//			{
//				ApplyGridSettings(cc,container, formID);
//			}
//		}

		public void SaveColumnOrder(Control ctl,ILRRUIContainer container)
		{
//			if(ctl.GetType() == typeof(Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl))
//			{
//				Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridInstance = ctl as Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl;
//				LRRGridSetting gridSetting = new LRRGridSetting();
//					LRRGridSetting gridSetting2 = new LRRGridSetting();
//
//				GridTableDescriptor gtd = gridInstance.TableDescriptor;
//				int iChildGrid = gtd.Relations.Count;
//				int co=gtd.Relations.Count;
//				string baseName=null;
//				int nCols = 0;
//				int nColCtr = 0;
//				int iCount = 0;
//				string hh=null;
//				if(gridSetting == null)
//					return;
//				for(int lk=0;lk< co;lk++)
//				{
//					string hhhh= gtd.Relations[lk].Name;
//					while(iChildGrid > -1)
//					{
//						nCols = gtd.Columns.Count + 1;
//						int cnt=gtd.VisibleColumns.Count;
//						bool flag = false;
//						string sConfigFilePath = Application.StartupPath + @"\config\UISettings.xml";
//						if(System.IO.File.Exists(sConfigFilePath))
//						{
//							gridSetting = SGTXmlSerializer.DeserializeXML(sConfigFilePath,typeof(LRRGridSetting)) as LRRGridSetting;
//							if(gridSetting == null)
//								return;
//							baseName=null;
//							baseName=	gtd.Name;
//							hh=gridInstance.Name+"_"+ baseName;
//							for(int j=0;j < gridSetting.gridInstance.Length;j++)
//							{
//								if(hh.Equals(gridSetting.gridInstance[j].GridName))
//								{
//									Columns[] col = gridSetting.gridInstance[j].GridColumns;
//									for(int i=0;i <cnt;i++)
//									{
//										string na=	gtd.Columns[i].Name;
//										col[i].Name = na;
//										col[i].Header=gtd.Columns[i].HeaderText;
//
//									}
//									flag = true;
//								}
//							}
//					
//							if(!flag)
//							{
//								Grid gridControl = new Grid();
//								//gridControl.GridName="";
//								gridControl.GridName =  hh;
//								gridControl.GridColumns = new Columns[nCols];
//								nColCtr = 0;
//
//								foreach(GridColumnDescriptor newColWidth in gtd.Columns)
//								{
//									Columns col = new Columns();
//									col.Name =  newColWidth.Name;
//									col.Width= newColWidth.Width;
//									col.Header= newColWidth.HeaderText;
//									gridControl.GridColumns[nColCtr] = col;
//									nColCtr++;
//								}
//
//								gridSetting2.gridInstance = new Grid[gridSetting.gridInstance.Length + 1];
//								gridSetting.gridInstance.CopyTo(gridSetting2.gridInstance,0);
//								gridSetting2.gridInstance[gridSetting.gridInstance.Length] = gridControl;
//
//								System.IO.File.Delete(sConfigFilePath);
//								SGTXmlSerializer.SerializeAsXML(sConfigFilePath,typeof(LRRGridSetting),gridSetting2,true);
//							}
//							else
//							{
//								System.IO.File.Delete(sConfigFilePath);
//								SGTXmlSerializer.SerializeAsXML(sConfigFilePath,typeof(LRRGridSetting),gridSetting,true);
//							}
//						}
//						else
//						{
//							gridSetting.gridInstance = new Grid[1];
//							Grid gridControl = new Grid();
//							//gridControl.GridName="";
//							gridControl.GridName = gridInstance.Name+"_"+ gtd.Name;
//							gridControl.GridColumns = new Columns[nCols];
//
//							foreach(GridColumnDescriptor newColWidth in gtd.Columns)
//							{
//								Columns col = new Columns();
//								col.Name =  newColWidth.Name;
//								col.Width= newColWidth.Width;
//								col.Header= newColWidth.HeaderText;
//								gridControl.GridColumns[nColCtr] = col;
//								nColCtr++;
//							}
//							gridSetting.gridInstance[0] = gridControl;
//							System.IO.File.Delete(sConfigFilePath);
//							SGTXmlSerializer.SerializeAsXML(sConfigFilePath,typeof(LRRGridSetting),gridSetting,true);
//						}
//				
//					child:
//						try
//						{
//							GridTableDescriptor gtd1 =  gtd.Relations[iCount].ChildTableDescriptor;
//							iChildGrid = gtd1.Relations.Count;
//						}
//						catch(Exception ex)
//						{
//							return;
//						}
//					
//						if(iChildGrid == 0)
//						{
//							iCount ++;
//					
//							goto child;
//						}
//						else
//						{
//							gtd =  gtd.Relations[iCount].ChildTableDescriptor;
//							iChildGrid = gtd.Relations.Count;
//						}
//					}
//				}
//				}
//
//			foreach(Control cc in ctl.Controls)
//			{
//				SaveColumnOrder(cc,container);
//			}
		}

	

			public void ApplyColumOrder(Control ctl,ILRRUIContainer container)
			{
//				if(ctl.GetType() == typeof(Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl))
//				{
//					string sConfigFilePath = Application.StartupPath + @"\config\UISettings.xml";
//					Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridInstance = ctl as Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl;
//					LRRGridSetting gridSetting = SGTXmlSerializer.DeserializeXML(sConfigFilePath,typeof(LRRGridSetting)) as LRRGridSetting;
//
//					GridTableDescriptor gtd = gridInstance.TableDescriptor;
//					GridTableDescriptor gtd2 = gridInstance.TableDescriptor;
//					int iChildGrid = gtd.Relations.Count;
//					string hh=null;
//					int iCount = 0;
//					if(gridSetting == null)
//						return;
//
//					while(iChildGrid > -1)
//					{
//						int cnt=gtd.VisibleColumns.Count;	
//						
//						GridColumnDescriptorCollection cd=gtd.Columns as GridColumnDescriptorCollection;
//						int k=cd.Count;
//						string baseName=null;
//						baseName=	gtd.Name;
//						
//						 hh=gridInstance.Name+"_"+ baseName;
//						for(int j=0;j < gridSetting.gridInstance.Length;j++)
//						{
//							if(hh.Equals(gridSetting.gridInstance[j].GridName))
//							{
//								for(int p=0;p <cnt;p++)
//								{
//									if(gtd.Columns[p].Name!="")
//									{
//										string nam=gtd.Columns[p].Name;
//										nam="";
//										gtd.Columns[p].Name="a"+p;
//									}
//								}
//								Columns[] col = gridSetting.gridInstance[j].GridColumns;
//								int i=0;
//								foreach(GridColumnDescriptor newColName in gtd.Columns)
//								{
//									if(i<col.Length)
//									{
//										string changedName=null;
//										newColName.Name = col[i].Name;
//										changedName=newColName.Name;
//										gtd.Columns[i].Dispose();
//										gtd.Columns.Add(changedName);
//										
//										gtd.Columns[i].HeaderText=col[i].Header;
//										//gtd.Columns[i].Name=col[i].Name;
//										i++;
//									}
//								}
//								gtd.Name=baseName;
//							}
//						
//						
//						}
//					
//						child:
//							try
//							{
//								GridTableDescriptor gtd1 =  gtd.Relations[iCount].ChildTableDescriptor;
//								iChildGrid = gtd1.Relations.Count;
//							}
//							catch(Exception ex)
//							{
//								return;
//							}
//							if(iChildGrid == 0)
//							{
//								iCount ++;
//								goto child;
//							}
//							else
//							{
//								gtd =  gtd.Relations[iCount].ChildTableDescriptor;
//								iChildGrid = gtd.Relations.Count;
//							}
//						
//					}
//					gridInstance.Refresh();
//					ctl.Refresh();
//				}
//				
//				foreach(Control cc in ctl.Controls)
//					{
//						ApplyColumOrder(cc,container);
//					}
			}


		/// <summary>
		/// This Method Selects the next child window in the internal list
		/// </summary>
		public void SelectNextChild()
		{
			if(_lastActiveIndex == (_windowList.Count -1))
				_lastActiveIndex=0;
			else
			{
				_lastActiveIndex++;
			}
			
			((Form)_windowList[_lastActiveIndex]).Select();
		}

		#endregion

		private void frm_Activated(object sender, EventArgs e)
		{
			ILRRUIContainer cont = sender as ILRRUIContainer;
			Control ctl = sender as Control;
			if(cont != null)
			{
				//InternationalizationUtils.GetControlText(ctl,null); 

				_mainFrame.TabbedMDIChildList.SelectedTab = cont.ParentTabPage;

				if(!_mainFrame.TabbedMDIChildList.Visible)
				{
					_mainFrame.TabbedMDIChildList.Visible = true;

				}
			}
			if(ctl.GetType() == typeof(RunControlPanelDialog))
			{
				RunControlPanelDialog runControlPanelForm = ctl as RunControlPanelDialog;
				runControlPanelForm.chkSaveDataSets_SetText();
			}
		}

		private void TabbedMDIChildList_SelectedIndexChanged(object sender, EventArgs e)
		{
			foreach (Form childForm in _mainFrame.MdiChildren) 
			{
				ILRRUIContainer cont = childForm as ILRRUIContainer;
				if(cont != null)
				{
					//Check for its corresponding MDI child form
					if (cont.ParentTabPage.Equals(_mainFrame.TabbedMDIChildList.SelectedTab)) 
					{
						//Activate the MDI child form
						childForm.Select();
					}
				}
			}

		}
		
		/// <summary>
		/// This methos selects the previoud child in the list
		/// </summary>
		public void SelectPreviousChild()
		{
			if(_lastActiveIndex == 0)
				_lastActiveIndex=_windowList.Count-1;
			else
			{
				_lastActiveIndex--;
			}
			((Form)_windowList[_lastActiveIndex]).Select();
		}
		
		private void _mainFrame_MdiChildActivate(object sender, EventArgs e)
		{
			for(int i=0;i<_windowList.Count;i++)
			{
				if(_windowList[i].Equals(sender))
					_lastActiveIndex = i;
			}
		}

		private void _runStatusDialog_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			_mainFrame.Activate();
		}

		private void frmTrend_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			Form frm = sender as Form;
			object toRemoveKey = null;
			foreach(object key in _htFrmTrends.Keys)
			{
				if(_htFrmTrends[key] == frm)
				{
					toRemoveKey=key;
				}
			}
			if(toRemoveKey != null)
			{
				_htFrmTrends.Remove(toRemoveKey);
			}
		}
	}
}
