using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Reflection;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Activation;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Http;
using System.Runtime.Remoting.Channels.Tcp;
using System.Globalization;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Lifetime;
using System.Collections.Specialized;
using Symcon.LightRidge.PEAdvisor;
using System.Security.Permissions;
using Symcon.LightRidge.MessageService;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.MessageLogger;
using System.Net.NetworkInformation;
namespace Symcon.LightRidge.PEAdvisor
{
	public class MessagingClient
	{
		static MessagingClient()
		{
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			_userID = _app.DBManagerObj.AccessControlModel.UserID;
		}


		/// <summary>
		/// Client Registration to the server if the server exists
		/// </summary>
		public static void InitializeClient()
		{
          
			_callback = new CallbackClass(_app);

			try
			{
				BinaryServerFormatterSinkProvider  provider = new BinaryServerFormatterSinkProvider();
				BinaryServerFormatterSinkProvider serverProv = new BinaryServerFormatterSinkProvider();
				provider.TypeFilterLevel = System.Runtime.Serialization.Formatters.TypeFilterLevel.Full;
				BinaryClientFormatterSinkProvider clientProv = new BinaryClientFormatterSinkProvider();
				//****
				ListDictionary channelProperties = new ListDictionary();
				channelProperties.Add("port", 0);
				//*******
				TcpChannel chan = new TcpChannel(channelProperties, clientProv, provider);
				try
				{
					foreach(TcpChannel chn in ChannelServices.RegisteredChannels)
					{
						ChannelServices.UnregisterChannel(chn);
					}
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"InitializeClient:"+ex.Message  ,Logger.Level.Debug);
				}
				ChannelServices.RegisterChannel(chan);
				object[] activationAttributes = new object[1];

				activationAttributes[0] = new UrlAttribute(@"tcp://" + _app.GetOptions().MessagingServerInfoObj.ServerIP + ":" + _app.GetOptions().MessagingServerInfoObj.ServerPort + @"/" + _app.GetOptions().MessagingServerInfoObj.ApplicationName);

				SrvObj  = (MessageComponent)Activator.CreateInstance(typeof(MessageComponent),null,activationAttributes);

				ILease serverLease = (ILease)RemotingServices.GetLifetimeService(SrvObj);
				SrvObj.CalibrationFactorUpdate+=new CalibrationFactorEvent(_callback.CalibrationFactorUpdateCallback);
				SrvObj.PropertyValueUpdate+=new PropertyValueChangedEvent(_callback.PropertyValueChangedUpdateCallback);
				SrvObj.GroupedPropertyValueUpdate+=new GroupedPropertyValueEvent(_callback.GroupedPropertyValueUpdateCallback);
				SrvObj.CommitUpdates+=new RequestCommitUpdatesEvent(_callback.CommitUpdatesCallback);
                SrvObj.RaiseAlertUpdate += new RaiseAlertEvent(_callback.RaiseAletCallback);
				SrvObj.RegisterClient(_app.DBManagerObj.AccessControlModel.UserID,_app.DBManagerObj.AccessControlModel.UserName);
				_isInitialized = true;

			
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(MessagingClient),"InitializeClient:"+ex.Message  ,Logger.Level.Debug);
				_isInitialized = false;
			}
		}


		/// <summary>
		/// Client ReInitialize to the server if the server exists
		/// </summary>
		public static void ReInitializeClient()
		{
			_callback = new CallbackClass(_app);

			try
			{
				BinaryServerFormatterSinkProvider  provider = new BinaryServerFormatterSinkProvider();
				BinaryServerFormatterSinkProvider serverProv = new BinaryServerFormatterSinkProvider();
				provider.TypeFilterLevel = System.Runtime.Serialization.Formatters.TypeFilterLevel.Full;
				BinaryClientFormatterSinkProvider clientProv = new BinaryClientFormatterSinkProvider();
				//****
				ListDictionary channelProperties = new ListDictionary();
				channelProperties.Add("port", 0);
				//*******
				TcpChannel chan = new TcpChannel(channelProperties, clientProv, provider);
				try
				{
					foreach(TcpChannel chn in ChannelServices.RegisteredChannels)
					{
						ChannelServices.UnregisterChannel(chn);
					}
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"ReInitializeClient:"+ex.Message  ,Logger.Level.Debug);
				}
				ChannelServices.RegisterChannel(chan);
				object[] activationAttributes = new object[1];

				activationAttributes[0] = new UrlAttribute(@"tcp://" + _app.GetOptions().MessagingServerInfoObj.ServerIP + ":" + _app.GetOptions().MessagingServerInfoObj.ServerPort + @"/" + _app.GetOptions().MessagingServerInfoObj.ApplicationName);

				SrvObj  = (MessageComponent)Activator.CreateInstance(typeof(MessageComponent),null,activationAttributes);

				ILease serverLease = (ILease)RemotingServices.GetLifetimeService(SrvObj);
				SrvObj.CalibrationFactorUpdate+=new CalibrationFactorEvent(_callback.CalibrationFactorUpdateCallback);
				SrvObj.PropertyValueUpdate+=new PropertyValueChangedEvent(_callback.PropertyValueChangedUpdateCallback);
				SrvObj.GroupedPropertyValueUpdate+=new GroupedPropertyValueEvent(_callback.GroupedPropertyValueUpdateCallback);
				SrvObj.CommitUpdates+=new RequestCommitUpdatesEvent(_callback.CommitUpdatesCallback);
                SrvObj.RaiseAlertUpdate += new RaiseAlertEvent(_callback.RaiseAletCallback);
				SrvObj.RegisterClient(_app.DBManagerObj.AccessControlModel.UserID,_app.DBManagerObj.AccessControlModel.UserName);
				_isInitialized = true;
			}
			catch(Exception ex)
			{
				
				Logger.LogMessage(typeof(MessagingClient),"Could Not Connect to the Server! \n Make sure Server is Configured and Running \n\n Running PE-Advisor in Standalone Mode....:"+ex.Message  ,Logger.Level.Error);
				_isInitialized = false;
			}
		}

        public static void RaiseAlert(string msg)
        {
            Symcon.Utils.Alert.Messages.LRRAlert.AddAlert(Symcon.Globals.Enums.MessageType.MSgUpdate, msg);
            if (_isInitialized)
            {
                try
                {
                    SrvObj.BroadcastAlert(msg);
                }
                catch (Exception ex)
                {
                    Logger.LogMessage(typeof(MessagingClient), msg + ":" + ex.Message, Logger.Level.Debug);
                    _isInitialized = false;
                }
            }
        }

		public static void CalibrationFactorChanged(string histTagUID,string calibrationFactor, string preFilterUserOverride,string postFilterUserOverride)
		{
			if(_isInitialized)
			{
				try
				{
					SrvObj.BroadcastCalibrationFactorChanged(histTagUID,calibrationFactor,preFilterUserOverride,postFilterUserOverride);
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"CalibrationFactorChanged:"+ex.Message  ,Logger.Level.Debug);
					_isInitialized = false;
				}
			}
		}


		public static void CalibrationFactorChangedForUsers(string histTagUID,string calibrationFactor, string preFilterUserOverride,string postFilterUserOverride)
		{
			if(_isInitialized)
			{
				try
				{
					SrvObj.BroadcastCalibrationFactorChangeForSubscribers(_userID,histTagUID,calibrationFactor,preFilterUserOverride,postFilterUserOverride);
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"CalibrationFactorChanged:"+ex.Message  ,Logger.Level.Debug);
					_isInitialized = false;
				}
			}
		}


		public static void GroupedPropertyValueChanged(string lookupQualifierID,string groupPropID,string modeID, string groupPropVal)
		{
			if(_isInitialized )
			{
				try
				{
					SrvObj.BroadcastGroupedPropertyValueChanged(lookupQualifierID,groupPropID,modeID,groupPropVal);
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"GroupedPropertyValueChanged:"+ex.Message  ,Logger.Level.Debug);
					_isInitialized = false;
				}
			}
		}

		public static void GroupedPropertyValueChangeForUsers(string lookupQualifierID,string groupPropID,string modeID, string groupPropVal)
		{
			if(_isInitialized )
			{
				try
				{
					SrvObj.BroadcastGroupedPropertyValueChangeForSubscribers(_userID,lookupQualifierID,groupPropID,modeID,groupPropVal);
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"GroupedPropertyValueChanged:"+ex.Message  ,Logger.Level.Debug);
					_isInitialized = false;
				}
			}
		}

		public static void PropertyValueChanged(string propId,string propValue, string modeID)
		{
			if(_isInitialized)
			{
				try
				{
					SrvObj.BroadcastPropertyValueChanged(propId,propValue,modeID);
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"PropertyValueChanged:"+ex.Message  ,Logger.Level.Debug);
					_isInitialized = false;
				}
			}
		}


		[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
		public static void CommitUpdates()
		{
			if(_isInitialized)
			{
				try
				{
					
					SrvObj.BroadcastCommitUpdates(_userID);
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"CommitUpdates:"+ex.Message  ,Logger.Level.Debug);
					_isInitialized = false;
				}
			}
		}


		[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
		public static void PropertyValueChangeForUser(string propId,string propValue, string modeID)
		{
			if(_isInitialized)
			{
				try
				{
					SrvObj.BroadcastPropertyValueChangeForSubscribers(_userID, propId,propValue,modeID);
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(MessagingClient),"PropertyValueChanged:"+ex.Message  ,Logger.Level.Debug);
					_isInitialized = false;
				}
			}
		}
		public static void Terminate()
		{
			try
			{
				if(SrvObj != null)
				{
					SrvObj.Release();
					_isInitialized = false;
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(Options),"Exception In  Saving Options:"+ex.Message  ,Logger.Level.Debug);
			}
		}
		public static bool IsInitialized
		{
			get
			{
				return _isInitialized;
			}
		}


		public static ArrayList AvailableUsers
		{
			get
			{
				return GetAvailableUsers();
			}
		}

		public static ArrayList Subscriptions
		{
			get
			{
				return _subscriptions;

			}
		}

		private static ArrayList GetAvailableUsers()
		{
			_availableUsers.Clear();
			try
			{
				if(_isInitialized)
				{
					object[,] objarray =  SrvObj.GetAvailabelUsers();
					if(objarray != null)
					{
						for(int i = 0;i<objarray.Length; i=i+2)
						{
							UserMap userMap = new UserMap();
							userMap.UserID = objarray.GetValue(i/2,0) == null ? "" : objarray.GetValue(i/2,0).ToString();
							userMap.UserName = objarray.GetValue(i/2,1) == null ? "" : objarray.GetValue(i/2,1).ToString();
							if(userMap.UserID != null && userMap.UserID != _userID)
								_availableUsers.Add(userMap);
						}
					}
				}
			}
			catch
			{

			}
			return _availableUsers;
		}

		#region Fields
		private static CallbackClass _callback;
		private static LRRApp _app;
		public static Symcon.LightRidge.MessageService.MessageComponent  SrvObj=null;
		private static bool _isInitialized = false;
		private static string _userID = null;
		private static ArrayList _subscriptions = new ArrayList();
		private static ArrayList _availableUsers = new ArrayList();

		#endregion

	}
}
