using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.Serialization;

using Syncfusion.Drawing;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Symcon.Utils.MessageLogger;
using Symcon.Database;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
   

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for PlantLevelUserTreeControl.
	/// </summary>
	public class PlantLevelUserTreeControl : GridStaticCellModel 
	{
		internal PlantLevelTreeStructure plts;
		
		protected PlantLevelUserTreeControl(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			ButtonBarSize = new Size(14, 14);
		}

		public PlantLevelUserTreeControl(GridModel grid,PlantLevelTreeStructure plts)
			: base(grid)
		{
			this.plts = plts;
			ButtonBarSize = new Size(16, 16);
		}
			
		//note that this method create our new derived renderer
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new PlantLevelUserTreeControlRenderer(control, this);
		}
	}

	public class PlantLevelUserTreeControlRenderer : GridStaticCellRenderer
	{
		internal PlantLevelTreeStructure plts;
		private LRRLevels				_lrrLevels=null; //Levels Object
		private LRRLevelLinks			_lrrLevelLinks = null; //Level Links object	
		private LRRApp					_app;
		private bool					isPopulate = false;
		public void initDataStore()
		{
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			_lrrLevels = new LRRLevels();
			_app.DBManagerObj.PopulateCollection(_lrrLevels,false);
			_lrrLevelLinks = new LRRLevelLinks();
			_app.DBManagerObj.PopulateCollection(_lrrLevelLinks,false);   
		}
		public PlantLevelUserTreeControlRenderer(GridControlBase grid,GridCellModelBase cellModel)
			: base(grid,cellModel)
		{
			this.plts= ((PlantLevelUserTreeControl)this.Model).plts;
			this.plts.Visible = false;
			DropDownPart = new GridDropDownCellImp(this);
			DropDownButton = new GridCellComboBoxButton(this);
			this.plts.save += new EventHandler(Opt_save);
			this.plts.cancel += new EventHandler(Opt_cancel);  
		}
		public new GridDropDownContainer DropDownContainer
		{
			get{return (GridDropDownContainer)base.DropDownContainer;}
		}
		protected override void InitializeDropDownContainer()
		{
			base.InitializeDropDownContainer ();
			if (this.DropDownContainer != null)
			{
				this.DropDownContainer.Controls.Add(this.plts);
			}
		}
		protected override void OnBeginEdit()
		{
			base.OnBeginEdit ();
			GridCurrentCell cc = this.Grid.CurrentCell;
		   
		}

		public override void DropDownContainerShowingDropDown(object sender, CancelEventArgs e)
		{
			GridCurrentCell cc = this.Grid.CurrentCell;
			this.DropDownContainer.Size = plts.Size;
			this.plts.SetValuesFromString(this.ControlText);
			plts.Visible = true;
			if(isPopulate == false)
			{
				initDataStore();
				populateTree(plts.treeView1,_lrrLevels,_lrrLevelLinks); 
				plts.treeView1.ExpandAll(); 
				isPopulate = true;
			}
			
		}
		public override void DropDownContainerShowedDropDown(object sender, EventArgs e)
		{
			//base.DropDownContainerShowedDropDown (sender, e);
		}

		public override void DropDownContainerCloseDropDown(object sender, PopupClosedEventArgs e)
		{	
			if (e.PopupCloseType == PopupCloseType.Done)
			{
				if (this.NotifyCurrentCellChanging())
				{
					ControlValue = this.plts.GetValuesToString();
					this.NotifyCurrentCellChanged();
				}
			}
			Grid.CurrentCell.EndEdit();
			Grid.CurrentCell.Refresh();      
			base.DropDownContainerCloseDropDown (sender, e);
		}
		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
		{
			base.OnDraw (g, clientRectangle, rowIndex, colIndex, style);
			plts.Width = clientRectangle.Width;
		}


		//used to move the grid value into the cell control
		protected override void OnInitialize(int rowIndex, int colIndex)
		{
			base.OnInitialize (rowIndex, colIndex);
			plts.SetValuesFromString(this.Grid.Model[rowIndex, colIndex].Text);
		}
		

		//used to save the cell control value back to the grid
		protected override bool OnSaveChanges()
		{
			string s = this.plts.GetValuesToString();
			this.Grid.Model[this.RowIndex, this.ColIndex].Text = s;
			return true;
		}
		private void Opt_save(object sender, EventArgs e)
		{
			// close the dropdown with a Done setting
			this.Grid.CurrentCell.CloseDropDown(Syncfusion.Windows.Forms.PopupCloseType.Done);
		}

		//handler for the user control Cancel event
		private void Opt_cancel(object sender, EventArgs e)
		{
			// close the dropdown with a Canceled setting
			this.Grid.CurrentCell.CloseDropDown(Syncfusion.Windows.Forms.PopupCloseType.Canceled);
		}
		
		protected override void OnClick(int rowIndex, int colIndex, MouseEventArgs e)
		{
			base.OnClick (rowIndex, colIndex, e);
			GridCurrentCell cc = this.Grid.CurrentCell;
			cc.BeginEdit(true); 
		}
		


		private bool populateTree(TreeView treeView,LRRLevels levels,LRRLevelLinks levelLinks)
		{
			treeView.Nodes.Clear();
			try
			{
				Hashtable				_levelHash = new Hashtable();
				Hashtable				_nodesHash = new Hashtable();
				Hashtable				_childParent = new Hashtable();

				foreach(LRRLevel l in levels)
				{
					_levelHash.Add(l.LevelID,l);
				}
				foreach(LRRLevelLink ll in levelLinks)
				{
					_childParent.Add(ll.ChildID,ll.ParentID);

					if(_nodesHash.ContainsKey(ll.ParentID))
					{
						TreeNode foundNode = _nodesHash[ll.ParentID] as TreeNode;
						LRRLevel foundLevel = _levelHash[ll.ChildID] as LRRLevel; 
						TreeNode childNode = new TreeNode(foundLevel.Name);
						childNode.Tag = foundLevel;
						foundNode.Nodes.Add(childNode);
					}
					else
					{
						LRRLevel parentLevel = _levelHash[ll.ParentID] as LRRLevel;
						LRRLevel childLevel = _levelHash[ll.ChildID] as LRRLevel;
						TreeNode parentNode = new TreeNode(parentLevel.Name);
						parentNode.Tag = parentLevel;
						TreeNode childNode = new TreeNode(childLevel.Name);
						childNode.Tag = childLevel;
						parentNode.Nodes.Add(childNode);
						_nodesHash.Add(ll.ParentID,parentNode);
					}
				}

				foreach(object k in _childParent.Keys)
				{
					string childID = (string)k;
					string parentID = (string)_childParent[k];
					TreeNode  childNode = null;
					TreeNode  parentNode = null;
					if(_nodesHash.ContainsKey(childID))
					{
						childNode = _nodesHash[childID] as TreeNode;
					}
					if(_nodesHash.ContainsKey(parentID))
					{
						parentNode = _nodesHash[parentID] as TreeNode;
					}
					if((parentNode != null) && (childNode != null))
					{
						int index = -1;
						foreach(TreeNode f in parentNode.Nodes)
						{
							if(f.Tag == childNode.Tag)
							{
								index = f.Index;
								break;
							}
						}
						if(index != -1)
							parentNode.Nodes[index] = childNode;
					}
				}
				TreeNode rootNode = _nodesHash["0"] as TreeNode;
				treeView.Nodes.Add(rootNode);
				return true;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(PlantLevelUserTreeControl),"populateTree:"+ex.Message  ,Logger.Level.Debug);
				return false;
			}
			
		}
		
	}
}
