using System;
using System.Drawing;


//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.


namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Generic class to hold the print data information for a report form
	/// </summary>
	public class PrintData
	{
		// Default constructor
		public PrintData()
		{
			headerPrintFont = new Font("Arial", 14);	
		}

		// Properties
		/// <summary>
		/// Sets and returns the header name of the print page
		/// </summary>
		public string HeaderName
		{
			set { header = value; }
			get { return header; }
		}

		/// <summary>
		/// Sets and returns the header Font of the print page
		/// </summary>
		public Font HeaderPrintFont
		{
			set { headerPrintFont = value; }
			get { return headerPrintFont; }
		}

		/// <summary>
		/// Gets the header starting location
		/// </summary>
		public Point Location
		{
			set { location = value; }
			get { return location; }
		}

		/// <summary>
		/// Sets and returns a reference to PrintData table.
		/// </summary>
		public PrintDataTable PrintDataTableObj
		{
			set { dataTable = value; }
			get { return dataTable; }
		}


		// Fields

		private string header;
		private Font headerPrintFont;
		private Point location;
		private PrintDataTable dataTable;
	}


	/// <summary>
	/// Container for all printable rows.
	/// </summary>
	public class PrintDataTable
	{
		/// <summary>
		/// Default Constructor
		/// </summary>
		public PrintDataTable()
		{
		
		}

		// Properties
		
		/// <summary>
		/// Sets and returns the Print row array.
		/// </summary>
		public PrintRow[] Rows
		{
			set { rows = value; }
			get { return rows; }
		}

		// Fields
		private PrintRow[] rows;

	}

	/// <summary>
	/// Contains the details of a printable row
	/// </summary>
	public class PrintRow
	{
		/// <summary>
		/// Default Constructor
		/// </summary>
		public PrintRow()
		{
			// dafault initialization
			labelWidth = 100.0f;
			labelForeColor = Color.Black;
			labelName = "";
			labelPrintFont = new Font("Tahoma", 10, FontStyle.Bold); 
			valueWidth = 400.0f;
			valueForeColor = Color.Black;
			valueName = "";
			valuePrintFont = new Font("Arial", 10);		
		}

		
		// Properties
		
		/// <summary>
		/// Sets or returns the label string
		/// </summary>
		public string LabelName
		{
			set { labelName = value; }
			get { return labelName; }
		}

		/// <summary>
		/// Sets or returns the value string
		/// </summary>
		public string ValueName
		{
			set { valueName = value; }
			get { return valueName; }
		}
		
		/// <summary>
		/// Sets or returns the label width
		/// </summary>
		public float LabelWidth
		{
			set { labelWidth = value; }
			get { return labelWidth; }
		}

		/// <summary>
		/// Sets or returns the value width
		/// </summary>
		public float ValueWidth
		{
			set { valueWidth = value; }
			get { return valueWidth; }
		}

		/// <summary>
		/// Sets or returns the label fore color
		/// </summary>
		public Color LabelForeColor
		{
			set { labelForeColor = value; }
			get { return labelForeColor; }
		}

		/// <summary>
		/// Sets or returns value fore color
		/// </summary>
		public Color ValueForeColor
		{
			set { valueForeColor = value; }
			get { return valueForeColor; }
		}

		/// <summary>
		/// Sets or returns the label font
		/// </summary>
		public Font LabelPrintFont
		{
			set { labelPrintFont = value; }
			get { return labelPrintFont; }
		}

		/// <summary>
		/// Sets or returns the value font
		/// </summary>
		public Font ValuePrintFont
		{
			set { valuePrintFont = value; }
			get { return valuePrintFont; }
		}

		// Fields
		private float labelWidth;
		private Color labelForeColor;
		private string labelName;
		private Font labelPrintFont;
		private float valueWidth;
		private Color valueForeColor;
		private string valueName;
		private Font valuePrintFont;
	}
	  
	
}
