using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.BusinessObjects;

namespace Symcon.LightRidge.PEAdvisor.Reports
{
    public partial class ReportConfiguration : Form
    {
        private const string _folderPath = "/Tutorial/";
        public ReportConfiguration()
        {
            InitializeComponent();
            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

            InitialiseMasterItemCombo();
        }

        private void radbtnSingleSiteAttrRep_MouseClick(object sender, MouseEventArgs e)
        {
        }
        private void InitialiseMasterItemCombo()
        {
            _masterComponentsComboList = LRRModel.Model;
            cmbMasterItemType.DataSource = _masterComponentsComboList;
            cmbMasterItemType.DisplayMember = "ComponentName";
            cmbMasterItemType.ValueMember = "ID";

            InitialiseMasterAttributeCombo();
        }

        private void InitialiseMasterAttributeCombo()
        {
            _selectedMasterComponent = cmbMasterItemType.SelectedItem as LRRMasterComponent;
            _masterProperties = LRRModel.MasterComponentPropertiesObj.GetMasterPropertiesByType(_selectedMasterComponent.ID);

            if (_masterProperties.Count.Equals(0))
            {
                cmbMasterAttribute.Visible = false;
                cmbSiteItem.Visible = false;
                lblMasterAttribute.Visible = false;
                lblSiteItem.Visible = false;
            }
            else
            {
                cmbMasterAttribute.Visible = true;
                lblMasterAttribute.Visible = true;

                cmbMasterAttribute.DataSource = _masterProperties;
                cmbMasterAttribute.DisplayMember = "Name";
                cmbMasterAttribute.ValueMember = "PropID";

                _selectedMasterAttribute = cmbMasterAttribute.SelectedItem as LRRMasterProperty;

                InitialiseSiteItemsCombo();
            }
            
        }

        private void InitialiseSiteItemsCombo()
        {

            if (_selectedMasterComponent != null)
            {
                _siteComponents = LRRModel.SiteComponentsObj.FilterSiteComponentsByMasterComponentID(_selectedMasterComponent.ID);
                if (_siteComponents.Count.Equals(0))
                {
                    cmbSiteItem.Visible = false;
                    lblSiteItem.Visible = false;
                }
                else
                {
                    cmbSiteItem.Visible = true;
                    lblSiteItem.Visible = true;

                    cmbSiteItem.DataSource = _siteComponents;
                    cmbSiteItem.DisplayMember = "Name";
                    cmbSiteItem.ValueMember = "SiteComponentID";

                    _selectedSiteComponent = cmbSiteItem.SelectedItem as LRRSiteComponent;
                }
            }
        }
        #region fields
        private LRRApp                      _app;
        private LRRMasterComponents         _masterComponentsComboList;
        private LRRMasterComponent          _selectedMasterComponent;
        private LRRMasterProperties         _masterProperties;
        private LRRSiteComponent            _selectedSiteComponent;
        private LRRSiteComponents           _siteComponents;
        private LRRMasterProperty           _selectedMasterAttribute;
        private string                      _reportName;
       // private LRRSiteComponent            _selectedSiteItem;
        #endregion

        private void cmbMasterItemType_SelectedIndexChanged(object sender, EventArgs e)
        {
            InitialiseMasterAttributeCombo();
        }

        private void radbtnMasterAttrRep_MouseClick(object sender, MouseEventArgs e)
        {

        }

        private void radbtnSiteAttrRep_MouseClick(object sender, MouseEventArgs e)
        {
        }

        private void cmbMasterAttribute_SelectedIndexChanged(object sender, EventArgs e)
        {
            _selectedMasterAttribute = cmbMasterAttribute.SelectedItem as LRRMasterProperty;
        }

        private void cmbSiteItem_SelectedIndexChanged(object sender, EventArgs e)
        {
            _selectedSiteComponent = cmbSiteItem.SelectedItem as LRRSiteComponent;
        }

        private void btnConfigureReport_Click(object sender, EventArgs e)
        {

        }

        private void btnGenerateReport_Click(object sender, EventArgs e)
        {
            string reportServerURL = "";
            string reportPath = "";
            string masterComponentName = "";
            string masterPropertyName = "";
            string siteComponentName = "";
            DateTime starttime;
            DateTime endtime;

            reportServerURL = txtReportServerURL.Text.ToString();
            reportPath = txtReportPAth.Text.ToString();
            starttime = datetimeStart.Value;
            endtime = datetimeEnd.Value;

            if (radbtnMasterAttrRep.Checked)
            {
                _reportName = "Master Attributes Report";
                masterComponentName = "";
                masterPropertyName = "";
                siteComponentName = "";
            }
            else if(radbtnSiteAttrRep.Checked)
            {
                _reportName = "Site Component Attributes Report";
                masterComponentName = "";
                masterPropertyName = "";
                siteComponentName = "";

            }
            else if (radbtnSingleSiteAttrRep.Checked)
            {
                _reportName = "Single Site Attribute Report";
                masterComponentName = _selectedMasterComponent.ComponentName;
                masterPropertyName = _selectedMasterAttribute.Name;
                siteComponentName = _selectedSiteComponent.Name;

            }
            Reports.Report frmreport = new Report();
            frmreport.GenerateSingleAttributeReport(reportServerURL, reportPath, masterComponentName, masterPropertyName, siteComponentName, starttime, endtime, _reportName);
            frmreport.Show();
        }

        private void radbtnMasterAttrRep_CheckedChanged(object sender, EventArgs e)
        {
            if (radbtnMasterAttrRep.Checked)
            {
                txtReportPAth.Text = _folderPath + "Master Attributes Report";
                groupBox2.Enabled = false;
            }
        }

        private void radbtnSiteAttrRep_CheckedChanged(object sender, EventArgs e)
        {
            if (radbtnSiteAttrRep.Checked)
            {
                txtReportPAth.Text = _folderPath + "Site Component Attributes Report";
                groupBox2.Enabled = false;
            }
        }

        private void radbtnSingleSiteAttrRep_CheckedChanged(object sender, EventArgs e)
        {
            if (radbtnSingleSiteAttrRep.Checked)
            {
                txtReportPAth.Text = _folderPath + "Single Site Attribute Report";
                groupBox2.Enabled = true;
            }
        }
    }
}