using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Diagnostics;

using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;

using Symcon.Utils.MessageLogger;
using Symcon.Database;
using Symcon.LightRidge.Options.DBOptions;
using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Globals.Enums;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;

namespace Symcon.LightRidge.PEAdvisor
{
    public partial class EffectivePricesMapping : Form , ILRRUIContainer
    {
        public EffectivePricesMapping()
        {
            _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitializeComponent();
            InitDataSource();
            BindPricesCombo();
            BindMasterPlantItems();
           
            BindGrid();
            if (_levels.Count > 0)
            {
                InitializeTreeView(trvLevels, _levels, _lrrLevelLinks);
                trvLevels.TreeViewNodeSorter = new TreeNodeSorter();
                trvLevels.Sort();
            }
        }

        private void BindMasterPlantItems()
        {
            ArrayList masterCompList = new ArrayList();
            foreach (LRRMasterComponent masterComp in _lrrMasterComponents)
            {
                if (!masterCompList.Contains(masterComp))
                    masterCompList.Add(masterComp);
            }
            cmbMasterPlantItems.DataSource = masterCompList;
            cmbMasterPlantItems.DisplayMember = "ComponentName";
        }

        private void BindGrid()
        {
            gridPriceMappings.DataSource = _effectivePrices;
            SyncGridUtilities.InitializeGridLookAndFeel(gridPriceMappings);
            gridPriceMappings.TableDescriptor.Columns.Clear();

            GridColumnDescriptor gcd,gcd1,gcd2,gcd3;
          
            gcd = new GridColumnDescriptor();
            gcd.HeaderText = LRRHeaderText.GetHeaderString("SID_POWER_E_PRICES");
            gcd.MappingName = "DisplayPowerEffectivePriceID";
            gcd.AllowFilter = true;
            gridPriceMappings.TableDescriptor.Columns.Add(gcd);
            gcd.Appearance.AnyRecordFieldCell.ReadOnly = true;

            gcd1 = new GridColumnDescriptor();
            gcd1.HeaderText = LRRHeaderText.GetHeaderString("SID_FUEL_E_PRICES");
            gcd1.MappingName = "DisplayFuelEffectivePriceID";
            gcd1.AllowFilter = true;
            gridPriceMappings.TableDescriptor.Columns.Add(gcd1);
            gcd1.Appearance.AnyRecordFieldCell.ReadOnly = true;

            gcd2 = new GridColumnDescriptor();
            gcd2.HeaderText = LRRHeaderText.GetHeaderString("SID_STEAM_E_PRICES");
            gcd2.MappingName = "DisplaySteamEffectivePriceID";
            gcd2.AllowFilter = true;
            gridPriceMappings.TableDescriptor.Columns.Add(gcd2);
            gcd2.Appearance.AnyRecordFieldCell.ReadOnly = true;

            gcd3 = new GridColumnDescriptor();
            gcd3.HeaderText = LRRHeaderText.GetHeaderString("SID_DIVIDING_FACTOR");
            gcd3.MappingName = "DisplayDividingFactorID";
            gcd3.AllowFilter = true;
            gridPriceMappings.TableDescriptor.Columns.Add(gcd3);
            gcd3.Appearance.AnyRecordFieldCell.ReadOnly = true;


            gridPriceMappings.TableModel.CommandStack.Enabled = true;
            gridPriceMappings.TableControl.Refresh();

        }

        private void BindPricesCombo()
        {
           this.comboBox1.Items.Add(LRRHeaderText.GetHeaderString("SID_FUEL_E_PRICES"));
           this.comboBox1.Items.Add(LRRHeaderText.GetHeaderString("SID_POWER_E_PRICES"));
           this.comboBox1.Items.Add(LRRHeaderText.GetHeaderString("SID_STEAM_E_PRICES"));
           this.comboBox1.Items.Add(LRRHeaderText.GetHeaderString("SID_DIVIDING_FACTOR"));
        }
        private void InitDataSource()
        {
            _lrrMasterComponents = LRRModel.Model;
            _lrrMasterProperties = LRRModel.MasterComponentPropertiesObj; 
            _levels = LRRModel.ChildModel.Levels;
            _lrrLevelLinks = new LRRLevelLinks();
            _lrrApp.DBManagerObj.PopulateCollection(_lrrLevelLinks);
        }


        private bool InitializeTreeView(TreeView treeView, LRRLevels levels, LRRLevelLinks levelLinks)
        {
            treeView.Nodes.Clear();
            try
            {
                Hashtable _levelHash = new Hashtable();
                Hashtable _nodesHash = new Hashtable();
                Hashtable _childParent = new Hashtable();

                foreach (LRRLevel l in levels)
                {
                    _levelHash.Add(l.LevelID, l);
                }
                if (levelLinks.Count == 0 && levels.Count == 1)
                {
                    //In here only when we have single entry on Level and No entry in Level Links
                    LRRLevel lonerLevel = _levelHash["0"] as LRRLevel;
                    if (lonerLevel != null)
                    {
                        TreeNode singleRootNode = new TreeNode(lonerLevel.Name);
                        singleRootNode.Tag = lonerLevel;
                        treeView.Nodes.Add(singleRootNode);
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
                foreach (LRRLevelLink ll in levelLinks)
                {
                    _childParent.Add(ll.ChildID, ll.ParentID);

                    if (_nodesHash.ContainsKey(ll.ParentID))
                    {
                        TreeNode foundNode = _nodesHash[ll.ParentID] as TreeNode;
                        LRRLevel foundLevel = _levelHash[ll.ChildID] as LRRLevel;
                        TreeNode childNode = new TreeNode(foundLevel.Name);
                        childNode.Tag = foundLevel;
                        foundNode.Nodes.Add(childNode);
                    }
                    else
                    {
                        LRRLevel parentLevel = _levelHash[ll.ParentID] as LRRLevel;
                        LRRLevel childLevel = _levelHash[ll.ChildID] as LRRLevel;
                        TreeNode parentNode = new TreeNode(parentLevel.Name);
                        parentNode.Tag = parentLevel;
                        TreeNode childNode = new TreeNode(childLevel.Name);
                        childNode.Tag = childLevel;
                        parentNode.Nodes.Add(childNode);
                        _nodesHash.Add(ll.ParentID, parentNode);
                    }
                }

                foreach (object k in _childParent.Keys)
                {
                    string childID = (string)k;
                    string parentID = (string)_childParent[k];
                    TreeNode childNode = null;
                    TreeNode parentNode = null;
                    if (_nodesHash.ContainsKey(childID))
                    {
                        childNode = _nodesHash[childID] as TreeNode;
                    }
                    if (_nodesHash.ContainsKey(parentID))
                    {
                        parentNode = _nodesHash[parentID] as TreeNode;
                    }
                    if ((parentNode != null) && (childNode != null))
                    {
                        int index = -1;
                        foreach (TreeNode f in parentNode.Nodes)
                        {
                            if (f.Tag == childNode.Tag)
                            {
                                index = f.Index;
                                break;
                            }
                        }
                        if (index != -1)
                            parentNode.Nodes[index] = childNode;
                    }
                }

                TreeNode rootNode = _nodesHash["0"] as TreeNode;
                rootNode.ExpandAll(); 
                treeView.Nodes.Add(rootNode);
                return true;
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(PlantLevels), "PopulateLevels:" + ex.Message, Logger.Level.Debug);
                return false;
            }
        }

        

        #region Fields

        private LRRLevels _levels;
        private LRRLevelLinks _lrrLevelLinks;
        private LRRApp _lrrApp; //All object to access the DB Manager and DB Adapter
        private LRRMasterComponents _lrrMasterComponents;
        private LRRMasterProperties _lrrMasterProperties;
        private GridUnBoundColumn gridUnBoundSiteItem = new GridUnBoundColumn();
        private TabControl _parentTabControl;
        private TabPage _parentTabPage;
        private TreeNode				prevSelNode = null;
        private  bool FUEL_E_P_ID;
        private bool POWER_E_P_ID;
        private bool STEAM_E_P_ID;
        private bool DIVIDING_FACTOR_ID;
       
        #endregion

        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearSort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Copy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyMasterProperties()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyUOMTypes()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Delete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DownLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Export()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void InsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenCompositeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenQuery()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void OpenView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PageSetup()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public TabControl ParentTabControl
        {
            set
            {
                _parentTabControl = value;
            }
            get { return _parentTabControl; }
        }

        public TabPage ParentTabPage
        {
            set
            {
                _parentTabPage = value;
            }
            get
            {
                return _parentTabPage;
            }
        }

        public void Paste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool PerformDirtyChk
        {
            get
            {
                // TODO:  Add MasterPlantItems.PerformDirtyChk getter implementation
                return true;
            }
        }

        public void Print()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PrintPreview()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightColMovement()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Save()
        {
            LRREffectivePrices effectivePrices = LRRModel.EffectivePrices.GetChanges();
            if (effectivePrices.Count > 0)
            {
                _lrrApp.DBManagerObj.CommitCollection(effectivePrices);
                LRRModel.EffectivePrices.RefreshCollection();
                LRRMessagBox.ShowMessage("SID_SAVEDINFO", true);
            }
            else
            {
                LRRMessagBox.ShowMessage("SID_NODATASAVED", true);
            }
        }

        public void SelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowComponentList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ShowPropertyList()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsArrangeColumns()
        {
            return false;
        }

        public bool SupportsClearFilter()
        {
            return false;
        }

        public bool SupportsClearSort()
        {
            return false;
        }

        public bool SupportsCompositeView()
        {
            return false;
        }

        public bool SupportsCopy()
        {
            return false;
        }

        public bool SupportsCopyUOMTypes()
        {
            return false;
        }

        public bool SupportsDelete()
        {
            return false;
        }

        public bool SupportsExport()
        {
            return false;
        }

        public bool SupportsImportToExcel()
        {
            return false;
        }

        public bool SupportsInsertNewRows()
        {
            return false;
        }

        public bool SupportsLevel()
        {
            return false;
        }

        public bool SupportsMasterPropertiesCopy()
        {
            return false;
        }

        public bool SupportsPageSetup()
        {
            return false;
        }

        public bool SupportsPaste()
        {
            return false;
        }

        public bool SupportsPrint()
        {
            return false;
        }

        public bool SupportsRefresh()
        {
            return false;
        }

        public bool SupportsResultsFilter()
        {
            return false;
        }

        public bool SupportsSave()
        {
            return true;
        }

        public bool SupportsSelectionPaste()
        {
            return false;
        }

        public bool SupportsUndo()
        {
            return false;
        }

        public void TransposeView()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Undo()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }
        #endregion

        private void trvLevels_AfterSelect(object sender, TreeViewEventArgs e)
        {
            _effectivePrices.Clear();
            cmbMasterPlantItems.SelectedIndex = -1;
            if (prevSelNode != null)
            {
                prevSelNode.ForeColor = Color.Black;
            }
            e.Node.ForeColor = Color.Blue;
            prevSelNode = e.Node;
            LRRLevel selLevel = new LRRLevel();
            selLevel = e.Node.Tag as LRRLevel;          
            _effectivePrice = LRRModel.EffectivePrices.GetEffectivePriceObjForLevel(selLevel.LevelID) as LRREffectivePrice;
            if (_effectivePrice != null)
            {
                _effectivePrices.Add(_effectivePrice);
                string mappedMasterPropID = null;
                if (this.comboBox1.SelectedItem != null)
                {
                    string currentPrice = this.comboBox1.Text;
                    if (currentPrice == LRRHeaderText.GetHeaderString("SID_FUEL_E_PRICES"))
                    {
                        if (_effectivePrice.FuelEffectivePriceID != "" || _effectivePrice.FuelEffectivePriceID != null)
                        {
                            mappedMasterPropID = _effectivePrice.FuelEffectivePriceID;
                        }
                    }
                    else if (currentPrice == LRRHeaderText.GetHeaderString("SID_POWER_E_PRICES"))
                    {
                        if (_effectivePrice.PowerEffectivePriceID != "" || _effectivePrice.PowerEffectivePriceID != null)
                        {
                            mappedMasterPropID = _effectivePrice.PowerEffectivePriceID;
                        }
                    }
                    else if (currentPrice == LRRHeaderText.GetHeaderString("SID_STEAM_E_PRICES"))
                    {
                        if (_effectivePrice.SteamEffectivePriceID != "" || _effectivePrice.SteamEffectivePriceID != null)
                        {
                            mappedMasterPropID = _effectivePrice.SteamEffectivePriceID;
                        }
                    }
                    else if (currentPrice == LRRHeaderText.GetHeaderString("SID_DIVIDING_FACTOR"))
                    {
                        if (_effectivePrice.DividingFactorID != "" || _effectivePrice.DividingFactorID != null)
                        {
                            mappedMasterPropID = _effectivePrice.DividingFactorID;
                        }
                    }
                }

                if (mappedMasterPropID != null)
                {
                    LRRMasterProperty masterProperty = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(mappedMasterPropID);
                    LRRMasterComponent masterComponent = LRRModel.Model.GetMasterComponentByID(masterProperty.ComponentID);
                    for (int i = 0; i < cmbMasterPlantItems.Items.Count; i++)
                    {
                        LRRMasterComponent masterComp = cmbMasterPlantItems.Items[i] as LRRMasterComponent;
                        if (masterComp.ComponentName == masterComponent.ComponentName)
                        {
                            cmbMasterPlantItems.SelectedIndex = i;
                        }
                    }
                    gridUnBoundSiteItem.SetRecord(masterProperty, true);
                }
                BindGrid();
            }
            //gridUnBoundSiteItem.ClearSelectedRecords();   
        }

       

        private void button3_Click(object sender, EventArgs e)
        {
            object[] selectedProp = gridUnBoundSiteItem.GetSelectedRecords();
            if (selectedProp.Length > 1)
            {
                LRRMessagBox.ShowMessage(this, "SID_PROPSELECTION_ERROR", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                gridUnBoundSiteItem.SetRecord(gridUnBoundSiteItem.LastRecord, false);
            }
            LRREffectivePrice effectivePrice = null;
            bool isNewEntry = false;
            if (prevSelNode != null)
            {
                LRRLevel level = prevSelNode.Tag as LRRLevel;
                effectivePrice = LRRModel.EffectivePrices.GetEffectivePriceObjForLevel(level.LevelID);
                if (effectivePrice == null)
                {
                    effectivePrice = new LRREffectivePrice();
                    effectivePrice.LevelID = level.LevelID;
                    LRRModel.EffectivePrices.Add(effectivePrice);
                    isNewEntry = true;
                }
            }
            if (this.comboBox1.SelectedItem != null)
            {
                string currentPrice = this.comboBox1.Text;
                if (currentPrice == LRRHeaderText.GetHeaderString("SID_FUEL_E_PRICES"))
                {
                    FUEL_E_P_ID = true;
                    POWER_E_P_ID = false;
                    STEAM_E_P_ID = false;
                    DIVIDING_FACTOR_ID = false;
                }
                else if (currentPrice == LRRHeaderText.GetHeaderString("SID_POWER_E_PRICES"))
                {
                    FUEL_E_P_ID = false;
                    POWER_E_P_ID = true;
                    STEAM_E_P_ID = false;
                    DIVIDING_FACTOR_ID = false;
                }
                else if (currentPrice == LRRHeaderText.GetHeaderString("SID_STEAM_E_PRICES"))
                {
                    FUEL_E_P_ID = false;
                    POWER_E_P_ID = false;
                    STEAM_E_P_ID = true;
                    DIVIDING_FACTOR_ID = false;
                }
                else if (currentPrice == LRRHeaderText.GetHeaderString("SID_DIVIDING_FACTOR"))
                {
                    FUEL_E_P_ID = false;
                    POWER_E_P_ID = false;
                    STEAM_E_P_ID = false;
                    DIVIDING_FACTOR_ID = true;
                }
            }
            LRRMasterProperty masterProp = null;
            if (selectedProp !=null && selectedProp.Length > 0)
            {
                masterProp = selectedProp[0] as LRRMasterProperty;
            }
            if (masterProp != null)
            {
                if (FUEL_E_P_ID)
                {
                    if (isNewEntry)
                    {
                        effectivePrice.FuelEffectivePriceID = masterProp.PropID;
                    }
                    else
                    {
                        if (effectivePrice.FuelEffectivePriceID == "" || effectivePrice.FuelEffectivePriceID == null)
                        {
                            effectivePrice.FuelEffectivePriceID = masterProp.PropID;
                        }
                        else
                        {
                            DialogResult result = LRRMessagBox.ShowMessage("SID_DUPLICATE_HISTMAPPING", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                            if (result == DialogResult.Yes)
                            {
                                effectivePrice.FuelEffectivePriceID = masterProp.PropID;
                            }
                        }
                    }

                }
                else if (POWER_E_P_ID)
                {
                    if (isNewEntry)
                    {
                        effectivePrice.PowerEffectivePriceID = masterProp.PropID;
                    }
                    else
                    {
                        if (effectivePrice.PowerEffectivePriceID == "" || effectivePrice.PowerEffectivePriceID == null)
                        {
                            effectivePrice.PowerEffectivePriceID = masterProp.PropID;
                        }
                        else
                        {
                            DialogResult result = LRRMessagBox.ShowMessage("SID_DUPLICATE_HISTMAPPING", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                            if (result == DialogResult.Yes)
                            {
                                effectivePrice.PowerEffectivePriceID = masterProp.PropID;
                            }
                        }
                    }
                }
                else if (STEAM_E_P_ID)
                {
                    if (isNewEntry)
                    {
                        effectivePrice.SteamEffectivePriceID = masterProp.PropID;
                    }
                    else
                    {
                        if (effectivePrice.SteamEffectivePriceID == "" || effectivePrice.SteamEffectivePriceID == null)
                        {
                            effectivePrice.SteamEffectivePriceID = masterProp.PropID;
                        }
                        else
                        {
                            DialogResult result = LRRMessagBox.ShowMessage("SID_DUPLICATE_HISTMAPPING", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                            if (result == DialogResult.Yes)
                            {
                                effectivePrice.SteamEffectivePriceID = masterProp.PropID;
                            }
                        }
                    }
                }
                else if (DIVIDING_FACTOR_ID)
                {
                    if (isNewEntry)
                    {
                        effectivePrice.DividingFactorID = masterProp.PropID;
                    }
                    else
                    {
                        if (effectivePrice.DividingFactorID == "" || effectivePrice.DividingFactorID == null)
                        {
                            effectivePrice.DividingFactorID = masterProp.PropID;
                        }
                        else
                        {
                            DialogResult result = LRRMessagBox.ShowMessage("SID_DUPLICATE_HISTMAPPING", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                            if (result == DialogResult.Yes)
                            {
                                effectivePrice.DividingFactorID = masterProp.PropID;
                            }
                        }
                    }
                }
                BindGrid();
            }
        }
        private void gridMasterPlantItems_TableControlCheckBoxClick(object sender, GridTableControlCellClickEventArgs e)
        {
            
        }

        private void trvLevels_MouseMove(object sender, MouseEventArgs e)
        {

        }

        private void cmbMasterPlantItems_SelectedIndexChanged(object sender, EventArgs e)
        {
            LRRMasterComponent masterComp = new LRRMasterComponent();
            masterComp = cmbMasterPlantItems.SelectedItem as LRRMasterComponent;
            BindMasterPropertiesGrid(masterComp);
            gridUnBoundSiteItem.ClearSelectedRecords();
        }

        private void BindMasterPropertiesGrid(LRRMasterComponent masterComp)
        {
            LRRMasterProperties masterProps = new LRRMasterProperties();
            if (masterComp != null)
            {
                masterProps = LRRModel.MasterComponentPropertiesObj.GetMasterPropertiesByType(masterComp.ID);
            }
            SyncGridUtilities.InitializeGridLookAndFeel(gridMasterProperties);
            gridMasterProperties.TableDescriptor.Columns.Clear();
            gridUnBoundSiteItem.WireGrid(gridMasterProperties);
            //Columns to be shown in this view...  

            gridMasterProperties.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_NAME"), "Name");
            gridMasterProperties.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_DESCRIPTION"), "Description");
            gridMasterProperties.TableDescriptor.Columns.Add(LRRHeaderText.GetHeaderString("SID_ROLL_UP_TYPE"), "RollUpTypeID");

            Hashtable RelativeWidths = new Hashtable();
            RelativeWidths.Add("Name", 30);
            RelativeWidths.Add("Description", 30);
            RelativeWidths.Add("RollUpTypeID", 30);
            SyncGridUtilities.InitializeGridColumnsWidth(gridMasterProperties, RelativeWidths);

            gridMasterProperties.DataSource = masterProps;         
            gridMasterProperties.TableDescriptor.AllowEdit = true;
            gridMasterProperties.TableDescriptor.AllowNew = false;
            gridMasterProperties.TableDescriptor.Columns["Name"].ReadOnly = true;
            gridMasterProperties.TableDescriptor.Columns["Description"].ReadOnly = true;
            gridMasterProperties.TableDescriptor.Columns["RollupType"].Appearance.AnyRecordFieldCell.CellType = "ComboBox";
            gridMasterProperties.TableDescriptor.Columns["RollupType"].Appearance.AnyRecordFieldCell.ExclusiveChoiceList = false;
            gridMasterProperties.TableDescriptor.Columns["RollupType"].Appearance.AnyRecordFieldCell.ShowButtons = GridShowButtons.Hide;
            gridMasterProperties.TableDescriptor.Columns["RollupType"].Appearance.AnyRecordFieldCell.ExclusiveChoiceList = false;
            gridMasterProperties.TableDescriptor.Columns["RollupType"].Appearance.AnyRecordFieldCell.ReadOnly = true;
            gridMasterProperties.TableDescriptor.Columns["RollupType"].Appearance.AnyRecordFieldCell.DataSource = LRRModel.RollUpTypes; 
            gridMasterProperties.TableDescriptor.Columns["RollupType"].Appearance.AnyRecordFieldCell.DisplayMember = "RollUpType";
            gridMasterProperties.TableDescriptor.Columns["RollupType"].Appearance.AnyRecordFieldCell.ValueMember = "ID";

           
            //Resume the Layout of the Grid...
            gridMasterProperties.ResumeLayout(false);


        }

        #region Fields
        LRREffectivePrice _effectivePrice = new LRREffectivePrice();
        LRREffectivePrices _effectivePrices = new LRREffectivePrices();
        #endregion

        private void gridMasterProperties_CurrentRecordContextChange(object sender, Syncfusion.Grouping.CurrentRecordContextChangeEventArgs e)
        {
            if (e.Action == Syncfusion.Grouping.CurrentRecordAction.EndEditComplete)
            {
                object[] objArray = gridUnBoundSiteItem.GetSelectedRecords();
                if (objArray.Length > 1)
                {
                    LRRMessagBox.ShowMessage(this, "SID_PROPSELECTION_ERROR", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    gridUnBoundSiteItem.SetRecord(gridUnBoundSiteItem.LastRecord, false);
                    return;
                }
            }
        }
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            gridUnBoundSiteItem.ClearSelectedRecords();
            cmbMasterPlantItems.SelectedIndex = -1;
            LRRLevel selLevel = new LRRLevel();
            selLevel = prevSelNode.Tag as LRRLevel;
            _effectivePrice = LRRModel.EffectivePrices.GetEffectivePriceObjForLevel(selLevel.LevelID) as LRREffectivePrice;
            if (_effectivePrice != null)
            {
                string mappedMasterPropID = null;
                if (this.comboBox1.SelectedItem != null)
                {
                    string currentPrice = this.comboBox1.Text;
                    if (currentPrice == LRRHeaderText.GetHeaderString("SID_FUEL_E_PRICES"))
                    {
                        if (_effectivePrice.FuelEffectivePriceID != "" || _effectivePrice.FuelEffectivePriceID != null)
                        {
                            mappedMasterPropID = _effectivePrice.FuelEffectivePriceID;
                        }
                    }
                    else if (currentPrice == LRRHeaderText.GetHeaderString("SID_POWER_E_PRICES"))
                    {
                        if (_effectivePrice.PowerEffectivePriceID != "" || _effectivePrice.PowerEffectivePriceID != null)
                        {
                            mappedMasterPropID = _effectivePrice.PowerEffectivePriceID;
                        }
                    }
                    else if (currentPrice == LRRHeaderText.GetHeaderString("SID_STEAM_E_PRICES"))
                    {
                        if (_effectivePrice.SteamEffectivePriceID != "" || _effectivePrice.SteamEffectivePriceID != null)
                        {
                            mappedMasterPropID = _effectivePrice.SteamEffectivePriceID;
                        }
                    }
                    else if (currentPrice == LRRHeaderText.GetHeaderString("SID_DIVIDING_FACTOR"))
                    {
                        if (_effectivePrice.DividingFactorID != "" || _effectivePrice.DividingFactorID != null)
                        {
                            mappedMasterPropID = _effectivePrice.DividingFactorID;
                        }
                    }
                }

                if (mappedMasterPropID != null)
                {
                    LRRMasterProperty masterProperty = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(mappedMasterPropID);
                    LRRMasterComponent masterComponent = LRRModel.Model.GetMasterComponentByID(masterProperty.ComponentID);
                    for (int i = 0; i < cmbMasterPlantItems.Items.Count; i++)
                    {
                        LRRMasterComponent masterComp = cmbMasterPlantItems.Items[i] as LRRMasterComponent;
                        if (masterComp.ComponentName == masterComponent.ComponentName)
                        {
                            cmbMasterPlantItems.SelectedIndex = i;
                        }
                    }
                    gridUnBoundSiteItem.SetRecord(masterProperty, true);
                }
                BindGrid();
            }
        }

    }
}