using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Reflection;
using Microsoft.Reporting.WinForms;
using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects;
using Microsoft.SqlServer.ReportingService2005;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher; 

namespace Symcon.LightRidge.PEAdvisor.Reports
{
	public partial class LRRReportViewer : Form,ILRRUIContainer	
	{
		
		#region [Fields] 
			ReportingService2005 rService = new ReportingService2005();		
			string path=Application.StartupPath + @"\config\ReportServerConfig.xml";
			//string path=@"C:\Program Files\LightRidge Resources\PE-Advisor\config\ReportServerConfig.xml";
			XmlDocument xDoc = new XmlDocument(); 
			string _reportName="";		
			string _reportingServerName="";
			string rServerName="";
			private TabControl			_parentTabControl;
			private TabPage				_parentTabPage;
			private LRRApp	_lrrApp;

		#endregion [Fields]

		public LRRReportViewer()
		{
			xDoc.Load(path);
			 _lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp; 
			InitializeComponent();
			InitializeReportTree();
			
		}

		#region[Methods]
		 		#region[InitializeReportTree]
				public void InitializeReportTree()
				{		  
					rService.Credentials = System.Net.CredentialCache.DefaultCredentials;
					LRRDefaultRSInfo defaultReportServer = new LRRDefaultRSInfo();
					_lrrApp.DBManagerObj.PopulateObject(defaultReportServer,typeof(LRRDefaultRSInfo),"");
					string server = defaultReportServer.ReportServerName;
					//if (server != null)
					if(!string.IsNullOrEmpty(server))
					{
						rServerName = server.Trim();
						//if (rServerName != null && rServerName != "")
						if(!string.IsNullOrEmpty(rServerName))
						{
							// ReportingServerName=@"http://"+txtRSName.Text+"/ReportServer";
							_reportingServerName = @"http://" + rServerName + "/ReportServer/ReportService2005.asmx";
							rService.Url = _reportingServerName;
							CatalogItem[] catalogItems;
							catalogItems = rService.ListChildren(ReportInfo.InitialReportPath, true);
							BuildTree(catalogItems);
							AddImagesToTreeView(trvReportList);
						}
					}
					else { 
						//Reporting Server is not configured message
						LRRMessagBox.ShowMessage("SID_NO_ReportServer","SID_LABEL",MessageBoxButtons.OK, MessageBoxIcon.Information);     						
					}
				}
			#endregion[InitializeReportTree]

			#region [AddImagesToTreeView]
			public void AddImagesToTreeView(TreeView trvReportList)
			{
				trvReportList.ImageList=imageList1;
				//trvReportList.StateImageList=imageList2;
				foreach(TreeNode node in trvReportList.Nodes)
					 {
						 if(node.GetType().Name=="TreeNode")
						 {
							 node.ImageIndex=0;							 
							 node.SelectedImageIndex=2;							 
							 //node.StateImageIndex=0;
							 foreach(TreeNode childNode in node.Nodes)
							 {
								 childNode.ImageIndex=1;
								 childNode.SelectedImageIndex=3;
								 //childNode.StateImageIndex=1;
							 }
						 }
						
					 }
			}
			#endregion[AddImagesToTreeView]
			#region[BuildTree]
			private void BuildTree(CatalogItem[] catalogItems)
			{
				Boolean rFlag=false;
				foreach(CatalogItem item in catalogItems)
				{
					if(item.Type == ItemTypeEnum.Report)
					{
							rFlag=true;
							string path = item.Path.Remove(0, ReportInfo.InitialReportPath.Length);
							string[] tokens = path.Split(ReportInfo.PathSeparatorArray);
							BuildNodesFromTokens(tokens, 0, trvReportList.Nodes);          
					}
				}
				if(!rFlag)
				{
						MessageBox.Show("There are no reports available in the Reporting Server");
						splitContainer1.Visible=false;
				}
				else
				{
					splitContainer1.Visible=true;
				}
			}
			#endregion[BuildTree]

			#region[BuildNodesFromTokens]
				private void BuildNodesFromTokens(string[] tokens, int index, 
						TreeNodeCollection nodes)
				{
					 TreeNode node = null;

					 // first, see if a node for the current token
					 // exists on this level
					 for(int i = 0; i < nodes.Count; i++)
					 {
						  if(nodes[i].Text == tokens[index])
						  {
								// a node was found at this level, 
								// no need to continue searching
								node = nodes[i];								
								break;
						  }
					 }
				    
					 // no node was found, will need to 
					 // create a new tree node
					 if(node == null)
					 {
						 
						  node = new TreeNode(); 
						  node.Text = tokens[index]; 
						  nodes.Add(node); 
					 }

					 // if we have not reached the end of the token list,
					 // increase the level by (move down the tree) and 
					 // call function again
					 index++;
					 if(tokens.Length > index)
					 {
						  BuildNodesFromTokens(tokens, index, node.Nodes);
					 } 
				}


			#endregion[BuildNodesFromTokens]		

			#region[AddReportDetailsInfo]
		public void AddReportDetailsInfo(ReportInfo reportInfo)
		{
		      XmlNode node =  xDoc.SelectSingleNode("//ReportDetails");		
		      string value="DBName="+reportInfo.DBName+";DataSource="+reportInfo.DBName+";ReportPath="+reportInfo.ReportPath+"";
		       XmlElement elem = (XmlElement)node.SelectSingleNode(string.Format("//Report[@ReportName='{0}']", _reportName ));
		       if(elem != null)
		       {
		         if(_reportName==elem.GetAttribute("ReportName"))						
		         {
					 
		            elem.SetAttribute("ReportServerURL",reportInfo.ReportServerURL);
						elem.SetAttribute("ReportPath",reportInfo.ReportPath);
						elem.SetAttribute("DatabaseServer",reportInfo.DatabaseServer);
						elem.SetAttribute("DatabaseName",reportInfo.DBName);
						elem.SetAttribute("DataSource",reportInfo.DBName);
						elem.SetAttribute("UserName",reportInfo.UserName);
						elem.SetAttribute("PassWord",reportInfo.Password); 
		            node.AppendChild(elem);
		         }
		         else
		         {
		            elem = xDoc.CreateElement("Report");
						elem.SetAttribute("ReportName",reportInfo.ReportName);
		            elem.SetAttribute("ReportServerURL",reportInfo.ReportServerURL);
						elem.SetAttribute("ReportPath",reportInfo.ReportPath);
						elem.SetAttribute("DatabaseServer",reportInfo.DatabaseServer);
						elem.SetAttribute("DatabaseName",reportInfo.DBName);
						elem.SetAttribute("DataSource",reportInfo.DBName);
						elem.SetAttribute("UserName",reportInfo.UserName);
						elem.SetAttribute("PassWord",reportInfo.Password);
		            node.AppendChild(elem);
		         }				
		       }
		       else
		       {  
						elem = xDoc.CreateElement("Report");
						elem.SetAttribute("ReportName",reportInfo.ReportName);
		            elem.SetAttribute("ReportServerURL",reportInfo.ReportServerURL);
						elem.SetAttribute("ReportPath",reportInfo.ReportPath);
						elem.SetAttribute("DatabaseServer",reportInfo.DatabaseServer);
						elem.SetAttribute("DatabaseName",reportInfo.DBName);
						elem.SetAttribute("DataSource",reportInfo.DBName);
						elem.SetAttribute("UserName",reportInfo.UserName);
						elem.SetAttribute("PassWord",reportInfo.Password);
		            node.AppendChild(elem);
		       }
				 
		       xDoc.Save(path);
		}
		#endregion[AddReportDetailsInfo]

			#region [GetReportDetails]
		public ReportInfo GetReportDetails(string reportName)
		{
			ReportInfo rInfo = new ReportInfo();
				rService.Credentials = System.Net.CredentialCache.DefaultCredentials;
			CatalogItem[] catalogItems;
			DataSource[] dataSources = null;
			catalogItems = rService.ListChildren(ReportInfo.InitialReportPath,true); 
			
			foreach(CatalogItem item in catalogItems)
				{
					if(item.Type == ItemTypeEnum.Report)
					{
						if(item.Name==_reportName)
						{
							rInfo.ReportName= item.Name;        
							rInfo.ReportPath=item.Path;
							dataSources =rService.GetItemDataSources(item.Path);
							foreach(DataSource ds in dataSources)
							{				
							//	DataSourceDefinition ddReport =rService.GetDataSourceContents("/Data Sources/"+item.Name);
								DataSourceDefinition ddReport=((Microsoft.SqlServer.ReportingService2005.DataSourceDefinition)(dataSources[0].Item));
								string connStringValue=ddReport.ConnectString;
								string[] connDetails = connStringValue.Split(";".ToCharArray());

								//reporting server name
								rInfo.ReportServerURL="http://"+rServerName+"/ReportServer";

								//report path
								rInfo.ReportPath=item.Path;

								//datasource name
								rInfo.DataSourceName = item.Name;
								//database server name
								string[] dsDetails = connDetails[0].Split("=".ToCharArray());
								rInfo.DatabaseServer=dsDetails[1];

								//database name
								string[] dbDetails = connDetails[1].Split("=".ToCharArray());
								rInfo.DBName= dbDetails[1];

								//username
								string[] userNameDetails = connDetails[2].Split("=".ToCharArray());
								rInfo.UserName= userNameDetails[1]; 

								//password
								string[] PWDDetails = connDetails[3].Split("=".ToCharArray());
								rInfo.Password= PWDDetails[1]; 
 
							}
							break;
						}							
					}
				} 


			/*XmlNode node =  xDoc.SelectSingleNode("//ReportDetails");	
				if(node!=null)
				{
					 XmlElement elem = (XmlElement)node.SelectSingleNode(string.Format("//Report[@ReportName='{0}']", reportName ));
					if(elem!=null)
					{
						rInfo.ReportServerURL=elem.GetAttribute("ReportServerURL");
						rInfo.ReportPath=elem.GetAttribute("ReportPath");
						rInfo.DatabaseServer=elem.GetAttribute("DatabaseServer");
						rInfo.DBName=elem.GetAttribute("DatabaseName");
						rInfo.DataSource=elem.GetAttribute("DataSource");
						rInfo.UserName=elem.GetAttribute("UserName");
						rInfo.Password=elem.GetAttribute("PassWord");

					}
				}*/
			return rInfo;
		}
		#endregion[GetReportDetails]
	
		 
		 
	
		 
		#endregion[Methods]

		#region [Events]
		private void LRRReportViewer_Load(object sender, EventArgs e)
		{

			this.rvReportDisplay.RefreshReport();
		}
		 
	

		private void trvReportList_AfterSelect(object sender, TreeViewEventArgs e)
		{
		  
			string selectReportName="";
			string selectReportServerURL="" ;
			string selectReportPath="" ;
			string selectDatabaseServer="" ;
			string selectDatabaseName="" ;
			string selectUserName ="";
			string selectPassWord =""; 
			string selectDatasourceName="";
			ReportInfo myReportInfo = new ReportInfo();
			if(trvReportList.SelectedNode.Nodes.Count==0)			
			{
				_reportName=e.Node.Text;				
			}
			else
			{				
				_reportName="";
				
			}
			if(_reportName!="")
			{
				myReportInfo=GetReportDetails(_reportName);
				//code to bind the datasource to the report viewer
				selectReportName=_reportName;
				selectReportServerURL=myReportInfo.ReportServerURL ;
				selectReportPath=myReportInfo.ReportPath;
				selectDatabaseServer=myReportInfo.DatabaseServer ;
				selectDatabaseName=myReportInfo.DBName ;
				selectDatasourceName=myReportInfo.DataSourceName;
				selectUserName =myReportInfo.UserName;
				selectPassWord =myReportInfo.Password; 


				rvReportDisplay.ProcessingMode = ProcessingMode.Remote;
				ServerReport serverReport = rvReportDisplay.ServerReport; 
			 
				rvReportDisplay.ServerReport.ReportServerUrl = new Uri(selectReportServerURL);
				rvReportDisplay.PromptAreaCollapsed = true; 
				rvReportDisplay.ShowProgress = true; 
				rvReportDisplay.ServerReport.ReportPath =selectReportPath;
				Microsoft.Reporting.WinForms.DataSourceCredentials dataSourceCredential = new Microsoft.Reporting.WinForms.DataSourceCredentials(); 	 			
				Microsoft.Reporting.WinForms.DataSourceCredentials[] dataSourceCredentials=null;
				dataSourceCredential.UserId = selectUserName; 
				dataSourceCredential.Password = selectPassWord; 

				try{
				ReportDataSourceInfoCollection reportdatasourcecollection = this.rvReportDisplay.ServerReport.GetDataSources();
				if(reportdatasourcecollection!=null)
				{ 
						if (reportdatasourcecollection.Count > 0)
						{
							dataSourceCredentials = new Microsoft.Reporting.WinForms.DataSourceCredentials[reportdatasourcecollection.Count];
						}
						for (int iIndex = 0; iIndex < reportdatasourcecollection.Count; iIndex++)
						{
							ReportDataSourceInfo datasourceinfo = reportdatasourcecollection[iIndex];
							//dataSourceCredential.Name = datasourceinfo.Name;
							dataSourceCredential.Name =datasourceinfo.Name;
							dataSourceCredentials[iIndex] = dataSourceCredential;
						}
				}
			 
				this.rvReportDisplay.ServerReport.SetDataSourceCredentials(dataSourceCredentials); 
				}
				catch(Exception ex)
				{
					//MessageBox.Show("Datasource has not been configured for"+_reportName);
				}

				//// Refresh the report
				rvReportDisplay.RefreshReport();
			} 
		

		}

		#endregion [Events]

      #region ILRRUIContainer Members

		public void LeftColMovement()
		{
			// TODO:  Add ManualEntryForHistPoints.LeftColMovement implementation
		}

		public void ClearSort()
		{

		}

		public void RightLevel()
		{
			// TODO:  Add ManualEntryForHistPoints.RightLevel implementation
		}

		public void UpLevel()
		{
			// TODO:  Add ManualEntryForHistPoints.UpLevel implementation
		}

		public bool SupportsImportToExcel()
		{
			return true;
		}

		public bool SupportsLevel()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsLevel implementation
			return false;
		}

		public bool PerformDirtyChk
		{
			get
			{
				// TODO:  Add ManualEntryForHistPoints.PerformDirtyChk getter implementation
				return false;
			}
		}

		public bool SupportsRefresh()
		{
			return true;
		}

		public void Undo()
		{
			// TODO:  Add ManualEntryForHistPoints.Undo implementation

		}

		public bool SupportsCopy()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsCopy implementation
			return false;
		}

		public void SelectionPaste()
		{

		}

		public TabPage ParentTabPage
		{
			set
			{
				_parentTabPage = value;
			}
			get
			{
				return _parentTabPage;
			}
		}

		public void PageSetup()
		{
			// TODO:  Add ManualEntryForHistPoints.PageSetup implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add ManualEntryForHistPoints.OpenQuery implementation
		}

		public void Copy()
		{
			// TODO:  Add ManualEntryForHistPoints.Copy implementation
		}

		public bool SupportsInsertNewRows()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsInsertNewRows implementation
			return false;
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsPageSetup implementation
			return false;
		}

		public void ShowPropertyList()
		{
			// TODO:  Add ManualEntryForHistPoints.ShowPropertyList implementation
		}

		public bool SupportsCompositeView()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsCompositeView implementation
			return false;
		}

		public bool SupportsClearSort()
		{
			return true;
		}

		public void RightColMovement()
		{
			// TODO:  Add ManualEntryForHistPoints.RightColMovement implementation
		}

		public void LeftLevel()
		{
			// TODO:  Add ManualEntryForHistPoints.LeftLevel implementation
		}

		public bool SupportsClearFilter()
		{
			return true;
		}

		public bool SupportsDelete()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsDelete implementation
			return false;
		}

		public void PrintPreview()
		{
			// TODO:  Add ManualEntryForHistPoints.PrintPreview implementation
		}

		public bool SupportsResultsFilter()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsResultsFilter implementation
			return false;
		}

		public bool SupportsPaste()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsPaste implementation
			return false;
		}

		public void Print()
		{
			// TODO:  Add ManualEntryForHistPoints.Print implementation
		}

		public bool SupportsArrangeColumns()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsArrangeColumns implementation
			return false;
		}

		public bool SupportsPrint()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsPrint implementation
			return false;
		}

		public void OpenView()
		{
			// TODO:  Add ManualEntryForHistPoints.OpenView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add ManualEntryForHistPoints.TransposeView implementation
		}

		public void ArrangeColumns()
		{
			// TODO:  Add ManualEntryForHistPoints.ArrangeColumns implementation
		}

		public void Export()
		{

		}

		public bool SupportsMasterPropertiesCopy()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsMasterPropertiesCopy implementation
			return false;
		}

		public void ShowComponentList()
		{
			// TODO:  Add ManualEntryForHistPoints.ShowComponentList implementation
		}

		public void ClearFilter()
		{
		}

		public void OpenCompositeView()
		{
			// TODO:  Add ManualEntryForHistPoints.OpenCompositeView implementation
		}

		public void Delete()
		{
			// TODO:  Add ManualEntryForHistPoints.Delete implementation
		}

		public bool SupportsUndo()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsUndo implementation
			return true;
		}

		public bool SupportsExport()
		{
			return true;
		}

		public void CopyUOMTypes()
		{
			// TODO:  Add ManualEntryForHistPoints.CopyUOMTypes implementation
		}

		public void ImportToExcel()
		{

		}

		public void Refresh()
		{

		}

		public bool SupportsSelectionPaste()
		{
			return true;
		}

		public void DownLevel()
		{
			// TODO:  Add ManualEntryForHistPoints.DownLevel implementation
		}

		public void Paste()
		{
			// TODO:  Add ManualEntryForHistPoints.Paste implementation
		}

		public bool SupportsSave()
		{
			return true;
		}

		public void InsertNewRows()
		{
			// TODO:  Add ManualEntryForHistPoints.InsertNewRows implementation
		}

		public void CopyMasterProperties()
		{
			// TODO:  Add ManualEntryForHistPoints.CopyMasterProperties implementation
		}

		public void Save()
		{
	
		}

		public TabControl ParentTabControl
		{
			set
			{
				_parentTabControl = value;
			}
			get{return _parentTabControl;}
		}

		public bool SupportsCopyUOMTypes()
		{
			// TODO:  Add ManualEntryForHistPoints.SupportsCopyUOMTypes implementation
			return false;
		}

		#endregion

		 

		

	}
}