using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;

using Syncfusion.Windows.Forms.Grid;

using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.IOScreen;
using Symcon.LightRidge.PEAdvisor.GridUtilities;

namespace Symcon.LightRidge.PEAdvisor.SavedViews
{
	/// <summary>
	/// Summary description for GridSavedViewController.
	/// </summary>
	public class GridSavedViewController
	{
		public GridSavedViewController()
		{
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
		}
		
		#region Methods
		public Panel LoadPanel(ArrayList arrXCollection,ArrayList arrYCollection)
		{
			//Split according to master plant items
			int rowCount = 0;
			LRRSiteComponents siteComponents = new LRRSiteComponents();
			object[,]  arrSplitComponents = SplitComponent(arrXCollection,arrYCollection,out rowCount);
			
			Panel pnlMain = new Panel();
			for(int i=0;i < rowCount;i++)
			{
				ArrayList arrXList = arrSplitComponents[i,0] as ArrayList;
				ArrayList arrYList = arrSplitComponents[i,1] as ArrayList;

				//Create Grid
				GridControl gridControlInstance = new GridControl();
				gridControlInstance.Dock = DockStyle.Fill;
				Panel pnlGrid = new Panel();
				pnlGrid.Dock = DockStyle.Top;
				pnlGrid.Controls.Add(gridControlInstance);
				
				//Add the Panel to panel Collection
				_pnlCollection.Add(pnlGrid);

				pnlMain.Controls.Add(pnlGrid);

				InitializeGrid(gridControlInstance);

				gridControlInstance.RowCount = arrYList.Count+ 2;
				gridControlInstance.ColCount = arrXList.Count * LRRModel.ProjectModes.Count + 2;

				//Row/Col Freeze
				gridControlInstance.Rows.FreezeRange(FREEZE_ROW_START,FREEZE_ROW_END);
				gridControlInstance.Cols.FreezeRange(FREEZE_COL_START,FREEZE_COL_END);

				GridControlManager gridControlManager = new GridControlManager();
				gridControlManager.RowList = arrYList;
				gridControlManager.ColList = arrXList;

				GridManagerHash.Add(gridControlInstance,gridControlManager);

				//register grid events
				registerGridEvents(gridControlInstance);

				gridControlInstance.ResetVolatileData();
				gridControlInstance.BeginUpdate();

				//Render x-axis
				CreateXAxis(gridControlInstance,gridControlManager,arrXList);

				//Render y-axis
				CreateYAxis(gridControlInstance,gridControlManager,arrYList);

				//Sort the Grid
//				gridControlInstance.RaiseCellClick(PROPERTIES_COL,COMPONENT_ROW,new MouseEventArgs(MouseButtons.Left,1,1,1,1));
				InitializePropertyValues(gridControlInstance,gridControlManager);

				gridControlInstance.ResumeLayout();
				gridControlInstance.EndUpdate(true);
			}

			pnlMain.Dock = DockStyle.Fill;
			return pnlMain;
		}

		private object[,] SplitComponent(ArrayList arrXCollection,ArrayList arrYCollection, out int rowCount)
		{
			Hashtable htMasterComps = new Hashtable();
			Hashtable htProps = new Hashtable();

			rowCount = 0;
			
			//Split X-Collection
			for(int i=0;i < arrXCollection.Count;i++)
			{
				SearchCriteria criteria = arrXCollection[i] as SearchCriteria;
				ArrayList arrCondition = criteria.ArrListSearchCondition;
				for(int j=0;j < arrCondition.Count;j++)
				{
					SearchCondition condition = arrCondition[j] as SearchCondition;
					LRRSiteComponent siteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(condition.Value);
					
					string sMasterCompType = siteComponent.MasterComponentType;
					if(!htMasterComps.ContainsKey(sMasterCompType))
					{
						ArrayList arrCriteria = new ArrayList();
						arrCriteria.Add(criteria);
						htMasterComps.Add(sMasterCompType,arrCriteria);
					}
					else
					{
						ArrayList arrCriteria = htMasterComps[sMasterCompType] as ArrayList;
						arrCriteria.Add(criteria);
					}
				}
			}

			//Split Y-Collection
			for(int i=0;i < arrYCollection.Count;i++)
			{
				SearchCriteria criteria = arrYCollection[i] as SearchCriteria;
				
				ArrayList arrCondition = criteria.ArrListSearchCondition;
				if(criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
				{
					SearchCondition condition = arrCondition[0] as SearchCondition;
					LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition.Value);
					string compID = null;
					if(masterProp != null)
					{
						compID = masterProp.ComponentID;
					}
					else
					{
						LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(condition.Value);
						LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProp.ComponentID);
						compID = siteComp.MasterComponentType;
					}

					if(!htProps.ContainsKey(compID))
					{
						ArrayList arrCriteria = new ArrayList();
						arrCriteria.Add(criteria);
						htProps.Add(compID,arrCriteria);
					}
					else
					{
						ArrayList arrCriteria = htProps[compID] as ArrayList;
						arrCriteria.Add(criteria);
					}
				}
				if(criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
				{
					string masterPropID = null;
					for(int j=0; j < arrCondition.Count;j++)
					{
						SearchCondition condition = arrCondition[j] as SearchCondition;
						if(condition.Name == "MasterPropID")
							masterPropID = condition.Value;
					}
					LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(masterPropID);
					string compID = masterProp.ComponentID;
					if(!htProps.ContainsKey(compID))
					{
						ArrayList arrCriteria = new ArrayList();
						arrCriteria.Add(criteria);
						htProps.Add(compID,arrCriteria);
					}
					else
					{
						ArrayList arrCriteria = htProps[compID] as ArrayList;
						arrCriteria.Add(criteria);
					}
				}															 
			}

			int nMasterComp = htMasterComps.Count;
			rowCount = nMasterComp;
			object[,] arrSplitComponent = new object[nMasterComp,2];
			
			int iCtr = 0;
			IDictionaryEnumerator iEnum = htMasterComps.GetEnumerator();
			while(iEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry) iEnum.Current;
				string compID = (string)dEntry.Key;
				ArrayList criteria = dEntry.Value as ArrayList;
				arrSplitComponent[iCtr,0] = criteria;

				ArrayList propCriteria = htProps[compID] as ArrayList;
				arrSplitComponent[iCtr,1] = propCriteria;

				iCtr++;
			}

			return arrSplitComponent;
		}

		public ArrayList GetPanelCollection()
		{
			return _pnlCollection;
		}

		/// <summary>
		/// Prints the selected grid
		/// </summary>
		public void Print()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					GridControlPrint ggPrint = new GridControlPrint();
					ggPrint.PrintData(gridControl,false);
					break;
				}
			}
		}

		/// <summary>
		/// Print Preview of the selected grid
		/// </summary>
		public void PrintPreview()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					GridControlPrint ggPrint = new GridControlPrint();
					ggPrint.PrintData(gridControl,true);
					break;
				}
			}
		}

		/// <summary>
		/// Exports the selected grid
		/// </summary>
		/// <param name="gridManagerHash"></param>
		/// <returns></returns>
		public GridControl Export()
		{
			IDictionaryEnumerator iDictEnum = GridManagerHash.GetEnumerator();
			while(iDictEnum.MoveNext())
			{
				DictionaryEntry dEntry = (DictionaryEntry)iDictEnum.Current;
				GridControl gridControl = (GridControl)dEntry.Key;
				if(gridControl.CurrentCell.IsActive)
				{
					return gridControl;
				}
			}
			return null;
		}

		/// <summary>
		/// This method registers all the grid events
		/// </summary>
		/// <param name="gridControlInstance"></param>
		private void registerGridEvents(GridControl gridControlInstance)
		{
			gridControlInstance.CurrentCellCloseDropDown +=new Syncfusion.Windows.Forms.PopupClosedEventHandler(gridControlInstance_CurrentCellCloseDropDown);
			gridControlInstance.CurrentCellValidating +=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellValidating);
			gridControlInstance.CurrentCellStartEditing+=new System.ComponentModel.CancelEventHandler(gridControlInstance_CurrentCellStartEditing);
			
			//add a sort header to the CellModels.
//			gridControlInstance.CellModels.Add("ColumnHeaderCell", new GridSortColumnHeaderCellModel(gridControlInstance.Model));

			//catch the click
//			gridControlInstance.CellClick +=new GridCellClickEventHandler(gridControlInstance_CellClick);

			//Event for row movement
//			gridControlInstance.RowsMoving+=new GridRangeMovingEventHandler(gridControlInstance_RowsMoving);
//			gridControlInstance.RowsMoved+=new GridRangeMovedEventHandler(gridControlInstance_RowsMoved);

			//Event for Col Movement
//			gridControlInstance.ColsMoving+=new GridRangeMovingEventHandler(gridControlInstance_ColsMoving);

			//Event for preventing hidden columns from expanding
//			gridControlInstance.ResizingColumns+=new GridResizingColumnsEventHandler(gridControlInstance_ResizingColumns);
			
			//Enabling/Disabling Left/Right Col Movement Buttons
//			gridControlInstance.SelectionChanged+=new GridSelectionChangedEventHandler(gridControlInstance_SelectionChanged);
			
			//Event for Handling Key down
			gridControlInstance.KeyDown+=new KeyEventHandler(gridControlInstance_KeyDown);

			//Context Menu for switching recording values
//			ContextMenu contMenu = new ContextMenu();
//			gridControlInstance.ContextMenu = contMenu;
//			contMenu.Popup+=new EventHandler(contMenu_Popup);
		}

		/// <summary>
		/// Initializes the grid
		/// </summary>
		/// <param name="gridControlInstance"></param>
		public static void InitializeGrid(GridControl gridControlInstance)
		{
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.BackColor = Color.DarkBlue;
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.TextColor =  SystemColors.ActiveCaptionText;
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.CellType = "Static";
			gridControlInstance.BaseStylesMap["Column Header"].StyleInfo.ReadOnly = true;

			gridControlInstance.BaseStylesMap["Row Header"].StyleInfo.BackColor = Color.FromArgb(192,192,192);

			gridControlInstance.Properties.RowHeaders = true;
			gridControlInstance.NumberedRowHeaders = false;
			gridControlInstance.AllowDragSelectedRows = true;
			gridControlInstance.BaseStylesMap["Row Header"].StyleInfo.Borders.Right = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.Solid, System.Drawing.SystemColors.WindowFrame);
			gridControlInstance.ColWidths.SetSize(0, 10);

			gridControlInstance.BaseStylesMap["Header"].StyleInfo.BackColor = Color.DarkBlue;
			gridControlInstance.ColStyles[1].CellType = "Static";
			gridControlInstance.RowStyles[1].CellType = "Static";
			gridControlInstance.RowStyles[2].CellType = "Static";
			gridControlInstance.ColStyles[1].ReadOnly = true;
			gridControlInstance.RowStyles[0].ReadOnly = true;
			gridControlInstance.RowStyles[1].ReadOnly = true;
			gridControlInstance.RowStyles[2].ReadOnly = true;;

			gridControlInstance.Model.Options.DefaultGridBorderStyle = GridBorderStyle.Solid;
			gridControlInstance.Model.Options.ExcelLikeSelectionFrame = false;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.WrapText = true;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.Trimming = StringTrimming.None;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.AutoSize = true;
			gridControlInstance.BaseStylesMap["Standard"].StyleInfo.HorizontalAlignment = GridHorizontalAlignment.Center;
			gridControlInstance.Properties.BackgroundColor = Color.FromArgb(150,150,150);
			gridControlInstance.Properties.GridLineColor = Color.FromArgb(0,0,0);
			gridControlInstance.DefaultGridBorderStyle = GridBorderStyle.Solid;
			gridControlInstance.Properties.Buttons3D = false;
		}
		
		private void CreateXAxis(GridControl gridControl,GridControlManager gridControlManager,ArrayList arrXCollection)
		{
			gridControl[ATTRIBUTE_ROW,ATTRIBUTE_COL].Text = "Attributes";
			gridControl[ATTRIBUTE_ROW,ATTRIBUTE_COL].BackColor = IOSchemes.ATTRIBUTES_BACK_COLOR;
			gridControl[ATTRIBUTE_ROW + 1,ATTRIBUTE_COL].BackColor = IOSchemes.ATTRIBUTES_BACK_COLOR;
			gridControl[UOM_ROW,UOM_COL].Text = "UOM";
			gridControl[UOM_ROW,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;
			gridControl[UOM_ROW + 1,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;

			int colIndex = COMPONENT_COL;
			int nModes = LRRModel.ProjectModes.Count;
			for(int i=0;i < arrXCollection.Count;i++)
			{
				SearchCriteria criteria = arrXCollection[i] as SearchCriteria;
				ArrayList arrCondition = criteria.ArrListSearchCondition;
				SearchCondition condition = arrCondition[0] as SearchCondition;
				
				gridControl.CoveredRanges.Add(GridRangeInfo.Cells(COMPONENT_ROW,colIndex,COMPONENT_ROW,colIndex+nModes-1));
				LRRSiteComponent siteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(condition.Value);
				gridControl.Model[COMPONENT_ROW,colIndex].Text	= siteComponent.Name;
				gridControl[COMPONENT_ROW,colIndex].BackColor = IOSchemes.COMPONENT_BACK_COLOR;

				LRRMasterComponent masterComp = LRRModel.Model.GetMasterComponentByID(siteComponent.MasterComponentType);
				gridControl.CoveredRanges.Add(GridRangeInfo.Cells(HEADER_ROW,HEADER_COL,HEADER_ROW,HEADER_COL + COL_SPAN - 1));
				gridControl[HEADER_ROW,HEADER_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
				gridControl[HEADER_ROW,HEADER_COL].Text = masterComp.ComponentName;
				
				int nModesCol = colIndex;
				foreach(LRRMode lrrMode in LRRModel.ProjectModes)
				{
					CellHeader colHeader = new CellHeader();
					colHeader.SearchCriteriaObj = criteria;
					colHeader.ModeID = lrrMode.ModeId;
					gridControlManager.ColHeader.Add(colHeader);

					gridControl[MODES_ROW,nModesCol].Text = lrrMode.ModeDisplayName;
					gridControl[MODES_ROW,nModesCol].CellTipText = lrrMode.ModeName;
					gridControl[MODES_ROW,nModesCol].BackColor = IOSchemes.GetModeBackColor(lrrMode.ModeId);
					gridControl[MODES_ROW,nModesCol].TextColor = IOSchemes.GetModeTextColor(lrrMode.ModeId);
					nModesCol ++;
				}

				colIndex = nModesCol;
			}
		}

		private void CreateYAxis(GridControl gridControl,GridControlManager gridControlManager,ArrayList arrYCollection)
		{
			int rowIndex = PROPERTIES_ROW;
			for(int i=0;i < arrYCollection.Count;i++)
			{
				SearchCriteria criteria = arrYCollection[i] as SearchCriteria;

				CellHeader rowHeader = new CellHeader();
				rowHeader.SearchCriteriaObj = criteria;
				gridControlManager.RowHeader.Add(rowHeader);

				string uomTypeID = null;

				ArrayList arrCondition = criteria.ArrListSearchCondition;
				if(criteria.ObjType == typeof(LRRSiteComponentPropertyValue))
				{
					SearchCondition condition = arrCondition[0] as SearchCondition;
					LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition.Value);
					if(masterProp != null)
					{
						gridControl.Model[rowIndex,PROPERTIES_COL].Text	= "[S] - " + masterProp.Name;
						gridControl.Model[rowIndex,PROPERTIES_COL].CellTipText	= "[S] - " + masterProp.Name;
						uomTypeID = masterProp.MasterPropertyUOMTypeID;
						rowHeader.UIUOMId = masterProp.UIUomID;
					}
					else
					{
						LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(condition.Value);
						gridControl.Model[rowIndex,PROPERTIES_COL].Text	= "[S] - " + siteCompProp.Name;
						gridControl.Model[rowIndex,PROPERTIES_COL].CellTipText	= "[S] - " + siteCompProp.Name;
						uomTypeID = siteCompProp.UOMTypeID;
						rowHeader.UIUOMId = siteCompProp.UIUomID;
					}
					gridControl.Model[rowIndex,PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
					
				}
				if(criteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
				{
					string lkUpPropertyID= null;
					string lookUpQualifierID = null;
					for(int j=0; j < arrCondition.Count;j++)
					{
						SearchCondition condition = arrCondition[j] as SearchCondition;
						if(condition.Name == "LookUpPropertiesID")
							lkUpPropertyID = condition.Value;
						if(condition.Name == "LookUpQualifierID")
							lookUpQualifierID = condition.Value;
					}
					LRRLookUpProperty lkupProp = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(lkUpPropertyID);

					if(lookUpQualifierID != null)
					{
						LRRLookUpQualifier lrrLkUpQual = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
						if(lrrLkUpQual != null)
						{
							LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);
							gridControl.Model[rowIndex,PROPERTIES_COL].Text	= "[GP] - " + qual.Name + "/" + lkupProp.Name;
							gridControl.Model[rowIndex,PROPERTIES_COL].CellTipText	= "[GP] - " + qual.Name + "/" + lkupProp.Name;
							gridControl.Model[rowIndex,PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
							uomTypeID = lrrLkUpQual.UOMTypeID;
						}
					}
					else
					{
						gridControl.Model[rowIndex,PROPERTIES_COL].Text	= "[DA] - " + lkupProp.Name;
						gridControl.Model[rowIndex,PROPERTIES_COL].CellTipText	= "[DA] - " + lkupProp.Name;
						gridControl.Model[rowIndex,PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
					}
				}
				
				LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomTypeID);
				string uomID= LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_lrrApp.AppLocaleID,uomTypeID);
				//Get UOM name from UOMID
				string uomName = null;
				if(uomID != null && uomID.Trim() != "")
				{
					uomName = LRRModel.ChildModel.GetUOMNameFromID(uomID);
				}

				gridControl.Model[rowIndex,UOM_COL].Text = uomName;
				gridControl.Model[rowIndex,UOM_COL].CellType = "ComboBox";
				gridControl.Model[rowIndex,UOM_COL].DataSource = uoms;
				uoms.SortColumn="Name";
				uoms.Sort();
				gridControl.Model[rowIndex,UOM_COL].DisplayMember = "Name";
				gridControl.Model[rowIndex,UOM_COL].ValueMember = "ID";
				gridControl.Model[rowIndex,UOM_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
				gridControl.Model[rowIndex,UOM_COL].BackColor = IOSchemes.UOM_BACK_COLOR;

				gridControl[rowIndex,PROPERTIES_COL].BackColor = IOSchemes.GRID_ROW_COLOR;
				gridControl[rowIndex,PROPERTIES_COL].HorizontalAlignment = GridHorizontalAlignment.Left;
				gridControl[rowIndex,PROPERTIES_COL].TextMargins.Left = 5;
				rowIndex ++;

			}
		}

		private void InitializePropertyValues(GridControl gridControl,GridControlManager gridControlManager)
		{
			//Clear the Bindings
			ClearBindings(gridControlManager);

			//Loop through the grid matrix,put property values,add bindings
			for(int iRow = DATA_ROW_START;iRow < gridControlManager.RowList.Count+ DATA_ROW_START; iRow++)
			{
				for(int iCol = DATA_COL_START; iCol < gridControlManager.ColList.Count * LRRModel.ProjectModes.Count + DATA_COL_START; iCol++)
				{
					CellHeader rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;
					CellHeader colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;

					object objPropValue = GetPropertyValueObject(rowData,colData);
					
					if(objPropValue != null)
					{
//						gridControl.Model[iRow,iCol].Text = siteCompPropValue.PropertyValue;
						IPropertyValue iPropValue = objPropValue as IPropertyValue;

						//Add Bindings
						gridControlManager.BindingManager.AddBinding(objPropValue,"PropertyValue",gridControl.Model[iRow,iCol],"Text");

						//Set Colour
						SetColor(iPropValue,gridControl.Model[iRow,iCol]);

						//Set the Cell tool Tip
//						SetCellToolTipText(iPropValue,gridControlInstance.Model[iRow,iCol],uiUOMID);

						gridControl.Model[iRow,iCol].HorizontalAlignment = GridHorizontalAlignment.Right;
						gridControl.Model[iRow,iCol].AutoSize = true;
					}
					else
					{
						gridControl.Model[iRow,iCol].BackColor = IOSchemes.GRID_ROW_COLOR;
						gridControl.Model[iRow,iCol].Borders.All = new GridBorder(GridBorderStyle.Solid,IOSchemes.GRID_PROPVALUE_LINE_COLOR,GridBorderWeight.ExtraThin);
					}
				}
			}
			//Resize to fit
			gridControl.ColWidths.ResizeToFit(GridRangeInfo.Cells(DATA_ROW_START,DATA_COL_START,gridControl.RowCount,gridControl.ColCount),GridResizeToFitOptions.NoShrinkSize);
		}
		
		private object GetPropertyValueObject(CellHeader rowData,CellHeader colData)
		{
			SearchCriteria rowSrchCriteria = rowData.SearchCriteriaObj;
			SearchCriteria colSrchCriteria = colData.SearchCriteriaObj;

			//Get the Site Component
			string modeID = colData.ModeID;
			ArrayList arrConditionList = colSrchCriteria.ArrListSearchCondition;
			SearchCondition condition = arrConditionList[0] as SearchCondition;
			LRRSiteComponent siteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(condition.Value);

			//Get the Property Values
			ArrayList arrRowConditionList = rowSrchCriteria.ArrListSearchCondition;
			if(rowSrchCriteria.ObjType == typeof(LRRSiteComponentPropertyValue))
			{
				SearchCondition condition2 = arrRowConditionList[0] as SearchCondition;
				LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(condition2.Value);
				LRRSiteComponentProperty siteCompProperty = null;
				if(masterProp != null)
				{
					siteCompProperty = siteComponent.GetPropertyFromMasterPropID(masterProp.PropID);
				}
				else
				{
					object objSiteCompProperty = LRRModel.GetPropertyObject(typeof(LRRSiteComponent),condition2.Value,siteComponent.SiteComponentID);
					if(objSiteCompProperty != null)
						siteCompProperty = objSiteCompProperty as LRRSiteComponentProperty; 
				}
				if(siteCompProperty != null)
				{
					ArrayList arrPropvalues = LRRModel.PropertyValuesObj.GetPropValuesByPropID(siteCompProperty.ID);
					if(arrPropvalues != null)
					{
						for(int i=0;i < arrPropvalues.Count;i++)
						{
							LRRSiteComponentPropertyValue siteCompPropValue  = arrPropvalues[i] as LRRSiteComponentPropertyValue;
							if(siteCompPropValue != null && siteCompPropValue.ModeID == modeID)
							{
								return siteCompPropValue;
							}
						}
					}
				}
			}
					
			if(rowSrchCriteria.ObjType == typeof(LRRSiteGroupPropertyQualifierValue))
			{
				string lkUpPropertyID= null;
				string lookUpQualifierID = null;
				string masterPropID = null;
				for(int j=0; j < arrRowConditionList.Count;j++)
				{
					SearchCondition condition1 = arrRowConditionList[j] as SearchCondition;
					if(condition1.Name == "LookUpPropertiesID")
						lkUpPropertyID = condition1.Value;
					if(condition1.Name == "LookUpQualifierID")
						lookUpQualifierID = condition1.Value;
					if(condition1.Name == "MasterPropID")
						masterPropID = condition1.Value;
				}
						
				LRRSiteComponentProperty siteCompProp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromMasterPropertyID(siteComponent.SiteComponentID,masterPropID);
				if(siteCompProp != null)
				{
					string siteGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteGroupPropID(siteCompProp.ID,lkUpPropertyID);
					LRRLookUpQualifier lkUpQualObj = null;
					if(lookUpQualifierID != null && lookUpQualifierID.Trim() != "")
						lkUpQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
					string qualID = lkUpQualObj == null ? null : lkUpQualObj.QualifierID;
					LRRSiteGroupPropertyQualifierValue qualValueObj = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObjectForPropertyMode(siteGrpProp,lkUpPropertyID,qualID,modeID);
					if(qualValueObj != null)
					{
						return qualValueObj;
					}
				}
			}

			return null;
		}

		/// <summary>
		/// This method clears the bindings
		/// </summary>
		/// <param name="gridControlManager"></param>
		private void ClearBindings(GridControlManager gridControlManager)
		{
			if(gridControlManager.BindingManager != null)
			{
				gridControlManager.BindingManager.RemoveBinding();
				
			}
		}

		/// <summary>
		/// Method sets the fore/back/border colours for the grid cell
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="gsInfo"></param>
		private void SetColor(IPropertyValue iPropValue,GridStyleInfo gsInfo)
		{
			gsInfo.BackColor = iPropValue.GetBackColor();
			gsInfo.TextColor = iPropValue.GetForeColor();

			//Set Border Color and Border Thickness
			Color borderColor = iPropValue.GetBorderColor();
			int borderThickness = iPropValue.GetBorderThickness();
			if(borderThickness == 2)
				gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid, borderColor,GridBorderWeight.Medium);
			else
			{
				gsInfo.Borders.All = new GridBorder(GridBorderStyle.Solid,IOSchemes.GRID_PROPVALUE_LINE_COLOR,GridBorderWeight.ExtraThin);
			}
			gsInfo.Font.Underline = iPropValue.GetUnderLine();
		}

		/// <summary>
		/// Changes property Values acording to UI UOM
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		private void changePropertyValueForUOM(GridControl gridControl,GridControlManager gridControlManager,int iRow,int iCol)
		{
			//Get the old UOM and new UOM
			string newUOM = gridControl.CurrentCell.Renderer.ControlValue == null ? "" : gridControl.CurrentCell.Renderer.ControlValue.ToString();
			
			for(int iColIndex =DATA_COL_START; iColIndex <= gridControl.ColCount;iColIndex++)
			{
				CellHeader colData = gridControlManager.ColHeader[iColIndex - DATA_COL_START] as CellHeader;
				CellHeader rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;

				object objPropValue = GetPropertyValueObject(rowData,colData);

				if(objPropValue != null)
				{
					IPropertyValue iPropValue = objPropValue as IPropertyValue;
					string uomID = iPropValue.GetUOMID();

					//Convert property value according to UIUOM 
					string propertyValue = iPropValue.GetPropertyValue();
					propertyValue = ConvertUOM(uomID,newUOM,propertyValue);

					gridControl.Model[iRow,iColIndex].Text = propertyValue;

					//Set Cell Tool Tip text
//					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iColIndex],newUOM);

					//Set the UI UOM to Sortmanager Row 
//					rowData.UIUOMId = newUOM;
//
//					//Add to UI UOM hash
//					if(gridControlManager.UIUOMHash.ContainsKey(rowData.ID))
//					{
//						gridControlManager.UIUOMHash[rowData.ID] = newUOM;
//					}
//					else
//					{
//						gridControlManager.UIUOMHash.Add(rowData.ID,newUOM);
//					}
				}
			}
		}

		/// <summary>
		/// Property value changes
		/// </summary>
		/// <param name="gridControl"></param>
		/// <param name="gridControlManager"></param>
		/// <param name="iRow"></param>
		/// <param name="iCol"></param>
		/// <returns></returns>
		private bool changePropertyValue(GridControl gridControl,GridControlManager gridControlManager,int iRow,int iCol)
		{
			string currentValue = gridControl.CurrentCell.Renderer.ControlText;

			//Get the row data
			CellHeader colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;
			CellHeader rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;

			string modeID = colData.ModeID;
			
			//Get the Property Value Object
			object objPropValue = GetPropertyValueObject(rowData,colData);
			if(objPropValue != null)
			{
				IPropertyValue iPropValue = objPropValue as IPropertyValue;
				string uomID = iPropValue.GetUOMID();

				string newUOM = rowData.UIUOMId;
				string newpropValue = ConvertUOM(newUOM,uomID,currentValue);
				
				if(iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
				{
					iPropValue.SetOverrideState(newpropValue);

					//Measurement Mode
					if(iPropValue.GetOverrideState() && modeID == "1")
					{
//						_lrrApp.SwapBindings(objPropValue,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}

					//Other Modes
					if(iPropValue.GetOverrideState() && modeID != "1")
					{
//						_lrrApp.RemoveCascadedBindings(objPropValue,objectType,rowCellID,colCellID,true);
						iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
					}
					
					iPropValue.SetPropertyValue(newpropValue);
					//Set the Cell tool Tip
//					SetCellToolTipText(iPropValue,gridControl.Model[iRow,iCol],newUOM);
					
					//Get the Property Object
//					object objProp = LRRModel.GetPropertyObject(objectType,rowCellID,colCellID);
//					if(objProp != null)
//					{
//						//Populate Overrides Collection	
//						IProperty iProp = objProp as IProperty;
//						string propID = iProp.GetID();
//						PopulateOverrides(iPropValue,propID,colCellID,modeID,uomID);
//					}
				}
			}
			return true;
		}
		
		/// <summary>
		/// Sets the Cell Tool Tip Text
		/// </summary>
		/// <param name="iPropValue"></param>
		/// <param name="gsInfo"></param>
		/// <param name="uiUOMID"></param>
//		private void SetCellToolTipText(IPropertyValue iPropValue,GridStyleInfo gsInfo,string uiUOMID)
//		{
//			string uomID = iPropValue.GetUOMID();
//			string actualPropValue = iPropValue.GetActualPropertyValue();
//			if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
//			{
//				actualPropValue = ConvertUOM(uomID,uiUOMID,actualPropValue);
//			}
//			if(iPropValue.GetOverrideState())
//			{
//				string preOverrideValue = iPropValue.GetActualOverridedValue();
//				if(uiUOMID != null && uiUOMID.Trim() != "" && uiUOMID != uomID)
//				{
//					preOverrideValue = ConvertUOM(uomID,uiUOMID,preOverrideValue);
//				}
//				gsInfo.CellTipText = "Pre-Override Value :" + preOverrideValue + "\n" +
//					"Post-Override Value :" + actualPropValue;
//			}
//			else
//			{
//				gsInfo.CellTipText = actualPropValue;
//			}
//		}

		/// <summary>
		/// Converts Property value from one UOM to another
		/// </summary>
		/// <param name="oldUOMValue"></param>
		/// <param name="newUOMValue"></param>
		/// <param name="propVal"></param>
		/// <returns></returns>
		private string ConvertUOM(string oldUOMValue,string newUOMValue,string propVal)
		{
			string newVal = "";
			if(propVal != null && propVal != "")
			{
				if(oldUOMValue == newUOMValue)
				{
					newVal = propVal;
				}
				else
				{
					if((oldUOMValue != null && oldUOMValue != "") &&
						(newUOMValue != null && newUOMValue != ""))
					{
						newVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(oldUOMValue,newUOMValue,propVal);
					}
				}
			}
			return newVal;
		}

		/// <summary>
		/// This method applies sorting
		/// </summary>
		/// <param name="col"></param>
		public void SortCol(int col,GridControl gridControl,GridControlManager gridControlManager)
		{
			if(gridControl.CurrentCell.IsEditing)
				gridControl.CurrentCell.EndEdit();
		
			GridStyleInfo style = new GridStyleInfo();
			style.Tag = "";

			//turn off the old sort arrow
			for(int col1 = 1; col1 <= gridControl.ColCount; ++col1)
				if(col1 != col && gridControl[1, col1].CellType == "ColumnHeaderCell")
				{
					gridControl[1, col1].CellType = "Header";
					break;
				}

			//turn on new sort arrow
			gridControl[1, col].CellType = "ColumnHeaderCell";

			System.ComponentModel.ListSortDirection dir = System.ComponentModel.ListSortDirection.Ascending;
			if(gridControl[1, col].HasTag)
			{
				if((System.ComponentModel.ListSortDirection) gridControl[1, col].Tag == System.ComponentModel.ListSortDirection.Ascending)
					dir = System.ComponentModel.ListSortDirection.Descending;
			}

			gridControl[1, col].Tag = dir;

			int headerCount = 2;
			string colName = null;
			if(col == 1)
				colName = "Name";
			if(col == 2)
				colName = "UOMName";
			int direction = 0;
			if(dir == System.ComponentModel.ListSortDirection.Ascending)
				direction = 0;
			if(dir == System.ComponentModel.ListSortDirection.Descending)
				direction = 1;
			
			Symcon.LightRidge.PEAdvisor.IOScreen.Comparer  ioComparer = IOScreenUtilities.IOComparer;
			gridControl.Data.SortByColumn(col,dir,headerCount);
			gridControl.Refresh();

			//Sort the Row List
			ArrayList sortManagerList = gridControlManager.RowHeader;
			ioComparer.SortColumn = colName;
			ioComparer.SortingOrder = direction;
			sortManagerList.Sort(ioComparer);

		}

		#endregion

		#region Constants
		private int COMPONENT_ROW = 1;
		private int COMPONENT_COL = 3;
		private int MODES_ROW = 2;
		private int PROPERTIES_ROW = 3;
		private int PROPERTIES_COL = 1;
		private int ATTRIBUTE_ROW = 1;
		private int ATTRIBUTE_COL = 1;
		private int HEADER_ROW = 0;
		private int HEADER_COL = 1;
		private int DATA_ROW_START = 3;
		private int DATA_COL_START = 3;
		private int COL_SPAN = 5;
		private const int FREEZE_ROW_START = 0;
		private const int FREEZE_ROW_END = 1;
		private const int FREEZE_COL_START = 0;
		private const int FREEZE_COL_END= 1;

		private int UOM_COL = 2;
		private int UOM_ROW = 1;
		#endregion 

		#region Fields
		private Hashtable _gridManagerHash = new Hashtable();
		private ArrayList _pnlCollection = new ArrayList();
		private LRRApp	  _lrrApp;
		#endregion

		#region Properties
		public Hashtable GridManagerHash
		{
			get{return _gridManagerHash;}
			set
			{
				_gridManagerHash = value;
			}
		}
		#endregion
		
		#region Event Handlers
		private void gridControlInstance_CurrentCellCloseDropDown(object sender, Syncfusion.Windows.Forms.PopupClosedEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			gridControl.CurrentCell.ConfirmChanges();
			gridControl.Refresh();
		}

		private void gridControlInstance_CurrentCellValidating(object sender, System.ComponentModel.CancelEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;
			
			//Get the Grid Data
			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
			if(gridControlManager == null)
				return;
			//Check for UOM
			if(iRow > UOM_ROW && iCol == UOM_COL)
			{
				//Change the Property Values according to UOM
				changePropertyValueForUOM(gridControl,gridControlManager,iRow,iCol);
			}
			//Check for Property Value 
			if(iRow >= DATA_ROW_START && iCol >= DATA_COL_START)
			{
				bool result = changePropertyValue(gridControl,gridControlManager,iRow,iCol);
				if(!result)
				{
					gridControl.CurrentCell.ResetCurrentCellWithoutDeactivate();
					gridControl.CurrentCell.Refresh();
					e.Cancel = true;
				}
			}
		}

		private void gridControlInstance_CurrentCellStartEditing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			GridControl gridControl = sender as GridControl;
			if(GridManagerHash[gridControl] == null)
			{
				e.Cancel = true;
				return;
			}
			int iRow = gridControl.CurrentCell.RowIndex;
			int iCol = gridControl.CurrentCell.ColIndex;

			if(iRow < DATA_ROW_START || iCol < DATA_COL_START)
				return;

//			GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
//			//Get the row data
//			CellHeader colData = gridControlManager.ColHeader[iCol - DATA_COL_START] as CellHeader;
//			CellHeader rowData = gridControlManager.RowHeader[iRow - DATA_ROW_START] as CellHeader;
//
//			//Get the Property Value Object
//			object objPropValue = GetPropertyValueObject(rowData,colData);
//			if(objPropValue != null)
//			{
//				IPropertyValue iPropValue = objPropValue as IPropertyValue;
//				if(!iPropValue.GetEditableState())
//					e.Cancel = true;
//			}
//			else
//			{
				e.Cancel = true;
//			}
				
		}

		private void gridControlInstance_CellClick(object sender, GridCellClickEventArgs e)
		{
			if(e.MouseEventArgs.Button == MouseButtons.Left
				&& e.RowIndex == 1 && (e.ColIndex == 1 || e.ColIndex == 2))
			{
				GridControl gridControl = sender as GridControl;
				gridControl.ResetVolatileData();
				gridControl.BeginUpdate();

				Cursor.Current = Cursors.WaitCursor;
				Cursor.Show();
				if(GridManagerHash[gridControl] != null)
				{
					GridControlManager gridControlManager = GridManagerHash[gridControl] as GridControlManager;
					SortCol(e.ColIndex,gridControl,gridControlManager);
					InitializePropertyValues(gridControl,gridControlManager);
				}
				gridControl.EndUpdate();
				Cursor.Hide();
			}
		}

		private void gridControlInstance_KeyDown(object sender, KeyEventArgs e)
		{
			if(e.Shift || e.Control || e.Alt)
			{
				e.Handled = true;
			}
		}

		#endregion
	}
}
