using System;
using System.Collections;

using Symcon.LightRidge.BusinessObjects;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageLogger;

namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for SearchRetrievalUtility.
	/// </summary>
	public class SearchRetrievalUtility
	{
		public SearchRetrievalUtility()
		{
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			string sRoleType = _lrrApp.DBManagerObj.AccessControlModel.UserRoleTypeID;
			if(sRoleType == "1")
			{
				bImplementer = true;
			}
			InitDataSource();
		}

		#region Methods
		/// <summary>
		/// Populate the siteComponents for given search id
		/// </summary>
		/// <param name="searchCriteriaID"></param>
		/// <returns></returns>
		public LRRSiteComponents PopulateSiteComponents(string searchCriteriaID)
		{
			LRRSearchCriteriaItems  lrrSearchCriteriaItems					= _lrrSearchCriteriaItems.FilterItemsBySearchCriteriaID(searchCriteriaID);
			LRRSearchCriteriaItemProperties lrrSearchCriteriaItemProperties	= _lrrSearchCriteriaItemProperties.FilterItemsBySearchCriteriaID(searchCriteriaID);

			LRRSearchCriteriaAggregateItems aggrItems = _lrrSearchCriteriaAggregateItems.FilterAggregateItemsBySearchCriteriaID(searchCriteriaID);
			LRRModes lrrModes = LRRModel.ProjectModes;

			LRRSiteComponents lrrSiteComponents = new LRRSiteComponents();
			LRRSiteComponentProperties _lrrSiteComponentProperties = LRRModel.SiteComponentPropertiesObj;

			foreach(LRRSearchCriteriaItem lrrSearchCriteriaItem in lrrSearchCriteriaItems)
			{
				LRRSiteComponent lrrSiteComponent = _lrrSiteComponents.GetSiteComponentByID(lrrSearchCriteriaItem.ItemID);
				if(lrrSiteComponent == null)
				{
					//Get from the Model and check for dynamic
					lrrSiteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(lrrSearchCriteriaItem.ItemID);
					if(lrrSiteComponent == null || !lrrSiteComponent.IsDynamic || this._lrrApp.DynamicCaseID == null)
						continue;
				}
				lrrSiteComponents.Add(lrrSiteComponent);

				LRRSiteComponentProperties lrrSiteCompProps = new LRRSiteComponentProperties();
				foreach(LRRSiteComponentProperty lrrSiteCompProperty in _lrrSiteComponentProperties)
				{
					if(lrrSiteComponent.SiteComponentID != lrrSiteCompProperty.ComponentID)
						continue;
					foreach(LRRSearchCriteriaItemProperty lrrSearchCriteriaItemProperty in lrrSearchCriteriaItemProperties)
					{
						if(lrrSiteCompProperty.PropertyID == lrrSearchCriteriaItemProperty.ItemPropertyID)
						{
							if(!bImplementer)
							{
								LRRMasterProperty lmp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(lrrSiteCompProperty.PropertyID);
								if(lmp != null && lmp.IsExposed == true)
								{
									//Handling of UIUOMs
									lrrSiteCompProperty.UIUomID= lrrSearchCriteriaItemProperty.UIUomID;
									/*if(lrrSiteCompProperty.UIUomID == null || lrrSiteCompProperty.UIUomID.Trim() == "")
									{
										lrrSiteCompProperty.UIUomID = lrrSiteCompProperty.UOM;
									}*/
							
									if(!_uiUOMHash.ContainsKey(lrrSiteCompProperty.PropertyID))
										_uiUOMHash.Add(lrrSiteCompProperty.PropertyID,lrrSiteCompProperty.UIUomID);

									lrrSiteCompProps.Add(lrrSiteCompProperty);
								}
							}
							else
							{
								//Handling of UIUOMs
								lrrSiteCompProperty.UIUomID= lrrSearchCriteriaItemProperty.UIUomID;
								/*if(lrrSiteCompProperty.UIUomID == null || lrrSiteCompProperty.UIUomID.Trim() == "")
									{
										lrrSiteCompProperty.UIUomID = lrrSiteCompProperty.UOM;
									}*/
							
								if(!_uiUOMHash.ContainsKey(lrrSiteCompProperty.PropertyID))
									_uiUOMHash.Add(lrrSiteCompProperty.PropertyID,lrrSiteCompProperty.UIUomID);

								lrrSiteCompProps.Add(lrrSiteCompProperty);
							}
						}
						if(lrrSiteCompProperty.ID == lrrSearchCriteriaItemProperty.ItemPropertyID)
						{
							//Handling of UIUOMs
							lrrSiteCompProperty.UIUomID= lrrSearchCriteriaItemProperty.UIUomID;
							/*if(lrrSiteCompProperty.UIUomID == null || lrrSiteCompProperty.UIUomID.Trim() == "")
							{
								lrrSiteCompProperty.UIUomID = lrrSiteCompProperty.UOM;
							}*/
							
							if(!_uiUOMHash.ContainsKey(lrrSiteCompProperty.ID))
								_uiUOMHash.Add(lrrSiteCompProperty.ID,lrrSiteCompProperty.UIUomID);
							lrrSiteCompProps.Add(lrrSiteCompProperty);
						}
					}
				}
				lrrSiteComponent.Properties = lrrSiteCompProps;
			}
			
			//Sort By Name
			lrrSiteComponents.SortColumn = "Name";
			lrrSiteComponents.Sort();
			

			return lrrSiteComponents;
		}
		
		/// <summary>
		/// Populate Aggregates for given SearchID
		/// </summary>
		/// <param name="searchID"></param>
		/// <returns></returns>
		public LRRAggregates PopulateAggregates(string searchID)
		{
			LRRSearchCriteriaAggregateItems aggrItems = _lrrSearchCriteriaAggregateItems.FilterAggregateItemsBySearchCriteriaID(searchID);

			LRRAggregates lrrResultAggregates = new LRRAggregates();
			foreach(LRRSearchCriteriaAggregateItem aggrItem in aggrItems)
			{
				LRRAggregate lrrAggregate = _lrrAggregates.GetAggregateFromID(aggrItem.AggregateItemID);
				if(lrrAggregate != null)
				{
					lrrResultAggregates.Add(lrrAggregate);
					LRRAggregateProperties lrrAggrProps = LRRModel.AggregatePropertiesObj.FilterPropertiesByAggregate(lrrAggregate.AggregateID);
					LRRAggregateProperties lrrResultAggrProps = new LRRAggregateProperties();
					foreach(LRRAggregateProperty lrrAggrProperty in lrrAggrProps)
					{
						foreach(LRRSearchCriteriaItemProperty lrrSearchCriteriaItemProperty in _lrrSearchCriteriaItemProperties)
						{
							
							if(lrrAggrProperty.MasterPropertyID == lrrSearchCriteriaItemProperty.ItemPropertyID)
							{
								if(!bImplementer)
								{
									LRRMasterProperty lmp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(lrrAggrProperty.MasterPropertyID );
									if(lmp != null && lmp.IsExposed == true)
									{
										//Handling of UIUOMs
										lrrAggrProperty.UIUomID= lrrSearchCriteriaItemProperty.UIUomID;
										/*if(lrrAggrProperty.UIUomID == null || lrrAggrProperty.UIUomID.Trim() == "")
										{
											lrrAggrProperty.UIUomID = lrrAggrProperty.UOM;
										}*/
								
										if(!_uiUOMHash.ContainsKey(lrrAggrProperty.MasterPropertyID))
											_uiUOMHash.Add(lrrAggrProperty.MasterPropertyID,lrrAggrProperty.UIUomID);

										lrrResultAggrProps.Add(lrrAggrProperty);
									}
								}
								else
								{
									//Handling of UIUOMs
									lrrAggrProperty.UIUomID= lrrSearchCriteriaItemProperty.UIUomID;
									/*if(lrrAggrProperty.UIUomID == null || lrrAggrProperty.UIUomID.Trim() == "")
									{
										lrrAggrProperty.UIUomID = lrrAggrProperty.UOM;
									}*/
								
									if(!_uiUOMHash.ContainsKey(lrrAggrProperty.MasterPropertyID))
										_uiUOMHash.Add(lrrAggrProperty.MasterPropertyID,lrrAggrProperty.UIUomID);

									lrrResultAggrProps.Add(lrrAggrProperty);
								}
							}
						}
					}
					lrrAggregate.AggregateProperties = lrrResultAggrProps;
				}
			}
			return lrrResultAggregates;
		}
		
		/// <summary>
		/// Populates SiteComponentProperties and Aggregate Propertiefor given siteComponents And Aggregates
		/// </summary>
		/// <param name="siteComponents"></param>
		/// <param name="lrrResultAggregates"></param>
		/// <returns></returns>
		public static LRRResultProperties ShowPropertiesWindow(LRRSiteComponents siteComponents,LRRAggregates lrrResultAggregates)
		{
			LRRResultProperties resultProperties = new LRRResultProperties();
			Hashtable htItemsToAdd = new Hashtable();
			
			//Populate SiteComponent Properties
            if (siteComponents != null)
            {
                foreach (LRRSiteComponent siteComponent in siteComponents)
                {
                    LRRSiteComponentProperties siteCompProps = siteComponent.Properties;
                    string masterCompName = LRRModel.Model.GetMasterComponentByID(siteComponent.MasterComponentType).ComponentName;
                    foreach (LRRSiteComponentProperty siteCompProp in siteCompProps)
                    {
                        if (siteCompProp.PropertyID == null || siteCompProp.PropertyID.Trim() == "")
                        {
                            if (!htItemsToAdd.ContainsKey(siteCompProp.ID))
                                htItemsToAdd.Add(siteCompProp.ID, siteCompProp.Name + "(" + masterCompName + ")");
                        }
                        else
                        {
                            if (bImplementer)
                            {
                                if (!htItemsToAdd.ContainsKey(siteCompProp.PropertyID))
                                {
                                    htItemsToAdd.Add(siteCompProp.PropertyID, siteCompProp.Name + "(" + masterCompName + ")");
                                }
                            }
                            else
                            {
                                LRRMasterProperty lmp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(siteCompProp.PropertyID);
                                if (lmp.IsExposed == true)
                                {
                                    if (!htItemsToAdd.ContainsKey(siteCompProp.PropertyID))
                                    {
                                        htItemsToAdd.Add(siteCompProp.PropertyID, siteCompProp.Name + "(" + masterCompName + ")");
                                    }
                                }
                            }
                        }
                    }
                }
            }

			//Populate Aggregate Properties
            if (lrrResultAggregates != null)
            {
                foreach (LRRAggregate aggregate in lrrResultAggregates)
                {
                    LRRAggregateProperties aggrProps = aggregate.AggregateProperties;
                    string masterCompName = LRRModel.Model.GetMasterComponentByID(aggregate.EquipmentTypeID).ComponentName;
                    foreach (LRRAggregateProperty aggrProp in aggrProps)
                    {
                        if (!htItemsToAdd.ContainsKey(aggrProp.MasterPropertyID))
                        {
                            if (bImplementer)
                            {
                                htItemsToAdd.Add(aggrProp.MasterPropertyID, aggrProp.Name + "(" + masterCompName + ")");
                            }
                            else
                            {
                                LRRMasterProperty lmp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(aggrProp.MasterPropertyID);
                                if (lmp.IsExposed == true)
                                {
                                    htItemsToAdd.Add(aggrProp.MasterPropertyID, aggrProp.Name + "(" + masterCompName + ")");
                                }
                            }
                        }
                    }
                }
            }
			
			
			//Populate Result Properties
			foreach(DictionaryEntry entry in htItemsToAdd)
			{
				LRRResultProperty lrrResultProp = new LRRResultProperty();
				lrrResultProp.ID = entry.Key.ToString();
				lrrResultProp.Name = entry.Value.ToString();
				resultProperties.Add(lrrResultProp);
			}

			return resultProperties;
		}

        /// <summary>
        /// Shows the plant properties W indow.
        /// </summary>
        /// <param name="baseObj">The base obj.</param>
        /// <param name="siteComponents">The site components.</param>
        /// <returns></returns>
        public static LRRResultProperties ShowPlantPropertiesWIndow(object baseObj, LRRSiteComponents siteComponents)
        {
            LRRResultProperties resultProperties = new LRRResultProperties();
            Hashtable htItemsToAdd = new Hashtable();
            LRRLookUpQualifiers objLookUpQualifiers = baseObj as LRRLookUpQualifiers;
            IEnumerator iSiteComponet = siteComponents.GetEnumerator();
            while(iSiteComponet.MoveNext())
            {
                LRRSiteComponent siteComponent = iSiteComponet.Current as LRRSiteComponent;
                if (objLookUpQualifiers != null)
                {
                    IEnumerator iLookUpQualifiersEnum = objLookUpQualifiers.GetEnumerator();
                    while (iLookUpQualifiersEnum.MoveNext())
                    {
                        LRRLookUpQualifier lookUpQualifier = iLookUpQualifiersEnum.Current as LRRLookUpQualifier;
                        string propertyType = lookUpQualifier.PropertyID;
                        LRRPropertyType lrrPropertyType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(propertyType);
                        //grpPropertyType = lrrPropertyType.PropertyType;

                        CustomCollection propsCollection = LRRModel.SiteComponentPropertiesObj.GetSiteComponentPropertiesByComponentID(siteComponent.SiteComponentID);
                        //Loop through Properties Collection
                        IEnumerator iPropEnum = propsCollection.GetEnumerator();
                        while (iPropEnum.MoveNext())
                        {
                            IProperty iProp = iPropEnum.Current as IProperty;
                            if (iProp.GetPropertyType() != propertyType)
                                continue;

                            //Loop through Group Properties Collection
                            CustomCollection grpPropsCollection = iProp.GetGroupProperties();
                            IEnumerator iGroupPropEnum = grpPropsCollection.GetEnumerator();
                            while (iGroupPropEnum.MoveNext())
                            {
                                IProperty iGroupProp = iGroupPropEnum.Current as IProperty;
                                string ID = iGroupProp.GetID();
                                string name = iGroupProp.GetName();
                                string materialID = LRRModel.ChildModel.LookUpProperties.getLookUpPropertyForLkupPropID(ID).MaterialID.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                ID = materialID;
                                if (ID == null)
                                    ID = name;
                                if (!htItemsToAdd.ContainsKey(ID + "|" + siteComponent.SiteComponentID))
                                {
                                    //Added QualifierID to the ID to get a uniqueID for same LookUpProperty
                                    htItemsToAdd.Add(ID + "|" + siteComponent.SiteComponentID, name + " (" + siteComponent.Name + ")");
                                }
                            }
                        }
                    }
                }
            }
            
            //Populate Result Properties
            foreach (DictionaryEntry entry in htItemsToAdd)
            {
                LRRResultProperty lrrResultProp = new LRRResultProperty();
                lrrResultProp.ID = entry.Key.ToString();
                lrrResultProp.Name = entry.Value.ToString();
                resultProperties.Add(lrrResultProp);
            }
            return resultProperties;
        }

        /// <summary>
        /// Shows the grouped properties window.
        /// </summary>
        /// <param name="baseObj">The base obj.</param>
        /// <param name="type">The type.</param>
        /// <returns></returns>
        public static LRRResultProperties ShowGroupedPropertiesWindow(LRRSiteComponents compCollection, object baseObj, Type type)
        {
            LRRResultProperties resultProperties = new LRRResultProperties();
            Hashtable htItemsToAdd = new Hashtable();
            #region GroupedPlantProPertiesView

            if (type == typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantPropertiesView) || type == typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GridQualifierViewController) || type == typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GridQualifierTransposedViewController))
            {

                LRRLookUpQualifiers objLookUpQualifiers = baseObj as LRRLookUpQualifiers;
                if (objLookUpQualifiers != null)
                {
                    IEnumerator iLookUpQualifiersEnum = objLookUpQualifiers.GetEnumerator();
                    while (iLookUpQualifiersEnum.MoveNext())
                    {
                        LRRLookUpQualifier lookUpQualifier = iLookUpQualifiersEnum.Current as LRRLookUpQualifier;
                        LRRQualifier qualifier = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lookUpQualifier.QualifierID);
                        string qualifierName = qualifier.Name;
                        string propertyType = lookUpQualifier.PropertyID;
                        //Loop through the Collection and render the grid
                        if (compCollection != null)
                        {
                            IEnumerator iCompEnum = compCollection.GetEnumerator();
                            while (iCompEnum.MoveNext())
                            {
                                IComponent iComp = iCompEnum.Current as IComponent;
                                CustomCollection propsCollection = iComp.GetProperties();

                                //Loop through Properties Collection
                                IEnumerator iPropEnum = propsCollection.GetEnumerator();
                                while (iPropEnum.MoveNext())
                                {
                                    IProperty iProp = iPropEnum.Current as IProperty;

                                    if (iProp.GetPropertyType() != propertyType)
                                        continue;

                                    CustomCollection grpPropsCollection = iProp.GetGroupProperties();
                                    IEnumerator iGroupPropEnum = grpPropsCollection.GetEnumerator();
                                    while (iGroupPropEnum.MoveNext())
                                    {
                                        try
                                        {
                                            IProperty iGroupProp = iGroupPropEnum.Current as IProperty;
                                            string ID = iGroupProp.GetID();
                                            string name = iGroupProp.GetName();
                                            if (ID == null)
                                                ID = name;
                                            if (!htItemsToAdd.ContainsKey(ID + "|" + qualifier.QualifierID))
                                            {
                                                //Added QualifierID to the ID to get a uniqueID for same groupedProperty
                                                htItemsToAdd.Add(ID + "|" + qualifier.QualifierID, name + " (" + qualifierName + ")");
                                            }
                                        }
                                        catch (Exception ex)
                                        {
                                            Logger.LogMessage(typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GridQualifierViewController), "Exception in Group Properties rendering:- " + ex.Message, Logger.Level.Debug);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            #endregion

            #region GroupedPlantDataArrayVIew

            else if (type == typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GroupedPlantDataArrayView) || type == typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GridDataArrayViewController) || type == typeof(Symcon.LightRidge.PEAdvisor.GroupProperties.GridDataArrayTransposeViewController))
            {
                LRRPropertyTypes objPropertyTypes = baseObj as LRRPropertyTypes;
                if (objPropertyTypes != null)
                {
                    IEnumerator ipropertyEnum = objPropertyTypes.GetEnumerator();
                    while (ipropertyEnum.MoveNext())
                    {
                        string ID = string.Empty;
                        string name = string.Empty;
                        string propertyType = string.Empty;
                        string grpPropertyType = string.Empty;

                        LRRPropertyType lrrPropertyType = ipropertyEnum.Current as LRRPropertyType;
                        propertyType = lrrPropertyType.ID;
                        grpPropertyType = lrrPropertyType.PropertyType;

                        IEnumerator iCompEnum = compCollection.GetEnumerator();
                        while (iCompEnum.MoveNext())
                        {
                            
                            IComponent iComp = iCompEnum.Current as IComponent;
                            CustomCollection propsCollection = iComp.GetProperties();

                            //Loop through Properties Collection
                            IEnumerator iPropEnum = propsCollection.GetEnumerator();
                            while (iPropEnum.MoveNext())
                            {
                                IProperty iProp = iPropEnum.Current as IProperty;

                                if (iProp.GetPropertyType() != propertyType)
                                    continue;

                                //Loop through Group Properties Collection
                                CustomCollection grpPropsCollection = iProp.GetGroupProperties();
                                IEnumerator iGroupPropEnum = grpPropsCollection.GetEnumerator();
                                while (iGroupPropEnum.MoveNext())
                                {
                                    IProperty iGroupProp = iGroupPropEnum.Current as IProperty;
                                    ID = iGroupProp.GetID();
                                    name = iGroupProp.GetName();
                                    if (ID == null)
                                        ID = name;

                                    if (!htItemsToAdd.ContainsKey(ID + "|" + lrrPropertyType.ID))
                                    {
                                        //attrName = attrName.Replace("\r\n", "/");
                                        htItemsToAdd.Add(ID + "|" + lrrPropertyType.ID, name + " (" + grpPropertyType + ")");
                                    }
                                }

                            }
                        }
                    }
                }
            }
            #endregion

            //Populate Result Properties
            foreach (DictionaryEntry entry in htItemsToAdd)
            {
                LRRResultProperty lrrResultProp = new LRRResultProperty();
                lrrResultProp.ID = entry.Key.ToString();
                lrrResultProp.Name = entry.Value.ToString();
                resultProperties.Add(lrrResultProp);
            }
            return resultProperties;
        }

		/// <summary>
		/// Populates distinct Components for given SiteComponents And Aggregates
		/// </summary>
		/// <param name="siteComponents"></param>
		/// <param name="lrrResultAggregates"></param>
		/// <returns></returns>
		public static LRRResultProperties ShowComponents(LRRSiteComponents siteComponents,LRRAggregates lrrResultAggregates)
		{
			LRRResultProperties resultProperties = new LRRResultProperties();
			Hashtable htItemsToAdd = new Hashtable();
			
			//Populate SiteComponents
            if(siteComponents != null)
            {
			    foreach(LRRSiteComponent siteComponent in siteComponents)
			    {
				    string masterCompName = LRRModel.Model.GetMasterComponentByID(siteComponent.MasterComponentType).ComponentName;
				    if(!htItemsToAdd.ContainsKey(siteComponent.SiteComponentID))
				    {
					    htItemsToAdd.Add(siteComponent.SiteComponentID,siteComponent.Name + "(" + masterCompName + ")");
				    }
			    }
            }

			//Populate Aggregates
            if (lrrResultAggregates != null)
            {
                foreach (LRRAggregate aggregate in lrrResultAggregates)
                {
                    string masterCompName = LRRModel.Model.GetMasterComponentByID(aggregate.EquipmentTypeID).ComponentName;
                    if (!htItemsToAdd.ContainsKey(aggregate.AggregateID))
                    {
                        htItemsToAdd.Add(aggregate.AggregateID, aggregate.Name + "(" + masterCompName + ")");
                    }
                }
            }
			
			
			//Populate Result Properties
			foreach(DictionaryEntry entry in htItemsToAdd)
			{
				LRRResultProperty lrrResultProp = new LRRResultProperty();
				lrrResultProp.ID = entry.Key.ToString();
				lrrResultProp.Name = entry.Value.ToString();
				resultProperties.Add(lrrResultProp);
			}

			return resultProperties;
		}
		

        /// <summary>
        /// Shows the qualifiers.
        /// </summary>
        /// <param name="objLookUpQualifiers">The obj look up qualifiers.</param>
        /// <returns></returns>
        public static LRRResultProperties ShowQualifiers(LRRLookUpQualifiers objLookUpQualifiers, LRRSiteComponents objSiteComponents)
	    {
            string ID = string.Empty;
            string name = string.Empty;
            string propID = string.Empty;
            string propName = string.Empty;

            LRRResultProperties resultProperties = new LRRResultProperties();
            Hashtable htItemsToAdd = new Hashtable();
            IEnumerator iSiteComponet = objSiteComponents.GetEnumerator();
            while (iSiteComponet.MoveNext())
            {
                LRRSiteComponent siteComponent = iSiteComponet.Current as LRRSiteComponent;
                if (objLookUpQualifiers != null)
                {
                    IEnumerator iLookUpQualifiersEnum = objLookUpQualifiers.GetEnumerator();
                    while (iLookUpQualifiersEnum.MoveNext())
                    {
                        LRRLookUpQualifier lookUpQualifier = iLookUpQualifiersEnum.Current as LRRLookUpQualifier;
                        string propertyType = lookUpQualifier.PropertyID;
                        LRRPropertyType lrrPropertyType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(propertyType);

                        CustomCollection propsCollection = siteComponent.GetProperties();
                        IEnumerator iPropEnum = propsCollection.GetEnumerator();
                        //Loop through Properties Collection
                        while (iPropEnum.MoveNext())
                        {
                            IProperty iProp = iPropEnum.Current as IProperty;
                            if (iProp.GetPropertyType() != propertyType)
                                continue;
                            propID = iProp.GetID();
                            propName = iProp.GetName();
                            ID = lookUpQualifier.QualifierID;
                            LRRQualifier qualifier = LRRModel.Qualifiers.getQualifierObjectForQualifierID(ID);

                            if (!htItemsToAdd.ContainsKey(ID + "|" + siteComponent.SiteComponentID))
                            {
                                htItemsToAdd.Add(ID + "|" + siteComponent.SiteComponentID, qualifier.Name + " (" + siteComponent.Name + ")");
                            }
                        }
                    }
                }
            }
            //Populate Result Properties
            foreach (DictionaryEntry entry in htItemsToAdd)
            {
                LRRResultProperty lrrResultProp = new LRRResultProperty();
                lrrResultProp.ID = entry.Key.ToString();
                lrrResultProp.Name = entry.Value.ToString();
                resultProperties.Add(lrrResultProp);
            }
            return resultProperties;


        }
		public void InitDataSource()
		{
			if(_lrrSearchCriteriaItems != null)
				_lrrSearchCriteriaItems.Clear();
			if(_lrrSearchCriteriaItemProperties != null)
				_lrrSearchCriteriaItemProperties.Clear();
			if(_lrrSiteComponents != null)
				_lrrSiteComponents.Clear();

			//Aggregates
			if(_lrrAggregates != null)
				_lrrAggregates.Clear();

			_lrrSearchCriteriaItems = new LRRSearchCriteriaItems();
			_lrrSearchCriteriaItemProperties = new LRRSearchCriteriaItemProperties();
			_lrrSearchCriteriaAggregateItems = new LRRSearchCriteriaAggregateItems();

			_lrrApp.DBManagerObj.PopulateCollection(_lrrSearchCriteriaItems,false);
			_lrrApp.DBManagerObj.PopulateCollection(_lrrSearchCriteriaItemProperties,false);
			_lrrApp.DBManagerObj.PopulateCollection(_lrrSearchCriteriaAggregateItems,false);

			_lrrSiteComponents = new LRRSiteComponents();
			_lrrApp.DBManagerObj.PopulateCollection(_lrrSiteComponents,false);
			
			_lrrAggregates = new LRRAggregates();
			_lrrApp.DBManagerObj.PopulateCollection(_lrrAggregates,false);
		}

		#endregion
		
		#region Properties
		public Hashtable UIUOMHash
		{
			get{return _uiUOMHash;}
			set
			{
				_uiUOMHash = value;
			}
		}
		#endregion


		#region Fields
		private LRRApp _lrrApp = null;
		private LRRSearchCriteriaItems _lrrSearchCriteriaItems = null;
		private LRRSearchCriteriaItemProperties _lrrSearchCriteriaItemProperties = null;
		private  static bool bImplementer = false;

		//Aggregates
		private LRRSearchCriteriaAggregateItems	_lrrSearchCriteriaAggregateItems = null;
		private LRRSiteComponents _lrrSiteComponents = null;

		//Aggregates
		private LRRAggregates	_lrrAggregates = null;

		//UIUOMHash
		private Hashtable		_uiUOMHash = new Hashtable();

        //GroupedProperties
        private static LRRGroupProperties _lrrGroupProperties = null;


		#endregion
	}
}
