using System;
using System.Reflection;
using System.Collections;
using System.Data;
using System.Diagnostics;

using Symcon.Globals.Enums;
using Symcon.AccessControl;
using Symcon.AccessControl.Model;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor.UIAccessController
{
	/// <summary>
	/// Summary description for UIACLController
	/// </summary>
	public class UIACLController
	{
		private static LRRApp	_app;
		

		static UIACLController()
		{
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;//Get the app object...
		}

		#region Access Control Utilities

		/// <summary>
		/// The member method checks if the user has permissions to access the UI Object
		/// </summary>
		public static bool IsUIEntityAccessible(string UIEntityID)
		{
            if (UIEntityID == "Mapping")
            {
                int xyz = 0;
            }
			bool bUIEntityAccessible = true;
			AccessControlUIEntities objACLUI = new AccessControlUIEntities();
			if( _app.DBManagerObj!=null)
			{
				objACLUI =(AccessControlUIEntities) _app.DBManagerObj.AccessControlManager.GetUIAccessControlModel();
				if(objACLUI.Count >0)
				{
					IEnumerator iUIEntityEnum = objACLUI.GetEnumerator();
					bool assigned = false;
					while (iUIEntityEnum.MoveNext())
					{
						AccessControlUIEntity objUIEntity= (AccessControlUIEntity)iUIEntityEnum.Current;
						UIEntity uiEntityObj = _app.DBManagerObj.AccessControlModel.UIEntitiesDefined.GetUIEntityFromID(objUIEntity.UIEntitiesID);
						if((objUIEntity.UIEntitiesID == UIEntityID))
						{
							ACLPermissionAssigned aclPermissions = getUIEntityPermission(objUIEntity);
							if(aclPermissions == ACLPermissionAssigned.NOPERMISSIONS)
							{
								bUIEntityAccessible = false;
							}
							assigned = true;
							break;
						}
						else if(uiEntityObj != null && uiEntityObj.ItemName == UIEntityID)
						{
							ACLPermissionAssigned aclPermissions = getUIEntityPermission(objUIEntity);
							if(aclPermissions == ACLPermissionAssigned.NOPERMISSIONS)
							{
								bUIEntityAccessible = false;
							}
							assigned = true;
							break;
						}
						else
						{
							assigned = false;
						}
					}
					if(assigned == false)
						bUIEntityAccessible = false;
				}
				else
					bUIEntityAccessible = false;
			}
			else
			{
				bUIEntityAccessible=false;
			}
			if(!bUIEntityAccessible)
			{
				//LRRMessagBox.ShowMessage(
			}
			return bUIEntityAccessible;
		}
		/// <summary>
		/// Get The Permission
		/// </summary>
		public static ACLPermissionAssigned getUIEntityPermission(AccessControlUIEntity objUIEntity)
		{
			
			string sAccessPermissions = objUIEntity.AccessPermissions == null ? "" : objUIEntity.AccessPermissions.ToString(Symcon.BusinessObjects.SymConverter.Culture);
			if(sAccessPermissions == "")
			{
				return ACLPermissionAssigned.NOPERMISSIONS;
			}
			ACLPermissionAssigned enumACLPermission = ACLPermissionAssigned.NOPERMISSIONS;
			
			AccessCode enumAccessCode = getAccessCode(sAccessPermissions);
			objUIEntity.AccessPermissionCode = enumAccessCode;

			if((enumAccessCode & (AccessCode.ACCESS)) == (AccessCode.ACCESS))
			{
				enumACLPermission =  ACLPermissionAssigned.READONLY;
			}
			if((enumAccessCode & (AccessCode.CREATE | AccessCode.UPDATE | AccessCode.DELETE)) == (AccessCode.CREATE | AccessCode.UPDATE | AccessCode.DELETE))
			{
				enumACLPermission = ACLPermissionAssigned.FULLCONTROL;
			}
			if((enumAccessCode & (AccessCode.NONE)) == (AccessCode.NONE))
			{
				enumACLPermission =  ACLPermissionAssigned.NOPERMISSIONS;
			}
			return enumACLPermission;
		}

		private static AccessCode getAccessCode(string sAccessPermissions)
		{

			AccessCode enumAccessPermissions = AccessCode.NONE;
				
			//Check if the user has access permission
			int nExists = sAccessPermissions.IndexOf("A");
				
			if(nExists == -1)
				return AccessCode.NONE;
			else
				enumAccessPermissions = AccessCode.ACCESS;

			//Check if the user has create permission
			nExists = sAccessPermissions.IndexOf("C");
			if(nExists != -1)
				enumAccessPermissions = enumAccessPermissions | AccessCode.CREATE;

			//Check if the user has update permissions
			nExists = sAccessPermissions.IndexOf("U");
			if(nExists != -1)
				enumAccessPermissions = enumAccessPermissions | AccessCode.UPDATE;
			
			//Check if the user has delete permissions
			nExists = sAccessPermissions.IndexOf("D");
			if(nExists != -1)
				enumAccessPermissions = enumAccessPermissions | AccessCode.DELETE;

			return enumAccessPermissions;

		}
			
		#endregion


	}
}
