using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.HeaderDispatcher;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
    public partial class UserControlChartSheetsConfig : UserControl
    {
        #region Constructor

        public UserControlChartSheetsConfig()
        {
            InitializeComponent();

            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
        }

        #endregion

        #region Fields

        private LRRApp _app = null;

        #endregion

        #region Public Methods

        public LRRChart AddNewExcelView()
        {
            if (LRRModel.ChildModel.ExcelCharts.GetChartFromDisplayName(textBoxDisplayName.Text) != null)
            {
                textBoxDisplayName.Text = "";
                throw new DuplicateNameException(LRRHeaderText.GetHeaderString("SID_EXCEL_VIEW_EXISTS"));
            }

            LRRExcelView lrrExcelView = new LRRExcelView();
            LRRChart lrrChart = new LRRChart();

            lrrExcelView.ViewID = lrrChart.ID;
            lrrExcelView.ViewType = "Chart";

            UpdateChartObject(lrrChart);

            lrrChart.DataType = "ChartData";

            lrrExcelView.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
            lrrChart.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;

            LRRModel.ChildModel.ExcelCharts.Add(lrrChart);
            LRRModel.ChildModel.ExcelViews.Add(lrrExcelView);

            return lrrChart;
        }

        public void Reset()
        {
            textBoxDisplayName.ReadOnly = false;

            textBoxWorkBookName.Text = string.Empty;
            textBoxChartName.Text = string.Empty;
            textBoxDisplayName.Text = string.Empty;
            textBoxDescription.Text = string.Empty;
        }

        public void PopulateChart(LRRChart lrrChart)
        {
            textBoxWorkBookName.Text = lrrChart.WorkBookName;
            textBoxChartName.Text = lrrChart.ChartName;

            textBoxDisplayName.ReadOnly = false;
            textBoxDisplayName.Text = lrrChart.DisplayName;
            textBoxDisplayName.ReadOnly = true;

            textBoxDescription.Text = lrrChart.Description;
        }

        public void UpdateChartObject(LRRChart lrrChart)
        {
            lrrChart.WorkBookName = textBoxWorkBookName.Text;
            lrrChart.SheetName = string.Empty;
            lrrChart.ChartName = textBoxChartName.Text;
            lrrChart.DisplayName = textBoxDisplayName.Text;
            lrrChart.Description = textBoxDescription.Text;
        }

        #endregion

        #region Event Handlers

        #endregion
    }
}
