using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Utils.MessageLogger;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
	/// <summary>
	/// Summary description for UserControlExcelNamedRangeView.
	/// </summary>
	public class UserControlExcelNamedRangeView : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.Panel panelImage;
		private System.Windows.Forms.PictureBox pictureBox1;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public UserControlExcelNamedRangeView(string miscScreenId)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			LRRChart chart = new LRRChart();
			_lrrApp.DBManagerObj.PopulateObject(chart,typeof(LRRChart), " where ID='" + miscScreenId + "' and DataType='TabularData'");
			if(chart.WorkBookName!=null)
			{
				_workbook = chart.WorkBookName;
				_lrrApp.SymExcelWrapperObj.OpenWorkBook(_workbook);
				_nameRange = chart.ChartName;
				this.Text = chart.DisplayName;
				try
				{
					string[] srgb = chart.SheetName.Split(new char[] {','});
					if(srgb.Length == 3)
					{
						this.BackColor = Color.FromArgb(255,int.Parse(srgb[0].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(),Symcon.BusinessObjects.SymConverter.Culture),int.Parse(srgb[1].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(),Symcon.BusinessObjects.SymConverter.Culture),int.Parse(srgb[2].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim(),Symcon.BusinessObjects.SymConverter.Culture));
					}
					else
					{
						throw new Exception("Invalid number of colors in rgb string, should be 3.");
					}
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(UserControlExcelNamedRangeView),"Invalid color string in db(table: Charts, field:SheetName, ID:" + chart.ID + "):" + ex.Message,Logger.Level.Debug);
				}
				RefreshImage();
//				pictureBox1.Height = _img.Height;
//				pictureBox1.Width = _img.Width;
			}
			else
			{
				throw new Exception("Cannot populate chart object");
			}
			pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;
			pictureBox1.Dock = DockStyle.None;

			this.Load += new EventHandler(UserControlExcelNamedRangeView_Load);
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			_lrrApp.RunControllerObj.RunCycleCompleted -= new EventHandler(RunControllerObj_RunCycleCompleted);
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.panelImage = new System.Windows.Forms.Panel();
			this.pictureBox1 = new System.Windows.Forms.PictureBox();
			this.panelImage.SuspendLayout();
			this.SuspendLayout();
			// 
			// panelImage
			// 
			this.panelImage.AutoScroll = true;
			this.panelImage.BackColor = System.Drawing.Color.Transparent;
			this.panelImage.Controls.Add(this.pictureBox1);
			this.panelImage.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panelImage.Location = new System.Drawing.Point(0, 0);
			this.panelImage.Name = "panelImage";
			this.panelImage.Size = new System.Drawing.Size(800, 384);
			this.panelImage.TabIndex = 7;
			// 
			// pictureBox1
			// 
			this.pictureBox1.Location = new System.Drawing.Point(104, 40);
			this.pictureBox1.Name = "pictureBox1";
			this.pictureBox1.Size = new System.Drawing.Size(432, 216);
			this.pictureBox1.TabIndex = 0;
			this.pictureBox1.TabStop = false;
			// 
			// UserControlExcelNamedRangeView
			// 
			this.BackColor = System.Drawing.Color.White;
			this.Controls.Add(this.panelImage);
			this.Name = "UserControlExcelNamedRangeView";
			this.Size = new System.Drawing.Size(800, 384);
			this.panelImage.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region Fields

		private LRRApp	_lrrApp;
		private string _workbook = null;
		private string _nameRange = null;
//		private int _width = -1;
//		private int _height = -1;
		private Image _img;
		private bool _shouldResize = true;
//		private int _lastZoom = 100;
//		private float _initZoom = 1.0f;
		
		#endregion

		#region Event Handlers

		private void UserControlExcelNamedRangeView_Load(object sender, EventArgs e)
		{
//			_initZoom = Math.Min((this.Width - SystemInformation.VerticalScrollBarWidth)/(float)_img.Width ,((this.Height - panelZoomContainer.Height - SystemInformation.HorizontalScrollBarHeight)/(float)_img.Height));
//			_width = pictureBox1.Width;
//			_height = pictureBox1.Height;
//			int nZoom = (int)((_width*100.0)/(_img.Width) + 0.5);
//			if(nZoom == 100)
//			{
//				nZoom = (int)((_height*100.0)/(_img.Height) + 0.5);
//				_width = ((_width * nZoom)/100);
//			}
//			else
//			{
//				_height = ((_height * nZoom)/100);
//			}
//			_width = (int)(_width *_initZoom);
//			_height = (int)(_height* _initZoom);
			ShowImage();
//			comboBoxZoom.Text = nZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
//			_lastZoom = nZoom;
			_lrrApp.RunControllerObj.RunCycleCompleted += new EventHandler(RunControllerObj_RunCycleCompleted);
			this.Resize += new EventHandler(UserControlExcelNamedRangeView_Resize);
		}

		private void RunControllerObj_RunCycleCompleted(object sender, EventArgs e)
		{
			RefreshViewDel refViewDel = new RefreshViewDel(RefreshView);
			_lrrApp.MainFrame.BeginInvoke(refViewDel);
		}

		private delegate void RefreshViewDel();
		private void RefreshView()
		{
			RefreshImage();
		}

//		private void buttonZoom_Click(object sender, System.EventArgs e)
//		{
//			try
//			{
//				double dblZoom = double.Parse(comboBoxZoom.Text.Replace("%","").Trim(),Symcon.BusinessObjects.SymConverter.Culture);
//				int nZoom = (int)dblZoom;
//				comboBoxZoom.Text = nZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
//				_width = (int)((_img.Width * nZoom *_initZoom)/100);
//				_height = (int)((_img.Height * nZoom * _initZoom)/100);
//				_lastZoom = nZoom;
//				ShowImage();
//				comboBoxZoom.SelectAll();
//			}
//			catch(Exception ex)
//			{
//				comboBoxZoom.Text = _lastZoom.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "%";
//				Logger.LogMessage(typeof(FinancialOverview),"buttonZoom_Click Event:" + ex.Message,Logger.Level.Debug);
//			}
//		}

		private void UserControlExcelNamedRangeView_Resize(object sender, EventArgs e)
		{
			if(_shouldResize)
			ShowImage();
		}

		#endregion

		#region Public Methods

		public void Zoom(int nZoom)
		{
			_shouldResize = false;
			this.Width = pictureBox1.Width = (_img.Width * nZoom)/100;
			this.Height = pictureBox1.Height = (_img.Height * nZoom)/100;
			if(nZoom == 100)
				pictureBox1.SizeMode = PictureBoxSizeMode.CenterImage;
			else
				pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;
			_shouldResize = true;
		}

		public void RefreshImage()
		{
			try
			{
				_img = _lrrApp.SymExcelWrapperObj.GetDataFromNamedRangeAsImage(_workbook,_nameRange);
				pictureBox1.Image = _img;
			}
			catch(Exception ex)
			{
				//	System.Diagnostics.Debug.Assert(false,ex.Message,ex.StackTrace);
				Logger.LogMessage(typeof(UserControlExcelNamedRangeView),"Could not load image:" + ex.Message ,Logger.Level.Debug);
			}
		}

		public void ShowImage()
		{
			int width, height;
			float scale, xscale, yscale;
			xscale = (float)this.Width/(float)_img.Width;
			yscale = (float)this.Height/(float)_img.Height;

			scale = Math.Min(xscale,yscale);

			width = (int)(_img.Width * scale);
			height = (int)(_img.Height * scale);

			
			pictureBox1.Size = new Size(width,height);
			
			pictureBox1.Location = new Point(0,0);
			pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;
//			if(_width == -1)
//				_width = (int)(_img.Width * _initZoom);
//
//			if(_height == -1)
//				_height = (int)(_img.Height * _initZoom);
//			
//			
//			pictureBox1.Height = _height;
//			pictureBox1.Width = _width;
//			pictureBox1.Location = new Point(Math.Max((this.Width - _width)/2,0),Math.Max((this.panelImage.Height - _height - SystemInformation.HorizontalScrollBarHeight)/2,0));
		}

		#endregion
	}
}
