using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.BusinessObjects;
using Syncfusion.Windows.Forms.Chart;
using Symcon.Utils.HeaderDispatcher;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
	/// <summary>
	/// Summary description for UserControlTrend.
	/// </summary>
	public class UserControlGroupedTrend : System.Windows.Forms.UserControl, IGroupedTrend
	{
		private Syncfusion.Windows.Forms.Chart.ChartControl chartControlTrend;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public UserControlGroupedTrend()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

			this.Load += new EventHandler(UserControlGroupedTrend_Load);
			this.Text = "Trend Chart";
			chartControlTrend.Model.Series.Clear();
			chartControlTrend.ChartFormatAxisLabel += new ChartFormatAxisLabelEventHandler(chartControlTrend_ChartFormatAxisLabelEventHandler);
			this.chartControlTrend.Legend.Visible = false;
            this.chartControlTrend.DisplayChartContextMenu = false;
            _lrrApp.RunControllerObj.TrendPointRecieved += new TrendPointRecievedDel(RunControllerObj_TrendPointRecieved);
        }

        public delegate void AddPointDel(LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange);

        void RunControllerObj_TrendPointRecieved(LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange)
        {
             if (siteCompPropValueChange.SiteComponentPropertyValueID == _siteGroupPropertyQualifierValueID)
             {
                AddPointDel addPointDel = new AddPointDel(AddPoint);
                _lrrApp.MainFrame.Invoke(addPointDel, new object[] { siteCompPropValueChange });
             }

        }

        private void AddPoint(LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange)
        {

            
            //chartControlTrend.Model.Series.Remove(series);
            try
            {
                double point;
                if (double.TryParse(ConvertUOM(_iPropValue.GetUOMID(), _currentUOM, siteCompPropValueChange.PropertyValue),System.Globalization.NumberStyles.Float, Symcon.BusinessObjects.SymConverter.Culture, out point))
                {
                    ChartSeries series = chartControlTrend.Model.Series[0];
                    double[] points = new double[] { point, 1 };

                    if (DateTime.Parse(siteCompPropValueChange.StartTime, Symcon.BusinessObjects.SymConverter.Culture) < _startTime)
                        _startTime = DateTime.Parse(siteCompPropValueChange.StartTime, Symcon.BusinessObjects.SymConverter.Culture);
                    if (DateTime.Parse(siteCompPropValueChange.StartTime, Symcon.BusinessObjects.SymConverter.Culture) > _endTime)
                        _endTime = DateTime.Parse(siteCompPropValueChange.StartTime, Symcon.BusinessObjects.SymConverter.Culture);

                    _ts = _endTime - _startTime;
                    if (_ts.TotalHours < 11 && _ts.TotalMilliseconds >= 0)
                    {
                        _ts = TimeSpan.FromHours(11);
                        _startTime = _endTime.Subtract(_ts);
                    }
                    AdjustXScale(_trendScheme);
                    series.Points.Add(DateTime.Parse(siteCompPropValueChange.StartTime, Symcon.BusinessObjects.SymConverter.Culture), points);
                    AdjustXScale(_trendScheme);
                }
            }
            catch
            {

            }

        }

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
            _lrrApp.RunControllerObj.TrendPointRecieved -= new TrendPointRecievedDel(RunControllerObj_TrendPointRecieved);
			//chartControlTrend.Dispose();
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.chartControlTrend = new Syncfusion.Windows.Forms.Chart.ChartControl();
			this.SuspendLayout();
			// 
			// chartControlTrend
			// 
			this.chartControlTrend.ColumnWidthMode = Syncfusion.Windows.Forms.Chart.ChartColumnWidthMode.DefaultWidthMode;
			this.chartControlTrend.Controls.Add(this.chartControlTrend.Legend);
			this.chartControlTrend.Dock = System.Windows.Forms.DockStyle.Fill;
			// 
			// chartControl1.Legend
			// 
			this.chartControlTrend.Legend.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.chartControlTrend.Legend.Location = new System.Drawing.Point(506, 205);
			this.chartControlTrend.Legend.Name = "";
			this.chartControlTrend.Legend.Size = new System.Drawing.Size(76, 28);
			this.chartControlTrend.Legend.TabIndex = 3;
			this.chartControlTrend.Location = new System.Drawing.Point(0, 0);
			this.chartControlTrend.Name = "chartControlTrend";
			//this.chartControlTrend.PrimaryXAxis.Orientation = System.Windows.Forms.Orientation.Horizontal;
			this.chartControlTrend.PrimaryXAxis.SmartDateZoomDayLevelLabelFormat = "g";
			this.chartControlTrend.PrimaryXAxis.SmartDateZoomMonthLevelLabelFormat = "m";
			this.chartControlTrend.PrimaryYAxis.SmartDateZoomDayLevelLabelFormat = "g";
			this.chartControlTrend.PrimaryYAxis.SmartDateZoomMonthLevelLabelFormat = "m";
			this.chartControlTrend.Size = new System.Drawing.Size(592, 400);
			this.chartControlTrend.Spacing = 10F;
			this.chartControlTrend.TabIndex = 0;
			this.chartControlTrend.Text = "chartControl1";
			this.chartControlTrend.Tilt = 30F;
			// 
			// chartControl1.ToolBar
			// 
			this.chartControlTrend.ToolBar.Location = new System.Drawing.Point(26, 26);
			this.chartControlTrend.ZoomFactorX = 1;
			this.chartControlTrend.ZoomFactorY = 1;
			this.chartControlTrend.ZoomOutIncrement = 1.5;
			// 
			// UserControlTrend
			// 
			this.Controls.Add(this.chartControlTrend);
			this.Name = "UserControlTrend";
			this.Size = new System.Drawing.Size(592, 400);
			this.ResumeLayout(false);

		}
		#endregion

		#region Fields

		private LRRApp _lrrApp;
		private string _siteGroupPropertyQualifierValueID = null;
        private DateTime _startTime = DateTime.Now.AddYears(1500);
		private DateTime _endTime = DateTime.Now.AddYears(-1500);
		private TimeSpan _ts;
		private string _errorMsg = "";
		private Form _frmOwner;
		private string _newText = "";
		private string _currentUOM = null;
		private IPropertyValue _iPropValue = null;
		private bool _isDateInitialized = false;
        private DateTime _actualStartTime = DateTime.Now.AddYears(1500);
        private DateTime _actualEndTime = DateTime.Now.AddYears(-1500);
		private string _trendScheme = null;
		private TrendSetting _trendSetting = null;
        private ChartSeries _seriesOrig;

		#endregion

		#region Private Methods

		private string ConvertUOM(string oldUOMValue,string newUOMValue,string propVal)
		{
			string newVal = "";
			if(propVal != null && propVal != "")
			{
				if(oldUOMValue == newUOMValue)
				{
					newVal = propVal;
				}
				else
				{
					if((oldUOMValue != null && oldUOMValue != "") &&
						(newUOMValue != null && newUOMValue != ""))
					{
						newVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(oldUOMValue,newUOMValue,propVal,null);
					}
				}
			}
			return newVal;
		}

		private bool LoadChart(TrendSetting trendSetting)
		{
			this.chartControlTrend.Text = _newText;

			DateTime startTime = DateTime.Now;
			DateTime endTime = DateTime.Now.AddYears(-1500);

			LRRSiteComponentPropertyValueChangeUserAndTimes scpvcuts = new LRRSiteComponentPropertyValueChangeUserAndTimes();

            _lrrApp.RecordingValuesManagerObj.PopulateCollection(scpvcuts, " WHERE  SiteComponentPropertyValueID='" + _siteGroupPropertyQualifierValueID + "' AND IsRecorded=False");

			chartControlTrend.SuspendLayout();
				
			ChartSeries series = _seriesOrig = this.chartControlTrend.Model.NewSeries("Series 0");

			foreach(LRRSiteComponentPropertyValueChangeUserAndTime scpvcut in scpvcuts)
			{
				try
				{
                    double point;
                    if (double.TryParse(ConvertUOM(_iPropValue.GetUOMID(), _currentUOM, scpvcut.PropertyValue),System.Globalization.NumberStyles.Float, Symcon.BusinessObjects.SymConverter.Culture, out point))
                    {
                        if (DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture) < startTime)
                            startTime = DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture);
                        if (DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture) > endTime)
                            endTime = DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture);

                        double[] points = new double[] { point, 1 };
                        series.Points.Add(DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture), points);
                    }
				}
				catch
				{

				}
			}
            bool check = true;
			if(series.Points.Count == 0)
			{
				//chartControlTrend.ResumeLayout();
				if(_lrrApp.ToggleTrending)
					_errorMsg = LRRHeaderText.GetHeaderString("SID_TREND_ERROR");
				else
					_errorMsg = LRRHeaderText.GetHeaderString("SID_TREND_ENABLE");
                check = false;
			}
			startTime.AddHours(-0.5);
			endTime.AddMinutes(0.5);
			//_errorMsg = "";
			_startTime = startTime;
			_endTime = endTime;
			series.Type = (ChartSeriesType)Enum.Parse(typeof(ChartSeriesType),trendSetting.SeriesType);
			series.Text = series.Name;
			
			_frmOwner = _lrrApp.MainFrame.ActiveMdiChild;

			_ts = endTime - startTime;
            if (_ts.TotalHours < 11 && check)
            {
                _ts = TimeSpan.FromHours(11);
                _startTime = endTime.Subtract(_ts);
            }
            //TimeSpan timeMargin = TimeSpan.FromTicks(_ts.Ticks/10);
			
            //_endTime = endTime.Add(timeMargin);
            //_ts = _endTime - _startTime;

			series.PrepareStyle += new ChartPrepareStyleInfoHandler(this.ChartControlSeries_PrepareStyleMulti);

            try
            {
                series.Style.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, new Color[] { Color.FromArgb(int.Parse(trendSetting.SeriesInteriorColor)), Color.Black });
            }
            catch
            {
                try
                {
                    series.Style.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, new Color[] { Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), trendSetting.SeriesInteriorColor)), Color.Black });
                }
                catch { }
            }

			chartControlTrend.Model.Series.Clear();
			this.chartControlTrend.Series.Add(series);

			this.chartControlTrend.ShowToolTips = true;

			chartControlTrend.ResumeLayout();

			_actualStartTime = _startTime;
			_actualEndTime = _endTime;
			return check;
		}

		private bool LoadChart(string trendScheme)
		{
			DateTime startTime = DateTime.Now;
			DateTime endTime = DateTime.Now.AddYears(-1500);

			LRRSiteComponentPropertyValueChangeUserAndTimes scpvcuts = new LRRSiteComponentPropertyValueChangeUserAndTimes();

            _lrrApp.RecordingValuesManagerObj.PopulateCollection(scpvcuts, " WHERE  SiteComponentPropertyValueID='" + _siteGroupPropertyQualifierValueID + "' AND IsRecorded=False");

			chartControlTrend.SuspendLayout();
				
			ChartSeries series = this.chartControlTrend.Model.NewSeries("Series 0");

			foreach(LRRSiteComponentPropertyValueChangeUserAndTime scpvcut in scpvcuts)
			{
				try
				{
                    double point;
                    if (double.TryParse(ConvertUOM(_iPropValue.GetUOMID(), _currentUOM, scpvcut.PropertyValue), System.Globalization.NumberStyles.Float, Symcon.BusinessObjects.SymConverter.Culture, out point))
                    {
                        if (DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture) < startTime)
                            startTime = DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture);
                        if (DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture) > endTime)
                            endTime = DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture);

                        double[] points = new double[] { point, 1 };
                        series.Points.Add(DateTime.Parse(scpvcut.StartTime, Symcon.BusinessObjects.SymConverter.Culture), points);
                    }
				}
				catch
				{

				}
			}
            bool check = true;
			if(series.Points.Count == 0)
			{
				//chartControlTrend.ResumeLayout();
				if(_lrrApp.ToggleTrending)
					_errorMsg = LRRHeaderText.GetHeaderString("SID_TREND_ERROR");
				else
					_errorMsg = LRRHeaderText.GetHeaderString("SID_TREND_ENABLE");
				check = false;
			}
			startTime.AddHours(-0.5);
			endTime.AddMinutes(0.5);
			_errorMsg = "";
			_startTime = startTime;
			_endTime = endTime;
			series.Type = (ChartSeriesType)Enum.Parse(typeof(ChartSeriesType),UserControlTrendOptions.HtSchemes[trendScheme].ToString().Split("|".ToCharArray())[0]);
			series.Text = series.Name;
			this.chartControlTrend.Text = _newText;
			_frmOwner = _lrrApp.MainFrame.ActiveMdiChild;

			_ts = endTime - startTime;
            if (_ts.TotalHours < 11 && _ts.TotalMilliseconds>=0)
            {
                _ts = TimeSpan.FromHours(11);
                _startTime = endTime.Subtract(_ts);
            }
            //TimeSpan timeMargin = TimeSpan.FromTicks(_ts.Ticks/10);
            //_startTime = startTime.Subtract(timeMargin);
            //_endTime = endTime.Add(timeMargin);
            //_ts = _endTime - _startTime;

			series.PrepareStyle += new ChartPrepareStyleInfoHandler(this.ChartControlSeries_PrepareStyle);

            try
            {
                series.Style.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, new Color[] { Color.FromArgb(int.Parse(UserControlTrendOptions.HtSchemes[_trendScheme].ToString().Split("|".ToCharArray())[3])), Color.Black });
            }
            catch
            {
                try
                {
                    series.Style.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, new Color[] { Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), UserControlTrendOptions.HtSchemes[_trendScheme].ToString().Split("|".ToCharArray())[3])), Color.Black });
                }
                catch { }
            }

			chartControlTrend.Model.Series.Clear();
			this.chartControlTrend.Series.Add(series);

			this.chartControlTrend.ShowToolTips = true;

			chartControlTrend.ResumeLayout();

			_actualStartTime = _startTime;
			_actualEndTime = _endTime;
			return check;
		}

		/// <summary>
		/// Sets properties to customize the ChartControl's appearance
		/// </summary>
		private void InitializeChartAppearance()
		{			
			this.chartControlTrend.PrimaryXAxis.ValueType = ChartValueType.DateTime;
			this.chartControlTrend.PrimaryXAxis.GridLineType.ForeColor = Color.FromArgb(100,255,0,0);
			this.chartControlTrend.PrimaryYAxis.GridLineType.ForeColor = Color.FromArgb(100,255,0,0);
			this.chartControlTrend.PrimaryXAxis.Font = this.Font;
			this.chartControlTrend.PrimaryYAxis.Font = this.Font;
		}

		#endregion

		#region Event Handlers

        void comboBoxUOM_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                ComboBox comboBoxUOM = sender as ComboBox;

                string newUOM = comboBoxUOM.SelectedValue as string;
                ChartSeries seriesOrig = _seriesOrig;
                ChartSeries oldSeries = chartControlTrend.Model.Series[0];
                chartControlTrend.Model.Series.Remove(oldSeries);
                ChartSeries seriesNew = chartControlTrend.Model.NewSeries("series 0");
                for(int i=0;i<seriesOrig.Points.Count;i++)
                {
                    ChartPoint cp = seriesOrig.Points[i];
                    seriesNew.Points.Add(cp.DateX, double.Parse(ConvertUOM(_currentUOM, newUOM, cp.YValues[0].ToString())));
                }

               
                _currentUOM = newUOM;
                seriesNew.Type = (ChartSeriesType)Enum.Parse(typeof(ChartSeriesType), _trendSetting.SeriesType);
                seriesNew.Text = seriesNew.Name;

                try
                {
                    seriesNew.Style.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, new Color[] { Color.FromArgb(int.Parse(_trendSetting.SeriesInteriorColor)), Color.Black });
                }
                catch
                {
                    try
                    {
                        seriesNew.Style.Interior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, new Color[] { Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), _trendSetting.SeriesInteriorColor)), Color.Black });
                    }
                    catch
                    { }
                }

                seriesNew.PrepareStyle += new ChartPrepareStyleInfoHandler(ChartControlSeries_PrepareStyleMulti);
                chartControlTrend.Model.Series.Add(seriesNew);

            }
            catch
            { }
        }

		private void chartControlTrend_ChartFormatAxisLabelEventHandler(object sender,ChartFormatAxisLabelEventArgs args )
		{
			_ts = _endTime - _startTime;
			if(_ts.TotalMilliseconds != 0)
			{
				if(args.IsAxisPrimary)
				{
					if(((_ts.TotalDays * 1152 * (Screen.PrimaryScreen.Bounds.Width - (this.Width - this.chartControlTrend.PrimaryXAxis.Size.Width)))/(this.chartControlTrend.PrimaryXAxis.Size.Width * Screen.PrimaryScreen.Bounds.Width)) < 17)
					{
						//this.chartControlTrend.PrimaryXAxis.DateTimeRange = new ChartDateTimeRange(_startTime,_endTime, Math.Max((_ts.TotalMilliseconds * 4.5 * 1152 *(Screen.PrimaryScreen.Bounds.Width - (this.Width - this.chartControlTrend.PrimaryXAxis.Size.Width)))/(2.0*24*1000* Screen.PrimaryScreen.Bounds.Width * this.chartControlTrend.PrimaryXAxis.Size.Width),1) , ChartDateTimeIntervalType.Seconds);
						args.Label = args.ValueAsDate.ToString("M/d/yy\nHH:mm:ss");
					}
					else
					{
						//this.chartControlTrend.PrimaryXAxis.DateTimeRange = new ChartDateTimeRange(_startTime,_endTime, Math.Max((_ts.TotalMilliseconds * 2.25 * 1152 *(Screen.PrimaryScreen.Bounds.Width - (this.Width - this.chartControlTrend.PrimaryXAxis.Size.Width)))/(2.0*24*1000* Screen.PrimaryScreen.Bounds.Width * this.chartControlTrend.PrimaryXAxis.Size.Width),1) , ChartDateTimeIntervalType.Seconds);
						args.Label = args.ValueAsDate.ToString("M/d/yy");
					}
					args.Handled = true;
				}
			}
		}

		/// <summary>
		/// Handler for the PrepareStyle event of ChartSeries. 
		/// </summary>
		/// <param name="sender">The ChartSeries object that raises this event.</param>
		/// <param name="args">The event data. Can be modified to change series appearance.</param>
		protected void ChartControlSeries_PrepareStyle(object sender, ChartPrepareStyleInfoEventArgs args)
		{
			// Style formatting using a callback. You can apply the same settings directly on the series style on the
			// point styles.
			ChartSeries series = sender as ChartSeries;
			if(series != null)
			{
				args.Style.Symbol.Shape = (ChartSymbolShape)Enum.Parse(typeof(ChartSymbolShape),UserControlTrendOptions.HtSchemes[_trendScheme].ToString().Split("|".ToCharArray())[1]);
				args.Style.Symbol.Size = new Size(5,5);
                try
                {
                    args.Style.Symbol.Color = Color.FromArgb(int.Parse(UserControlTrendOptions.HtSchemes[_trendScheme].ToString().Split("|".ToCharArray())[2]));
                }
                catch
                {
                    try
                    {
                        args.Style.Symbol.Color = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), UserControlTrendOptions.HtSchemes[_trendScheme].ToString().Split("|".ToCharArray())[2]));
                    }
                    catch
                    { }
                }
				args.Style.DisplayShadow = true;
				args.Style.ToolTip = LRRHeaderText.GetHeaderString("SID_STARTTIME") + ": " + series.Points[args.Index].DateX.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "\n" + LRRHeaderText.GetHeaderString("SID_PROPERTYVALUE") + ": " + series.Points[args.Index].YValues[0];
				args.Handled = true;
			}
		}

		protected void ChartControlSeries_PrepareStyleMulti(object sender, ChartPrepareStyleInfoEventArgs args)
		{
			// Style formatting using a callback. You can apply the same settings directly on the series style on the
			// point styles.
			ChartSeries series = sender as ChartSeries;
			if(series != null)
			{
				args.Style.Symbol.Shape = (ChartSymbolShape)Enum.Parse(typeof(ChartSymbolShape),_trendSetting.SymbolShape);
				args.Style.Symbol.Size = new Size(5,5);
                try
                {
                    args.Style.Symbol.Color = Color.FromArgb(int.Parse(_trendSetting.SymbolColor));
                }
                catch
                {
                    try
                    {
                        args.Style.Symbol.Color = Color.FromKnownColor((KnownColor)Enum.Parse(typeof(KnownColor), _trendSetting.SymbolColor));
                    }
                    catch { }
                }
				args.Style.DisplayShadow = true;
				args.Style.ToolTip = LRRHeaderText.GetHeaderString("SID_STARTTIME") + ": " + series.Points[args.Index].DateX.ToString(Symcon.BusinessObjects.SymConverter.Culture) + "\n" + LRRHeaderText.GetHeaderString("SID_PROPERTYVALUE") + ": " + series.Points[args.Index].YValues[0];
				args.Handled = true;
			}
		}

		private void UserControlGroupedTrend_Resize(object sender, EventArgs e)
		{
			AdjustXScale(_trendScheme);
		}

		private void UserControlGroupedTrend_Load(object sender, EventArgs e)
		{
			this.Resize += new EventHandler(UserControlGroupedTrend_Resize);
		}

		#endregion

		#region IGroupedTrend Members

		public string ErrorMsg
		{
			get
			{
				return _errorMsg;
			}
			set
			{
				_errorMsg = value;
			}
		}

		public Form FormOwner
		{
			get
			{
				return _frmOwner;
			}
			set
			{
				_frmOwner = value;
			}
		}

		public DateTime StartTime
		{
			get
			{
				return _startTime;
			}
			set
			{
				_startTime = value;
			}
		}

		public DateTime EndTime
		{
			get
			{
				return _endTime;
			}
			set
			{
				_endTime = value;
			}
		}

		public bool IsDateInitialized
		{
			get
			{
				return _isDateInitialized;
			}
			set
			{
				_isDateInitialized = value;
			}
		}

		public void AdjustXScale(string trendScheme)
		{
			try
			{
				_ts = _endTime - _startTime;
				if(_ts.TotalMilliseconds != 0)
				{
					if(((_ts.TotalDays * 1152 * (Screen.PrimaryScreen.Bounds.Width - (this.Width - this.chartControlTrend.PrimaryXAxis.Size.Width)))/(this.chartControlTrend.PrimaryXAxis.Size.Width * Screen.PrimaryScreen.Bounds.Width)) < 17)
						this.chartControlTrend.PrimaryXAxis.DateTimeRange = new ChartDateTimeRange(_startTime,_endTime, Math.Max((_ts.TotalMilliseconds * 4.5 * 1152 *(Screen.PrimaryScreen.Bounds.Width - (this.Width - this.chartControlTrend.PrimaryXAxis.Size.Width)))/(2.0*24*1000* Screen.PrimaryScreen.Bounds.Width * this.chartControlTrend.PrimaryXAxis.Size.Width),1) , ChartDateTimeIntervalType.Seconds);
					else
						this.chartControlTrend.PrimaryXAxis.DateTimeRange = new ChartDateTimeRange(_startTime,_endTime, Math.Max((_ts.TotalMilliseconds * 2.25 * 1152 *(Screen.PrimaryScreen.Bounds.Width - (this.Width - this.chartControlTrend.PrimaryXAxis.Size.Width)))/(2.0*24*1000* Screen.PrimaryScreen.Bounds.Width * this.chartControlTrend.PrimaryXAxis.Size.Width),1) , ChartDateTimeIntervalType.Seconds);
				}
			}
			catch
			{

			}
		}

		public DateTime ActualStartTime
		{
			get
			{
				return _actualStartTime;
			}
		}

		public DateTime ActualEndTime
		{
			get
			{
				return _actualEndTime;
			}
		}

		public bool IsTrendUOMChanged(string trendScheme, IPropertyValue iPropValue, string currentUOM)
		{
			if((iPropValue.GetID() == _iPropValue.GetID()) && (currentUOM != _currentUOM) && (trendScheme == _trendScheme))
				return true;
			return false;
		}

		public void InitTrend(string trendScheme, IPropertyValue iPropValue, string currentUOM)
		{
			_currentUOM=currentUOM;
			_iPropValue = iPropValue;
			_trendScheme = trendScheme;
			if(!_lrrApp.RecordingValuesManagerObj.ExistsRecordingDB())
			{
				_errorMsg = LRRHeaderText.GetHeaderString("SID_TREND_DB_ERROR");
				return;
			}
			_siteGroupPropertyQualifierValueID = iPropValue.GetID();

			InitializeChartAppearance();
			
			chartControlTrend.Font = this.Font;
			LRRSiteGroupPropertyQualifierValue gpqv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(iPropValue.GetID());
			_newText = LRRModel.ChildModel.Modes.GetModeByID(_iPropValue.GetModeID()).ModeName + ":" + LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(gpqv.MasterPropID).Name;
			if(gpqv.LookUpQualifierID != null)
			{
				LRRLookUpQualifier lrrLkUpQual = LRRModel.ChildModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(gpqv.LookUpQualifierID);
				_newText += ":" + LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID).Name;
			}
			_newText+= ":" + LRRModel.ChildModel.LookUpProperties.getLookUpPropertyForLkupPropID(gpqv.LookUpPropertiesID).Name;
			LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(_currentUOM);
			if(uom != null)
			{
				_newText += " (" + uom.Name + ")";
			}
			if(LoadChart(trendScheme))
			    AdjustXScale(trendScheme);
		}

		#endregion

		#region IGroupedTrend Members

        void Symcon.LightRidge.PEAdvisor.UserControls.IGroupedTrend.InitTrend(TrendSetting trendSetting, ComboBox comboBoxUOM)
		{
			string propValueID = trendSetting.SchemeName;
			LRRSiteGroupPropertyQualifierValue gpqv = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(propValueID);

			if(gpqv != null)
			{
				_trendSetting = trendSetting;
                string modelUOM = _currentUOM = gpqv.ModelUOM;
                string uomType = LRRModel.ChildModel.UOMTypeXRefs.GetUOMTypeIDFromUOMID(modelUOM);

                string localeID = LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID;
                string uiUom = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localeID, uomType);
                if (uiUom == null)
                    uiUom = modelUOM;

                if (comboBoxUOM == null)
                {
                    _currentUOM = uiUom;
                }
                else
                {
                    if (comboBoxUOM.SelectedItem == null)
                    {
                        _currentUOM = uiUom;
                        LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomType);

                        comboBoxUOM.DataSource = uoms;
                        comboBoxUOM.DisplayMember = "Name";
                        comboBoxUOM.ValueMember = "ID";
                        try
                        {
                            comboBoxUOM.SelectedValue = uiUom;
                        }
                        catch
                        {
                        }
                        comboBoxUOM.SelectedIndexChanged += new EventHandler(comboBoxUOM_SelectedIndexChanged);
                    }
                    else
                    {
                        comboBoxUOM.SelectedIndexChanged -= new EventHandler(comboBoxUOM_SelectedIndexChanged);
                        _currentUOM = comboBoxUOM.SelectedValue as string;
                        comboBoxUOM.SelectedIndexChanged += new EventHandler(comboBoxUOM_SelectedIndexChanged);
                    }
                }
				_iPropValue = gpqv;
				if(!_lrrApp.RecordingValuesManagerObj.ExistsRecordingDB())
				{
					_errorMsg = LRRHeaderText.GetHeaderString("SID_TREND_DB_ERROR");
					return;
				}
				_siteGroupPropertyQualifierValueID = _iPropValue.GetID();

				InitializeChartAppearance();
			
				chartControlTrend.Font = this.Font;
			
				_newText = LRRModel.ChildModel.Modes.GetModeByID(_iPropValue.GetModeID()).ModeName + ":" + LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(gpqv.MasterPropID).Name;
				if(gpqv.LookUpQualifierID != null)
				{
					LRRLookUpQualifier lrrLkUpQual = LRRModel.ChildModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(gpqv.LookUpQualifierID);
					_newText += ":" + LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID).Name;
				}
				_newText+= ":" + LRRModel.ChildModel.LookUpProperties.getLookUpPropertyForLkupPropID(gpqv.LookUpPropertiesID).Name;

                //LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(_currentUOM);
                //if(uom != null)
                //{
                //    _newText += " (" + uom.Name + ")";
                //}
				LoadChart(trendSetting);
			}
		}

		#endregion
	}
}
