using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Symcon.BusinessObjects;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.Options;
using Symcon.Utils.HeaderDispatcher;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.MessageDispatcher;
using Syncfusion.Windows.Forms.Grid;
using Symcon.Utils.MessageLogger; 

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
    public partial class UserControlRollUpType : UserControl , ILRRUIContainer
    {

        private GridExcelFormatter _gridExcelFormatter = new GridExcelFormatter();
        private LRRApp _app;
        private LRRRollUpTypes _rollUpTypes;
	
		

        public UserControlRollUpType()
        {
            InitializeComponent();

            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            _gridExcelFormatter.WireGrid(this.gridRollUpTypes);
            InitDataSource();
            BindGrid();
        }

        private void BindGrid()
        {
            gridRollUpTypes.SuspendLayout();
            Dictionary<string, SymForeignKeyFieldAttribute> fkColumns;
            Dictionary<string, SymChildDataAttribute> childColumns;
            Symcon.LightRidge.BusinessObjects.Utilities.LRRBusinessObjectUtilities.GetObjectInfo(_rollUpTypes.ItemType,out fkColumns,out childColumns);
            Hashtable fkData = new Hashtable();
            foreach (object obj in fkColumns.Keys)
            {
                string propName = (string)obj;
                SymForeignKeyFieldAttribute fkAttr = fkColumns[propName] as SymForeignKeyFieldAttribute;
                object collectionObject = Activator.CreateInstance(fkAttr.CollectionType);
                _app.DBManagerObj.PopulateCollection(collectionObject, false);
                fkData.Add(propName, collectionObject);
            }


            Symcon.LightRidge.PEAdvisor.GridUtilities.SyncGridUtilities.InitializeGridLookAndFeel(gridRollUpTypes);

            Symcon.LightRidge.PEAdvisor.GridUtilities.SyncGridUtilities.InitializeGridColumns(gridRollUpTypes , _rollUpTypes.ItemType, null, fkData);
            
            gridRollUpTypes.TopLevelGroupOptions.CaptionText = LRRHeaderText.GetHeaderString("SID_ROLL_UP_TYPE") + "{RecordCount}" + LRRHeaderText.GetHeaderString("SID_ITEMS");
           
            gridRollUpTypes.NestedTableGroupOptions.ShowFilterBar = true;
            gridRollUpTypes.TableDescriptor.AllowRemove = true;
            gridRollUpTypes.TableDescriptor.AllowNew = true;
            gridRollUpTypes.TableDescriptor.AllowEdit = true;

            gridRollUpTypes.Engine.SourceListSet.Add(_rollUpTypes);
			

            gridRollUpTypes.DataSource = _rollUpTypes;
            this.gridRollUpTypes.TableOptions.AllowSelection = ((Syncfusion.Windows.Forms.Grid.GridSelectionFlags)((Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Any | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.MixRangeType)));
            gridRollUpTypes.ResumeLayout(false);
        }

        private void InitDataSource()
        {
            _rollUpTypes = LRRModel.RollUpTypes;
        }

        #region ILRRUIContainer Members

        public void ClearSort()
        {
            SyncGridUtilities.ClearSort(gridRollUpTypes);
            SyncGridUtilities.ClearSort(gridRollUpTypes);
        }

        public bool SupportsClearSort()
        {
            // TODO:  Add UserControlDataBaseConnectionInfos.SupportsClearSort implementation
            return true;
        }

        #endregion

        #region ILRRUIContainer Members

        public void CopyUOMTypes()
        {
            
        }

        public bool SupportsCopyUOMTypes()
        {
            return true;
        }

        #endregion

        #region ILRRUIContainer Members

        public void PrintPreview()
        {
            // TODO:  Add UserControlUOMSetDefinition.PrintPreview implementation
            if (gridRollUpTypes.TableControl.CurrentCell.IsActive)
            {
                GridGroupingPrint ggPrint = new GridGroupingPrint();
                ggPrint.PrintData(gridRollUpTypes, true);
            }
            if (gridRollUpTypes.TableControl.CurrentCell.IsActive)
            {
                GridGroupingPrint ggPrint = new GridGroupingPrint();
                ggPrint.PrintData(gridRollUpTypes, true);
            }
        }

        #endregion

        #region ILRRUIContainer Members

        public void ShowPropertyList()
        {
            // TODO:  Add UserControlUOMSetDefinition.ShowPropertyList implementation
        }

        public void ShowComponentList()
        {
            // TODO:  Add UserControlUOMSetDefinition.ShowComponentList implementation
        }

        public bool SupportsResultsFilter()
        {
            // TODO:  Add UserControlUOMSetDefinition.SupportsResultsFilter implementation
            return false;
        }

        #endregion

        #region ILRRUIContainer Members

        public void LeftColMovement()
        {
            // TODO:  Add UserControlUOMSetDefinition.LeftColMovement implementation
        }

        public void RightColMovement()
        {
            // TODO:  Add UserControlUOMSetDefinition.RightColMovement implementation
        }

        #endregion

        #region ILRRUIContainer Members

        public void PageSetup()
        {
            GridGroupingPrint ggPrint = new GridGroupingPrint();
            ggPrint.PageSetUp();
        }

        public void Undo()
        {
            // TODO:  Add UserControlUOMSetDefinition.Undo implementation
        }

        public bool SupportsPageSetup()
        {
            // TODO:  Add UserControlUOMSetDefinition.SupportsPageSetup implementation
            return true;
        }

        public bool SupportsUndo()
        {
            // TODO:  Add UserControlUOMSetDefinition.SupportsUndo implementation
            return false;
        }

        #endregion

        #region ILRRUIContainer Members

        public void OpenCompositeView()
        {
            // TODO:  Add UserControlUOMSetDefinition.OpenCompositeView implementation
        }

        public void TransposeView()
        {
            // TODO:  Add UserControlUOMSetDefinition.TransposeView implementation
        }

        public void OpenQuery()
        {
            // TODO:  Add UserControlUOMSetDefinition.OpenQuery implementation
        }

        public void OpenView()
        {
            // TODO:  Add UserControlUOMSetDefinition.OpenView implementation
        }

        public bool SupportsOpenCompositeView()
        {
            // TODO:  Add UserControlUOMSetDefinition.SupportsOpenCompositeView implementation
            return false;
        }

        public bool SupportsTransposeView()
        {
            // TODO:  Add UserControlUOMSetDefinition.SupportsTransposeView implementation
            return false;
        }

        public bool SupportsOpenQuery()
        {
            // TODO:  Add UserControlUOMSetDefinition.SupportsOpenQuery implementation
            return false;
        }

        public bool SupportsOpenView()
        {
            // TODO:  Add UserControlUOMSetDefinition.SupportsOpenView implementation
            return false;
        }

        #endregion

        #region ILRRUIContainer Members

        public bool SupportsCompositeView()
        {
            // TODO:  Add UserControlUOMSetDefinition.SupportsCompositeView implementation
            return false;
        }

        #endregion


        #region ILRRUIContainer Members

        public bool PerformDirtyChk
        {
            get
            {
                // TODO:  Add UserControlUOMSetDefinition.PerformDirtyChk getter implementation
                return true;
            }
        }

        #endregion

        #region ILRRUIContainer Members

        public void ArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Copy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void CopyMasterProperties()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Delete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DownLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Export()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void InsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void LeftLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public TabControl ParentTabControl
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        public TabPage ParentTabPage
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        public void Paste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Print()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void RightLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Save()
        {
            gridRollUpTypes.TableControl.CurrentCell.ConfirmChanges();
			gridRollUpTypes.TableControl.CurrentCell.ConfirmChanges();
            LRRRollUpTypes rollUpTypes = _rollUpTypes.GetChanges();
            if(rollUpTypes.Count > 0)
            {
                _app.DBManagerObj.CommitCollection( rollUpTypes);
                _rollUpTypes.RefreshCollection();
                rollUpTypes.RefreshCollection();
                OptionsUserControl.saved  = true; 
            }
        }

        public void SelectionPaste()
        {
            
        }

        public bool SupportsArrangeColumns()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsClearFilter()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsCopy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsDelete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsExport()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsImportToExcel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsInsertNewRows()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsMasterPropertiesCopy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsPrint()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsRefresh()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsSave()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool SupportsSelectionPaste()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpLevel()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        private void gridRollUpTypes_TableControlCurrentCellCloseDropDown(object sender, GridTableControlPopupClosedEventArgs e)
        {
           
        }

        private void gridRollUpTypes_TableControlCurrentCellShowingDropDown(object sender, GridTableControlCurrentCellShowingDropDownEventArgs e)
        {
            string colHeader = e.TableControl.Table.TableModel.GetColumnDescriptorAt(e.TableControl.CurrentCell.RowIndex, e.TableControl.CurrentCell.ColIndex).HeaderText;
            if (colHeader != LRRHeaderText.GetHeaderString("SID_UOM_TYPE"))
            {
                LRRRollUpType rollUpType = e.TableControl.Table.CurrentRecord.GetData() as LRRRollUpType;
                if (rollUpType.UOMTypeID != null)
                {
                    GridCurrentCell oCurrCell = e.TableControl.CurrentCell as GridCurrentCell;
                    GridCellModelBase oCurrCellModel = oCurrCell.Model as GridCellModelBase;
                    GridTableCellStyleInfoIdentity tableCellInfo = e.TableControl.Model[oCurrCell.RowIndex, oCurrCell.ColIndex].CellIdentity as GridTableCellStyleInfoIdentity;
                    if (tableCellInfo.TableCellType == GridTableCellType.FilterBarCell)
                    {
                        return;
                    }

                    string sUOMType = "";
                    try
                    {
                        sUOMType = ((LRRRollUpType)e.TableControl.Table.CurrentRecord.GetData()).UOMTypeID;
                        if (tableCellInfo.Column.MappingName == "UOMTypeID")
                            return;
                    }
                    catch (Exception ex)
                    {
                        Logger.LogMessage(typeof(UserControlUOMSetDefinition), "gridUOMSetDefinitions_TableControlCurrentCellShowingDropDown Event :" + ex.Message, Logger.Level.Debug);
                    }
                    if (sUOMType != "" && sUOMType != null)
                    {
                        LRRUOMs uoms = LRRModel.ChildModel.GetUOMsFromUOMTypeID(sUOMType);
                        GridComboBoxCellRenderer cr = e.TableControl.CurrentCell.Renderer as GridComboBoxCellRenderer;
                        uoms.SortColumn = "Name";
                        uoms.Sort();
                        cr.ListBoxPart.DataSource = uoms;
                        cr.Model.SupportsChoiceList = false;

                        if (uoms.Count == 0)
                        {
                            e.TableControl.CurrentCell.ConfirmChanges();
                            LRRMessagBox.ShowMessage("SID_NO_UOM", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            e.Inner.Cancel = true;
                        }
                    }
                }
                else
                {
                    e.TableControl.CurrentCell.ConfirmChanges();
                    LRRMessagBox.ShowMessage("SID_UOMTYPE_NULL", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    e.Inner.Cancel = true;
                }
            }
        }
    }
}
