using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Symcon.BusinessObjects;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.Options;
using Symcon.LightRidge.Options.TrendCompressionOptions;
using Symcon.Utils.HeaderDispatcher;

namespace Symcon.LightRidge.PEAdvisor.UserControls
{
	/// <summary>
	/// Summary description for UserControlTrendCompressionOptions.
	/// </summary>
	public class UserControlTrendCompressionOptions : System.Windows.Forms.UserControl, ILRRUIContainer
	{
		private System.Windows.Forms.Label labelNoOFRuns;
		private System.Windows.Forms.TextBox textBoxNoOfRuns;
		private System.Windows.Forms.Label labelNoOfRes;
		private System.Windows.Forms.ComboBox comboBoxNoOfRes;
		private System.Windows.Forms.Panel panelResolutionDetails;
		private System.Windows.Forms.Button buttonCompress;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public UserControlTrendCompressionOptions()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			comboBoxNoOfRes.SelectedIndex = 0;
            
			_lrrApp = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;

			LRROptions options =_lrrApp.GetOptions();
			LRRTrendCompressionDetails trendCompOptions = options.TrendCompDetails;

			if(trendCompOptions != null)
			{
				textBoxNoOfRuns.Text = trendCompOptions.NumberOfRuns.ToString();
				int noOfRes= trendCompOptions.PeriodsDetails.Length;

				comboBoxNoOfRes.SelectedIndex = noOfRes - 1;
				int j=0;
				for(int i=noOfRes-1;i>=0;i--)
				{
					UserControlTrendCompressionResolution ucTrendRes = panelResolutionDetails.Controls[i] as UserControlTrendCompressionResolution;
					LRRTrendCompressionPeriodDetails trendPeriodDetails = trendCompOptions.PeriodsDetails[j];
					
					ucTrendRes.textBoxPeriodValue.Text = trendPeriodDetails.Period;
					ucTrendRes.textBoxNoOfPoints.Text = trendPeriodDetails.NumberOfPoints.ToString();
					
					ucTrendRes.AdjustUnit();
					
					j++;
				}
			}
            InternationalizationUtils.GetControlText(this, null);
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.labelNoOFRuns = new System.Windows.Forms.Label();
            this.textBoxNoOfRuns = new System.Windows.Forms.TextBox();
            this.labelNoOfRes = new System.Windows.Forms.Label();
            this.comboBoxNoOfRes = new System.Windows.Forms.ComboBox();
            this.panelResolutionDetails = new System.Windows.Forms.Panel();
            this.buttonCompress = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // labelNoOFRuns
            // 
            this.labelNoOFRuns.AutoSize = true;
            this.labelNoOFRuns.Location = new System.Drawing.Point(48, 40);
            this.labelNoOFRuns.Name = "labelNoOFRuns";
            this.labelNoOFRuns.Size = new System.Drawing.Size(0, 13);
            this.labelNoOFRuns.TabIndex = 0;
            this.labelNoOFRuns.Tag = "NO_OF_RUN_COMPRESS";
            // 
            // textBoxNoOfRuns
            // 
            this.textBoxNoOfRuns.Location = new System.Drawing.Point(51, 66);
            this.textBoxNoOfRuns.Name = "textBoxNoOfRuns";
            this.textBoxNoOfRuns.Size = new System.Drawing.Size(56, 20);
            this.textBoxNoOfRuns.TabIndex = 1;
            this.textBoxNoOfRuns.Text = "30";
            // 
            // labelNoOfRes
            // 
            this.labelNoOfRes.AutoSize = true;
            this.labelNoOfRes.Location = new System.Drawing.Point(48, 107);
            this.labelNoOfRes.Name = "labelNoOfRes";
            this.labelNoOfRes.Size = new System.Drawing.Size(0, 13);
            this.labelNoOfRes.TabIndex = 2;
            this.labelNoOfRes.Tag = "NO_OF_RES";
            // 
            // comboBoxNoOfRes
            // 
            this.comboBoxNoOfRes.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxNoOfRes.Items.AddRange(new object[] {
            "1",
            "2",
            "3",
            "4",
            "5"});
            this.comboBoxNoOfRes.Location = new System.Drawing.Point(51, 133);
            this.comboBoxNoOfRes.Name = "comboBoxNoOfRes";
            this.comboBoxNoOfRes.Size = new System.Drawing.Size(40, 21);
            this.comboBoxNoOfRes.TabIndex = 3;
            this.comboBoxNoOfRes.SelectedIndexChanged += new System.EventHandler(this.comboBoxNoOfRes_SelectedIndexChanged);
            // 
            // panelResolutionDetails
            // 
            this.panelResolutionDetails.Location = new System.Drawing.Point(48, 216);
            this.panelResolutionDetails.Name = "panelResolutionDetails";
            this.panelResolutionDetails.Size = new System.Drawing.Size(879, 293);
            this.panelResolutionDetails.TabIndex = 4;
            // 
            // buttonCompress
            // 
            this.buttonCompress.AutoSize = true;
            this.buttonCompress.Location = new System.Drawing.Point(282, 133);
            this.buttonCompress.Name = "buttonCompress";
            this.buttonCompress.Size = new System.Drawing.Size(96, 23);
            this.buttonCompress.TabIndex = 5;
            this.buttonCompress.Tag = "DO_COMPRESS";
            this.buttonCompress.Click += new System.EventHandler(this.buttonCompress_Click);
            // 
            // UserControlTrendCompressionOptions
            // 
            this.Controls.Add(this.buttonCompress);
            this.Controls.Add(this.panelResolutionDetails);
            this.Controls.Add(this.comboBoxNoOfRes);
            this.Controls.Add(this.labelNoOfRes);
            this.Controls.Add(this.textBoxNoOfRuns);
            this.Controls.Add(this.labelNoOFRuns);
            this.Name = "UserControlTrendCompressionOptions";
            this.Size = new System.Drawing.Size(944, 578);
            this.ResumeLayout(false);
            this.PerformLayout();

		}
		#endregion

		#region Fields

		private Hashtable _htResolutionsDetails = new Hashtable();
		private LRRApp _lrrApp = null;
        private bool _toSave = false;
        private bool _isFirstTime = true;

		#endregion

		#region Delegates
		private delegate void CompressionDel();
		private delegate void ShowMessageDel(string messageID);
		#endregion

		#region Private Methods
		
		private void ShowMessage(string messageID)
		{
			LRRMessagBox.ShowMessage(messageID,true);
		}
		private void DoCompression()
		{
			try
			{
				ShowMessageDel showMessage = new ShowMessageDel(ShowMessage);
				object[] args = new object[1];
				args[0] = "SID_COMPRESSION_STARTED";
				_lrrApp.MainFrame.BeginInvoke(showMessage,args);
				LRRTrendCompressionDetails trendCompOptions= new LRRTrendCompressionDetails();
				//trendCompOptions.NumberOfRuns = int.Parse(textBoxNoOfRuns.Text);

				int noOfRes= panelResolutionDetails.Controls.Count;
				trendCompOptions.PeriodsDetails = new LRRTrendCompressionPeriodDetails[noOfRes];

				int j=0;
				for(int i=noOfRes-1;i>=0;i--)
				{
					UserControlTrendCompressionResolution ucTrendRes = panelResolutionDetails.Controls[i] as UserControlTrendCompressionResolution;
					if(!ucTrendRes.Validate())
						return;

					LRRTrendCompressionPeriodDetails trendPeriodDetails = trendCompOptions.PeriodsDetails[j] = new LRRTrendCompressionPeriodDetails();

					TimeSpan period = TimeSpan.FromDays(10*365);
					int periodValue = int.Parse(ucTrendRes.textBoxPeriodValue.Text);

					switch(ucTrendRes.comboBoxPeriodUnit.SelectedIndex)
					{
						case 0:
							period = new TimeSpan(periodValue,0,0);
							break;
						case 1:
							period = new TimeSpan(periodValue,0,0,0);
							break;
						case 2:
							period = new TimeSpan(periodValue*7,0,0,0);
							break;
						case 3:
							period = new TimeSpan(30*periodValue,0,0,0);
							break;
						case 4:
							period = new TimeSpan(periodValue*365,0,0,0);
							break;
					}

					trendPeriodDetails.Period = period.TotalHours.ToString();
					trendPeriodDetails.NumberOfPoints = int.Parse(ucTrendRes.textBoxNoOfPoints.Text);

					j++;
				}
			
				LRRSiteComponentPropertyValueChangeUserAndTimes scpvuts = new LRRSiteComponentPropertyValueChangeUserAndTimes();
				_lrrApp.RecordingValuesManagerObj.PopulateCollection(scpvuts);
			
				Hashtable htScpvuts = new Hashtable();
				ArrayList arrSelectedRecords = new ArrayList();

				foreach(LRRSiteComponentPropertyValueChangeUserAndTime scpvut in scpvuts)
				{
					if(!htScpvuts.Contains(scpvut.SiteComponentPropertyValueID))
					{
						htScpvuts.Add(scpvut.SiteComponentPropertyValueID,new ArrayList());
					}
					(htScpvuts[scpvut.SiteComponentPropertyValueID] as ArrayList).Add(scpvut);
				}

				DateTime startTime = DateTime.Now;
				for(int i=0;i< trendCompOptions.PeriodsDetails.Length;i++)
				{
					LRRTrendCompressionPeriodDetails trendPeriodDetails = trendCompOptions.PeriodsDetails[i];
				
					DateTime endTime = startTime.AddHours(-1 * int.Parse(trendPeriodDetails.Period));

					TimeSpan totalTimeSpan = startTime - endTime;

					TimeSpan intervalSpan = TimeSpan.FromMilliseconds(totalTimeSpan.TotalMilliseconds/trendPeriodDetails.NumberOfPoints);

					for(DateTime timeVar = endTime; timeVar < startTime; timeVar = timeVar.Add(intervalSpan))
					{
						foreach(string propValueID in htScpvuts.Keys)
						{
							int count = 0;
							ArrayList arrValues = htScpvuts[propValueID] as ArrayList;
						
							foreach(LRRSiteComponentPropertyValueChangeUserAndTime scpvut in arrValues)
							{
							
								DateTime currStartTime = DateTime.Parse(scpvut.StartTime,Symcon.BusinessObjects.SymConverter.Culture);
								if(currStartTime>= timeVar && currStartTime < timeVar.Add(intervalSpan))
								{
									if(count > 0)
									{
										scpvut.ObjectState = Symcon.Globals.Enums.DBOperation.DELETE;
									
									}
									if(!arrSelectedRecords.Contains(scpvut))
										arrSelectedRecords.Add(scpvut);
									count ++;
								}
							}
						}
					}

					startTime = endTime;
				}
				LRRSiteComponentPropertyValueChangeUserAndTimes scpvcutsToDelete = new LRRSiteComponentPropertyValueChangeUserAndTimes();
				foreach(LRRSiteComponentPropertyValueChangeUserAndTime scpvut in scpvuts)
				{
					if(!arrSelectedRecords.Contains(scpvut) || scpvut.ObjectState == Symcon.Globals.Enums.DBOperation.DELETE)
						scpvcutsToDelete.Add(scpvut);
				}
				_lrrApp.RecordingValuesManagerObj.ExecuteBatchDelete(scpvcutsToDelete);
				args[0] = "SID_COMPRESSION_COMPLETE";
				_lrrApp.MainFrame.BeginInvoke(showMessage,args);
			}
			catch(Exception ex)
			{
				System.Diagnostics.Debug.Assert(false,ex.Message,ex.StackTrace);
			}
		}

		#endregion

		#region Event Handlers

		private void comboBoxNoOfRes_SelectedIndexChanged(object sender, System.EventArgs e)
		{
            if (!_isFirstTime)
                _toSave = true;
            
			int noOfRes = comboBoxNoOfRes.SelectedIndex +1 ;
			
			_htResolutionsDetails.Clear();
			
			for(int i=1;i<=noOfRes;i++)
			{
				UserControlTrendCompressionResolution compResDetails =  new UserControlTrendCompressionResolution();
                if (_isFirstTime)
                {
                    compResDetails.textBoxNoOfPoints.TextChanged += new EventHandler(textBoxPeriodValue_TextChanged);
                    compResDetails.textBoxPeriodValue.TextChanged += new EventHandler(textBoxPeriodValue_TextChanged);
                }
				_htResolutionsDetails.Add(i,compResDetails);
				compResDetails.Dock = DockStyle.Top;
				compResDetails.labelPeriod.Text = LRRHeaderText.GetHeaderString("SID_PERIOD") + " " + i + ":";
			}
			
			panelResolutionDetails.Controls.Clear();
			
			for(int i= noOfRes;i>=1;i--)
			{
				panelResolutionDetails.Controls.Add(_htResolutionsDetails[i] as Control);
			}
            if (_isFirstTime)
                _isFirstTime = false;
		}

        void textBoxPeriodValue_TextChanged(object sender, EventArgs e)
        {
            _toSave = true;
        }

		private void buttonCompress_Click(object sender, System.EventArgs e)
		{
			if(!_lrrApp.RecordingValuesManagerObj.ExistsRecordingDB())
			{
				MessageBox.Show(this,Symcon.Utils.HeaderDispatcher.LRRHeaderText.GetHeaderString("SID_TREND_DB_ERROR"));
				return;
			}
			CompressionDel compDel = new CompressionDel(DoCompression);
			compDel.BeginInvoke(null,null);
		}

		#endregion

		#region ILRRUIContainer Members

		public void LeftColMovement()
		{
			// TODO:  Add UserControlTrendCompressionOptions.LeftColMovement implementation
		}

		public void ClearSort()
		{
			// TODO:  Add UserControlTrendCompressionOptions.ClearSort implementation
		}

		public void RightLevel()
		{
			// TODO:  Add UserControlTrendCompressionOptions.RightLevel implementation
		}

		public void UpLevel()
		{
			// TODO:  Add UserControlTrendCompressionOptions.UpLevel implementation
		}

		public bool SupportsImportToExcel()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsImportToExcel implementation
			return false;
		}

		public bool SupportsLevel()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsLevel implementation
			return false;
		}

		public bool PerformDirtyChk
		{
			get
			{
				// TODO:  Add UserControlTrendCompressionOptions.PerformDirtyChk getter implementation
				return false;
			}
		}

		public bool SupportsRefresh()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsRefresh implementation
			return false;
		}

		public void Undo()
		{
			// TODO:  Add UserControlTrendCompressionOptions.Undo implementation
		}

		public bool SupportsCopy()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsCopy implementation
			return false;
		}

		public void SelectionPaste()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SelectionPaste implementation
		}

		public TabPage ParentTabPage
		{
			get
			{
				// TODO:  Add UserControlTrendCompressionOptions.ParentTabPage getter implementation
				return null;
			}
			set
			{
				// TODO:  Add UserControlTrendCompressionOptions.ParentTabPage setter implementation
			}
		}

		public void PageSetup()
		{
			// TODO:  Add UserControlTrendCompressionOptions.PageSetup implementation
		}

		public void OpenQuery()
		{
			// TODO:  Add UserControlTrendCompressionOptions.OpenQuery implementation
		}

		public void Copy()
		{
			// TODO:  Add UserControlTrendCompressionOptions.Copy implementation
		}

		public bool SupportsInsertNewRows()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsInsertNewRows implementation
			return false;
		}

		public bool SupportsPageSetup()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsPageSetup implementation
			return false;
		}

		public void ShowPropertyList()
		{
			// TODO:  Add UserControlTrendCompressionOptions.ShowPropertyList implementation
		}

		public bool SupportsCompositeView()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsCompositeView implementation
			return false;
		}

		public bool SupportsClearSort()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsClearSort implementation
			return false;
		}

		public void RightColMovement()
		{
			// TODO:  Add UserControlTrendCompressionOptions.RightColMovement implementation
		}

		public void LeftLevel()
		{
			// TODO:  Add UserControlTrendCompressionOptions.LeftLevel implementation
		}

		public bool SupportsClearFilter()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsClearFilter implementation
			return false;
		}

		public bool SupportsDelete()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsDelete implementation
			return false;
		}

		public void PrintPreview()
		{
			// TODO:  Add UserControlTrendCompressionOptions.PrintPreview implementation
		}

		public bool SupportsResultsFilter()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsResultsFilter implementation
			return false;
		}

		public bool SupportsPaste()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsPaste implementation
			return false;
		}

		public void Print()
		{
			// TODO:  Add UserControlTrendCompressionOptions.Print implementation
		}

		public bool SupportsArrangeColumns()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsArrangeColumns implementation
			return false;
		}

		public bool SupportsPrint()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsPrint implementation
			return false;
		}

		public void OpenView()
		{
			// TODO:  Add UserControlTrendCompressionOptions.OpenView implementation
		}

		public void TransposeView()
		{
			// TODO:  Add UserControlTrendCompressionOptions.TransposeView implementation
		}

		public void ArrangeColumns()
		{
			// TODO:  Add UserControlTrendCompressionOptions.ArrangeColumns implementation
		}

		public void Export()
		{
			// TODO:  Add UserControlTrendCompressionOptions.Export implementation
		}

		public bool SupportsMasterPropertiesCopy()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsMasterPropertiesCopy implementation
			return false;
		}

		public void ShowComponentList()
		{
			// TODO:  Add UserControlTrendCompressionOptions.ShowComponentList implementation
		}

		public void ClearFilter()
		{
			// TODO:  Add UserControlTrendCompressionOptions.ClearFilter implementation
		}

		public void OpenCompositeView()
		{
			// TODO:  Add UserControlTrendCompressionOptions.OpenCompositeView implementation
		}

		public void Delete()
		{
			// TODO:  Add UserControlTrendCompressionOptions.Delete implementation
		}

		public bool SupportsUndo()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsUndo implementation
			return false;
		}

		public bool SupportsExport()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsExport implementation
			return false;
		}

		public void CopyUOMTypes()
		{
			// TODO:  Add UserControlTrendCompressionOptions.CopyUOMTypes implementation
		}

		public void ImportToExcel()
		{
			// TODO:  Add UserControlTrendCompressionOptions.ImportToExcel implementation
		}

		public void Refresh()
		{
			// TODO:  Add UserControlTrendCompressionOptions.Refresh implementation
		}

		public bool SupportsSelectionPaste()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsSelectionPaste implementation
			return false;
		}

		public void DownLevel()
		{
			// TODO:  Add UserControlTrendCompressionOptions.DownLevel implementation
		}

		public void Paste()
		{
			// TODO:  Add UserControlTrendCompressionOptions.Paste implementation
		}

		public bool SupportsSave()
		{
			return true;
		}

		public void InsertNewRows()
		{
			// TODO:  Add UserControlTrendCompressionOptions.InsertNewRows implementation
		}

		public void CopyMasterProperties()
		{
			// TODO:  Add UserControlTrendCompressionOptions.CopyMasterProperties implementation
		}

		public void Save()
		{
            if (_toSave)
            {
                try
                {
                    LRRTrendCompressionDetails trendCompOptions = new LRRTrendCompressionDetails();
                    trendCompOptions.NumberOfRuns = int.Parse(textBoxNoOfRuns.Text);

                    int noOfRes = panelResolutionDetails.Controls.Count;
                    trendCompOptions.PeriodsDetails = new LRRTrendCompressionPeriodDetails[noOfRes];

                    int j = 0;
                    for (int i = noOfRes - 1; i >= 0; i--)
                    {
                        UserControlTrendCompressionResolution ucTrendRes = panelResolutionDetails.Controls[i] as UserControlTrendCompressionResolution;
                        if (!ucTrendRes.Validate())
                            return;

                        LRRTrendCompressionPeriodDetails trendPeriodDetails = trendCompOptions.PeriodsDetails[j] = new LRRTrendCompressionPeriodDetails();

                        TimeSpan period = TimeSpan.FromDays(10 * 365);
                        int periodValue = int.Parse(ucTrendRes.textBoxPeriodValue.Text);

                        switch (ucTrendRes.comboBoxPeriodUnit.SelectedIndex)
                        {
                            case 0:
                                period = new TimeSpan(periodValue, 0, 0);
                                break;
                            case 1:
                                period = new TimeSpan(periodValue, 0, 0, 0);
                                break;
                            case 2:
                                period = new TimeSpan(periodValue * 7, 0, 0, 0);
                                break;
                            case 3:
                                period = new TimeSpan(30 * periodValue, 0, 0, 0);
                                break;
                            case 4:
                                period = new TimeSpan(periodValue * 365, 0, 0, 0);
                                break;
                        }

                        trendPeriodDetails.Period = period.TotalHours.ToString();
                        trendPeriodDetails.NumberOfPoints = int.Parse(ucTrendRes.textBoxNoOfPoints.Text);

                        j++;
                    }

                    LRROptions options = _lrrApp.GetOptions();
                    options.TrendCompDetails = trendCompOptions;
                    OptionsUserControl.saved = true;
                }
                catch
                {

                }
                _toSave = false;
            }
		}

		public TabControl ParentTabControl
		{
			get
			{
				// TODO:  Add UserControlTrendCompressionOptions.ParentTabControl getter implementation
				return null;
			}
			set
			{
				// TODO:  Add UserControlTrendCompressionOptions.ParentTabControl setter implementation
			}
		}

		public bool SupportsCopyUOMTypes()
		{
			// TODO:  Add UserControlTrendCompressionOptions.SupportsCopyUOMTypes implementation
			return false;
		}

		#endregion
		
	}
}
