using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;
using System.Text.RegularExpressions;
using System.Reflection;

using Symcon.BusinessObjects;
using Symcom.Evaluator;
using Symcon.LightRidge.BusinessObjects;
using Symcon.Scripting;
using Symcon.Utils.MessageDispatcher;
using Symcon.LightRidge.BusinessObjects.Utilities;
using Symcon.LightRidge.PEAdvisor.GridUtilities;
using Symcon.Utils.HeaderDispatcher; 

namespace Symcon.LightRidge.PEAdvisor
{
    public partial class WeighedAverageMapping : Form
    {
        public WeighedAverageMapping(LRRMasterProperties masterProps,LRRMasterProperty masterProp)
        {
            InitializeComponent();
            masterProperties = masterProps;
            masterProperty = masterProp;
            InternationalizationUtils.GetControlText(this, null);
            _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
            InitializeTree();
            weightedFacterMappings = LRRModel.WeightedFacterMappings;
            BindGrid();
        }

        #region Fields
        private LRRApp _app;
        private LRRMasterProperties masterProperties;
        private LRRMasterProperty masterProperty;
        private LRRMasterProperty selectedMasterProperty;
        private TreeNode selectedNode = null;
        private int oldNodeIndex = -1;
        private ToolTip toolTip1 = new ToolTip();
        private LRRWeightedFacterMappings weightedFacterMappings = null;
        private LRRWeightedFacterMapping weightedFacterMapping = null;
        #endregion

        private void BindGrid()
        {
            gridWeightedParamMapping.TableDescriptor.Relations.Clear();
            gridWeightedParamMapping.DataSource = null;

            SyncGridUtilities.InitializeGridColumns(gridWeightedParamMapping, gridWeightedParamMapping.TableDescriptor, weightedFacterMappings.ItemType, null);

            SyncGridUtilities.InitializeGridLookAndFeel(gridWeightedParamMapping); 

            gridWeightedParamMapping.Engine.SourceListSet.Add("ParentTable", weightedFacterMappings);

            gridWeightedParamMapping.DataSource = weightedFacterMappings.GetWeightedFacterMappingsFromPropID(masterProperty.PropID);

            gridWeightedParamMapping.TopLevelGroupOptions.CaptionText = LRRHeaderText.GetHeaderString("SID_MASTER_PLANT_ITEMS") + " {RecordCount} " + LRRHeaderText.GetHeaderString("SID_ITEMS");

        }

        private void LoadCombo()
        {

        }

        private void InitializeTree()
        {
            LRRMasterComponent masterComp = LRRModel.Model.GetMasterComponentByID(masterProperties[0].ComponentID);   
            TreeNode root = null;
            trvModel.SuspendLayout();
            trvModel.Nodes.Clear();
            root = new TreeNode("MasterProperties");
            root.Expand();
            LRRPropertyType propType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(masterProperty.Type);
            foreach (LRRMasterProperty masterProp in masterProperties)
            {
                if (masterProp.PropID != masterProperty.PropID && masterProperty.ModeID == masterProp.ModeID)
                {
                    if (propType.IsDefault)
                    {
                        if (masterProp.Type.Equals("1"))
                        {
                            TreeNode mp = new TreeNode(masterProp.Name);
                            mp.Tag = masterProp;
                            root.Nodes.Add(mp);
                        }
                    }
                    else
                    {
                        string type = masterProp.Type;
                        LRRPropertyType pType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(type);
                        if (pType.IsDataArray)
                        {
                            TreeNode mp = new TreeNode(masterProp.Name);
                            mp.Tag = masterProp;
                            root.Nodes.Add(mp);
                        }
                    }
                }
            }
            trvModel.Nodes.Add(root);
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked)
            {
                this.trvModel.Enabled = false;
                this.txtConstantName.Enabled = true;
                this.lblConstantName.Enabled = true;
                this.txtConstantValue.Enabled = true;
                this.lblConstantValue.Enabled = true;
            }
            else
            {
                trvModel.Enabled = true;
                this.txtConstantName.Enabled = false;
                this.lblConstantName.Enabled = false;
                this.txtConstantValue.Enabled = false;
                this.lblConstantValue.Enabled = false;
            }
        }
        private void GenerateNormalWFA()
        {
            
        }

        private void cmbMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            
        }
        private void trvModel_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node != null)
            {
                selectedNode = e.Node;
                selectedMasterProperty = e.Node.Tag as LRRMasterProperty; 
            }
        }

        private void trvModel_MouseMove(object sender, MouseEventArgs e)
        {
            TreeNode tn = this.trvModel.GetNodeAt(e.X, e.Y); 
            if(tn != null) 
			{   
                string toolTipText = "";
				int currentNodeIndex = tn.Index; 
                if(currentNodeIndex != oldNodeIndex) 
				{ 
					oldNodeIndex = currentNodeIndex; 
					object objNode = tn.Tag;
					if(objNode == null)
						return;
                    LRRMasterProperty masterProp = objNode as LRRMasterProperty; 
                    string desc = masterProp.Description;
                
                    string uom = "";
                    string uomID = masterProp.UIUomID;
                    if(uomID != "" && uomID != null)
                    {
                        LRRUOM uomObj = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(uomID);
                        uom = uomObj.Name;
                    }
				    toolTipText = "Desc:" + desc + "\n" + "UOM:" + uom;
                }
                if(this.toolTip1 != null && this.toolTip1.Active) 
               			this.toolTip1.Active = false;  
               	this.toolTip1.SetToolTip(this.trvModel, toolTipText); 
				this.toolTip1.Active = true; 
            }
        }

        private void PropertyMapping()
        {
            LRRWeightedFacterMapping weighteFacObj = weightedFacterMappings.GetWeightedFacterFromPropID(masterProperty.PropID);
            if (weighteFacObj == null)
            {
                LRRWeightedFacterMapping weighteObj = new LRRWeightedFacterMapping();
                weighteObj.ComponentID = masterProperty.ComponentID;
                weighteObj.PropertyID = masterProperty.PropID;
                weighteObj.PropertyName = masterProperty.Name;
                weighteObj.WeightedPropertyID = selectedMasterProperty.PropID;
                weighteObj.WeightedPropertyName = selectedMasterProperty.Name;
                weighteObj.ConstantName = "";
                weighteObj.ConstantValue = "";   
                weighteObj.IsConstant = false;
                weightedFacterMappings.Add(weighteObj);
            }
            else
            {
                DialogResult result = LRRMessagBox.ShowMessage("SID_DUPLICATE_HISTMAPPING", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (result == DialogResult.Yes)
                {
                    weighteFacObj.WeightedPropertyID = selectedMasterProperty.PropID;
                    weighteFacObj.WeightedPropertyName = selectedMasterProperty.Name;
                    weighteFacObj.ConstantName = "";
                    weighteFacObj.ConstantValue = "";
                    weighteFacObj.IsConstant = false;
                }
            }
        }

        private void ConstantMapping()
        {
            LRRWeightedFacterMapping weighteFacObj = weightedFacterMappings.GetWeightedFacterFromPropID(masterProperty.PropID);
            if (weighteFacObj == null)
            {
                LRRWeightedFacterMapping weighteObj = new LRRWeightedFacterMapping();
                weighteObj.ComponentID = masterProperty.ComponentID;
                weighteObj.PropertyID = masterProperty.PropID;
                weighteObj.PropertyName = masterProperty.Name;
                weighteObj.WeightedPropertyID = "";
                weighteObj.WeightedPropertyName = "";
                weighteObj.ConstantName = txtConstantName.Text;
                weighteObj.ConstantValue = txtConstantValue.Text;   
                weighteObj.IsConstant = true;
                weightedFacterMappings.Add(weighteObj);
            }
            else
            {
                DialogResult result = LRRMessagBox.ShowMessage("SID_DUPLICATE_HISTMAPPING", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (result == DialogResult.Yes)
                {
                    weighteFacObj.WeightedPropertyName = "";
                    weighteFacObj.WeightedPropertyID = "";
                    weighteFacObj.ConstantName = txtConstantName.Text;
                    weighteFacObj.ConstantValue = txtConstantValue.Text;
                    weighteFacObj.IsConstant = true;
                }
            }
        }


        private void btnMapObj_Click(object sender, EventArgs e)
        {
            if (this.checkBox1.Checked)
            {
                ConstantMapping();
                gridWeightedParamMapping.TableDescriptor.Relations.Clear();
                gridWeightedParamMapping.DataSource = null;
                gridWeightedParamMapping.DataSource = weightedFacterMappings.GetWeightedFacterMappingsFromPropID(masterProperty.PropID);

            }
            else
            {
                PropertyMapping();
                gridWeightedParamMapping.TableDescriptor.Relations.Clear();
                gridWeightedParamMapping.DataSource = null;
                gridWeightedParamMapping.DataSource = weightedFacterMappings.GetWeightedFacterMappingsFromPropID(masterProperty.PropID);
            }
        }

        private void btnSAVE_Click(object sender, EventArgs e)
        {
            if (weightedFacterMappings.GetChanges().Count > 0)
            {
                _app.DBManagerObj.CommitCollection(weightedFacterMappings.GetChanges());
            }
            weightedFacterMappings.RefreshCollection();
            LRRMessagBox.ShowMessage("SID_SAVEDINFO", "SID_LABLE", MessageBoxButtons.OK);
            this.Close();
        }
    }
}