using System;
using System.Windows.Forms;
using System.Reflection;
using Symcon.Utils.MessageLogger;
using Symcon.LightRidge.HistorianConnect;
//using Symcon.LightRidge.Historian;
using Symcon.LightRidge.Options.HistOptions;
using Symcon.LightRidge.PEAdvisor;
using Symcon.LightRidge.BusinessObjects;
using System.Collections;
using System.Runtime.CompilerServices;
using Symcon.Globals.Enums;
using Symcon.LightRidge.Options;
using Symcon.LightRidge.Runcontrol;
using Symcom.Evaluator;
using Symcon.Scripting;
using Symcon.Utils.MessageDispatcher;
using Symcon.Utils.HeaderDispatcher;
using System.Collections.Generic;
using Symcon.Lightrige.ScriptRunner;
using Symcom.Utils;
//Copyright (C) 2004-2005 Symcom Global Technologies.  All rights reserved.

namespace Symcon.LightRidge.PEAdvisor
{
    /// <summary>
    /// This class manages the Run operation for PE-Advisor.
    /// 1.	Holds the information for the various Historians
    /// 2.	Executes the run. Send the calls to the Historian 
    ///		depending on the Run Options selected by the user..
    /// 3.	Assigns the results back to the Historian Data Store..
    /// </summary>

    public delegate void ThreadRun(string startTime, string endTime);

    public delegate void ReceivedPointDataDel(LRRHistData point, string startTime, string endTime);

    public delegate void RunStartedDel(RunStartedEventArgs e);

    public delegate void UIUpdateDel(LRRHistorianPoint point, LRRHistData histData);

    public delegate void AsyncInvokeDelegate(Delegate del, object[] args);

    public delegate void GiveFeedbackDel(string timeRemaining);

    public delegate void SaveChangesDel(LRRSiteComponentPropertyValues lrrSiteCompPropvalues, LRRSiteGroupPropertyQualifierValues lrrSiteGroupQualValues, LRRAggregatePropertyValues lrrAggregatePropValues);

    public delegate void TimerProcessDel(object sender, EventArgs e);

    public delegate void InitiatePreExtractionDel();

    public delegate void RunCompletedDel(object sender, EventArgs e);

    public delegate void RunStoppedDel(object sender, EventArgs e);

    public delegate void RunCycleCompletedDel(object sender, EventArgs e);

    public delegate void TrendPointRecievedDel(LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange);

    delegate void scriptingAppDomainUnloadDelegate(AppDomain scriptingAppDomainToUnload);


    public class RunController
    {

        Queue _runQueue = new Queue();
        //System.Timers.Timer _timer;
        //System.Threading.Timer _timer;
        System.Windows.Forms.Timer _timer;

        public RunController()
        {
            
        }

        #region Methods

        /// <summary>
        /// This method is used to initialize the 
        /// Run Controller
        /// </summary>
        /// <returns></returns>
        public bool Initialize()
        {
            bool ret = false;
            try
            {
                _app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp; //Get the App Object..

                _offLineHistorian = new OffLineHistorian(); //Create the instance of the offline Historian..
                _tuningHistorian = new TuningHistorian(); //Create the instance of the tuning Historian...



                //_timer = new System.Threading.Timer(new System.Threading.TimerCallback(timer_Tick),null,0,100);
                _timer = new Timer();
                _timer.Enabled = true;
                _timer.Interval = 250;
                //_timer.Tick+=new EventHandler(timer_Tick);

                //Initialize the Poller
                _poller = new System.Threading.Timer(new System.Threading.TimerCallback(poller_Tick), null, 0, 100);


                _lrrOptions = _app.GetOptions() as LRROptions;

                //Initialize Tuning
                Symcon.LightRidge.Options.DBOptions.DBInfo dbinfo = _lrrOptions.TuningConnectionObj.GetActiveTuningDBInfo();
                if (dbinfo != null)
                {
                    CurrentTunningDB = dbinfo.DatabaseName;
                    _tuningHistorian.Initialize(dbinfo);
                }

                //Initialize Offline - Added by Nidhig
                Symcon.LightRidge.Options.DBOptions.DBInfo dbOfflineInfo = _lrrOptions.OfflineConnectionObj.GetActiveOfflineDBInfo();
                if (dbOfflineInfo != null)
                    _offLineHistorian.Initialize(dbOfflineInfo);

                if (_tuningHistorian.Initialized)
                {
                    if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                    {
                        _app.LoadedOffLineHistorianAssemblies.Remove(_app.TUNNING);
                    }
                    _app.LoadedOffLineHistorianAssemblies.Add(_app.TUNNING, _tuningHistorian);
                }
                if (_offLineHistorian.Initialized)
                {
                    if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                    {
                        _app.LoadedOffLineHistorianAssemblies.Remove(_app.OFFLINE);
                    }
                    _app.LoadedOffLineHistorianAssemblies.Add(_app.OFFLINE, _offLineHistorian);
                }
                //Initialize Online Adaptors...
                LRROnlineAdaptorConnections onlineHistConnectionInfos = LRRModel.ChildModel.OnlineAdaptorConnections;
                foreach (LRROnlineAdaptorConnection connObj in onlineHistConnectionInfos)
                {
                    LRRHistTagSourceInfo sourceInfo = LRRModel.ChildModel.TagSourceInfos.GetSourceInfoByID(connObj.SourceType);
                    if (sourceInfo != null)
                    {
                        Assembly asm = Assembly.LoadFrom(Application.StartupPath + "\\" + sourceInfo.AssemblyName);
                        if (asm.GetType(sourceInfo.TypeName) != null)
                        {
                            IHistorian iHist = (IHistorian)Activator.CreateInstance(asm.GetType(sourceInfo.TypeName));
                            iHist.Initailize(connObj, Application.StartupPath);
                            iHist.UseSimulator = connObj.UseSimulator;
                            if (!_app.LoadedOnLineHistorianAssemblies.ContainsKey(connObj.ConnectID))
                            {
                                _app.LoadedOnLineHistorianAssemblies.Add(connObj.ConnectID, iHist);
                            }
                            if (iHist.Initialized)
                            {
                                connObj.IsConnected = "TRUE";
                            }
                            else
                            {
                                connObj.IsConnected = "FALSE";
                            }
                        }
                    }
                }
                ret = true;
                return ret;
            }
            catch (Exception ex)
            {
                string msg = ex.Message;
                ret = false;
                Logger.LogMessage(typeof(RunController), "Initialize :" + ex.Message, Logger.Level.Error);
                return ret;
            }
        }

        ThreadRun del;
        public bool Run(string startTime, string endTime)
        {
            InitiatePreExtractionDel preExtractionDel = new InitiatePreExtractionDel(InitiatePreExtraction);
            _app.MainFrame.Invoke(preExtractionDel);

            if (del == null)
            {
                del = new ThreadRun(internalRun);
            }
            //ThreadRun del = new ThreadRun(internalRun);
            IAsyncResult result = del.BeginInvoke(startTime, endTime, new AsyncCallback(RunCallBack), del);
            //result.AsyncWaitHandle.WaitOne();
            return true;
            /*internalRun(startTime,endTime);
            return true;*/
        }

        private string _runSetID = string.Empty;

	

        private void RecordRunActivity( string startTime, string endTime)
        {
            //TODO: 
            //1> check if recording is on  ln 482
            //2> Generate new runid 
            //3> Save runrecord
            //TODO: Apply culture

            //string runsetID = this._app.SaveRunset();

            if (_runControlSet.RecordValues)
            {
                _runID = Guid.NewGuid().ToString();
                LRRRunRecord lrrRunRecord = new LRRRunRecord();

                //string runset = _app.RecordingValuesManagerObj.RunRecordName;

                /*_savedSet.Name = "DataSet " + startTime + " " + endTime;
                _savedSet.StartTime = DateTime.Parse(startTime, Symcon.BusinessObjects.SymConverter.Culture);
                _savedSet.EndTime = DateTime.Parse(endTime, Symcon.BusinessObjects.SymConverter.Culture); ;
                 *
                 * */

                lrrRunRecord.RunName = "RunSet " + startTime + "  " + endTime;
                lrrRunRecord.StartTime = startTime;
                lrrRunRecord.EndTime = endTime;
                
                lrrRunRecord.RunID = _runID;
                //lrrRunRecord.RunName = "RUNSET_" + _runControlSet.DataWindowObj.StartTime.ToString() + "_"+ _runControlSet.DataWindowObj.EndTime.ToString();//sRunForm.name;
                //lrrRunRecord.RunDescription = this.RecordingValuesManagerObj.RunRecordDesc;//sRunForm.desc;
                //lrrRunRecord.StartTime = _runControlSet.DataWindowObj.StartTime.ToString();
                //lrrRunRecord.EndTime = _runControlSet.DataWindowObj.EndTime.ToString();
                lrrRunRecord.UserID = _app.LoggedUser;
                lrrRunRecord.TimeID = DateTime.Now.ToString();
                lrrRunRecord.RunSetID = _runSetID;
                lrrRunRecord.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
                _app.RecordingValuesManagerObj.CommitObject(lrrRunRecord);
            }
        }

        /*public LRRHistorianPoint Run(string histTag,string startTime,string endTime)
        {
            LRRHistorianPoint lhsd = GetActiveHistorian().GetPoint(histTag,startTime,endTime,null,null,"",null,0,null);	
            return lhsd;
        }*/

        public bool Terminate()
        {
            foreach (IHistorian iHist in _app.LoadedOnLineHistorianAssemblies.Values)
            {
                iHist.Logout();
            }
            return true;
        }
        #endregion

        #region Properies

        #region Event
        public event EventHandler CurrentTunningDBChanged;
        #endregion

        #region Delegate

        #endregion

        private string _currentTunningDB;

        public string CurrentTunningDB
        {
            get { return _currentTunningDB; }
            set
            {
                _currentTunningDB = value;
                if (CurrentTunningDBChanged != null)
                {
                    CurrentTunningDBChanged(_currentTunningDB, null);
                }
            }
        }


        public Hashtable HistorianConnections
        {
            get { return _historianConnections; }
        }

        public int ActiveHistorianIndex
        {
            get { return _activeHistorianIndex; }
            set
            {
                _activeHistorianIndex = value;
            }
        }

        public string CurrentDataSetID
        {
            get { return _currentDatasetID; }
            set
            {
                _currentDatasetID = value;
            }
        }

        public LRRHistorianPoints CurrentDataSet
        {
            get { return _currentDataset; }
            set
            {
                _currentDataset = value;
            }
        }

        public string ErroMessage
        {
            get { return _erroMessage; }
            set
            {
                _erroMessage = value;
            }
        }


        public bool InUIThread
        {
            [System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
            get
            {
                return _inUIThread;
            }
            [System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
            set
            {
                _inUIThread = value;
            }
        }

        public bool InPauseState
        {
            get
            {
                return _pauseFlag;
            }

        }
        #endregion

        #region Private Methods

        /// <summary>
        /// This Methods Checks of any of the Run IS Valid with the Current
        /// Settings in the Options and the Run Control Panel.
        /// This method also gives the User a message box with the 
        /// error message.
        /// This Method also Clear the DataSetID's Arraylist 
        /// of the Tuning Historian if the save datasets option is
        /// selected by the user...
        /// </summary>
        /// <returns>false if Failure , true for Success</returns>
        public bool CheckRunSettrings(RunControlSet runControlSet)
        {
            bool ret = false;
            try
            {
                if (runControlSet != null)
                {
                    //Tuning...
                    if (runControlSet.DataSourceType.ToUpper() == LRRHeaderText.GetHeaderString("SID_TUNING").ToUpper())
                    {
                        if (!_tuningHistorian.Initialized)
                        {
                            //Please Configure Tuning Historian First...
                            //LRRMessagBox.ShowMessage("SID_MISSING_TUNING","SID_LABEL",MessageBoxButtons.OK,MessageBoxIcon.Information);
                            LRRMessagBox.ShowMessage(_app.MainFrame, "SID_MISSING_TUNING", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            return false;
                        }
                        else
                        {
                            return true;
                        }
                    }
                    else if (runControlSet.DataSourceType.ToUpper() == LRRHeaderText.GetHeaderString("SID_ONLINE").ToUpper()) //Online...
                    {
                        if (_app.LoadedOnLineHistorianAssemblies.Count == 0)
                        {
                            //if(!_onLineHistorian.Initialized)
                            //{
                            //Please Configure Online Historian First...
                            DialogResult res = LRRMessagBox.ShowMessage("SID_MISSING_ONLINE", "SID_LABEL", MessageBoxButtons.YesNo, MessageBoxIcon.Information);
                            if (res == DialogResult.No)
                            {
                                return false;

                            }
                            else
                            {
                                return true;
                            }

                        }
                        else
                        {
                            if (runControlSet.SaveDataSets) //User checked the flag of Saving the DataSets 
                            {
                                //Check Tuning Connection
                                if (!_tuningHistorian.Initialized)
                                {
                                    //Please Configure Tuning Historian First...
                                    LRRMessagBox.ShowMessage("SID_MISSING_TUNING_FOR_SAVED_SETS", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                    return false;
                                }
                                else
                                {
                                    _tuningHistorian.ExcludeDataSetIDs.Clear();
                                    return true;
                                }
                            }
                            else if (runControlSet.SaveToOfflineHistorian)
                            {
                                //Check Tuning Connection
                                if (!_offLineHistorian.Initialized)
                                {
                                    //Please Configure Tuning Historian First...
                                    LRRMessagBox.ShowMessage("SID_MISSING_TUNING_FOR_SAVED_SETS", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                    return false;
                                }
                                else
                                {
                                    _offLineHistorian.ExcludeDataSetIDs.Clear();
                                    return true;
                                }
                            }
                            else
                            {
                                return true;
                            }
                        }
                    }
                    else if (runControlSet.DataSourceType.ToUpper() == LRRHeaderText.GetHeaderString("SID_OFFLINE").ToUpper()) //OffLine...
                    {
                        if (!_offLineHistorian.Initialized)
                        {
                            //Please Configure Tuning Historian First...
                            LRRMessagBox.ShowMessage("SID_MISSING_OFFLINE", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            return false;
                        }
                        else
                        {
                            return true;
                        }
                    }
                    else
                    {
                        //Invalid Run Control Data Source Type...
                        LRRMessagBox.ShowMessage("SID_INVALID_RUN_CONTROL_DATASOURCE", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return false;
                    }
                }
                else
                {
                    //Not a valid Run Control Set . Display the Message to the user..
                    LRRMessagBox.ShowMessage("SID_INVALID_RUN_CONTROL_SET", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return false;
            }
        }

        private void internalRun(string startTime, string endTime)
        {
            //			_getPointsInProgress = true; //Call the get points in the loop so set the flag...

            //Populate from Model
            LRRHistDatas histDatas = LRRModel.HistDatas;

            //LRRHistDataUserAndTimes histDataUserAndTimes = LRRModel.HistDataUserAndTimes;

            
            currentStartTime = startTime;
            currentEndTime = endTime;


            int cnt = histDatas.Count;

            if (RunStarted != null)
            {
                RunStartedEventArgs e = new RunStartedEventArgs();
                e.StartTime = startTime;
                e.EndTime = endTime;
                e.TotalPoints = cnt;
                e.DatabaseType = _runControlSet.DataSourceType;
                e.DataWindowSize = _runControlSet.DataWindowObj.DataWindowSize;
                e.DataWindowUnits = _runControlSet.DataWindowObj.DataWindowUnits;

                //InitiatePreCalc();
                object[] args = new object[1];
                args[0] = e;
                _app.MainFrame.BeginInvoke(new RunStartedDel(RunStarted), args);
                //RunStarted(e);
                this._app.SetPropertyHistorianBinding();
            }

            //IHistorian activeHistorian = GetActiveHistorian();

            if (_runControlSet.RecordValues)
            {
                RecordRunActivity(startTime,endTime);
            }


            Symcon.LightRidge.HistorianConnect.TuningHistorian tuningHist = null;
            if (_runControlSet.SaveDataSets)
            {
                if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                {
                    tuningHist = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING] as TuningHistorian;
                    //Save the dataSet
                    if ((_savedSet = tuningHist.GetSavedSetFromName(startTime, endTime, out _arrOldHistTags)) == null)
                    {
                        _savedSet = new LRRHistSavedSet();
                        _savedSet.ID = Guid.NewGuid().ToString();
                        _savedSet.Name = "DataSet " + startTime + " " + endTime;
                        _savedSet.StartTime = DateTime.Parse(startTime, Symcon.BusinessObjects.SymConverter.Culture);
                        _savedSet.EndTime = DateTime.Parse(endTime, Symcon.BusinessObjects.SymConverter.Culture); ;
                        _savedSet.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
                    }
                    else
                    {
                        _isSetAlreadyPresent = true;
                    }
                    _savedPoints = new LRRHistorianPoints();
                }
                //--End--
            }
            Symcon.LightRidge.HistorianConnect.OffLineHistorian offlineHist = null;
            if (_runControlSet.SaveToOfflineHistorian)
            {
                if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                {
                    offlineHist = _app.LoadedOffLineHistorianAssemblies[_app.OFFLINE] as OffLineHistorian;
                    //Save the dataSet
                    if ((_savedSetForOffline = offlineHist.GetSavedSetFromName(startTime, endTime, out _arrOldHistTagsForOffline)) == null)
                    {
                        _savedSetForOffline = new LRRHistSavedSet();
                        _savedSetForOffline.ID = Guid.NewGuid().ToString();
                        _savedSetForOffline.Name = "DataSet " + startTime + " " + endTime;
                        _savedSetForOffline.StartTime = DateTime.Parse(startTime, Symcon.BusinessObjects.SymConverter.Culture);
                        _savedSetForOffline.EndTime = DateTime.Parse(endTime, Symcon.BusinessObjects.SymConverter.Culture); ;
                        _savedSetForOffline.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
                    }
                    else
                    {
                        _isSetAlreadyPresentForOffline = true;
                    }
                    _savedPointsForOffline = new LRRHistorianPoints();
                }
                //--End--
            }

            //Get the Manual Entry Historian DataSet points

            LRRManualEntryHistorianDataBase manualEntryHistDB = this._app.GetOptions().ManualEntryHistorianDataBaseObj;
            LRRManualEntryHistorianDataSet manualEntryHistDataSet = null;
            LRRManualEntryHistorianDataSetDatas manualEntryHistDataSetDatas = null;
            //Symcon.LightRidge.HistorianConnect.OffLineHistorian offlineHist = null;
            if (manualEntryHistDB != null && manualEntryHistDB.Type == LRRHeaderText.GetHeaderString("SID_TUNING"))
            {
                if (tuningHist == null)
                {
                    if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                    {
                        tuningHist = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING] as TuningHistorian;
                    }
                }
                if (tuningHist.Initialized)
                {
                    manualEntryHistDataSet = tuningHist.GetManualEntryHistDataSet(startTime, endTime);
                    if (manualEntryHistDataSet != null && manualEntryHistDataSet.EffectiveTime.CompareTo(new DateTime(1, 1, 1)) != 0)
                    {
                        manualEntryHistDataSetDatas = tuningHist.GetManualEntryHistDataSetDatas(manualEntryHistDataSet.ID);
                    }
                }
                else
                {
                    manualEntryHistDataSetDatas = null;
                }
            }
            else if (manualEntryHistDB != null && manualEntryHistDB.Type == LRRHeaderText.GetHeaderString("SID_OFFLINE"))
            {
                if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                {
                    offlineHist = _app.LoadedOffLineHistorianAssemblies[_app.OFFLINE] as OffLineHistorian;
                }
                if (offlineHist != null)
                {
                    if (offlineHist.Initialized)
                    {

                        manualEntryHistDataSet = offlineHist.GetManualEntryHistDataSet(startTime, endTime);
                        if (manualEntryHistDataSet != null && manualEntryHistDataSet.EffectiveTime.CompareTo(new DateTime(1, 1, 1)) != 0)
                        {
                            manualEntryHistDataSetDatas = offlineHist.GetManualEntryHistDataSetDatas(manualEntryHistDataSet.ID);
                        }

                    }
                    else
                    {
                        manualEntryHistDataSetDatas = null;
                    }
                }
            }
            Dictionary<string, IHistorian> loadedAssemblies = _app.LoadedOnLineHistorianAssemblies;
            bool isFirstTimeRun = true;
            int tempCounter = 0;
            Hashtable histPoints = new Hashtable();
            LRRHistDatas ovationPoints = new LRRHistDatas();
            string ovatConnectID = "";
            string convertedStartTime = string.Empty;
            string convertedEndTime = string.Empty;
            Dictionary<string, List<string>> convertedTimes = new Dictionary<string, List<string>>();
            foreach (LRRHistData lhd in histDatas)
            {
                LRRHistorianPoint lhp = null;
                tempCounter = tempCounter + 1;

                if (lhd.SourceType != null && lhd.SourceType.Trim() != "" && _runControlSet.DataSourceType != "Offline" && _runControlSet.DataSourceType != "PE-Advisor Online Plant Data")
                {
                    LRROnlineAdaptorConnection onlineAdaptor = LRRModel.ChildModel.OnlineAdaptorConnections.GetOnlineAdaptorByID(lhd.SourceType);
                    if (onlineAdaptor != null)
                    {
                        if (!convertedTimes.ContainsKey(onlineAdaptor.ConnectID))
                        {
                            // commented by ashwin
                            if (onlineAdaptor.HistorianTimeZone != onlineAdaptor.PlantTimeZone)
                            {
                                convertedStartTime = DateUtils.ConvertTime(startTime, onlineAdaptor.HistorianTimeZone).ToString("MM/dd/yyyy HH:mm:ss");
                                convertedEndTime = DateUtils.ConvertTime(endTime, onlineAdaptor.HistorianTimeZone).ToString("MM/dd/yyyy HH:mm:ss");
                            }
                            else
                            {
                                convertedStartTime = startTime;
                                convertedEndTime = endTime;
                            }

                             
                            /*convertedStartTime = startTime.ToString();
                            convertedEndTime = endTime.ToString();
                             * */
                            List<string> times = new List<string>();
                            times.Add(convertedStartTime);
                            times.Add(convertedEndTime);
                            convertedTimes.Add(onlineAdaptor.ConnectID, times);
                            //startTime = convertedStartTime;
                            //endTime = convertedEndTime;
                        }
                        else
                        {
                            convertedStartTime = convertedTimes[onlineAdaptor.ConnectID][0];
                            convertedEndTime = convertedTimes[onlineAdaptor.ConnectID][1];

                            //startTime = convertedTimes[onlineAdaptor.ConnectID][0];
                            //endTime = convertedTimes[onlineAdaptor.ConnectID][1];
                        }

                        LRRHistTagSourceInfo sourceInfo = LRRModel.ChildModel.TagSourceInfos.GetSourceInfoByID(onlineAdaptor.SourceType);
                        if (sourceInfo != null)
                        {
                            if (loadedAssemblies.ContainsKey(onlineAdaptor.ConnectID))
                            {
                                IHistorian iHist = loadedAssemblies[onlineAdaptor.ConnectID];
                                if (isFirstTimeRun && (sourceInfo.AssemblyName == "Symcon.LightRidge.SquareDAdapter.dll"))
                                {
                                    onlineAdaptor.IsFirstTimeRun = isFirstTimeRun;
                                    iHist.IsFirstRunCycle = onlineAdaptor.IsFirstTimeRun;
                                    isFirstTimeRun = false;
                                    onlineAdaptor.IsFirstTimeRun = isFirstTimeRun;
                                }
                                else if (sourceInfo.AssemblyName == "Symcon.LightRidge.SquareDAdapter.dll")
                                {
                                    iHist.IsFirstRunCycle = onlineAdaptor.IsFirstTimeRun;
                                }
                                else if (sourceInfo.AssemblyName == "Symcon.LightRidge.OvationAdaptor.dll")
                                {
                                    ovationPoints.Add(lhd);
                                    ovatConnectID = onlineAdaptor.ConnectID;
                                    continue;
                                }
                                if (iHist.Initialized && !(sourceInfo.AssemblyName == "Symcon.LightRidge.OvationAdaptor.dll"))
                                {
                                    //lhp = iHist.GetPoint(lhd.HistTag, startTime, endTime, lhd.ServerID, lhd.MeasType, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                                    try
                                    {
                                        lhp = iHist.GetPoint(lhd.HistTag, convertedStartTime, convertedEndTime, lhd.ServerID, lhd.MeasType, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                                    }
                                    catch (Exception ex)
                                    {
                                        Symcon.Utils.MessageLogger.Logger.LogMessage(ex.Message, Symcon.Utils.MessageLogger.Logger.Level.Debug);
                                    }

                                }
                            }
                            //Should never go to this else part.
                            else
                            {
                                Assembly asm = Assembly.LoadFrom(Application.StartupPath + "\\" + sourceInfo.AssemblyName);
                                if (asm.GetType(sourceInfo.TypeName) != null)
                                {
                                    IHistorian iHist = (IHistorian)Activator.CreateInstance(asm.GetType(sourceInfo.TypeName));
                                    iHist.Initailize(onlineAdaptor, Application.StartupPath);
                                    if (iHist.Initialized)
                                    {
                                        if (!_app.LoadedOnLineHistorianAssemblies.ContainsKey(onlineAdaptor.ConnectID))
                                        {
                                            _app.LoadedOnLineHistorianAssemblies.Add(onlineAdaptor.ConnectID, iHist);
                                        }
                                        else
                                        {
                                            _app.LoadedOnLineHistorianAssemblies[onlineAdaptor.ConnectID] = iHist;
                                        }
                                        onlineAdaptor.IsConnected = "TRUE";
                                        iHist.IsFirstRunCycle = onlineAdaptor.IsFirstTimeRun;
                                        if (sourceInfo.AssemblyName == "Symcon.LightRidge.OvationAdaptor.dll")
                                        {
                                            ovationPoints.Add(lhd);
                                            continue;
                                        }
                                        else
                                        {
                                            //lhp = iHist.GetPoint(lhd.HistTag, startTime, endTime, lhd.ServerID, lhd.MeasType, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                                            try
                                            {
                                                lhp = iHist.GetPoint(lhd.HistTag, convertedStartTime, convertedEndTime, lhd.ServerID, lhd.MeasType, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                                            }
                                            catch (Exception ex)
                                            {
                                                Symcon.Utils.MessageLogger.Logger.LogMessage(ex.Message, Symcon.Utils.MessageLogger.Logger.Level.Debug);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                else
                {
                    if (_runControlSet.DataSourceType == "Offline")
                    {
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                        {

                            IHistorian iHistTunningObj = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING];
                            lhp = iHistTunningObj.GetPoint(lhd.HistTag, startTime, endTime, lhd.ServerID, lhd.MeasType, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                        }
                    }
                    else if (_runControlSet.DataSourceType == "PE-Advisor Online Plant Data")
                    {
                        if (offlineHist != null)
                        {
                            lhp = offlineHist.GetPoint(lhd.HistTag, startTime, endTime, lhd.ServerID, lhd.MeasType, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                        }
                    }
                }

                //Check for Derived Point
                //Derived Point is a point for which the formula Exists And the formula is not self referential.
                //If the point exists on the server then it cannot be a derived point.
                //If the point does not exists on the server and if it exists in the Manual Entry DB 
                //then it is not a derived point.

                bool bDerived = false;
                bDerived = CheckForDerivedPoint(lhd);
                if (lhp == null)
                {
                    //Check if the point exists in Manual Entry
                    bool exists = false;
                    lhp = new LRRHistorianPoint();
                    if (_app.GetOptions().UseSimulator)
                    {
                        Random random = new Random();
                        lhp.StartOfPeriodValue = random.Next(100).ToString();
                        lhp.EndOfPeriodValue = random.Next(100).ToString();
                        lhp.AverageValueForPeriodEntered = random.Next(100).ToString();
                    }
                    if (manualEntryHistDataSetDatas != null && manualEntryHistDataSetDatas.Count > 0)
                    {
                        LRRManualEntryHistorianDataSetData manualEntryHistPoint = manualEntryHistDataSetDatas.GetManualEntryHistPointFromHistTag(lhd.HistTag);
                        if (manualEntryHistPoint != null)
                        {
                            lhp.StartOfPeriodValue = manualEntryHistPoint.InterfaceValue;
                            lhp.EndOfPeriodValue = manualEntryHistPoint.InterfaceValue;
                            lhp.AverageValueForPeriodEntered = manualEntryHistPoint.InterfaceValue;
                            lhp.SpotValue = manualEntryHistPoint.InterfaceValue;
                            exists = true;
                        }
                    }
                    //If the point does not exists in the Manual Entry
                    if (!exists)
                    {
                        if (bDerived)
                        {
                            lhp.StartOfPeriodValue = LRRHeaderText.GetHeaderString("SID_DERIVE_POINT");
                            lhp.EndOfPeriodValue = LRRHeaderText.GetHeaderString("SID_DERIVE_POINT");
                            if (_runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_ONLINE"))
                            {
                                lhp.AverageValueForPeriodEntered = LRRHeaderText.GetHeaderString("SID_DERIVE_POINT");
                                lhp.SpotValue = LRRHeaderText.GetHeaderString("SID_DERIVE_POINT");
                            }
                        }
                        //Check for the datasource type
                        if (_runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_TUNING") && !bDerived)
                        {
                            lhp.StartOfPeriodValue = LRRHeaderText.GetHeaderString("SID_TUNING_TAG_NOT_FOUND");//"Could not find the tag in Tuning Historian.";
                            lhp.EndOfPeriodValue = LRRHeaderText.GetHeaderString("SID_TUNING_TAG_NOT_FOUND");//"Could not find the tag in Tuning Historian.";
                            lhp.SpotValue = LRRHeaderText.GetHeaderString("SID_TUNING_TAG_NOT_FOUND");//"Could not find the tag in Tuning Historian.";
                            lhp.AverageValueForPeriodEntered = LRRHeaderText.GetHeaderString("SID_TUNING_TAG_NOT_FOUND");//"Could not find the tag in Tuning Historian.";
                        }
                        if (_runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_OFFLINE") && !bDerived)
                        {
                            lhp.StartOfPeriodValue = LRRHeaderText.GetHeaderString("SID_OFFLINE_TAG_NOT_FOUND");//"Could not find the tag in Offline Historian.";
                            lhp.EndOfPeriodValue = LRRHeaderText.GetHeaderString("SID_OFFLINE_TAG_NOT_FOUND");//"Could not find the tag in Offline Historian.";
                            lhp.SpotValue = LRRHeaderText.GetHeaderString("SID_OFFLINE_TAG_NOT_FOUND");//"Could not find the tag in Offline Historian.";
                            lhp.AverageValueForPeriodEntered = LRRHeaderText.GetHeaderString("SID_OFFLINE_TAG_NOT_FOUND");//"Could not find the tag in Offline Historian.";
                        }
                        if (_runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_ONLINE") && !bDerived)
                        {
                            lhp.StartOfPeriodValue = LRRHeaderText.GetHeaderString("SID_ONLINE_TAG_NOT_FOUND");//"Could not find the tag in Offline Historian.";
                            lhp.EndOfPeriodValue = LRRHeaderText.GetHeaderString("SID_ONLINE_TAG_NOT_FOUND");//"Could not find the tag in Offline Historian.";
                            lhp.SpotValue = LRRHeaderText.GetHeaderString("SID_ONLINE_TAG_NOT_FOUND");//"Could not find the tag in Offline Historian.";
                            lhp.AverageValueForPeriodEntered = LRRHeaderText.GetHeaderString("SID_ONLINE_TAG_NOT_FOUND");//"Could not find the tag in Offline Historian.";
                        }
                    }
                }
                else
                {
                    //Null Checks for the 4 values...
                    if (lhp.SpotValue == null)
                    {
                        lhp.SpotValue = "";
                    }
                    if (lhp.EndOfPeriodValue == null)
                    {
                        lhp.EndOfPeriodValue = "";
                    }
                    if (lhp.AverageValueForPeriodEntered == null)
                    {
                        lhp.AverageValueForPeriodEntered = "";
                    }

                    if (lhp.StartOfPeriodValue == null)
                    {
                        lhp.StartOfPeriodValue = "";
                    }
                }
                if (lhd.MeasType != "N")
                {
                    if (convertedStartTime != "" && convertedStartTime != null)
                    {
                        lhp.StartTime = convertedStartTime;
                    }
                    else
                    {
                        lhp.StartTime = startTime;
                    }

                    if (convertedEndTime != "" && convertedEndTime != null)
                    {
                        lhp.EndTime = convertedEndTime;
                    }
                    else
                    {
                        lhp.EndTime = endTime;
                    }
                    //lhp.StartTime = startTime;
                    //lhp.EndTime = endTime;
                }
                else
                {
                    if (lhp.StartTime == null)
                    {
                        if (convertedStartTime != null && convertedStartTime != "")
                        {
                            lhp.StartTime = convertedStartTime;
                        }
                        else
                        {
                            lhp.StartTime = startTime;
                        }
                        //lhp.StartTime = startTime;
                    }
                    if (lhp.EndTime == null)
                    {
                        if (convertedEndTime != null && convertedEndTime != "")
                        {
                            lhp.EndTime = convertedEndTime;
                        }
                        else
                        {
                            lhp.EndTime = endTime;
                        }
                        //lhp.EndTime = endTime;
                    }
                }
                RunQueue runQueue = new RunQueue();
                runQueue.histData = lhd;
                runQueue.histPoint = lhp;

                _runQueue.Enqueue(runQueue);

                this.IternalRaiseReceivedPoint(lhd, lhp.StartTime, lhp.EndTime);
                //System.Threading.Thread.Sleep(1);
            }

            LRRHistorianPoints lhps = new LRRHistorianPoints();

            if (_runControlSet.DataSourceType != "Offline" && _runControlSet.DataSourceType != "PE-Advisor Online Plant Data")
            {
                if (ovatConnectID != null && ovatConnectID != "")
                {
                    IHistorian iOvtHist = loadedAssemblies[ovatConnectID];
                    tempCounter = 1;
                    string queryString = "";
                    //startTime = convertedTimes[ovatConnectID][0];
                    convertedStartTime = convertedTimes[ovatConnectID][0];
                    //endTime = convertedTimes[ovatConnectID][1];
                    convertedEndTime = convertedTimes[ovatConnectID][1];
                    foreach (LRRHistData lhd in ovationPoints)
                    {
                        if (tempCounter < 330)
                        {
                            if (tempCounter == 1)
                            {
                                tempCounter = tempCounter + 1;
                                queryString = "'" + lhd.HistTag + "',";
                            }
                            else
                            {
                                tempCounter = tempCounter + 1;
                                queryString = queryString + "'" + lhd.HistTag + "',";
                            }
                        }
                        else if (tempCounter == 330)
                        {
                            tempCounter = 1;
                            queryString = queryString.Substring(0, (queryString.Length - 1));
                            if (iOvtHist.Initialized)
                            {
                                //lhps = iOvtHist.GetPoints(queryString, startTime, endTime, null, null, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                                
                                try
                                {
                                    lhps = iOvtHist.GetPoints(queryString, convertedStartTime, convertedEndTime, null, null, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                                }
                                catch (Exception ex)
                                {
                                    Symcon.Utils.MessageLogger.Logger.LogMessage(ex.Message, Symcon.Utils.MessageLogger.Logger.Level.Debug);
                                }
                                
                                if (lhps != null)
                                {
                                    foreach (LRRHistorianPoint lhp in lhps)
                                    {
                                        LRRHistData ovlhd = LRRModel.HistDatas.GetHistDataFromHistTag(lhp.HistTag);
                                        if (ovlhd != null)
                                        {
                                            if (lhp.StartTime == null)
                                            {
                                                if (convertedStartTime != null && convertedStartTime != "")
                                                {
                                                    lhp.StartTime = convertedStartTime;
                                                }
                                                else
                                                {
                                                    lhp.StartTime = startTime;
                                                }
                                                //lhp.StartTime = startTime;
                                            }
                                            if (lhp.EndTime == null)
                                            {
                                                if (convertedEndTime != null && convertedEndTime != "")
                                                {
                                                    lhp.EndTime = convertedEndTime;
                                                }
                                                else
                                                {
                                                    lhp.EndTime = endTime;
                                                }
                                                //lhp.EndTime = endTime;
                                            }
                                            RunQueue runQueue = new RunQueue();
                                            runQueue.histData = ovlhd;
                                            runQueue.histPoint = lhp;

                                            _runQueue.Enqueue(runQueue);

                                            this.IternalRaiseReceivedPoint(ovlhd, lhp.StartTime, lhp.EndTime);
                                        }
                                    }
                                }
                            }
                        }

                    }

                    if (tempCounter > 1)
                    {
                        tempCounter = 1;
                        queryString = queryString.Substring(0, (queryString.Length - 1));

                        //lhps = iOvtHist.GetPoints(queryString, startTime, endTime, null, null, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                        if ((convertedStartTime != null && convertedStartTime != "") && (convertedEndTime != null && convertedEndTime != ""))
                        {
                            lhps = iOvtHist.GetPoints(queryString, convertedStartTime, convertedEndTime, null, null, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                        }
                        else
                        {
                            lhps = iOvtHist.GetPoints(queryString, startTime, endTime, null, null, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                        }
                        //lhps = iOvtHist.GetPoints(queryString, convertedStartTime, convertedEndTime, null, null, CurrentDataSetID, _currentRunMsg.RunType.ToString(), _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
                        if (lhps != null)
                        {
                            foreach (LRRHistorianPoint lhp in lhps)
                            {
                                LRRHistData ovlhd = LRRModel.HistDatas.GetHistDataFromHistTag(lhp.HistTag);
                                if (ovlhd != null)
                                {
                                    if (lhp.StartTime == null)
                                    {
                                        if (convertedStartTime != null && convertedStartTime != "")
                                        {
                                            lhp.StartTime = convertedStartTime;
                                        }
                                        else
                                        {
                                            lhp.StartTime = startTime;
                                        }
                                        //lhp.StartTime = startTime;
                                    }
                                    if (lhp.EndTime == null)
                                    {
                                        if (convertedEndTime != null && convertedEndTime != "")
                                        {
                                            lhp.EndTime = convertedEndTime;
                                        }
                                        else
                                        {
                                            lhp.EndTime = endTime;
                                        }
                                        //lhp.EndTime = endTime;
                                    }
                                    RunQueue runQueue = new RunQueue();
                                    runQueue.histData = ovlhd;
                                    runQueue.histPoint = lhp;

                                    _runQueue.Enqueue(runQueue);

                                    this.IternalRaiseReceivedPoint(ovlhd, lhp.StartTime, lhp.EndTime);
                                }
                            }
                        }
                    }
                }
            }



            foreach (IHistorian hist in _app.LoadedOnLineHistorianAssemblies.Values)
            {
                hist.CloseConnection();
            }
            if (ovationPoints != null)
            {
                ovationPoints.Clear();
                ovationPoints = null;
            }
            if (lhps != null)
            {
                lhps.Clear();
                lhps = null;
            }
            GC.Collect();
            //_getPointsInProgress = fsalse;
        }

        /// <summary>
        /// This method checks whether the historian Tag is a derived point
        /// A point is a derived point if the formula exists and formula
        /// is not self referential.
        /// </summary>
        /// <param name="lhd"></param>
        /// <returns></returns>
        private bool CheckForDerivedPoint(LRRHistData lhd)
        {
            bool bDerived = false;
            if (lhd.FormulaID != null && lhd.FormulaID != "")// || (lhd.FormulaObj != null && lhd.FormulaObj.ID != null && lhd.FormulaObj.ID != ""))
            {
                //Get the Formula Params
                string formulaID = lhd.FormulaID;
                LRRFormula lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(formulaID);
                if (lrrFormula != null)
                {
                    bDerived = true;
                    LRRFormulaParams lrrFormulaParams = lrrFormula.Params;
                    IEnumerator iEnum = lrrFormulaParams.GetEnumerator();
                    while (iEnum.MoveNext())
                    {
                        LRRFormulaParam lrrFormulaParam = iEnum.Current as LRRFormulaParam;
                        if (lrrFormulaParam.SourceOBJID == lhd.HistTagUID)
                        {
                            bDerived = false;
                            break;
                        }
                    }
                }
            }
            else
                bDerived = false;
            return bDerived;
        }

        private void InvokeDelegate(Delegate sink, object[] args)
        {
            try
            {
                sink.DynamicInvoke(args);
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(RunController), "InvokeDelegate :" + ex.Message, Logger.Level.Debug);
            }
        }

        private void RunCallBack(IAsyncResult arResult)
        {
            ThreadRun ar = (ThreadRun)arResult.AsyncState;
            ar.EndInvoke(arResult);
            //if(_savedSet != null)
            //{
            //    try
            //    {
            //        if(_runControlSet.SaveDataSets)
            //        {
            //            Symcon.LightRidge.HistorianConnect.IHistorian tuningHist =  this.GetActiveHistorian(HistorianConnect.HistorianConnectionType.TUNING);
            //            tuningHist.SetPoints(_savedSet,_savedPoints,_onLineHistorian);
            //        }
            //    }
            //    catch(Exception ex)
            //    {
            //        Logger.LogMessage(typeof(RunController),"RunCallBack :"+ex.Message  ,Logger.Level.Debug);
            //    }
            //}
            TimerProcessDel timerDel = new TimerProcessDel(timer_Tick);
            object[] args1 = new object[2];
            args1[0] = null;
            args1[1] = null;

            _app.MainFrame.BeginInvoke(timerDel, args1);


            //			bool bExit = false;
            //
            //			while(_processingPoint)
            //			{
            //				System.Threading.Thread.Sleep(1);
            //				Application.DoEvents();
            //
            //			}
            //			_getPointsInProgress = false;
            //			if(_pollerQueue.Count > 0)
            //			{
            //				RunMessage prevRunMsg = _pollerQueue.Peek() as RunMessage;
            //				RunMessage currentRunMsg = CreateRunMsg(prevRunMsg.RunType);
            //				if((currentRunMsg.RunType != RunTypeEnum.SINGLE_RUN_ENTERED_WINDOW) && (currentRunMsg.RunType != RunTypeEnum.SINGLE_RUN_STEP_NEXT) && (currentRunMsg.RunType != RunTypeEnum.SINGLE_RUN_ADVANCE_NOW))
            //				{
            //					AssignStartTimeEndTime(currentRunMsg,prevRunMsg);
            //					if((currentRunMsg.RunCondition == (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITH_INCREMENT))
            //						|| (currentRunMsg.RunCondition == (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT)))
            //
            //					{
            //						DateTime dtStartTime = (DateTime)Symcon.BusinessObjects.SymConverter.Convert(typeof(string),currentRunMsg.StartTime,typeof(DateTime));
            //						DateTime dtStartTime1 = dtStartTime.Add(_runControlSet.GetTimeIncrementSpan() + _runControlSet.GetDataWindowSpan());
            //						TimeSpan tsForecast = dtStartTime1.Subtract(_runControlSet.AutoRunsObj.EndTime);
            //						if(tsForecast.Seconds > 0)
            //						{
            //							_goodToComplete = true;
            //						}
            //						
            //						TimeSpan ts = dtStartTime.Subtract(_runControlSet.AutoRunsObj.EndTime);
            //						if(ts.Seconds > 0)
            //						{
            //							_pollerQueue.Clear();
            //							bExit = true;
            //						}
            //					}
            //					//Check for Tuning Database conditions
            //					if((currentRunMsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITH_SEQUENTIAL_DATASETS)
            //						||(currentRunMsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITH_RANDOM_DATASETS))
            //					{
            //						if((_tuningRunCounter + 1) == _lrrSavedSets.Count)
            //						{
            //							_goodToComplete = true;
            //						}
            //
            //						if(_tuningRunCounter == _lrrSavedSets.Count)
            //						{
            //							_pollerQueue.Clear();
            //							
            //							bExit = true;
            //						}
            //					}
            //					_pollerQueue.Dequeue();
            //
            //					if(!bExit)
            //					{
            //						if(_pauseFlag)
            //							currentRunMsg.MsgState = MessageStateEnum.PAUSED;
            //						_pollerQueue.Enqueue(currentRunMsg);
            //}
            //				}
            //				else
            //				{
            //					/*if(_runStarted)
            //					{
            //						_goodToComplete = true;
            //					}  */
            //				}
            //			}
            //			else
            //			{
            //				/*if(_runStarted)
            //				{
            //					_goodToComplete = true;
            //				}  */
            //				
            //			}
        }

        /// <summary>
        /// Method executes the Historian formulas of type Pre
        /// Formulas which carry the SourceCollectionName = typeof(LRRHistData)
        /// will be the ones which will be executed by this method.
        /// This method should be called at the begining of the 
        /// extration of the historian points from the Active Historian 
        /// The Active Historian can be one of the following:- Online,Offline,Tuning
        /// </summary>
        public void InitiatePreExtraction()
        {
            try
            {
                Logger.LogMessage(typeof(RunController), "InitiatePreExtraction Called", Logger.Level.Debug);

                LRRFormulas lrrFormulas = LRRModel.FormulasObj.FilterFormulasByPreExtraction();

                //foreach(LRRFormula lrrFormula in lrrFormulas) Modified by NidhiG to sort formula collection before execution
                lrrFormulas.SortColumn = "SortIndex";
                lrrFormulas.Sort();

                lrrFormulas.SortColumn = "Priority";
                lrrFormulas.SortingOrder = Symcon.BusinessObjects.GenericComparer.SortOrder.Ascending;
                lrrFormulas.Sort();

                int nTotalFormula = lrrFormulas.Count;
                for (int formulaCtr = 0; formulaCtr < nTotalFormula; formulaCtr++)
                {
                    LRRFormula lrrFormula = lrrFormulas[formulaCtr];

                    if (lrrFormula.SourceCollectionName.Trim() != typeof(LRRHistData).ToString())
                        continue;

                    LRRFormulaParams lrrFormulaParams = lrrFormula.Params;
                    string formula = lrrFormula.Expression;
                    if (lrrFormula.IsFormula && lrrFormula.PreCalc && lrrFormula.SourceCollectionName.Trim() == typeof(LRRHistData).ToString())
                    {
                        string val = "";
                        if (formula != null && formula != "" && lrrFormulaParams != null)
                        {
                            val = Calculate(lrrFormulaParams, formula);
                            if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRHistData).ToString())
                            {
                                //Populate from Model
                                try
                                {
                                    LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
                                    LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
                                    if (lrrhistData != null)
                                    {
                                        //Convert to Model UOM
                                        string uomID = lrrhistData.ModelUOM;
                                        string uiUOMId = lrrhistData.InterfaceUnits;
                                        lrrhistData.InterfaceValue = ConvertUOM(uiUOMId, uomID, val);
                                    }
                                }
                                catch
                                {
                                    Symcon.Utils.MessageLogger.Logger.LogMessage("Error Executing Function for " + formula, Symcon.Utils.MessageLogger.Logger.Level.Debug);
                                }

                            }
                        }
                    }
                    else if (!lrrFormula.IsFormula && !lrrFormula.IsCustomFormula && lrrFormula.PreCalc && lrrFormula.SourceCollectionName.Trim() == typeof(LRRHistData).ToString())
                    {
                        ExceuteScript(lrrFormulaParams, formula, lrrFormula);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(RunController), "InitiatePreExtraction :" + ex.Message, Logger.Level.Debug);
            }
            finally
            {
                GC.Collect();
            }
        }

        /// <summary>
        /// Method executes the Property Value formulas of type Pre
        /// Formulas which carry the SourceCollectionName != typeof(LRRHistData)
        /// will be the ones which will be executed by this method.
        /// This method should be called at the end of the run and before the 
        /// read/write execution engine cycle
        /// </summary>
        public void InitiatePreCalc()
        {
            try
            {
                Logger.LogMessage(typeof(RunController), "InitiatePreCalc Called", Logger.Level.Debug);
                

                LRRFormulas lrrFormulas = LRRModel.FormulasObj.FilterFormulasByPreCalc();

                //System.Threading.Thread.Sleep(5000);

                //foreach(LRRFormula lrrFormula in lrrFormulas) Modified by NidhiG to sort formula collection before execution
                lrrFormulas.SortColumn = "SortIndex";
                lrrFormulas.Sort();

                lrrFormulas.SortColumn = "Priority";
                lrrFormulas.SortingOrder = Symcon.BusinessObjects.GenericComparer.SortOrder.Ascending;
                lrrFormulas.Sort();

                int nTotalFormula = lrrFormulas.Count;
                for (int formulaCtr = 0; formulaCtr < nTotalFormula; formulaCtr++)
                {
                    LRRFormula lrrFormula = lrrFormulas[formulaCtr];
                    if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRHistData).ToString())
                        continue;
                    LRRFormulaParams lrrFormulaParams = lrrFormula.Params;
                    string formula = lrrFormula.Expression;
                    LRRSiteComponentProperty siteCompPropObj = null;
                    if (lrrFormula.IsFormula)
                    {
                        string val = "";

                        if (formula != null && formula != "" && lrrFormulaParams != null)
                        {
                            try
                            {
                                val = Calculate(lrrFormulaParams, formula);

                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteComponentPropertyValue).ToString())
                                {
                                    LRRSiteComponentPropertyValue lrrSiteCompPropValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);

                                    if (lrrSiteCompPropValue == null)
                                        continue;

                                    //Convert to Model UOM
                                    string siteCompPropID = lrrSiteCompPropValue.SiteComponentPropertyID;
                                    siteCompPropObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
                                    if (siteCompPropObj != null)
                                    {
                                        string uiUOMID = siteCompPropObj.GetUIUOMID();
                                        val = ConvertUOM(uiUOMID, lrrSiteCompPropValue.GetUOMID(), val);
                                    }

                                    if (!lrrSiteCompPropValue.IsOverrided)
                                    {
                                        lrrSiteCompPropValue.PropertyValue = val;
                                    }
                                    else
                                    {
                                        lrrSiteCompPropValue.OverridedValue = val;
                                    }
                                }
                                //Aggregates
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRAggregatePropertyValue).ToString())
                                {
                                    //Populate from Model
                                    if (LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID) != null)
                                        LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID).PropertyValue = val;
                                }

                                //Added by Nidhi For Grouped Properties - Chemical components and Data Arrays
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString())
                                {
                                    LRRSiteGroupPropertyQualifierValue lrrSiteGroupQualifierValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);

                                    if (lrrSiteGroupQualifierValue == null)
                                        continue;

                                    //Convert to Model UOM
                                    string lkupQualID = lrrSiteGroupQualifierValue.LookUpQualifierID;
                                    LRRLookUpQualifier lkupQualObj = null;
                                    if (lkupQualID != null && lkupQualID.Trim() != "")
                                    {
                                        lkupQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lkupQualID);
                                    }
                                    //Group Property
                                    if (lkupQualObj != null)
                                    {
                                        string uomTypeID = lkupQualObj.UOMTypeID;
                                        //Get the UI UOM
                                        LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                        string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, uomTypeID);

                                        val = ConvertUOM(uiUOMID, lrrSiteGroupQualifierValue.GetUOMID(), val);
                                    }
                                    else //Data Arrays
                                    {
                                        LRRSiteComponentGroupProperty objGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrSiteGroupQualifierValue.GroupPropertyID);
                                        if (objGrpProp != null)
                                        {
                                            string propType = objGrpProp.PropertyID;
                                            LRRPropertyType propTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(propType);
                                            if (propTypeObj != null)
                                            {
                                                //Get the UI UOM
                                                LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                                string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, propTypeObj.UOMTypeID);
                                                val = ConvertUOM(uiUOMID, lrrSiteGroupQualifierValue.GetUOMID(), val);
                                            }
                                        }
                                    }

                                    if (!lrrSiteGroupQualifierValue.IsOverrided)
                                    {
                                        lrrSiteGroupQualifierValue.PropertyValue = val;
                                    }
                                    else
                                    {
                                        lrrSiteGroupQualifierValue.OverridedValue = val;
                                    }
                                }

                            }
                            catch (Exception ex)
                            {
                                Logger.LogMessage(ex.Message + " : " + lrrFormula.ID, Logger.Level.Info);
                                //   Logger.LogMessage(typeof(RunController), "InitiatePreCalc :" + ex.Message, Logger.Level.Debug);
                            }
                        }

                    }
                    else
                    {
                        if (!lrrFormula.IsCustomFormula)
                        {
                            string val = "";
                            if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteComponentPropertyValue).ToString())
                            {
                                LRRSiteComponentPropertyValue lrrSiteCompPropValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);
                                if (lrrSiteCompPropValue == null)
                                    continue;
                                val = lrrSiteCompPropValue.PropertyValue;
                            }
                            //Added by Nidhi For Grouped Properties - Chemical components and Data Arrays
                            if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString())
                            {
                                LRRSiteGroupPropertyQualifierValue lrrSiteGroupQualifierValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);
                                if (lrrSiteGroupQualifierValue == null)
                                    continue;
                                val = lrrSiteGroupQualifierValue.PropertyValue;
                            }
                            try
                            {
                                ExceuteScript(lrrFormulaParams, formula, lrrFormula);

                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteComponentPropertyValue).ToString())
                                {
                                    LRRSiteComponentPropertyValue lrrSiteCompPropValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);

                                    if (lrrSiteCompPropValue == null)
                                        continue;

                                    if (lrrSiteCompPropValue.IsOverrided)
                                    {
                                        lrrSiteCompPropValue.OverridedValue = lrrSiteCompPropValue.PropertyValue;
                                        lrrSiteCompPropValue.PropertyValue = val;
                                    }
                                }

                                //Added by Nidhi For Grouped Properties - Chemical components and Data Arrays
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString())
                                {
                                    LRRSiteGroupPropertyQualifierValue lrrSiteGroupQualifierValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);
                                    if (lrrSiteGroupQualifierValue == null)
                                        continue;

                                    if (lrrSiteGroupQualifierValue.IsOverrided)
                                    {
                                        lrrSiteGroupQualifierValue.OverridedValue = lrrSiteGroupQualifierValue.PropertyValue;
                                        lrrSiteGroupQualifierValue.PropertyValue = val;
                                    }
                                }
                            }
                            catch (Exception ex)
                            {
                                Logger.LogMessage(typeof(RunController), "InitiatePreCalc :" + ex.Message, Logger.Level.Debug);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(RunController), "InitiatePreCalc :" + ex.Message, Logger.Level.Debug);
            }
            finally
            {
                GC.Collect();
            }

        }

        /// <summary>
        /// Method executes the Historian formulas of type Post
        /// Formulas which carry the SourceCollectionName = typeof(LRRHistData)
        /// will be the ones which will be executed by this method.
        /// This method should be called at the begining of the 
        /// extration of the historian points from the Active Historian 
        /// The Active Historian can be one of the following:- Online,Offline,Tuning
        /// </summary>
        public void InitiatePostExtraction()
        {
            try
            {
                Logger.LogMessage(typeof(RunController), "InitiatePostExtraction Called", Logger.Level.Debug);

                LRRFormulas lrrFormulas = LRRModel.FormulasObj.FilterFormulasByPostExtraction();

                //				foreach(LRRFormula lrrFormula in lrrFormulas)
                lrrFormulas.SortColumn = "SortIndex";
                lrrFormulas.Sort();

                lrrFormulas.SortColumn = "Priority";
                lrrFormulas.SortingOrder = Symcon.BusinessObjects.GenericComparer.SortOrder.Ascending;
                lrrFormulas.Sort();

                int nTotalFormula = lrrFormulas.Count;
                for (int formulaCtr = 0; formulaCtr < nTotalFormula; formulaCtr++)
                {
                    LRRFormula lrrFormula = lrrFormulas[formulaCtr];


                    if (lrrFormula.SourceCollectionName.Trim() != typeof(LRRHistData).ToString())
                        continue;



                    LRRFormulaParams lrrFormulaParams = lrrFormula.Params;
                    string formula = lrrFormula.Expression;
                    if (lrrFormula.IsFormula && lrrFormula.PostCalc && lrrFormula.SourceCollectionName.Trim() == typeof(LRRHistData).ToString())
                    {
                        string val = "";
                        if (formula != null && formula != "" && lrrFormulaParams != null)
                        {
                            val = Calculate(lrrFormulaParams, formula);
                            if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRHistData).ToString())
                            {
                                //Populate from Model
                                try
                                {
                                    LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
                                    LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
                                    if (lrrhistData != null)
                                    {

                                        //Convert to Model UOM
                                        string uomID = lrrhistData.ModelUOM;
                                        string uiUOMId = lrrhistData.InterfaceUnits;
                                        lrrhistData.InterfaceValue = ConvertUOM(uiUOMId, uomID, val);
                                        string[] ret;

                                        if (lrrhistData != null)
                                        {

                                            ret = HistorianConnect.HistorianUtilities.SmartDataFilter("C", _currentStartTime, _currentEndTime, lrrhistData);
                                            lrrhistData.LastValidAverageValue = ret[1]; //Assign the Last Valid
                                            lrrhistData.ActiveFilterReturnValue = ret[1]; //Assign Active Filter Return Value...
                                            lrrhistData.MeterOnOffStatusFlag = ret[2];
                                            lrrhistData.DataQuality = ret[0];
                                        }
                                        lrrhistData.UpdatedRecordStack.Clear();
                                    }
                                }
                                catch
                                {
                                    Symcon.Utils.MessageLogger.Logger.LogMessage("Error Executing Function for " + formula, Symcon.Utils.MessageLogger.Logger.Level.Debug);
                                }

                            }
                        }
                    }
                    else if (!lrrFormula.IsFormula && !lrrFormula.IsCustomFormula && lrrFormula.PostCalc && lrrFormula.SourceCollectionName.Trim() == typeof(LRRHistData).ToString())
                    {
                        ExceuteScript(lrrFormulaParams, formula, lrrFormula);
                    }
                    if (lrrFormula.IsCustomFormula)
                    {
                        LRRHistData lrrhistData = LRRModel.HistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
                        if (lrrhistData != null)
                        {
                            ComputeCustomFormula(lrrhistData, lrrFormulaParams, lrrFormula);

                            string[] ret;

                            ret = HistorianConnect.HistorianUtilities.SmartDataFilter("C", _currentStartTime, _currentEndTime, lrrhistData);
                            lrrhistData.LastValidAverageValue = ret[1]; //Assign the Last Valid
                            lrrhistData.ActiveFilterReturnValue = ret[1]; //Assign Active Filter Return Value...
                            lrrhistData.MeterOnOffStatusFlag = ret[2];
                            lrrhistData.DataQuality = ret[0];
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(RunController), "InitiatePostExtraction :" + ex.Message, Logger.Level.Debug);
            }
            finally
            {
                GC.Collect();
            }
        }

        /// <summary>
        /// Method executes the Property Value formulas of type Post
        /// Formulas which carry the SourceCollectionName != typeof(LRRHistData)
        /// will be the ones which will be executed by this method.
        /// This method should be called after the 
        /// read/write execution engine cycle
        /// </summary>
        public void InitiatePostCalc()
        {
            try
            {
                Logger.LogMessage(typeof(RunController), "InitiatePostCalc Called", Logger.Level.Debug);

                //System.Threading.Thread.Sleep(120000);

                LRRFormulas lrrFormulas = LRRModel.FormulasObj.FilterFormulasByPostCalc();
                //foreach(LRRFormula lrrFormula in lrrFormulas) Modified by NidhiG to sort formula collection before execution
                lrrFormulas.SortColumn = "SortIndex";
                lrrFormulas.Sort();

                lrrFormulas.SortColumn = "Priority";
                lrrFormulas.SortingOrder = Symcon.BusinessObjects.GenericComparer.SortOrder.Ascending;
                lrrFormulas.Sort();

                int nTotalFormula = lrrFormulas.Count;
                for (int formulaCtr = 0; formulaCtr < nTotalFormula; formulaCtr++)
                {
                    LRRFormula lrrFormula = lrrFormulas[formulaCtr];

                    try
                    {
                        if (lrrFormula.SourceCollectionName == null) // Added by Sandeep as lrrFormula.SourceCollectionName.Trim() was throwing lrrFormula.SourceCollectionName null Exception.
                            continue;
                        if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRHistData).ToString())
                            continue;
                        LRRFormulaParams lrrFormulaParams = lrrFormula.Params;

                        string formula = lrrFormula.Expression;
                        //if (formula == "C00+(C10AH*AH)+(C20AH*(AH^2))+(C11TDW*DT)+((C12HOD*(H+LOOKAHEADH))+(C121HSIN*(SIN(C122HSIN*((H+LOOKAHEADH)-C123HSIN)))))+((C13DOW*(D+LOOKAHEADD))+(C131DSIN*(SIN(C132DSIN*((D+LOOKAHEADD)-C133DSIN)))))+ ((C14MON*(M+LOOKAHEADM))+(C141MSIN*(SIN(C142MSIN*((M+LOOKAHEADM)-C143MSIN)))))")
                        //{
                        //    int i = 5;
                        //}
                        if (lrrFormula.IsFormula)
                        {
                            string val = "";
                            if (formula != null && formula != "" && lrrFormulaParams != null)
                            {
                                val = Calculate(lrrFormulaParams, formula);

                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteComponentPropertyValue).ToString())
                                {
                                    LRRSiteComponentPropertyValue lrrSiteCompPropValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);
                                    if (lrrSiteCompPropValue == null)
                                        continue;

                                    //Convert to Model UOM
                                    string siteCompPropID = lrrSiteCompPropValue.SiteComponentPropertyID;
                                    LRRSiteComponentProperty siteCompPropObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteCompPropID);
                                    if (siteCompPropObj != null)
                                    {
                                        string uiUOMID = siteCompPropObj.GetUIUOMID();
                                        val = ConvertUOM(uiUOMID, lrrSiteCompPropValue.GetUOMID(), val);
                                    }

                                    if (!lrrSiteCompPropValue.IsOverrided)
                                    {
                                        lrrSiteCompPropValue.PropertyValue = val;
                                    }
                                    else
                                    {
                                        lrrSiteCompPropValue.OverridedValue = val;
                                    }
                                }
                                //Aggregates
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRAggregatePropertyValue).ToString())
                                {
                                    LRRAggregatePropertyValue lrrAggPropValue = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID);
                                    if (lrrAggPropValue == null)
                                        continue;

                                    //Convert to Model UOM
                                    string aggPropID = lrrAggPropValue.AggregateID;
                                    LRRAggregateProperty aggPropObj = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(aggPropID);
                                    if (aggPropObj != null)
                                    {
                                        string uiUOMID = aggPropObj.GetUIUOMID();
                                        val = ConvertUOM(uiUOMID, lrrAggPropValue.GetUOMID(), val);
                                    }

                                    if (!lrrAggPropValue.IsOverrided)
                                    {

                                        lrrAggPropValue.PropertyValue = val;
                                    }
                                    else
                                    {
                                        lrrAggPropValue.OverridedValue = val;
                                    }
                                }

                                //Added by Nidhi For Grouped Properties - Chemical components and Data Arrays
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString())
                                {
                                    if (lrrFormula.SourceObjectID == null)
                                        continue;
                                    LRRSiteGroupPropertyQualifierValue lrrSiteGroupQualifierValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);
                                    if (lrrSiteGroupQualifierValue == null)
                                        continue;

                                    //Convert to Model UOM
                                    string lkupQualID = lrrSiteGroupQualifierValue.LookUpQualifierID;
                                    LRRLookUpQualifier lkupQualObj = null;
                                    if (lkupQualID != null && lkupQualID.Trim() != "")
                                        lkupQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lkupQualID);
                                    //Group Property
                                    if (lkupQualObj != null)
                                    {
                                        string uomTypeID = lkupQualObj.UOMTypeID;
                                        //Get the UI UOM
                                        LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                        string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, uomTypeID);

                                        val = ConvertUOM(uiUOMID, lrrSiteGroupQualifierValue.GetUOMID(), val);
                                    }
                                    else //Data Arrays
                                    {
                                        LRRSiteComponentGroupProperty objGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrSiteGroupQualifierValue.GroupPropertyID);
                                        if (objGrpProp != null)
                                        {
                                            string propType = objGrpProp.PropertyID;
                                            LRRPropertyType propTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(propType);
                                            if (propTypeObj != null)
                                            {
                                                //Get the UI UOM
                                                LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                                string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, propTypeObj.UOMTypeID);
                                                val = ConvertUOM(uiUOMID, lrrSiteGroupQualifierValue.GetUOMID(), val);
                                            }
                                        }
                                    }

                                    if (!lrrSiteGroupQualifierValue.IsOverrided)
                                    {
                                        lrrSiteGroupQualifierValue.PropertyValue = val;
                                    }
                                    else
                                    {
                                        lrrSiteGroupQualifierValue.OverridedValue = val;
                                    }
                                }

                            }
                        }
                        else
                        {
                            if (!lrrFormula.IsCustomFormula)
                            {
                                string val = "";
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteComponentPropertyValue).ToString())
                                {
                                    LRRSiteComponentPropertyValue lrrSiteCompPropValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);
                                    if (lrrSiteCompPropValue == null)
                                        continue;
                                    val = lrrSiteCompPropValue.PropertyValue;
                                }

                                //Added by Nidhi For Grouped Properties - Chemical components and Data Arrays
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString())
                                {
                                    LRRSiteGroupPropertyQualifierValue lrrSiteGroupQualifierValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);
                                    if (lrrSiteGroupQualifierValue == null)
                                        continue;

                                    val = lrrSiteGroupQualifierValue.PropertyValue;
                                }

                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRAggregatePropertyValue).ToString())
                                {
                                    LRRAggregatePropertyValue lrrAggregatePropValue = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID);
                                    if (lrrAggregatePropValue == null)
                                        continue;
                                    val = lrrAggregatePropValue.PropertyValue;
                                }

                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRAggregateGroupedPropertyValue).ToString())
                                {
                                    LRRAggregateGroupedPropertyValue lrrAggregateGpPropValue = LRRModel.AggregateGroupedPropertyValuesObj.GetAggregateGroupedPropertyValueFromID(lrrFormula.SourceObjectID);
                                    if (lrrAggregateGpPropValue == null)
                                        continue;
                                    val = lrrAggregateGpPropValue.PropertyValue;
                                }
                                ExceuteScript(lrrFormulaParams, formula, lrrFormula);
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteComponentPropertyValue).ToString())
                                {
                                    LRRSiteComponentPropertyValue lrrSiteCompPropValue = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);
                                    if (lrrSiteCompPropValue.IsOverrided)
                                    {
                                        lrrSiteCompPropValue.OverridedValue = lrrSiteCompPropValue.PropertyValue;
                                        lrrSiteCompPropValue.PropertyValue = val;
                                    }
                                }

                                //Added by Nidhi For Grouped Properties - Chemical components and Data Arrays
                                if (lrrFormula.SourceCollectionName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString())
                                {
                                    LRRSiteGroupPropertyQualifierValue lrrSiteGroupQualifierValue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);
                                    if (lrrSiteGroupQualifierValue == null)
                                        continue;

                                    if (lrrSiteGroupQualifierValue.IsOverrided)
                                    {
                                        lrrSiteGroupQualifierValue.OverridedValue = lrrSiteGroupQualifierValue.PropertyValue;
                                        lrrSiteGroupQualifierValue.PropertyValue = val;
                                    }
                                }

                            }
                        }
                    }
                    catch
                    {
                        //Symcon.Utils.MessageLogger.Logger.LogMessage("Error executing function",Symcon.Utils.MessageLogger.Level.Debug);
                        Symcon.Utils.MessageLogger.Logger.LogMessage("Error Executing Function for " + lrrFormula.FormulaName + '-' + lrrFormula.ID , Symcon.Utils.MessageLogger.Logger.Level.Debug);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(RunController), "InitiatePostCalc :" + ex.Message, Logger.Level.Debug);
            }
            finally
            {
                GC.Collect();
            }
        }

        private Dictionary<LRRFormulaParam, string> ParseScript(string script, LRRFormulaParams fParams)
        {
            Dictionary<LRRFormulaParam, string> retVal = new Dictionary<LRRFormulaParam, string>();
            foreach (LRRFormulaParam fParam in fParams)
            {
                string fParamName = fParam.ParamName;
                if (script.Contains(fParamName + ".PropertyValue"))
                {
                    retVal.Add(fParam, "PropertyValue");
                }
                else if (script.Contains(fParamName + ".TimeStamp"))
                {
                    retVal.Add(fParam, "TimeStamp");
                }
            }
            return retVal;
        }

        /// <summary>
        /// This method executes the script
        /// </summary>
        /// <param name="lrrFormulaParams"></param>
        /// <param name="script"></param>
        /// <param name="lrrFormula"></param>
        public void ExceuteScript(LRRFormulaParams lrrFormulaParams, string script, LRRFormula lrrFormula)
        {
            try
            {
                LRRPropertyValueTransferObject resultPVTO = new LRRPropertyValueTransferObject();
                LRRHistDataTransferObject resultHistTO = new LRRHistDataTransferObject();
                Dictionary<LRRFormulaParam, string> paramValDic = ParseScript(script, lrrFormulaParams);
                List<string> paramNames = new List<string>();
                List<object> paramValues = new List<object>();
                bool flag = true;

                foreach (LRRFormulaParam param in lrrFormulaParams)
                {
                    if (param.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
                    {
                        if (LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(param.SourceOBJID) != null)
                        {
                            LRRSiteComponentPropertyValue lrrSiteCompPropVal = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(param.SourceOBJID).GetValueObjectForPropertyMode(param.ModeID);
                            if (lrrSiteCompPropVal != null)
                            {
                                LRRPropertyValueTransferObject propValueTO = new LRRPropertyValueTransferObject();
                                if (paramValDic.ContainsKey(param))
                                {
                                    paramNames.Add(param.ParamName);
                                    string valueToSend = paramValDic[param];
                                    if (valueToSend == "PropertyValue")
                                    {
                                        propValueTO.PropertyValue = lrrSiteCompPropVal.PropertyValue;
                                        propValueTO.PropertyName = "PropertyValue";
                                    }
                                    else if (valueToSend == "TimeStamp")
                                    {
                                        propValueTO.TimeStamp = lrrSiteCompPropVal.TimeStamp;
                                        propValueTO.PropertyName = "TimeStamp";
                                    }
                                    if (propValueTO.PropertyValue != null || propValueTO.TimeStamp != null)
                                    {
                                        paramValues.Add(propValueTO);
                                    }
                                    else
                                    {
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    if (param.SourceCollectioName == typeof(LRRHistData).ToString())
                    {
                        LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
                        LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(param.SourceOBJID);
                        if (lrrhistData != null)
                        {
                            paramNames.Add(param.ParamName);
                            LRRHistDataTransferObject histTO = new LRRHistDataTransferObject();
                            histTO.InterfaceValue = lrrhistData.InterfaceValue;
                            paramValues.Add(histTO);
                        }
                    }

                    if (param.SourceCollectioName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString().Trim())
                    {
                        if (LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(param.SourceOBJID) != null)
                        {
                            string sQualifierID = "";
                            if ((param.SourceLookUpQualifierID != null) && (param.SourceLookUpQualifierID != ""))
                                sQualifierID = LRRModel.ChildModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(param.SourceLookUpQualifierID).QualifierID;

                            LRRSiteGroupPropertyQualifierValue lrrSiteCompGroupPropQVal = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(param.SourceOBJID).GroupPptiesQualifierValues.GetQualifierValueObjectForPropertyMode(param.SourceOBJID, param.SourceLookUpPropertyID, sQualifierID, param.ModeID);
                            //LRRSiteGroupPropertyQualifierValue lrrSiteCompGroupPropQVal = LRRModel.SiteGroupPropertyQualifierValuesObj.GetGroupQualifie
                            if (lrrSiteCompGroupPropQVal != null)
                            {

                                LRRPropertyValueTransferObject propValueTO = new LRRPropertyValueTransferObject();
                                if (paramValDic.ContainsKey(param))
                                {
                                    paramNames.Add(param.ParamName);
                                    string valueToSend = paramValDic[param];
                                    if (valueToSend == "PropertyValue")
                                    {
                                        propValueTO.PropertyValue = lrrSiteCompGroupPropQVal.PropertyValue;
                                        propValueTO.PropertyName = "PropertyValue";
                                    }
                                    else if (valueToSend == "TimeStamp")
                                    {
                                        propValueTO.TimeStamp = lrrSiteCompGroupPropQVal.TimeStamp;
                                        propValueTO.PropertyName = "TimeStamp";
                                    }
                                    if (propValueTO.PropertyValue != null || propValueTO.TimeStamp != null)
                                    {
                                        paramValues.Add(propValueTO);
                                    }
                                    else
                                    {
                                        break;
                                    }
                                }
                            }
                            //if (lrrSiteCompGroupPropQVal != null)
                            //{
                            //    paramNames.Add(param.ParamName);

                            //    propValueTO.PropertyValue = lrrSiteCompGroupPropQVal.PropertyValue;
                            //    if(propValueTO.PropertyValue != null)
                            //        paramValues.Add(propValueTO);
                            //}
                        }
                    }

                    //Constants
                    if (param.SourceCollectioName == typeof(LRRConstant).ToString())
                    {
                        LRRConstantTransferObject lrrConstant = new LRRConstantTransferObject();
                        lrrConstant.Value = param.CurrentValue;
                        paramNames.Add(param.ParamName);
                        paramValues.Add(lrrConstant);
                    }
                }
                if (paramValues.Count > 0)
                {
                    if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRAggregatePropertyValue")
                    {
                        LRRAggregatePropertyValue aggPropVal = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID);
                        if (aggPropVal != null && flag)
                        {
                            paramNames.Add("RESULT");
                            resultPVTO.PropertyValue = aggPropVal.PropertyValue;
                            paramValues.Add(resultPVTO);
                            flag = false;
                        }
                    }
                    else if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRSiteComponentPropertyValue")
                    {
                        LRRSiteComponentPropertyValue siteCompPropVal = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);
                        if (siteCompPropVal != null && flag)
                        {
                            paramNames.Add("RESULT");
                            resultPVTO.PropertyValue = siteCompPropVal.PropertyValue;
                            paramValues.Add(resultPVTO);
                            flag = false;
                        }
                    }
                    else if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRHistData")
                    {
                        LRRHistData histData = LRRModel.HistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
                        if (histData != null && flag)
                        {
                            paramNames.Add("RESULT");
                            resultHistTO.InterfaceValue = histData.InterfaceValue;
                            paramValues.Add(resultHistTO);
                            flag = false;
                        }
                    }
                    else if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRAggregateGroupedPropertyValue")
                    {
                        LRRAggregateGroupedPropertyValue aggGpPropValue = LRRModel.AggregateGroupedPropertyValuesObj.GetAggregateGroupedPropertyValueFromID(lrrFormula.SourceObjectID);
                        if (aggGpPropValue != null && flag)
                        {
                            paramNames.Add("RESULT");
                            resultPVTO.PropertyValue = aggGpPropValue.PropertyValue;
                            paramValues.Add(resultPVTO);
                            flag = false;
                        }
                    }
                    else if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue")
                    {
                        LRRSiteGroupPropertyQualifierValue siteGroupPropQualifierVal = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);
                        if (siteGroupPropQualifierVal != null && flag)
                        {
                            paramNames.Add("RESULT");
                            resultPVTO.PropertyValue = siteGroupPropQualifierVal.PropertyValue;
                            paramValues.Add(resultPVTO);
                            flag = false;
                        }
                    }
                }
                if (LRRFormulaDataStore.ObjectSource == typeof(LRRAggregatePropertyValue).ToString())
                {
                    LRRAggregatePropertyValue aggPropVal = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(LRRFormulaDataStore.SourceObjectID);
                    if (aggPropVal != null)
                    {
                        if (!paramNames.Contains("RESULT"))
                        {
                            paramNames.Add("RESULT");
                            resultPVTO.PropertyValue = aggPropVal.PropertyValue;
                            paramValues.Add(resultPVTO);
                        }
                    }
                }
                string[] arrParamNames = paramNames.ToArray();
                object[] arrParamValues = paramValues.ToArray();
                object result = null;
                if (paramValues.Count > 0)
                {
                    /*AssemblyName assembName = AssemblyName.GetAssemblyName(Application.StartupPath + "\\Symcon.Lightrige.ScriptRunner.dll");
                    _app.ScriptingAppDomain.Load(assembName);////"Symcon.Lightrige.ScriptRunner.dll");
                    System.Runtime.Remoting.ObjectHandle scriptRunnerObjectHandel = _app.ScriptingAppDomain.CreateInstance(assembName.Name, "Symcon.Lightrige.ScriptRunner.LRRScriptRunner");
                    LRRScriptRunner scriptRunner = (Symcon.Lightrige.ScriptRunner.LRRScriptRunner)scriptRunnerObjectHandel.Unwrap();
                    result = scriptRunner.Execute(arrParamNames, arrParamValues, script);*/
                    try
                    {
                        result = _app.ScriptRunner.Execute(arrParamNames, arrParamValues, script);
                    }
                    catch (System.Runtime.Remoting.RemotingException ex)
                    {
                        _runningScriptCounter = 0;
                        scriptingAppDomainUnloadDelegate myDel = new scriptingAppDomainUnloadDelegate(_app.UnLoadScriptingAppDomain);
                        myDel.BeginInvoke(_app.ScriptingAppDomain, null, null);

                        _app.ReloadScriptingAppDomain();
                        result = _app.ScriptRunner.Execute(arrParamNames, arrParamValues, script);
                    }


                    _runningScriptCounter++;
                    if (_runningScriptCounter >= 50)
                    {
                        _runningScriptCounter = 0;
                        scriptingAppDomainUnloadDelegate myDel = new scriptingAppDomainUnloadDelegate(_app.UnLoadScriptingAppDomain);
                        myDel.BeginInvoke(_app.ScriptingAppDomain, null, null);

                        _app.ReloadScriptingAppDomain();
                    }
                }
                if (result != null)
                {
                    resultHistTO = result as LRRHistDataTransferObject;
                    resultPVTO = result as LRRPropertyValueTransferObject;
                    flag = true;

                    if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRAggregatePropertyValue")
                    {
                        LRRAggregatePropertyValue aggPropVal = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID);
                        if (aggPropVal != null && flag)
                        {
                            aggPropVal.PropertyValue = resultPVTO.PropertyValue;
                        }
                    }
                    else if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRSiteComponentPropertyValue")
                    {
                        LRRSiteComponentPropertyValue siteCompPropVal = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);
                        if (siteCompPropVal != null && flag)
                        {
                            siteCompPropVal.PropertyValue = resultPVTO.PropertyValue;
                        }
                    }
                    else if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRHistData")
                    {
                        LRRHistData histData = LRRModel.HistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
                        if (histData != null && flag)
                        {
                            histData.InterfaceValue = resultHistTO.InterfaceValue;
                        }
                    }
                    else if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRAggregateGroupedPropertyValue")
                    {
                        LRRAggregateGroupedPropertyValue aggGpPropValue = LRRModel.AggregateGroupedPropertyValuesObj.GetAggregateGroupedPropertyValueFromID(lrrFormula.SourceObjectID);
                        if (aggGpPropValue != null && flag)
                        {
                            aggGpPropValue.PropertyValue = resultPVTO.PropertyValue;
                        }
                    }
                    else if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRSiteGroupPropertyQualifierValue")
                    {
                        LRRSiteGroupPropertyQualifierValue siteGroupPropQualifierVal = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);
                        if (siteGroupPropQualifierVal != null && flag)
                        {
                            siteGroupPropQualifierVal.PropertyValue = resultPVTO.PropertyValue;
                        }
                    }
                    #region comment

                    /*foreach (LRRFormulaParam param in lrrFormulaParams)
                    {
                        if (param.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
                        {
                            if (LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(param.SourceOBJID) != null)
                            {
                                if (lrrFormula.SourceCollectionName == "Symcon.LightRidge.BusinessObjects.LRRAggregatePropertyValue")
                                {
                                    LRRAggregatePropertyValue aggPropVal = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID);
                                    if (aggPropVal != null && flag)
                                    {
                                        aggPropVal.PropertyValue = resultPVTO.PropertyValue; 
                                        flag = false;
                                        break;
                                    }
                                }
                                else
                                {
                                    LRRSiteComponentPropertyValue siteCompPropVal = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(lrrFormula.SourceObjectID);
                                    if (siteCompPropVal != null && flag)
                                    {
                                        siteCompPropVal.PropertyValue = resultPVTO.PropertyValue;
                                        flag = false;
                                        break;
                                    }
                                }
                            }
                        }
                        if (param.SourceCollectioName == typeof(LRRHistData).ToString())
                        {   
                            LRRHistData histData = LRRModel.HistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceObjectID);
                            if (histData != null && flag)
                            {
                                histData.InterfaceValue = resultHistTO.InterfaceValue;
                                flag = false;
                                break;
                            }
                        }

                        if (param.SourceCollectioName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString().Trim())
                        {
                            if (LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(param.SourceOBJID) != null)
                            {
                                LRRSiteGroupPropertyQualifierValue siteGroupPropQualifierVal = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(lrrFormula.SourceObjectID);
                                if (siteGroupPropQualifierVal != null && flag)
                                {
                                    siteGroupPropQualifierVal.PropertyValue = resultPVTO.PropertyValue;
                                    flag = false;
                                    break;
                                }
                            }
                        }
                    }*/
                    #endregion
                    if (LRRFormulaDataStore.ObjectSource == typeof(LRRAggregatePropertyValue).ToString())
                    {
                        LRRAggregatePropertyValue aggPropVal = LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormula.SourceObjectID);
                        if (aggPropVal != null)
                        {
                            if (resultPVTO != null)
                            {
                                aggPropVal.PropertyValue = resultPVTO.PropertyValue;
                            }
                        }

                    }
                }
                //Added by Sachin to test unloading the scriptingAppDomain in the end
                //scriptingAppDomainUnloadDelegate unloadscriptDel = new scriptingAppDomainUnloadDelegate(_app.UnLoadScriptingAppDomain);
                //unloadscriptDel.BeginInvoke(_app.ScriptingAppDomain, null, null);
                paramNames.Clear();
                paramNames = null;
                paramValues.Clear();
                paramValues = null;
                paramValDic.Clear();
                paramValDic = null;

            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(RunController), "ExecuteScript : Formula ID=" + lrrFormula.ID + "  " + ex.Message, Logger.Level.Debug);
            }

        }

        /// <summary>
        /// This method executes the custom formulas
        /// The following Custom Formulas are defined:-
        /// 1.GetAverage()
        /// 2.SelectBasedOnVote()
        /// </summary>
        /// <param name="lrrHistData"></param>
        /// <param name="lrrFormulaParams"></param>
        /// <param name="lrrFormula"></param>
        public void ComputeCustomFormula(LRRHistData lrrHistData, LRRFormulaParams lrrFormulaParams, LRRFormula lrrFormula)
        {
            try
            {
                string methodName = lrrFormula.MethodName;
                Type objType = typeof(HistorianUtilities);
                Object obj = Activator.CreateInstance(objType);
                MethodInfo miInfo = objType.GetMethod(methodName);
                ParameterInfo[] paramInfo = miInfo.GetParameters();

                Object[] parameters = new Object[paramInfo.Length];
                LRRFormulaParams localCopyOfParams = new LRRFormulaParams();
                for (int i = 0; i < lrrFormulaParams.Count; i++)
                {
                    LRRFormulaParam formulaParam = lrrFormulaParams[i];
                    if (formulaParam.SourceCollectioName == methodName)
                    {
                        string paramName = formulaParam.ParamName;
                        for (int k = 0; k < paramInfo.Length; k++)
                        {
                            if (paramName == paramInfo[k].Name)
                            {
                                int pos = paramInfo[k].Position;
                                if (paramInfo[k].ParameterType.BaseType == typeof(Enum))
                                    parameters[pos] = Enum.Parse(paramInfo[k].ParameterType, formulaParam.CurrentValue);
                                else
                                    parameters[pos] = formulaParam.CurrentValue;
                                break;
                            }
                        }
                    }
                    else
                    {
                        if (formulaParam.ParamName != "MIN" && formulaParam.ParamName != "MAX")
                        {
                            LRRHistData histData = LRRModel.HistDatas.GetHistDataFromHistTagUID(formulaParam.SourceOBJID);
                            if (histData != null)
                            {
                                formulaParam.CurrentValue = histData.InterfaceValue;
                                localCopyOfParams.Add(formulaParam);
                            }
                        }
                        else
                        {
                            localCopyOfParams.Add(formulaParam);
                        }
                    }


                }

                parameters[0] = localCopyOfParams;
                //parameters[1] = localCopyOfParams;

                object retValue = miInfo.Invoke(obj, parameters);
                if (lrrHistData != null)
                    lrrHistData.InterfaceValue = retValue.ToString();
            }
            catch (Exception ex)
            {
                Logger.LogMessage(typeof(RunController), "Compute Custom Formula: Formula ID=" + lrrFormula.ID + "  " + ex.Message, Logger.Level.Debug);
            }
        }

        /// <summary>
        /// This method evaluates the formula expression
        /// </summary>
        /// <param name="lrrFormulaParams"></param>
        /// <param name="formula"></param>
        /// <returns></returns>
        public string Calculate(LRRFormulaParams lrrFormulaParams, string formula)
        {
            //if (formula == "C00+(C10AH*AH)+(C20AH*(AH^2))+(C11TDW*DT)+((C12HOD*(H+LOOKAHEADH))+(C121HSIN*(SIN(C122HSIN*((H+LOOKAHEADH)-C123HSIN)))))+((C13DOW*(D+LOOKAHEADD))+(C131DSIN*(SIN(C132DSIN*((D+LOOKAHEADD)-C133DSIN)))))+ ((C14MON*(M+LOOKAHEADM))+(C141MSIN*(SIN(C142MSIN*((M+LOOKAHEADM)-C143MSIN)))))")
            //{
            //    int i = 5;
            //}
            Hashtable htFormulaParam = new Hashtable();
            foreach (LRRFormulaParam lrrFormulaParam in lrrFormulaParams)
            {
                if (!htFormulaParam.ContainsKey(lrrFormulaParam.ParamName))
                    htFormulaParam.Add(lrrFormulaParam.ParamName, lrrFormulaParam);
            }

            //--Inline Operator--
            string[] inlineOperator = formula.Split(new char[] { '?' });
            if (inlineOperator.Length > 1)
            {
                string exp = inlineOperator[0].ToString(Symcon.BusinessObjects.SymConverter.Culture);
                string[] param = exp.Split(new char[] { '=', '>', '<' });
                int greaterThanIndex = exp.IndexOf('>');
                int lesserThanIndex = exp.IndexOf('<');
                int equalToIndex = exp.IndexOf('=');
                string oper = null;
                if (greaterThanIndex != -1)
                {
                    oper = ">";
                }
                else if (lesserThanIndex != -1)
                {
                    oper = "<";
                }
                else if (equalToIndex != -1)
                {
                    oper = "=";
                }

                if (param.Length > 1)
                {
                    string param1 = param[0].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim();
                    string param2 = param[1].ToString(Symcon.BusinessObjects.SymConverter.Culture).Trim();
                    string finalValue = null;
                    if (htFormulaParam.Contains(param1))
                    {
                        string[] result = inlineOperator[1].Split(new char[] { ':' });

                        try
                        {
                            //Get the current Value
                            LRRFormulaParam lrrFormulaParam1 = htFormulaParam[param1] as LRRFormulaParam;
                            string curVal = "";
                            if (lrrFormulaParam1.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
                            {
                                if (lrrFormulaParam1.SourceOBJID == null || lrrFormulaParam1.SourceOBJID.Trim() == "")
                                {
                                    curVal = lrrFormulaParam1.CurrentValue;
                                }
                                else
                                {
                                    //Convert to UI UOM
                                    if (LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormulaParam1.SourceOBJID) != null)
                                    {
                                        LRRSiteComponentProperty propObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormulaParam1.SourceOBJID);
                                        LRRSiteComponentPropertyValue propValueObj = propObj.GetValueObjectForPropertyMode(lrrFormulaParam1.ModeID);
                                        if (propValueObj != null)
                                        {
                                            curVal = propValueObj.GetPropertyValueForUOM(propValueObj.GetUOMID(), propObj.GetUIUOMID(), propValueObj.ActualPropertyValue);
                                        }
                                    }
                                }
                            }
                            else if (lrrFormulaParam1.SourceCollectioName == typeof(LRRAggregatePropertyValue).ToString())
                            {
                                if (lrrFormulaParam1.SourceOBJID == null || lrrFormulaParam1.SourceOBJID.Trim() == "")
                                {
                                    curVal = lrrFormulaParam1.CurrentValue;
                                }
                                else
                                {
                                    //Convert to UI UOM
                                    if (LRRModel.AggregatePropertyValuesObj.FindAggregatePropertyValueByID(lrrFormulaParam1.SourceOBJID) != null)
                                    {
                                        LRRAggregateProperty propObj = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(lrrFormulaParam1.SourceOBJID);
                                        if (propObj != null)
                                        {
                                            LRRAggregatePropertyValue propValueObj = propObj.GetValueObjectForPropertyMode(lrrFormulaParam1.ModeID);
                                            if (propValueObj != null)
                                            {
                                                curVal = propValueObj.GetPropertyValueForUOM(propValueObj.GetUOMID(), propObj.GetUIUOMID(), propValueObj.ActualPropertyValue);
                                            }
                                        }
                                    }
                                }
                            }
                            else if (lrrFormulaParam1.SourceCollectioName == typeof(LRRHistData).ToString())
                            {
                                if (lrrFormulaParam1.SourceOBJID == null || lrrFormulaParam1.SourceOBJID.Trim() == "")
                                {
                                    curVal = lrrFormulaParam1.CurrentValue;
                                }
                                else
                                {
                                    //LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
                                    //Populate from Model
                                    LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
                                    LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormulaParam1.SourceOBJID);

                                    //Convert to UI UOM
                                    string uomID = lrrhistData.ModelUOM;
                                    string uiUOMId = lrrhistData.InterfaceUnits;
                                    curVal = ConvertUOM(uomID, uiUOMId, lrrhistData.InterfaceValue);
                                }
                            }
                            else if (lrrFormulaParam1.SourceCollectioName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString().Trim())  //Added by Nidhi For Grouped Properties - Chemical components and Data Arrays
                            {
                                if (lrrFormulaParam1.SourceOBJID == null || lrrFormulaParam1.SourceOBJID.Trim() == "")
                                {
                                    curVal = lrrFormulaParam1.CurrentValue;
                                }
                                else
                                {
                                    if (LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrFormulaParam1.SourceOBJID) != null)
                                    {
                                        string sQualifierID = "";
                                        if ((lrrFormulaParam1.SourceLookUpQualifierID != null) && (lrrFormulaParam1.SourceLookUpQualifierID != ""))
                                            sQualifierID = LRRModel.ChildModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lrrFormulaParam1.SourceLookUpQualifierID).QualifierID;

                                        LRRSiteGroupPropertyQualifierValue lrrSiteCompGroupPropQVal = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrFormulaParam1.SourceOBJID).GroupPptiesQualifierValues.GetQualifierValueObjectForPropertyMode(lrrFormulaParam1.SourceOBJID, lrrFormulaParam1.SourceLookUpPropertyID, sQualifierID, lrrFormulaParam1.ModeID);
                                        if (lrrSiteCompGroupPropQVal != null)
                                        {
                                            curVal = lrrSiteCompGroupPropQVal.ActualPropertyValue;
                                            //Convert to UI UOM
                                            string lkupQualID = lrrSiteCompGroupPropQVal.LookUpQualifierID;
                                            LRRLookUpQualifier lkupQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lkupQualID);
                                            //Group Property
                                            if (lkupQualObj != null)
                                            {
                                                string uomTypeID = lkupQualObj.UOMTypeID;
                                                //Get the UI UOM
                                                LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                                string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, uomTypeID);

                                                curVal = ConvertUOM(lrrSiteCompGroupPropQVal.GetUOMID(), uiUOMID, lrrSiteCompGroupPropQVal.ActualPropertyValue);
                                            }
                                            else //Data Arrays
                                            {
                                                LRRSiteComponentGroupProperty objGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrSiteCompGroupPropQVal.GroupPropertyID);
                                                if (objGrpProp != null)
                                                {
                                                    string propType = objGrpProp.PropertyID;
                                                    LRRPropertyType propTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(propType);
                                                    if (propTypeObj != null)
                                                    {
                                                        //Get the UI UOM
                                                        LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                                        string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, propTypeObj.UOMTypeID);
                                                        curVal = ConvertUOM(lrrSiteCompGroupPropQVal.GetUOMID(), uiUOMID, lrrSiteCompGroupPropQVal.ActualPropertyValue);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            //If constant
                            else
                            {
                                curVal = lrrFormulaParam1.CurrentValue;
                            }

                            if (curVal == null || curVal.Trim() == "")
                                curVal = "0";

                            //check for operator
                            switch (oper)
                            {
                                case ">":
                                    double cmp1 = Convert.ToDouble(curVal, Symcon.BusinessObjects.SymConverter.Culture);
                                    double cmp2 = Convert.ToDouble(param2, Symcon.BusinessObjects.SymConverter.Culture);
                                    if (cmp1 > cmp2)
                                    {
                                        finalValue = result[0].ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    }
                                    else
                                    {
                                        finalValue = result[1].ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    }
                                    break;
                                case "<":
                                    double cmp3 = Convert.ToDouble(curVal, Symcon.BusinessObjects.SymConverter.Culture);
                                    double cmp4 = Convert.ToDouble(param2, Symcon.BusinessObjects.SymConverter.Culture);
                                    if (cmp3 < cmp4)
                                    {
                                        finalValue = result[0].ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    }
                                    else
                                    {
                                        finalValue = result[1].ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    }
                                    break;
                                case "=":
                                    if (curVal == param2)
                                    {
                                        finalValue = result[0].ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    }
                                    else
                                    {
                                        finalValue = result[1].ToString(Symcon.BusinessObjects.SymConverter.Culture);
                                    }
                                    break;
                                default:
                                    break;

                            }
                        }
                        catch (Exception ex)
                        {
                            Logger.LogMessage(typeof(RunController), "Calculate :" + ex.Message, Logger.Level.Debug);
                        }
                        return finalValue;
                    }
                }
            }
            //--End--

            return CalculateFunctionsExpression(formula, htFormulaParam);

            /*ExecutionQueue eq = Evaluator.ParseIt(formula);
			ExecutionQueue newEq = new ExecutionQueue(); 
			ExecutionItem ei;
			while(eq.Count > 0)
			{
				ei = eq.Dequeue();
				switch(ei.itemType)
				{
					case ItemType.itString:
						if(htFormulaParam.Contains(ei.itemString))
						{
							LRRFormulaParam lrrFormula = htFormulaParam[ei.itemString] as LRRFormulaParam;
							string curVal = "";
                            if (lrrFormula.IsPESteamParam)
                            {
                                curVal = Symcon.LightRidge.BusinessObjects.TypeEditors.PESteamFunctionCalculation.CalulatePESteamFunction(lrrFormula);  
                            }
                            else
                            {
                                if (lrrFormula.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
                                {
                                    if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                    {
                                        curVal = lrrFormula.CurrentValue;
                                    }
                                    else
                                    {
                                        if (LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID) != null)
                                        {
                                            LRRSiteComponentProperty propObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID);
                                            LRRSiteComponentPropertyValue propValueObj = propObj.GetValueObjectForPropertyMode(lrrFormula.ModeID);
                                            if (propValueObj != null)
                                            {
                                                curVal = propValueObj.GetPropertyValueForUOM(propValueObj.GetUOMID(), propObj.GetUIUOMID(), propValueObj.PropertyValue);
                                            }
                                        }
                                    }
                                }
                                else if (lrrFormula.SourceCollectioName == typeof(LRRAggregatePropertyValue).ToString())
                                {
                                    if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                    {
                                        curVal = lrrFormula.CurrentValue;
                                    }
                                    else
                                    {
                                        if (LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(lrrFormula.SourceOBJID) != null)
                                        {
                                            LRRAggregateProperty propObj = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(lrrFormula.SourceOBJID);
                                            if (propObj != null)
                                            {
                                                LRRAggregatePropertyValue propValueObj = propObj.GetValueObjectForPropertyMode(lrrFormula.ModeID);
                                                if (propValueObj != null)
                                                {
                                                    curVal = propValueObj.GetPropertyValueForUOM(propValueObj.GetUOMID(), propObj.GetUIUOMID(), propValueObj.PropertyValue);
                                                }
                                            }
                                        }
                                    }
                                }
                                else if (lrrFormula.SourceCollectioName == typeof(LRRHistData).ToString())
                                {
                                    if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                    {
                                        curVal = lrrFormula.CurrentValue;
                                    }
                                    else
                                    {
                                        //LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
                                        //Populate from Model
                                        LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
                                        LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceOBJID);
                                        if (lrrhistData != null)
                                        {
                                            string uomID = lrrhistData.ModelUOM;
                                            string uiUOMId = lrrhistData.InterfaceUnits;
                                            curVal = ConvertUOM(uomID, uiUOMId, lrrhistData.InterfaceValue);
                                        }
                                    }
                                }
                                else if (lrrFormula.SourceCollectioName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString().Trim())
                                {
                                    if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                    {
                                        curVal = lrrFormula.CurrentValue;
                                    }
                                    else
                                    {
                                        if (LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrFormula.SourceOBJID) != null)
                                        {
                                            string sQualifierID = "";
                                            if ((lrrFormula.SourceLookUpQualifierID != null) && (lrrFormula.SourceLookUpQualifierID != ""))
                                                sQualifierID = LRRModel.ChildModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lrrFormula.SourceLookUpQualifierID).QualifierID;

                                            LRRSiteGroupPropertyQualifierValue lrrSiteCompGroupPropQVal = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrFormula.SourceOBJID).GroupPptiesQualifierValues.GetQualifierValueObjectForPropertyMode(lrrFormula.SourceOBJID, lrrFormula.SourceLookUpPropertyID, sQualifierID, lrrFormula.ModeID);
                                            if (lrrSiteCompGroupPropQVal != null)
                                            {
                                                string lkupQualID = lrrSiteCompGroupPropQVal.LookUpQualifierID;
                                                LRRLookUpQualifier lkupQualObj = null;
                                                if (lkupQualID != null && lkupQualID.Trim() != "")
                                                    lkupQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lkupQualID);
                                                //Group Property
                                                if (lkupQualObj != null)
                                                {
                                                    string uomTypeID = lkupQualObj.UOMTypeID;
                                                    //Get the UI UOM
                                                    LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                                    string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, uomTypeID);

                                                    curVal = ConvertUOM(lrrSiteCompGroupPropQVal.GetUOMID(), uiUOMID, lrrSiteCompGroupPropQVal.PropertyValue);
                                                }
                                                else //Data Arrays
                                                {
                                                    LRRSiteComponentGroupProperty objGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrSiteCompGroupPropQVal.GroupPropertyID);
                                                    if (objGrpProp != null)
                                                    {
                                                        string propType = objGrpProp.PropertyID;
                                                        LRRPropertyType propTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(propType);
                                                        if (propTypeObj != null)
                                                        {
                                                            //Get the UI UOM
                                                            LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                                            string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, propTypeObj.UOMTypeID);
                                                            curVal = ConvertUOM(lrrSiteCompGroupPropQVal.GetUOMID(), uiUOMID, lrrSiteCompGroupPropQVal.PropertyValue);
                                                        }
                                                    }
                                                }
                                            }
                                        }

                                    }
                                }
                                else //If Constant
                                {
                                    curVal = lrrFormula.CurrentValue;
                                }
                            }
							
							if(curVal == null || curVal.Trim() == "")
								curVal = "0";

							newEq.Enqueue(new ExecutionItem(ItemType.itDigit,curVal));
						}
						break;
					default:
						newEq.Enqueue(ei);
						break;
				}
			}

			Variant var  =  Evaluator.CalcIt(newEq);
			string val =  var.oValue.ToString();
			return val;*/
        }

        private string CalculateFunctionsExpression(string expression, Hashtable htFormulaParam)
        {
            ExecutionQueue eq = Evaluator.ParseIt(expression);
            ExecutionQueue newEq = new ExecutionQueue();
            ExecutionItem ei;

            while (eq.Count > 0)
            {
                ei = eq.Dequeue();
                switch (ei.ItemType)
                {
                    case ItemType.itString:
                        if (htFormulaParam.Contains(ei.ItemString.ToUpper()))
                        {
                            LRRFormulaParam lrrFormula = htFormulaParam[ei.ItemString.ToUpper()] as LRRFormulaParam;
                            string curVal = "";
                            if (lrrFormula.IsPESteamParam)
                            {
                                curVal = Symcon.LightRidge.BusinessObjects.TypeEditors.PESteamFunctionCalculation.CalulatePESteamFunction(lrrFormula);
                            }
                            else
                            {
                                if (lrrFormula.SourceCollectioName == typeof(LRRSiteComponentPropertyValue).ToString())
                                {
                                    if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                    {
                                        curVal = lrrFormula.CurrentValue;
                                    }
                                    else
                                    {
                                        if (LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID) != null)
                                        {
                                            LRRSiteComponentProperty propObj = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(lrrFormula.SourceOBJID);
                                            LRRSiteComponentPropertyValue propValueObj = propObj.GetValueObjectForPropertyMode(lrrFormula.ModeID);
                                            if (propValueObj != null)
                                            {
                                                curVal = propValueObj.GetPropertyValueForUOM(propValueObj.GetUOMID(), propObj.GetUIUOMID(), propValueObj.ActualPropertyValue);
                                            }
                                        }
                                    }
                                }
                                else if (lrrFormula.SourceCollectioName == typeof(LRRAggregatePropertyValue).ToString())
                                {
                                    if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                    {
                                        curVal = lrrFormula.CurrentValue;
                                    }
                                    else
                                    {
                                        if (LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(lrrFormula.SourceOBJID) != null)
                                        {
                                            LRRAggregateProperty propObj = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(lrrFormula.SourceOBJID);
                                            if (propObj != null)
                                            {
                                                LRRAggregatePropertyValue propValueObj = propObj.GetValueObjectForPropertyMode(lrrFormula.ModeID);
                                                if (propValueObj != null)
                                                {
                                                    curVal = propValueObj.GetPropertyValueForUOM(propValueObj.GetUOMID(), propObj.GetUIUOMID(), propValueObj.ActualPropertyValue);
                                                }
                                            }
                                        }
                                    }
                                }
                                else if (lrrFormula.SourceCollectioName == typeof(LRRHistData).ToString())
                                {
                                    if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                    {
                                        curVal = lrrFormula.CurrentValue;
                                    }
                                    else
                                    {
                                        //LRRHistDatas lrrhistDatas = _app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRHistDatas)) as LRRHistDatas;
                                        //Populate from Model
                                        LRRHistDatas lrrhistDatas = LRRModel.HistDatas;
                                        LRRHistData lrrhistData = lrrhistDatas.GetHistDataFromHistTagUID(lrrFormula.SourceOBJID);
                                        if (lrrhistData != null)
                                        {
                                            string uomID = lrrhistData.ModelUOM;
                                            string uiUOMId = lrrhistData.InterfaceUnits;
                                            curVal = ConvertUOM(uomID, uiUOMId, lrrhistData.InterfaceValue);
                                        }
                                    }
                                }
                                else if (lrrFormula.SourceCollectioName.Trim() == typeof(LRRSiteGroupPropertyQualifierValue).ToString().Trim())
                                {
                                    if (lrrFormula.SourceOBJID == null || lrrFormula.SourceOBJID.Trim() == "")
                                    {
                                        curVal = lrrFormula.CurrentValue;
                                    }
                                    else
                                    {
                                        if (LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrFormula.SourceOBJID) != null)
                                        {
                                            string sQualifierID = "";
                                            if ((lrrFormula.SourceLookUpQualifierID != null) && (lrrFormula.SourceLookUpQualifierID != ""))
                                                sQualifierID = LRRModel.ChildModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lrrFormula.SourceLookUpQualifierID).QualifierID;

                                            LRRSiteGroupPropertyQualifierValue lrrSiteCompGroupPropQVal = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrFormula.SourceOBJID).GroupPptiesQualifierValues.GetQualifierValueObjectForPropertyMode(lrrFormula.SourceOBJID, lrrFormula.SourceLookUpPropertyID, sQualifierID, lrrFormula.ModeID);
                                            if (lrrSiteCompGroupPropQVal != null)
                                            {
                                                string lkupQualID = lrrSiteCompGroupPropQVal.LookUpQualifierID;
                                                LRRLookUpQualifier lkupQualObj = null;
                                                if (lkupQualID != null && lkupQualID.Trim() != "")
                                                    lkupQualObj = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lkupQualID);
                                                //Group Property
                                                if (lkupQualObj != null)
                                                {
                                                    string uomTypeID = lkupQualObj.UOMTypeID;
                                                    //Get the UI UOM
                                                    LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                                    string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, uomTypeID);

                                                    curVal = ConvertUOM(lrrSiteCompGroupPropQVal.GetUOMID(), uiUOMID, lrrSiteCompGroupPropQVal.ActualPropertyValue);
                                                }
                                                else //Data Arrays
                                                {
                                                    LRRSiteComponentGroupProperty objGrpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(lrrSiteCompGroupPropQVal.GroupPropertyID);
                                                    if (objGrpProp != null)
                                                    {
                                                        string propType = objGrpProp.PropertyID;
                                                        LRRPropertyType propTypeObj = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(propType);
                                                        if (propTypeObj != null)
                                                        {
                                                            //Get the UI UOM
                                                            LRRMeasurementSystemDefinitions lrrMSDs = LRRModel.ChildModel.MeasSysDef;
                                                            string uiUOMID = lrrMSDs.GetLocaleUOM(LRRModel.ChildModel.LRRProjConfigObj.MeasurementSystemUserInterfaceID, propTypeObj.UOMTypeID);
                                                            curVal = ConvertUOM(lrrSiteCompGroupPropQVal.GetUOMID(), uiUOMID, lrrSiteCompGroupPropQVal.ActualPropertyValue);
                                                        }
                                                    }
                                                }
                                            }
                                        }

                                    }
                                }
                                else //If Constant
                                {
                                    curVal = lrrFormula.CurrentValue;
                                }
                            }

                            if (curVal == null || curVal.Trim() == "")
                                curVal = "0";

                            double res;
                            if (double.TryParse(curVal, out res))
                                newEq.Enqueue(new ExecutionItem(ItemType.itDigit, curVal));
                            else
                                newEq.Enqueue(new ExecutionItem(ItemType.itString, curVal));
                        }
                        else
                        {
                            newEq.Enqueue(ei);
                        }
                        //else
                        //{ 
                        //    int i=5;i
                        //}
                        break;
                    case ItemType.itFunction:
                        System.Collections.Specialized.StringCollection itParamsNew = new System.Collections.Specialized.StringCollection();
                        foreach (string itParam in ei.ItemParams)
                        {
                            itParamsNew.Add(CalculateFunctionsExpression(itParam, htFormulaParam));
                        }
                        ei.ItemParams = itParamsNew;
                        newEq.Enqueue(ei);
                        break;
                    default:
                        newEq.Enqueue(ei);
                        break;
                }
            }

                Variant var = Evaluator.CalcIt(newEq);
                return var.oValue.ToString();

        }

        /// <summary>
        /// Method converts value from one UOM to another
        /// </summary>
        /// <param name="fromUOMID"></param>
        /// <param name="newUOMID"></param>
        /// <param name="currentValue"></param>
        /// <returns></returns>
        private string ConvertUOM(string fromUOMID, string newUOMID, string currentValue)
        {
            string newVal = "";
            if (currentValue != null && currentValue != "")
            {
                if (fromUOMID == newUOMID)
                {
                    newVal = currentValue;
                }
                else
                {
                    if ((fromUOMID != null && fromUOMID != "") &&
                        (newUOMID != null && newUOMID != ""))
                    {
                        newVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(fromUOMID, newUOMID, currentValue, null);
                    }
                }
            }
            return newVal;
        }

        /// <summary>
        /// Saves the recorded values in the Recording Values DB
        /// </summary>
        /// <param name="startTime"></param>
        /// <param name="endTime"></param>
        public void SaveRecordingValues(string startTime, string endTime)
        {
            if (!this._app.RecordingValuesManagerObj.ExistsRecordingDB())
                return;
            //Save the PropertyValues Collection in Property Values User and Time table
            LRRSiteComponentPropertyValueChangeUserAndTimes propValueChanges = new LRRSiteComponentPropertyValueChangeUserAndTimes();
            LRRSiteComponentProperties lscps = LRRModel.SiteComponentPropertiesObj.GetChanges();
            //foreach (LRRSiteComponentProperty siteCompProperty in LRRModel.SiteComponentPropertiesObj)
            foreach (LRRSiteComponentProperty siteCompProperty in LRRModel.SiteComponentPropertiesObj)
            {
                if (siteCompProperty.IsTracked)
                {
                    LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange = new LRRSiteComponentPropertyValueChangeUserAndTime();
                    siteCompPropValueChange.RunID = _runID;
                    //siteCompPropValueChange.SiteComponentPropertyID = siteCompProperty.ID;
                    siteCompPropValueChange.SiteComponentPropertyID = siteCompProperty.Name;

                    LRRSiteComponent sc = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProperty.ComponentID);
                    if (sc != null)
                    {
                        siteCompPropValueChange.EquipmentTag = sc.Name;
                    }

                    //Record the property Values
                    foreach (LRRMode lrrMode in LRRModel.ProjectModes)
                    {
                        LRRSiteComponentPropertyValue siteCompPropValue = siteCompProperty.GetValueObjectForPropertyMode(lrrMode.ModeId);
                        if (siteCompPropValue != null)
                        {
                            //LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange = new LRRSiteComponentPropertyValueChangeUserAndTime();
                            try
                            {
                                if (lrrMode.ModeName == "Measurement")
                                {
                                    //siteCompPropValueChange.PropertyValue = siteCompPropValue.PropertyValue;
                                    siteCompPropValueChange.PropertyValue = siteCompPropValue.ActualPropertyValue;
                                }
                                if (lrrMode.ModeName == "SystemStatus")
                                {
                                    //siteCompPropValueChange.PropertyValue_SS = siteCompPropValue.PropertyValue;
                                    siteCompPropValueChange.PropertyValue_SS = siteCompPropValue.ActualPropertyValue;
                                }
                                if (lrrMode.ModeName == "Scenarios")
                                {
                                    //siteCompPropValueChange.PropertyValue_SCEN = siteCompPropValue.PropertyValue;
                                    siteCompPropValueChange.PropertyValue_SCEN = siteCompPropValue.ActualPropertyValue;
                                }
                                //LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(siteCompProperty.PropertyID);
                                //string masterPropertyName = masterProp.Name;

                                //LRRMasterComponent masterComp = LRRModel.ChildModel.MasterComponents.GetMasterComponentByID(masterProp.ComponentID);

                                //string masterCompName = masterComp.ComponentName;

                                //LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProperty.ComponentID);

                                //string siteCompName = siteComp.Name;

                                //string modeName = lrrMode.ModeName;

                                //string equipTag = siteComp.EquipmentTag;

                                //siteCompPropValueChange.MasterPlantItemName = masterCompName;
                                //siteCompPropValueChange.MasterPlantItemAttributeName = masterPropertyName;
                                //siteCompPropValueChange.PlantItemName = siteCompName;
                                //siteCompPropValueChange.ModeName = modeName;
                                //siteCompPropValueChange.EquipmentTag = equipTag;
                            }
                            catch
                            {

                            }
                            /*
                            siteCompPropValueChange.RunID = _runID;
                            siteCompPropValueChange.PropertyValue = siteCompPropValue.PropertyValue;
                            siteCompPropValueChange.SiteComponentPropertyID = siteCompPropValue.SiteComponentPropertyID;
                            siteCompPropValueChange.SiteComponentPropertyValueID = siteCompPropValue.ID;
                            siteCompPropValueChange.StartTime = startTime;
                            siteCompPropValueChange.EndTime = endTime;
                            //siteCompPropValueChange.IsRecorded = true;
                             
                            propValueChanges.Add(siteCompPropValueChange);
                             * */
                            siteCompPropValueChange.StartTime = startTime;
                            siteCompPropValueChange.EndTime = endTime;

                        }
                    }

                    siteCompPropValueChange.StartTime = startTime;
                    siteCompPropValueChange.EndTime = endTime;


                    propValueChanges.Add(siteCompPropValueChange);
                }
            }

            if (propValueChanges.Count > 0)
            {
                this._app.RecordingValuesManagerObj.BeginTransaction();
                _app.RecordingValuesManagerObj.ExecuteBatchInsert(propValueChanges);
                this._app.RecordingValuesManagerObj.CommitTransaction();
            }

            //Save the GroupPropertyQualifierValues Collection in Property Values User and Time table
            propValueChanges = new LRRSiteComponentPropertyValueChangeUserAndTimes();
            foreach (LRRSiteGroupPropertyQualifierValue siteGroupPropertyQualifierValue in LRRModel.SiteGroupPropertyQualifierValuesObj)
            {
                if (siteGroupPropertyQualifierValue.IsTracked)
                {
                    LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange = new LRRSiteComponentPropertyValueChangeUserAndTime();

                    try
                    {
                        LRRMode lrrMode = LRRModel.Modes.GetModeByID(siteGroupPropertyQualifierValue.ModeID);

                        LRRSiteComponentProperty siteCompProperty = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteGroupPropertyQualifierValue.SiteComponentPropID);

                        LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(siteCompProperty.PropertyID);
                        string masterPropertyName = masterProp.Name;

                        LRRMasterComponent masterComp = LRRModel.ChildModel.MasterComponents.GetMasterComponentByID(masterProp.ComponentID);

                        string masterCompName = masterComp.ComponentName;

                        LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProperty.ComponentID);

                        string siteCompName = siteComp.Name;

                        string modeName = lrrMode.ModeName;

                        string equipTag = siteComp.EquipmentTag;

                        siteCompPropValueChange.MasterPlantItemName = masterCompName;
                        siteCompPropValueChange.MasterPlantItemAttributeName = masterPropertyName;
                        siteCompPropValueChange.PlantItemName = siteCompName;
                        siteCompPropValueChange.ModeName = modeName;
                        siteCompPropValueChange.EquipmentTag = equipTag;

                        string lkUpPropertyID = siteGroupPropertyQualifierValue.LookUpPropertiesID;
                        string lookUpQualifierID = siteGroupPropertyQualifierValue.LookUpQualifierID;

                        LRRPropertyType propType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(masterProp.Type);
                        string propTypeName = propType.PropertyType;

                        LRRLookUpProperty lkupProp = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(lkUpPropertyID);

                        if (lookUpQualifierID != null) // Case of GPs
                        {
                            LRRLookUpQualifier lrrLkUpQual = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
                            if (lrrLkUpQual != null)
                            {
                                LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);
                                siteCompPropValueChange.GPAndDADetails = "[GP] - " + propTypeName + "/" + lkupProp.Name + "/" + qual.Name;
                            }
                        }
                        else // Case of DAs
                        {
                            siteCompPropValueChange.GPAndDADetails = "[DA] - " + propTypeName + "/" + lkupProp.Name;
                        }

                        //tag = masterCompName + "/" + masterPropertyName + "/" + siteCompName + "/" + modeName + "/" + equipTag;
                    }
                    catch
                    {

                    }

                    siteCompPropValueChange.RunID = _runID;
                    siteCompPropValueChange.PropertyValue = siteGroupPropertyQualifierValue.PropertyValue;
                    siteCompPropValueChange.SiteComponentPropertyID = siteGroupPropertyQualifierValue.SiteComponentPropID;
                    siteCompPropValueChange.SiteComponentPropertyValueID = siteGroupPropertyQualifierValue.SiteGroupPropertyQualifierValueID;
                    //siteCompPropValueChange.SiteComponentGroupedPropertyID = siteGroupPropertyQualifierValue.GroupPropertyID;
                    //siteCompPropValueChange.LookUpQualifierID = siteGroupPropertyQualifierValue.LookUpQualifierID;
                    //siteCompPropValueChange.LookUpPropertiesID = siteGroupPropertyQualifierValue.LookUpPropertiesID;
                    //siteCompPropValueChange.SiteComponentQualifierValuedID = siteGroupPropertyQualifierValue.SiteGroupPropertyQualifierValueID;
                    siteCompPropValueChange.IsRecorded = true;
                    siteCompPropValueChange.StartTime = startTime;
                    siteCompPropValueChange.EndTime = endTime;
                    //siteCompPropValueChange.UID = propertyValue
                    propValueChanges.Add(siteCompPropValueChange);
                }
            }

            if (propValueChanges.Count > 0)
            {
                this._app.RecordingValuesManagerObj.BeginTransaction();
                this._app.RecordingValuesManagerObj.ExecuteBatchInsert(propValueChanges);
                this._app.RecordingValuesManagerObj.CommitTransaction();
            }
        }

        /// <summary>
        /// Saves the recorded values in the Recording Values DB
        /// </summary>
        /// <param name="startTime"></param>
        /// <param name="endTime"></param>
        private void SaveTrendValues(string startTime, string endTime)
        {
            if (!this._app.RecordingValuesManagerObj.ExistsRecordingDB())
                return;
            //Save the PropertyValues Collection in Property Values User and Time table
            LRRSiteComponentPropertyValueChangeUserAndTimes propValueChanges = new LRRSiteComponentPropertyValueChangeUserAndTimes();
            foreach (LRRSiteComponentProperty siteCompProperty in LRRModel.SiteComponentPropertiesObj)
            {
                if (siteCompProperty.IsTrended)
                {
                    //Record the property Values
                    foreach (LRRMode lrrMode in LRRModel.ProjectModes)
                    {
                        LRRSiteComponentPropertyValue siteCompPropValue = siteCompProperty.GetValueObjectForPropertyMode(lrrMode.ModeId);
                        if (siteCompPropValue != null)
                        {
                            LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange = new LRRSiteComponentPropertyValueChangeUserAndTime();
                            try
                            {
                                //LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(siteCompProperty.PropertyID);
                                //string masterPropertyName = masterProp.Name;

                                //LRRMasterComponent masterComp = LRRModel.ChildModel.MasterComponents.GetMasterComponentByID(masterProp.ComponentID);

                                //string masterCompName = masterComp.ComponentName;

                                //LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProperty.ComponentID);

                                //string siteCompName = siteComp.Name;

                                string modeName = lrrMode.ModeName;

                                //string equipTag = siteComp.EquipmentTag;

                                //siteCompPropValueChange.MasterPlantItemName = masterCompName;
                                //siteCompPropValueChange.MasterPlantItemAttributeName = masterPropertyName;
                                //siteCompPropValueChange.PlantItemName = siteCompName;
                                siteCompPropValueChange.ModeName = modeName;
                                //siteCompPropValueChange.EquipmentTag = equipTag;

                            }
                            catch
                            {

                            }

                            siteCompPropValueChange.PropertyValue = siteCompPropValue.PropertyValue;
                            siteCompPropValueChange.SiteComponentPropertyID = siteCompPropValue.SiteComponentPropertyID;
                            siteCompPropValueChange.SiteComponentPropertyValueID = siteCompPropValue.ID;
                            siteCompPropValueChange.StartTime = startTime;
                            siteCompPropValueChange.EndTime = endTime;

                            propValueChanges.Add(siteCompPropValueChange);
                            if (TrendPointRecieved != null)
                                TrendPointRecieved(siteCompPropValueChange);
                        }
                    }
                }
            }

            if (propValueChanges.Count > 0)
            {
                this._app.RecordingValuesManagerObj.BeginTransaction();
                this._app.RecordingValuesManagerObj.ExecuteBatchInsert(propValueChanges);
                this._app.RecordingValuesManagerObj.CommitTransaction();
            }

            //Save the GroupPropertyQualifierValues Collection in Property Values User and Time table
            propValueChanges = new LRRSiteComponentPropertyValueChangeUserAndTimes();
            foreach (LRRSiteGroupPropertyQualifierValue siteGroupPropertyQualifierValue in LRRModel.SiteGroupPropertyQualifierValuesObj)
            {
                if (siteGroupPropertyQualifierValue.IsTrended)
                {
                    LRRSiteComponentPropertyValueChangeUserAndTime siteCompPropValueChange = new LRRSiteComponentPropertyValueChangeUserAndTime();


                    try
                    {
                        LRRMode lrrMode = LRRModel.Modes.GetModeByID(siteGroupPropertyQualifierValue.ModeID);

                        LRRSiteComponentProperty siteCompProperty = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(siteGroupPropertyQualifierValue.SiteComponentPropID);

                        LRRMasterProperty masterProp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(siteCompProperty.PropertyID);
                        string masterPropertyName = masterProp.Name;

                        LRRMasterComponent masterComp = LRRModel.ChildModel.MasterComponents.GetMasterComponentByID(masterProp.ComponentID);

                        string masterCompName = masterComp.ComponentName;

                        LRRSiteComponent siteComp = LRRModel.SiteComponentsObj.GetSiteComponentByID(siteCompProperty.ComponentID);

                        string siteCompName = siteComp.Name;

                        string modeName = lrrMode.ModeName;

                        string equipTag = siteComp.EquipmentTag;

                        siteCompPropValueChange.MasterPlantItemName = masterCompName;
                        siteCompPropValueChange.MasterPlantItemAttributeName = masterPropertyName;
                        siteCompPropValueChange.PlantItemName = siteCompName;
                        siteCompPropValueChange.ModeName = modeName;
                        siteCompPropValueChange.EquipmentTag = equipTag;

                        string lkUpPropertyID = siteGroupPropertyQualifierValue.LookUpPropertiesID;
                        string lookUpQualifierID = siteGroupPropertyQualifierValue.LookUpQualifierID;

                        LRRPropertyType propType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(masterProp.Type);
                        string propTypeName = propType.PropertyType;

                        LRRLookUpProperty lkupProp = LRRModel.LookUpProperties.getLookUpPropertyForLkupPropID(lkUpPropertyID);

                        if (lookUpQualifierID != null) // Case of GPs
                        {
                            LRRLookUpQualifier lrrLkUpQual = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualifierID);
                            if (lrrLkUpQual != null)
                            {
                                LRRQualifier qual = LRRModel.Qualifiers.getQualifierObjectForQualifierID(lrrLkUpQual.QualifierID);
                                siteCompPropValueChange.GPAndDADetails = "[GP] - " + propTypeName + "/" + lkupProp.Name + "/" + qual.Name;
                            }
                        }
                        else // Case of DAs
                        {
                            siteCompPropValueChange.GPAndDADetails = "[DA] - " + propTypeName + "/" + lkupProp.Name;
                        }

                        //tag = masterCompName + "/" + masterPropertyName + "/" + siteCompName + "/" + modeName + "/" + equipTag;
                    }
                    catch
                    {

                    }

                    //							siteCompPropValueChange.RunID = _runID;
                    siteCompPropValueChange.PropertyValue = siteGroupPropertyQualifierValue.PropertyValue;
                    siteCompPropValueChange.SiteComponentPropertyID = siteGroupPropertyQualifierValue.SiteComponentPropID;
                    siteCompPropValueChange.SiteComponentPropertyValueID = siteGroupPropertyQualifierValue.SiteGroupPropertyQualifierValueID;
                    //siteCompPropValueChange.SiteComponentGroupedPropertyID = siteGroupPropertyQualifierValue.GroupPropertyID;
                    //siteCompPropValueChange.LookUpQualifierID = siteGroupPropertyQualifierValue.LookUpQualifierID;
                    //siteCompPropValueChange.LookUpPropertiesID = siteGroupPropertyQualifierValue.LookUpPropertiesID;
                    //siteCompPropValueChange.SiteComponentQualifierValuedID = siteGroupPropertyQualifierValue.SiteGroupPropertyQualifierValueID;
                    siteCompPropValueChange.StartTime = startTime;
                    siteCompPropValueChange.EndTime = endTime;
                    //siteCompPropValueChange.UID = propertyValue
                    propValueChanges.Add(siteCompPropValueChange);
                    if (TrendPointRecieved != null)
                        TrendPointRecieved(siteCompPropValueChange);
                }
            }

            if (propValueChanges.Count > 0)
            {
                this._app.RecordingValuesManagerObj.BeginTransaction();
                this._app.RecordingValuesManagerObj.ExecuteBatchInsert(propValueChanges);
                this._app.RecordingValuesManagerObj.CommitTransaction();
            }
        }

        /// <summary>
        /// This method saves the property Values after the run cycle
        /// </summary>
        /// <param name="lrrSiteComponentPropValues"></param>
        /// <param name="lrrSiteGroupPropertyQualifierValues"></param>
        private void SaveChanges(LRRSiteComponentPropertyValues lrrSiteComponentPropValues, LRRSiteGroupPropertyQualifierValues lrrSiteGroupPropertyQualifierValues, LRRAggregatePropertyValues lrrAggregatePropertyValues)
        {
            try
            {
                //Start Time
                //Symcon.Utils.TimerUtils.HiPerfTimer hTimer = new Symcon.Utils.TimerUtils.HiPerfTimer();
                //hTimer.Start();
                Logger.LogMessage("End of Run Save Start Time:" + DateTime.Now.ToString(Symcon.BusinessObjects.SymConverter.LRR_DATE_TIME_FORMAT, Symcon.BusinessObjects.SymConverter.Culture), Logger.Level.Debug);
                if (lrrSiteComponentPropValues.Count > 0)
                {

                    this._app.DBManagerObj.BeginTransaction();
                    this._app.DBManagerObj.ExecuteBatchUpdate(lrrSiteComponentPropValues);
                    this._app.DBManagerObj.CommitTransaction();

                    //this._app.DBManagerObj.CommitCollection(lrrSiteComponentPropValues);
                }
                if (lrrSiteGroupPropertyQualifierValues.Count > 0)
                {

                    this._app.DBManagerObj.BeginTransaction();
                    this._app.DBManagerObj.ExecuteBatchUpdate(lrrSiteGroupPropertyQualifierValues);
                    this._app.DBManagerObj.CommitTransaction();

                    /*
                    this._app.DBManagerObj.CommitCollection(lrrSiteGroupPropertyQualifierValues);
                     * */
                }
                if (lrrAggregatePropertyValues.Count > 0)
                {

                    this._app.DBManagerObj.BeginTransaction();
                    this._app.DBManagerObj.ExecuteBatchUpdate(lrrAggregatePropertyValues);
                    this._app.DBManagerObj.CommitTransaction();

                    /*
                    this._app.DBManagerObj.CommitCollection(lrrAggregatePropertyValues);
                     * */
                }
                //End time
                Logger.LogMessage("End of Run End Time:" + DateTime.Now.ToString(Symcon.BusinessObjects.SymConverter.LRR_DATE_TIME_FORMAT, Symcon.BusinessObjects.SymConverter.Culture), Logger.Level.Debug);
                //Difference
                //hTimer.Stop();
                //Logger.LogMessage("End of Run Difference:" + hTimer.Duration.ToString(Symcon.BusinessObjects.SymConverter.Culture),Logger.Level.Debug);
                MessagingClient.CommitUpdates();
                /*
                for (int idx = lrrSiteComponentPropValues.Count - 1; idx >= 0; idx--)
                {
                    LRRSiteComponentPropertyValue lrrSiteComponentPropValue = lrrSiteComponentPropValues[idx];
                    if (lrrSiteComponentPropValue != null)
                    {
                        lrrSiteComponentPropValue.ArrListUpdatedColNames.Clear();
                        lrrSiteComponentPropValues.Remove(lrrSiteComponentPropValue);
                        lrrSiteComponentPropValue = null;
                    }
                } 
                 * */
                lrrSiteComponentPropValues.ClearCache();
                lrrSiteComponentPropValues.Clear();
                lrrSiteComponentPropValues = null;
                /*
                for (int idx = lrrSiteGroupPropertyQualifierValues.Count - 1; idx >= 0; idx--)
                {
                    LRRSiteGroupPropertyQualifierValue lrrSiteGroupPropertyQualifierValue = lrrSiteGroupPropertyQualifierValues[idx];
                    if (lrrSiteGroupPropertyQualifierValue != null)
                    {
                        lrrSiteGroupPropertyQualifierValue.ArrListUpdatedColNames.Clear();
                        lrrSiteGroupPropertyQualifierValues.Remove(lrrSiteGroupPropertyQualifierValue);
                        lrrSiteGroupPropertyQualifierValue = null;
                    }
                }
                 * */
                lrrSiteGroupPropertyQualifierValues.ClearCache();
                lrrSiteGroupPropertyQualifierValues.Clear();
                lrrSiteGroupPropertyQualifierValues = null;
                /*
                for (int idx = lrrAggregatePropertyValues.Count - 1; idx >= 0; idx--)
                {
                    LRRAggregatePropertyValue lrrAggregatePropertyValue = lrrAggregatePropertyValues[idx];
                    if (lrrAggregatePropertyValue != null)
                    {
                        lrrAggregatePropertyValue.ArrListUpdatedColNames.Clear();
                        lrrAggregatePropertyValues.Remove(lrrAggregatePropertyValue);
                        lrrAggregatePropertyValue = null;
                    }
                }
                 * */
                lrrAggregatePropertyValues.ClearCache();
                lrrAggregatePropertyValues.Clear();
                lrrAggregatePropertyValues = null;
                //Refresh Collections

                LRRModel.PropertyValuesObj.RefreshCollection();
                LRRModel.SiteGroupPropertyQualifierValuesObj.RefreshCollection();
                LRRModel.AggregatePropertyValuesObj.RefreshCollection();

            }
            catch
            {
            }
            finally
            {
                //_mutex.ReleaseMutex();
            }
        }

        #endregion

        #region Fields


        private LRRApp _app; //Instance of the application object.

        private Hashtable _historianConnections; //Connection Infos for the various Hostorians...

        private OnLineHistorian _onLineHistorian; //online
        private OffLineHistorian _offLineHistorian; //offline
        private TuningHistorian _tuningHistorian; //tuning

        private int _activeHistorianIndex; //
        private LRROptions _lrrOptions;
        private string _currentDatasetID;
        private LRRHistorianPoints _currentDataset;

        private System.Threading.Timer _poller;

        private Queue _pollerQueue = new Queue();

        private RunControlSet _runControlSet;

        //Error Handling

        private string _erroMessage;

        private LRRHistSavedSet _savedSet;
        private LRRHistSavedSet _savedSetForOffline;
        private LRRHistorianPoints _savedPoints;
        private LRRHistorianPoints _savedPointsForOffline;

        private string _runID = null;

        private LRRFormulas _lrrFormulas;
        private LRRFormulaParams _lrrFormulaParams;
        private Hashtable htSiteCompPropValues = new Hashtable();

        private bool _bRunID = false;

        //Tuning Database Run Counter
        private int _tuningRunCounter;
        private Random _tuningRandomCounter = new Random(1);
        private int _tuningRandomIndex;
        private LRRHistSavedSets _lrrSavedSets;
        private LRRLogin _lrrLogin;

        //Offline Database Run Counter
        private int _offlineRunCounter;
        private Random _offlineRandomCounter = new Random(1);
        private int _offlineRandomIndex;

        private bool _goodToComplete = false;
        private bool _runStarted = false;
        private bool _getPointsInProgress = false;
        private bool _processingPoint = false;
        private bool _inUIThread = false;
        private bool _stopRunSequence = false;
        private string _currentStartTime = "";
        private string _currentEndTime = "";

        public string currentStartTime = "";
        public string currentEndTime = "";

        private System.Threading.Mutex _mutex = new System.Threading.Mutex();
        private Symcon.LightRidge.PEAdvisor.SubscriberUpdater _subs = new SubscriberUpdater();
        private bool _isSetAlreadyPresent = false;
        private bool _isSetAlreadyPresentForOffline = false;

        private ArrayList _arrOldHistTags = null;
        private ArrayList _arrOldHistTagsForOffline = null;

        private Symcon.Utils.MessageLogger.Logger _histWriteLog = new Symcon.Utils.MessageLogger.Logger();
        private string _strRuntType = "";
        RunMessage _currentRunMsg;
        private int _runningScriptCounter = 0;

        #endregion


        #region Events

        public event EventHandler RunCompleted;
        public event EventHandler RunStopped;
        public event ReceivedPointDataDel ReceivedPoint;
        public event RunStartedDel RunStarted;
        public event GiveFeedbackDel FeedBack;
        public event EventHandler RunCycleCompleted;
        public event TrendPointRecievedDel TrendPointRecieved;


        #endregion

        private bool allowEvents = true;
        public bool SuspendEvents()
        {
            allowEvents = false;
            return allowEvents;
        }

        public bool ResumeEvents()
        {
            allowEvents = true;
            return allowEvents;
        }

        private void IternalRaiseGiveFeedback(string timeRemaining)
        {
            if(allowEvents)
                _app.MainFrame.Invoke(new GiveFeedbackDel(RaiseGiveFeedback), new object[] { timeRemaining });

        }

        private void RaiseGiveFeedback(string timeRemaining)
        {
            //_app.LRRUIManagerObj.
            if (FeedBack != null)
            {
                FeedBack(timeRemaining);
            }
        }


        private void IternalRaiseRunStarted(RunStartedEventArgs e)
        {
            if (allowEvents)
            _app.MainFrame.Invoke(new RunStartedDel(RaiseRunStarted), new object[] { e });

        }

        private void RaiseRunStarted(RunStartedEventArgs e)
        {
            if (RunStarted != null)
            {
                RunStarted(e);
            }
        }

        private void IternalRaiseReceivedPoint(LRRHistData point, string startTime, string endTime)
        {
            if (allowEvents)
            _app.MainFrame.Invoke(new ReceivedPointDataDel(RaiseReceivedPoint), new object[] { point, startTime, endTime });

        }

        private void RaiseReceivedPoint(LRRHistData point, string startTime, string endTime)
        {
            if (ReceivedPoint != null)
            {
                ReceivedPoint(point, startTime, endTime);
            }
        }

        private void IternalRaiseRunCycleCompleted(object sender, EventArgs e)
        {

            _runningScriptCounter = 0;
            if (allowEvents)
            _app.MainFrame.Invoke(new RunCycleCompletedDel(RaiseRunCycleCompleted), new object[] { sender, e });
            //this._app.SetPropertyHistorianBinding();

        }

        private void RaiseRunCycleCompleted(object sender, EventArgs e)
        {
            if (RunCycleCompleted != null)
            {
                RunCycleCompleted(sender, e);
            }

            if (_savedSet != null)
            {
                try
                {
                    if (_runControlSet.SaveDataSets)
                    {

                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                        {
                            Symcon.LightRidge.HistorianConnect.IHistorian tuningHist = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING] as IHistorian;
                            tuningHist.SetPoints(_savedSet, _savedPoints, null);
                            _savedPoints.Clear();
                            _savedPoints = null;
                            GC.Collect();
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.LogMessage(typeof(RunController), "RunCallBack :" + ex.Message, Logger.Level.Debug);
                }
            }
            if (_savedSetForOffline != null)
            {
                try
                {
                    if (_runControlSet.SaveToOfflineHistorian)
                    {

                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                        {
                            Symcon.LightRidge.HistorianConnect.IHistorian offLineHist = _app.LoadedOffLineHistorianAssemblies[_app.OFFLINE] as IHistorian;
                            offLineHist.SetPoints(_savedSetForOffline, _savedPointsForOffline, null);
                            _savedPointsForOffline.Clear();
                            _savedPointsForOffline = null;
                            GC.Collect();
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.LogMessage(typeof(RunController), "RunCallBack :" + ex.Message, Logger.Level.Debug);
                }
            }

            
        }


        private void IternalRaiseRunStopped(object sender, EventArgs e)
        {
            if (allowEvents)
            _app.MainFrame.Invoke(new RunStoppedDel(RaiseRunStopped), new object[] { sender, e });

        }

        private void RaiseRunStopped(object sender, EventArgs e)
        {
            if (RunStopped != null)
            {
                RunStopped(sender, null);
            }
        }

        private void InternalRaiseRunCompleted(object sender, EventArgs e)
        {
            if (allowEvents)
            _app.MainFrame.Invoke(new RunCompletedDel(RaiseRunCompleted), new object[] { sender, e });
        }

        private void RaiseRunCompleted(object sender, EventArgs e)
        {
            if (RunCompleted != null)
            {
                RunCompleted(sender, e);
            }
        }


        private void timer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            if (_runQueue.Count > 0)
            {
                RunQueue runQueue = _runQueue.Dequeue() as RunQueue;
                LRRHistData histData = runQueue.histData;
                LRRHistorianPoint histPoint = runQueue.histPoint;

                histData.SpotValue = histPoint.SpotValue;
                histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
                histData.StartOfPeriodValue = histPoint.StartOfPeriodValue;
                histData.EndOfPeriodValue = histPoint.EndOfPeriodValue;
            }
        }

        private string GetTimeStamp(DateTime starttime, int datawindowsize, string datawindowunits)
        {
            int midPoint = datawindowsize / 2;
            TimeSpan span = new TimeSpan();
            if (datawindowunits == "Min")
            {
                span = new TimeSpan(00, midPoint, 00);
            }
            else if (datawindowunits == "Sec")
            {
                span = new TimeSpan(00, 00, midPoint);
            }
            else if (datawindowunits == "Hr")
            {
                span = new TimeSpan(midPoint, 00, 00);
            }

            DateTime midDateTime = starttime.Add(span);



            return midDateTime.ToString(Symcon.BusinessObjects.SymConverter.LRR_DATE_TIME_FORMAT, Symcon.BusinessObjects.SymConverter.Culture);

        }

        [System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
        private void timer_Tick(object sender, EventArgs e)
        {
            try
            {
                //				if(_getPointsInProgress)
                //					return;
                InUIThread = true;
                bool bPostCalc = false;
                string startTime = null;
                string endTime = null;
                Dictionary<LRRHistData, LRRHistorianPoint> hstHstDatasHistPointForOffline = new Dictionary<LRRHistData, LRRHistorianPoint>();
                List<LRRHistorianPoint> lstHstPointsForOffline = new List<LRRHistorianPoint>();
                while (_runQueue.Count > 0)
                {
                    _processingPoint = true;
                    _pauseFlag1 = true;
                    RunQueue runQueue = _runQueue.Dequeue() as RunQueue;
                    LRRHistData histData = runQueue.histData;
                    LRRHistorianPoint histPoint = runQueue.histPoint;
                    if (_runControlSet.SaveToOfflineHistorian)
                    {
                        hstHstDatasHistPointForOffline.Add(histData, histPoint);
                    }

                    ProcessHistPointsForTuning(ref bPostCalc, ref startTime, ref endTime, histData, histPoint);
                }

                if (bPostCalc)
                {
                    InitiatePostExtraction();

                    _app.LRRCascadeBindingManagerObj.CallCascadeBindings(false, true);

                    //Check for the post calc only if there needs to be a run happening...
                    //					InitiatePreCalc();


                    foreach (KeyValuePair<LRRHistData, LRRHistorianPoint> var in hstHstDatasHistPointForOffline)
                    {
                        ProcessHistPointsForOfflineHistorian(ref bPostCalc, ref startTime, ref endTime, var.Key, var.Value);

                    }

                    //Start the Execution Engine
                    if (!_runControlSet.SaveDataSets)
                    {
                        try
                        {
                            _runControlSet.RebuildWaterModel = LRRModel.RebuildWaterModel;
                            //MessageBox.Show(LRRModel.RebuildWaterModel.ToString());
                            LRRModel.RebuildWaterModel = false;
                            _app.StartExecutionEngine();
                        }
                        catch { }

                    }
                    else
                    {
                        EndRunCycle();
                    }
                }

            }
            catch (Exception ex)
            {
                _pollerQueue.Clear();
                _getPointsInProgress = false;
                _processingPoint = false;
                _pauseFlag1 = false;

                //if(RunCompleted != null)
                //    RunCompleted(this,new EventArgs());
                this.InternalRaiseRunCompleted(this, null);
                Logger.LogMessage(typeof(RunController), "timer_tick :" + ex.Message, Logger.Level.Debug);
            }

        }

        private void ProcessHistPointsForTuning(ref bool bPostCalc, ref string startTime, ref string endTime, LRRHistData histData, LRRHistorianPoint histPoint)
        {

            
            // Removing Spot value from Tuning and copying end of period value as spot value.
            if (_runControlSet.DataSourceType.ToUpper() == LRRHeaderText.GetHeaderString("SID_ONLINE").ToUpper())
                histData.SpotValue = histPoint.SpotValue;
            else
                histData.SpotValue = histPoint.EndOfPeriodValue;

            if ((histData.MeasType == "C" || histData.MeasType == "N") && (histData.StartOfPeriodValue != "Timeout Caused") && (histData.EndOfPeriodValue != "Timeout Caused") && (histData.AverageValueForPeriodEntered != "Timeout Caused") )
            {
                try
                {
                    if (histPoint.EndOfPeriodValue != null && histPoint.EndOfPeriodValue != string.Empty && histPoint.StartOfPeriodValue != null && histPoint.StartOfPeriodValue != string.Empty)
                    {
                        double endOfPeriodValue;
                        double startOfPeriodValue;
                        if (double.TryParse(histPoint.EndOfPeriodValue, out endOfPeriodValue) && double.TryParse(histPoint.StartOfPeriodValue, out startOfPeriodValue))
                        {
                            histData.AverageValueForPeriodEntered = ((endOfPeriodValue - startOfPeriodValue) / ((DateTime.Parse(histPoint.EndTime)) - (DateTime.Parse(histPoint.StartTime))).TotalHours).ToString();
                        }
                        else
                        {
                            histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
                        }
                    }
                    else
                    {
                        histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
                    }
                }
                catch
                {
                    histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
                }
            }
            else
            {
                histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
            }
            histData.StartOfPeriodValue = histPoint.StartOfPeriodValue;
            histData.EndOfPeriodValue = histPoint.EndOfPeriodValue;
            histData.TimeStamp = GetTimeStamp(_runControlSet.DataWindowObj.StartTime, _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
            string[] ret;
            //if(_runControlSet.DataSourceType == "Tuning")
            //{
            //	ret = HistorianConnect.HistorianUtilities.SmartDataFilter("H",histPoint.StartTime,histPoint.EndTime,histData);
            //}
            //else
            //{


            ret = HistorianConnect.HistorianUtilities.SmartDataFilter("C", histPoint.StartTime, histPoint.EndTime, histData);

            //}
            if (histData != null)
            {

                histData.LastValidAverageValue = ret[1]; //Assign the Last Valid
                histData.ActiveFilterReturnValue = ret[1]; //Assign Active Filter Return Value...
                histData.MeterOnOffStatusFlag = ret[2];
                histData.DataQuality = ret[0];

                string finalInterfaceValue = ret[1];



                if (histData.MeterOnOffStatusFlag != "0")
                {
                    if (histData.MeasType != "S" && histData.MeasType != "D") //Its a Number so apply Caliberation Factor..
                    {
                        double activeValue;
                        double calibrationFactor;
                        double result;
                        bool containsPreFilterOverides = false;
                        if (histData.PreFilterUserOverride != null && histData.PreFilterUserOverride.Trim() != "") //Check for the Pre Filter Overides....
                        {
                            Double.TryParse(histData.AverageValueForPeriodEntered, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out activeValue);
                            containsPreFilterOverides = true;
                        }
                        else
                        {
                            Double.TryParse(histData.ActiveFilterReturnValue, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out activeValue);
                        }
                        // commented by ashwin on account of rounding off of calibration factor while parsing
                        //Double.TryParse(histData.CalibrationFactor, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out calibrationFactor);
                        calibrationFactor = Double.Parse(histData.CalibrationFactor, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture);
                        result = activeValue * calibrationFactor;
                        histData.FinalValueWithoutOverrides = result.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                        if (containsPreFilterOverides) //Calculte the finalInterfaceValue seperately
                        {
                            Double.TryParse(histData.ActiveFilterReturnValue, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out activeValue);
                            result = activeValue * calibrationFactor;
                            finalInterfaceValue = result.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                        }
                        else
                        {
                            finalInterfaceValue = histData.FinalValueWithoutOverrides;
                        }
                    }
                }
                else //Using Off Condition Subsitute Value...
                {
                    if (histData.MeasType != "S" && histData.MeasType != "D") //Its a Number so apply Caliberation Factor..
                    {
                        double activeValue;
                        double calibrationFactor;
                        double result;
                        Double.TryParse(histData.OffConditionSubstituteValue, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out activeValue);
                        Double.TryParse(histData.CalibrationFactor, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out calibrationFactor);
                        result = activeValue * calibrationFactor;
                        histData.FinalValueWithoutOverrides = result.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                        finalInterfaceValue = result.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                    }
                }

                //Apply Post Filter Override to the final Value.....
                if (histData.PostFilterUserOverride != null && histData.PostFilterUserOverride.Trim() != "")
                {
                    if (histData.ModelUOM == null || histData.ModelUOM == string.Empty)
                    {
                        histData.ModelUOM = null;
                        if (histData.InterfaceUnits == null || histData.InterfaceUnits == string.Empty)
                        {
                            histData.InterfaceUnits = null;
                        }
                    }
                    if(histData.ModelUOM == histData.InterfaceUnits)
                        histData.InterfaceValue = histData.PostFilterUserOverride;
                    else
                        histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histData.InterfaceUnits, histData.ModelUOM, histData.PostFilterUserOverride, null);

               }
                else
                {
                    if (histData.MeasType == "N" || histData.MeasType == "C")
                    {
                        // change added by ashwin. Skips all the processing if the conversion formula exists for the tag
                        if (histData.ConversionFormulaID == null || histData.ConversionFormulaID != "")
                        {
                            if (histData.HistUnits != null && histData.HistUnits != "")
                            {
                                LRRUOM histUomForConversion = null;
                                string histUomName = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(histData.HistUnits).Name;
                                if (histUomName.EndsWith("h"))
                                {
                                    string newUom = histUomName.Remove(histUomName.LastIndexOf('h'));
                                    List<string> l1 = LRRModel.ChildModel.UOMTypeXRefs.GetUOMIDsFromUOMTypeID(histData.UOMTypeID);
                                    foreach (string s in l1)
                                    {
                                        LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(s);
                                        if (uom.Name == newUom)
                                        {
                                            histUomForConversion = uom;
                                            break;
                                        }
                                    }
                                    if (histUomForConversion != null)
                                    {
                                        //if (histUomForConversion.ID != histData.InterfaceUnits)
                                        if (histUomForConversion.ID != histData.ModelUOM)
                                        {
                                            bool ucfFound = LRRModel.ChildModel.UCFs.UCFCombHash.ContainsKey(histUomForConversion.ID + "|" + histData.InterfaceUnits);
                                            histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histUomForConversion.ID, histData.ModelUOM, finalInterfaceValue, false, null);
                                            if (!ucfFound)
                                                histData.ErrorStatus = "UCF not found.";
                                        }
                                        else
                                        {
                                            // histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                            histData.InterfaceValue = finalInterfaceValue;
                                        }
                                    }
                                    else
                                    {
                                        histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                    }
                                }
                                else if (histUomName.EndsWith("H"))
                                {
                                    string newUom = histUomName.Remove(histUomName.LastIndexOf('H'));
                                    List<string> l1 = LRRModel.ChildModel.UOMTypeXRefs.GetUOMIDsFromUOMTypeID(histData.UOMTypeID);
                                    foreach (string s in l1)
                                    {
                                        LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(s);
                                        if (uom.Name == newUom)
                                        {
                                            histUomForConversion = uom;
                                            break;
                                        }
                                    }
                                    if (histUomForConversion != null)
                                    {
                                        //if (histUomForConversion.ID != histData.InterfaceUnits)
                                        if (histUomForConversion.ID != histData.ModelUOM)
                                        {
                                            bool ucfFound = LRRModel.ChildModel.UCFs.UCFCombHash.ContainsKey(histUomForConversion.ID + "|" + histData.InterfaceUnits);
                                            histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histUomForConversion.ID, histData.ModelUOM, finalInterfaceValue, false, null);
                                            if (!ucfFound)
                                                histData.ErrorStatus = "UCF not found.";
                                        }
                                        else
                                        {
                                            histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                        }
                                    }
                                    else
                                    {
                                        histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                    }
                                }
                                else
                                {
                                    histUomName = histUomName + "/hr";
                                    List<string> l1 = LRRModel.ChildModel.UOMTypeXRefs.GetUOMIDsFromUOMTypeID(histData.UOMTypeID);
                                    foreach (string s in l1)
                                    {
                                        LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(s);
                                        if (uom.Name == histUomName)
                                        {
                                            histUomForConversion = uom;
                                            break;
                                        }
                                    }
                                    if (histUomForConversion != null)
                                    {
                                        //if (histUomForConversion.ID != histData.InterfaceUnits)
                                        if (histUomForConversion.ID != histData.ModelUOM)
                                        {
                                            bool ucfFound = LRRModel.ChildModel.UCFs.UCFCombHash.ContainsKey(histUomForConversion.ID + "|" + histData.InterfaceUnits);
                                            histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histUomForConversion.ID, histData.ModelUOM, finalInterfaceValue, false, null);
                                            if (!ucfFound)
                                                histData.ErrorStatus = "UCF not found.";
                                        }
                                        else
                                        {
                                            //histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                            //commented by ashwin
                                            histData.InterfaceValue = finalInterfaceValue;
                                        }
                                    }
                                    else
                                    {
                                        //histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                        //commented by ashwin
                                        histData.InterfaceValue = finalInterfaceValue;
                                    }
                                }
                            }
                        }
                        else
                        {
                            LRRFormula lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(histData.ConversionFormulaID);
                            if (lrrFormula != null)
                            {
                                if (lrrFormula.Expression.Trim() != "")
                                {
                                    if (lrrFormula.Params != null)
                                    {
                                        //new method for calculating conversion formula added by ashwin
                                        //histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.Calculate(lrrFormula.Params, lrrFormula.Expression, finalInterfaceValue);
                                        histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.CalculateHistorianConversionFormula(lrrFormula.Params, lrrFormula.Expression, lrrFormula);
                                        //applyUCF = false;
                                    }

                                }
                            }

                        }

                        // till
                    }
                    else
                    {

                        if (histData.HistUOMTypeID != null && histData.UOMTypeID != null)
                        {
                            if (histData.UOMTypeID == histData.HistUOMTypeID)
                            {
                                if (histData.HistUOMTypeID != "" && histData.UOMTypeID != "")
                                {
                                    if (histData.HistUnits != null && histData.ModelUOM != null)
                                    {
                                        //check if COnversion Formula is existing for this Hist Point

                                        //string modelUOMID =  LRRModel.ChildModel.GetUOMIDFromName(histData.ModelUOM);
                                        if (histData.HistUnits == histData.ModelUOM)
                                        {
                                            histData.InterfaceValue = finalInterfaceValue;
                                        }
                                        else //Need to apply the UCF...
                                        {
                                            bool applyUCF = true;
                                            if (histData.ConversionFormulaID != null && histData.ConversionFormulaID != "")
                                            {
                                                LRRFormula lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(histData.ConversionFormulaID);
                                                if (lrrFormula != null)
                                                {
                                                    if (lrrFormula.Expression.Trim() != "")
                                                    {
                                                        if (lrrFormula.Params != null)
                                                        {
                                                            //new method for calculating conversion formula added by ashwin
                                                            //histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.Calculate(lrrFormula.Params, lrrFormula.Expression, finalInterfaceValue);
                                                            histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.CalculateHistorianConversionFormula(lrrFormula.Params, lrrFormula.Expression, lrrFormula);
                                                            applyUCF = false;
                                                        }

                                                    }
                                                }

                                            }
                                            if (applyUCF)
                                            {
                                                histData.ErrorStatus = "";
                                                bool ucfFound = LRRModel.ChildModel.UCFs.UCFCombHash.ContainsKey(histData.HistUnits + "|" + histData.ModelUOM);
                                                histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histData.HistUnits, histData.ModelUOM, finalInterfaceValue, false, null);
                                                if (!ucfFound)
                                                    histData.ErrorStatus = "UCF not found.";
                                            }
                                        }
                                    }
                                    else
                                    {
                                        histData.InterfaceValue = finalInterfaceValue;
                                        // commented by ashwin for handling rounding off of calibratin factor
                                        //histData.InterfaceValue = ret[1];
                                    }
                                }
                            }
                            else
                            {
                                histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                            }
                        }
                    }
                }
            }

            startTime = histPoint.StartTime;
            endTime = histPoint.EndTime;
            _currentStartTime = startTime;
            _currentEndTime = endTime;

            //ReceivedPoint.BeginInvoke(histData,histPoint.StartTime,histPoint.EndTime,null,null);
            bPostCalc = true;

            if (_runControlSet.SaveDataSets)
            {
                //Save the dataSet
                if ((_arrOldHistTags == null) || (!_isSetAlreadyPresent) || (!_arrOldHistTags.Contains(histData.HistTag)))
                {
                    LRRHistorianPoint savedPoint = new LRRHistorianPoint();
                    savedPoint.HistTag = histData.HistTag;
                    savedPoint.DataSetID = _savedSet.ID;
                    savedPoint.StartOfPeriodValue = histData.StartOfPeriodValue;
                    savedPoint.EndOfPeriodValue = histData.EndOfPeriodValue;
                    savedPoint.SpotValue = histData.SpotValue;
                    savedPoint.AverageValueForPeriodEntered = histData.AverageValueForPeriodEntered;
                    savedPoint.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
                    _savedPoints.Add(savedPoint);
                }
                //End
            }

        }


        private void ProcessHistPointsForOfflineHistorian(ref bool bPostCalc, ref string startTime, ref string endTime, LRRHistData histData, LRRHistorianPoint histPoint)
        {



            // Removing Spot value from Tuning and copying end of period value as spot value.
            if (_runControlSet.DataSourceType.ToUpper() == LRRHeaderText.GetHeaderString("SID_ONLINE").ToUpper())
                histData.SpotValue = histPoint.SpotValue;
            else
                histData.SpotValue = histPoint.EndOfPeriodValue;

            if ((histData.MeasType == "C" || histData.MeasType == "N") /*&& (_runControlSet.DataSourceType != "Offline")*/)
            {
                try
                {
                    if (histPoint.EndOfPeriodValue != null && histPoint.EndOfPeriodValue != string.Empty && histPoint.StartOfPeriodValue != null && histPoint.StartOfPeriodValue != string.Empty)
                    {
                        double endOfPeriodValue;
                        double startOfPeriodValue;
                        if (double.TryParse(histPoint.EndOfPeriodValue, out endOfPeriodValue) && double.TryParse(histPoint.StartOfPeriodValue, out startOfPeriodValue))
                        {
                            histData.AverageValueForPeriodEntered = ((endOfPeriodValue - startOfPeriodValue) / ((DateTime.Parse(histPoint.EndTime)) - (DateTime.Parse(histPoint.StartTime))).TotalHours).ToString();
                        }
                        else
                        {
                            histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
                        }
                    }
                    else
                    {
                        histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
                    }
                }
                catch
                {
                    histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
                }
            }
            else
            {
                histData.AverageValueForPeriodEntered = histPoint.AverageValueForPeriodEntered;
            }
            histData.StartOfPeriodValue = histPoint.StartOfPeriodValue;
            histData.EndOfPeriodValue = histPoint.EndOfPeriodValue;
            histData.TimeStamp = GetTimeStamp(_runControlSet.DataWindowObj.StartTime, _runControlSet.DataWindowObj.DataWindowSize, _runControlSet.DataWindowObj.DataWindowUnits);
            string[] ret;
            //if(_runControlSet.DataSourceType == "Tuning")
            //{
            //	ret = HistorianConnect.HistorianUtilities.SmartDataFilter("H",histPoint.StartTime,histPoint.EndTime,histData);
            //}
            //else
            //{
            ret = HistorianConnect.HistorianUtilities.SmartDataFilter("C", histPoint.StartTime, histPoint.EndTime, histData);

            //}
            if (histData != null)
            {

                histData.LastValidAverageValue = ret[1]; //Assign the Last Valid
                histData.ActiveFilterReturnValue = ret[1]; //Assign Active Filter Return Value...
                histData.MeterOnOffStatusFlag = ret[2];
                histData.DataQuality = ret[0];

                string finalInterfaceValue = ret[1];


                if (histData.MeterOnOffStatusFlag != "0")
                {
                    if (histData.MeasType != "S" && histData.MeasType != "D") //Its a Number so apply Caliberation Factor..
                    {
                        double activeValue;
                        double calibrationFactor;
                        double result;
                        bool containsPreFilterOverides = false;
                        if (histData.PreFilterUserOverride != null && histData.PreFilterUserOverride.Trim() != "") //Check for the Pre Filter Overides....
                        {
                            Double.TryParse(histData.AverageValueForPeriodEntered, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out activeValue);
                            containsPreFilterOverides = true;
                        }
                        else
                        {
                            Double.TryParse(histData.ActiveFilterReturnValue, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out activeValue);
                        }
                        Double.TryParse(histData.CalibrationFactor, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out calibrationFactor);
                        result = activeValue * calibrationFactor;
                        histData.FinalValueWithoutOverrides = result.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                        if (containsPreFilterOverides) //Calculte the finalInterfaceValue seperately
                        {
                            Double.TryParse(histData.ActiveFilterReturnValue, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out activeValue);
                            result = activeValue * calibrationFactor;
                            finalInterfaceValue = result.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                        }
                        else
                        {
                            finalInterfaceValue = histData.FinalValueWithoutOverrides;
                        }
                    }
                }
                else //Using Off Condition Subsitute Value...
                {
                    if (histData.MeasType != "S" && histData.MeasType != "D") //Its a Number so apply Caliberation Factor..
                    {
                        double activeValue;
                        double calibrationFactor;
                        double result;
                        Double.TryParse(histData.OffConditionSubstituteValue, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out activeValue);
                        Double.TryParse(histData.CalibrationFactor, System.Globalization.NumberStyles.Number, Symcon.BusinessObjects.SymConverter.Culture, out calibrationFactor);
                        result = activeValue * calibrationFactor;
                        histData.FinalValueWithoutOverrides = result.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                        finalInterfaceValue = result.ToString(Symcon.BusinessObjects.SymConverter.Culture);
                    }
                }

                //Apply Post Filter Override to the final Value.....
                if (histData.PostFilterUserOverride != null && histData.PostFilterUserOverride.Trim() != "")
                {
                    if (histData.ModelUOM == histData.InterfaceUnits)
                        histData.InterfaceValue = histData.PostFilterUserOverride;
                    else
                        histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histData.InterfaceUnits, histData.ModelUOM, histData.PostFilterUserOverride, null);
                }
                else
                {
                    if (histData.MeasType == "N" || histData.MeasType == "C")
                    {
                        if (histData.HistUnits != null && histData.HistUnits != "")
                        {
                            LRRUOM histUomForConversion = null;
                            string histUomName = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(histData.HistUnits).Name;
                            if (histUomName.EndsWith("h"))
                            {
                                string newUom = histUomName.Remove(histUomName.LastIndexOf('h'));
                                List<string> l1 = LRRModel.ChildModel.UOMTypeXRefs.GetUOMIDsFromUOMTypeID(histData.UOMTypeID);
                                foreach (string s in l1)
                                {
                                    LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(s);
                                    if (uom.Name == newUom)
                                    {
                                        histUomForConversion = uom;
                                        break;
                                    }
                                }
                                if (histUomForConversion != null)
                                {
                                    //if (histUomForConversion.ID != histData.InterfaceUnits)
                                    if (histUomForConversion.ID != histData.ModelUOM)
                                    {
                                        bool ucfFound = LRRModel.ChildModel.UCFs.UCFCombHash.ContainsKey(histUomForConversion.ID + "|" + histData.InterfaceUnits);
                                        histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histUomForConversion.ID, histData.ModelUOM, finalInterfaceValue, false, null);
                                        if (!ucfFound)
                                            histData.ErrorStatus = "UCF not found.";
                                    }
                                    else
                                    {
                                        // histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                        histData.InterfaceValue = finalInterfaceValue;
                                    }
                                }
                                else
                                {
                                    histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                }
                            }
                            else if (histUomName.EndsWith("H"))
                            {
                                string newUom = histUomName.Remove(histUomName.LastIndexOf('H'));
                                List<string> l1 = LRRModel.ChildModel.UOMTypeXRefs.GetUOMIDsFromUOMTypeID(histData.UOMTypeID);
                                foreach (string s in l1)
                                {
                                    LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(s);
                                    if (uom.Name == newUom)
                                    {
                                        histUomForConversion = uom;
                                        break;
                                    }
                                }
                                if (histUomForConversion != null)
                                {
                                    //if (histUomForConversion.ID != histData.InterfaceUnits)
                                    if (histUomForConversion.ID != histData.ModelUOM)
                                    {
                                        bool ucfFound = LRRModel.ChildModel.UCFs.UCFCombHash.ContainsKey(histUomForConversion.ID + "|" + histData.InterfaceUnits);
                                        histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histUomForConversion.ID, histData.ModelUOM, finalInterfaceValue, false, null);
                                        if (!ucfFound)
                                            histData.ErrorStatus = "UCF not found.";
                                    }
                                    else
                                    {
                                        histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                    }
                                }
                                else
                                {
                                    histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                }
                            }
                            else
                            {
                                histUomName = histUomName + "/hr";
                                List<string> l1 = LRRModel.ChildModel.UOMTypeXRefs.GetUOMIDsFromUOMTypeID(histData.UOMTypeID);
                                foreach (string s in l1)
                                {
                                    LRRUOM uom = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(s);
                                    if (uom.Name == histUomName)
                                    {
                                        histUomForConversion = uom;
                                        break;
                                    }
                                }
                                if (histUomForConversion != null)
                                {
                                    //if (histUomForConversion.ID != histData.InterfaceUnits)
                                    if (histUomForConversion.ID != histData.ModelUOM)
                                    {
                                        bool ucfFound = LRRModel.ChildModel.UCFs.UCFCombHash.ContainsKey(histUomForConversion.ID + "|" + histData.InterfaceUnits);
                                        histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histUomForConversion.ID, histData.ModelUOM, finalInterfaceValue, false, null);
                                        if (!ucfFound)
                                            histData.ErrorStatus = "UCF not found.";
                                    }
                                    else
                                    {
                                        //histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                        //commented by ashwin
                                        histData.InterfaceValue = finalInterfaceValue;
                                    }
                                }
                                else
                                {
                                    //histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                                    //commented by ashwin
                                    histData.InterfaceValue = finalInterfaceValue;
                                }
                            }
                        }
                    }
                    else
                    {
                        if (histData.HistUOMTypeID != null && histData.UOMTypeID != null)
                        {
                            if (histData.UOMTypeID == histData.HistUOMTypeID)
                            {
                                if (histData.HistUOMTypeID != "" && histData.UOMTypeID != "")
                                {
                                    if (histData.HistUnits != null && histData.ModelUOM != null)
                                    {
                                        //check if COnversion Formula is existing for this Hist Point

                                        //string modelUOMID =  LRRModel.ChildModel.GetUOMIDFromName(histData.ModelUOM);
                                        if (histData.HistUnits == histData.ModelUOM)
                                        {
                                            histData.InterfaceValue = finalInterfaceValue;
                                        }
                                        else //Need to apply the UCF...
                                        {
                                            bool applyUCF = true;
                                            if (histData.ConversionFormulaID != null && histData.ConversionFormulaID != "")
                                            {
                                                LRRFormula lrrFormula = LRRModel.FormulasObj.GetFormulaObjFromID(histData.ConversionFormulaID);
                                                if (lrrFormula != null)
                                                {
                                                    if (lrrFormula.Expression.Trim() != "")
                                                    {
                                                        if (lrrFormula.Params != null)
                                                        {
                                                            histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.Calculate(lrrFormula.Params, lrrFormula.Expression, finalInterfaceValue);
                                                            applyUCF = false;
                                                        }

                                                    }
                                                }

                                            }
                                            if (applyUCF)
                                            {
                                                histData.ErrorStatus = "";
                                                bool ucfFound = LRRModel.ChildModel.UCFs.UCFCombHash.ContainsKey(histData.HistUnits + "|" + histData.ModelUOM);
                                                histData.InterfaceValue = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(histData.HistUnits, histData.ModelUOM, finalInterfaceValue, false, null);
                                                if (!ucfFound)
                                                    histData.ErrorStatus = "UCF not found.";
                                            }
                                        }
                                    }
                                    else
                                    {
                                        histData.InterfaceValue = ret[1];
                                    }
                                }
                            }
                            else
                            {
                                histData.InterfaceValue = histData.AverageValueForPeriodEntered;
                            }
                        }
                    }
                }
            }

            startTime = histPoint.StartTime;
            endTime = histPoint.EndTime;
            _currentStartTime = startTime;
            _currentEndTime = endTime;

            //ReceivedPoint.BeginInvoke(histData,histPoint.StartTime,histPoint.EndTime,null,null);
            bPostCalc = true;

            if (_runControlSet.SaveToOfflineHistorian)
            {
                //Save the dataSet
                if ((_arrOldHistTagsForOffline == null) || (!_isSetAlreadyPresentForOffline) || (!_arrOldHistTagsForOffline.Contains(histData.HistTag)))
                {
                    LRRHistorianPoint savedPoint = new LRRHistorianPoint();
                    savedPoint.HistTag = histData.HistTag;
                    savedPoint.DataSetID = _savedSetForOffline.ID;
                    savedPoint.StartOfPeriodValue = histData.StartOfPeriodValue;
                    savedPoint.EndOfPeriodValue = histData.EndOfPeriodValue;
                    savedPoint.SpotValue = histData.SpotValue;
                    savedPoint.AverageValueForPeriodEntered = histData.AverageValueForPeriodEntered;
                    savedPoint.ObjectState = Symcon.Globals.Enums.DBOperation.INSERT;
                    _savedPointsForOffline.Add(savedPoint);
                }
                //End
            }

        }


        public void EndRunCycle()
        {
            try
            {
                if (_runID != null)
                {
                    //Save the PropertyValues Collection in Property Values User and Time table
                    //Check for Recoding On/Off
                    // commented by ashwin for handling the offset during recording
                    //if (this._app.ToggleRecording)
                      //  SaveRecordingValues(_currentStartTime, _currentEndTime);
                }

                //Check for Trending On/Off
                if (this._app.ToggleTrending)
                    SaveTrendValues(_currentStartTime, _currentEndTime);
                //if(RunCycleCompleted != null)
                //    RunCycleCompleted(this,new EventArgs());
                this.IternalRaiseRunCycleCompleted(this, null);

                #region Legacy Code

                _subs.Update = true;

                //Save the Property Values
                LRRSiteComponentPropertyValues toSaveSiteCompPropvalues = new LRRSiteComponentPropertyValues();
                //Clone Property Values

                IEnumerator iSiteCompPropValueEnum = LRRModel.PropertyValuesObj.GetChanges().GetEnumerator();
                while (iSiteCompPropValueEnum.MoveNext())
                {
                    LRRSiteComponentPropertyValue siteCompPropValue = iSiteCompPropValueEnum.Current as LRRSiteComponentPropertyValue;
                    // comented by ashwin on Sep-02-09 for handling the memory leak during runs
                    /*				
                    LRRSiteComponentPropertyValue clonedSiteCompPropValue = siteCompPropValue.Clone() as LRRSiteComponentPropertyValue;
					clonedSiteCompPropValue.ArrListUpdatedColNames = siteCompPropValue.ArrListUpdatedColNames.Clone() as ArrayList;
					toSaveSiteCompPropvalues.Add(clonedSiteCompPropValue);
					_subs.ServerUpdateQue.Enqueue(clonedSiteCompPropValue);
                     * */


                    toSaveSiteCompPropvalues.Add(siteCompPropValue);
                    _subs.ServerUpdateQue.Enqueue(siteCompPropValue);



                }


                LRRSiteGroupPropertyQualifierValues toSaveSiteGroupPropertyQualifierValues = new LRRSiteGroupPropertyQualifierValues();
                //Clone Qualifier Values
                IEnumerator iSiteCompQualValueEnum = LRRModel.SiteGroupPropertyQualifierValuesObj.GetChanges().GetEnumerator();
                while (iSiteCompQualValueEnum.MoveNext())
                {
                    LRRSiteGroupPropertyQualifierValue siteCompQualValue = iSiteCompQualValueEnum.Current as LRRSiteGroupPropertyQualifierValue;
                    // comented by ashwin on Sep-02-09 for handling the memory leak during runs
                    /*
					LRRSiteGroupPropertyQualifierValue clonedSiteCompQualValue  = siteCompQualValue.Clone() as LRRSiteGroupPropertyQualifierValue;
					clonedSiteCompQualValue.ArrListUpdatedColNames = siteCompQualValue.ArrListUpdatedColNames.Clone()  as ArrayList;
					toSaveSiteGroupPropertyQualifierValues.Add(clonedSiteCompQualValue);
					_subs.ServerUpdateQue.Enqueue(clonedSiteCompQualValue);
                     * */


                    toSaveSiteGroupPropertyQualifierValues.Add(siteCompQualValue);
                    _subs.ServerUpdateQue.Enqueue(siteCompQualValue);


                }

                LRRAggregatePropertyValues toSaveAggregatePropvalues = new LRRAggregatePropertyValues();
                //Clone Property Values
                IEnumerator iAggPropValueEnum = LRRModel.AggregatePropertyValuesObj.GetChanges().GetEnumerator();
                while (iAggPropValueEnum.MoveNext())
                {
                    LRRAggregatePropertyValue aggPropValue = iAggPropValueEnum.Current as LRRAggregatePropertyValue;
                    // comented by ashwin on Sep-02-09 for handling the memory leak during runs
                    /*
                    LRRAggregatePropertyValue clonedAggPropValue = aggPropValue.Clone() as LRRAggregatePropertyValue;
					clonedAggPropValue.ArrListUpdatedColNames = aggPropValue.ArrListUpdatedColNames.Clone() as ArrayList;
					toSaveAggregatePropvalues.Add(clonedAggPropValue);
					_subs.ServerUpdateQue.Enqueue(clonedAggPropValue);
                     * */

                    toSaveAggregatePropvalues.Add(aggPropValue);
                    _subs.ServerUpdateQue.Enqueue(aggPropValue);

                }

                #endregion
                //Refresh Collections
                /*
				LRRModel.PropertyValuesObj.RefreshCollection();
				LRRModel.SiteGroupPropertyQualifierValuesObj.RefreshCollection();
                LRRModel.AggregatePropertyValuesObj.RefreshCollection();
                 * */

                SaveChangesDel saveDelegate = new SaveChangesDel(SaveChanges);
                saveDelegate.BeginInvoke(toSaveSiteCompPropvalues, toSaveSiteGroupPropertyQualifierValues, toSaveAggregatePropvalues, null, null);

                _pauseFlag1 = false;
                _processingPoint = false;
                _getPointsInProgress = false;
                bool bExit = false;
                _subs.Update = false;



                RunMessage prevRunMsg = _pollerQueue.Peek() as RunMessage;
                RunMessage currentRunMsg = CreateRunMsg(prevRunMsg.RunType);
                _currentRunMsg = currentRunMsg;

                if (_pollerQueue.Count > 0)
                {
                    //RunMessage prevRunMsg = _pollerQueue.Peek() as RunMessage;
                    //RunMessage currentRunMsg = CreateRunMsg(prevRunMsg.RunType);
                    if ((currentRunMsg.RunType != RunTypeEnum.SINGLE_RUN_ENTERED_WINDOW) && (currentRunMsg.RunType != RunTypeEnum.SINGLE_RUN_STEP_NEXT) && (currentRunMsg.RunType != RunTypeEnum.SINGLE_RUN_ADVANCE_NOW))
                    {
                        AssignStartTimeEndTime(currentRunMsg, prevRunMsg);
                        if (_stopRunSequence == true)
                        {
                            _pollerQueue.Clear();
                            bExit = true;
                            //if(RunStopped != null)
                            //    RunStopped(this,new EventArgs());
                            this.IternalRaiseRunStopped(this, null);

                        }
                        if ((currentRunMsg.RunCondition == (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITH_INCREMENT))
                            || (currentRunMsg.RunCondition == (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT)))
                        {
                            DateTime dtStartTime = (DateTime)Symcon.BusinessObjects.SymConverter.Convert(typeof(string), currentRunMsg.StartTime, typeof(DateTime));
                            DateTime dtStartTime1 = dtStartTime.Add(_runControlSet.GetTimeIncrementSpan() + _runControlSet.GetDataWindowSpan());
                            TimeSpan tsForecast = dtStartTime1.Subtract(_runControlSet.AutoRunsObj.EndTime);
                            if (tsForecast.Seconds > 0)
                            {
                                _goodToComplete = true;
                            }

                            TimeSpan ts = dtStartTime.Subtract(_runControlSet.AutoRunsObj.EndTime);
                            if (ts.Ticks > 0)
                            {
                                _pollerQueue.Clear();
                                bExit = true;
                            }
                        }
                        //Check for Tuning Database conditions
                        if ((currentRunMsg.RunCondition == RunConditionEnum.TUNING_RUN_WITH_SEQUENTIAL_DATASETS)
                            || (currentRunMsg.RunCondition == RunConditionEnum.TUNING_RUN_WITH_RANDOM_DATASETS))
                        {
                            if ((_tuningRunCounter + 1) == _lrrSavedSets.Count)
                            {
                                _goodToComplete = true;
                            }

                            if (_tuningRunCounter == _lrrSavedSets.Count)
                            {
                                _pollerQueue.Clear();

                                bExit = true;
                            }
                        }


                        //Check for Offline Database conditions -- Added by nidhig
                        if ((currentRunMsg.RunCondition == RunConditionEnum.OFFLINE_RUN_WITH_SEQUENTIAL_DATASETS)
                            || (currentRunMsg.RunCondition == RunConditionEnum.OFFLINE_RUN_WITH_RANDOM_DATASETS))
                        {
                            if ((_offlineRunCounter + 1) == _lrrSavedSets.Count)
                            {
                                _goodToComplete = true;
                            }

                            if (_offlineRunCounter == _lrrSavedSets.Count)
                            {
                                _pollerQueue.Clear();

                                bExit = true;
                            }
                        }

                        if (!bExit)
                        {
                            if (_pauseFlag)
                                currentRunMsg.MsgState = MessageStateEnum.PAUSED;
                            _pollerQueue.Dequeue();
                            if (currentRunMsg.StartTime != null && currentRunMsg.EndTime != null)
                                _pollerQueue.Enqueue(currentRunMsg);
                        }
                    }
                    else
                    {
                        prevRunMsg.MsgState = MessageStateEnum.PROCESSED;
                    }
                }
                else
                {
                    /*if(_runStarted)
                        {
                            _goodToComplete = true;
                        }  */

                }

            }
            catch (Exception ex)
            {
                _pollerQueue.Clear();
                _getPointsInProgress = false;
                _processingPoint = false;
                _pauseFlag1 = false;
                //if(RunCompleted != null)
                //    RunCompleted(this,new EventArgs());
                this.InternalRaiseRunCompleted(this, null);
                Logger.LogMessage(typeof(RunController), "timer_tick :" + ex.Message, Logger.Level.Debug);
            }
            finally
            {
                InUIThread = false;
                GC.Collect();
            }

            /*if(_goodToComplete && _runStarted)
                {
                    if(RunCompleted != null)
                    {
                        RunCompleted(this,new EventArgs());
                        _goodToComplete = false;
                        _runStarted = false;
                    }
                }  */

            SetHistorianTagValues();
        }

        #region Public Methods

        /// <summary>
        ///  function for data historian writes Access - controls who can write back to historian 
        /// </summary>
        public void SetHistorianTagValues()
        {
            String user = _app.LoggedUser;
            TimeSpan tsDataWindow = _runControlSet.GetDataWindowSpan();
            DateTime endTime = new DateTime();
            DateTime startTime = new DateTime();
            DateTime currentDateTime = new DateTime();
            string histTag = "";
            string serverName = "";
            string histValue = "";
            LRRUserHistDatas lrrHistDatas = LRRModel.UserHistDatas;
            string userID = _app.DBManagerObj.AccessControlModel.UserID;
            string modePermission = _app.DBManagerObj.AccessControlModel.ModePermission;
            foreach (LRRUserHistData lrrHistData in lrrHistDatas)
            {
                if (_runControlSet.DataSourceType.ToUpper() == LRRHeaderText.GetHeaderString("SID_ONLINE").ToUpper())
                {
                    histTag = lrrHistData.HistTag;
                    LRRHistData histDataObj = LRRModel.HistDatas.GetHistDataFromHistTagUID(lrrHistData.HistTagUID);
                    serverName = lrrHistData.ServerID;
                    currentDateTime = System.Convert.ToDateTime(System.DateTime.Now.ToString(Symcon.BusinessObjects.SymConverter.LRR_DATE_TIME_FORMAT, Symcon.BusinessObjects.SymConverter.Culture));

                    /*
                    switch(modePermission)
                    {
                        case "1": //Measurement
                            histValue = lrrHistData.Measurement;
                            break;
                        case "2": //SystemStatus
                            histValue = lrrHistData.SystemStatus;
                            break;
                        case "3":  //Scenarios
                            histValue = lrrHistData.Scenario;
                            break;
                    }*/
                    if (lrrHistData.ModeID == modePermission)
                    {
                        if (histDataObj.SourceType != null && histDataObj.SourceType.Trim() != "")
                        {
                            IHistorian writeIHist = _app.LoadedOnLineHistorianAssemblies[histDataObj.SourceType] as IHistorian;
                            //writeIHist.SetPoint(lrrHistData.HistTag,lrrHistData.ServerID,lrrHistData.histValue,DateTime.Now);
                            string msg = "Hist Tag :" + histTag + "," + "ServerName:" + serverName + "," + "Value:" + lrrHistData.InterfaceValue + "DateTime:" + currentDateTime;
                            Logger.LogMessage(msg, Logger.Level.Debug);
                        }
                        else
                        {
                            IHistorian writeIHist = _app.LoadedOnLineHistorianAssemblies[_app.PI] as IHistorian;
                            string msg = "Hist Tag :" + histTag + "," + "ServerName:" + serverName + "," + "Value:" + lrrHistData.InterfaceValue + "DateTime:" + currentDateTime;
                            Logger.LogMessage(msg, Logger.Level.Debug);
                        }
                    }

                }
            }
        }
        public bool Start(RunTypeEnum runType, RunControlSet runControlSet)
        {
            if (!CheckRunSettrings(runControlSet))
            {
                return false;
            }
            if (runControlSet.RecordValues)
            {
                _runSetID = this._app.SaveRunset();
            }



            _app.LRRUIManagerObj.SetPauseSate(true);
            bool ret = false;
            _stopRunSequence = false;
            _runControlSet = runControlSet;

            RunMessage currentRunMsg = CreateRunMsg(runType);
            _currentRunMsg = currentRunMsg;
            if (currentRunMsg != null)
            {

                AssignStartTimeEndTime(currentRunMsg, null);
                if (currentRunMsg.StartTime != null && currentRunMsg.EndTime != null)
                {
                    _pollerQueue.Enqueue(currentRunMsg);
                    _runStarted = true;
                    _processingPoint = false;
                    _getPointsInProgress = false;
                    ret = true;
                    _bRunID = true;
                }
            }
            else
            {
                ret = false;
            }

            return ret;

        }

        public void Stop()
        {
            _app.LRRUIManagerObj.SetStopEnabled(false);
            _stopRunSequence = true;


            //			while(_getPointsInProgress)	//Wait if any _getPoint is in progress
            //			{
            //				Application.DoEvents();
            //				System.Threading.Thread.Sleep(1);
            //			}
            //			while(_processingPoint)	//Wait if anyprocessing is in progress...
            //			{
            //				Application.DoEvents();
            //				System.Threading.Thread.Sleep(1);
            //			}
            //			while(InUIThread)
            //			{
            //				Application.DoEvents();
            //				System.Threading.Thread.Sleep(1);
            //			}
            //			 _pollerQueue.Clear();
            //			if(RunStopped != null)
            //				RunStopped(this,new EventArgs());
        }

        private bool _pauseFlag = false;
        private bool _pauseFlag1 = false;

        public void Pause()
        {
            _pauseFlag = true;
        }

        public void Resume()
        {
            if (_pollerQueue.Count > 0)
            {
                RunMessage rmsg = (RunMessage)_pollerQueue.Peek();
                //_currentRunMsg = rmsg;
                if ((rmsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITH_INCREMENT) || (rmsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT))
                    AssignStartTimeEndTime(rmsg, null);
                _pauseFlag = false;
                rmsg.MsgState = MessageStateEnum.ACTIVE;
            }
        }


        private RunMessage CreateRunMsg(RunTypeEnum runType)
        {
            RunMessage runMsg = new RunMessage();
            RunConditionEnum ret = RunConditionEnum.NONE;
            runMsg.MsgState = MessageStateEnum.ACTIVE;
            runMsg.RunType = runType;

            //Check for RunType
            switch (runType)
            {
                case RunTypeEnum.SINGLE_RUN:
                    //Check for Current Run
                    if (_runControlSet.RunSettingsObj.IsCurrent || _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_TUNING") || _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_OFFLINE")) //Modified by Nidhig
                    {
                        ret = RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT;
                    }
                    break;
                case RunTypeEnum.SINGLE_RUN_ENTERED_WINDOW:
                    if (_runControlSet.RunSettingsObj.IsCurrent || _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_TUNING") || _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_OFFLINE")) //Modified by Nidhig
                    {
                        ret = RunConditionEnum.CURRENT_RUN_WITH_CUSTOM_TIME;
                    }
                    break;
                case RunTypeEnum.SINGLE_RUN_STEP_NEXT:
                    if (_runControlSet.RunSchedulesObj.TimeIncrementFlag)
                    {
                        ret = RunConditionEnum.CURRENT_RUN_WITH_INCREMENT_NEXT_STEP;
                    }
                    else
                    {
                        //Cannot Apply this run with out time increment...
                        ErroMessage = "Cannot Create Run Message without Time Increment.";
                        return null;
                    }
                    break;
                case RunTypeEnum.SINGLE_RUN_ADVANCE_NOW:
                    ret = RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT;
                    break;
                case RunTypeEnum.AUTO_RUN:
                    //Check if it is sequential Run
                    if (_runControlSet.RunSchedulesObj.SequentialDataSetFlag && _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_TUNING"))
                    {
                        ret = RunConditionEnum.TUNING_RUN_WITH_SEQUENTIAL_DATASETS;
                    }
                    //Check if it is Random Run
                    else if (_runControlSet.RunSchedulesObj.RandomDataSetFlag && _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_TUNING"))
                    {
                        ret = RunConditionEnum.TUNING_RUN_WITH_RANDOM_DATASETS;
                    }
                    //Nidhig
                    else if (_runControlSet.RunSchedulesObj.SequentialDataSetFlag && _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_OFFLINE"))
                    {
                        ret = RunConditionEnum.OFFLINE_RUN_WITH_SEQUENTIAL_DATASETS;
                    }
                    //Check if it is Random Run
                    else if (_runControlSet.RunSchedulesObj.RandomDataSetFlag && _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_OFFLINE"))
                    {
                        ret = RunConditionEnum.OFFLINE_RUN_WITH_RANDOM_DATASETS;
                    }
                    else if (_runControlSet.RunSettingsObj.IsCurrent && _runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_ONLINE"))
                    {
                        if (_runControlSet.RunSchedulesObj.TimeIncrementFlag)
                        {
                            ret = RunConditionEnum.CURRENT_RUN_WITH_INCREMENT;
                        }
                        else
                        {
                            ret = RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT;
                        }

                        //AutoRun Boundaries
                        if (_runControlSet.AutoRunsObj.AutoRun)
                        {
                            if (_runControlSet.RunSchedulesObj.TimeIncrementFlag)
                            {
                                ret = RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITH_INCREMENT;
                            }
                            else
                            {
                                ret = RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT;
                            }
                        }
                    }
                    else if (_runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_TUNING"))
                    {
                        //AutoRun Boundaries
                        if (_runControlSet.AutoRunsObj.AutoRun)
                        {
                            if (_runControlSet.RunSchedulesObj.TimeIncrementFlag)
                            {
                                ret = RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITH_INCREMENT;
                            }
                            else
                            {
                                ret = RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT;
                            }
                        }
                        else
                        {
                            ret = RunConditionEnum.TUNING_RUN_WITH_INCREMENT;
                        }
                    }
                    else if (_runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_OFFLINE")) //Nidhig
                    {
                        //AutoRun Boundaries
                        if (_runControlSet.AutoRunsObj.AutoRun)
                        {
                            if (_runControlSet.RunSchedulesObj.TimeIncrementFlag)
                            {
                                ret = RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITH_INCREMENT;
                            }
                            else
                            {
                                ret = RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT;
                            }
                        }
                        else
                        {
                            ret = RunConditionEnum.OFFLINE_RUN_WITH_INCREMENT;
                        }
                    }
                    break;
                case RunTypeEnum.NONE:
                    break;

            }
            runMsg.RunCondition = ret;
            return runMsg;
        }

        private void AssignStartTimeEndTime(RunMessage currentMsg, RunMessage prevRunMsg)
        {
            TimeSpan tsDataWindow = _runControlSet.GetDataWindowSpan();
            TimeSpan tsLag = _runControlSet.GetLagSpan();
            TimeSpan tsTimeIncrement = _runControlSet.GetTimeIncrementSpan();

            DateTime endTime = new DateTime();
            DateTime startTime = new DateTime();
            bool assignedDateTime = true;
            if (prevRunMsg == null)
            {
                switch (currentMsg.RunCondition)
                {
                    case RunConditionEnum.CURRENT_RUN_WITH_INCREMENT:
                        endTime = DateTime.Now.Subtract(tsLag);
                        startTime = endTime.Subtract(tsDataWindow);
                        break;
                    case RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT:
                        endTime = DateTime.Now.Subtract(tsLag);
                        startTime = endTime.Subtract(tsDataWindow);
                        break;
                    case RunConditionEnum.CURRENT_RUN_WITH_INCREMENT_NEXT_STEP:
                        startTime = _runControlSet.DataWindowObj.StartTime.Add(tsTimeIncrement);
                        endTime = startTime.Add(tsDataWindow);
                        break;
                    case RunConditionEnum.CURRENT_RUN_WITH_CUSTOM_TIME:
                        startTime = _runControlSet.DataWindowObj.StartTime;
                        endTime = _runControlSet.DataWindowObj.StartTime.Add(tsDataWindow);
                        break;
                    case (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITH_INCREMENT):
                        startTime = _runControlSet.AutoRunsObj.StartTime;
                        endTime = startTime.Add(tsDataWindow);
                        break;
                    case (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT):
                        startTime = _runControlSet.AutoRunsObj.StartTime;
                        endTime = startTime.Add(tsDataWindow);
                        break;
                    //Tuning DataBase cases
                    //Sequential DataSet
                    case RunConditionEnum.TUNING_RUN_WITH_SEQUENTIAL_DATASETS:
                        //_lrrSavedSets = this.GetActiveHistorian().GetSavedSets();
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                        {
                            _lrrSavedSets = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING].GetSavedSets();
                            _lrrSavedSets.SortColumn = "StartTime";
                            _lrrSavedSets.Sort();
                            _tuningRunCounter = 0;
                            if (_lrrSavedSets != null)
                            {
                                if (_lrrSavedSets.Count > 0)
                                {
                                    LRRHistSavedSet lrrSavedSet = _lrrSavedSets[_tuningRunCounter] as LRRHistSavedSet;
                                    foreach (IHistorian iHistObj in _app.LoadedOffLineHistorianAssemblies.Values)
                                    {
                                        iHistObj.PrepareCollection(lrrSavedSet.ID);
                                        startTime = lrrSavedSet.StartTime;
                                        endTime = lrrSavedSet.EndTime;
                                    }

                                }
                                else
                                {
                                    LRRMessagBox.ShowMessage("SID_MISSING_TUNING", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                }
                            }
                        }
                        break;
                    //Random DataSet
                    case RunConditionEnum.TUNING_RUN_WITH_RANDOM_DATASETS:
                        //_lrrSavedSets = this.GetActiveHistorian().GetSavedSets();
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                        {
                            _lrrSavedSets = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING].GetSavedSets();
                            _tuningRunCounter = 0;
                            if (_lrrSavedSets != null)
                            {
                                if (_lrrSavedSets.Count > 0)
                                {
                                    _tuningRandomIndex = _lrrSavedSets.Count;
                                    _tuningRandomIndex = _lrrSavedSets.Count - _tuningRandomCounter.Next(1, _tuningRandomIndex);
                                    LRRHistSavedSet lrrSavedSet = _lrrSavedSets[_tuningRandomIndex] as LRRHistSavedSet;
                                    foreach (IHistorian iHistObj in _app.LoadedOffLineHistorianAssemblies.Values)
                                    {
                                        iHistObj.PrepareCollection(lrrSavedSet.ID);
                                        startTime = lrrSavedSet.StartTime;
                                        endTime = lrrSavedSet.EndTime;
                                    }
                                }
                                else
                                {
                                    LRRMessagBox.ShowMessage("SID_MISSING_TUNING", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                }
                            }
                        }
                        break;
                    case RunConditionEnum.TUNING_RUN_WITH_INCREMENT:
                        startTime = _runControlSet.DataWindowObj.StartTime;
                        endTime = _runControlSet.DataWindowObj.StartTime.Add(tsDataWindow);
                        break;
                    //Nidhig
                    case RunConditionEnum.OFFLINE_RUN_WITH_SEQUENTIAL_DATASETS:
                        //_lrrSavedSets = this.GetActiveHistorian().GetSavedSets();
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                        {
                            _lrrSavedSets = _app.LoadedOffLineHistorianAssemblies[_app.OFFLINE].GetSavedSets();
                            _lrrSavedSets.SortColumn = "StartTime";
                            _lrrSavedSets.Sort();
                            _offlineRunCounter = 0;
                            if (_lrrSavedSets != null)
                            {
                                if (_lrrSavedSets.Count > 0)
                                {
                                    LRRHistSavedSet lrrSavedSet = _lrrSavedSets[_offlineRunCounter] as LRRHistSavedSet;
                                    foreach (IHistorian iHistObj in _app.LoadedOffLineHistorianAssemblies.Values)
                                    {
                                        iHistObj.PrepareCollection(lrrSavedSet.ID);
                                        startTime = lrrSavedSet.StartTime;
                                        endTime = lrrSavedSet.EndTime;
                                    }
                                }
                                else
                                {
                                    LRRMessagBox.ShowMessage("SID_MISSING_OFFLINE", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                }
                            }
                        }
                        break;
                    //Random DataSet
                    case RunConditionEnum.OFFLINE_RUN_WITH_RANDOM_DATASETS:
                        //_lrrSavedSets = this.GetActiveHistorian().GetSavedSets();
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                        {
                            _lrrSavedSets = _app.LoadedOffLineHistorianAssemblies[_app.OFFLINE].GetSavedSets();
                            _offlineRunCounter = 0;
                            if (_lrrSavedSets != null)
                            {
                                if (_lrrSavedSets.Count > 0)
                                {
                                    _offlineRandomIndex = _lrrSavedSets.Count;
                                    _offlineRandomIndex = _lrrSavedSets.Count - _offlineRandomCounter.Next(1, _offlineRandomIndex);
                                    LRRHistSavedSet lrrSavedSet = _lrrSavedSets[_offlineRandomIndex] as LRRHistSavedSet;
                                    foreach (IHistorian iHistObj in _app.LoadedOffLineHistorianAssemblies.Values)
                                    {
                                        iHistObj.PrepareCollection(lrrSavedSet.ID);
                                        startTime = lrrSavedSet.StartTime;
                                        endTime = lrrSavedSet.EndTime;
                                    }
                                }
                                else
                                {
                                    LRRMessagBox.ShowMessage("SID_MISSING_OFFLINE", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                }
                            }
                        }
                        break;
                    case RunConditionEnum.OFFLINE_RUN_WITH_INCREMENT:
                        startTime = _runControlSet.DataWindowObj.StartTime;
                        endTime = _runControlSet.DataWindowObj.StartTime.Add(tsDataWindow);
                        break;
                    case RunConditionEnum.NONE:
                        break;
                }
            }
            else
            {
                switch (currentMsg.RunCondition)
                {
                    case RunConditionEnum.CURRENT_RUN_WITH_INCREMENT:
                        endTime = DateTime.Parse(prevRunMsg.EndTime, Symcon.BusinessObjects.SymConverter.Culture).Add(tsTimeIncrement).Subtract(tsLag);
                        startTime = endTime.Subtract(tsDataWindow);
                        break;
                    case RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT:
                        endTime = DateTime.Now.Subtract(tsLag);
                        startTime = endTime.Subtract(tsDataWindow);
                        break;
                    case (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITH_INCREMENT):
                        startTime = DateTime.Parse(prevRunMsg.StartTime, Symcon.BusinessObjects.SymConverter.Culture).Add(tsTimeIncrement);
                        endTime = startTime.Add(tsDataWindow);
                        break;
                    case (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT):
                        startTime = DateTime.Parse(prevRunMsg.EndTime, Symcon.BusinessObjects.SymConverter.Culture);
                        endTime = startTime.Add(tsDataWindow);
                        break;
                    //Tuning DataBase cases
                    //Sequential DataSet
                    case RunConditionEnum.TUNING_RUN_WITH_SEQUENTIAL_DATASETS:
                        _tuningRunCounter = _tuningRunCounter + 1;
                        if (_lrrSavedSets != null)
                        {
                            if (_lrrSavedSets.Count > 0)
                            {
                                if (_tuningRunCounter < _lrrSavedSets.Count)
                                {
                                    LRRHistSavedSet lrrSavedSet = _lrrSavedSets[_tuningRunCounter] as LRRHistSavedSet;
                                    foreach (IHistorian iHistObj in _app.LoadedOffLineHistorianAssemblies.Values)
                                    {
                                        iHistObj.PrepareCollection(lrrSavedSet.ID);
                                        startTime = lrrSavedSet.StartTime;
                                        endTime = lrrSavedSet.EndTime;
                                    }
                                }
                            }
                            else
                            {
                                LRRMessagBox.ShowMessage("SID_MISSING_TUNING", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                        }
                        break;
                    //Random DataSet
                    case RunConditionEnum.TUNING_RUN_WITH_RANDOM_DATASETS:
                        //_lrrSavedSets = this.GetActiveHistorian().GetSavedSets();
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                        {
                            _lrrSavedSets = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING].GetSavedSets();
                            _tuningRunCounter = _tuningRunCounter + 1;
                            if (_lrrSavedSets != null)
                            {
                                if (_lrrSavedSets.Count > 0)
                                {
                                    _tuningRandomIndex = _lrrSavedSets.Count - _tuningRandomCounter.Next(1, _tuningRandomIndex);
                                    LRRHistSavedSet lrrSavedSet = _lrrSavedSets[_tuningRandomIndex] as LRRHistSavedSet;
                                    foreach (IHistorian iHistObj in _app.LoadedOffLineHistorianAssemblies.Values)
                                    {
                                        iHistObj.PrepareCollection(lrrSavedSet.ID);
                                        startTime = lrrSavedSet.StartTime;
                                        endTime = lrrSavedSet.EndTime;
                                    }
                                }
                                else
                                {
                                    LRRMessagBox.ShowMessage("SID_MISSING_TUNING", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                }
                            }
                        }
                        break;
                    case RunConditionEnum.TUNING_RUN_WITH_INCREMENT:
                        startTime = DateTime.Parse(prevRunMsg.StartTime, Symcon.BusinessObjects.SymConverter.Culture).Add(tsTimeIncrement);
                        endTime = startTime.Add(tsDataWindow);
                        break;
                    //Offline DataBase cases  Nidhig
                    //Sequential DataSet
                    case RunConditionEnum.OFFLINE_RUN_WITH_SEQUENTIAL_DATASETS:
                        _offlineRunCounter = _offlineRunCounter + 1;
                        if (_lrrSavedSets != null)
                        {
                            if (_lrrSavedSets.Count > 0)
                            {
                                if (_offlineRunCounter < _lrrSavedSets.Count)
                                {
                                    LRRHistSavedSet lrrSavedSet = _lrrSavedSets[_offlineRunCounter] as LRRHistSavedSet;
                                    foreach (IHistorian iHistObj in _app.LoadedOffLineHistorianAssemblies.Values)
                                    {
                                        iHistObj.PrepareCollection(lrrSavedSet.ID);
                                        startTime = lrrSavedSet.StartTime;
                                        endTime = lrrSavedSet.EndTime;
                                    }
                                }
                            }
                            else
                            {
                                LRRMessagBox.ShowMessage("SID_MISSING_OFFLINE", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                        }
                        break;
                    //Random DataSet
                    case RunConditionEnum.OFFLINE_RUN_WITH_RANDOM_DATASETS:
                        //_lrrSavedSets = this.GetActiveHistorian().GetSavedSets();
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                        {
                            _lrrSavedSets = _app.LoadedOffLineHistorianAssemblies[_app.OFFLINE].GetSavedSets();
                            _offlineRunCounter = _offlineRunCounter + 1;
                            if (_lrrSavedSets != null)
                            {
                                if (_lrrSavedSets.Count > 0)
                                {
                                    _offlineRandomIndex = _lrrSavedSets.Count - _offlineRandomCounter.Next(1, _offlineRandomIndex);
                                    LRRHistSavedSet lrrSavedSet = _lrrSavedSets[_offlineRandomIndex] as LRRHistSavedSet;
                                    foreach (IHistorian iHistObj in _app.LoadedOffLineHistorianAssemblies.Values)
                                    {
                                        iHistObj.PrepareCollection(lrrSavedSet.ID);
                                        startTime = lrrSavedSet.StartTime;
                                        endTime = lrrSavedSet.EndTime;
                                    }
                                }
                                else
                                {
                                    LRRMessagBox.ShowMessage("SID_MISSING_OFFLINE", "SID_LABEL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                }
                            }
                        }
                        break;

                    case RunConditionEnum.OFFLINE_RUN_WITH_INCREMENT:
                        startTime = DateTime.Parse(prevRunMsg.StartTime, Symcon.BusinessObjects.SymConverter.Culture).Add(tsTimeIncrement);
                        endTime = startTime.Add(tsDataWindow);
                        break;
                    case RunConditionEnum.NONE:
                        break;
                }
            }

            if (startTime != (new DateTime()))
            {
                
                currentMsg.StartTime = startTime.ToString(Symcon.BusinessObjects.SymConverter.LRR_DATE_TIME_FORMAT);
                currentMsg.EndTime = endTime.ToString(Symcon.BusinessObjects.SymConverter.LRR_DATE_TIME_FORMAT);
                
                 TimeSpan ts = endTime.Subtract(DateTime.Now);
                 TimeSpan ts1 = DateTime.Now.Subtract(endTime);
                 if (ts1.Milliseconds > 0)
                 {
                     if (prevRunMsg != null && prevRunMsg.RunType == RunTypeEnum.AUTO_RUN && (prevRunMsg.RunCondition == RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES ||
                         prevRunMsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITH_CUSTOM_TIME) && (_runControlSet.DataSourceType == "Online"))
                     {
                         prevRunMsg.StartTime = currentMsg.StartTime;
                         prevRunMsg.EndTime = currentMsg.EndTime;
                         AssignStartTimeEndTime(currentMsg, prevRunMsg);
                     }

                 }
                
              

            }
            else
            {
                currentMsg.StartTime = null;
                currentMsg.EndTime = null;
            }

            if (_runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_TUNING") && _runControlSet.RunSchedulesObj.RandomDataSetFlag == false && _runControlSet.RunSchedulesObj.SequentialDataSetFlag == false)
            {
                //Its a tuning run with the times to be found in the tuning database...
                string setID = "";
                if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                {
                    setID = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING].GetSetIDFromStartEndTime(currentMsg.StartTime, currentMsg.EndTime);
                    if (setID == null)
                    {
                        string[] dates = new string[2];
                        dates[0] = currentMsg.StartTime;
                        dates[1] = currentMsg.EndTime;
                        LRRMessagBox.ShowMessage("SID_NO_SET", "SID_LABEL", dates, MessageBoxButtons.OK, MessageBoxIcon.Information);
                        currentMsg.StartTime = null;
                        currentMsg.EndTime = null;
                        if (!_runControlSet.AutoRun)
                        {
                            this.InternalRaiseRunCompleted(this, null);
                            _runStarted = false;
                            _processingPoint = false;
                            _getPointsInProgress = false;
                        }
                        else
                        {

                        }

                    }
                    else
                    {
                        //this.GetActiveHistorian().PrepareCollection(setID);
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                        {
                            IHistorian iHist = _app.LoadedOffLineHistorianAssemblies[_app.TUNNING];
                            if (iHist.GetType() == typeof(Symcon.LightRidge.HistorianConnect.TuningHistorian))
                            {
                                Symcon.LightRidge.HistorianConnect.TuningHistorian tuningHistorian = iHist as Symcon.LightRidge.HistorianConnect.TuningHistorian;
                                tuningHistorian.PrepareCollectionByDataSetID(setID);
                            }
                            else
                            {
                                //this.GetActiveHistorian().PrepareCollection(setID);
                                if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.TUNNING))
                                {
                                    _app.LoadedOffLineHistorianAssemblies[_app.TUNNING].PrepareCollection(setID);
                                }
                            }
                            /*startTime = DateTime.Parse(_runControlSet.DataWindowObj.StartTimeValue,Symcon.BusinessObjects.SymConverter.Culture);
                            endTime = DateTime.Parse(_runControlSet.DataWindowObj.EndTimeValue,Symcon.BusinessObjects.SymConverter.Culture);*/
                        }
                    }
                }
            }
            //nidhig
            if (_runControlSet.DataSourceType == LRRHeaderText.GetHeaderString("SID_OFFLINE") && _runControlSet.RunSchedulesObj.RandomDataSetFlag == false && _runControlSet.RunSchedulesObj.SequentialDataSetFlag == false)
            {
                //Its a offline run with the times to be found in the offline database...
                string setID = "";
                if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                {
                    setID = _app.LoadedOffLineHistorianAssemblies[_app.OFFLINE].GetSetIDFromStartEndTime(currentMsg.StartTime, currentMsg.EndTime);
                    if (setID == null)
                    {
                        string[] dates = new string[2];
                        dates[0] = currentMsg.StartTime;
                        dates[1] = currentMsg.EndTime;
                        LRRMessagBox.ShowMessage("SID_OFFLINE_NO_SET", "SID_LABEL", dates, MessageBoxButtons.OK, MessageBoxIcon.Information);
                        currentMsg.StartTime = null;
                        currentMsg.EndTime = null;
                        if (!_runControlSet.AutoRun)
                        {
                            this.InternalRaiseRunCompleted(this, null);
                            _runStarted = false;
                            _processingPoint = false;
                            _getPointsInProgress = false;
                        }
                        else
                        {

                        }
                    }
                    else
                    {
                        //this.GetActiveHistorian().PrepareCollection(setID);
                        if (_app.LoadedOffLineHistorianAssemblies.ContainsKey(_app.OFFLINE))
                        {
                            _app.LoadedOffLineHistorianAssemblies[_app.OFFLINE].PrepareCollection(setID);
                        }
                        /*startTime = DateTime.Parse(_runControlSet.DataWindowObj.StartTimeValue,Symcon.BusinessObjects.SymConverter.Culture);
                        endTime = DateTime.Parse(_runControlSet.DataWindowObj.EndTimeValue,Symcon.BusinessObjects.SymConverter.Culture);*/
                    }
                }

            }

        }
        #endregion

        #region Event Handlers
        //Handler which gets called when the run gets completed...
        [System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.Synchronized)]
        private void 
            poller_Tick(object sender)
        {
            if (_pauseFlag || _pauseFlag1 || _processingPoint || InUIThread)
                return;
            if (_pollerQueue.Count > 0)
            {
                RunMessage runMsg = _pollerQueue.Peek() as RunMessage;
                //_currentRunMsg = runMsg;
                if (runMsg.MsgState == MessageStateEnum.ACTIVE)
                {
                    //This is a simple run for the entered window in both the current and historical states
                    if ((runMsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITH_CUSTOM_TIME)
                        || (runMsg.RunCondition == RunConditionEnum.HISTORY_RUN_WITH_CUSTOM_TIME)
                        || (runMsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT)
                        || (runMsg.RunCondition == (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITHOUT_INCREMENT)))
                    {
                        //runMsg.MsgState = MessageStateEnum.PROCESSING;
                        //Refresh the Property Values Collection
                        //						LRRModel.PropertyValuesObj.RefreshCollection();

                        TimeSpan ts = DateTime.Now.Subtract(DateTime.Parse(runMsg.EndTime, Symcon.BusinessObjects.SymConverter.Culture));
                        if (ts.Milliseconds > 0)
                        {
                            //Send for Processing
                            runMsg.MsgState = MessageStateEnum.PROCESSING;
                            //Refresh the Property Values Collection
                            //							LRRModel.PropertyValuesObj.RefreshCollection();

                            Run(runMsg.StartTime, runMsg.EndTime);
                        }
                        else
                        {
                            //if(FeedBack != null)
                            //{
                            if (!_app.LRRUIManagerObj.RunStatusWindowVisible)
                            {
                                //_app.LRRUIManagerObj.ShowRunStatusDialog(runMsg.StartTime,runMsg.EndTime,runMsg.);
                                _app.LRRUIManagerObj.ShowRunStatusDialog(_runControlSet.DataWindowObj.StartTime.ToString(Symcon.BusinessObjects.SymConverter.Culture), _runControlSet.DataWindowObj.EndTime.ToString(Symcon.BusinessObjects.SymConverter.Culture), _runControlSet.DataWindowObj.DataWindowSize.ToString(Symcon.BusinessObjects.SymConverter.Culture));
                            }
                            this.IternalRaiseGiveFeedback(Math.Abs(ts.Hours).ToString(Symcon.BusinessObjects.SymConverter.Culture) + ":" + Math.Abs(ts.Minutes).ToString(Symcon.BusinessObjects.SymConverter.Culture) + ":" + Math.Abs(ts.Seconds).ToString(Symcon.BusinessObjects.SymConverter.Culture));
                            //}

                            if (_stopRunSequence == true)
                            {
                                _pollerQueue.Clear();
                                //bExit = true;
                                //if(RunStopped != null)
                                //    RunStopped(this,new EventArgs());
                                this.IternalRaiseRunStopped(this, null);

                            }
                        }



                        //Run(runMsg.StartTime, runMsg.EndTime);
                    }
                    else if ((runMsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITH_INCREMENT)
                        || (runMsg.RunCondition == RunConditionEnum.CURRENT_RUN_WITH_INCREMENT_NEXT_STEP)
                        || (runMsg.RunCondition == (RunConditionEnum.CURRENT_RUN_AUTO_WITH_CUSTOM_BOUNDARIES | RunConditionEnum.CURRENT_RUN_WITH_INCREMENT)))
                    {
                        //TimeSpan ts = DateTime.Now.Subtract(DateTime.Parse(runMsg.StartTime, Symcon.BusinessObjects.SymConverter.Culture));
                        TimeSpan ts = DateTime.Now.Subtract(DateTime.Parse(runMsg.EndTime, Symcon.BusinessObjects.SymConverter.Culture));
                        if (ts.Milliseconds > 0)
                        {
                            //Send for Processing
                            runMsg.MsgState = MessageStateEnum.PROCESSING;
                            //Refresh the Property Values Collection
                            //							LRRModel.PropertyValuesObj.RefreshCollection();

                            Run(runMsg.StartTime, runMsg.EndTime);
                        }
                        else
                        {
                            //if(FeedBack != null)
                            //{
                            if (!_app.LRRUIManagerObj.RunStatusWindowVisible)
                            {
                                //_app.LRRUIManagerObj.ShowRunStatusDialog(runMsg.StartTime,runMsg.EndTime,runMsg.);
                                _app.LRRUIManagerObj.ShowRunStatusDialog(_runControlSet.DataWindowObj.StartTime.ToString(Symcon.BusinessObjects.SymConverter.Culture), _runControlSet.DataWindowObj.EndTime.ToString(Symcon.BusinessObjects.SymConverter.Culture), _runControlSet.DataWindowObj.DataWindowSize.ToString(Symcon.BusinessObjects.SymConverter.Culture));
                            }
                            this.IternalRaiseGiveFeedback(Math.Abs(ts.Hours).ToString(Symcon.BusinessObjects.SymConverter.Culture) + ":" + Math.Abs(ts.Minutes).ToString(Symcon.BusinessObjects.SymConverter.Culture) + ":" + Math.Abs(ts.Seconds).ToString(Symcon.BusinessObjects.SymConverter.Culture));
                            //}

                            if (_stopRunSequence == true)
                            {
                                _pollerQueue.Clear();
                                //bExit = true;
                                //if(RunStopped != null)
                                //    RunStopped(this,new EventArgs());
                                this.IternalRaiseRunStopped(this, null);

                            }
                        }
                    }
                    //Check for Tuning Database Run Conditions
                    else if ((runMsg.RunCondition == RunConditionEnum.TUNING_RUN_WITH_SEQUENTIAL_DATASETS)
                        || (runMsg.RunCondition == RunConditionEnum.TUNING_RUN_WITH_RANDOM_DATASETS
                        || (runMsg.RunCondition == RunConditionEnum.TUNING_RUN_WITH_INCREMENT)))
                    {
                        //Send for Processing
                        runMsg.MsgState = MessageStateEnum.PROCESSING;
                        //Refresh the Property Values Collection
                        //						LRRModel.PropertyValuesObj.RefreshCollection();

                        Run(runMsg.StartTime, runMsg.EndTime);
                    }
                    //Check for Offline Database Run Conditions - Nidhig
                    else if ((runMsg.RunCondition == RunConditionEnum.OFFLINE_RUN_WITH_SEQUENTIAL_DATASETS)
                        || (runMsg.RunCondition == RunConditionEnum.OFFLINE_RUN_WITH_RANDOM_DATASETS
                        || (runMsg.RunCondition == RunConditionEnum.OFFLINE_RUN_WITH_INCREMENT)))
                    {
                        //Send for Processing
                        runMsg.MsgState = MessageStateEnum.PROCESSING;
                        //Refresh the Property Values Collection
                        //						LRRModel.PropertyValuesObj.RefreshCollection();

                        Run(runMsg.StartTime, runMsg.EndTime);
                    }
                }
                else if (runMsg.MsgState == MessageStateEnum.PROCESSED)
                {
                    if (!_getPointsInProgress && !_processingPoint && _runStarted)
                    {
                        _runStarted = false;
                        _pollerQueue.Clear();
                        _getPointsInProgress = false;
                        _processingPoint = false;
                        this.InternalRaiseRunCompleted(this, null);
                    }
                }
                else
                {
                    return;
                }
            }
            else
            {
                if (!_getPointsInProgress && !_processingPoint && _runStarted)
                {
                    _runStarted = false;
                    this.InternalRaiseRunCompleted(this, null);
                }

            }
        }
        #endregion
    }
}
